/*
 * Copyright 2023 Stanislav Mikhailov (xavetar)
 *
 * Licensed under the Creative Commons Zero v1.0 Universal (CC0) License.
 * You may obtain a copy of the License at
 *
 *     http://creativecommons.org/publicdomain/zero/1.0/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the CC0 license is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// It is strictly forbidden to use the from_code method, respected Microsoft decided to define
// duplicate codes, but by defining different constant names, therefore, when using these codes,
// it is mandatory to use through from_name, and not from_code. Otherwise, it may cause undefined
// behavior or an unknown exception. Because one code corresponds to several constants.

use super::{RawError};

#[derive(Clone, Debug, Copy, Eq, Hash, Ord, PartialEq, PartialOrd)]
pub enum COMPLUS {
    COMADMIN_E_OBJECTERRORS,
    COMADMIN_E_OBJECTINVALID,
    COMADMIN_E_KEYMISSING,
    COMADMIN_E_ALREADYINSTALLED,
    COMADMIN_E_APP_FILE_WRITEFAIL,
    COMADMIN_E_APP_FILE_READFAIL,
    COMADMIN_E_APP_FILE_VERSION,
    COMADMIN_E_BADPATH,
    COMADMIN_E_APPLICATIONEXISTS,
    COMADMIN_E_ROLEEXISTS,
    COMADMIN_E_CANTCOPYFILE,
    COMADMIN_E_NOUSER,
    COMADMIN_E_INVALIDUSERIDS,
    COMADMIN_E_NOREGISTRYCLSID,
    COMADMIN_E_BADREGISTRYPROGID,
    COMADMIN_E_AUTHENTICATIONLEVEL,
    COMADMIN_E_USERPASSWDNOTVALID,
    COMADMIN_E_CLSIDORIIDMISMATCH,
    COMADMIN_E_REMOTEINTERFACE,
    COMADMIN_E_DLLREGISTERSERVER,
    COMADMIN_E_NOSERVERSHARE,
    COMADMIN_E_DLLLOADFAILED,
    COMADMIN_E_BADREGISTRYLIBID,
    COMADMIN_E_APPDIRNOTFOUND,
    COMADMIN_E_REGISTRARFAILED,
    COMADMIN_E_COMPFILE_DOESNOTEXIST,
    COMADMIN_E_COMPFILE_LOADDLLFAIL,
    COMADMIN_E_COMPFILE_GETCLASSOBJ,
    COMADMIN_E_COMPFILE_CLASSNOTAVAIL,
    COMADMIN_E_COMPFILE_BADTLB,
    COMADMIN_E_COMPFILE_NOTINSTALLABLE,
    COMADMIN_E_NOTCHANGEABLE,
    COMADMIN_E_NOTDELETEABLE,
    COMADMIN_E_SESSION,
    COMADMIN_E_COMP_MOVE_LOCKED,
    COMADMIN_E_COMP_MOVE_BAD_DEST,
    COMADMIN_E_REGISTERTLB,
    COMADMIN_E_SYSTEMAPP,
    COMADMIN_E_COMPFILE_NOREGISTRAR,
    COMADMIN_E_COREQCOMPINSTALLED,
    COMADMIN_E_SERVICENOTINSTALLED,
    COMADMIN_E_PROPERTYSAVEFAILED,
    COMADMIN_E_OBJECTEXISTS,
    COMADMIN_E_COMPONENTEXISTS,
    COMADMIN_E_REGFILE_CORRUPT,
    COMADMIN_E_PROPERTY_OVERFLOW,
    COMADMIN_E_NOTINREGISTRY,
    COMADMIN_E_OBJECTNOTPOOLABLE,
    COMADMIN_E_APPLID_MATCHES_CLSID,
    COMADMIN_E_ROLE_DOES_NOT_EXIST,
    COMADMIN_E_START_APP_NEEDS_COMPONENTS,
    COMADMIN_E_REQUIRES_DIFFERENT_PLATFORM,
    COMADMIN_E_CAN_NOT_EXPORT_APP_PROXY,
    COMADMIN_E_CAN_NOT_START_APP,
    COMADMIN_E_CAN_NOT_EXPORT_SYS_APP,
    COMADMIN_E_CANT_SUBSCRIBE_TO_COMPONENT,
    COMADMIN_E_EVENTCLASS_CANT_BE_SUBSCRIBER,
    COMADMIN_E_LIB_APP_PROXY_INCOMPATIBLE,
    COMADMIN_E_BASE_PARTITION_ONLY,
    COMADMIN_E_START_APP_DISABLED,
    COMADMIN_E_CAT_DUPLICATE_PARTITION_NAME,
    COMADMIN_E_CAT_INVALID_PARTITION_NAME,
    COMADMIN_E_CAT_PARTITION_IN_USE,
    COMADMIN_E_FILE_PARTITION_DUPLICATE_FILES,
    COMADMIN_E_CAT_IMPORTED_COMPONENTS_NOT_ALLOWED,
    COMADMIN_E_AMBIGUOUS_APPLICATION_NAME,
    COMADMIN_E_AMBIGUOUS_PARTITION_NAME,
    COMADMIN_E_REGDB_NOTINITIALIZED,
    COMADMIN_E_REGDB_NOTOPEN,
    COMADMIN_E_REGDB_SYSTEMERR,
    COMADMIN_E_REGDB_ALREADYRUNNING,
    COMADMIN_E_MIG_VERSIONNOTSUPPORTED,
    COMADMIN_E_MIG_SCHEMANOTFOUND,
    COMADMIN_E_CAT_BITNESSMISMATCH,
    COMADMIN_E_CAT_UNACCEPTABLEBITNESS,
    COMADMIN_E_CAT_WRONGAPPBITNESS,
    COMADMIN_E_CAT_PAUSE_RESUME_NOT_SUPPORTED,
    COMADMIN_E_CAT_SERVERFAULT,
    COMQC_E_APPLICATION_NOT_QUEUED,
    COMQC_E_NO_QUEUEABLE_INTERFACES,
    COMQC_E_QUEUING_SERVICE_NOT_AVAILABLE,
    COMQC_E_NO_IPERSISTSTREAM,
    COMQC_E_BAD_MESSAGE,
    COMQC_E_UNAUTHENTICATED,
    COMQC_E_UNTRUSTED_ENQUEUER,
    MSDTC_E_DUPLICATE_RESOURCE,
    COMADMIN_E_OBJECT_PARENT_MISSING,
    COMADMIN_E_OBJECT_DOES_NOT_EXIST,
    COMADMIN_E_APP_NOT_RUNNING,
    COMADMIN_E_INVALID_PARTITION,
    COMADMIN_E_SVCAPP_NOT_POOLABLE_OR_RECYCLABLE,
    COMADMIN_E_USER_IN_SET,
    COMADMIN_E_CANTRECYCLELIBRARYAPPS,
    COMADMIN_E_CANTRECYCLESERVICEAPPS,
    COMADMIN_E_PROCESSALREADYRECYCLED,
    COMADMIN_E_PAUSEDPROCESSMAYNOTBERECYCLED,
    COMADMIN_E_CANTMAKEINPROCSERVICE,
    COMADMIN_E_PROGIDINUSEBYCLSID,
    COMADMIN_E_DEFAULT_PARTITION_NOT_IN_SET,
    COMADMIN_E_RECYCLEDPROCESSMAYNOTBEPAUSED,
    COMADMIN_E_PARTITION_ACCESSDENIED,
    COMADMIN_E_PARTITION_MSI_ONLY,
    COMADMIN_E_LEGACYCOMPS_NOT_ALLOWED_IN_1_0_FORMAT,
    COMADMIN_E_LEGACYCOMPS_NOT_ALLOWED_IN_NONBASE_PARTITIONS,
    COMADMIN_E_COMP_MOVE_SOURCE,
    COMADMIN_E_COMP_MOVE_DEST,
    COMADMIN_E_COMP_MOVE_PRIVATE,
    COMADMIN_E_BASEPARTITION_REQUIRED_IN_SET,
    COMADMIN_E_CANNOT_ALIAS_EVENTCLASS,
    COMADMIN_E_PRIVATE_ACCESSDENIED,
    COMADMIN_E_SAFERINVALID,
    COMADMIN_E_REGISTRY_ACCESSDENIED,
    COMADMIN_E_PARTITIONS_DISABLED,
}

impl COMPLUS {
    pub fn code(&self) -> u32 {
        return match self {
            COMPLUS::COMADMIN_E_OBJECTERRORS => 0x80110401 as u32,
            COMPLUS::COMADMIN_E_OBJECTINVALID => 0x80110402 as u32,
            COMPLUS::COMADMIN_E_KEYMISSING => 0x80110403 as u32,
            COMPLUS::COMADMIN_E_ALREADYINSTALLED => 0x80110404 as u32,
            COMPLUS::COMADMIN_E_APP_FILE_WRITEFAIL => 0x80110407 as u32,
            COMPLUS::COMADMIN_E_APP_FILE_READFAIL => 0x80110408 as u32,
            COMPLUS::COMADMIN_E_APP_FILE_VERSION => 0x80110409 as u32,
            COMPLUS::COMADMIN_E_BADPATH => 0x8011040A as u32,
            COMPLUS::COMADMIN_E_APPLICATIONEXISTS => 0x8011040B as u32,
            COMPLUS::COMADMIN_E_ROLEEXISTS => 0x8011040C as u32,
            COMPLUS::COMADMIN_E_CANTCOPYFILE => 0x8011040D as u32,
            COMPLUS::COMADMIN_E_NOUSER => 0x8011040F as u32,
            COMPLUS::COMADMIN_E_INVALIDUSERIDS => 0x80110410 as u32,
            COMPLUS::COMADMIN_E_NOREGISTRYCLSID => 0x80110411 as u32,
            COMPLUS::COMADMIN_E_BADREGISTRYPROGID => 0x80110412 as u32,
            COMPLUS::COMADMIN_E_AUTHENTICATIONLEVEL => 0x80110413 as u32,
            COMPLUS::COMADMIN_E_USERPASSWDNOTVALID => 0x80110414 as u32,
            COMPLUS::COMADMIN_E_CLSIDORIIDMISMATCH => 0x80110418 as u32,
            COMPLUS::COMADMIN_E_REMOTEINTERFACE => 0x80110419 as u32,
            COMPLUS::COMADMIN_E_DLLREGISTERSERVER => 0x8011041A as u32,
            COMPLUS::COMADMIN_E_NOSERVERSHARE => 0x8011041B as u32,
            COMPLUS::COMADMIN_E_DLLLOADFAILED => 0x8011041D as u32,
            COMPLUS::COMADMIN_E_BADREGISTRYLIBID => 0x8011041E as u32,
            COMPLUS::COMADMIN_E_APPDIRNOTFOUND => 0x8011041F as u32,
            COMPLUS::COMADMIN_E_REGISTRARFAILED => 0x80110423 as u32,
            COMPLUS::COMADMIN_E_COMPFILE_DOESNOTEXIST => 0x80110424 as u32,
            COMPLUS::COMADMIN_E_COMPFILE_LOADDLLFAIL => 0x80110425 as u32,
            COMPLUS::COMADMIN_E_COMPFILE_GETCLASSOBJ => 0x80110426 as u32,
            COMPLUS::COMADMIN_E_COMPFILE_CLASSNOTAVAIL => 0x80110427 as u32,
            COMPLUS::COMADMIN_E_COMPFILE_BADTLB => 0x80110428 as u32,
            COMPLUS::COMADMIN_E_COMPFILE_NOTINSTALLABLE => 0x80110429 as u32,
            COMPLUS::COMADMIN_E_NOTCHANGEABLE => 0x8011042A as u32,
            COMPLUS::COMADMIN_E_NOTDELETEABLE => 0x8011042B as u32,
            COMPLUS::COMADMIN_E_SESSION => 0x8011042C as u32,
            COMPLUS::COMADMIN_E_COMP_MOVE_LOCKED => 0x8011042D as u32,
            COMPLUS::COMADMIN_E_COMP_MOVE_BAD_DEST => 0x8011042E as u32,
            COMPLUS::COMADMIN_E_REGISTERTLB => 0x80110430 as u32,
            COMPLUS::COMADMIN_E_SYSTEMAPP => 0x80110433 as u32,
            COMPLUS::COMADMIN_E_COMPFILE_NOREGISTRAR => 0x80110434 as u32,
            COMPLUS::COMADMIN_E_COREQCOMPINSTALLED => 0x80110435 as u32,
            COMPLUS::COMADMIN_E_SERVICENOTINSTALLED => 0x80110436 as u32,
            COMPLUS::COMADMIN_E_PROPERTYSAVEFAILED => 0x80110437 as u32,
            COMPLUS::COMADMIN_E_OBJECTEXISTS => 0x80110438 as u32,
            COMPLUS::COMADMIN_E_COMPONENTEXISTS => 0x80110439 as u32,
            COMPLUS::COMADMIN_E_REGFILE_CORRUPT => 0x8011043B as u32,
            COMPLUS::COMADMIN_E_PROPERTY_OVERFLOW => 0x8011043C as u32,
            COMPLUS::COMADMIN_E_NOTINREGISTRY => 0x8011043E as u32,
            COMPLUS::COMADMIN_E_OBJECTNOTPOOLABLE => 0x8011043F as u32,
            COMPLUS::COMADMIN_E_APPLID_MATCHES_CLSID => 0x80110446 as u32,
            COMPLUS::COMADMIN_E_ROLE_DOES_NOT_EXIST => 0x80110447 as u32,
            COMPLUS::COMADMIN_E_START_APP_NEEDS_COMPONENTS => 0x80110448 as u32,
            COMPLUS::COMADMIN_E_REQUIRES_DIFFERENT_PLATFORM => 0x80110449 as u32,
            COMPLUS::COMADMIN_E_CAN_NOT_EXPORT_APP_PROXY => 0x8011044A as u32,
            COMPLUS::COMADMIN_E_CAN_NOT_START_APP => 0x8011044B as u32,
            COMPLUS::COMADMIN_E_CAN_NOT_EXPORT_SYS_APP => 0x8011044C as u32,
            COMPLUS::COMADMIN_E_CANT_SUBSCRIBE_TO_COMPONENT => 0x8011044D as u32,
            COMPLUS::COMADMIN_E_EVENTCLASS_CANT_BE_SUBSCRIBER => 0x8011044E as u32,
            COMPLUS::COMADMIN_E_LIB_APP_PROXY_INCOMPATIBLE => 0x8011044F as u32,
            COMPLUS::COMADMIN_E_BASE_PARTITION_ONLY => 0x80110450 as u32,
            COMPLUS::COMADMIN_E_START_APP_DISABLED => 0x80110451 as u32,
            COMPLUS::COMADMIN_E_CAT_DUPLICATE_PARTITION_NAME => 0x80110457 as u32,
            COMPLUS::COMADMIN_E_CAT_INVALID_PARTITION_NAME => 0x80110458 as u32,
            COMPLUS::COMADMIN_E_CAT_PARTITION_IN_USE => 0x80110459 as u32,
            COMPLUS::COMADMIN_E_FILE_PARTITION_DUPLICATE_FILES => 0x8011045A as u32,
            COMPLUS::COMADMIN_E_CAT_IMPORTED_COMPONENTS_NOT_ALLOWED => 0x8011045B as u32,
            COMPLUS::COMADMIN_E_AMBIGUOUS_APPLICATION_NAME => 0x8011045C as u32,
            COMPLUS::COMADMIN_E_AMBIGUOUS_PARTITION_NAME => 0x8011045D as u32,
            COMPLUS::COMADMIN_E_REGDB_NOTINITIALIZED => 0x80110472 as u32,
            COMPLUS::COMADMIN_E_REGDB_NOTOPEN => 0x80110473 as u32,
            COMPLUS::COMADMIN_E_REGDB_SYSTEMERR => 0x80110474 as u32,
            COMPLUS::COMADMIN_E_REGDB_ALREADYRUNNING => 0x80110475 as u32,
            COMPLUS::COMADMIN_E_MIG_VERSIONNOTSUPPORTED => 0x80110480 as u32,
            COMPLUS::COMADMIN_E_MIG_SCHEMANOTFOUND => 0x80110481 as u32,
            COMPLUS::COMADMIN_E_CAT_BITNESSMISMATCH => 0x80110482 as u32,
            COMPLUS::COMADMIN_E_CAT_UNACCEPTABLEBITNESS => 0x80110483 as u32,
            COMPLUS::COMADMIN_E_CAT_WRONGAPPBITNESS => 0x80110484 as u32,
            COMPLUS::COMADMIN_E_CAT_PAUSE_RESUME_NOT_SUPPORTED => 0x80110485 as u32,
            COMPLUS::COMADMIN_E_CAT_SERVERFAULT => 0x80110486 as u32,
            COMPLUS::COMQC_E_APPLICATION_NOT_QUEUED => 0x80110600 as u32,
            COMPLUS::COMQC_E_NO_QUEUEABLE_INTERFACES => 0x80110601 as u32,
            COMPLUS::COMQC_E_QUEUING_SERVICE_NOT_AVAILABLE => 0x80110602 as u32,
            COMPLUS::COMQC_E_NO_IPERSISTSTREAM => 0x80110603 as u32,
            COMPLUS::COMQC_E_BAD_MESSAGE => 0x80110604 as u32,
            COMPLUS::COMQC_E_UNAUTHENTICATED => 0x80110605 as u32,
            COMPLUS::COMQC_E_UNTRUSTED_ENQUEUER => 0x80110606 as u32,
            COMPLUS::MSDTC_E_DUPLICATE_RESOURCE => 0x80110701 as u32,
            COMPLUS::COMADMIN_E_OBJECT_PARENT_MISSING => 0x80110808 as u32,
            COMPLUS::COMADMIN_E_OBJECT_DOES_NOT_EXIST => 0x80110809 as u32,
            COMPLUS::COMADMIN_E_APP_NOT_RUNNING => 0x8011080A as u32,
            COMPLUS::COMADMIN_E_INVALID_PARTITION => 0x8011080B as u32,
            COMPLUS::COMADMIN_E_SVCAPP_NOT_POOLABLE_OR_RECYCLABLE => 0x8011080D as u32,
            COMPLUS::COMADMIN_E_USER_IN_SET => 0x8011080E as u32,
            COMPLUS::COMADMIN_E_CANTRECYCLELIBRARYAPPS => 0x8011080F as u32,
            COMPLUS::COMADMIN_E_CANTRECYCLESERVICEAPPS => 0x80110811 as u32,
            COMPLUS::COMADMIN_E_PROCESSALREADYRECYCLED => 0x80110812 as u32,
            COMPLUS::COMADMIN_E_PAUSEDPROCESSMAYNOTBERECYCLED => 0x80110813 as u32,
            COMPLUS::COMADMIN_E_CANTMAKEINPROCSERVICE => 0x80110814 as u32,
            COMPLUS::COMADMIN_E_PROGIDINUSEBYCLSID => 0x80110815 as u32,
            COMPLUS::COMADMIN_E_DEFAULT_PARTITION_NOT_IN_SET => 0x80110816 as u32,
            COMPLUS::COMADMIN_E_RECYCLEDPROCESSMAYNOTBEPAUSED => 0x80110817 as u32,
            COMPLUS::COMADMIN_E_PARTITION_ACCESSDENIED => 0x80110818 as u32,
            COMPLUS::COMADMIN_E_PARTITION_MSI_ONLY => 0x80110819 as u32,
            COMPLUS::COMADMIN_E_LEGACYCOMPS_NOT_ALLOWED_IN_1_0_FORMAT => 0x8011081A as u32,
            COMPLUS::COMADMIN_E_LEGACYCOMPS_NOT_ALLOWED_IN_NONBASE_PARTITIONS => 0x8011081B as u32,
            COMPLUS::COMADMIN_E_COMP_MOVE_SOURCE => 0x8011081C as u32,
            COMPLUS::COMADMIN_E_COMP_MOVE_DEST => 0x8011081D as u32,
            COMPLUS::COMADMIN_E_COMP_MOVE_PRIVATE => 0x8011081E as u32,
            COMPLUS::COMADMIN_E_BASEPARTITION_REQUIRED_IN_SET => 0x8011081F as u32,
            COMPLUS::COMADMIN_E_CANNOT_ALIAS_EVENTCLASS => 0x80110820 as u32,
            COMPLUS::COMADMIN_E_PRIVATE_ACCESSDENIED => 0x80110821 as u32,
            COMPLUS::COMADMIN_E_SAFERINVALID => 0x80110822 as u32,
            COMPLUS::COMADMIN_E_REGISTRY_ACCESSDENIED => 0x80110823 as u32,
            COMPLUS::COMADMIN_E_PARTITIONS_DISABLED => 0x80110824 as u32,
        }
    }

    pub fn error(&self) -> RawError {
        return match self {
            COMPLUS::COMADMIN_E_OBJECTERRORS => RawError::Kind(COMPLUS::COMADMIN_E_OBJECTERRORS),
            COMPLUS::COMADMIN_E_OBJECTINVALID => RawError::Kind(COMPLUS::COMADMIN_E_OBJECTINVALID),
            COMPLUS::COMADMIN_E_KEYMISSING => RawError::Kind(COMPLUS::COMADMIN_E_KEYMISSING),
            COMPLUS::COMADMIN_E_ALREADYINSTALLED => RawError::Kind(COMPLUS::COMADMIN_E_ALREADYINSTALLED),
            COMPLUS::COMADMIN_E_APP_FILE_WRITEFAIL => RawError::Kind(COMPLUS::COMADMIN_E_APP_FILE_WRITEFAIL),
            COMPLUS::COMADMIN_E_APP_FILE_READFAIL => RawError::Kind(COMPLUS::COMADMIN_E_APP_FILE_READFAIL),
            COMPLUS::COMADMIN_E_APP_FILE_VERSION => RawError::Kind(COMPLUS::COMADMIN_E_APP_FILE_VERSION),
            COMPLUS::COMADMIN_E_BADPATH => RawError::Kind(COMPLUS::COMADMIN_E_BADPATH),
            COMPLUS::COMADMIN_E_APPLICATIONEXISTS => RawError::Kind(COMPLUS::COMADMIN_E_APPLICATIONEXISTS),
            COMPLUS::COMADMIN_E_ROLEEXISTS => RawError::Kind(COMPLUS::COMADMIN_E_ROLEEXISTS),
            COMPLUS::COMADMIN_E_CANTCOPYFILE => RawError::Kind(COMPLUS::COMADMIN_E_CANTCOPYFILE),
            COMPLUS::COMADMIN_E_NOUSER => RawError::Kind(COMPLUS::COMADMIN_E_NOUSER),
            COMPLUS::COMADMIN_E_INVALIDUSERIDS => RawError::Kind(COMPLUS::COMADMIN_E_INVALIDUSERIDS),
            COMPLUS::COMADMIN_E_NOREGISTRYCLSID => RawError::Kind(COMPLUS::COMADMIN_E_NOREGISTRYCLSID),
            COMPLUS::COMADMIN_E_BADREGISTRYPROGID => RawError::Kind(COMPLUS::COMADMIN_E_BADREGISTRYPROGID),
            COMPLUS::COMADMIN_E_AUTHENTICATIONLEVEL => RawError::Kind(COMPLUS::COMADMIN_E_AUTHENTICATIONLEVEL),
            COMPLUS::COMADMIN_E_USERPASSWDNOTVALID => RawError::Kind(COMPLUS::COMADMIN_E_USERPASSWDNOTVALID),
            COMPLUS::COMADMIN_E_CLSIDORIIDMISMATCH => RawError::Kind(COMPLUS::COMADMIN_E_CLSIDORIIDMISMATCH),
            COMPLUS::COMADMIN_E_REMOTEINTERFACE => RawError::Kind(COMPLUS::COMADMIN_E_REMOTEINTERFACE),
            COMPLUS::COMADMIN_E_DLLREGISTERSERVER => RawError::Kind(COMPLUS::COMADMIN_E_DLLREGISTERSERVER),
            COMPLUS::COMADMIN_E_NOSERVERSHARE => RawError::Kind(COMPLUS::COMADMIN_E_NOSERVERSHARE),
            COMPLUS::COMADMIN_E_DLLLOADFAILED => RawError::Kind(COMPLUS::COMADMIN_E_DLLLOADFAILED),
            COMPLUS::COMADMIN_E_BADREGISTRYLIBID => RawError::Kind(COMPLUS::COMADMIN_E_BADREGISTRYLIBID),
            COMPLUS::COMADMIN_E_APPDIRNOTFOUND => RawError::Kind(COMPLUS::COMADMIN_E_APPDIRNOTFOUND),
            COMPLUS::COMADMIN_E_REGISTRARFAILED => RawError::Kind(COMPLUS::COMADMIN_E_REGISTRARFAILED),
            COMPLUS::COMADMIN_E_COMPFILE_DOESNOTEXIST => RawError::Kind(COMPLUS::COMADMIN_E_COMPFILE_DOESNOTEXIST),
            COMPLUS::COMADMIN_E_COMPFILE_LOADDLLFAIL => RawError::Kind(COMPLUS::COMADMIN_E_COMPFILE_LOADDLLFAIL),
            COMPLUS::COMADMIN_E_COMPFILE_GETCLASSOBJ => RawError::Kind(COMPLUS::COMADMIN_E_COMPFILE_GETCLASSOBJ),
            COMPLUS::COMADMIN_E_COMPFILE_CLASSNOTAVAIL => RawError::Kind(COMPLUS::COMADMIN_E_COMPFILE_CLASSNOTAVAIL),
            COMPLUS::COMADMIN_E_COMPFILE_BADTLB => RawError::Kind(COMPLUS::COMADMIN_E_COMPFILE_BADTLB),
            COMPLUS::COMADMIN_E_COMPFILE_NOTINSTALLABLE => RawError::Kind(COMPLUS::COMADMIN_E_COMPFILE_NOTINSTALLABLE),
            COMPLUS::COMADMIN_E_NOTCHANGEABLE => RawError::Kind(COMPLUS::COMADMIN_E_NOTCHANGEABLE),
            COMPLUS::COMADMIN_E_NOTDELETEABLE => RawError::Kind(COMPLUS::COMADMIN_E_NOTDELETEABLE),
            COMPLUS::COMADMIN_E_SESSION => RawError::Kind(COMPLUS::COMADMIN_E_SESSION),
            COMPLUS::COMADMIN_E_COMP_MOVE_LOCKED => RawError::Kind(COMPLUS::COMADMIN_E_COMP_MOVE_LOCKED),
            COMPLUS::COMADMIN_E_COMP_MOVE_BAD_DEST => RawError::Kind(COMPLUS::COMADMIN_E_COMP_MOVE_BAD_DEST),
            COMPLUS::COMADMIN_E_REGISTERTLB => RawError::Kind(COMPLUS::COMADMIN_E_REGISTERTLB),
            COMPLUS::COMADMIN_E_SYSTEMAPP => RawError::Kind(COMPLUS::COMADMIN_E_SYSTEMAPP),
            COMPLUS::COMADMIN_E_COMPFILE_NOREGISTRAR => RawError::Kind(COMPLUS::COMADMIN_E_COMPFILE_NOREGISTRAR),
            COMPLUS::COMADMIN_E_COREQCOMPINSTALLED => RawError::Kind(COMPLUS::COMADMIN_E_COREQCOMPINSTALLED),
            COMPLUS::COMADMIN_E_SERVICENOTINSTALLED => RawError::Kind(COMPLUS::COMADMIN_E_SERVICENOTINSTALLED),
            COMPLUS::COMADMIN_E_PROPERTYSAVEFAILED => RawError::Kind(COMPLUS::COMADMIN_E_PROPERTYSAVEFAILED),
            COMPLUS::COMADMIN_E_OBJECTEXISTS => RawError::Kind(COMPLUS::COMADMIN_E_OBJECTEXISTS),
            COMPLUS::COMADMIN_E_COMPONENTEXISTS => RawError::Kind(COMPLUS::COMADMIN_E_COMPONENTEXISTS),
            COMPLUS::COMADMIN_E_REGFILE_CORRUPT => RawError::Kind(COMPLUS::COMADMIN_E_REGFILE_CORRUPT),
            COMPLUS::COMADMIN_E_PROPERTY_OVERFLOW => RawError::Kind(COMPLUS::COMADMIN_E_PROPERTY_OVERFLOW),
            COMPLUS::COMADMIN_E_NOTINREGISTRY => RawError::Kind(COMPLUS::COMADMIN_E_NOTINREGISTRY),
            COMPLUS::COMADMIN_E_OBJECTNOTPOOLABLE => RawError::Kind(COMPLUS::COMADMIN_E_OBJECTNOTPOOLABLE),
            COMPLUS::COMADMIN_E_APPLID_MATCHES_CLSID => RawError::Kind(COMPLUS::COMADMIN_E_APPLID_MATCHES_CLSID),
            COMPLUS::COMADMIN_E_ROLE_DOES_NOT_EXIST => RawError::Kind(COMPLUS::COMADMIN_E_ROLE_DOES_NOT_EXIST),
            COMPLUS::COMADMIN_E_START_APP_NEEDS_COMPONENTS => RawError::Kind(COMPLUS::COMADMIN_E_START_APP_NEEDS_COMPONENTS),
            COMPLUS::COMADMIN_E_REQUIRES_DIFFERENT_PLATFORM => RawError::Kind(COMPLUS::COMADMIN_E_REQUIRES_DIFFERENT_PLATFORM),
            COMPLUS::COMADMIN_E_CAN_NOT_EXPORT_APP_PROXY => RawError::Kind(COMPLUS::COMADMIN_E_CAN_NOT_EXPORT_APP_PROXY),
            COMPLUS::COMADMIN_E_CAN_NOT_START_APP => RawError::Kind(COMPLUS::COMADMIN_E_CAN_NOT_START_APP),
            COMPLUS::COMADMIN_E_CAN_NOT_EXPORT_SYS_APP => RawError::Kind(COMPLUS::COMADMIN_E_CAN_NOT_EXPORT_SYS_APP),
            COMPLUS::COMADMIN_E_CANT_SUBSCRIBE_TO_COMPONENT => RawError::Kind(COMPLUS::COMADMIN_E_CANT_SUBSCRIBE_TO_COMPONENT),
            COMPLUS::COMADMIN_E_EVENTCLASS_CANT_BE_SUBSCRIBER => RawError::Kind(COMPLUS::COMADMIN_E_EVENTCLASS_CANT_BE_SUBSCRIBER),
            COMPLUS::COMADMIN_E_LIB_APP_PROXY_INCOMPATIBLE => RawError::Kind(COMPLUS::COMADMIN_E_LIB_APP_PROXY_INCOMPATIBLE),
            COMPLUS::COMADMIN_E_BASE_PARTITION_ONLY => RawError::Kind(COMPLUS::COMADMIN_E_BASE_PARTITION_ONLY),
            COMPLUS::COMADMIN_E_START_APP_DISABLED => RawError::Kind(COMPLUS::COMADMIN_E_START_APP_DISABLED),
            COMPLUS::COMADMIN_E_CAT_DUPLICATE_PARTITION_NAME => RawError::Kind(COMPLUS::COMADMIN_E_CAT_DUPLICATE_PARTITION_NAME),
            COMPLUS::COMADMIN_E_CAT_INVALID_PARTITION_NAME => RawError::Kind(COMPLUS::COMADMIN_E_CAT_INVALID_PARTITION_NAME),
            COMPLUS::COMADMIN_E_CAT_PARTITION_IN_USE => RawError::Kind(COMPLUS::COMADMIN_E_CAT_PARTITION_IN_USE),
            COMPLUS::COMADMIN_E_FILE_PARTITION_DUPLICATE_FILES => RawError::Kind(COMPLUS::COMADMIN_E_FILE_PARTITION_DUPLICATE_FILES),
            COMPLUS::COMADMIN_E_CAT_IMPORTED_COMPONENTS_NOT_ALLOWED => RawError::Kind(COMPLUS::COMADMIN_E_CAT_IMPORTED_COMPONENTS_NOT_ALLOWED),
            COMPLUS::COMADMIN_E_AMBIGUOUS_APPLICATION_NAME => RawError::Kind(COMPLUS::COMADMIN_E_AMBIGUOUS_APPLICATION_NAME),
            COMPLUS::COMADMIN_E_AMBIGUOUS_PARTITION_NAME => RawError::Kind(COMPLUS::COMADMIN_E_AMBIGUOUS_PARTITION_NAME),
            COMPLUS::COMADMIN_E_REGDB_NOTINITIALIZED => RawError::Kind(COMPLUS::COMADMIN_E_REGDB_NOTINITIALIZED),
            COMPLUS::COMADMIN_E_REGDB_NOTOPEN => RawError::Kind(COMPLUS::COMADMIN_E_REGDB_NOTOPEN),
            COMPLUS::COMADMIN_E_REGDB_SYSTEMERR => RawError::Kind(COMPLUS::COMADMIN_E_REGDB_SYSTEMERR),
            COMPLUS::COMADMIN_E_REGDB_ALREADYRUNNING => RawError::Kind(COMPLUS::COMADMIN_E_REGDB_ALREADYRUNNING),
            COMPLUS::COMADMIN_E_MIG_VERSIONNOTSUPPORTED => RawError::Kind(COMPLUS::COMADMIN_E_MIG_VERSIONNOTSUPPORTED),
            COMPLUS::COMADMIN_E_MIG_SCHEMANOTFOUND => RawError::Kind(COMPLUS::COMADMIN_E_MIG_SCHEMANOTFOUND),
            COMPLUS::COMADMIN_E_CAT_BITNESSMISMATCH => RawError::Kind(COMPLUS::COMADMIN_E_CAT_BITNESSMISMATCH),
            COMPLUS::COMADMIN_E_CAT_UNACCEPTABLEBITNESS => RawError::Kind(COMPLUS::COMADMIN_E_CAT_UNACCEPTABLEBITNESS),
            COMPLUS::COMADMIN_E_CAT_WRONGAPPBITNESS => RawError::Kind(COMPLUS::COMADMIN_E_CAT_WRONGAPPBITNESS),
            COMPLUS::COMADMIN_E_CAT_PAUSE_RESUME_NOT_SUPPORTED => RawError::Kind(COMPLUS::COMADMIN_E_CAT_PAUSE_RESUME_NOT_SUPPORTED),
            COMPLUS::COMADMIN_E_CAT_SERVERFAULT => RawError::Kind(COMPLUS::COMADMIN_E_CAT_SERVERFAULT),
            COMPLUS::COMQC_E_APPLICATION_NOT_QUEUED => RawError::Kind(COMPLUS::COMQC_E_APPLICATION_NOT_QUEUED),
            COMPLUS::COMQC_E_NO_QUEUEABLE_INTERFACES => RawError::Kind(COMPLUS::COMQC_E_NO_QUEUEABLE_INTERFACES),
            COMPLUS::COMQC_E_QUEUING_SERVICE_NOT_AVAILABLE => RawError::Kind(COMPLUS::COMQC_E_QUEUING_SERVICE_NOT_AVAILABLE),
            COMPLUS::COMQC_E_NO_IPERSISTSTREAM => RawError::Kind(COMPLUS::COMQC_E_NO_IPERSISTSTREAM),
            COMPLUS::COMQC_E_BAD_MESSAGE => RawError::Kind(COMPLUS::COMQC_E_BAD_MESSAGE),
            COMPLUS::COMQC_E_UNAUTHENTICATED => RawError::Kind(COMPLUS::COMQC_E_UNAUTHENTICATED),
            COMPLUS::COMQC_E_UNTRUSTED_ENQUEUER => RawError::Kind(COMPLUS::COMQC_E_UNTRUSTED_ENQUEUER),
            COMPLUS::MSDTC_E_DUPLICATE_RESOURCE => RawError::Kind(COMPLUS::MSDTC_E_DUPLICATE_RESOURCE),
            COMPLUS::COMADMIN_E_OBJECT_PARENT_MISSING => RawError::Kind(COMPLUS::COMADMIN_E_OBJECT_PARENT_MISSING),
            COMPLUS::COMADMIN_E_OBJECT_DOES_NOT_EXIST => RawError::Kind(COMPLUS::COMADMIN_E_OBJECT_DOES_NOT_EXIST),
            COMPLUS::COMADMIN_E_APP_NOT_RUNNING => RawError::Kind(COMPLUS::COMADMIN_E_APP_NOT_RUNNING),
            COMPLUS::COMADMIN_E_INVALID_PARTITION => RawError::Kind(COMPLUS::COMADMIN_E_INVALID_PARTITION),
            COMPLUS::COMADMIN_E_SVCAPP_NOT_POOLABLE_OR_RECYCLABLE => RawError::Kind(COMPLUS::COMADMIN_E_SVCAPP_NOT_POOLABLE_OR_RECYCLABLE),
            COMPLUS::COMADMIN_E_USER_IN_SET => RawError::Kind(COMPLUS::COMADMIN_E_USER_IN_SET),
            COMPLUS::COMADMIN_E_CANTRECYCLELIBRARYAPPS => RawError::Kind(COMPLUS::COMADMIN_E_CANTRECYCLELIBRARYAPPS),
            COMPLUS::COMADMIN_E_CANTRECYCLESERVICEAPPS => RawError::Kind(COMPLUS::COMADMIN_E_CANTRECYCLESERVICEAPPS),
            COMPLUS::COMADMIN_E_PROCESSALREADYRECYCLED => RawError::Kind(COMPLUS::COMADMIN_E_PROCESSALREADYRECYCLED),
            COMPLUS::COMADMIN_E_PAUSEDPROCESSMAYNOTBERECYCLED => RawError::Kind(COMPLUS::COMADMIN_E_PAUSEDPROCESSMAYNOTBERECYCLED),
            COMPLUS::COMADMIN_E_CANTMAKEINPROCSERVICE => RawError::Kind(COMPLUS::COMADMIN_E_CANTMAKEINPROCSERVICE),
            COMPLUS::COMADMIN_E_PROGIDINUSEBYCLSID => RawError::Kind(COMPLUS::COMADMIN_E_PROGIDINUSEBYCLSID),
            COMPLUS::COMADMIN_E_DEFAULT_PARTITION_NOT_IN_SET => RawError::Kind(COMPLUS::COMADMIN_E_DEFAULT_PARTITION_NOT_IN_SET),
            COMPLUS::COMADMIN_E_RECYCLEDPROCESSMAYNOTBEPAUSED => RawError::Kind(COMPLUS::COMADMIN_E_RECYCLEDPROCESSMAYNOTBEPAUSED),
            COMPLUS::COMADMIN_E_PARTITION_ACCESSDENIED => RawError::Kind(COMPLUS::COMADMIN_E_PARTITION_ACCESSDENIED),
            COMPLUS::COMADMIN_E_PARTITION_MSI_ONLY => RawError::Kind(COMPLUS::COMADMIN_E_PARTITION_MSI_ONLY),
            COMPLUS::COMADMIN_E_LEGACYCOMPS_NOT_ALLOWED_IN_1_0_FORMAT => RawError::Kind(COMPLUS::COMADMIN_E_LEGACYCOMPS_NOT_ALLOWED_IN_1_0_FORMAT),
            COMPLUS::COMADMIN_E_LEGACYCOMPS_NOT_ALLOWED_IN_NONBASE_PARTITIONS => RawError::Kind(COMPLUS::COMADMIN_E_LEGACYCOMPS_NOT_ALLOWED_IN_NONBASE_PARTITIONS),
            COMPLUS::COMADMIN_E_COMP_MOVE_SOURCE => RawError::Kind(COMPLUS::COMADMIN_E_COMP_MOVE_SOURCE),
            COMPLUS::COMADMIN_E_COMP_MOVE_DEST => RawError::Kind(COMPLUS::COMADMIN_E_COMP_MOVE_DEST),
            COMPLUS::COMADMIN_E_COMP_MOVE_PRIVATE => RawError::Kind(COMPLUS::COMADMIN_E_COMP_MOVE_PRIVATE),
            COMPLUS::COMADMIN_E_BASEPARTITION_REQUIRED_IN_SET => RawError::Kind(COMPLUS::COMADMIN_E_BASEPARTITION_REQUIRED_IN_SET),
            COMPLUS::COMADMIN_E_CANNOT_ALIAS_EVENTCLASS => RawError::Kind(COMPLUS::COMADMIN_E_CANNOT_ALIAS_EVENTCLASS),
            COMPLUS::COMADMIN_E_PRIVATE_ACCESSDENIED => RawError::Kind(COMPLUS::COMADMIN_E_PRIVATE_ACCESSDENIED),
            COMPLUS::COMADMIN_E_SAFERINVALID => RawError::Kind(COMPLUS::COMADMIN_E_SAFERINVALID),
            COMPLUS::COMADMIN_E_REGISTRY_ACCESSDENIED => RawError::Kind(COMPLUS::COMADMIN_E_REGISTRY_ACCESSDENIED),
            COMPLUS::COMADMIN_E_PARTITIONS_DISABLED => RawError::Kind(COMPLUS::COMADMIN_E_PARTITIONS_DISABLED),
        }
    }

    pub fn description(&self) -> &'static str {
        return match self {
            COMPLUS::COMADMIN_E_OBJECTERRORS => "Errors occurred accessing one or more objects - the ErrorInfo collection may have more detail",
            COMPLUS::COMADMIN_E_OBJECTINVALID => "One or more of the object's properties are missing or invalid",
            COMPLUS::COMADMIN_E_KEYMISSING => "The object was not found in the catalog",
            COMPLUS::COMADMIN_E_ALREADYINSTALLED => "The object is already registered",
            COMPLUS::COMADMIN_E_APP_FILE_WRITEFAIL => "Error occurred writing to the application file",
            COMPLUS::COMADMIN_E_APP_FILE_READFAIL => "Error occurred reading the application file",
            COMPLUS::COMADMIN_E_APP_FILE_VERSION => "Invalid version number in application file",
            COMPLUS::COMADMIN_E_BADPATH => "The file path is invalid",
            COMPLUS::COMADMIN_E_APPLICATIONEXISTS => "The application is already installed",
            COMPLUS::COMADMIN_E_ROLEEXISTS => "The role already exists",
            COMPLUS::COMADMIN_E_CANTCOPYFILE => "An error occurred copying the file",
            COMPLUS::COMADMIN_E_NOUSER => "One or more users are not valid",
            COMPLUS::COMADMIN_E_INVALIDUSERIDS => "One or more users in the application file are not valid",
            COMPLUS::COMADMIN_E_NOREGISTRYCLSID => "The component's CLSID is missing or corrupt",
            COMPLUS::COMADMIN_E_BADREGISTRYPROGID => "The component's progID is missing or corrupt",
            COMPLUS::COMADMIN_E_AUTHENTICATIONLEVEL => "Unable to set required authentication level for update request",
            COMPLUS::COMADMIN_E_USERPASSWDNOTVALID => "The identity or password set on the application is not valid",
            COMPLUS::COMADMIN_E_CLSIDORIIDMISMATCH => "Application file CLSIDs or IIDs do not match corresponding DLLs",
            COMPLUS::COMADMIN_E_REMOTEINTERFACE => "Interface information is either missing or changed",
            COMPLUS::COMADMIN_E_DLLREGISTERSERVER => "DllRegisterServer failed on component install",
            COMPLUS::COMADMIN_E_NOSERVERSHARE => "No server file share available",
            COMPLUS::COMADMIN_E_DLLLOADFAILED => "DLL could not be loaded",
            COMPLUS::COMADMIN_E_BADREGISTRYLIBID => "The registered TypeLib ID is not valid",
            COMPLUS::COMADMIN_E_APPDIRNOTFOUND => "Application install directory not found",
            COMPLUS::COMADMIN_E_REGISTRARFAILED => "Errors occurred while in the component registrar",
            COMPLUS::COMADMIN_E_COMPFILE_DOESNOTEXIST => "The file does not exist",
            COMPLUS::COMADMIN_E_COMPFILE_LOADDLLFAIL => "The DLL could not be loaded",
            COMPLUS::COMADMIN_E_COMPFILE_GETCLASSOBJ => "GetClassObject failed in the DLL",
            COMPLUS::COMADMIN_E_COMPFILE_CLASSNOTAVAIL => "The DLL does not support the components listed in the TypeLib",
            COMPLUS::COMADMIN_E_COMPFILE_BADTLB => "The TypeLib could not be loaded",
            COMPLUS::COMADMIN_E_COMPFILE_NOTINSTALLABLE => "The file does not contain components or component information",
            COMPLUS::COMADMIN_E_NOTCHANGEABLE => "Changes to this object and its sub-objects have been disabled",
            COMPLUS::COMADMIN_E_NOTDELETEABLE => "The delete function has been disabled for this object",
            COMPLUS::COMADMIN_E_SESSION => "The server catalog version is not supported",
            COMPLUS::COMADMIN_E_COMP_MOVE_LOCKED => "The component move was disallowed, because the source or destination application is either a system application or currently locked against changes",
            COMPLUS::COMADMIN_E_COMP_MOVE_BAD_DEST => "The component move failed because the destination application no longer exists",
            COMPLUS::COMADMIN_E_REGISTERTLB => "The system was unable to register the TypeLib",
            COMPLUS::COMADMIN_E_SYSTEMAPP => "This operation cannot be performed on the system application",
            COMPLUS::COMADMIN_E_COMPFILE_NOREGISTRAR => "The component registrar referenced in this file is not available",
            COMPLUS::COMADMIN_E_COREQCOMPINSTALLED => "A component in the same DLL is already installed",
            COMPLUS::COMADMIN_E_SERVICENOTINSTALLED => "The service is not installed",
            COMPLUS::COMADMIN_E_PROPERTYSAVEFAILED => "One or more property settings are either invalid or in conflict with each other",
            COMPLUS::COMADMIN_E_OBJECTEXISTS => "The object you are attempting to add or rename already exists",
            COMPLUS::COMADMIN_E_COMPONENTEXISTS => "The component already exists",
            COMPLUS::COMADMIN_E_REGFILE_CORRUPT => "The registration file is corrupt",
            COMPLUS::COMADMIN_E_PROPERTY_OVERFLOW => "The property value is too large",
            COMPLUS::COMADMIN_E_NOTINREGISTRY => "Object was not found in registry",
            COMPLUS::COMADMIN_E_OBJECTNOTPOOLABLE => "This object is not poolable",
            COMPLUS::COMADMIN_E_APPLID_MATCHES_CLSID => "A CLSID with the same GUID as the new application ID is already installed on this machine",
            COMPLUS::COMADMIN_E_ROLE_DOES_NOT_EXIST => "A role assigned to a component, interface, or method did not exist in the application",
            COMPLUS::COMADMIN_E_START_APP_NEEDS_COMPONENTS => "You must have components in an application in order to start the application",
            COMPLUS::COMADMIN_E_REQUIRES_DIFFERENT_PLATFORM => "This operation is not enabled on this platform",
            COMPLUS::COMADMIN_E_CAN_NOT_EXPORT_APP_PROXY => "Application Proxy is not exportable",
            COMPLUS::COMADMIN_E_CAN_NOT_START_APP => "Failed to start application because it is either a library application or an application proxy",
            COMPLUS::COMADMIN_E_CAN_NOT_EXPORT_SYS_APP => "System application is not exportable",
            COMPLUS::COMADMIN_E_CANT_SUBSCRIBE_TO_COMPONENT => "Cannot subscribe to this component (the component may have been imported)",
            COMPLUS::COMADMIN_E_EVENTCLASS_CANT_BE_SUBSCRIBER => "An event class cannot also be a subscriber component",
            COMPLUS::COMADMIN_E_LIB_APP_PROXY_INCOMPATIBLE => "Library applications and application proxies are incompatible",
            COMPLUS::COMADMIN_E_BASE_PARTITION_ONLY => "This function is valid for the base partition only",
            COMPLUS::COMADMIN_E_START_APP_DISABLED => "You cannot start an application that has been disabled",
            COMPLUS::COMADMIN_E_CAT_DUPLICATE_PARTITION_NAME => "The specified partition name is already in use on this computer",
            COMPLUS::COMADMIN_E_CAT_INVALID_PARTITION_NAME => "The specified partition name is invalid. Check that the name contains at least one visible character",
            COMPLUS::COMADMIN_E_CAT_PARTITION_IN_USE => "The partition cannot be deleted because it is the default partition for one or more users",
            COMPLUS::COMADMIN_E_FILE_PARTITION_DUPLICATE_FILES => "The partition cannot be exported, because one or more components in the partition have the same file name",
            COMPLUS::COMADMIN_E_CAT_IMPORTED_COMPONENTS_NOT_ALLOWED => "Applications that contain one or more imported components cannot be installed into a non-base partition",
            COMPLUS::COMADMIN_E_AMBIGUOUS_APPLICATION_NAME => "The application name is not unique and cannot be resolved to an application id",
            COMPLUS::COMADMIN_E_AMBIGUOUS_PARTITION_NAME => "The partition name is not unique and cannot be resolved to a partition id",
            COMPLUS::COMADMIN_E_REGDB_NOTINITIALIZED => "The COM+ registry database has not been initialized",
            COMPLUS::COMADMIN_E_REGDB_NOTOPEN => "The COM+ registry database is not open",
            COMPLUS::COMADMIN_E_REGDB_SYSTEMERR => "The COM+ registry database detected a system error",
            COMPLUS::COMADMIN_E_REGDB_ALREADYRUNNING => "The COM+ registry database is already running",
            COMPLUS::COMADMIN_E_MIG_VERSIONNOTSUPPORTED => "This version of the COM+ registry database cannot be migrated",
            COMPLUS::COMADMIN_E_MIG_SCHEMANOTFOUND => "The schema version to be migrated could not be found in the COM+ registry database",
            COMPLUS::COMADMIN_E_CAT_BITNESSMISMATCH => "There was a type mismatch between binaries",
            COMPLUS::COMADMIN_E_CAT_UNACCEPTABLEBITNESS => "A binary of unknown or invalid type was provided",
            COMPLUS::COMADMIN_E_CAT_WRONGAPPBITNESS => "There was a type mismatch between a binary and an application",
            COMPLUS::COMADMIN_E_CAT_PAUSE_RESUME_NOT_SUPPORTED => "The application cannot be paused or resumed",
            COMPLUS::COMADMIN_E_CAT_SERVERFAULT => "The COM+ Catalog Server threw an exception during execution",
            COMPLUS::COMQC_E_APPLICATION_NOT_QUEUED => "Only COM+ Applications marked 'queued' can be invoked using the 'queue' moniker",
            COMPLUS::COMQC_E_NO_QUEUEABLE_INTERFACES => "At least one interface must be marked 'queued' in order to create a queued component instance with the 'queue' moniker",
            COMPLUS::COMQC_E_QUEUING_SERVICE_NOT_AVAILABLE => "MSMQ is required for the requested operation and is not installed",
            COMPLUS::COMQC_E_NO_IPERSISTSTREAM => "Unable to marshal an interface that does not support IPersistStream",
            COMPLUS::COMQC_E_BAD_MESSAGE => "The message is improperly formatted or was damaged in transit",
            COMPLUS::COMQC_E_UNAUTHENTICATED => "An unauthenticated message was received by an application that accepts only authenticated messages",
            COMPLUS::COMQC_E_UNTRUSTED_ENQUEUER => "The message was requeued or moved by a user not in the 'QC Trusted User' role",
            COMPLUS::MSDTC_E_DUPLICATE_RESOURCE => "Cannot create a duplicate resource of type Distributed Transaction Coordinator",
            COMPLUS::COMADMIN_E_OBJECT_PARENT_MISSING => "One of the objects being inserted or updated does not belong to a valid parent collection",
            COMPLUS::COMADMIN_E_OBJECT_DOES_NOT_EXIST => "One of the specified objects cannot be found",
            COMPLUS::COMADMIN_E_APP_NOT_RUNNING => "The specified application is not currently running",
            COMPLUS::COMADMIN_E_INVALID_PARTITION => "The partition(s) specified are not valid.",
            COMPLUS::COMADMIN_E_SVCAPP_NOT_POOLABLE_OR_RECYCLABLE => "COM+ applications that run as NT service may not be pooled or recycled",
            COMPLUS::COMADMIN_E_USER_IN_SET => "One or more users are already assigned to a local partition set.",
            COMPLUS::COMADMIN_E_CANTRECYCLELIBRARYAPPS => "Library applications may not be recycled.",
            COMPLUS::COMADMIN_E_CANTRECYCLESERVICEAPPS => "Applications running as NT services may not be recycled.",
            COMPLUS::COMADMIN_E_PROCESSALREADYRECYCLED => "The process has already been recycled.",
            COMPLUS::COMADMIN_E_PAUSEDPROCESSMAYNOTBERECYCLED => "A paused process may not be recycled.",
            COMPLUS::COMADMIN_E_CANTMAKEINPROCSERVICE => "Library applications may not be NT services.",
            COMPLUS::COMADMIN_E_PROGIDINUSEBYCLSID => "The ProgID provided to the copy operation is invalid. The ProgID is in use by another registered CLSID.",
            COMPLUS::COMADMIN_E_DEFAULT_PARTITION_NOT_IN_SET => "The partition specified as default is not a member of the partition set.",
            COMPLUS::COMADMIN_E_RECYCLEDPROCESSMAYNOTBEPAUSED => "A recycled process may not be paused.",
            COMPLUS::COMADMIN_E_PARTITION_ACCESSDENIED => "Access to the specified partition is denied.",
            COMPLUS::COMADMIN_E_PARTITION_MSI_ONLY => "Only Application Files (*.MSI files) can be installed into partitions.",
            COMPLUS::COMADMIN_E_LEGACYCOMPS_NOT_ALLOWED_IN_1_0_FORMAT => "Applications containing one or more legacy components may not be exported to 1.0 format.",
            COMPLUS::COMADMIN_E_LEGACYCOMPS_NOT_ALLOWED_IN_NONBASE_PARTITIONS => "Legacy components may not exist in non-base partitions.",
            COMPLUS::COMADMIN_E_COMP_MOVE_SOURCE => "A component cannot be moved (or copied) from the System Application, an application proxy or a non-changeable application",
            COMPLUS::COMADMIN_E_COMP_MOVE_DEST => "A component cannot be moved (or copied) to the System Application, an application proxy or a non-changeable application",
            COMPLUS::COMADMIN_E_COMP_MOVE_PRIVATE => "A private component cannot be moved (or copied) to a library application or to the base partition",
            COMPLUS::COMADMIN_E_BASEPARTITION_REQUIRED_IN_SET => "The Base Application Partition exists in all partition sets and cannot be removed.",
            COMPLUS::COMADMIN_E_CANNOT_ALIAS_EVENTCLASS => "Alas, Event Class components cannot be aliased.",
            COMPLUS::COMADMIN_E_PRIVATE_ACCESSDENIED => "Access is denied because the component is private.",
            COMPLUS::COMADMIN_E_SAFERINVALID => "The specified SAFER level is invalid.",
            COMPLUS::COMADMIN_E_REGISTRY_ACCESSDENIED => "The specified user cannot write to the system registry",
            COMPLUS::COMADMIN_E_PARTITIONS_DISABLED => "COM+ partitions are currently disabled.",
        }
    }

    pub fn from_name(name: &str) -> COMPLUS {
        return match name {
            "COMADMIN_E_OBJECTERRORS" => COMPLUS::COMADMIN_E_OBJECTERRORS,
            "COMADMIN_E_OBJECTINVALID" => COMPLUS::COMADMIN_E_OBJECTINVALID,
            "COMADMIN_E_KEYMISSING" => COMPLUS::COMADMIN_E_KEYMISSING,
            "COMADMIN_E_ALREADYINSTALLED" => COMPLUS::COMADMIN_E_ALREADYINSTALLED,
            "COMADMIN_E_APP_FILE_WRITEFAIL" => COMPLUS::COMADMIN_E_APP_FILE_WRITEFAIL,
            "COMADMIN_E_APP_FILE_READFAIL" => COMPLUS::COMADMIN_E_APP_FILE_READFAIL,
            "COMADMIN_E_APP_FILE_VERSION" => COMPLUS::COMADMIN_E_APP_FILE_VERSION,
            "COMADMIN_E_BADPATH" => COMPLUS::COMADMIN_E_BADPATH,
            "COMADMIN_E_APPLICATIONEXISTS" => COMPLUS::COMADMIN_E_APPLICATIONEXISTS,
            "COMADMIN_E_ROLEEXISTS" => COMPLUS::COMADMIN_E_ROLEEXISTS,
            "COMADMIN_E_CANTCOPYFILE" => COMPLUS::COMADMIN_E_CANTCOPYFILE,
            "COMADMIN_E_NOUSER" => COMPLUS::COMADMIN_E_NOUSER,
            "COMADMIN_E_INVALIDUSERIDS" => COMPLUS::COMADMIN_E_INVALIDUSERIDS,
            "COMADMIN_E_NOREGISTRYCLSID" => COMPLUS::COMADMIN_E_NOREGISTRYCLSID,
            "COMADMIN_E_BADREGISTRYPROGID" => COMPLUS::COMADMIN_E_BADREGISTRYPROGID,
            "COMADMIN_E_AUTHENTICATIONLEVEL" => COMPLUS::COMADMIN_E_AUTHENTICATIONLEVEL,
            "COMADMIN_E_USERPASSWDNOTVALID" => COMPLUS::COMADMIN_E_USERPASSWDNOTVALID,
            "COMADMIN_E_CLSIDORIIDMISMATCH" => COMPLUS::COMADMIN_E_CLSIDORIIDMISMATCH,
            "COMADMIN_E_REMOTEINTERFACE" => COMPLUS::COMADMIN_E_REMOTEINTERFACE,
            "COMADMIN_E_DLLREGISTERSERVER" => COMPLUS::COMADMIN_E_DLLREGISTERSERVER,
            "COMADMIN_E_NOSERVERSHARE" => COMPLUS::COMADMIN_E_NOSERVERSHARE,
            "COMADMIN_E_DLLLOADFAILED" => COMPLUS::COMADMIN_E_DLLLOADFAILED,
            "COMADMIN_E_BADREGISTRYLIBID" => COMPLUS::COMADMIN_E_BADREGISTRYLIBID,
            "COMADMIN_E_APPDIRNOTFOUND" => COMPLUS::COMADMIN_E_APPDIRNOTFOUND,
            "COMADMIN_E_REGISTRARFAILED" => COMPLUS::COMADMIN_E_REGISTRARFAILED,
            "COMADMIN_E_COMPFILE_DOESNOTEXIST" => COMPLUS::COMADMIN_E_COMPFILE_DOESNOTEXIST,
            "COMADMIN_E_COMPFILE_LOADDLLFAIL" => COMPLUS::COMADMIN_E_COMPFILE_LOADDLLFAIL,
            "COMADMIN_E_COMPFILE_GETCLASSOBJ" => COMPLUS::COMADMIN_E_COMPFILE_GETCLASSOBJ,
            "COMADMIN_E_COMPFILE_CLASSNOTAVAIL" => COMPLUS::COMADMIN_E_COMPFILE_CLASSNOTAVAIL,
            "COMADMIN_E_COMPFILE_BADTLB" => COMPLUS::COMADMIN_E_COMPFILE_BADTLB,
            "COMADMIN_E_COMPFILE_NOTINSTALLABLE" => COMPLUS::COMADMIN_E_COMPFILE_NOTINSTALLABLE,
            "COMADMIN_E_NOTCHANGEABLE" => COMPLUS::COMADMIN_E_NOTCHANGEABLE,
            "COMADMIN_E_NOTDELETEABLE" => COMPLUS::COMADMIN_E_NOTDELETEABLE,
            "COMADMIN_E_SESSION" => COMPLUS::COMADMIN_E_SESSION,
            "COMADMIN_E_COMP_MOVE_LOCKED" => COMPLUS::COMADMIN_E_COMP_MOVE_LOCKED,
            "COMADMIN_E_COMP_MOVE_BAD_DEST" => COMPLUS::COMADMIN_E_COMP_MOVE_BAD_DEST,
            "COMADMIN_E_REGISTERTLB" => COMPLUS::COMADMIN_E_REGISTERTLB,
            "COMADMIN_E_SYSTEMAPP" => COMPLUS::COMADMIN_E_SYSTEMAPP,
            "COMADMIN_E_COMPFILE_NOREGISTRAR" => COMPLUS::COMADMIN_E_COMPFILE_NOREGISTRAR,
            "COMADMIN_E_COREQCOMPINSTALLED" => COMPLUS::COMADMIN_E_COREQCOMPINSTALLED,
            "COMADMIN_E_SERVICENOTINSTALLED" => COMPLUS::COMADMIN_E_SERVICENOTINSTALLED,
            "COMADMIN_E_PROPERTYSAVEFAILED" => COMPLUS::COMADMIN_E_PROPERTYSAVEFAILED,
            "COMADMIN_E_OBJECTEXISTS" => COMPLUS::COMADMIN_E_OBJECTEXISTS,
            "COMADMIN_E_COMPONENTEXISTS" => COMPLUS::COMADMIN_E_COMPONENTEXISTS,
            "COMADMIN_E_REGFILE_CORRUPT" => COMPLUS::COMADMIN_E_REGFILE_CORRUPT,
            "COMADMIN_E_PROPERTY_OVERFLOW" => COMPLUS::COMADMIN_E_PROPERTY_OVERFLOW,
            "COMADMIN_E_NOTINREGISTRY" => COMPLUS::COMADMIN_E_NOTINREGISTRY,
            "COMADMIN_E_OBJECTNOTPOOLABLE" => COMPLUS::COMADMIN_E_OBJECTNOTPOOLABLE,
            "COMADMIN_E_APPLID_MATCHES_CLSID" => COMPLUS::COMADMIN_E_APPLID_MATCHES_CLSID,
            "COMADMIN_E_ROLE_DOES_NOT_EXIST" => COMPLUS::COMADMIN_E_ROLE_DOES_NOT_EXIST,
            "COMADMIN_E_START_APP_NEEDS_COMPONENTS" => COMPLUS::COMADMIN_E_START_APP_NEEDS_COMPONENTS,
            "COMADMIN_E_REQUIRES_DIFFERENT_PLATFORM" => COMPLUS::COMADMIN_E_REQUIRES_DIFFERENT_PLATFORM,
            "COMADMIN_E_CAN_NOT_EXPORT_APP_PROXY" => COMPLUS::COMADMIN_E_CAN_NOT_EXPORT_APP_PROXY,
            "COMADMIN_E_CAN_NOT_START_APP" => COMPLUS::COMADMIN_E_CAN_NOT_START_APP,
            "COMADMIN_E_CAN_NOT_EXPORT_SYS_APP" => COMPLUS::COMADMIN_E_CAN_NOT_EXPORT_SYS_APP,
            "COMADMIN_E_CANT_SUBSCRIBE_TO_COMPONENT" => COMPLUS::COMADMIN_E_CANT_SUBSCRIBE_TO_COMPONENT,
            "COMADMIN_E_EVENTCLASS_CANT_BE_SUBSCRIBER" => COMPLUS::COMADMIN_E_EVENTCLASS_CANT_BE_SUBSCRIBER,
            "COMADMIN_E_LIB_APP_PROXY_INCOMPATIBLE" => COMPLUS::COMADMIN_E_LIB_APP_PROXY_INCOMPATIBLE,
            "COMADMIN_E_BASE_PARTITION_ONLY" => COMPLUS::COMADMIN_E_BASE_PARTITION_ONLY,
            "COMADMIN_E_START_APP_DISABLED" => COMPLUS::COMADMIN_E_START_APP_DISABLED,
            "COMADMIN_E_CAT_DUPLICATE_PARTITION_NAME" => COMPLUS::COMADMIN_E_CAT_DUPLICATE_PARTITION_NAME,
            "COMADMIN_E_CAT_INVALID_PARTITION_NAME" => COMPLUS::COMADMIN_E_CAT_INVALID_PARTITION_NAME,
            "COMADMIN_E_CAT_PARTITION_IN_USE" => COMPLUS::COMADMIN_E_CAT_PARTITION_IN_USE,
            "COMADMIN_E_FILE_PARTITION_DUPLICATE_FILES" => COMPLUS::COMADMIN_E_FILE_PARTITION_DUPLICATE_FILES,
            "COMADMIN_E_CAT_IMPORTED_COMPONENTS_NOT_ALLOWED" => COMPLUS::COMADMIN_E_CAT_IMPORTED_COMPONENTS_NOT_ALLOWED,
            "COMADMIN_E_AMBIGUOUS_APPLICATION_NAME" => COMPLUS::COMADMIN_E_AMBIGUOUS_APPLICATION_NAME,
            "COMADMIN_E_AMBIGUOUS_PARTITION_NAME" => COMPLUS::COMADMIN_E_AMBIGUOUS_PARTITION_NAME,
            "COMADMIN_E_REGDB_NOTINITIALIZED" => COMPLUS::COMADMIN_E_REGDB_NOTINITIALIZED,
            "COMADMIN_E_REGDB_NOTOPEN" => COMPLUS::COMADMIN_E_REGDB_NOTOPEN,
            "COMADMIN_E_REGDB_SYSTEMERR" => COMPLUS::COMADMIN_E_REGDB_SYSTEMERR,
            "COMADMIN_E_REGDB_ALREADYRUNNING" => COMPLUS::COMADMIN_E_REGDB_ALREADYRUNNING,
            "COMADMIN_E_MIG_VERSIONNOTSUPPORTED" => COMPLUS::COMADMIN_E_MIG_VERSIONNOTSUPPORTED,
            "COMADMIN_E_MIG_SCHEMANOTFOUND" => COMPLUS::COMADMIN_E_MIG_SCHEMANOTFOUND,
            "COMADMIN_E_CAT_BITNESSMISMATCH" => COMPLUS::COMADMIN_E_CAT_BITNESSMISMATCH,
            "COMADMIN_E_CAT_UNACCEPTABLEBITNESS" => COMPLUS::COMADMIN_E_CAT_UNACCEPTABLEBITNESS,
            "COMADMIN_E_CAT_WRONGAPPBITNESS" => COMPLUS::COMADMIN_E_CAT_WRONGAPPBITNESS,
            "COMADMIN_E_CAT_PAUSE_RESUME_NOT_SUPPORTED" => COMPLUS::COMADMIN_E_CAT_PAUSE_RESUME_NOT_SUPPORTED,
            "COMADMIN_E_CAT_SERVERFAULT" => COMPLUS::COMADMIN_E_CAT_SERVERFAULT,
            "COMQC_E_APPLICATION_NOT_QUEUED" => COMPLUS::COMQC_E_APPLICATION_NOT_QUEUED,
            "COMQC_E_NO_QUEUEABLE_INTERFACES" => COMPLUS::COMQC_E_NO_QUEUEABLE_INTERFACES,
            "COMQC_E_QUEUING_SERVICE_NOT_AVAILABLE" => COMPLUS::COMQC_E_QUEUING_SERVICE_NOT_AVAILABLE,
            "COMQC_E_NO_IPERSISTSTREAM" => COMPLUS::COMQC_E_NO_IPERSISTSTREAM,
            "COMQC_E_BAD_MESSAGE" => COMPLUS::COMQC_E_BAD_MESSAGE,
            "COMQC_E_UNAUTHENTICATED" => COMPLUS::COMQC_E_UNAUTHENTICATED,
            "COMQC_E_UNTRUSTED_ENQUEUER" => COMPLUS::COMQC_E_UNTRUSTED_ENQUEUER,
            "MSDTC_E_DUPLICATE_RESOURCE" => COMPLUS::MSDTC_E_DUPLICATE_RESOURCE,
            "COMADMIN_E_OBJECT_PARENT_MISSING" => COMPLUS::COMADMIN_E_OBJECT_PARENT_MISSING,
            "COMADMIN_E_OBJECT_DOES_NOT_EXIST" => COMPLUS::COMADMIN_E_OBJECT_DOES_NOT_EXIST,
            "COMADMIN_E_APP_NOT_RUNNING" => COMPLUS::COMADMIN_E_APP_NOT_RUNNING,
            "COMADMIN_E_INVALID_PARTITION" => COMPLUS::COMADMIN_E_INVALID_PARTITION,
            "COMADMIN_E_SVCAPP_NOT_POOLABLE_OR_RECYCLABLE" => COMPLUS::COMADMIN_E_SVCAPP_NOT_POOLABLE_OR_RECYCLABLE,
            "COMADMIN_E_USER_IN_SET" => COMPLUS::COMADMIN_E_USER_IN_SET,
            "COMADMIN_E_CANTRECYCLELIBRARYAPPS" => COMPLUS::COMADMIN_E_CANTRECYCLELIBRARYAPPS,
            "COMADMIN_E_CANTRECYCLESERVICEAPPS" => COMPLUS::COMADMIN_E_CANTRECYCLESERVICEAPPS,
            "COMADMIN_E_PROCESSALREADYRECYCLED" => COMPLUS::COMADMIN_E_PROCESSALREADYRECYCLED,
            "COMADMIN_E_PAUSEDPROCESSMAYNOTBERECYCLED" => COMPLUS::COMADMIN_E_PAUSEDPROCESSMAYNOTBERECYCLED,
            "COMADMIN_E_CANTMAKEINPROCSERVICE" => COMPLUS::COMADMIN_E_CANTMAKEINPROCSERVICE,
            "COMADMIN_E_PROGIDINUSEBYCLSID" => COMPLUS::COMADMIN_E_PROGIDINUSEBYCLSID,
            "COMADMIN_E_DEFAULT_PARTITION_NOT_IN_SET" => COMPLUS::COMADMIN_E_DEFAULT_PARTITION_NOT_IN_SET,
            "COMADMIN_E_RECYCLEDPROCESSMAYNOTBEPAUSED" => COMPLUS::COMADMIN_E_RECYCLEDPROCESSMAYNOTBEPAUSED,
            "COMADMIN_E_PARTITION_ACCESSDENIED" => COMPLUS::COMADMIN_E_PARTITION_ACCESSDENIED,
            "COMADMIN_E_PARTITION_MSI_ONLY" => COMPLUS::COMADMIN_E_PARTITION_MSI_ONLY,
            "COMADMIN_E_LEGACYCOMPS_NOT_ALLOWED_IN_1_0_FORMAT" => COMPLUS::COMADMIN_E_LEGACYCOMPS_NOT_ALLOWED_IN_1_0_FORMAT,
            "COMADMIN_E_LEGACYCOMPS_NOT_ALLOWED_IN_NONBASE_PARTITIONS" => COMPLUS::COMADMIN_E_LEGACYCOMPS_NOT_ALLOWED_IN_NONBASE_PARTITIONS,
            "COMADMIN_E_COMP_MOVE_SOURCE" => COMPLUS::COMADMIN_E_COMP_MOVE_SOURCE,
            "COMADMIN_E_COMP_MOVE_DEST" => COMPLUS::COMADMIN_E_COMP_MOVE_DEST,
            "COMADMIN_E_COMP_MOVE_PRIVATE" => COMPLUS::COMADMIN_E_COMP_MOVE_PRIVATE,
            "COMADMIN_E_BASEPARTITION_REQUIRED_IN_SET" => COMPLUS::COMADMIN_E_BASEPARTITION_REQUIRED_IN_SET,
            "COMADMIN_E_CANNOT_ALIAS_EVENTCLASS" => COMPLUS::COMADMIN_E_CANNOT_ALIAS_EVENTCLASS,
            "COMADMIN_E_PRIVATE_ACCESSDENIED" => COMPLUS::COMADMIN_E_PRIVATE_ACCESSDENIED,
            "COMADMIN_E_SAFERINVALID" => COMPLUS::COMADMIN_E_SAFERINVALID,
            "COMADMIN_E_REGISTRY_ACCESSDENIED" => COMPLUS::COMADMIN_E_REGISTRY_ACCESSDENIED,
            "COMADMIN_E_PARTITIONS_DISABLED" => COMPLUS::COMADMIN_E_PARTITIONS_DISABLED,
        }
    }
    pub fn from_code(code: u32) -> COMPLUS {
        return match code {
            0x80110401 => COMPLUS::COMADMIN_E_OBJECTERRORS,
            0x80110402 => COMPLUS::COMADMIN_E_OBJECTINVALID,
            0x80110403 => COMPLUS::COMADMIN_E_KEYMISSING,
            0x80110404 => COMPLUS::COMADMIN_E_ALREADYINSTALLED,
            0x80110407 => COMPLUS::COMADMIN_E_APP_FILE_WRITEFAIL,
            0x80110408 => COMPLUS::COMADMIN_E_APP_FILE_READFAIL,
            0x80110409 => COMPLUS::COMADMIN_E_APP_FILE_VERSION,
            0x8011040A => COMPLUS::COMADMIN_E_BADPATH,
            0x8011040B => COMPLUS::COMADMIN_E_APPLICATIONEXISTS,
            0x8011040C => COMPLUS::COMADMIN_E_ROLEEXISTS,
            0x8011040D => COMPLUS::COMADMIN_E_CANTCOPYFILE,
            0x8011040F => COMPLUS::COMADMIN_E_NOUSER,
            0x80110410 => COMPLUS::COMADMIN_E_INVALIDUSERIDS,
            0x80110411 => COMPLUS::COMADMIN_E_NOREGISTRYCLSID,
            0x80110412 => COMPLUS::COMADMIN_E_BADREGISTRYPROGID,
            0x80110413 => COMPLUS::COMADMIN_E_AUTHENTICATIONLEVEL,
            0x80110414 => COMPLUS::COMADMIN_E_USERPASSWDNOTVALID,
            0x80110418 => COMPLUS::COMADMIN_E_CLSIDORIIDMISMATCH,
            0x80110419 => COMPLUS::COMADMIN_E_REMOTEINTERFACE,
            0x8011041A => COMPLUS::COMADMIN_E_DLLREGISTERSERVER,
            0x8011041B => COMPLUS::COMADMIN_E_NOSERVERSHARE,
            0x8011041D => COMPLUS::COMADMIN_E_DLLLOADFAILED,
            0x8011041E => COMPLUS::COMADMIN_E_BADREGISTRYLIBID,
            0x8011041F => COMPLUS::COMADMIN_E_APPDIRNOTFOUND,
            0x80110423 => COMPLUS::COMADMIN_E_REGISTRARFAILED,
            0x80110424 => COMPLUS::COMADMIN_E_COMPFILE_DOESNOTEXIST,
            0x80110425 => COMPLUS::COMADMIN_E_COMPFILE_LOADDLLFAIL,
            0x80110426 => COMPLUS::COMADMIN_E_COMPFILE_GETCLASSOBJ,
            0x80110427 => COMPLUS::COMADMIN_E_COMPFILE_CLASSNOTAVAIL,
            0x80110428 => COMPLUS::COMADMIN_E_COMPFILE_BADTLB,
            0x80110429 => COMPLUS::COMADMIN_E_COMPFILE_NOTINSTALLABLE,
            0x8011042A => COMPLUS::COMADMIN_E_NOTCHANGEABLE,
            0x8011042B => COMPLUS::COMADMIN_E_NOTDELETEABLE,
            0x8011042C => COMPLUS::COMADMIN_E_SESSION,
            0x8011042D => COMPLUS::COMADMIN_E_COMP_MOVE_LOCKED,
            0x8011042E => COMPLUS::COMADMIN_E_COMP_MOVE_BAD_DEST,
            0x80110430 => COMPLUS::COMADMIN_E_REGISTERTLB,
            0x80110433 => COMPLUS::COMADMIN_E_SYSTEMAPP,
            0x80110434 => COMPLUS::COMADMIN_E_COMPFILE_NOREGISTRAR,
            0x80110435 => COMPLUS::COMADMIN_E_COREQCOMPINSTALLED,
            0x80110436 => COMPLUS::COMADMIN_E_SERVICENOTINSTALLED,
            0x80110437 => COMPLUS::COMADMIN_E_PROPERTYSAVEFAILED,
            0x80110438 => COMPLUS::COMADMIN_E_OBJECTEXISTS,
            0x80110439 => COMPLUS::COMADMIN_E_COMPONENTEXISTS,
            0x8011043B => COMPLUS::COMADMIN_E_REGFILE_CORRUPT,
            0x8011043C => COMPLUS::COMADMIN_E_PROPERTY_OVERFLOW,
            0x8011043E => COMPLUS::COMADMIN_E_NOTINREGISTRY,
            0x8011043F => COMPLUS::COMADMIN_E_OBJECTNOTPOOLABLE,
            0x80110446 => COMPLUS::COMADMIN_E_APPLID_MATCHES_CLSID,
            0x80110447 => COMPLUS::COMADMIN_E_ROLE_DOES_NOT_EXIST,
            0x80110448 => COMPLUS::COMADMIN_E_START_APP_NEEDS_COMPONENTS,
            0x80110449 => COMPLUS::COMADMIN_E_REQUIRES_DIFFERENT_PLATFORM,
            0x8011044A => COMPLUS::COMADMIN_E_CAN_NOT_EXPORT_APP_PROXY,
            0x8011044B => COMPLUS::COMADMIN_E_CAN_NOT_START_APP,
            0x8011044C => COMPLUS::COMADMIN_E_CAN_NOT_EXPORT_SYS_APP,
            0x8011044D => COMPLUS::COMADMIN_E_CANT_SUBSCRIBE_TO_COMPONENT,
            0x8011044E => COMPLUS::COMADMIN_E_EVENTCLASS_CANT_BE_SUBSCRIBER,
            0x8011044F => COMPLUS::COMADMIN_E_LIB_APP_PROXY_INCOMPATIBLE,
            0x80110450 => COMPLUS::COMADMIN_E_BASE_PARTITION_ONLY,
            0x80110451 => COMPLUS::COMADMIN_E_START_APP_DISABLED,
            0x80110457 => COMPLUS::COMADMIN_E_CAT_DUPLICATE_PARTITION_NAME,
            0x80110458 => COMPLUS::COMADMIN_E_CAT_INVALID_PARTITION_NAME,
            0x80110459 => COMPLUS::COMADMIN_E_CAT_PARTITION_IN_USE,
            0x8011045A => COMPLUS::COMADMIN_E_FILE_PARTITION_DUPLICATE_FILES,
            0x8011045B => COMPLUS::COMADMIN_E_CAT_IMPORTED_COMPONENTS_NOT_ALLOWED,
            0x8011045C => COMPLUS::COMADMIN_E_AMBIGUOUS_APPLICATION_NAME,
            0x8011045D => COMPLUS::COMADMIN_E_AMBIGUOUS_PARTITION_NAME,
            0x80110472 => COMPLUS::COMADMIN_E_REGDB_NOTINITIALIZED,
            0x80110473 => COMPLUS::COMADMIN_E_REGDB_NOTOPEN,
            0x80110474 => COMPLUS::COMADMIN_E_REGDB_SYSTEMERR,
            0x80110475 => COMPLUS::COMADMIN_E_REGDB_ALREADYRUNNING,
            0x80110480 => COMPLUS::COMADMIN_E_MIG_VERSIONNOTSUPPORTED,
            0x80110481 => COMPLUS::COMADMIN_E_MIG_SCHEMANOTFOUND,
            0x80110482 => COMPLUS::COMADMIN_E_CAT_BITNESSMISMATCH,
            0x80110483 => COMPLUS::COMADMIN_E_CAT_UNACCEPTABLEBITNESS,
            0x80110484 => COMPLUS::COMADMIN_E_CAT_WRONGAPPBITNESS,
            0x80110485 => COMPLUS::COMADMIN_E_CAT_PAUSE_RESUME_NOT_SUPPORTED,
            0x80110486 => COMPLUS::COMADMIN_E_CAT_SERVERFAULT,
            0x80110600 => COMPLUS::COMQC_E_APPLICATION_NOT_QUEUED,
            0x80110601 => COMPLUS::COMQC_E_NO_QUEUEABLE_INTERFACES,
            0x80110602 => COMPLUS::COMQC_E_QUEUING_SERVICE_NOT_AVAILABLE,
            0x80110603 => COMPLUS::COMQC_E_NO_IPERSISTSTREAM,
            0x80110604 => COMPLUS::COMQC_E_BAD_MESSAGE,
            0x80110605 => COMPLUS::COMQC_E_UNAUTHENTICATED,
            0x80110606 => COMPLUS::COMQC_E_UNTRUSTED_ENQUEUER,
            0x80110701 => COMPLUS::MSDTC_E_DUPLICATE_RESOURCE,
            0x80110808 => COMPLUS::COMADMIN_E_OBJECT_PARENT_MISSING,
            0x80110809 => COMPLUS::COMADMIN_E_OBJECT_DOES_NOT_EXIST,
            0x8011080A => COMPLUS::COMADMIN_E_APP_NOT_RUNNING,
            0x8011080B => COMPLUS::COMADMIN_E_INVALID_PARTITION,
            0x8011080D => COMPLUS::COMADMIN_E_SVCAPP_NOT_POOLABLE_OR_RECYCLABLE,
            0x8011080E => COMPLUS::COMADMIN_E_USER_IN_SET,
            0x8011080F => COMPLUS::COMADMIN_E_CANTRECYCLELIBRARYAPPS,
            0x80110811 => COMPLUS::COMADMIN_E_CANTRECYCLESERVICEAPPS,
            0x80110812 => COMPLUS::COMADMIN_E_PROCESSALREADYRECYCLED,
            0x80110813 => COMPLUS::COMADMIN_E_PAUSEDPROCESSMAYNOTBERECYCLED,
            0x80110814 => COMPLUS::COMADMIN_E_CANTMAKEINPROCSERVICE,
            0x80110815 => COMPLUS::COMADMIN_E_PROGIDINUSEBYCLSID,
            0x80110816 => COMPLUS::COMADMIN_E_DEFAULT_PARTITION_NOT_IN_SET,
            0x80110817 => COMPLUS::COMADMIN_E_RECYCLEDPROCESSMAYNOTBEPAUSED,
            0x80110818 => COMPLUS::COMADMIN_E_PARTITION_ACCESSDENIED,
            0x80110819 => COMPLUS::COMADMIN_E_PARTITION_MSI_ONLY,
            0x8011081A => COMPLUS::COMADMIN_E_LEGACYCOMPS_NOT_ALLOWED_IN_1_0_FORMAT,
            0x8011081B => COMPLUS::COMADMIN_E_LEGACYCOMPS_NOT_ALLOWED_IN_NONBASE_PARTITIONS,
            0x8011081C => COMPLUS::COMADMIN_E_COMP_MOVE_SOURCE,
            0x8011081D => COMPLUS::COMADMIN_E_COMP_MOVE_DEST,
            0x8011081E => COMPLUS::COMADMIN_E_COMP_MOVE_PRIVATE,
            0x8011081F => COMPLUS::COMADMIN_E_BASEPARTITION_REQUIRED_IN_SET,
            0x80110820 => COMPLUS::COMADMIN_E_CANNOT_ALIAS_EVENTCLASS,
            0x80110821 => COMPLUS::COMADMIN_E_PRIVATE_ACCESSDENIED,
            0x80110822 => COMPLUS::COMADMIN_E_SAFERINVALID,
            0x80110823 => COMPLUS::COMADMIN_E_REGISTRY_ACCESSDENIED,
            0x80110824 => COMPLUS::COMADMIN_E_PARTITIONS_DISABLED,
        }
    }
}
