/*
 * Copyright 2023 Stanislav Mikhailov (xavetar)
 *
 * Licensed under the Creative Commons Zero v1.0 Universal (CC0) License.
 * You may obtain a copy of the License at
 *
 *     http://creativecommons.org/publicdomain/zero/1.0/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the CC0 license is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// It is strictly forbidden to use the from_code method, respected Microsoft decided to define
// duplicate codes, but by defining different constant names, therefore, when using these codes,
// it is mandatory to use through from_name, and not from_code. Otherwise, it may cause undefined
// behavior or an unknown exception. Because one code corresponds to several constants.

use super::{RawError};

#[derive(Clone, Debug, Copy, Eq, Hash, Ord, PartialEq, PartialOrd)]
pub enum DIRECT3D11 {
    D3D11_ERROR_TOO_MANY_UNIQUE_STATE_OBJECTS,
    D3D11_ERROR_FILE_NOT_FOUND,
    D3D11_ERROR_TOO_MANY_UNIQUE_VIEW_OBJECTS,
    D3D11_ERROR_DEFERRED_CONTEXT_MAP_WITHOUT_INITIAL_DISCARD,
    AEERR_ALREADY_INITIALIZED,
    AEERR_NOT_INITIALIZED,
    AEERR_MISALIGNED_BUFFER,
    AEERR_NO_REALTIME,
    AEERR_ALREADY_RUNNING,
    AEERR_NOT_RUNNING,
    AEERR_APO_NOT_READY,
    AEERR_ITEM_NOT_FOUND,
    AEERR_MAX_CONNECTIONS_SET,
    AEERR_MIN_CONNECTIONS_SET,
    AEERR_OBJECT_BUSY,
    AEERR_INVALID_CONNECTION_FORMAT,
    AEERR_INPLACE_NOTALLOWED,
    AEERR_INVALID_MIX_FLAGS,
    AEERR_TIME_IS_PAST,
    AEERR_BUFFER_NOT_LOCKED,
    AEERR_DATA_POINTER_NOT_RELEASED,
    AEERR_DATA_POINTER_NOT_ACQUIRED,
    AEERR_CONNECTION_ATTACHED_TO_APO,
    AEERR_ENDPOINT_ALREADY_ATTACHED,
    AEERR_PIPE_NOT_CONNECTABLE,
    AEERR_PIPE_LIMIT_REACHED,
    AEERR_ADG_NO_PIPE_INSTANCES,
    AEERR_ADG_ALREADY_CONNECTED,
    AEERR_TRANSACTION_ALREADY_IN_PROGRESS,
    AEERR_NO_CONVERTERS_FOUND,
    AEERR_TRANSACTION_MUST_BE_CANCELLED,
    AEERR_TRANSACTION_MUST_BE_COMMITTED,
    AEERR_TIMED_OUT,
    AEERR_FORMAT_NOT_SUPPORTED,
    AEERR_ENDPOINT_STATE_NOT_VALID,
    AEERR_ENDPOINT_CREATE_FAILED,
    AEERR_ADG_APO_NOT_USABLE,
    AEERR_ADG_SAMPLE_RATE_MISMATCH,
    AEERR_APO_NOT_REGISTERED,
    AEERR_APO_REGISTRY,
    AEERR_ADG_INVALID_GRAPH,
    AEERR_INVALID_DITHER_STATE,
    AEERR_INVALID_TIMESTAMP,
    AEERR_ENDPOINT_IO,
    AEERR_CP_MUTEX_ABONDENED,
    AEERR_INVALID_FRAMERATE,
    AEERR_DEVICE_IN_USE,
    AEERR_PERFLIB_NOFUNC,
    AEERR_CP_SHARED_BUFFER_CORRUPT,
    AEERR_ADG_INVALID_PROCESSING_PERIOD,
    AEERR_CP_QUEUE_FULL,
    AEERR_APO_INVALIDARG,
    AEERR_NO_HEAP,
    AEERR_HEAP_ALREADY_CREATED,
    AEERR_ACTIVE_STREAM_COUNT_ERR,
    AEERR_ENDPOINT_IN_USE,
}

impl DIRECT3D11 {
    pub fn code(&self) -> u32 {
        return match self {
            DIRECT3D11::D3D11_ERROR_TOO_MANY_UNIQUE_STATE_OBJECTS => 0x887C0001 as u32,
            DIRECT3D11::D3D11_ERROR_FILE_NOT_FOUND => 0x887C0002 as u32,
            DIRECT3D11::D3D11_ERROR_TOO_MANY_UNIQUE_VIEW_OBJECTS => 0x887C0003 as u32,
            DIRECT3D11::D3D11_ERROR_DEFERRED_CONTEXT_MAP_WITHOUT_INITIAL_DISCARD => 0x887C0004 as u32,
            DIRECT3D11::AEERR_ALREADY_INITIALIZED => 0x887C0010 as u32,
            DIRECT3D11::AEERR_NOT_INITIALIZED => 0x887C0013 as u32,
            DIRECT3D11::AEERR_MISALIGNED_BUFFER => 0x887C0014 as u32,
            DIRECT3D11::AEERR_NO_REALTIME => 0x887C0015 as u32,
            DIRECT3D11::AEERR_ALREADY_RUNNING => 0x887C0016 as u32,
            DIRECT3D11::AEERR_NOT_RUNNING => 0x887C0017 as u32,
            DIRECT3D11::AEERR_APO_NOT_READY => 0x887C0018 as u32,
            DIRECT3D11::AEERR_ITEM_NOT_FOUND => 0x887C001A as u32,
            DIRECT3D11::AEERR_MAX_CONNECTIONS_SET => 0x887C001B as u32,
            DIRECT3D11::AEERR_MIN_CONNECTIONS_SET => 0x887C001C as u32,
            DIRECT3D11::AEERR_OBJECT_BUSY => 0x887C001D as u32,
            DIRECT3D11::AEERR_INVALID_CONNECTION_FORMAT => 0x887C001E as u32,
            DIRECT3D11::AEERR_INPLACE_NOTALLOWED => 0x887C001F as u32,
            DIRECT3D11::AEERR_INVALID_MIX_FLAGS => 0x887C0020 as u32,
            DIRECT3D11::AEERR_TIME_IS_PAST => 0x887C0022 as u32,
            DIRECT3D11::AEERR_BUFFER_NOT_LOCKED => 0x887C0023 as u32,
            DIRECT3D11::AEERR_DATA_POINTER_NOT_RELEASED => 0x887C0024 as u32,
            DIRECT3D11::AEERR_DATA_POINTER_NOT_ACQUIRED => 0x887C0025 as u32,
            DIRECT3D11::AEERR_CONNECTION_ATTACHED_TO_APO => 0x887C0027 as u32,
            DIRECT3D11::AEERR_ENDPOINT_ALREADY_ATTACHED => 0x887C0028 as u32,
            DIRECT3D11::AEERR_PIPE_NOT_CONNECTABLE => 0x887C0029 as u32,
            DIRECT3D11::AEERR_PIPE_LIMIT_REACHED => 0x887C002A as u32,
            DIRECT3D11::AEERR_ADG_NO_PIPE_INSTANCES => 0x887C002B as u32,
            DIRECT3D11::AEERR_ADG_ALREADY_CONNECTED => 0x887C002E as u32,
            DIRECT3D11::AEERR_TRANSACTION_ALREADY_IN_PROGRESS => 0x887C002F as u32,
            DIRECT3D11::AEERR_NO_CONVERTERS_FOUND => 0x887C0030 as u32,
            DIRECT3D11::AEERR_TRANSACTION_MUST_BE_CANCELLED => 0x887C0031 as u32,
            DIRECT3D11::AEERR_TRANSACTION_MUST_BE_COMMITTED => 0x887C0032 as u32,
            DIRECT3D11::AEERR_TIMED_OUT => 0x887C0033 as u32,
            DIRECT3D11::AEERR_FORMAT_NOT_SUPPORTED => 0x887C0034 as u32,
            DIRECT3D11::AEERR_ENDPOINT_STATE_NOT_VALID => 0x887C0035 as u32,
            DIRECT3D11::AEERR_ENDPOINT_CREATE_FAILED => 0x887C0036 as u32,
            DIRECT3D11::AEERR_ADG_APO_NOT_USABLE => 0x887C0037 as u32,
            DIRECT3D11::AEERR_ADG_SAMPLE_RATE_MISMATCH => 0x887C0039 as u32,
            DIRECT3D11::AEERR_APO_NOT_REGISTERED => 0x887C003A as u32,
            DIRECT3D11::AEERR_APO_REGISTRY => 0x887C003B as u32,
            DIRECT3D11::AEERR_ADG_INVALID_GRAPH => 0x887C003C as u32,
            DIRECT3D11::AEERR_INVALID_DITHER_STATE => 0x887C003D as u32,
            DIRECT3D11::AEERR_INVALID_TIMESTAMP => 0x887C003F as u32,
            DIRECT3D11::AEERR_ENDPOINT_IO => 0x887C0040 as u32,
            DIRECT3D11::AEERR_CP_MUTEX_ABONDENED => 0x887C0041 as u32,
            DIRECT3D11::AEERR_INVALID_FRAMERATE => 0x887C0042 as u32,
            DIRECT3D11::AEERR_DEVICE_IN_USE => 0x887C0043 as u32,
            DIRECT3D11::AEERR_PERFLIB_NOFUNC => 0x887C0044 as u32,
            DIRECT3D11::AEERR_CP_SHARED_BUFFER_CORRUPT => 0x887C0045 as u32,
            DIRECT3D11::AEERR_ADG_INVALID_PROCESSING_PERIOD => 0x887C0046 as u32,
            DIRECT3D11::AEERR_CP_QUEUE_FULL => 0x887C0047 as u32,
            DIRECT3D11::AEERR_APO_INVALIDARG => 0x887C0048 as u32,
            DIRECT3D11::AEERR_NO_HEAP => 0x887C0049 as u32,
            DIRECT3D11::AEERR_HEAP_ALREADY_CREATED => 0x887C004A as u32,
            DIRECT3D11::AEERR_ACTIVE_STREAM_COUNT_ERR => 0x887C004B as u32,
            DIRECT3D11::AEERR_ENDPOINT_IN_USE => 0x887C00AA as u32,
        }
    }

    pub fn error(&self) -> RawError {
        return match self {
            DIRECT3D11::D3D11_ERROR_TOO_MANY_UNIQUE_STATE_OBJECTS => RawError::Kind(DIRECT3D11::D3D11_ERROR_TOO_MANY_UNIQUE_STATE_OBJECTS),
            DIRECT3D11::D3D11_ERROR_FILE_NOT_FOUND => RawError::Kind(DIRECT3D11::D3D11_ERROR_FILE_NOT_FOUND),
            DIRECT3D11::D3D11_ERROR_TOO_MANY_UNIQUE_VIEW_OBJECTS => RawError::Kind(DIRECT3D11::D3D11_ERROR_TOO_MANY_UNIQUE_VIEW_OBJECTS),
            DIRECT3D11::D3D11_ERROR_DEFERRED_CONTEXT_MAP_WITHOUT_INITIAL_DISCARD => RawError::Kind(DIRECT3D11::D3D11_ERROR_DEFERRED_CONTEXT_MAP_WITHOUT_INITIAL_DISCARD),
            DIRECT3D11::AEERR_ALREADY_INITIALIZED => RawError::Kind(DIRECT3D11::AEERR_ALREADY_INITIALIZED),
            DIRECT3D11::AEERR_NOT_INITIALIZED => RawError::Kind(DIRECT3D11::AEERR_NOT_INITIALIZED),
            DIRECT3D11::AEERR_MISALIGNED_BUFFER => RawError::Kind(DIRECT3D11::AEERR_MISALIGNED_BUFFER),
            DIRECT3D11::AEERR_NO_REALTIME => RawError::Kind(DIRECT3D11::AEERR_NO_REALTIME),
            DIRECT3D11::AEERR_ALREADY_RUNNING => RawError::Kind(DIRECT3D11::AEERR_ALREADY_RUNNING),
            DIRECT3D11::AEERR_NOT_RUNNING => RawError::Kind(DIRECT3D11::AEERR_NOT_RUNNING),
            DIRECT3D11::AEERR_APO_NOT_READY => RawError::Kind(DIRECT3D11::AEERR_APO_NOT_READY),
            DIRECT3D11::AEERR_ITEM_NOT_FOUND => RawError::Kind(DIRECT3D11::AEERR_ITEM_NOT_FOUND),
            DIRECT3D11::AEERR_MAX_CONNECTIONS_SET => RawError::Kind(DIRECT3D11::AEERR_MAX_CONNECTIONS_SET),
            DIRECT3D11::AEERR_MIN_CONNECTIONS_SET => RawError::Kind(DIRECT3D11::AEERR_MIN_CONNECTIONS_SET),
            DIRECT3D11::AEERR_OBJECT_BUSY => RawError::Kind(DIRECT3D11::AEERR_OBJECT_BUSY),
            DIRECT3D11::AEERR_INVALID_CONNECTION_FORMAT => RawError::Kind(DIRECT3D11::AEERR_INVALID_CONNECTION_FORMAT),
            DIRECT3D11::AEERR_INPLACE_NOTALLOWED => RawError::Kind(DIRECT3D11::AEERR_INPLACE_NOTALLOWED),
            DIRECT3D11::AEERR_INVALID_MIX_FLAGS => RawError::Kind(DIRECT3D11::AEERR_INVALID_MIX_FLAGS),
            DIRECT3D11::AEERR_TIME_IS_PAST => RawError::Kind(DIRECT3D11::AEERR_TIME_IS_PAST),
            DIRECT3D11::AEERR_BUFFER_NOT_LOCKED => RawError::Kind(DIRECT3D11::AEERR_BUFFER_NOT_LOCKED),
            DIRECT3D11::AEERR_DATA_POINTER_NOT_RELEASED => RawError::Kind(DIRECT3D11::AEERR_DATA_POINTER_NOT_RELEASED),
            DIRECT3D11::AEERR_DATA_POINTER_NOT_ACQUIRED => RawError::Kind(DIRECT3D11::AEERR_DATA_POINTER_NOT_ACQUIRED),
            DIRECT3D11::AEERR_CONNECTION_ATTACHED_TO_APO => RawError::Kind(DIRECT3D11::AEERR_CONNECTION_ATTACHED_TO_APO),
            DIRECT3D11::AEERR_ENDPOINT_ALREADY_ATTACHED => RawError::Kind(DIRECT3D11::AEERR_ENDPOINT_ALREADY_ATTACHED),
            DIRECT3D11::AEERR_PIPE_NOT_CONNECTABLE => RawError::Kind(DIRECT3D11::AEERR_PIPE_NOT_CONNECTABLE),
            DIRECT3D11::AEERR_PIPE_LIMIT_REACHED => RawError::Kind(DIRECT3D11::AEERR_PIPE_LIMIT_REACHED),
            DIRECT3D11::AEERR_ADG_NO_PIPE_INSTANCES => RawError::Kind(DIRECT3D11::AEERR_ADG_NO_PIPE_INSTANCES),
            DIRECT3D11::AEERR_ADG_ALREADY_CONNECTED => RawError::Kind(DIRECT3D11::AEERR_ADG_ALREADY_CONNECTED),
            DIRECT3D11::AEERR_TRANSACTION_ALREADY_IN_PROGRESS => RawError::Kind(DIRECT3D11::AEERR_TRANSACTION_ALREADY_IN_PROGRESS),
            DIRECT3D11::AEERR_NO_CONVERTERS_FOUND => RawError::Kind(DIRECT3D11::AEERR_NO_CONVERTERS_FOUND),
            DIRECT3D11::AEERR_TRANSACTION_MUST_BE_CANCELLED => RawError::Kind(DIRECT3D11::AEERR_TRANSACTION_MUST_BE_CANCELLED),
            DIRECT3D11::AEERR_TRANSACTION_MUST_BE_COMMITTED => RawError::Kind(DIRECT3D11::AEERR_TRANSACTION_MUST_BE_COMMITTED),
            DIRECT3D11::AEERR_TIMED_OUT => RawError::Kind(DIRECT3D11::AEERR_TIMED_OUT),
            DIRECT3D11::AEERR_FORMAT_NOT_SUPPORTED => RawError::Kind(DIRECT3D11::AEERR_FORMAT_NOT_SUPPORTED),
            DIRECT3D11::AEERR_ENDPOINT_STATE_NOT_VALID => RawError::Kind(DIRECT3D11::AEERR_ENDPOINT_STATE_NOT_VALID),
            DIRECT3D11::AEERR_ENDPOINT_CREATE_FAILED => RawError::Kind(DIRECT3D11::AEERR_ENDPOINT_CREATE_FAILED),
            DIRECT3D11::AEERR_ADG_APO_NOT_USABLE => RawError::Kind(DIRECT3D11::AEERR_ADG_APO_NOT_USABLE),
            DIRECT3D11::AEERR_ADG_SAMPLE_RATE_MISMATCH => RawError::Kind(DIRECT3D11::AEERR_ADG_SAMPLE_RATE_MISMATCH),
            DIRECT3D11::AEERR_APO_NOT_REGISTERED => RawError::Kind(DIRECT3D11::AEERR_APO_NOT_REGISTERED),
            DIRECT3D11::AEERR_APO_REGISTRY => RawError::Kind(DIRECT3D11::AEERR_APO_REGISTRY),
            DIRECT3D11::AEERR_ADG_INVALID_GRAPH => RawError::Kind(DIRECT3D11::AEERR_ADG_INVALID_GRAPH),
            DIRECT3D11::AEERR_INVALID_DITHER_STATE => RawError::Kind(DIRECT3D11::AEERR_INVALID_DITHER_STATE),
            DIRECT3D11::AEERR_INVALID_TIMESTAMP => RawError::Kind(DIRECT3D11::AEERR_INVALID_TIMESTAMP),
            DIRECT3D11::AEERR_ENDPOINT_IO => RawError::Kind(DIRECT3D11::AEERR_ENDPOINT_IO),
            DIRECT3D11::AEERR_CP_MUTEX_ABONDENED => RawError::Kind(DIRECT3D11::AEERR_CP_MUTEX_ABONDENED),
            DIRECT3D11::AEERR_INVALID_FRAMERATE => RawError::Kind(DIRECT3D11::AEERR_INVALID_FRAMERATE),
            DIRECT3D11::AEERR_DEVICE_IN_USE => RawError::Kind(DIRECT3D11::AEERR_DEVICE_IN_USE),
            DIRECT3D11::AEERR_PERFLIB_NOFUNC => RawError::Kind(DIRECT3D11::AEERR_PERFLIB_NOFUNC),
            DIRECT3D11::AEERR_CP_SHARED_BUFFER_CORRUPT => RawError::Kind(DIRECT3D11::AEERR_CP_SHARED_BUFFER_CORRUPT),
            DIRECT3D11::AEERR_ADG_INVALID_PROCESSING_PERIOD => RawError::Kind(DIRECT3D11::AEERR_ADG_INVALID_PROCESSING_PERIOD),
            DIRECT3D11::AEERR_CP_QUEUE_FULL => RawError::Kind(DIRECT3D11::AEERR_CP_QUEUE_FULL),
            DIRECT3D11::AEERR_APO_INVALIDARG => RawError::Kind(DIRECT3D11::AEERR_APO_INVALIDARG),
            DIRECT3D11::AEERR_NO_HEAP => RawError::Kind(DIRECT3D11::AEERR_NO_HEAP),
            DIRECT3D11::AEERR_HEAP_ALREADY_CREATED => RawError::Kind(DIRECT3D11::AEERR_HEAP_ALREADY_CREATED),
            DIRECT3D11::AEERR_ACTIVE_STREAM_COUNT_ERR => RawError::Kind(DIRECT3D11::AEERR_ACTIVE_STREAM_COUNT_ERR),
            DIRECT3D11::AEERR_ENDPOINT_IN_USE => RawError::Kind(DIRECT3D11::AEERR_ENDPOINT_IN_USE),
        }
    }

    pub fn description(&self) -> &'static str {
        return match self {
            DIRECT3D11::D3D11_ERROR_TOO_MANY_UNIQUE_STATE_OBJECTS => "The application has exceeded the maximum number of unique state objects per Direct3D device.",
            DIRECT3D11::D3D11_ERROR_FILE_NOT_FOUND => "The specified file was not found.",
            DIRECT3D11::D3D11_ERROR_TOO_MANY_UNIQUE_VIEW_OBJECTS => "The application has exceeded the maximum number of unique view objects per Direct3D device.",
            DIRECT3D11::D3D11_ERROR_DEFERRED_CONTEXT_MAP_WITHOUT_INITIAL_DISCARD => "The application's first call per command list to Map on a deferred context did not use D3D11_MAP_WRITE_DISCARD.",
            DIRECT3D11::AEERR_ALREADY_INITIALIZED => "The object has already been initialized.",
            DIRECT3D11::AEERR_NOT_INITIALIZED => "Object/structure is not initialized.",
            DIRECT3D11::AEERR_MISALIGNED_BUFFER => "Misaligned buffer usage.",
            DIRECT3D11::AEERR_NO_REALTIME => "Failed to allocate a real-time thread.",
            DIRECT3D11::AEERR_ALREADY_RUNNING => "Already in a running state.",
            DIRECT3D11::AEERR_NOT_RUNNING => "Not in a running state.",
            DIRECT3D11::AEERR_APO_NOT_READY => "APO is not ready to process.",
            DIRECT3D11::AEERR_ITEM_NOT_FOUND => "Item not found in list.",
            DIRECT3D11::AEERR_MAX_CONNECTIONS_SET => "Maximum number of input or output connections already set on this APO.",
            DIRECT3D11::AEERR_MIN_CONNECTIONS_SET => "Minimum number of input or output connections are set on this APO.",
            DIRECT3D11::AEERR_OBJECT_BUSY => "Object is busy performing some operation.",
            DIRECT3D11::AEERR_INVALID_CONNECTION_FORMAT => "Invalid connection format for this operation.",
            DIRECT3D11::AEERR_INPLACE_NOTALLOWED => "Inplace not allowed on this APO.",
            DIRECT3D11::AEERR_INVALID_MIX_FLAGS => "Invalid mix flags.",
            DIRECT3D11::AEERR_TIME_IS_PAST => "Invalid timestamp (time has already gone by).",
            DIRECT3D11::AEERR_BUFFER_NOT_LOCKED => "Invalid data pointer (the pointer has not been locked).",
            DIRECT3D11::AEERR_DATA_POINTER_NOT_RELEASED => "Data pointer not released (GetDataPointer was called twice).",
            DIRECT3D11::AEERR_DATA_POINTER_NOT_ACQUIRED => "Data pointer not acquired (Release was called without get).",
            DIRECT3D11::AEERR_CONNECTION_ATTACHED_TO_APO => "Connection is attached to an apo and cannot be released.",
            DIRECT3D11::AEERR_ENDPOINT_ALREADY_ATTACHED => "Connection already attached to an endpoint.",
            DIRECT3D11::AEERR_PIPE_NOT_CONNECTABLE => "The pipe contains APOs that cannot be connected.",
            DIRECT3D11::AEERR_PIPE_LIMIT_REACHED => "The pipe contains APOs that cannot be connected.",
            DIRECT3D11::AEERR_ADG_NO_PIPE_INSTANCES => "The audiodevicegraph does not have any pipe instances.",
            DIRECT3D11::AEERR_ADG_ALREADY_CONNECTED => "The audiodevicegraph is already connected.",
            DIRECT3D11::AEERR_TRANSACTION_ALREADY_IN_PROGRESS => "There is already a transaction in progress.",
            DIRECT3D11::AEERR_NO_CONVERTERS_FOUND => "No format converters were found.",
            DIRECT3D11::AEERR_TRANSACTION_MUST_BE_CANCELLED => "The transaction must be cancelled (because it was already attempted but failed to cancel).",
            DIRECT3D11::AEERR_TRANSACTION_MUST_BE_COMMITTED => "The transaction must be committed (because it was already attempted but failed to commit).",
            DIRECT3D11::AEERR_TIMED_OUT => "Time out occurred.",
            DIRECT3D11::AEERR_FORMAT_NOT_SUPPORTED => "A pin supporting the format cannot be found.",
            DIRECT3D11::AEERR_ENDPOINT_STATE_NOT_VALID => "Endpoint state is invalid for the required operation.",
            DIRECT3D11::AEERR_ENDPOINT_CREATE_FAILED => "Endpoint creation failed.",
            DIRECT3D11::AEERR_ADG_APO_NOT_USABLE => "An APO is not usable for ADG. See AUDIO_PIPE_DESCRIPTOR for details.",
            DIRECT3D11::AEERR_ADG_SAMPLE_RATE_MISMATCH => "ADG does not support SRC within a pipe.",
            DIRECT3D11::AEERR_APO_NOT_REGISTERED => "APO is not registered.",
            DIRECT3D11::AEERR_APO_REGISTRY => "An unexpected error occurred when reading APO properties.",
            DIRECT3D11::AEERR_ADG_INVALID_GRAPH => "An unexpected error occurred in audio device graph.",
            DIRECT3D11::AEERR_INVALID_DITHER_STATE => "Invalid dither state.",
            DIRECT3D11::AEERR_INVALID_TIMESTAMP => "The time-stamp is invalid.",
            DIRECT3D11::AEERR_ENDPOINT_IO => "The endpoint failed IO with the driver.",
            DIRECT3D11::AEERR_CP_MUTEX_ABONDENED => "The crossprocess synchronization mutex was abandoned.",
            DIRECT3D11::AEERR_INVALID_FRAMERATE => "Frame rates differ from initialization rates.",
            DIRECT3D11::AEERR_DEVICE_IN_USE => "The device is already in use.",
            DIRECT3D11::AEERR_PERFLIB_NOFUNC => "No supported function available in Audio PerfLib.",
            DIRECT3D11::AEERR_CP_SHARED_BUFFER_CORRUPT => "The crossprocess shared buffer is corrupted.",
            DIRECT3D11::AEERR_ADG_INVALID_PROCESSING_PERIOD => "Invalid ADG global graph processing period.",
            DIRECT3D11::AEERR_CP_QUEUE_FULL => "The crossprocess capture queue is full.",
            DIRECT3D11::AEERR_APO_INVALIDARG => "Invalid APO parameter.",
            DIRECT3D11::AEERR_NO_HEAP => "AudioProcessor heap is not created.",
            DIRECT3D11::AEERR_HEAP_ALREADY_CREATED => "AudioProcessor heap is already created.",
            DIRECT3D11::AEERR_ACTIVE_STREAM_COUNT_ERR => "AudioProcessor wrong active stream count.",
            DIRECT3D11::AEERR_ENDPOINT_IN_USE => "PinInstantiate returned ERROR_NO_SYSTEM_RESOURCES.",
        }
    }

    pub fn from_name(name: &str) -> DIRECT3D11 {
        return match name {
            "D3D11_ERROR_TOO_MANY_UNIQUE_STATE_OBJECTS" => DIRECT3D11::D3D11_ERROR_TOO_MANY_UNIQUE_STATE_OBJECTS,
            "D3D11_ERROR_FILE_NOT_FOUND" => DIRECT3D11::D3D11_ERROR_FILE_NOT_FOUND,
            "D3D11_ERROR_TOO_MANY_UNIQUE_VIEW_OBJECTS" => DIRECT3D11::D3D11_ERROR_TOO_MANY_UNIQUE_VIEW_OBJECTS,
            "D3D11_ERROR_DEFERRED_CONTEXT_MAP_WITHOUT_INITIAL_DISCARD" => DIRECT3D11::D3D11_ERROR_DEFERRED_CONTEXT_MAP_WITHOUT_INITIAL_DISCARD,
            "AEERR_ALREADY_INITIALIZED" => DIRECT3D11::AEERR_ALREADY_INITIALIZED,
            "AEERR_NOT_INITIALIZED" => DIRECT3D11::AEERR_NOT_INITIALIZED,
            "AEERR_MISALIGNED_BUFFER" => DIRECT3D11::AEERR_MISALIGNED_BUFFER,
            "AEERR_NO_REALTIME" => DIRECT3D11::AEERR_NO_REALTIME,
            "AEERR_ALREADY_RUNNING" => DIRECT3D11::AEERR_ALREADY_RUNNING,
            "AEERR_NOT_RUNNING" => DIRECT3D11::AEERR_NOT_RUNNING,
            "AEERR_APO_NOT_READY" => DIRECT3D11::AEERR_APO_NOT_READY,
            "AEERR_ITEM_NOT_FOUND" => DIRECT3D11::AEERR_ITEM_NOT_FOUND,
            "AEERR_MAX_CONNECTIONS_SET" => DIRECT3D11::AEERR_MAX_CONNECTIONS_SET,
            "AEERR_MIN_CONNECTIONS_SET" => DIRECT3D11::AEERR_MIN_CONNECTIONS_SET,
            "AEERR_OBJECT_BUSY" => DIRECT3D11::AEERR_OBJECT_BUSY,
            "AEERR_INVALID_CONNECTION_FORMAT" => DIRECT3D11::AEERR_INVALID_CONNECTION_FORMAT,
            "AEERR_INPLACE_NOTALLOWED" => DIRECT3D11::AEERR_INPLACE_NOTALLOWED,
            "AEERR_INVALID_MIX_FLAGS" => DIRECT3D11::AEERR_INVALID_MIX_FLAGS,
            "AEERR_TIME_IS_PAST" => DIRECT3D11::AEERR_TIME_IS_PAST,
            "AEERR_BUFFER_NOT_LOCKED" => DIRECT3D11::AEERR_BUFFER_NOT_LOCKED,
            "AEERR_DATA_POINTER_NOT_RELEASED" => DIRECT3D11::AEERR_DATA_POINTER_NOT_RELEASED,
            "AEERR_DATA_POINTER_NOT_ACQUIRED" => DIRECT3D11::AEERR_DATA_POINTER_NOT_ACQUIRED,
            "AEERR_CONNECTION_ATTACHED_TO_APO" => DIRECT3D11::AEERR_CONNECTION_ATTACHED_TO_APO,
            "AEERR_ENDPOINT_ALREADY_ATTACHED" => DIRECT3D11::AEERR_ENDPOINT_ALREADY_ATTACHED,
            "AEERR_PIPE_NOT_CONNECTABLE" => DIRECT3D11::AEERR_PIPE_NOT_CONNECTABLE,
            "AEERR_PIPE_LIMIT_REACHED" => DIRECT3D11::AEERR_PIPE_LIMIT_REACHED,
            "AEERR_ADG_NO_PIPE_INSTANCES" => DIRECT3D11::AEERR_ADG_NO_PIPE_INSTANCES,
            "AEERR_ADG_ALREADY_CONNECTED" => DIRECT3D11::AEERR_ADG_ALREADY_CONNECTED,
            "AEERR_TRANSACTION_ALREADY_IN_PROGRESS" => DIRECT3D11::AEERR_TRANSACTION_ALREADY_IN_PROGRESS,
            "AEERR_NO_CONVERTERS_FOUND" => DIRECT3D11::AEERR_NO_CONVERTERS_FOUND,
            "AEERR_TRANSACTION_MUST_BE_CANCELLED" => DIRECT3D11::AEERR_TRANSACTION_MUST_BE_CANCELLED,
            "AEERR_TRANSACTION_MUST_BE_COMMITTED" => DIRECT3D11::AEERR_TRANSACTION_MUST_BE_COMMITTED,
            "AEERR_TIMED_OUT" => DIRECT3D11::AEERR_TIMED_OUT,
            "AEERR_FORMAT_NOT_SUPPORTED" => DIRECT3D11::AEERR_FORMAT_NOT_SUPPORTED,
            "AEERR_ENDPOINT_STATE_NOT_VALID" => DIRECT3D11::AEERR_ENDPOINT_STATE_NOT_VALID,
            "AEERR_ENDPOINT_CREATE_FAILED" => DIRECT3D11::AEERR_ENDPOINT_CREATE_FAILED,
            "AEERR_ADG_APO_NOT_USABLE" => DIRECT3D11::AEERR_ADG_APO_NOT_USABLE,
            "AEERR_ADG_SAMPLE_RATE_MISMATCH" => DIRECT3D11::AEERR_ADG_SAMPLE_RATE_MISMATCH,
            "AEERR_APO_NOT_REGISTERED" => DIRECT3D11::AEERR_APO_NOT_REGISTERED,
            "AEERR_APO_REGISTRY" => DIRECT3D11::AEERR_APO_REGISTRY,
            "AEERR_ADG_INVALID_GRAPH" => DIRECT3D11::AEERR_ADG_INVALID_GRAPH,
            "AEERR_INVALID_DITHER_STATE" => DIRECT3D11::AEERR_INVALID_DITHER_STATE,
            "AEERR_INVALID_TIMESTAMP" => DIRECT3D11::AEERR_INVALID_TIMESTAMP,
            "AEERR_ENDPOINT_IO" => DIRECT3D11::AEERR_ENDPOINT_IO,
            "AEERR_CP_MUTEX_ABONDENED" => DIRECT3D11::AEERR_CP_MUTEX_ABONDENED,
            "AEERR_INVALID_FRAMERATE" => DIRECT3D11::AEERR_INVALID_FRAMERATE,
            "AEERR_DEVICE_IN_USE" => DIRECT3D11::AEERR_DEVICE_IN_USE,
            "AEERR_PERFLIB_NOFUNC" => DIRECT3D11::AEERR_PERFLIB_NOFUNC,
            "AEERR_CP_SHARED_BUFFER_CORRUPT" => DIRECT3D11::AEERR_CP_SHARED_BUFFER_CORRUPT,
            "AEERR_ADG_INVALID_PROCESSING_PERIOD" => DIRECT3D11::AEERR_ADG_INVALID_PROCESSING_PERIOD,
            "AEERR_CP_QUEUE_FULL" => DIRECT3D11::AEERR_CP_QUEUE_FULL,
            "AEERR_APO_INVALIDARG" => DIRECT3D11::AEERR_APO_INVALIDARG,
            "AEERR_NO_HEAP" => DIRECT3D11::AEERR_NO_HEAP,
            "AEERR_HEAP_ALREADY_CREATED" => DIRECT3D11::AEERR_HEAP_ALREADY_CREATED,
            "AEERR_ACTIVE_STREAM_COUNT_ERR" => DIRECT3D11::AEERR_ACTIVE_STREAM_COUNT_ERR,
            "AEERR_ENDPOINT_IN_USE" => DIRECT3D11::AEERR_ENDPOINT_IN_USE,
        }
    }
    pub fn from_code(code: u32) -> DIRECT3D11 {
        return match code {
            0x887C0001 => DIRECT3D11::D3D11_ERROR_TOO_MANY_UNIQUE_STATE_OBJECTS,
            0x887C0002 => DIRECT3D11::D3D11_ERROR_FILE_NOT_FOUND,
            0x887C0003 => DIRECT3D11::D3D11_ERROR_TOO_MANY_UNIQUE_VIEW_OBJECTS,
            0x887C0004 => DIRECT3D11::D3D11_ERROR_DEFERRED_CONTEXT_MAP_WITHOUT_INITIAL_DISCARD,
            0x887C0010 => DIRECT3D11::AEERR_ALREADY_INITIALIZED,
            0x887C0013 => DIRECT3D11::AEERR_NOT_INITIALIZED,
            0x887C0014 => DIRECT3D11::AEERR_MISALIGNED_BUFFER,
            0x887C0015 => DIRECT3D11::AEERR_NO_REALTIME,
            0x887C0016 => DIRECT3D11::AEERR_ALREADY_RUNNING,
            0x887C0017 => DIRECT3D11::AEERR_NOT_RUNNING,
            0x887C0018 => DIRECT3D11::AEERR_APO_NOT_READY,
            0x887C001A => DIRECT3D11::AEERR_ITEM_NOT_FOUND,
            0x887C001B => DIRECT3D11::AEERR_MAX_CONNECTIONS_SET,
            0x887C001C => DIRECT3D11::AEERR_MIN_CONNECTIONS_SET,
            0x887C001D => DIRECT3D11::AEERR_OBJECT_BUSY,
            0x887C001E => DIRECT3D11::AEERR_INVALID_CONNECTION_FORMAT,
            0x887C001F => DIRECT3D11::AEERR_INPLACE_NOTALLOWED,
            0x887C0020 => DIRECT3D11::AEERR_INVALID_MIX_FLAGS,
            0x887C0022 => DIRECT3D11::AEERR_TIME_IS_PAST,
            0x887C0023 => DIRECT3D11::AEERR_BUFFER_NOT_LOCKED,
            0x887C0024 => DIRECT3D11::AEERR_DATA_POINTER_NOT_RELEASED,
            0x887C0025 => DIRECT3D11::AEERR_DATA_POINTER_NOT_ACQUIRED,
            0x887C0027 => DIRECT3D11::AEERR_CONNECTION_ATTACHED_TO_APO,
            0x887C0028 => DIRECT3D11::AEERR_ENDPOINT_ALREADY_ATTACHED,
            0x887C0029 => DIRECT3D11::AEERR_PIPE_NOT_CONNECTABLE,
            0x887C002A => DIRECT3D11::AEERR_PIPE_LIMIT_REACHED,
            0x887C002B => DIRECT3D11::AEERR_ADG_NO_PIPE_INSTANCES,
            0x887C002E => DIRECT3D11::AEERR_ADG_ALREADY_CONNECTED,
            0x887C002F => DIRECT3D11::AEERR_TRANSACTION_ALREADY_IN_PROGRESS,
            0x887C0030 => DIRECT3D11::AEERR_NO_CONVERTERS_FOUND,
            0x887C0031 => DIRECT3D11::AEERR_TRANSACTION_MUST_BE_CANCELLED,
            0x887C0032 => DIRECT3D11::AEERR_TRANSACTION_MUST_BE_COMMITTED,
            0x887C0033 => DIRECT3D11::AEERR_TIMED_OUT,
            0x887C0034 => DIRECT3D11::AEERR_FORMAT_NOT_SUPPORTED,
            0x887C0035 => DIRECT3D11::AEERR_ENDPOINT_STATE_NOT_VALID,
            0x887C0036 => DIRECT3D11::AEERR_ENDPOINT_CREATE_FAILED,
            0x887C0037 => DIRECT3D11::AEERR_ADG_APO_NOT_USABLE,
            0x887C0039 => DIRECT3D11::AEERR_ADG_SAMPLE_RATE_MISMATCH,
            0x887C003A => DIRECT3D11::AEERR_APO_NOT_REGISTERED,
            0x887C003B => DIRECT3D11::AEERR_APO_REGISTRY,
            0x887C003C => DIRECT3D11::AEERR_ADG_INVALID_GRAPH,
            0x887C003D => DIRECT3D11::AEERR_INVALID_DITHER_STATE,
            0x887C003F => DIRECT3D11::AEERR_INVALID_TIMESTAMP,
            0x887C0040 => DIRECT3D11::AEERR_ENDPOINT_IO,
            0x887C0041 => DIRECT3D11::AEERR_CP_MUTEX_ABONDENED,
            0x887C0042 => DIRECT3D11::AEERR_INVALID_FRAMERATE,
            0x887C0043 => DIRECT3D11::AEERR_DEVICE_IN_USE,
            0x887C0044 => DIRECT3D11::AEERR_PERFLIB_NOFUNC,
            0x887C0045 => DIRECT3D11::AEERR_CP_SHARED_BUFFER_CORRUPT,
            0x887C0046 => DIRECT3D11::AEERR_ADG_INVALID_PROCESSING_PERIOD,
            0x887C0047 => DIRECT3D11::AEERR_CP_QUEUE_FULL,
            0x887C0048 => DIRECT3D11::AEERR_APO_INVALIDARG,
            0x887C0049 => DIRECT3D11::AEERR_NO_HEAP,
            0x887C004A => DIRECT3D11::AEERR_HEAP_ALREADY_CREATED,
            0x887C004B => DIRECT3D11::AEERR_ACTIVE_STREAM_COUNT_ERR,
            0x887C00AA => DIRECT3D11::AEERR_ENDPOINT_IN_USE,
        }
    }
}
