/*
 * Copyright 2023 Stanislav Mikhailov (xavetar)
 *
 * Licensed under the Creative Commons Zero v1.0 Universal (CC0) License.
 * You may obtain a copy of the License at
 *
 *     http://creativecommons.org/publicdomain/zero/1.0/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the CC0 license is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// It is strictly forbidden to use the from_code method, respected Microsoft decided to define
// duplicate codes, but by defining different constant names, therefore, when using these codes,
// it is mandatory to use through from_name, and not from_code. Otherwise, it may cause undefined
// behavior or an unknown exception. Because one code corresponds to several constants.

use super::{RawError};

#[derive(Clone, Debug, Copy, Eq, Hash, Ord, PartialEq, PartialOrd)]
pub enum DIRECT_DRAW {
    DDERR_ALREADYINITIALIZED,
    DDERR_CANNOTATTACHSURFACE,
    DDERR_CANNOTDETACHSURFACE,
    DDERR_CURRENTLYNOTAVAIL,
    DDERR_EXCEPTION,
    DDERR_HEIGHTALIGN,
    DDERR_INCOMPATIBLEPRIMARY,
    DDERR_INVALIDCAPS,
    DDERR_INVALIDCLIPLIST,
    DDERR_INVALIDMODE,
    DDERR_INVALIDOBJECT,
    DDERR_INVALIDPIXELFORMAT,
    DDERR_INVALIDRECT,
    DDERR_LOCKEDSURFACES,
    DDERR_NO3D,
    DDERR_NOALPHAHW,
    DDERR_NOSTEREOHARDWARE,
    DDERR_NOSURFACELEFT,
    DDERR_NOCLIPLIST,
    DDERR_NOCOLORCONVHW,
    DDERR_NOCOOPERATIVELEVELSET,
    DDERR_NOCOLORKEY,
    DDERR_NOCOLORKEYHW,
    DDERR_NODIRECTDRAWSUPPORT,
    DDERR_NOEXCLUSIVEMODE,
    DDERR_NOFLIPHW,
    DDERR_NOGDI,
    DDERR_NOMIRRORHW,
    DDERR_NOTFOUND,
    DDERR_NOOVERLAYHW,
    DDERR_OVERLAPPINGRECTS,
    DDERR_NORASTEROPHW,
    DDERR_NOROTATIONHW,
    DDERR_NOSTRETCHHW,
    DDERR_NOT4BITCOLOR,
    DDERR_NOT4BITCOLORINDEX,
    DDERR_NOT8BITCOLOR,
    DDERR_NOTEXTUREHW,
    DDERR_NOVSYNCHW,
    DDERR_NOZBUFFERHW,
    DDERR_NOZOVERLAYHW,
    DDERR_OUTOFCAPS,
    DDERR_OUTOFVIDEOMEMORY,
    DDERR_OVERLAYCANTCLIP,
    DDERR_OVERLAYCOLORKEYONLYONEACTIVE,
    DDERR_PALETTEBUSY,
    DDERR_COLORKEYNOTSET,
    DDERR_SURFACEALREADYATTACHED,
    DDERR_SURFACEALREADYDEPENDENT,
    DDERR_SURFACEBUSY,
    DDERR_CANTLOCKSURFACE,
    DDERR_SURFACEISOBSCURED,
    DDERR_SURFACELOST,
    DDERR_SURFACENOTATTACHED,
    DDERR_TOOBIGHEIGHT,
    DDERR_TOOBIGSIZE,
    DDERR_TOOBIGWIDTH,
    DDERR_UNSUPPORTEDFORMAT,
    DDERR_UNSUPPORTEDMASK,
    DDERR_INVALIDSTREAM,
    DDERR_VERTICALBLANKINPROGRESS,
    DDERR_WASSTILLDRAWING,
    DDERR_DDSCAPSCOMPLEXREQUIRED,
    DDERR_XALIGN,
    DDERR_INVALIDDIRECTDRAWGUID,
    DDERR_DIRECTDRAWALREADYCREATED,
    DDERR_NODIRECTDRAWHW,
    DDERR_PRIMARYSURFACEALREADYEXISTS,
    DDERR_NOEMULATION,
    DDERR_REGIONTOOSMALL,
    DDERR_CLIPPERISUSINGHWND,
    DDERR_NOCLIPPERATTACHED,
    DDERR_NOHWND,
    DDERR_HWNDSUBCLASSED,
    DDERR_HWNDALREADYSET,
    DDERR_NOPALETTEATTACHED,
    DDERR_NOPALETTEHW,
    DDERR_BLTFASTCANTCLIP,
    DDERR_NOBLTHW,
    DDERR_NODDROPSHW,
    DDERR_OVERLAYNOTVISIBLE,
    DDERR_NOOVERLAYDEST,
    DDERR_INVALIDPOSITION,
    DDERR_NOTAOVERLAYSURFACE,
    DDERR_EXCLUSIVEMODEALREADYSET,
    DDERR_NOTFLIPPABLE,
    DDERR_CANTDUPLICATE,
    DDERR_NOTLOCKED,
    DDERR_CANTCREATEDC,
    DDERR_NODC,
    DDERR_WRONGMODE,
    DDERR_IMPLICITLYCREATED,
    DDERR_NOTPALETTIZED,
    DDERR_UNSUPPORTEDMODE,
    DDERR_NOMIPMAPHW,
    DDERR_INVALIDSURFACETYPE,
    DDERR_NOOPTIMIZEHW,
    DDERR_NOTLOADED,
    DDERR_NOFOCUSWINDOW,
    DDERR_NOTONMIPMAPSUBLEVEL,
    DDERR_DCALREADYCREATED,
    DDERR_NONONLOCALVIDMEM,
    DDERR_CANTPAGELOCK,
    DDERR_CANTPAGEUNLOCK,
    DDERR_NOTPAGELOCKED,
    DDERR_MOREDATA,
    DDERR_EXPIRED,
    DDERR_TESTFINISHED,
    DDERR_NEWMODE,
    DDERR_D3DNOTINITIALIZED,
    DDERR_VIDEONOTACTIVE,
    DDERR_NOMONITORINFORMATION,
    DDERR_NODRIVERSUPPORT,
    DDERR_DEVICEDOESNTOWNSURFACE,
}

impl DIRECT_DRAW {
    pub fn code(&self) -> u32 {
        return match self {
            DIRECT_DRAW::DDERR_ALREADYINITIALIZED => 0x88760005 as u32,
            DIRECT_DRAW::DDERR_CANNOTATTACHSURFACE => 0x8876000A as u32,
            DIRECT_DRAW::DDERR_CANNOTDETACHSURFACE => 0x88760014 as u32,
            DIRECT_DRAW::DDERR_CURRENTLYNOTAVAIL => 0x88760028 as u32,
            DIRECT_DRAW::DDERR_EXCEPTION => 0x88760037 as u32,
            DIRECT_DRAW::DDERR_HEIGHTALIGN => 0x8876005A as u32,
            DIRECT_DRAW::DDERR_INCOMPATIBLEPRIMARY => 0x8876005F as u32,
            DIRECT_DRAW::DDERR_INVALIDCAPS => 0x88760064 as u32,
            DIRECT_DRAW::DDERR_INVALIDCLIPLIST => 0x8876006E as u32,
            DIRECT_DRAW::DDERR_INVALIDMODE => 0x88760078 as u32,
            DIRECT_DRAW::DDERR_INVALIDOBJECT => 0x88760082 as u32,
            DIRECT_DRAW::DDERR_INVALIDPIXELFORMAT => 0x88760091 as u32,
            DIRECT_DRAW::DDERR_INVALIDRECT => 0x88760096 as u32,
            DIRECT_DRAW::DDERR_LOCKEDSURFACES => 0x887600A0 as u32,
            DIRECT_DRAW::DDERR_NO3D => 0x887600AA as u32,
            DIRECT_DRAW::DDERR_NOALPHAHW => 0x887600B4 as u32,
            DIRECT_DRAW::DDERR_NOSTEREOHARDWARE => 0x887600B5 as u32,
            DIRECT_DRAW::DDERR_NOSURFACELEFT => 0x887600B6 as u32,
            DIRECT_DRAW::DDERR_NOCLIPLIST => 0x887600CD as u32,
            DIRECT_DRAW::DDERR_NOCOLORCONVHW => 0x887600D2 as u32,
            DIRECT_DRAW::DDERR_NOCOOPERATIVELEVELSET => 0x887600D4 as u32,
            DIRECT_DRAW::DDERR_NOCOLORKEY => 0x887600D7 as u32,
            DIRECT_DRAW::DDERR_NOCOLORKEYHW => 0x887600DC as u32,
            DIRECT_DRAW::DDERR_NODIRECTDRAWSUPPORT => 0x887600DE as u32,
            DIRECT_DRAW::DDERR_NOEXCLUSIVEMODE => 0x887600E1 as u32,
            DIRECT_DRAW::DDERR_NOFLIPHW => 0x887600E6 as u32,
            DIRECT_DRAW::DDERR_NOGDI => 0x887600F0 as u32,
            DIRECT_DRAW::DDERR_NOMIRRORHW => 0x887600FA as u32,
            DIRECT_DRAW::DDERR_NOTFOUND => 0x887600FF as u32,
            DIRECT_DRAW::DDERR_NOOVERLAYHW => 0x88760104 as u32,
            DIRECT_DRAW::DDERR_OVERLAPPINGRECTS => 0x8876010E as u32,
            DIRECT_DRAW::DDERR_NORASTEROPHW => 0x88760118 as u32,
            DIRECT_DRAW::DDERR_NOROTATIONHW => 0x88760122 as u32,
            DIRECT_DRAW::DDERR_NOSTRETCHHW => 0x88760136 as u32,
            DIRECT_DRAW::DDERR_NOT4BITCOLOR => 0x8876013C as u32,
            DIRECT_DRAW::DDERR_NOT4BITCOLORINDEX => 0x8876013D as u32,
            DIRECT_DRAW::DDERR_NOT8BITCOLOR => 0x88760140 as u32,
            DIRECT_DRAW::DDERR_NOTEXTUREHW => 0x8876014A as u32,
            DIRECT_DRAW::DDERR_NOVSYNCHW => 0x8876014F as u32,
            DIRECT_DRAW::DDERR_NOZBUFFERHW => 0x88760154 as u32,
            DIRECT_DRAW::DDERR_NOZOVERLAYHW => 0x8876015E as u32,
            DIRECT_DRAW::DDERR_OUTOFCAPS => 0x88760168 as u32,
            DIRECT_DRAW::DDERR_OUTOFVIDEOMEMORY => 0x8876017C as u32,
            DIRECT_DRAW::DDERR_OVERLAYCANTCLIP => 0x8876017E as u32,
            DIRECT_DRAW::DDERR_OVERLAYCOLORKEYONLYONEACTIVE => 0x88760180 as u32,
            DIRECT_DRAW::DDERR_PALETTEBUSY => 0x88760183 as u32,
            DIRECT_DRAW::DDERR_COLORKEYNOTSET => 0x88760190 as u32,
            DIRECT_DRAW::DDERR_SURFACEALREADYATTACHED => 0x8876019A as u32,
            DIRECT_DRAW::DDERR_SURFACEALREADYDEPENDENT => 0x887601A4 as u32,
            DIRECT_DRAW::DDERR_SURFACEBUSY => 0x887601AE as u32,
            DIRECT_DRAW::DDERR_CANTLOCKSURFACE => 0x887601B3 as u32,
            DIRECT_DRAW::DDERR_SURFACEISOBSCURED => 0x887601B8 as u32,
            DIRECT_DRAW::DDERR_SURFACELOST => 0x887601C2 as u32,
            DIRECT_DRAW::DDERR_SURFACENOTATTACHED => 0x887601CC as u32,
            DIRECT_DRAW::DDERR_TOOBIGHEIGHT => 0x887601D6 as u32,
            DIRECT_DRAW::DDERR_TOOBIGSIZE => 0x887601E0 as u32,
            DIRECT_DRAW::DDERR_TOOBIGWIDTH => 0x887601EA as u32,
            DIRECT_DRAW::DDERR_UNSUPPORTEDFORMAT => 0x887601FE as u32,
            DIRECT_DRAW::DDERR_UNSUPPORTEDMASK => 0x88760208 as u32,
            DIRECT_DRAW::DDERR_INVALIDSTREAM => 0x88760209 as u32,
            DIRECT_DRAW::DDERR_VERTICALBLANKINPROGRESS => 0x88760219 as u32,
            DIRECT_DRAW::DDERR_WASSTILLDRAWING => 0x8876021C as u32,
            DIRECT_DRAW::DDERR_DDSCAPSCOMPLEXREQUIRED => 0x8876021E as u32,
            DIRECT_DRAW::DDERR_XALIGN => 0x88760230 as u32,
            DIRECT_DRAW::DDERR_INVALIDDIRECTDRAWGUID => 0x88760231 as u32,
            DIRECT_DRAW::DDERR_DIRECTDRAWALREADYCREATED => 0x88760232 as u32,
            DIRECT_DRAW::DDERR_NODIRECTDRAWHW => 0x88760233 as u32,
            DIRECT_DRAW::DDERR_PRIMARYSURFACEALREADYEXISTS => 0x88760234 as u32,
            DIRECT_DRAW::DDERR_NOEMULATION => 0x88760235 as u32,
            DIRECT_DRAW::DDERR_REGIONTOOSMALL => 0x88760236 as u32,
            DIRECT_DRAW::DDERR_CLIPPERISUSINGHWND => 0x88760237 as u32,
            DIRECT_DRAW::DDERR_NOCLIPPERATTACHED => 0x88760238 as u32,
            DIRECT_DRAW::DDERR_NOHWND => 0x88760239 as u32,
            DIRECT_DRAW::DDERR_HWNDSUBCLASSED => 0x8876023A as u32,
            DIRECT_DRAW::DDERR_HWNDALREADYSET => 0x8876023B as u32,
            DIRECT_DRAW::DDERR_NOPALETTEATTACHED => 0x8876023C as u32,
            DIRECT_DRAW::DDERR_NOPALETTEHW => 0x8876023D as u32,
            DIRECT_DRAW::DDERR_BLTFASTCANTCLIP => 0x8876023E as u32,
            DIRECT_DRAW::DDERR_NOBLTHW => 0x8876023F as u32,
            DIRECT_DRAW::DDERR_NODDROPSHW => 0x88760240 as u32,
            DIRECT_DRAW::DDERR_OVERLAYNOTVISIBLE => 0x88760241 as u32,
            DIRECT_DRAW::DDERR_NOOVERLAYDEST => 0x88760242 as u32,
            DIRECT_DRAW::DDERR_INVALIDPOSITION => 0x88760243 as u32,
            DIRECT_DRAW::DDERR_NOTAOVERLAYSURFACE => 0x88760244 as u32,
            DIRECT_DRAW::DDERR_EXCLUSIVEMODEALREADYSET => 0x88760245 as u32,
            DIRECT_DRAW::DDERR_NOTFLIPPABLE => 0x88760246 as u32,
            DIRECT_DRAW::DDERR_CANTDUPLICATE => 0x88760247 as u32,
            DIRECT_DRAW::DDERR_NOTLOCKED => 0x88760248 as u32,
            DIRECT_DRAW::DDERR_CANTCREATEDC => 0x88760249 as u32,
            DIRECT_DRAW::DDERR_NODC => 0x8876024A as u32,
            DIRECT_DRAW::DDERR_WRONGMODE => 0x8876024B as u32,
            DIRECT_DRAW::DDERR_IMPLICITLYCREATED => 0x8876024C as u32,
            DIRECT_DRAW::DDERR_NOTPALETTIZED => 0x8876024D as u32,
            DIRECT_DRAW::DDERR_UNSUPPORTEDMODE => 0x8876024E as u32,
            DIRECT_DRAW::DDERR_NOMIPMAPHW => 0x8876024F as u32,
            DIRECT_DRAW::DDERR_INVALIDSURFACETYPE => 0x88760250 as u32,
            DIRECT_DRAW::DDERR_NOOPTIMIZEHW => 0x88760258 as u32,
            DIRECT_DRAW::DDERR_NOTLOADED => 0x88760259 as u32,
            DIRECT_DRAW::DDERR_NOFOCUSWINDOW => 0x8876025A as u32,
            DIRECT_DRAW::DDERR_NOTONMIPMAPSUBLEVEL => 0x8876025B as u32,
            DIRECT_DRAW::DDERR_DCALREADYCREATED => 0x8876026C as u32,
            DIRECT_DRAW::DDERR_NONONLOCALVIDMEM => 0x88760276 as u32,
            DIRECT_DRAW::DDERR_CANTPAGELOCK => 0x88760280 as u32,
            DIRECT_DRAW::DDERR_CANTPAGEUNLOCK => 0x88760294 as u32,
            DIRECT_DRAW::DDERR_NOTPAGELOCKED => 0x887602A8 as u32,
            DIRECT_DRAW::DDERR_MOREDATA => 0x887602B2 as u32,
            DIRECT_DRAW::DDERR_EXPIRED => 0x887602B3 as u32,
            DIRECT_DRAW::DDERR_TESTFINISHED => 0x887602B4 as u32,
            DIRECT_DRAW::DDERR_NEWMODE => 0x887602B5 as u32,
            DIRECT_DRAW::DDERR_D3DNOTINITIALIZED => 0x887602B6 as u32,
            DIRECT_DRAW::DDERR_VIDEONOTACTIVE => 0x887602B7 as u32,
            DIRECT_DRAW::DDERR_NOMONITORINFORMATION => 0x887602B8 as u32,
            DIRECT_DRAW::DDERR_NODRIVERSUPPORT => 0x887602B9 as u32,
            DIRECT_DRAW::DDERR_DEVICEDOESNTOWNSURFACE => 0x887602BB as u32,
        }
    }

    pub fn error(&self) -> RawError {
        return match self {
            DIRECT_DRAW::DDERR_ALREADYINITIALIZED => RawError::Kind(DIRECT_DRAW::DDERR_ALREADYINITIALIZED),
            DIRECT_DRAW::DDERR_CANNOTATTACHSURFACE => RawError::Kind(DIRECT_DRAW::DDERR_CANNOTATTACHSURFACE),
            DIRECT_DRAW::DDERR_CANNOTDETACHSURFACE => RawError::Kind(DIRECT_DRAW::DDERR_CANNOTDETACHSURFACE),
            DIRECT_DRAW::DDERR_CURRENTLYNOTAVAIL => RawError::Kind(DIRECT_DRAW::DDERR_CURRENTLYNOTAVAIL),
            DIRECT_DRAW::DDERR_EXCEPTION => RawError::Kind(DIRECT_DRAW::DDERR_EXCEPTION),
            DIRECT_DRAW::DDERR_HEIGHTALIGN => RawError::Kind(DIRECT_DRAW::DDERR_HEIGHTALIGN),
            DIRECT_DRAW::DDERR_INCOMPATIBLEPRIMARY => RawError::Kind(DIRECT_DRAW::DDERR_INCOMPATIBLEPRIMARY),
            DIRECT_DRAW::DDERR_INVALIDCAPS => RawError::Kind(DIRECT_DRAW::DDERR_INVALIDCAPS),
            DIRECT_DRAW::DDERR_INVALIDCLIPLIST => RawError::Kind(DIRECT_DRAW::DDERR_INVALIDCLIPLIST),
            DIRECT_DRAW::DDERR_INVALIDMODE => RawError::Kind(DIRECT_DRAW::DDERR_INVALIDMODE),
            DIRECT_DRAW::DDERR_INVALIDOBJECT => RawError::Kind(DIRECT_DRAW::DDERR_INVALIDOBJECT),
            DIRECT_DRAW::DDERR_INVALIDPIXELFORMAT => RawError::Kind(DIRECT_DRAW::DDERR_INVALIDPIXELFORMAT),
            DIRECT_DRAW::DDERR_INVALIDRECT => RawError::Kind(DIRECT_DRAW::DDERR_INVALIDRECT),
            DIRECT_DRAW::DDERR_LOCKEDSURFACES => RawError::Kind(DIRECT_DRAW::DDERR_LOCKEDSURFACES),
            DIRECT_DRAW::DDERR_NO3D => RawError::Kind(DIRECT_DRAW::DDERR_NO3D),
            DIRECT_DRAW::DDERR_NOALPHAHW => RawError::Kind(DIRECT_DRAW::DDERR_NOALPHAHW),
            DIRECT_DRAW::DDERR_NOSTEREOHARDWARE => RawError::Kind(DIRECT_DRAW::DDERR_NOSTEREOHARDWARE),
            DIRECT_DRAW::DDERR_NOSURFACELEFT => RawError::Kind(DIRECT_DRAW::DDERR_NOSURFACELEFT),
            DIRECT_DRAW::DDERR_NOCLIPLIST => RawError::Kind(DIRECT_DRAW::DDERR_NOCLIPLIST),
            DIRECT_DRAW::DDERR_NOCOLORCONVHW => RawError::Kind(DIRECT_DRAW::DDERR_NOCOLORCONVHW),
            DIRECT_DRAW::DDERR_NOCOOPERATIVELEVELSET => RawError::Kind(DIRECT_DRAW::DDERR_NOCOOPERATIVELEVELSET),
            DIRECT_DRAW::DDERR_NOCOLORKEY => RawError::Kind(DIRECT_DRAW::DDERR_NOCOLORKEY),
            DIRECT_DRAW::DDERR_NOCOLORKEYHW => RawError::Kind(DIRECT_DRAW::DDERR_NOCOLORKEYHW),
            DIRECT_DRAW::DDERR_NODIRECTDRAWSUPPORT => RawError::Kind(DIRECT_DRAW::DDERR_NODIRECTDRAWSUPPORT),
            DIRECT_DRAW::DDERR_NOEXCLUSIVEMODE => RawError::Kind(DIRECT_DRAW::DDERR_NOEXCLUSIVEMODE),
            DIRECT_DRAW::DDERR_NOFLIPHW => RawError::Kind(DIRECT_DRAW::DDERR_NOFLIPHW),
            DIRECT_DRAW::DDERR_NOGDI => RawError::Kind(DIRECT_DRAW::DDERR_NOGDI),
            DIRECT_DRAW::DDERR_NOMIRRORHW => RawError::Kind(DIRECT_DRAW::DDERR_NOMIRRORHW),
            DIRECT_DRAW::DDERR_NOTFOUND => RawError::Kind(DIRECT_DRAW::DDERR_NOTFOUND),
            DIRECT_DRAW::DDERR_NOOVERLAYHW => RawError::Kind(DIRECT_DRAW::DDERR_NOOVERLAYHW),
            DIRECT_DRAW::DDERR_OVERLAPPINGRECTS => RawError::Kind(DIRECT_DRAW::DDERR_OVERLAPPINGRECTS),
            DIRECT_DRAW::DDERR_NORASTEROPHW => RawError::Kind(DIRECT_DRAW::DDERR_NORASTEROPHW),
            DIRECT_DRAW::DDERR_NOROTATIONHW => RawError::Kind(DIRECT_DRAW::DDERR_NOROTATIONHW),
            DIRECT_DRAW::DDERR_NOSTRETCHHW => RawError::Kind(DIRECT_DRAW::DDERR_NOSTRETCHHW),
            DIRECT_DRAW::DDERR_NOT4BITCOLOR => RawError::Kind(DIRECT_DRAW::DDERR_NOT4BITCOLOR),
            DIRECT_DRAW::DDERR_NOT4BITCOLORINDEX => RawError::Kind(DIRECT_DRAW::DDERR_NOT4BITCOLORINDEX),
            DIRECT_DRAW::DDERR_NOT8BITCOLOR => RawError::Kind(DIRECT_DRAW::DDERR_NOT8BITCOLOR),
            DIRECT_DRAW::DDERR_NOTEXTUREHW => RawError::Kind(DIRECT_DRAW::DDERR_NOTEXTUREHW),
            DIRECT_DRAW::DDERR_NOVSYNCHW => RawError::Kind(DIRECT_DRAW::DDERR_NOVSYNCHW),
            DIRECT_DRAW::DDERR_NOZBUFFERHW => RawError::Kind(DIRECT_DRAW::DDERR_NOZBUFFERHW),
            DIRECT_DRAW::DDERR_NOZOVERLAYHW => RawError::Kind(DIRECT_DRAW::DDERR_NOZOVERLAYHW),
            DIRECT_DRAW::DDERR_OUTOFCAPS => RawError::Kind(DIRECT_DRAW::DDERR_OUTOFCAPS),
            DIRECT_DRAW::DDERR_OUTOFVIDEOMEMORY => RawError::Kind(DIRECT_DRAW::DDERR_OUTOFVIDEOMEMORY),
            DIRECT_DRAW::DDERR_OVERLAYCANTCLIP => RawError::Kind(DIRECT_DRAW::DDERR_OVERLAYCANTCLIP),
            DIRECT_DRAW::DDERR_OVERLAYCOLORKEYONLYONEACTIVE => RawError::Kind(DIRECT_DRAW::DDERR_OVERLAYCOLORKEYONLYONEACTIVE),
            DIRECT_DRAW::DDERR_PALETTEBUSY => RawError::Kind(DIRECT_DRAW::DDERR_PALETTEBUSY),
            DIRECT_DRAW::DDERR_COLORKEYNOTSET => RawError::Kind(DIRECT_DRAW::DDERR_COLORKEYNOTSET),
            DIRECT_DRAW::DDERR_SURFACEALREADYATTACHED => RawError::Kind(DIRECT_DRAW::DDERR_SURFACEALREADYATTACHED),
            DIRECT_DRAW::DDERR_SURFACEALREADYDEPENDENT => RawError::Kind(DIRECT_DRAW::DDERR_SURFACEALREADYDEPENDENT),
            DIRECT_DRAW::DDERR_SURFACEBUSY => RawError::Kind(DIRECT_DRAW::DDERR_SURFACEBUSY),
            DIRECT_DRAW::DDERR_CANTLOCKSURFACE => RawError::Kind(DIRECT_DRAW::DDERR_CANTLOCKSURFACE),
            DIRECT_DRAW::DDERR_SURFACEISOBSCURED => RawError::Kind(DIRECT_DRAW::DDERR_SURFACEISOBSCURED),
            DIRECT_DRAW::DDERR_SURFACELOST => RawError::Kind(DIRECT_DRAW::DDERR_SURFACELOST),
            DIRECT_DRAW::DDERR_SURFACENOTATTACHED => RawError::Kind(DIRECT_DRAW::DDERR_SURFACENOTATTACHED),
            DIRECT_DRAW::DDERR_TOOBIGHEIGHT => RawError::Kind(DIRECT_DRAW::DDERR_TOOBIGHEIGHT),
            DIRECT_DRAW::DDERR_TOOBIGSIZE => RawError::Kind(DIRECT_DRAW::DDERR_TOOBIGSIZE),
            DIRECT_DRAW::DDERR_TOOBIGWIDTH => RawError::Kind(DIRECT_DRAW::DDERR_TOOBIGWIDTH),
            DIRECT_DRAW::DDERR_UNSUPPORTEDFORMAT => RawError::Kind(DIRECT_DRAW::DDERR_UNSUPPORTEDFORMAT),
            DIRECT_DRAW::DDERR_UNSUPPORTEDMASK => RawError::Kind(DIRECT_DRAW::DDERR_UNSUPPORTEDMASK),
            DIRECT_DRAW::DDERR_INVALIDSTREAM => RawError::Kind(DIRECT_DRAW::DDERR_INVALIDSTREAM),
            DIRECT_DRAW::DDERR_VERTICALBLANKINPROGRESS => RawError::Kind(DIRECT_DRAW::DDERR_VERTICALBLANKINPROGRESS),
            DIRECT_DRAW::DDERR_WASSTILLDRAWING => RawError::Kind(DIRECT_DRAW::DDERR_WASSTILLDRAWING),
            DIRECT_DRAW::DDERR_DDSCAPSCOMPLEXREQUIRED => RawError::Kind(DIRECT_DRAW::DDERR_DDSCAPSCOMPLEXREQUIRED),
            DIRECT_DRAW::DDERR_XALIGN => RawError::Kind(DIRECT_DRAW::DDERR_XALIGN),
            DIRECT_DRAW::DDERR_INVALIDDIRECTDRAWGUID => RawError::Kind(DIRECT_DRAW::DDERR_INVALIDDIRECTDRAWGUID),
            DIRECT_DRAW::DDERR_DIRECTDRAWALREADYCREATED => RawError::Kind(DIRECT_DRAW::DDERR_DIRECTDRAWALREADYCREATED),
            DIRECT_DRAW::DDERR_NODIRECTDRAWHW => RawError::Kind(DIRECT_DRAW::DDERR_NODIRECTDRAWHW),
            DIRECT_DRAW::DDERR_PRIMARYSURFACEALREADYEXISTS => RawError::Kind(DIRECT_DRAW::DDERR_PRIMARYSURFACEALREADYEXISTS),
            DIRECT_DRAW::DDERR_NOEMULATION => RawError::Kind(DIRECT_DRAW::DDERR_NOEMULATION),
            DIRECT_DRAW::DDERR_REGIONTOOSMALL => RawError::Kind(DIRECT_DRAW::DDERR_REGIONTOOSMALL),
            DIRECT_DRAW::DDERR_CLIPPERISUSINGHWND => RawError::Kind(DIRECT_DRAW::DDERR_CLIPPERISUSINGHWND),
            DIRECT_DRAW::DDERR_NOCLIPPERATTACHED => RawError::Kind(DIRECT_DRAW::DDERR_NOCLIPPERATTACHED),
            DIRECT_DRAW::DDERR_NOHWND => RawError::Kind(DIRECT_DRAW::DDERR_NOHWND),
            DIRECT_DRAW::DDERR_HWNDSUBCLASSED => RawError::Kind(DIRECT_DRAW::DDERR_HWNDSUBCLASSED),
            DIRECT_DRAW::DDERR_HWNDALREADYSET => RawError::Kind(DIRECT_DRAW::DDERR_HWNDALREADYSET),
            DIRECT_DRAW::DDERR_NOPALETTEATTACHED => RawError::Kind(DIRECT_DRAW::DDERR_NOPALETTEATTACHED),
            DIRECT_DRAW::DDERR_NOPALETTEHW => RawError::Kind(DIRECT_DRAW::DDERR_NOPALETTEHW),
            DIRECT_DRAW::DDERR_BLTFASTCANTCLIP => RawError::Kind(DIRECT_DRAW::DDERR_BLTFASTCANTCLIP),
            DIRECT_DRAW::DDERR_NOBLTHW => RawError::Kind(DIRECT_DRAW::DDERR_NOBLTHW),
            DIRECT_DRAW::DDERR_NODDROPSHW => RawError::Kind(DIRECT_DRAW::DDERR_NODDROPSHW),
            DIRECT_DRAW::DDERR_OVERLAYNOTVISIBLE => RawError::Kind(DIRECT_DRAW::DDERR_OVERLAYNOTVISIBLE),
            DIRECT_DRAW::DDERR_NOOVERLAYDEST => RawError::Kind(DIRECT_DRAW::DDERR_NOOVERLAYDEST),
            DIRECT_DRAW::DDERR_INVALIDPOSITION => RawError::Kind(DIRECT_DRAW::DDERR_INVALIDPOSITION),
            DIRECT_DRAW::DDERR_NOTAOVERLAYSURFACE => RawError::Kind(DIRECT_DRAW::DDERR_NOTAOVERLAYSURFACE),
            DIRECT_DRAW::DDERR_EXCLUSIVEMODEALREADYSET => RawError::Kind(DIRECT_DRAW::DDERR_EXCLUSIVEMODEALREADYSET),
            DIRECT_DRAW::DDERR_NOTFLIPPABLE => RawError::Kind(DIRECT_DRAW::DDERR_NOTFLIPPABLE),
            DIRECT_DRAW::DDERR_CANTDUPLICATE => RawError::Kind(DIRECT_DRAW::DDERR_CANTDUPLICATE),
            DIRECT_DRAW::DDERR_NOTLOCKED => RawError::Kind(DIRECT_DRAW::DDERR_NOTLOCKED),
            DIRECT_DRAW::DDERR_CANTCREATEDC => RawError::Kind(DIRECT_DRAW::DDERR_CANTCREATEDC),
            DIRECT_DRAW::DDERR_NODC => RawError::Kind(DIRECT_DRAW::DDERR_NODC),
            DIRECT_DRAW::DDERR_WRONGMODE => RawError::Kind(DIRECT_DRAW::DDERR_WRONGMODE),
            DIRECT_DRAW::DDERR_IMPLICITLYCREATED => RawError::Kind(DIRECT_DRAW::DDERR_IMPLICITLYCREATED),
            DIRECT_DRAW::DDERR_NOTPALETTIZED => RawError::Kind(DIRECT_DRAW::DDERR_NOTPALETTIZED),
            DIRECT_DRAW::DDERR_UNSUPPORTEDMODE => RawError::Kind(DIRECT_DRAW::DDERR_UNSUPPORTEDMODE),
            DIRECT_DRAW::DDERR_NOMIPMAPHW => RawError::Kind(DIRECT_DRAW::DDERR_NOMIPMAPHW),
            DIRECT_DRAW::DDERR_INVALIDSURFACETYPE => RawError::Kind(DIRECT_DRAW::DDERR_INVALIDSURFACETYPE),
            DIRECT_DRAW::DDERR_NOOPTIMIZEHW => RawError::Kind(DIRECT_DRAW::DDERR_NOOPTIMIZEHW),
            DIRECT_DRAW::DDERR_NOTLOADED => RawError::Kind(DIRECT_DRAW::DDERR_NOTLOADED),
            DIRECT_DRAW::DDERR_NOFOCUSWINDOW => RawError::Kind(DIRECT_DRAW::DDERR_NOFOCUSWINDOW),
            DIRECT_DRAW::DDERR_NOTONMIPMAPSUBLEVEL => RawError::Kind(DIRECT_DRAW::DDERR_NOTONMIPMAPSUBLEVEL),
            DIRECT_DRAW::DDERR_DCALREADYCREATED => RawError::Kind(DIRECT_DRAW::DDERR_DCALREADYCREATED),
            DIRECT_DRAW::DDERR_NONONLOCALVIDMEM => RawError::Kind(DIRECT_DRAW::DDERR_NONONLOCALVIDMEM),
            DIRECT_DRAW::DDERR_CANTPAGELOCK => RawError::Kind(DIRECT_DRAW::DDERR_CANTPAGELOCK),
            DIRECT_DRAW::DDERR_CANTPAGEUNLOCK => RawError::Kind(DIRECT_DRAW::DDERR_CANTPAGEUNLOCK),
            DIRECT_DRAW::DDERR_NOTPAGELOCKED => RawError::Kind(DIRECT_DRAW::DDERR_NOTPAGELOCKED),
            DIRECT_DRAW::DDERR_MOREDATA => RawError::Kind(DIRECT_DRAW::DDERR_MOREDATA),
            DIRECT_DRAW::DDERR_EXPIRED => RawError::Kind(DIRECT_DRAW::DDERR_EXPIRED),
            DIRECT_DRAW::DDERR_TESTFINISHED => RawError::Kind(DIRECT_DRAW::DDERR_TESTFINISHED),
            DIRECT_DRAW::DDERR_NEWMODE => RawError::Kind(DIRECT_DRAW::DDERR_NEWMODE),
            DIRECT_DRAW::DDERR_D3DNOTINITIALIZED => RawError::Kind(DIRECT_DRAW::DDERR_D3DNOTINITIALIZED),
            DIRECT_DRAW::DDERR_VIDEONOTACTIVE => RawError::Kind(DIRECT_DRAW::DDERR_VIDEONOTACTIVE),
            DIRECT_DRAW::DDERR_NOMONITORINFORMATION => RawError::Kind(DIRECT_DRAW::DDERR_NOMONITORINFORMATION),
            DIRECT_DRAW::DDERR_NODRIVERSUPPORT => RawError::Kind(DIRECT_DRAW::DDERR_NODRIVERSUPPORT),
            DIRECT_DRAW::DDERR_DEVICEDOESNTOWNSURFACE => RawError::Kind(DIRECT_DRAW::DDERR_DEVICEDOESNTOWNSURFACE),
        }
    }

    pub fn description(&self) -> &'static str {
        return match self {
            DIRECT_DRAW::DDERR_ALREADYINITIALIZED => "The object has already been initialized.",
            DIRECT_DRAW::DDERR_CANNOTATTACHSURFACE => "A surface cannot be attached to another requested surface.",
            DIRECT_DRAW::DDERR_CANNOTDETACHSURFACE => "A surface cannot be detached from another requested surface.",
            DIRECT_DRAW::DDERR_CURRENTLYNOTAVAIL => "No support is currently available.",
            DIRECT_DRAW::DDERR_EXCEPTION => "An exception was encountered while performing the requested operation.",
            DIRECT_DRAW::DDERR_HEIGHTALIGN => "The height of the provided rectangle is not a multiple of the required alignment.",
            DIRECT_DRAW::DDERR_INCOMPATIBLEPRIMARY => "The primary surface creation request does not match the existing primary surface.",
            DIRECT_DRAW::DDERR_INVALIDCAPS => "One or more of the capability bits passed to the callback function are incorrect.",
            DIRECT_DRAW::DDERR_INVALIDCLIPLIST => "DirectDraw does not support the provided clip list.",
            DIRECT_DRAW::DDERR_INVALIDMODE => "DirectDraw does not support the requested mode.",
            DIRECT_DRAW::DDERR_INVALIDOBJECT => "DirectDraw received a pointer that was an invalid DirectDraw object.",
            DIRECT_DRAW::DDERR_INVALIDPIXELFORMAT => "The pixel format was invalid as specified.",
            DIRECT_DRAW::DDERR_INVALIDRECT => "The provided rectangle was invalid.",
            DIRECT_DRAW::DDERR_LOCKEDSURFACES => "One or more surfaces are locked, causing the failure of the requested operation.",
            DIRECT_DRAW::DDERR_NO3D => "No 3-D hardware or emulation is present.",
            DIRECT_DRAW::DDERR_NOALPHAHW => "No alpha-acceleration hardware is present or available, causing the failure of the requested operation.",
            DIRECT_DRAW::DDERR_NOSTEREOHARDWARE => "There is no stereo hardware present or available.",
            DIRECT_DRAW::DDERR_NOSURFACELEFT => "There is no hardware present that supports stereo surfaces.",
            DIRECT_DRAW::DDERR_NOCLIPLIST => "No clip list is available.",
            DIRECT_DRAW::DDERR_NOCOLORCONVHW => "No color-conversion hardware is present or available.",
            DIRECT_DRAW::DDERR_NOCOOPERATIVELEVELSET => "A create function was called without the IDirectDraw7::SetCooperativeLevel method.",
            DIRECT_DRAW::DDERR_NOCOLORKEY => "The surface does not currently have a color key.",
            DIRECT_DRAW::DDERR_NOCOLORKEYHW => "There is no hardware support for the destination color key.",
            DIRECT_DRAW::DDERR_NODIRECTDRAWSUPPORT => "DirectDraw support is not possible with the current display driver.",
            DIRECT_DRAW::DDERR_NOEXCLUSIVEMODE => "The operation requires the application to have exclusive mode, but the application does not have exclusive mode.",
            DIRECT_DRAW::DDERR_NOFLIPHW => "Flipping visible surfaces is not supported.",
            DIRECT_DRAW::DDERR_NOGDI => "No GDI is present.",
            DIRECT_DRAW::DDERR_NOMIRRORHW => "No mirroring hardware is present or available.",
            DIRECT_DRAW::DDERR_NOTFOUND => "The requested item was not found.",
            DIRECT_DRAW::DDERR_NOOVERLAYHW => "No overlay hardware is present or available.",
            DIRECT_DRAW::DDERR_OVERLAPPINGRECTS => "The source and destination rectangles are on the same surface and overlap each other.",
            DIRECT_DRAW::DDERR_NORASTEROPHW => "No appropriate raster-operation hardware is present or available.",
            DIRECT_DRAW::DDERR_NOROTATIONHW => "No rotation hardware is present or available.",
            DIRECT_DRAW::DDERR_NOSTRETCHHW => "There is no hardware support for stretching.",
            DIRECT_DRAW::DDERR_NOT4BITCOLOR => "The DirectDrawSurface object is not using a 4-bit color palette, and the requested operation requires a 4-bit color palette.",
            DIRECT_DRAW::DDERR_NOT4BITCOLORINDEX => "The DirectDrawSurface object is not using a 4-bit color index palette, and the requested operation requires a 4-bit color index palette.",
            DIRECT_DRAW::DDERR_NOT8BITCOLOR => "The DirectDrawSurface object is not using an 8-bit color palette, and the requested operation requires an 8-bit color palette.",
            DIRECT_DRAW::DDERR_NOTEXTUREHW => "The operation cannot be carried out because no texture-mapping hardware is present or available.",
            DIRECT_DRAW::DDERR_NOVSYNCHW => "There is no hardware support for vertical blank&#x2013;synchronized operations.",
            DIRECT_DRAW::DDERR_NOZBUFFERHW => "The operation to create a z-buffer in display memory or to perform a bit block transfer (bitblt), using a z-buffer cannot be carried out because there is no hardware support for z-buffers.",
            DIRECT_DRAW::DDERR_NOZOVERLAYHW => "The overlay surfaces cannot be z-layered, based on the z-order because the hardware does not support z-ordering of overlays.",
            DIRECT_DRAW::DDERR_OUTOFCAPS => "The hardware needed for the requested operation has already been allocated.",
            DIRECT_DRAW::DDERR_OUTOFVIDEOMEMORY => "DirectDraw does not have enough display memory to perform the operation.",
            DIRECT_DRAW::DDERR_OVERLAYCANTCLIP => "The hardware does not support clipped overlays.",
            DIRECT_DRAW::DDERR_OVERLAYCOLORKEYONLYONEACTIVE => "An attempt was made to have more than one color key active on an overlay.",
            DIRECT_DRAW::DDERR_PALETTEBUSY => "Access to this palette is refused because the palette is locked by another thread.",
            DIRECT_DRAW::DDERR_COLORKEYNOTSET => "No source color key is specified for this operation.",
            DIRECT_DRAW::DDERR_SURFACEALREADYATTACHED => "An attempt was made to attach a surface to another surface to which it is already attached.",
            DIRECT_DRAW::DDERR_SURFACEALREADYDEPENDENT => "An attempt was made to make a surface a dependency of another surface on which it is already dependent.",
            DIRECT_DRAW::DDERR_SURFACEBUSY => "Access to the surface is refused because the surface is locked by another thread.",
            DIRECT_DRAW::DDERR_CANTLOCKSURFACE => "Access to this surface is refused because an attempt was made to lock the primary surface without Display Control Interface (DCI) support.",
            DIRECT_DRAW::DDERR_SURFACEISOBSCURED => "Access to the surface is refused because the surface is obscured.",
            DIRECT_DRAW::DDERR_SURFACELOST => "Access to the surface is refused because the surface memory is gone. Call the IDirectDrawSurface7::Restore method on this surface to restore the memory associated with it.",
            DIRECT_DRAW::DDERR_SURFACENOTATTACHED => "The requested surface is not attached.",
            DIRECT_DRAW::DDERR_TOOBIGHEIGHT => "The height requested by DirectDraw is too large.",
            DIRECT_DRAW::DDERR_TOOBIGSIZE => "The size requested by DirectDraw is too large. However, the individual height and width are valid sizes.",
            DIRECT_DRAW::DDERR_TOOBIGWIDTH => "The width requested by DirectDraw is too large.",
            DIRECT_DRAW::DDERR_UNSUPPORTEDFORMAT => "The pixel format requested is not supported by DirectDraw.",
            DIRECT_DRAW::DDERR_UNSUPPORTEDMASK => "The bitmask in the pixel format requested is not supported by DirectDraw.",
            DIRECT_DRAW::DDERR_INVALIDSTREAM => "The specified stream contains invalid data.",
            DIRECT_DRAW::DDERR_VERTICALBLANKINPROGRESS => "A vertical blank is in progress.",
            DIRECT_DRAW::DDERR_WASSTILLDRAWING => "The previous bitblt operation that is transferring information to or from this surface is incomplete.",
            DIRECT_DRAW::DDERR_DDSCAPSCOMPLEXREQUIRED => "New for DirectX 7.0. The surface requires the DDSCAPS_COMPLEX flag.",
            DIRECT_DRAW::DDERR_XALIGN => "The provided rectangle was not horizontally aligned on a required boundary.",
            DIRECT_DRAW::DDERR_INVALIDDIRECTDRAWGUID => "The globally unique identifier (GUID) passed to the DirectDrawCreate function is not a valid DirectDraw driver identifier.",
            DIRECT_DRAW::DDERR_DIRECTDRAWALREADYCREATED => "A DirectDraw object representing this driver has already been created for this process.",
            DIRECT_DRAW::DDERR_NODIRECTDRAWHW => "Hardware-only DirectDraw object creation is not possible; the driver does not support any hardware.",
            DIRECT_DRAW::DDERR_PRIMARYSURFACEALREADYEXISTS => "This process has already created a primary surface.",
            DIRECT_DRAW::DDERR_NOEMULATION => "Software emulation is not available.",
            DIRECT_DRAW::DDERR_REGIONTOOSMALL => "The region passed to the IDirectDrawClipper::GetClipList method is too small.",
            DIRECT_DRAW::DDERR_CLIPPERISUSINGHWND => "An attempt was made to set a clip list for a DirectDrawClipper object that is already monitoring a window handle.",
            DIRECT_DRAW::DDERR_NOCLIPPERATTACHED => "No DirectDrawClipper object is attached to the surface object.",
            DIRECT_DRAW::DDERR_NOHWND => "Clipper notification requires a window handle, or no window handle has been previously set as the cooperative level window handle.",
            DIRECT_DRAW::DDERR_HWNDSUBCLASSED => "DirectDraw is prevented from restoring state because the DirectDraw cooperative-level window handle has been subclassed.",
            DIRECT_DRAW::DDERR_HWNDALREADYSET => "The DirectDraw cooperative-level window handle has already been set. It cannot be reset while the process has surfaces or palettes created.",
            DIRECT_DRAW::DDERR_NOPALETTEATTACHED => "No palette object is attached to this surface.",
            DIRECT_DRAW::DDERR_NOPALETTEHW => "There is no hardware support for 16- or 256-color palettes.",
            DIRECT_DRAW::DDERR_BLTFASTCANTCLIP => "A DirectDrawClipper object is attached to a source surface that has passed into a call to the IDirectDrawSurface7::BltFast method.",
            DIRECT_DRAW::DDERR_NOBLTHW => "No bit block transferring hardware is present.",
            DIRECT_DRAW::DDERR_NODDROPSHW => "No DirectDraw raster-operation (ROP) hardware is available.",
            DIRECT_DRAW::DDERR_OVERLAYNOTVISIBLE => "The IDirectDrawSurface7::GetOverlayPosition method was called on a hidden overlay.",
            DIRECT_DRAW::DDERR_NOOVERLAYDEST => "The IDirectDrawSurface7::GetOverlayPosition method is called on an overlay that the IDirectDrawSurface7::UpdateOverlay method has not been called on to establish as a destination.",
            DIRECT_DRAW::DDERR_INVALIDPOSITION => "The position of the overlay on the destination is no longer valid.",
            DIRECT_DRAW::DDERR_NOTAOVERLAYSURFACE => "An overlay component is called for a nonoverlay surface.",
            DIRECT_DRAW::DDERR_EXCLUSIVEMODEALREADYSET => "An attempt was made to set the cooperative level when it was already set to exclusive.",
            DIRECT_DRAW::DDERR_NOTFLIPPABLE => "An attempt was made to flip a surface that cannot be flipped.",
            DIRECT_DRAW::DDERR_CANTDUPLICATE => "Primary and 3-D surfaces, or surfaces that are implicitly created, cannot be duplicated.",
            DIRECT_DRAW::DDERR_NOTLOCKED => "An attempt was made to unlock a surface that was not locked.",
            DIRECT_DRAW::DDERR_CANTCREATEDC => "Windows cannot create any more device contexts (DCs), or a DC has requested a palette-indexed surface when the surface had no palette and the display mode was not palette-indexed (in this case, DirectDraw cannot select a proper palette into the DC).",
            DIRECT_DRAW::DDERR_NODC => "No device context (DC) has ever been created for this surface.",
            DIRECT_DRAW::DDERR_WRONGMODE => "This surface cannot be restored because it was created in a different mode.",
            DIRECT_DRAW::DDERR_IMPLICITLYCREATED => "The surface cannot be restored because it is an implicitly created surface.",
            DIRECT_DRAW::DDERR_NOTPALETTIZED => "The surface being used is not a palette-based surface.",
            DIRECT_DRAW::DDERR_UNSUPPORTEDMODE => "The display is currently in an unsupported mode.",
            DIRECT_DRAW::DDERR_NOMIPMAPHW => "No mipmap-capable texture mapping hardware is present or available.",
            DIRECT_DRAW::DDERR_INVALIDSURFACETYPE => "The surface was of the wrong type.",
            DIRECT_DRAW::DDERR_NOOPTIMIZEHW => "The device does not support optimized surfaces.",
            DIRECT_DRAW::DDERR_NOTLOADED => "The surface is an optimized surface, but it has not yet been allocated any memory.",
            DIRECT_DRAW::DDERR_NOFOCUSWINDOW => "An attempt was made to create or set a device window without first setting the focus window.",
            DIRECT_DRAW::DDERR_NOTONMIPMAPSUBLEVEL => "Attempt was made to set a palette on a mipmap sublevel",
            DIRECT_DRAW::DDERR_DCALREADYCREATED => "A device context (DC) has already been returned for this surface. Only one DC can be retrieved for each surface.",
            DIRECT_DRAW::DDERR_NONONLOCALVIDMEM => "An attempt was made to allocate nonlocal video memory from a device that does not support nonlocal video memory.",
            DIRECT_DRAW::DDERR_CANTPAGELOCK => "An attempt to page-lock a surface failed. Page lock does not work on a display-memory surface or an emulated primary surface.",
            DIRECT_DRAW::DDERR_CANTPAGEUNLOCK => "An attempt to page-unlock a surface failed. Page unlock does not work on a display-memory surface or an emulated primary surface.",
            DIRECT_DRAW::DDERR_NOTPAGELOCKED => "An attempt was made to page-unlock a surface with no outstanding page locks.",
            DIRECT_DRAW::DDERR_MOREDATA => "There is more data available than the specified buffer size can hold.",
            DIRECT_DRAW::DDERR_EXPIRED => "The data has expired and is therefore no longer valid.",
            DIRECT_DRAW::DDERR_TESTFINISHED => "New for DirectX 7.0. When returned by the IDirectDraw7::StartModeTest method, this value means that no test could be initiated because all the resolutions chosen for testing already have refresh rate information in the registry. When returned by IDirectDraw7::EvaluateMode, the value means that DirectDraw has completed a refresh rate test.",
            DIRECT_DRAW::DDERR_NEWMODE => "New for DirectX 7.0. When IDirectDraw7::StartModeTest is called with the DDSMT_ISTESTREQUIRED flag, it might return this value to denote that some or all of the resolutions can and should be tested. IDirectDraw7::EvaluateMode returns this value to indicate that the test has switched to a new display mode.",
            DIRECT_DRAW::DDERR_D3DNOTINITIALIZED => "D3D has not yet been initialized.",
            DIRECT_DRAW::DDERR_VIDEONOTACTIVE => "The video port is not active.",
            DIRECT_DRAW::DDERR_NOMONITORINFORMATION => "New for DirectX 7.0. Testing cannot proceed because the monitor has no associated EDID data.",
            DIRECT_DRAW::DDERR_NODRIVERSUPPORT => "New for DirectX 7.0. Testing cannot proceed because the display adapter driver does not enumerate refresh rates.",
            DIRECT_DRAW::DDERR_DEVICEDOESNTOWNSURFACE => "Surfaces created by one DirectDraw device cannot be used directly by another DirectDraw device.",
        }
    }

    pub fn from_name(name: &str) -> DIRECT_DRAW {
        return match name {
            "DDERR_ALREADYINITIALIZED" => DIRECT_DRAW::DDERR_ALREADYINITIALIZED,
            "DDERR_CANNOTATTACHSURFACE" => DIRECT_DRAW::DDERR_CANNOTATTACHSURFACE,
            "DDERR_CANNOTDETACHSURFACE" => DIRECT_DRAW::DDERR_CANNOTDETACHSURFACE,
            "DDERR_CURRENTLYNOTAVAIL" => DIRECT_DRAW::DDERR_CURRENTLYNOTAVAIL,
            "DDERR_EXCEPTION" => DIRECT_DRAW::DDERR_EXCEPTION,
            "DDERR_HEIGHTALIGN" => DIRECT_DRAW::DDERR_HEIGHTALIGN,
            "DDERR_INCOMPATIBLEPRIMARY" => DIRECT_DRAW::DDERR_INCOMPATIBLEPRIMARY,
            "DDERR_INVALIDCAPS" => DIRECT_DRAW::DDERR_INVALIDCAPS,
            "DDERR_INVALIDCLIPLIST" => DIRECT_DRAW::DDERR_INVALIDCLIPLIST,
            "DDERR_INVALIDMODE" => DIRECT_DRAW::DDERR_INVALIDMODE,
            "DDERR_INVALIDOBJECT" => DIRECT_DRAW::DDERR_INVALIDOBJECT,
            "DDERR_INVALIDPIXELFORMAT" => DIRECT_DRAW::DDERR_INVALIDPIXELFORMAT,
            "DDERR_INVALIDRECT" => DIRECT_DRAW::DDERR_INVALIDRECT,
            "DDERR_LOCKEDSURFACES" => DIRECT_DRAW::DDERR_LOCKEDSURFACES,
            "DDERR_NO3D" => DIRECT_DRAW::DDERR_NO3D,
            "DDERR_NOALPHAHW" => DIRECT_DRAW::DDERR_NOALPHAHW,
            "DDERR_NOSTEREOHARDWARE" => DIRECT_DRAW::DDERR_NOSTEREOHARDWARE,
            "DDERR_NOSURFACELEFT" => DIRECT_DRAW::DDERR_NOSURFACELEFT,
            "DDERR_NOCLIPLIST" => DIRECT_DRAW::DDERR_NOCLIPLIST,
            "DDERR_NOCOLORCONVHW" => DIRECT_DRAW::DDERR_NOCOLORCONVHW,
            "DDERR_NOCOOPERATIVELEVELSET" => DIRECT_DRAW::DDERR_NOCOOPERATIVELEVELSET,
            "DDERR_NOCOLORKEY" => DIRECT_DRAW::DDERR_NOCOLORKEY,
            "DDERR_NOCOLORKEYHW" => DIRECT_DRAW::DDERR_NOCOLORKEYHW,
            "DDERR_NODIRECTDRAWSUPPORT" => DIRECT_DRAW::DDERR_NODIRECTDRAWSUPPORT,
            "DDERR_NOEXCLUSIVEMODE" => DIRECT_DRAW::DDERR_NOEXCLUSIVEMODE,
            "DDERR_NOFLIPHW" => DIRECT_DRAW::DDERR_NOFLIPHW,
            "DDERR_NOGDI" => DIRECT_DRAW::DDERR_NOGDI,
            "DDERR_NOMIRRORHW" => DIRECT_DRAW::DDERR_NOMIRRORHW,
            "DDERR_NOTFOUND" => DIRECT_DRAW::DDERR_NOTFOUND,
            "DDERR_NOOVERLAYHW" => DIRECT_DRAW::DDERR_NOOVERLAYHW,
            "DDERR_OVERLAPPINGRECTS" => DIRECT_DRAW::DDERR_OVERLAPPINGRECTS,
            "DDERR_NORASTEROPHW" => DIRECT_DRAW::DDERR_NORASTEROPHW,
            "DDERR_NOROTATIONHW" => DIRECT_DRAW::DDERR_NOROTATIONHW,
            "DDERR_NOSTRETCHHW" => DIRECT_DRAW::DDERR_NOSTRETCHHW,
            "DDERR_NOT4BITCOLOR" => DIRECT_DRAW::DDERR_NOT4BITCOLOR,
            "DDERR_NOT4BITCOLORINDEX" => DIRECT_DRAW::DDERR_NOT4BITCOLORINDEX,
            "DDERR_NOT8BITCOLOR" => DIRECT_DRAW::DDERR_NOT8BITCOLOR,
            "DDERR_NOTEXTUREHW" => DIRECT_DRAW::DDERR_NOTEXTUREHW,
            "DDERR_NOVSYNCHW" => DIRECT_DRAW::DDERR_NOVSYNCHW,
            "DDERR_NOZBUFFERHW" => DIRECT_DRAW::DDERR_NOZBUFFERHW,
            "DDERR_NOZOVERLAYHW" => DIRECT_DRAW::DDERR_NOZOVERLAYHW,
            "DDERR_OUTOFCAPS" => DIRECT_DRAW::DDERR_OUTOFCAPS,
            "DDERR_OUTOFVIDEOMEMORY" => DIRECT_DRAW::DDERR_OUTOFVIDEOMEMORY,
            "DDERR_OVERLAYCANTCLIP" => DIRECT_DRAW::DDERR_OVERLAYCANTCLIP,
            "DDERR_OVERLAYCOLORKEYONLYONEACTIVE" => DIRECT_DRAW::DDERR_OVERLAYCOLORKEYONLYONEACTIVE,
            "DDERR_PALETTEBUSY" => DIRECT_DRAW::DDERR_PALETTEBUSY,
            "DDERR_COLORKEYNOTSET" => DIRECT_DRAW::DDERR_COLORKEYNOTSET,
            "DDERR_SURFACEALREADYATTACHED" => DIRECT_DRAW::DDERR_SURFACEALREADYATTACHED,
            "DDERR_SURFACEALREADYDEPENDENT" => DIRECT_DRAW::DDERR_SURFACEALREADYDEPENDENT,
            "DDERR_SURFACEBUSY" => DIRECT_DRAW::DDERR_SURFACEBUSY,
            "DDERR_CANTLOCKSURFACE" => DIRECT_DRAW::DDERR_CANTLOCKSURFACE,
            "DDERR_SURFACEISOBSCURED" => DIRECT_DRAW::DDERR_SURFACEISOBSCURED,
            "DDERR_SURFACELOST" => DIRECT_DRAW::DDERR_SURFACELOST,
            "DDERR_SURFACENOTATTACHED" => DIRECT_DRAW::DDERR_SURFACENOTATTACHED,
            "DDERR_TOOBIGHEIGHT" => DIRECT_DRAW::DDERR_TOOBIGHEIGHT,
            "DDERR_TOOBIGSIZE" => DIRECT_DRAW::DDERR_TOOBIGSIZE,
            "DDERR_TOOBIGWIDTH" => DIRECT_DRAW::DDERR_TOOBIGWIDTH,
            "DDERR_UNSUPPORTEDFORMAT" => DIRECT_DRAW::DDERR_UNSUPPORTEDFORMAT,
            "DDERR_UNSUPPORTEDMASK" => DIRECT_DRAW::DDERR_UNSUPPORTEDMASK,
            "DDERR_INVALIDSTREAM" => DIRECT_DRAW::DDERR_INVALIDSTREAM,
            "DDERR_VERTICALBLANKINPROGRESS" => DIRECT_DRAW::DDERR_VERTICALBLANKINPROGRESS,
            "DDERR_WASSTILLDRAWING" => DIRECT_DRAW::DDERR_WASSTILLDRAWING,
            "DDERR_DDSCAPSCOMPLEXREQUIRED" => DIRECT_DRAW::DDERR_DDSCAPSCOMPLEXREQUIRED,
            "DDERR_XALIGN" => DIRECT_DRAW::DDERR_XALIGN,
            "DDERR_INVALIDDIRECTDRAWGUID" => DIRECT_DRAW::DDERR_INVALIDDIRECTDRAWGUID,
            "DDERR_DIRECTDRAWALREADYCREATED" => DIRECT_DRAW::DDERR_DIRECTDRAWALREADYCREATED,
            "DDERR_NODIRECTDRAWHW" => DIRECT_DRAW::DDERR_NODIRECTDRAWHW,
            "DDERR_PRIMARYSURFACEALREADYEXISTS" => DIRECT_DRAW::DDERR_PRIMARYSURFACEALREADYEXISTS,
            "DDERR_NOEMULATION" => DIRECT_DRAW::DDERR_NOEMULATION,
            "DDERR_REGIONTOOSMALL" => DIRECT_DRAW::DDERR_REGIONTOOSMALL,
            "DDERR_CLIPPERISUSINGHWND" => DIRECT_DRAW::DDERR_CLIPPERISUSINGHWND,
            "DDERR_NOCLIPPERATTACHED" => DIRECT_DRAW::DDERR_NOCLIPPERATTACHED,
            "DDERR_NOHWND" => DIRECT_DRAW::DDERR_NOHWND,
            "DDERR_HWNDSUBCLASSED" => DIRECT_DRAW::DDERR_HWNDSUBCLASSED,
            "DDERR_HWNDALREADYSET" => DIRECT_DRAW::DDERR_HWNDALREADYSET,
            "DDERR_NOPALETTEATTACHED" => DIRECT_DRAW::DDERR_NOPALETTEATTACHED,
            "DDERR_NOPALETTEHW" => DIRECT_DRAW::DDERR_NOPALETTEHW,
            "DDERR_BLTFASTCANTCLIP" => DIRECT_DRAW::DDERR_BLTFASTCANTCLIP,
            "DDERR_NOBLTHW" => DIRECT_DRAW::DDERR_NOBLTHW,
            "DDERR_NODDROPSHW" => DIRECT_DRAW::DDERR_NODDROPSHW,
            "DDERR_OVERLAYNOTVISIBLE" => DIRECT_DRAW::DDERR_OVERLAYNOTVISIBLE,
            "DDERR_NOOVERLAYDEST" => DIRECT_DRAW::DDERR_NOOVERLAYDEST,
            "DDERR_INVALIDPOSITION" => DIRECT_DRAW::DDERR_INVALIDPOSITION,
            "DDERR_NOTAOVERLAYSURFACE" => DIRECT_DRAW::DDERR_NOTAOVERLAYSURFACE,
            "DDERR_EXCLUSIVEMODEALREADYSET" => DIRECT_DRAW::DDERR_EXCLUSIVEMODEALREADYSET,
            "DDERR_NOTFLIPPABLE" => DIRECT_DRAW::DDERR_NOTFLIPPABLE,
            "DDERR_CANTDUPLICATE" => DIRECT_DRAW::DDERR_CANTDUPLICATE,
            "DDERR_NOTLOCKED" => DIRECT_DRAW::DDERR_NOTLOCKED,
            "DDERR_CANTCREATEDC" => DIRECT_DRAW::DDERR_CANTCREATEDC,
            "DDERR_NODC" => DIRECT_DRAW::DDERR_NODC,
            "DDERR_WRONGMODE" => DIRECT_DRAW::DDERR_WRONGMODE,
            "DDERR_IMPLICITLYCREATED" => DIRECT_DRAW::DDERR_IMPLICITLYCREATED,
            "DDERR_NOTPALETTIZED" => DIRECT_DRAW::DDERR_NOTPALETTIZED,
            "DDERR_UNSUPPORTEDMODE" => DIRECT_DRAW::DDERR_UNSUPPORTEDMODE,
            "DDERR_NOMIPMAPHW" => DIRECT_DRAW::DDERR_NOMIPMAPHW,
            "DDERR_INVALIDSURFACETYPE" => DIRECT_DRAW::DDERR_INVALIDSURFACETYPE,
            "DDERR_NOOPTIMIZEHW" => DIRECT_DRAW::DDERR_NOOPTIMIZEHW,
            "DDERR_NOTLOADED" => DIRECT_DRAW::DDERR_NOTLOADED,
            "DDERR_NOFOCUSWINDOW" => DIRECT_DRAW::DDERR_NOFOCUSWINDOW,
            "DDERR_NOTONMIPMAPSUBLEVEL" => DIRECT_DRAW::DDERR_NOTONMIPMAPSUBLEVEL,
            "DDERR_DCALREADYCREATED" => DIRECT_DRAW::DDERR_DCALREADYCREATED,
            "DDERR_NONONLOCALVIDMEM" => DIRECT_DRAW::DDERR_NONONLOCALVIDMEM,
            "DDERR_CANTPAGELOCK" => DIRECT_DRAW::DDERR_CANTPAGELOCK,
            "DDERR_CANTPAGEUNLOCK" => DIRECT_DRAW::DDERR_CANTPAGEUNLOCK,
            "DDERR_NOTPAGELOCKED" => DIRECT_DRAW::DDERR_NOTPAGELOCKED,
            "DDERR_MOREDATA" => DIRECT_DRAW::DDERR_MOREDATA,
            "DDERR_EXPIRED" => DIRECT_DRAW::DDERR_EXPIRED,
            "DDERR_TESTFINISHED" => DIRECT_DRAW::DDERR_TESTFINISHED,
            "DDERR_NEWMODE" => DIRECT_DRAW::DDERR_NEWMODE,
            "DDERR_D3DNOTINITIALIZED" => DIRECT_DRAW::DDERR_D3DNOTINITIALIZED,
            "DDERR_VIDEONOTACTIVE" => DIRECT_DRAW::DDERR_VIDEONOTACTIVE,
            "DDERR_NOMONITORINFORMATION" => DIRECT_DRAW::DDERR_NOMONITORINFORMATION,
            "DDERR_NODRIVERSUPPORT" => DIRECT_DRAW::DDERR_NODRIVERSUPPORT,
            "DDERR_DEVICEDOESNTOWNSURFACE" => DIRECT_DRAW::DDERR_DEVICEDOESNTOWNSURFACE,
        }
    }
    pub fn from_code(code: u32) -> DIRECT_DRAW {
        return match code {
            0x88760005 => DIRECT_DRAW::DDERR_ALREADYINITIALIZED,
            0x8876000A => DIRECT_DRAW::DDERR_CANNOTATTACHSURFACE,
            0x88760014 => DIRECT_DRAW::DDERR_CANNOTDETACHSURFACE,
            0x88760028 => DIRECT_DRAW::DDERR_CURRENTLYNOTAVAIL,
            0x88760037 => DIRECT_DRAW::DDERR_EXCEPTION,
            0x8876005A => DIRECT_DRAW::DDERR_HEIGHTALIGN,
            0x8876005F => DIRECT_DRAW::DDERR_INCOMPATIBLEPRIMARY,
            0x88760064 => DIRECT_DRAW::DDERR_INVALIDCAPS,
            0x8876006E => DIRECT_DRAW::DDERR_INVALIDCLIPLIST,
            0x88760078 => DIRECT_DRAW::DDERR_INVALIDMODE,
            0x88760082 => DIRECT_DRAW::DDERR_INVALIDOBJECT,
            0x88760091 => DIRECT_DRAW::DDERR_INVALIDPIXELFORMAT,
            0x88760096 => DIRECT_DRAW::DDERR_INVALIDRECT,
            0x887600A0 => DIRECT_DRAW::DDERR_LOCKEDSURFACES,
            0x887600AA => DIRECT_DRAW::DDERR_NO3D,
            0x887600B4 => DIRECT_DRAW::DDERR_NOALPHAHW,
            0x887600B5 => DIRECT_DRAW::DDERR_NOSTEREOHARDWARE,
            0x887600B6 => DIRECT_DRAW::DDERR_NOSURFACELEFT,
            0x887600CD => DIRECT_DRAW::DDERR_NOCLIPLIST,
            0x887600D2 => DIRECT_DRAW::DDERR_NOCOLORCONVHW,
            0x887600D4 => DIRECT_DRAW::DDERR_NOCOOPERATIVELEVELSET,
            0x887600D7 => DIRECT_DRAW::DDERR_NOCOLORKEY,
            0x887600DC => DIRECT_DRAW::DDERR_NOCOLORKEYHW,
            0x887600DE => DIRECT_DRAW::DDERR_NODIRECTDRAWSUPPORT,
            0x887600E1 => DIRECT_DRAW::DDERR_NOEXCLUSIVEMODE,
            0x887600E6 => DIRECT_DRAW::DDERR_NOFLIPHW,
            0x887600F0 => DIRECT_DRAW::DDERR_NOGDI,
            0x887600FA => DIRECT_DRAW::DDERR_NOMIRRORHW,
            0x887600FF => DIRECT_DRAW::DDERR_NOTFOUND,
            0x88760104 => DIRECT_DRAW::DDERR_NOOVERLAYHW,
            0x8876010E => DIRECT_DRAW::DDERR_OVERLAPPINGRECTS,
            0x88760118 => DIRECT_DRAW::DDERR_NORASTEROPHW,
            0x88760122 => DIRECT_DRAW::DDERR_NOROTATIONHW,
            0x88760136 => DIRECT_DRAW::DDERR_NOSTRETCHHW,
            0x8876013C => DIRECT_DRAW::DDERR_NOT4BITCOLOR,
            0x8876013D => DIRECT_DRAW::DDERR_NOT4BITCOLORINDEX,
            0x88760140 => DIRECT_DRAW::DDERR_NOT8BITCOLOR,
            0x8876014A => DIRECT_DRAW::DDERR_NOTEXTUREHW,
            0x8876014F => DIRECT_DRAW::DDERR_NOVSYNCHW,
            0x88760154 => DIRECT_DRAW::DDERR_NOZBUFFERHW,
            0x8876015E => DIRECT_DRAW::DDERR_NOZOVERLAYHW,
            0x88760168 => DIRECT_DRAW::DDERR_OUTOFCAPS,
            0x8876017C => DIRECT_DRAW::DDERR_OUTOFVIDEOMEMORY,
            0x8876017E => DIRECT_DRAW::DDERR_OVERLAYCANTCLIP,
            0x88760180 => DIRECT_DRAW::DDERR_OVERLAYCOLORKEYONLYONEACTIVE,
            0x88760183 => DIRECT_DRAW::DDERR_PALETTEBUSY,
            0x88760190 => DIRECT_DRAW::DDERR_COLORKEYNOTSET,
            0x8876019A => DIRECT_DRAW::DDERR_SURFACEALREADYATTACHED,
            0x887601A4 => DIRECT_DRAW::DDERR_SURFACEALREADYDEPENDENT,
            0x887601AE => DIRECT_DRAW::DDERR_SURFACEBUSY,
            0x887601B3 => DIRECT_DRAW::DDERR_CANTLOCKSURFACE,
            0x887601B8 => DIRECT_DRAW::DDERR_SURFACEISOBSCURED,
            0x887601C2 => DIRECT_DRAW::DDERR_SURFACELOST,
            0x887601CC => DIRECT_DRAW::DDERR_SURFACENOTATTACHED,
            0x887601D6 => DIRECT_DRAW::DDERR_TOOBIGHEIGHT,
            0x887601E0 => DIRECT_DRAW::DDERR_TOOBIGSIZE,
            0x887601EA => DIRECT_DRAW::DDERR_TOOBIGWIDTH,
            0x887601FE => DIRECT_DRAW::DDERR_UNSUPPORTEDFORMAT,
            0x88760208 => DIRECT_DRAW::DDERR_UNSUPPORTEDMASK,
            0x88760209 => DIRECT_DRAW::DDERR_INVALIDSTREAM,
            0x88760219 => DIRECT_DRAW::DDERR_VERTICALBLANKINPROGRESS,
            0x8876021C => DIRECT_DRAW::DDERR_WASSTILLDRAWING,
            0x8876021E => DIRECT_DRAW::DDERR_DDSCAPSCOMPLEXREQUIRED,
            0x88760230 => DIRECT_DRAW::DDERR_XALIGN,
            0x88760231 => DIRECT_DRAW::DDERR_INVALIDDIRECTDRAWGUID,
            0x88760232 => DIRECT_DRAW::DDERR_DIRECTDRAWALREADYCREATED,
            0x88760233 => DIRECT_DRAW::DDERR_NODIRECTDRAWHW,
            0x88760234 => DIRECT_DRAW::DDERR_PRIMARYSURFACEALREADYEXISTS,
            0x88760235 => DIRECT_DRAW::DDERR_NOEMULATION,
            0x88760236 => DIRECT_DRAW::DDERR_REGIONTOOSMALL,
            0x88760237 => DIRECT_DRAW::DDERR_CLIPPERISUSINGHWND,
            0x88760238 => DIRECT_DRAW::DDERR_NOCLIPPERATTACHED,
            0x88760239 => DIRECT_DRAW::DDERR_NOHWND,
            0x8876023A => DIRECT_DRAW::DDERR_HWNDSUBCLASSED,
            0x8876023B => DIRECT_DRAW::DDERR_HWNDALREADYSET,
            0x8876023C => DIRECT_DRAW::DDERR_NOPALETTEATTACHED,
            0x8876023D => DIRECT_DRAW::DDERR_NOPALETTEHW,
            0x8876023E => DIRECT_DRAW::DDERR_BLTFASTCANTCLIP,
            0x8876023F => DIRECT_DRAW::DDERR_NOBLTHW,
            0x88760240 => DIRECT_DRAW::DDERR_NODDROPSHW,
            0x88760241 => DIRECT_DRAW::DDERR_OVERLAYNOTVISIBLE,
            0x88760242 => DIRECT_DRAW::DDERR_NOOVERLAYDEST,
            0x88760243 => DIRECT_DRAW::DDERR_INVALIDPOSITION,
            0x88760244 => DIRECT_DRAW::DDERR_NOTAOVERLAYSURFACE,
            0x88760245 => DIRECT_DRAW::DDERR_EXCLUSIVEMODEALREADYSET,
            0x88760246 => DIRECT_DRAW::DDERR_NOTFLIPPABLE,
            0x88760247 => DIRECT_DRAW::DDERR_CANTDUPLICATE,
            0x88760248 => DIRECT_DRAW::DDERR_NOTLOCKED,
            0x88760249 => DIRECT_DRAW::DDERR_CANTCREATEDC,
            0x8876024A => DIRECT_DRAW::DDERR_NODC,
            0x8876024B => DIRECT_DRAW::DDERR_WRONGMODE,
            0x8876024C => DIRECT_DRAW::DDERR_IMPLICITLYCREATED,
            0x8876024D => DIRECT_DRAW::DDERR_NOTPALETTIZED,
            0x8876024E => DIRECT_DRAW::DDERR_UNSUPPORTEDMODE,
            0x8876024F => DIRECT_DRAW::DDERR_NOMIPMAPHW,
            0x88760250 => DIRECT_DRAW::DDERR_INVALIDSURFACETYPE,
            0x88760258 => DIRECT_DRAW::DDERR_NOOPTIMIZEHW,
            0x88760259 => DIRECT_DRAW::DDERR_NOTLOADED,
            0x8876025A => DIRECT_DRAW::DDERR_NOFOCUSWINDOW,
            0x8876025B => DIRECT_DRAW::DDERR_NOTONMIPMAPSUBLEVEL,
            0x8876026C => DIRECT_DRAW::DDERR_DCALREADYCREATED,
            0x88760276 => DIRECT_DRAW::DDERR_NONONLOCALVIDMEM,
            0x88760280 => DIRECT_DRAW::DDERR_CANTPAGELOCK,
            0x88760294 => DIRECT_DRAW::DDERR_CANTPAGEUNLOCK,
            0x887602A8 => DIRECT_DRAW::DDERR_NOTPAGELOCKED,
            0x887602B2 => DIRECT_DRAW::DDERR_MOREDATA,
            0x887602B3 => DIRECT_DRAW::DDERR_EXPIRED,
            0x887602B4 => DIRECT_DRAW::DDERR_TESTFINISHED,
            0x887602B5 => DIRECT_DRAW::DDERR_NEWMODE,
            0x887602B6 => DIRECT_DRAW::DDERR_D3DNOTINITIALIZED,
            0x887602B7 => DIRECT_DRAW::DDERR_VIDEONOTACTIVE,
            0x887602B8 => DIRECT_DRAW::DDERR_NOMONITORINFORMATION,
            0x887602B9 => DIRECT_DRAW::DDERR_NODRIVERSUPPORT,
            0x887602BB => DIRECT_DRAW::DDERR_DEVICEDOESNTOWNSURFACE,
        }
    }
}
