/*
 * Copyright 2023 Stanislav Mikhailov (xavetar)
 *
 * Licensed under the Creative Commons Zero v1.0 Universal (CC0) License.
 * You may obtain a copy of the License at
 *
 *     http://creativecommons.org/publicdomain/zero/1.0/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the CC0 license is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// It is strictly forbidden to use the from_code method, respected Microsoft decided to define
// duplicate codes, but by defining different constant names, therefore, when using these codes,
// it is mandatory to use through from_name, and not from_code. Otherwise, it may cause undefined
// behavior or an unknown exception. Because one code corresponds to several constants.

use super::{RawError};

#[derive(Clone, Debug, Copy, Eq, Hash, Ord, PartialEq, PartialOrd)]
pub enum DXGI {
    DXGI_STATUS_OCCLUDED,
    DXGI_STATUS_CLIPPED,
    DXGI_STATUS_NO_REDIRECTION,
    DXGI_STATUS_NO_DESKTOP_ACCESS,
    DXGI_STATUS_GRAPHICS_VIDPN_SOURCE_IN_USE,
    DXGI_STATUS_MODE_CHANGED,
    DXGI_STATUS_MODE_CHANGE_IN_PROGRESS,
    DXGI_STATUS_UNOCCLUDED,
    DXGI_STATUS_DDA_WAS_STILL_DRAWING,
    DXGI_STATUS_PRESENT_REQUIRED,
    DXGI_ERROR_INVALID_CALL,
    DXGI_ERROR_NOT_FOUND,
    DXGI_ERROR_MORE_DATA,
    DXGI_ERROR_UNSUPPORTED,
    DXGI_ERROR_DEVICE_REMOVED,
    DXGI_ERROR_DEVICE_HUNG,
    DXGI_ERROR_DEVICE_RESET,
    DXGI_ERROR_WAS_STILL_DRAWING,
    DXGI_ERROR_FRAME_STATISTICS_DISJOINT,
    DXGI_ERROR_GRAPHICS_VIDPN_SOURCE_IN_USE,
    DXGI_ERROR_DRIVER_INTERNAL_ERROR,
    DXGI_ERROR_NONEXCLUSIVE,
    DXGI_ERROR_NOT_CURRENTLY_AVAILABLE,
    DXGI_ERROR_REMOTE_CLIENT_DISCONNECTED,
    DXGI_ERROR_REMOTE_OUTOFMEMORY,
    DXGI_ERROR_MODE_CHANGE_IN_PROGRESS,
    DXGI_ERROR_ACCESS_LOST,
    DXGI_ERROR_WAIT_TIMEOUT,
    DXGI_ERROR_SESSION_DISCONNECTED,
    DXGI_ERROR_RESTRICT_TO_OUTPUT_STALE,
    DXGI_ERROR_CANNOT_PROTECT_CONTENT,
    DXGI_ERROR_ACCESS_DENIED,
    DXGI_ERROR_NAME_ALREADY_EXISTS,
    DXGI_ERROR_SDK_COMPONENT_MISSING,
    DXGI_ERROR_NOT_CURRENT,
    DXGI_ERROR_HW_PROTECTION_OUTOFMEMORY,
    DXGI_ERROR_DYNAMIC_CODE_POLICY_VIOLATION,
    DXGI_ERROR_NON_COMPOSITED_UI,
    DXGI_ERROR_CACHE_CORRUPT,
    DXGI_ERROR_CACHE_FULL,
    DXGI_ERROR_CACHE_HASH_COLLISION,
    DXGI_ERROR_ALREADY_EXISTS,
    DXGI_ERROR_MPO_UNPINNED,
}

impl DXGI {
    pub fn code(&self) -> u32 {
        return match self {
            DXGI::DXGI_STATUS_OCCLUDED => 0x087A0001 as u32,
            DXGI::DXGI_STATUS_CLIPPED => 0x087A0002 as u32,
            DXGI::DXGI_STATUS_NO_REDIRECTION => 0x087A0004 as u32,
            DXGI::DXGI_STATUS_NO_DESKTOP_ACCESS => 0x087A0005 as u32,
            DXGI::DXGI_STATUS_GRAPHICS_VIDPN_SOURCE_IN_USE => 0x087A0006 as u32,
            DXGI::DXGI_STATUS_MODE_CHANGED => 0x087A0007 as u32,
            DXGI::DXGI_STATUS_MODE_CHANGE_IN_PROGRESS => 0x087A0008 as u32,
            DXGI::DXGI_STATUS_UNOCCLUDED => 0x087A0009 as u32,
            DXGI::DXGI_STATUS_DDA_WAS_STILL_DRAWING => 0x087A000A as u32,
            DXGI::DXGI_STATUS_PRESENT_REQUIRED => 0x087A002F as u32,
            DXGI::DXGI_ERROR_INVALID_CALL => 0x887A0001 as u32,
            DXGI::DXGI_ERROR_NOT_FOUND => 0x887A0002 as u32,
            DXGI::DXGI_ERROR_MORE_DATA => 0x887A0003 as u32,
            DXGI::DXGI_ERROR_UNSUPPORTED => 0x887A0004 as u32,
            DXGI::DXGI_ERROR_DEVICE_REMOVED => 0x887A0005 as u32,
            DXGI::DXGI_ERROR_DEVICE_HUNG => 0x887A0006 as u32,
            DXGI::DXGI_ERROR_DEVICE_RESET => 0x887A0007 as u32,
            DXGI::DXGI_ERROR_WAS_STILL_DRAWING => 0x887A000A as u32,
            DXGI::DXGI_ERROR_FRAME_STATISTICS_DISJOINT => 0x887A000B as u32,
            DXGI::DXGI_ERROR_GRAPHICS_VIDPN_SOURCE_IN_USE => 0x887A000C as u32,
            DXGI::DXGI_ERROR_DRIVER_INTERNAL_ERROR => 0x887A0020 as u32,
            DXGI::DXGI_ERROR_NONEXCLUSIVE => 0x887A0021 as u32,
            DXGI::DXGI_ERROR_NOT_CURRENTLY_AVAILABLE => 0x887A0022 as u32,
            DXGI::DXGI_ERROR_REMOTE_CLIENT_DISCONNECTED => 0x887A0023 as u32,
            DXGI::DXGI_ERROR_REMOTE_OUTOFMEMORY => 0x887A0024 as u32,
            DXGI::DXGI_ERROR_MODE_CHANGE_IN_PROGRESS => 0x887A0025 as u32,
            DXGI::DXGI_ERROR_ACCESS_LOST => 0x887A0026 as u32,
            DXGI::DXGI_ERROR_WAIT_TIMEOUT => 0x887A0027 as u32,
            DXGI::DXGI_ERROR_SESSION_DISCONNECTED => 0x887A0028 as u32,
            DXGI::DXGI_ERROR_RESTRICT_TO_OUTPUT_STALE => 0x887A0029 as u32,
            DXGI::DXGI_ERROR_CANNOT_PROTECT_CONTENT => 0x887A002A as u32,
            DXGI::DXGI_ERROR_ACCESS_DENIED => 0x887A002B as u32,
            DXGI::DXGI_ERROR_NAME_ALREADY_EXISTS => 0x887A002C as u32,
            DXGI::DXGI_ERROR_SDK_COMPONENT_MISSING => 0x887A002D as u32,
            DXGI::DXGI_ERROR_NOT_CURRENT => 0x887A002E as u32,
            DXGI::DXGI_ERROR_HW_PROTECTION_OUTOFMEMORY => 0x887A0030 as u32,
            DXGI::DXGI_ERROR_DYNAMIC_CODE_POLICY_VIOLATION => 0x887A0031 as u32,
            DXGI::DXGI_ERROR_NON_COMPOSITED_UI => 0x887A0032 as u32,
            DXGI::DXGI_ERROR_CACHE_CORRUPT => 0x887A0033 as u32,
            DXGI::DXGI_ERROR_CACHE_FULL => 0x887A0034 as u32,
            DXGI::DXGI_ERROR_CACHE_HASH_COLLISION => 0x887A0035 as u32,
            DXGI::DXGI_ERROR_ALREADY_EXISTS => 0x887A0036 as u32,
            DXGI::DXGI_ERROR_MPO_UNPINNED => 0x887A0064 as u32,
        }
    }

    pub fn error(&self) -> RawError {
        return match self {
            DXGI::DXGI_STATUS_OCCLUDED => RawError::Kind(DXGI::DXGI_STATUS_OCCLUDED),
            DXGI::DXGI_STATUS_CLIPPED => RawError::Kind(DXGI::DXGI_STATUS_CLIPPED),
            DXGI::DXGI_STATUS_NO_REDIRECTION => RawError::Kind(DXGI::DXGI_STATUS_NO_REDIRECTION),
            DXGI::DXGI_STATUS_NO_DESKTOP_ACCESS => RawError::Kind(DXGI::DXGI_STATUS_NO_DESKTOP_ACCESS),
            DXGI::DXGI_STATUS_GRAPHICS_VIDPN_SOURCE_IN_USE => RawError::Kind(DXGI::DXGI_STATUS_GRAPHICS_VIDPN_SOURCE_IN_USE),
            DXGI::DXGI_STATUS_MODE_CHANGED => RawError::Kind(DXGI::DXGI_STATUS_MODE_CHANGED),
            DXGI::DXGI_STATUS_MODE_CHANGE_IN_PROGRESS => RawError::Kind(DXGI::DXGI_STATUS_MODE_CHANGE_IN_PROGRESS),
            DXGI::DXGI_STATUS_UNOCCLUDED => RawError::Kind(DXGI::DXGI_STATUS_UNOCCLUDED),
            DXGI::DXGI_STATUS_DDA_WAS_STILL_DRAWING => RawError::Kind(DXGI::DXGI_STATUS_DDA_WAS_STILL_DRAWING),
            DXGI::DXGI_STATUS_PRESENT_REQUIRED => RawError::Kind(DXGI::DXGI_STATUS_PRESENT_REQUIRED),
            DXGI::DXGI_ERROR_INVALID_CALL => RawError::Kind(DXGI::DXGI_ERROR_INVALID_CALL),
            DXGI::DXGI_ERROR_NOT_FOUND => RawError::Kind(DXGI::DXGI_ERROR_NOT_FOUND),
            DXGI::DXGI_ERROR_MORE_DATA => RawError::Kind(DXGI::DXGI_ERROR_MORE_DATA),
            DXGI::DXGI_ERROR_UNSUPPORTED => RawError::Kind(DXGI::DXGI_ERROR_UNSUPPORTED),
            DXGI::DXGI_ERROR_DEVICE_REMOVED => RawError::Kind(DXGI::DXGI_ERROR_DEVICE_REMOVED),
            DXGI::DXGI_ERROR_DEVICE_HUNG => RawError::Kind(DXGI::DXGI_ERROR_DEVICE_HUNG),
            DXGI::DXGI_ERROR_DEVICE_RESET => RawError::Kind(DXGI::DXGI_ERROR_DEVICE_RESET),
            DXGI::DXGI_ERROR_WAS_STILL_DRAWING => RawError::Kind(DXGI::DXGI_ERROR_WAS_STILL_DRAWING),
            DXGI::DXGI_ERROR_FRAME_STATISTICS_DISJOINT => RawError::Kind(DXGI::DXGI_ERROR_FRAME_STATISTICS_DISJOINT),
            DXGI::DXGI_ERROR_GRAPHICS_VIDPN_SOURCE_IN_USE => RawError::Kind(DXGI::DXGI_ERROR_GRAPHICS_VIDPN_SOURCE_IN_USE),
            DXGI::DXGI_ERROR_DRIVER_INTERNAL_ERROR => RawError::Kind(DXGI::DXGI_ERROR_DRIVER_INTERNAL_ERROR),
            DXGI::DXGI_ERROR_NONEXCLUSIVE => RawError::Kind(DXGI::DXGI_ERROR_NONEXCLUSIVE),
            DXGI::DXGI_ERROR_NOT_CURRENTLY_AVAILABLE => RawError::Kind(DXGI::DXGI_ERROR_NOT_CURRENTLY_AVAILABLE),
            DXGI::DXGI_ERROR_REMOTE_CLIENT_DISCONNECTED => RawError::Kind(DXGI::DXGI_ERROR_REMOTE_CLIENT_DISCONNECTED),
            DXGI::DXGI_ERROR_REMOTE_OUTOFMEMORY => RawError::Kind(DXGI::DXGI_ERROR_REMOTE_OUTOFMEMORY),
            DXGI::DXGI_ERROR_MODE_CHANGE_IN_PROGRESS => RawError::Kind(DXGI::DXGI_ERROR_MODE_CHANGE_IN_PROGRESS),
            DXGI::DXGI_ERROR_ACCESS_LOST => RawError::Kind(DXGI::DXGI_ERROR_ACCESS_LOST),
            DXGI::DXGI_ERROR_WAIT_TIMEOUT => RawError::Kind(DXGI::DXGI_ERROR_WAIT_TIMEOUT),
            DXGI::DXGI_ERROR_SESSION_DISCONNECTED => RawError::Kind(DXGI::DXGI_ERROR_SESSION_DISCONNECTED),
            DXGI::DXGI_ERROR_RESTRICT_TO_OUTPUT_STALE => RawError::Kind(DXGI::DXGI_ERROR_RESTRICT_TO_OUTPUT_STALE),
            DXGI::DXGI_ERROR_CANNOT_PROTECT_CONTENT => RawError::Kind(DXGI::DXGI_ERROR_CANNOT_PROTECT_CONTENT),
            DXGI::DXGI_ERROR_ACCESS_DENIED => RawError::Kind(DXGI::DXGI_ERROR_ACCESS_DENIED),
            DXGI::DXGI_ERROR_NAME_ALREADY_EXISTS => RawError::Kind(DXGI::DXGI_ERROR_NAME_ALREADY_EXISTS),
            DXGI::DXGI_ERROR_SDK_COMPONENT_MISSING => RawError::Kind(DXGI::DXGI_ERROR_SDK_COMPONENT_MISSING),
            DXGI::DXGI_ERROR_NOT_CURRENT => RawError::Kind(DXGI::DXGI_ERROR_NOT_CURRENT),
            DXGI::DXGI_ERROR_HW_PROTECTION_OUTOFMEMORY => RawError::Kind(DXGI::DXGI_ERROR_HW_PROTECTION_OUTOFMEMORY),
            DXGI::DXGI_ERROR_DYNAMIC_CODE_POLICY_VIOLATION => RawError::Kind(DXGI::DXGI_ERROR_DYNAMIC_CODE_POLICY_VIOLATION),
            DXGI::DXGI_ERROR_NON_COMPOSITED_UI => RawError::Kind(DXGI::DXGI_ERROR_NON_COMPOSITED_UI),
            DXGI::DXGI_ERROR_CACHE_CORRUPT => RawError::Kind(DXGI::DXGI_ERROR_CACHE_CORRUPT),
            DXGI::DXGI_ERROR_CACHE_FULL => RawError::Kind(DXGI::DXGI_ERROR_CACHE_FULL),
            DXGI::DXGI_ERROR_CACHE_HASH_COLLISION => RawError::Kind(DXGI::DXGI_ERROR_CACHE_HASH_COLLISION),
            DXGI::DXGI_ERROR_ALREADY_EXISTS => RawError::Kind(DXGI::DXGI_ERROR_ALREADY_EXISTS),
            DXGI::DXGI_ERROR_MPO_UNPINNED => RawError::Kind(DXGI::DXGI_ERROR_MPO_UNPINNED),
        }
    }

    pub fn description(&self) -> &'static str {
        return match self {
            DXGI::DXGI_STATUS_OCCLUDED => "The Present operation was invisible to the user.",
            DXGI::DXGI_STATUS_CLIPPED => "The Present operation was partially invisible to the user.",
            DXGI::DXGI_STATUS_NO_REDIRECTION => "The driver is requesting that the DXGI runtime not use shared resources to communicate with the Desktop Window Manager.",
            DXGI::DXGI_STATUS_NO_DESKTOP_ACCESS => "The Present operation was not visible because the Windows session has switched to another desktop (for example, ctrl-alt-del).",
            DXGI::DXGI_STATUS_GRAPHICS_VIDPN_SOURCE_IN_USE => "The Present operation was not visible because the target monitor was being used for some other purpose.",
            DXGI::DXGI_STATUS_MODE_CHANGED => "The Present operation was not visible because the display mode changed. DXGI will have re-attempted the presentation.",
            DXGI::DXGI_STATUS_MODE_CHANGE_IN_PROGRESS => "The Present operation was not visible because another Direct3D device was attempting to take fullscreen mode at the time.",
            DXGI::DXGI_STATUS_UNOCCLUDED => "The swapchain has become unoccluded.",
            DXGI::DXGI_STATUS_DDA_WAS_STILL_DRAWING => "The adapter did not have access to the required resources to complete the Desktop Duplication Present() call, the Present() call needs to be made again",
            DXGI::DXGI_STATUS_PRESENT_REQUIRED => "The present succeeded but the caller should present again on the next V-sync, even if there are no changes to the content.",
            DXGI::DXGI_ERROR_INVALID_CALL => "The application made a call that is invalid. Either the parameters of the call or the state of some object was incorrect.",
            DXGI::DXGI_ERROR_NOT_FOUND => "The object was not found. If calling IDXGIFactory::EnumAdaptes, there is no adapter with the specified ordinal.",
            DXGI::DXGI_ERROR_MORE_DATA => "The caller did not supply a sufficiently large buffer.",
            DXGI::DXGI_ERROR_UNSUPPORTED => "The specified device interface or feature level is not supported on this system.",
            DXGI::DXGI_ERROR_DEVICE_REMOVED => "The GPU device instance has been suspended. Use GetDeviceRemovedReason to determine the appropriate action.",
            DXGI::DXGI_ERROR_DEVICE_HUNG => "The GPU will not respond to more commands, most likely because of an invalid command passed by the calling application.",
            DXGI::DXGI_ERROR_DEVICE_RESET => "The GPU will not respond to more commands, most likely because some other application submitted invalid commands.",
            DXGI::DXGI_ERROR_WAS_STILL_DRAWING => "The GPU was busy at the moment when the call was made, and the call was neither executed nor scheduled.",
            DXGI::DXGI_ERROR_FRAME_STATISTICS_DISJOINT => "An event (such as power cycle) interrupted the gathering of presentation statistics. Any previous statistics should be",
            DXGI::DXGI_ERROR_GRAPHICS_VIDPN_SOURCE_IN_USE => "Fullscreen mode could not be achieved because the specified output was already in use.",
            DXGI::DXGI_ERROR_DRIVER_INTERNAL_ERROR => "An internal issue prevented the driver from carrying out the specified operation. The driver's state is probably suspect,",
            DXGI::DXGI_ERROR_NONEXCLUSIVE => "A global counter resource was in use, and the specified counter cannot be used by this Direct3D device at this time.",
            DXGI::DXGI_ERROR_NOT_CURRENTLY_AVAILABLE => "A resource is not available at the time of the call, but may become available later.",
            DXGI::DXGI_ERROR_REMOTE_CLIENT_DISCONNECTED => "The application's remote device has been removed due to session disconnect or network disconnect.",
            DXGI::DXGI_ERROR_REMOTE_OUTOFMEMORY => "The device has been removed during a remote session because the remote computer ran out of memory.",
            DXGI::DXGI_ERROR_MODE_CHANGE_IN_PROGRESS => "An on-going mode change prevented completion of the call. The call may succeed if attempted later.",
            DXGI::DXGI_ERROR_ACCESS_LOST => "The keyed mutex was abandoned.",
            DXGI::DXGI_ERROR_WAIT_TIMEOUT => "The timeout value has elapsed and the resource is not yet available.",
            DXGI::DXGI_ERROR_SESSION_DISCONNECTED => "The output duplication has been turned off because the Windows session ended or was disconnected.",
            DXGI::DXGI_ERROR_RESTRICT_TO_OUTPUT_STALE => "The DXGI output (monitor) to which the swapchain content was restricted, has been disconnected or changed.",
            DXGI::DXGI_ERROR_CANNOT_PROTECT_CONTENT => "DXGI is unable to provide content protection on the swapchain. This is typically caused by an older driver,",
            DXGI::DXGI_ERROR_ACCESS_DENIED => "The application is trying to use a resource to which it does not have the required access privileges.",
            DXGI::DXGI_ERROR_NAME_ALREADY_EXISTS => "The application is trying to create a shared handle using a name that is already associated with some other resource.",
            DXGI::DXGI_ERROR_SDK_COMPONENT_MISSING => "The application requested an operation that depends on an SDK component that is missing or mismatched.",
            DXGI::DXGI_ERROR_NOT_CURRENT => "The DXGI objects that the application has created are no longer current & need to be recreated for this operation to be performed.",
            DXGI::DXGI_ERROR_HW_PROTECTION_OUTOFMEMORY => "Insufficient HW protected memory exits for proper function.",
            DXGI::DXGI_ERROR_DYNAMIC_CODE_POLICY_VIOLATION => "Creating this device would violate the process's dynamic code policy.",
            DXGI::DXGI_ERROR_NON_COMPOSITED_UI => "The operation failed because the compositor is not in control of the output.",
            DXGI::DXGI_ERROR_CACHE_CORRUPT => "The cache is corrupt and either could not be opened or could not be reset.",
            DXGI::DXGI_ERROR_CACHE_FULL => "This entry would cause the cache to exceed its quota. On a load operation, this may indicate exceeding the maximum in-memory size.",
            DXGI::DXGI_ERROR_CACHE_HASH_COLLISION => "A cache entry was found, but the key provided does not match the key stored in the entry.",
            DXGI::DXGI_ERROR_ALREADY_EXISTS => "The desired element already exists.",
            DXGI::DXGI_ERROR_MPO_UNPINNED => "The allocation of the MPO plane has been unpinned",
        }
    }

    pub fn from_name(name: &str) -> DXGI {
        return match name {
            "DXGI_STATUS_OCCLUDED" => DXGI::DXGI_STATUS_OCCLUDED,
            "DXGI_STATUS_CLIPPED" => DXGI::DXGI_STATUS_CLIPPED,
            "DXGI_STATUS_NO_REDIRECTION" => DXGI::DXGI_STATUS_NO_REDIRECTION,
            "DXGI_STATUS_NO_DESKTOP_ACCESS" => DXGI::DXGI_STATUS_NO_DESKTOP_ACCESS,
            "DXGI_STATUS_GRAPHICS_VIDPN_SOURCE_IN_USE" => DXGI::DXGI_STATUS_GRAPHICS_VIDPN_SOURCE_IN_USE,
            "DXGI_STATUS_MODE_CHANGED" => DXGI::DXGI_STATUS_MODE_CHANGED,
            "DXGI_STATUS_MODE_CHANGE_IN_PROGRESS" => DXGI::DXGI_STATUS_MODE_CHANGE_IN_PROGRESS,
            "DXGI_STATUS_UNOCCLUDED" => DXGI::DXGI_STATUS_UNOCCLUDED,
            "DXGI_STATUS_DDA_WAS_STILL_DRAWING" => DXGI::DXGI_STATUS_DDA_WAS_STILL_DRAWING,
            "DXGI_STATUS_PRESENT_REQUIRED" => DXGI::DXGI_STATUS_PRESENT_REQUIRED,
            "DXGI_ERROR_INVALID_CALL" => DXGI::DXGI_ERROR_INVALID_CALL,
            "DXGI_ERROR_NOT_FOUND" => DXGI::DXGI_ERROR_NOT_FOUND,
            "DXGI_ERROR_MORE_DATA" => DXGI::DXGI_ERROR_MORE_DATA,
            "DXGI_ERROR_UNSUPPORTED" => DXGI::DXGI_ERROR_UNSUPPORTED,
            "DXGI_ERROR_DEVICE_REMOVED" => DXGI::DXGI_ERROR_DEVICE_REMOVED,
            "DXGI_ERROR_DEVICE_HUNG" => DXGI::DXGI_ERROR_DEVICE_HUNG,
            "DXGI_ERROR_DEVICE_RESET" => DXGI::DXGI_ERROR_DEVICE_RESET,
            "DXGI_ERROR_WAS_STILL_DRAWING" => DXGI::DXGI_ERROR_WAS_STILL_DRAWING,
            "DXGI_ERROR_FRAME_STATISTICS_DISJOINT" => DXGI::DXGI_ERROR_FRAME_STATISTICS_DISJOINT,
            "DXGI_ERROR_GRAPHICS_VIDPN_SOURCE_IN_USE" => DXGI::DXGI_ERROR_GRAPHICS_VIDPN_SOURCE_IN_USE,
            "DXGI_ERROR_DRIVER_INTERNAL_ERROR" => DXGI::DXGI_ERROR_DRIVER_INTERNAL_ERROR,
            "DXGI_ERROR_NONEXCLUSIVE" => DXGI::DXGI_ERROR_NONEXCLUSIVE,
            "DXGI_ERROR_NOT_CURRENTLY_AVAILABLE" => DXGI::DXGI_ERROR_NOT_CURRENTLY_AVAILABLE,
            "DXGI_ERROR_REMOTE_CLIENT_DISCONNECTED" => DXGI::DXGI_ERROR_REMOTE_CLIENT_DISCONNECTED,
            "DXGI_ERROR_REMOTE_OUTOFMEMORY" => DXGI::DXGI_ERROR_REMOTE_OUTOFMEMORY,
            "DXGI_ERROR_MODE_CHANGE_IN_PROGRESS" => DXGI::DXGI_ERROR_MODE_CHANGE_IN_PROGRESS,
            "DXGI_ERROR_ACCESS_LOST" => DXGI::DXGI_ERROR_ACCESS_LOST,
            "DXGI_ERROR_WAIT_TIMEOUT" => DXGI::DXGI_ERROR_WAIT_TIMEOUT,
            "DXGI_ERROR_SESSION_DISCONNECTED" => DXGI::DXGI_ERROR_SESSION_DISCONNECTED,
            "DXGI_ERROR_RESTRICT_TO_OUTPUT_STALE" => DXGI::DXGI_ERROR_RESTRICT_TO_OUTPUT_STALE,
            "DXGI_ERROR_CANNOT_PROTECT_CONTENT" => DXGI::DXGI_ERROR_CANNOT_PROTECT_CONTENT,
            "DXGI_ERROR_ACCESS_DENIED" => DXGI::DXGI_ERROR_ACCESS_DENIED,
            "DXGI_ERROR_NAME_ALREADY_EXISTS" => DXGI::DXGI_ERROR_NAME_ALREADY_EXISTS,
            "DXGI_ERROR_SDK_COMPONENT_MISSING" => DXGI::DXGI_ERROR_SDK_COMPONENT_MISSING,
            "DXGI_ERROR_NOT_CURRENT" => DXGI::DXGI_ERROR_NOT_CURRENT,
            "DXGI_ERROR_HW_PROTECTION_OUTOFMEMORY" => DXGI::DXGI_ERROR_HW_PROTECTION_OUTOFMEMORY,
            "DXGI_ERROR_DYNAMIC_CODE_POLICY_VIOLATION" => DXGI::DXGI_ERROR_DYNAMIC_CODE_POLICY_VIOLATION,
            "DXGI_ERROR_NON_COMPOSITED_UI" => DXGI::DXGI_ERROR_NON_COMPOSITED_UI,
            "DXGI_ERROR_CACHE_CORRUPT" => DXGI::DXGI_ERROR_CACHE_CORRUPT,
            "DXGI_ERROR_CACHE_FULL" => DXGI::DXGI_ERROR_CACHE_FULL,
            "DXGI_ERROR_CACHE_HASH_COLLISION" => DXGI::DXGI_ERROR_CACHE_HASH_COLLISION,
            "DXGI_ERROR_ALREADY_EXISTS" => DXGI::DXGI_ERROR_ALREADY_EXISTS,
            "DXGI_ERROR_MPO_UNPINNED" => DXGI::DXGI_ERROR_MPO_UNPINNED,
        }
    }
    pub fn from_code(code: u32) -> DXGI {
        return match code {
            0x087A0001 => DXGI::DXGI_STATUS_OCCLUDED,
            0x087A0002 => DXGI::DXGI_STATUS_CLIPPED,
            0x087A0004 => DXGI::DXGI_STATUS_NO_REDIRECTION,
            0x087A0005 => DXGI::DXGI_STATUS_NO_DESKTOP_ACCESS,
            0x087A0006 => DXGI::DXGI_STATUS_GRAPHICS_VIDPN_SOURCE_IN_USE,
            0x087A0007 => DXGI::DXGI_STATUS_MODE_CHANGED,
            0x087A0008 => DXGI::DXGI_STATUS_MODE_CHANGE_IN_PROGRESS,
            0x087A0009 => DXGI::DXGI_STATUS_UNOCCLUDED,
            0x087A000A => DXGI::DXGI_STATUS_DDA_WAS_STILL_DRAWING,
            0x087A002F => DXGI::DXGI_STATUS_PRESENT_REQUIRED,
            0x887A0001 => DXGI::DXGI_ERROR_INVALID_CALL,
            0x887A0002 => DXGI::DXGI_ERROR_NOT_FOUND,
            0x887A0003 => DXGI::DXGI_ERROR_MORE_DATA,
            0x887A0004 => DXGI::DXGI_ERROR_UNSUPPORTED,
            0x887A0005 => DXGI::DXGI_ERROR_DEVICE_REMOVED,
            0x887A0006 => DXGI::DXGI_ERROR_DEVICE_HUNG,
            0x887A0007 => DXGI::DXGI_ERROR_DEVICE_RESET,
            0x887A000A => DXGI::DXGI_ERROR_WAS_STILL_DRAWING,
            0x887A000B => DXGI::DXGI_ERROR_FRAME_STATISTICS_DISJOINT,
            0x887A000C => DXGI::DXGI_ERROR_GRAPHICS_VIDPN_SOURCE_IN_USE,
            0x887A0020 => DXGI::DXGI_ERROR_DRIVER_INTERNAL_ERROR,
            0x887A0021 => DXGI::DXGI_ERROR_NONEXCLUSIVE,
            0x887A0022 => DXGI::DXGI_ERROR_NOT_CURRENTLY_AVAILABLE,
            0x887A0023 => DXGI::DXGI_ERROR_REMOTE_CLIENT_DISCONNECTED,
            0x887A0024 => DXGI::DXGI_ERROR_REMOTE_OUTOFMEMORY,
            0x887A0025 => DXGI::DXGI_ERROR_MODE_CHANGE_IN_PROGRESS,
            0x887A0026 => DXGI::DXGI_ERROR_ACCESS_LOST,
            0x887A0027 => DXGI::DXGI_ERROR_WAIT_TIMEOUT,
            0x887A0028 => DXGI::DXGI_ERROR_SESSION_DISCONNECTED,
            0x887A0029 => DXGI::DXGI_ERROR_RESTRICT_TO_OUTPUT_STALE,
            0x887A002A => DXGI::DXGI_ERROR_CANNOT_PROTECT_CONTENT,
            0x887A002B => DXGI::DXGI_ERROR_ACCESS_DENIED,
            0x887A002C => DXGI::DXGI_ERROR_NAME_ALREADY_EXISTS,
            0x887A002D => DXGI::DXGI_ERROR_SDK_COMPONENT_MISSING,
            0x887A002E => DXGI::DXGI_ERROR_NOT_CURRENT,
            0x887A0030 => DXGI::DXGI_ERROR_HW_PROTECTION_OUTOFMEMORY,
            0x887A0031 => DXGI::DXGI_ERROR_DYNAMIC_CODE_POLICY_VIOLATION,
            0x887A0032 => DXGI::DXGI_ERROR_NON_COMPOSITED_UI,
            0x887A0033 => DXGI::DXGI_ERROR_CACHE_CORRUPT,
            0x887A0034 => DXGI::DXGI_ERROR_CACHE_FULL,
            0x887A0035 => DXGI::DXGI_ERROR_CACHE_HASH_COLLISION,
            0x887A0036 => DXGI::DXGI_ERROR_ALREADY_EXISTS,
            0x887A0064 => DXGI::DXGI_ERROR_MPO_UNPINNED,
        }
    }
}
