/*
 * Copyright 2023 Stanislav Mikhailov (xavetar)
 *
 * Licensed under the Creative Commons Zero v1.0 Universal (CC0) License.
 * You may obtain a copy of the License at
 *
 *     http://creativecommons.org/publicdomain/zero/1.0/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the CC0 license is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// It is strictly forbidden to use the from_code method, respected Microsoft decided to define
// duplicate codes, but by defining different constant names, therefore, when using these codes,
// it is mandatory to use through from_name, and not from_code. Otherwise, it may cause undefined
// behavior or an unknown exception. Because one code corresponds to several constants.

use super::{RawError};

#[derive(Clone, Debug, Copy, Eq, Hash, Ord, PartialEq, PartialOrd)]
pub enum FWP {
    FWP_E_CALLOUT_NOT_FOUND,
    FWP_E_CONDITION_NOT_FOUND,
    FWP_E_FILTER_NOT_FOUND,
    FWP_E_LAYER_NOT_FOUND,
    FWP_E_PROVIDER_NOT_FOUND,
    FWP_E_PROVIDER_CONTEXT_NOT_FOUND,
    FWP_E_SUBLAYER_NOT_FOUND,
    FWP_E_NOT_FOUND,
    FWP_E_ALREADY_EXISTS,
    FWP_E_IN_USE,
    FWP_E_DYNAMIC_SESSION_IN_PROGRESS,
    FWP_E_WRONG_SESSION,
    FWP_E_NO_TXN_IN_PROGRESS,
    FWP_E_TXN_IN_PROGRESS,
    FWP_E_TXN_ABORTED,
    FWP_E_SESSION_ABORTED,
    FWP_E_INCOMPATIBLE_TXN,
    FWP_E_TIMEOUT,
    FWP_E_NET_EVENTS_DISABLED,
    FWP_E_INCOMPATIBLE_LAYER,
    FWP_E_KM_CLIENTS_ONLY,
    FWP_E_LIFETIME_MISMATCH,
    FWP_E_BUILTIN_OBJECT,
    FWP_E_TOO_MANY_CALLOUTS,
    FWP_E_TOO_MANY_BOOTTIME_FILTERS,
    FWP_E_NOTIFICATION_DROPPED,
    FWP_E_TRAFFIC_MISMATCH,
    FWP_E_INCOMPATIBLE_SA_STATE,
    FWP_E_NULL_POINTER,
    FWP_E_INVALID_ENUMERATOR,
    FWP_E_INVALID_FLAGS,
    FWP_E_INVALID_NET_MASK,
    FWP_E_INVALID_RANGE,
    FWP_E_INVALID_INTERVAL,
    FWP_E_ZERO_LENGTH_ARRAY,
    FWP_E_NULL_DISPLAY_NAME,
    FWP_E_INVALID_ACTION_TYPE,
    FWP_E_INVALID_WEIGHT,
    FWP_E_MATCH_TYPE_MISMATCH,
    FWP_E_TYPE_MISMATCH,
    FWP_E_OUT_OF_BOUNDS,
    FWP_E_RESERVED,
    FWP_E_DUPLICATE_CONDITION,
    FWP_E_DUPLICATE_KEYMOD,
    FWP_E_ACTION_INCOMPATIBLE_WITH_LAYER,
    FWP_E_ACTION_INCOMPATIBLE_WITH_SUBLAYER,
    FWP_E_CONTEXT_INCOMPATIBLE_WITH_LAYER,
    FWP_E_CONTEXT_INCOMPATIBLE_WITH_CALLOUT,
    FWP_E_INCOMPATIBLE_AUTH_METHOD,
    FWP_E_INCOMPATIBLE_DH_GROUP,
    FWP_E_EM_NOT_SUPPORTED,
    FWP_E_NEVER_MATCH,
    FWP_E_PROVIDER_CONTEXT_MISMATCH,
    FWP_E_INVALID_PARAMETER,
    FWP_E_TOO_MANY_SUBLAYERS,
    FWP_E_CALLOUT_NOTIFICATION_FAILED,
    FWP_E_INVALID_AUTH_TRANSFORM,
    FWP_E_INCOMPATIBLE_AUTH_CONFIG,
    FWP_E_INVALID_CIPHER_TRANSFORM,
    FWP_E_INCOMPATIBLE_CIPHER_CONFIG,
    FWP_E_INCOMPATIBLE_CIPHER_TRANSFORM,
    FWP_E_INVALID_TRANSFORM_COMBINATION,
    FWP_E_DUPLICATE_AUTH_METHOD,
    FWP_E_INVALID_TUNNEL_ENDPOINT,
    FWP_E_L2_DRIVER_NOT_READY,
    FWP_E_KEY_DICTATOR_ALREADY_REGISTERED,
    FWP_E_KEY_DICTATION_INVALID_KEYING_MATERIAL,
    FWP_E_CONNECTIONS_DISABLED,
    FWP_E_INVALID_DNS_NAME,
    FWP_E_STILL_ON,
    FWP_E_IKEEXT_NOT_RUNNING,
    FWP_E_DROP_NOICMP,
}

impl FWP {
    pub fn code(&self) -> u32 {
        return match self {
            FWP::FWP_E_CALLOUT_NOT_FOUND => 0x80320001 as u32,
            FWP::FWP_E_CONDITION_NOT_FOUND => 0x80320002 as u32,
            FWP::FWP_E_FILTER_NOT_FOUND => 0x80320003 as u32,
            FWP::FWP_E_LAYER_NOT_FOUND => 0x80320004 as u32,
            FWP::FWP_E_PROVIDER_NOT_FOUND => 0x80320005 as u32,
            FWP::FWP_E_PROVIDER_CONTEXT_NOT_FOUND => 0x80320006 as u32,
            FWP::FWP_E_SUBLAYER_NOT_FOUND => 0x80320007 as u32,
            FWP::FWP_E_NOT_FOUND => 0x80320008 as u32,
            FWP::FWP_E_ALREADY_EXISTS => 0x80320009 as u32,
            FWP::FWP_E_IN_USE => 0x8032000A as u32,
            FWP::FWP_E_DYNAMIC_SESSION_IN_PROGRESS => 0x8032000B as u32,
            FWP::FWP_E_WRONG_SESSION => 0x8032000C as u32,
            FWP::FWP_E_NO_TXN_IN_PROGRESS => 0x8032000D as u32,
            FWP::FWP_E_TXN_IN_PROGRESS => 0x8032000E as u32,
            FWP::FWP_E_TXN_ABORTED => 0x8032000F as u32,
            FWP::FWP_E_SESSION_ABORTED => 0x80320010 as u32,
            FWP::FWP_E_INCOMPATIBLE_TXN => 0x80320011 as u32,
            FWP::FWP_E_TIMEOUT => 0x80320012 as u32,
            FWP::FWP_E_NET_EVENTS_DISABLED => 0x80320013 as u32,
            FWP::FWP_E_INCOMPATIBLE_LAYER => 0x80320014 as u32,
            FWP::FWP_E_KM_CLIENTS_ONLY => 0x80320015 as u32,
            FWP::FWP_E_LIFETIME_MISMATCH => 0x80320016 as u32,
            FWP::FWP_E_BUILTIN_OBJECT => 0x80320017 as u32,
            FWP::FWP_E_TOO_MANY_CALLOUTS => 0x80320018 as u32,
            FWP::FWP_E_TOO_MANY_BOOTTIME_FILTERS => 0x80320018 as u32,
            FWP::FWP_E_NOTIFICATION_DROPPED => 0x80320019 as u32,
            FWP::FWP_E_TRAFFIC_MISMATCH => 0x8032001A as u32,
            FWP::FWP_E_INCOMPATIBLE_SA_STATE => 0x8032001B as u32,
            FWP::FWP_E_NULL_POINTER => 0x8032001C as u32,
            FWP::FWP_E_INVALID_ENUMERATOR => 0x8032001D as u32,
            FWP::FWP_E_INVALID_FLAGS => 0x8032001E as u32,
            FWP::FWP_E_INVALID_NET_MASK => 0x8032001F as u32,
            FWP::FWP_E_INVALID_RANGE => 0x80320020 as u32,
            FWP::FWP_E_INVALID_INTERVAL => 0x80320021 as u32,
            FWP::FWP_E_ZERO_LENGTH_ARRAY => 0x80320022 as u32,
            FWP::FWP_E_NULL_DISPLAY_NAME => 0x80320023 as u32,
            FWP::FWP_E_INVALID_ACTION_TYPE => 0x80320024 as u32,
            FWP::FWP_E_INVALID_WEIGHT => 0x80320025 as u32,
            FWP::FWP_E_MATCH_TYPE_MISMATCH => 0x80320026 as u32,
            FWP::FWP_E_TYPE_MISMATCH => 0x80320027 as u32,
            FWP::FWP_E_OUT_OF_BOUNDS => 0x80320028 as u32,
            FWP::FWP_E_RESERVED => 0x80320029 as u32,
            FWP::FWP_E_DUPLICATE_CONDITION => 0x8032002A as u32,
            FWP::FWP_E_DUPLICATE_KEYMOD => 0x8032002B as u32,
            FWP::FWP_E_ACTION_INCOMPATIBLE_WITH_LAYER => 0x8032002C as u32,
            FWP::FWP_E_ACTION_INCOMPATIBLE_WITH_SUBLAYER => 0x8032002D as u32,
            FWP::FWP_E_CONTEXT_INCOMPATIBLE_WITH_LAYER => 0x8032002E as u32,
            FWP::FWP_E_CONTEXT_INCOMPATIBLE_WITH_CALLOUT => 0x8032002F as u32,
            FWP::FWP_E_INCOMPATIBLE_AUTH_METHOD => 0x80320030 as u32,
            FWP::FWP_E_INCOMPATIBLE_DH_GROUP => 0x80320031 as u32,
            FWP::FWP_E_EM_NOT_SUPPORTED => 0x80320032 as u32,
            FWP::FWP_E_NEVER_MATCH => 0x80320033 as u32,
            FWP::FWP_E_PROVIDER_CONTEXT_MISMATCH => 0x80320034 as u32,
            FWP::FWP_E_INVALID_PARAMETER => 0x80320035 as u32,
            FWP::FWP_E_TOO_MANY_SUBLAYERS => 0x80320036 as u32,
            FWP::FWP_E_CALLOUT_NOTIFICATION_FAILED => 0x80320037 as u32,
            FWP::FWP_E_INVALID_AUTH_TRANSFORM => 0x80320038 as u32,
            FWP::FWP_E_INCOMPATIBLE_AUTH_CONFIG => 0x80320038 as u32,
            FWP::FWP_E_INVALID_CIPHER_TRANSFORM => 0x80320039 as u32,
            FWP::FWP_E_INCOMPATIBLE_CIPHER_CONFIG => 0x80320039 as u32,
            FWP::FWP_E_INCOMPATIBLE_CIPHER_TRANSFORM => 0x8032003A as u32,
            FWP::FWP_E_INVALID_TRANSFORM_COMBINATION => 0x8032003B as u32,
            FWP::FWP_E_DUPLICATE_AUTH_METHOD => 0x8032003C as u32,
            FWP::FWP_E_INVALID_TUNNEL_ENDPOINT => 0x8032003D as u32,
            FWP::FWP_E_L2_DRIVER_NOT_READY => 0x8032003E as u32,
            FWP::FWP_E_KEY_DICTATOR_ALREADY_REGISTERED => 0x8032003F as u32,
            FWP::FWP_E_KEY_DICTATION_INVALID_KEYING_MATERIAL => 0x80320040 as u32,
            FWP::FWP_E_CONNECTIONS_DISABLED => 0x80320041 as u32,
            FWP::FWP_E_INVALID_DNS_NAME => 0x80320042 as u32,
            FWP::FWP_E_STILL_ON => 0x80320043 as u32,
            FWP::FWP_E_IKEEXT_NOT_RUNNING => 0x80320044 as u32,
            FWP::FWP_E_DROP_NOICMP => 0x80320104 as u32,
        }
    }

    pub fn error(&self) -> RawError {
        return match self {
            FWP::FWP_E_CALLOUT_NOT_FOUND => RawError::Kind(FWP::FWP_E_CALLOUT_NOT_FOUND),
            FWP::FWP_E_CONDITION_NOT_FOUND => RawError::Kind(FWP::FWP_E_CONDITION_NOT_FOUND),
            FWP::FWP_E_FILTER_NOT_FOUND => RawError::Kind(FWP::FWP_E_FILTER_NOT_FOUND),
            FWP::FWP_E_LAYER_NOT_FOUND => RawError::Kind(FWP::FWP_E_LAYER_NOT_FOUND),
            FWP::FWP_E_PROVIDER_NOT_FOUND => RawError::Kind(FWP::FWP_E_PROVIDER_NOT_FOUND),
            FWP::FWP_E_PROVIDER_CONTEXT_NOT_FOUND => RawError::Kind(FWP::FWP_E_PROVIDER_CONTEXT_NOT_FOUND),
            FWP::FWP_E_SUBLAYER_NOT_FOUND => RawError::Kind(FWP::FWP_E_SUBLAYER_NOT_FOUND),
            FWP::FWP_E_NOT_FOUND => RawError::Kind(FWP::FWP_E_NOT_FOUND),
            FWP::FWP_E_ALREADY_EXISTS => RawError::Kind(FWP::FWP_E_ALREADY_EXISTS),
            FWP::FWP_E_IN_USE => RawError::Kind(FWP::FWP_E_IN_USE),
            FWP::FWP_E_DYNAMIC_SESSION_IN_PROGRESS => RawError::Kind(FWP::FWP_E_DYNAMIC_SESSION_IN_PROGRESS),
            FWP::FWP_E_WRONG_SESSION => RawError::Kind(FWP::FWP_E_WRONG_SESSION),
            FWP::FWP_E_NO_TXN_IN_PROGRESS => RawError::Kind(FWP::FWP_E_NO_TXN_IN_PROGRESS),
            FWP::FWP_E_TXN_IN_PROGRESS => RawError::Kind(FWP::FWP_E_TXN_IN_PROGRESS),
            FWP::FWP_E_TXN_ABORTED => RawError::Kind(FWP::FWP_E_TXN_ABORTED),
            FWP::FWP_E_SESSION_ABORTED => RawError::Kind(FWP::FWP_E_SESSION_ABORTED),
            FWP::FWP_E_INCOMPATIBLE_TXN => RawError::Kind(FWP::FWP_E_INCOMPATIBLE_TXN),
            FWP::FWP_E_TIMEOUT => RawError::Kind(FWP::FWP_E_TIMEOUT),
            FWP::FWP_E_NET_EVENTS_DISABLED => RawError::Kind(FWP::FWP_E_NET_EVENTS_DISABLED),
            FWP::FWP_E_INCOMPATIBLE_LAYER => RawError::Kind(FWP::FWP_E_INCOMPATIBLE_LAYER),
            FWP::FWP_E_KM_CLIENTS_ONLY => RawError::Kind(FWP::FWP_E_KM_CLIENTS_ONLY),
            FWP::FWP_E_LIFETIME_MISMATCH => RawError::Kind(FWP::FWP_E_LIFETIME_MISMATCH),
            FWP::FWP_E_BUILTIN_OBJECT => RawError::Kind(FWP::FWP_E_BUILTIN_OBJECT),
            FWP::FWP_E_TOO_MANY_CALLOUTS => RawError::Kind(FWP::FWP_E_TOO_MANY_CALLOUTS),
            FWP::FWP_E_TOO_MANY_BOOTTIME_FILTERS => RawError::Kind(FWP::FWP_E_TOO_MANY_BOOTTIME_FILTERS),
            FWP::FWP_E_NOTIFICATION_DROPPED => RawError::Kind(FWP::FWP_E_NOTIFICATION_DROPPED),
            FWP::FWP_E_TRAFFIC_MISMATCH => RawError::Kind(FWP::FWP_E_TRAFFIC_MISMATCH),
            FWP::FWP_E_INCOMPATIBLE_SA_STATE => RawError::Kind(FWP::FWP_E_INCOMPATIBLE_SA_STATE),
            FWP::FWP_E_NULL_POINTER => RawError::Kind(FWP::FWP_E_NULL_POINTER),
            FWP::FWP_E_INVALID_ENUMERATOR => RawError::Kind(FWP::FWP_E_INVALID_ENUMERATOR),
            FWP::FWP_E_INVALID_FLAGS => RawError::Kind(FWP::FWP_E_INVALID_FLAGS),
            FWP::FWP_E_INVALID_NET_MASK => RawError::Kind(FWP::FWP_E_INVALID_NET_MASK),
            FWP::FWP_E_INVALID_RANGE => RawError::Kind(FWP::FWP_E_INVALID_RANGE),
            FWP::FWP_E_INVALID_INTERVAL => RawError::Kind(FWP::FWP_E_INVALID_INTERVAL),
            FWP::FWP_E_ZERO_LENGTH_ARRAY => RawError::Kind(FWP::FWP_E_ZERO_LENGTH_ARRAY),
            FWP::FWP_E_NULL_DISPLAY_NAME => RawError::Kind(FWP::FWP_E_NULL_DISPLAY_NAME),
            FWP::FWP_E_INVALID_ACTION_TYPE => RawError::Kind(FWP::FWP_E_INVALID_ACTION_TYPE),
            FWP::FWP_E_INVALID_WEIGHT => RawError::Kind(FWP::FWP_E_INVALID_WEIGHT),
            FWP::FWP_E_MATCH_TYPE_MISMATCH => RawError::Kind(FWP::FWP_E_MATCH_TYPE_MISMATCH),
            FWP::FWP_E_TYPE_MISMATCH => RawError::Kind(FWP::FWP_E_TYPE_MISMATCH),
            FWP::FWP_E_OUT_OF_BOUNDS => RawError::Kind(FWP::FWP_E_OUT_OF_BOUNDS),
            FWP::FWP_E_RESERVED => RawError::Kind(FWP::FWP_E_RESERVED),
            FWP::FWP_E_DUPLICATE_CONDITION => RawError::Kind(FWP::FWP_E_DUPLICATE_CONDITION),
            FWP::FWP_E_DUPLICATE_KEYMOD => RawError::Kind(FWP::FWP_E_DUPLICATE_KEYMOD),
            FWP::FWP_E_ACTION_INCOMPATIBLE_WITH_LAYER => RawError::Kind(FWP::FWP_E_ACTION_INCOMPATIBLE_WITH_LAYER),
            FWP::FWP_E_ACTION_INCOMPATIBLE_WITH_SUBLAYER => RawError::Kind(FWP::FWP_E_ACTION_INCOMPATIBLE_WITH_SUBLAYER),
            FWP::FWP_E_CONTEXT_INCOMPATIBLE_WITH_LAYER => RawError::Kind(FWP::FWP_E_CONTEXT_INCOMPATIBLE_WITH_LAYER),
            FWP::FWP_E_CONTEXT_INCOMPATIBLE_WITH_CALLOUT => RawError::Kind(FWP::FWP_E_CONTEXT_INCOMPATIBLE_WITH_CALLOUT),
            FWP::FWP_E_INCOMPATIBLE_AUTH_METHOD => RawError::Kind(FWP::FWP_E_INCOMPATIBLE_AUTH_METHOD),
            FWP::FWP_E_INCOMPATIBLE_DH_GROUP => RawError::Kind(FWP::FWP_E_INCOMPATIBLE_DH_GROUP),
            FWP::FWP_E_EM_NOT_SUPPORTED => RawError::Kind(FWP::FWP_E_EM_NOT_SUPPORTED),
            FWP::FWP_E_NEVER_MATCH => RawError::Kind(FWP::FWP_E_NEVER_MATCH),
            FWP::FWP_E_PROVIDER_CONTEXT_MISMATCH => RawError::Kind(FWP::FWP_E_PROVIDER_CONTEXT_MISMATCH),
            FWP::FWP_E_INVALID_PARAMETER => RawError::Kind(FWP::FWP_E_INVALID_PARAMETER),
            FWP::FWP_E_TOO_MANY_SUBLAYERS => RawError::Kind(FWP::FWP_E_TOO_MANY_SUBLAYERS),
            FWP::FWP_E_CALLOUT_NOTIFICATION_FAILED => RawError::Kind(FWP::FWP_E_CALLOUT_NOTIFICATION_FAILED),
            FWP::FWP_E_INVALID_AUTH_TRANSFORM => RawError::Kind(FWP::FWP_E_INVALID_AUTH_TRANSFORM),
            FWP::FWP_E_INCOMPATIBLE_AUTH_CONFIG => RawError::Kind(FWP::FWP_E_INCOMPATIBLE_AUTH_CONFIG),
            FWP::FWP_E_INVALID_CIPHER_TRANSFORM => RawError::Kind(FWP::FWP_E_INVALID_CIPHER_TRANSFORM),
            FWP::FWP_E_INCOMPATIBLE_CIPHER_CONFIG => RawError::Kind(FWP::FWP_E_INCOMPATIBLE_CIPHER_CONFIG),
            FWP::FWP_E_INCOMPATIBLE_CIPHER_TRANSFORM => RawError::Kind(FWP::FWP_E_INCOMPATIBLE_CIPHER_TRANSFORM),
            FWP::FWP_E_INVALID_TRANSFORM_COMBINATION => RawError::Kind(FWP::FWP_E_INVALID_TRANSFORM_COMBINATION),
            FWP::FWP_E_DUPLICATE_AUTH_METHOD => RawError::Kind(FWP::FWP_E_DUPLICATE_AUTH_METHOD),
            FWP::FWP_E_INVALID_TUNNEL_ENDPOINT => RawError::Kind(FWP::FWP_E_INVALID_TUNNEL_ENDPOINT),
            FWP::FWP_E_L2_DRIVER_NOT_READY => RawError::Kind(FWP::FWP_E_L2_DRIVER_NOT_READY),
            FWP::FWP_E_KEY_DICTATOR_ALREADY_REGISTERED => RawError::Kind(FWP::FWP_E_KEY_DICTATOR_ALREADY_REGISTERED),
            FWP::FWP_E_KEY_DICTATION_INVALID_KEYING_MATERIAL => RawError::Kind(FWP::FWP_E_KEY_DICTATION_INVALID_KEYING_MATERIAL),
            FWP::FWP_E_CONNECTIONS_DISABLED => RawError::Kind(FWP::FWP_E_CONNECTIONS_DISABLED),
            FWP::FWP_E_INVALID_DNS_NAME => RawError::Kind(FWP::FWP_E_INVALID_DNS_NAME),
            FWP::FWP_E_STILL_ON => RawError::Kind(FWP::FWP_E_STILL_ON),
            FWP::FWP_E_IKEEXT_NOT_RUNNING => RawError::Kind(FWP::FWP_E_IKEEXT_NOT_RUNNING),
            FWP::FWP_E_DROP_NOICMP => RawError::Kind(FWP::FWP_E_DROP_NOICMP),
        }
    }

    pub fn description(&self) -> &'static str {
        return match self {
            FWP::FWP_E_CALLOUT_NOT_FOUND => "The callout does not exist.",
            FWP::FWP_E_CONDITION_NOT_FOUND => "The filter condition does not exist.",
            FWP::FWP_E_FILTER_NOT_FOUND => "The filter does not exist.",
            FWP::FWP_E_LAYER_NOT_FOUND => "The layer does not exist.",
            FWP::FWP_E_PROVIDER_NOT_FOUND => "The provider does not exist.",
            FWP::FWP_E_PROVIDER_CONTEXT_NOT_FOUND => "The provider context does not exist.",
            FWP::FWP_E_SUBLAYER_NOT_FOUND => "The sublayer does not exist.",
            FWP::FWP_E_NOT_FOUND => "The object does not exist.",
            FWP::FWP_E_ALREADY_EXISTS => "An object with that GUID or LUID already exists.",
            FWP::FWP_E_IN_USE => "The object is referenced by other objects so cannot be deleted.",
            FWP::FWP_E_DYNAMIC_SESSION_IN_PROGRESS => "The call is not allowed from within a dynamic session.",
            FWP::FWP_E_WRONG_SESSION => "The call was made from the wrong session so cannot be completed.",
            FWP::FWP_E_NO_TXN_IN_PROGRESS => "The call must be made from within an explicit transaction.",
            FWP::FWP_E_TXN_IN_PROGRESS => "The call is not allowed from within an explicit transaction.",
            FWP::FWP_E_TXN_ABORTED => "The explicit transaction has been forcibly cancelled.",
            FWP::FWP_E_SESSION_ABORTED => "The session has been cancelled.",
            FWP::FWP_E_INCOMPATIBLE_TXN => "The call is not allowed from within a read-only transaction.",
            FWP::FWP_E_TIMEOUT => "The call timed out while waiting to acquire the transaction lock.",
            FWP::FWP_E_NET_EVENTS_DISABLED => "Collection of network diagnostic events is disabled.",
            FWP::FWP_E_INCOMPATIBLE_LAYER => "The operation is not supported by the specified layer.",
            FWP::FWP_E_KM_CLIENTS_ONLY => "The call is allowed for kernel-mode callers only.",
            FWP::FWP_E_LIFETIME_MISMATCH => "The call tried to associate two objects with incompatible lifetimes.",
            FWP::FWP_E_BUILTIN_OBJECT => "The object is built in so cannot be deleted.",
            FWP::FWP_E_TOO_MANY_CALLOUTS => "The maximum number of callouts has been reached.",
            FWP::FWP_E_TOO_MANY_BOOTTIME_FILTERS => "The maximum number of boot-time filters has been reached.",
            FWP::FWP_E_NOTIFICATION_DROPPED => "A notification could not be delivered because a message queue is at its maximum capacity.",
            FWP::FWP_E_TRAFFIC_MISMATCH => "The traffic parameters do not match those for the security association context.",
            FWP::FWP_E_INCOMPATIBLE_SA_STATE => "The call is not allowed for the current security association state.",
            FWP::FWP_E_NULL_POINTER => "A required pointer is null.",
            FWP::FWP_E_INVALID_ENUMERATOR => "An enumerator is not valid.",
            FWP::FWP_E_INVALID_FLAGS => "The flags field contains an invalid value.",
            FWP::FWP_E_INVALID_NET_MASK => "A network mask is not valid.",
            FWP::FWP_E_INVALID_RANGE => "An FWP_RANGE is not valid.",
            FWP::FWP_E_INVALID_INTERVAL => "The time interval is not valid.",
            FWP::FWP_E_ZERO_LENGTH_ARRAY => "An array that must contain at least one element is zero length.",
            FWP::FWP_E_NULL_DISPLAY_NAME => "The displayData.name field cannot be null.",
            FWP::FWP_E_INVALID_ACTION_TYPE => "The action type is not one of the allowed action types for a filter.",
            FWP::FWP_E_INVALID_WEIGHT => "The filter weight is not valid.",
            FWP::FWP_E_MATCH_TYPE_MISMATCH => "A filter condition contains a match type that is not compatible with the operands.",
            FWP::FWP_E_TYPE_MISMATCH => "An FWP_VALUE or FWPM_CONDITION_VALUE is of the wrong type.",
            FWP::FWP_E_OUT_OF_BOUNDS => "An integer value is outside the allowed range.",
            FWP::FWP_E_RESERVED => "A reserved field is non-zero.",
            FWP::FWP_E_DUPLICATE_CONDITION => "A filter cannot contain multiple conditions operating on a single field.",
            FWP::FWP_E_DUPLICATE_KEYMOD => "A policy cannot contain the same keying module more than once.",
            FWP::FWP_E_ACTION_INCOMPATIBLE_WITH_LAYER => "The action type is not compatible with the layer.",
            FWP::FWP_E_ACTION_INCOMPATIBLE_WITH_SUBLAYER => "The action type is not compatible with the sublayer.",
            FWP::FWP_E_CONTEXT_INCOMPATIBLE_WITH_LAYER => "The raw context or the provider context is not compatible with the layer.",
            FWP::FWP_E_CONTEXT_INCOMPATIBLE_WITH_CALLOUT => "The raw context or the provider context is not compatible with the callout.",
            FWP::FWP_E_INCOMPATIBLE_AUTH_METHOD => "The authentication method is not compatible with the policy type.",
            FWP::FWP_E_INCOMPATIBLE_DH_GROUP => "The Diffie-Hellman group is not compatible with the policy type.",
            FWP::FWP_E_EM_NOT_SUPPORTED => "An IKE policy cannot contain an Extended Mode policy.",
            FWP::FWP_E_NEVER_MATCH => "The enumeration template or subscription will never match any objects.",
            FWP::FWP_E_PROVIDER_CONTEXT_MISMATCH => "The provider context is of the wrong type.",
            FWP::FWP_E_INVALID_PARAMETER => "The parameter is incorrect.",
            FWP::FWP_E_TOO_MANY_SUBLAYERS => "The maximum number of sublayers has been reached.",
            FWP::FWP_E_CALLOUT_NOTIFICATION_FAILED => "The notification function for a callout returned an error.",
            FWP::FWP_E_INVALID_AUTH_TRANSFORM => "The IPsec authentication transform is not valid.",
            FWP::FWP_E_INCOMPATIBLE_AUTH_CONFIG => "The IPsec authentication configuration is not compatible with the authentication type.",
            FWP::FWP_E_INVALID_CIPHER_TRANSFORM => "The IPsec cipher transform is not valid.",
            FWP::FWP_E_INCOMPATIBLE_CIPHER_CONFIG => "The IPsec cipher configuration is not compatible with the cipher type.",
            FWP::FWP_E_INCOMPATIBLE_CIPHER_TRANSFORM => "The IPsec cipher transform is not compatible with the policy.",
            FWP::FWP_E_INVALID_TRANSFORM_COMBINATION => "The combination of IPsec transform types is not valid.",
            FWP::FWP_E_DUPLICATE_AUTH_METHOD => "A policy cannot contain the same auth method more than once.",
            FWP::FWP_E_INVALID_TUNNEL_ENDPOINT => "A tunnel endpoint configuration is invalid.",
            FWP::FWP_E_L2_DRIVER_NOT_READY => "The WFP MAC Layers are not ready.",
            FWP::FWP_E_KEY_DICTATOR_ALREADY_REGISTERED => "A key manager capable of key dictation is already registered",
            FWP::FWP_E_KEY_DICTATION_INVALID_KEYING_MATERIAL => "A key manager dictated invalid keys",
            FWP::FWP_E_CONNECTIONS_DISABLED => "The BFE IPsec Connection Tracking is disabled.",
            FWP::FWP_E_INVALID_DNS_NAME => "The DNS name is invalid.",
            FWP::FWP_E_STILL_ON => "The engine option is still enabled due to other configuration settings.",
            FWP::FWP_E_IKEEXT_NOT_RUNNING => "The IKEEXT service is not running. This service only runs when there is IPsec policy applied to the machine.",
            FWP::FWP_E_DROP_NOICMP => "The packet should be dropped, no ICMP should be sent.",
        }
    }

    pub fn from_name(name: &str) -> FWP {
        return match name {
            "FWP_E_CALLOUT_NOT_FOUND" => FWP::FWP_E_CALLOUT_NOT_FOUND,
            "FWP_E_CONDITION_NOT_FOUND" => FWP::FWP_E_CONDITION_NOT_FOUND,
            "FWP_E_FILTER_NOT_FOUND" => FWP::FWP_E_FILTER_NOT_FOUND,
            "FWP_E_LAYER_NOT_FOUND" => FWP::FWP_E_LAYER_NOT_FOUND,
            "FWP_E_PROVIDER_NOT_FOUND" => FWP::FWP_E_PROVIDER_NOT_FOUND,
            "FWP_E_PROVIDER_CONTEXT_NOT_FOUND" => FWP::FWP_E_PROVIDER_CONTEXT_NOT_FOUND,
            "FWP_E_SUBLAYER_NOT_FOUND" => FWP::FWP_E_SUBLAYER_NOT_FOUND,
            "FWP_E_NOT_FOUND" => FWP::FWP_E_NOT_FOUND,
            "FWP_E_ALREADY_EXISTS" => FWP::FWP_E_ALREADY_EXISTS,
            "FWP_E_IN_USE" => FWP::FWP_E_IN_USE,
            "FWP_E_DYNAMIC_SESSION_IN_PROGRESS" => FWP::FWP_E_DYNAMIC_SESSION_IN_PROGRESS,
            "FWP_E_WRONG_SESSION" => FWP::FWP_E_WRONG_SESSION,
            "FWP_E_NO_TXN_IN_PROGRESS" => FWP::FWP_E_NO_TXN_IN_PROGRESS,
            "FWP_E_TXN_IN_PROGRESS" => FWP::FWP_E_TXN_IN_PROGRESS,
            "FWP_E_TXN_ABORTED" => FWP::FWP_E_TXN_ABORTED,
            "FWP_E_SESSION_ABORTED" => FWP::FWP_E_SESSION_ABORTED,
            "FWP_E_INCOMPATIBLE_TXN" => FWP::FWP_E_INCOMPATIBLE_TXN,
            "FWP_E_TIMEOUT" => FWP::FWP_E_TIMEOUT,
            "FWP_E_NET_EVENTS_DISABLED" => FWP::FWP_E_NET_EVENTS_DISABLED,
            "FWP_E_INCOMPATIBLE_LAYER" => FWP::FWP_E_INCOMPATIBLE_LAYER,
            "FWP_E_KM_CLIENTS_ONLY" => FWP::FWP_E_KM_CLIENTS_ONLY,
            "FWP_E_LIFETIME_MISMATCH" => FWP::FWP_E_LIFETIME_MISMATCH,
            "FWP_E_BUILTIN_OBJECT" => FWP::FWP_E_BUILTIN_OBJECT,
            "FWP_E_TOO_MANY_CALLOUTS" => FWP::FWP_E_TOO_MANY_CALLOUTS,
            "FWP_E_TOO_MANY_BOOTTIME_FILTERS" => FWP::FWP_E_TOO_MANY_BOOTTIME_FILTERS,
            "FWP_E_NOTIFICATION_DROPPED" => FWP::FWP_E_NOTIFICATION_DROPPED,
            "FWP_E_TRAFFIC_MISMATCH" => FWP::FWP_E_TRAFFIC_MISMATCH,
            "FWP_E_INCOMPATIBLE_SA_STATE" => FWP::FWP_E_INCOMPATIBLE_SA_STATE,
            "FWP_E_NULL_POINTER" => FWP::FWP_E_NULL_POINTER,
            "FWP_E_INVALID_ENUMERATOR" => FWP::FWP_E_INVALID_ENUMERATOR,
            "FWP_E_INVALID_FLAGS" => FWP::FWP_E_INVALID_FLAGS,
            "FWP_E_INVALID_NET_MASK" => FWP::FWP_E_INVALID_NET_MASK,
            "FWP_E_INVALID_RANGE" => FWP::FWP_E_INVALID_RANGE,
            "FWP_E_INVALID_INTERVAL" => FWP::FWP_E_INVALID_INTERVAL,
            "FWP_E_ZERO_LENGTH_ARRAY" => FWP::FWP_E_ZERO_LENGTH_ARRAY,
            "FWP_E_NULL_DISPLAY_NAME" => FWP::FWP_E_NULL_DISPLAY_NAME,
            "FWP_E_INVALID_ACTION_TYPE" => FWP::FWP_E_INVALID_ACTION_TYPE,
            "FWP_E_INVALID_WEIGHT" => FWP::FWP_E_INVALID_WEIGHT,
            "FWP_E_MATCH_TYPE_MISMATCH" => FWP::FWP_E_MATCH_TYPE_MISMATCH,
            "FWP_E_TYPE_MISMATCH" => FWP::FWP_E_TYPE_MISMATCH,
            "FWP_E_OUT_OF_BOUNDS" => FWP::FWP_E_OUT_OF_BOUNDS,
            "FWP_E_RESERVED" => FWP::FWP_E_RESERVED,
            "FWP_E_DUPLICATE_CONDITION" => FWP::FWP_E_DUPLICATE_CONDITION,
            "FWP_E_DUPLICATE_KEYMOD" => FWP::FWP_E_DUPLICATE_KEYMOD,
            "FWP_E_ACTION_INCOMPATIBLE_WITH_LAYER" => FWP::FWP_E_ACTION_INCOMPATIBLE_WITH_LAYER,
            "FWP_E_ACTION_INCOMPATIBLE_WITH_SUBLAYER" => FWP::FWP_E_ACTION_INCOMPATIBLE_WITH_SUBLAYER,
            "FWP_E_CONTEXT_INCOMPATIBLE_WITH_LAYER" => FWP::FWP_E_CONTEXT_INCOMPATIBLE_WITH_LAYER,
            "FWP_E_CONTEXT_INCOMPATIBLE_WITH_CALLOUT" => FWP::FWP_E_CONTEXT_INCOMPATIBLE_WITH_CALLOUT,
            "FWP_E_INCOMPATIBLE_AUTH_METHOD" => FWP::FWP_E_INCOMPATIBLE_AUTH_METHOD,
            "FWP_E_INCOMPATIBLE_DH_GROUP" => FWP::FWP_E_INCOMPATIBLE_DH_GROUP,
            "FWP_E_EM_NOT_SUPPORTED" => FWP::FWP_E_EM_NOT_SUPPORTED,
            "FWP_E_NEVER_MATCH" => FWP::FWP_E_NEVER_MATCH,
            "FWP_E_PROVIDER_CONTEXT_MISMATCH" => FWP::FWP_E_PROVIDER_CONTEXT_MISMATCH,
            "FWP_E_INVALID_PARAMETER" => FWP::FWP_E_INVALID_PARAMETER,
            "FWP_E_TOO_MANY_SUBLAYERS" => FWP::FWP_E_TOO_MANY_SUBLAYERS,
            "FWP_E_CALLOUT_NOTIFICATION_FAILED" => FWP::FWP_E_CALLOUT_NOTIFICATION_FAILED,
            "FWP_E_INVALID_AUTH_TRANSFORM" => FWP::FWP_E_INVALID_AUTH_TRANSFORM,
            "FWP_E_INCOMPATIBLE_AUTH_CONFIG" => FWP::FWP_E_INCOMPATIBLE_AUTH_CONFIG,
            "FWP_E_INVALID_CIPHER_TRANSFORM" => FWP::FWP_E_INVALID_CIPHER_TRANSFORM,
            "FWP_E_INCOMPATIBLE_CIPHER_CONFIG" => FWP::FWP_E_INCOMPATIBLE_CIPHER_CONFIG,
            "FWP_E_INCOMPATIBLE_CIPHER_TRANSFORM" => FWP::FWP_E_INCOMPATIBLE_CIPHER_TRANSFORM,
            "FWP_E_INVALID_TRANSFORM_COMBINATION" => FWP::FWP_E_INVALID_TRANSFORM_COMBINATION,
            "FWP_E_DUPLICATE_AUTH_METHOD" => FWP::FWP_E_DUPLICATE_AUTH_METHOD,
            "FWP_E_INVALID_TUNNEL_ENDPOINT" => FWP::FWP_E_INVALID_TUNNEL_ENDPOINT,
            "FWP_E_L2_DRIVER_NOT_READY" => FWP::FWP_E_L2_DRIVER_NOT_READY,
            "FWP_E_KEY_DICTATOR_ALREADY_REGISTERED" => FWP::FWP_E_KEY_DICTATOR_ALREADY_REGISTERED,
            "FWP_E_KEY_DICTATION_INVALID_KEYING_MATERIAL" => FWP::FWP_E_KEY_DICTATION_INVALID_KEYING_MATERIAL,
            "FWP_E_CONNECTIONS_DISABLED" => FWP::FWP_E_CONNECTIONS_DISABLED,
            "FWP_E_INVALID_DNS_NAME" => FWP::FWP_E_INVALID_DNS_NAME,
            "FWP_E_STILL_ON" => FWP::FWP_E_STILL_ON,
            "FWP_E_IKEEXT_NOT_RUNNING" => FWP::FWP_E_IKEEXT_NOT_RUNNING,
            "FWP_E_DROP_NOICMP" => FWP::FWP_E_DROP_NOICMP,
        }
    }
    pub fn from_code(code: u32) -> FWP {
        return match code {
            0x80320001 => FWP::FWP_E_CALLOUT_NOT_FOUND,
            0x80320002 => FWP::FWP_E_CONDITION_NOT_FOUND,
            0x80320003 => FWP::FWP_E_FILTER_NOT_FOUND,
            0x80320004 => FWP::FWP_E_LAYER_NOT_FOUND,
            0x80320005 => FWP::FWP_E_PROVIDER_NOT_FOUND,
            0x80320006 => FWP::FWP_E_PROVIDER_CONTEXT_NOT_FOUND,
            0x80320007 => FWP::FWP_E_SUBLAYER_NOT_FOUND,
            0x80320008 => FWP::FWP_E_NOT_FOUND,
            0x80320009 => FWP::FWP_E_ALREADY_EXISTS,
            0x8032000A => FWP::FWP_E_IN_USE,
            0x8032000B => FWP::FWP_E_DYNAMIC_SESSION_IN_PROGRESS,
            0x8032000C => FWP::FWP_E_WRONG_SESSION,
            0x8032000D => FWP::FWP_E_NO_TXN_IN_PROGRESS,
            0x8032000E => FWP::FWP_E_TXN_IN_PROGRESS,
            0x8032000F => FWP::FWP_E_TXN_ABORTED,
            0x80320010 => FWP::FWP_E_SESSION_ABORTED,
            0x80320011 => FWP::FWP_E_INCOMPATIBLE_TXN,
            0x80320012 => FWP::FWP_E_TIMEOUT,
            0x80320013 => FWP::FWP_E_NET_EVENTS_DISABLED,
            0x80320014 => FWP::FWP_E_INCOMPATIBLE_LAYER,
            0x80320015 => FWP::FWP_E_KM_CLIENTS_ONLY,
            0x80320016 => FWP::FWP_E_LIFETIME_MISMATCH,
            0x80320017 => FWP::FWP_E_BUILTIN_OBJECT,
            0x80320018 => FWP::FWP_E_TOO_MANY_CALLOUTS,
            0x80320018 => FWP::FWP_E_TOO_MANY_BOOTTIME_FILTERS,
            0x80320019 => FWP::FWP_E_NOTIFICATION_DROPPED,
            0x8032001A => FWP::FWP_E_TRAFFIC_MISMATCH,
            0x8032001B => FWP::FWP_E_INCOMPATIBLE_SA_STATE,
            0x8032001C => FWP::FWP_E_NULL_POINTER,
            0x8032001D => FWP::FWP_E_INVALID_ENUMERATOR,
            0x8032001E => FWP::FWP_E_INVALID_FLAGS,
            0x8032001F => FWP::FWP_E_INVALID_NET_MASK,
            0x80320020 => FWP::FWP_E_INVALID_RANGE,
            0x80320021 => FWP::FWP_E_INVALID_INTERVAL,
            0x80320022 => FWP::FWP_E_ZERO_LENGTH_ARRAY,
            0x80320023 => FWP::FWP_E_NULL_DISPLAY_NAME,
            0x80320024 => FWP::FWP_E_INVALID_ACTION_TYPE,
            0x80320025 => FWP::FWP_E_INVALID_WEIGHT,
            0x80320026 => FWP::FWP_E_MATCH_TYPE_MISMATCH,
            0x80320027 => FWP::FWP_E_TYPE_MISMATCH,
            0x80320028 => FWP::FWP_E_OUT_OF_BOUNDS,
            0x80320029 => FWP::FWP_E_RESERVED,
            0x8032002A => FWP::FWP_E_DUPLICATE_CONDITION,
            0x8032002B => FWP::FWP_E_DUPLICATE_KEYMOD,
            0x8032002C => FWP::FWP_E_ACTION_INCOMPATIBLE_WITH_LAYER,
            0x8032002D => FWP::FWP_E_ACTION_INCOMPATIBLE_WITH_SUBLAYER,
            0x8032002E => FWP::FWP_E_CONTEXT_INCOMPATIBLE_WITH_LAYER,
            0x8032002F => FWP::FWP_E_CONTEXT_INCOMPATIBLE_WITH_CALLOUT,
            0x80320030 => FWP::FWP_E_INCOMPATIBLE_AUTH_METHOD,
            0x80320031 => FWP::FWP_E_INCOMPATIBLE_DH_GROUP,
            0x80320032 => FWP::FWP_E_EM_NOT_SUPPORTED,
            0x80320033 => FWP::FWP_E_NEVER_MATCH,
            0x80320034 => FWP::FWP_E_PROVIDER_CONTEXT_MISMATCH,
            0x80320035 => FWP::FWP_E_INVALID_PARAMETER,
            0x80320036 => FWP::FWP_E_TOO_MANY_SUBLAYERS,
            0x80320037 => FWP::FWP_E_CALLOUT_NOTIFICATION_FAILED,
            0x80320038 => FWP::FWP_E_INVALID_AUTH_TRANSFORM,
            0x80320038 => FWP::FWP_E_INCOMPATIBLE_AUTH_CONFIG,
            0x80320039 => FWP::FWP_E_INVALID_CIPHER_TRANSFORM,
            0x80320039 => FWP::FWP_E_INCOMPATIBLE_CIPHER_CONFIG,
            0x8032003A => FWP::FWP_E_INCOMPATIBLE_CIPHER_TRANSFORM,
            0x8032003B => FWP::FWP_E_INVALID_TRANSFORM_COMBINATION,
            0x8032003C => FWP::FWP_E_DUPLICATE_AUTH_METHOD,
            0x8032003D => FWP::FWP_E_INVALID_TUNNEL_ENDPOINT,
            0x8032003E => FWP::FWP_E_L2_DRIVER_NOT_READY,
            0x8032003F => FWP::FWP_E_KEY_DICTATOR_ALREADY_REGISTERED,
            0x80320040 => FWP::FWP_E_KEY_DICTATION_INVALID_KEYING_MATERIAL,
            0x80320041 => FWP::FWP_E_CONNECTIONS_DISABLED,
            0x80320042 => FWP::FWP_E_INVALID_DNS_NAME,
            0x80320043 => FWP::FWP_E_STILL_ON,
            0x80320044 => FWP::FWP_E_IKEEXT_NOT_RUNNING,
            0x80320104 => FWP::FWP_E_DROP_NOICMP,
        }
    }
}
