/*
 * Copyright 2023 Stanislav Mikhailov (xavetar)
 *
 * Licensed under the Creative Commons Zero v1.0 Universal (CC0) License.
 * You may obtain a copy of the License at
 *
 *     http://creativecommons.org/publicdomain/zero/1.0/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the CC0 license is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// It is strictly forbidden to use the from_code method, respected Microsoft decided to define
// duplicate codes, but by defining different constant names, therefore, when using these codes,
// it is mandatory to use through from_name, and not from_code. Otherwise, it may cause undefined
// behavior or an unknown exception. Because one code corresponds to several constants.

use super::{RawError};

#[derive(Clone, Debug, Copy, Eq, Hash, Ord, PartialEq, PartialOrd)]
pub enum MBN_POWERSHELL {
    E_MBN_CONTEXT_NOT_ACTIVATED,
    E_MBN_BAD_SIM,
    E_MBN_DATA_CLASS_NOT_AVAILABLE,
    E_MBN_INVALID_ACCESS_STRING,
    E_MBN_MAX_ACTIVATED_CONTEXTS,
    E_MBN_PACKET_SVC_DETACHED,
    E_MBN_PROVIDER_NOT_VISIBLE,
    E_MBN_RADIO_POWER_OFF,
    E_MBN_SERVICE_NOT_ACTIVATED,
    E_MBN_SIM_NOT_INSERTED,
    E_MBN_VOICE_CALL_IN_PROGRESS,
    E_MBN_INVALID_CACHE,
    E_MBN_NOT_REGISTERED,
    E_MBN_PROVIDERS_NOT_FOUND,
    E_MBN_PIN_NOT_SUPPORTED,
    E_MBN_PIN_REQUIRED,
    E_MBN_PIN_DISABLED,
    E_MBN_FAILURE,
    E_MBN_INVALID_PROFILE,
    E_MBN_DEFAULT_PROFILE_EXIST,
    E_MBN_SMS_ENCODING_NOT_SUPPORTED,
    E_MBN_SMS_FILTER_NOT_SUPPORTED,
    E_MBN_SMS_INVALID_MEMORY_INDEX,
    E_MBN_SMS_LANG_NOT_SUPPORTED,
    E_MBN_SMS_MEMORY_FAILURE,
    E_MBN_SMS_NETWORK_TIMEOUT,
    E_MBN_SMS_UNKNOWN_SMSC_ADDRESS,
    E_MBN_SMS_FORMAT_NOT_SUPPORTED,
    E_MBN_SMS_OPERATION_NOT_ALLOWED,
    E_MBN_SMS_MEMORY_FULL,
}

impl MBN_POWERSHELL {
    pub fn code(&self) -> u32 {
        return match self {
            MBN_POWERSHELL::E_MBN_CONTEXT_NOT_ACTIVATED => 0x80548201 as u32,
            MBN_POWERSHELL::E_MBN_BAD_SIM => 0x80548202 as u32,
            MBN_POWERSHELL::E_MBN_DATA_CLASS_NOT_AVAILABLE => 0x80548203 as u32,
            MBN_POWERSHELL::E_MBN_INVALID_ACCESS_STRING => 0x80548204 as u32,
            MBN_POWERSHELL::E_MBN_MAX_ACTIVATED_CONTEXTS => 0x80548205 as u32,
            MBN_POWERSHELL::E_MBN_PACKET_SVC_DETACHED => 0x80548206 as u32,
            MBN_POWERSHELL::E_MBN_PROVIDER_NOT_VISIBLE => 0x80548207 as u32,
            MBN_POWERSHELL::E_MBN_RADIO_POWER_OFF => 0x80548208 as u32,
            MBN_POWERSHELL::E_MBN_SERVICE_NOT_ACTIVATED => 0x80548209 as u32,
            MBN_POWERSHELL::E_MBN_SIM_NOT_INSERTED => 0x8054820A as u32,
            MBN_POWERSHELL::E_MBN_VOICE_CALL_IN_PROGRESS => 0x8054820B as u32,
            MBN_POWERSHELL::E_MBN_INVALID_CACHE => 0x8054820C as u32,
            MBN_POWERSHELL::E_MBN_NOT_REGISTERED => 0x8054820D as u32,
            MBN_POWERSHELL::E_MBN_PROVIDERS_NOT_FOUND => 0x8054820E as u32,
            MBN_POWERSHELL::E_MBN_PIN_NOT_SUPPORTED => 0x8054820F as u32,
            MBN_POWERSHELL::E_MBN_PIN_REQUIRED => 0x80548210 as u32,
            MBN_POWERSHELL::E_MBN_PIN_DISABLED => 0x80548211 as u32,
            MBN_POWERSHELL::E_MBN_FAILURE => 0x80548212 as u32,
            MBN_POWERSHELL::E_MBN_INVALID_PROFILE => 0x80548218 as u32,
            MBN_POWERSHELL::E_MBN_DEFAULT_PROFILE_EXIST => 0x80548219 as u32,
            MBN_POWERSHELL::E_MBN_SMS_ENCODING_NOT_SUPPORTED => 0x80548220 as u32,
            MBN_POWERSHELL::E_MBN_SMS_FILTER_NOT_SUPPORTED => 0x80548221 as u32,
            MBN_POWERSHELL::E_MBN_SMS_INVALID_MEMORY_INDEX => 0x80548222 as u32,
            MBN_POWERSHELL::E_MBN_SMS_LANG_NOT_SUPPORTED => 0x80548223 as u32,
            MBN_POWERSHELL::E_MBN_SMS_MEMORY_FAILURE => 0x80548224 as u32,
            MBN_POWERSHELL::E_MBN_SMS_NETWORK_TIMEOUT => 0x80548225 as u32,
            MBN_POWERSHELL::E_MBN_SMS_UNKNOWN_SMSC_ADDRESS => 0x80548226 as u32,
            MBN_POWERSHELL::E_MBN_SMS_FORMAT_NOT_SUPPORTED => 0x80548227 as u32,
            MBN_POWERSHELL::E_MBN_SMS_OPERATION_NOT_ALLOWED => 0x80548228 as u32,
            MBN_POWERSHELL::E_MBN_SMS_MEMORY_FULL => 0x80548229 as u32,
        }
    }

    pub fn error(&self) -> RawError {
        return match self {
            MBN_POWERSHELL::E_MBN_CONTEXT_NOT_ACTIVATED => RawError::Kind(MBN_POWERSHELL::E_MBN_CONTEXT_NOT_ACTIVATED),
            MBN_POWERSHELL::E_MBN_BAD_SIM => RawError::Kind(MBN_POWERSHELL::E_MBN_BAD_SIM),
            MBN_POWERSHELL::E_MBN_DATA_CLASS_NOT_AVAILABLE => RawError::Kind(MBN_POWERSHELL::E_MBN_DATA_CLASS_NOT_AVAILABLE),
            MBN_POWERSHELL::E_MBN_INVALID_ACCESS_STRING => RawError::Kind(MBN_POWERSHELL::E_MBN_INVALID_ACCESS_STRING),
            MBN_POWERSHELL::E_MBN_MAX_ACTIVATED_CONTEXTS => RawError::Kind(MBN_POWERSHELL::E_MBN_MAX_ACTIVATED_CONTEXTS),
            MBN_POWERSHELL::E_MBN_PACKET_SVC_DETACHED => RawError::Kind(MBN_POWERSHELL::E_MBN_PACKET_SVC_DETACHED),
            MBN_POWERSHELL::E_MBN_PROVIDER_NOT_VISIBLE => RawError::Kind(MBN_POWERSHELL::E_MBN_PROVIDER_NOT_VISIBLE),
            MBN_POWERSHELL::E_MBN_RADIO_POWER_OFF => RawError::Kind(MBN_POWERSHELL::E_MBN_RADIO_POWER_OFF),
            MBN_POWERSHELL::E_MBN_SERVICE_NOT_ACTIVATED => RawError::Kind(MBN_POWERSHELL::E_MBN_SERVICE_NOT_ACTIVATED),
            MBN_POWERSHELL::E_MBN_SIM_NOT_INSERTED => RawError::Kind(MBN_POWERSHELL::E_MBN_SIM_NOT_INSERTED),
            MBN_POWERSHELL::E_MBN_VOICE_CALL_IN_PROGRESS => RawError::Kind(MBN_POWERSHELL::E_MBN_VOICE_CALL_IN_PROGRESS),
            MBN_POWERSHELL::E_MBN_INVALID_CACHE => RawError::Kind(MBN_POWERSHELL::E_MBN_INVALID_CACHE),
            MBN_POWERSHELL::E_MBN_NOT_REGISTERED => RawError::Kind(MBN_POWERSHELL::E_MBN_NOT_REGISTERED),
            MBN_POWERSHELL::E_MBN_PROVIDERS_NOT_FOUND => RawError::Kind(MBN_POWERSHELL::E_MBN_PROVIDERS_NOT_FOUND),
            MBN_POWERSHELL::E_MBN_PIN_NOT_SUPPORTED => RawError::Kind(MBN_POWERSHELL::E_MBN_PIN_NOT_SUPPORTED),
            MBN_POWERSHELL::E_MBN_PIN_REQUIRED => RawError::Kind(MBN_POWERSHELL::E_MBN_PIN_REQUIRED),
            MBN_POWERSHELL::E_MBN_PIN_DISABLED => RawError::Kind(MBN_POWERSHELL::E_MBN_PIN_DISABLED),
            MBN_POWERSHELL::E_MBN_FAILURE => RawError::Kind(MBN_POWERSHELL::E_MBN_FAILURE),
            MBN_POWERSHELL::E_MBN_INVALID_PROFILE => RawError::Kind(MBN_POWERSHELL::E_MBN_INVALID_PROFILE),
            MBN_POWERSHELL::E_MBN_DEFAULT_PROFILE_EXIST => RawError::Kind(MBN_POWERSHELL::E_MBN_DEFAULT_PROFILE_EXIST),
            MBN_POWERSHELL::E_MBN_SMS_ENCODING_NOT_SUPPORTED => RawError::Kind(MBN_POWERSHELL::E_MBN_SMS_ENCODING_NOT_SUPPORTED),
            MBN_POWERSHELL::E_MBN_SMS_FILTER_NOT_SUPPORTED => RawError::Kind(MBN_POWERSHELL::E_MBN_SMS_FILTER_NOT_SUPPORTED),
            MBN_POWERSHELL::E_MBN_SMS_INVALID_MEMORY_INDEX => RawError::Kind(MBN_POWERSHELL::E_MBN_SMS_INVALID_MEMORY_INDEX),
            MBN_POWERSHELL::E_MBN_SMS_LANG_NOT_SUPPORTED => RawError::Kind(MBN_POWERSHELL::E_MBN_SMS_LANG_NOT_SUPPORTED),
            MBN_POWERSHELL::E_MBN_SMS_MEMORY_FAILURE => RawError::Kind(MBN_POWERSHELL::E_MBN_SMS_MEMORY_FAILURE),
            MBN_POWERSHELL::E_MBN_SMS_NETWORK_TIMEOUT => RawError::Kind(MBN_POWERSHELL::E_MBN_SMS_NETWORK_TIMEOUT),
            MBN_POWERSHELL::E_MBN_SMS_UNKNOWN_SMSC_ADDRESS => RawError::Kind(MBN_POWERSHELL::E_MBN_SMS_UNKNOWN_SMSC_ADDRESS),
            MBN_POWERSHELL::E_MBN_SMS_FORMAT_NOT_SUPPORTED => RawError::Kind(MBN_POWERSHELL::E_MBN_SMS_FORMAT_NOT_SUPPORTED),
            MBN_POWERSHELL::E_MBN_SMS_OPERATION_NOT_ALLOWED => RawError::Kind(MBN_POWERSHELL::E_MBN_SMS_OPERATION_NOT_ALLOWED),
            MBN_POWERSHELL::E_MBN_SMS_MEMORY_FULL => RawError::Kind(MBN_POWERSHELL::E_MBN_SMS_MEMORY_FULL),
        }
    }

    pub fn description(&self) -> &'static str {
        return match self {
            MBN_POWERSHELL::E_MBN_CONTEXT_NOT_ACTIVATED => "Context is not activated.",
            MBN_POWERSHELL::E_MBN_BAD_SIM => "Bad SIM is inserted.",
            MBN_POWERSHELL::E_MBN_DATA_CLASS_NOT_AVAILABLE => "Requested data class is not avaialable.",
            MBN_POWERSHELL::E_MBN_INVALID_ACCESS_STRING => "Access point name (APN) or Access string is incorrect.",
            MBN_POWERSHELL::E_MBN_MAX_ACTIVATED_CONTEXTS => "Max activated contexts have reached.",
            MBN_POWERSHELL::E_MBN_PACKET_SVC_DETACHED => "Device is in packet detach state.",
            MBN_POWERSHELL::E_MBN_PROVIDER_NOT_VISIBLE => "Provider is not visible.",
            MBN_POWERSHELL::E_MBN_RADIO_POWER_OFF => "Radio is powered off.",
            MBN_POWERSHELL::E_MBN_SERVICE_NOT_ACTIVATED => "MBN subscription is not activated.",
            MBN_POWERSHELL::E_MBN_SIM_NOT_INSERTED => "SIM is not inserted.",
            MBN_POWERSHELL::E_MBN_VOICE_CALL_IN_PROGRESS => "Voice call in progress.",
            MBN_POWERSHELL::E_MBN_INVALID_CACHE => "Visible provider cache is invalid.",
            MBN_POWERSHELL::E_MBN_NOT_REGISTERED => "Device is not registered.",
            MBN_POWERSHELL::E_MBN_PROVIDERS_NOT_FOUND => "Providers not found.",
            MBN_POWERSHELL::E_MBN_PIN_NOT_SUPPORTED => "Pin is not supported.",
            MBN_POWERSHELL::E_MBN_PIN_REQUIRED => "Pin is required.",
            MBN_POWERSHELL::E_MBN_PIN_DISABLED => "PIN is disabled.",
            MBN_POWERSHELL::E_MBN_FAILURE => "Generic Failure.",
            MBN_POWERSHELL::E_MBN_INVALID_PROFILE => "Profile is invalid.",
            MBN_POWERSHELL::E_MBN_DEFAULT_PROFILE_EXIST => "Default profile exist.",
            MBN_POWERSHELL::E_MBN_SMS_ENCODING_NOT_SUPPORTED => "SMS encoding is not supported.",
            MBN_POWERSHELL::E_MBN_SMS_FILTER_NOT_SUPPORTED => "SMS filter is not supported.",
            MBN_POWERSHELL::E_MBN_SMS_INVALID_MEMORY_INDEX => "Invalid SMS memory index is used.",
            MBN_POWERSHELL::E_MBN_SMS_LANG_NOT_SUPPORTED => "SMS language is not supported.",
            MBN_POWERSHELL::E_MBN_SMS_MEMORY_FAILURE => "SMS memory failure occurred.",
            MBN_POWERSHELL::E_MBN_SMS_NETWORK_TIMEOUT => "SMS network timeout happened.",
            MBN_POWERSHELL::E_MBN_SMS_UNKNOWN_SMSC_ADDRESS => "Unknown SMSC address is used.",
            MBN_POWERSHELL::E_MBN_SMS_FORMAT_NOT_SUPPORTED => "SMS format is not supported.",
            MBN_POWERSHELL::E_MBN_SMS_OPERATION_NOT_ALLOWED => "SMS operation is not allowed.",
            MBN_POWERSHELL::E_MBN_SMS_MEMORY_FULL => "Device SMS memory is full.",
        }
    }

    pub fn from_name(name: &str) -> MBN_POWERSHELL {
        return match name {
            "E_MBN_CONTEXT_NOT_ACTIVATED" => MBN_POWERSHELL::E_MBN_CONTEXT_NOT_ACTIVATED,
            "E_MBN_BAD_SIM" => MBN_POWERSHELL::E_MBN_BAD_SIM,
            "E_MBN_DATA_CLASS_NOT_AVAILABLE" => MBN_POWERSHELL::E_MBN_DATA_CLASS_NOT_AVAILABLE,
            "E_MBN_INVALID_ACCESS_STRING" => MBN_POWERSHELL::E_MBN_INVALID_ACCESS_STRING,
            "E_MBN_MAX_ACTIVATED_CONTEXTS" => MBN_POWERSHELL::E_MBN_MAX_ACTIVATED_CONTEXTS,
            "E_MBN_PACKET_SVC_DETACHED" => MBN_POWERSHELL::E_MBN_PACKET_SVC_DETACHED,
            "E_MBN_PROVIDER_NOT_VISIBLE" => MBN_POWERSHELL::E_MBN_PROVIDER_NOT_VISIBLE,
            "E_MBN_RADIO_POWER_OFF" => MBN_POWERSHELL::E_MBN_RADIO_POWER_OFF,
            "E_MBN_SERVICE_NOT_ACTIVATED" => MBN_POWERSHELL::E_MBN_SERVICE_NOT_ACTIVATED,
            "E_MBN_SIM_NOT_INSERTED" => MBN_POWERSHELL::E_MBN_SIM_NOT_INSERTED,
            "E_MBN_VOICE_CALL_IN_PROGRESS" => MBN_POWERSHELL::E_MBN_VOICE_CALL_IN_PROGRESS,
            "E_MBN_INVALID_CACHE" => MBN_POWERSHELL::E_MBN_INVALID_CACHE,
            "E_MBN_NOT_REGISTERED" => MBN_POWERSHELL::E_MBN_NOT_REGISTERED,
            "E_MBN_PROVIDERS_NOT_FOUND" => MBN_POWERSHELL::E_MBN_PROVIDERS_NOT_FOUND,
            "E_MBN_PIN_NOT_SUPPORTED" => MBN_POWERSHELL::E_MBN_PIN_NOT_SUPPORTED,
            "E_MBN_PIN_REQUIRED" => MBN_POWERSHELL::E_MBN_PIN_REQUIRED,
            "E_MBN_PIN_DISABLED" => MBN_POWERSHELL::E_MBN_PIN_DISABLED,
            "E_MBN_FAILURE" => MBN_POWERSHELL::E_MBN_FAILURE,
            "E_MBN_INVALID_PROFILE" => MBN_POWERSHELL::E_MBN_INVALID_PROFILE,
            "E_MBN_DEFAULT_PROFILE_EXIST" => MBN_POWERSHELL::E_MBN_DEFAULT_PROFILE_EXIST,
            "E_MBN_SMS_ENCODING_NOT_SUPPORTED" => MBN_POWERSHELL::E_MBN_SMS_ENCODING_NOT_SUPPORTED,
            "E_MBN_SMS_FILTER_NOT_SUPPORTED" => MBN_POWERSHELL::E_MBN_SMS_FILTER_NOT_SUPPORTED,
            "E_MBN_SMS_INVALID_MEMORY_INDEX" => MBN_POWERSHELL::E_MBN_SMS_INVALID_MEMORY_INDEX,
            "E_MBN_SMS_LANG_NOT_SUPPORTED" => MBN_POWERSHELL::E_MBN_SMS_LANG_NOT_SUPPORTED,
            "E_MBN_SMS_MEMORY_FAILURE" => MBN_POWERSHELL::E_MBN_SMS_MEMORY_FAILURE,
            "E_MBN_SMS_NETWORK_TIMEOUT" => MBN_POWERSHELL::E_MBN_SMS_NETWORK_TIMEOUT,
            "E_MBN_SMS_UNKNOWN_SMSC_ADDRESS" => MBN_POWERSHELL::E_MBN_SMS_UNKNOWN_SMSC_ADDRESS,
            "E_MBN_SMS_FORMAT_NOT_SUPPORTED" => MBN_POWERSHELL::E_MBN_SMS_FORMAT_NOT_SUPPORTED,
            "E_MBN_SMS_OPERATION_NOT_ALLOWED" => MBN_POWERSHELL::E_MBN_SMS_OPERATION_NOT_ALLOWED,
            "E_MBN_SMS_MEMORY_FULL" => MBN_POWERSHELL::E_MBN_SMS_MEMORY_FULL,
        }
    }
    pub fn from_code(code: u32) -> MBN_POWERSHELL {
        return match code {
            0x80548201 => MBN_POWERSHELL::E_MBN_CONTEXT_NOT_ACTIVATED,
            0x80548202 => MBN_POWERSHELL::E_MBN_BAD_SIM,
            0x80548203 => MBN_POWERSHELL::E_MBN_DATA_CLASS_NOT_AVAILABLE,
            0x80548204 => MBN_POWERSHELL::E_MBN_INVALID_ACCESS_STRING,
            0x80548205 => MBN_POWERSHELL::E_MBN_MAX_ACTIVATED_CONTEXTS,
            0x80548206 => MBN_POWERSHELL::E_MBN_PACKET_SVC_DETACHED,
            0x80548207 => MBN_POWERSHELL::E_MBN_PROVIDER_NOT_VISIBLE,
            0x80548208 => MBN_POWERSHELL::E_MBN_RADIO_POWER_OFF,
            0x80548209 => MBN_POWERSHELL::E_MBN_SERVICE_NOT_ACTIVATED,
            0x8054820A => MBN_POWERSHELL::E_MBN_SIM_NOT_INSERTED,
            0x8054820B => MBN_POWERSHELL::E_MBN_VOICE_CALL_IN_PROGRESS,
            0x8054820C => MBN_POWERSHELL::E_MBN_INVALID_CACHE,
            0x8054820D => MBN_POWERSHELL::E_MBN_NOT_REGISTERED,
            0x8054820E => MBN_POWERSHELL::E_MBN_PROVIDERS_NOT_FOUND,
            0x8054820F => MBN_POWERSHELL::E_MBN_PIN_NOT_SUPPORTED,
            0x80548210 => MBN_POWERSHELL::E_MBN_PIN_REQUIRED,
            0x80548211 => MBN_POWERSHELL::E_MBN_PIN_DISABLED,
            0x80548212 => MBN_POWERSHELL::E_MBN_FAILURE,
            0x80548218 => MBN_POWERSHELL::E_MBN_INVALID_PROFILE,
            0x80548219 => MBN_POWERSHELL::E_MBN_DEFAULT_PROFILE_EXIST,
            0x80548220 => MBN_POWERSHELL::E_MBN_SMS_ENCODING_NOT_SUPPORTED,
            0x80548221 => MBN_POWERSHELL::E_MBN_SMS_FILTER_NOT_SUPPORTED,
            0x80548222 => MBN_POWERSHELL::E_MBN_SMS_INVALID_MEMORY_INDEX,
            0x80548223 => MBN_POWERSHELL::E_MBN_SMS_LANG_NOT_SUPPORTED,
            0x80548224 => MBN_POWERSHELL::E_MBN_SMS_MEMORY_FAILURE,
            0x80548225 => MBN_POWERSHELL::E_MBN_SMS_NETWORK_TIMEOUT,
            0x80548226 => MBN_POWERSHELL::E_MBN_SMS_UNKNOWN_SMSC_ADDRESS,
            0x80548227 => MBN_POWERSHELL::E_MBN_SMS_FORMAT_NOT_SUPPORTED,
            0x80548228 => MBN_POWERSHELL::E_MBN_SMS_OPERATION_NOT_ALLOWED,
            0x80548229 => MBN_POWERSHELL::E_MBN_SMS_MEMORY_FULL,
        }
    }
}
