/*
 * Copyright 2023 Stanislav Mikhailov (xavetar)
 *
 * Licensed under the Creative Commons Zero v1.0 Universal (CC0) License.
 * You may obtain a copy of the License at
 *
 *     http://creativecommons.org/publicdomain/zero/1.0/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the CC0 license is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// It is strictly forbidden to use the from_code method, respected Microsoft decided to define
// duplicate codes, but by defining different constant names, therefore, when using these codes,
// it is mandatory to use through from_name, and not from_code. Otherwise, it may cause undefined
// behavior or an unknown exception. Because one code corresponds to several constants.

use super::{RawError};

#[derive(Clone, Debug, Copy, Eq, Hash, Ord, PartialEq, PartialOrd)]
pub enum MCA {
    MCA_INFO_CPU_THERMAL_THROTTLING_REMOVED,
    MCA_INFO_NO_MORE_CORRECTED_ERROR_LOGS,
    MCA_INFO_MEMORY_PAGE_MARKED_BAD,
    AUDIT_CARD_WRITTEN,
    AUDIT_CARD_DELETE,
    AUDIT_CARD_IMPORT,
    AUDIT_STORE_IMPORT,
    AUDIT_STORE_EXPORT,
    AUDIT_STORE_DELETE,
    AUDIT_SERVICE_IDLE_STOP,
    E_EVAL_EXPIRED,
    LTMM_E_INVALID_DATA,
    LTMM_E_INVALID_HEADER,
    LTMM_E_DISABLED,
    LTMM_E_INVALID_VERSION,
    LTMM_E_STILLIMAGE_LOCKED,
    MCA_WARNING_CACHE,
    MCA_WARNING_TLB,
    MCA_WARNING_CPU_BUS,
    MCA_WARNING_REGISTER_FILE,
    MCA_WARNING_MAS,
    MCA_WARNING_MEM_UNKNOWN,
    MCA_WARNING_MEM_1_2,
    MCA_WARNING_MEM_1_2_5,
    MCA_WARNING_MEM_1_2_5_4,
    MCA_WARNING_SYSTEM_EVENT,
    MCA_WARNING_PCI_BUS_PARITY,
    MCA_WARNING_PCI_BUS_PARITY_NO_INFO,
    MCA_WARNING_PCI_BUS_SERR,
    MCA_WARNING_PCI_BUS_SERR_NO_INFO,
    MCA_WARNING_PCI_BUS_MASTER_ABORT,
    MCA_WARNING_PCI_BUS_MASTER_ABORT_NO_INFO,
    MCA_WARNING_PCI_BUS_TIMEOUT,
    MCA_WARNING_PCI_BUS_TIMEOUT_NO_INFO,
    MCA_WARNING_PCI_BUS_UNKNOWN,
    MCA_WARNING_PCI_DEVICE,
    MCA_WARNING_SMBIOS,
    MCA_WARNING_PLATFORM_SPECIFIC,
    MCA_WARNING_UNKNOWN,
    MCA_WARNING_UNKNOWN_NO_CPU,
    MCA_WARNING_CMC_THRESHOLD_EXCEEDED,
    MCA_WARNING_CPE_THRESHOLD_EXCEEDED,
    MCA_WARNING_CPU_THERMAL_THROTTLED,
    MCA_WARNING_CPU,
    MCA_ERROR_CACHE,
    MCA_ERROR_TLB,
    MCA_ERROR_CPU_BUS,
    MCA_ERROR_REGISTER_FILE,
    MCA_ERROR_MAS,
    MCA_ERROR_MEM_UNKNOWN,
    MCA_ERROR_MEM_1_2,
    MCA_ERROR_MEM_1_2_5,
    MCA_ERROR_MEM_1_2_5_4,
    MCA_ERROR_SYSTEM_EVENT,
    MCA_ERROR_PCI_BUS_PARITY,
    MCA_ERROR_PCI_BUS_PARITY_NO_INFO,
    MCA_ERROR_PCI_BUS_SERR,
    MCA_ERROR_PCI_BUS_SERR_NO_INFO,
    MCA_ERROR_PCI_BUS_MASTER_ABORT,
    MCA_ERROR_PCI_BUS_MASTER_ABORT_NO_INFO,
    MCA_ERROR_PCI_BUS_TIMEOUT,
    MCA_ERROR_PCI_BUS_TIMEOUT_NO_INFO,
    MCA_ERROR_PCI_BUS_UNKNOWN,
    MCA_ERROR_PCI_DEVICE,
    MCA_ERROR_SMBIOS,
    MCA_ERROR_PLATFORM_SPECIFIC,
    MCA_ERROR_UNKNOWN,
    MCA_ERROR_UNKNOWN_NO_CPU,
    MCA_ERROR_CPU,
    MCA_MEMORYHIERARCHY_ERROR,
    MCA_TLB_ERROR,
    MCA_BUS_ERROR,
    MCA_BUS_TIMEOUT_ERROR,
    MCA_INTERNALTIMER_ERROR,
    MCA_MICROCODE_ROM_PARITY_ERROR,
    MCA_EXTERNAL_ERROR,
    MCA_FRC_ERROR,
    E_ICARD_COMMUNICATION,
    E_ICARD_DATA_ACCESS,
    E_ICARD_EXPORT,
    E_ICARD_IDENTITY,
    E_ICARD_IMPORT,
    E_ICARD_ARGUMENT,
    E_ICARD_REQUEST,
    E_ICARD_INFORMATIONCARD,
    E_ICARD_STOREKEY,
    E_ICARD_LOGOVALIDATION,
    E_ICARD_PASSWORDVALIDATION,
    E_ICARD_POLICY,
    E_ICARD_PROCESSDIED,
    E_ICARD_SERVICEBUSY,
    E_ICARD_SERVICE,
    E_ICARD_SHUTTINGDOWN,
    E_ICARD_TOKENCREATION,
    E_ICARD_TRUSTEXCHANGE,
    E_ICARD_UNTRUSTED,
    E_ICARD_USERCANCELLED,
    E_ICARD_STORE_IMPORT,
    E_ICARD_FAIL,
    E_ICARD_UI_INITIALIZATION,
    E_ICARD_REFRESH_REQUIRED,
    E_ICARD_MISSING_APPLIESTO,
    E_ICARD_INVALID_PROOF_KEY,
    E_ICARD_UNKNOWN_REFERENCE,
    E_ICARD_FAILED_REQUIRED_CLAIMS,
}

impl MCA {
    pub fn code(&self) -> u32 {
        return match self {
            MCA::MCA_INFO_CPU_THERMAL_THROTTLING_REMOVED => 0x40050070 as u32,
            MCA::MCA_INFO_NO_MORE_CORRECTED_ERROR_LOGS => 0x40050073 as u32,
            MCA::MCA_INFO_MEMORY_PAGE_MARKED_BAD => 0x40050074 as u32,
            MCA::AUDIT_CARD_WRITTEN => 0x40050200 as u32,
            MCA::AUDIT_CARD_DELETE => 0x40050201 as u32,
            MCA::AUDIT_CARD_IMPORT => 0x40050202 as u32,
            MCA::AUDIT_STORE_IMPORT => 0x40050203 as u32,
            MCA::AUDIT_STORE_EXPORT => 0x40050204 as u32,
            MCA::AUDIT_STORE_DELETE => 0x40050205 as u32,
            MCA::AUDIT_SERVICE_IDLE_STOP => 0x40050206 as u32,
            MCA::E_EVAL_EXPIRED => 0x80050010 as u32,
            MCA::LTMM_E_INVALID_DATA => 0x80050013 as u32,
            MCA::LTMM_E_INVALID_HEADER => 0x80050014 as u32,
            MCA::LTMM_E_DISABLED => 0x80050015 as u32,
            MCA::LTMM_E_INVALID_VERSION => 0x80050016 as u32,
            MCA::LTMM_E_STILLIMAGE_LOCKED => 0x80050017 as u32,
            MCA::MCA_WARNING_CACHE => 0x8005003C as u32,
            MCA::MCA_WARNING_TLB => 0x8005003E as u32,
            MCA::MCA_WARNING_CPU_BUS => 0x80050040 as u32,
            MCA::MCA_WARNING_REGISTER_FILE => 0x80050042 as u32,
            MCA::MCA_WARNING_MAS => 0x80050044 as u32,
            MCA::MCA_WARNING_MEM_UNKNOWN => 0x80050046 as u32,
            MCA::MCA_WARNING_MEM_1_2 => 0x80050048 as u32,
            MCA::MCA_WARNING_MEM_1_2_5 => 0x8005004A as u32,
            MCA::MCA_WARNING_MEM_1_2_5_4 => 0x8005004C as u32,
            MCA::MCA_WARNING_SYSTEM_EVENT => 0x8005004E as u32,
            MCA::MCA_WARNING_PCI_BUS_PARITY => 0x80050050 as u32,
            MCA::MCA_WARNING_PCI_BUS_PARITY_NO_INFO => 0x80050052 as u32,
            MCA::MCA_WARNING_PCI_BUS_SERR => 0x80050054 as u32,
            MCA::MCA_WARNING_PCI_BUS_SERR_NO_INFO => 0x80050056 as u32,
            MCA::MCA_WARNING_PCI_BUS_MASTER_ABORT => 0x80050058 as u32,
            MCA::MCA_WARNING_PCI_BUS_MASTER_ABORT_NO_INFO => 0x8005005A as u32,
            MCA::MCA_WARNING_PCI_BUS_TIMEOUT => 0x8005005C as u32,
            MCA::MCA_WARNING_PCI_BUS_TIMEOUT_NO_INFO => 0x8005005E as u32,
            MCA::MCA_WARNING_PCI_BUS_UNKNOWN => 0x80050060 as u32,
            MCA::MCA_WARNING_PCI_DEVICE => 0x80050062 as u32,
            MCA::MCA_WARNING_SMBIOS => 0x80050064 as u32,
            MCA::MCA_WARNING_PLATFORM_SPECIFIC => 0x80050066 as u32,
            MCA::MCA_WARNING_UNKNOWN => 0x80050068 as u32,
            MCA::MCA_WARNING_UNKNOWN_NO_CPU => 0x8005006A as u32,
            MCA::MCA_WARNING_CMC_THRESHOLD_EXCEEDED => 0x8005006D as u32,
            MCA::MCA_WARNING_CPE_THRESHOLD_EXCEEDED => 0x8005006E as u32,
            MCA::MCA_WARNING_CPU_THERMAL_THROTTLED => 0x8005006F as u32,
            MCA::MCA_WARNING_CPU => 0x80050071 as u32,
            MCA::MCA_ERROR_CACHE => 0xC005003D as u32,
            MCA::MCA_ERROR_TLB => 0xC005003F as u32,
            MCA::MCA_ERROR_CPU_BUS => 0xC0050041 as u32,
            MCA::MCA_ERROR_REGISTER_FILE => 0xC0050043 as u32,
            MCA::MCA_ERROR_MAS => 0xC0050045 as u32,
            MCA::MCA_ERROR_MEM_UNKNOWN => 0xC0050047 as u32,
            MCA::MCA_ERROR_MEM_1_2 => 0xC0050049 as u32,
            MCA::MCA_ERROR_MEM_1_2_5 => 0xC005004B as u32,
            MCA::MCA_ERROR_MEM_1_2_5_4 => 0xC005004D as u32,
            MCA::MCA_ERROR_SYSTEM_EVENT => 0xC005004F as u32,
            MCA::MCA_ERROR_PCI_BUS_PARITY => 0xC0050051 as u32,
            MCA::MCA_ERROR_PCI_BUS_PARITY_NO_INFO => 0xC0050053 as u32,
            MCA::MCA_ERROR_PCI_BUS_SERR => 0xC0050055 as u32,
            MCA::MCA_ERROR_PCI_BUS_SERR_NO_INFO => 0xC0050057 as u32,
            MCA::MCA_ERROR_PCI_BUS_MASTER_ABORT => 0xC0050059 as u32,
            MCA::MCA_ERROR_PCI_BUS_MASTER_ABORT_NO_INFO => 0xC005005B as u32,
            MCA::MCA_ERROR_PCI_BUS_TIMEOUT => 0xC005005D as u32,
            MCA::MCA_ERROR_PCI_BUS_TIMEOUT_NO_INFO => 0xC005005F as u32,
            MCA::MCA_ERROR_PCI_BUS_UNKNOWN => 0xC0050061 as u32,
            MCA::MCA_ERROR_PCI_DEVICE => 0xC0050063 as u32,
            MCA::MCA_ERROR_SMBIOS => 0xC0050065 as u32,
            MCA::MCA_ERROR_PLATFORM_SPECIFIC => 0xC0050067 as u32,
            MCA::MCA_ERROR_UNKNOWN => 0xC0050069 as u32,
            MCA::MCA_ERROR_UNKNOWN_NO_CPU => 0xC005006B as u32,
            MCA::MCA_ERROR_CPU => 0xC0050072 as u32,
            MCA::MCA_MEMORYHIERARCHY_ERROR => 0xC0050078 as u32,
            MCA::MCA_TLB_ERROR => 0xC0050079 as u32,
            MCA::MCA_BUS_ERROR => 0xC005007A as u32,
            MCA::MCA_BUS_TIMEOUT_ERROR => 0xC005007B as u32,
            MCA::MCA_INTERNALTIMER_ERROR => 0xC005007C as u32,
            MCA::MCA_MICROCODE_ROM_PARITY_ERROR => 0xC005007E as u32,
            MCA::MCA_EXTERNAL_ERROR => 0xC005007F as u32,
            MCA::MCA_FRC_ERROR => 0xC0050080 as u32,
            MCA::E_ICARD_COMMUNICATION => 0xC0050100 as u32,
            MCA::E_ICARD_DATA_ACCESS => 0xC0050101 as u32,
            MCA::E_ICARD_EXPORT => 0xC0050102 as u32,
            MCA::E_ICARD_IDENTITY => 0xC0050103 as u32,
            MCA::E_ICARD_IMPORT => 0xC0050104 as u32,
            MCA::E_ICARD_ARGUMENT => 0xC0050105 as u32,
            MCA::E_ICARD_REQUEST => 0xC0050106 as u32,
            MCA::E_ICARD_INFORMATIONCARD => 0xC0050107 as u32,
            MCA::E_ICARD_STOREKEY => 0xC0050108 as u32,
            MCA::E_ICARD_LOGOVALIDATION => 0xC0050109 as u32,
            MCA::E_ICARD_PASSWORDVALIDATION => 0xC005010A as u32,
            MCA::E_ICARD_POLICY => 0xC005010B as u32,
            MCA::E_ICARD_PROCESSDIED => 0xC005010C as u32,
            MCA::E_ICARD_SERVICEBUSY => 0xC005010D as u32,
            MCA::E_ICARD_SERVICE => 0xC005010E as u32,
            MCA::E_ICARD_SHUTTINGDOWN => 0xC005010F as u32,
            MCA::E_ICARD_TOKENCREATION => 0xC0050110 as u32,
            MCA::E_ICARD_TRUSTEXCHANGE => 0xC0050111 as u32,
            MCA::E_ICARD_UNTRUSTED => 0xC0050112 as u32,
            MCA::E_ICARD_USERCANCELLED => 0xC0050113 as u32,
            MCA::E_ICARD_STORE_IMPORT => 0xC0050114 as u32,
            MCA::E_ICARD_FAIL => 0xC0050115 as u32,
            MCA::E_ICARD_UI_INITIALIZATION => 0xC005011A as u32,
            MCA::E_ICARD_REFRESH_REQUIRED => 0xC0050180 as u32,
            MCA::E_ICARD_MISSING_APPLIESTO => 0xC0050181 as u32,
            MCA::E_ICARD_INVALID_PROOF_KEY => 0xC0050182 as u32,
            MCA::E_ICARD_UNKNOWN_REFERENCE => 0xC0050183 as u32,
            MCA::E_ICARD_FAILED_REQUIRED_CLAIMS => 0xC0050184 as u32,
        }
    }

    pub fn error(&self) -> RawError {
        return match self {
            MCA::MCA_INFO_CPU_THERMAL_THROTTLING_REMOVED => RawError::Kind(MCA::MCA_INFO_CPU_THERMAL_THROTTLING_REMOVED),
            MCA::MCA_INFO_NO_MORE_CORRECTED_ERROR_LOGS => RawError::Kind(MCA::MCA_INFO_NO_MORE_CORRECTED_ERROR_LOGS),
            MCA::MCA_INFO_MEMORY_PAGE_MARKED_BAD => RawError::Kind(MCA::MCA_INFO_MEMORY_PAGE_MARKED_BAD),
            MCA::AUDIT_CARD_WRITTEN => RawError::Kind(MCA::AUDIT_CARD_WRITTEN),
            MCA::AUDIT_CARD_DELETE => RawError::Kind(MCA::AUDIT_CARD_DELETE),
            MCA::AUDIT_CARD_IMPORT => RawError::Kind(MCA::AUDIT_CARD_IMPORT),
            MCA::AUDIT_STORE_IMPORT => RawError::Kind(MCA::AUDIT_STORE_IMPORT),
            MCA::AUDIT_STORE_EXPORT => RawError::Kind(MCA::AUDIT_STORE_EXPORT),
            MCA::AUDIT_STORE_DELETE => RawError::Kind(MCA::AUDIT_STORE_DELETE),
            MCA::AUDIT_SERVICE_IDLE_STOP => RawError::Kind(MCA::AUDIT_SERVICE_IDLE_STOP),
            MCA::E_EVAL_EXPIRED => RawError::Kind(MCA::E_EVAL_EXPIRED),
            MCA::LTMM_E_INVALID_DATA => RawError::Kind(MCA::LTMM_E_INVALID_DATA),
            MCA::LTMM_E_INVALID_HEADER => RawError::Kind(MCA::LTMM_E_INVALID_HEADER),
            MCA::LTMM_E_DISABLED => RawError::Kind(MCA::LTMM_E_DISABLED),
            MCA::LTMM_E_INVALID_VERSION => RawError::Kind(MCA::LTMM_E_INVALID_VERSION),
            MCA::LTMM_E_STILLIMAGE_LOCKED => RawError::Kind(MCA::LTMM_E_STILLIMAGE_LOCKED),
            MCA::MCA_WARNING_CACHE => RawError::Kind(MCA::MCA_WARNING_CACHE),
            MCA::MCA_WARNING_TLB => RawError::Kind(MCA::MCA_WARNING_TLB),
            MCA::MCA_WARNING_CPU_BUS => RawError::Kind(MCA::MCA_WARNING_CPU_BUS),
            MCA::MCA_WARNING_REGISTER_FILE => RawError::Kind(MCA::MCA_WARNING_REGISTER_FILE),
            MCA::MCA_WARNING_MAS => RawError::Kind(MCA::MCA_WARNING_MAS),
            MCA::MCA_WARNING_MEM_UNKNOWN => RawError::Kind(MCA::MCA_WARNING_MEM_UNKNOWN),
            MCA::MCA_WARNING_MEM_1_2 => RawError::Kind(MCA::MCA_WARNING_MEM_1_2),
            MCA::MCA_WARNING_MEM_1_2_5 => RawError::Kind(MCA::MCA_WARNING_MEM_1_2_5),
            MCA::MCA_WARNING_MEM_1_2_5_4 => RawError::Kind(MCA::MCA_WARNING_MEM_1_2_5_4),
            MCA::MCA_WARNING_SYSTEM_EVENT => RawError::Kind(MCA::MCA_WARNING_SYSTEM_EVENT),
            MCA::MCA_WARNING_PCI_BUS_PARITY => RawError::Kind(MCA::MCA_WARNING_PCI_BUS_PARITY),
            MCA::MCA_WARNING_PCI_BUS_PARITY_NO_INFO => RawError::Kind(MCA::MCA_WARNING_PCI_BUS_PARITY_NO_INFO),
            MCA::MCA_WARNING_PCI_BUS_SERR => RawError::Kind(MCA::MCA_WARNING_PCI_BUS_SERR),
            MCA::MCA_WARNING_PCI_BUS_SERR_NO_INFO => RawError::Kind(MCA::MCA_WARNING_PCI_BUS_SERR_NO_INFO),
            MCA::MCA_WARNING_PCI_BUS_MASTER_ABORT => RawError::Kind(MCA::MCA_WARNING_PCI_BUS_MASTER_ABORT),
            MCA::MCA_WARNING_PCI_BUS_MASTER_ABORT_NO_INFO => RawError::Kind(MCA::MCA_WARNING_PCI_BUS_MASTER_ABORT_NO_INFO),
            MCA::MCA_WARNING_PCI_BUS_TIMEOUT => RawError::Kind(MCA::MCA_WARNING_PCI_BUS_TIMEOUT),
            MCA::MCA_WARNING_PCI_BUS_TIMEOUT_NO_INFO => RawError::Kind(MCA::MCA_WARNING_PCI_BUS_TIMEOUT_NO_INFO),
            MCA::MCA_WARNING_PCI_BUS_UNKNOWN => RawError::Kind(MCA::MCA_WARNING_PCI_BUS_UNKNOWN),
            MCA::MCA_WARNING_PCI_DEVICE => RawError::Kind(MCA::MCA_WARNING_PCI_DEVICE),
            MCA::MCA_WARNING_SMBIOS => RawError::Kind(MCA::MCA_WARNING_SMBIOS),
            MCA::MCA_WARNING_PLATFORM_SPECIFIC => RawError::Kind(MCA::MCA_WARNING_PLATFORM_SPECIFIC),
            MCA::MCA_WARNING_UNKNOWN => RawError::Kind(MCA::MCA_WARNING_UNKNOWN),
            MCA::MCA_WARNING_UNKNOWN_NO_CPU => RawError::Kind(MCA::MCA_WARNING_UNKNOWN_NO_CPU),
            MCA::MCA_WARNING_CMC_THRESHOLD_EXCEEDED => RawError::Kind(MCA::MCA_WARNING_CMC_THRESHOLD_EXCEEDED),
            MCA::MCA_WARNING_CPE_THRESHOLD_EXCEEDED => RawError::Kind(MCA::MCA_WARNING_CPE_THRESHOLD_EXCEEDED),
            MCA::MCA_WARNING_CPU_THERMAL_THROTTLED => RawError::Kind(MCA::MCA_WARNING_CPU_THERMAL_THROTTLED),
            MCA::MCA_WARNING_CPU => RawError::Kind(MCA::MCA_WARNING_CPU),
            MCA::MCA_ERROR_CACHE => RawError::Kind(MCA::MCA_ERROR_CACHE),
            MCA::MCA_ERROR_TLB => RawError::Kind(MCA::MCA_ERROR_TLB),
            MCA::MCA_ERROR_CPU_BUS => RawError::Kind(MCA::MCA_ERROR_CPU_BUS),
            MCA::MCA_ERROR_REGISTER_FILE => RawError::Kind(MCA::MCA_ERROR_REGISTER_FILE),
            MCA::MCA_ERROR_MAS => RawError::Kind(MCA::MCA_ERROR_MAS),
            MCA::MCA_ERROR_MEM_UNKNOWN => RawError::Kind(MCA::MCA_ERROR_MEM_UNKNOWN),
            MCA::MCA_ERROR_MEM_1_2 => RawError::Kind(MCA::MCA_ERROR_MEM_1_2),
            MCA::MCA_ERROR_MEM_1_2_5 => RawError::Kind(MCA::MCA_ERROR_MEM_1_2_5),
            MCA::MCA_ERROR_MEM_1_2_5_4 => RawError::Kind(MCA::MCA_ERROR_MEM_1_2_5_4),
            MCA::MCA_ERROR_SYSTEM_EVENT => RawError::Kind(MCA::MCA_ERROR_SYSTEM_EVENT),
            MCA::MCA_ERROR_PCI_BUS_PARITY => RawError::Kind(MCA::MCA_ERROR_PCI_BUS_PARITY),
            MCA::MCA_ERROR_PCI_BUS_PARITY_NO_INFO => RawError::Kind(MCA::MCA_ERROR_PCI_BUS_PARITY_NO_INFO),
            MCA::MCA_ERROR_PCI_BUS_SERR => RawError::Kind(MCA::MCA_ERROR_PCI_BUS_SERR),
            MCA::MCA_ERROR_PCI_BUS_SERR_NO_INFO => RawError::Kind(MCA::MCA_ERROR_PCI_BUS_SERR_NO_INFO),
            MCA::MCA_ERROR_PCI_BUS_MASTER_ABORT => RawError::Kind(MCA::MCA_ERROR_PCI_BUS_MASTER_ABORT),
            MCA::MCA_ERROR_PCI_BUS_MASTER_ABORT_NO_INFO => RawError::Kind(MCA::MCA_ERROR_PCI_BUS_MASTER_ABORT_NO_INFO),
            MCA::MCA_ERROR_PCI_BUS_TIMEOUT => RawError::Kind(MCA::MCA_ERROR_PCI_BUS_TIMEOUT),
            MCA::MCA_ERROR_PCI_BUS_TIMEOUT_NO_INFO => RawError::Kind(MCA::MCA_ERROR_PCI_BUS_TIMEOUT_NO_INFO),
            MCA::MCA_ERROR_PCI_BUS_UNKNOWN => RawError::Kind(MCA::MCA_ERROR_PCI_BUS_UNKNOWN),
            MCA::MCA_ERROR_PCI_DEVICE => RawError::Kind(MCA::MCA_ERROR_PCI_DEVICE),
            MCA::MCA_ERROR_SMBIOS => RawError::Kind(MCA::MCA_ERROR_SMBIOS),
            MCA::MCA_ERROR_PLATFORM_SPECIFIC => RawError::Kind(MCA::MCA_ERROR_PLATFORM_SPECIFIC),
            MCA::MCA_ERROR_UNKNOWN => RawError::Kind(MCA::MCA_ERROR_UNKNOWN),
            MCA::MCA_ERROR_UNKNOWN_NO_CPU => RawError::Kind(MCA::MCA_ERROR_UNKNOWN_NO_CPU),
            MCA::MCA_ERROR_CPU => RawError::Kind(MCA::MCA_ERROR_CPU),
            MCA::MCA_MEMORYHIERARCHY_ERROR => RawError::Kind(MCA::MCA_MEMORYHIERARCHY_ERROR),
            MCA::MCA_TLB_ERROR => RawError::Kind(MCA::MCA_TLB_ERROR),
            MCA::MCA_BUS_ERROR => RawError::Kind(MCA::MCA_BUS_ERROR),
            MCA::MCA_BUS_TIMEOUT_ERROR => RawError::Kind(MCA::MCA_BUS_TIMEOUT_ERROR),
            MCA::MCA_INTERNALTIMER_ERROR => RawError::Kind(MCA::MCA_INTERNALTIMER_ERROR),
            MCA::MCA_MICROCODE_ROM_PARITY_ERROR => RawError::Kind(MCA::MCA_MICROCODE_ROM_PARITY_ERROR),
            MCA::MCA_EXTERNAL_ERROR => RawError::Kind(MCA::MCA_EXTERNAL_ERROR),
            MCA::MCA_FRC_ERROR => RawError::Kind(MCA::MCA_FRC_ERROR),
            MCA::E_ICARD_COMMUNICATION => RawError::Kind(MCA::E_ICARD_COMMUNICATION),
            MCA::E_ICARD_DATA_ACCESS => RawError::Kind(MCA::E_ICARD_DATA_ACCESS),
            MCA::E_ICARD_EXPORT => RawError::Kind(MCA::E_ICARD_EXPORT),
            MCA::E_ICARD_IDENTITY => RawError::Kind(MCA::E_ICARD_IDENTITY),
            MCA::E_ICARD_IMPORT => RawError::Kind(MCA::E_ICARD_IMPORT),
            MCA::E_ICARD_ARGUMENT => RawError::Kind(MCA::E_ICARD_ARGUMENT),
            MCA::E_ICARD_REQUEST => RawError::Kind(MCA::E_ICARD_REQUEST),
            MCA::E_ICARD_INFORMATIONCARD => RawError::Kind(MCA::E_ICARD_INFORMATIONCARD),
            MCA::E_ICARD_STOREKEY => RawError::Kind(MCA::E_ICARD_STOREKEY),
            MCA::E_ICARD_LOGOVALIDATION => RawError::Kind(MCA::E_ICARD_LOGOVALIDATION),
            MCA::E_ICARD_PASSWORDVALIDATION => RawError::Kind(MCA::E_ICARD_PASSWORDVALIDATION),
            MCA::E_ICARD_POLICY => RawError::Kind(MCA::E_ICARD_POLICY),
            MCA::E_ICARD_PROCESSDIED => RawError::Kind(MCA::E_ICARD_PROCESSDIED),
            MCA::E_ICARD_SERVICEBUSY => RawError::Kind(MCA::E_ICARD_SERVICEBUSY),
            MCA::E_ICARD_SERVICE => RawError::Kind(MCA::E_ICARD_SERVICE),
            MCA::E_ICARD_SHUTTINGDOWN => RawError::Kind(MCA::E_ICARD_SHUTTINGDOWN),
            MCA::E_ICARD_TOKENCREATION => RawError::Kind(MCA::E_ICARD_TOKENCREATION),
            MCA::E_ICARD_TRUSTEXCHANGE => RawError::Kind(MCA::E_ICARD_TRUSTEXCHANGE),
            MCA::E_ICARD_UNTRUSTED => RawError::Kind(MCA::E_ICARD_UNTRUSTED),
            MCA::E_ICARD_USERCANCELLED => RawError::Kind(MCA::E_ICARD_USERCANCELLED),
            MCA::E_ICARD_STORE_IMPORT => RawError::Kind(MCA::E_ICARD_STORE_IMPORT),
            MCA::E_ICARD_FAIL => RawError::Kind(MCA::E_ICARD_FAIL),
            MCA::E_ICARD_UI_INITIALIZATION => RawError::Kind(MCA::E_ICARD_UI_INITIALIZATION),
            MCA::E_ICARD_REFRESH_REQUIRED => RawError::Kind(MCA::E_ICARD_REFRESH_REQUIRED),
            MCA::E_ICARD_MISSING_APPLIESTO => RawError::Kind(MCA::E_ICARD_MISSING_APPLIESTO),
            MCA::E_ICARD_INVALID_PROOF_KEY => RawError::Kind(MCA::E_ICARD_INVALID_PROOF_KEY),
            MCA::E_ICARD_UNKNOWN_REFERENCE => RawError::Kind(MCA::E_ICARD_UNKNOWN_REFERENCE),
            MCA::E_ICARD_FAILED_REQUIRED_CLAIMS => RawError::Kind(MCA::E_ICARD_FAILED_REQUIRED_CLAIMS),
        }
    }

    pub fn description(&self) -> &'static str {
        return match self {
            MCA::MCA_INFO_CPU_THERMAL_THROTTLING_REMOVED => "Machine Check Event reported is a CPU thermal throttling event reported from CPU %1. The CPU has dropped below the temperature limit and throttling has been removed. %2 additional error(s) are contained within the record.",
            MCA::MCA_INFO_NO_MORE_CORRECTED_ERROR_LOGS => "The maximum number of Machine Check Event corrected error events that can be saved to the Event Log has been reached. Logging of these events has been disabled.",
            MCA::MCA_INFO_MEMORY_PAGE_MARKED_BAD => "The memory page at physical address %1 has encountered multiple corrected hardware error events. As a result it will no longer be used by Windows.",
            MCA::AUDIT_CARD_WRITTEN => "Description is missing",
            MCA::AUDIT_CARD_DELETE => "Description is missing",
            MCA::AUDIT_CARD_IMPORT => "Description is missing",
            MCA::AUDIT_STORE_IMPORT => "Description is missing",
            MCA::AUDIT_STORE_EXPORT => "Description is missing",
            MCA::AUDIT_STORE_DELETE => "Description is missing",
            MCA::AUDIT_SERVICE_IDLE_STOP => "Description is missing",
            MCA::E_EVAL_EXPIRED => "The evaluation period for the filter has expired.",
            MCA::LTMM_E_INVALID_DATA => "Invalid data.",
            MCA::LTMM_E_INVALID_HEADER => "Invalid packed data.",
            MCA::LTMM_E_DISABLED => "This feature is disabled.",
            MCA::LTMM_E_INVALID_VERSION => "This version is not supported.",
            MCA::LTMM_E_STILLIMAGE_LOCKED => "The LEADTOOLS still image toolkit needs to be unlocked.",
            MCA::MCA_WARNING_CACHE => "Machine Check Event reported is a corrected level %3 Cache error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_WARNING_TLB => "Machine Check Event reported is a corrected level %3 translation Buffer error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_WARNING_CPU_BUS => "Machine Check Event reported is a corrected External/Internal bus error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_WARNING_REGISTER_FILE => "Machine Check Event reported is a corrected internal CPU register access error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_WARNING_MAS => "Machine Check Event reported is a corrected Micro Architecture Structure error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_WARNING_MEM_UNKNOWN => "Machine Check Event reported is a corrected ECC memory error at an unknown physical address reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_WARNING_MEM_1_2 => "Machine Check Event reported is a corrected ECC memory error at physical address %3 reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_WARNING_MEM_1_2_5 => "Machine Check Event reported is a corrected ECC memory error at physical address %3 on memory module %4 reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_WARNING_MEM_1_2_5_4 => "Machine Check Event reported is a corrected ECC memory error at physical address %3 on memory module %4 on memory card %5 reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_WARNING_SYSTEM_EVENT => "Machine Check Event reported is a corrected System Event error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_WARNING_PCI_BUS_PARITY => "Machine Check Event reported is a corrected PCI bus Parity error during a transaction type %3 at address %4 on PCI bus %5 reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_WARNING_PCI_BUS_PARITY_NO_INFO => "Machine Check Event reported is a corrected PCI bus Parity error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_WARNING_PCI_BUS_SERR => "Machine Check Event reported is a corrected PCI bus SERR error during a transaction type %3 at address %4 on PCI bus %5 reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_WARNING_PCI_BUS_SERR_NO_INFO => "Machine Check Event reported is a corrected PCI bus SERR error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_WARNING_PCI_BUS_MASTER_ABORT => "Machine Check Event reported is a corrected PCI bus Master abort error during a transaction type %3 at address %4 on PCI bus %5 reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_WARNING_PCI_BUS_MASTER_ABORT_NO_INFO => "Machine Check Event reported is a corrected PCI bus Master abort error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_WARNING_PCI_BUS_TIMEOUT => "Machine Check Event reported is a corrected PCI bus Timeout error during a transaction type %3 at address %4 on PCI bus %5 reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_WARNING_PCI_BUS_TIMEOUT_NO_INFO => "Machine Check Event reported is a corrected PCI bus Timeout error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_WARNING_PCI_BUS_UNKNOWN => "Machine Check Event reported is an unknown corrected PCI bus error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_WARNING_PCI_DEVICE => "Machine Check Event reported is a corrected PCI component error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_WARNING_SMBIOS => "Machine Check Event reported is a corrected SMBIOS Device Type %3 error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_WARNING_PLATFORM_SPECIFIC => "Machine Check Event reported is a corrected Platform Specific error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_WARNING_UNKNOWN => "Machine Check Event reported is a corrected error reported to CPU %1.",
            MCA::MCA_WARNING_UNKNOWN_NO_CPU => "Machine Check Event reported is a corrected error.",
            MCA::MCA_WARNING_CMC_THRESHOLD_EXCEEDED => "Corrected Machine Check Interrupt threshold exceeded. Interrupt has been disabled. Polling mode has been enabled.",
            MCA::MCA_WARNING_CPE_THRESHOLD_EXCEEDED => "Corrected Platform Error Interrupt threshold exceeded. Interrupt has been disabled. Polling mode has been enabled.",
            MCA::MCA_WARNING_CPU_THERMAL_THROTTLED => "Machine Check Event reported is a CPU thermal throttling event reported from CPU %1. The CPU has exceeded the temperature limit and has been throttled down. %2 additional error(s) are contained within the record.",
            MCA::MCA_WARNING_CPU => "Machine Check Event reported is a corrected CPU error reported from CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_ERROR_CACHE => "Machine Check Event reported is a fatal level %3 Cache error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_ERROR_TLB => "Machine Check Event reported is a fatal level %3 translation Buffer error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_ERROR_CPU_BUS => "Machine Check Event reported is a fatal External/Internal bus error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_ERROR_REGISTER_FILE => "Machine Check Event reported is a fatal internal CPU register access error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_ERROR_MAS => "Machine Check Event reported is a fatal Micro Architecture Structure error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_ERROR_MEM_UNKNOWN => "Machine Check Event reported is a fatal ECC memory error at an unknown physical address reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_ERROR_MEM_1_2 => "Machine Check Event reported is a fatal ECC memory error at physical address %3 reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_ERROR_MEM_1_2_5 => "Machine Check Event reported is a fatal ECC memory error at physical address %3 on memory module %4 reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_ERROR_MEM_1_2_5_4 => "Machine Check Event reported is a fatal ECC memory error at physical address %3 on memory module %4 on memory card %5 reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_ERROR_SYSTEM_EVENT => "Machine Check Event reported is a fatal System Event error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_ERROR_PCI_BUS_PARITY => "Machine Check Event reported is a fatal PCI bus Parity error during a transaction type %3 at address %4 on PCI bus %5 reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_ERROR_PCI_BUS_PARITY_NO_INFO => "Machine Check Event reported is a fatal PCI bus Parity error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_ERROR_PCI_BUS_SERR => "Machine Check Event reported is a fatal PCI bus SERR error during a transaction type %3 at address %4 on PCI bus %5 reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_ERROR_PCI_BUS_SERR_NO_INFO => "Machine Check Event reported is a fatal PCI bus SERR error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_ERROR_PCI_BUS_MASTER_ABORT => "Machine Check Event reported is a fatal PCI bus Master abort error during a transaction type %3 at address %4 on PCI bus %5 reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_ERROR_PCI_BUS_MASTER_ABORT_NO_INFO => "Machine Check Event reported is a fatal PCI bus Master abort error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_ERROR_PCI_BUS_TIMEOUT => "Machine Check Event reported is a fatal PCI bus Timeout error during a transaction type %3 at address %4 on PCI bus %5 reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_ERROR_PCI_BUS_TIMEOUT_NO_INFO => "Machine Check Event reported is a fatal PCI bus Timeout error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_ERROR_PCI_BUS_UNKNOWN => "Machine Check Event reported is an unknown fatal PCI bus error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_ERROR_PCI_DEVICE => "Machine Check Event reported is a fatal PCI component error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_ERROR_SMBIOS => "Machine Check Event reported is a fatal SMBIOS Device Type %3 error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_ERROR_PLATFORM_SPECIFIC => "Machine Check Event reported is a fatal Platform Specific error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_ERROR_UNKNOWN => "Machine Check Event reported is a fatal error reported to CPU %1.",
            MCA::MCA_ERROR_UNKNOWN_NO_CPU => "Machine Check Event reported is a fatal error.",
            MCA::MCA_ERROR_CPU => "Machine Check Event reported is a fatal CPU error reported to CPU %1. %2 additional error(s) are contained within the record.",
            MCA::MCA_MEMORYHIERARCHY_ERROR => "Machine Check Event reported is a fatal memory hierarchy error.%r Trasaction Type: %1%r Memory Hierarchy Level: %2%r Request Type: %3%r Address: %4.",
            MCA::MCA_TLB_ERROR => "Machine Check Event reported is a fatal TLB error.%r Transaction Type: %1%r Memory Hierarchy Level: %2%r Address: %3.",
            MCA::MCA_BUS_ERROR => "Machine Check Event reported is a fatal Bus or Interconnect error.%r Memory Hierarchy Level: %1%r Participation: %2%r Request Type: %3%r Memory/IO: %4%r Address: %5.",
            MCA::MCA_BUS_TIMEOUT_ERROR => "Machine Check Event reported is a fatal Bus or Interconnect timeout error.%r Memory Hierarchy Level: %1%r Participation: %2%r Request Type: %3%r Memory/IO: %4%r Address: %5.",
            MCA::MCA_INTERNALTIMER_ERROR => "Machine Check Event reported is a fatal internal watchdog timer error.",
            MCA::MCA_MICROCODE_ROM_PARITY_ERROR => "Machine Check Event reported is a fatal microsoft ROM parity error.",
            MCA::MCA_EXTERNAL_ERROR => "Machine Check Event reported is a fatal condition. A processor received an external signal that an unrecoverable error has occurred.",
            MCA::MCA_FRC_ERROR => "Machine Check Event reported is a fatal functional redundancy check error.",
            MCA::E_ICARD_COMMUNICATION => "Description is missing",
            MCA::E_ICARD_DATA_ACCESS => "Description is missing",
            MCA::E_ICARD_EXPORT => "Description is missing",
            MCA::E_ICARD_IDENTITY => "Description is missing",
            MCA::E_ICARD_IMPORT => "Description is missing",
            MCA::E_ICARD_ARGUMENT => "Description is missing",
            MCA::E_ICARD_REQUEST => "Description is missing",
            MCA::E_ICARD_INFORMATIONCARD => "Description is missing",
            MCA::E_ICARD_STOREKEY => "Description is missing",
            MCA::E_ICARD_LOGOVALIDATION => "Description is missing",
            MCA::E_ICARD_PASSWORDVALIDATION => "Description is missing",
            MCA::E_ICARD_POLICY => "Description is missing",
            MCA::E_ICARD_PROCESSDIED => "Description is missing",
            MCA::E_ICARD_SERVICEBUSY => "Description is missing",
            MCA::E_ICARD_SERVICE => "Description is missing",
            MCA::E_ICARD_SHUTTINGDOWN => "Description is missing",
            MCA::E_ICARD_TOKENCREATION => "Description is missing",
            MCA::E_ICARD_TRUSTEXCHANGE => "Description is missing",
            MCA::E_ICARD_UNTRUSTED => "Description is missing",
            MCA::E_ICARD_USERCANCELLED => "Description is missing",
            MCA::E_ICARD_STORE_IMPORT => "Description is missing",
            MCA::E_ICARD_FAIL => "Description is missing",
            MCA::E_ICARD_UI_INITIALIZATION => "Description is missing",
            MCA::E_ICARD_REFRESH_REQUIRED => "Description is missing",
            MCA::E_ICARD_MISSING_APPLIESTO => "Description is missing",
            MCA::E_ICARD_INVALID_PROOF_KEY => "Description is missing",
            MCA::E_ICARD_UNKNOWN_REFERENCE => "Description is missing",
            MCA::E_ICARD_FAILED_REQUIRED_CLAIMS => "Description is missing",
        }
    }

    pub fn from_name(name: &str) -> MCA {
        return match name {
            "MCA_INFO_CPU_THERMAL_THROTTLING_REMOVED" => MCA::MCA_INFO_CPU_THERMAL_THROTTLING_REMOVED,
            "MCA_INFO_NO_MORE_CORRECTED_ERROR_LOGS" => MCA::MCA_INFO_NO_MORE_CORRECTED_ERROR_LOGS,
            "MCA_INFO_MEMORY_PAGE_MARKED_BAD" => MCA::MCA_INFO_MEMORY_PAGE_MARKED_BAD,
            "AUDIT_CARD_WRITTEN" => MCA::AUDIT_CARD_WRITTEN,
            "AUDIT_CARD_DELETE" => MCA::AUDIT_CARD_DELETE,
            "AUDIT_CARD_IMPORT" => MCA::AUDIT_CARD_IMPORT,
            "AUDIT_STORE_IMPORT" => MCA::AUDIT_STORE_IMPORT,
            "AUDIT_STORE_EXPORT" => MCA::AUDIT_STORE_EXPORT,
            "AUDIT_STORE_DELETE" => MCA::AUDIT_STORE_DELETE,
            "AUDIT_SERVICE_IDLE_STOP" => MCA::AUDIT_SERVICE_IDLE_STOP,
            "E_EVAL_EXPIRED" => MCA::E_EVAL_EXPIRED,
            "LTMM_E_INVALID_DATA" => MCA::LTMM_E_INVALID_DATA,
            "LTMM_E_INVALID_HEADER" => MCA::LTMM_E_INVALID_HEADER,
            "LTMM_E_DISABLED" => MCA::LTMM_E_DISABLED,
            "LTMM_E_INVALID_VERSION" => MCA::LTMM_E_INVALID_VERSION,
            "LTMM_E_STILLIMAGE_LOCKED" => MCA::LTMM_E_STILLIMAGE_LOCKED,
            "MCA_WARNING_CACHE" => MCA::MCA_WARNING_CACHE,
            "MCA_WARNING_TLB" => MCA::MCA_WARNING_TLB,
            "MCA_WARNING_CPU_BUS" => MCA::MCA_WARNING_CPU_BUS,
            "MCA_WARNING_REGISTER_FILE" => MCA::MCA_WARNING_REGISTER_FILE,
            "MCA_WARNING_MAS" => MCA::MCA_WARNING_MAS,
            "MCA_WARNING_MEM_UNKNOWN" => MCA::MCA_WARNING_MEM_UNKNOWN,
            "MCA_WARNING_MEM_1_2" => MCA::MCA_WARNING_MEM_1_2,
            "MCA_WARNING_MEM_1_2_5" => MCA::MCA_WARNING_MEM_1_2_5,
            "MCA_WARNING_MEM_1_2_5_4" => MCA::MCA_WARNING_MEM_1_2_5_4,
            "MCA_WARNING_SYSTEM_EVENT" => MCA::MCA_WARNING_SYSTEM_EVENT,
            "MCA_WARNING_PCI_BUS_PARITY" => MCA::MCA_WARNING_PCI_BUS_PARITY,
            "MCA_WARNING_PCI_BUS_PARITY_NO_INFO" => MCA::MCA_WARNING_PCI_BUS_PARITY_NO_INFO,
            "MCA_WARNING_PCI_BUS_SERR" => MCA::MCA_WARNING_PCI_BUS_SERR,
            "MCA_WARNING_PCI_BUS_SERR_NO_INFO" => MCA::MCA_WARNING_PCI_BUS_SERR_NO_INFO,
            "MCA_WARNING_PCI_BUS_MASTER_ABORT" => MCA::MCA_WARNING_PCI_BUS_MASTER_ABORT,
            "MCA_WARNING_PCI_BUS_MASTER_ABORT_NO_INFO" => MCA::MCA_WARNING_PCI_BUS_MASTER_ABORT_NO_INFO,
            "MCA_WARNING_PCI_BUS_TIMEOUT" => MCA::MCA_WARNING_PCI_BUS_TIMEOUT,
            "MCA_WARNING_PCI_BUS_TIMEOUT_NO_INFO" => MCA::MCA_WARNING_PCI_BUS_TIMEOUT_NO_INFO,
            "MCA_WARNING_PCI_BUS_UNKNOWN" => MCA::MCA_WARNING_PCI_BUS_UNKNOWN,
            "MCA_WARNING_PCI_DEVICE" => MCA::MCA_WARNING_PCI_DEVICE,
            "MCA_WARNING_SMBIOS" => MCA::MCA_WARNING_SMBIOS,
            "MCA_WARNING_PLATFORM_SPECIFIC" => MCA::MCA_WARNING_PLATFORM_SPECIFIC,
            "MCA_WARNING_UNKNOWN" => MCA::MCA_WARNING_UNKNOWN,
            "MCA_WARNING_UNKNOWN_NO_CPU" => MCA::MCA_WARNING_UNKNOWN_NO_CPU,
            "MCA_WARNING_CMC_THRESHOLD_EXCEEDED" => MCA::MCA_WARNING_CMC_THRESHOLD_EXCEEDED,
            "MCA_WARNING_CPE_THRESHOLD_EXCEEDED" => MCA::MCA_WARNING_CPE_THRESHOLD_EXCEEDED,
            "MCA_WARNING_CPU_THERMAL_THROTTLED" => MCA::MCA_WARNING_CPU_THERMAL_THROTTLED,
            "MCA_WARNING_CPU" => MCA::MCA_WARNING_CPU,
            "MCA_ERROR_CACHE" => MCA::MCA_ERROR_CACHE,
            "MCA_ERROR_TLB" => MCA::MCA_ERROR_TLB,
            "MCA_ERROR_CPU_BUS" => MCA::MCA_ERROR_CPU_BUS,
            "MCA_ERROR_REGISTER_FILE" => MCA::MCA_ERROR_REGISTER_FILE,
            "MCA_ERROR_MAS" => MCA::MCA_ERROR_MAS,
            "MCA_ERROR_MEM_UNKNOWN" => MCA::MCA_ERROR_MEM_UNKNOWN,
            "MCA_ERROR_MEM_1_2" => MCA::MCA_ERROR_MEM_1_2,
            "MCA_ERROR_MEM_1_2_5" => MCA::MCA_ERROR_MEM_1_2_5,
            "MCA_ERROR_MEM_1_2_5_4" => MCA::MCA_ERROR_MEM_1_2_5_4,
            "MCA_ERROR_SYSTEM_EVENT" => MCA::MCA_ERROR_SYSTEM_EVENT,
            "MCA_ERROR_PCI_BUS_PARITY" => MCA::MCA_ERROR_PCI_BUS_PARITY,
            "MCA_ERROR_PCI_BUS_PARITY_NO_INFO" => MCA::MCA_ERROR_PCI_BUS_PARITY_NO_INFO,
            "MCA_ERROR_PCI_BUS_SERR" => MCA::MCA_ERROR_PCI_BUS_SERR,
            "MCA_ERROR_PCI_BUS_SERR_NO_INFO" => MCA::MCA_ERROR_PCI_BUS_SERR_NO_INFO,
            "MCA_ERROR_PCI_BUS_MASTER_ABORT" => MCA::MCA_ERROR_PCI_BUS_MASTER_ABORT,
            "MCA_ERROR_PCI_BUS_MASTER_ABORT_NO_INFO" => MCA::MCA_ERROR_PCI_BUS_MASTER_ABORT_NO_INFO,
            "MCA_ERROR_PCI_BUS_TIMEOUT" => MCA::MCA_ERROR_PCI_BUS_TIMEOUT,
            "MCA_ERROR_PCI_BUS_TIMEOUT_NO_INFO" => MCA::MCA_ERROR_PCI_BUS_TIMEOUT_NO_INFO,
            "MCA_ERROR_PCI_BUS_UNKNOWN" => MCA::MCA_ERROR_PCI_BUS_UNKNOWN,
            "MCA_ERROR_PCI_DEVICE" => MCA::MCA_ERROR_PCI_DEVICE,
            "MCA_ERROR_SMBIOS" => MCA::MCA_ERROR_SMBIOS,
            "MCA_ERROR_PLATFORM_SPECIFIC" => MCA::MCA_ERROR_PLATFORM_SPECIFIC,
            "MCA_ERROR_UNKNOWN" => MCA::MCA_ERROR_UNKNOWN,
            "MCA_ERROR_UNKNOWN_NO_CPU" => MCA::MCA_ERROR_UNKNOWN_NO_CPU,
            "MCA_ERROR_CPU" => MCA::MCA_ERROR_CPU,
            "MCA_MEMORYHIERARCHY_ERROR" => MCA::MCA_MEMORYHIERARCHY_ERROR,
            "MCA_TLB_ERROR" => MCA::MCA_TLB_ERROR,
            "MCA_BUS_ERROR" => MCA::MCA_BUS_ERROR,
            "MCA_BUS_TIMEOUT_ERROR" => MCA::MCA_BUS_TIMEOUT_ERROR,
            "MCA_INTERNALTIMER_ERROR" => MCA::MCA_INTERNALTIMER_ERROR,
            "MCA_MICROCODE_ROM_PARITY_ERROR" => MCA::MCA_MICROCODE_ROM_PARITY_ERROR,
            "MCA_EXTERNAL_ERROR" => MCA::MCA_EXTERNAL_ERROR,
            "MCA_FRC_ERROR" => MCA::MCA_FRC_ERROR,
            "E_ICARD_COMMUNICATION" => MCA::E_ICARD_COMMUNICATION,
            "E_ICARD_DATA_ACCESS" => MCA::E_ICARD_DATA_ACCESS,
            "E_ICARD_EXPORT" => MCA::E_ICARD_EXPORT,
            "E_ICARD_IDENTITY" => MCA::E_ICARD_IDENTITY,
            "E_ICARD_IMPORT" => MCA::E_ICARD_IMPORT,
            "E_ICARD_ARGUMENT" => MCA::E_ICARD_ARGUMENT,
            "E_ICARD_REQUEST" => MCA::E_ICARD_REQUEST,
            "E_ICARD_INFORMATIONCARD" => MCA::E_ICARD_INFORMATIONCARD,
            "E_ICARD_STOREKEY" => MCA::E_ICARD_STOREKEY,
            "E_ICARD_LOGOVALIDATION" => MCA::E_ICARD_LOGOVALIDATION,
            "E_ICARD_PASSWORDVALIDATION" => MCA::E_ICARD_PASSWORDVALIDATION,
            "E_ICARD_POLICY" => MCA::E_ICARD_POLICY,
            "E_ICARD_PROCESSDIED" => MCA::E_ICARD_PROCESSDIED,
            "E_ICARD_SERVICEBUSY" => MCA::E_ICARD_SERVICEBUSY,
            "E_ICARD_SERVICE" => MCA::E_ICARD_SERVICE,
            "E_ICARD_SHUTTINGDOWN" => MCA::E_ICARD_SHUTTINGDOWN,
            "E_ICARD_TOKENCREATION" => MCA::E_ICARD_TOKENCREATION,
            "E_ICARD_TRUSTEXCHANGE" => MCA::E_ICARD_TRUSTEXCHANGE,
            "E_ICARD_UNTRUSTED" => MCA::E_ICARD_UNTRUSTED,
            "E_ICARD_USERCANCELLED" => MCA::E_ICARD_USERCANCELLED,
            "E_ICARD_STORE_IMPORT" => MCA::E_ICARD_STORE_IMPORT,
            "E_ICARD_FAIL" => MCA::E_ICARD_FAIL,
            "E_ICARD_UI_INITIALIZATION" => MCA::E_ICARD_UI_INITIALIZATION,
            "E_ICARD_REFRESH_REQUIRED" => MCA::E_ICARD_REFRESH_REQUIRED,
            "E_ICARD_MISSING_APPLIESTO" => MCA::E_ICARD_MISSING_APPLIESTO,
            "E_ICARD_INVALID_PROOF_KEY" => MCA::E_ICARD_INVALID_PROOF_KEY,
            "E_ICARD_UNKNOWN_REFERENCE" => MCA::E_ICARD_UNKNOWN_REFERENCE,
            "E_ICARD_FAILED_REQUIRED_CLAIMS" => MCA::E_ICARD_FAILED_REQUIRED_CLAIMS,
        }
    }
    pub fn from_code(code: u32) -> MCA {
        return match code {
            0x40050070 => MCA::MCA_INFO_CPU_THERMAL_THROTTLING_REMOVED,
            0x40050073 => MCA::MCA_INFO_NO_MORE_CORRECTED_ERROR_LOGS,
            0x40050074 => MCA::MCA_INFO_MEMORY_PAGE_MARKED_BAD,
            0x40050200 => MCA::AUDIT_CARD_WRITTEN,
            0x40050201 => MCA::AUDIT_CARD_DELETE,
            0x40050202 => MCA::AUDIT_CARD_IMPORT,
            0x40050203 => MCA::AUDIT_STORE_IMPORT,
            0x40050204 => MCA::AUDIT_STORE_EXPORT,
            0x40050205 => MCA::AUDIT_STORE_DELETE,
            0x40050206 => MCA::AUDIT_SERVICE_IDLE_STOP,
            0x80050010 => MCA::E_EVAL_EXPIRED,
            0x80050013 => MCA::LTMM_E_INVALID_DATA,
            0x80050014 => MCA::LTMM_E_INVALID_HEADER,
            0x80050015 => MCA::LTMM_E_DISABLED,
            0x80050016 => MCA::LTMM_E_INVALID_VERSION,
            0x80050017 => MCA::LTMM_E_STILLIMAGE_LOCKED,
            0x8005003C => MCA::MCA_WARNING_CACHE,
            0x8005003E => MCA::MCA_WARNING_TLB,
            0x80050040 => MCA::MCA_WARNING_CPU_BUS,
            0x80050042 => MCA::MCA_WARNING_REGISTER_FILE,
            0x80050044 => MCA::MCA_WARNING_MAS,
            0x80050046 => MCA::MCA_WARNING_MEM_UNKNOWN,
            0x80050048 => MCA::MCA_WARNING_MEM_1_2,
            0x8005004A => MCA::MCA_WARNING_MEM_1_2_5,
            0x8005004C => MCA::MCA_WARNING_MEM_1_2_5_4,
            0x8005004E => MCA::MCA_WARNING_SYSTEM_EVENT,
            0x80050050 => MCA::MCA_WARNING_PCI_BUS_PARITY,
            0x80050052 => MCA::MCA_WARNING_PCI_BUS_PARITY_NO_INFO,
            0x80050054 => MCA::MCA_WARNING_PCI_BUS_SERR,
            0x80050056 => MCA::MCA_WARNING_PCI_BUS_SERR_NO_INFO,
            0x80050058 => MCA::MCA_WARNING_PCI_BUS_MASTER_ABORT,
            0x8005005A => MCA::MCA_WARNING_PCI_BUS_MASTER_ABORT_NO_INFO,
            0x8005005C => MCA::MCA_WARNING_PCI_BUS_TIMEOUT,
            0x8005005E => MCA::MCA_WARNING_PCI_BUS_TIMEOUT_NO_INFO,
            0x80050060 => MCA::MCA_WARNING_PCI_BUS_UNKNOWN,
            0x80050062 => MCA::MCA_WARNING_PCI_DEVICE,
            0x80050064 => MCA::MCA_WARNING_SMBIOS,
            0x80050066 => MCA::MCA_WARNING_PLATFORM_SPECIFIC,
            0x80050068 => MCA::MCA_WARNING_UNKNOWN,
            0x8005006A => MCA::MCA_WARNING_UNKNOWN_NO_CPU,
            0x8005006D => MCA::MCA_WARNING_CMC_THRESHOLD_EXCEEDED,
            0x8005006E => MCA::MCA_WARNING_CPE_THRESHOLD_EXCEEDED,
            0x8005006F => MCA::MCA_WARNING_CPU_THERMAL_THROTTLED,
            0x80050071 => MCA::MCA_WARNING_CPU,
            0xC005003D => MCA::MCA_ERROR_CACHE,
            0xC005003F => MCA::MCA_ERROR_TLB,
            0xC0050041 => MCA::MCA_ERROR_CPU_BUS,
            0xC0050043 => MCA::MCA_ERROR_REGISTER_FILE,
            0xC0050045 => MCA::MCA_ERROR_MAS,
            0xC0050047 => MCA::MCA_ERROR_MEM_UNKNOWN,
            0xC0050049 => MCA::MCA_ERROR_MEM_1_2,
            0xC005004B => MCA::MCA_ERROR_MEM_1_2_5,
            0xC005004D => MCA::MCA_ERROR_MEM_1_2_5_4,
            0xC005004F => MCA::MCA_ERROR_SYSTEM_EVENT,
            0xC0050051 => MCA::MCA_ERROR_PCI_BUS_PARITY,
            0xC0050053 => MCA::MCA_ERROR_PCI_BUS_PARITY_NO_INFO,
            0xC0050055 => MCA::MCA_ERROR_PCI_BUS_SERR,
            0xC0050057 => MCA::MCA_ERROR_PCI_BUS_SERR_NO_INFO,
            0xC0050059 => MCA::MCA_ERROR_PCI_BUS_MASTER_ABORT,
            0xC005005B => MCA::MCA_ERROR_PCI_BUS_MASTER_ABORT_NO_INFO,
            0xC005005D => MCA::MCA_ERROR_PCI_BUS_TIMEOUT,
            0xC005005F => MCA::MCA_ERROR_PCI_BUS_TIMEOUT_NO_INFO,
            0xC0050061 => MCA::MCA_ERROR_PCI_BUS_UNKNOWN,
            0xC0050063 => MCA::MCA_ERROR_PCI_DEVICE,
            0xC0050065 => MCA::MCA_ERROR_SMBIOS,
            0xC0050067 => MCA::MCA_ERROR_PLATFORM_SPECIFIC,
            0xC0050069 => MCA::MCA_ERROR_UNKNOWN,
            0xC005006B => MCA::MCA_ERROR_UNKNOWN_NO_CPU,
            0xC0050072 => MCA::MCA_ERROR_CPU,
            0xC0050078 => MCA::MCA_MEMORYHIERARCHY_ERROR,
            0xC0050079 => MCA::MCA_TLB_ERROR,
            0xC005007A => MCA::MCA_BUS_ERROR,
            0xC005007B => MCA::MCA_BUS_TIMEOUT_ERROR,
            0xC005007C => MCA::MCA_INTERNALTIMER_ERROR,
            0xC005007E => MCA::MCA_MICROCODE_ROM_PARITY_ERROR,
            0xC005007F => MCA::MCA_EXTERNAL_ERROR,
            0xC0050080 => MCA::MCA_FRC_ERROR,
            0xC0050100 => MCA::E_ICARD_COMMUNICATION,
            0xC0050101 => MCA::E_ICARD_DATA_ACCESS,
            0xC0050102 => MCA::E_ICARD_EXPORT,
            0xC0050103 => MCA::E_ICARD_IDENTITY,
            0xC0050104 => MCA::E_ICARD_IMPORT,
            0xC0050105 => MCA::E_ICARD_ARGUMENT,
            0xC0050106 => MCA::E_ICARD_REQUEST,
            0xC0050107 => MCA::E_ICARD_INFORMATIONCARD,
            0xC0050108 => MCA::E_ICARD_STOREKEY,
            0xC0050109 => MCA::E_ICARD_LOGOVALIDATION,
            0xC005010A => MCA::E_ICARD_PASSWORDVALIDATION,
            0xC005010B => MCA::E_ICARD_POLICY,
            0xC005010C => MCA::E_ICARD_PROCESSDIED,
            0xC005010D => MCA::E_ICARD_SERVICEBUSY,
            0xC005010E => MCA::E_ICARD_SERVICE,
            0xC005010F => MCA::E_ICARD_SHUTTINGDOWN,
            0xC0050110 => MCA::E_ICARD_TOKENCREATION,
            0xC0050111 => MCA::E_ICARD_TRUSTEXCHANGE,
            0xC0050112 => MCA::E_ICARD_UNTRUSTED,
            0xC0050113 => MCA::E_ICARD_USERCANCELLED,
            0xC0050114 => MCA::E_ICARD_STORE_IMPORT,
            0xC0050115 => MCA::E_ICARD_FAIL,
            0xC005011A => MCA::E_ICARD_UI_INITIALIZATION,
            0xC0050180 => MCA::E_ICARD_REFRESH_REQUIRED,
            0xC0050181 => MCA::E_ICARD_MISSING_APPLIESTO,
            0xC0050182 => MCA::E_ICARD_INVALID_PROOF_KEY,
            0xC0050183 => MCA::E_ICARD_UNKNOWN_REFERENCE,
            0xC0050184 => MCA::E_ICARD_FAILED_REQUIRED_CLAIMS,
        }
    }
}
