/*
 * Copyright 2023 Stanislav Mikhailov (xavetar)
 *
 * Licensed under the Creative Commons Zero v1.0 Universal (CC0) License.
 * You may obtain a copy of the License at
 *
 *     http://creativecommons.org/publicdomain/zero/1.0/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the CC0 license is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// It is strictly forbidden to use the from_code method, respected Microsoft decided to define
// duplicate codes, but by defining different constant names, therefore, when using these codes,
// it is mandatory to use through from_name, and not from_code. Otherwise, it may cause undefined
// behavior or an unknown exception. Because one code corresponds to several constants.

use super::{RawError};

#[derive(Clone, Debug, Copy, Eq, Hash, Ord, PartialEq, PartialOrd)]
pub enum NDIS {
    ERROR_NDIS_INDICATION_REQUIRED,
    ERROR_NDIS_INTERFACE_CLOSING,
    ERROR_NDIS_BAD_VERSION,
    ERROR_NDIS_BAD_CHARACTERISTICS,
    ERROR_NDIS_ADAPTER_NOT_FOUND,
    ERROR_NDIS_OPEN_FAILED,
    ERROR_NDIS_DEVICE_FAILED,
    ERROR_NDIS_MULTICAST_FULL,
    ERROR_NDIS_MULTICAST_EXISTS,
    ERROR_NDIS_MULTICAST_NOT_FOUND,
    ERROR_NDIS_REQUEST_ABORTED,
    ERROR_NDIS_RESET_IN_PROGRESS,
    ERROR_NDIS_INVALID_PACKET,
    ERROR_NDIS_INVALID_DEVICE_REQUEST,
    ERROR_NDIS_ADAPTER_NOT_READY,
    ERROR_NDIS_INVALID_LENGTH,
    ERROR_NDIS_INVALID_DATA,
    ERROR_NDIS_BUFFER_TOO_SHORT,
    ERROR_NDIS_INVALID_OID,
    ERROR_NDIS_ADAPTER_REMOVED,
    ERROR_NDIS_UNSUPPORTED_MEDIA,
    ERROR_NDIS_GROUP_ADDRESS_IN_USE,
    ERROR_NDIS_FILE_NOT_FOUND,
    ERROR_NDIS_ERROR_READING_FILE,
    ERROR_NDIS_ALREADY_MAPPED,
    ERROR_NDIS_RESOURCE_CONFLICT,
    ERROR_NDIS_MEDIA_DISCONNECTED,
    ERROR_NDIS_INVALID_ADDRESS,
    ERROR_NDIS_PAUSED,
    ERROR_NDIS_INTERFACE_NOT_FOUND,
    ERROR_NDIS_UNSUPPORTED_REVISION,
    ERROR_NDIS_INVALID_PORT,
    ERROR_NDIS_INVALID_PORT_STATE,
    ERROR_NDIS_LOW_POWER_STATE,
    ERROR_NDIS_REINIT_REQUIRED,
    ERROR_NDIS_NO_QUEUES,
    ERROR_NDIS_NOT_SUPPORTED,
    ERROR_NDIS_DOT11_AUTO_CONFIG_ENABLED,
    ERROR_NDIS_DOT11_MEDIA_IN_USE,
    ERROR_NDIS_DOT11_POWER_STATE_INVALID,
    ERROR_NDIS_PM_WOL_PATTERN_LIST_FULL,
    ERROR_NDIS_PM_PROTOCOL_OFFLOAD_LIST_FULL,
    ERROR_NDIS_DOT11_AP_CHANNEL_CURRENTLY_NOT_AVAILABLE,
    ERROR_NDIS_DOT11_AP_BAND_CURRENTLY_NOT_AVAILABLE,
    ERROR_NDIS_DOT11_AP_CHANNEL_NOT_ALLOWED,
    ERROR_NDIS_DOT11_AP_BAND_NOT_ALLOWED,
    ERROR_NDIS_OFFLOAD_POLICY,
    ERROR_NDIS_OFFLOAD_CONNECTION_REJECTED,
    ERROR_NDIS_OFFLOAD_PATH_REJECTED,
}

impl NDIS {
    pub fn code(&self) -> u32 {
        return match self {
            NDIS::ERROR_NDIS_INDICATION_REQUIRED => 0x00340001 as u32,
            NDIS::ERROR_NDIS_INTERFACE_CLOSING => 0x80340002 as u32,
            NDIS::ERROR_NDIS_BAD_VERSION => 0x80340004 as u32,
            NDIS::ERROR_NDIS_BAD_CHARACTERISTICS => 0x80340005 as u32,
            NDIS::ERROR_NDIS_ADAPTER_NOT_FOUND => 0x80340006 as u32,
            NDIS::ERROR_NDIS_OPEN_FAILED => 0x80340007 as u32,
            NDIS::ERROR_NDIS_DEVICE_FAILED => 0x80340008 as u32,
            NDIS::ERROR_NDIS_MULTICAST_FULL => 0x80340009 as u32,
            NDIS::ERROR_NDIS_MULTICAST_EXISTS => 0x8034000A as u32,
            NDIS::ERROR_NDIS_MULTICAST_NOT_FOUND => 0x8034000B as u32,
            NDIS::ERROR_NDIS_REQUEST_ABORTED => 0x8034000C as u32,
            NDIS::ERROR_NDIS_RESET_IN_PROGRESS => 0x8034000D as u32,
            NDIS::ERROR_NDIS_INVALID_PACKET => 0x8034000F as u32,
            NDIS::ERROR_NDIS_INVALID_DEVICE_REQUEST => 0x80340010 as u32,
            NDIS::ERROR_NDIS_ADAPTER_NOT_READY => 0x80340011 as u32,
            NDIS::ERROR_NDIS_INVALID_LENGTH => 0x80340014 as u32,
            NDIS::ERROR_NDIS_INVALID_DATA => 0x80340015 as u32,
            NDIS::ERROR_NDIS_BUFFER_TOO_SHORT => 0x80340016 as u32,
            NDIS::ERROR_NDIS_INVALID_OID => 0x80340017 as u32,
            NDIS::ERROR_NDIS_ADAPTER_REMOVED => 0x80340018 as u32,
            NDIS::ERROR_NDIS_UNSUPPORTED_MEDIA => 0x80340019 as u32,
            NDIS::ERROR_NDIS_GROUP_ADDRESS_IN_USE => 0x8034001A as u32,
            NDIS::ERROR_NDIS_FILE_NOT_FOUND => 0x8034001B as u32,
            NDIS::ERROR_NDIS_ERROR_READING_FILE => 0x8034001C as u32,
            NDIS::ERROR_NDIS_ALREADY_MAPPED => 0x8034001D as u32,
            NDIS::ERROR_NDIS_RESOURCE_CONFLICT => 0x8034001E as u32,
            NDIS::ERROR_NDIS_MEDIA_DISCONNECTED => 0x8034001F as u32,
            NDIS::ERROR_NDIS_INVALID_ADDRESS => 0x80340022 as u32,
            NDIS::ERROR_NDIS_PAUSED => 0x8034002A as u32,
            NDIS::ERROR_NDIS_INTERFACE_NOT_FOUND => 0x8034002B as u32,
            NDIS::ERROR_NDIS_UNSUPPORTED_REVISION => 0x8034002C as u32,
            NDIS::ERROR_NDIS_INVALID_PORT => 0x8034002D as u32,
            NDIS::ERROR_NDIS_INVALID_PORT_STATE => 0x8034002E as u32,
            NDIS::ERROR_NDIS_LOW_POWER_STATE => 0x8034002F as u32,
            NDIS::ERROR_NDIS_REINIT_REQUIRED => 0x80340030 as u32,
            NDIS::ERROR_NDIS_NO_QUEUES => 0x80340031 as u32,
            NDIS::ERROR_NDIS_NOT_SUPPORTED => 0x803400BB as u32,
            NDIS::ERROR_NDIS_DOT11_AUTO_CONFIG_ENABLED => 0x80342000 as u32,
            NDIS::ERROR_NDIS_DOT11_MEDIA_IN_USE => 0x80342001 as u32,
            NDIS::ERROR_NDIS_DOT11_POWER_STATE_INVALID => 0x80342002 as u32,
            NDIS::ERROR_NDIS_PM_WOL_PATTERN_LIST_FULL => 0x80342003 as u32,
            NDIS::ERROR_NDIS_PM_PROTOCOL_OFFLOAD_LIST_FULL => 0x80342004 as u32,
            NDIS::ERROR_NDIS_DOT11_AP_CHANNEL_CURRENTLY_NOT_AVAILABLE => 0x80342005 as u32,
            NDIS::ERROR_NDIS_DOT11_AP_BAND_CURRENTLY_NOT_AVAILABLE => 0x80342006 as u32,
            NDIS::ERROR_NDIS_DOT11_AP_CHANNEL_NOT_ALLOWED => 0x80342007 as u32,
            NDIS::ERROR_NDIS_DOT11_AP_BAND_NOT_ALLOWED => 0x80342008 as u32,
            NDIS::ERROR_NDIS_OFFLOAD_POLICY => 0xC034100F as u32,
            NDIS::ERROR_NDIS_OFFLOAD_CONNECTION_REJECTED => 0xC0341012 as u32,
            NDIS::ERROR_NDIS_OFFLOAD_PATH_REJECTED => 0xC0341013 as u32,
        }
    }

    pub fn error(&self) -> RawError {
        return match self {
            NDIS::ERROR_NDIS_INDICATION_REQUIRED => RawError::Kind(NDIS::ERROR_NDIS_INDICATION_REQUIRED),
            NDIS::ERROR_NDIS_INTERFACE_CLOSING => RawError::Kind(NDIS::ERROR_NDIS_INTERFACE_CLOSING),
            NDIS::ERROR_NDIS_BAD_VERSION => RawError::Kind(NDIS::ERROR_NDIS_BAD_VERSION),
            NDIS::ERROR_NDIS_BAD_CHARACTERISTICS => RawError::Kind(NDIS::ERROR_NDIS_BAD_CHARACTERISTICS),
            NDIS::ERROR_NDIS_ADAPTER_NOT_FOUND => RawError::Kind(NDIS::ERROR_NDIS_ADAPTER_NOT_FOUND),
            NDIS::ERROR_NDIS_OPEN_FAILED => RawError::Kind(NDIS::ERROR_NDIS_OPEN_FAILED),
            NDIS::ERROR_NDIS_DEVICE_FAILED => RawError::Kind(NDIS::ERROR_NDIS_DEVICE_FAILED),
            NDIS::ERROR_NDIS_MULTICAST_FULL => RawError::Kind(NDIS::ERROR_NDIS_MULTICAST_FULL),
            NDIS::ERROR_NDIS_MULTICAST_EXISTS => RawError::Kind(NDIS::ERROR_NDIS_MULTICAST_EXISTS),
            NDIS::ERROR_NDIS_MULTICAST_NOT_FOUND => RawError::Kind(NDIS::ERROR_NDIS_MULTICAST_NOT_FOUND),
            NDIS::ERROR_NDIS_REQUEST_ABORTED => RawError::Kind(NDIS::ERROR_NDIS_REQUEST_ABORTED),
            NDIS::ERROR_NDIS_RESET_IN_PROGRESS => RawError::Kind(NDIS::ERROR_NDIS_RESET_IN_PROGRESS),
            NDIS::ERROR_NDIS_INVALID_PACKET => RawError::Kind(NDIS::ERROR_NDIS_INVALID_PACKET),
            NDIS::ERROR_NDIS_INVALID_DEVICE_REQUEST => RawError::Kind(NDIS::ERROR_NDIS_INVALID_DEVICE_REQUEST),
            NDIS::ERROR_NDIS_ADAPTER_NOT_READY => RawError::Kind(NDIS::ERROR_NDIS_ADAPTER_NOT_READY),
            NDIS::ERROR_NDIS_INVALID_LENGTH => RawError::Kind(NDIS::ERROR_NDIS_INVALID_LENGTH),
            NDIS::ERROR_NDIS_INVALID_DATA => RawError::Kind(NDIS::ERROR_NDIS_INVALID_DATA),
            NDIS::ERROR_NDIS_BUFFER_TOO_SHORT => RawError::Kind(NDIS::ERROR_NDIS_BUFFER_TOO_SHORT),
            NDIS::ERROR_NDIS_INVALID_OID => RawError::Kind(NDIS::ERROR_NDIS_INVALID_OID),
            NDIS::ERROR_NDIS_ADAPTER_REMOVED => RawError::Kind(NDIS::ERROR_NDIS_ADAPTER_REMOVED),
            NDIS::ERROR_NDIS_UNSUPPORTED_MEDIA => RawError::Kind(NDIS::ERROR_NDIS_UNSUPPORTED_MEDIA),
            NDIS::ERROR_NDIS_GROUP_ADDRESS_IN_USE => RawError::Kind(NDIS::ERROR_NDIS_GROUP_ADDRESS_IN_USE),
            NDIS::ERROR_NDIS_FILE_NOT_FOUND => RawError::Kind(NDIS::ERROR_NDIS_FILE_NOT_FOUND),
            NDIS::ERROR_NDIS_ERROR_READING_FILE => RawError::Kind(NDIS::ERROR_NDIS_ERROR_READING_FILE),
            NDIS::ERROR_NDIS_ALREADY_MAPPED => RawError::Kind(NDIS::ERROR_NDIS_ALREADY_MAPPED),
            NDIS::ERROR_NDIS_RESOURCE_CONFLICT => RawError::Kind(NDIS::ERROR_NDIS_RESOURCE_CONFLICT),
            NDIS::ERROR_NDIS_MEDIA_DISCONNECTED => RawError::Kind(NDIS::ERROR_NDIS_MEDIA_DISCONNECTED),
            NDIS::ERROR_NDIS_INVALID_ADDRESS => RawError::Kind(NDIS::ERROR_NDIS_INVALID_ADDRESS),
            NDIS::ERROR_NDIS_PAUSED => RawError::Kind(NDIS::ERROR_NDIS_PAUSED),
            NDIS::ERROR_NDIS_INTERFACE_NOT_FOUND => RawError::Kind(NDIS::ERROR_NDIS_INTERFACE_NOT_FOUND),
            NDIS::ERROR_NDIS_UNSUPPORTED_REVISION => RawError::Kind(NDIS::ERROR_NDIS_UNSUPPORTED_REVISION),
            NDIS::ERROR_NDIS_INVALID_PORT => RawError::Kind(NDIS::ERROR_NDIS_INVALID_PORT),
            NDIS::ERROR_NDIS_INVALID_PORT_STATE => RawError::Kind(NDIS::ERROR_NDIS_INVALID_PORT_STATE),
            NDIS::ERROR_NDIS_LOW_POWER_STATE => RawError::Kind(NDIS::ERROR_NDIS_LOW_POWER_STATE),
            NDIS::ERROR_NDIS_REINIT_REQUIRED => RawError::Kind(NDIS::ERROR_NDIS_REINIT_REQUIRED),
            NDIS::ERROR_NDIS_NO_QUEUES => RawError::Kind(NDIS::ERROR_NDIS_NO_QUEUES),
            NDIS::ERROR_NDIS_NOT_SUPPORTED => RawError::Kind(NDIS::ERROR_NDIS_NOT_SUPPORTED),
            NDIS::ERROR_NDIS_DOT11_AUTO_CONFIG_ENABLED => RawError::Kind(NDIS::ERROR_NDIS_DOT11_AUTO_CONFIG_ENABLED),
            NDIS::ERROR_NDIS_DOT11_MEDIA_IN_USE => RawError::Kind(NDIS::ERROR_NDIS_DOT11_MEDIA_IN_USE),
            NDIS::ERROR_NDIS_DOT11_POWER_STATE_INVALID => RawError::Kind(NDIS::ERROR_NDIS_DOT11_POWER_STATE_INVALID),
            NDIS::ERROR_NDIS_PM_WOL_PATTERN_LIST_FULL => RawError::Kind(NDIS::ERROR_NDIS_PM_WOL_PATTERN_LIST_FULL),
            NDIS::ERROR_NDIS_PM_PROTOCOL_OFFLOAD_LIST_FULL => RawError::Kind(NDIS::ERROR_NDIS_PM_PROTOCOL_OFFLOAD_LIST_FULL),
            NDIS::ERROR_NDIS_DOT11_AP_CHANNEL_CURRENTLY_NOT_AVAILABLE => RawError::Kind(NDIS::ERROR_NDIS_DOT11_AP_CHANNEL_CURRENTLY_NOT_AVAILABLE),
            NDIS::ERROR_NDIS_DOT11_AP_BAND_CURRENTLY_NOT_AVAILABLE => RawError::Kind(NDIS::ERROR_NDIS_DOT11_AP_BAND_CURRENTLY_NOT_AVAILABLE),
            NDIS::ERROR_NDIS_DOT11_AP_CHANNEL_NOT_ALLOWED => RawError::Kind(NDIS::ERROR_NDIS_DOT11_AP_CHANNEL_NOT_ALLOWED),
            NDIS::ERROR_NDIS_DOT11_AP_BAND_NOT_ALLOWED => RawError::Kind(NDIS::ERROR_NDIS_DOT11_AP_BAND_NOT_ALLOWED),
            NDIS::ERROR_NDIS_OFFLOAD_POLICY => RawError::Kind(NDIS::ERROR_NDIS_OFFLOAD_POLICY),
            NDIS::ERROR_NDIS_OFFLOAD_CONNECTION_REJECTED => RawError::Kind(NDIS::ERROR_NDIS_OFFLOAD_CONNECTION_REJECTED),
            NDIS::ERROR_NDIS_OFFLOAD_PATH_REJECTED => RawError::Kind(NDIS::ERROR_NDIS_OFFLOAD_PATH_REJECTED),
        }
    }

    pub fn description(&self) -> &'static str {
        return match self {
            NDIS::ERROR_NDIS_INDICATION_REQUIRED => "The request will be completed later by NDIS status indication.",
            NDIS::ERROR_NDIS_INTERFACE_CLOSING => "The binding to the network interface is being closed.",
            NDIS::ERROR_NDIS_BAD_VERSION => "An invalid version was specified.",
            NDIS::ERROR_NDIS_BAD_CHARACTERISTICS => "An invalid characteristics table was used.",
            NDIS::ERROR_NDIS_ADAPTER_NOT_FOUND => "Failed to find the network interface or network interface is not ready.",
            NDIS::ERROR_NDIS_OPEN_FAILED => "Failed to open the network interface.",
            NDIS::ERROR_NDIS_DEVICE_FAILED => "Network interface has encountered an internal unrecoverable failure.",
            NDIS::ERROR_NDIS_MULTICAST_FULL => "The multicast list on the network interface is full.",
            NDIS::ERROR_NDIS_MULTICAST_EXISTS => "An attempt was made to add a duplicate multicast address to the list.",
            NDIS::ERROR_NDIS_MULTICAST_NOT_FOUND => "At attempt was made to remove a multicast address that was never added.",
            NDIS::ERROR_NDIS_REQUEST_ABORTED => "Netowork interface aborted the request.",
            NDIS::ERROR_NDIS_RESET_IN_PROGRESS => "Network interface can not process the request because it is being reset.",
            NDIS::ERROR_NDIS_INVALID_PACKET => "An attempt was made to send an invalid packet on a network interface.",
            NDIS::ERROR_NDIS_INVALID_DEVICE_REQUEST => "The specified request is not a valid operation for the target device.",
            NDIS::ERROR_NDIS_ADAPTER_NOT_READY => "Network interface is not ready to complete this operation.",
            NDIS::ERROR_NDIS_INVALID_LENGTH => "The length of the buffer submitted for this operation is not valid.",
            NDIS::ERROR_NDIS_INVALID_DATA => "The data used for this operation is not valid.",
            NDIS::ERROR_NDIS_BUFFER_TOO_SHORT => "The length of buffer submitted for this operation is too small.",
            NDIS::ERROR_NDIS_INVALID_OID => "Network interface does not support this OID (Object Identifier)",
            NDIS::ERROR_NDIS_ADAPTER_REMOVED => "The network interface has been removed.",
            NDIS::ERROR_NDIS_UNSUPPORTED_MEDIA => "Network interface does not support this media type.",
            NDIS::ERROR_NDIS_GROUP_ADDRESS_IN_USE => "An attempt was made to remove a token ring group address that is in use by other components.",
            NDIS::ERROR_NDIS_FILE_NOT_FOUND => "An attempt was made to map a file that can not be found.",
            NDIS::ERROR_NDIS_ERROR_READING_FILE => "An error occurred while NDIS tried to map the file.",
            NDIS::ERROR_NDIS_ALREADY_MAPPED => "An attempt was made to map a file that is alreay mapped.",
            NDIS::ERROR_NDIS_RESOURCE_CONFLICT => "An attempt to allocate a hardware resource failed because the resource is used by another component.",
            NDIS::ERROR_NDIS_MEDIA_DISCONNECTED => "The I/O operation failed because network media is disconnected or wireless access point is out of range.",
            NDIS::ERROR_NDIS_INVALID_ADDRESS => "The network address used in the request is invalid.",
            NDIS::ERROR_NDIS_PAUSED => "The offload operation on the network interface has been paused.",
            NDIS::ERROR_NDIS_INTERFACE_NOT_FOUND => "Network interface was not found.",
            NDIS::ERROR_NDIS_UNSUPPORTED_REVISION => "The revision number specified in the structure is not supported.",
            NDIS::ERROR_NDIS_INVALID_PORT => "The specified port does not exist on this network interface.",
            NDIS::ERROR_NDIS_INVALID_PORT_STATE => "The current state of the specified port on this network interface does not support the requested operation.",
            NDIS::ERROR_NDIS_LOW_POWER_STATE => "The miniport adapter is in low power state.",
            NDIS::ERROR_NDIS_REINIT_REQUIRED => "This operation requires the miniport adapter to be reinitialized.",
            NDIS::ERROR_NDIS_NO_QUEUES => "There are not enough queues to complete the operation.",
            NDIS::ERROR_NDIS_NOT_SUPPORTED => "Netword interface does not support this request.",
            NDIS::ERROR_NDIS_DOT11_AUTO_CONFIG_ENABLED => "The wireless local area network interface is in auto configuration mode and doesn't support the requested parameter change operation.",
            NDIS::ERROR_NDIS_DOT11_MEDIA_IN_USE => "The wireless local area network interface is busy and can not perform the requested operation.",
            NDIS::ERROR_NDIS_DOT11_POWER_STATE_INVALID => "The wireless local area network interface is powered down and doesn't support the requested operation.",
            NDIS::ERROR_NDIS_PM_WOL_PATTERN_LIST_FULL => "The list of wake on LAN patterns is full.",
            NDIS::ERROR_NDIS_PM_PROTOCOL_OFFLOAD_LIST_FULL => "The list of low power protocol offloads is full.",
            NDIS::ERROR_NDIS_DOT11_AP_CHANNEL_CURRENTLY_NOT_AVAILABLE => "The wireless local area network interface cannot start an AP on the specified channel right now.",
            NDIS::ERROR_NDIS_DOT11_AP_BAND_CURRENTLY_NOT_AVAILABLE => "The wireless local area network interface cannot start an AP on the specified band right now.",
            NDIS::ERROR_NDIS_DOT11_AP_CHANNEL_NOT_ALLOWED => "The wireless local area network interface cannot start an AP on this channel due to regulatory reasons.",
            NDIS::ERROR_NDIS_DOT11_AP_BAND_NOT_ALLOWED => "The wireless local area network interface cannot start an AP on this band due to regulatory reasons.",
            NDIS::ERROR_NDIS_OFFLOAD_POLICY => "The TCP connection is not offloadable because of a local policy setting.",
            NDIS::ERROR_NDIS_OFFLOAD_CONNECTION_REJECTED => "The TCP connection is not offloadable by the Chimney Offload target.",
            NDIS::ERROR_NDIS_OFFLOAD_PATH_REJECTED => "The IP Path object is not in an offloadable state.",
        }
    }

    pub fn from_name(name: &str) -> NDIS {
        return match name {
            "ERROR_NDIS_INDICATION_REQUIRED" => NDIS::ERROR_NDIS_INDICATION_REQUIRED,
            "ERROR_NDIS_INTERFACE_CLOSING" => NDIS::ERROR_NDIS_INTERFACE_CLOSING,
            "ERROR_NDIS_BAD_VERSION" => NDIS::ERROR_NDIS_BAD_VERSION,
            "ERROR_NDIS_BAD_CHARACTERISTICS" => NDIS::ERROR_NDIS_BAD_CHARACTERISTICS,
            "ERROR_NDIS_ADAPTER_NOT_FOUND" => NDIS::ERROR_NDIS_ADAPTER_NOT_FOUND,
            "ERROR_NDIS_OPEN_FAILED" => NDIS::ERROR_NDIS_OPEN_FAILED,
            "ERROR_NDIS_DEVICE_FAILED" => NDIS::ERROR_NDIS_DEVICE_FAILED,
            "ERROR_NDIS_MULTICAST_FULL" => NDIS::ERROR_NDIS_MULTICAST_FULL,
            "ERROR_NDIS_MULTICAST_EXISTS" => NDIS::ERROR_NDIS_MULTICAST_EXISTS,
            "ERROR_NDIS_MULTICAST_NOT_FOUND" => NDIS::ERROR_NDIS_MULTICAST_NOT_FOUND,
            "ERROR_NDIS_REQUEST_ABORTED" => NDIS::ERROR_NDIS_REQUEST_ABORTED,
            "ERROR_NDIS_RESET_IN_PROGRESS" => NDIS::ERROR_NDIS_RESET_IN_PROGRESS,
            "ERROR_NDIS_INVALID_PACKET" => NDIS::ERROR_NDIS_INVALID_PACKET,
            "ERROR_NDIS_INVALID_DEVICE_REQUEST" => NDIS::ERROR_NDIS_INVALID_DEVICE_REQUEST,
            "ERROR_NDIS_ADAPTER_NOT_READY" => NDIS::ERROR_NDIS_ADAPTER_NOT_READY,
            "ERROR_NDIS_INVALID_LENGTH" => NDIS::ERROR_NDIS_INVALID_LENGTH,
            "ERROR_NDIS_INVALID_DATA" => NDIS::ERROR_NDIS_INVALID_DATA,
            "ERROR_NDIS_BUFFER_TOO_SHORT" => NDIS::ERROR_NDIS_BUFFER_TOO_SHORT,
            "ERROR_NDIS_INVALID_OID" => NDIS::ERROR_NDIS_INVALID_OID,
            "ERROR_NDIS_ADAPTER_REMOVED" => NDIS::ERROR_NDIS_ADAPTER_REMOVED,
            "ERROR_NDIS_UNSUPPORTED_MEDIA" => NDIS::ERROR_NDIS_UNSUPPORTED_MEDIA,
            "ERROR_NDIS_GROUP_ADDRESS_IN_USE" => NDIS::ERROR_NDIS_GROUP_ADDRESS_IN_USE,
            "ERROR_NDIS_FILE_NOT_FOUND" => NDIS::ERROR_NDIS_FILE_NOT_FOUND,
            "ERROR_NDIS_ERROR_READING_FILE" => NDIS::ERROR_NDIS_ERROR_READING_FILE,
            "ERROR_NDIS_ALREADY_MAPPED" => NDIS::ERROR_NDIS_ALREADY_MAPPED,
            "ERROR_NDIS_RESOURCE_CONFLICT" => NDIS::ERROR_NDIS_RESOURCE_CONFLICT,
            "ERROR_NDIS_MEDIA_DISCONNECTED" => NDIS::ERROR_NDIS_MEDIA_DISCONNECTED,
            "ERROR_NDIS_INVALID_ADDRESS" => NDIS::ERROR_NDIS_INVALID_ADDRESS,
            "ERROR_NDIS_PAUSED" => NDIS::ERROR_NDIS_PAUSED,
            "ERROR_NDIS_INTERFACE_NOT_FOUND" => NDIS::ERROR_NDIS_INTERFACE_NOT_FOUND,
            "ERROR_NDIS_UNSUPPORTED_REVISION" => NDIS::ERROR_NDIS_UNSUPPORTED_REVISION,
            "ERROR_NDIS_INVALID_PORT" => NDIS::ERROR_NDIS_INVALID_PORT,
            "ERROR_NDIS_INVALID_PORT_STATE" => NDIS::ERROR_NDIS_INVALID_PORT_STATE,
            "ERROR_NDIS_LOW_POWER_STATE" => NDIS::ERROR_NDIS_LOW_POWER_STATE,
            "ERROR_NDIS_REINIT_REQUIRED" => NDIS::ERROR_NDIS_REINIT_REQUIRED,
            "ERROR_NDIS_NO_QUEUES" => NDIS::ERROR_NDIS_NO_QUEUES,
            "ERROR_NDIS_NOT_SUPPORTED" => NDIS::ERROR_NDIS_NOT_SUPPORTED,
            "ERROR_NDIS_DOT11_AUTO_CONFIG_ENABLED" => NDIS::ERROR_NDIS_DOT11_AUTO_CONFIG_ENABLED,
            "ERROR_NDIS_DOT11_MEDIA_IN_USE" => NDIS::ERROR_NDIS_DOT11_MEDIA_IN_USE,
            "ERROR_NDIS_DOT11_POWER_STATE_INVALID" => NDIS::ERROR_NDIS_DOT11_POWER_STATE_INVALID,
            "ERROR_NDIS_PM_WOL_PATTERN_LIST_FULL" => NDIS::ERROR_NDIS_PM_WOL_PATTERN_LIST_FULL,
            "ERROR_NDIS_PM_PROTOCOL_OFFLOAD_LIST_FULL" => NDIS::ERROR_NDIS_PM_PROTOCOL_OFFLOAD_LIST_FULL,
            "ERROR_NDIS_DOT11_AP_CHANNEL_CURRENTLY_NOT_AVAILABLE" => NDIS::ERROR_NDIS_DOT11_AP_CHANNEL_CURRENTLY_NOT_AVAILABLE,
            "ERROR_NDIS_DOT11_AP_BAND_CURRENTLY_NOT_AVAILABLE" => NDIS::ERROR_NDIS_DOT11_AP_BAND_CURRENTLY_NOT_AVAILABLE,
            "ERROR_NDIS_DOT11_AP_CHANNEL_NOT_ALLOWED" => NDIS::ERROR_NDIS_DOT11_AP_CHANNEL_NOT_ALLOWED,
            "ERROR_NDIS_DOT11_AP_BAND_NOT_ALLOWED" => NDIS::ERROR_NDIS_DOT11_AP_BAND_NOT_ALLOWED,
            "ERROR_NDIS_OFFLOAD_POLICY" => NDIS::ERROR_NDIS_OFFLOAD_POLICY,
            "ERROR_NDIS_OFFLOAD_CONNECTION_REJECTED" => NDIS::ERROR_NDIS_OFFLOAD_CONNECTION_REJECTED,
            "ERROR_NDIS_OFFLOAD_PATH_REJECTED" => NDIS::ERROR_NDIS_OFFLOAD_PATH_REJECTED,
        }
    }
    pub fn from_code(code: u32) -> NDIS {
        return match code {
            0x00340001 => NDIS::ERROR_NDIS_INDICATION_REQUIRED,
            0x80340002 => NDIS::ERROR_NDIS_INTERFACE_CLOSING,
            0x80340004 => NDIS::ERROR_NDIS_BAD_VERSION,
            0x80340005 => NDIS::ERROR_NDIS_BAD_CHARACTERISTICS,
            0x80340006 => NDIS::ERROR_NDIS_ADAPTER_NOT_FOUND,
            0x80340007 => NDIS::ERROR_NDIS_OPEN_FAILED,
            0x80340008 => NDIS::ERROR_NDIS_DEVICE_FAILED,
            0x80340009 => NDIS::ERROR_NDIS_MULTICAST_FULL,
            0x8034000A => NDIS::ERROR_NDIS_MULTICAST_EXISTS,
            0x8034000B => NDIS::ERROR_NDIS_MULTICAST_NOT_FOUND,
            0x8034000C => NDIS::ERROR_NDIS_REQUEST_ABORTED,
            0x8034000D => NDIS::ERROR_NDIS_RESET_IN_PROGRESS,
            0x8034000F => NDIS::ERROR_NDIS_INVALID_PACKET,
            0x80340010 => NDIS::ERROR_NDIS_INVALID_DEVICE_REQUEST,
            0x80340011 => NDIS::ERROR_NDIS_ADAPTER_NOT_READY,
            0x80340014 => NDIS::ERROR_NDIS_INVALID_LENGTH,
            0x80340015 => NDIS::ERROR_NDIS_INVALID_DATA,
            0x80340016 => NDIS::ERROR_NDIS_BUFFER_TOO_SHORT,
            0x80340017 => NDIS::ERROR_NDIS_INVALID_OID,
            0x80340018 => NDIS::ERROR_NDIS_ADAPTER_REMOVED,
            0x80340019 => NDIS::ERROR_NDIS_UNSUPPORTED_MEDIA,
            0x8034001A => NDIS::ERROR_NDIS_GROUP_ADDRESS_IN_USE,
            0x8034001B => NDIS::ERROR_NDIS_FILE_NOT_FOUND,
            0x8034001C => NDIS::ERROR_NDIS_ERROR_READING_FILE,
            0x8034001D => NDIS::ERROR_NDIS_ALREADY_MAPPED,
            0x8034001E => NDIS::ERROR_NDIS_RESOURCE_CONFLICT,
            0x8034001F => NDIS::ERROR_NDIS_MEDIA_DISCONNECTED,
            0x80340022 => NDIS::ERROR_NDIS_INVALID_ADDRESS,
            0x8034002A => NDIS::ERROR_NDIS_PAUSED,
            0x8034002B => NDIS::ERROR_NDIS_INTERFACE_NOT_FOUND,
            0x8034002C => NDIS::ERROR_NDIS_UNSUPPORTED_REVISION,
            0x8034002D => NDIS::ERROR_NDIS_INVALID_PORT,
            0x8034002E => NDIS::ERROR_NDIS_INVALID_PORT_STATE,
            0x8034002F => NDIS::ERROR_NDIS_LOW_POWER_STATE,
            0x80340030 => NDIS::ERROR_NDIS_REINIT_REQUIRED,
            0x80340031 => NDIS::ERROR_NDIS_NO_QUEUES,
            0x803400BB => NDIS::ERROR_NDIS_NOT_SUPPORTED,
            0x80342000 => NDIS::ERROR_NDIS_DOT11_AUTO_CONFIG_ENABLED,
            0x80342001 => NDIS::ERROR_NDIS_DOT11_MEDIA_IN_USE,
            0x80342002 => NDIS::ERROR_NDIS_DOT11_POWER_STATE_INVALID,
            0x80342003 => NDIS::ERROR_NDIS_PM_WOL_PATTERN_LIST_FULL,
            0x80342004 => NDIS::ERROR_NDIS_PM_PROTOCOL_OFFLOAD_LIST_FULL,
            0x80342005 => NDIS::ERROR_NDIS_DOT11_AP_CHANNEL_CURRENTLY_NOT_AVAILABLE,
            0x80342006 => NDIS::ERROR_NDIS_DOT11_AP_BAND_CURRENTLY_NOT_AVAILABLE,
            0x80342007 => NDIS::ERROR_NDIS_DOT11_AP_CHANNEL_NOT_ALLOWED,
            0x80342008 => NDIS::ERROR_NDIS_DOT11_AP_BAND_NOT_ALLOWED,
            0xC034100F => NDIS::ERROR_NDIS_OFFLOAD_POLICY,
            0xC0341012 => NDIS::ERROR_NDIS_OFFLOAD_CONNECTION_REJECTED,
            0xC0341013 => NDIS::ERROR_NDIS_OFFLOAD_PATH_REJECTED,
        }
    }
}
