/*
 * Copyright 2023 Stanislav Mikhailov (xavetar)
 *
 * Licensed under the Creative Commons Zero v1.0 Universal (CC0) License.
 * You may obtain a copy of the License at
 *
 *     http://creativecommons.org/publicdomain/zero/1.0/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the CC0 license is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// It is strictly forbidden to use the from_code method, respected Microsoft decided to define
// duplicate codes, but by defining different constant names, therefore, when using these codes,
// it is mandatory to use through from_name, and not from_code. Otherwise, it may cause undefined
// behavior or an unknown exception. Because one code corresponds to several constants.

use super::{RawError};

#[derive(Clone, Debug, Copy, Eq, Hash, Ord, PartialEq, PartialOrd)]
pub enum NTDSB {
    FHCFG_E_INVALID_REHYDRATION_STATE,
    hrBFNotSynchronous,
    hrBFPageNotFound,
    hrRemainingVersions,
    hrFLDKeyTooBig,
    hrFLDNullKey,
    hrExistingLogFileHasBadSignature,
    hrExistingLogFileIsNotContiguous,
    hrColumnNull,
    hrBufferTruncated,
    hrDatabaseAttached,
    hrSeekNotEqual,
    hrNoIdleActivity,
    hrNoWriteLock,
    hrColumnSetNull,
    hrTableEmpty,
    hrCreateIndexFailed,
    hrColumnMaxTruncated,
    hrwrnDataHasChanged,
    hrKeyChanged,
    hrFileOpenReadOnly,
    hrFileClose,
    hrOutOfThreads,
    hrTooManyIO,
    hrBFInUse,
    hrPMRecDeleted,
    hrFLDTooManySegments,
    hrLogFileCorrupt,
    hrNoBackupDirectory,
    hrBackupDirectoryNotEmpty,
    hrBackupInProgress,
    hrMissingPreviousLogFile,
    hrLogWriteFail,
    hrBadLogVersion,
    hrInvalidLogSequence,
    hrLoggingDisabled,
    hrLogBufferTooSmall,
    hrLogSequenceEnd,
    hrNoBackup,
    hrInvalidBackupSequence,
    hrBackupNotAllowedYet,
    hrDeleteBackupFileFail,
    hrMakeBackupDirectoryFail,
    hrInvalidBackup,
    hrRecoveredWithErrors,
    hrMissingLogFile,
    hrLogDiskFull,
    hrBadLogSignature,
    hrBadDbSignature,
    hrBadCheckpointSignature,
    hrCheckpointCorrupt,
    hrDatabaseInconsistent,
    hrConsistentTimeMismatch,
    hrPatchFileMismatch,
    hrRestoreLogTooLow,
    hrRestoreLogTooHigh,
    hrGivenLogFileHasBadSignature,
    hrGivenLogFileIsNotContiguous,
    hrMissingRestoreLogFiles,
    hrMissingFullBackup,
    hrBadBackupDatabaseSize,
    hrTermInProgress,
    hrFeatureNotAvailable,
    hrInvalidName,
    hrInvalidParameter,
    hrInvalidDatabaseId,
    hrOutOfMemory,
    hrOutOfDatabaseSpace,
    hrOutOfCursors,
    hrOutOfBuffers,
    hrTooManyIndexes,
    hrTooManyKeys,
    hrRecordDeleted,
    hrReadVerifyFailure,
    hrOutOfFileHandles,
    hrDiskIO,
    hrInvalidPath,
    hrRecordTooBig,
    hrTooManyOpenDatabases,
    hrInvalidDatabase,
    hrNotInitialized,
    hrAlreadyInitialized,
    hrFileAccessDenied,
    hrBufferTooSmall,
    hrTooManyColumns,
    hrContainerNotEmpty,
    hrInvalidFilename,
    hrInvalidBookmark,
    hrColumnInUse,
    hrInvalidBufferSize,
    hrColumnNotUpdatable,
    hrIndexInUse,
    hrNullKeyDisallowed,
    hrNotInTransaction,
    hrTooManyActiveUsers,
    hrInvalidCountry,
    hrInvalidLanguageId,
    hrInvalidCodePage,
    hrVersionStoreOutOfMemory,
    hrCurrencyStackOutOfMemory,
    hrOutOfSessions,
    hrWriteConflict,
    hrTransTooDeep,
    hrInvalidSesid,
    hrSessionWriteConflict,
    hrInTransaction,
    hrDatabaseDuplicate,
    hrDatabaseInUse,
    hrDatabaseNotFound,
    hrDatabaseInvalidName,
    hrDatabaseInvalidPages,
    hrDatabaseCorrupted,
    hrDatabaseLocked,
    hrTableLocked,
    hrTableDuplicate,
    hrTableInUse,
    hrObjectNotFound,
    hrCannotRename,
    hrDensityInvalid,
    hrTableNotEmpty,
    hrInvalidTableId,
    hrTooManyOpenTables,
    hrIllegalOperation,
    hrObjectDuplicate,
    hrInvalidObject,
    hrIndexCantBuild,
    hrIndexHasPrimary,
    hrIndexDuplicate,
    hrIndexNotFound,
    hrIndexMustStay,
    hrIndexInvalidDef,
    hrIndexHasClustered,
    hrTooManyOpenIndexes,
    hrColumnLong,
    hrColumnDoesNotFit,
    hrNullInvalid,
    hrColumnIndexed,
    hrColumnTooBig,
    hrColumnNotFound,
    hrColumnDuplicate,
    hrColumn2ndSysMaint,
    hrInvalidColumnType,
    hrColumnCannotIndex,
    hrTaggedNotNULL,
    hrNoCurrentIndex,
    hrKeyIsMade,
    hrBadColumnId,
    hrBadItagSequence,
    hrCannotBeTagged,
    hrRecordNotFound,
    hrNoCurrentRecord,
    hrRecordClusteredChanged,
    hrKeyDuplicate,
    hrAlreadyPrepared,
    hrKeyNotMade,
    hrUpdateNotPrepared,
    hrerrDataHasChanged,
    hrTooManySorts,
    hrInvalidOnSort,
    hrTempFileOpenError,
    hrTooManyAttachedDatabases,
    hrDiskFull,
    hrPermissionDenied,
    hrFileNotFound,
    hrAfterInitialization,
    hrLogCorrupted,
    hrInvalidOperation,
    hrAccessDenied,
}

impl NTDSB {
    pub fn code(&self) -> u32 {
        return match self {
            NTDSB::FHCFG_E_INVALID_REHYDRATION_STATE => 0x0800430A as u32,
            NTDSB::hrBFNotSynchronous => 0x880000C8 as u32,
            NTDSB::hrBFPageNotFound => 0x880000C9 as u32,
            NTDSB::hrRemainingVersions => 0x88000141 as u32,
            NTDSB::hrFLDKeyTooBig => 0x88000190 as u32,
            NTDSB::hrFLDNullKey => 0x88000192 as u32,
            NTDSB::hrExistingLogFileHasBadSignature => 0x8800022E as u32,
            NTDSB::hrExistingLogFileIsNotContiguous => 0x8800022F as u32,
            NTDSB::hrColumnNull => 0x880003EC as u32,
            NTDSB::hrBufferTruncated => 0x880003EE as u32,
            NTDSB::hrDatabaseAttached => 0x880003EF as u32,
            NTDSB::hrSeekNotEqual => 0x8800040F as u32,
            NTDSB::hrNoIdleActivity => 0x88000422 as u32,
            NTDSB::hrNoWriteLock => 0x8800042B as u32,
            NTDSB::hrColumnSetNull => 0x8800042C as u32,
            NTDSB::hrTableEmpty => 0x88000515 as u32,
            NTDSB::hrCreateIndexFailed => 0x88000581 as u32,
            NTDSB::hrColumnMaxTruncated => 0x880005E8 as u32,
            NTDSB::hrwrnDataHasChanged => 0x8800064A as u32,
            NTDSB::hrKeyChanged => 0x88000652 as u32,
            NTDSB::hrFileOpenReadOnly => 0x88000715 as u32,
            NTDSB::hrFileClose => 0xC8000066 as u32,
            NTDSB::hrOutOfThreads => 0xC8000067 as u32,
            NTDSB::hrTooManyIO => 0xC8000069 as u32,
            NTDSB::hrBFInUse => 0xC80000CA as u32,
            NTDSB::hrPMRecDeleted => 0xC800012E as u32,
            NTDSB::hrFLDTooManySegments => 0xC8000191 as u32,
            NTDSB::hrLogFileCorrupt => 0xC80001F5 as u32,
            NTDSB::hrNoBackupDirectory => 0xC80001F7 as u32,
            NTDSB::hrBackupDirectoryNotEmpty => 0xC80001F8 as u32,
            NTDSB::hrBackupInProgress => 0xC80001F9 as u32,
            NTDSB::hrMissingPreviousLogFile => 0xC80001FD as u32,
            NTDSB::hrLogWriteFail => 0xC80001FE as u32,
            NTDSB::hrBadLogVersion => 0xC8000202 as u32,
            NTDSB::hrInvalidLogSequence => 0xC8000203 as u32,
            NTDSB::hrLoggingDisabled => 0xC8000204 as u32,
            NTDSB::hrLogBufferTooSmall => 0xC8000205 as u32,
            NTDSB::hrLogSequenceEnd => 0xC8000207 as u32,
            NTDSB::hrNoBackup => 0xC8000208 as u32,
            NTDSB::hrInvalidBackupSequence => 0xC8000209 as u32,
            NTDSB::hrBackupNotAllowedYet => 0xC800020B as u32,
            NTDSB::hrDeleteBackupFileFail => 0xC800020C as u32,
            NTDSB::hrMakeBackupDirectoryFail => 0xC800020D as u32,
            NTDSB::hrInvalidBackup => 0xC800020E as u32,
            NTDSB::hrRecoveredWithErrors => 0xC800020F as u32,
            NTDSB::hrMissingLogFile => 0xC8000210 as u32,
            NTDSB::hrLogDiskFull => 0xC8000211 as u32,
            NTDSB::hrBadLogSignature => 0xC8000212 as u32,
            NTDSB::hrBadDbSignature => 0xC8000213 as u32,
            NTDSB::hrBadCheckpointSignature => 0xC8000214 as u32,
            NTDSB::hrCheckpointCorrupt => 0xC8000215 as u32,
            NTDSB::hrDatabaseInconsistent => 0xC8000226 as u32,
            NTDSB::hrConsistentTimeMismatch => 0xC8000227 as u32,
            NTDSB::hrPatchFileMismatch => 0xC8000228 as u32,
            NTDSB::hrRestoreLogTooLow => 0xC8000229 as u32,
            NTDSB::hrRestoreLogTooHigh => 0xC800022A as u32,
            NTDSB::hrGivenLogFileHasBadSignature => 0xC800022B as u32,
            NTDSB::hrGivenLogFileIsNotContiguous => 0xC800022C as u32,
            NTDSB::hrMissingRestoreLogFiles => 0xC800022D as u32,
            NTDSB::hrMissingFullBackup => 0xC8000230 as u32,
            NTDSB::hrBadBackupDatabaseSize => 0xC8000231 as u32,
            NTDSB::hrTermInProgress => 0xC80003E8 as u32,
            NTDSB::hrFeatureNotAvailable => 0xC80003E9 as u32,
            NTDSB::hrInvalidName => 0xC80003EA as u32,
            NTDSB::hrInvalidParameter => 0xC80003EB as u32,
            NTDSB::hrInvalidDatabaseId => 0xC80003F2 as u32,
            NTDSB::hrOutOfMemory => 0xC80003F3 as u32,
            NTDSB::hrOutOfDatabaseSpace => 0xC80003F4 as u32,
            NTDSB::hrOutOfCursors => 0xC80003F5 as u32,
            NTDSB::hrOutOfBuffers => 0xC80003F6 as u32,
            NTDSB::hrTooManyIndexes => 0xC80003F7 as u32,
            NTDSB::hrTooManyKeys => 0xC80003F8 as u32,
            NTDSB::hrRecordDeleted => 0xC80003F9 as u32,
            NTDSB::hrReadVerifyFailure => 0xC80003FA as u32,
            NTDSB::hrOutOfFileHandles => 0xC80003FC as u32,
            NTDSB::hrDiskIO => 0xC80003FE as u32,
            NTDSB::hrInvalidPath => 0xC80003FF as u32,
            NTDSB::hrRecordTooBig => 0xC8000402 as u32,
            NTDSB::hrTooManyOpenDatabases => 0xC8000403 as u32,
            NTDSB::hrInvalidDatabase => 0xC8000404 as u32,
            NTDSB::hrNotInitialized => 0xC8000405 as u32,
            NTDSB::hrAlreadyInitialized => 0xC8000406 as u32,
            NTDSB::hrFileAccessDenied => 0xC8000408 as u32,
            NTDSB::hrBufferTooSmall => 0xC800040E as u32,
            NTDSB::hrTooManyColumns => 0xC8000410 as u32,
            NTDSB::hrContainerNotEmpty => 0xC8000413 as u32,
            NTDSB::hrInvalidFilename => 0xC8000414 as u32,
            NTDSB::hrInvalidBookmark => 0xC8000415 as u32,
            NTDSB::hrColumnInUse => 0xC8000416 as u32,
            NTDSB::hrInvalidBufferSize => 0xC8000417 as u32,
            NTDSB::hrColumnNotUpdatable => 0xC8000418 as u32,
            NTDSB::hrIndexInUse => 0xC800041B as u32,
            NTDSB::hrNullKeyDisallowed => 0xC800041D as u32,
            NTDSB::hrNotInTransaction => 0xC800041E as u32,
            NTDSB::hrTooManyActiveUsers => 0xC8000423 as u32,
            NTDSB::hrInvalidCountry => 0xC8000425 as u32,
            NTDSB::hrInvalidLanguageId => 0xC8000426 as u32,
            NTDSB::hrInvalidCodePage => 0xC8000427 as u32,
            NTDSB::hrVersionStoreOutOfMemory => 0xC800042D as u32,
            NTDSB::hrCurrencyStackOutOfMemory => 0xC800042E as u32,
            NTDSB::hrOutOfSessions => 0xC800044D as u32,
            NTDSB::hrWriteConflict => 0xC800044E as u32,
            NTDSB::hrTransTooDeep => 0xC800044F as u32,
            NTDSB::hrInvalidSesid => 0xC8000450 as u32,
            NTDSB::hrSessionWriteConflict => 0xC8000453 as u32,
            NTDSB::hrInTransaction => 0xC8000454 as u32,
            NTDSB::hrDatabaseDuplicate => 0xC80004B1 as u32,
            NTDSB::hrDatabaseInUse => 0xC80004B2 as u32,
            NTDSB::hrDatabaseNotFound => 0xC80004B3 as u32,
            NTDSB::hrDatabaseInvalidName => 0xC80004B4 as u32,
            NTDSB::hrDatabaseInvalidPages => 0xC80004B5 as u32,
            NTDSB::hrDatabaseCorrupted => 0xC80004B6 as u32,
            NTDSB::hrDatabaseLocked => 0xC80004B7 as u32,
            NTDSB::hrTableLocked => 0xC8000516 as u32,
            NTDSB::hrTableDuplicate => 0xC8000517 as u32,
            NTDSB::hrTableInUse => 0xC8000518 as u32,
            NTDSB::hrObjectNotFound => 0xC8000519 as u32,
            NTDSB::hrCannotRename => 0xC800051A as u32,
            NTDSB::hrDensityInvalid => 0xC800051B as u32,
            NTDSB::hrTableNotEmpty => 0xC800051C as u32,
            NTDSB::hrInvalidTableId => 0xC800051E as u32,
            NTDSB::hrTooManyOpenTables => 0xC800051F as u32,
            NTDSB::hrIllegalOperation => 0xC8000520 as u32,
            NTDSB::hrObjectDuplicate => 0xC8000522 as u32,
            NTDSB::hrInvalidObject => 0xC8000524 as u32,
            NTDSB::hrIndexCantBuild => 0xC8000579 as u32,
            NTDSB::hrIndexHasPrimary => 0xC800057A as u32,
            NTDSB::hrIndexDuplicate => 0xC800057B as u32,
            NTDSB::hrIndexNotFound => 0xC800057C as u32,
            NTDSB::hrIndexMustStay => 0xC800057D as u32,
            NTDSB::hrIndexInvalidDef => 0xC800057E as u32,
            NTDSB::hrIndexHasClustered => 0xC8000580 as u32,
            NTDSB::hrTooManyOpenIndexes => 0xC8000582 as u32,
            NTDSB::hrColumnLong => 0xC80005DD as u32,
            NTDSB::hrColumnDoesNotFit => 0xC80005DF as u32,
            NTDSB::hrNullInvalid => 0xC80005E0 as u32,
            NTDSB::hrColumnIndexed => 0xC80005E1 as u32,
            NTDSB::hrColumnTooBig => 0xC80005E2 as u32,
            NTDSB::hrColumnNotFound => 0xC80005E3 as u32,
            NTDSB::hrColumnDuplicate => 0xC80005E4 as u32,
            NTDSB::hrColumn2ndSysMaint => 0xC80005E6 as u32,
            NTDSB::hrInvalidColumnType => 0xC80005E7 as u32,
            NTDSB::hrColumnCannotIndex => 0xC80005E9 as u32,
            NTDSB::hrTaggedNotNULL => 0xC80005EA as u32,
            NTDSB::hrNoCurrentIndex => 0xC80005EB as u32,
            NTDSB::hrKeyIsMade => 0xC80005EC as u32,
            NTDSB::hrBadColumnId => 0xC80005ED as u32,
            NTDSB::hrBadItagSequence => 0xC80005EE as u32,
            NTDSB::hrCannotBeTagged => 0xC80005F1 as u32,
            NTDSB::hrRecordNotFound => 0xC8000641 as u32,
            NTDSB::hrNoCurrentRecord => 0xC8000643 as u32,
            NTDSB::hrRecordClusteredChanged => 0xC8000644 as u32,
            NTDSB::hrKeyDuplicate => 0xC8000645 as u32,
            NTDSB::hrAlreadyPrepared => 0xC8000647 as u32,
            NTDSB::hrKeyNotMade => 0xC8000648 as u32,
            NTDSB::hrUpdateNotPrepared => 0xC8000649 as u32,
            NTDSB::hrerrDataHasChanged => 0xC800064B as u32,
            NTDSB::hrTooManySorts => 0xC80006A5 as u32,
            NTDSB::hrInvalidOnSort => 0xC80006A6 as u32,
            NTDSB::hrTempFileOpenError => 0xC800070B as u32,
            NTDSB::hrTooManyAttachedDatabases => 0xC800070D as u32,
            NTDSB::hrDiskFull => 0xC8000710 as u32,
            NTDSB::hrPermissionDenied => 0xC8000711 as u32,
            NTDSB::hrFileNotFound => 0xC8000713 as u32,
            NTDSB::hrAfterInitialization => 0xC800073A as u32,
            NTDSB::hrLogCorrupted => 0xC800073C as u32,
            NTDSB::hrInvalidOperation => 0xC8000772 as u32,
            NTDSB::hrAccessDenied => 0xC8000773 as u32,
        }
    }

    pub fn error(&self) -> RawError {
        return match self {
            NTDSB::FHCFG_E_INVALID_REHYDRATION_STATE => RawError::Kind(NTDSB::FHCFG_E_INVALID_REHYDRATION_STATE),
            NTDSB::hrBFNotSynchronous => RawError::Kind(NTDSB::hrBFNotSynchronous),
            NTDSB::hrBFPageNotFound => RawError::Kind(NTDSB::hrBFPageNotFound),
            NTDSB::hrRemainingVersions => RawError::Kind(NTDSB::hrRemainingVersions),
            NTDSB::hrFLDKeyTooBig => RawError::Kind(NTDSB::hrFLDKeyTooBig),
            NTDSB::hrFLDNullKey => RawError::Kind(NTDSB::hrFLDNullKey),
            NTDSB::hrExistingLogFileHasBadSignature => RawError::Kind(NTDSB::hrExistingLogFileHasBadSignature),
            NTDSB::hrExistingLogFileIsNotContiguous => RawError::Kind(NTDSB::hrExistingLogFileIsNotContiguous),
            NTDSB::hrColumnNull => RawError::Kind(NTDSB::hrColumnNull),
            NTDSB::hrBufferTruncated => RawError::Kind(NTDSB::hrBufferTruncated),
            NTDSB::hrDatabaseAttached => RawError::Kind(NTDSB::hrDatabaseAttached),
            NTDSB::hrSeekNotEqual => RawError::Kind(NTDSB::hrSeekNotEqual),
            NTDSB::hrNoIdleActivity => RawError::Kind(NTDSB::hrNoIdleActivity),
            NTDSB::hrNoWriteLock => RawError::Kind(NTDSB::hrNoWriteLock),
            NTDSB::hrColumnSetNull => RawError::Kind(NTDSB::hrColumnSetNull),
            NTDSB::hrTableEmpty => RawError::Kind(NTDSB::hrTableEmpty),
            NTDSB::hrCreateIndexFailed => RawError::Kind(NTDSB::hrCreateIndexFailed),
            NTDSB::hrColumnMaxTruncated => RawError::Kind(NTDSB::hrColumnMaxTruncated),
            NTDSB::hrwrnDataHasChanged => RawError::Kind(NTDSB::hrwrnDataHasChanged),
            NTDSB::hrKeyChanged => RawError::Kind(NTDSB::hrKeyChanged),
            NTDSB::hrFileOpenReadOnly => RawError::Kind(NTDSB::hrFileOpenReadOnly),
            NTDSB::hrFileClose => RawError::Kind(NTDSB::hrFileClose),
            NTDSB::hrOutOfThreads => RawError::Kind(NTDSB::hrOutOfThreads),
            NTDSB::hrTooManyIO => RawError::Kind(NTDSB::hrTooManyIO),
            NTDSB::hrBFInUse => RawError::Kind(NTDSB::hrBFInUse),
            NTDSB::hrPMRecDeleted => RawError::Kind(NTDSB::hrPMRecDeleted),
            NTDSB::hrFLDTooManySegments => RawError::Kind(NTDSB::hrFLDTooManySegments),
            NTDSB::hrLogFileCorrupt => RawError::Kind(NTDSB::hrLogFileCorrupt),
            NTDSB::hrNoBackupDirectory => RawError::Kind(NTDSB::hrNoBackupDirectory),
            NTDSB::hrBackupDirectoryNotEmpty => RawError::Kind(NTDSB::hrBackupDirectoryNotEmpty),
            NTDSB::hrBackupInProgress => RawError::Kind(NTDSB::hrBackupInProgress),
            NTDSB::hrMissingPreviousLogFile => RawError::Kind(NTDSB::hrMissingPreviousLogFile),
            NTDSB::hrLogWriteFail => RawError::Kind(NTDSB::hrLogWriteFail),
            NTDSB::hrBadLogVersion => RawError::Kind(NTDSB::hrBadLogVersion),
            NTDSB::hrInvalidLogSequence => RawError::Kind(NTDSB::hrInvalidLogSequence),
            NTDSB::hrLoggingDisabled => RawError::Kind(NTDSB::hrLoggingDisabled),
            NTDSB::hrLogBufferTooSmall => RawError::Kind(NTDSB::hrLogBufferTooSmall),
            NTDSB::hrLogSequenceEnd => RawError::Kind(NTDSB::hrLogSequenceEnd),
            NTDSB::hrNoBackup => RawError::Kind(NTDSB::hrNoBackup),
            NTDSB::hrInvalidBackupSequence => RawError::Kind(NTDSB::hrInvalidBackupSequence),
            NTDSB::hrBackupNotAllowedYet => RawError::Kind(NTDSB::hrBackupNotAllowedYet),
            NTDSB::hrDeleteBackupFileFail => RawError::Kind(NTDSB::hrDeleteBackupFileFail),
            NTDSB::hrMakeBackupDirectoryFail => RawError::Kind(NTDSB::hrMakeBackupDirectoryFail),
            NTDSB::hrInvalidBackup => RawError::Kind(NTDSB::hrInvalidBackup),
            NTDSB::hrRecoveredWithErrors => RawError::Kind(NTDSB::hrRecoveredWithErrors),
            NTDSB::hrMissingLogFile => RawError::Kind(NTDSB::hrMissingLogFile),
            NTDSB::hrLogDiskFull => RawError::Kind(NTDSB::hrLogDiskFull),
            NTDSB::hrBadLogSignature => RawError::Kind(NTDSB::hrBadLogSignature),
            NTDSB::hrBadDbSignature => RawError::Kind(NTDSB::hrBadDbSignature),
            NTDSB::hrBadCheckpointSignature => RawError::Kind(NTDSB::hrBadCheckpointSignature),
            NTDSB::hrCheckpointCorrupt => RawError::Kind(NTDSB::hrCheckpointCorrupt),
            NTDSB::hrDatabaseInconsistent => RawError::Kind(NTDSB::hrDatabaseInconsistent),
            NTDSB::hrConsistentTimeMismatch => RawError::Kind(NTDSB::hrConsistentTimeMismatch),
            NTDSB::hrPatchFileMismatch => RawError::Kind(NTDSB::hrPatchFileMismatch),
            NTDSB::hrRestoreLogTooLow => RawError::Kind(NTDSB::hrRestoreLogTooLow),
            NTDSB::hrRestoreLogTooHigh => RawError::Kind(NTDSB::hrRestoreLogTooHigh),
            NTDSB::hrGivenLogFileHasBadSignature => RawError::Kind(NTDSB::hrGivenLogFileHasBadSignature),
            NTDSB::hrGivenLogFileIsNotContiguous => RawError::Kind(NTDSB::hrGivenLogFileIsNotContiguous),
            NTDSB::hrMissingRestoreLogFiles => RawError::Kind(NTDSB::hrMissingRestoreLogFiles),
            NTDSB::hrMissingFullBackup => RawError::Kind(NTDSB::hrMissingFullBackup),
            NTDSB::hrBadBackupDatabaseSize => RawError::Kind(NTDSB::hrBadBackupDatabaseSize),
            NTDSB::hrTermInProgress => RawError::Kind(NTDSB::hrTermInProgress),
            NTDSB::hrFeatureNotAvailable => RawError::Kind(NTDSB::hrFeatureNotAvailable),
            NTDSB::hrInvalidName => RawError::Kind(NTDSB::hrInvalidName),
            NTDSB::hrInvalidParameter => RawError::Kind(NTDSB::hrInvalidParameter),
            NTDSB::hrInvalidDatabaseId => RawError::Kind(NTDSB::hrInvalidDatabaseId),
            NTDSB::hrOutOfMemory => RawError::Kind(NTDSB::hrOutOfMemory),
            NTDSB::hrOutOfDatabaseSpace => RawError::Kind(NTDSB::hrOutOfDatabaseSpace),
            NTDSB::hrOutOfCursors => RawError::Kind(NTDSB::hrOutOfCursors),
            NTDSB::hrOutOfBuffers => RawError::Kind(NTDSB::hrOutOfBuffers),
            NTDSB::hrTooManyIndexes => RawError::Kind(NTDSB::hrTooManyIndexes),
            NTDSB::hrTooManyKeys => RawError::Kind(NTDSB::hrTooManyKeys),
            NTDSB::hrRecordDeleted => RawError::Kind(NTDSB::hrRecordDeleted),
            NTDSB::hrReadVerifyFailure => RawError::Kind(NTDSB::hrReadVerifyFailure),
            NTDSB::hrOutOfFileHandles => RawError::Kind(NTDSB::hrOutOfFileHandles),
            NTDSB::hrDiskIO => RawError::Kind(NTDSB::hrDiskIO),
            NTDSB::hrInvalidPath => RawError::Kind(NTDSB::hrInvalidPath),
            NTDSB::hrRecordTooBig => RawError::Kind(NTDSB::hrRecordTooBig),
            NTDSB::hrTooManyOpenDatabases => RawError::Kind(NTDSB::hrTooManyOpenDatabases),
            NTDSB::hrInvalidDatabase => RawError::Kind(NTDSB::hrInvalidDatabase),
            NTDSB::hrNotInitialized => RawError::Kind(NTDSB::hrNotInitialized),
            NTDSB::hrAlreadyInitialized => RawError::Kind(NTDSB::hrAlreadyInitialized),
            NTDSB::hrFileAccessDenied => RawError::Kind(NTDSB::hrFileAccessDenied),
            NTDSB::hrBufferTooSmall => RawError::Kind(NTDSB::hrBufferTooSmall),
            NTDSB::hrTooManyColumns => RawError::Kind(NTDSB::hrTooManyColumns),
            NTDSB::hrContainerNotEmpty => RawError::Kind(NTDSB::hrContainerNotEmpty),
            NTDSB::hrInvalidFilename => RawError::Kind(NTDSB::hrInvalidFilename),
            NTDSB::hrInvalidBookmark => RawError::Kind(NTDSB::hrInvalidBookmark),
            NTDSB::hrColumnInUse => RawError::Kind(NTDSB::hrColumnInUse),
            NTDSB::hrInvalidBufferSize => RawError::Kind(NTDSB::hrInvalidBufferSize),
            NTDSB::hrColumnNotUpdatable => RawError::Kind(NTDSB::hrColumnNotUpdatable),
            NTDSB::hrIndexInUse => RawError::Kind(NTDSB::hrIndexInUse),
            NTDSB::hrNullKeyDisallowed => RawError::Kind(NTDSB::hrNullKeyDisallowed),
            NTDSB::hrNotInTransaction => RawError::Kind(NTDSB::hrNotInTransaction),
            NTDSB::hrTooManyActiveUsers => RawError::Kind(NTDSB::hrTooManyActiveUsers),
            NTDSB::hrInvalidCountry => RawError::Kind(NTDSB::hrInvalidCountry),
            NTDSB::hrInvalidLanguageId => RawError::Kind(NTDSB::hrInvalidLanguageId),
            NTDSB::hrInvalidCodePage => RawError::Kind(NTDSB::hrInvalidCodePage),
            NTDSB::hrVersionStoreOutOfMemory => RawError::Kind(NTDSB::hrVersionStoreOutOfMemory),
            NTDSB::hrCurrencyStackOutOfMemory => RawError::Kind(NTDSB::hrCurrencyStackOutOfMemory),
            NTDSB::hrOutOfSessions => RawError::Kind(NTDSB::hrOutOfSessions),
            NTDSB::hrWriteConflict => RawError::Kind(NTDSB::hrWriteConflict),
            NTDSB::hrTransTooDeep => RawError::Kind(NTDSB::hrTransTooDeep),
            NTDSB::hrInvalidSesid => RawError::Kind(NTDSB::hrInvalidSesid),
            NTDSB::hrSessionWriteConflict => RawError::Kind(NTDSB::hrSessionWriteConflict),
            NTDSB::hrInTransaction => RawError::Kind(NTDSB::hrInTransaction),
            NTDSB::hrDatabaseDuplicate => RawError::Kind(NTDSB::hrDatabaseDuplicate),
            NTDSB::hrDatabaseInUse => RawError::Kind(NTDSB::hrDatabaseInUse),
            NTDSB::hrDatabaseNotFound => RawError::Kind(NTDSB::hrDatabaseNotFound),
            NTDSB::hrDatabaseInvalidName => RawError::Kind(NTDSB::hrDatabaseInvalidName),
            NTDSB::hrDatabaseInvalidPages => RawError::Kind(NTDSB::hrDatabaseInvalidPages),
            NTDSB::hrDatabaseCorrupted => RawError::Kind(NTDSB::hrDatabaseCorrupted),
            NTDSB::hrDatabaseLocked => RawError::Kind(NTDSB::hrDatabaseLocked),
            NTDSB::hrTableLocked => RawError::Kind(NTDSB::hrTableLocked),
            NTDSB::hrTableDuplicate => RawError::Kind(NTDSB::hrTableDuplicate),
            NTDSB::hrTableInUse => RawError::Kind(NTDSB::hrTableInUse),
            NTDSB::hrObjectNotFound => RawError::Kind(NTDSB::hrObjectNotFound),
            NTDSB::hrCannotRename => RawError::Kind(NTDSB::hrCannotRename),
            NTDSB::hrDensityInvalid => RawError::Kind(NTDSB::hrDensityInvalid),
            NTDSB::hrTableNotEmpty => RawError::Kind(NTDSB::hrTableNotEmpty),
            NTDSB::hrInvalidTableId => RawError::Kind(NTDSB::hrInvalidTableId),
            NTDSB::hrTooManyOpenTables => RawError::Kind(NTDSB::hrTooManyOpenTables),
            NTDSB::hrIllegalOperation => RawError::Kind(NTDSB::hrIllegalOperation),
            NTDSB::hrObjectDuplicate => RawError::Kind(NTDSB::hrObjectDuplicate),
            NTDSB::hrInvalidObject => RawError::Kind(NTDSB::hrInvalidObject),
            NTDSB::hrIndexCantBuild => RawError::Kind(NTDSB::hrIndexCantBuild),
            NTDSB::hrIndexHasPrimary => RawError::Kind(NTDSB::hrIndexHasPrimary),
            NTDSB::hrIndexDuplicate => RawError::Kind(NTDSB::hrIndexDuplicate),
            NTDSB::hrIndexNotFound => RawError::Kind(NTDSB::hrIndexNotFound),
            NTDSB::hrIndexMustStay => RawError::Kind(NTDSB::hrIndexMustStay),
            NTDSB::hrIndexInvalidDef => RawError::Kind(NTDSB::hrIndexInvalidDef),
            NTDSB::hrIndexHasClustered => RawError::Kind(NTDSB::hrIndexHasClustered),
            NTDSB::hrTooManyOpenIndexes => RawError::Kind(NTDSB::hrTooManyOpenIndexes),
            NTDSB::hrColumnLong => RawError::Kind(NTDSB::hrColumnLong),
            NTDSB::hrColumnDoesNotFit => RawError::Kind(NTDSB::hrColumnDoesNotFit),
            NTDSB::hrNullInvalid => RawError::Kind(NTDSB::hrNullInvalid),
            NTDSB::hrColumnIndexed => RawError::Kind(NTDSB::hrColumnIndexed),
            NTDSB::hrColumnTooBig => RawError::Kind(NTDSB::hrColumnTooBig),
            NTDSB::hrColumnNotFound => RawError::Kind(NTDSB::hrColumnNotFound),
            NTDSB::hrColumnDuplicate => RawError::Kind(NTDSB::hrColumnDuplicate),
            NTDSB::hrColumn2ndSysMaint => RawError::Kind(NTDSB::hrColumn2ndSysMaint),
            NTDSB::hrInvalidColumnType => RawError::Kind(NTDSB::hrInvalidColumnType),
            NTDSB::hrColumnCannotIndex => RawError::Kind(NTDSB::hrColumnCannotIndex),
            NTDSB::hrTaggedNotNULL => RawError::Kind(NTDSB::hrTaggedNotNULL),
            NTDSB::hrNoCurrentIndex => RawError::Kind(NTDSB::hrNoCurrentIndex),
            NTDSB::hrKeyIsMade => RawError::Kind(NTDSB::hrKeyIsMade),
            NTDSB::hrBadColumnId => RawError::Kind(NTDSB::hrBadColumnId),
            NTDSB::hrBadItagSequence => RawError::Kind(NTDSB::hrBadItagSequence),
            NTDSB::hrCannotBeTagged => RawError::Kind(NTDSB::hrCannotBeTagged),
            NTDSB::hrRecordNotFound => RawError::Kind(NTDSB::hrRecordNotFound),
            NTDSB::hrNoCurrentRecord => RawError::Kind(NTDSB::hrNoCurrentRecord),
            NTDSB::hrRecordClusteredChanged => RawError::Kind(NTDSB::hrRecordClusteredChanged),
            NTDSB::hrKeyDuplicate => RawError::Kind(NTDSB::hrKeyDuplicate),
            NTDSB::hrAlreadyPrepared => RawError::Kind(NTDSB::hrAlreadyPrepared),
            NTDSB::hrKeyNotMade => RawError::Kind(NTDSB::hrKeyNotMade),
            NTDSB::hrUpdateNotPrepared => RawError::Kind(NTDSB::hrUpdateNotPrepared),
            NTDSB::hrerrDataHasChanged => RawError::Kind(NTDSB::hrerrDataHasChanged),
            NTDSB::hrTooManySorts => RawError::Kind(NTDSB::hrTooManySorts),
            NTDSB::hrInvalidOnSort => RawError::Kind(NTDSB::hrInvalidOnSort),
            NTDSB::hrTempFileOpenError => RawError::Kind(NTDSB::hrTempFileOpenError),
            NTDSB::hrTooManyAttachedDatabases => RawError::Kind(NTDSB::hrTooManyAttachedDatabases),
            NTDSB::hrDiskFull => RawError::Kind(NTDSB::hrDiskFull),
            NTDSB::hrPermissionDenied => RawError::Kind(NTDSB::hrPermissionDenied),
            NTDSB::hrFileNotFound => RawError::Kind(NTDSB::hrFileNotFound),
            NTDSB::hrAfterInitialization => RawError::Kind(NTDSB::hrAfterInitialization),
            NTDSB::hrLogCorrupted => RawError::Kind(NTDSB::hrLogCorrupted),
            NTDSB::hrInvalidOperation => RawError::Kind(NTDSB::hrInvalidOperation),
            NTDSB::hrAccessDenied => RawError::Kind(NTDSB::hrAccessDenied),
        }
    }

    pub fn description(&self) -> &'static str {
        return match self {
            NTDSB::FHCFG_E_INVALID_REHYDRATION_STATE => "Configuration Manager: Rehydration can't be performed in the current configuration state.",
            NTDSB::hrBFNotSynchronous => "The buffer page has been evicted.",
            NTDSB::hrBFPageNotFound => "Unable to find the page.",
            NTDSB::hrRemainingVersions => "There is idle work remaining.",
            NTDSB::hrFLDKeyTooBig => "The key was truncated because it exceeded the maximum length.",
            NTDSB::hrFLDNullKey => "The key is NULL.",
            NTDSB::hrExistingLogFileHasBadSignature => "The log file in the log file path is damaged.",
            NTDSB::hrExistingLogFileIsNotContiguous => "Unable to find a mandatory log file in the log file path.",
            NTDSB::hrColumnNull => "The value of the column is null.",
            NTDSB::hrBufferTruncated => "The buffer is too small for data.",
            NTDSB::hrDatabaseAttached => "The database is already attached.",
            NTDSB::hrSeekNotEqual => "Either SeekLE or SeekGE did not find an exact match.",
            NTDSB::hrNoIdleActivity => "No idle activity occurred.",
            NTDSB::hrNoWriteLock => "There is no write lock at transaction level 0.",
            NTDSB::hrColumnSetNull => "The column value is set to null.",
            NTDSB::hrTableEmpty => "An empty table was opened.",
            NTDSB::hrCreateIndexFailed => "Unable to create the index because an error occurred while creating a table.",
            NTDSB::hrColumnMaxTruncated => "The column was truncated because it exceeded the maximum length.",
            NTDSB::hrwrnDataHasChanged => "Data has changed.",
            NTDSB::hrKeyChanged => "Moved to a new key.",
            NTDSB::hrFileOpenReadOnly => "The database file is read only.",
            NTDSB::hrFileClose => "Unable to close the DOS file.",
            NTDSB::hrOutOfThreads => "Unable to start a thread because there are none available.",
            NTDSB::hrTooManyIO => "The system is busy because there are too many I/Os.",
            NTDSB::hrBFInUse => "Unable to abandon the buffer.",
            NTDSB::hrPMRecDeleted => "The record has been deleted.",
            NTDSB::hrFLDTooManySegments => "There are too many key segments.",
            NTDSB::hrLogFileCorrupt => "The log file is damaged.",
            NTDSB::hrNoBackupDirectory => "No backup directory was given.",
            NTDSB::hrBackupDirectoryNotEmpty => "The backup directory is not empty.",
            NTDSB::hrBackupInProgress => "Backup is already active.",
            NTDSB::hrMissingPreviousLogFile => "A log file for the checkpoint is missing.",
            NTDSB::hrLogWriteFail => "Unable to write to the log file.",
            NTDSB::hrBadLogVersion => "The version of the log file is not compatible with the version of the Windows NT Directory Service database (NTDS).",
            NTDSB::hrInvalidLogSequence => "The time stamp in the next log does not match what was expected.",
            NTDSB::hrLoggingDisabled => "The log is not active.",
            NTDSB::hrLogBufferTooSmall => "The log buffer is too small to be recovered.",
            NTDSB::hrLogSequenceEnd => "The maximum number of log files has been exceeded.",
            NTDSB::hrNoBackup => "There is no backup in progress.",
            NTDSB::hrInvalidBackupSequence => "The backup call is out of sequence.",
            NTDSB::hrBackupNotAllowedYet => "Unable to perform a backup now.",
            NTDSB::hrDeleteBackupFileFail => "Unable to delete the backup file.",
            NTDSB::hrMakeBackupDirectoryFail => "Unable to make a backup temporary directory.",
            NTDSB::hrInvalidBackup => "An incremental backup cannot be performed when circular logging is enabled.",
            NTDSB::hrRecoveredWithErrors => "Errors were encountered during the repair process.",
            NTDSB::hrMissingLogFile => "The current log file is missing.",
            NTDSB::hrLogDiskFull => "The log disk is full.",
            NTDSB::hrBadLogSignature => "A log file is damaged.",
            NTDSB::hrBadDbSignature => "A database file is damaged.",
            NTDSB::hrBadCheckpointSignature => "A checkpoint file is damaged.",
            NTDSB::hrCheckpointCorrupt => "A checkpoint file either could not be found or is damaged.",
            NTDSB::hrDatabaseInconsistent => "The database is damaged.",
            NTDSB::hrConsistentTimeMismatch => "There is a mismatch in the database's last consistent time.",
            NTDSB::hrPatchFileMismatch => "The patch file is not generated from this backup.",
            NTDSB::hrRestoreLogTooLow => "The starting log number is too low for the restore.",
            NTDSB::hrRestoreLogTooHigh => "The starting log number is too high for the restore.",
            NTDSB::hrGivenLogFileHasBadSignature => "The log file downloaded from the tape is damaged.",
            NTDSB::hrGivenLogFileIsNotContiguous => "Unable to find a mandatory log file after the tape was downloaded.",
            NTDSB::hrMissingRestoreLogFiles => "The data is not fully restored because some log files are missing.",
            NTDSB::hrMissingFullBackup => "The database missed a previous full backup before the incremental backup.",
            NTDSB::hrBadBackupDatabaseSize => "The backup database size must be a multiple of 4K (4096 bytes).",
            NTDSB::hrTermInProgress => "The database is being shut down.",
            NTDSB::hrFeatureNotAvailable => "The feature is not available.",
            NTDSB::hrInvalidName => "The name is not valid.",
            NTDSB::hrInvalidParameter => "The parameter is not valid.",
            NTDSB::hrInvalidDatabaseId => "The database ID is not valid.",
            NTDSB::hrOutOfMemory => "The computer is out of memory.",
            NTDSB::hrOutOfDatabaseSpace => "The database has reached the maximum size of 16 GB.",
            NTDSB::hrOutOfCursors => "Out of table cursors.",
            NTDSB::hrOutOfBuffers => "Out of database page buffers.",
            NTDSB::hrTooManyIndexes => "There are too many indexes.",
            NTDSB::hrTooManyKeys => "There are too many columns in an index.",
            NTDSB::hrRecordDeleted => "The record has been deleted.",
            NTDSB::hrReadVerifyFailure => "A read verification error occurred.",
            NTDSB::hrOutOfFileHandles => "Out of file handles.",
            NTDSB::hrDiskIO => "A disk I/O error occurred.",
            NTDSB::hrInvalidPath => "The path to the file is not valid.",
            NTDSB::hrRecordTooBig => "The record has exceeded the maximum size.",
            NTDSB::hrTooManyOpenDatabases => "There are too many open databases.",
            NTDSB::hrInvalidDatabase => "The file is not a database file.",
            NTDSB::hrNotInitialized => "The database was not yet called.",
            NTDSB::hrAlreadyInitialized => "The database was already called.",
            NTDSB::hrFileAccessDenied => "Unable to access the file.",
            NTDSB::hrBufferTooSmall => "The buffer is too small.",
            NTDSB::hrTooManyColumns => "There are too many columns defined.",
            NTDSB::hrContainerNotEmpty => "The container is not empty.",
            NTDSB::hrInvalidFilename => "The filename is not valid.",
            NTDSB::hrInvalidBookmark => "The bookmark is not valid.",
            NTDSB::hrColumnInUse => "The column is used in an index.",
            NTDSB::hrInvalidBufferSize => "The data buffer does not match the column size.",
            NTDSB::hrColumnNotUpdatable => "Unable to set the column value.",
            NTDSB::hrIndexInUse => "The index is in use.",
            NTDSB::hrNullKeyDisallowed => "Null keys are not allowed on an index.",
            NTDSB::hrNotInTransaction => "The operation must be within a transaction.",
            NTDSB::hrTooManyActiveUsers => "There are too many active database users.",
            NTDSB::hrInvalidCountry => "The country code is either not known or is not valid.",
            NTDSB::hrInvalidLanguageId => "The language ID is either not known or is not valid.",
            NTDSB::hrInvalidCodePage => "The code page is either not known or is not valid.",
            NTDSB::hrVersionStoreOutOfMemory => "lMaxVerPages exceeded (XJET only).",
            NTDSB::hrCurrencyStackOutOfMemory => "Out of cursors.",
            NTDSB::hrOutOfSessions => "Out of sessions.",
            NTDSB::hrWriteConflict => "The write lock failed due to an outstanding write lock.",
            NTDSB::hrTransTooDeep => "The transactions are nested too deeply.",
            NTDSB::hrInvalidSesid => "The session handle is not valid.",
            NTDSB::hrSessionWriteConflict => "Another session has a private version of the page.",
            NTDSB::hrInTransaction => "The operation is not allowed within a transaction.",
            NTDSB::hrDatabaseDuplicate => "The database already exists.",
            NTDSB::hrDatabaseInUse => "The database is in use.",
            NTDSB::hrDatabaseNotFound => "The database does not exist.",
            NTDSB::hrDatabaseInvalidName => "The database name is not valid.",
            NTDSB::hrDatabaseInvalidPages => "The number of pages is not valid.",
            NTDSB::hrDatabaseCorrupted => "The database file is either damaged or cannot be found.",
            NTDSB::hrDatabaseLocked => "The database is locked.",
            NTDSB::hrTableLocked => "The table is locked.",
            NTDSB::hrTableDuplicate => "The table already exists.",
            NTDSB::hrTableInUse => "Unable to lock the table because it is already in use.",
            NTDSB::hrObjectNotFound => "The table or object does not exist.",
            NTDSB::hrCannotRename => "Unable to rename the temporary file.",
            NTDSB::hrDensityInvalid => "The file/index density is not valid.",
            NTDSB::hrTableNotEmpty => "Unable to define the clustered index.",
            NTDSB::hrInvalidTableId => "The table ID is not valid.",
            NTDSB::hrTooManyOpenTables => "Unable to open any more tables.",
            NTDSB::hrIllegalOperation => "The operation is not supported on tables.",
            NTDSB::hrObjectDuplicate => "The table or object name is already being used.",
            NTDSB::hrInvalidObject => "The object is not valid for operation.",
            NTDSB::hrIndexCantBuild => "Unable to build a clustered index.",
            NTDSB::hrIndexHasPrimary => "The primary index is already defined.",
            NTDSB::hrIndexDuplicate => "The index is already defined.",
            NTDSB::hrIndexNotFound => "The index does not exist.",
            NTDSB::hrIndexMustStay => "Unable to delete a clustered index.",
            NTDSB::hrIndexInvalidDef => "The index definition is illegal.",
            NTDSB::hrIndexHasClustered => "The clustered index is already defined.",
            NTDSB::hrTooManyOpenIndexes => "Out of index description blocks.",
            NTDSB::hrColumnLong => "The column value is too long.",
            NTDSB::hrColumnDoesNotFit => "The field will not fit in the record.",
            NTDSB::hrNullInvalid => "The value cannot be null.",
            NTDSB::hrColumnIndexed => "Unable to delete because the column is indexed.",
            NTDSB::hrColumnTooBig => "The length of the field exceeds the maximum length.",
            NTDSB::hrColumnNotFound => "Unable to find the column.",
            NTDSB::hrColumnDuplicate => "The field is already defined.",
            NTDSB::hrColumn2ndSysMaint => "Only one auto-increment or version column is allowed per table.",
            NTDSB::hrInvalidColumnType => "The column data type is not valid.",
            NTDSB::hrColumnCannotIndex => "Unable to index a long value column.",
            NTDSB::hrTaggedNotNULL => "Tagged columns cannot be null.",
            NTDSB::hrNoCurrentIndex => "The entry is not valid without a current index.",
            NTDSB::hrKeyIsMade => "The key is completely made.",
            NTDSB::hrBadColumnId => "The column ID is not correct.",
            NTDSB::hrBadItagSequence => "There is a bad instance identifier for a multivalued column.",
            NTDSB::hrCannotBeTagged => "AutoIncrement and Version cannot be multivalued.",
            NTDSB::hrRecordNotFound => "Unable to find the key.",
            NTDSB::hrNoCurrentRecord => "The currency is not on a record.",
            NTDSB::hrRecordClusteredChanged => "A clustered key cannot be changed.",
            NTDSB::hrKeyDuplicate => "The key already exists.",
            NTDSB::hrAlreadyPrepared => "The current entry has already been copied or cleared.",
            NTDSB::hrKeyNotMade => "No key was made.",
            NTDSB::hrUpdateNotPrepared => "Update was not prepared.",
            NTDSB::hrerrDataHasChanged => "The operation was abandoned because data has changed.",
            NTDSB::hrTooManySorts => "There are too many sort processes.",
            NTDSB::hrInvalidOnSort => "An operation that is not valid occurred in the sort.",
            NTDSB::hrTempFileOpenError => "Unable to open the temporary file.",
            NTDSB::hrTooManyAttachedDatabases => "There are too many databases open.",
            NTDSB::hrDiskFull => "The disk is full.",
            NTDSB::hrPermissionDenied => "Permission is denied.",
            NTDSB::hrFileNotFound => "Unable to find the file.",
            NTDSB::hrAfterInitialization => "Unable to restore after initialization.",
            NTDSB::hrLogCorrupted => "The database log files are damaged.",
            NTDSB::hrInvalidOperation => "The operation is not valid.",
            NTDSB::hrAccessDenied => "Access is denied.",
        }
    }

    pub fn from_name(name: &str) -> NTDSB {
        return match name {
            "FHCFG_E_INVALID_REHYDRATION_STATE" => NTDSB::FHCFG_E_INVALID_REHYDRATION_STATE,
            "hrBFNotSynchronous" => NTDSB::hrBFNotSynchronous,
            "hrBFPageNotFound" => NTDSB::hrBFPageNotFound,
            "hrRemainingVersions" => NTDSB::hrRemainingVersions,
            "hrFLDKeyTooBig" => NTDSB::hrFLDKeyTooBig,
            "hrFLDNullKey" => NTDSB::hrFLDNullKey,
            "hrExistingLogFileHasBadSignature" => NTDSB::hrExistingLogFileHasBadSignature,
            "hrExistingLogFileIsNotContiguous" => NTDSB::hrExistingLogFileIsNotContiguous,
            "hrColumnNull" => NTDSB::hrColumnNull,
            "hrBufferTruncated" => NTDSB::hrBufferTruncated,
            "hrDatabaseAttached" => NTDSB::hrDatabaseAttached,
            "hrSeekNotEqual" => NTDSB::hrSeekNotEqual,
            "hrNoIdleActivity" => NTDSB::hrNoIdleActivity,
            "hrNoWriteLock" => NTDSB::hrNoWriteLock,
            "hrColumnSetNull" => NTDSB::hrColumnSetNull,
            "hrTableEmpty" => NTDSB::hrTableEmpty,
            "hrCreateIndexFailed" => NTDSB::hrCreateIndexFailed,
            "hrColumnMaxTruncated" => NTDSB::hrColumnMaxTruncated,
            "hrwrnDataHasChanged" => NTDSB::hrwrnDataHasChanged,
            "hrKeyChanged" => NTDSB::hrKeyChanged,
            "hrFileOpenReadOnly" => NTDSB::hrFileOpenReadOnly,
            "hrFileClose" => NTDSB::hrFileClose,
            "hrOutOfThreads" => NTDSB::hrOutOfThreads,
            "hrTooManyIO" => NTDSB::hrTooManyIO,
            "hrBFInUse" => NTDSB::hrBFInUse,
            "hrPMRecDeleted" => NTDSB::hrPMRecDeleted,
            "hrFLDTooManySegments" => NTDSB::hrFLDTooManySegments,
            "hrLogFileCorrupt" => NTDSB::hrLogFileCorrupt,
            "hrNoBackupDirectory" => NTDSB::hrNoBackupDirectory,
            "hrBackupDirectoryNotEmpty" => NTDSB::hrBackupDirectoryNotEmpty,
            "hrBackupInProgress" => NTDSB::hrBackupInProgress,
            "hrMissingPreviousLogFile" => NTDSB::hrMissingPreviousLogFile,
            "hrLogWriteFail" => NTDSB::hrLogWriteFail,
            "hrBadLogVersion" => NTDSB::hrBadLogVersion,
            "hrInvalidLogSequence" => NTDSB::hrInvalidLogSequence,
            "hrLoggingDisabled" => NTDSB::hrLoggingDisabled,
            "hrLogBufferTooSmall" => NTDSB::hrLogBufferTooSmall,
            "hrLogSequenceEnd" => NTDSB::hrLogSequenceEnd,
            "hrNoBackup" => NTDSB::hrNoBackup,
            "hrInvalidBackupSequence" => NTDSB::hrInvalidBackupSequence,
            "hrBackupNotAllowedYet" => NTDSB::hrBackupNotAllowedYet,
            "hrDeleteBackupFileFail" => NTDSB::hrDeleteBackupFileFail,
            "hrMakeBackupDirectoryFail" => NTDSB::hrMakeBackupDirectoryFail,
            "hrInvalidBackup" => NTDSB::hrInvalidBackup,
            "hrRecoveredWithErrors" => NTDSB::hrRecoveredWithErrors,
            "hrMissingLogFile" => NTDSB::hrMissingLogFile,
            "hrLogDiskFull" => NTDSB::hrLogDiskFull,
            "hrBadLogSignature" => NTDSB::hrBadLogSignature,
            "hrBadDbSignature" => NTDSB::hrBadDbSignature,
            "hrBadCheckpointSignature" => NTDSB::hrBadCheckpointSignature,
            "hrCheckpointCorrupt" => NTDSB::hrCheckpointCorrupt,
            "hrDatabaseInconsistent" => NTDSB::hrDatabaseInconsistent,
            "hrConsistentTimeMismatch" => NTDSB::hrConsistentTimeMismatch,
            "hrPatchFileMismatch" => NTDSB::hrPatchFileMismatch,
            "hrRestoreLogTooLow" => NTDSB::hrRestoreLogTooLow,
            "hrRestoreLogTooHigh" => NTDSB::hrRestoreLogTooHigh,
            "hrGivenLogFileHasBadSignature" => NTDSB::hrGivenLogFileHasBadSignature,
            "hrGivenLogFileIsNotContiguous" => NTDSB::hrGivenLogFileIsNotContiguous,
            "hrMissingRestoreLogFiles" => NTDSB::hrMissingRestoreLogFiles,
            "hrMissingFullBackup" => NTDSB::hrMissingFullBackup,
            "hrBadBackupDatabaseSize" => NTDSB::hrBadBackupDatabaseSize,
            "hrTermInProgress" => NTDSB::hrTermInProgress,
            "hrFeatureNotAvailable" => NTDSB::hrFeatureNotAvailable,
            "hrInvalidName" => NTDSB::hrInvalidName,
            "hrInvalidParameter" => NTDSB::hrInvalidParameter,
            "hrInvalidDatabaseId" => NTDSB::hrInvalidDatabaseId,
            "hrOutOfMemory" => NTDSB::hrOutOfMemory,
            "hrOutOfDatabaseSpace" => NTDSB::hrOutOfDatabaseSpace,
            "hrOutOfCursors" => NTDSB::hrOutOfCursors,
            "hrOutOfBuffers" => NTDSB::hrOutOfBuffers,
            "hrTooManyIndexes" => NTDSB::hrTooManyIndexes,
            "hrTooManyKeys" => NTDSB::hrTooManyKeys,
            "hrRecordDeleted" => NTDSB::hrRecordDeleted,
            "hrReadVerifyFailure" => NTDSB::hrReadVerifyFailure,
            "hrOutOfFileHandles" => NTDSB::hrOutOfFileHandles,
            "hrDiskIO" => NTDSB::hrDiskIO,
            "hrInvalidPath" => NTDSB::hrInvalidPath,
            "hrRecordTooBig" => NTDSB::hrRecordTooBig,
            "hrTooManyOpenDatabases" => NTDSB::hrTooManyOpenDatabases,
            "hrInvalidDatabase" => NTDSB::hrInvalidDatabase,
            "hrNotInitialized" => NTDSB::hrNotInitialized,
            "hrAlreadyInitialized" => NTDSB::hrAlreadyInitialized,
            "hrFileAccessDenied" => NTDSB::hrFileAccessDenied,
            "hrBufferTooSmall" => NTDSB::hrBufferTooSmall,
            "hrTooManyColumns" => NTDSB::hrTooManyColumns,
            "hrContainerNotEmpty" => NTDSB::hrContainerNotEmpty,
            "hrInvalidFilename" => NTDSB::hrInvalidFilename,
            "hrInvalidBookmark" => NTDSB::hrInvalidBookmark,
            "hrColumnInUse" => NTDSB::hrColumnInUse,
            "hrInvalidBufferSize" => NTDSB::hrInvalidBufferSize,
            "hrColumnNotUpdatable" => NTDSB::hrColumnNotUpdatable,
            "hrIndexInUse" => NTDSB::hrIndexInUse,
            "hrNullKeyDisallowed" => NTDSB::hrNullKeyDisallowed,
            "hrNotInTransaction" => NTDSB::hrNotInTransaction,
            "hrTooManyActiveUsers" => NTDSB::hrTooManyActiveUsers,
            "hrInvalidCountry" => NTDSB::hrInvalidCountry,
            "hrInvalidLanguageId" => NTDSB::hrInvalidLanguageId,
            "hrInvalidCodePage" => NTDSB::hrInvalidCodePage,
            "hrVersionStoreOutOfMemory" => NTDSB::hrVersionStoreOutOfMemory,
            "hrCurrencyStackOutOfMemory" => NTDSB::hrCurrencyStackOutOfMemory,
            "hrOutOfSessions" => NTDSB::hrOutOfSessions,
            "hrWriteConflict" => NTDSB::hrWriteConflict,
            "hrTransTooDeep" => NTDSB::hrTransTooDeep,
            "hrInvalidSesid" => NTDSB::hrInvalidSesid,
            "hrSessionWriteConflict" => NTDSB::hrSessionWriteConflict,
            "hrInTransaction" => NTDSB::hrInTransaction,
            "hrDatabaseDuplicate" => NTDSB::hrDatabaseDuplicate,
            "hrDatabaseInUse" => NTDSB::hrDatabaseInUse,
            "hrDatabaseNotFound" => NTDSB::hrDatabaseNotFound,
            "hrDatabaseInvalidName" => NTDSB::hrDatabaseInvalidName,
            "hrDatabaseInvalidPages" => NTDSB::hrDatabaseInvalidPages,
            "hrDatabaseCorrupted" => NTDSB::hrDatabaseCorrupted,
            "hrDatabaseLocked" => NTDSB::hrDatabaseLocked,
            "hrTableLocked" => NTDSB::hrTableLocked,
            "hrTableDuplicate" => NTDSB::hrTableDuplicate,
            "hrTableInUse" => NTDSB::hrTableInUse,
            "hrObjectNotFound" => NTDSB::hrObjectNotFound,
            "hrCannotRename" => NTDSB::hrCannotRename,
            "hrDensityInvalid" => NTDSB::hrDensityInvalid,
            "hrTableNotEmpty" => NTDSB::hrTableNotEmpty,
            "hrInvalidTableId" => NTDSB::hrInvalidTableId,
            "hrTooManyOpenTables" => NTDSB::hrTooManyOpenTables,
            "hrIllegalOperation" => NTDSB::hrIllegalOperation,
            "hrObjectDuplicate" => NTDSB::hrObjectDuplicate,
            "hrInvalidObject" => NTDSB::hrInvalidObject,
            "hrIndexCantBuild" => NTDSB::hrIndexCantBuild,
            "hrIndexHasPrimary" => NTDSB::hrIndexHasPrimary,
            "hrIndexDuplicate" => NTDSB::hrIndexDuplicate,
            "hrIndexNotFound" => NTDSB::hrIndexNotFound,
            "hrIndexMustStay" => NTDSB::hrIndexMustStay,
            "hrIndexInvalidDef" => NTDSB::hrIndexInvalidDef,
            "hrIndexHasClustered" => NTDSB::hrIndexHasClustered,
            "hrTooManyOpenIndexes" => NTDSB::hrTooManyOpenIndexes,
            "hrColumnLong" => NTDSB::hrColumnLong,
            "hrColumnDoesNotFit" => NTDSB::hrColumnDoesNotFit,
            "hrNullInvalid" => NTDSB::hrNullInvalid,
            "hrColumnIndexed" => NTDSB::hrColumnIndexed,
            "hrColumnTooBig" => NTDSB::hrColumnTooBig,
            "hrColumnNotFound" => NTDSB::hrColumnNotFound,
            "hrColumnDuplicate" => NTDSB::hrColumnDuplicate,
            "hrColumn2ndSysMaint" => NTDSB::hrColumn2ndSysMaint,
            "hrInvalidColumnType" => NTDSB::hrInvalidColumnType,
            "hrColumnCannotIndex" => NTDSB::hrColumnCannotIndex,
            "hrTaggedNotNULL" => NTDSB::hrTaggedNotNULL,
            "hrNoCurrentIndex" => NTDSB::hrNoCurrentIndex,
            "hrKeyIsMade" => NTDSB::hrKeyIsMade,
            "hrBadColumnId" => NTDSB::hrBadColumnId,
            "hrBadItagSequence" => NTDSB::hrBadItagSequence,
            "hrCannotBeTagged" => NTDSB::hrCannotBeTagged,
            "hrRecordNotFound" => NTDSB::hrRecordNotFound,
            "hrNoCurrentRecord" => NTDSB::hrNoCurrentRecord,
            "hrRecordClusteredChanged" => NTDSB::hrRecordClusteredChanged,
            "hrKeyDuplicate" => NTDSB::hrKeyDuplicate,
            "hrAlreadyPrepared" => NTDSB::hrAlreadyPrepared,
            "hrKeyNotMade" => NTDSB::hrKeyNotMade,
            "hrUpdateNotPrepared" => NTDSB::hrUpdateNotPrepared,
            "hrerrDataHasChanged" => NTDSB::hrerrDataHasChanged,
            "hrTooManySorts" => NTDSB::hrTooManySorts,
            "hrInvalidOnSort" => NTDSB::hrInvalidOnSort,
            "hrTempFileOpenError" => NTDSB::hrTempFileOpenError,
            "hrTooManyAttachedDatabases" => NTDSB::hrTooManyAttachedDatabases,
            "hrDiskFull" => NTDSB::hrDiskFull,
            "hrPermissionDenied" => NTDSB::hrPermissionDenied,
            "hrFileNotFound" => NTDSB::hrFileNotFound,
            "hrAfterInitialization" => NTDSB::hrAfterInitialization,
            "hrLogCorrupted" => NTDSB::hrLogCorrupted,
            "hrInvalidOperation" => NTDSB::hrInvalidOperation,
            "hrAccessDenied" => NTDSB::hrAccessDenied,
        }
    }
    pub fn from_code(code: u32) -> NTDSB {
        return match code {
            0x0800430A => NTDSB::FHCFG_E_INVALID_REHYDRATION_STATE,
            0x880000C8 => NTDSB::hrBFNotSynchronous,
            0x880000C9 => NTDSB::hrBFPageNotFound,
            0x88000141 => NTDSB::hrRemainingVersions,
            0x88000190 => NTDSB::hrFLDKeyTooBig,
            0x88000192 => NTDSB::hrFLDNullKey,
            0x8800022E => NTDSB::hrExistingLogFileHasBadSignature,
            0x8800022F => NTDSB::hrExistingLogFileIsNotContiguous,
            0x880003EC => NTDSB::hrColumnNull,
            0x880003EE => NTDSB::hrBufferTruncated,
            0x880003EF => NTDSB::hrDatabaseAttached,
            0x8800040F => NTDSB::hrSeekNotEqual,
            0x88000422 => NTDSB::hrNoIdleActivity,
            0x8800042B => NTDSB::hrNoWriteLock,
            0x8800042C => NTDSB::hrColumnSetNull,
            0x88000515 => NTDSB::hrTableEmpty,
            0x88000581 => NTDSB::hrCreateIndexFailed,
            0x880005E8 => NTDSB::hrColumnMaxTruncated,
            0x8800064A => NTDSB::hrwrnDataHasChanged,
            0x88000652 => NTDSB::hrKeyChanged,
            0x88000715 => NTDSB::hrFileOpenReadOnly,
            0xC8000066 => NTDSB::hrFileClose,
            0xC8000067 => NTDSB::hrOutOfThreads,
            0xC8000069 => NTDSB::hrTooManyIO,
            0xC80000CA => NTDSB::hrBFInUse,
            0xC800012E => NTDSB::hrPMRecDeleted,
            0xC8000191 => NTDSB::hrFLDTooManySegments,
            0xC80001F5 => NTDSB::hrLogFileCorrupt,
            0xC80001F7 => NTDSB::hrNoBackupDirectory,
            0xC80001F8 => NTDSB::hrBackupDirectoryNotEmpty,
            0xC80001F9 => NTDSB::hrBackupInProgress,
            0xC80001FD => NTDSB::hrMissingPreviousLogFile,
            0xC80001FE => NTDSB::hrLogWriteFail,
            0xC8000202 => NTDSB::hrBadLogVersion,
            0xC8000203 => NTDSB::hrInvalidLogSequence,
            0xC8000204 => NTDSB::hrLoggingDisabled,
            0xC8000205 => NTDSB::hrLogBufferTooSmall,
            0xC8000207 => NTDSB::hrLogSequenceEnd,
            0xC8000208 => NTDSB::hrNoBackup,
            0xC8000209 => NTDSB::hrInvalidBackupSequence,
            0xC800020B => NTDSB::hrBackupNotAllowedYet,
            0xC800020C => NTDSB::hrDeleteBackupFileFail,
            0xC800020D => NTDSB::hrMakeBackupDirectoryFail,
            0xC800020E => NTDSB::hrInvalidBackup,
            0xC800020F => NTDSB::hrRecoveredWithErrors,
            0xC8000210 => NTDSB::hrMissingLogFile,
            0xC8000211 => NTDSB::hrLogDiskFull,
            0xC8000212 => NTDSB::hrBadLogSignature,
            0xC8000213 => NTDSB::hrBadDbSignature,
            0xC8000214 => NTDSB::hrBadCheckpointSignature,
            0xC8000215 => NTDSB::hrCheckpointCorrupt,
            0xC8000226 => NTDSB::hrDatabaseInconsistent,
            0xC8000227 => NTDSB::hrConsistentTimeMismatch,
            0xC8000228 => NTDSB::hrPatchFileMismatch,
            0xC8000229 => NTDSB::hrRestoreLogTooLow,
            0xC800022A => NTDSB::hrRestoreLogTooHigh,
            0xC800022B => NTDSB::hrGivenLogFileHasBadSignature,
            0xC800022C => NTDSB::hrGivenLogFileIsNotContiguous,
            0xC800022D => NTDSB::hrMissingRestoreLogFiles,
            0xC8000230 => NTDSB::hrMissingFullBackup,
            0xC8000231 => NTDSB::hrBadBackupDatabaseSize,
            0xC80003E8 => NTDSB::hrTermInProgress,
            0xC80003E9 => NTDSB::hrFeatureNotAvailable,
            0xC80003EA => NTDSB::hrInvalidName,
            0xC80003EB => NTDSB::hrInvalidParameter,
            0xC80003F2 => NTDSB::hrInvalidDatabaseId,
            0xC80003F3 => NTDSB::hrOutOfMemory,
            0xC80003F4 => NTDSB::hrOutOfDatabaseSpace,
            0xC80003F5 => NTDSB::hrOutOfCursors,
            0xC80003F6 => NTDSB::hrOutOfBuffers,
            0xC80003F7 => NTDSB::hrTooManyIndexes,
            0xC80003F8 => NTDSB::hrTooManyKeys,
            0xC80003F9 => NTDSB::hrRecordDeleted,
            0xC80003FA => NTDSB::hrReadVerifyFailure,
            0xC80003FC => NTDSB::hrOutOfFileHandles,
            0xC80003FE => NTDSB::hrDiskIO,
            0xC80003FF => NTDSB::hrInvalidPath,
            0xC8000402 => NTDSB::hrRecordTooBig,
            0xC8000403 => NTDSB::hrTooManyOpenDatabases,
            0xC8000404 => NTDSB::hrInvalidDatabase,
            0xC8000405 => NTDSB::hrNotInitialized,
            0xC8000406 => NTDSB::hrAlreadyInitialized,
            0xC8000408 => NTDSB::hrFileAccessDenied,
            0xC800040E => NTDSB::hrBufferTooSmall,
            0xC8000410 => NTDSB::hrTooManyColumns,
            0xC8000413 => NTDSB::hrContainerNotEmpty,
            0xC8000414 => NTDSB::hrInvalidFilename,
            0xC8000415 => NTDSB::hrInvalidBookmark,
            0xC8000416 => NTDSB::hrColumnInUse,
            0xC8000417 => NTDSB::hrInvalidBufferSize,
            0xC8000418 => NTDSB::hrColumnNotUpdatable,
            0xC800041B => NTDSB::hrIndexInUse,
            0xC800041D => NTDSB::hrNullKeyDisallowed,
            0xC800041E => NTDSB::hrNotInTransaction,
            0xC8000423 => NTDSB::hrTooManyActiveUsers,
            0xC8000425 => NTDSB::hrInvalidCountry,
            0xC8000426 => NTDSB::hrInvalidLanguageId,
            0xC8000427 => NTDSB::hrInvalidCodePage,
            0xC800042D => NTDSB::hrVersionStoreOutOfMemory,
            0xC800042E => NTDSB::hrCurrencyStackOutOfMemory,
            0xC800044D => NTDSB::hrOutOfSessions,
            0xC800044E => NTDSB::hrWriteConflict,
            0xC800044F => NTDSB::hrTransTooDeep,
            0xC8000450 => NTDSB::hrInvalidSesid,
            0xC8000453 => NTDSB::hrSessionWriteConflict,
            0xC8000454 => NTDSB::hrInTransaction,
            0xC80004B1 => NTDSB::hrDatabaseDuplicate,
            0xC80004B2 => NTDSB::hrDatabaseInUse,
            0xC80004B3 => NTDSB::hrDatabaseNotFound,
            0xC80004B4 => NTDSB::hrDatabaseInvalidName,
            0xC80004B5 => NTDSB::hrDatabaseInvalidPages,
            0xC80004B6 => NTDSB::hrDatabaseCorrupted,
            0xC80004B7 => NTDSB::hrDatabaseLocked,
            0xC8000516 => NTDSB::hrTableLocked,
            0xC8000517 => NTDSB::hrTableDuplicate,
            0xC8000518 => NTDSB::hrTableInUse,
            0xC8000519 => NTDSB::hrObjectNotFound,
            0xC800051A => NTDSB::hrCannotRename,
            0xC800051B => NTDSB::hrDensityInvalid,
            0xC800051C => NTDSB::hrTableNotEmpty,
            0xC800051E => NTDSB::hrInvalidTableId,
            0xC800051F => NTDSB::hrTooManyOpenTables,
            0xC8000520 => NTDSB::hrIllegalOperation,
            0xC8000522 => NTDSB::hrObjectDuplicate,
            0xC8000524 => NTDSB::hrInvalidObject,
            0xC8000579 => NTDSB::hrIndexCantBuild,
            0xC800057A => NTDSB::hrIndexHasPrimary,
            0xC800057B => NTDSB::hrIndexDuplicate,
            0xC800057C => NTDSB::hrIndexNotFound,
            0xC800057D => NTDSB::hrIndexMustStay,
            0xC800057E => NTDSB::hrIndexInvalidDef,
            0xC8000580 => NTDSB::hrIndexHasClustered,
            0xC8000582 => NTDSB::hrTooManyOpenIndexes,
            0xC80005DD => NTDSB::hrColumnLong,
            0xC80005DF => NTDSB::hrColumnDoesNotFit,
            0xC80005E0 => NTDSB::hrNullInvalid,
            0xC80005E1 => NTDSB::hrColumnIndexed,
            0xC80005E2 => NTDSB::hrColumnTooBig,
            0xC80005E3 => NTDSB::hrColumnNotFound,
            0xC80005E4 => NTDSB::hrColumnDuplicate,
            0xC80005E6 => NTDSB::hrColumn2ndSysMaint,
            0xC80005E7 => NTDSB::hrInvalidColumnType,
            0xC80005E9 => NTDSB::hrColumnCannotIndex,
            0xC80005EA => NTDSB::hrTaggedNotNULL,
            0xC80005EB => NTDSB::hrNoCurrentIndex,
            0xC80005EC => NTDSB::hrKeyIsMade,
            0xC80005ED => NTDSB::hrBadColumnId,
            0xC80005EE => NTDSB::hrBadItagSequence,
            0xC80005F1 => NTDSB::hrCannotBeTagged,
            0xC8000641 => NTDSB::hrRecordNotFound,
            0xC8000643 => NTDSB::hrNoCurrentRecord,
            0xC8000644 => NTDSB::hrRecordClusteredChanged,
            0xC8000645 => NTDSB::hrKeyDuplicate,
            0xC8000647 => NTDSB::hrAlreadyPrepared,
            0xC8000648 => NTDSB::hrKeyNotMade,
            0xC8000649 => NTDSB::hrUpdateNotPrepared,
            0xC800064B => NTDSB::hrerrDataHasChanged,
            0xC80006A5 => NTDSB::hrTooManySorts,
            0xC80006A6 => NTDSB::hrInvalidOnSort,
            0xC800070B => NTDSB::hrTempFileOpenError,
            0xC800070D => NTDSB::hrTooManyAttachedDatabases,
            0xC8000710 => NTDSB::hrDiskFull,
            0xC8000711 => NTDSB::hrPermissionDenied,
            0xC8000713 => NTDSB::hrFileNotFound,
            0xC800073A => NTDSB::hrAfterInitialization,
            0xC800073C => NTDSB::hrLogCorrupted,
            0xC8000772 => NTDSB::hrInvalidOperation,
            0xC8000773 => NTDSB::hrAccessDenied,
        }
    }
}
