/*
 * Copyright 2023 Stanislav Mikhailov (xavetar)
 *
 * Licensed under the Creative Commons Zero v1.0 Universal (CC0) License.
 * You may obtain a copy of the License at
 *
 *     http://creativecommons.org/publicdomain/zero/1.0/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the CC0 license is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// It is strictly forbidden to use the from_code method, respected Microsoft decided to define
// duplicate codes, but by defining different constant names, therefore, when using these codes,
// it is mandatory to use through from_name, and not from_code. Otherwise, it may cause undefined
// behavior or an unknown exception. Because one code corresponds to several constants.

use super::{RawError};

#[derive(Clone, Debug, Copy, Eq, Hash, Ord, PartialEq, PartialOrd)]
pub enum OPC {
    OPC_E_NONCONFORMING_URI,
    OPC_E_RELATIVE_URI_REQUIRED,
    OPC_E_RELATIONSHIP_URI_REQUIRED,
    OPC_E_PART_CANNOT_BE_DIRECTORY,
    OPC_E_UNEXPECTED_CONTENT_TYPE,
    OPC_E_INVALID_CONTENT_TYPE_XML,
    OPC_E_MISSING_CONTENT_TYPES,
    OPC_E_NONCONFORMING_CONTENT_TYPES_XML,
    OPC_E_NONCONFORMING_RELS_XML,
    OPC_E_INVALID_RELS_XML,
    OPC_E_DUPLICATE_PART,
    OPC_E_INVALID_OVERRIDE_PART_NAME,
    OPC_E_DUPLICATE_OVERRIDE_PART,
    OPC_E_INVALID_DEFAULT_EXTENSION,
    OPC_E_DUPLICATE_DEFAULT_EXTENSION,
    OPC_E_INVALID_RELATIONSHIP_ID,
    OPC_E_INVALID_RELATIONSHIP_TYPE,
    OPC_E_INVALID_RELATIONSHIP_TARGET,
    OPC_E_DUPLICATE_RELATIONSHIP,
    OPC_E_CONFLICTING_SETTINGS,
    OPC_E_DUPLICATE_PIECE,
    OPC_E_INVALID_PIECE,
    OPC_E_MISSING_PIECE,
    OPC_E_NO_SUCH_PART,
    OPC_E_DS_SIGNATURE_CORRUPT,
    OPC_E_DS_DIGEST_VALUE_ERROR,
    OPC_E_DS_DUPLICATE_SIGNATURE_ORIGIN_RELATIONSHIP,
    OPC_E_DS_INVALID_SIGNATURE_ORIGIN_RELATIONSHIP,
    OPC_E_DS_INVALID_CERTIFICATE_RELATIONSHIP,
    OPC_E_DS_EXTERNAL_SIGNATURE,
    OPC_E_DS_MISSING_SIGNATURE_ORIGIN_PART,
    OPC_E_DS_MISSING_SIGNATURE_PART,
    OPC_E_DS_INVALID_RELATIONSHIP_TRANSFORM_XML,
    OPC_E_DS_INVALID_CANONICALIZATION_METHOD,
    OPC_E_DS_INVALID_RELATIONSHIPS_SIGNING_OPTION,
    OPC_E_DS_INVALID_OPC_SIGNATURE_TIME_FORMAT,
    OPC_E_DS_PACKAGE_REFERENCE_URI_RESERVED,
    OPC_E_DS_MISSING_SIGNATURE_PROPERTIES_ELEMENT,
    OPC_E_DS_MISSING_SIGNATURE_PROPERTY_ELEMENT,
    OPC_E_DS_DUPLICATE_SIGNATURE_PROPERTY_ELEMENT,
    OPC_E_DS_MISSING_SIGNATURE_TIME_PROPERTY,
    OPC_E_DS_INVALID_SIGNATURE_XML,
    OPC_E_DS_INVALID_SIGNATURE_COUNT,
    OPC_E_DS_MISSING_SIGNATURE_ALGORITHM,
    OPC_E_DS_DUPLICATE_PACKAGE_OBJECT_REFERENCES,
    OPC_E_DS_MISSING_PACKAGE_OBJECT_REFERENCE,
    OPC_E_DS_EXTERNAL_SIGNATURE_REFERENCE,
    OPC_E_DS_REFERENCE_MISSING_CONTENT_TYPE,
    OPC_E_DS_MULTIPLE_RELATIONSHIP_TRANSFORMS,
    OPC_E_DS_MISSING_CANONICALIZATION_TRANSFORM,
    OPC_E_MC_UNEXPECTED_ELEMENT,
    OPC_E_MC_UNEXPECTED_REQUIRES_ATTR,
    OPC_E_MC_MISSING_REQUIRES_ATTR,
    OPC_E_MC_UNEXPECTED_ATTR,
    OPC_E_MC_INVALID_PREFIX_LIST,
    OPC_E_MC_INVALID_QNAME_LIST,
    OPC_E_MC_NESTED_ALTERNATE_CONTENT,
    OPC_E_MC_UNEXPECTED_CHOICE,
    OPC_E_MC_MISSING_CHOICE,
    OPC_E_MC_INVALID_ENUM_TYPE,
    OPC_E_MC_UNKNOWN_NAMESPACE,
    OPC_E_MC_UNKNOWN_PREFIX,
    OPC_E_MC_INVALID_ATTRIBUTES_ON_IGNORABLE_ELEMENT,
    OPC_E_MC_INVALID_XMLNS_ATTRIBUTE,
    OPC_E_INVALID_XML_ENCODING,
    OPC_E_DS_SIGNATURE_REFERENCE_MISSING_URI,
    OPC_E_INVALID_CONTENT_TYPE,
    OPC_E_DS_SIGNATURE_PROPERTY_MISSING_TARGET,
    OPC_E_DS_SIGNATURE_METHOD_NOT_SET,
    OPC_E_DS_DEFAULT_DIGEST_METHOD_NOT_SET,
    OPC_E_NO_SUCH_RELATIONSHIP,
    OPC_E_MC_MULTIPLE_FALLBACK_ELEMENTS,
    OPC_E_MC_INCONSISTENT_PROCESS_CONTENT,
    OPC_E_MC_INCONSISTENT_PRESERVE_ATTRIBUTES,
    OPC_E_MC_INCONSISTENT_PRESERVE_ELEMENTS,
    OPC_E_INVALID_RELATIONSHIP_TARGET_MODE,
    OPC_E_COULD_NOT_RECOVER,
    OPC_E_UNSUPPORTED_PACKAGE,
    OPC_E_ENUM_COLLECTION_CHANGED,
    OPC_E_ENUM_CANNOT_MOVE_NEXT,
    OPC_E_ENUM_CANNOT_MOVE_PREVIOUS,
    OPC_E_ENUM_INVALID_POSITION,
    OPC_E_DS_SIGNATURE_ORIGIN_EXISTS,
    OPC_E_DS_UNSIGNED_PACKAGE,
    OPC_E_DS_MISSING_CERTIFICATE_PART,
    OPC_E_ZIP_INCORRECT_DATA_SIZE,
    OPC_E_ZIP_CORRUPTED_ARCHIVE,
    OPC_E_ZIP_COMPRESSION_FAILED,
    OPC_E_ZIP_DECOMPRESSION_FAILED,
    OPC_E_ZIP_INCONSISTENT_FILEITEM,
    OPC_E_ZIP_INCONSISTENT_DIRECTORY,
    OPC_E_ZIP_MISSING_DATA_DESCRIPTOR,
    OPC_E_ZIP_UNSUPPORTEDARCHIVE,
    OPC_E_ZIP_CENTRAL_DIRECTORY_TOO_LARGE,
    OPC_E_ZIP_NAME_TOO_LARGE,
    OPC_E_ZIP_DUPLICATE_NAME,
    OPC_E_ZIP_COMMENT_TOO_LARGE,
    OPC_E_ZIP_EXTRA_FIELDS_TOO_LARGE,
    OPC_E_ZIP_FILE_HEADER_TOO_LARGE,
    OPC_E_ZIP_MISSING_END_OF_CENTRAL_DIRECTORY,
    OPC_E_ZIP_REQUIRES_64_BIT,
}

impl OPC {
    pub fn code(&self) -> u32 {
        return match self {
            OPC::OPC_E_NONCONFORMING_URI => 0x80510001 as u32,
            OPC::OPC_E_RELATIVE_URI_REQUIRED => 0x80510002 as u32,
            OPC::OPC_E_RELATIONSHIP_URI_REQUIRED => 0x80510003 as u32,
            OPC::OPC_E_PART_CANNOT_BE_DIRECTORY => 0x80510004 as u32,
            OPC::OPC_E_UNEXPECTED_CONTENT_TYPE => 0x80510005 as u32,
            OPC::OPC_E_INVALID_CONTENT_TYPE_XML => 0x80510006 as u32,
            OPC::OPC_E_MISSING_CONTENT_TYPES => 0x80510007 as u32,
            OPC::OPC_E_NONCONFORMING_CONTENT_TYPES_XML => 0x80510008 as u32,
            OPC::OPC_E_NONCONFORMING_RELS_XML => 0x80510009 as u32,
            OPC::OPC_E_INVALID_RELS_XML => 0x8051000A as u32,
            OPC::OPC_E_DUPLICATE_PART => 0x8051000B as u32,
            OPC::OPC_E_INVALID_OVERRIDE_PART_NAME => 0x8051000C as u32,
            OPC::OPC_E_DUPLICATE_OVERRIDE_PART => 0x8051000D as u32,
            OPC::OPC_E_INVALID_DEFAULT_EXTENSION => 0x8051000E as u32,
            OPC::OPC_E_DUPLICATE_DEFAULT_EXTENSION => 0x8051000F as u32,
            OPC::OPC_E_INVALID_RELATIONSHIP_ID => 0x80510010 as u32,
            OPC::OPC_E_INVALID_RELATIONSHIP_TYPE => 0x80510011 as u32,
            OPC::OPC_E_INVALID_RELATIONSHIP_TARGET => 0x80510012 as u32,
            OPC::OPC_E_DUPLICATE_RELATIONSHIP => 0x80510013 as u32,
            OPC::OPC_E_CONFLICTING_SETTINGS => 0x80510014 as u32,
            OPC::OPC_E_DUPLICATE_PIECE => 0x80510015 as u32,
            OPC::OPC_E_INVALID_PIECE => 0x80510016 as u32,
            OPC::OPC_E_MISSING_PIECE => 0x80510017 as u32,
            OPC::OPC_E_NO_SUCH_PART => 0x80510018 as u32,
            OPC::OPC_E_DS_SIGNATURE_CORRUPT => 0x80510019 as u32,
            OPC::OPC_E_DS_DIGEST_VALUE_ERROR => 0x8051001A as u32,
            OPC::OPC_E_DS_DUPLICATE_SIGNATURE_ORIGIN_RELATIONSHIP => 0x8051001B as u32,
            OPC::OPC_E_DS_INVALID_SIGNATURE_ORIGIN_RELATIONSHIP => 0x8051001C as u32,
            OPC::OPC_E_DS_INVALID_CERTIFICATE_RELATIONSHIP => 0x8051001D as u32,
            OPC::OPC_E_DS_EXTERNAL_SIGNATURE => 0x8051001E as u32,
            OPC::OPC_E_DS_MISSING_SIGNATURE_ORIGIN_PART => 0x8051001F as u32,
            OPC::OPC_E_DS_MISSING_SIGNATURE_PART => 0x80510020 as u32,
            OPC::OPC_E_DS_INVALID_RELATIONSHIP_TRANSFORM_XML => 0x80510021 as u32,
            OPC::OPC_E_DS_INVALID_CANONICALIZATION_METHOD => 0x80510022 as u32,
            OPC::OPC_E_DS_INVALID_RELATIONSHIPS_SIGNING_OPTION => 0x80510023 as u32,
            OPC::OPC_E_DS_INVALID_OPC_SIGNATURE_TIME_FORMAT => 0x80510024 as u32,
            OPC::OPC_E_DS_PACKAGE_REFERENCE_URI_RESERVED => 0x80510025 as u32,
            OPC::OPC_E_DS_MISSING_SIGNATURE_PROPERTIES_ELEMENT => 0x80510026 as u32,
            OPC::OPC_E_DS_MISSING_SIGNATURE_PROPERTY_ELEMENT => 0x80510027 as u32,
            OPC::OPC_E_DS_DUPLICATE_SIGNATURE_PROPERTY_ELEMENT => 0x80510028 as u32,
            OPC::OPC_E_DS_MISSING_SIGNATURE_TIME_PROPERTY => 0x80510029 as u32,
            OPC::OPC_E_DS_INVALID_SIGNATURE_XML => 0x8051002A as u32,
            OPC::OPC_E_DS_INVALID_SIGNATURE_COUNT => 0x8051002B as u32,
            OPC::OPC_E_DS_MISSING_SIGNATURE_ALGORITHM => 0x8051002C as u32,
            OPC::OPC_E_DS_DUPLICATE_PACKAGE_OBJECT_REFERENCES => 0x8051002D as u32,
            OPC::OPC_E_DS_MISSING_PACKAGE_OBJECT_REFERENCE => 0x8051002E as u32,
            OPC::OPC_E_DS_EXTERNAL_SIGNATURE_REFERENCE => 0x8051002F as u32,
            OPC::OPC_E_DS_REFERENCE_MISSING_CONTENT_TYPE => 0x80510030 as u32,
            OPC::OPC_E_DS_MULTIPLE_RELATIONSHIP_TRANSFORMS => 0x80510031 as u32,
            OPC::OPC_E_DS_MISSING_CANONICALIZATION_TRANSFORM => 0x80510032 as u32,
            OPC::OPC_E_MC_UNEXPECTED_ELEMENT => 0x80510033 as u32,
            OPC::OPC_E_MC_UNEXPECTED_REQUIRES_ATTR => 0x80510034 as u32,
            OPC::OPC_E_MC_MISSING_REQUIRES_ATTR => 0x80510035 as u32,
            OPC::OPC_E_MC_UNEXPECTED_ATTR => 0x80510036 as u32,
            OPC::OPC_E_MC_INVALID_PREFIX_LIST => 0x80510037 as u32,
            OPC::OPC_E_MC_INVALID_QNAME_LIST => 0x80510038 as u32,
            OPC::OPC_E_MC_NESTED_ALTERNATE_CONTENT => 0x80510039 as u32,
            OPC::OPC_E_MC_UNEXPECTED_CHOICE => 0x8051003A as u32,
            OPC::OPC_E_MC_MISSING_CHOICE => 0x8051003B as u32,
            OPC::OPC_E_MC_INVALID_ENUM_TYPE => 0x8051003C as u32,
            OPC::OPC_E_MC_UNKNOWN_NAMESPACE => 0x8051003E as u32,
            OPC::OPC_E_MC_UNKNOWN_PREFIX => 0x8051003F as u32,
            OPC::OPC_E_MC_INVALID_ATTRIBUTES_ON_IGNORABLE_ELEMENT => 0x80510040 as u32,
            OPC::OPC_E_MC_INVALID_XMLNS_ATTRIBUTE => 0x80510041 as u32,
            OPC::OPC_E_INVALID_XML_ENCODING => 0x80510042 as u32,
            OPC::OPC_E_DS_SIGNATURE_REFERENCE_MISSING_URI => 0x80510043 as u32,
            OPC::OPC_E_INVALID_CONTENT_TYPE => 0x80510044 as u32,
            OPC::OPC_E_DS_SIGNATURE_PROPERTY_MISSING_TARGET => 0x80510045 as u32,
            OPC::OPC_E_DS_SIGNATURE_METHOD_NOT_SET => 0x80510046 as u32,
            OPC::OPC_E_DS_DEFAULT_DIGEST_METHOD_NOT_SET => 0x80510047 as u32,
            OPC::OPC_E_NO_SUCH_RELATIONSHIP => 0x80510048 as u32,
            OPC::OPC_E_MC_MULTIPLE_FALLBACK_ELEMENTS => 0x80510049 as u32,
            OPC::OPC_E_MC_INCONSISTENT_PROCESS_CONTENT => 0x8051004A as u32,
            OPC::OPC_E_MC_INCONSISTENT_PRESERVE_ATTRIBUTES => 0x8051004B as u32,
            OPC::OPC_E_MC_INCONSISTENT_PRESERVE_ELEMENTS => 0x8051004C as u32,
            OPC::OPC_E_INVALID_RELATIONSHIP_TARGET_MODE => 0x8051004D as u32,
            OPC::OPC_E_COULD_NOT_RECOVER => 0x8051004E as u32,
            OPC::OPC_E_UNSUPPORTED_PACKAGE => 0x8051004F as u32,
            OPC::OPC_E_ENUM_COLLECTION_CHANGED => 0x80510050 as u32,
            OPC::OPC_E_ENUM_CANNOT_MOVE_NEXT => 0x80510051 as u32,
            OPC::OPC_E_ENUM_CANNOT_MOVE_PREVIOUS => 0x80510052 as u32,
            OPC::OPC_E_ENUM_INVALID_POSITION => 0x80510053 as u32,
            OPC::OPC_E_DS_SIGNATURE_ORIGIN_EXISTS => 0x80510054 as u32,
            OPC::OPC_E_DS_UNSIGNED_PACKAGE => 0x80510055 as u32,
            OPC::OPC_E_DS_MISSING_CERTIFICATE_PART => 0x80510056 as u32,
            OPC::OPC_E_ZIP_INCORRECT_DATA_SIZE => 0x80511001 as u32,
            OPC::OPC_E_ZIP_CORRUPTED_ARCHIVE => 0x80511002 as u32,
            OPC::OPC_E_ZIP_COMPRESSION_FAILED => 0x80511003 as u32,
            OPC::OPC_E_ZIP_DECOMPRESSION_FAILED => 0x80511004 as u32,
            OPC::OPC_E_ZIP_INCONSISTENT_FILEITEM => 0x80511005 as u32,
            OPC::OPC_E_ZIP_INCONSISTENT_DIRECTORY => 0x80511006 as u32,
            OPC::OPC_E_ZIP_MISSING_DATA_DESCRIPTOR => 0x80511007 as u32,
            OPC::OPC_E_ZIP_UNSUPPORTEDARCHIVE => 0x80511008 as u32,
            OPC::OPC_E_ZIP_CENTRAL_DIRECTORY_TOO_LARGE => 0x80511009 as u32,
            OPC::OPC_E_ZIP_NAME_TOO_LARGE => 0x8051100A as u32,
            OPC::OPC_E_ZIP_DUPLICATE_NAME => 0x8051100B as u32,
            OPC::OPC_E_ZIP_COMMENT_TOO_LARGE => 0x8051100C as u32,
            OPC::OPC_E_ZIP_EXTRA_FIELDS_TOO_LARGE => 0x8051100D as u32,
            OPC::OPC_E_ZIP_FILE_HEADER_TOO_LARGE => 0x8051100E as u32,
            OPC::OPC_E_ZIP_MISSING_END_OF_CENTRAL_DIRECTORY => 0x8051100F as u32,
            OPC::OPC_E_ZIP_REQUIRES_64_BIT => 0x80511010 as u32,
        }
    }

    pub fn error(&self) -> RawError {
        return match self {
            OPC::OPC_E_NONCONFORMING_URI => RawError::Kind(OPC::OPC_E_NONCONFORMING_URI),
            OPC::OPC_E_RELATIVE_URI_REQUIRED => RawError::Kind(OPC::OPC_E_RELATIVE_URI_REQUIRED),
            OPC::OPC_E_RELATIONSHIP_URI_REQUIRED => RawError::Kind(OPC::OPC_E_RELATIONSHIP_URI_REQUIRED),
            OPC::OPC_E_PART_CANNOT_BE_DIRECTORY => RawError::Kind(OPC::OPC_E_PART_CANNOT_BE_DIRECTORY),
            OPC::OPC_E_UNEXPECTED_CONTENT_TYPE => RawError::Kind(OPC::OPC_E_UNEXPECTED_CONTENT_TYPE),
            OPC::OPC_E_INVALID_CONTENT_TYPE_XML => RawError::Kind(OPC::OPC_E_INVALID_CONTENT_TYPE_XML),
            OPC::OPC_E_MISSING_CONTENT_TYPES => RawError::Kind(OPC::OPC_E_MISSING_CONTENT_TYPES),
            OPC::OPC_E_NONCONFORMING_CONTENT_TYPES_XML => RawError::Kind(OPC::OPC_E_NONCONFORMING_CONTENT_TYPES_XML),
            OPC::OPC_E_NONCONFORMING_RELS_XML => RawError::Kind(OPC::OPC_E_NONCONFORMING_RELS_XML),
            OPC::OPC_E_INVALID_RELS_XML => RawError::Kind(OPC::OPC_E_INVALID_RELS_XML),
            OPC::OPC_E_DUPLICATE_PART => RawError::Kind(OPC::OPC_E_DUPLICATE_PART),
            OPC::OPC_E_INVALID_OVERRIDE_PART_NAME => RawError::Kind(OPC::OPC_E_INVALID_OVERRIDE_PART_NAME),
            OPC::OPC_E_DUPLICATE_OVERRIDE_PART => RawError::Kind(OPC::OPC_E_DUPLICATE_OVERRIDE_PART),
            OPC::OPC_E_INVALID_DEFAULT_EXTENSION => RawError::Kind(OPC::OPC_E_INVALID_DEFAULT_EXTENSION),
            OPC::OPC_E_DUPLICATE_DEFAULT_EXTENSION => RawError::Kind(OPC::OPC_E_DUPLICATE_DEFAULT_EXTENSION),
            OPC::OPC_E_INVALID_RELATIONSHIP_ID => RawError::Kind(OPC::OPC_E_INVALID_RELATIONSHIP_ID),
            OPC::OPC_E_INVALID_RELATIONSHIP_TYPE => RawError::Kind(OPC::OPC_E_INVALID_RELATIONSHIP_TYPE),
            OPC::OPC_E_INVALID_RELATIONSHIP_TARGET => RawError::Kind(OPC::OPC_E_INVALID_RELATIONSHIP_TARGET),
            OPC::OPC_E_DUPLICATE_RELATIONSHIP => RawError::Kind(OPC::OPC_E_DUPLICATE_RELATIONSHIP),
            OPC::OPC_E_CONFLICTING_SETTINGS => RawError::Kind(OPC::OPC_E_CONFLICTING_SETTINGS),
            OPC::OPC_E_DUPLICATE_PIECE => RawError::Kind(OPC::OPC_E_DUPLICATE_PIECE),
            OPC::OPC_E_INVALID_PIECE => RawError::Kind(OPC::OPC_E_INVALID_PIECE),
            OPC::OPC_E_MISSING_PIECE => RawError::Kind(OPC::OPC_E_MISSING_PIECE),
            OPC::OPC_E_NO_SUCH_PART => RawError::Kind(OPC::OPC_E_NO_SUCH_PART),
            OPC::OPC_E_DS_SIGNATURE_CORRUPT => RawError::Kind(OPC::OPC_E_DS_SIGNATURE_CORRUPT),
            OPC::OPC_E_DS_DIGEST_VALUE_ERROR => RawError::Kind(OPC::OPC_E_DS_DIGEST_VALUE_ERROR),
            OPC::OPC_E_DS_DUPLICATE_SIGNATURE_ORIGIN_RELATIONSHIP => RawError::Kind(OPC::OPC_E_DS_DUPLICATE_SIGNATURE_ORIGIN_RELATIONSHIP),
            OPC::OPC_E_DS_INVALID_SIGNATURE_ORIGIN_RELATIONSHIP => RawError::Kind(OPC::OPC_E_DS_INVALID_SIGNATURE_ORIGIN_RELATIONSHIP),
            OPC::OPC_E_DS_INVALID_CERTIFICATE_RELATIONSHIP => RawError::Kind(OPC::OPC_E_DS_INVALID_CERTIFICATE_RELATIONSHIP),
            OPC::OPC_E_DS_EXTERNAL_SIGNATURE => RawError::Kind(OPC::OPC_E_DS_EXTERNAL_SIGNATURE),
            OPC::OPC_E_DS_MISSING_SIGNATURE_ORIGIN_PART => RawError::Kind(OPC::OPC_E_DS_MISSING_SIGNATURE_ORIGIN_PART),
            OPC::OPC_E_DS_MISSING_SIGNATURE_PART => RawError::Kind(OPC::OPC_E_DS_MISSING_SIGNATURE_PART),
            OPC::OPC_E_DS_INVALID_RELATIONSHIP_TRANSFORM_XML => RawError::Kind(OPC::OPC_E_DS_INVALID_RELATIONSHIP_TRANSFORM_XML),
            OPC::OPC_E_DS_INVALID_CANONICALIZATION_METHOD => RawError::Kind(OPC::OPC_E_DS_INVALID_CANONICALIZATION_METHOD),
            OPC::OPC_E_DS_INVALID_RELATIONSHIPS_SIGNING_OPTION => RawError::Kind(OPC::OPC_E_DS_INVALID_RELATIONSHIPS_SIGNING_OPTION),
            OPC::OPC_E_DS_INVALID_OPC_SIGNATURE_TIME_FORMAT => RawError::Kind(OPC::OPC_E_DS_INVALID_OPC_SIGNATURE_TIME_FORMAT),
            OPC::OPC_E_DS_PACKAGE_REFERENCE_URI_RESERVED => RawError::Kind(OPC::OPC_E_DS_PACKAGE_REFERENCE_URI_RESERVED),
            OPC::OPC_E_DS_MISSING_SIGNATURE_PROPERTIES_ELEMENT => RawError::Kind(OPC::OPC_E_DS_MISSING_SIGNATURE_PROPERTIES_ELEMENT),
            OPC::OPC_E_DS_MISSING_SIGNATURE_PROPERTY_ELEMENT => RawError::Kind(OPC::OPC_E_DS_MISSING_SIGNATURE_PROPERTY_ELEMENT),
            OPC::OPC_E_DS_DUPLICATE_SIGNATURE_PROPERTY_ELEMENT => RawError::Kind(OPC::OPC_E_DS_DUPLICATE_SIGNATURE_PROPERTY_ELEMENT),
            OPC::OPC_E_DS_MISSING_SIGNATURE_TIME_PROPERTY => RawError::Kind(OPC::OPC_E_DS_MISSING_SIGNATURE_TIME_PROPERTY),
            OPC::OPC_E_DS_INVALID_SIGNATURE_XML => RawError::Kind(OPC::OPC_E_DS_INVALID_SIGNATURE_XML),
            OPC::OPC_E_DS_INVALID_SIGNATURE_COUNT => RawError::Kind(OPC::OPC_E_DS_INVALID_SIGNATURE_COUNT),
            OPC::OPC_E_DS_MISSING_SIGNATURE_ALGORITHM => RawError::Kind(OPC::OPC_E_DS_MISSING_SIGNATURE_ALGORITHM),
            OPC::OPC_E_DS_DUPLICATE_PACKAGE_OBJECT_REFERENCES => RawError::Kind(OPC::OPC_E_DS_DUPLICATE_PACKAGE_OBJECT_REFERENCES),
            OPC::OPC_E_DS_MISSING_PACKAGE_OBJECT_REFERENCE => RawError::Kind(OPC::OPC_E_DS_MISSING_PACKAGE_OBJECT_REFERENCE),
            OPC::OPC_E_DS_EXTERNAL_SIGNATURE_REFERENCE => RawError::Kind(OPC::OPC_E_DS_EXTERNAL_SIGNATURE_REFERENCE),
            OPC::OPC_E_DS_REFERENCE_MISSING_CONTENT_TYPE => RawError::Kind(OPC::OPC_E_DS_REFERENCE_MISSING_CONTENT_TYPE),
            OPC::OPC_E_DS_MULTIPLE_RELATIONSHIP_TRANSFORMS => RawError::Kind(OPC::OPC_E_DS_MULTIPLE_RELATIONSHIP_TRANSFORMS),
            OPC::OPC_E_DS_MISSING_CANONICALIZATION_TRANSFORM => RawError::Kind(OPC::OPC_E_DS_MISSING_CANONICALIZATION_TRANSFORM),
            OPC::OPC_E_MC_UNEXPECTED_ELEMENT => RawError::Kind(OPC::OPC_E_MC_UNEXPECTED_ELEMENT),
            OPC::OPC_E_MC_UNEXPECTED_REQUIRES_ATTR => RawError::Kind(OPC::OPC_E_MC_UNEXPECTED_REQUIRES_ATTR),
            OPC::OPC_E_MC_MISSING_REQUIRES_ATTR => RawError::Kind(OPC::OPC_E_MC_MISSING_REQUIRES_ATTR),
            OPC::OPC_E_MC_UNEXPECTED_ATTR => RawError::Kind(OPC::OPC_E_MC_UNEXPECTED_ATTR),
            OPC::OPC_E_MC_INVALID_PREFIX_LIST => RawError::Kind(OPC::OPC_E_MC_INVALID_PREFIX_LIST),
            OPC::OPC_E_MC_INVALID_QNAME_LIST => RawError::Kind(OPC::OPC_E_MC_INVALID_QNAME_LIST),
            OPC::OPC_E_MC_NESTED_ALTERNATE_CONTENT => RawError::Kind(OPC::OPC_E_MC_NESTED_ALTERNATE_CONTENT),
            OPC::OPC_E_MC_UNEXPECTED_CHOICE => RawError::Kind(OPC::OPC_E_MC_UNEXPECTED_CHOICE),
            OPC::OPC_E_MC_MISSING_CHOICE => RawError::Kind(OPC::OPC_E_MC_MISSING_CHOICE),
            OPC::OPC_E_MC_INVALID_ENUM_TYPE => RawError::Kind(OPC::OPC_E_MC_INVALID_ENUM_TYPE),
            OPC::OPC_E_MC_UNKNOWN_NAMESPACE => RawError::Kind(OPC::OPC_E_MC_UNKNOWN_NAMESPACE),
            OPC::OPC_E_MC_UNKNOWN_PREFIX => RawError::Kind(OPC::OPC_E_MC_UNKNOWN_PREFIX),
            OPC::OPC_E_MC_INVALID_ATTRIBUTES_ON_IGNORABLE_ELEMENT => RawError::Kind(OPC::OPC_E_MC_INVALID_ATTRIBUTES_ON_IGNORABLE_ELEMENT),
            OPC::OPC_E_MC_INVALID_XMLNS_ATTRIBUTE => RawError::Kind(OPC::OPC_E_MC_INVALID_XMLNS_ATTRIBUTE),
            OPC::OPC_E_INVALID_XML_ENCODING => RawError::Kind(OPC::OPC_E_INVALID_XML_ENCODING),
            OPC::OPC_E_DS_SIGNATURE_REFERENCE_MISSING_URI => RawError::Kind(OPC::OPC_E_DS_SIGNATURE_REFERENCE_MISSING_URI),
            OPC::OPC_E_INVALID_CONTENT_TYPE => RawError::Kind(OPC::OPC_E_INVALID_CONTENT_TYPE),
            OPC::OPC_E_DS_SIGNATURE_PROPERTY_MISSING_TARGET => RawError::Kind(OPC::OPC_E_DS_SIGNATURE_PROPERTY_MISSING_TARGET),
            OPC::OPC_E_DS_SIGNATURE_METHOD_NOT_SET => RawError::Kind(OPC::OPC_E_DS_SIGNATURE_METHOD_NOT_SET),
            OPC::OPC_E_DS_DEFAULT_DIGEST_METHOD_NOT_SET => RawError::Kind(OPC::OPC_E_DS_DEFAULT_DIGEST_METHOD_NOT_SET),
            OPC::OPC_E_NO_SUCH_RELATIONSHIP => RawError::Kind(OPC::OPC_E_NO_SUCH_RELATIONSHIP),
            OPC::OPC_E_MC_MULTIPLE_FALLBACK_ELEMENTS => RawError::Kind(OPC::OPC_E_MC_MULTIPLE_FALLBACK_ELEMENTS),
            OPC::OPC_E_MC_INCONSISTENT_PROCESS_CONTENT => RawError::Kind(OPC::OPC_E_MC_INCONSISTENT_PROCESS_CONTENT),
            OPC::OPC_E_MC_INCONSISTENT_PRESERVE_ATTRIBUTES => RawError::Kind(OPC::OPC_E_MC_INCONSISTENT_PRESERVE_ATTRIBUTES),
            OPC::OPC_E_MC_INCONSISTENT_PRESERVE_ELEMENTS => RawError::Kind(OPC::OPC_E_MC_INCONSISTENT_PRESERVE_ELEMENTS),
            OPC::OPC_E_INVALID_RELATIONSHIP_TARGET_MODE => RawError::Kind(OPC::OPC_E_INVALID_RELATIONSHIP_TARGET_MODE),
            OPC::OPC_E_COULD_NOT_RECOVER => RawError::Kind(OPC::OPC_E_COULD_NOT_RECOVER),
            OPC::OPC_E_UNSUPPORTED_PACKAGE => RawError::Kind(OPC::OPC_E_UNSUPPORTED_PACKAGE),
            OPC::OPC_E_ENUM_COLLECTION_CHANGED => RawError::Kind(OPC::OPC_E_ENUM_COLLECTION_CHANGED),
            OPC::OPC_E_ENUM_CANNOT_MOVE_NEXT => RawError::Kind(OPC::OPC_E_ENUM_CANNOT_MOVE_NEXT),
            OPC::OPC_E_ENUM_CANNOT_MOVE_PREVIOUS => RawError::Kind(OPC::OPC_E_ENUM_CANNOT_MOVE_PREVIOUS),
            OPC::OPC_E_ENUM_INVALID_POSITION => RawError::Kind(OPC::OPC_E_ENUM_INVALID_POSITION),
            OPC::OPC_E_DS_SIGNATURE_ORIGIN_EXISTS => RawError::Kind(OPC::OPC_E_DS_SIGNATURE_ORIGIN_EXISTS),
            OPC::OPC_E_DS_UNSIGNED_PACKAGE => RawError::Kind(OPC::OPC_E_DS_UNSIGNED_PACKAGE),
            OPC::OPC_E_DS_MISSING_CERTIFICATE_PART => RawError::Kind(OPC::OPC_E_DS_MISSING_CERTIFICATE_PART),
            OPC::OPC_E_ZIP_INCORRECT_DATA_SIZE => RawError::Kind(OPC::OPC_E_ZIP_INCORRECT_DATA_SIZE),
            OPC::OPC_E_ZIP_CORRUPTED_ARCHIVE => RawError::Kind(OPC::OPC_E_ZIP_CORRUPTED_ARCHIVE),
            OPC::OPC_E_ZIP_COMPRESSION_FAILED => RawError::Kind(OPC::OPC_E_ZIP_COMPRESSION_FAILED),
            OPC::OPC_E_ZIP_DECOMPRESSION_FAILED => RawError::Kind(OPC::OPC_E_ZIP_DECOMPRESSION_FAILED),
            OPC::OPC_E_ZIP_INCONSISTENT_FILEITEM => RawError::Kind(OPC::OPC_E_ZIP_INCONSISTENT_FILEITEM),
            OPC::OPC_E_ZIP_INCONSISTENT_DIRECTORY => RawError::Kind(OPC::OPC_E_ZIP_INCONSISTENT_DIRECTORY),
            OPC::OPC_E_ZIP_MISSING_DATA_DESCRIPTOR => RawError::Kind(OPC::OPC_E_ZIP_MISSING_DATA_DESCRIPTOR),
            OPC::OPC_E_ZIP_UNSUPPORTEDARCHIVE => RawError::Kind(OPC::OPC_E_ZIP_UNSUPPORTEDARCHIVE),
            OPC::OPC_E_ZIP_CENTRAL_DIRECTORY_TOO_LARGE => RawError::Kind(OPC::OPC_E_ZIP_CENTRAL_DIRECTORY_TOO_LARGE),
            OPC::OPC_E_ZIP_NAME_TOO_LARGE => RawError::Kind(OPC::OPC_E_ZIP_NAME_TOO_LARGE),
            OPC::OPC_E_ZIP_DUPLICATE_NAME => RawError::Kind(OPC::OPC_E_ZIP_DUPLICATE_NAME),
            OPC::OPC_E_ZIP_COMMENT_TOO_LARGE => RawError::Kind(OPC::OPC_E_ZIP_COMMENT_TOO_LARGE),
            OPC::OPC_E_ZIP_EXTRA_FIELDS_TOO_LARGE => RawError::Kind(OPC::OPC_E_ZIP_EXTRA_FIELDS_TOO_LARGE),
            OPC::OPC_E_ZIP_FILE_HEADER_TOO_LARGE => RawError::Kind(OPC::OPC_E_ZIP_FILE_HEADER_TOO_LARGE),
            OPC::OPC_E_ZIP_MISSING_END_OF_CENTRAL_DIRECTORY => RawError::Kind(OPC::OPC_E_ZIP_MISSING_END_OF_CENTRAL_DIRECTORY),
            OPC::OPC_E_ZIP_REQUIRES_64_BIT => RawError::Kind(OPC::OPC_E_ZIP_REQUIRES_64_BIT),
        }
    }

    pub fn description(&self) -> &'static str {
        return match self {
            OPC::OPC_E_NONCONFORMING_URI => "The part name does not conform to the rules specified in the OPC.",
            OPC::OPC_E_RELATIVE_URI_REQUIRED => "A part name cannot be an absolute URI. An absolute URI begins with a schema component followed by a ':', as described in RFC 3986: URI Generic Syntax.",
            OPC::OPC_E_RELATIONSHIP_URI_REQUIRED => "The part name of a Relationships part is required, but the part name is not that of a Relationships part. For more information about the part names of Relationships parts, see the OPC.",
            OPC::OPC_E_PART_CANNOT_BE_DIRECTORY => "The part name of one part is a segment in the path of the part name of another part. An example would be a part named 'path/dirorfile/file.txt' and a part named 'path/dirorfile'.",
            OPC::OPC_E_UNEXPECTED_CONTENT_TYPE => "Either the content type of a part differed from the expected content type (specified in the OPC, ECMA-376 Part 2), or the part content did not match the part's content type.",
            OPC::OPC_E_INVALID_CONTENT_TYPE_XML => "The XML markup that defines package content types in the Content Types stream is not well-formed.",
            OPC::OPC_E_MISSING_CONTENT_TYPES => "Could not find the part that contains Content Types stream.",
            OPC::OPC_E_NONCONFORMING_CONTENT_TYPES_XML => "The Content Types stream XML markup does not conform to the syntax specified in RFC 2616: HTTP/1.1 and the OPC.",
            OPC::OPC_E_NONCONFORMING_RELS_XML => "The relationship markup does not conform to the rules specified in the OPC.",
            OPC::OPC_E_INVALID_RELS_XML => "The relationship markup for a Relationships part is not well-formed XML.",
            OPC::OPC_E_DUPLICATE_PART => "A part with the specified part name already exists in the current package.",
            OPC::OPC_E_INVALID_OVERRIDE_PART_NAME => "The part name that is used as the value of the PartName attribute of an Override element in the Content Types stream does not conform to the rules for a valid part name, specified in RFC 3986: URI Generic Syntax (http://go.microsoft.com/fwlink/p/?linkid=143950) and the OPC. For more information about Override elements, see the OPC.",
            OPC::OPC_E_DUPLICATE_OVERRIDE_PART => "More than one Override element in the Content Types stream specifies the same part name as the value of its PartName attribute. If this error is encountered while a package is being read, the package is not valid, because the PartName attribute value for each Override element must be unique. For more information about Override elements, see the OPC.",
            OPC::OPC_E_INVALID_DEFAULT_EXTENSION => "An extension that is provided for a default content type is not valid, because it does not conform to the rules for a part name, specified in the OPC.",
            OPC::OPC_E_DUPLICATE_DEFAULT_EXTENSION => "More than one Default element specifies the same extension in the content type XML markup. For more information about Default elements and content type XML markup, see the OPC.",
            OPC::OPC_E_INVALID_RELATIONSHIP_ID => "The Id attribute of a relationship does not conform to the rules specified in the OPC.",
            OPC::OPC_E_INVALID_RELATIONSHIP_TYPE => "The Type attribute of a relationship does not conform to the rules specified in the OPC.",
            OPC::OPC_E_INVALID_RELATIONSHIP_TARGET => "The Target attribute of a relationship does not conform to the rules specified in the OPC.",
            OPC::OPC_E_DUPLICATE_RELATIONSHIP => "A relationship with the same identifier already exists in the current package.",
            OPC::OPC_E_CONFLICTING_SETTINGS => "The selected package creation settings are inconsistent; an example would be creating a package that uses both Force 32-bit mode and Force 64-bit mode.",
            OPC::OPC_E_DUPLICATE_PIECE => "A piece with the same piece number already exists in the current package. Pieces are a result of interleaved parts. If the Packaging API is used to consume a package that supports interleaving, this error may result. The Packaging API does not support the production of packages that have interleaved parts. For more information about interleaving, see the OPC.",
            OPC::OPC_E_INVALID_PIECE => "An unexpected piece was found. Pieces are a result of interleaved parts. If the Packaging API is used to consume a package that has interleaved parts, this error may result. The Packaging API does not support the production of packages that have interleaved parts. For more information about interleaving, see the OPC.",
            OPC::OPC_E_MISSING_PIECE => "One or more pieces of an interleaved part were not found. A piece is a file that contains data for an interleaved part. This error may result if the Packaging APIs are used to consume a package that has interleaved parts. Production of packages that have interleaved parts is not supported by the Packaging APIs. For more information about interleaving, see the OPC.",
            OPC::OPC_E_NO_SUCH_PART => "The specified part does not exist.",
            OPC::OPC_E_DS_SIGNATURE_CORRUPT => "A signature in the package is not properly formed. Cannot get the signature value.",
            OPC::OPC_E_DS_DIGEST_VALUE_ERROR => "Cannot get the digest value of a package component or an element in the signature markup that was referenced for signing.",
            OPC::OPC_E_DS_DUPLICATE_SIGNATURE_ORIGIN_RELATIONSHIP => "More than one relationship of the digital signature origin relationship type exists, but only one such relationship is allowed. For more information about this relationship type, see the OPC.",
            OPC::OPC_E_DS_INVALID_SIGNATURE_ORIGIN_RELATIONSHIP => "A package relationship of type digital signature origin is targeting a location that is external to the package. Digital Signature Origin parts must be located internally. For more information about this relationship type, see the OPC.",
            OPC::OPC_E_DS_INVALID_CERTIFICATE_RELATIONSHIP => "A relationship of type digital signature certificate has the external target mode. For more information about this relationship type, see the OPC.",
            OPC::OPC_E_DS_EXTERNAL_SIGNATURE => "A relationship whose target is a Signature part has the external target mode; Signature parts must be inside of the package.",
            OPC::OPC_E_DS_MISSING_SIGNATURE_ORIGIN_PART => "A relationship of type digital signature origin was found, but the Digital Signature Origin part itself was not. For more information about this relationship type, see the OPC.",
            OPC::OPC_E_DS_MISSING_SIGNATURE_PART => "The specified Signature part does not exist in the package.",
            OPC::OPC_E_DS_INVALID_RELATIONSHIP_TRANSFORM_XML => "A Transform element that indicates the use of the relationships transform and the selection criteria for the transform does not conform to the schema specified in the OPC.",
            OPC::OPC_E_DS_INVALID_CANONICALIZATION_METHOD => "An unsupported canonicalization method was requested or used in a signature.",
            OPC::OPC_E_DS_INVALID_RELATIONSHIPS_SIGNING_OPTION => "An indicated relationship signing option is not a valid OPC_RELATIONSHIPS_SIGNING_OPTION enumeration value.",
            OPC::OPC_E_DS_INVALID_OPC_SIGNATURE_TIME_FORMAT => "The signature's time format is not a valid OPC_SIGNATURE_TIME_FORMAT enumeration value.",
            OPC::OPC_E_DS_PACKAGE_REFERENCE_URI_RESERVED => "The reserved URI attribute value of the signature's Reference element to the package Object is being used as the URI attribute value of a Reference to a custom Object element.",
            OPC::OPC_E_DS_MISSING_SIGNATURE_PROPERTIES_ELEMENT => "The SignatureProperties element was not found in the signature markup.",
            OPC::OPC_E_DS_MISSING_SIGNATURE_PROPERTY_ELEMENT => "The SignatureProperty child element of the SignatureProperties element was not found.",
            OPC::OPC_E_DS_DUPLICATE_SIGNATURE_PROPERTY_ELEMENT => "The signature markup contains more than one SignatureProperty element that has the same Id attribute.",
            OPC::OPC_E_DS_MISSING_SIGNATURE_TIME_PROPERTY => "The SignatureProperty element with the Id attribute value of 'idSignatureTime' does not exist or is not correctly constructed.",
            OPC::OPC_E_DS_INVALID_SIGNATURE_XML => "The signature markup in a Signature part does not conform to the schema specified in the OPC or XML-Signature Syntax and Processing (http://go.microsoft.com/fwlink/p/?linkid=132847).",
            OPC::OPC_E_DS_INVALID_SIGNATURE_COUNT => "A Signature part does not contain the signature markup for exactly one signature.",
            OPC::OPC_E_DS_MISSING_SIGNATURE_ALGORITHM => "The signature markup does not specify a signature method algorithm.",
            OPC::OPC_E_DS_DUPLICATE_PACKAGE_OBJECT_REFERENCES => "The signature markup contains more than one Reference element that refers to the package Object element, but only one such Reference is allowed.",
            OPC::OPC_E_DS_MISSING_PACKAGE_OBJECT_REFERENCE => "The signature markup is missing a Reference to the package-specific Object element.",
            OPC::OPC_E_DS_EXTERNAL_SIGNATURE_REFERENCE => "A Reference element in the signature markup refers to an object that is external to the package. Reference elements must point to parts or Object elements that are internal.",
            OPC::OPC_E_DS_REFERENCE_MISSING_CONTENT_TYPE => "The URI attribute value of a Reference element in the signature markup does not include the content type of the referenced part.",
            OPC::OPC_E_DS_MULTIPLE_RELATIONSHIP_TRANSFORMS => "More than one relationships transform is specified for a Reference element, but only one relationships transform is allowed.",
            OPC::OPC_E_DS_MISSING_CANONICALIZATION_TRANSFORM => "A relationships transform is not followed by a canonicalization method; the relationships transform must be followed by a canonicalization method.",
            OPC::OPC_E_MC_UNEXPECTED_ELEMENT => "An element that is used in the markup compatibility namespace is not recognized as a valid markup compatibility element.",
            OPC::OPC_E_MC_UNEXPECTED_REQUIRES_ATTR => "The Requires attribute was found on a markup compatibility element that does not support this attribute.",
            OPC::OPC_E_MC_MISSING_REQUIRES_ATTR => "A markup compatibility element that requires the Requires attribute is missing the attribute, or that attribute's value is empty.",
            OPC::OPC_E_MC_UNEXPECTED_ATTR => "A markup compatibility element has an attribute that is not recognized or supported in the markup compatibility namespace.",
            OPC::OPC_E_MC_INVALID_PREFIX_LIST => "The prefix list cannot be parsed, because it contains a character that is not valid, such as ':'.",
            OPC::OPC_E_MC_INVALID_QNAME_LIST => "A markup compatibility attribute contains an invalid list of element names.",
            OPC::OPC_E_MC_NESTED_ALTERNATE_CONTENT => "An AlternateContent element cannot have another AlternateContent element as a child.",
            OPC::OPC_E_MC_UNEXPECTED_CHOICE => "A Choice element was encountered after the Fallback element, or more than one Fallback element was encountered.",
            OPC::OPC_E_MC_MISSING_CHOICE => "An AlternateContent element must have at least one Choice element as a child.",
            OPC::OPC_E_MC_INVALID_ENUM_TYPE => "Encountered an enumeration type that is not valid.",
            OPC::OPC_E_MC_UNKNOWN_NAMESPACE => "The MustUnderstand attribute of an element contains a namespace that is not recognized.",
            OPC::OPC_E_MC_UNKNOWN_PREFIX => "The package contains an XML prefix that is not defined.",
            OPC::OPC_E_MC_INVALID_ATTRIBUTES_ON_IGNORABLE_ELEMENT => "An element (or its parent) that is ignorable by markup compatibility has an attribute that is prohibited in Part 5: Markup Compatibility and Extensibility of ECMA-376 OpenXML (http://go.microsoft.com/fwlink/p/?linkid=123375).",
            OPC::OPC_E_MC_INVALID_XMLNS_ATTRIBUTE => "The declaration of an xmlns attribute is malformed.",
            OPC::OPC_E_INVALID_XML_ENCODING => "The XML Encoding attribute must be UTF-8 or UTF-16.",
            OPC::OPC_E_DS_SIGNATURE_REFERENCE_MISSING_URI => "The URI attribute is required for a Reference element but is missing.",
            OPC::OPC_E_INVALID_CONTENT_TYPE => "A content type does not conform to the rules for a valid media type, specified in RFC 2616: HTTP/1.1(http://go.microsoft.com/fwlink/p/?linkid=143979) and the OPC.",
            OPC::OPC_E_DS_SIGNATURE_PROPERTY_MISSING_TARGET => "The SignatureProperty element is missing the required Target attribute.",
            OPC::OPC_E_DS_SIGNATURE_METHOD_NOT_SET => "The signature method has not been set. Call IOpcSigningOptions::SetSignatureMethod to set the signature method.",
            OPC::OPC_E_DS_DEFAULT_DIGEST_METHOD_NOT_SET => "The default digest method has not been set; to set it, call IOpcSigningOptions::SetDefaultDigestMethod.",
            OPC::OPC_E_NO_SUCH_RELATIONSHIP => "The specified relationship does not exist.",
            OPC::OPC_E_MC_MULTIPLE_FALLBACK_ELEMENTS => "There is more than one Fallback element within the child elements of an AlternateContent, but only one is allowed.",
            OPC::OPC_E_MC_INCONSISTENT_PROCESS_CONTENT => "The ProcessContent attribute does not have a matching Ignorable attribute on the same element.",
            OPC::OPC_E_MC_INCONSISTENT_PRESERVE_ATTRIBUTES => "The PreserveAttributes attribute does not have a matching Ignorable attribute on the same element.",
            OPC::OPC_E_MC_INCONSISTENT_PRESERVE_ELEMENTS => "The PreserveElements attribute does not have a matching Ignorable attribute on the same element.",
            OPC::OPC_E_INVALID_RELATIONSHIP_TARGET_MODE => "The TargetMode attribute of a relationship does not conform to the rules specified in the OPC.",
            OPC::OPC_E_COULD_NOT_RECOVER => "Errors that occurred during previous operations prevent the requested operation from completing. Future attempts to perform the operation that triggered this error, or attempts to perform related operations, will fail.",
            OPC::OPC_E_UNSUPPORTED_PACKAGE => "The OPC allows certain aspects of a package, such as the number of pieces used to compose a part, to grow without bounds. For practical purposes, however, the Packaging API limits the possible number of pieces to an unsigned DWORD.",
            OPC::OPC_E_ENUM_COLLECTION_CHANGED => "The enumerator is invalid because the underlying set has changed.",
            OPC::OPC_E_ENUM_CANNOT_MOVE_NEXT => "The current position is already past the last item of the enumerator.",
            OPC::OPC_E_ENUM_CANNOT_MOVE_PREVIOUS => "The current position already precedes the first item of the enumerator.",
            OPC::OPC_E_ENUM_INVALID_POSITION => "The enumerator cannot perform this operation from its current position.",
            OPC::OPC_E_DS_SIGNATURE_ORIGIN_EXISTS => "A Digital Signature Origin part already exists in the package and cannot be renamed.",
            OPC::OPC_E_DS_UNSIGNED_PACKAGE => "The package is not signed; therefore, the signature markup cannot be replaced.",
            OPC::OPC_E_DS_MISSING_CERTIFICATE_PART => "The part that contains the certificate and is the target of a relationship of type digital signature certificate does not exist. For more information about this relationship type, see the OPC.",
            OPC::OPC_E_ZIP_INCORRECT_DATA_SIZE => "A field or header in the archive contains a different number of bytes than expected.",
            OPC::OPC_E_ZIP_CORRUPTED_ARCHIVE => "The archive has an incorrect checksum or signature, or file headers in the central directory are in an unexpected order.",
            OPC::OPC_E_ZIP_COMPRESSION_FAILED => "Compression of the data failed.",
            OPC::OPC_E_ZIP_DECOMPRESSION_FAILED => "Data decompression failed.",
            OPC::OPC_E_ZIP_INCONSISTENT_FILEITEM => "The file attributes recorded in a file header in the central directory are inconsistent with the attributes of the zipped file item.",
            OPC::OPC_E_ZIP_INCONSISTENT_DIRECTORY => "The number of file items found in the archive does not match the number of file items recorded in the central directory.",
            OPC::OPC_E_ZIP_MISSING_DATA_DESCRIPTOR => "A data descriptor was expected for the file item, but none was found.",
            OPC::OPC_E_ZIP_UNSUPPORTEDARCHIVE => "The package cannot be read because the archive is either unsupported (for example, a multidisk) or encrypted.",
            OPC::OPC_E_ZIP_CENTRAL_DIRECTORY_TOO_LARGE => "The central directory exceeds 65,535 bytes.",
            OPC::OPC_E_ZIP_NAME_TOO_LARGE => "The length of a file item name exceeds 65,535 characters.",
            OPC::OPC_E_ZIP_DUPLICATE_NAME => "Two file items in the archive have the same name.",
            OPC::OPC_E_ZIP_COMMENT_TOO_LARGE => "The file comment length exceeds 65,535 bytes.",
            OPC::OPC_E_ZIP_EXTRA_FIELDS_TOO_LARGE => "The extra field length exceeds 65,535 bytes.",
            OPC::OPC_E_ZIP_FILE_HEADER_TOO_LARGE => "The length of a local file header exceeds 65,535 bytes.",
            OPC::OPC_E_ZIP_MISSING_END_OF_CENTRAL_DIRECTORY => "The archive is missing the end of the central directory record.",
            OPC::OPC_E_ZIP_REQUIRES_64_BIT => "The 64-bit ZIP format is required because the data exceeds the capacity of the 32-bit ZIP format. Recreate the package using the 64-bit ZIP format.",
        }
    }

    pub fn from_name(name: &str) -> OPC {
        return match name {
            "OPC_E_NONCONFORMING_URI" => OPC::OPC_E_NONCONFORMING_URI,
            "OPC_E_RELATIVE_URI_REQUIRED" => OPC::OPC_E_RELATIVE_URI_REQUIRED,
            "OPC_E_RELATIONSHIP_URI_REQUIRED" => OPC::OPC_E_RELATIONSHIP_URI_REQUIRED,
            "OPC_E_PART_CANNOT_BE_DIRECTORY" => OPC::OPC_E_PART_CANNOT_BE_DIRECTORY,
            "OPC_E_UNEXPECTED_CONTENT_TYPE" => OPC::OPC_E_UNEXPECTED_CONTENT_TYPE,
            "OPC_E_INVALID_CONTENT_TYPE_XML" => OPC::OPC_E_INVALID_CONTENT_TYPE_XML,
            "OPC_E_MISSING_CONTENT_TYPES" => OPC::OPC_E_MISSING_CONTENT_TYPES,
            "OPC_E_NONCONFORMING_CONTENT_TYPES_XML" => OPC::OPC_E_NONCONFORMING_CONTENT_TYPES_XML,
            "OPC_E_NONCONFORMING_RELS_XML" => OPC::OPC_E_NONCONFORMING_RELS_XML,
            "OPC_E_INVALID_RELS_XML" => OPC::OPC_E_INVALID_RELS_XML,
            "OPC_E_DUPLICATE_PART" => OPC::OPC_E_DUPLICATE_PART,
            "OPC_E_INVALID_OVERRIDE_PART_NAME" => OPC::OPC_E_INVALID_OVERRIDE_PART_NAME,
            "OPC_E_DUPLICATE_OVERRIDE_PART" => OPC::OPC_E_DUPLICATE_OVERRIDE_PART,
            "OPC_E_INVALID_DEFAULT_EXTENSION" => OPC::OPC_E_INVALID_DEFAULT_EXTENSION,
            "OPC_E_DUPLICATE_DEFAULT_EXTENSION" => OPC::OPC_E_DUPLICATE_DEFAULT_EXTENSION,
            "OPC_E_INVALID_RELATIONSHIP_ID" => OPC::OPC_E_INVALID_RELATIONSHIP_ID,
            "OPC_E_INVALID_RELATIONSHIP_TYPE" => OPC::OPC_E_INVALID_RELATIONSHIP_TYPE,
            "OPC_E_INVALID_RELATIONSHIP_TARGET" => OPC::OPC_E_INVALID_RELATIONSHIP_TARGET,
            "OPC_E_DUPLICATE_RELATIONSHIP" => OPC::OPC_E_DUPLICATE_RELATIONSHIP,
            "OPC_E_CONFLICTING_SETTINGS" => OPC::OPC_E_CONFLICTING_SETTINGS,
            "OPC_E_DUPLICATE_PIECE" => OPC::OPC_E_DUPLICATE_PIECE,
            "OPC_E_INVALID_PIECE" => OPC::OPC_E_INVALID_PIECE,
            "OPC_E_MISSING_PIECE" => OPC::OPC_E_MISSING_PIECE,
            "OPC_E_NO_SUCH_PART" => OPC::OPC_E_NO_SUCH_PART,
            "OPC_E_DS_SIGNATURE_CORRUPT" => OPC::OPC_E_DS_SIGNATURE_CORRUPT,
            "OPC_E_DS_DIGEST_VALUE_ERROR" => OPC::OPC_E_DS_DIGEST_VALUE_ERROR,
            "OPC_E_DS_DUPLICATE_SIGNATURE_ORIGIN_RELATIONSHIP" => OPC::OPC_E_DS_DUPLICATE_SIGNATURE_ORIGIN_RELATIONSHIP,
            "OPC_E_DS_INVALID_SIGNATURE_ORIGIN_RELATIONSHIP" => OPC::OPC_E_DS_INVALID_SIGNATURE_ORIGIN_RELATIONSHIP,
            "OPC_E_DS_INVALID_CERTIFICATE_RELATIONSHIP" => OPC::OPC_E_DS_INVALID_CERTIFICATE_RELATIONSHIP,
            "OPC_E_DS_EXTERNAL_SIGNATURE" => OPC::OPC_E_DS_EXTERNAL_SIGNATURE,
            "OPC_E_DS_MISSING_SIGNATURE_ORIGIN_PART" => OPC::OPC_E_DS_MISSING_SIGNATURE_ORIGIN_PART,
            "OPC_E_DS_MISSING_SIGNATURE_PART" => OPC::OPC_E_DS_MISSING_SIGNATURE_PART,
            "OPC_E_DS_INVALID_RELATIONSHIP_TRANSFORM_XML" => OPC::OPC_E_DS_INVALID_RELATIONSHIP_TRANSFORM_XML,
            "OPC_E_DS_INVALID_CANONICALIZATION_METHOD" => OPC::OPC_E_DS_INVALID_CANONICALIZATION_METHOD,
            "OPC_E_DS_INVALID_RELATIONSHIPS_SIGNING_OPTION" => OPC::OPC_E_DS_INVALID_RELATIONSHIPS_SIGNING_OPTION,
            "OPC_E_DS_INVALID_OPC_SIGNATURE_TIME_FORMAT" => OPC::OPC_E_DS_INVALID_OPC_SIGNATURE_TIME_FORMAT,
            "OPC_E_DS_PACKAGE_REFERENCE_URI_RESERVED" => OPC::OPC_E_DS_PACKAGE_REFERENCE_URI_RESERVED,
            "OPC_E_DS_MISSING_SIGNATURE_PROPERTIES_ELEMENT" => OPC::OPC_E_DS_MISSING_SIGNATURE_PROPERTIES_ELEMENT,
            "OPC_E_DS_MISSING_SIGNATURE_PROPERTY_ELEMENT" => OPC::OPC_E_DS_MISSING_SIGNATURE_PROPERTY_ELEMENT,
            "OPC_E_DS_DUPLICATE_SIGNATURE_PROPERTY_ELEMENT" => OPC::OPC_E_DS_DUPLICATE_SIGNATURE_PROPERTY_ELEMENT,
            "OPC_E_DS_MISSING_SIGNATURE_TIME_PROPERTY" => OPC::OPC_E_DS_MISSING_SIGNATURE_TIME_PROPERTY,
            "OPC_E_DS_INVALID_SIGNATURE_XML" => OPC::OPC_E_DS_INVALID_SIGNATURE_XML,
            "OPC_E_DS_INVALID_SIGNATURE_COUNT" => OPC::OPC_E_DS_INVALID_SIGNATURE_COUNT,
            "OPC_E_DS_MISSING_SIGNATURE_ALGORITHM" => OPC::OPC_E_DS_MISSING_SIGNATURE_ALGORITHM,
            "OPC_E_DS_DUPLICATE_PACKAGE_OBJECT_REFERENCES" => OPC::OPC_E_DS_DUPLICATE_PACKAGE_OBJECT_REFERENCES,
            "OPC_E_DS_MISSING_PACKAGE_OBJECT_REFERENCE" => OPC::OPC_E_DS_MISSING_PACKAGE_OBJECT_REFERENCE,
            "OPC_E_DS_EXTERNAL_SIGNATURE_REFERENCE" => OPC::OPC_E_DS_EXTERNAL_SIGNATURE_REFERENCE,
            "OPC_E_DS_REFERENCE_MISSING_CONTENT_TYPE" => OPC::OPC_E_DS_REFERENCE_MISSING_CONTENT_TYPE,
            "OPC_E_DS_MULTIPLE_RELATIONSHIP_TRANSFORMS" => OPC::OPC_E_DS_MULTIPLE_RELATIONSHIP_TRANSFORMS,
            "OPC_E_DS_MISSING_CANONICALIZATION_TRANSFORM" => OPC::OPC_E_DS_MISSING_CANONICALIZATION_TRANSFORM,
            "OPC_E_MC_UNEXPECTED_ELEMENT" => OPC::OPC_E_MC_UNEXPECTED_ELEMENT,
            "OPC_E_MC_UNEXPECTED_REQUIRES_ATTR" => OPC::OPC_E_MC_UNEXPECTED_REQUIRES_ATTR,
            "OPC_E_MC_MISSING_REQUIRES_ATTR" => OPC::OPC_E_MC_MISSING_REQUIRES_ATTR,
            "OPC_E_MC_UNEXPECTED_ATTR" => OPC::OPC_E_MC_UNEXPECTED_ATTR,
            "OPC_E_MC_INVALID_PREFIX_LIST" => OPC::OPC_E_MC_INVALID_PREFIX_LIST,
            "OPC_E_MC_INVALID_QNAME_LIST" => OPC::OPC_E_MC_INVALID_QNAME_LIST,
            "OPC_E_MC_NESTED_ALTERNATE_CONTENT" => OPC::OPC_E_MC_NESTED_ALTERNATE_CONTENT,
            "OPC_E_MC_UNEXPECTED_CHOICE" => OPC::OPC_E_MC_UNEXPECTED_CHOICE,
            "OPC_E_MC_MISSING_CHOICE" => OPC::OPC_E_MC_MISSING_CHOICE,
            "OPC_E_MC_INVALID_ENUM_TYPE" => OPC::OPC_E_MC_INVALID_ENUM_TYPE,
            "OPC_E_MC_UNKNOWN_NAMESPACE" => OPC::OPC_E_MC_UNKNOWN_NAMESPACE,
            "OPC_E_MC_UNKNOWN_PREFIX" => OPC::OPC_E_MC_UNKNOWN_PREFIX,
            "OPC_E_MC_INVALID_ATTRIBUTES_ON_IGNORABLE_ELEMENT" => OPC::OPC_E_MC_INVALID_ATTRIBUTES_ON_IGNORABLE_ELEMENT,
            "OPC_E_MC_INVALID_XMLNS_ATTRIBUTE" => OPC::OPC_E_MC_INVALID_XMLNS_ATTRIBUTE,
            "OPC_E_INVALID_XML_ENCODING" => OPC::OPC_E_INVALID_XML_ENCODING,
            "OPC_E_DS_SIGNATURE_REFERENCE_MISSING_URI" => OPC::OPC_E_DS_SIGNATURE_REFERENCE_MISSING_URI,
            "OPC_E_INVALID_CONTENT_TYPE" => OPC::OPC_E_INVALID_CONTENT_TYPE,
            "OPC_E_DS_SIGNATURE_PROPERTY_MISSING_TARGET" => OPC::OPC_E_DS_SIGNATURE_PROPERTY_MISSING_TARGET,
            "OPC_E_DS_SIGNATURE_METHOD_NOT_SET" => OPC::OPC_E_DS_SIGNATURE_METHOD_NOT_SET,
            "OPC_E_DS_DEFAULT_DIGEST_METHOD_NOT_SET" => OPC::OPC_E_DS_DEFAULT_DIGEST_METHOD_NOT_SET,
            "OPC_E_NO_SUCH_RELATIONSHIP" => OPC::OPC_E_NO_SUCH_RELATIONSHIP,
            "OPC_E_MC_MULTIPLE_FALLBACK_ELEMENTS" => OPC::OPC_E_MC_MULTIPLE_FALLBACK_ELEMENTS,
            "OPC_E_MC_INCONSISTENT_PROCESS_CONTENT" => OPC::OPC_E_MC_INCONSISTENT_PROCESS_CONTENT,
            "OPC_E_MC_INCONSISTENT_PRESERVE_ATTRIBUTES" => OPC::OPC_E_MC_INCONSISTENT_PRESERVE_ATTRIBUTES,
            "OPC_E_MC_INCONSISTENT_PRESERVE_ELEMENTS" => OPC::OPC_E_MC_INCONSISTENT_PRESERVE_ELEMENTS,
            "OPC_E_INVALID_RELATIONSHIP_TARGET_MODE" => OPC::OPC_E_INVALID_RELATIONSHIP_TARGET_MODE,
            "OPC_E_COULD_NOT_RECOVER" => OPC::OPC_E_COULD_NOT_RECOVER,
            "OPC_E_UNSUPPORTED_PACKAGE" => OPC::OPC_E_UNSUPPORTED_PACKAGE,
            "OPC_E_ENUM_COLLECTION_CHANGED" => OPC::OPC_E_ENUM_COLLECTION_CHANGED,
            "OPC_E_ENUM_CANNOT_MOVE_NEXT" => OPC::OPC_E_ENUM_CANNOT_MOVE_NEXT,
            "OPC_E_ENUM_CANNOT_MOVE_PREVIOUS" => OPC::OPC_E_ENUM_CANNOT_MOVE_PREVIOUS,
            "OPC_E_ENUM_INVALID_POSITION" => OPC::OPC_E_ENUM_INVALID_POSITION,
            "OPC_E_DS_SIGNATURE_ORIGIN_EXISTS" => OPC::OPC_E_DS_SIGNATURE_ORIGIN_EXISTS,
            "OPC_E_DS_UNSIGNED_PACKAGE" => OPC::OPC_E_DS_UNSIGNED_PACKAGE,
            "OPC_E_DS_MISSING_CERTIFICATE_PART" => OPC::OPC_E_DS_MISSING_CERTIFICATE_PART,
            "OPC_E_ZIP_INCORRECT_DATA_SIZE" => OPC::OPC_E_ZIP_INCORRECT_DATA_SIZE,
            "OPC_E_ZIP_CORRUPTED_ARCHIVE" => OPC::OPC_E_ZIP_CORRUPTED_ARCHIVE,
            "OPC_E_ZIP_COMPRESSION_FAILED" => OPC::OPC_E_ZIP_COMPRESSION_FAILED,
            "OPC_E_ZIP_DECOMPRESSION_FAILED" => OPC::OPC_E_ZIP_DECOMPRESSION_FAILED,
            "OPC_E_ZIP_INCONSISTENT_FILEITEM" => OPC::OPC_E_ZIP_INCONSISTENT_FILEITEM,
            "OPC_E_ZIP_INCONSISTENT_DIRECTORY" => OPC::OPC_E_ZIP_INCONSISTENT_DIRECTORY,
            "OPC_E_ZIP_MISSING_DATA_DESCRIPTOR" => OPC::OPC_E_ZIP_MISSING_DATA_DESCRIPTOR,
            "OPC_E_ZIP_UNSUPPORTEDARCHIVE" => OPC::OPC_E_ZIP_UNSUPPORTEDARCHIVE,
            "OPC_E_ZIP_CENTRAL_DIRECTORY_TOO_LARGE" => OPC::OPC_E_ZIP_CENTRAL_DIRECTORY_TOO_LARGE,
            "OPC_E_ZIP_NAME_TOO_LARGE" => OPC::OPC_E_ZIP_NAME_TOO_LARGE,
            "OPC_E_ZIP_DUPLICATE_NAME" => OPC::OPC_E_ZIP_DUPLICATE_NAME,
            "OPC_E_ZIP_COMMENT_TOO_LARGE" => OPC::OPC_E_ZIP_COMMENT_TOO_LARGE,
            "OPC_E_ZIP_EXTRA_FIELDS_TOO_LARGE" => OPC::OPC_E_ZIP_EXTRA_FIELDS_TOO_LARGE,
            "OPC_E_ZIP_FILE_HEADER_TOO_LARGE" => OPC::OPC_E_ZIP_FILE_HEADER_TOO_LARGE,
            "OPC_E_ZIP_MISSING_END_OF_CENTRAL_DIRECTORY" => OPC::OPC_E_ZIP_MISSING_END_OF_CENTRAL_DIRECTORY,
            "OPC_E_ZIP_REQUIRES_64_BIT" => OPC::OPC_E_ZIP_REQUIRES_64_BIT,
        }
    }
    pub fn from_code(code: u32) -> OPC {
        return match code {
            0x80510001 => OPC::OPC_E_NONCONFORMING_URI,
            0x80510002 => OPC::OPC_E_RELATIVE_URI_REQUIRED,
            0x80510003 => OPC::OPC_E_RELATIONSHIP_URI_REQUIRED,
            0x80510004 => OPC::OPC_E_PART_CANNOT_BE_DIRECTORY,
            0x80510005 => OPC::OPC_E_UNEXPECTED_CONTENT_TYPE,
            0x80510006 => OPC::OPC_E_INVALID_CONTENT_TYPE_XML,
            0x80510007 => OPC::OPC_E_MISSING_CONTENT_TYPES,
            0x80510008 => OPC::OPC_E_NONCONFORMING_CONTENT_TYPES_XML,
            0x80510009 => OPC::OPC_E_NONCONFORMING_RELS_XML,
            0x8051000A => OPC::OPC_E_INVALID_RELS_XML,
            0x8051000B => OPC::OPC_E_DUPLICATE_PART,
            0x8051000C => OPC::OPC_E_INVALID_OVERRIDE_PART_NAME,
            0x8051000D => OPC::OPC_E_DUPLICATE_OVERRIDE_PART,
            0x8051000E => OPC::OPC_E_INVALID_DEFAULT_EXTENSION,
            0x8051000F => OPC::OPC_E_DUPLICATE_DEFAULT_EXTENSION,
            0x80510010 => OPC::OPC_E_INVALID_RELATIONSHIP_ID,
            0x80510011 => OPC::OPC_E_INVALID_RELATIONSHIP_TYPE,
            0x80510012 => OPC::OPC_E_INVALID_RELATIONSHIP_TARGET,
            0x80510013 => OPC::OPC_E_DUPLICATE_RELATIONSHIP,
            0x80510014 => OPC::OPC_E_CONFLICTING_SETTINGS,
            0x80510015 => OPC::OPC_E_DUPLICATE_PIECE,
            0x80510016 => OPC::OPC_E_INVALID_PIECE,
            0x80510017 => OPC::OPC_E_MISSING_PIECE,
            0x80510018 => OPC::OPC_E_NO_SUCH_PART,
            0x80510019 => OPC::OPC_E_DS_SIGNATURE_CORRUPT,
            0x8051001A => OPC::OPC_E_DS_DIGEST_VALUE_ERROR,
            0x8051001B => OPC::OPC_E_DS_DUPLICATE_SIGNATURE_ORIGIN_RELATIONSHIP,
            0x8051001C => OPC::OPC_E_DS_INVALID_SIGNATURE_ORIGIN_RELATIONSHIP,
            0x8051001D => OPC::OPC_E_DS_INVALID_CERTIFICATE_RELATIONSHIP,
            0x8051001E => OPC::OPC_E_DS_EXTERNAL_SIGNATURE,
            0x8051001F => OPC::OPC_E_DS_MISSING_SIGNATURE_ORIGIN_PART,
            0x80510020 => OPC::OPC_E_DS_MISSING_SIGNATURE_PART,
            0x80510021 => OPC::OPC_E_DS_INVALID_RELATIONSHIP_TRANSFORM_XML,
            0x80510022 => OPC::OPC_E_DS_INVALID_CANONICALIZATION_METHOD,
            0x80510023 => OPC::OPC_E_DS_INVALID_RELATIONSHIPS_SIGNING_OPTION,
            0x80510024 => OPC::OPC_E_DS_INVALID_OPC_SIGNATURE_TIME_FORMAT,
            0x80510025 => OPC::OPC_E_DS_PACKAGE_REFERENCE_URI_RESERVED,
            0x80510026 => OPC::OPC_E_DS_MISSING_SIGNATURE_PROPERTIES_ELEMENT,
            0x80510027 => OPC::OPC_E_DS_MISSING_SIGNATURE_PROPERTY_ELEMENT,
            0x80510028 => OPC::OPC_E_DS_DUPLICATE_SIGNATURE_PROPERTY_ELEMENT,
            0x80510029 => OPC::OPC_E_DS_MISSING_SIGNATURE_TIME_PROPERTY,
            0x8051002A => OPC::OPC_E_DS_INVALID_SIGNATURE_XML,
            0x8051002B => OPC::OPC_E_DS_INVALID_SIGNATURE_COUNT,
            0x8051002C => OPC::OPC_E_DS_MISSING_SIGNATURE_ALGORITHM,
            0x8051002D => OPC::OPC_E_DS_DUPLICATE_PACKAGE_OBJECT_REFERENCES,
            0x8051002E => OPC::OPC_E_DS_MISSING_PACKAGE_OBJECT_REFERENCE,
            0x8051002F => OPC::OPC_E_DS_EXTERNAL_SIGNATURE_REFERENCE,
            0x80510030 => OPC::OPC_E_DS_REFERENCE_MISSING_CONTENT_TYPE,
            0x80510031 => OPC::OPC_E_DS_MULTIPLE_RELATIONSHIP_TRANSFORMS,
            0x80510032 => OPC::OPC_E_DS_MISSING_CANONICALIZATION_TRANSFORM,
            0x80510033 => OPC::OPC_E_MC_UNEXPECTED_ELEMENT,
            0x80510034 => OPC::OPC_E_MC_UNEXPECTED_REQUIRES_ATTR,
            0x80510035 => OPC::OPC_E_MC_MISSING_REQUIRES_ATTR,
            0x80510036 => OPC::OPC_E_MC_UNEXPECTED_ATTR,
            0x80510037 => OPC::OPC_E_MC_INVALID_PREFIX_LIST,
            0x80510038 => OPC::OPC_E_MC_INVALID_QNAME_LIST,
            0x80510039 => OPC::OPC_E_MC_NESTED_ALTERNATE_CONTENT,
            0x8051003A => OPC::OPC_E_MC_UNEXPECTED_CHOICE,
            0x8051003B => OPC::OPC_E_MC_MISSING_CHOICE,
            0x8051003C => OPC::OPC_E_MC_INVALID_ENUM_TYPE,
            0x8051003E => OPC::OPC_E_MC_UNKNOWN_NAMESPACE,
            0x8051003F => OPC::OPC_E_MC_UNKNOWN_PREFIX,
            0x80510040 => OPC::OPC_E_MC_INVALID_ATTRIBUTES_ON_IGNORABLE_ELEMENT,
            0x80510041 => OPC::OPC_E_MC_INVALID_XMLNS_ATTRIBUTE,
            0x80510042 => OPC::OPC_E_INVALID_XML_ENCODING,
            0x80510043 => OPC::OPC_E_DS_SIGNATURE_REFERENCE_MISSING_URI,
            0x80510044 => OPC::OPC_E_INVALID_CONTENT_TYPE,
            0x80510045 => OPC::OPC_E_DS_SIGNATURE_PROPERTY_MISSING_TARGET,
            0x80510046 => OPC::OPC_E_DS_SIGNATURE_METHOD_NOT_SET,
            0x80510047 => OPC::OPC_E_DS_DEFAULT_DIGEST_METHOD_NOT_SET,
            0x80510048 => OPC::OPC_E_NO_SUCH_RELATIONSHIP,
            0x80510049 => OPC::OPC_E_MC_MULTIPLE_FALLBACK_ELEMENTS,
            0x8051004A => OPC::OPC_E_MC_INCONSISTENT_PROCESS_CONTENT,
            0x8051004B => OPC::OPC_E_MC_INCONSISTENT_PRESERVE_ATTRIBUTES,
            0x8051004C => OPC::OPC_E_MC_INCONSISTENT_PRESERVE_ELEMENTS,
            0x8051004D => OPC::OPC_E_INVALID_RELATIONSHIP_TARGET_MODE,
            0x8051004E => OPC::OPC_E_COULD_NOT_RECOVER,
            0x8051004F => OPC::OPC_E_UNSUPPORTED_PACKAGE,
            0x80510050 => OPC::OPC_E_ENUM_COLLECTION_CHANGED,
            0x80510051 => OPC::OPC_E_ENUM_CANNOT_MOVE_NEXT,
            0x80510052 => OPC::OPC_E_ENUM_CANNOT_MOVE_PREVIOUS,
            0x80510053 => OPC::OPC_E_ENUM_INVALID_POSITION,
            0x80510054 => OPC::OPC_E_DS_SIGNATURE_ORIGIN_EXISTS,
            0x80510055 => OPC::OPC_E_DS_UNSIGNED_PACKAGE,
            0x80510056 => OPC::OPC_E_DS_MISSING_CERTIFICATE_PART,
            0x80511001 => OPC::OPC_E_ZIP_INCORRECT_DATA_SIZE,
            0x80511002 => OPC::OPC_E_ZIP_CORRUPTED_ARCHIVE,
            0x80511003 => OPC::OPC_E_ZIP_COMPRESSION_FAILED,
            0x80511004 => OPC::OPC_E_ZIP_DECOMPRESSION_FAILED,
            0x80511005 => OPC::OPC_E_ZIP_INCONSISTENT_FILEITEM,
            0x80511006 => OPC::OPC_E_ZIP_INCONSISTENT_DIRECTORY,
            0x80511007 => OPC::OPC_E_ZIP_MISSING_DATA_DESCRIPTOR,
            0x80511008 => OPC::OPC_E_ZIP_UNSUPPORTEDARCHIVE,
            0x80511009 => OPC::OPC_E_ZIP_CENTRAL_DIRECTORY_TOO_LARGE,
            0x8051100A => OPC::OPC_E_ZIP_NAME_TOO_LARGE,
            0x8051100B => OPC::OPC_E_ZIP_DUPLICATE_NAME,
            0x8051100C => OPC::OPC_E_ZIP_COMMENT_TOO_LARGE,
            0x8051100D => OPC::OPC_E_ZIP_EXTRA_FIELDS_TOO_LARGE,
            0x8051100E => OPC::OPC_E_ZIP_FILE_HEADER_TOO_LARGE,
            0x8051100F => OPC::OPC_E_ZIP_MISSING_END_OF_CENTRAL_DIRECTORY,
            0x80511010 => OPC::OPC_E_ZIP_REQUIRES_64_BIT,
        }
    }
}
