/*
 * Copyright 2023 Stanislav Mikhailov (xavetar)
 *
 * Licensed under the Creative Commons Zero v1.0 Universal (CC0) License.
 * You may obtain a copy of the License at
 *
 *     http://creativecommons.org/publicdomain/zero/1.0/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the CC0 license is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// It is strictly forbidden to use the from_code method, respected Microsoft decided to define
// duplicate codes, but by defining different constant names, therefore, when using these codes,
// it is mandatory to use through from_name, and not from_code. Otherwise, it may cause undefined
// behavior or an unknown exception. Because one code corresponds to several constants.

use super::{RawError};

#[derive(Clone, Debug, Copy, Eq, Hash, Ord, PartialEq, PartialOrd)]
pub enum P2P_INT {
    DRT_E_TIMEOUT,
    DRT_E_INVALID_KEY_SIZE,
    DRT_E_INVALID_CERT_CHAIN,
    DRT_E_INVALID_MESSAGE,
    DRT_E_NO_MORE,
    DRT_E_INVALID_MAX_ADDRESSES,
    DRT_E_SEARCH_IN_PROGRESS,
    DRT_E_INVALID_KEY,
    DRT_S_RETRY,
    DRT_E_INVALID_MAX_ENDPOINTS,
    DRT_E_INVALID_SEARCH_RANGE,
    DRT_E_INVALID_PORT,
    DRT_E_INVALID_TRANSPORT_PROVIDER,
    DRT_E_INVALID_SECURITY_PROVIDER,
    DRT_E_STILL_IN_USE,
    DRT_E_INVALID_BOOTSTRAP_PROVIDER,
    DRT_E_INVALID_ADDRESS,
    DRT_E_INVALID_SCOPE,
    DRT_E_TRANSPORT_SHUTTING_DOWN,
    DRT_E_NO_ADDRESSES_AVAILABLE,
    DRT_E_DUPLICATE_KEY,
    DRT_E_TRANSPORTPROVIDER_IN_USE,
    DRT_E_TRANSPORTPROVIDER_NOT_ATTACHED,
    DRT_E_SECURITYPROVIDER_IN_USE,
    DRT_E_SECURITYPROVIDER_NOT_ATTACHED,
    DRT_E_BOOTSTRAPPROVIDER_IN_USE,
    DRT_E_BOOTSTRAPPROVIDER_NOT_ATTACHED,
    DRT_E_TRANSPORT_ALREADY_BOUND,
    DRT_E_TRANSPORT_NOT_BOUND,
    DRT_E_TRANSPORT_UNEXPECTED,
    DRT_E_TRANSPORT_INVALID_ARGUMENT,
    DRT_E_TRANSPORT_NO_DEST_ADDRESSES,
    DRT_E_TRANSPORT_EXECUTING_CALLBACK,
    DRT_E_TRANSPORT_ALREADY_EXISTS_FOR_SCOPE,
    DRT_E_INVALID_SETTINGS,
    DRT_E_INVALID_SEARCH_INFO,
    DRT_E_FAULTED,
    DRT_E_TRANSPORT_STILL_BOUND,
    DRT_E_INSUFFICIENT_BUFFER,
    DRT_E_INVALID_INSTANCE_PREFIX,
    DRT_E_INVALID_SECURITY_MODE,
    DRT_E_CAPABILITY_MISMATCH,
}

impl P2P_INT {
    pub fn code(&self) -> u32 {
        return match self {
            P2P_INT::DRT_E_TIMEOUT => 0x80621001 as u32,
            P2P_INT::DRT_E_INVALID_KEY_SIZE => 0x80621002 as u32,
            P2P_INT::DRT_E_INVALID_CERT_CHAIN => 0x80621004 as u32,
            P2P_INT::DRT_E_INVALID_MESSAGE => 0x80621005 as u32,
            P2P_INT::DRT_E_NO_MORE => 0x80621006 as u32,
            P2P_INT::DRT_E_INVALID_MAX_ADDRESSES => 0x80621007 as u32,
            P2P_INT::DRT_E_SEARCH_IN_PROGRESS => 0x80621008 as u32,
            P2P_INT::DRT_E_INVALID_KEY => 0x80621009 as u32,
            P2P_INT::DRT_S_RETRY => 0x80621010 as u32,
            P2P_INT::DRT_E_INVALID_MAX_ENDPOINTS => 0x80621011 as u32,
            P2P_INT::DRT_E_INVALID_SEARCH_RANGE => 0x80621012 as u32,
            P2P_INT::DRT_E_INVALID_PORT => 0x80622000 as u32,
            P2P_INT::DRT_E_INVALID_TRANSPORT_PROVIDER => 0x80622001 as u32,
            P2P_INT::DRT_E_INVALID_SECURITY_PROVIDER => 0x80622002 as u32,
            P2P_INT::DRT_E_STILL_IN_USE => 0x80622003 as u32,
            P2P_INT::DRT_E_INVALID_BOOTSTRAP_PROVIDER => 0x80622004 as u32,
            P2P_INT::DRT_E_INVALID_ADDRESS => 0x80622005 as u32,
            P2P_INT::DRT_E_INVALID_SCOPE => 0x80622006 as u32,
            P2P_INT::DRT_E_TRANSPORT_SHUTTING_DOWN => 0x80622007 as u32,
            P2P_INT::DRT_E_NO_ADDRESSES_AVAILABLE => 0x80622008 as u32,
            P2P_INT::DRT_E_DUPLICATE_KEY => 0x80622009 as u32,
            P2P_INT::DRT_E_TRANSPORTPROVIDER_IN_USE => 0x8062200A as u32,
            P2P_INT::DRT_E_TRANSPORTPROVIDER_NOT_ATTACHED => 0x8062200B as u32,
            P2P_INT::DRT_E_SECURITYPROVIDER_IN_USE => 0x8062200C as u32,
            P2P_INT::DRT_E_SECURITYPROVIDER_NOT_ATTACHED => 0x8062200D as u32,
            P2P_INT::DRT_E_BOOTSTRAPPROVIDER_IN_USE => 0x8062200E as u32,
            P2P_INT::DRT_E_BOOTSTRAPPROVIDER_NOT_ATTACHED => 0x8062200F as u32,
            P2P_INT::DRT_E_TRANSPORT_ALREADY_BOUND => 0x80622101 as u32,
            P2P_INT::DRT_E_TRANSPORT_NOT_BOUND => 0x80622102 as u32,
            P2P_INT::DRT_E_TRANSPORT_UNEXPECTED => 0x80622103 as u32,
            P2P_INT::DRT_E_TRANSPORT_INVALID_ARGUMENT => 0x80622104 as u32,
            P2P_INT::DRT_E_TRANSPORT_NO_DEST_ADDRESSES => 0x80622105 as u32,
            P2P_INT::DRT_E_TRANSPORT_EXECUTING_CALLBACK => 0x80622106 as u32,
            P2P_INT::DRT_E_TRANSPORT_ALREADY_EXISTS_FOR_SCOPE => 0x80622107 as u32,
            P2P_INT::DRT_E_INVALID_SETTINGS => 0x80622108 as u32,
            P2P_INT::DRT_E_INVALID_SEARCH_INFO => 0x80622109 as u32,
            P2P_INT::DRT_E_FAULTED => 0x8062210A as u32,
            P2P_INT::DRT_E_TRANSPORT_STILL_BOUND => 0x8062210B as u32,
            P2P_INT::DRT_E_INSUFFICIENT_BUFFER => 0x8062210C as u32,
            P2P_INT::DRT_E_INVALID_INSTANCE_PREFIX => 0x8062210D as u32,
            P2P_INT::DRT_E_INVALID_SECURITY_MODE => 0x8062210E as u32,
            P2P_INT::DRT_E_CAPABILITY_MISMATCH => 0x8062210F as u32,
        }
    }

    pub fn error(&self) -> RawError {
        return match self {
            P2P_INT::DRT_E_TIMEOUT => RawError::Kind(P2P_INT::DRT_E_TIMEOUT),
            P2P_INT::DRT_E_INVALID_KEY_SIZE => RawError::Kind(P2P_INT::DRT_E_INVALID_KEY_SIZE),
            P2P_INT::DRT_E_INVALID_CERT_CHAIN => RawError::Kind(P2P_INT::DRT_E_INVALID_CERT_CHAIN),
            P2P_INT::DRT_E_INVALID_MESSAGE => RawError::Kind(P2P_INT::DRT_E_INVALID_MESSAGE),
            P2P_INT::DRT_E_NO_MORE => RawError::Kind(P2P_INT::DRT_E_NO_MORE),
            P2P_INT::DRT_E_INVALID_MAX_ADDRESSES => RawError::Kind(P2P_INT::DRT_E_INVALID_MAX_ADDRESSES),
            P2P_INT::DRT_E_SEARCH_IN_PROGRESS => RawError::Kind(P2P_INT::DRT_E_SEARCH_IN_PROGRESS),
            P2P_INT::DRT_E_INVALID_KEY => RawError::Kind(P2P_INT::DRT_E_INVALID_KEY),
            P2P_INT::DRT_S_RETRY => RawError::Kind(P2P_INT::DRT_S_RETRY),
            P2P_INT::DRT_E_INVALID_MAX_ENDPOINTS => RawError::Kind(P2P_INT::DRT_E_INVALID_MAX_ENDPOINTS),
            P2P_INT::DRT_E_INVALID_SEARCH_RANGE => RawError::Kind(P2P_INT::DRT_E_INVALID_SEARCH_RANGE),
            P2P_INT::DRT_E_INVALID_PORT => RawError::Kind(P2P_INT::DRT_E_INVALID_PORT),
            P2P_INT::DRT_E_INVALID_TRANSPORT_PROVIDER => RawError::Kind(P2P_INT::DRT_E_INVALID_TRANSPORT_PROVIDER),
            P2P_INT::DRT_E_INVALID_SECURITY_PROVIDER => RawError::Kind(P2P_INT::DRT_E_INVALID_SECURITY_PROVIDER),
            P2P_INT::DRT_E_STILL_IN_USE => RawError::Kind(P2P_INT::DRT_E_STILL_IN_USE),
            P2P_INT::DRT_E_INVALID_BOOTSTRAP_PROVIDER => RawError::Kind(P2P_INT::DRT_E_INVALID_BOOTSTRAP_PROVIDER),
            P2P_INT::DRT_E_INVALID_ADDRESS => RawError::Kind(P2P_INT::DRT_E_INVALID_ADDRESS),
            P2P_INT::DRT_E_INVALID_SCOPE => RawError::Kind(P2P_INT::DRT_E_INVALID_SCOPE),
            P2P_INT::DRT_E_TRANSPORT_SHUTTING_DOWN => RawError::Kind(P2P_INT::DRT_E_TRANSPORT_SHUTTING_DOWN),
            P2P_INT::DRT_E_NO_ADDRESSES_AVAILABLE => RawError::Kind(P2P_INT::DRT_E_NO_ADDRESSES_AVAILABLE),
            P2P_INT::DRT_E_DUPLICATE_KEY => RawError::Kind(P2P_INT::DRT_E_DUPLICATE_KEY),
            P2P_INT::DRT_E_TRANSPORTPROVIDER_IN_USE => RawError::Kind(P2P_INT::DRT_E_TRANSPORTPROVIDER_IN_USE),
            P2P_INT::DRT_E_TRANSPORTPROVIDER_NOT_ATTACHED => RawError::Kind(P2P_INT::DRT_E_TRANSPORTPROVIDER_NOT_ATTACHED),
            P2P_INT::DRT_E_SECURITYPROVIDER_IN_USE => RawError::Kind(P2P_INT::DRT_E_SECURITYPROVIDER_IN_USE),
            P2P_INT::DRT_E_SECURITYPROVIDER_NOT_ATTACHED => RawError::Kind(P2P_INT::DRT_E_SECURITYPROVIDER_NOT_ATTACHED),
            P2P_INT::DRT_E_BOOTSTRAPPROVIDER_IN_USE => RawError::Kind(P2P_INT::DRT_E_BOOTSTRAPPROVIDER_IN_USE),
            P2P_INT::DRT_E_BOOTSTRAPPROVIDER_NOT_ATTACHED => RawError::Kind(P2P_INT::DRT_E_BOOTSTRAPPROVIDER_NOT_ATTACHED),
            P2P_INT::DRT_E_TRANSPORT_ALREADY_BOUND => RawError::Kind(P2P_INT::DRT_E_TRANSPORT_ALREADY_BOUND),
            P2P_INT::DRT_E_TRANSPORT_NOT_BOUND => RawError::Kind(P2P_INT::DRT_E_TRANSPORT_NOT_BOUND),
            P2P_INT::DRT_E_TRANSPORT_UNEXPECTED => RawError::Kind(P2P_INT::DRT_E_TRANSPORT_UNEXPECTED),
            P2P_INT::DRT_E_TRANSPORT_INVALID_ARGUMENT => RawError::Kind(P2P_INT::DRT_E_TRANSPORT_INVALID_ARGUMENT),
            P2P_INT::DRT_E_TRANSPORT_NO_DEST_ADDRESSES => RawError::Kind(P2P_INT::DRT_E_TRANSPORT_NO_DEST_ADDRESSES),
            P2P_INT::DRT_E_TRANSPORT_EXECUTING_CALLBACK => RawError::Kind(P2P_INT::DRT_E_TRANSPORT_EXECUTING_CALLBACK),
            P2P_INT::DRT_E_TRANSPORT_ALREADY_EXISTS_FOR_SCOPE => RawError::Kind(P2P_INT::DRT_E_TRANSPORT_ALREADY_EXISTS_FOR_SCOPE),
            P2P_INT::DRT_E_INVALID_SETTINGS => RawError::Kind(P2P_INT::DRT_E_INVALID_SETTINGS),
            P2P_INT::DRT_E_INVALID_SEARCH_INFO => RawError::Kind(P2P_INT::DRT_E_INVALID_SEARCH_INFO),
            P2P_INT::DRT_E_FAULTED => RawError::Kind(P2P_INT::DRT_E_FAULTED),
            P2P_INT::DRT_E_TRANSPORT_STILL_BOUND => RawError::Kind(P2P_INT::DRT_E_TRANSPORT_STILL_BOUND),
            P2P_INT::DRT_E_INSUFFICIENT_BUFFER => RawError::Kind(P2P_INT::DRT_E_INSUFFICIENT_BUFFER),
            P2P_INT::DRT_E_INVALID_INSTANCE_PREFIX => RawError::Kind(P2P_INT::DRT_E_INVALID_INSTANCE_PREFIX),
            P2P_INT::DRT_E_INVALID_SECURITY_MODE => RawError::Kind(P2P_INT::DRT_E_INVALID_SECURITY_MODE),
            P2P_INT::DRT_E_CAPABILITY_MISMATCH => RawError::Kind(P2P_INT::DRT_E_CAPABILITY_MISMATCH),
        }
    }

    pub fn description(&self) -> &'static str {
        return match self {
            P2P_INT::DRT_E_TIMEOUT => "The DRT operation has timed out.",
            P2P_INT::DRT_E_INVALID_KEY_SIZE => "The specified key does not meet, or exceeds, the allowed DRT key size.",
            P2P_INT::DRT_E_INVALID_CERT_CHAIN => "There is no certificate store attached or there is an error in the certificate chain.",
            P2P_INT::DRT_E_INVALID_MESSAGE => "The message exceeds the allowed character limit or is in an incorrect format.",
            P2P_INT::DRT_E_NO_MORE => "There is no more event data available.",
            P2P_INT::DRT_E_INVALID_MAX_ADDRESSES => "No addresses are available or the number of addresses exceeds 20.",
            P2P_INT::DRT_E_SEARCH_IN_PROGRESS => "A search operation is already in progress.",
            P2P_INT::DRT_E_INVALID_KEY => "Supplied key does not match generated key.",
            P2P_INT::DRT_S_RETRY => "The operation could not complete, but may be successful in a second attempt.",
            P2P_INT::DRT_E_INVALID_MAX_ENDPOINTS => "The number of endpoints exceeds maximum allowed endpoints.",
            P2P_INT::DRT_E_INVALID_SEARCH_RANGE => "Min and max key values are equal, but target is different.",
            P2P_INT::DRT_E_INVALID_PORT => "The specified port value is NULL.",
            P2P_INT::DRT_E_INVALID_TRANSPORT_PROVIDER => "The specified transport provider is invalid.",
            P2P_INT::DRT_E_INVALID_SECURITY_PROVIDER => "The specified security provider is invalid.",
            P2P_INT::DRT_E_STILL_IN_USE => "The DRT infrastructure is currently busy and cannot complete the operation.",
            P2P_INT::DRT_E_INVALID_BOOTSTRAP_PROVIDER => "The specified bootstrap provider is invalid.",
            P2P_INT::DRT_E_INVALID_ADDRESS => "The supplied address is not in the accepted and complete format, or is NULL.",
            P2P_INT::DRT_E_INVALID_SCOPE => "The specified scope is not one of the values defined in DRT_SCOPE.",
            P2P_INT::DRT_E_TRANSPORT_SHUTTING_DOWN => "The specified transport is in the process of shutting down.",
            P2P_INT::DRT_E_NO_ADDRESSES_AVAILABLE => "There are currently no remote nodes present in the DRT.",
            P2P_INT::DRT_E_DUPLICATE_KEY => "This key already exists within the DRT infrastructure.",
            P2P_INT::DRT_E_TRANSPORTPROVIDER_IN_USE => "The specified transport provider is already in use.",
            P2P_INT::DRT_E_TRANSPORTPROVIDER_NOT_ATTACHED => "The transport provider is not attached.",
            P2P_INT::DRT_E_SECURITYPROVIDER_IN_USE => "The specified security provider is already in use.",
            P2P_INT::DRT_E_SECURITYPROVIDER_NOT_ATTACHED => "The security provider is not attached.",
            P2P_INT::DRT_E_BOOTSTRAPPROVIDER_IN_USE => "The specified security provider is already in use.",
            P2P_INT::DRT_E_BOOTSTRAPPROVIDER_NOT_ATTACHED => "The bootstrap provider is not attached.",
            P2P_INT::DRT_E_TRANSPORT_ALREADY_BOUND => "The transport is already bound.",
            P2P_INT::DRT_E_TRANSPORT_NOT_BOUND => "The transport is not bound.",
            P2P_INT::DRT_E_TRANSPORT_UNEXPECTED => "The transport has encountered an unexpected error.",
            P2P_INT::DRT_E_TRANSPORT_INVALID_ARGUMENT => "An invalid argument was passed to the transport.",
            P2P_INT::DRT_E_TRANSPORT_NO_DEST_ADDRESSES => "Destination addresses have not been supplied to the transport.",
            P2P_INT::DRT_E_TRANSPORT_EXECUTING_CALLBACK => "The transport is currently executing a callback operation.",
            P2P_INT::DRT_E_TRANSPORT_ALREADY_EXISTS_FOR_SCOPE => "A transport already exists for this DRT scope.",
            P2P_INT::DRT_E_INVALID_SETTINGS => "The data contained inDRT SETTINGS structure is invalid or the relevant parameter value is NULL.",
            P2P_INT::DRT_E_INVALID_SEARCH_INFO => "The data contained in DRT_SEARCH_INFO structure is invalid or the relevant parameter value is NULL.",
            P2P_INT::DRT_E_FAULTED => "The DRT infrastructure has faulted. The DrtClose function must be called, after which an attempt to re-open the DRT can be made.",
            P2P_INT::DRT_E_TRANSPORT_STILL_BOUND => "The transport is currently bound.",
            P2P_INT::DRT_E_INSUFFICIENT_BUFFER => "The size of the buffer is insufficient for the operation.",
            P2P_INT::DRT_E_INVALID_INSTANCE_PREFIX => "The DRT instance prefix is invalid.",
            P2P_INT::DRT_E_INVALID_SECURITY_MODE => "The specified security mode is not one of the values defined in DRT_SECURITY_MODE.",
            P2P_INT::DRT_E_CAPABILITY_MISMATCH => "The requested security algorithms are not available.",
        }
    }

    pub fn from_name(name: &str) -> P2P_INT {
        return match name {
            "DRT_E_TIMEOUT" => P2P_INT::DRT_E_TIMEOUT,
            "DRT_E_INVALID_KEY_SIZE" => P2P_INT::DRT_E_INVALID_KEY_SIZE,
            "DRT_E_INVALID_CERT_CHAIN" => P2P_INT::DRT_E_INVALID_CERT_CHAIN,
            "DRT_E_INVALID_MESSAGE" => P2P_INT::DRT_E_INVALID_MESSAGE,
            "DRT_E_NO_MORE" => P2P_INT::DRT_E_NO_MORE,
            "DRT_E_INVALID_MAX_ADDRESSES" => P2P_INT::DRT_E_INVALID_MAX_ADDRESSES,
            "DRT_E_SEARCH_IN_PROGRESS" => P2P_INT::DRT_E_SEARCH_IN_PROGRESS,
            "DRT_E_INVALID_KEY" => P2P_INT::DRT_E_INVALID_KEY,
            "DRT_S_RETRY" => P2P_INT::DRT_S_RETRY,
            "DRT_E_INVALID_MAX_ENDPOINTS" => P2P_INT::DRT_E_INVALID_MAX_ENDPOINTS,
            "DRT_E_INVALID_SEARCH_RANGE" => P2P_INT::DRT_E_INVALID_SEARCH_RANGE,
            "DRT_E_INVALID_PORT" => P2P_INT::DRT_E_INVALID_PORT,
            "DRT_E_INVALID_TRANSPORT_PROVIDER" => P2P_INT::DRT_E_INVALID_TRANSPORT_PROVIDER,
            "DRT_E_INVALID_SECURITY_PROVIDER" => P2P_INT::DRT_E_INVALID_SECURITY_PROVIDER,
            "DRT_E_STILL_IN_USE" => P2P_INT::DRT_E_STILL_IN_USE,
            "DRT_E_INVALID_BOOTSTRAP_PROVIDER" => P2P_INT::DRT_E_INVALID_BOOTSTRAP_PROVIDER,
            "DRT_E_INVALID_ADDRESS" => P2P_INT::DRT_E_INVALID_ADDRESS,
            "DRT_E_INVALID_SCOPE" => P2P_INT::DRT_E_INVALID_SCOPE,
            "DRT_E_TRANSPORT_SHUTTING_DOWN" => P2P_INT::DRT_E_TRANSPORT_SHUTTING_DOWN,
            "DRT_E_NO_ADDRESSES_AVAILABLE" => P2P_INT::DRT_E_NO_ADDRESSES_AVAILABLE,
            "DRT_E_DUPLICATE_KEY" => P2P_INT::DRT_E_DUPLICATE_KEY,
            "DRT_E_TRANSPORTPROVIDER_IN_USE" => P2P_INT::DRT_E_TRANSPORTPROVIDER_IN_USE,
            "DRT_E_TRANSPORTPROVIDER_NOT_ATTACHED" => P2P_INT::DRT_E_TRANSPORTPROVIDER_NOT_ATTACHED,
            "DRT_E_SECURITYPROVIDER_IN_USE" => P2P_INT::DRT_E_SECURITYPROVIDER_IN_USE,
            "DRT_E_SECURITYPROVIDER_NOT_ATTACHED" => P2P_INT::DRT_E_SECURITYPROVIDER_NOT_ATTACHED,
            "DRT_E_BOOTSTRAPPROVIDER_IN_USE" => P2P_INT::DRT_E_BOOTSTRAPPROVIDER_IN_USE,
            "DRT_E_BOOTSTRAPPROVIDER_NOT_ATTACHED" => P2P_INT::DRT_E_BOOTSTRAPPROVIDER_NOT_ATTACHED,
            "DRT_E_TRANSPORT_ALREADY_BOUND" => P2P_INT::DRT_E_TRANSPORT_ALREADY_BOUND,
            "DRT_E_TRANSPORT_NOT_BOUND" => P2P_INT::DRT_E_TRANSPORT_NOT_BOUND,
            "DRT_E_TRANSPORT_UNEXPECTED" => P2P_INT::DRT_E_TRANSPORT_UNEXPECTED,
            "DRT_E_TRANSPORT_INVALID_ARGUMENT" => P2P_INT::DRT_E_TRANSPORT_INVALID_ARGUMENT,
            "DRT_E_TRANSPORT_NO_DEST_ADDRESSES" => P2P_INT::DRT_E_TRANSPORT_NO_DEST_ADDRESSES,
            "DRT_E_TRANSPORT_EXECUTING_CALLBACK" => P2P_INT::DRT_E_TRANSPORT_EXECUTING_CALLBACK,
            "DRT_E_TRANSPORT_ALREADY_EXISTS_FOR_SCOPE" => P2P_INT::DRT_E_TRANSPORT_ALREADY_EXISTS_FOR_SCOPE,
            "DRT_E_INVALID_SETTINGS" => P2P_INT::DRT_E_INVALID_SETTINGS,
            "DRT_E_INVALID_SEARCH_INFO" => P2P_INT::DRT_E_INVALID_SEARCH_INFO,
            "DRT_E_FAULTED" => P2P_INT::DRT_E_FAULTED,
            "DRT_E_TRANSPORT_STILL_BOUND" => P2P_INT::DRT_E_TRANSPORT_STILL_BOUND,
            "DRT_E_INSUFFICIENT_BUFFER" => P2P_INT::DRT_E_INSUFFICIENT_BUFFER,
            "DRT_E_INVALID_INSTANCE_PREFIX" => P2P_INT::DRT_E_INVALID_INSTANCE_PREFIX,
            "DRT_E_INVALID_SECURITY_MODE" => P2P_INT::DRT_E_INVALID_SECURITY_MODE,
            "DRT_E_CAPABILITY_MISMATCH" => P2P_INT::DRT_E_CAPABILITY_MISMATCH,
        }
    }
    pub fn from_code(code: u32) -> P2P_INT {
        return match code {
            0x80621001 => P2P_INT::DRT_E_TIMEOUT,
            0x80621002 => P2P_INT::DRT_E_INVALID_KEY_SIZE,
            0x80621004 => P2P_INT::DRT_E_INVALID_CERT_CHAIN,
            0x80621005 => P2P_INT::DRT_E_INVALID_MESSAGE,
            0x80621006 => P2P_INT::DRT_E_NO_MORE,
            0x80621007 => P2P_INT::DRT_E_INVALID_MAX_ADDRESSES,
            0x80621008 => P2P_INT::DRT_E_SEARCH_IN_PROGRESS,
            0x80621009 => P2P_INT::DRT_E_INVALID_KEY,
            0x80621010 => P2P_INT::DRT_S_RETRY,
            0x80621011 => P2P_INT::DRT_E_INVALID_MAX_ENDPOINTS,
            0x80621012 => P2P_INT::DRT_E_INVALID_SEARCH_RANGE,
            0x80622000 => P2P_INT::DRT_E_INVALID_PORT,
            0x80622001 => P2P_INT::DRT_E_INVALID_TRANSPORT_PROVIDER,
            0x80622002 => P2P_INT::DRT_E_INVALID_SECURITY_PROVIDER,
            0x80622003 => P2P_INT::DRT_E_STILL_IN_USE,
            0x80622004 => P2P_INT::DRT_E_INVALID_BOOTSTRAP_PROVIDER,
            0x80622005 => P2P_INT::DRT_E_INVALID_ADDRESS,
            0x80622006 => P2P_INT::DRT_E_INVALID_SCOPE,
            0x80622007 => P2P_INT::DRT_E_TRANSPORT_SHUTTING_DOWN,
            0x80622008 => P2P_INT::DRT_E_NO_ADDRESSES_AVAILABLE,
            0x80622009 => P2P_INT::DRT_E_DUPLICATE_KEY,
            0x8062200A => P2P_INT::DRT_E_TRANSPORTPROVIDER_IN_USE,
            0x8062200B => P2P_INT::DRT_E_TRANSPORTPROVIDER_NOT_ATTACHED,
            0x8062200C => P2P_INT::DRT_E_SECURITYPROVIDER_IN_USE,
            0x8062200D => P2P_INT::DRT_E_SECURITYPROVIDER_NOT_ATTACHED,
            0x8062200E => P2P_INT::DRT_E_BOOTSTRAPPROVIDER_IN_USE,
            0x8062200F => P2P_INT::DRT_E_BOOTSTRAPPROVIDER_NOT_ATTACHED,
            0x80622101 => P2P_INT::DRT_E_TRANSPORT_ALREADY_BOUND,
            0x80622102 => P2P_INT::DRT_E_TRANSPORT_NOT_BOUND,
            0x80622103 => P2P_INT::DRT_E_TRANSPORT_UNEXPECTED,
            0x80622104 => P2P_INT::DRT_E_TRANSPORT_INVALID_ARGUMENT,
            0x80622105 => P2P_INT::DRT_E_TRANSPORT_NO_DEST_ADDRESSES,
            0x80622106 => P2P_INT::DRT_E_TRANSPORT_EXECUTING_CALLBACK,
            0x80622107 => P2P_INT::DRT_E_TRANSPORT_ALREADY_EXISTS_FOR_SCOPE,
            0x80622108 => P2P_INT::DRT_E_INVALID_SETTINGS,
            0x80622109 => P2P_INT::DRT_E_INVALID_SEARCH_INFO,
            0x8062210A => P2P_INT::DRT_E_FAULTED,
            0x8062210B => P2P_INT::DRT_E_TRANSPORT_STILL_BOUND,
            0x8062210C => P2P_INT::DRT_E_INSUFFICIENT_BUFFER,
            0x8062210D => P2P_INT::DRT_E_INVALID_INSTANCE_PREFIX,
            0x8062210E => P2P_INT::DRT_E_INVALID_SECURITY_MODE,
            0x8062210F => P2P_INT::DRT_E_CAPABILITY_MISMATCH,
        }
    }
}
