/*
 * Copyright 2023 Stanislav Mikhailov (xavetar)
 *
 * Licensed under the Creative Commons Zero v1.0 Universal (CC0) License.
 * You may obtain a copy of the License at
 *
 *     http://creativecommons.org/publicdomain/zero/1.0/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the CC0 license is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// It is strictly forbidden to use the from_code method, respected Microsoft decided to define
// duplicate codes, but by defining different constant names, therefore, when using these codes,
// it is mandatory to use through from_name, and not from_code. Otherwise, it may cause undefined
// behavior or an unknown exception. Because one code corresponds to several constants.

use super::{RawError};

#[derive(Clone, Debug, Copy, Eq, Hash, Ord, PartialEq, PartialOrd)]
pub enum SIP_STATUS_CODE {
    RTC_E_STATUS_INFO_TRYING,
    RTC_E_STATUS_INFO_RINGING,
    RTC_E_STATUS_INFO_CALL_FORWARDING,
    RTC_E_STATUS_INFO_QUEUED,
    RTC_E_STATUS_SESSION_PROGRESS,
    RTC_E_STATUS_SUCCESS,
    RTC_E_STATUS_REDIRECT_MULTIPLE_CHOICES,
    RTC_E_STATUS_REDIRECT_MOVED_PERMANENTLY,
    RTC_E_STATUS_REDIRECT_MOVED_TEMPORARILY,
    RTC_E_STATUS_REDIRECT_SEE_OTHER,
    RTC_E_STATUS_REDIRECT_USE_PROXY,
    RTC_E_STATUS_REDIRECT_ALTERNATIVE_SERVICE,
    RTC_E_STATUS_CLIENT_BAD_REQUEST,
    RTC_E_STATUS_CLIENT_UNAUTHORIZED,
    RTC_E_STATUS_CLIENT_PAYMENT_REQUIRED,
    RTC_E_STATUS_CLIENT_FORBIDDEN,
    RTC_E_STATUS_CLIENT_NOT_FOUND,
    RTC_E_STATUS_CLIENT_METHOD_NOT_ALLOWED,
    RTC_E_STATUS_CLIENT_NOT_ACCEPTABLE,
    RTC_E_STATUS_CLIENT_PROXY_AUTHENTICATION_REQUIRED,
    RTC_E_STATUS_CLIENT_REQUEST_TIMEOUT,
    RTC_E_STATUS_CLIENT_CONFLICT,
    RTC_E_STATUS_CLIENT_GONE,
    RTC_E_STATUS_CLIENT_LENGTH_REQUIRED,
    RTC_E_STATUS_CLIENT_REQUEST_ENTITY_TOO_LARGE,
    RTC_E_STATUS_CLIENT_REQUEST_URI_TOO_LARGE,
    RTC_E_STATUS_CLIENT_UNSUPPORTED_MEDIA_TYPE,
    RTC_E_STATUS_CLIENT_BAD_EXTENSION,
    RTC_E_STATUS_CLIENT_TEMPORARILY_NOT_AVAILABLE,
    RTC_E_STATUS_CLIENT_TRANSACTION_DOES_NOT_EXIST,
    RTC_E_STATUS_CLIENT_LOOP_DETECTED,
    RTC_E_STATUS_CLIENT_TOO_MANY_HOPS,
    RTC_E_STATUS_CLIENT_ADDRESS_INCOMPLETE,
    RTC_E_STATUS_CLIENT_AMBIGUOUS,
    RTC_E_STATUS_CLIENT_BUSY_HERE,
    RTC_E_STATUS_REQUEST_TERMINATED,
    RTC_E_STATUS_NOT_ACCEPTABLE_HERE,
    RTC_E_STATUS_SERVER_INTERNAL_ERROR,
    RTC_E_STATUS_SERVER_NOT_IMPLEMENTED,
    RTC_E_STATUS_SERVER_BAD_GATEWAY,
    RTC_E_STATUS_SERVER_SERVICE_UNAVAILABLE,
    RTC_E_STATUS_SERVER_SERVER_TIMEOUT,
    RTC_E_STATUS_SERVER_VERSION_NOT_SUPPORTED,
    RTC_E_STATUS_GLOBAL_BUSY_EVERYWHERE,
    RTC_E_STATUS_GLOBAL_DECLINE,
    RTC_E_STATUS_GLOBAL_DOES_NOT_EXIST_ANYWHERE,
    RTC_E_STATUS_GLOBAL_NOT_ACCEPTABLE,
}

impl SIP_STATUS_CODE {
    pub fn code(&self) -> u32 {
        return match self {
            SIP_STATUS_CODE::RTC_E_STATUS_INFO_TRYING => 0x00EF0064 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_INFO_RINGING => 0x00EF00B4 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_INFO_CALL_FORWARDING => 0x00EF00B5 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_INFO_QUEUED => 0x00EF00B6 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_SESSION_PROGRESS => 0x00EF00B7 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_SUCCESS => 0x00EF00C8 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_MULTIPLE_CHOICES => 0x80EF012C as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_MOVED_PERMANENTLY => 0x80EF012D as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_MOVED_TEMPORARILY => 0x80EF012E as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_SEE_OTHER => 0x80EF012F as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_USE_PROXY => 0x80EF0131 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_ALTERNATIVE_SERVICE => 0x80EF017C as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_BAD_REQUEST => 0x80EF0190 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_UNAUTHORIZED => 0x80EF0191 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_PAYMENT_REQUIRED => 0x80EF0192 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_FORBIDDEN => 0x80EF0193 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_NOT_FOUND => 0x80EF0194 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_METHOD_NOT_ALLOWED => 0x80EF0195 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_NOT_ACCEPTABLE => 0x80EF0196 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_PROXY_AUTHENTICATION_REQUIRED => 0x80EF0197 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_REQUEST_TIMEOUT => 0x80EF0198 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_CONFLICT => 0x80EF0199 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_GONE => 0x80EF019A as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_LENGTH_REQUIRED => 0x80EF019B as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_REQUEST_ENTITY_TOO_LARGE => 0x80EF019D as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_REQUEST_URI_TOO_LARGE => 0x80EF019E as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_UNSUPPORTED_MEDIA_TYPE => 0x80EF019F as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_BAD_EXTENSION => 0x80EF01A4 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_TEMPORARILY_NOT_AVAILABLE => 0x80EF01E0 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_TRANSACTION_DOES_NOT_EXIST => 0x80EF01E1 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_LOOP_DETECTED => 0x80EF01E2 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_TOO_MANY_HOPS => 0x80EF01E3 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_ADDRESS_INCOMPLETE => 0x80EF01E4 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_AMBIGUOUS => 0x80EF01E5 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_BUSY_HERE => 0x80EF01E6 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_REQUEST_TERMINATED => 0x80EF01E7 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_NOT_ACCEPTABLE_HERE => 0x80EF01E8 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_SERVER_INTERNAL_ERROR => 0x80EF01F4 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_SERVER_NOT_IMPLEMENTED => 0x80EF01F5 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_SERVER_BAD_GATEWAY => 0x80EF01F6 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_SERVER_SERVICE_UNAVAILABLE => 0x80EF01F7 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_SERVER_SERVER_TIMEOUT => 0x80EF01F8 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_SERVER_VERSION_NOT_SUPPORTED => 0x80EF01F9 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_GLOBAL_BUSY_EVERYWHERE => 0x80EF0258 as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_GLOBAL_DECLINE => 0x80EF025B as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_GLOBAL_DOES_NOT_EXIST_ANYWHERE => 0x80EF025C as u32,
            SIP_STATUS_CODE::RTC_E_STATUS_GLOBAL_NOT_ACCEPTABLE => 0x80EF025E as u32,
        }
    }

    pub fn error(&self) -> RawError {
        return match self {
            SIP_STATUS_CODE::RTC_E_STATUS_INFO_TRYING => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_INFO_TRYING),
            SIP_STATUS_CODE::RTC_E_STATUS_INFO_RINGING => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_INFO_RINGING),
            SIP_STATUS_CODE::RTC_E_STATUS_INFO_CALL_FORWARDING => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_INFO_CALL_FORWARDING),
            SIP_STATUS_CODE::RTC_E_STATUS_INFO_QUEUED => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_INFO_QUEUED),
            SIP_STATUS_CODE::RTC_E_STATUS_SESSION_PROGRESS => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_SESSION_PROGRESS),
            SIP_STATUS_CODE::RTC_E_STATUS_SUCCESS => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_SUCCESS),
            SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_MULTIPLE_CHOICES => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_MULTIPLE_CHOICES),
            SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_MOVED_PERMANENTLY => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_MOVED_PERMANENTLY),
            SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_MOVED_TEMPORARILY => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_MOVED_TEMPORARILY),
            SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_SEE_OTHER => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_SEE_OTHER),
            SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_USE_PROXY => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_USE_PROXY),
            SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_ALTERNATIVE_SERVICE => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_ALTERNATIVE_SERVICE),
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_BAD_REQUEST => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_BAD_REQUEST),
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_UNAUTHORIZED => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_UNAUTHORIZED),
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_PAYMENT_REQUIRED => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_PAYMENT_REQUIRED),
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_FORBIDDEN => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_FORBIDDEN),
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_NOT_FOUND => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_NOT_FOUND),
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_METHOD_NOT_ALLOWED => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_METHOD_NOT_ALLOWED),
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_NOT_ACCEPTABLE => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_NOT_ACCEPTABLE),
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_PROXY_AUTHENTICATION_REQUIRED => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_PROXY_AUTHENTICATION_REQUIRED),
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_REQUEST_TIMEOUT => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_REQUEST_TIMEOUT),
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_CONFLICT => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_CONFLICT),
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_GONE => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_GONE),
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_LENGTH_REQUIRED => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_LENGTH_REQUIRED),
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_REQUEST_ENTITY_TOO_LARGE => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_REQUEST_ENTITY_TOO_LARGE),
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_REQUEST_URI_TOO_LARGE => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_REQUEST_URI_TOO_LARGE),
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_UNSUPPORTED_MEDIA_TYPE => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_UNSUPPORTED_MEDIA_TYPE),
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_BAD_EXTENSION => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_BAD_EXTENSION),
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_TEMPORARILY_NOT_AVAILABLE => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_TEMPORARILY_NOT_AVAILABLE),
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_TRANSACTION_DOES_NOT_EXIST => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_TRANSACTION_DOES_NOT_EXIST),
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_LOOP_DETECTED => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_LOOP_DETECTED),
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_TOO_MANY_HOPS => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_TOO_MANY_HOPS),
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_ADDRESS_INCOMPLETE => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_ADDRESS_INCOMPLETE),
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_AMBIGUOUS => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_AMBIGUOUS),
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_BUSY_HERE => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_BUSY_HERE),
            SIP_STATUS_CODE::RTC_E_STATUS_REQUEST_TERMINATED => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_REQUEST_TERMINATED),
            SIP_STATUS_CODE::RTC_E_STATUS_NOT_ACCEPTABLE_HERE => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_NOT_ACCEPTABLE_HERE),
            SIP_STATUS_CODE::RTC_E_STATUS_SERVER_INTERNAL_ERROR => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_SERVER_INTERNAL_ERROR),
            SIP_STATUS_CODE::RTC_E_STATUS_SERVER_NOT_IMPLEMENTED => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_SERVER_NOT_IMPLEMENTED),
            SIP_STATUS_CODE::RTC_E_STATUS_SERVER_BAD_GATEWAY => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_SERVER_BAD_GATEWAY),
            SIP_STATUS_CODE::RTC_E_STATUS_SERVER_SERVICE_UNAVAILABLE => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_SERVER_SERVICE_UNAVAILABLE),
            SIP_STATUS_CODE::RTC_E_STATUS_SERVER_SERVER_TIMEOUT => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_SERVER_SERVER_TIMEOUT),
            SIP_STATUS_CODE::RTC_E_STATUS_SERVER_VERSION_NOT_SUPPORTED => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_SERVER_VERSION_NOT_SUPPORTED),
            SIP_STATUS_CODE::RTC_E_STATUS_GLOBAL_BUSY_EVERYWHERE => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_GLOBAL_BUSY_EVERYWHERE),
            SIP_STATUS_CODE::RTC_E_STATUS_GLOBAL_DECLINE => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_GLOBAL_DECLINE),
            SIP_STATUS_CODE::RTC_E_STATUS_GLOBAL_DOES_NOT_EXIST_ANYWHERE => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_GLOBAL_DOES_NOT_EXIST_ANYWHERE),
            SIP_STATUS_CODE::RTC_E_STATUS_GLOBAL_NOT_ACCEPTABLE => RawError::Kind(SIP_STATUS_CODE::RTC_E_STATUS_GLOBAL_NOT_ACCEPTABLE),
        }
    }

    pub fn description(&self) -> &'static str {
        return match self {
            SIP_STATUS_CODE::RTC_E_STATUS_INFO_TRYING => "Trying",
            SIP_STATUS_CODE::RTC_E_STATUS_INFO_RINGING => "Ringing",
            SIP_STATUS_CODE::RTC_E_STATUS_INFO_CALL_FORWARDING => "Call Is Being Forwarded",
            SIP_STATUS_CODE::RTC_E_STATUS_INFO_QUEUED => "Queued",
            SIP_STATUS_CODE::RTC_E_STATUS_SESSION_PROGRESS => "Session Progress",
            SIP_STATUS_CODE::RTC_E_STATUS_SUCCESS => "OK",
            SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_MULTIPLE_CHOICES => "Multiple Choices",
            SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_MOVED_PERMANENTLY => "Moved Permanently",
            SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_MOVED_TEMPORARILY => "Moved Temporarily",
            SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_SEE_OTHER => "See Other",
            SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_USE_PROXY => "Use Proxy",
            SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_ALTERNATIVE_SERVICE => "Alternative Service",
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_BAD_REQUEST => "Bad Request",
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_UNAUTHORIZED => "Unauthorized",
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_PAYMENT_REQUIRED => "Payment Required",
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_FORBIDDEN => "Forbidden",
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_NOT_FOUND => "Not Found",
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_METHOD_NOT_ALLOWED => "Method Not Allowed",
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_NOT_ACCEPTABLE => "Not Acceptable",
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_PROXY_AUTHENTICATION_REQUIRED => "Proxy Authentication Required",
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_REQUEST_TIMEOUT => "Request Timeout",
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_CONFLICT => "Conflict",
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_GONE => "Gone",
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_LENGTH_REQUIRED => "Length Required",
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_REQUEST_ENTITY_TOO_LARGE => "Request Entity Too Large",
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_REQUEST_URI_TOO_LARGE => "Request-URI Too Long",
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_UNSUPPORTED_MEDIA_TYPE => "Unsupported Media Type",
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_BAD_EXTENSION => "Bad Extension",
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_TEMPORARILY_NOT_AVAILABLE => "Temporarily Unavailable",
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_TRANSACTION_DOES_NOT_EXIST => "Call Leg/Transaction Does Not Exist",
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_LOOP_DETECTED => "Loop Detected",
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_TOO_MANY_HOPS => "Too Many Hops",
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_ADDRESS_INCOMPLETE => "Address Incomplete",
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_AMBIGUOUS => "Ambiguous",
            SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_BUSY_HERE => "Busy Here",
            SIP_STATUS_CODE::RTC_E_STATUS_REQUEST_TERMINATED => "Request Terminated",
            SIP_STATUS_CODE::RTC_E_STATUS_NOT_ACCEPTABLE_HERE => "Not Acceptable Here",
            SIP_STATUS_CODE::RTC_E_STATUS_SERVER_INTERNAL_ERROR => "Server Internal Error",
            SIP_STATUS_CODE::RTC_E_STATUS_SERVER_NOT_IMPLEMENTED => "Not Implemented",
            SIP_STATUS_CODE::RTC_E_STATUS_SERVER_BAD_GATEWAY => "Bad Gateway",
            SIP_STATUS_CODE::RTC_E_STATUS_SERVER_SERVICE_UNAVAILABLE => "Service Unavailable",
            SIP_STATUS_CODE::RTC_E_STATUS_SERVER_SERVER_TIMEOUT => "Server Time-out",
            SIP_STATUS_CODE::RTC_E_STATUS_SERVER_VERSION_NOT_SUPPORTED => "Version Not Supported",
            SIP_STATUS_CODE::RTC_E_STATUS_GLOBAL_BUSY_EVERYWHERE => "Busy Everywhere",
            SIP_STATUS_CODE::RTC_E_STATUS_GLOBAL_DECLINE => "Decline",
            SIP_STATUS_CODE::RTC_E_STATUS_GLOBAL_DOES_NOT_EXIST_ANYWHERE => "Does Not Exist Anywhere",
            SIP_STATUS_CODE::RTC_E_STATUS_GLOBAL_NOT_ACCEPTABLE => "Not Acceptable",
        }
    }

    pub fn from_name(name: &str) -> SIP_STATUS_CODE {
        return match name {
            "RTC_E_STATUS_INFO_TRYING" => SIP_STATUS_CODE::RTC_E_STATUS_INFO_TRYING,
            "RTC_E_STATUS_INFO_RINGING" => SIP_STATUS_CODE::RTC_E_STATUS_INFO_RINGING,
            "RTC_E_STATUS_INFO_CALL_FORWARDING" => SIP_STATUS_CODE::RTC_E_STATUS_INFO_CALL_FORWARDING,
            "RTC_E_STATUS_INFO_QUEUED" => SIP_STATUS_CODE::RTC_E_STATUS_INFO_QUEUED,
            "RTC_E_STATUS_SESSION_PROGRESS" => SIP_STATUS_CODE::RTC_E_STATUS_SESSION_PROGRESS,
            "RTC_E_STATUS_SUCCESS" => SIP_STATUS_CODE::RTC_E_STATUS_SUCCESS,
            "RTC_E_STATUS_REDIRECT_MULTIPLE_CHOICES" => SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_MULTIPLE_CHOICES,
            "RTC_E_STATUS_REDIRECT_MOVED_PERMANENTLY" => SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_MOVED_PERMANENTLY,
            "RTC_E_STATUS_REDIRECT_MOVED_TEMPORARILY" => SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_MOVED_TEMPORARILY,
            "RTC_E_STATUS_REDIRECT_SEE_OTHER" => SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_SEE_OTHER,
            "RTC_E_STATUS_REDIRECT_USE_PROXY" => SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_USE_PROXY,
            "RTC_E_STATUS_REDIRECT_ALTERNATIVE_SERVICE" => SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_ALTERNATIVE_SERVICE,
            "RTC_E_STATUS_CLIENT_BAD_REQUEST" => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_BAD_REQUEST,
            "RTC_E_STATUS_CLIENT_UNAUTHORIZED" => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_UNAUTHORIZED,
            "RTC_E_STATUS_CLIENT_PAYMENT_REQUIRED" => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_PAYMENT_REQUIRED,
            "RTC_E_STATUS_CLIENT_FORBIDDEN" => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_FORBIDDEN,
            "RTC_E_STATUS_CLIENT_NOT_FOUND" => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_NOT_FOUND,
            "RTC_E_STATUS_CLIENT_METHOD_NOT_ALLOWED" => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_METHOD_NOT_ALLOWED,
            "RTC_E_STATUS_CLIENT_NOT_ACCEPTABLE" => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_NOT_ACCEPTABLE,
            "RTC_E_STATUS_CLIENT_PROXY_AUTHENTICATION_REQUIRED" => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_PROXY_AUTHENTICATION_REQUIRED,
            "RTC_E_STATUS_CLIENT_REQUEST_TIMEOUT" => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_REQUEST_TIMEOUT,
            "RTC_E_STATUS_CLIENT_CONFLICT" => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_CONFLICT,
            "RTC_E_STATUS_CLIENT_GONE" => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_GONE,
            "RTC_E_STATUS_CLIENT_LENGTH_REQUIRED" => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_LENGTH_REQUIRED,
            "RTC_E_STATUS_CLIENT_REQUEST_ENTITY_TOO_LARGE" => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_REQUEST_ENTITY_TOO_LARGE,
            "RTC_E_STATUS_CLIENT_REQUEST_URI_TOO_LARGE" => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_REQUEST_URI_TOO_LARGE,
            "RTC_E_STATUS_CLIENT_UNSUPPORTED_MEDIA_TYPE" => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_UNSUPPORTED_MEDIA_TYPE,
            "RTC_E_STATUS_CLIENT_BAD_EXTENSION" => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_BAD_EXTENSION,
            "RTC_E_STATUS_CLIENT_TEMPORARILY_NOT_AVAILABLE" => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_TEMPORARILY_NOT_AVAILABLE,
            "RTC_E_STATUS_CLIENT_TRANSACTION_DOES_NOT_EXIST" => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_TRANSACTION_DOES_NOT_EXIST,
            "RTC_E_STATUS_CLIENT_LOOP_DETECTED" => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_LOOP_DETECTED,
            "RTC_E_STATUS_CLIENT_TOO_MANY_HOPS" => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_TOO_MANY_HOPS,
            "RTC_E_STATUS_CLIENT_ADDRESS_INCOMPLETE" => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_ADDRESS_INCOMPLETE,
            "RTC_E_STATUS_CLIENT_AMBIGUOUS" => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_AMBIGUOUS,
            "RTC_E_STATUS_CLIENT_BUSY_HERE" => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_BUSY_HERE,
            "RTC_E_STATUS_REQUEST_TERMINATED" => SIP_STATUS_CODE::RTC_E_STATUS_REQUEST_TERMINATED,
            "RTC_E_STATUS_NOT_ACCEPTABLE_HERE" => SIP_STATUS_CODE::RTC_E_STATUS_NOT_ACCEPTABLE_HERE,
            "RTC_E_STATUS_SERVER_INTERNAL_ERROR" => SIP_STATUS_CODE::RTC_E_STATUS_SERVER_INTERNAL_ERROR,
            "RTC_E_STATUS_SERVER_NOT_IMPLEMENTED" => SIP_STATUS_CODE::RTC_E_STATUS_SERVER_NOT_IMPLEMENTED,
            "RTC_E_STATUS_SERVER_BAD_GATEWAY" => SIP_STATUS_CODE::RTC_E_STATUS_SERVER_BAD_GATEWAY,
            "RTC_E_STATUS_SERVER_SERVICE_UNAVAILABLE" => SIP_STATUS_CODE::RTC_E_STATUS_SERVER_SERVICE_UNAVAILABLE,
            "RTC_E_STATUS_SERVER_SERVER_TIMEOUT" => SIP_STATUS_CODE::RTC_E_STATUS_SERVER_SERVER_TIMEOUT,
            "RTC_E_STATUS_SERVER_VERSION_NOT_SUPPORTED" => SIP_STATUS_CODE::RTC_E_STATUS_SERVER_VERSION_NOT_SUPPORTED,
            "RTC_E_STATUS_GLOBAL_BUSY_EVERYWHERE" => SIP_STATUS_CODE::RTC_E_STATUS_GLOBAL_BUSY_EVERYWHERE,
            "RTC_E_STATUS_GLOBAL_DECLINE" => SIP_STATUS_CODE::RTC_E_STATUS_GLOBAL_DECLINE,
            "RTC_E_STATUS_GLOBAL_DOES_NOT_EXIST_ANYWHERE" => SIP_STATUS_CODE::RTC_E_STATUS_GLOBAL_DOES_NOT_EXIST_ANYWHERE,
            "RTC_E_STATUS_GLOBAL_NOT_ACCEPTABLE" => SIP_STATUS_CODE::RTC_E_STATUS_GLOBAL_NOT_ACCEPTABLE,
        }
    }
    pub fn from_code(code: u32) -> SIP_STATUS_CODE {
        return match code {
            0x00EF0064 => SIP_STATUS_CODE::RTC_E_STATUS_INFO_TRYING,
            0x00EF00B4 => SIP_STATUS_CODE::RTC_E_STATUS_INFO_RINGING,
            0x00EF00B5 => SIP_STATUS_CODE::RTC_E_STATUS_INFO_CALL_FORWARDING,
            0x00EF00B6 => SIP_STATUS_CODE::RTC_E_STATUS_INFO_QUEUED,
            0x00EF00B7 => SIP_STATUS_CODE::RTC_E_STATUS_SESSION_PROGRESS,
            0x00EF00C8 => SIP_STATUS_CODE::RTC_E_STATUS_SUCCESS,
            0x80EF012C => SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_MULTIPLE_CHOICES,
            0x80EF012D => SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_MOVED_PERMANENTLY,
            0x80EF012E => SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_MOVED_TEMPORARILY,
            0x80EF012F => SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_SEE_OTHER,
            0x80EF0131 => SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_USE_PROXY,
            0x80EF017C => SIP_STATUS_CODE::RTC_E_STATUS_REDIRECT_ALTERNATIVE_SERVICE,
            0x80EF0190 => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_BAD_REQUEST,
            0x80EF0191 => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_UNAUTHORIZED,
            0x80EF0192 => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_PAYMENT_REQUIRED,
            0x80EF0193 => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_FORBIDDEN,
            0x80EF0194 => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_NOT_FOUND,
            0x80EF0195 => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_METHOD_NOT_ALLOWED,
            0x80EF0196 => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_NOT_ACCEPTABLE,
            0x80EF0197 => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_PROXY_AUTHENTICATION_REQUIRED,
            0x80EF0198 => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_REQUEST_TIMEOUT,
            0x80EF0199 => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_CONFLICT,
            0x80EF019A => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_GONE,
            0x80EF019B => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_LENGTH_REQUIRED,
            0x80EF019D => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_REQUEST_ENTITY_TOO_LARGE,
            0x80EF019E => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_REQUEST_URI_TOO_LARGE,
            0x80EF019F => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_UNSUPPORTED_MEDIA_TYPE,
            0x80EF01A4 => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_BAD_EXTENSION,
            0x80EF01E0 => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_TEMPORARILY_NOT_AVAILABLE,
            0x80EF01E1 => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_TRANSACTION_DOES_NOT_EXIST,
            0x80EF01E2 => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_LOOP_DETECTED,
            0x80EF01E3 => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_TOO_MANY_HOPS,
            0x80EF01E4 => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_ADDRESS_INCOMPLETE,
            0x80EF01E5 => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_AMBIGUOUS,
            0x80EF01E6 => SIP_STATUS_CODE::RTC_E_STATUS_CLIENT_BUSY_HERE,
            0x80EF01E7 => SIP_STATUS_CODE::RTC_E_STATUS_REQUEST_TERMINATED,
            0x80EF01E8 => SIP_STATUS_CODE::RTC_E_STATUS_NOT_ACCEPTABLE_HERE,
            0x80EF01F4 => SIP_STATUS_CODE::RTC_E_STATUS_SERVER_INTERNAL_ERROR,
            0x80EF01F5 => SIP_STATUS_CODE::RTC_E_STATUS_SERVER_NOT_IMPLEMENTED,
            0x80EF01F6 => SIP_STATUS_CODE::RTC_E_STATUS_SERVER_BAD_GATEWAY,
            0x80EF01F7 => SIP_STATUS_CODE::RTC_E_STATUS_SERVER_SERVICE_UNAVAILABLE,
            0x80EF01F8 => SIP_STATUS_CODE::RTC_E_STATUS_SERVER_SERVER_TIMEOUT,
            0x80EF01F9 => SIP_STATUS_CODE::RTC_E_STATUS_SERVER_VERSION_NOT_SUPPORTED,
            0x80EF0258 => SIP_STATUS_CODE::RTC_E_STATUS_GLOBAL_BUSY_EVERYWHERE,
            0x80EF025B => SIP_STATUS_CODE::RTC_E_STATUS_GLOBAL_DECLINE,
            0x80EF025C => SIP_STATUS_CODE::RTC_E_STATUS_GLOBAL_DOES_NOT_EXIST_ANYWHERE,
            0x80EF025E => SIP_STATUS_CODE::RTC_E_STATUS_GLOBAL_NOT_ACCEPTABLE,
        }
    }
}
