/*
 * Copyright 2023 Stanislav Mikhailov (xavetar)
 *
 * Licensed under the Creative Commons Zero v1.0 Universal (CC0) License.
 * You may obtain a copy of the License at
 *
 *     http://creativecommons.org/publicdomain/zero/1.0/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the CC0 license is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// It is strictly forbidden to use the from_code method, respected Microsoft decided to define
// duplicate codes, but by defining different constant names, therefore, when using these codes,
// it is mandatory to use through from_name, and not from_code. Otherwise, it may cause undefined
// behavior or an unknown exception. Because one code corresponds to several constants.

use super::{RawError};

#[derive(Clone, Debug, Copy, Eq, Hash, Ord, PartialEq, PartialOrd)]
pub enum TPM_SERVICES {
    TPM_E_ERROR_MASK,
    TPM_E_AUTHFAIL,
    E_INK_EXCEPTION,
    TPM_E_BADINDEX,
    E_INK_MISMATCHED_INK_OBJECT,
    TPM_E_BAD_PARAMETER,
    E_INK_COLLECTOR_BUSY,
    TPM_E_AUDITFAILURE,
    E_INK_INCOMPATIBLE_OBJECT,
    TPM_E_CLEAR_DISABLED,
    E_INK_WINDOW_NOT_SET,
    TPM_E_DEACTIVATED,
    E_INK_INVALID_MODE,
    TPM_E_DISABLED,
    E_INK_COLLECTOR_ENABLED,
    TPM_E_DISABLED_CMD,
    E_INK_NO_STROKES_TO_RECOGNIZE,
    TPM_E_FAIL,
    E_INK_EMPTY_RECOGNITION_RESULT,
    TPM_E_BAD_ORDINAL,
    TPM_E_INSTALL_DISABLED,
    TPM_E_INVALID_KEYHANDLE,
    TPM_E_KEYNOTFOUND,
    TPM_E_INAPPROPRIATE_ENC,
    TPM_E_MIGRATEFAIL,
    TPM_E_INVALID_PCR_INFO,
    E_INK_OVERLAPPING_INPUT_RECT,
    TPM_E_NOSPACE,
    TPM_E_NOSRK,
    TPM_E_NOTSEALED_BLOB,
    TPM_E_OWNER_SET,
    TPM_E_RESOURCES,
    TPM_E_SHORTRANDOM,
    TPM_E_SIZE,
    TPM_E_WRONGPCRVAL,
    TPM_E_BAD_PARAM_SIZE,
    TPM_E_SHA_THREAD,
    TPM_E_SHA_ERROR,
    TPM_E_FAILEDSELFTEST,
    TPM_E_AUTH2FAIL,
    TPM_E_BADTAG,
    TPM_E_IOERROR,
    TPM_E_ENCRYPT_ERROR,
    TPM_E_DECRYPT_ERROR,
    TPM_E_INVALID_AUTHHANDLE,
    TPM_E_NO_ENDORSEMENT,
    TPM_E_INVALID_KEYUSAGE,
    TPM_E_WRONG_ENTITYTYPE,
    TPM_E_INVALID_POSTINIT,
    TPM_E_INAPPROPRIATE_SIG,
    TPM_E_BAD_KEY_PROPERTY,
    TPM_E_BAD_MIGRATION,
    TPM_E_BAD_SCHEME,
    TPM_E_BAD_DATASIZE,
    TPM_E_BAD_MODE,
    TPM_E_BAD_PRESENCE,
    TPM_E_BAD_VERSION,
    TPM_E_NO_WRAP_TRANSPORT,
    TPM_E_AUDITFAIL_UNSUCCESSFUL,
    TPM_E_AUDITFAIL_SUCCESSFUL,
    TPM_E_NOTRESETABLE,
    TPM_E_NOTLOCAL,
    TPM_E_BAD_TYPE,
    TPM_E_INVALID_RESOURCE,
    TPM_E_NOTFIPS,
    TPM_E_INVALID_FAMILY,
    TPM_E_NO_NV_PERMISSION,
    TPM_E_REQUIRES_SIGN,
    TPM_E_KEY_NOTSUPPORTED,
    TPM_E_AUTH_CONFLICT,
    TPM_E_AREA_LOCKED,
    TPM_E_BAD_LOCALITY,
    TPM_E_READ_ONLY,
    TPM_E_PER_NOWRITE,
    TPM_E_FAMILYCOUNT,
    TPM_E_WRITE_LOCKED,
    TPM_E_BAD_ATTRIBUTES,
    TPM_E_INVALID_STRUCTURE,
    TPM_E_KEY_OWNER_CONTROL,
    TPM_E_BAD_COUNTER,
    TPM_E_NOT_FULLWRITE,
    TPM_E_CONTEXT_GAP,
    TPM_E_MAXNVWRITES,
    TPM_E_NOOPERATOR,
    TPM_E_RESOURCEMISSING,
    TPM_E_DELEGATE_LOCK,
    TPM_E_DELEGATE_FAMILY,
    TPM_E_DELEGATE_ADMIN,
    TPM_E_TRANSPORT_NOTEXCLUSIVE,
    TPM_E_OWNER_CONTROL,
    TPM_E_DAA_RESOURCES,
    TPM_E_DAA_INPUT_DATA0,
    TPM_E_DAA_INPUT_DATA1,
    TPM_E_DAA_ISSUER_SETTINGS,
    TPM_E_DAA_TPM_SETTINGS,
    TPM_E_DAA_STAGE,
    TPM_E_DAA_ISSUER_VALIDITY,
    TPM_E_DAA_WRONG_W,
    TPM_E_BAD_HANDLE,
    TPM_E_BAD_DELEGATE,
    TPM_E_BADCONTEXT,
    TPM_E_TOOMANYCONTEXTS,
    TPM_E_MA_TICKET_SIGNATURE,
    TPM_E_MA_DESTINATION,
    TPM_E_MA_SOURCE,
    TPM_E_MA_AUTHORITY,
    TPM_E_PERMANENTEK,
    TPM_E_BAD_SIGNATURE,
    TPM_E_NOCONTEXTSPACE,
    TPM_20_E_ASYMMETRIC,
    TPM_20_E_ATTRIBUTES,
    TPM_20_E_HASH,
    TPM_20_E_VALUE,
    TPM_20_E_HIERARCHY,
    TPM_20_E_KEY_SIZE,
    TPM_20_E_MGF,
    TPM_20_E_MODE,
    TPM_20_E_TYPE,
    TPM_20_E_HANDLE,
    TPM_20_E_KDF,
    TPM_20_E_RANGE,
    TPM_20_E_AUTH_FAIL,
    TPM_20_E_NONCE,
    TPM_20_E_PP,
    TPM_20_E_SCHEME,
    TPM_20_E_SIZE,
    TPM_20_E_SYMMETRIC,
    TPM_20_E_TAG,
    TPM_20_E_SELECTOR,
    TPM_20_E_INSUFFICIENT,
    TPM_20_E_SIGNATURE,
    TPM_20_E_KEY,
    TPM_20_E_POLICY_FAIL,
    TPM_20_E_INTEGRITY,
    TPM_20_E_TICKET,
    TPM_20_E_RESERVED_BITS,
    TPM_20_E_BAD_AUTH,
    TPM_20_E_EXPIRED,
    TPM_20_E_POLICY_CC,
    TPM_20_E_BINDING,
    TPM_20_E_CURVE,
    TPM_20_E_ECC_POINT,
    TPM_20_E_INITIALIZE,
    TPM_20_E_FAILURE,
    TPM_20_E_SEQUENCE,
    TPM_20_E_PRIVATE,
    TPM_20_E_HMAC,
    TPM_20_E_DISABLED,
    TPM_20_E_EXCLUSIVE,
    TPM_20_E_ECC_CURVE,
    TPM_20_E_AUTH_TYPE,
    TPM_20_E_AUTH_MISSING,
    TPM_20_E_POLICY,
    TPM_20_E_PCR,
    TPM_20_E_PCR_CHANGED,
    TPM_20_E_UPGRADE,
    TPM_20_E_TOO_MANY_CONTEXTS,
    TPM_20_E_AUTH_UNAVAILABLE,
    TPM_20_E_REBOOT,
    TPM_20_E_UNBALANCED,
    TPM_20_E_COMMAND_SIZE,
    TPM_20_E_COMMAND_CODE,
    TPM_20_E_AUTHSIZE,
    TPM_20_E_AUTH_CONTEXT,
    TPM_20_E_NV_RANGE,
    TPM_20_E_NV_SIZE,
    TPM_20_E_NV_LOCKED,
    TPM_20_E_NV_AUTHORIZATION,
    TPM_20_E_NV_UNINITIALIZED,
    TPM_20_E_NV_SPACE,
    TPM_20_E_NV_DEFINED,
    TPM_20_E_BAD_CONTEXT,
    TPM_20_E_CPHASH,
    TPM_20_E_PARENT,
    TPM_20_E_NEEDS_TEST,
    TPM_20_E_NO_RESULT,
    TPM_20_E_SENSITIVE,
    TPM_E_COMMAND_BLOCKED,
    TPM_E_INVALID_HANDLE,
    TPM_E_DUPLICATE_VHANDLE,
    TPM_E_EMBEDDED_COMMAND_BLOCKED,
    TPM_E_EMBEDDED_COMMAND_UNSUPPORTED,
    TPM_E_RETRY,
    TPM_E_NEEDS_SELFTEST,
    TPM_E_DOING_SELFTEST,
    TPM_E_DEFEND_LOCK_RUNNING,
    TPM_20_E_CONTEXT_GAP,
    TPM_20_E_OBJECT_MEMORY,
    TPM_20_E_SESSION_MEMORY,
    TPM_20_E_MEMORY,
    TPM_20_E_SESSION_HANDLES,
    TPM_20_E_OBJECT_HANDLES,
    TPM_20_E_LOCALITY,
    TPM_20_E_YIELDED,
    TPM_20_E_CANCELED,
    TPM_20_E_TESTING,
    TPM_20_E_NV_RATE,
    TPM_20_E_LOCKOUT,
    TPM_20_E_RETRY,
    TPM_20_E_NV_UNAVAILABLE,
    TBS_E_INTERNAL_ERROR,
    TBS_E_BAD_PARAMETER,
    TBS_E_INVALID_OUTPUT_POINTER,
    TBS_E_INVALID_CONTEXT,
    TBS_E_INSUFFICIENT_BUFFER,
    TBS_E_IOERROR,
    TBS_E_INVALID_CONTEXT_PARAM,
    TBS_E_SERVICE_NOT_RUNNING,
    TBS_E_TOO_MANY_TBS_CONTEXTS,
    TBS_E_TOO_MANY_RESOURCES,
    TBS_E_SERVICE_START_PENDING,
    TBS_E_PPI_NOT_SUPPORTED,
    TBS_E_COMMAND_CANCELED,
    TBS_E_BUFFER_TOO_LARGE,
    TBS_E_TPM_NOT_FOUND,
    TBS_E_SERVICE_DISABLED,
    TBS_E_NO_EVENT_LOG,
    TBS_E_ACCESS_DENIED,
    TBS_E_PROVISIONING_NOT_ALLOWED,
    TBS_E_PPI_FUNCTION_UNSUPPORTED,
    TBS_E_OWNERAUTH_NOT_FOUND,
    TBS_E_PROVISIONING_INCOMPLETE,
}

impl TPM_SERVICES {
    pub fn code(&self) -> u32 {
        return match self {
            TPM_SERVICES::TPM_E_ERROR_MASK => 0x80280000 as u32,
            TPM_SERVICES::TPM_E_AUTHFAIL => 0x80280001 as u32,
            TPM_SERVICES::E_INK_EXCEPTION => 0x80280001 as u32,
            TPM_SERVICES::TPM_E_BADINDEX => 0x80280002 as u32,
            TPM_SERVICES::E_INK_MISMATCHED_INK_OBJECT => 0x80280002 as u32,
            TPM_SERVICES::TPM_E_BAD_PARAMETER => 0x80280003 as u32,
            TPM_SERVICES::E_INK_COLLECTOR_BUSY => 0x80280003 as u32,
            TPM_SERVICES::TPM_E_AUDITFAILURE => 0x80280004 as u32,
            TPM_SERVICES::E_INK_INCOMPATIBLE_OBJECT => 0x80280004 as u32,
            TPM_SERVICES::TPM_E_CLEAR_DISABLED => 0x80280005 as u32,
            TPM_SERVICES::E_INK_WINDOW_NOT_SET => 0x80280005 as u32,
            TPM_SERVICES::TPM_E_DEACTIVATED => 0x80280006 as u32,
            TPM_SERVICES::E_INK_INVALID_MODE => 0x80280006 as u32,
            TPM_SERVICES::TPM_E_DISABLED => 0x80280007 as u32,
            TPM_SERVICES::E_INK_COLLECTOR_ENABLED => 0x80280007 as u32,
            TPM_SERVICES::TPM_E_DISABLED_CMD => 0x80280008 as u32,
            TPM_SERVICES::E_INK_NO_STROKES_TO_RECOGNIZE => 0x80280008 as u32,
            TPM_SERVICES::TPM_E_FAIL => 0x80280009 as u32,
            TPM_SERVICES::E_INK_EMPTY_RECOGNITION_RESULT => 0x80280009 as u32,
            TPM_SERVICES::TPM_E_BAD_ORDINAL => 0x8028000A as u32,
            TPM_SERVICES::TPM_E_INSTALL_DISABLED => 0x8028000B as u32,
            TPM_SERVICES::TPM_E_INVALID_KEYHANDLE => 0x8028000C as u32,
            TPM_SERVICES::TPM_E_KEYNOTFOUND => 0x8028000D as u32,
            TPM_SERVICES::TPM_E_INAPPROPRIATE_ENC => 0x8028000E as u32,
            TPM_SERVICES::TPM_E_MIGRATEFAIL => 0x8028000F as u32,
            TPM_SERVICES::TPM_E_INVALID_PCR_INFO => 0x80280010 as u32,
            TPM_SERVICES::E_INK_OVERLAPPING_INPUT_RECT => 0x80280010 as u32,
            TPM_SERVICES::TPM_E_NOSPACE => 0x80280011 as u32,
            TPM_SERVICES::TPM_E_NOSRK => 0x80280012 as u32,
            TPM_SERVICES::TPM_E_NOTSEALED_BLOB => 0x80280013 as u32,
            TPM_SERVICES::TPM_E_OWNER_SET => 0x80280014 as u32,
            TPM_SERVICES::TPM_E_RESOURCES => 0x80280015 as u32,
            TPM_SERVICES::TPM_E_SHORTRANDOM => 0x80280016 as u32,
            TPM_SERVICES::TPM_E_SIZE => 0x80280017 as u32,
            TPM_SERVICES::TPM_E_WRONGPCRVAL => 0x80280018 as u32,
            TPM_SERVICES::TPM_E_BAD_PARAM_SIZE => 0x80280019 as u32,
            TPM_SERVICES::TPM_E_SHA_THREAD => 0x8028001A as u32,
            TPM_SERVICES::TPM_E_SHA_ERROR => 0x8028001B as u32,
            TPM_SERVICES::TPM_E_FAILEDSELFTEST => 0x8028001C as u32,
            TPM_SERVICES::TPM_E_AUTH2FAIL => 0x8028001D as u32,
            TPM_SERVICES::TPM_E_BADTAG => 0x8028001E as u32,
            TPM_SERVICES::TPM_E_IOERROR => 0x8028001F as u32,
            TPM_SERVICES::TPM_E_ENCRYPT_ERROR => 0x80280020 as u32,
            TPM_SERVICES::TPM_E_DECRYPT_ERROR => 0x80280021 as u32,
            TPM_SERVICES::TPM_E_INVALID_AUTHHANDLE => 0x80280022 as u32,
            TPM_SERVICES::TPM_E_NO_ENDORSEMENT => 0x80280023 as u32,
            TPM_SERVICES::TPM_E_INVALID_KEYUSAGE => 0x80280024 as u32,
            TPM_SERVICES::TPM_E_WRONG_ENTITYTYPE => 0x80280025 as u32,
            TPM_SERVICES::TPM_E_INVALID_POSTINIT => 0x80280026 as u32,
            TPM_SERVICES::TPM_E_INAPPROPRIATE_SIG => 0x80280027 as u32,
            TPM_SERVICES::TPM_E_BAD_KEY_PROPERTY => 0x80280028 as u32,
            TPM_SERVICES::TPM_E_BAD_MIGRATION => 0x80280029 as u32,
            TPM_SERVICES::TPM_E_BAD_SCHEME => 0x8028002A as u32,
            TPM_SERVICES::TPM_E_BAD_DATASIZE => 0x8028002B as u32,
            TPM_SERVICES::TPM_E_BAD_MODE => 0x8028002C as u32,
            TPM_SERVICES::TPM_E_BAD_PRESENCE => 0x8028002D as u32,
            TPM_SERVICES::TPM_E_BAD_VERSION => 0x8028002E as u32,
            TPM_SERVICES::TPM_E_NO_WRAP_TRANSPORT => 0x8028002F as u32,
            TPM_SERVICES::TPM_E_AUDITFAIL_UNSUCCESSFUL => 0x80280030 as u32,
            TPM_SERVICES::TPM_E_AUDITFAIL_SUCCESSFUL => 0x80280031 as u32,
            TPM_SERVICES::TPM_E_NOTRESETABLE => 0x80280032 as u32,
            TPM_SERVICES::TPM_E_NOTLOCAL => 0x80280033 as u32,
            TPM_SERVICES::TPM_E_BAD_TYPE => 0x80280034 as u32,
            TPM_SERVICES::TPM_E_INVALID_RESOURCE => 0x80280035 as u32,
            TPM_SERVICES::TPM_E_NOTFIPS => 0x80280036 as u32,
            TPM_SERVICES::TPM_E_INVALID_FAMILY => 0x80280037 as u32,
            TPM_SERVICES::TPM_E_NO_NV_PERMISSION => 0x80280038 as u32,
            TPM_SERVICES::TPM_E_REQUIRES_SIGN => 0x80280039 as u32,
            TPM_SERVICES::TPM_E_KEY_NOTSUPPORTED => 0x8028003A as u32,
            TPM_SERVICES::TPM_E_AUTH_CONFLICT => 0x8028003B as u32,
            TPM_SERVICES::TPM_E_AREA_LOCKED => 0x8028003C as u32,
            TPM_SERVICES::TPM_E_BAD_LOCALITY => 0x8028003D as u32,
            TPM_SERVICES::TPM_E_READ_ONLY => 0x8028003E as u32,
            TPM_SERVICES::TPM_E_PER_NOWRITE => 0x8028003F as u32,
            TPM_SERVICES::TPM_E_FAMILYCOUNT => 0x80280040 as u32,
            TPM_SERVICES::TPM_E_WRITE_LOCKED => 0x80280041 as u32,
            TPM_SERVICES::TPM_E_BAD_ATTRIBUTES => 0x80280042 as u32,
            TPM_SERVICES::TPM_E_INVALID_STRUCTURE => 0x80280043 as u32,
            TPM_SERVICES::TPM_E_KEY_OWNER_CONTROL => 0x80280044 as u32,
            TPM_SERVICES::TPM_E_BAD_COUNTER => 0x80280045 as u32,
            TPM_SERVICES::TPM_E_NOT_FULLWRITE => 0x80280046 as u32,
            TPM_SERVICES::TPM_E_CONTEXT_GAP => 0x80280047 as u32,
            TPM_SERVICES::TPM_E_MAXNVWRITES => 0x80280048 as u32,
            TPM_SERVICES::TPM_E_NOOPERATOR => 0x80280049 as u32,
            TPM_SERVICES::TPM_E_RESOURCEMISSING => 0x8028004A as u32,
            TPM_SERVICES::TPM_E_DELEGATE_LOCK => 0x8028004B as u32,
            TPM_SERVICES::TPM_E_DELEGATE_FAMILY => 0x8028004C as u32,
            TPM_SERVICES::TPM_E_DELEGATE_ADMIN => 0x8028004D as u32,
            TPM_SERVICES::TPM_E_TRANSPORT_NOTEXCLUSIVE => 0x8028004E as u32,
            TPM_SERVICES::TPM_E_OWNER_CONTROL => 0x8028004F as u32,
            TPM_SERVICES::TPM_E_DAA_RESOURCES => 0x80280050 as u32,
            TPM_SERVICES::TPM_E_DAA_INPUT_DATA0 => 0x80280051 as u32,
            TPM_SERVICES::TPM_E_DAA_INPUT_DATA1 => 0x80280052 as u32,
            TPM_SERVICES::TPM_E_DAA_ISSUER_SETTINGS => 0x80280053 as u32,
            TPM_SERVICES::TPM_E_DAA_TPM_SETTINGS => 0x80280054 as u32,
            TPM_SERVICES::TPM_E_DAA_STAGE => 0x80280055 as u32,
            TPM_SERVICES::TPM_E_DAA_ISSUER_VALIDITY => 0x80280056 as u32,
            TPM_SERVICES::TPM_E_DAA_WRONG_W => 0x80280057 as u32,
            TPM_SERVICES::TPM_E_BAD_HANDLE => 0x80280058 as u32,
            TPM_SERVICES::TPM_E_BAD_DELEGATE => 0x80280059 as u32,
            TPM_SERVICES::TPM_E_BADCONTEXT => 0x8028005A as u32,
            TPM_SERVICES::TPM_E_TOOMANYCONTEXTS => 0x8028005B as u32,
            TPM_SERVICES::TPM_E_MA_TICKET_SIGNATURE => 0x8028005C as u32,
            TPM_SERVICES::TPM_E_MA_DESTINATION => 0x8028005D as u32,
            TPM_SERVICES::TPM_E_MA_SOURCE => 0x8028005E as u32,
            TPM_SERVICES::TPM_E_MA_AUTHORITY => 0x8028005F as u32,
            TPM_SERVICES::TPM_E_PERMANENTEK => 0x80280061 as u32,
            TPM_SERVICES::TPM_E_BAD_SIGNATURE => 0x80280062 as u32,
            TPM_SERVICES::TPM_E_NOCONTEXTSPACE => 0x80280063 as u32,
            TPM_SERVICES::TPM_20_E_ASYMMETRIC => 0x80280081 as u32,
            TPM_SERVICES::TPM_20_E_ATTRIBUTES => 0x80280082 as u32,
            TPM_SERVICES::TPM_20_E_HASH => 0x80280083 as u32,
            TPM_SERVICES::TPM_20_E_VALUE => 0x80280084 as u32,
            TPM_SERVICES::TPM_20_E_HIERARCHY => 0x80280085 as u32,
            TPM_SERVICES::TPM_20_E_KEY_SIZE => 0x80280087 as u32,
            TPM_SERVICES::TPM_20_E_MGF => 0x80280088 as u32,
            TPM_SERVICES::TPM_20_E_MODE => 0x80280089 as u32,
            TPM_SERVICES::TPM_20_E_TYPE => 0x8028008A as u32,
            TPM_SERVICES::TPM_20_E_HANDLE => 0x8028008B as u32,
            TPM_SERVICES::TPM_20_E_KDF => 0x8028008C as u32,
            TPM_SERVICES::TPM_20_E_RANGE => 0x8028008D as u32,
            TPM_SERVICES::TPM_20_E_AUTH_FAIL => 0x8028008E as u32,
            TPM_SERVICES::TPM_20_E_NONCE => 0x8028008F as u32,
            TPM_SERVICES::TPM_20_E_PP => 0x80280090 as u32,
            TPM_SERVICES::TPM_20_E_SCHEME => 0x80280092 as u32,
            TPM_SERVICES::TPM_20_E_SIZE => 0x80280095 as u32,
            TPM_SERVICES::TPM_20_E_SYMMETRIC => 0x80280096 as u32,
            TPM_SERVICES::TPM_20_E_TAG => 0x80280097 as u32,
            TPM_SERVICES::TPM_20_E_SELECTOR => 0x80280098 as u32,
            TPM_SERVICES::TPM_20_E_INSUFFICIENT => 0x8028009A as u32,
            TPM_SERVICES::TPM_20_E_SIGNATURE => 0x8028009B as u32,
            TPM_SERVICES::TPM_20_E_KEY => 0x8028009C as u32,
            TPM_SERVICES::TPM_20_E_POLICY_FAIL => 0x8028009D as u32,
            TPM_SERVICES::TPM_20_E_INTEGRITY => 0x8028009F as u32,
            TPM_SERVICES::TPM_20_E_TICKET => 0x802800A0 as u32,
            TPM_SERVICES::TPM_20_E_RESERVED_BITS => 0x802800A1 as u32,
            TPM_SERVICES::TPM_20_E_BAD_AUTH => 0x802800A2 as u32,
            TPM_SERVICES::TPM_20_E_EXPIRED => 0x802800A3 as u32,
            TPM_SERVICES::TPM_20_E_POLICY_CC => 0x802800A4 as u32,
            TPM_SERVICES::TPM_20_E_BINDING => 0x802800A5 as u32,
            TPM_SERVICES::TPM_20_E_CURVE => 0x802800A6 as u32,
            TPM_SERVICES::TPM_20_E_ECC_POINT => 0x802800A7 as u32,
            TPM_SERVICES::TPM_20_E_INITIALIZE => 0x80280100 as u32,
            TPM_SERVICES::TPM_20_E_FAILURE => 0x80280101 as u32,
            TPM_SERVICES::TPM_20_E_SEQUENCE => 0x80280103 as u32,
            TPM_SERVICES::TPM_20_E_PRIVATE => 0x8028010B as u32,
            TPM_SERVICES::TPM_20_E_HMAC => 0x80280119 as u32,
            TPM_SERVICES::TPM_20_E_DISABLED => 0x80280120 as u32,
            TPM_SERVICES::TPM_20_E_EXCLUSIVE => 0x80280121 as u32,
            TPM_SERVICES::TPM_20_E_ECC_CURVE => 0x80280123 as u32,
            TPM_SERVICES::TPM_20_E_AUTH_TYPE => 0x80280124 as u32,
            TPM_SERVICES::TPM_20_E_AUTH_MISSING => 0x80280125 as u32,
            TPM_SERVICES::TPM_20_E_POLICY => 0x80280126 as u32,
            TPM_SERVICES::TPM_20_E_PCR => 0x80280127 as u32,
            TPM_SERVICES::TPM_20_E_PCR_CHANGED => 0x80280128 as u32,
            TPM_SERVICES::TPM_20_E_UPGRADE => 0x8028012D as u32,
            TPM_SERVICES::TPM_20_E_TOO_MANY_CONTEXTS => 0x8028012E as u32,
            TPM_SERVICES::TPM_20_E_AUTH_UNAVAILABLE => 0x8028012F as u32,
            TPM_SERVICES::TPM_20_E_REBOOT => 0x80280130 as u32,
            TPM_SERVICES::TPM_20_E_UNBALANCED => 0x80280131 as u32,
            TPM_SERVICES::TPM_20_E_COMMAND_SIZE => 0x80280142 as u32,
            TPM_SERVICES::TPM_20_E_COMMAND_CODE => 0x80280143 as u32,
            TPM_SERVICES::TPM_20_E_AUTHSIZE => 0x80280144 as u32,
            TPM_SERVICES::TPM_20_E_AUTH_CONTEXT => 0x80280145 as u32,
            TPM_SERVICES::TPM_20_E_NV_RANGE => 0x80280146 as u32,
            TPM_SERVICES::TPM_20_E_NV_SIZE => 0x80280147 as u32,
            TPM_SERVICES::TPM_20_E_NV_LOCKED => 0x80280148 as u32,
            TPM_SERVICES::TPM_20_E_NV_AUTHORIZATION => 0x80280149 as u32,
            TPM_SERVICES::TPM_20_E_NV_UNINITIALIZED => 0x8028014A as u32,
            TPM_SERVICES::TPM_20_E_NV_SPACE => 0x8028014B as u32,
            TPM_SERVICES::TPM_20_E_NV_DEFINED => 0x8028014C as u32,
            TPM_SERVICES::TPM_20_E_BAD_CONTEXT => 0x80280150 as u32,
            TPM_SERVICES::TPM_20_E_CPHASH => 0x80280151 as u32,
            TPM_SERVICES::TPM_20_E_PARENT => 0x80280152 as u32,
            TPM_SERVICES::TPM_20_E_NEEDS_TEST => 0x80280153 as u32,
            TPM_SERVICES::TPM_20_E_NO_RESULT => 0x80280154 as u32,
            TPM_SERVICES::TPM_20_E_SENSITIVE => 0x80280155 as u32,
            TPM_SERVICES::TPM_E_COMMAND_BLOCKED => 0x80280400 as u32,
            TPM_SERVICES::TPM_E_INVALID_HANDLE => 0x80280401 as u32,
            TPM_SERVICES::TPM_E_DUPLICATE_VHANDLE => 0x80280402 as u32,
            TPM_SERVICES::TPM_E_EMBEDDED_COMMAND_BLOCKED => 0x80280403 as u32,
            TPM_SERVICES::TPM_E_EMBEDDED_COMMAND_UNSUPPORTED => 0x80280404 as u32,
            TPM_SERVICES::TPM_E_RETRY => 0x80280800 as u32,
            TPM_SERVICES::TPM_E_NEEDS_SELFTEST => 0x80280801 as u32,
            TPM_SERVICES::TPM_E_DOING_SELFTEST => 0x80280802 as u32,
            TPM_SERVICES::TPM_E_DEFEND_LOCK_RUNNING => 0x80280803 as u32,
            TPM_SERVICES::TPM_20_E_CONTEXT_GAP => 0x80280901 as u32,
            TPM_SERVICES::TPM_20_E_OBJECT_MEMORY => 0x80280902 as u32,
            TPM_SERVICES::TPM_20_E_SESSION_MEMORY => 0x80280903 as u32,
            TPM_SERVICES::TPM_20_E_MEMORY => 0x80280904 as u32,
            TPM_SERVICES::TPM_20_E_SESSION_HANDLES => 0x80280905 as u32,
            TPM_SERVICES::TPM_20_E_OBJECT_HANDLES => 0x80280906 as u32,
            TPM_SERVICES::TPM_20_E_LOCALITY => 0x80280907 as u32,
            TPM_SERVICES::TPM_20_E_YIELDED => 0x80280908 as u32,
            TPM_SERVICES::TPM_20_E_CANCELED => 0x80280909 as u32,
            TPM_SERVICES::TPM_20_E_TESTING => 0x8028090A as u32,
            TPM_SERVICES::TPM_20_E_NV_RATE => 0x80280920 as u32,
            TPM_SERVICES::TPM_20_E_LOCKOUT => 0x80280921 as u32,
            TPM_SERVICES::TPM_20_E_RETRY => 0x80280922 as u32,
            TPM_SERVICES::TPM_20_E_NV_UNAVAILABLE => 0x80280923 as u32,
            TPM_SERVICES::TBS_E_INTERNAL_ERROR => 0x80284001 as u32,
            TPM_SERVICES::TBS_E_BAD_PARAMETER => 0x80284002 as u32,
            TPM_SERVICES::TBS_E_INVALID_OUTPUT_POINTER => 0x80284003 as u32,
            TPM_SERVICES::TBS_E_INVALID_CONTEXT => 0x80284004 as u32,
            TPM_SERVICES::TBS_E_INSUFFICIENT_BUFFER => 0x80284005 as u32,
            TPM_SERVICES::TBS_E_IOERROR => 0x80284006 as u32,
            TPM_SERVICES::TBS_E_INVALID_CONTEXT_PARAM => 0x80284007 as u32,
            TPM_SERVICES::TBS_E_SERVICE_NOT_RUNNING => 0x80284008 as u32,
            TPM_SERVICES::TBS_E_TOO_MANY_TBS_CONTEXTS => 0x80284009 as u32,
            TPM_SERVICES::TBS_E_TOO_MANY_RESOURCES => 0x8028400A as u32,
            TPM_SERVICES::TBS_E_SERVICE_START_PENDING => 0x8028400B as u32,
            TPM_SERVICES::TBS_E_PPI_NOT_SUPPORTED => 0x8028400C as u32,
            TPM_SERVICES::TBS_E_COMMAND_CANCELED => 0x8028400D as u32,
            TPM_SERVICES::TBS_E_BUFFER_TOO_LARGE => 0x8028400E as u32,
            TPM_SERVICES::TBS_E_TPM_NOT_FOUND => 0x8028400F as u32,
            TPM_SERVICES::TBS_E_SERVICE_DISABLED => 0x80284010 as u32,
            TPM_SERVICES::TBS_E_NO_EVENT_LOG => 0x80284011 as u32,
            TPM_SERVICES::TBS_E_ACCESS_DENIED => 0x80284012 as u32,
            TPM_SERVICES::TBS_E_PROVISIONING_NOT_ALLOWED => 0x80284013 as u32,
            TPM_SERVICES::TBS_E_PPI_FUNCTION_UNSUPPORTED => 0x80284014 as u32,
            TPM_SERVICES::TBS_E_OWNERAUTH_NOT_FOUND => 0x80284015 as u32,
            TPM_SERVICES::TBS_E_PROVISIONING_INCOMPLETE => 0x80284016 as u32,
        }
    }

    pub fn error(&self) -> RawError {
        return match self {
            TPM_SERVICES::TPM_E_ERROR_MASK => RawError::Kind(TPM_SERVICES::TPM_E_ERROR_MASK),
            TPM_SERVICES::TPM_E_AUTHFAIL => RawError::Kind(TPM_SERVICES::TPM_E_AUTHFAIL),
            TPM_SERVICES::E_INK_EXCEPTION => RawError::Kind(TPM_SERVICES::E_INK_EXCEPTION),
            TPM_SERVICES::TPM_E_BADINDEX => RawError::Kind(TPM_SERVICES::TPM_E_BADINDEX),
            TPM_SERVICES::E_INK_MISMATCHED_INK_OBJECT => RawError::Kind(TPM_SERVICES::E_INK_MISMATCHED_INK_OBJECT),
            TPM_SERVICES::TPM_E_BAD_PARAMETER => RawError::Kind(TPM_SERVICES::TPM_E_BAD_PARAMETER),
            TPM_SERVICES::E_INK_COLLECTOR_BUSY => RawError::Kind(TPM_SERVICES::E_INK_COLLECTOR_BUSY),
            TPM_SERVICES::TPM_E_AUDITFAILURE => RawError::Kind(TPM_SERVICES::TPM_E_AUDITFAILURE),
            TPM_SERVICES::E_INK_INCOMPATIBLE_OBJECT => RawError::Kind(TPM_SERVICES::E_INK_INCOMPATIBLE_OBJECT),
            TPM_SERVICES::TPM_E_CLEAR_DISABLED => RawError::Kind(TPM_SERVICES::TPM_E_CLEAR_DISABLED),
            TPM_SERVICES::E_INK_WINDOW_NOT_SET => RawError::Kind(TPM_SERVICES::E_INK_WINDOW_NOT_SET),
            TPM_SERVICES::TPM_E_DEACTIVATED => RawError::Kind(TPM_SERVICES::TPM_E_DEACTIVATED),
            TPM_SERVICES::E_INK_INVALID_MODE => RawError::Kind(TPM_SERVICES::E_INK_INVALID_MODE),
            TPM_SERVICES::TPM_E_DISABLED => RawError::Kind(TPM_SERVICES::TPM_E_DISABLED),
            TPM_SERVICES::E_INK_COLLECTOR_ENABLED => RawError::Kind(TPM_SERVICES::E_INK_COLLECTOR_ENABLED),
            TPM_SERVICES::TPM_E_DISABLED_CMD => RawError::Kind(TPM_SERVICES::TPM_E_DISABLED_CMD),
            TPM_SERVICES::E_INK_NO_STROKES_TO_RECOGNIZE => RawError::Kind(TPM_SERVICES::E_INK_NO_STROKES_TO_RECOGNIZE),
            TPM_SERVICES::TPM_E_FAIL => RawError::Kind(TPM_SERVICES::TPM_E_FAIL),
            TPM_SERVICES::E_INK_EMPTY_RECOGNITION_RESULT => RawError::Kind(TPM_SERVICES::E_INK_EMPTY_RECOGNITION_RESULT),
            TPM_SERVICES::TPM_E_BAD_ORDINAL => RawError::Kind(TPM_SERVICES::TPM_E_BAD_ORDINAL),
            TPM_SERVICES::TPM_E_INSTALL_DISABLED => RawError::Kind(TPM_SERVICES::TPM_E_INSTALL_DISABLED),
            TPM_SERVICES::TPM_E_INVALID_KEYHANDLE => RawError::Kind(TPM_SERVICES::TPM_E_INVALID_KEYHANDLE),
            TPM_SERVICES::TPM_E_KEYNOTFOUND => RawError::Kind(TPM_SERVICES::TPM_E_KEYNOTFOUND),
            TPM_SERVICES::TPM_E_INAPPROPRIATE_ENC => RawError::Kind(TPM_SERVICES::TPM_E_INAPPROPRIATE_ENC),
            TPM_SERVICES::TPM_E_MIGRATEFAIL => RawError::Kind(TPM_SERVICES::TPM_E_MIGRATEFAIL),
            TPM_SERVICES::TPM_E_INVALID_PCR_INFO => RawError::Kind(TPM_SERVICES::TPM_E_INVALID_PCR_INFO),
            TPM_SERVICES::E_INK_OVERLAPPING_INPUT_RECT => RawError::Kind(TPM_SERVICES::E_INK_OVERLAPPING_INPUT_RECT),
            TPM_SERVICES::TPM_E_NOSPACE => RawError::Kind(TPM_SERVICES::TPM_E_NOSPACE),
            TPM_SERVICES::TPM_E_NOSRK => RawError::Kind(TPM_SERVICES::TPM_E_NOSRK),
            TPM_SERVICES::TPM_E_NOTSEALED_BLOB => RawError::Kind(TPM_SERVICES::TPM_E_NOTSEALED_BLOB),
            TPM_SERVICES::TPM_E_OWNER_SET => RawError::Kind(TPM_SERVICES::TPM_E_OWNER_SET),
            TPM_SERVICES::TPM_E_RESOURCES => RawError::Kind(TPM_SERVICES::TPM_E_RESOURCES),
            TPM_SERVICES::TPM_E_SHORTRANDOM => RawError::Kind(TPM_SERVICES::TPM_E_SHORTRANDOM),
            TPM_SERVICES::TPM_E_SIZE => RawError::Kind(TPM_SERVICES::TPM_E_SIZE),
            TPM_SERVICES::TPM_E_WRONGPCRVAL => RawError::Kind(TPM_SERVICES::TPM_E_WRONGPCRVAL),
            TPM_SERVICES::TPM_E_BAD_PARAM_SIZE => RawError::Kind(TPM_SERVICES::TPM_E_BAD_PARAM_SIZE),
            TPM_SERVICES::TPM_E_SHA_THREAD => RawError::Kind(TPM_SERVICES::TPM_E_SHA_THREAD),
            TPM_SERVICES::TPM_E_SHA_ERROR => RawError::Kind(TPM_SERVICES::TPM_E_SHA_ERROR),
            TPM_SERVICES::TPM_E_FAILEDSELFTEST => RawError::Kind(TPM_SERVICES::TPM_E_FAILEDSELFTEST),
            TPM_SERVICES::TPM_E_AUTH2FAIL => RawError::Kind(TPM_SERVICES::TPM_E_AUTH2FAIL),
            TPM_SERVICES::TPM_E_BADTAG => RawError::Kind(TPM_SERVICES::TPM_E_BADTAG),
            TPM_SERVICES::TPM_E_IOERROR => RawError::Kind(TPM_SERVICES::TPM_E_IOERROR),
            TPM_SERVICES::TPM_E_ENCRYPT_ERROR => RawError::Kind(TPM_SERVICES::TPM_E_ENCRYPT_ERROR),
            TPM_SERVICES::TPM_E_DECRYPT_ERROR => RawError::Kind(TPM_SERVICES::TPM_E_DECRYPT_ERROR),
            TPM_SERVICES::TPM_E_INVALID_AUTHHANDLE => RawError::Kind(TPM_SERVICES::TPM_E_INVALID_AUTHHANDLE),
            TPM_SERVICES::TPM_E_NO_ENDORSEMENT => RawError::Kind(TPM_SERVICES::TPM_E_NO_ENDORSEMENT),
            TPM_SERVICES::TPM_E_INVALID_KEYUSAGE => RawError::Kind(TPM_SERVICES::TPM_E_INVALID_KEYUSAGE),
            TPM_SERVICES::TPM_E_WRONG_ENTITYTYPE => RawError::Kind(TPM_SERVICES::TPM_E_WRONG_ENTITYTYPE),
            TPM_SERVICES::TPM_E_INVALID_POSTINIT => RawError::Kind(TPM_SERVICES::TPM_E_INVALID_POSTINIT),
            TPM_SERVICES::TPM_E_INAPPROPRIATE_SIG => RawError::Kind(TPM_SERVICES::TPM_E_INAPPROPRIATE_SIG),
            TPM_SERVICES::TPM_E_BAD_KEY_PROPERTY => RawError::Kind(TPM_SERVICES::TPM_E_BAD_KEY_PROPERTY),
            TPM_SERVICES::TPM_E_BAD_MIGRATION => RawError::Kind(TPM_SERVICES::TPM_E_BAD_MIGRATION),
            TPM_SERVICES::TPM_E_BAD_SCHEME => RawError::Kind(TPM_SERVICES::TPM_E_BAD_SCHEME),
            TPM_SERVICES::TPM_E_BAD_DATASIZE => RawError::Kind(TPM_SERVICES::TPM_E_BAD_DATASIZE),
            TPM_SERVICES::TPM_E_BAD_MODE => RawError::Kind(TPM_SERVICES::TPM_E_BAD_MODE),
            TPM_SERVICES::TPM_E_BAD_PRESENCE => RawError::Kind(TPM_SERVICES::TPM_E_BAD_PRESENCE),
            TPM_SERVICES::TPM_E_BAD_VERSION => RawError::Kind(TPM_SERVICES::TPM_E_BAD_VERSION),
            TPM_SERVICES::TPM_E_NO_WRAP_TRANSPORT => RawError::Kind(TPM_SERVICES::TPM_E_NO_WRAP_TRANSPORT),
            TPM_SERVICES::TPM_E_AUDITFAIL_UNSUCCESSFUL => RawError::Kind(TPM_SERVICES::TPM_E_AUDITFAIL_UNSUCCESSFUL),
            TPM_SERVICES::TPM_E_AUDITFAIL_SUCCESSFUL => RawError::Kind(TPM_SERVICES::TPM_E_AUDITFAIL_SUCCESSFUL),
            TPM_SERVICES::TPM_E_NOTRESETABLE => RawError::Kind(TPM_SERVICES::TPM_E_NOTRESETABLE),
            TPM_SERVICES::TPM_E_NOTLOCAL => RawError::Kind(TPM_SERVICES::TPM_E_NOTLOCAL),
            TPM_SERVICES::TPM_E_BAD_TYPE => RawError::Kind(TPM_SERVICES::TPM_E_BAD_TYPE),
            TPM_SERVICES::TPM_E_INVALID_RESOURCE => RawError::Kind(TPM_SERVICES::TPM_E_INVALID_RESOURCE),
            TPM_SERVICES::TPM_E_NOTFIPS => RawError::Kind(TPM_SERVICES::TPM_E_NOTFIPS),
            TPM_SERVICES::TPM_E_INVALID_FAMILY => RawError::Kind(TPM_SERVICES::TPM_E_INVALID_FAMILY),
            TPM_SERVICES::TPM_E_NO_NV_PERMISSION => RawError::Kind(TPM_SERVICES::TPM_E_NO_NV_PERMISSION),
            TPM_SERVICES::TPM_E_REQUIRES_SIGN => RawError::Kind(TPM_SERVICES::TPM_E_REQUIRES_SIGN),
            TPM_SERVICES::TPM_E_KEY_NOTSUPPORTED => RawError::Kind(TPM_SERVICES::TPM_E_KEY_NOTSUPPORTED),
            TPM_SERVICES::TPM_E_AUTH_CONFLICT => RawError::Kind(TPM_SERVICES::TPM_E_AUTH_CONFLICT),
            TPM_SERVICES::TPM_E_AREA_LOCKED => RawError::Kind(TPM_SERVICES::TPM_E_AREA_LOCKED),
            TPM_SERVICES::TPM_E_BAD_LOCALITY => RawError::Kind(TPM_SERVICES::TPM_E_BAD_LOCALITY),
            TPM_SERVICES::TPM_E_READ_ONLY => RawError::Kind(TPM_SERVICES::TPM_E_READ_ONLY),
            TPM_SERVICES::TPM_E_PER_NOWRITE => RawError::Kind(TPM_SERVICES::TPM_E_PER_NOWRITE),
            TPM_SERVICES::TPM_E_FAMILYCOUNT => RawError::Kind(TPM_SERVICES::TPM_E_FAMILYCOUNT),
            TPM_SERVICES::TPM_E_WRITE_LOCKED => RawError::Kind(TPM_SERVICES::TPM_E_WRITE_LOCKED),
            TPM_SERVICES::TPM_E_BAD_ATTRIBUTES => RawError::Kind(TPM_SERVICES::TPM_E_BAD_ATTRIBUTES),
            TPM_SERVICES::TPM_E_INVALID_STRUCTURE => RawError::Kind(TPM_SERVICES::TPM_E_INVALID_STRUCTURE),
            TPM_SERVICES::TPM_E_KEY_OWNER_CONTROL => RawError::Kind(TPM_SERVICES::TPM_E_KEY_OWNER_CONTROL),
            TPM_SERVICES::TPM_E_BAD_COUNTER => RawError::Kind(TPM_SERVICES::TPM_E_BAD_COUNTER),
            TPM_SERVICES::TPM_E_NOT_FULLWRITE => RawError::Kind(TPM_SERVICES::TPM_E_NOT_FULLWRITE),
            TPM_SERVICES::TPM_E_CONTEXT_GAP => RawError::Kind(TPM_SERVICES::TPM_E_CONTEXT_GAP),
            TPM_SERVICES::TPM_E_MAXNVWRITES => RawError::Kind(TPM_SERVICES::TPM_E_MAXNVWRITES),
            TPM_SERVICES::TPM_E_NOOPERATOR => RawError::Kind(TPM_SERVICES::TPM_E_NOOPERATOR),
            TPM_SERVICES::TPM_E_RESOURCEMISSING => RawError::Kind(TPM_SERVICES::TPM_E_RESOURCEMISSING),
            TPM_SERVICES::TPM_E_DELEGATE_LOCK => RawError::Kind(TPM_SERVICES::TPM_E_DELEGATE_LOCK),
            TPM_SERVICES::TPM_E_DELEGATE_FAMILY => RawError::Kind(TPM_SERVICES::TPM_E_DELEGATE_FAMILY),
            TPM_SERVICES::TPM_E_DELEGATE_ADMIN => RawError::Kind(TPM_SERVICES::TPM_E_DELEGATE_ADMIN),
            TPM_SERVICES::TPM_E_TRANSPORT_NOTEXCLUSIVE => RawError::Kind(TPM_SERVICES::TPM_E_TRANSPORT_NOTEXCLUSIVE),
            TPM_SERVICES::TPM_E_OWNER_CONTROL => RawError::Kind(TPM_SERVICES::TPM_E_OWNER_CONTROL),
            TPM_SERVICES::TPM_E_DAA_RESOURCES => RawError::Kind(TPM_SERVICES::TPM_E_DAA_RESOURCES),
            TPM_SERVICES::TPM_E_DAA_INPUT_DATA0 => RawError::Kind(TPM_SERVICES::TPM_E_DAA_INPUT_DATA0),
            TPM_SERVICES::TPM_E_DAA_INPUT_DATA1 => RawError::Kind(TPM_SERVICES::TPM_E_DAA_INPUT_DATA1),
            TPM_SERVICES::TPM_E_DAA_ISSUER_SETTINGS => RawError::Kind(TPM_SERVICES::TPM_E_DAA_ISSUER_SETTINGS),
            TPM_SERVICES::TPM_E_DAA_TPM_SETTINGS => RawError::Kind(TPM_SERVICES::TPM_E_DAA_TPM_SETTINGS),
            TPM_SERVICES::TPM_E_DAA_STAGE => RawError::Kind(TPM_SERVICES::TPM_E_DAA_STAGE),
            TPM_SERVICES::TPM_E_DAA_ISSUER_VALIDITY => RawError::Kind(TPM_SERVICES::TPM_E_DAA_ISSUER_VALIDITY),
            TPM_SERVICES::TPM_E_DAA_WRONG_W => RawError::Kind(TPM_SERVICES::TPM_E_DAA_WRONG_W),
            TPM_SERVICES::TPM_E_BAD_HANDLE => RawError::Kind(TPM_SERVICES::TPM_E_BAD_HANDLE),
            TPM_SERVICES::TPM_E_BAD_DELEGATE => RawError::Kind(TPM_SERVICES::TPM_E_BAD_DELEGATE),
            TPM_SERVICES::TPM_E_BADCONTEXT => RawError::Kind(TPM_SERVICES::TPM_E_BADCONTEXT),
            TPM_SERVICES::TPM_E_TOOMANYCONTEXTS => RawError::Kind(TPM_SERVICES::TPM_E_TOOMANYCONTEXTS),
            TPM_SERVICES::TPM_E_MA_TICKET_SIGNATURE => RawError::Kind(TPM_SERVICES::TPM_E_MA_TICKET_SIGNATURE),
            TPM_SERVICES::TPM_E_MA_DESTINATION => RawError::Kind(TPM_SERVICES::TPM_E_MA_DESTINATION),
            TPM_SERVICES::TPM_E_MA_SOURCE => RawError::Kind(TPM_SERVICES::TPM_E_MA_SOURCE),
            TPM_SERVICES::TPM_E_MA_AUTHORITY => RawError::Kind(TPM_SERVICES::TPM_E_MA_AUTHORITY),
            TPM_SERVICES::TPM_E_PERMANENTEK => RawError::Kind(TPM_SERVICES::TPM_E_PERMANENTEK),
            TPM_SERVICES::TPM_E_BAD_SIGNATURE => RawError::Kind(TPM_SERVICES::TPM_E_BAD_SIGNATURE),
            TPM_SERVICES::TPM_E_NOCONTEXTSPACE => RawError::Kind(TPM_SERVICES::TPM_E_NOCONTEXTSPACE),
            TPM_SERVICES::TPM_20_E_ASYMMETRIC => RawError::Kind(TPM_SERVICES::TPM_20_E_ASYMMETRIC),
            TPM_SERVICES::TPM_20_E_ATTRIBUTES => RawError::Kind(TPM_SERVICES::TPM_20_E_ATTRIBUTES),
            TPM_SERVICES::TPM_20_E_HASH => RawError::Kind(TPM_SERVICES::TPM_20_E_HASH),
            TPM_SERVICES::TPM_20_E_VALUE => RawError::Kind(TPM_SERVICES::TPM_20_E_VALUE),
            TPM_SERVICES::TPM_20_E_HIERARCHY => RawError::Kind(TPM_SERVICES::TPM_20_E_HIERARCHY),
            TPM_SERVICES::TPM_20_E_KEY_SIZE => RawError::Kind(TPM_SERVICES::TPM_20_E_KEY_SIZE),
            TPM_SERVICES::TPM_20_E_MGF => RawError::Kind(TPM_SERVICES::TPM_20_E_MGF),
            TPM_SERVICES::TPM_20_E_MODE => RawError::Kind(TPM_SERVICES::TPM_20_E_MODE),
            TPM_SERVICES::TPM_20_E_TYPE => RawError::Kind(TPM_SERVICES::TPM_20_E_TYPE),
            TPM_SERVICES::TPM_20_E_HANDLE => RawError::Kind(TPM_SERVICES::TPM_20_E_HANDLE),
            TPM_SERVICES::TPM_20_E_KDF => RawError::Kind(TPM_SERVICES::TPM_20_E_KDF),
            TPM_SERVICES::TPM_20_E_RANGE => RawError::Kind(TPM_SERVICES::TPM_20_E_RANGE),
            TPM_SERVICES::TPM_20_E_AUTH_FAIL => RawError::Kind(TPM_SERVICES::TPM_20_E_AUTH_FAIL),
            TPM_SERVICES::TPM_20_E_NONCE => RawError::Kind(TPM_SERVICES::TPM_20_E_NONCE),
            TPM_SERVICES::TPM_20_E_PP => RawError::Kind(TPM_SERVICES::TPM_20_E_PP),
            TPM_SERVICES::TPM_20_E_SCHEME => RawError::Kind(TPM_SERVICES::TPM_20_E_SCHEME),
            TPM_SERVICES::TPM_20_E_SIZE => RawError::Kind(TPM_SERVICES::TPM_20_E_SIZE),
            TPM_SERVICES::TPM_20_E_SYMMETRIC => RawError::Kind(TPM_SERVICES::TPM_20_E_SYMMETRIC),
            TPM_SERVICES::TPM_20_E_TAG => RawError::Kind(TPM_SERVICES::TPM_20_E_TAG),
            TPM_SERVICES::TPM_20_E_SELECTOR => RawError::Kind(TPM_SERVICES::TPM_20_E_SELECTOR),
            TPM_SERVICES::TPM_20_E_INSUFFICIENT => RawError::Kind(TPM_SERVICES::TPM_20_E_INSUFFICIENT),
            TPM_SERVICES::TPM_20_E_SIGNATURE => RawError::Kind(TPM_SERVICES::TPM_20_E_SIGNATURE),
            TPM_SERVICES::TPM_20_E_KEY => RawError::Kind(TPM_SERVICES::TPM_20_E_KEY),
            TPM_SERVICES::TPM_20_E_POLICY_FAIL => RawError::Kind(TPM_SERVICES::TPM_20_E_POLICY_FAIL),
            TPM_SERVICES::TPM_20_E_INTEGRITY => RawError::Kind(TPM_SERVICES::TPM_20_E_INTEGRITY),
            TPM_SERVICES::TPM_20_E_TICKET => RawError::Kind(TPM_SERVICES::TPM_20_E_TICKET),
            TPM_SERVICES::TPM_20_E_RESERVED_BITS => RawError::Kind(TPM_SERVICES::TPM_20_E_RESERVED_BITS),
            TPM_SERVICES::TPM_20_E_BAD_AUTH => RawError::Kind(TPM_SERVICES::TPM_20_E_BAD_AUTH),
            TPM_SERVICES::TPM_20_E_EXPIRED => RawError::Kind(TPM_SERVICES::TPM_20_E_EXPIRED),
            TPM_SERVICES::TPM_20_E_POLICY_CC => RawError::Kind(TPM_SERVICES::TPM_20_E_POLICY_CC),
            TPM_SERVICES::TPM_20_E_BINDING => RawError::Kind(TPM_SERVICES::TPM_20_E_BINDING),
            TPM_SERVICES::TPM_20_E_CURVE => RawError::Kind(TPM_SERVICES::TPM_20_E_CURVE),
            TPM_SERVICES::TPM_20_E_ECC_POINT => RawError::Kind(TPM_SERVICES::TPM_20_E_ECC_POINT),
            TPM_SERVICES::TPM_20_E_INITIALIZE => RawError::Kind(TPM_SERVICES::TPM_20_E_INITIALIZE),
            TPM_SERVICES::TPM_20_E_FAILURE => RawError::Kind(TPM_SERVICES::TPM_20_E_FAILURE),
            TPM_SERVICES::TPM_20_E_SEQUENCE => RawError::Kind(TPM_SERVICES::TPM_20_E_SEQUENCE),
            TPM_SERVICES::TPM_20_E_PRIVATE => RawError::Kind(TPM_SERVICES::TPM_20_E_PRIVATE),
            TPM_SERVICES::TPM_20_E_HMAC => RawError::Kind(TPM_SERVICES::TPM_20_E_HMAC),
            TPM_SERVICES::TPM_20_E_DISABLED => RawError::Kind(TPM_SERVICES::TPM_20_E_DISABLED),
            TPM_SERVICES::TPM_20_E_EXCLUSIVE => RawError::Kind(TPM_SERVICES::TPM_20_E_EXCLUSIVE),
            TPM_SERVICES::TPM_20_E_ECC_CURVE => RawError::Kind(TPM_SERVICES::TPM_20_E_ECC_CURVE),
            TPM_SERVICES::TPM_20_E_AUTH_TYPE => RawError::Kind(TPM_SERVICES::TPM_20_E_AUTH_TYPE),
            TPM_SERVICES::TPM_20_E_AUTH_MISSING => RawError::Kind(TPM_SERVICES::TPM_20_E_AUTH_MISSING),
            TPM_SERVICES::TPM_20_E_POLICY => RawError::Kind(TPM_SERVICES::TPM_20_E_POLICY),
            TPM_SERVICES::TPM_20_E_PCR => RawError::Kind(TPM_SERVICES::TPM_20_E_PCR),
            TPM_SERVICES::TPM_20_E_PCR_CHANGED => RawError::Kind(TPM_SERVICES::TPM_20_E_PCR_CHANGED),
            TPM_SERVICES::TPM_20_E_UPGRADE => RawError::Kind(TPM_SERVICES::TPM_20_E_UPGRADE),
            TPM_SERVICES::TPM_20_E_TOO_MANY_CONTEXTS => RawError::Kind(TPM_SERVICES::TPM_20_E_TOO_MANY_CONTEXTS),
            TPM_SERVICES::TPM_20_E_AUTH_UNAVAILABLE => RawError::Kind(TPM_SERVICES::TPM_20_E_AUTH_UNAVAILABLE),
            TPM_SERVICES::TPM_20_E_REBOOT => RawError::Kind(TPM_SERVICES::TPM_20_E_REBOOT),
            TPM_SERVICES::TPM_20_E_UNBALANCED => RawError::Kind(TPM_SERVICES::TPM_20_E_UNBALANCED),
            TPM_SERVICES::TPM_20_E_COMMAND_SIZE => RawError::Kind(TPM_SERVICES::TPM_20_E_COMMAND_SIZE),
            TPM_SERVICES::TPM_20_E_COMMAND_CODE => RawError::Kind(TPM_SERVICES::TPM_20_E_COMMAND_CODE),
            TPM_SERVICES::TPM_20_E_AUTHSIZE => RawError::Kind(TPM_SERVICES::TPM_20_E_AUTHSIZE),
            TPM_SERVICES::TPM_20_E_AUTH_CONTEXT => RawError::Kind(TPM_SERVICES::TPM_20_E_AUTH_CONTEXT),
            TPM_SERVICES::TPM_20_E_NV_RANGE => RawError::Kind(TPM_SERVICES::TPM_20_E_NV_RANGE),
            TPM_SERVICES::TPM_20_E_NV_SIZE => RawError::Kind(TPM_SERVICES::TPM_20_E_NV_SIZE),
            TPM_SERVICES::TPM_20_E_NV_LOCKED => RawError::Kind(TPM_SERVICES::TPM_20_E_NV_LOCKED),
            TPM_SERVICES::TPM_20_E_NV_AUTHORIZATION => RawError::Kind(TPM_SERVICES::TPM_20_E_NV_AUTHORIZATION),
            TPM_SERVICES::TPM_20_E_NV_UNINITIALIZED => RawError::Kind(TPM_SERVICES::TPM_20_E_NV_UNINITIALIZED),
            TPM_SERVICES::TPM_20_E_NV_SPACE => RawError::Kind(TPM_SERVICES::TPM_20_E_NV_SPACE),
            TPM_SERVICES::TPM_20_E_NV_DEFINED => RawError::Kind(TPM_SERVICES::TPM_20_E_NV_DEFINED),
            TPM_SERVICES::TPM_20_E_BAD_CONTEXT => RawError::Kind(TPM_SERVICES::TPM_20_E_BAD_CONTEXT),
            TPM_SERVICES::TPM_20_E_CPHASH => RawError::Kind(TPM_SERVICES::TPM_20_E_CPHASH),
            TPM_SERVICES::TPM_20_E_PARENT => RawError::Kind(TPM_SERVICES::TPM_20_E_PARENT),
            TPM_SERVICES::TPM_20_E_NEEDS_TEST => RawError::Kind(TPM_SERVICES::TPM_20_E_NEEDS_TEST),
            TPM_SERVICES::TPM_20_E_NO_RESULT => RawError::Kind(TPM_SERVICES::TPM_20_E_NO_RESULT),
            TPM_SERVICES::TPM_20_E_SENSITIVE => RawError::Kind(TPM_SERVICES::TPM_20_E_SENSITIVE),
            TPM_SERVICES::TPM_E_COMMAND_BLOCKED => RawError::Kind(TPM_SERVICES::TPM_E_COMMAND_BLOCKED),
            TPM_SERVICES::TPM_E_INVALID_HANDLE => RawError::Kind(TPM_SERVICES::TPM_E_INVALID_HANDLE),
            TPM_SERVICES::TPM_E_DUPLICATE_VHANDLE => RawError::Kind(TPM_SERVICES::TPM_E_DUPLICATE_VHANDLE),
            TPM_SERVICES::TPM_E_EMBEDDED_COMMAND_BLOCKED => RawError::Kind(TPM_SERVICES::TPM_E_EMBEDDED_COMMAND_BLOCKED),
            TPM_SERVICES::TPM_E_EMBEDDED_COMMAND_UNSUPPORTED => RawError::Kind(TPM_SERVICES::TPM_E_EMBEDDED_COMMAND_UNSUPPORTED),
            TPM_SERVICES::TPM_E_RETRY => RawError::Kind(TPM_SERVICES::TPM_E_RETRY),
            TPM_SERVICES::TPM_E_NEEDS_SELFTEST => RawError::Kind(TPM_SERVICES::TPM_E_NEEDS_SELFTEST),
            TPM_SERVICES::TPM_E_DOING_SELFTEST => RawError::Kind(TPM_SERVICES::TPM_E_DOING_SELFTEST),
            TPM_SERVICES::TPM_E_DEFEND_LOCK_RUNNING => RawError::Kind(TPM_SERVICES::TPM_E_DEFEND_LOCK_RUNNING),
            TPM_SERVICES::TPM_20_E_CONTEXT_GAP => RawError::Kind(TPM_SERVICES::TPM_20_E_CONTEXT_GAP),
            TPM_SERVICES::TPM_20_E_OBJECT_MEMORY => RawError::Kind(TPM_SERVICES::TPM_20_E_OBJECT_MEMORY),
            TPM_SERVICES::TPM_20_E_SESSION_MEMORY => RawError::Kind(TPM_SERVICES::TPM_20_E_SESSION_MEMORY),
            TPM_SERVICES::TPM_20_E_MEMORY => RawError::Kind(TPM_SERVICES::TPM_20_E_MEMORY),
            TPM_SERVICES::TPM_20_E_SESSION_HANDLES => RawError::Kind(TPM_SERVICES::TPM_20_E_SESSION_HANDLES),
            TPM_SERVICES::TPM_20_E_OBJECT_HANDLES => RawError::Kind(TPM_SERVICES::TPM_20_E_OBJECT_HANDLES),
            TPM_SERVICES::TPM_20_E_LOCALITY => RawError::Kind(TPM_SERVICES::TPM_20_E_LOCALITY),
            TPM_SERVICES::TPM_20_E_YIELDED => RawError::Kind(TPM_SERVICES::TPM_20_E_YIELDED),
            TPM_SERVICES::TPM_20_E_CANCELED => RawError::Kind(TPM_SERVICES::TPM_20_E_CANCELED),
            TPM_SERVICES::TPM_20_E_TESTING => RawError::Kind(TPM_SERVICES::TPM_20_E_TESTING),
            TPM_SERVICES::TPM_20_E_NV_RATE => RawError::Kind(TPM_SERVICES::TPM_20_E_NV_RATE),
            TPM_SERVICES::TPM_20_E_LOCKOUT => RawError::Kind(TPM_SERVICES::TPM_20_E_LOCKOUT),
            TPM_SERVICES::TPM_20_E_RETRY => RawError::Kind(TPM_SERVICES::TPM_20_E_RETRY),
            TPM_SERVICES::TPM_20_E_NV_UNAVAILABLE => RawError::Kind(TPM_SERVICES::TPM_20_E_NV_UNAVAILABLE),
            TPM_SERVICES::TBS_E_INTERNAL_ERROR => RawError::Kind(TPM_SERVICES::TBS_E_INTERNAL_ERROR),
            TPM_SERVICES::TBS_E_BAD_PARAMETER => RawError::Kind(TPM_SERVICES::TBS_E_BAD_PARAMETER),
            TPM_SERVICES::TBS_E_INVALID_OUTPUT_POINTER => RawError::Kind(TPM_SERVICES::TBS_E_INVALID_OUTPUT_POINTER),
            TPM_SERVICES::TBS_E_INVALID_CONTEXT => RawError::Kind(TPM_SERVICES::TBS_E_INVALID_CONTEXT),
            TPM_SERVICES::TBS_E_INSUFFICIENT_BUFFER => RawError::Kind(TPM_SERVICES::TBS_E_INSUFFICIENT_BUFFER),
            TPM_SERVICES::TBS_E_IOERROR => RawError::Kind(TPM_SERVICES::TBS_E_IOERROR),
            TPM_SERVICES::TBS_E_INVALID_CONTEXT_PARAM => RawError::Kind(TPM_SERVICES::TBS_E_INVALID_CONTEXT_PARAM),
            TPM_SERVICES::TBS_E_SERVICE_NOT_RUNNING => RawError::Kind(TPM_SERVICES::TBS_E_SERVICE_NOT_RUNNING),
            TPM_SERVICES::TBS_E_TOO_MANY_TBS_CONTEXTS => RawError::Kind(TPM_SERVICES::TBS_E_TOO_MANY_TBS_CONTEXTS),
            TPM_SERVICES::TBS_E_TOO_MANY_RESOURCES => RawError::Kind(TPM_SERVICES::TBS_E_TOO_MANY_RESOURCES),
            TPM_SERVICES::TBS_E_SERVICE_START_PENDING => RawError::Kind(TPM_SERVICES::TBS_E_SERVICE_START_PENDING),
            TPM_SERVICES::TBS_E_PPI_NOT_SUPPORTED => RawError::Kind(TPM_SERVICES::TBS_E_PPI_NOT_SUPPORTED),
            TPM_SERVICES::TBS_E_COMMAND_CANCELED => RawError::Kind(TPM_SERVICES::TBS_E_COMMAND_CANCELED),
            TPM_SERVICES::TBS_E_BUFFER_TOO_LARGE => RawError::Kind(TPM_SERVICES::TBS_E_BUFFER_TOO_LARGE),
            TPM_SERVICES::TBS_E_TPM_NOT_FOUND => RawError::Kind(TPM_SERVICES::TBS_E_TPM_NOT_FOUND),
            TPM_SERVICES::TBS_E_SERVICE_DISABLED => RawError::Kind(TPM_SERVICES::TBS_E_SERVICE_DISABLED),
            TPM_SERVICES::TBS_E_NO_EVENT_LOG => RawError::Kind(TPM_SERVICES::TBS_E_NO_EVENT_LOG),
            TPM_SERVICES::TBS_E_ACCESS_DENIED => RawError::Kind(TPM_SERVICES::TBS_E_ACCESS_DENIED),
            TPM_SERVICES::TBS_E_PROVISIONING_NOT_ALLOWED => RawError::Kind(TPM_SERVICES::TBS_E_PROVISIONING_NOT_ALLOWED),
            TPM_SERVICES::TBS_E_PPI_FUNCTION_UNSUPPORTED => RawError::Kind(TPM_SERVICES::TBS_E_PPI_FUNCTION_UNSUPPORTED),
            TPM_SERVICES::TBS_E_OWNERAUTH_NOT_FOUND => RawError::Kind(TPM_SERVICES::TBS_E_OWNERAUTH_NOT_FOUND),
            TPM_SERVICES::TBS_E_PROVISIONING_INCOMPLETE => RawError::Kind(TPM_SERVICES::TBS_E_PROVISIONING_INCOMPLETE),
        }
    }

    pub fn description(&self) -> &'static str {
        return match self {
            TPM_SERVICES::TPM_E_ERROR_MASK => "This is an error mask to convert TPM hardware errors to win errors.",
            TPM_SERVICES::TPM_E_AUTHFAIL => "TPM 1.2: Authentication failed.",
            TPM_SERVICES::E_INK_EXCEPTION => "Microsoft Tablet PC: An internal exception occurred while executing the method or property.",
            TPM_SERVICES::TPM_E_BADINDEX => "TPM 1.2: The index to a PCR, DIR or other register is incorrect.",
            TPM_SERVICES::E_INK_MISMATCHED_INK_OBJECT => "Microsoft Tablet PC: The object is already associated with an ink object and cannot be reassociated.",
            TPM_SERVICES::TPM_E_BAD_PARAMETER => "TPM 1.2: One or more parameter is bad.",
            TPM_SERVICES::E_INK_COLLECTOR_BUSY => "Microsoft Tablet PC: The operation cannot be performed while the user is actively inking.",
            TPM_SERVICES::TPM_E_AUDITFAILURE => "TPM 1.2: An operation completed successfully but the auditing of that operation failed.",
            TPM_SERVICES::E_INK_INCOMPATIBLE_OBJECT => "Microsoft Tablet PC: The interface pointer points to an object that is incompatible with the Ink API",
            TPM_SERVICES::TPM_E_CLEAR_DISABLED => "TPM 1.2: The clear disable flag is set and all clear operations now require physical access.",
            TPM_SERVICES::E_INK_WINDOW_NOT_SET => "Microsoft Tablet PC: The window handle must be set before ink collection can occur.",
            TPM_SERVICES::TPM_E_DEACTIVATED => "TPM 1.2: Activate the Trusted Platform Module (TPM).",
            TPM_SERVICES::E_INK_INVALID_MODE => "Microsoft Tablet PC: The InkCollector must be gesture mode for gesture features, and single tablet mode for single tablet features.",
            TPM_SERVICES::TPM_E_DISABLED => "TPM 1.2: Enable the Trusted Platform Module (TPM).",
            TPM_SERVICES::E_INK_COLLECTOR_ENABLED => "Microsoft Tablet PC: The operation cannot be performed while the InkCollector is enabled.",
            TPM_SERVICES::TPM_E_DISABLED_CMD => "TPM 1.2: The target command has been disabled.",
            TPM_SERVICES::E_INK_NO_STROKES_TO_RECOGNIZE => "Microsoft Tablet PC: There are no strokes for the recognizer to process.",
            TPM_SERVICES::TPM_E_FAIL => "TPM 1.2: The operation failed.",
            TPM_SERVICES::E_INK_EMPTY_RECOGNITION_RESULT => "Microsoft Tablet PC: There are no strokes for the recognizer to process.",
            TPM_SERVICES::TPM_E_BAD_ORDINAL => "TPM 1.2: The ordinal was unknown or inconsistent.",
            TPM_SERVICES::TPM_E_INSTALL_DISABLED => "TPM 1.2: The ability to install an owner is disabled.",
            TPM_SERVICES::TPM_E_INVALID_KEYHANDLE => "TPM 1.2: The key handle cannot be interpreted.",
            TPM_SERVICES::TPM_E_KEYNOTFOUND => "TPM 1.2: The key handle points to an invalid key.",
            TPM_SERVICES::TPM_E_INAPPROPRIATE_ENC => "TPM 1.2: Unacceptable encryption scheme.",
            TPM_SERVICES::TPM_E_MIGRATEFAIL => "TPM 1.2: Migration authorization failed.",
            TPM_SERVICES::TPM_E_INVALID_PCR_INFO => "TPM 1.2: PCR information could not be interpreted.",
            TPM_SERVICES::E_INK_OVERLAPPING_INPUT_RECT => "Microsoft Tablet PC: The window input rectangle overlaps with an enabled InkCollector&#x27;s window input rectangle.",
            TPM_SERVICES::TPM_E_NOSPACE => "TPM 1.2: No room to load key.",
            TPM_SERVICES::TPM_E_NOSRK => "TPM 1.2: There is no Storage Root Key (SRK) set.",
            TPM_SERVICES::TPM_E_NOTSEALED_BLOB => "TPM 1.2: An encrypted blob is invalid or was not created by this TPM.",
            TPM_SERVICES::TPM_E_OWNER_SET => "TPM 1.2: The Trusted Platform Module (TPM) already has an owner.",
            TPM_SERVICES::TPM_E_RESOURCES => "TPM 1.2: The TPM has insufficient internal resources to perform the requested action.",
            TPM_SERVICES::TPM_E_SHORTRANDOM => "TPM 1.2: A random string was too short.",
            TPM_SERVICES::TPM_E_SIZE => "TPM 1.2: The TPM does not have the space to perform the operation.",
            TPM_SERVICES::TPM_E_WRONGPCRVAL => "TPM 1.2: The named PCR value does not match the current PCR value.",
            TPM_SERVICES::TPM_E_BAD_PARAM_SIZE => "TPM 1.2: The paramSize argument to the command has the incorrect value .",
            TPM_SERVICES::TPM_E_SHA_THREAD => "TPM 1.2: There is no existing SHA-1 thread.",
            TPM_SERVICES::TPM_E_SHA_ERROR => "TPM 1.2: The calculation is unable to proceed because the existing SHA-1 thread has already encountered an error.",
            TPM_SERVICES::TPM_E_FAILEDSELFTEST => "TPM 1.2: The TPM hardware device reported a failure during its internal self test. Try restarting the computer to resolve the problem. If the problem continues, check for the latest BIOS or firmware update for your TPM hardware. Consult the computer manufacturer's documentation for instructions.",
            TPM_SERVICES::TPM_E_AUTH2FAIL => "TPM 1.2: The authorization for the second key in a 2 key function failed authorization.",
            TPM_SERVICES::TPM_E_BADTAG => "TPM 1.2: The tag value sent to for a command is invalid.",
            TPM_SERVICES::TPM_E_IOERROR => "TPM 1.2: An IO error occurred transmitting information to the TPM.",
            TPM_SERVICES::TPM_E_ENCRYPT_ERROR => "TPM 1.2: The encryption process had a problem.",
            TPM_SERVICES::TPM_E_DECRYPT_ERROR => "TPM 1.2: The decryption process did not complete.",
            TPM_SERVICES::TPM_E_INVALID_AUTHHANDLE => "TPM 1.2: An invalid handle was used.",
            TPM_SERVICES::TPM_E_NO_ENDORSEMENT => "TPM 1.2: The TPM does not have an Endorsement Key (EK) installed.",
            TPM_SERVICES::TPM_E_INVALID_KEYUSAGE => "TPM 1.2: The usage of a key is not allowed.",
            TPM_SERVICES::TPM_E_WRONG_ENTITYTYPE => "TPM 1.2: The submitted entity type is not allowed.",
            TPM_SERVICES::TPM_E_INVALID_POSTINIT => "TPM 1.2: The command was received in the wrong sequence relative to TPM_Init and a subsequent TPM_Startup.",
            TPM_SERVICES::TPM_E_INAPPROPRIATE_SIG => "TPM 1.2: Signed data cannot include additional DER information.",
            TPM_SERVICES::TPM_E_BAD_KEY_PROPERTY => "TPM 1.2: The key properties in TPM_KEY_PARMs are not supported by this TPM.",
            TPM_SERVICES::TPM_E_BAD_MIGRATION => "TPM 1.2: The migration properties of this key are incorrect.",
            TPM_SERVICES::TPM_E_BAD_SCHEME => "TPM 1.2: The signature or encryption scheme for this key is incorrect or not permitted in this situation.",
            TPM_SERVICES::TPM_E_BAD_DATASIZE => "TPM 1.2: The size of the data (or blob) parameter is bad or inconsistent with the referenced key.",
            TPM_SERVICES::TPM_E_BAD_MODE => "TPM 1.2: A mode parameter is bad, such as capArea or subCapArea for TPM_GetCapability, phsicalPresence parameter for TPM_PhysicalPresence, or migrationType for TPM_CreateMigrationBlob.",
            TPM_SERVICES::TPM_E_BAD_PRESENCE => "TPM 1.2: Either the physicalPresence or physicalPresenceLock bits have the wrong value.",
            TPM_SERVICES::TPM_E_BAD_VERSION => "TPM 1.2: The TPM cannot perform this version of the capability.",
            TPM_SERVICES::TPM_E_NO_WRAP_TRANSPORT => "TPM 1.2: The TPM does not allow for wrapped transport sessions.",
            TPM_SERVICES::TPM_E_AUDITFAIL_UNSUCCESSFUL => "TPM 1.2: TPM audit construction failed and the underlying command was returning a failure code also.",
            TPM_SERVICES::TPM_E_AUDITFAIL_SUCCESSFUL => "TPM 1.2: TPM audit construction failed and the underlying command was returning success.",
            TPM_SERVICES::TPM_E_NOTRESETABLE => "TPM 1.2: Attempt to reset a PCR register that does not have the resettable attribute.",
            TPM_SERVICES::TPM_E_NOTLOCAL => "TPM 1.2: Attempt to reset a PCR register that requires locality and locality modifier not part of command transport.",
            TPM_SERVICES::TPM_E_BAD_TYPE => "TPM 1.2: Make identity blob not properly typed.",
            TPM_SERVICES::TPM_E_INVALID_RESOURCE => "TPM 1.2: When saving context identified resource type does not match actual resource.",
            TPM_SERVICES::TPM_E_NOTFIPS => "TPM 1.2: The TPM is attempting to execute a command that is not allowed when in FIPS mode.",
            TPM_SERVICES::TPM_E_INVALID_FAMILY => "TPM 1.2: The command is attempting to use an invalid family ID.",
            TPM_SERVICES::TPM_E_NO_NV_PERMISSION => "TPM 1.2: The permission to manipulate the NV storage is not available.",
            TPM_SERVICES::TPM_E_REQUIRES_SIGN => "TPM 1.2: The operation requires a signed command.",
            TPM_SERVICES::TPM_E_KEY_NOTSUPPORTED => "TPM 1.2: Wrong operation to load an NV key.",
            TPM_SERVICES::TPM_E_AUTH_CONFLICT => "TPM 1.2: NV_LoadKey blob requires both owner and blob authorization.",
            TPM_SERVICES::TPM_E_AREA_LOCKED => "TPM 1.2: The NV area is locked and not writable.",
            TPM_SERVICES::TPM_E_BAD_LOCALITY => "TPM 1.2: The locality is incorrect for the attempted operation.",
            TPM_SERVICES::TPM_E_READ_ONLY => "TPM 1.2: The NV area is read only and can't be written to.",
            TPM_SERVICES::TPM_E_PER_NOWRITE => "TPM 1.2: There is no protection on the write to the NV area.",
            TPM_SERVICES::TPM_E_FAMILYCOUNT => "TPM 1.2: The family count value does not match.",
            TPM_SERVICES::TPM_E_WRITE_LOCKED => "TPM 1.2: The NV area has already been written to.",
            TPM_SERVICES::TPM_E_BAD_ATTRIBUTES => "TPM 1.2: The NV area attributes conflict.",
            TPM_SERVICES::TPM_E_INVALID_STRUCTURE => "TPM 1.2: The structure tag and version are invalid or inconsistent.",
            TPM_SERVICES::TPM_E_KEY_OWNER_CONTROL => "TPM 1.2: The key is under control of the TPM Owner and can only be evicted by the TPM Owner.",
            TPM_SERVICES::TPM_E_BAD_COUNTER => "TPM 1.2: The counter handle is incorrect.",
            TPM_SERVICES::TPM_E_NOT_FULLWRITE => "TPM 1.2: The write is not a complete write of the area.",
            TPM_SERVICES::TPM_E_CONTEXT_GAP => "TPM 1.2: The gap between saved context counts is too large.",
            TPM_SERVICES::TPM_E_MAXNVWRITES => "TPM 1.2: The maximum number of NV writes without an owner has been exceeded.",
            TPM_SERVICES::TPM_E_NOOPERATOR => "TPM 1.2: No operator AuthData value is set.",
            TPM_SERVICES::TPM_E_RESOURCEMISSING => "TPM 1.2: The resource pointed to by context is not loaded.",
            TPM_SERVICES::TPM_E_DELEGATE_LOCK => "TPM 1.2: The delegate administration is locked.",
            TPM_SERVICES::TPM_E_DELEGATE_FAMILY => "TPM 1.2: Attempt to manage a family other then the delegated family.",
            TPM_SERVICES::TPM_E_DELEGATE_ADMIN => "TPM 1.2: Delegation table management not enabled.",
            TPM_SERVICES::TPM_E_TRANSPORT_NOTEXCLUSIVE => "TPM 1.2: There was a command executed outside of an exclusive transport session.",
            TPM_SERVICES::TPM_E_OWNER_CONTROL => "TPM 1.2: Attempt to context save a owner evict controlled key.",
            TPM_SERVICES::TPM_E_DAA_RESOURCES => "TPM 1.2: The DAA command has no resources available to execute the command.",
            TPM_SERVICES::TPM_E_DAA_INPUT_DATA0 => "TPM 1.2: The consistency check on DAA parameter inputData0 has failed.",
            TPM_SERVICES::TPM_E_DAA_INPUT_DATA1 => "TPM 1.2: The consistency check on DAA parameter inputData1 has failed.",
            TPM_SERVICES::TPM_E_DAA_ISSUER_SETTINGS => "TPM 1.2: The consistency check on DAA_issuerSettings has failed.",
            TPM_SERVICES::TPM_E_DAA_TPM_SETTINGS => "TPM 1.2: The consistency check on DAA_tpmSpecific has failed.",
            TPM_SERVICES::TPM_E_DAA_STAGE => "TPM 1.2: The atomic process indicated by the submitted DAA command is not the expected process.",
            TPM_SERVICES::TPM_E_DAA_ISSUER_VALIDITY => "TPM 1.2: The issuer's validity check has detected an inconsistency.",
            TPM_SERVICES::TPM_E_DAA_WRONG_W => "TPM 1.2: The consistency check on w has failed.",
            TPM_SERVICES::TPM_E_BAD_HANDLE => "TPM 1.2: The handle is incorrect.",
            TPM_SERVICES::TPM_E_BAD_DELEGATE => "TPM 1.2: Delegation is not correct.",
            TPM_SERVICES::TPM_E_BADCONTEXT => "TPM 1.2: The context blob is invalid.",
            TPM_SERVICES::TPM_E_TOOMANYCONTEXTS => "TPM 1.2: Too many contexts held by the TPM.",
            TPM_SERVICES::TPM_E_MA_TICKET_SIGNATURE => "TPM 1.2: Migration authority signature validation failure.",
            TPM_SERVICES::TPM_E_MA_DESTINATION => "TPM 1.2: Migration destination not authenticated.",
            TPM_SERVICES::TPM_E_MA_SOURCE => "TPM 1.2: Migration source incorrect.",
            TPM_SERVICES::TPM_E_MA_AUTHORITY => "TPM 1.2: Incorrect migration authority.",
            TPM_SERVICES::TPM_E_PERMANENTEK => "TPM 1.2: Attempt to revoke the EK and the EK is not revocable.",
            TPM_SERVICES::TPM_E_BAD_SIGNATURE => "TPM 1.2: Bad signature of CMK ticket.",
            TPM_SERVICES::TPM_E_NOCONTEXTSPACE => "TPM 1.2: There is no room in the context list for additional contexts.",
            TPM_SERVICES::TPM_20_E_ASYMMETRIC => "TPM 2.0: Asymmetric algorithm not supported or not correct.",
            TPM_SERVICES::TPM_20_E_ATTRIBUTES => "TPM 2.0: Inconsistent attributes.",
            TPM_SERVICES::TPM_20_E_HASH => "TPM 2.0: Hash algorithm not supported or not appropriate.",
            TPM_SERVICES::TPM_20_E_VALUE => "TPM 2.0: Value is out of range or is not correct for the context.",
            TPM_SERVICES::TPM_20_E_HIERARCHY => "TPM 2.0: Hierarchy is not enabled or is not correct for the use.",
            TPM_SERVICES::TPM_20_E_KEY_SIZE => "TPM 2.0: Key size is not supported.",
            TPM_SERVICES::TPM_20_E_MGF => "TPM 2.0: Mask generation function not supported.",
            TPM_SERVICES::TPM_20_E_MODE => "TPM 2.0: Mode of operation not supported.",
            TPM_SERVICES::TPM_20_E_TYPE => "TPM 2.0: The type of the value is not appropriate for the use.",
            TPM_SERVICES::TPM_20_E_HANDLE => "TPM 2.0: The Handle is not correct for the use.",
            TPM_SERVICES::TPM_20_E_KDF => "TPM 2.0: Unsupported key derivation function or function not appropriate for use.",
            TPM_SERVICES::TPM_20_E_RANGE => "TPM 2.0: Value was out of allowed range.",
            TPM_SERVICES::TPM_20_E_AUTH_FAIL => "TPM 2.0: The authorization HMAC check failed and DA counter incremented.",
            TPM_SERVICES::TPM_20_E_NONCE => "TPM 2.0: Invalid nonce size.",
            TPM_SERVICES::TPM_20_E_PP => "TPM 2.0: Authorization requires assertion of PP.",
            TPM_SERVICES::TPM_20_E_SCHEME => "TPM 2.0: Unsupported or incompatible scheme.",
            TPM_SERVICES::TPM_20_E_SIZE => "TPM 2.0: Structure is wrong size.",
            TPM_SERVICES::TPM_20_E_SYMMETRIC => "TPM 2.0: Unsupported symmetric algorithm or key size, or not appropriate for instance.",
            TPM_SERVICES::TPM_20_E_TAG => "TPM 2.0: Incorrect structure tag.",
            TPM_SERVICES::TPM_20_E_SELECTOR => "TPM 2.0: Union selector is incorrect.",
            TPM_SERVICES::TPM_20_E_INSUFFICIENT => "TPM 2.0: The TPM was unable to unmarshal a value because there were not enough octets in the input buffer.",
            TPM_SERVICES::TPM_20_E_SIGNATURE => "TPM 2.0: The signature is not valid.",
            TPM_SERVICES::TPM_20_E_KEY => "TPM 2.0: Key fields are not compatible with the selected use.",
            TPM_SERVICES::TPM_20_E_POLICY_FAIL => "TPM 2.0: A policy check failed.",
            TPM_SERVICES::TPM_20_E_INTEGRITY => "TPM 2.0: Integrity check failed.",
            TPM_SERVICES::TPM_20_E_TICKET => "TPM 2.0: Invalid ticket.",
            TPM_SERVICES::TPM_20_E_RESERVED_BITS => "TPM 2.0: Reserved bits not set to zero as required.",
            TPM_SERVICES::TPM_20_E_BAD_AUTH => "TPM 2.0: Authorization failure without DA implications.",
            TPM_SERVICES::TPM_20_E_EXPIRED => "TPM 2.0: The policy has expired.",
            TPM_SERVICES::TPM_20_E_POLICY_CC => "TPM 2.0: The command code in the policy is not the command code of the command or the command code in a policy command references a command that is not implemented.",
            TPM_SERVICES::TPM_20_E_BINDING => "TPM 2.0: Public and sensitive portions of an object are not cryptographically bound.",
            TPM_SERVICES::TPM_20_E_CURVE => "TPM 2.0: Curve not supported.",
            TPM_SERVICES::TPM_20_E_ECC_POINT => "TPM 2.0: Point is not on the required curve.",
            TPM_SERVICES::TPM_20_E_INITIALIZE => "TPM 2.0: TPM not initialized.",
            TPM_SERVICES::TPM_20_E_FAILURE => "TPM 2.0: Commands not being accepted because of a TPM failure.",
            TPM_SERVICES::TPM_20_E_SEQUENCE => "TPM 2.0: Improper use of a sequence handle.",
            TPM_SERVICES::TPM_20_E_PRIVATE => "TPM 2.0: TPM_RC_PRIVATE error.",
            TPM_SERVICES::TPM_20_E_HMAC => "TPM 2.0: TPM_RC_HMAC.",
            TPM_SERVICES::TPM_20_E_DISABLED => "TPM 2.0: TPM_RC_DISABLED.",
            TPM_SERVICES::TPM_20_E_EXCLUSIVE => "TPM 2.0: Command failed because audit sequence required exclusivity.",
            TPM_SERVICES::TPM_20_E_ECC_CURVE => "TPM 2.0: Unsupported ECC curve.",
            TPM_SERVICES::TPM_20_E_AUTH_TYPE => "TPM 2.0: Authorization handle is not correct for command.",
            TPM_SERVICES::TPM_20_E_AUTH_MISSING => "TPM 2.0: Command requires an authorization session for handle and is not present.",
            TPM_SERVICES::TPM_20_E_POLICY => "TPM 2.0: Policy failure in Math Operation or an invalid authPolicy value.",
            TPM_SERVICES::TPM_20_E_PCR => "TPM 2.0: PCR check fail.",
            TPM_SERVICES::TPM_20_E_PCR_CHANGED => "TPM 2.0: PCR have changed since checked.",
            TPM_SERVICES::TPM_20_E_UPGRADE => "TPM 2.0: The TPM is not in the right mode for upgrade.",
            TPM_SERVICES::TPM_20_E_TOO_MANY_CONTEXTS => "TPM 2.0: Context ID counter is at maximum.",
            TPM_SERVICES::TPM_20_E_AUTH_UNAVAILABLE => "TPM 2.0: authValue or authPolicy is not available for selected entity.",
            TPM_SERVICES::TPM_20_E_REBOOT => "TPM 2.0: A _TPM_Init and Startup(CLEAR) is required before the TPM can resume operation.",
            TPM_SERVICES::TPM_20_E_UNBALANCED => "TPM 2.0: The protection algorithms (hash and symmetric) are not reasonably balanced. The digest size of the hash must be larger than the key size of the symmetric algorithm.",
            TPM_SERVICES::TPM_20_E_COMMAND_SIZE => "TPM 2.0: The TPM command's commandSize value is inconsistent with contents of the command buffer; either the size is not the same as the bytes loaded by the hardware interface layer or the value is not large enough to hold a command header.",
            TPM_SERVICES::TPM_20_E_COMMAND_CODE => "TPM 2.0: Command code not supported.",
            TPM_SERVICES::TPM_20_E_AUTHSIZE => "TPM 2.0: The value of authorizationSize is out of range or the number of octets in the authorization Area is greater than required.",
            TPM_SERVICES::TPM_20_E_AUTH_CONTEXT => "TPM 2.0: Use of an authorization session with a context command or another command that cannot have an authorization session.",
            TPM_SERVICES::TPM_20_E_NV_RANGE => "TPM 2.0: NV offset+size is out of range.",
            TPM_SERVICES::TPM_20_E_NV_SIZE => "TPM 2.0: Requested allocation size is larger than allowed.",
            TPM_SERVICES::TPM_20_E_NV_LOCKED => "TPM 2.0: NV access locked.",
            TPM_SERVICES::TPM_20_E_NV_AUTHORIZATION => "TPM 2.0: NV access authorization fails in command actions",
            TPM_SERVICES::TPM_20_E_NV_UNINITIALIZED => "TPM 2.0: An NV index is used before being initialized or the state saved by TPM2_Shutdown(STATE) could not be restored.",
            TPM_SERVICES::TPM_20_E_NV_SPACE => "TPM 2.0: Insufficient space for NV allocation.",
            TPM_SERVICES::TPM_20_E_NV_DEFINED => "TPM 2.0: NV index or persistent object already defined.",
            TPM_SERVICES::TPM_20_E_BAD_CONTEXT => "TPM 2.0: Context in TPM2_ContextLoad() is not valid.",
            TPM_SERVICES::TPM_20_E_CPHASH => "TPM 2.0: chHash value already set or not correct for use.",
            TPM_SERVICES::TPM_20_E_PARENT => "TPM 2.0: Handle for parent is not a valid parent.",
            TPM_SERVICES::TPM_20_E_NEEDS_TEST => "TPM 2.0: Some function needs testing.",
            TPM_SERVICES::TPM_20_E_NO_RESULT => "TPM 2.0: returned when an internal function cannot process a request due to an unspecified problem. This code is usually related to invalid parameters that are not properly filtered by the input unmarshaling code.",
            TPM_SERVICES::TPM_20_E_SENSITIVE => "TPM 2.0: The sensitive area did not unmarshal correctly after decryption - this code is used in lieu of the other unmarshaling errors so that an attacker cannot determine where the unmarshaling error occurred.",
            TPM_SERVICES::TPM_E_COMMAND_BLOCKED => "The command was blocked.",
            TPM_SERVICES::TPM_E_INVALID_HANDLE => "The specified handle was not found.",
            TPM_SERVICES::TPM_E_DUPLICATE_VHANDLE => "The TPM returned a duplicate handle and the command needs to be resubmitted.",
            TPM_SERVICES::TPM_E_EMBEDDED_COMMAND_BLOCKED => "The command within the transport was blocked.",
            TPM_SERVICES::TPM_E_EMBEDDED_COMMAND_UNSUPPORTED => "The command within the transport is not supported.",
            TPM_SERVICES::TPM_E_RETRY => "The TPM is too busy to respond to the command immediately, but the command could be resubmitted at a later time.",
            TPM_SERVICES::TPM_E_NEEDS_SELFTEST => "SelfTestFull has not been run.",
            TPM_SERVICES::TPM_E_DOING_SELFTEST => "The TPM is currently executing a full selftest.",
            TPM_SERVICES::TPM_E_DEFEND_LOCK_RUNNING => "The TPM is defending against dictionary attacks and is in a time-out period.",
            TPM_SERVICES::TPM_20_E_CONTEXT_GAP => "TPM 2.0: Gap for context ID is too large.",
            TPM_SERVICES::TPM_20_E_OBJECT_MEMORY => "TPM 2.0: Out of memory for object contexts.",
            TPM_SERVICES::TPM_20_E_SESSION_MEMORY => "TPM 2.0: Out of memory for session contexts.",
            TPM_SERVICES::TPM_20_E_MEMORY => "TPM 2.0: Out of shared object/session memory or need space for internal operations.",
            TPM_SERVICES::TPM_20_E_SESSION_HANDLES => "TPM 2.0: Out of session handles - a session must be flushed before a nes session may be created.",
            TPM_SERVICES::TPM_20_E_OBJECT_HANDLES => "TPM 2.0: Out of object handles - the handle space for objects is depleted and a reboot is required.",
            TPM_SERVICES::TPM_20_E_LOCALITY => "TPM 2.0: Bad locality.",
            TPM_SERVICES::TPM_20_E_YIELDED => "TPM 2.0: The TPM has suspended operation on the command; forward progress was made and the command may be retried.",
            TPM_SERVICES::TPM_20_E_CANCELED => "TPM 2.0: The command was canceled.",
            TPM_SERVICES::TPM_20_E_TESTING => "TPM 2.0: TPM is performing self-tests.",
            TPM_SERVICES::TPM_20_E_NV_RATE => "TPM 2.0: The TPM is rate-limiting accesses to prevent wearout of NV",
            TPM_SERVICES::TPM_20_E_LOCKOUT => "TPM 2.0: Authorization for objects subject to DA protection are not allowed at this time because the TPM is in DA lockout mode.",
            TPM_SERVICES::TPM_20_E_RETRY => "TPM 2.0: The TPM was not able to start the command.",
            TPM_SERVICES::TPM_20_E_NV_UNAVAILABLE => "TPM 2.0: the command may require writing of NV and NV is not current accessible.",
            TPM_SERVICES::TBS_E_INTERNAL_ERROR => "An internal error has occurred within the Trusted Platform Module support program.",
            TPM_SERVICES::TBS_E_BAD_PARAMETER => "One or more input parameters is bad.",
            TPM_SERVICES::TBS_E_INVALID_OUTPUT_POINTER => "A specified output pointer is bad.",
            TPM_SERVICES::TBS_E_INVALID_CONTEXT => "The specified context handle does not refer to a valid context.",
            TPM_SERVICES::TBS_E_INSUFFICIENT_BUFFER => "A specified output buffer is too small.",
            TPM_SERVICES::TBS_E_IOERROR => "An error occurred while communicating with the TPM.",
            TPM_SERVICES::TBS_E_INVALID_CONTEXT_PARAM => "One or more context parameters is invalid.",
            TPM_SERVICES::TBS_E_SERVICE_NOT_RUNNING => "The TBS service is not running and could not be started.",
            TPM_SERVICES::TBS_E_TOO_MANY_TBS_CONTEXTS => "A new context could not be created because there are too many open contexts.",
            TPM_SERVICES::TBS_E_TOO_MANY_RESOURCES => "A new virtual resource could not be created because there are too many open virtual resources.",
            TPM_SERVICES::TBS_E_SERVICE_START_PENDING => "The TBS service has been started but is not yet running.",
            TPM_SERVICES::TBS_E_PPI_NOT_SUPPORTED => "The physical presence interface is not supported.",
            TPM_SERVICES::TBS_E_COMMAND_CANCELED => "The command was canceled.",
            TPM_SERVICES::TBS_E_BUFFER_TOO_LARGE => "The input or output buffer is too large.",
            TPM_SERVICES::TBS_E_TPM_NOT_FOUND => "A compatible Trusted Platform Module (TPM) Security Device cannot be found on this computer.",
            TPM_SERVICES::TBS_E_SERVICE_DISABLED => "The TBS service has been disabled.",
            TPM_SERVICES::TBS_E_NO_EVENT_LOG => "No TCG event log is available.",
            TPM_SERVICES::TBS_E_ACCESS_DENIED => "The caller does not have the appropriate rights to perform the requested operation.",
            TPM_SERVICES::TBS_E_PROVISIONING_NOT_ALLOWED => "The TPM provisioning action is not allowed by the specified flags. For provisioning to be successful, one of several actions may be required. The TPM management console (tpm.msc) action to make the TPM Ready may help. For further information, see the documentation for the Win32_Tpm WMI method 'Provision'. (The actions that may be required include importing the TPM Owner Authorization value into the system, calling the Win32_Tpm WMI method for provisioning the TPM and specifying TRUE for either 'ForceClear_Allowed' or 'PhysicalPresencePrompts_Allowed' (as indicated by the value returned in the Additional Information), or enabling the TPM in the system BIOS.)",
            TPM_SERVICES::TBS_E_PPI_FUNCTION_UNSUPPORTED => "The Physical Presence Interface of this firmware does not support the requested method.",
            TPM_SERVICES::TBS_E_OWNERAUTH_NOT_FOUND => "The requested TPM OwnerAuth value was not found.",
            TPM_SERVICES::TBS_E_PROVISIONING_INCOMPLETE => "The TPM provisioning did not complete. For more information on completing the provisioning, call the Win32_Tpm WMI method for provisioning the TPM ('Provision') and check the returned Information.",
        }
    }

    pub fn from_name(name: &str) -> TPM_SERVICES {
        return match name {
            "TPM_E_ERROR_MASK" => TPM_SERVICES::TPM_E_ERROR_MASK,
            "TPM_E_AUTHFAIL" => TPM_SERVICES::TPM_E_AUTHFAIL,
            "E_INK_EXCEPTION" => TPM_SERVICES::E_INK_EXCEPTION,
            "TPM_E_BADINDEX" => TPM_SERVICES::TPM_E_BADINDEX,
            "E_INK_MISMATCHED_INK_OBJECT" => TPM_SERVICES::E_INK_MISMATCHED_INK_OBJECT,
            "TPM_E_BAD_PARAMETER" => TPM_SERVICES::TPM_E_BAD_PARAMETER,
            "E_INK_COLLECTOR_BUSY" => TPM_SERVICES::E_INK_COLLECTOR_BUSY,
            "TPM_E_AUDITFAILURE" => TPM_SERVICES::TPM_E_AUDITFAILURE,
            "E_INK_INCOMPATIBLE_OBJECT" => TPM_SERVICES::E_INK_INCOMPATIBLE_OBJECT,
            "TPM_E_CLEAR_DISABLED" => TPM_SERVICES::TPM_E_CLEAR_DISABLED,
            "E_INK_WINDOW_NOT_SET" => TPM_SERVICES::E_INK_WINDOW_NOT_SET,
            "TPM_E_DEACTIVATED" => TPM_SERVICES::TPM_E_DEACTIVATED,
            "E_INK_INVALID_MODE" => TPM_SERVICES::E_INK_INVALID_MODE,
            "TPM_E_DISABLED" => TPM_SERVICES::TPM_E_DISABLED,
            "E_INK_COLLECTOR_ENABLED" => TPM_SERVICES::E_INK_COLLECTOR_ENABLED,
            "TPM_E_DISABLED_CMD" => TPM_SERVICES::TPM_E_DISABLED_CMD,
            "E_INK_NO_STROKES_TO_RECOGNIZE" => TPM_SERVICES::E_INK_NO_STROKES_TO_RECOGNIZE,
            "TPM_E_FAIL" => TPM_SERVICES::TPM_E_FAIL,
            "E_INK_EMPTY_RECOGNITION_RESULT" => TPM_SERVICES::E_INK_EMPTY_RECOGNITION_RESULT,
            "TPM_E_BAD_ORDINAL" => TPM_SERVICES::TPM_E_BAD_ORDINAL,
            "TPM_E_INSTALL_DISABLED" => TPM_SERVICES::TPM_E_INSTALL_DISABLED,
            "TPM_E_INVALID_KEYHANDLE" => TPM_SERVICES::TPM_E_INVALID_KEYHANDLE,
            "TPM_E_KEYNOTFOUND" => TPM_SERVICES::TPM_E_KEYNOTFOUND,
            "TPM_E_INAPPROPRIATE_ENC" => TPM_SERVICES::TPM_E_INAPPROPRIATE_ENC,
            "TPM_E_MIGRATEFAIL" => TPM_SERVICES::TPM_E_MIGRATEFAIL,
            "TPM_E_INVALID_PCR_INFO" => TPM_SERVICES::TPM_E_INVALID_PCR_INFO,
            "E_INK_OVERLAPPING_INPUT_RECT" => TPM_SERVICES::E_INK_OVERLAPPING_INPUT_RECT,
            "TPM_E_NOSPACE" => TPM_SERVICES::TPM_E_NOSPACE,
            "TPM_E_NOSRK" => TPM_SERVICES::TPM_E_NOSRK,
            "TPM_E_NOTSEALED_BLOB" => TPM_SERVICES::TPM_E_NOTSEALED_BLOB,
            "TPM_E_OWNER_SET" => TPM_SERVICES::TPM_E_OWNER_SET,
            "TPM_E_RESOURCES" => TPM_SERVICES::TPM_E_RESOURCES,
            "TPM_E_SHORTRANDOM" => TPM_SERVICES::TPM_E_SHORTRANDOM,
            "TPM_E_SIZE" => TPM_SERVICES::TPM_E_SIZE,
            "TPM_E_WRONGPCRVAL" => TPM_SERVICES::TPM_E_WRONGPCRVAL,
            "TPM_E_BAD_PARAM_SIZE" => TPM_SERVICES::TPM_E_BAD_PARAM_SIZE,
            "TPM_E_SHA_THREAD" => TPM_SERVICES::TPM_E_SHA_THREAD,
            "TPM_E_SHA_ERROR" => TPM_SERVICES::TPM_E_SHA_ERROR,
            "TPM_E_FAILEDSELFTEST" => TPM_SERVICES::TPM_E_FAILEDSELFTEST,
            "TPM_E_AUTH2FAIL" => TPM_SERVICES::TPM_E_AUTH2FAIL,
            "TPM_E_BADTAG" => TPM_SERVICES::TPM_E_BADTAG,
            "TPM_E_IOERROR" => TPM_SERVICES::TPM_E_IOERROR,
            "TPM_E_ENCRYPT_ERROR" => TPM_SERVICES::TPM_E_ENCRYPT_ERROR,
            "TPM_E_DECRYPT_ERROR" => TPM_SERVICES::TPM_E_DECRYPT_ERROR,
            "TPM_E_INVALID_AUTHHANDLE" => TPM_SERVICES::TPM_E_INVALID_AUTHHANDLE,
            "TPM_E_NO_ENDORSEMENT" => TPM_SERVICES::TPM_E_NO_ENDORSEMENT,
            "TPM_E_INVALID_KEYUSAGE" => TPM_SERVICES::TPM_E_INVALID_KEYUSAGE,
            "TPM_E_WRONG_ENTITYTYPE" => TPM_SERVICES::TPM_E_WRONG_ENTITYTYPE,
            "TPM_E_INVALID_POSTINIT" => TPM_SERVICES::TPM_E_INVALID_POSTINIT,
            "TPM_E_INAPPROPRIATE_SIG" => TPM_SERVICES::TPM_E_INAPPROPRIATE_SIG,
            "TPM_E_BAD_KEY_PROPERTY" => TPM_SERVICES::TPM_E_BAD_KEY_PROPERTY,
            "TPM_E_BAD_MIGRATION" => TPM_SERVICES::TPM_E_BAD_MIGRATION,
            "TPM_E_BAD_SCHEME" => TPM_SERVICES::TPM_E_BAD_SCHEME,
            "TPM_E_BAD_DATASIZE" => TPM_SERVICES::TPM_E_BAD_DATASIZE,
            "TPM_E_BAD_MODE" => TPM_SERVICES::TPM_E_BAD_MODE,
            "TPM_E_BAD_PRESENCE" => TPM_SERVICES::TPM_E_BAD_PRESENCE,
            "TPM_E_BAD_VERSION" => TPM_SERVICES::TPM_E_BAD_VERSION,
            "TPM_E_NO_WRAP_TRANSPORT" => TPM_SERVICES::TPM_E_NO_WRAP_TRANSPORT,
            "TPM_E_AUDITFAIL_UNSUCCESSFUL" => TPM_SERVICES::TPM_E_AUDITFAIL_UNSUCCESSFUL,
            "TPM_E_AUDITFAIL_SUCCESSFUL" => TPM_SERVICES::TPM_E_AUDITFAIL_SUCCESSFUL,
            "TPM_E_NOTRESETABLE" => TPM_SERVICES::TPM_E_NOTRESETABLE,
            "TPM_E_NOTLOCAL" => TPM_SERVICES::TPM_E_NOTLOCAL,
            "TPM_E_BAD_TYPE" => TPM_SERVICES::TPM_E_BAD_TYPE,
            "TPM_E_INVALID_RESOURCE" => TPM_SERVICES::TPM_E_INVALID_RESOURCE,
            "TPM_E_NOTFIPS" => TPM_SERVICES::TPM_E_NOTFIPS,
            "TPM_E_INVALID_FAMILY" => TPM_SERVICES::TPM_E_INVALID_FAMILY,
            "TPM_E_NO_NV_PERMISSION" => TPM_SERVICES::TPM_E_NO_NV_PERMISSION,
            "TPM_E_REQUIRES_SIGN" => TPM_SERVICES::TPM_E_REQUIRES_SIGN,
            "TPM_E_KEY_NOTSUPPORTED" => TPM_SERVICES::TPM_E_KEY_NOTSUPPORTED,
            "TPM_E_AUTH_CONFLICT" => TPM_SERVICES::TPM_E_AUTH_CONFLICT,
            "TPM_E_AREA_LOCKED" => TPM_SERVICES::TPM_E_AREA_LOCKED,
            "TPM_E_BAD_LOCALITY" => TPM_SERVICES::TPM_E_BAD_LOCALITY,
            "TPM_E_READ_ONLY" => TPM_SERVICES::TPM_E_READ_ONLY,
            "TPM_E_PER_NOWRITE" => TPM_SERVICES::TPM_E_PER_NOWRITE,
            "TPM_E_FAMILYCOUNT" => TPM_SERVICES::TPM_E_FAMILYCOUNT,
            "TPM_E_WRITE_LOCKED" => TPM_SERVICES::TPM_E_WRITE_LOCKED,
            "TPM_E_BAD_ATTRIBUTES" => TPM_SERVICES::TPM_E_BAD_ATTRIBUTES,
            "TPM_E_INVALID_STRUCTURE" => TPM_SERVICES::TPM_E_INVALID_STRUCTURE,
            "TPM_E_KEY_OWNER_CONTROL" => TPM_SERVICES::TPM_E_KEY_OWNER_CONTROL,
            "TPM_E_BAD_COUNTER" => TPM_SERVICES::TPM_E_BAD_COUNTER,
            "TPM_E_NOT_FULLWRITE" => TPM_SERVICES::TPM_E_NOT_FULLWRITE,
            "TPM_E_CONTEXT_GAP" => TPM_SERVICES::TPM_E_CONTEXT_GAP,
            "TPM_E_MAXNVWRITES" => TPM_SERVICES::TPM_E_MAXNVWRITES,
            "TPM_E_NOOPERATOR" => TPM_SERVICES::TPM_E_NOOPERATOR,
            "TPM_E_RESOURCEMISSING" => TPM_SERVICES::TPM_E_RESOURCEMISSING,
            "TPM_E_DELEGATE_LOCK" => TPM_SERVICES::TPM_E_DELEGATE_LOCK,
            "TPM_E_DELEGATE_FAMILY" => TPM_SERVICES::TPM_E_DELEGATE_FAMILY,
            "TPM_E_DELEGATE_ADMIN" => TPM_SERVICES::TPM_E_DELEGATE_ADMIN,
            "TPM_E_TRANSPORT_NOTEXCLUSIVE" => TPM_SERVICES::TPM_E_TRANSPORT_NOTEXCLUSIVE,
            "TPM_E_OWNER_CONTROL" => TPM_SERVICES::TPM_E_OWNER_CONTROL,
            "TPM_E_DAA_RESOURCES" => TPM_SERVICES::TPM_E_DAA_RESOURCES,
            "TPM_E_DAA_INPUT_DATA0" => TPM_SERVICES::TPM_E_DAA_INPUT_DATA0,
            "TPM_E_DAA_INPUT_DATA1" => TPM_SERVICES::TPM_E_DAA_INPUT_DATA1,
            "TPM_E_DAA_ISSUER_SETTINGS" => TPM_SERVICES::TPM_E_DAA_ISSUER_SETTINGS,
            "TPM_E_DAA_TPM_SETTINGS" => TPM_SERVICES::TPM_E_DAA_TPM_SETTINGS,
            "TPM_E_DAA_STAGE" => TPM_SERVICES::TPM_E_DAA_STAGE,
            "TPM_E_DAA_ISSUER_VALIDITY" => TPM_SERVICES::TPM_E_DAA_ISSUER_VALIDITY,
            "TPM_E_DAA_WRONG_W" => TPM_SERVICES::TPM_E_DAA_WRONG_W,
            "TPM_E_BAD_HANDLE" => TPM_SERVICES::TPM_E_BAD_HANDLE,
            "TPM_E_BAD_DELEGATE" => TPM_SERVICES::TPM_E_BAD_DELEGATE,
            "TPM_E_BADCONTEXT" => TPM_SERVICES::TPM_E_BADCONTEXT,
            "TPM_E_TOOMANYCONTEXTS" => TPM_SERVICES::TPM_E_TOOMANYCONTEXTS,
            "TPM_E_MA_TICKET_SIGNATURE" => TPM_SERVICES::TPM_E_MA_TICKET_SIGNATURE,
            "TPM_E_MA_DESTINATION" => TPM_SERVICES::TPM_E_MA_DESTINATION,
            "TPM_E_MA_SOURCE" => TPM_SERVICES::TPM_E_MA_SOURCE,
            "TPM_E_MA_AUTHORITY" => TPM_SERVICES::TPM_E_MA_AUTHORITY,
            "TPM_E_PERMANENTEK" => TPM_SERVICES::TPM_E_PERMANENTEK,
            "TPM_E_BAD_SIGNATURE" => TPM_SERVICES::TPM_E_BAD_SIGNATURE,
            "TPM_E_NOCONTEXTSPACE" => TPM_SERVICES::TPM_E_NOCONTEXTSPACE,
            "TPM_20_E_ASYMMETRIC" => TPM_SERVICES::TPM_20_E_ASYMMETRIC,
            "TPM_20_E_ATTRIBUTES" => TPM_SERVICES::TPM_20_E_ATTRIBUTES,
            "TPM_20_E_HASH" => TPM_SERVICES::TPM_20_E_HASH,
            "TPM_20_E_VALUE" => TPM_SERVICES::TPM_20_E_VALUE,
            "TPM_20_E_HIERARCHY" => TPM_SERVICES::TPM_20_E_HIERARCHY,
            "TPM_20_E_KEY_SIZE" => TPM_SERVICES::TPM_20_E_KEY_SIZE,
            "TPM_20_E_MGF" => TPM_SERVICES::TPM_20_E_MGF,
            "TPM_20_E_MODE" => TPM_SERVICES::TPM_20_E_MODE,
            "TPM_20_E_TYPE" => TPM_SERVICES::TPM_20_E_TYPE,
            "TPM_20_E_HANDLE" => TPM_SERVICES::TPM_20_E_HANDLE,
            "TPM_20_E_KDF" => TPM_SERVICES::TPM_20_E_KDF,
            "TPM_20_E_RANGE" => TPM_SERVICES::TPM_20_E_RANGE,
            "TPM_20_E_AUTH_FAIL" => TPM_SERVICES::TPM_20_E_AUTH_FAIL,
            "TPM_20_E_NONCE" => TPM_SERVICES::TPM_20_E_NONCE,
            "TPM_20_E_PP" => TPM_SERVICES::TPM_20_E_PP,
            "TPM_20_E_SCHEME" => TPM_SERVICES::TPM_20_E_SCHEME,
            "TPM_20_E_SIZE" => TPM_SERVICES::TPM_20_E_SIZE,
            "TPM_20_E_SYMMETRIC" => TPM_SERVICES::TPM_20_E_SYMMETRIC,
            "TPM_20_E_TAG" => TPM_SERVICES::TPM_20_E_TAG,
            "TPM_20_E_SELECTOR" => TPM_SERVICES::TPM_20_E_SELECTOR,
            "TPM_20_E_INSUFFICIENT" => TPM_SERVICES::TPM_20_E_INSUFFICIENT,
            "TPM_20_E_SIGNATURE" => TPM_SERVICES::TPM_20_E_SIGNATURE,
            "TPM_20_E_KEY" => TPM_SERVICES::TPM_20_E_KEY,
            "TPM_20_E_POLICY_FAIL" => TPM_SERVICES::TPM_20_E_POLICY_FAIL,
            "TPM_20_E_INTEGRITY" => TPM_SERVICES::TPM_20_E_INTEGRITY,
            "TPM_20_E_TICKET" => TPM_SERVICES::TPM_20_E_TICKET,
            "TPM_20_E_RESERVED_BITS" => TPM_SERVICES::TPM_20_E_RESERVED_BITS,
            "TPM_20_E_BAD_AUTH" => TPM_SERVICES::TPM_20_E_BAD_AUTH,
            "TPM_20_E_EXPIRED" => TPM_SERVICES::TPM_20_E_EXPIRED,
            "TPM_20_E_POLICY_CC" => TPM_SERVICES::TPM_20_E_POLICY_CC,
            "TPM_20_E_BINDING" => TPM_SERVICES::TPM_20_E_BINDING,
            "TPM_20_E_CURVE" => TPM_SERVICES::TPM_20_E_CURVE,
            "TPM_20_E_ECC_POINT" => TPM_SERVICES::TPM_20_E_ECC_POINT,
            "TPM_20_E_INITIALIZE" => TPM_SERVICES::TPM_20_E_INITIALIZE,
            "TPM_20_E_FAILURE" => TPM_SERVICES::TPM_20_E_FAILURE,
            "TPM_20_E_SEQUENCE" => TPM_SERVICES::TPM_20_E_SEQUENCE,
            "TPM_20_E_PRIVATE" => TPM_SERVICES::TPM_20_E_PRIVATE,
            "TPM_20_E_HMAC" => TPM_SERVICES::TPM_20_E_HMAC,
            "TPM_20_E_DISABLED" => TPM_SERVICES::TPM_20_E_DISABLED,
            "TPM_20_E_EXCLUSIVE" => TPM_SERVICES::TPM_20_E_EXCLUSIVE,
            "TPM_20_E_ECC_CURVE" => TPM_SERVICES::TPM_20_E_ECC_CURVE,
            "TPM_20_E_AUTH_TYPE" => TPM_SERVICES::TPM_20_E_AUTH_TYPE,
            "TPM_20_E_AUTH_MISSING" => TPM_SERVICES::TPM_20_E_AUTH_MISSING,
            "TPM_20_E_POLICY" => TPM_SERVICES::TPM_20_E_POLICY,
            "TPM_20_E_PCR" => TPM_SERVICES::TPM_20_E_PCR,
            "TPM_20_E_PCR_CHANGED" => TPM_SERVICES::TPM_20_E_PCR_CHANGED,
            "TPM_20_E_UPGRADE" => TPM_SERVICES::TPM_20_E_UPGRADE,
            "TPM_20_E_TOO_MANY_CONTEXTS" => TPM_SERVICES::TPM_20_E_TOO_MANY_CONTEXTS,
            "TPM_20_E_AUTH_UNAVAILABLE" => TPM_SERVICES::TPM_20_E_AUTH_UNAVAILABLE,
            "TPM_20_E_REBOOT" => TPM_SERVICES::TPM_20_E_REBOOT,
            "TPM_20_E_UNBALANCED" => TPM_SERVICES::TPM_20_E_UNBALANCED,
            "TPM_20_E_COMMAND_SIZE" => TPM_SERVICES::TPM_20_E_COMMAND_SIZE,
            "TPM_20_E_COMMAND_CODE" => TPM_SERVICES::TPM_20_E_COMMAND_CODE,
            "TPM_20_E_AUTHSIZE" => TPM_SERVICES::TPM_20_E_AUTHSIZE,
            "TPM_20_E_AUTH_CONTEXT" => TPM_SERVICES::TPM_20_E_AUTH_CONTEXT,
            "TPM_20_E_NV_RANGE" => TPM_SERVICES::TPM_20_E_NV_RANGE,
            "TPM_20_E_NV_SIZE" => TPM_SERVICES::TPM_20_E_NV_SIZE,
            "TPM_20_E_NV_LOCKED" => TPM_SERVICES::TPM_20_E_NV_LOCKED,
            "TPM_20_E_NV_AUTHORIZATION" => TPM_SERVICES::TPM_20_E_NV_AUTHORIZATION,
            "TPM_20_E_NV_UNINITIALIZED" => TPM_SERVICES::TPM_20_E_NV_UNINITIALIZED,
            "TPM_20_E_NV_SPACE" => TPM_SERVICES::TPM_20_E_NV_SPACE,
            "TPM_20_E_NV_DEFINED" => TPM_SERVICES::TPM_20_E_NV_DEFINED,
            "TPM_20_E_BAD_CONTEXT" => TPM_SERVICES::TPM_20_E_BAD_CONTEXT,
            "TPM_20_E_CPHASH" => TPM_SERVICES::TPM_20_E_CPHASH,
            "TPM_20_E_PARENT" => TPM_SERVICES::TPM_20_E_PARENT,
            "TPM_20_E_NEEDS_TEST" => TPM_SERVICES::TPM_20_E_NEEDS_TEST,
            "TPM_20_E_NO_RESULT" => TPM_SERVICES::TPM_20_E_NO_RESULT,
            "TPM_20_E_SENSITIVE" => TPM_SERVICES::TPM_20_E_SENSITIVE,
            "TPM_E_COMMAND_BLOCKED" => TPM_SERVICES::TPM_E_COMMAND_BLOCKED,
            "TPM_E_INVALID_HANDLE" => TPM_SERVICES::TPM_E_INVALID_HANDLE,
            "TPM_E_DUPLICATE_VHANDLE" => TPM_SERVICES::TPM_E_DUPLICATE_VHANDLE,
            "TPM_E_EMBEDDED_COMMAND_BLOCKED" => TPM_SERVICES::TPM_E_EMBEDDED_COMMAND_BLOCKED,
            "TPM_E_EMBEDDED_COMMAND_UNSUPPORTED" => TPM_SERVICES::TPM_E_EMBEDDED_COMMAND_UNSUPPORTED,
            "TPM_E_RETRY" => TPM_SERVICES::TPM_E_RETRY,
            "TPM_E_NEEDS_SELFTEST" => TPM_SERVICES::TPM_E_NEEDS_SELFTEST,
            "TPM_E_DOING_SELFTEST" => TPM_SERVICES::TPM_E_DOING_SELFTEST,
            "TPM_E_DEFEND_LOCK_RUNNING" => TPM_SERVICES::TPM_E_DEFEND_LOCK_RUNNING,
            "TPM_20_E_CONTEXT_GAP" => TPM_SERVICES::TPM_20_E_CONTEXT_GAP,
            "TPM_20_E_OBJECT_MEMORY" => TPM_SERVICES::TPM_20_E_OBJECT_MEMORY,
            "TPM_20_E_SESSION_MEMORY" => TPM_SERVICES::TPM_20_E_SESSION_MEMORY,
            "TPM_20_E_MEMORY" => TPM_SERVICES::TPM_20_E_MEMORY,
            "TPM_20_E_SESSION_HANDLES" => TPM_SERVICES::TPM_20_E_SESSION_HANDLES,
            "TPM_20_E_OBJECT_HANDLES" => TPM_SERVICES::TPM_20_E_OBJECT_HANDLES,
            "TPM_20_E_LOCALITY" => TPM_SERVICES::TPM_20_E_LOCALITY,
            "TPM_20_E_YIELDED" => TPM_SERVICES::TPM_20_E_YIELDED,
            "TPM_20_E_CANCELED" => TPM_SERVICES::TPM_20_E_CANCELED,
            "TPM_20_E_TESTING" => TPM_SERVICES::TPM_20_E_TESTING,
            "TPM_20_E_NV_RATE" => TPM_SERVICES::TPM_20_E_NV_RATE,
            "TPM_20_E_LOCKOUT" => TPM_SERVICES::TPM_20_E_LOCKOUT,
            "TPM_20_E_RETRY" => TPM_SERVICES::TPM_20_E_RETRY,
            "TPM_20_E_NV_UNAVAILABLE" => TPM_SERVICES::TPM_20_E_NV_UNAVAILABLE,
            "TBS_E_INTERNAL_ERROR" => TPM_SERVICES::TBS_E_INTERNAL_ERROR,
            "TBS_E_BAD_PARAMETER" => TPM_SERVICES::TBS_E_BAD_PARAMETER,
            "TBS_E_INVALID_OUTPUT_POINTER" => TPM_SERVICES::TBS_E_INVALID_OUTPUT_POINTER,
            "TBS_E_INVALID_CONTEXT" => TPM_SERVICES::TBS_E_INVALID_CONTEXT,
            "TBS_E_INSUFFICIENT_BUFFER" => TPM_SERVICES::TBS_E_INSUFFICIENT_BUFFER,
            "TBS_E_IOERROR" => TPM_SERVICES::TBS_E_IOERROR,
            "TBS_E_INVALID_CONTEXT_PARAM" => TPM_SERVICES::TBS_E_INVALID_CONTEXT_PARAM,
            "TBS_E_SERVICE_NOT_RUNNING" => TPM_SERVICES::TBS_E_SERVICE_NOT_RUNNING,
            "TBS_E_TOO_MANY_TBS_CONTEXTS" => TPM_SERVICES::TBS_E_TOO_MANY_TBS_CONTEXTS,
            "TBS_E_TOO_MANY_RESOURCES" => TPM_SERVICES::TBS_E_TOO_MANY_RESOURCES,
            "TBS_E_SERVICE_START_PENDING" => TPM_SERVICES::TBS_E_SERVICE_START_PENDING,
            "TBS_E_PPI_NOT_SUPPORTED" => TPM_SERVICES::TBS_E_PPI_NOT_SUPPORTED,
            "TBS_E_COMMAND_CANCELED" => TPM_SERVICES::TBS_E_COMMAND_CANCELED,
            "TBS_E_BUFFER_TOO_LARGE" => TPM_SERVICES::TBS_E_BUFFER_TOO_LARGE,
            "TBS_E_TPM_NOT_FOUND" => TPM_SERVICES::TBS_E_TPM_NOT_FOUND,
            "TBS_E_SERVICE_DISABLED" => TPM_SERVICES::TBS_E_SERVICE_DISABLED,
            "TBS_E_NO_EVENT_LOG" => TPM_SERVICES::TBS_E_NO_EVENT_LOG,
            "TBS_E_ACCESS_DENIED" => TPM_SERVICES::TBS_E_ACCESS_DENIED,
            "TBS_E_PROVISIONING_NOT_ALLOWED" => TPM_SERVICES::TBS_E_PROVISIONING_NOT_ALLOWED,
            "TBS_E_PPI_FUNCTION_UNSUPPORTED" => TPM_SERVICES::TBS_E_PPI_FUNCTION_UNSUPPORTED,
            "TBS_E_OWNERAUTH_NOT_FOUND" => TPM_SERVICES::TBS_E_OWNERAUTH_NOT_FOUND,
            "TBS_E_PROVISIONING_INCOMPLETE" => TPM_SERVICES::TBS_E_PROVISIONING_INCOMPLETE,
        }
    }
    pub fn from_code(code: u32) -> TPM_SERVICES {
        return match code {
            0x80280000 => TPM_SERVICES::TPM_E_ERROR_MASK,
            0x80280001 => TPM_SERVICES::TPM_E_AUTHFAIL,
            0x80280001 => TPM_SERVICES::E_INK_EXCEPTION,
            0x80280002 => TPM_SERVICES::TPM_E_BADINDEX,
            0x80280002 => TPM_SERVICES::E_INK_MISMATCHED_INK_OBJECT,
            0x80280003 => TPM_SERVICES::TPM_E_BAD_PARAMETER,
            0x80280003 => TPM_SERVICES::E_INK_COLLECTOR_BUSY,
            0x80280004 => TPM_SERVICES::TPM_E_AUDITFAILURE,
            0x80280004 => TPM_SERVICES::E_INK_INCOMPATIBLE_OBJECT,
            0x80280005 => TPM_SERVICES::TPM_E_CLEAR_DISABLED,
            0x80280005 => TPM_SERVICES::E_INK_WINDOW_NOT_SET,
            0x80280006 => TPM_SERVICES::TPM_E_DEACTIVATED,
            0x80280006 => TPM_SERVICES::E_INK_INVALID_MODE,
            0x80280007 => TPM_SERVICES::TPM_E_DISABLED,
            0x80280007 => TPM_SERVICES::E_INK_COLLECTOR_ENABLED,
            0x80280008 => TPM_SERVICES::TPM_E_DISABLED_CMD,
            0x80280008 => TPM_SERVICES::E_INK_NO_STROKES_TO_RECOGNIZE,
            0x80280009 => TPM_SERVICES::TPM_E_FAIL,
            0x80280009 => TPM_SERVICES::E_INK_EMPTY_RECOGNITION_RESULT,
            0x8028000A => TPM_SERVICES::TPM_E_BAD_ORDINAL,
            0x8028000B => TPM_SERVICES::TPM_E_INSTALL_DISABLED,
            0x8028000C => TPM_SERVICES::TPM_E_INVALID_KEYHANDLE,
            0x8028000D => TPM_SERVICES::TPM_E_KEYNOTFOUND,
            0x8028000E => TPM_SERVICES::TPM_E_INAPPROPRIATE_ENC,
            0x8028000F => TPM_SERVICES::TPM_E_MIGRATEFAIL,
            0x80280010 => TPM_SERVICES::TPM_E_INVALID_PCR_INFO,
            0x80280010 => TPM_SERVICES::E_INK_OVERLAPPING_INPUT_RECT,
            0x80280011 => TPM_SERVICES::TPM_E_NOSPACE,
            0x80280012 => TPM_SERVICES::TPM_E_NOSRK,
            0x80280013 => TPM_SERVICES::TPM_E_NOTSEALED_BLOB,
            0x80280014 => TPM_SERVICES::TPM_E_OWNER_SET,
            0x80280015 => TPM_SERVICES::TPM_E_RESOURCES,
            0x80280016 => TPM_SERVICES::TPM_E_SHORTRANDOM,
            0x80280017 => TPM_SERVICES::TPM_E_SIZE,
            0x80280018 => TPM_SERVICES::TPM_E_WRONGPCRVAL,
            0x80280019 => TPM_SERVICES::TPM_E_BAD_PARAM_SIZE,
            0x8028001A => TPM_SERVICES::TPM_E_SHA_THREAD,
            0x8028001B => TPM_SERVICES::TPM_E_SHA_ERROR,
            0x8028001C => TPM_SERVICES::TPM_E_FAILEDSELFTEST,
            0x8028001D => TPM_SERVICES::TPM_E_AUTH2FAIL,
            0x8028001E => TPM_SERVICES::TPM_E_BADTAG,
            0x8028001F => TPM_SERVICES::TPM_E_IOERROR,
            0x80280020 => TPM_SERVICES::TPM_E_ENCRYPT_ERROR,
            0x80280021 => TPM_SERVICES::TPM_E_DECRYPT_ERROR,
            0x80280022 => TPM_SERVICES::TPM_E_INVALID_AUTHHANDLE,
            0x80280023 => TPM_SERVICES::TPM_E_NO_ENDORSEMENT,
            0x80280024 => TPM_SERVICES::TPM_E_INVALID_KEYUSAGE,
            0x80280025 => TPM_SERVICES::TPM_E_WRONG_ENTITYTYPE,
            0x80280026 => TPM_SERVICES::TPM_E_INVALID_POSTINIT,
            0x80280027 => TPM_SERVICES::TPM_E_INAPPROPRIATE_SIG,
            0x80280028 => TPM_SERVICES::TPM_E_BAD_KEY_PROPERTY,
            0x80280029 => TPM_SERVICES::TPM_E_BAD_MIGRATION,
            0x8028002A => TPM_SERVICES::TPM_E_BAD_SCHEME,
            0x8028002B => TPM_SERVICES::TPM_E_BAD_DATASIZE,
            0x8028002C => TPM_SERVICES::TPM_E_BAD_MODE,
            0x8028002D => TPM_SERVICES::TPM_E_BAD_PRESENCE,
            0x8028002E => TPM_SERVICES::TPM_E_BAD_VERSION,
            0x8028002F => TPM_SERVICES::TPM_E_NO_WRAP_TRANSPORT,
            0x80280030 => TPM_SERVICES::TPM_E_AUDITFAIL_UNSUCCESSFUL,
            0x80280031 => TPM_SERVICES::TPM_E_AUDITFAIL_SUCCESSFUL,
            0x80280032 => TPM_SERVICES::TPM_E_NOTRESETABLE,
            0x80280033 => TPM_SERVICES::TPM_E_NOTLOCAL,
            0x80280034 => TPM_SERVICES::TPM_E_BAD_TYPE,
            0x80280035 => TPM_SERVICES::TPM_E_INVALID_RESOURCE,
            0x80280036 => TPM_SERVICES::TPM_E_NOTFIPS,
            0x80280037 => TPM_SERVICES::TPM_E_INVALID_FAMILY,
            0x80280038 => TPM_SERVICES::TPM_E_NO_NV_PERMISSION,
            0x80280039 => TPM_SERVICES::TPM_E_REQUIRES_SIGN,
            0x8028003A => TPM_SERVICES::TPM_E_KEY_NOTSUPPORTED,
            0x8028003B => TPM_SERVICES::TPM_E_AUTH_CONFLICT,
            0x8028003C => TPM_SERVICES::TPM_E_AREA_LOCKED,
            0x8028003D => TPM_SERVICES::TPM_E_BAD_LOCALITY,
            0x8028003E => TPM_SERVICES::TPM_E_READ_ONLY,
            0x8028003F => TPM_SERVICES::TPM_E_PER_NOWRITE,
            0x80280040 => TPM_SERVICES::TPM_E_FAMILYCOUNT,
            0x80280041 => TPM_SERVICES::TPM_E_WRITE_LOCKED,
            0x80280042 => TPM_SERVICES::TPM_E_BAD_ATTRIBUTES,
            0x80280043 => TPM_SERVICES::TPM_E_INVALID_STRUCTURE,
            0x80280044 => TPM_SERVICES::TPM_E_KEY_OWNER_CONTROL,
            0x80280045 => TPM_SERVICES::TPM_E_BAD_COUNTER,
            0x80280046 => TPM_SERVICES::TPM_E_NOT_FULLWRITE,
            0x80280047 => TPM_SERVICES::TPM_E_CONTEXT_GAP,
            0x80280048 => TPM_SERVICES::TPM_E_MAXNVWRITES,
            0x80280049 => TPM_SERVICES::TPM_E_NOOPERATOR,
            0x8028004A => TPM_SERVICES::TPM_E_RESOURCEMISSING,
            0x8028004B => TPM_SERVICES::TPM_E_DELEGATE_LOCK,
            0x8028004C => TPM_SERVICES::TPM_E_DELEGATE_FAMILY,
            0x8028004D => TPM_SERVICES::TPM_E_DELEGATE_ADMIN,
            0x8028004E => TPM_SERVICES::TPM_E_TRANSPORT_NOTEXCLUSIVE,
            0x8028004F => TPM_SERVICES::TPM_E_OWNER_CONTROL,
            0x80280050 => TPM_SERVICES::TPM_E_DAA_RESOURCES,
            0x80280051 => TPM_SERVICES::TPM_E_DAA_INPUT_DATA0,
            0x80280052 => TPM_SERVICES::TPM_E_DAA_INPUT_DATA1,
            0x80280053 => TPM_SERVICES::TPM_E_DAA_ISSUER_SETTINGS,
            0x80280054 => TPM_SERVICES::TPM_E_DAA_TPM_SETTINGS,
            0x80280055 => TPM_SERVICES::TPM_E_DAA_STAGE,
            0x80280056 => TPM_SERVICES::TPM_E_DAA_ISSUER_VALIDITY,
            0x80280057 => TPM_SERVICES::TPM_E_DAA_WRONG_W,
            0x80280058 => TPM_SERVICES::TPM_E_BAD_HANDLE,
            0x80280059 => TPM_SERVICES::TPM_E_BAD_DELEGATE,
            0x8028005A => TPM_SERVICES::TPM_E_BADCONTEXT,
            0x8028005B => TPM_SERVICES::TPM_E_TOOMANYCONTEXTS,
            0x8028005C => TPM_SERVICES::TPM_E_MA_TICKET_SIGNATURE,
            0x8028005D => TPM_SERVICES::TPM_E_MA_DESTINATION,
            0x8028005E => TPM_SERVICES::TPM_E_MA_SOURCE,
            0x8028005F => TPM_SERVICES::TPM_E_MA_AUTHORITY,
            0x80280061 => TPM_SERVICES::TPM_E_PERMANENTEK,
            0x80280062 => TPM_SERVICES::TPM_E_BAD_SIGNATURE,
            0x80280063 => TPM_SERVICES::TPM_E_NOCONTEXTSPACE,
            0x80280081 => TPM_SERVICES::TPM_20_E_ASYMMETRIC,
            0x80280082 => TPM_SERVICES::TPM_20_E_ATTRIBUTES,
            0x80280083 => TPM_SERVICES::TPM_20_E_HASH,
            0x80280084 => TPM_SERVICES::TPM_20_E_VALUE,
            0x80280085 => TPM_SERVICES::TPM_20_E_HIERARCHY,
            0x80280087 => TPM_SERVICES::TPM_20_E_KEY_SIZE,
            0x80280088 => TPM_SERVICES::TPM_20_E_MGF,
            0x80280089 => TPM_SERVICES::TPM_20_E_MODE,
            0x8028008A => TPM_SERVICES::TPM_20_E_TYPE,
            0x8028008B => TPM_SERVICES::TPM_20_E_HANDLE,
            0x8028008C => TPM_SERVICES::TPM_20_E_KDF,
            0x8028008D => TPM_SERVICES::TPM_20_E_RANGE,
            0x8028008E => TPM_SERVICES::TPM_20_E_AUTH_FAIL,
            0x8028008F => TPM_SERVICES::TPM_20_E_NONCE,
            0x80280090 => TPM_SERVICES::TPM_20_E_PP,
            0x80280092 => TPM_SERVICES::TPM_20_E_SCHEME,
            0x80280095 => TPM_SERVICES::TPM_20_E_SIZE,
            0x80280096 => TPM_SERVICES::TPM_20_E_SYMMETRIC,
            0x80280097 => TPM_SERVICES::TPM_20_E_TAG,
            0x80280098 => TPM_SERVICES::TPM_20_E_SELECTOR,
            0x8028009A => TPM_SERVICES::TPM_20_E_INSUFFICIENT,
            0x8028009B => TPM_SERVICES::TPM_20_E_SIGNATURE,
            0x8028009C => TPM_SERVICES::TPM_20_E_KEY,
            0x8028009D => TPM_SERVICES::TPM_20_E_POLICY_FAIL,
            0x8028009F => TPM_SERVICES::TPM_20_E_INTEGRITY,
            0x802800A0 => TPM_SERVICES::TPM_20_E_TICKET,
            0x802800A1 => TPM_SERVICES::TPM_20_E_RESERVED_BITS,
            0x802800A2 => TPM_SERVICES::TPM_20_E_BAD_AUTH,
            0x802800A3 => TPM_SERVICES::TPM_20_E_EXPIRED,
            0x802800A4 => TPM_SERVICES::TPM_20_E_POLICY_CC,
            0x802800A5 => TPM_SERVICES::TPM_20_E_BINDING,
            0x802800A6 => TPM_SERVICES::TPM_20_E_CURVE,
            0x802800A7 => TPM_SERVICES::TPM_20_E_ECC_POINT,
            0x80280100 => TPM_SERVICES::TPM_20_E_INITIALIZE,
            0x80280101 => TPM_SERVICES::TPM_20_E_FAILURE,
            0x80280103 => TPM_SERVICES::TPM_20_E_SEQUENCE,
            0x8028010B => TPM_SERVICES::TPM_20_E_PRIVATE,
            0x80280119 => TPM_SERVICES::TPM_20_E_HMAC,
            0x80280120 => TPM_SERVICES::TPM_20_E_DISABLED,
            0x80280121 => TPM_SERVICES::TPM_20_E_EXCLUSIVE,
            0x80280123 => TPM_SERVICES::TPM_20_E_ECC_CURVE,
            0x80280124 => TPM_SERVICES::TPM_20_E_AUTH_TYPE,
            0x80280125 => TPM_SERVICES::TPM_20_E_AUTH_MISSING,
            0x80280126 => TPM_SERVICES::TPM_20_E_POLICY,
            0x80280127 => TPM_SERVICES::TPM_20_E_PCR,
            0x80280128 => TPM_SERVICES::TPM_20_E_PCR_CHANGED,
            0x8028012D => TPM_SERVICES::TPM_20_E_UPGRADE,
            0x8028012E => TPM_SERVICES::TPM_20_E_TOO_MANY_CONTEXTS,
            0x8028012F => TPM_SERVICES::TPM_20_E_AUTH_UNAVAILABLE,
            0x80280130 => TPM_SERVICES::TPM_20_E_REBOOT,
            0x80280131 => TPM_SERVICES::TPM_20_E_UNBALANCED,
            0x80280142 => TPM_SERVICES::TPM_20_E_COMMAND_SIZE,
            0x80280143 => TPM_SERVICES::TPM_20_E_COMMAND_CODE,
            0x80280144 => TPM_SERVICES::TPM_20_E_AUTHSIZE,
            0x80280145 => TPM_SERVICES::TPM_20_E_AUTH_CONTEXT,
            0x80280146 => TPM_SERVICES::TPM_20_E_NV_RANGE,
            0x80280147 => TPM_SERVICES::TPM_20_E_NV_SIZE,
            0x80280148 => TPM_SERVICES::TPM_20_E_NV_LOCKED,
            0x80280149 => TPM_SERVICES::TPM_20_E_NV_AUTHORIZATION,
            0x8028014A => TPM_SERVICES::TPM_20_E_NV_UNINITIALIZED,
            0x8028014B => TPM_SERVICES::TPM_20_E_NV_SPACE,
            0x8028014C => TPM_SERVICES::TPM_20_E_NV_DEFINED,
            0x80280150 => TPM_SERVICES::TPM_20_E_BAD_CONTEXT,
            0x80280151 => TPM_SERVICES::TPM_20_E_CPHASH,
            0x80280152 => TPM_SERVICES::TPM_20_E_PARENT,
            0x80280153 => TPM_SERVICES::TPM_20_E_NEEDS_TEST,
            0x80280154 => TPM_SERVICES::TPM_20_E_NO_RESULT,
            0x80280155 => TPM_SERVICES::TPM_20_E_SENSITIVE,
            0x80280400 => TPM_SERVICES::TPM_E_COMMAND_BLOCKED,
            0x80280401 => TPM_SERVICES::TPM_E_INVALID_HANDLE,
            0x80280402 => TPM_SERVICES::TPM_E_DUPLICATE_VHANDLE,
            0x80280403 => TPM_SERVICES::TPM_E_EMBEDDED_COMMAND_BLOCKED,
            0x80280404 => TPM_SERVICES::TPM_E_EMBEDDED_COMMAND_UNSUPPORTED,
            0x80280800 => TPM_SERVICES::TPM_E_RETRY,
            0x80280801 => TPM_SERVICES::TPM_E_NEEDS_SELFTEST,
            0x80280802 => TPM_SERVICES::TPM_E_DOING_SELFTEST,
            0x80280803 => TPM_SERVICES::TPM_E_DEFEND_LOCK_RUNNING,
            0x80280901 => TPM_SERVICES::TPM_20_E_CONTEXT_GAP,
            0x80280902 => TPM_SERVICES::TPM_20_E_OBJECT_MEMORY,
            0x80280903 => TPM_SERVICES::TPM_20_E_SESSION_MEMORY,
            0x80280904 => TPM_SERVICES::TPM_20_E_MEMORY,
            0x80280905 => TPM_SERVICES::TPM_20_E_SESSION_HANDLES,
            0x80280906 => TPM_SERVICES::TPM_20_E_OBJECT_HANDLES,
            0x80280907 => TPM_SERVICES::TPM_20_E_LOCALITY,
            0x80280908 => TPM_SERVICES::TPM_20_E_YIELDED,
            0x80280909 => TPM_SERVICES::TPM_20_E_CANCELED,
            0x8028090A => TPM_SERVICES::TPM_20_E_TESTING,
            0x80280920 => TPM_SERVICES::TPM_20_E_NV_RATE,
            0x80280921 => TPM_SERVICES::TPM_20_E_LOCKOUT,
            0x80280922 => TPM_SERVICES::TPM_20_E_RETRY,
            0x80280923 => TPM_SERVICES::TPM_20_E_NV_UNAVAILABLE,
            0x80284001 => TPM_SERVICES::TBS_E_INTERNAL_ERROR,
            0x80284002 => TPM_SERVICES::TBS_E_BAD_PARAMETER,
            0x80284003 => TPM_SERVICES::TBS_E_INVALID_OUTPUT_POINTER,
            0x80284004 => TPM_SERVICES::TBS_E_INVALID_CONTEXT,
            0x80284005 => TPM_SERVICES::TBS_E_INSUFFICIENT_BUFFER,
            0x80284006 => TPM_SERVICES::TBS_E_IOERROR,
            0x80284007 => TPM_SERVICES::TBS_E_INVALID_CONTEXT_PARAM,
            0x80284008 => TPM_SERVICES::TBS_E_SERVICE_NOT_RUNNING,
            0x80284009 => TPM_SERVICES::TBS_E_TOO_MANY_TBS_CONTEXTS,
            0x8028400A => TPM_SERVICES::TBS_E_TOO_MANY_RESOURCES,
            0x8028400B => TPM_SERVICES::TBS_E_SERVICE_START_PENDING,
            0x8028400C => TPM_SERVICES::TBS_E_PPI_NOT_SUPPORTED,
            0x8028400D => TPM_SERVICES::TBS_E_COMMAND_CANCELED,
            0x8028400E => TPM_SERVICES::TBS_E_BUFFER_TOO_LARGE,
            0x8028400F => TPM_SERVICES::TBS_E_TPM_NOT_FOUND,
            0x80284010 => TPM_SERVICES::TBS_E_SERVICE_DISABLED,
            0x80284011 => TPM_SERVICES::TBS_E_NO_EVENT_LOG,
            0x80284012 => TPM_SERVICES::TBS_E_ACCESS_DENIED,
            0x80284013 => TPM_SERVICES::TBS_E_PROVISIONING_NOT_ALLOWED,
            0x80284014 => TPM_SERVICES::TBS_E_PPI_FUNCTION_UNSUPPORTED,
            0x80284015 => TPM_SERVICES::TBS_E_OWNERAUTH_NOT_FOUND,
            0x80284016 => TPM_SERVICES::TBS_E_PROVISIONING_INCOMPLETE,
        }
    }
}
