/*
 * Copyright 2023 Stanislav Mikhailov (xavetar)
 *
 * Licensed under the Creative Commons Zero v1.0 Universal (CC0) License.
 * You may obtain a copy of the License at
 *
 *     http://creativecommons.org/publicdomain/zero/1.0/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the CC0 license is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// It is strictly forbidden to use the from_code method, respected Microsoft decided to define
// duplicate codes, but by defining different constant names, therefore, when using these codes,
// it is mandatory to use through from_name, and not from_code. Otherwise, it may cause undefined
// behavior or an unknown exception. Because one code corresponds to several constants.

use super::{RawError};

#[derive(Clone, Debug, Copy, Eq, Hash, Ord, PartialEq, PartialOrd)]
pub enum TPM_SOFTWARE {
    TPM_E_PCP_UNSUPPORTED_PSS_SALT,
    TPM_E_PCP_PLATFORM_CLAIM_MAY_BE_OUTDATED,
    TPM_E_PCP_PLATFORM_CLAIM_OUTDATED,
    TPM_E_PCP_PLATFORM_CLAIM_REBOOT,
    TPMAPI_E_INVALID_STATE,
    TPMAPI_E_NOT_ENOUGH_DATA,
    TPMAPI_E_TOO_MUCH_DATA,
    TPMAPI_E_INVALID_OUTPUT_POINTER,
    TPMAPI_E_INVALID_PARAMETER,
    TPMAPI_E_OUT_OF_MEMORY,
    TPMAPI_E_BUFFER_TOO_SMALL,
    TPMAPI_E_INTERNAL_ERROR,
    TPMAPI_E_ACCESS_DENIED,
    TPMAPI_E_AUTHORIZATION_FAILED,
    TPMAPI_E_INVALID_CONTEXT_HANDLE,
    TPMAPI_E_TBS_COMMUNICATION_ERROR,
    TPMAPI_E_TPM_COMMAND_ERROR,
    TPMAPI_E_MESSAGE_TOO_LARGE,
    TPMAPI_E_INVALID_ENCODING,
    TPMAPI_E_INVALID_KEY_SIZE,
    TPMAPI_E_ENCRYPTION_FAILED,
    TPMAPI_E_INVALID_KEY_PARAMS,
    TPMAPI_E_INVALID_MIGRATION_AUTHORIZATION_BLOB,
    TPMAPI_E_INVALID_PCR_INDEX,
    TPMAPI_E_INVALID_DELEGATE_BLOB,
    TPMAPI_E_INVALID_CONTEXT_PARAMS,
    TPMAPI_E_INVALID_KEY_BLOB,
    TPMAPI_E_INVALID_PCR_DATA,
    TPMAPI_E_INVALID_OWNER_AUTH,
    TPMAPI_E_FIPS_RNG_CHECK_FAILED,
    TPMAPI_E_EMPTY_TCG_LOG,
    TPMAPI_E_INVALID_TCG_LOG_ENTRY,
    TPMAPI_E_TCG_SEPARATOR_ABSENT,
    TPMAPI_E_TCG_INVALID_DIGEST_ENTRY,
    TPMAPI_E_POLICY_DENIES_OPERATION,
    TPMAPI_E_NV_BITS_NOT_DEFINED,
    TPMAPI_E_NV_BITS_NOT_READY,
    TPMAPI_E_SEALING_KEY_NOT_AVAILABLE,
    TPMAPI_E_NO_AUTHORIZATION_CHAIN_FOUND,
    TPMAPI_E_SVN_COUNTER_NOT_AVAILABLE,
    TPMAPI_E_OWNER_AUTH_NOT_NULL,
    TPMAPI_E_ENDORSEMENT_AUTH_NOT_NULL,
    TPMAPI_E_AUTHORIZATION_REVOKED,
    TPMAPI_E_MALFORMED_AUTHORIZATION_KEY,
    TPMAPI_E_AUTHORIZING_KEY_NOT_SUPPORTED,
    TPMAPI_E_INVALID_AUTHORIZATION_SIGNATURE,
    TPMAPI_E_MALFORMED_AUTHORIZATION_POLICY,
    TPMAPI_E_MALFORMED_AUTHORIZATION_OTHER,
    TPMAPI_E_SEALING_KEY_CHANGED,
    TPMAPI_E_INVALID_TPM_VERSION,
    TPMAPI_E_INVALID_POLICYAUTH_BLOB_TYPE,
    TBSIMP_E_BUFFER_TOO_SMALL,
    TBSIMP_E_CLEANUP_FAILED,
    TBSIMP_E_INVALID_CONTEXT_HANDLE,
    TBSIMP_E_INVALID_CONTEXT_PARAM,
    TBSIMP_E_TPM_ERROR,
    TBSIMP_E_HASH_BAD_KEY,
    TBSIMP_E_DUPLICATE_VHANDLE,
    TBSIMP_E_INVALID_OUTPUT_POINTER,
    TBSIMP_E_INVALID_PARAMETER,
    TBSIMP_E_RPC_INIT_FAILED,
    TBSIMP_E_SCHEDULER_NOT_RUNNING,
    TBSIMP_E_COMMAND_CANCELED,
    TBSIMP_E_OUT_OF_MEMORY,
    TBSIMP_E_LIST_NO_MORE_ITEMS,
    TBSIMP_E_LIST_NOT_FOUND,
    TBSIMP_E_NOT_ENOUGH_SPACE,
    TBSIMP_E_NOT_ENOUGH_TPM_CONTEXTS,
    TBSIMP_E_COMMAND_FAILED,
    TBSIMP_E_UNKNOWN_ORDINAL,
    TBSIMP_E_RESOURCE_EXPIRED,
    TBSIMP_E_INVALID_RESOURCE,
    TBSIMP_E_NOTHING_TO_UNLOAD,
    TBSIMP_E_HASH_TABLE_FULL,
    TBSIMP_E_TOO_MANY_TBS_CONTEXTS,
    TBSIMP_E_TOO_MANY_RESOURCES,
    TBSIMP_E_PPI_NOT_SUPPORTED,
    TBSIMP_E_TPM_INCOMPATIBLE,
    TBSIMP_E_NO_EVENT_LOG,
    TPM_E_PPI_ACPI_FAILURE,
    TPM_E_PPI_USER_ABORT,
    TPM_E_PPI_BIOS_FAILURE,
    TPM_E_PPI_NOT_SUPPORTED,
    TPM_E_PPI_BLOCKED_IN_BIOS,
    TPM_E_PCP_ERROR_MASK,
    TPM_E_PCP_DEVICE_NOT_READY,
    TPM_E_PCP_INVALID_HANDLE,
    TPM_E_PCP_INVALID_PARAMETER,
    TPM_E_PCP_FLAG_NOT_SUPPORTED,
    TPM_E_PCP_NOT_SUPPORTED,
    TPM_E_PCP_BUFFER_TOO_SMALL,
    TPM_E_PCP_INTERNAL_ERROR,
    TPM_E_PCP_AUTHENTICATION_FAILED,
    TPM_E_PCP_AUTHENTICATION_IGNORED,
    TPM_E_PCP_POLICY_NOT_FOUND,
    TPM_E_PCP_PROFILE_NOT_FOUND,
    TPM_E_PCP_VALIDATION_FAILED,
    TPM_E_PCP_WRONG_PARENT,
    TPM_E_KEY_NOT_LOADED,
    TPM_E_NO_KEY_CERTIFICATION,
    TPM_E_KEY_NOT_FINALIZED,
    TPM_E_ATTESTATION_CHALLENGE_NOT_SET,
    TPM_E_NOT_PCR_BOUND,
    TPM_E_KEY_ALREADY_FINALIZED,
    TPM_E_KEY_USAGE_POLICY_NOT_SUPPORTED,
    TPM_E_KEY_USAGE_POLICY_INVALID,
    TPM_E_SOFT_KEY_ERROR,
    TPM_E_KEY_NOT_AUTHENTICATED,
    TPM_E_PCP_KEY_NOT_AIK,
    TPM_E_KEY_NOT_SIGNING_KEY,
    TPM_E_LOCKED_OUT,
    TPM_E_CLAIM_TYPE_NOT_SUPPORTED,
    TPM_E_VERSION_NOT_SUPPORTED,
    TPM_E_BUFFER_LENGTH_MISMATCH,
    TPM_E_PCP_IFX_RSA_KEY_CREATION_BLOCKED,
    TPM_E_PCP_TICKET_MISSING,
    TPM_E_PCP_RAW_POLICY_NOT_SUPPORTED,
    TPM_E_PCP_KEY_HANDLE_INVALIDATED,
    TPM_E_ZERO_EXHAUST_ENABLED,
    TPM_E_PROVISIONING_INCOMPLETE,
    TPM_E_INVALID_OWNER_AUTH,
    TPM_E_TOO_MUCH_DATA,
    TPM_E_TPM_GENERATED_EPS,
}

impl TPM_SOFTWARE {
    pub fn code(&self) -> u32 {
        return match self {
            TPM_SOFTWARE::TPM_E_PCP_UNSUPPORTED_PSS_SALT => 0x40290423 as u32,
            TPM_SOFTWARE::TPM_E_PCP_PLATFORM_CLAIM_MAY_BE_OUTDATED => 0x40290424 as u32,
            TPM_SOFTWARE::TPM_E_PCP_PLATFORM_CLAIM_OUTDATED => 0x40290425 as u32,
            TPM_SOFTWARE::TPM_E_PCP_PLATFORM_CLAIM_REBOOT => 0x40290426 as u32,
            TPM_SOFTWARE::TPMAPI_E_INVALID_STATE => 0x80290100 as u32,
            TPM_SOFTWARE::TPMAPI_E_NOT_ENOUGH_DATA => 0x80290101 as u32,
            TPM_SOFTWARE::TPMAPI_E_TOO_MUCH_DATA => 0x80290102 as u32,
            TPM_SOFTWARE::TPMAPI_E_INVALID_OUTPUT_POINTER => 0x80290103 as u32,
            TPM_SOFTWARE::TPMAPI_E_INVALID_PARAMETER => 0x80290104 as u32,
            TPM_SOFTWARE::TPMAPI_E_OUT_OF_MEMORY => 0x80290105 as u32,
            TPM_SOFTWARE::TPMAPI_E_BUFFER_TOO_SMALL => 0x80290106 as u32,
            TPM_SOFTWARE::TPMAPI_E_INTERNAL_ERROR => 0x80290107 as u32,
            TPM_SOFTWARE::TPMAPI_E_ACCESS_DENIED => 0x80290108 as u32,
            TPM_SOFTWARE::TPMAPI_E_AUTHORIZATION_FAILED => 0x80290109 as u32,
            TPM_SOFTWARE::TPMAPI_E_INVALID_CONTEXT_HANDLE => 0x8029010A as u32,
            TPM_SOFTWARE::TPMAPI_E_TBS_COMMUNICATION_ERROR => 0x8029010B as u32,
            TPM_SOFTWARE::TPMAPI_E_TPM_COMMAND_ERROR => 0x8029010C as u32,
            TPM_SOFTWARE::TPMAPI_E_MESSAGE_TOO_LARGE => 0x8029010D as u32,
            TPM_SOFTWARE::TPMAPI_E_INVALID_ENCODING => 0x8029010E as u32,
            TPM_SOFTWARE::TPMAPI_E_INVALID_KEY_SIZE => 0x8029010F as u32,
            TPM_SOFTWARE::TPMAPI_E_ENCRYPTION_FAILED => 0x80290110 as u32,
            TPM_SOFTWARE::TPMAPI_E_INVALID_KEY_PARAMS => 0x80290111 as u32,
            TPM_SOFTWARE::TPMAPI_E_INVALID_MIGRATION_AUTHORIZATION_BLOB => 0x80290112 as u32,
            TPM_SOFTWARE::TPMAPI_E_INVALID_PCR_INDEX => 0x80290113 as u32,
            TPM_SOFTWARE::TPMAPI_E_INVALID_DELEGATE_BLOB => 0x80290114 as u32,
            TPM_SOFTWARE::TPMAPI_E_INVALID_CONTEXT_PARAMS => 0x80290115 as u32,
            TPM_SOFTWARE::TPMAPI_E_INVALID_KEY_BLOB => 0x80290116 as u32,
            TPM_SOFTWARE::TPMAPI_E_INVALID_PCR_DATA => 0x80290117 as u32,
            TPM_SOFTWARE::TPMAPI_E_INVALID_OWNER_AUTH => 0x80290118 as u32,
            TPM_SOFTWARE::TPMAPI_E_FIPS_RNG_CHECK_FAILED => 0x80290119 as u32,
            TPM_SOFTWARE::TPMAPI_E_EMPTY_TCG_LOG => 0x8029011A as u32,
            TPM_SOFTWARE::TPMAPI_E_INVALID_TCG_LOG_ENTRY => 0x8029011B as u32,
            TPM_SOFTWARE::TPMAPI_E_TCG_SEPARATOR_ABSENT => 0x8029011C as u32,
            TPM_SOFTWARE::TPMAPI_E_TCG_INVALID_DIGEST_ENTRY => 0x8029011D as u32,
            TPM_SOFTWARE::TPMAPI_E_POLICY_DENIES_OPERATION => 0x8029011E as u32,
            TPM_SOFTWARE::TPMAPI_E_NV_BITS_NOT_DEFINED => 0x8029011F as u32,
            TPM_SOFTWARE::TPMAPI_E_NV_BITS_NOT_READY => 0x80290120 as u32,
            TPM_SOFTWARE::TPMAPI_E_SEALING_KEY_NOT_AVAILABLE => 0x80290121 as u32,
            TPM_SOFTWARE::TPMAPI_E_NO_AUTHORIZATION_CHAIN_FOUND => 0x80290122 as u32,
            TPM_SOFTWARE::TPMAPI_E_SVN_COUNTER_NOT_AVAILABLE => 0x80290123 as u32,
            TPM_SOFTWARE::TPMAPI_E_OWNER_AUTH_NOT_NULL => 0x80290124 as u32,
            TPM_SOFTWARE::TPMAPI_E_ENDORSEMENT_AUTH_NOT_NULL => 0x80290125 as u32,
            TPM_SOFTWARE::TPMAPI_E_AUTHORIZATION_REVOKED => 0x80290126 as u32,
            TPM_SOFTWARE::TPMAPI_E_MALFORMED_AUTHORIZATION_KEY => 0x80290127 as u32,
            TPM_SOFTWARE::TPMAPI_E_AUTHORIZING_KEY_NOT_SUPPORTED => 0x80290128 as u32,
            TPM_SOFTWARE::TPMAPI_E_INVALID_AUTHORIZATION_SIGNATURE => 0x80290129 as u32,
            TPM_SOFTWARE::TPMAPI_E_MALFORMED_AUTHORIZATION_POLICY => 0x8029012A as u32,
            TPM_SOFTWARE::TPMAPI_E_MALFORMED_AUTHORIZATION_OTHER => 0x8029012B as u32,
            TPM_SOFTWARE::TPMAPI_E_SEALING_KEY_CHANGED => 0x8029012C as u32,
            TPM_SOFTWARE::TPMAPI_E_INVALID_TPM_VERSION => 0x8029012D as u32,
            TPM_SOFTWARE::TPMAPI_E_INVALID_POLICYAUTH_BLOB_TYPE => 0x8029012E as u32,
            TPM_SOFTWARE::TBSIMP_E_BUFFER_TOO_SMALL => 0x80290200 as u32,
            TPM_SOFTWARE::TBSIMP_E_CLEANUP_FAILED => 0x80290201 as u32,
            TPM_SOFTWARE::TBSIMP_E_INVALID_CONTEXT_HANDLE => 0x80290202 as u32,
            TPM_SOFTWARE::TBSIMP_E_INVALID_CONTEXT_PARAM => 0x80290203 as u32,
            TPM_SOFTWARE::TBSIMP_E_TPM_ERROR => 0x80290204 as u32,
            TPM_SOFTWARE::TBSIMP_E_HASH_BAD_KEY => 0x80290205 as u32,
            TPM_SOFTWARE::TBSIMP_E_DUPLICATE_VHANDLE => 0x80290206 as u32,
            TPM_SOFTWARE::TBSIMP_E_INVALID_OUTPUT_POINTER => 0x80290207 as u32,
            TPM_SOFTWARE::TBSIMP_E_INVALID_PARAMETER => 0x80290208 as u32,
            TPM_SOFTWARE::TBSIMP_E_RPC_INIT_FAILED => 0x80290209 as u32,
            TPM_SOFTWARE::TBSIMP_E_SCHEDULER_NOT_RUNNING => 0x8029020A as u32,
            TPM_SOFTWARE::TBSIMP_E_COMMAND_CANCELED => 0x8029020B as u32,
            TPM_SOFTWARE::TBSIMP_E_OUT_OF_MEMORY => 0x8029020C as u32,
            TPM_SOFTWARE::TBSIMP_E_LIST_NO_MORE_ITEMS => 0x8029020D as u32,
            TPM_SOFTWARE::TBSIMP_E_LIST_NOT_FOUND => 0x8029020E as u32,
            TPM_SOFTWARE::TBSIMP_E_NOT_ENOUGH_SPACE => 0x8029020F as u32,
            TPM_SOFTWARE::TBSIMP_E_NOT_ENOUGH_TPM_CONTEXTS => 0x80290210 as u32,
            TPM_SOFTWARE::TBSIMP_E_COMMAND_FAILED => 0x80290211 as u32,
            TPM_SOFTWARE::TBSIMP_E_UNKNOWN_ORDINAL => 0x80290212 as u32,
            TPM_SOFTWARE::TBSIMP_E_RESOURCE_EXPIRED => 0x80290213 as u32,
            TPM_SOFTWARE::TBSIMP_E_INVALID_RESOURCE => 0x80290214 as u32,
            TPM_SOFTWARE::TBSIMP_E_NOTHING_TO_UNLOAD => 0x80290215 as u32,
            TPM_SOFTWARE::TBSIMP_E_HASH_TABLE_FULL => 0x80290216 as u32,
            TPM_SOFTWARE::TBSIMP_E_TOO_MANY_TBS_CONTEXTS => 0x80290217 as u32,
            TPM_SOFTWARE::TBSIMP_E_TOO_MANY_RESOURCES => 0x80290218 as u32,
            TPM_SOFTWARE::TBSIMP_E_PPI_NOT_SUPPORTED => 0x80290219 as u32,
            TPM_SOFTWARE::TBSIMP_E_TPM_INCOMPATIBLE => 0x8029021A as u32,
            TPM_SOFTWARE::TBSIMP_E_NO_EVENT_LOG => 0x8029021B as u32,
            TPM_SOFTWARE::TPM_E_PPI_ACPI_FAILURE => 0x80290300 as u32,
            TPM_SOFTWARE::TPM_E_PPI_USER_ABORT => 0x80290301 as u32,
            TPM_SOFTWARE::TPM_E_PPI_BIOS_FAILURE => 0x80290302 as u32,
            TPM_SOFTWARE::TPM_E_PPI_NOT_SUPPORTED => 0x80290303 as u32,
            TPM_SOFTWARE::TPM_E_PPI_BLOCKED_IN_BIOS => 0x80290304 as u32,
            TPM_SOFTWARE::TPM_E_PCP_ERROR_MASK => 0x80290400 as u32,
            TPM_SOFTWARE::TPM_E_PCP_DEVICE_NOT_READY => 0x80290401 as u32,
            TPM_SOFTWARE::TPM_E_PCP_INVALID_HANDLE => 0x80290402 as u32,
            TPM_SOFTWARE::TPM_E_PCP_INVALID_PARAMETER => 0x80290403 as u32,
            TPM_SOFTWARE::TPM_E_PCP_FLAG_NOT_SUPPORTED => 0x80290404 as u32,
            TPM_SOFTWARE::TPM_E_PCP_NOT_SUPPORTED => 0x80290405 as u32,
            TPM_SOFTWARE::TPM_E_PCP_BUFFER_TOO_SMALL => 0x80290406 as u32,
            TPM_SOFTWARE::TPM_E_PCP_INTERNAL_ERROR => 0x80290407 as u32,
            TPM_SOFTWARE::TPM_E_PCP_AUTHENTICATION_FAILED => 0x80290408 as u32,
            TPM_SOFTWARE::TPM_E_PCP_AUTHENTICATION_IGNORED => 0x80290409 as u32,
            TPM_SOFTWARE::TPM_E_PCP_POLICY_NOT_FOUND => 0x8029040A as u32,
            TPM_SOFTWARE::TPM_E_PCP_PROFILE_NOT_FOUND => 0x8029040B as u32,
            TPM_SOFTWARE::TPM_E_PCP_VALIDATION_FAILED => 0x8029040C as u32,
            TPM_SOFTWARE::TPM_E_PCP_WRONG_PARENT => 0x8029040E as u32,
            TPM_SOFTWARE::TPM_E_KEY_NOT_LOADED => 0x8029040F as u32,
            TPM_SOFTWARE::TPM_E_NO_KEY_CERTIFICATION => 0x80290410 as u32,
            TPM_SOFTWARE::TPM_E_KEY_NOT_FINALIZED => 0x80290411 as u32,
            TPM_SOFTWARE::TPM_E_ATTESTATION_CHALLENGE_NOT_SET => 0x80290412 as u32,
            TPM_SOFTWARE::TPM_E_NOT_PCR_BOUND => 0x80290413 as u32,
            TPM_SOFTWARE::TPM_E_KEY_ALREADY_FINALIZED => 0x80290414 as u32,
            TPM_SOFTWARE::TPM_E_KEY_USAGE_POLICY_NOT_SUPPORTED => 0x80290415 as u32,
            TPM_SOFTWARE::TPM_E_KEY_USAGE_POLICY_INVALID => 0x80290416 as u32,
            TPM_SOFTWARE::TPM_E_SOFT_KEY_ERROR => 0x80290417 as u32,
            TPM_SOFTWARE::TPM_E_KEY_NOT_AUTHENTICATED => 0x80290418 as u32,
            TPM_SOFTWARE::TPM_E_PCP_KEY_NOT_AIK => 0x80290419 as u32,
            TPM_SOFTWARE::TPM_E_KEY_NOT_SIGNING_KEY => 0x8029041A as u32,
            TPM_SOFTWARE::TPM_E_LOCKED_OUT => 0x8029041B as u32,
            TPM_SOFTWARE::TPM_E_CLAIM_TYPE_NOT_SUPPORTED => 0x8029041C as u32,
            TPM_SOFTWARE::TPM_E_VERSION_NOT_SUPPORTED => 0x8029041D as u32,
            TPM_SOFTWARE::TPM_E_BUFFER_LENGTH_MISMATCH => 0x8029041E as u32,
            TPM_SOFTWARE::TPM_E_PCP_IFX_RSA_KEY_CREATION_BLOCKED => 0x8029041F as u32,
            TPM_SOFTWARE::TPM_E_PCP_TICKET_MISSING => 0x80290420 as u32,
            TPM_SOFTWARE::TPM_E_PCP_RAW_POLICY_NOT_SUPPORTED => 0x80290421 as u32,
            TPM_SOFTWARE::TPM_E_PCP_KEY_HANDLE_INVALIDATED => 0x80290422 as u32,
            TPM_SOFTWARE::TPM_E_ZERO_EXHAUST_ENABLED => 0x80290500 as u32,
            TPM_SOFTWARE::TPM_E_PROVISIONING_INCOMPLETE => 0x80290600 as u32,
            TPM_SOFTWARE::TPM_E_INVALID_OWNER_AUTH => 0x80290601 as u32,
            TPM_SOFTWARE::TPM_E_TOO_MUCH_DATA => 0x80290602 as u32,
            TPM_SOFTWARE::TPM_E_TPM_GENERATED_EPS => 0x80290603 as u32,
        }
    }

    pub fn error(&self) -> RawError {
        return match self {
            TPM_SOFTWARE::TPM_E_PCP_UNSUPPORTED_PSS_SALT => RawError::Kind(TPM_SOFTWARE::TPM_E_PCP_UNSUPPORTED_PSS_SALT),
            TPM_SOFTWARE::TPM_E_PCP_PLATFORM_CLAIM_MAY_BE_OUTDATED => RawError::Kind(TPM_SOFTWARE::TPM_E_PCP_PLATFORM_CLAIM_MAY_BE_OUTDATED),
            TPM_SOFTWARE::TPM_E_PCP_PLATFORM_CLAIM_OUTDATED => RawError::Kind(TPM_SOFTWARE::TPM_E_PCP_PLATFORM_CLAIM_OUTDATED),
            TPM_SOFTWARE::TPM_E_PCP_PLATFORM_CLAIM_REBOOT => RawError::Kind(TPM_SOFTWARE::TPM_E_PCP_PLATFORM_CLAIM_REBOOT),
            TPM_SOFTWARE::TPMAPI_E_INVALID_STATE => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_INVALID_STATE),
            TPM_SOFTWARE::TPMAPI_E_NOT_ENOUGH_DATA => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_NOT_ENOUGH_DATA),
            TPM_SOFTWARE::TPMAPI_E_TOO_MUCH_DATA => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_TOO_MUCH_DATA),
            TPM_SOFTWARE::TPMAPI_E_INVALID_OUTPUT_POINTER => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_INVALID_OUTPUT_POINTER),
            TPM_SOFTWARE::TPMAPI_E_INVALID_PARAMETER => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_INVALID_PARAMETER),
            TPM_SOFTWARE::TPMAPI_E_OUT_OF_MEMORY => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_OUT_OF_MEMORY),
            TPM_SOFTWARE::TPMAPI_E_BUFFER_TOO_SMALL => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_BUFFER_TOO_SMALL),
            TPM_SOFTWARE::TPMAPI_E_INTERNAL_ERROR => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_INTERNAL_ERROR),
            TPM_SOFTWARE::TPMAPI_E_ACCESS_DENIED => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_ACCESS_DENIED),
            TPM_SOFTWARE::TPMAPI_E_AUTHORIZATION_FAILED => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_AUTHORIZATION_FAILED),
            TPM_SOFTWARE::TPMAPI_E_INVALID_CONTEXT_HANDLE => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_INVALID_CONTEXT_HANDLE),
            TPM_SOFTWARE::TPMAPI_E_TBS_COMMUNICATION_ERROR => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_TBS_COMMUNICATION_ERROR),
            TPM_SOFTWARE::TPMAPI_E_TPM_COMMAND_ERROR => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_TPM_COMMAND_ERROR),
            TPM_SOFTWARE::TPMAPI_E_MESSAGE_TOO_LARGE => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_MESSAGE_TOO_LARGE),
            TPM_SOFTWARE::TPMAPI_E_INVALID_ENCODING => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_INVALID_ENCODING),
            TPM_SOFTWARE::TPMAPI_E_INVALID_KEY_SIZE => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_INVALID_KEY_SIZE),
            TPM_SOFTWARE::TPMAPI_E_ENCRYPTION_FAILED => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_ENCRYPTION_FAILED),
            TPM_SOFTWARE::TPMAPI_E_INVALID_KEY_PARAMS => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_INVALID_KEY_PARAMS),
            TPM_SOFTWARE::TPMAPI_E_INVALID_MIGRATION_AUTHORIZATION_BLOB => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_INVALID_MIGRATION_AUTHORIZATION_BLOB),
            TPM_SOFTWARE::TPMAPI_E_INVALID_PCR_INDEX => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_INVALID_PCR_INDEX),
            TPM_SOFTWARE::TPMAPI_E_INVALID_DELEGATE_BLOB => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_INVALID_DELEGATE_BLOB),
            TPM_SOFTWARE::TPMAPI_E_INVALID_CONTEXT_PARAMS => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_INVALID_CONTEXT_PARAMS),
            TPM_SOFTWARE::TPMAPI_E_INVALID_KEY_BLOB => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_INVALID_KEY_BLOB),
            TPM_SOFTWARE::TPMAPI_E_INVALID_PCR_DATA => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_INVALID_PCR_DATA),
            TPM_SOFTWARE::TPMAPI_E_INVALID_OWNER_AUTH => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_INVALID_OWNER_AUTH),
            TPM_SOFTWARE::TPMAPI_E_FIPS_RNG_CHECK_FAILED => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_FIPS_RNG_CHECK_FAILED),
            TPM_SOFTWARE::TPMAPI_E_EMPTY_TCG_LOG => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_EMPTY_TCG_LOG),
            TPM_SOFTWARE::TPMAPI_E_INVALID_TCG_LOG_ENTRY => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_INVALID_TCG_LOG_ENTRY),
            TPM_SOFTWARE::TPMAPI_E_TCG_SEPARATOR_ABSENT => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_TCG_SEPARATOR_ABSENT),
            TPM_SOFTWARE::TPMAPI_E_TCG_INVALID_DIGEST_ENTRY => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_TCG_INVALID_DIGEST_ENTRY),
            TPM_SOFTWARE::TPMAPI_E_POLICY_DENIES_OPERATION => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_POLICY_DENIES_OPERATION),
            TPM_SOFTWARE::TPMAPI_E_NV_BITS_NOT_DEFINED => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_NV_BITS_NOT_DEFINED),
            TPM_SOFTWARE::TPMAPI_E_NV_BITS_NOT_READY => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_NV_BITS_NOT_READY),
            TPM_SOFTWARE::TPMAPI_E_SEALING_KEY_NOT_AVAILABLE => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_SEALING_KEY_NOT_AVAILABLE),
            TPM_SOFTWARE::TPMAPI_E_NO_AUTHORIZATION_CHAIN_FOUND => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_NO_AUTHORIZATION_CHAIN_FOUND),
            TPM_SOFTWARE::TPMAPI_E_SVN_COUNTER_NOT_AVAILABLE => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_SVN_COUNTER_NOT_AVAILABLE),
            TPM_SOFTWARE::TPMAPI_E_OWNER_AUTH_NOT_NULL => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_OWNER_AUTH_NOT_NULL),
            TPM_SOFTWARE::TPMAPI_E_ENDORSEMENT_AUTH_NOT_NULL => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_ENDORSEMENT_AUTH_NOT_NULL),
            TPM_SOFTWARE::TPMAPI_E_AUTHORIZATION_REVOKED => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_AUTHORIZATION_REVOKED),
            TPM_SOFTWARE::TPMAPI_E_MALFORMED_AUTHORIZATION_KEY => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_MALFORMED_AUTHORIZATION_KEY),
            TPM_SOFTWARE::TPMAPI_E_AUTHORIZING_KEY_NOT_SUPPORTED => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_AUTHORIZING_KEY_NOT_SUPPORTED),
            TPM_SOFTWARE::TPMAPI_E_INVALID_AUTHORIZATION_SIGNATURE => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_INVALID_AUTHORIZATION_SIGNATURE),
            TPM_SOFTWARE::TPMAPI_E_MALFORMED_AUTHORIZATION_POLICY => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_MALFORMED_AUTHORIZATION_POLICY),
            TPM_SOFTWARE::TPMAPI_E_MALFORMED_AUTHORIZATION_OTHER => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_MALFORMED_AUTHORIZATION_OTHER),
            TPM_SOFTWARE::TPMAPI_E_SEALING_KEY_CHANGED => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_SEALING_KEY_CHANGED),
            TPM_SOFTWARE::TPMAPI_E_INVALID_TPM_VERSION => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_INVALID_TPM_VERSION),
            TPM_SOFTWARE::TPMAPI_E_INVALID_POLICYAUTH_BLOB_TYPE => RawError::Kind(TPM_SOFTWARE::TPMAPI_E_INVALID_POLICYAUTH_BLOB_TYPE),
            TPM_SOFTWARE::TBSIMP_E_BUFFER_TOO_SMALL => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_BUFFER_TOO_SMALL),
            TPM_SOFTWARE::TBSIMP_E_CLEANUP_FAILED => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_CLEANUP_FAILED),
            TPM_SOFTWARE::TBSIMP_E_INVALID_CONTEXT_HANDLE => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_INVALID_CONTEXT_HANDLE),
            TPM_SOFTWARE::TBSIMP_E_INVALID_CONTEXT_PARAM => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_INVALID_CONTEXT_PARAM),
            TPM_SOFTWARE::TBSIMP_E_TPM_ERROR => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_TPM_ERROR),
            TPM_SOFTWARE::TBSIMP_E_HASH_BAD_KEY => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_HASH_BAD_KEY),
            TPM_SOFTWARE::TBSIMP_E_DUPLICATE_VHANDLE => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_DUPLICATE_VHANDLE),
            TPM_SOFTWARE::TBSIMP_E_INVALID_OUTPUT_POINTER => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_INVALID_OUTPUT_POINTER),
            TPM_SOFTWARE::TBSIMP_E_INVALID_PARAMETER => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_INVALID_PARAMETER),
            TPM_SOFTWARE::TBSIMP_E_RPC_INIT_FAILED => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_RPC_INIT_FAILED),
            TPM_SOFTWARE::TBSIMP_E_SCHEDULER_NOT_RUNNING => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_SCHEDULER_NOT_RUNNING),
            TPM_SOFTWARE::TBSIMP_E_COMMAND_CANCELED => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_COMMAND_CANCELED),
            TPM_SOFTWARE::TBSIMP_E_OUT_OF_MEMORY => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_OUT_OF_MEMORY),
            TPM_SOFTWARE::TBSIMP_E_LIST_NO_MORE_ITEMS => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_LIST_NO_MORE_ITEMS),
            TPM_SOFTWARE::TBSIMP_E_LIST_NOT_FOUND => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_LIST_NOT_FOUND),
            TPM_SOFTWARE::TBSIMP_E_NOT_ENOUGH_SPACE => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_NOT_ENOUGH_SPACE),
            TPM_SOFTWARE::TBSIMP_E_NOT_ENOUGH_TPM_CONTEXTS => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_NOT_ENOUGH_TPM_CONTEXTS),
            TPM_SOFTWARE::TBSIMP_E_COMMAND_FAILED => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_COMMAND_FAILED),
            TPM_SOFTWARE::TBSIMP_E_UNKNOWN_ORDINAL => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_UNKNOWN_ORDINAL),
            TPM_SOFTWARE::TBSIMP_E_RESOURCE_EXPIRED => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_RESOURCE_EXPIRED),
            TPM_SOFTWARE::TBSIMP_E_INVALID_RESOURCE => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_INVALID_RESOURCE),
            TPM_SOFTWARE::TBSIMP_E_NOTHING_TO_UNLOAD => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_NOTHING_TO_UNLOAD),
            TPM_SOFTWARE::TBSIMP_E_HASH_TABLE_FULL => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_HASH_TABLE_FULL),
            TPM_SOFTWARE::TBSIMP_E_TOO_MANY_TBS_CONTEXTS => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_TOO_MANY_TBS_CONTEXTS),
            TPM_SOFTWARE::TBSIMP_E_TOO_MANY_RESOURCES => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_TOO_MANY_RESOURCES),
            TPM_SOFTWARE::TBSIMP_E_PPI_NOT_SUPPORTED => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_PPI_NOT_SUPPORTED),
            TPM_SOFTWARE::TBSIMP_E_TPM_INCOMPATIBLE => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_TPM_INCOMPATIBLE),
            TPM_SOFTWARE::TBSIMP_E_NO_EVENT_LOG => RawError::Kind(TPM_SOFTWARE::TBSIMP_E_NO_EVENT_LOG),
            TPM_SOFTWARE::TPM_E_PPI_ACPI_FAILURE => RawError::Kind(TPM_SOFTWARE::TPM_E_PPI_ACPI_FAILURE),
            TPM_SOFTWARE::TPM_E_PPI_USER_ABORT => RawError::Kind(TPM_SOFTWARE::TPM_E_PPI_USER_ABORT),
            TPM_SOFTWARE::TPM_E_PPI_BIOS_FAILURE => RawError::Kind(TPM_SOFTWARE::TPM_E_PPI_BIOS_FAILURE),
            TPM_SOFTWARE::TPM_E_PPI_NOT_SUPPORTED => RawError::Kind(TPM_SOFTWARE::TPM_E_PPI_NOT_SUPPORTED),
            TPM_SOFTWARE::TPM_E_PPI_BLOCKED_IN_BIOS => RawError::Kind(TPM_SOFTWARE::TPM_E_PPI_BLOCKED_IN_BIOS),
            TPM_SOFTWARE::TPM_E_PCP_ERROR_MASK => RawError::Kind(TPM_SOFTWARE::TPM_E_PCP_ERROR_MASK),
            TPM_SOFTWARE::TPM_E_PCP_DEVICE_NOT_READY => RawError::Kind(TPM_SOFTWARE::TPM_E_PCP_DEVICE_NOT_READY),
            TPM_SOFTWARE::TPM_E_PCP_INVALID_HANDLE => RawError::Kind(TPM_SOFTWARE::TPM_E_PCP_INVALID_HANDLE),
            TPM_SOFTWARE::TPM_E_PCP_INVALID_PARAMETER => RawError::Kind(TPM_SOFTWARE::TPM_E_PCP_INVALID_PARAMETER),
            TPM_SOFTWARE::TPM_E_PCP_FLAG_NOT_SUPPORTED => RawError::Kind(TPM_SOFTWARE::TPM_E_PCP_FLAG_NOT_SUPPORTED),
            TPM_SOFTWARE::TPM_E_PCP_NOT_SUPPORTED => RawError::Kind(TPM_SOFTWARE::TPM_E_PCP_NOT_SUPPORTED),
            TPM_SOFTWARE::TPM_E_PCP_BUFFER_TOO_SMALL => RawError::Kind(TPM_SOFTWARE::TPM_E_PCP_BUFFER_TOO_SMALL),
            TPM_SOFTWARE::TPM_E_PCP_INTERNAL_ERROR => RawError::Kind(TPM_SOFTWARE::TPM_E_PCP_INTERNAL_ERROR),
            TPM_SOFTWARE::TPM_E_PCP_AUTHENTICATION_FAILED => RawError::Kind(TPM_SOFTWARE::TPM_E_PCP_AUTHENTICATION_FAILED),
            TPM_SOFTWARE::TPM_E_PCP_AUTHENTICATION_IGNORED => RawError::Kind(TPM_SOFTWARE::TPM_E_PCP_AUTHENTICATION_IGNORED),
            TPM_SOFTWARE::TPM_E_PCP_POLICY_NOT_FOUND => RawError::Kind(TPM_SOFTWARE::TPM_E_PCP_POLICY_NOT_FOUND),
            TPM_SOFTWARE::TPM_E_PCP_PROFILE_NOT_FOUND => RawError::Kind(TPM_SOFTWARE::TPM_E_PCP_PROFILE_NOT_FOUND),
            TPM_SOFTWARE::TPM_E_PCP_VALIDATION_FAILED => RawError::Kind(TPM_SOFTWARE::TPM_E_PCP_VALIDATION_FAILED),
            TPM_SOFTWARE::TPM_E_PCP_WRONG_PARENT => RawError::Kind(TPM_SOFTWARE::TPM_E_PCP_WRONG_PARENT),
            TPM_SOFTWARE::TPM_E_KEY_NOT_LOADED => RawError::Kind(TPM_SOFTWARE::TPM_E_KEY_NOT_LOADED),
            TPM_SOFTWARE::TPM_E_NO_KEY_CERTIFICATION => RawError::Kind(TPM_SOFTWARE::TPM_E_NO_KEY_CERTIFICATION),
            TPM_SOFTWARE::TPM_E_KEY_NOT_FINALIZED => RawError::Kind(TPM_SOFTWARE::TPM_E_KEY_NOT_FINALIZED),
            TPM_SOFTWARE::TPM_E_ATTESTATION_CHALLENGE_NOT_SET => RawError::Kind(TPM_SOFTWARE::TPM_E_ATTESTATION_CHALLENGE_NOT_SET),
            TPM_SOFTWARE::TPM_E_NOT_PCR_BOUND => RawError::Kind(TPM_SOFTWARE::TPM_E_NOT_PCR_BOUND),
            TPM_SOFTWARE::TPM_E_KEY_ALREADY_FINALIZED => RawError::Kind(TPM_SOFTWARE::TPM_E_KEY_ALREADY_FINALIZED),
            TPM_SOFTWARE::TPM_E_KEY_USAGE_POLICY_NOT_SUPPORTED => RawError::Kind(TPM_SOFTWARE::TPM_E_KEY_USAGE_POLICY_NOT_SUPPORTED),
            TPM_SOFTWARE::TPM_E_KEY_USAGE_POLICY_INVALID => RawError::Kind(TPM_SOFTWARE::TPM_E_KEY_USAGE_POLICY_INVALID),
            TPM_SOFTWARE::TPM_E_SOFT_KEY_ERROR => RawError::Kind(TPM_SOFTWARE::TPM_E_SOFT_KEY_ERROR),
            TPM_SOFTWARE::TPM_E_KEY_NOT_AUTHENTICATED => RawError::Kind(TPM_SOFTWARE::TPM_E_KEY_NOT_AUTHENTICATED),
            TPM_SOFTWARE::TPM_E_PCP_KEY_NOT_AIK => RawError::Kind(TPM_SOFTWARE::TPM_E_PCP_KEY_NOT_AIK),
            TPM_SOFTWARE::TPM_E_KEY_NOT_SIGNING_KEY => RawError::Kind(TPM_SOFTWARE::TPM_E_KEY_NOT_SIGNING_KEY),
            TPM_SOFTWARE::TPM_E_LOCKED_OUT => RawError::Kind(TPM_SOFTWARE::TPM_E_LOCKED_OUT),
            TPM_SOFTWARE::TPM_E_CLAIM_TYPE_NOT_SUPPORTED => RawError::Kind(TPM_SOFTWARE::TPM_E_CLAIM_TYPE_NOT_SUPPORTED),
            TPM_SOFTWARE::TPM_E_VERSION_NOT_SUPPORTED => RawError::Kind(TPM_SOFTWARE::TPM_E_VERSION_NOT_SUPPORTED),
            TPM_SOFTWARE::TPM_E_BUFFER_LENGTH_MISMATCH => RawError::Kind(TPM_SOFTWARE::TPM_E_BUFFER_LENGTH_MISMATCH),
            TPM_SOFTWARE::TPM_E_PCP_IFX_RSA_KEY_CREATION_BLOCKED => RawError::Kind(TPM_SOFTWARE::TPM_E_PCP_IFX_RSA_KEY_CREATION_BLOCKED),
            TPM_SOFTWARE::TPM_E_PCP_TICKET_MISSING => RawError::Kind(TPM_SOFTWARE::TPM_E_PCP_TICKET_MISSING),
            TPM_SOFTWARE::TPM_E_PCP_RAW_POLICY_NOT_SUPPORTED => RawError::Kind(TPM_SOFTWARE::TPM_E_PCP_RAW_POLICY_NOT_SUPPORTED),
            TPM_SOFTWARE::TPM_E_PCP_KEY_HANDLE_INVALIDATED => RawError::Kind(TPM_SOFTWARE::TPM_E_PCP_KEY_HANDLE_INVALIDATED),
            TPM_SOFTWARE::TPM_E_ZERO_EXHAUST_ENABLED => RawError::Kind(TPM_SOFTWARE::TPM_E_ZERO_EXHAUST_ENABLED),
            TPM_SOFTWARE::TPM_E_PROVISIONING_INCOMPLETE => RawError::Kind(TPM_SOFTWARE::TPM_E_PROVISIONING_INCOMPLETE),
            TPM_SOFTWARE::TPM_E_INVALID_OWNER_AUTH => RawError::Kind(TPM_SOFTWARE::TPM_E_INVALID_OWNER_AUTH),
            TPM_SOFTWARE::TPM_E_TOO_MUCH_DATA => RawError::Kind(TPM_SOFTWARE::TPM_E_TOO_MUCH_DATA),
            TPM_SOFTWARE::TPM_E_TPM_GENERATED_EPS => RawError::Kind(TPM_SOFTWARE::TPM_E_TPM_GENERATED_EPS),
        }
    }

    pub fn description(&self) -> &'static str {
        return match self {
            TPM_SOFTWARE::TPM_E_PCP_UNSUPPORTED_PSS_SALT => "The requested salt size for signing with RSAPSS does not match what the TPM uses.",
            TPM_SOFTWARE::TPM_E_PCP_PLATFORM_CLAIM_MAY_BE_OUTDATED => "Validation of the platform claim failed.",
            TPM_SOFTWARE::TPM_E_PCP_PLATFORM_CLAIM_OUTDATED => "The requested platform claim is for a previous boot.",
            TPM_SOFTWARE::TPM_E_PCP_PLATFORM_CLAIM_REBOOT => "The platform claim is for a previous boot, and cannot be created without reboot.",
            TPM_SOFTWARE::TPMAPI_E_INVALID_STATE => "The command buffer is not in the correct state.",
            TPM_SOFTWARE::TPMAPI_E_NOT_ENOUGH_DATA => "The command buffer does not contain enough data to satisfy the request.",
            TPM_SOFTWARE::TPMAPI_E_TOO_MUCH_DATA => "The command buffer cannot contain any more data.",
            TPM_SOFTWARE::TPMAPI_E_INVALID_OUTPUT_POINTER => "One or more output parameters was NULL or invalid.",
            TPM_SOFTWARE::TPMAPI_E_INVALID_PARAMETER => "One or more input parameters is invalid.",
            TPM_SOFTWARE::TPMAPI_E_OUT_OF_MEMORY => "Not enough memory was available to satisfy the request.",
            TPM_SOFTWARE::TPMAPI_E_BUFFER_TOO_SMALL => "The specified buffer was too small.",
            TPM_SOFTWARE::TPMAPI_E_INTERNAL_ERROR => "An internal error was detected.",
            TPM_SOFTWARE::TPMAPI_E_ACCESS_DENIED => "The caller does not have the appropriate rights to perform the requested operation.",
            TPM_SOFTWARE::TPMAPI_E_AUTHORIZATION_FAILED => "The specified authorization information was invalid.",
            TPM_SOFTWARE::TPMAPI_E_INVALID_CONTEXT_HANDLE => "The specified context handle was not valid.",
            TPM_SOFTWARE::TPMAPI_E_TBS_COMMUNICATION_ERROR => "An error occurred while communicating with the TBS.",
            TPM_SOFTWARE::TPMAPI_E_TPM_COMMAND_ERROR => "The TPM returned an unexpected result.",
            TPM_SOFTWARE::TPMAPI_E_MESSAGE_TOO_LARGE => "The message was too large for the encoding scheme.",
            TPM_SOFTWARE::TPMAPI_E_INVALID_ENCODING => "The encoding in the blob was not recognized.",
            TPM_SOFTWARE::TPMAPI_E_INVALID_KEY_SIZE => "The key size is not valid.",
            TPM_SOFTWARE::TPMAPI_E_ENCRYPTION_FAILED => "The encryption operation failed.",
            TPM_SOFTWARE::TPMAPI_E_INVALID_KEY_PARAMS => "The key parameters structure was not valid",
            TPM_SOFTWARE::TPMAPI_E_INVALID_MIGRATION_AUTHORIZATION_BLOB => "The requested supplied data does not appear to be a valid migration authorization blob.",
            TPM_SOFTWARE::TPMAPI_E_INVALID_PCR_INDEX => "The specified PCR index was invalid",
            TPM_SOFTWARE::TPMAPI_E_INVALID_DELEGATE_BLOB => "The data given does not appear to be a valid delegate blob.",
            TPM_SOFTWARE::TPMAPI_E_INVALID_CONTEXT_PARAMS => "One or more of the specified context parameters was not valid.",
            TPM_SOFTWARE::TPMAPI_E_INVALID_KEY_BLOB => "The data given does not appear to be a valid key blob",
            TPM_SOFTWARE::TPMAPI_E_INVALID_PCR_DATA => "The specified PCR data was invalid.",
            TPM_SOFTWARE::TPMAPI_E_INVALID_OWNER_AUTH => "The format of the owner auth data was invalid.",
            TPM_SOFTWARE::TPMAPI_E_FIPS_RNG_CHECK_FAILED => "The random number generated did not pass FIPS RNG check.",
            TPM_SOFTWARE::TPMAPI_E_EMPTY_TCG_LOG => "The TCG Event Log does not contain any data.",
            TPM_SOFTWARE::TPMAPI_E_INVALID_TCG_LOG_ENTRY => "An entry in the TCG Event Log was invalid.",
            TPM_SOFTWARE::TPMAPI_E_TCG_SEPARATOR_ABSENT => "A TCG Separator was not found.",
            TPM_SOFTWARE::TPMAPI_E_TCG_INVALID_DIGEST_ENTRY => "A digest value in a TCG Log entry did not match hashed data.",
            TPM_SOFTWARE::TPMAPI_E_POLICY_DENIES_OPERATION => "The requested operation was blocked by current TPM policy. Please contact your system administrator for assistance.",
            TPM_SOFTWARE::TPMAPI_E_NV_BITS_NOT_DEFINED => "The Windows TPM NV Bits index is not defined.",
            TPM_SOFTWARE::TPMAPI_E_NV_BITS_NOT_READY => "The Windows TPM NV Bits index is not ready for use.",
            TPM_SOFTWARE::TPMAPI_E_SEALING_KEY_NOT_AVAILABLE => "The TPM key that was used to seal the data is no longer available.",
            TPM_SOFTWARE::TPMAPI_E_NO_AUTHORIZATION_CHAIN_FOUND => "An authorization chain could not be found that authorizes the PolicyAuthorize unseal.",
            TPM_SOFTWARE::TPMAPI_E_SVN_COUNTER_NOT_AVAILABLE => "The SVN counter to which the authorization was bound is not available.",
            TPM_SOFTWARE::TPMAPI_E_OWNER_AUTH_NOT_NULL => "The TPM Storage hierarchy (Owner) auth value is required to be NULL for this operation.",
            TPM_SOFTWARE::TPMAPI_E_ENDORSEMENT_AUTH_NOT_NULL => "The TPM Endorsement hierarchy auth value is required to be NULL for this operation.",
            TPM_SOFTWARE::TPMAPI_E_AUTHORIZATION_REVOKED => "The authorization to perform this operation has been revoked.",
            TPM_SOFTWARE::TPMAPI_E_MALFORMED_AUTHORIZATION_KEY => "The authorization public key is malformed.",
            TPM_SOFTWARE::TPMAPI_E_AUTHORIZING_KEY_NOT_SUPPORTED => "The authorization public key is not supported.",
            TPM_SOFTWARE::TPMAPI_E_INVALID_AUTHORIZATION_SIGNATURE => "The authorization signature is invalid.",
            TPM_SOFTWARE::TPMAPI_E_MALFORMED_AUTHORIZATION_POLICY => "The authorization policy is malformed.",
            TPM_SOFTWARE::TPMAPI_E_MALFORMED_AUTHORIZATION_OTHER => "The authorization data is malformed.",
            TPM_SOFTWARE::TPMAPI_E_SEALING_KEY_CHANGED => "The key used to unseal this data has changed since sealing the data. This may be the result of a TPM clear.",
            TPM_SOFTWARE::TPMAPI_E_INVALID_TPM_VERSION => "The TPM version is invalid.",
            TPM_SOFTWARE::TPMAPI_E_INVALID_POLICYAUTH_BLOB_TYPE => "The policy authorization blob type is invalid.",
            TPM_SOFTWARE::TBSIMP_E_BUFFER_TOO_SMALL => "The specified buffer was too small.",
            TPM_SOFTWARE::TBSIMP_E_CLEANUP_FAILED => "The context could not be cleaned up.",
            TPM_SOFTWARE::TBSIMP_E_INVALID_CONTEXT_HANDLE => "The specified context handle is invalid.",
            TPM_SOFTWARE::TBSIMP_E_INVALID_CONTEXT_PARAM => "An invalid context parameter was specified.",
            TPM_SOFTWARE::TBSIMP_E_TPM_ERROR => "An error occurred while communicating with the TPM",
            TPM_SOFTWARE::TBSIMP_E_HASH_BAD_KEY => "No entry with the specified key was found.",
            TPM_SOFTWARE::TBSIMP_E_DUPLICATE_VHANDLE => "The specified virtual handle matches a virtual handle already in use.",
            TPM_SOFTWARE::TBSIMP_E_INVALID_OUTPUT_POINTER => "The pointer to the returned handle location was NULL or invalid",
            TPM_SOFTWARE::TBSIMP_E_INVALID_PARAMETER => "One or more parameters is invalid",
            TPM_SOFTWARE::TBSIMP_E_RPC_INIT_FAILED => "The RPC subsystem could not be initialized.",
            TPM_SOFTWARE::TBSIMP_E_SCHEDULER_NOT_RUNNING => "The TBS scheduler is not running.",
            TPM_SOFTWARE::TBSIMP_E_COMMAND_CANCELED => "The command was canceled.",
            TPM_SOFTWARE::TBSIMP_E_OUT_OF_MEMORY => "There was not enough memory to fulfill the request",
            TPM_SOFTWARE::TBSIMP_E_LIST_NO_MORE_ITEMS => "The specified list is empty, or the iteration has reached the end of the list.",
            TPM_SOFTWARE::TBSIMP_E_LIST_NOT_FOUND => "The specified item was not found in the list.",
            TPM_SOFTWARE::TBSIMP_E_NOT_ENOUGH_SPACE => "The TPM does not have enough space to load the requested resource.",
            TPM_SOFTWARE::TBSIMP_E_NOT_ENOUGH_TPM_CONTEXTS => "There are too many TPM contexts in use.",
            TPM_SOFTWARE::TBSIMP_E_COMMAND_FAILED => "The TPM command failed.",
            TPM_SOFTWARE::TBSIMP_E_UNKNOWN_ORDINAL => "The TBS does not recognize the specified ordinal.",
            TPM_SOFTWARE::TBSIMP_E_RESOURCE_EXPIRED => "The requested resource is no longer available.",
            TPM_SOFTWARE::TBSIMP_E_INVALID_RESOURCE => "The resource type did not match.",
            TPM_SOFTWARE::TBSIMP_E_NOTHING_TO_UNLOAD => "No resources can be unloaded.",
            TPM_SOFTWARE::TBSIMP_E_HASH_TABLE_FULL => "No new entries can be added to the hash table.",
            TPM_SOFTWARE::TBSIMP_E_TOO_MANY_TBS_CONTEXTS => "A new TBS context could not be created because there are too many open contexts.",
            TPM_SOFTWARE::TBSIMP_E_TOO_MANY_RESOURCES => "A new virtual resource could not be created because there are too many open virtual resources.",
            TPM_SOFTWARE::TBSIMP_E_PPI_NOT_SUPPORTED => "The physical presence interface is not supported.",
            TPM_SOFTWARE::TBSIMP_E_TPM_INCOMPATIBLE => "TBS is not compatible with the version of TPM found on the system.",
            TPM_SOFTWARE::TBSIMP_E_NO_EVENT_LOG => "No TCG event log is available.",
            TPM_SOFTWARE::TPM_E_PPI_ACPI_FAILURE => "A general error was detected when attempting to acquire the BIOS's response to a Physical Presence command.",
            TPM_SOFTWARE::TPM_E_PPI_USER_ABORT => "The user failed to confirm the TPM operation request.",
            TPM_SOFTWARE::TPM_E_PPI_BIOS_FAILURE => "The BIOS failure prevented the successful execution of the requested TPM operation (e.g. invalid TPM operation request, BIOS communication error with the TPM).",
            TPM_SOFTWARE::TPM_E_PPI_NOT_SUPPORTED => "The BIOS does not support the physical presence interface.",
            TPM_SOFTWARE::TPM_E_PPI_BLOCKED_IN_BIOS => "The Physical Presence command was blocked by current BIOS settings. The system owner may be able to reconfigure the BIOS settings to allow the command.",
            TPM_SOFTWARE::TPM_E_PCP_ERROR_MASK => "This is an error mask to convert Platform Crypto Provider errors to win errors.",
            TPM_SOFTWARE::TPM_E_PCP_DEVICE_NOT_READY => "The Platform Crypto Device is currently not ready. It needs to be fully provisioned to be operational.",
            TPM_SOFTWARE::TPM_E_PCP_INVALID_HANDLE => "The handle provided to the Platform Crypto Provider is invalid.",
            TPM_SOFTWARE::TPM_E_PCP_INVALID_PARAMETER => "A parameter provided to the Platform Crypto Provider is invalid.",
            TPM_SOFTWARE::TPM_E_PCP_FLAG_NOT_SUPPORTED => "A provided flag to the Platform Crypto Provider is not supported.",
            TPM_SOFTWARE::TPM_E_PCP_NOT_SUPPORTED => "The requested operation is not supported by this Platform Crypto Provider.",
            TPM_SOFTWARE::TPM_E_PCP_BUFFER_TOO_SMALL => "The buffer is too small to contain all data. No information has been written to the buffer.",
            TPM_SOFTWARE::TPM_E_PCP_INTERNAL_ERROR => "An unexpected internal error has occurred in the Platform Crypto Provider.",
            TPM_SOFTWARE::TPM_E_PCP_AUTHENTICATION_FAILED => "The authorization to use a provider object has failed.",
            TPM_SOFTWARE::TPM_E_PCP_AUTHENTICATION_IGNORED => "The Platform Crypto Device has ignored the authorization for the provider object, to mitigate against a dictionary attack.",
            TPM_SOFTWARE::TPM_E_PCP_POLICY_NOT_FOUND => "The referenced policy was not found.",
            TPM_SOFTWARE::TPM_E_PCP_PROFILE_NOT_FOUND => "The referenced profile was not found.",
            TPM_SOFTWARE::TPM_E_PCP_VALIDATION_FAILED => "The validation was not succesful.",
            TPM_SOFTWARE::TPM_E_PCP_WRONG_PARENT => "An attempt was made to import or load a key under an incorrect storage parent.",
            TPM_SOFTWARE::TPM_E_KEY_NOT_LOADED => "The TPM key is not loaded.",
            TPM_SOFTWARE::TPM_E_NO_KEY_CERTIFICATION => "The TPM key certification has not been generated.",
            TPM_SOFTWARE::TPM_E_KEY_NOT_FINALIZED => "The TPM key is not yet finalized.",
            TPM_SOFTWARE::TPM_E_ATTESTATION_CHALLENGE_NOT_SET => "The TPM attestation challenge is not set.",
            TPM_SOFTWARE::TPM_E_NOT_PCR_BOUND => "The TPM PCR info is not available.",
            TPM_SOFTWARE::TPM_E_KEY_ALREADY_FINALIZED => "The TPM key is already finalized.",
            TPM_SOFTWARE::TPM_E_KEY_USAGE_POLICY_NOT_SUPPORTED => "The TPM key usage policy is not supported.",
            TPM_SOFTWARE::TPM_E_KEY_USAGE_POLICY_INVALID => "The TPM key usage policy is invalid.",
            TPM_SOFTWARE::TPM_E_SOFT_KEY_ERROR => "There was a problem with the software key being imported into the TPM.",
            TPM_SOFTWARE::TPM_E_KEY_NOT_AUTHENTICATED => "The TPM key is not authenticated.",
            TPM_SOFTWARE::TPM_E_PCP_KEY_NOT_AIK => "The TPM key is not an AIK.",
            TPM_SOFTWARE::TPM_E_KEY_NOT_SIGNING_KEY => "The TPM key is not a signing key.",
            TPM_SOFTWARE::TPM_E_LOCKED_OUT => "The TPM is locked out.",
            TPM_SOFTWARE::TPM_E_CLAIM_TYPE_NOT_SUPPORTED => "The claim type requested is not supported.",
            TPM_SOFTWARE::TPM_E_VERSION_NOT_SUPPORTED => "TPM version is not supported.",
            TPM_SOFTWARE::TPM_E_BUFFER_LENGTH_MISMATCH => "The buffer lengths do not match.",
            TPM_SOFTWARE::TPM_E_PCP_IFX_RSA_KEY_CREATION_BLOCKED => "The RSA key creation is blocked on this TPM due to known security vulnerabilities.",
            TPM_SOFTWARE::TPM_E_PCP_TICKET_MISSING => "A ticket required to use a key was not provided.",
            TPM_SOFTWARE::TPM_E_PCP_RAW_POLICY_NOT_SUPPORTED => "This key has a raw policy so the KSP can't authenticate against it.",
            TPM_SOFTWARE::TPM_E_PCP_KEY_HANDLE_INVALIDATED => "The TPM key's handle was unexpectedly invalidated due to a hardware or firmware issue.",
            TPM_SOFTWARE::TPM_E_ZERO_EXHAUST_ENABLED => "TPM related network operations are blocked as Zero Exhaust mode is enabled on client.",
            TPM_SOFTWARE::TPM_E_PROVISIONING_INCOMPLETE => "TPM provisioning did not run to completion.",
            TPM_SOFTWARE::TPM_E_INVALID_OWNER_AUTH => "An invalid owner authorization value was specified.",
            TPM_SOFTWARE::TPM_E_TOO_MUCH_DATA => "TPM command returned too much data.",
            TPM_SOFTWARE::TPM_E_TPM_GENERATED_EPS => "Unable to fetch EK Certificate if TPM-generated EPS (after a TPM2_ChangeEPS).",
        }
    }

    pub fn from_name(name: &str) -> TPM_SOFTWARE {
        return match name {
            "TPM_E_PCP_UNSUPPORTED_PSS_SALT" => TPM_SOFTWARE::TPM_E_PCP_UNSUPPORTED_PSS_SALT,
            "TPM_E_PCP_PLATFORM_CLAIM_MAY_BE_OUTDATED" => TPM_SOFTWARE::TPM_E_PCP_PLATFORM_CLAIM_MAY_BE_OUTDATED,
            "TPM_E_PCP_PLATFORM_CLAIM_OUTDATED" => TPM_SOFTWARE::TPM_E_PCP_PLATFORM_CLAIM_OUTDATED,
            "TPM_E_PCP_PLATFORM_CLAIM_REBOOT" => TPM_SOFTWARE::TPM_E_PCP_PLATFORM_CLAIM_REBOOT,
            "TPMAPI_E_INVALID_STATE" => TPM_SOFTWARE::TPMAPI_E_INVALID_STATE,
            "TPMAPI_E_NOT_ENOUGH_DATA" => TPM_SOFTWARE::TPMAPI_E_NOT_ENOUGH_DATA,
            "TPMAPI_E_TOO_MUCH_DATA" => TPM_SOFTWARE::TPMAPI_E_TOO_MUCH_DATA,
            "TPMAPI_E_INVALID_OUTPUT_POINTER" => TPM_SOFTWARE::TPMAPI_E_INVALID_OUTPUT_POINTER,
            "TPMAPI_E_INVALID_PARAMETER" => TPM_SOFTWARE::TPMAPI_E_INVALID_PARAMETER,
            "TPMAPI_E_OUT_OF_MEMORY" => TPM_SOFTWARE::TPMAPI_E_OUT_OF_MEMORY,
            "TPMAPI_E_BUFFER_TOO_SMALL" => TPM_SOFTWARE::TPMAPI_E_BUFFER_TOO_SMALL,
            "TPMAPI_E_INTERNAL_ERROR" => TPM_SOFTWARE::TPMAPI_E_INTERNAL_ERROR,
            "TPMAPI_E_ACCESS_DENIED" => TPM_SOFTWARE::TPMAPI_E_ACCESS_DENIED,
            "TPMAPI_E_AUTHORIZATION_FAILED" => TPM_SOFTWARE::TPMAPI_E_AUTHORIZATION_FAILED,
            "TPMAPI_E_INVALID_CONTEXT_HANDLE" => TPM_SOFTWARE::TPMAPI_E_INVALID_CONTEXT_HANDLE,
            "TPMAPI_E_TBS_COMMUNICATION_ERROR" => TPM_SOFTWARE::TPMAPI_E_TBS_COMMUNICATION_ERROR,
            "TPMAPI_E_TPM_COMMAND_ERROR" => TPM_SOFTWARE::TPMAPI_E_TPM_COMMAND_ERROR,
            "TPMAPI_E_MESSAGE_TOO_LARGE" => TPM_SOFTWARE::TPMAPI_E_MESSAGE_TOO_LARGE,
            "TPMAPI_E_INVALID_ENCODING" => TPM_SOFTWARE::TPMAPI_E_INVALID_ENCODING,
            "TPMAPI_E_INVALID_KEY_SIZE" => TPM_SOFTWARE::TPMAPI_E_INVALID_KEY_SIZE,
            "TPMAPI_E_ENCRYPTION_FAILED" => TPM_SOFTWARE::TPMAPI_E_ENCRYPTION_FAILED,
            "TPMAPI_E_INVALID_KEY_PARAMS" => TPM_SOFTWARE::TPMAPI_E_INVALID_KEY_PARAMS,
            "TPMAPI_E_INVALID_MIGRATION_AUTHORIZATION_BLOB" => TPM_SOFTWARE::TPMAPI_E_INVALID_MIGRATION_AUTHORIZATION_BLOB,
            "TPMAPI_E_INVALID_PCR_INDEX" => TPM_SOFTWARE::TPMAPI_E_INVALID_PCR_INDEX,
            "TPMAPI_E_INVALID_DELEGATE_BLOB" => TPM_SOFTWARE::TPMAPI_E_INVALID_DELEGATE_BLOB,
            "TPMAPI_E_INVALID_CONTEXT_PARAMS" => TPM_SOFTWARE::TPMAPI_E_INVALID_CONTEXT_PARAMS,
            "TPMAPI_E_INVALID_KEY_BLOB" => TPM_SOFTWARE::TPMAPI_E_INVALID_KEY_BLOB,
            "TPMAPI_E_INVALID_PCR_DATA" => TPM_SOFTWARE::TPMAPI_E_INVALID_PCR_DATA,
            "TPMAPI_E_INVALID_OWNER_AUTH" => TPM_SOFTWARE::TPMAPI_E_INVALID_OWNER_AUTH,
            "TPMAPI_E_FIPS_RNG_CHECK_FAILED" => TPM_SOFTWARE::TPMAPI_E_FIPS_RNG_CHECK_FAILED,
            "TPMAPI_E_EMPTY_TCG_LOG" => TPM_SOFTWARE::TPMAPI_E_EMPTY_TCG_LOG,
            "TPMAPI_E_INVALID_TCG_LOG_ENTRY" => TPM_SOFTWARE::TPMAPI_E_INVALID_TCG_LOG_ENTRY,
            "TPMAPI_E_TCG_SEPARATOR_ABSENT" => TPM_SOFTWARE::TPMAPI_E_TCG_SEPARATOR_ABSENT,
            "TPMAPI_E_TCG_INVALID_DIGEST_ENTRY" => TPM_SOFTWARE::TPMAPI_E_TCG_INVALID_DIGEST_ENTRY,
            "TPMAPI_E_POLICY_DENIES_OPERATION" => TPM_SOFTWARE::TPMAPI_E_POLICY_DENIES_OPERATION,
            "TPMAPI_E_NV_BITS_NOT_DEFINED" => TPM_SOFTWARE::TPMAPI_E_NV_BITS_NOT_DEFINED,
            "TPMAPI_E_NV_BITS_NOT_READY" => TPM_SOFTWARE::TPMAPI_E_NV_BITS_NOT_READY,
            "TPMAPI_E_SEALING_KEY_NOT_AVAILABLE" => TPM_SOFTWARE::TPMAPI_E_SEALING_KEY_NOT_AVAILABLE,
            "TPMAPI_E_NO_AUTHORIZATION_CHAIN_FOUND" => TPM_SOFTWARE::TPMAPI_E_NO_AUTHORIZATION_CHAIN_FOUND,
            "TPMAPI_E_SVN_COUNTER_NOT_AVAILABLE" => TPM_SOFTWARE::TPMAPI_E_SVN_COUNTER_NOT_AVAILABLE,
            "TPMAPI_E_OWNER_AUTH_NOT_NULL" => TPM_SOFTWARE::TPMAPI_E_OWNER_AUTH_NOT_NULL,
            "TPMAPI_E_ENDORSEMENT_AUTH_NOT_NULL" => TPM_SOFTWARE::TPMAPI_E_ENDORSEMENT_AUTH_NOT_NULL,
            "TPMAPI_E_AUTHORIZATION_REVOKED" => TPM_SOFTWARE::TPMAPI_E_AUTHORIZATION_REVOKED,
            "TPMAPI_E_MALFORMED_AUTHORIZATION_KEY" => TPM_SOFTWARE::TPMAPI_E_MALFORMED_AUTHORIZATION_KEY,
            "TPMAPI_E_AUTHORIZING_KEY_NOT_SUPPORTED" => TPM_SOFTWARE::TPMAPI_E_AUTHORIZING_KEY_NOT_SUPPORTED,
            "TPMAPI_E_INVALID_AUTHORIZATION_SIGNATURE" => TPM_SOFTWARE::TPMAPI_E_INVALID_AUTHORIZATION_SIGNATURE,
            "TPMAPI_E_MALFORMED_AUTHORIZATION_POLICY" => TPM_SOFTWARE::TPMAPI_E_MALFORMED_AUTHORIZATION_POLICY,
            "TPMAPI_E_MALFORMED_AUTHORIZATION_OTHER" => TPM_SOFTWARE::TPMAPI_E_MALFORMED_AUTHORIZATION_OTHER,
            "TPMAPI_E_SEALING_KEY_CHANGED" => TPM_SOFTWARE::TPMAPI_E_SEALING_KEY_CHANGED,
            "TPMAPI_E_INVALID_TPM_VERSION" => TPM_SOFTWARE::TPMAPI_E_INVALID_TPM_VERSION,
            "TPMAPI_E_INVALID_POLICYAUTH_BLOB_TYPE" => TPM_SOFTWARE::TPMAPI_E_INVALID_POLICYAUTH_BLOB_TYPE,
            "TBSIMP_E_BUFFER_TOO_SMALL" => TPM_SOFTWARE::TBSIMP_E_BUFFER_TOO_SMALL,
            "TBSIMP_E_CLEANUP_FAILED" => TPM_SOFTWARE::TBSIMP_E_CLEANUP_FAILED,
            "TBSIMP_E_INVALID_CONTEXT_HANDLE" => TPM_SOFTWARE::TBSIMP_E_INVALID_CONTEXT_HANDLE,
            "TBSIMP_E_INVALID_CONTEXT_PARAM" => TPM_SOFTWARE::TBSIMP_E_INVALID_CONTEXT_PARAM,
            "TBSIMP_E_TPM_ERROR" => TPM_SOFTWARE::TBSIMP_E_TPM_ERROR,
            "TBSIMP_E_HASH_BAD_KEY" => TPM_SOFTWARE::TBSIMP_E_HASH_BAD_KEY,
            "TBSIMP_E_DUPLICATE_VHANDLE" => TPM_SOFTWARE::TBSIMP_E_DUPLICATE_VHANDLE,
            "TBSIMP_E_INVALID_OUTPUT_POINTER" => TPM_SOFTWARE::TBSIMP_E_INVALID_OUTPUT_POINTER,
            "TBSIMP_E_INVALID_PARAMETER" => TPM_SOFTWARE::TBSIMP_E_INVALID_PARAMETER,
            "TBSIMP_E_RPC_INIT_FAILED" => TPM_SOFTWARE::TBSIMP_E_RPC_INIT_FAILED,
            "TBSIMP_E_SCHEDULER_NOT_RUNNING" => TPM_SOFTWARE::TBSIMP_E_SCHEDULER_NOT_RUNNING,
            "TBSIMP_E_COMMAND_CANCELED" => TPM_SOFTWARE::TBSIMP_E_COMMAND_CANCELED,
            "TBSIMP_E_OUT_OF_MEMORY" => TPM_SOFTWARE::TBSIMP_E_OUT_OF_MEMORY,
            "TBSIMP_E_LIST_NO_MORE_ITEMS" => TPM_SOFTWARE::TBSIMP_E_LIST_NO_MORE_ITEMS,
            "TBSIMP_E_LIST_NOT_FOUND" => TPM_SOFTWARE::TBSIMP_E_LIST_NOT_FOUND,
            "TBSIMP_E_NOT_ENOUGH_SPACE" => TPM_SOFTWARE::TBSIMP_E_NOT_ENOUGH_SPACE,
            "TBSIMP_E_NOT_ENOUGH_TPM_CONTEXTS" => TPM_SOFTWARE::TBSIMP_E_NOT_ENOUGH_TPM_CONTEXTS,
            "TBSIMP_E_COMMAND_FAILED" => TPM_SOFTWARE::TBSIMP_E_COMMAND_FAILED,
            "TBSIMP_E_UNKNOWN_ORDINAL" => TPM_SOFTWARE::TBSIMP_E_UNKNOWN_ORDINAL,
            "TBSIMP_E_RESOURCE_EXPIRED" => TPM_SOFTWARE::TBSIMP_E_RESOURCE_EXPIRED,
            "TBSIMP_E_INVALID_RESOURCE" => TPM_SOFTWARE::TBSIMP_E_INVALID_RESOURCE,
            "TBSIMP_E_NOTHING_TO_UNLOAD" => TPM_SOFTWARE::TBSIMP_E_NOTHING_TO_UNLOAD,
            "TBSIMP_E_HASH_TABLE_FULL" => TPM_SOFTWARE::TBSIMP_E_HASH_TABLE_FULL,
            "TBSIMP_E_TOO_MANY_TBS_CONTEXTS" => TPM_SOFTWARE::TBSIMP_E_TOO_MANY_TBS_CONTEXTS,
            "TBSIMP_E_TOO_MANY_RESOURCES" => TPM_SOFTWARE::TBSIMP_E_TOO_MANY_RESOURCES,
            "TBSIMP_E_PPI_NOT_SUPPORTED" => TPM_SOFTWARE::TBSIMP_E_PPI_NOT_SUPPORTED,
            "TBSIMP_E_TPM_INCOMPATIBLE" => TPM_SOFTWARE::TBSIMP_E_TPM_INCOMPATIBLE,
            "TBSIMP_E_NO_EVENT_LOG" => TPM_SOFTWARE::TBSIMP_E_NO_EVENT_LOG,
            "TPM_E_PPI_ACPI_FAILURE" => TPM_SOFTWARE::TPM_E_PPI_ACPI_FAILURE,
            "TPM_E_PPI_USER_ABORT" => TPM_SOFTWARE::TPM_E_PPI_USER_ABORT,
            "TPM_E_PPI_BIOS_FAILURE" => TPM_SOFTWARE::TPM_E_PPI_BIOS_FAILURE,
            "TPM_E_PPI_NOT_SUPPORTED" => TPM_SOFTWARE::TPM_E_PPI_NOT_SUPPORTED,
            "TPM_E_PPI_BLOCKED_IN_BIOS" => TPM_SOFTWARE::TPM_E_PPI_BLOCKED_IN_BIOS,
            "TPM_E_PCP_ERROR_MASK" => TPM_SOFTWARE::TPM_E_PCP_ERROR_MASK,
            "TPM_E_PCP_DEVICE_NOT_READY" => TPM_SOFTWARE::TPM_E_PCP_DEVICE_NOT_READY,
            "TPM_E_PCP_INVALID_HANDLE" => TPM_SOFTWARE::TPM_E_PCP_INVALID_HANDLE,
            "TPM_E_PCP_INVALID_PARAMETER" => TPM_SOFTWARE::TPM_E_PCP_INVALID_PARAMETER,
            "TPM_E_PCP_FLAG_NOT_SUPPORTED" => TPM_SOFTWARE::TPM_E_PCP_FLAG_NOT_SUPPORTED,
            "TPM_E_PCP_NOT_SUPPORTED" => TPM_SOFTWARE::TPM_E_PCP_NOT_SUPPORTED,
            "TPM_E_PCP_BUFFER_TOO_SMALL" => TPM_SOFTWARE::TPM_E_PCP_BUFFER_TOO_SMALL,
            "TPM_E_PCP_INTERNAL_ERROR" => TPM_SOFTWARE::TPM_E_PCP_INTERNAL_ERROR,
            "TPM_E_PCP_AUTHENTICATION_FAILED" => TPM_SOFTWARE::TPM_E_PCP_AUTHENTICATION_FAILED,
            "TPM_E_PCP_AUTHENTICATION_IGNORED" => TPM_SOFTWARE::TPM_E_PCP_AUTHENTICATION_IGNORED,
            "TPM_E_PCP_POLICY_NOT_FOUND" => TPM_SOFTWARE::TPM_E_PCP_POLICY_NOT_FOUND,
            "TPM_E_PCP_PROFILE_NOT_FOUND" => TPM_SOFTWARE::TPM_E_PCP_PROFILE_NOT_FOUND,
            "TPM_E_PCP_VALIDATION_FAILED" => TPM_SOFTWARE::TPM_E_PCP_VALIDATION_FAILED,
            "TPM_E_PCP_WRONG_PARENT" => TPM_SOFTWARE::TPM_E_PCP_WRONG_PARENT,
            "TPM_E_KEY_NOT_LOADED" => TPM_SOFTWARE::TPM_E_KEY_NOT_LOADED,
            "TPM_E_NO_KEY_CERTIFICATION" => TPM_SOFTWARE::TPM_E_NO_KEY_CERTIFICATION,
            "TPM_E_KEY_NOT_FINALIZED" => TPM_SOFTWARE::TPM_E_KEY_NOT_FINALIZED,
            "TPM_E_ATTESTATION_CHALLENGE_NOT_SET" => TPM_SOFTWARE::TPM_E_ATTESTATION_CHALLENGE_NOT_SET,
            "TPM_E_NOT_PCR_BOUND" => TPM_SOFTWARE::TPM_E_NOT_PCR_BOUND,
            "TPM_E_KEY_ALREADY_FINALIZED" => TPM_SOFTWARE::TPM_E_KEY_ALREADY_FINALIZED,
            "TPM_E_KEY_USAGE_POLICY_NOT_SUPPORTED" => TPM_SOFTWARE::TPM_E_KEY_USAGE_POLICY_NOT_SUPPORTED,
            "TPM_E_KEY_USAGE_POLICY_INVALID" => TPM_SOFTWARE::TPM_E_KEY_USAGE_POLICY_INVALID,
            "TPM_E_SOFT_KEY_ERROR" => TPM_SOFTWARE::TPM_E_SOFT_KEY_ERROR,
            "TPM_E_KEY_NOT_AUTHENTICATED" => TPM_SOFTWARE::TPM_E_KEY_NOT_AUTHENTICATED,
            "TPM_E_PCP_KEY_NOT_AIK" => TPM_SOFTWARE::TPM_E_PCP_KEY_NOT_AIK,
            "TPM_E_KEY_NOT_SIGNING_KEY" => TPM_SOFTWARE::TPM_E_KEY_NOT_SIGNING_KEY,
            "TPM_E_LOCKED_OUT" => TPM_SOFTWARE::TPM_E_LOCKED_OUT,
            "TPM_E_CLAIM_TYPE_NOT_SUPPORTED" => TPM_SOFTWARE::TPM_E_CLAIM_TYPE_NOT_SUPPORTED,
            "TPM_E_VERSION_NOT_SUPPORTED" => TPM_SOFTWARE::TPM_E_VERSION_NOT_SUPPORTED,
            "TPM_E_BUFFER_LENGTH_MISMATCH" => TPM_SOFTWARE::TPM_E_BUFFER_LENGTH_MISMATCH,
            "TPM_E_PCP_IFX_RSA_KEY_CREATION_BLOCKED" => TPM_SOFTWARE::TPM_E_PCP_IFX_RSA_KEY_CREATION_BLOCKED,
            "TPM_E_PCP_TICKET_MISSING" => TPM_SOFTWARE::TPM_E_PCP_TICKET_MISSING,
            "TPM_E_PCP_RAW_POLICY_NOT_SUPPORTED" => TPM_SOFTWARE::TPM_E_PCP_RAW_POLICY_NOT_SUPPORTED,
            "TPM_E_PCP_KEY_HANDLE_INVALIDATED" => TPM_SOFTWARE::TPM_E_PCP_KEY_HANDLE_INVALIDATED,
            "TPM_E_ZERO_EXHAUST_ENABLED" => TPM_SOFTWARE::TPM_E_ZERO_EXHAUST_ENABLED,
            "TPM_E_PROVISIONING_INCOMPLETE" => TPM_SOFTWARE::TPM_E_PROVISIONING_INCOMPLETE,
            "TPM_E_INVALID_OWNER_AUTH" => TPM_SOFTWARE::TPM_E_INVALID_OWNER_AUTH,
            "TPM_E_TOO_MUCH_DATA" => TPM_SOFTWARE::TPM_E_TOO_MUCH_DATA,
            "TPM_E_TPM_GENERATED_EPS" => TPM_SOFTWARE::TPM_E_TPM_GENERATED_EPS,
        }
    }
    pub fn from_code(code: u32) -> TPM_SOFTWARE {
        return match code {
            0x40290423 => TPM_SOFTWARE::TPM_E_PCP_UNSUPPORTED_PSS_SALT,
            0x40290424 => TPM_SOFTWARE::TPM_E_PCP_PLATFORM_CLAIM_MAY_BE_OUTDATED,
            0x40290425 => TPM_SOFTWARE::TPM_E_PCP_PLATFORM_CLAIM_OUTDATED,
            0x40290426 => TPM_SOFTWARE::TPM_E_PCP_PLATFORM_CLAIM_REBOOT,
            0x80290100 => TPM_SOFTWARE::TPMAPI_E_INVALID_STATE,
            0x80290101 => TPM_SOFTWARE::TPMAPI_E_NOT_ENOUGH_DATA,
            0x80290102 => TPM_SOFTWARE::TPMAPI_E_TOO_MUCH_DATA,
            0x80290103 => TPM_SOFTWARE::TPMAPI_E_INVALID_OUTPUT_POINTER,
            0x80290104 => TPM_SOFTWARE::TPMAPI_E_INVALID_PARAMETER,
            0x80290105 => TPM_SOFTWARE::TPMAPI_E_OUT_OF_MEMORY,
            0x80290106 => TPM_SOFTWARE::TPMAPI_E_BUFFER_TOO_SMALL,
            0x80290107 => TPM_SOFTWARE::TPMAPI_E_INTERNAL_ERROR,
            0x80290108 => TPM_SOFTWARE::TPMAPI_E_ACCESS_DENIED,
            0x80290109 => TPM_SOFTWARE::TPMAPI_E_AUTHORIZATION_FAILED,
            0x8029010A => TPM_SOFTWARE::TPMAPI_E_INVALID_CONTEXT_HANDLE,
            0x8029010B => TPM_SOFTWARE::TPMAPI_E_TBS_COMMUNICATION_ERROR,
            0x8029010C => TPM_SOFTWARE::TPMAPI_E_TPM_COMMAND_ERROR,
            0x8029010D => TPM_SOFTWARE::TPMAPI_E_MESSAGE_TOO_LARGE,
            0x8029010E => TPM_SOFTWARE::TPMAPI_E_INVALID_ENCODING,
            0x8029010F => TPM_SOFTWARE::TPMAPI_E_INVALID_KEY_SIZE,
            0x80290110 => TPM_SOFTWARE::TPMAPI_E_ENCRYPTION_FAILED,
            0x80290111 => TPM_SOFTWARE::TPMAPI_E_INVALID_KEY_PARAMS,
            0x80290112 => TPM_SOFTWARE::TPMAPI_E_INVALID_MIGRATION_AUTHORIZATION_BLOB,
            0x80290113 => TPM_SOFTWARE::TPMAPI_E_INVALID_PCR_INDEX,
            0x80290114 => TPM_SOFTWARE::TPMAPI_E_INVALID_DELEGATE_BLOB,
            0x80290115 => TPM_SOFTWARE::TPMAPI_E_INVALID_CONTEXT_PARAMS,
            0x80290116 => TPM_SOFTWARE::TPMAPI_E_INVALID_KEY_BLOB,
            0x80290117 => TPM_SOFTWARE::TPMAPI_E_INVALID_PCR_DATA,
            0x80290118 => TPM_SOFTWARE::TPMAPI_E_INVALID_OWNER_AUTH,
            0x80290119 => TPM_SOFTWARE::TPMAPI_E_FIPS_RNG_CHECK_FAILED,
            0x8029011A => TPM_SOFTWARE::TPMAPI_E_EMPTY_TCG_LOG,
            0x8029011B => TPM_SOFTWARE::TPMAPI_E_INVALID_TCG_LOG_ENTRY,
            0x8029011C => TPM_SOFTWARE::TPMAPI_E_TCG_SEPARATOR_ABSENT,
            0x8029011D => TPM_SOFTWARE::TPMAPI_E_TCG_INVALID_DIGEST_ENTRY,
            0x8029011E => TPM_SOFTWARE::TPMAPI_E_POLICY_DENIES_OPERATION,
            0x8029011F => TPM_SOFTWARE::TPMAPI_E_NV_BITS_NOT_DEFINED,
            0x80290120 => TPM_SOFTWARE::TPMAPI_E_NV_BITS_NOT_READY,
            0x80290121 => TPM_SOFTWARE::TPMAPI_E_SEALING_KEY_NOT_AVAILABLE,
            0x80290122 => TPM_SOFTWARE::TPMAPI_E_NO_AUTHORIZATION_CHAIN_FOUND,
            0x80290123 => TPM_SOFTWARE::TPMAPI_E_SVN_COUNTER_NOT_AVAILABLE,
            0x80290124 => TPM_SOFTWARE::TPMAPI_E_OWNER_AUTH_NOT_NULL,
            0x80290125 => TPM_SOFTWARE::TPMAPI_E_ENDORSEMENT_AUTH_NOT_NULL,
            0x80290126 => TPM_SOFTWARE::TPMAPI_E_AUTHORIZATION_REVOKED,
            0x80290127 => TPM_SOFTWARE::TPMAPI_E_MALFORMED_AUTHORIZATION_KEY,
            0x80290128 => TPM_SOFTWARE::TPMAPI_E_AUTHORIZING_KEY_NOT_SUPPORTED,
            0x80290129 => TPM_SOFTWARE::TPMAPI_E_INVALID_AUTHORIZATION_SIGNATURE,
            0x8029012A => TPM_SOFTWARE::TPMAPI_E_MALFORMED_AUTHORIZATION_POLICY,
            0x8029012B => TPM_SOFTWARE::TPMAPI_E_MALFORMED_AUTHORIZATION_OTHER,
            0x8029012C => TPM_SOFTWARE::TPMAPI_E_SEALING_KEY_CHANGED,
            0x8029012D => TPM_SOFTWARE::TPMAPI_E_INVALID_TPM_VERSION,
            0x8029012E => TPM_SOFTWARE::TPMAPI_E_INVALID_POLICYAUTH_BLOB_TYPE,
            0x80290200 => TPM_SOFTWARE::TBSIMP_E_BUFFER_TOO_SMALL,
            0x80290201 => TPM_SOFTWARE::TBSIMP_E_CLEANUP_FAILED,
            0x80290202 => TPM_SOFTWARE::TBSIMP_E_INVALID_CONTEXT_HANDLE,
            0x80290203 => TPM_SOFTWARE::TBSIMP_E_INVALID_CONTEXT_PARAM,
            0x80290204 => TPM_SOFTWARE::TBSIMP_E_TPM_ERROR,
            0x80290205 => TPM_SOFTWARE::TBSIMP_E_HASH_BAD_KEY,
            0x80290206 => TPM_SOFTWARE::TBSIMP_E_DUPLICATE_VHANDLE,
            0x80290207 => TPM_SOFTWARE::TBSIMP_E_INVALID_OUTPUT_POINTER,
            0x80290208 => TPM_SOFTWARE::TBSIMP_E_INVALID_PARAMETER,
            0x80290209 => TPM_SOFTWARE::TBSIMP_E_RPC_INIT_FAILED,
            0x8029020A => TPM_SOFTWARE::TBSIMP_E_SCHEDULER_NOT_RUNNING,
            0x8029020B => TPM_SOFTWARE::TBSIMP_E_COMMAND_CANCELED,
            0x8029020C => TPM_SOFTWARE::TBSIMP_E_OUT_OF_MEMORY,
            0x8029020D => TPM_SOFTWARE::TBSIMP_E_LIST_NO_MORE_ITEMS,
            0x8029020E => TPM_SOFTWARE::TBSIMP_E_LIST_NOT_FOUND,
            0x8029020F => TPM_SOFTWARE::TBSIMP_E_NOT_ENOUGH_SPACE,
            0x80290210 => TPM_SOFTWARE::TBSIMP_E_NOT_ENOUGH_TPM_CONTEXTS,
            0x80290211 => TPM_SOFTWARE::TBSIMP_E_COMMAND_FAILED,
            0x80290212 => TPM_SOFTWARE::TBSIMP_E_UNKNOWN_ORDINAL,
            0x80290213 => TPM_SOFTWARE::TBSIMP_E_RESOURCE_EXPIRED,
            0x80290214 => TPM_SOFTWARE::TBSIMP_E_INVALID_RESOURCE,
            0x80290215 => TPM_SOFTWARE::TBSIMP_E_NOTHING_TO_UNLOAD,
            0x80290216 => TPM_SOFTWARE::TBSIMP_E_HASH_TABLE_FULL,
            0x80290217 => TPM_SOFTWARE::TBSIMP_E_TOO_MANY_TBS_CONTEXTS,
            0x80290218 => TPM_SOFTWARE::TBSIMP_E_TOO_MANY_RESOURCES,
            0x80290219 => TPM_SOFTWARE::TBSIMP_E_PPI_NOT_SUPPORTED,
            0x8029021A => TPM_SOFTWARE::TBSIMP_E_TPM_INCOMPATIBLE,
            0x8029021B => TPM_SOFTWARE::TBSIMP_E_NO_EVENT_LOG,
            0x80290300 => TPM_SOFTWARE::TPM_E_PPI_ACPI_FAILURE,
            0x80290301 => TPM_SOFTWARE::TPM_E_PPI_USER_ABORT,
            0x80290302 => TPM_SOFTWARE::TPM_E_PPI_BIOS_FAILURE,
            0x80290303 => TPM_SOFTWARE::TPM_E_PPI_NOT_SUPPORTED,
            0x80290304 => TPM_SOFTWARE::TPM_E_PPI_BLOCKED_IN_BIOS,
            0x80290400 => TPM_SOFTWARE::TPM_E_PCP_ERROR_MASK,
            0x80290401 => TPM_SOFTWARE::TPM_E_PCP_DEVICE_NOT_READY,
            0x80290402 => TPM_SOFTWARE::TPM_E_PCP_INVALID_HANDLE,
            0x80290403 => TPM_SOFTWARE::TPM_E_PCP_INVALID_PARAMETER,
            0x80290404 => TPM_SOFTWARE::TPM_E_PCP_FLAG_NOT_SUPPORTED,
            0x80290405 => TPM_SOFTWARE::TPM_E_PCP_NOT_SUPPORTED,
            0x80290406 => TPM_SOFTWARE::TPM_E_PCP_BUFFER_TOO_SMALL,
            0x80290407 => TPM_SOFTWARE::TPM_E_PCP_INTERNAL_ERROR,
            0x80290408 => TPM_SOFTWARE::TPM_E_PCP_AUTHENTICATION_FAILED,
            0x80290409 => TPM_SOFTWARE::TPM_E_PCP_AUTHENTICATION_IGNORED,
            0x8029040A => TPM_SOFTWARE::TPM_E_PCP_POLICY_NOT_FOUND,
            0x8029040B => TPM_SOFTWARE::TPM_E_PCP_PROFILE_NOT_FOUND,
            0x8029040C => TPM_SOFTWARE::TPM_E_PCP_VALIDATION_FAILED,
            0x8029040E => TPM_SOFTWARE::TPM_E_PCP_WRONG_PARENT,
            0x8029040F => TPM_SOFTWARE::TPM_E_KEY_NOT_LOADED,
            0x80290410 => TPM_SOFTWARE::TPM_E_NO_KEY_CERTIFICATION,
            0x80290411 => TPM_SOFTWARE::TPM_E_KEY_NOT_FINALIZED,
            0x80290412 => TPM_SOFTWARE::TPM_E_ATTESTATION_CHALLENGE_NOT_SET,
            0x80290413 => TPM_SOFTWARE::TPM_E_NOT_PCR_BOUND,
            0x80290414 => TPM_SOFTWARE::TPM_E_KEY_ALREADY_FINALIZED,
            0x80290415 => TPM_SOFTWARE::TPM_E_KEY_USAGE_POLICY_NOT_SUPPORTED,
            0x80290416 => TPM_SOFTWARE::TPM_E_KEY_USAGE_POLICY_INVALID,
            0x80290417 => TPM_SOFTWARE::TPM_E_SOFT_KEY_ERROR,
            0x80290418 => TPM_SOFTWARE::TPM_E_KEY_NOT_AUTHENTICATED,
            0x80290419 => TPM_SOFTWARE::TPM_E_PCP_KEY_NOT_AIK,
            0x8029041A => TPM_SOFTWARE::TPM_E_KEY_NOT_SIGNING_KEY,
            0x8029041B => TPM_SOFTWARE::TPM_E_LOCKED_OUT,
            0x8029041C => TPM_SOFTWARE::TPM_E_CLAIM_TYPE_NOT_SUPPORTED,
            0x8029041D => TPM_SOFTWARE::TPM_E_VERSION_NOT_SUPPORTED,
            0x8029041E => TPM_SOFTWARE::TPM_E_BUFFER_LENGTH_MISMATCH,
            0x8029041F => TPM_SOFTWARE::TPM_E_PCP_IFX_RSA_KEY_CREATION_BLOCKED,
            0x80290420 => TPM_SOFTWARE::TPM_E_PCP_TICKET_MISSING,
            0x80290421 => TPM_SOFTWARE::TPM_E_PCP_RAW_POLICY_NOT_SUPPORTED,
            0x80290422 => TPM_SOFTWARE::TPM_E_PCP_KEY_HANDLE_INVALIDATED,
            0x80290500 => TPM_SOFTWARE::TPM_E_ZERO_EXHAUST_ENABLED,
            0x80290600 => TPM_SOFTWARE::TPM_E_PROVISIONING_INCOMPLETE,
            0x80290601 => TPM_SOFTWARE::TPM_E_INVALID_OWNER_AUTH,
            0x80290602 => TPM_SOFTWARE::TPM_E_TOO_MUCH_DATA,
            0x80290603 => TPM_SOFTWARE::TPM_E_TPM_GENERATED_EPS,
        }
    }
}
