/*
 * Copyright 2023 Stanislav Mikhailov (xavetar)
 *
 * Licensed under the Creative Commons Zero v1.0 Universal (CC0) License.
 * You may obtain a copy of the License at
 *
 *     http://creativecommons.org/publicdomain/zero/1.0/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the CC0 license is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// It is strictly forbidden to use the from_code method, respected Microsoft decided to define
// duplicate codes, but by defining different constant names, therefore, when using these codes,
// it is mandatory to use through from_name, and not from_code. Otherwise, it may cause undefined
// behavior or an unknown exception. Because one code corresponds to several constants.

use super::{RawError};

#[derive(Clone, Debug, Copy, Eq, Hash, Ord, PartialEq, PartialOrd)]
pub enum USERMODE_HNS {
    HCN_E_NETWORK_NOT_FOUND,
    HCN_E_ENDPOINT_NOT_FOUND,
    HCN_E_LAYER_NOT_FOUND,
    HCN_E_SWITCH_NOT_FOUND,
    HCN_E_SUBNET_NOT_FOUND,
    HCN_E_ADAPTER_NOT_FOUND,
    HCN_E_PORT_NOT_FOUND,
    HCN_E_POLICY_NOT_FOUND,
    HCN_E_VFP_PORTSETTING_NOT_FOUND,
    HCN_E_INVALID_NETWORK,
    HCN_E_INVALID_NETWORK_TYPE,
    HCN_E_INVALID_ENDPOINT,
    HCN_E_INVALID_POLICY,
    HCN_E_INVALID_POLICY_TYPE,
    HCN_E_INVALID_REMOTE_ENDPOINT_OPERATION,
    HCN_E_NETWORK_ALREADY_EXISTS,
    HCN_E_LAYER_ALREADY_EXISTS,
    HCN_E_POLICY_ALREADY_EXISTS,
    HCN_E_PORT_ALREADY_EXISTS,
    HCN_E_ENDPOINT_ALREADY_ATTACHED,
    HCN_E_REQUEST_UNSUPPORTED,
    HCN_E_MAPPING_NOT_SUPPORTED,
    HCN_E_DEGRADED_OPERATION,
    HCN_E_SHARED_SWITCH_MODIFICATION,
    HCN_E_GUID_CONVERSION_FAILURE,
    HCN_E_REGKEY_FAILURE,
    HCN_E_INVALID_JSON,
    HCN_E_INVALID_JSON_REFERENCE,
    HCN_E_ENDPOINT_SHARING_DISABLED,
    HCN_E_INVALID_IP,
    HCN_E_SWITCH_EXTENSION_NOT_FOUND,
    HCN_E_MANAGER_STOPPED,
    GCN_E_MODULE_NOT_FOUND,
    GCN_E_NO_REQUEST_HANDLERS,
    GCN_E_REQUEST_UNSUPPORTED,
    GCN_E_RUNTIMEKEYS_FAILED,
    GCN_E_NETADAPTER_TIMEOUT,
    GCN_E_NETADAPTER_NOT_FOUND,
    GCN_E_NETCOMPARTMENT_NOT_FOUND,
    GCN_E_NETINTERFACE_NOT_FOUND,
    GCN_E_DEFAULTNAMESPACE_EXISTS,
    HCN_E_ICS_DISABLED,
    HCN_E_ENDPOINT_NAMESPACE_ALREADY_EXISTS,
    HCN_E_ENTITY_HAS_REFERENCES,
    HCN_E_INVALID_INTERNAL_PORT,
    HCN_E_NAMESPACE_ATTACH_FAILED,
    HCN_E_ADDR_INVALID_OR_RESERVED,
    HCN_E_INVALID_PREFIX,
    HCN_E_OBJECT_USED_AFTER_UNLOAD,
    HCN_E_INVALID_SUBNET,
    HCN_E_INVALID_IP_SUBNET,
    HCN_E_ENDPOINT_NOT_ATTACHED,
    HCN_E_ENDPOINT_NOT_LOCAL,
    HCN_INTERFACEPARAMETERS_ALREADY_APPLIED,
    HCN_E_VFP_NOT_ALLOWED,
    ERROR_HNS_PORT_ALLOCATED,
}

impl USERMODE_HNS {
    pub fn code(&self) -> u32 {
        return match self {
            USERMODE_HNS::HCN_E_NETWORK_NOT_FOUND => 0x803B0001 as u32,
            USERMODE_HNS::HCN_E_ENDPOINT_NOT_FOUND => 0x803B0002 as u32,
            USERMODE_HNS::HCN_E_LAYER_NOT_FOUND => 0x803B0003 as u32,
            USERMODE_HNS::HCN_E_SWITCH_NOT_FOUND => 0x803B0004 as u32,
            USERMODE_HNS::HCN_E_SUBNET_NOT_FOUND => 0x803B0005 as u32,
            USERMODE_HNS::HCN_E_ADAPTER_NOT_FOUND => 0x803B0006 as u32,
            USERMODE_HNS::HCN_E_PORT_NOT_FOUND => 0x803B0007 as u32,
            USERMODE_HNS::HCN_E_POLICY_NOT_FOUND => 0x803B0008 as u32,
            USERMODE_HNS::HCN_E_VFP_PORTSETTING_NOT_FOUND => 0x803B0009 as u32,
            USERMODE_HNS::HCN_E_INVALID_NETWORK => 0x803B000A as u32,
            USERMODE_HNS::HCN_E_INVALID_NETWORK_TYPE => 0x803B000B as u32,
            USERMODE_HNS::HCN_E_INVALID_ENDPOINT => 0x803B000C as u32,
            USERMODE_HNS::HCN_E_INVALID_POLICY => 0x803B000D as u32,
            USERMODE_HNS::HCN_E_INVALID_POLICY_TYPE => 0x803B000E as u32,
            USERMODE_HNS::HCN_E_INVALID_REMOTE_ENDPOINT_OPERATION => 0x803B000F as u32,
            USERMODE_HNS::HCN_E_NETWORK_ALREADY_EXISTS => 0x803B0010 as u32,
            USERMODE_HNS::HCN_E_LAYER_ALREADY_EXISTS => 0x803B0011 as u32,
            USERMODE_HNS::HCN_E_POLICY_ALREADY_EXISTS => 0x803B0012 as u32,
            USERMODE_HNS::HCN_E_PORT_ALREADY_EXISTS => 0x803B0013 as u32,
            USERMODE_HNS::HCN_E_ENDPOINT_ALREADY_ATTACHED => 0x803B0014 as u32,
            USERMODE_HNS::HCN_E_REQUEST_UNSUPPORTED => 0x803B0015 as u32,
            USERMODE_HNS::HCN_E_MAPPING_NOT_SUPPORTED => 0x803B0016 as u32,
            USERMODE_HNS::HCN_E_DEGRADED_OPERATION => 0x803B0017 as u32,
            USERMODE_HNS::HCN_E_SHARED_SWITCH_MODIFICATION => 0x803B0018 as u32,
            USERMODE_HNS::HCN_E_GUID_CONVERSION_FAILURE => 0x803B0019 as u32,
            USERMODE_HNS::HCN_E_REGKEY_FAILURE => 0x803B001A as u32,
            USERMODE_HNS::HCN_E_INVALID_JSON => 0x803B001B as u32,
            USERMODE_HNS::HCN_E_INVALID_JSON_REFERENCE => 0x803B001C as u32,
            USERMODE_HNS::HCN_E_ENDPOINT_SHARING_DISABLED => 0x803B001D as u32,
            USERMODE_HNS::HCN_E_INVALID_IP => 0x803B001E as u32,
            USERMODE_HNS::HCN_E_SWITCH_EXTENSION_NOT_FOUND => 0x803B001F as u32,
            USERMODE_HNS::HCN_E_MANAGER_STOPPED => 0x803B0020 as u32,
            USERMODE_HNS::GCN_E_MODULE_NOT_FOUND => 0x803B0021 as u32,
            USERMODE_HNS::GCN_E_NO_REQUEST_HANDLERS => 0x803B0022 as u32,
            USERMODE_HNS::GCN_E_REQUEST_UNSUPPORTED => 0x803B0023 as u32,
            USERMODE_HNS::GCN_E_RUNTIMEKEYS_FAILED => 0x803B0024 as u32,
            USERMODE_HNS::GCN_E_NETADAPTER_TIMEOUT => 0x803B0025 as u32,
            USERMODE_HNS::GCN_E_NETADAPTER_NOT_FOUND => 0x803B0026 as u32,
            USERMODE_HNS::GCN_E_NETCOMPARTMENT_NOT_FOUND => 0x803B0027 as u32,
            USERMODE_HNS::GCN_E_NETINTERFACE_NOT_FOUND => 0x803B0028 as u32,
            USERMODE_HNS::GCN_E_DEFAULTNAMESPACE_EXISTS => 0x803B0029 as u32,
            USERMODE_HNS::HCN_E_ICS_DISABLED => 0x803B002A as u32,
            USERMODE_HNS::HCN_E_ENDPOINT_NAMESPACE_ALREADY_EXISTS => 0x803B002B as u32,
            USERMODE_HNS::HCN_E_ENTITY_HAS_REFERENCES => 0x803B002C as u32,
            USERMODE_HNS::HCN_E_INVALID_INTERNAL_PORT => 0x803B002D as u32,
            USERMODE_HNS::HCN_E_NAMESPACE_ATTACH_FAILED => 0x803B002E as u32,
            USERMODE_HNS::HCN_E_ADDR_INVALID_OR_RESERVED => 0x803B002F as u32,
            USERMODE_HNS::HCN_E_INVALID_PREFIX => 0x803B0030 as u32,
            USERMODE_HNS::HCN_E_OBJECT_USED_AFTER_UNLOAD => 0x803B0031 as u32,
            USERMODE_HNS::HCN_E_INVALID_SUBNET => 0x803B0032 as u32,
            USERMODE_HNS::HCN_E_INVALID_IP_SUBNET => 0x803B0033 as u32,
            USERMODE_HNS::HCN_E_ENDPOINT_NOT_ATTACHED => 0x803B0034 as u32,
            USERMODE_HNS::HCN_E_ENDPOINT_NOT_LOCAL => 0x803B0035 as u32,
            USERMODE_HNS::HCN_INTERFACEPARAMETERS_ALREADY_APPLIED => 0x803B0036 as u32,
            USERMODE_HNS::HCN_E_VFP_NOT_ALLOWED => 0x803B0037 as u32,
            USERMODE_HNS::ERROR_HNS_PORT_ALLOCATED => 0xC03B0001 as u32,
        }
    }

    pub fn error(&self) -> RawError {
        return match self {
            USERMODE_HNS::HCN_E_NETWORK_NOT_FOUND => RawError::Kind(USERMODE_HNS::HCN_E_NETWORK_NOT_FOUND),
            USERMODE_HNS::HCN_E_ENDPOINT_NOT_FOUND => RawError::Kind(USERMODE_HNS::HCN_E_ENDPOINT_NOT_FOUND),
            USERMODE_HNS::HCN_E_LAYER_NOT_FOUND => RawError::Kind(USERMODE_HNS::HCN_E_LAYER_NOT_FOUND),
            USERMODE_HNS::HCN_E_SWITCH_NOT_FOUND => RawError::Kind(USERMODE_HNS::HCN_E_SWITCH_NOT_FOUND),
            USERMODE_HNS::HCN_E_SUBNET_NOT_FOUND => RawError::Kind(USERMODE_HNS::HCN_E_SUBNET_NOT_FOUND),
            USERMODE_HNS::HCN_E_ADAPTER_NOT_FOUND => RawError::Kind(USERMODE_HNS::HCN_E_ADAPTER_NOT_FOUND),
            USERMODE_HNS::HCN_E_PORT_NOT_FOUND => RawError::Kind(USERMODE_HNS::HCN_E_PORT_NOT_FOUND),
            USERMODE_HNS::HCN_E_POLICY_NOT_FOUND => RawError::Kind(USERMODE_HNS::HCN_E_POLICY_NOT_FOUND),
            USERMODE_HNS::HCN_E_VFP_PORTSETTING_NOT_FOUND => RawError::Kind(USERMODE_HNS::HCN_E_VFP_PORTSETTING_NOT_FOUND),
            USERMODE_HNS::HCN_E_INVALID_NETWORK => RawError::Kind(USERMODE_HNS::HCN_E_INVALID_NETWORK),
            USERMODE_HNS::HCN_E_INVALID_NETWORK_TYPE => RawError::Kind(USERMODE_HNS::HCN_E_INVALID_NETWORK_TYPE),
            USERMODE_HNS::HCN_E_INVALID_ENDPOINT => RawError::Kind(USERMODE_HNS::HCN_E_INVALID_ENDPOINT),
            USERMODE_HNS::HCN_E_INVALID_POLICY => RawError::Kind(USERMODE_HNS::HCN_E_INVALID_POLICY),
            USERMODE_HNS::HCN_E_INVALID_POLICY_TYPE => RawError::Kind(USERMODE_HNS::HCN_E_INVALID_POLICY_TYPE),
            USERMODE_HNS::HCN_E_INVALID_REMOTE_ENDPOINT_OPERATION => RawError::Kind(USERMODE_HNS::HCN_E_INVALID_REMOTE_ENDPOINT_OPERATION),
            USERMODE_HNS::HCN_E_NETWORK_ALREADY_EXISTS => RawError::Kind(USERMODE_HNS::HCN_E_NETWORK_ALREADY_EXISTS),
            USERMODE_HNS::HCN_E_LAYER_ALREADY_EXISTS => RawError::Kind(USERMODE_HNS::HCN_E_LAYER_ALREADY_EXISTS),
            USERMODE_HNS::HCN_E_POLICY_ALREADY_EXISTS => RawError::Kind(USERMODE_HNS::HCN_E_POLICY_ALREADY_EXISTS),
            USERMODE_HNS::HCN_E_PORT_ALREADY_EXISTS => RawError::Kind(USERMODE_HNS::HCN_E_PORT_ALREADY_EXISTS),
            USERMODE_HNS::HCN_E_ENDPOINT_ALREADY_ATTACHED => RawError::Kind(USERMODE_HNS::HCN_E_ENDPOINT_ALREADY_ATTACHED),
            USERMODE_HNS::HCN_E_REQUEST_UNSUPPORTED => RawError::Kind(USERMODE_HNS::HCN_E_REQUEST_UNSUPPORTED),
            USERMODE_HNS::HCN_E_MAPPING_NOT_SUPPORTED => RawError::Kind(USERMODE_HNS::HCN_E_MAPPING_NOT_SUPPORTED),
            USERMODE_HNS::HCN_E_DEGRADED_OPERATION => RawError::Kind(USERMODE_HNS::HCN_E_DEGRADED_OPERATION),
            USERMODE_HNS::HCN_E_SHARED_SWITCH_MODIFICATION => RawError::Kind(USERMODE_HNS::HCN_E_SHARED_SWITCH_MODIFICATION),
            USERMODE_HNS::HCN_E_GUID_CONVERSION_FAILURE => RawError::Kind(USERMODE_HNS::HCN_E_GUID_CONVERSION_FAILURE),
            USERMODE_HNS::HCN_E_REGKEY_FAILURE => RawError::Kind(USERMODE_HNS::HCN_E_REGKEY_FAILURE),
            USERMODE_HNS::HCN_E_INVALID_JSON => RawError::Kind(USERMODE_HNS::HCN_E_INVALID_JSON),
            USERMODE_HNS::HCN_E_INVALID_JSON_REFERENCE => RawError::Kind(USERMODE_HNS::HCN_E_INVALID_JSON_REFERENCE),
            USERMODE_HNS::HCN_E_ENDPOINT_SHARING_DISABLED => RawError::Kind(USERMODE_HNS::HCN_E_ENDPOINT_SHARING_DISABLED),
            USERMODE_HNS::HCN_E_INVALID_IP => RawError::Kind(USERMODE_HNS::HCN_E_INVALID_IP),
            USERMODE_HNS::HCN_E_SWITCH_EXTENSION_NOT_FOUND => RawError::Kind(USERMODE_HNS::HCN_E_SWITCH_EXTENSION_NOT_FOUND),
            USERMODE_HNS::HCN_E_MANAGER_STOPPED => RawError::Kind(USERMODE_HNS::HCN_E_MANAGER_STOPPED),
            USERMODE_HNS::GCN_E_MODULE_NOT_FOUND => RawError::Kind(USERMODE_HNS::GCN_E_MODULE_NOT_FOUND),
            USERMODE_HNS::GCN_E_NO_REQUEST_HANDLERS => RawError::Kind(USERMODE_HNS::GCN_E_NO_REQUEST_HANDLERS),
            USERMODE_HNS::GCN_E_REQUEST_UNSUPPORTED => RawError::Kind(USERMODE_HNS::GCN_E_REQUEST_UNSUPPORTED),
            USERMODE_HNS::GCN_E_RUNTIMEKEYS_FAILED => RawError::Kind(USERMODE_HNS::GCN_E_RUNTIMEKEYS_FAILED),
            USERMODE_HNS::GCN_E_NETADAPTER_TIMEOUT => RawError::Kind(USERMODE_HNS::GCN_E_NETADAPTER_TIMEOUT),
            USERMODE_HNS::GCN_E_NETADAPTER_NOT_FOUND => RawError::Kind(USERMODE_HNS::GCN_E_NETADAPTER_NOT_FOUND),
            USERMODE_HNS::GCN_E_NETCOMPARTMENT_NOT_FOUND => RawError::Kind(USERMODE_HNS::GCN_E_NETCOMPARTMENT_NOT_FOUND),
            USERMODE_HNS::GCN_E_NETINTERFACE_NOT_FOUND => RawError::Kind(USERMODE_HNS::GCN_E_NETINTERFACE_NOT_FOUND),
            USERMODE_HNS::GCN_E_DEFAULTNAMESPACE_EXISTS => RawError::Kind(USERMODE_HNS::GCN_E_DEFAULTNAMESPACE_EXISTS),
            USERMODE_HNS::HCN_E_ICS_DISABLED => RawError::Kind(USERMODE_HNS::HCN_E_ICS_DISABLED),
            USERMODE_HNS::HCN_E_ENDPOINT_NAMESPACE_ALREADY_EXISTS => RawError::Kind(USERMODE_HNS::HCN_E_ENDPOINT_NAMESPACE_ALREADY_EXISTS),
            USERMODE_HNS::HCN_E_ENTITY_HAS_REFERENCES => RawError::Kind(USERMODE_HNS::HCN_E_ENTITY_HAS_REFERENCES),
            USERMODE_HNS::HCN_E_INVALID_INTERNAL_PORT => RawError::Kind(USERMODE_HNS::HCN_E_INVALID_INTERNAL_PORT),
            USERMODE_HNS::HCN_E_NAMESPACE_ATTACH_FAILED => RawError::Kind(USERMODE_HNS::HCN_E_NAMESPACE_ATTACH_FAILED),
            USERMODE_HNS::HCN_E_ADDR_INVALID_OR_RESERVED => RawError::Kind(USERMODE_HNS::HCN_E_ADDR_INVALID_OR_RESERVED),
            USERMODE_HNS::HCN_E_INVALID_PREFIX => RawError::Kind(USERMODE_HNS::HCN_E_INVALID_PREFIX),
            USERMODE_HNS::HCN_E_OBJECT_USED_AFTER_UNLOAD => RawError::Kind(USERMODE_HNS::HCN_E_OBJECT_USED_AFTER_UNLOAD),
            USERMODE_HNS::HCN_E_INVALID_SUBNET => RawError::Kind(USERMODE_HNS::HCN_E_INVALID_SUBNET),
            USERMODE_HNS::HCN_E_INVALID_IP_SUBNET => RawError::Kind(USERMODE_HNS::HCN_E_INVALID_IP_SUBNET),
            USERMODE_HNS::HCN_E_ENDPOINT_NOT_ATTACHED => RawError::Kind(USERMODE_HNS::HCN_E_ENDPOINT_NOT_ATTACHED),
            USERMODE_HNS::HCN_E_ENDPOINT_NOT_LOCAL => RawError::Kind(USERMODE_HNS::HCN_E_ENDPOINT_NOT_LOCAL),
            USERMODE_HNS::HCN_INTERFACEPARAMETERS_ALREADY_APPLIED => RawError::Kind(USERMODE_HNS::HCN_INTERFACEPARAMETERS_ALREADY_APPLIED),
            USERMODE_HNS::HCN_E_VFP_NOT_ALLOWED => RawError::Kind(USERMODE_HNS::HCN_E_VFP_NOT_ALLOWED),
            USERMODE_HNS::ERROR_HNS_PORT_ALLOCATED => RawError::Kind(USERMODE_HNS::ERROR_HNS_PORT_ALLOCATED),
        }
    }

    pub fn description(&self) -> &'static str {
        return match self {
            USERMODE_HNS::HCN_E_NETWORK_NOT_FOUND => "The network was not found.",
            USERMODE_HNS::HCN_E_ENDPOINT_NOT_FOUND => "The endpoint was not found.",
            USERMODE_HNS::HCN_E_LAYER_NOT_FOUND => "The network's underlying layer was not found.",
            USERMODE_HNS::HCN_E_SWITCH_NOT_FOUND => "The virtual switch was not found.",
            USERMODE_HNS::HCN_E_SUBNET_NOT_FOUND => "The network does not have a subnet for this endpoint.",
            USERMODE_HNS::HCN_E_ADAPTER_NOT_FOUND => "An adapter was not found.",
            USERMODE_HNS::HCN_E_PORT_NOT_FOUND => "The switch-port was not found.",
            USERMODE_HNS::HCN_E_POLICY_NOT_FOUND => "An expected policy was not found.",
            USERMODE_HNS::HCN_E_VFP_PORTSETTING_NOT_FOUND => "A required VFP port setting was not found.",
            USERMODE_HNS::HCN_E_INVALID_NETWORK => "The provided network configuration is invalid or missing parameters.",
            USERMODE_HNS::HCN_E_INVALID_NETWORK_TYPE => "Invalid network type.",
            USERMODE_HNS::HCN_E_INVALID_ENDPOINT => "The provided endpoint configuration is invalid or missing parameters.",
            USERMODE_HNS::HCN_E_INVALID_POLICY => "The provided policy configuration is invalid or missing parameters.",
            USERMODE_HNS::HCN_E_INVALID_POLICY_TYPE => "Invalid policy type.",
            USERMODE_HNS::HCN_E_INVALID_REMOTE_ENDPOINT_OPERATION => "This requested operation is invalid for a remote endpoint.",
            USERMODE_HNS::HCN_E_NETWORK_ALREADY_EXISTS => "A network with this name already exists.",
            USERMODE_HNS::HCN_E_LAYER_ALREADY_EXISTS => "A network with this name already exists.",
            USERMODE_HNS::HCN_E_POLICY_ALREADY_EXISTS => "Policy information already exists on this object.",
            USERMODE_HNS::HCN_E_PORT_ALREADY_EXISTS => "The specified port already exists.",
            USERMODE_HNS::HCN_E_ENDPOINT_ALREADY_ATTACHED => "This endpoint is already attached to the switch.",
            USERMODE_HNS::HCN_E_REQUEST_UNSUPPORTED => "The specified request is unsupported.",
            USERMODE_HNS::HCN_E_MAPPING_NOT_SUPPORTED => "Port mapping is not supported on the given network.",
            USERMODE_HNS::HCN_E_DEGRADED_OPERATION => "There was an operation attempted on a degraded object.",
            USERMODE_HNS::HCN_E_SHARED_SWITCH_MODIFICATION => "Cannot modify a switch shared by multiple networks.",
            USERMODE_HNS::HCN_E_GUID_CONVERSION_FAILURE => "Failed to interpret a parameter as a GUID.",
            USERMODE_HNS::HCN_E_REGKEY_FAILURE => "Failed to process registry key.",
            USERMODE_HNS::HCN_E_INVALID_JSON => "Invalid JSON document string.",
            USERMODE_HNS::HCN_E_INVALID_JSON_REFERENCE => "The reference is invalid in the JSON document.",
            USERMODE_HNS::HCN_E_ENDPOINT_SHARING_DISABLED => "Endpoint sharing is disabled.",
            USERMODE_HNS::HCN_E_INVALID_IP => "IP address is either invalid or not part of any configured subnet(s).",
            USERMODE_HNS::HCN_E_SWITCH_EXTENSION_NOT_FOUND => "The specified switch extension does not exist on this switch.",
            USERMODE_HNS::HCN_E_MANAGER_STOPPED => "Operation cannot be performed while service is stopping.",
            USERMODE_HNS::GCN_E_MODULE_NOT_FOUND => "Operation cannot be performed while service module not found.",
            USERMODE_HNS::GCN_E_NO_REQUEST_HANDLERS => "Request Handlers not present to handle the JSON request.",
            USERMODE_HNS::GCN_E_REQUEST_UNSUPPORTED => "The specified request is unsupported.",
            USERMODE_HNS::GCN_E_RUNTIMEKEYS_FAILED => "Add runtime keys to container failed.",
            USERMODE_HNS::GCN_E_NETADAPTER_TIMEOUT => "Timeout while waiting for network adapter with the given instance id",
            USERMODE_HNS::GCN_E_NETADAPTER_NOT_FOUND => "Network adapter not found for the given instance id",
            USERMODE_HNS::GCN_E_NETCOMPARTMENT_NOT_FOUND => "Network compartment not found for the given  id",
            USERMODE_HNS::GCN_E_NETINTERFACE_NOT_FOUND => "Network interface not found for the given  id",
            USERMODE_HNS::GCN_E_DEFAULTNAMESPACE_EXISTS => "Default Namespace already exists",
            USERMODE_HNS::HCN_E_ICS_DISABLED => "Internet Connection Sharing service (SharedAccess) is disabled and cannot be started",
            USERMODE_HNS::HCN_E_ENDPOINT_NAMESPACE_ALREADY_EXISTS => "This requested operation is invalid as endpoint is already part of a network namespace.",
            USERMODE_HNS::HCN_E_ENTITY_HAS_REFERENCES => "The specified entity cannot be removed while it still has references.",
            USERMODE_HNS::HCN_E_INVALID_INTERNAL_PORT => "The internal port must exist and cannot be zero.",
            USERMODE_HNS::HCN_E_NAMESPACE_ATTACH_FAILED => "The requested operation for attach namespace failed.",
            USERMODE_HNS::HCN_E_ADDR_INVALID_OR_RESERVED => "An address provided is invalid or reserved.",
            USERMODE_HNS::HCN_E_INVALID_PREFIX => "The prefix provided is invalid.",
            USERMODE_HNS::HCN_E_OBJECT_USED_AFTER_UNLOAD => "A call was performed against an object that was torn down.",
            USERMODE_HNS::HCN_E_INVALID_SUBNET => "The provided subnet configuration is invalid or missing parameters.",
            USERMODE_HNS::HCN_E_INVALID_IP_SUBNET => "The provided IP subnet configuration is invalid or missing parameters.",
            USERMODE_HNS::HCN_E_ENDPOINT_NOT_ATTACHED => "The endpoint must be attached to complete the operation.",
            USERMODE_HNS::HCN_E_ENDPOINT_NOT_LOCAL => "The endpoint must be local to complete the operation.",
            USERMODE_HNS::HCN_INTERFACEPARAMETERS_ALREADY_APPLIED => "Cannot apply more than one InterfaceParameters policy.",
            USERMODE_HNS::HCN_E_VFP_NOT_ALLOWED => "A network of this type can not be created because VFP is not available.",
            USERMODE_HNS::ERROR_HNS_PORT_ALLOCATED => "The port is already allocated.",
        }
    }

    pub fn from_name(name: &str) -> USERMODE_HNS {
        return match name {
            "HCN_E_NETWORK_NOT_FOUND" => USERMODE_HNS::HCN_E_NETWORK_NOT_FOUND,
            "HCN_E_ENDPOINT_NOT_FOUND" => USERMODE_HNS::HCN_E_ENDPOINT_NOT_FOUND,
            "HCN_E_LAYER_NOT_FOUND" => USERMODE_HNS::HCN_E_LAYER_NOT_FOUND,
            "HCN_E_SWITCH_NOT_FOUND" => USERMODE_HNS::HCN_E_SWITCH_NOT_FOUND,
            "HCN_E_SUBNET_NOT_FOUND" => USERMODE_HNS::HCN_E_SUBNET_NOT_FOUND,
            "HCN_E_ADAPTER_NOT_FOUND" => USERMODE_HNS::HCN_E_ADAPTER_NOT_FOUND,
            "HCN_E_PORT_NOT_FOUND" => USERMODE_HNS::HCN_E_PORT_NOT_FOUND,
            "HCN_E_POLICY_NOT_FOUND" => USERMODE_HNS::HCN_E_POLICY_NOT_FOUND,
            "HCN_E_VFP_PORTSETTING_NOT_FOUND" => USERMODE_HNS::HCN_E_VFP_PORTSETTING_NOT_FOUND,
            "HCN_E_INVALID_NETWORK" => USERMODE_HNS::HCN_E_INVALID_NETWORK,
            "HCN_E_INVALID_NETWORK_TYPE" => USERMODE_HNS::HCN_E_INVALID_NETWORK_TYPE,
            "HCN_E_INVALID_ENDPOINT" => USERMODE_HNS::HCN_E_INVALID_ENDPOINT,
            "HCN_E_INVALID_POLICY" => USERMODE_HNS::HCN_E_INVALID_POLICY,
            "HCN_E_INVALID_POLICY_TYPE" => USERMODE_HNS::HCN_E_INVALID_POLICY_TYPE,
            "HCN_E_INVALID_REMOTE_ENDPOINT_OPERATION" => USERMODE_HNS::HCN_E_INVALID_REMOTE_ENDPOINT_OPERATION,
            "HCN_E_NETWORK_ALREADY_EXISTS" => USERMODE_HNS::HCN_E_NETWORK_ALREADY_EXISTS,
            "HCN_E_LAYER_ALREADY_EXISTS" => USERMODE_HNS::HCN_E_LAYER_ALREADY_EXISTS,
            "HCN_E_POLICY_ALREADY_EXISTS" => USERMODE_HNS::HCN_E_POLICY_ALREADY_EXISTS,
            "HCN_E_PORT_ALREADY_EXISTS" => USERMODE_HNS::HCN_E_PORT_ALREADY_EXISTS,
            "HCN_E_ENDPOINT_ALREADY_ATTACHED" => USERMODE_HNS::HCN_E_ENDPOINT_ALREADY_ATTACHED,
            "HCN_E_REQUEST_UNSUPPORTED" => USERMODE_HNS::HCN_E_REQUEST_UNSUPPORTED,
            "HCN_E_MAPPING_NOT_SUPPORTED" => USERMODE_HNS::HCN_E_MAPPING_NOT_SUPPORTED,
            "HCN_E_DEGRADED_OPERATION" => USERMODE_HNS::HCN_E_DEGRADED_OPERATION,
            "HCN_E_SHARED_SWITCH_MODIFICATION" => USERMODE_HNS::HCN_E_SHARED_SWITCH_MODIFICATION,
            "HCN_E_GUID_CONVERSION_FAILURE" => USERMODE_HNS::HCN_E_GUID_CONVERSION_FAILURE,
            "HCN_E_REGKEY_FAILURE" => USERMODE_HNS::HCN_E_REGKEY_FAILURE,
            "HCN_E_INVALID_JSON" => USERMODE_HNS::HCN_E_INVALID_JSON,
            "HCN_E_INVALID_JSON_REFERENCE" => USERMODE_HNS::HCN_E_INVALID_JSON_REFERENCE,
            "HCN_E_ENDPOINT_SHARING_DISABLED" => USERMODE_HNS::HCN_E_ENDPOINT_SHARING_DISABLED,
            "HCN_E_INVALID_IP" => USERMODE_HNS::HCN_E_INVALID_IP,
            "HCN_E_SWITCH_EXTENSION_NOT_FOUND" => USERMODE_HNS::HCN_E_SWITCH_EXTENSION_NOT_FOUND,
            "HCN_E_MANAGER_STOPPED" => USERMODE_HNS::HCN_E_MANAGER_STOPPED,
            "GCN_E_MODULE_NOT_FOUND" => USERMODE_HNS::GCN_E_MODULE_NOT_FOUND,
            "GCN_E_NO_REQUEST_HANDLERS" => USERMODE_HNS::GCN_E_NO_REQUEST_HANDLERS,
            "GCN_E_REQUEST_UNSUPPORTED" => USERMODE_HNS::GCN_E_REQUEST_UNSUPPORTED,
            "GCN_E_RUNTIMEKEYS_FAILED" => USERMODE_HNS::GCN_E_RUNTIMEKEYS_FAILED,
            "GCN_E_NETADAPTER_TIMEOUT" => USERMODE_HNS::GCN_E_NETADAPTER_TIMEOUT,
            "GCN_E_NETADAPTER_NOT_FOUND" => USERMODE_HNS::GCN_E_NETADAPTER_NOT_FOUND,
            "GCN_E_NETCOMPARTMENT_NOT_FOUND" => USERMODE_HNS::GCN_E_NETCOMPARTMENT_NOT_FOUND,
            "GCN_E_NETINTERFACE_NOT_FOUND" => USERMODE_HNS::GCN_E_NETINTERFACE_NOT_FOUND,
            "GCN_E_DEFAULTNAMESPACE_EXISTS" => USERMODE_HNS::GCN_E_DEFAULTNAMESPACE_EXISTS,
            "HCN_E_ICS_DISABLED" => USERMODE_HNS::HCN_E_ICS_DISABLED,
            "HCN_E_ENDPOINT_NAMESPACE_ALREADY_EXISTS" => USERMODE_HNS::HCN_E_ENDPOINT_NAMESPACE_ALREADY_EXISTS,
            "HCN_E_ENTITY_HAS_REFERENCES" => USERMODE_HNS::HCN_E_ENTITY_HAS_REFERENCES,
            "HCN_E_INVALID_INTERNAL_PORT" => USERMODE_HNS::HCN_E_INVALID_INTERNAL_PORT,
            "HCN_E_NAMESPACE_ATTACH_FAILED" => USERMODE_HNS::HCN_E_NAMESPACE_ATTACH_FAILED,
            "HCN_E_ADDR_INVALID_OR_RESERVED" => USERMODE_HNS::HCN_E_ADDR_INVALID_OR_RESERVED,
            "HCN_E_INVALID_PREFIX" => USERMODE_HNS::HCN_E_INVALID_PREFIX,
            "HCN_E_OBJECT_USED_AFTER_UNLOAD" => USERMODE_HNS::HCN_E_OBJECT_USED_AFTER_UNLOAD,
            "HCN_E_INVALID_SUBNET" => USERMODE_HNS::HCN_E_INVALID_SUBNET,
            "HCN_E_INVALID_IP_SUBNET" => USERMODE_HNS::HCN_E_INVALID_IP_SUBNET,
            "HCN_E_ENDPOINT_NOT_ATTACHED" => USERMODE_HNS::HCN_E_ENDPOINT_NOT_ATTACHED,
            "HCN_E_ENDPOINT_NOT_LOCAL" => USERMODE_HNS::HCN_E_ENDPOINT_NOT_LOCAL,
            "HCN_INTERFACEPARAMETERS_ALREADY_APPLIED" => USERMODE_HNS::HCN_INTERFACEPARAMETERS_ALREADY_APPLIED,
            "HCN_E_VFP_NOT_ALLOWED" => USERMODE_HNS::HCN_E_VFP_NOT_ALLOWED,
            "ERROR_HNS_PORT_ALLOCATED" => USERMODE_HNS::ERROR_HNS_PORT_ALLOCATED,
        }
    }
    pub fn from_code(code: u32) -> USERMODE_HNS {
        return match code {
            0x803B0001 => USERMODE_HNS::HCN_E_NETWORK_NOT_FOUND,
            0x803B0002 => USERMODE_HNS::HCN_E_ENDPOINT_NOT_FOUND,
            0x803B0003 => USERMODE_HNS::HCN_E_LAYER_NOT_FOUND,
            0x803B0004 => USERMODE_HNS::HCN_E_SWITCH_NOT_FOUND,
            0x803B0005 => USERMODE_HNS::HCN_E_SUBNET_NOT_FOUND,
            0x803B0006 => USERMODE_HNS::HCN_E_ADAPTER_NOT_FOUND,
            0x803B0007 => USERMODE_HNS::HCN_E_PORT_NOT_FOUND,
            0x803B0008 => USERMODE_HNS::HCN_E_POLICY_NOT_FOUND,
            0x803B0009 => USERMODE_HNS::HCN_E_VFP_PORTSETTING_NOT_FOUND,
            0x803B000A => USERMODE_HNS::HCN_E_INVALID_NETWORK,
            0x803B000B => USERMODE_HNS::HCN_E_INVALID_NETWORK_TYPE,
            0x803B000C => USERMODE_HNS::HCN_E_INVALID_ENDPOINT,
            0x803B000D => USERMODE_HNS::HCN_E_INVALID_POLICY,
            0x803B000E => USERMODE_HNS::HCN_E_INVALID_POLICY_TYPE,
            0x803B000F => USERMODE_HNS::HCN_E_INVALID_REMOTE_ENDPOINT_OPERATION,
            0x803B0010 => USERMODE_HNS::HCN_E_NETWORK_ALREADY_EXISTS,
            0x803B0011 => USERMODE_HNS::HCN_E_LAYER_ALREADY_EXISTS,
            0x803B0012 => USERMODE_HNS::HCN_E_POLICY_ALREADY_EXISTS,
            0x803B0013 => USERMODE_HNS::HCN_E_PORT_ALREADY_EXISTS,
            0x803B0014 => USERMODE_HNS::HCN_E_ENDPOINT_ALREADY_ATTACHED,
            0x803B0015 => USERMODE_HNS::HCN_E_REQUEST_UNSUPPORTED,
            0x803B0016 => USERMODE_HNS::HCN_E_MAPPING_NOT_SUPPORTED,
            0x803B0017 => USERMODE_HNS::HCN_E_DEGRADED_OPERATION,
            0x803B0018 => USERMODE_HNS::HCN_E_SHARED_SWITCH_MODIFICATION,
            0x803B0019 => USERMODE_HNS::HCN_E_GUID_CONVERSION_FAILURE,
            0x803B001A => USERMODE_HNS::HCN_E_REGKEY_FAILURE,
            0x803B001B => USERMODE_HNS::HCN_E_INVALID_JSON,
            0x803B001C => USERMODE_HNS::HCN_E_INVALID_JSON_REFERENCE,
            0x803B001D => USERMODE_HNS::HCN_E_ENDPOINT_SHARING_DISABLED,
            0x803B001E => USERMODE_HNS::HCN_E_INVALID_IP,
            0x803B001F => USERMODE_HNS::HCN_E_SWITCH_EXTENSION_NOT_FOUND,
            0x803B0020 => USERMODE_HNS::HCN_E_MANAGER_STOPPED,
            0x803B0021 => USERMODE_HNS::GCN_E_MODULE_NOT_FOUND,
            0x803B0022 => USERMODE_HNS::GCN_E_NO_REQUEST_HANDLERS,
            0x803B0023 => USERMODE_HNS::GCN_E_REQUEST_UNSUPPORTED,
            0x803B0024 => USERMODE_HNS::GCN_E_RUNTIMEKEYS_FAILED,
            0x803B0025 => USERMODE_HNS::GCN_E_NETADAPTER_TIMEOUT,
            0x803B0026 => USERMODE_HNS::GCN_E_NETADAPTER_NOT_FOUND,
            0x803B0027 => USERMODE_HNS::GCN_E_NETCOMPARTMENT_NOT_FOUND,
            0x803B0028 => USERMODE_HNS::GCN_E_NETINTERFACE_NOT_FOUND,
            0x803B0029 => USERMODE_HNS::GCN_E_DEFAULTNAMESPACE_EXISTS,
            0x803B002A => USERMODE_HNS::HCN_E_ICS_DISABLED,
            0x803B002B => USERMODE_HNS::HCN_E_ENDPOINT_NAMESPACE_ALREADY_EXISTS,
            0x803B002C => USERMODE_HNS::HCN_E_ENTITY_HAS_REFERENCES,
            0x803B002D => USERMODE_HNS::HCN_E_INVALID_INTERNAL_PORT,
            0x803B002E => USERMODE_HNS::HCN_E_NAMESPACE_ATTACH_FAILED,
            0x803B002F => USERMODE_HNS::HCN_E_ADDR_INVALID_OR_RESERVED,
            0x803B0030 => USERMODE_HNS::HCN_E_INVALID_PREFIX,
            0x803B0031 => USERMODE_HNS::HCN_E_OBJECT_USED_AFTER_UNLOAD,
            0x803B0032 => USERMODE_HNS::HCN_E_INVALID_SUBNET,
            0x803B0033 => USERMODE_HNS::HCN_E_INVALID_IP_SUBNET,
            0x803B0034 => USERMODE_HNS::HCN_E_ENDPOINT_NOT_ATTACHED,
            0x803B0035 => USERMODE_HNS::HCN_E_ENDPOINT_NOT_LOCAL,
            0x803B0036 => USERMODE_HNS::HCN_INTERFACEPARAMETERS_ALREADY_APPLIED,
            0x803B0037 => USERMODE_HNS::HCN_E_VFP_NOT_ALLOWED,
            0xC03B0001 => USERMODE_HNS::ERROR_HNS_PORT_ALLOCATED,
        }
    }
}
