/*
 * Copyright 2023 Stanislav Mikhailov (xavetar)
 *
 * Licensed under the Creative Commons Zero v1.0 Universal (CC0) License.
 * You may obtain a copy of the License at
 *
 *     http://creativecommons.org/publicdomain/zero/1.0/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the CC0 license is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// It is strictly forbidden to use the from_code method, respected Microsoft decided to define
// duplicate codes, but by defining different constant names, therefore, when using these codes,
// it is mandatory to use through from_name, and not from_code. Otherwise, it may cause undefined
// behavior or an unknown exception. Because one code corresponds to several constants.

use super::{RawError};

#[derive(Clone, Debug, Copy, Eq, Hash, Ord, PartialEq, PartialOrd)]
pub enum USERMODE_HYPERVISOR {
    ERROR_HV_PENDING_PAGE_REQUESTS,
    ERROR_HV_INVALID_HYPERCALL_CODE,
    ERROR_HV_INVALID_HYPERCALL_INPUT,
    ERROR_HV_INVALID_ALIGNMENT,
    ERROR_HV_INVALID_PARAMETER,
    ERROR_HV_ACCESS_DENIED,
    ERROR_HV_INVALID_PARTITION_STATE,
    ERROR_HV_OPERATION_DENIED,
    ERROR_HV_UNKNOWN_PROPERTY,
    ERROR_HV_PROPERTY_VALUE_OUT_OF_RANGE,
    ERROR_HV_INSUFFICIENT_MEMORY,
    ERROR_HV_PARTITION_TOO_DEEP,
    ERROR_HV_INVALID_PARTITION_ID,
    ERROR_HV_INVALID_VP_INDEX,
    ERROR_HV_INVALID_PORT_ID,
    ERROR_HV_INVALID_CONNECTION_ID,
    ERROR_HV_INSUFFICIENT_BUFFERS,
    ERROR_HV_NOT_ACKNOWLEDGED,
    ERROR_HV_INVALID_VP_STATE,
    ERROR_HV_ACKNOWLEDGED,
    ERROR_HV_INVALID_SAVE_RESTORE_STATE,
    ERROR_HV_INVALID_SYNIC_STATE,
    ERROR_HV_OBJECT_IN_USE,
    ERROR_HV_INVALID_PROXIMITY_DOMAIN_INFO,
    ERROR_HV_NO_DATA,
    ERROR_HV_INACTIVE,
    ERROR_HV_NO_RESOURCES,
    ERROR_HV_FEATURE_UNAVAILABLE,
    ERROR_HV_INSUFFICIENT_BUFFER,
    ERROR_HV_INSUFFICIENT_DEVICE_DOMAINS,
    ERROR_HV_CPUID_FEATURE_VALIDATION,
    ERROR_HV_CPUID_XSAVE_FEATURE_VALIDATION,
    ERROR_HV_PROCESSOR_STARTUP_TIMEOUT,
    ERROR_HV_SMX_ENABLED,
    ERROR_HV_INVALID_LP_INDEX,
    ERROR_HV_INVALID_REGISTER_VALUE,
    ERROR_HV_INVALID_VTL_STATE,
    ERROR_HV_NX_NOT_DETECTED,
    ERROR_HV_INVALID_DEVICE_ID,
    ERROR_HV_INVALID_DEVICE_STATE,
    ERROR_HV_PAGE_REQUEST_INVALID,
    ERROR_HV_INVALID_CPU_GROUP_ID,
    ERROR_HV_INVALID_CPU_GROUP_STATE,
    ERROR_HV_OPERATION_FAILED,
    ERROR_HV_NOT_ALLOWED_WITH_NESTED_VIRT_ACTIVE,
    ERROR_HV_INSUFFICIENT_ROOT_MEMORY,
    ERROR_HV_EVENT_BUFFER_ALREADY_FREED,
    ERROR_HV_INSUFFICIENT_CONTIGUOUS_MEMORY,
    ERROR_HV_DEVICE_NOT_IN_DOMAIN,
    ERROR_HV_NESTED_VM_EXIT,
    ERROR_HV_MSR_ACCESS_FAILED,
    ERROR_HV_INSUFFICIENT_MEMORY_MIRRORING,
    ERROR_HV_INSUFFICIENT_CONTIGUOUS_MEMORY_MIRRORING,
    ERROR_HV_INSUFFICIENT_CONTIGUOUS_ROOT_MEMORY,
    ERROR_HV_INSUFFICIENT_ROOT_MEMORY_MIRRORING,
    ERROR_HV_INSUFFICIENT_CONTIGUOUS_ROOT_MEMORY_MIRRORING,
    ERROR_HV_NOT_PRESENT,
}

impl USERMODE_HYPERVISOR {
    pub fn code(&self) -> u32 {
        return match self {
            USERMODE_HYPERVISOR::ERROR_HV_PENDING_PAGE_REQUESTS => 0x00350059 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_HYPERCALL_CODE => 0xC0350002 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_HYPERCALL_INPUT => 0xC0350003 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_ALIGNMENT => 0xC0350004 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_PARAMETER => 0xC0350005 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_ACCESS_DENIED => 0xC0350006 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_PARTITION_STATE => 0xC0350007 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_OPERATION_DENIED => 0xC0350008 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_UNKNOWN_PROPERTY => 0xC0350009 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_PROPERTY_VALUE_OUT_OF_RANGE => 0xC035000A as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_MEMORY => 0xC035000B as u32,
            USERMODE_HYPERVISOR::ERROR_HV_PARTITION_TOO_DEEP => 0xC035000C as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_PARTITION_ID => 0xC035000D as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_VP_INDEX => 0xC035000E as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_PORT_ID => 0xC0350011 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_CONNECTION_ID => 0xC0350012 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_BUFFERS => 0xC0350013 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_NOT_ACKNOWLEDGED => 0xC0350014 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_VP_STATE => 0xC0350015 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_ACKNOWLEDGED => 0xC0350016 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_SAVE_RESTORE_STATE => 0xC0350017 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_SYNIC_STATE => 0xC0350018 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_OBJECT_IN_USE => 0xC0350019 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_PROXIMITY_DOMAIN_INFO => 0xC035001A as u32,
            USERMODE_HYPERVISOR::ERROR_HV_NO_DATA => 0xC035001B as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INACTIVE => 0xC035001C as u32,
            USERMODE_HYPERVISOR::ERROR_HV_NO_RESOURCES => 0xC035001D as u32,
            USERMODE_HYPERVISOR::ERROR_HV_FEATURE_UNAVAILABLE => 0xC035001E as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_BUFFER => 0xC0350033 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_DEVICE_DOMAINS => 0xC0350038 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_CPUID_FEATURE_VALIDATION => 0xC035003C as u32,
            USERMODE_HYPERVISOR::ERROR_HV_CPUID_XSAVE_FEATURE_VALIDATION => 0xC035003D as u32,
            USERMODE_HYPERVISOR::ERROR_HV_PROCESSOR_STARTUP_TIMEOUT => 0xC035003E as u32,
            USERMODE_HYPERVISOR::ERROR_HV_SMX_ENABLED => 0xC035003F as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_LP_INDEX => 0xC0350041 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_REGISTER_VALUE => 0xC0350050 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_VTL_STATE => 0xC0350051 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_NX_NOT_DETECTED => 0xC0350055 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_DEVICE_ID => 0xC0350057 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_DEVICE_STATE => 0xC0350058 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_PAGE_REQUEST_INVALID => 0xC0350060 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_CPU_GROUP_ID => 0xC035006F as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_CPU_GROUP_STATE => 0xC0350070 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_OPERATION_FAILED => 0xC0350071 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_NOT_ALLOWED_WITH_NESTED_VIRT_ACTIVE => 0xC0350072 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_ROOT_MEMORY => 0xC0350073 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_EVENT_BUFFER_ALREADY_FREED => 0xC0350074 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_CONTIGUOUS_MEMORY => 0xC0350075 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_DEVICE_NOT_IN_DOMAIN => 0xC0350076 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_NESTED_VM_EXIT => 0xC0350077 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_MSR_ACCESS_FAILED => 0xC0350080 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_MEMORY_MIRRORING => 0xC0350081 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_CONTIGUOUS_MEMORY_MIRRORING => 0xC0350082 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_CONTIGUOUS_ROOT_MEMORY => 0xC0350083 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_ROOT_MEMORY_MIRRORING => 0xC0350084 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_CONTIGUOUS_ROOT_MEMORY_MIRRORING => 0xC0350085 as u32,
            USERMODE_HYPERVISOR::ERROR_HV_NOT_PRESENT => 0xC0351000 as u32,
        }
    }

    pub fn error(&self) -> RawError {
        return match self {
            USERMODE_HYPERVISOR::ERROR_HV_PENDING_PAGE_REQUESTS => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_PENDING_PAGE_REQUESTS),
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_HYPERCALL_CODE => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INVALID_HYPERCALL_CODE),
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_HYPERCALL_INPUT => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INVALID_HYPERCALL_INPUT),
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_ALIGNMENT => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INVALID_ALIGNMENT),
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_PARAMETER => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INVALID_PARAMETER),
            USERMODE_HYPERVISOR::ERROR_HV_ACCESS_DENIED => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_ACCESS_DENIED),
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_PARTITION_STATE => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INVALID_PARTITION_STATE),
            USERMODE_HYPERVISOR::ERROR_HV_OPERATION_DENIED => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_OPERATION_DENIED),
            USERMODE_HYPERVISOR::ERROR_HV_UNKNOWN_PROPERTY => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_UNKNOWN_PROPERTY),
            USERMODE_HYPERVISOR::ERROR_HV_PROPERTY_VALUE_OUT_OF_RANGE => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_PROPERTY_VALUE_OUT_OF_RANGE),
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_MEMORY => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_MEMORY),
            USERMODE_HYPERVISOR::ERROR_HV_PARTITION_TOO_DEEP => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_PARTITION_TOO_DEEP),
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_PARTITION_ID => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INVALID_PARTITION_ID),
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_VP_INDEX => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INVALID_VP_INDEX),
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_PORT_ID => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INVALID_PORT_ID),
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_CONNECTION_ID => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INVALID_CONNECTION_ID),
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_BUFFERS => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_BUFFERS),
            USERMODE_HYPERVISOR::ERROR_HV_NOT_ACKNOWLEDGED => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_NOT_ACKNOWLEDGED),
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_VP_STATE => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INVALID_VP_STATE),
            USERMODE_HYPERVISOR::ERROR_HV_ACKNOWLEDGED => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_ACKNOWLEDGED),
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_SAVE_RESTORE_STATE => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INVALID_SAVE_RESTORE_STATE),
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_SYNIC_STATE => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INVALID_SYNIC_STATE),
            USERMODE_HYPERVISOR::ERROR_HV_OBJECT_IN_USE => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_OBJECT_IN_USE),
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_PROXIMITY_DOMAIN_INFO => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INVALID_PROXIMITY_DOMAIN_INFO),
            USERMODE_HYPERVISOR::ERROR_HV_NO_DATA => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_NO_DATA),
            USERMODE_HYPERVISOR::ERROR_HV_INACTIVE => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INACTIVE),
            USERMODE_HYPERVISOR::ERROR_HV_NO_RESOURCES => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_NO_RESOURCES),
            USERMODE_HYPERVISOR::ERROR_HV_FEATURE_UNAVAILABLE => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_FEATURE_UNAVAILABLE),
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_BUFFER => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_BUFFER),
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_DEVICE_DOMAINS => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_DEVICE_DOMAINS),
            USERMODE_HYPERVISOR::ERROR_HV_CPUID_FEATURE_VALIDATION => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_CPUID_FEATURE_VALIDATION),
            USERMODE_HYPERVISOR::ERROR_HV_CPUID_XSAVE_FEATURE_VALIDATION => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_CPUID_XSAVE_FEATURE_VALIDATION),
            USERMODE_HYPERVISOR::ERROR_HV_PROCESSOR_STARTUP_TIMEOUT => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_PROCESSOR_STARTUP_TIMEOUT),
            USERMODE_HYPERVISOR::ERROR_HV_SMX_ENABLED => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_SMX_ENABLED),
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_LP_INDEX => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INVALID_LP_INDEX),
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_REGISTER_VALUE => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INVALID_REGISTER_VALUE),
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_VTL_STATE => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INVALID_VTL_STATE),
            USERMODE_HYPERVISOR::ERROR_HV_NX_NOT_DETECTED => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_NX_NOT_DETECTED),
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_DEVICE_ID => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INVALID_DEVICE_ID),
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_DEVICE_STATE => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INVALID_DEVICE_STATE),
            USERMODE_HYPERVISOR::ERROR_HV_PAGE_REQUEST_INVALID => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_PAGE_REQUEST_INVALID),
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_CPU_GROUP_ID => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INVALID_CPU_GROUP_ID),
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_CPU_GROUP_STATE => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INVALID_CPU_GROUP_STATE),
            USERMODE_HYPERVISOR::ERROR_HV_OPERATION_FAILED => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_OPERATION_FAILED),
            USERMODE_HYPERVISOR::ERROR_HV_NOT_ALLOWED_WITH_NESTED_VIRT_ACTIVE => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_NOT_ALLOWED_WITH_NESTED_VIRT_ACTIVE),
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_ROOT_MEMORY => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_ROOT_MEMORY),
            USERMODE_HYPERVISOR::ERROR_HV_EVENT_BUFFER_ALREADY_FREED => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_EVENT_BUFFER_ALREADY_FREED),
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_CONTIGUOUS_MEMORY => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_CONTIGUOUS_MEMORY),
            USERMODE_HYPERVISOR::ERROR_HV_DEVICE_NOT_IN_DOMAIN => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_DEVICE_NOT_IN_DOMAIN),
            USERMODE_HYPERVISOR::ERROR_HV_NESTED_VM_EXIT => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_NESTED_VM_EXIT),
            USERMODE_HYPERVISOR::ERROR_HV_MSR_ACCESS_FAILED => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_MSR_ACCESS_FAILED),
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_MEMORY_MIRRORING => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_MEMORY_MIRRORING),
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_CONTIGUOUS_MEMORY_MIRRORING => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_CONTIGUOUS_MEMORY_MIRRORING),
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_CONTIGUOUS_ROOT_MEMORY => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_CONTIGUOUS_ROOT_MEMORY),
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_ROOT_MEMORY_MIRRORING => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_ROOT_MEMORY_MIRRORING),
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_CONTIGUOUS_ROOT_MEMORY_MIRRORING => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_CONTIGUOUS_ROOT_MEMORY_MIRRORING),
            USERMODE_HYPERVISOR::ERROR_HV_NOT_PRESENT => RawError::Kind(USERMODE_HYPERVISOR::ERROR_HV_NOT_PRESENT),
        }
    }

    pub fn description(&self) -> &'static str {
        return match self {
            USERMODE_HYPERVISOR::ERROR_HV_PENDING_PAGE_REQUESTS => "The device had pending page requests which were discarded.",
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_HYPERCALL_CODE => "The hypervisor does not support the operation because the specified hypercall code is not supported.",
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_HYPERCALL_INPUT => "The hypervisor does not support the operation because the encoding for the hypercall input register is not supported.",
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_ALIGNMENT => "The hypervisor could not perform the operation because a parameter has an invalid alignment.",
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_PARAMETER => "The hypervisor could not perform the operation because an invalid parameter was specified.",
            USERMODE_HYPERVISOR::ERROR_HV_ACCESS_DENIED => "Access to the specified object was denied.",
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_PARTITION_STATE => "The hypervisor could not perform the operation because the partition is entering or in an invalid state.",
            USERMODE_HYPERVISOR::ERROR_HV_OPERATION_DENIED => "The operation is not allowed in the current state.",
            USERMODE_HYPERVISOR::ERROR_HV_UNKNOWN_PROPERTY => "The hypervisor does not recognize the specified partition property.",
            USERMODE_HYPERVISOR::ERROR_HV_PROPERTY_VALUE_OUT_OF_RANGE => "The specified value of a partition property is out of range or violates an invariant.",
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_MEMORY => "There is not enough memory in the hypervisor pool to complete the operation.",
            USERMODE_HYPERVISOR::ERROR_HV_PARTITION_TOO_DEEP => "The maximum partition depth has been exceeded for the partition hierarchy.",
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_PARTITION_ID => "A partition with the specified partition Id does not exist.",
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_VP_INDEX => "The hypervisor could not perform the operation because the specified VP index is invalid.",
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_PORT_ID => "The hypervisor could not perform the operation because the specified port identifier is invalid.",
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_CONNECTION_ID => "The hypervisor could not perform the operation because the specified connection identifier is invalid.",
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_BUFFERS => "Not enough buffers were supplied to send a message.",
            USERMODE_HYPERVISOR::ERROR_HV_NOT_ACKNOWLEDGED => "The previous virtual interrupt has not been acknowledged.",
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_VP_STATE => "A virtual processor is not in the correct state for the indicated operation.",
            USERMODE_HYPERVISOR::ERROR_HV_ACKNOWLEDGED => "The previous virtual interrupt has already been acknowledged.",
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_SAVE_RESTORE_STATE => "The indicated partition is not in a valid state for saving or restoring.",
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_SYNIC_STATE => "The hypervisor could not complete the operation because a required feature of the synthetic interrupt controller (SynIC) was disabled.",
            USERMODE_HYPERVISOR::ERROR_HV_OBJECT_IN_USE => "The hypervisor could not perform the operation because the object or value was either already in use or being used for a purpose that would not permit completing the operation.",
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_PROXIMITY_DOMAIN_INFO => "The proximity domain information is invalid.",
            USERMODE_HYPERVISOR::ERROR_HV_NO_DATA => "An attempt to retrieve debugging data failed because none was available.",
            USERMODE_HYPERVISOR::ERROR_HV_INACTIVE => "The physical connection being used for debugging has not recorded any receive activity since the last operation.",
            USERMODE_HYPERVISOR::ERROR_HV_NO_RESOURCES => "There are not enough resources to complete the operation.",
            USERMODE_HYPERVISOR::ERROR_HV_FEATURE_UNAVAILABLE => "A hypervisor feature is not available to the user.",
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_BUFFER => "The specified buffer was too small to contain all of the requested data.",
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_DEVICE_DOMAINS => "The maximum number of domains supported by the platform I/O remapping hardware is currently in use. No domains are available to assign this device to this partition.",
            USERMODE_HYPERVISOR::ERROR_HV_CPUID_FEATURE_VALIDATION => "Validation of CPUID data of the processor failed.",
            USERMODE_HYPERVISOR::ERROR_HV_CPUID_XSAVE_FEATURE_VALIDATION => "Validation of XSAVE CPUID data of the processor failed.",
            USERMODE_HYPERVISOR::ERROR_HV_PROCESSOR_STARTUP_TIMEOUT => "Processor did not respond within the timeout period.",
            USERMODE_HYPERVISOR::ERROR_HV_SMX_ENABLED => "SMX has been enabled in the BIOS.",
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_LP_INDEX => "The hypervisor could not perform the operation because the specified LP index is invalid.",
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_REGISTER_VALUE => "The supplied register value is invalid.",
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_VTL_STATE => "The supplied virtual trust level is not in the correct state to perform the requested operation.",
            USERMODE_HYPERVISOR::ERROR_HV_NX_NOT_DETECTED => "No execute feature (NX) is not present or not enabled in the BIOS.",
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_DEVICE_ID => "The supplied device ID is invalid.",
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_DEVICE_STATE => "The operation is not allowed in the current device state.",
            USERMODE_HYPERVISOR::ERROR_HV_PAGE_REQUEST_INVALID => "The supplied page request specifies a memory access that the guest does not have permissions to perform.",
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_CPU_GROUP_ID => "A CPU group with the specified CPU group Id does not exist.",
            USERMODE_HYPERVISOR::ERROR_HV_INVALID_CPU_GROUP_STATE => "The hypervisor could not perform the operation because the CPU group is entering or in an invalid state.",
            USERMODE_HYPERVISOR::ERROR_HV_OPERATION_FAILED => "The requested operation failed.",
            USERMODE_HYPERVISOR::ERROR_HV_NOT_ALLOWED_WITH_NESTED_VIRT_ACTIVE => "The hypervisor could not perform the operation because it is not allowed with nested virtualization active.",
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_ROOT_MEMORY => "There is not enough memory in the root partition's pool to complete the operation.",
            USERMODE_HYPERVISOR::ERROR_HV_EVENT_BUFFER_ALREADY_FREED => "The provided event log buffer was already marked as freed.",
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_CONTIGUOUS_MEMORY => "There is not enough contiguous memory in the partition's pool to complete the operation.",
            USERMODE_HYPERVISOR::ERROR_HV_DEVICE_NOT_IN_DOMAIN => "The device is not in a device domain.",
            USERMODE_HYPERVISOR::ERROR_HV_NESTED_VM_EXIT => "The requested operation would result in a nested vm-exit.",
            USERMODE_HYPERVISOR::ERROR_HV_MSR_ACCESS_FAILED => "The requested access to the model specific register failed.",
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_MEMORY_MIRRORING => "There is not enough memory in the hypervisor pool to complete the mirroring operation.",
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_CONTIGUOUS_MEMORY_MIRRORING => "There is not enough contiguous memory in the hypervisor pool to complete the mirroring operation.",
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_CONTIGUOUS_ROOT_MEMORY => "There is not enough contiguous memory in the root partition's pool to complete the operation.",
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_ROOT_MEMORY_MIRRORING => "There is not enough memory in the root partition's pool to complete the mirroring operation.",
            USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_CONTIGUOUS_ROOT_MEMORY_MIRRORING => "There is not enough contiguous memory in the root partition's pool to complete the mirroring operation.",
            USERMODE_HYPERVISOR::ERROR_HV_NOT_PRESENT => "No hypervisor is present on this system.",
        }
    }

    pub fn from_name(name: &str) -> USERMODE_HYPERVISOR {
        return match name {
            "ERROR_HV_PENDING_PAGE_REQUESTS" => USERMODE_HYPERVISOR::ERROR_HV_PENDING_PAGE_REQUESTS,
            "ERROR_HV_INVALID_HYPERCALL_CODE" => USERMODE_HYPERVISOR::ERROR_HV_INVALID_HYPERCALL_CODE,
            "ERROR_HV_INVALID_HYPERCALL_INPUT" => USERMODE_HYPERVISOR::ERROR_HV_INVALID_HYPERCALL_INPUT,
            "ERROR_HV_INVALID_ALIGNMENT" => USERMODE_HYPERVISOR::ERROR_HV_INVALID_ALIGNMENT,
            "ERROR_HV_INVALID_PARAMETER" => USERMODE_HYPERVISOR::ERROR_HV_INVALID_PARAMETER,
            "ERROR_HV_ACCESS_DENIED" => USERMODE_HYPERVISOR::ERROR_HV_ACCESS_DENIED,
            "ERROR_HV_INVALID_PARTITION_STATE" => USERMODE_HYPERVISOR::ERROR_HV_INVALID_PARTITION_STATE,
            "ERROR_HV_OPERATION_DENIED" => USERMODE_HYPERVISOR::ERROR_HV_OPERATION_DENIED,
            "ERROR_HV_UNKNOWN_PROPERTY" => USERMODE_HYPERVISOR::ERROR_HV_UNKNOWN_PROPERTY,
            "ERROR_HV_PROPERTY_VALUE_OUT_OF_RANGE" => USERMODE_HYPERVISOR::ERROR_HV_PROPERTY_VALUE_OUT_OF_RANGE,
            "ERROR_HV_INSUFFICIENT_MEMORY" => USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_MEMORY,
            "ERROR_HV_PARTITION_TOO_DEEP" => USERMODE_HYPERVISOR::ERROR_HV_PARTITION_TOO_DEEP,
            "ERROR_HV_INVALID_PARTITION_ID" => USERMODE_HYPERVISOR::ERROR_HV_INVALID_PARTITION_ID,
            "ERROR_HV_INVALID_VP_INDEX" => USERMODE_HYPERVISOR::ERROR_HV_INVALID_VP_INDEX,
            "ERROR_HV_INVALID_PORT_ID" => USERMODE_HYPERVISOR::ERROR_HV_INVALID_PORT_ID,
            "ERROR_HV_INVALID_CONNECTION_ID" => USERMODE_HYPERVISOR::ERROR_HV_INVALID_CONNECTION_ID,
            "ERROR_HV_INSUFFICIENT_BUFFERS" => USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_BUFFERS,
            "ERROR_HV_NOT_ACKNOWLEDGED" => USERMODE_HYPERVISOR::ERROR_HV_NOT_ACKNOWLEDGED,
            "ERROR_HV_INVALID_VP_STATE" => USERMODE_HYPERVISOR::ERROR_HV_INVALID_VP_STATE,
            "ERROR_HV_ACKNOWLEDGED" => USERMODE_HYPERVISOR::ERROR_HV_ACKNOWLEDGED,
            "ERROR_HV_INVALID_SAVE_RESTORE_STATE" => USERMODE_HYPERVISOR::ERROR_HV_INVALID_SAVE_RESTORE_STATE,
            "ERROR_HV_INVALID_SYNIC_STATE" => USERMODE_HYPERVISOR::ERROR_HV_INVALID_SYNIC_STATE,
            "ERROR_HV_OBJECT_IN_USE" => USERMODE_HYPERVISOR::ERROR_HV_OBJECT_IN_USE,
            "ERROR_HV_INVALID_PROXIMITY_DOMAIN_INFO" => USERMODE_HYPERVISOR::ERROR_HV_INVALID_PROXIMITY_DOMAIN_INFO,
            "ERROR_HV_NO_DATA" => USERMODE_HYPERVISOR::ERROR_HV_NO_DATA,
            "ERROR_HV_INACTIVE" => USERMODE_HYPERVISOR::ERROR_HV_INACTIVE,
            "ERROR_HV_NO_RESOURCES" => USERMODE_HYPERVISOR::ERROR_HV_NO_RESOURCES,
            "ERROR_HV_FEATURE_UNAVAILABLE" => USERMODE_HYPERVISOR::ERROR_HV_FEATURE_UNAVAILABLE,
            "ERROR_HV_INSUFFICIENT_BUFFER" => USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_BUFFER,
            "ERROR_HV_INSUFFICIENT_DEVICE_DOMAINS" => USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_DEVICE_DOMAINS,
            "ERROR_HV_CPUID_FEATURE_VALIDATION" => USERMODE_HYPERVISOR::ERROR_HV_CPUID_FEATURE_VALIDATION,
            "ERROR_HV_CPUID_XSAVE_FEATURE_VALIDATION" => USERMODE_HYPERVISOR::ERROR_HV_CPUID_XSAVE_FEATURE_VALIDATION,
            "ERROR_HV_PROCESSOR_STARTUP_TIMEOUT" => USERMODE_HYPERVISOR::ERROR_HV_PROCESSOR_STARTUP_TIMEOUT,
            "ERROR_HV_SMX_ENABLED" => USERMODE_HYPERVISOR::ERROR_HV_SMX_ENABLED,
            "ERROR_HV_INVALID_LP_INDEX" => USERMODE_HYPERVISOR::ERROR_HV_INVALID_LP_INDEX,
            "ERROR_HV_INVALID_REGISTER_VALUE" => USERMODE_HYPERVISOR::ERROR_HV_INVALID_REGISTER_VALUE,
            "ERROR_HV_INVALID_VTL_STATE" => USERMODE_HYPERVISOR::ERROR_HV_INVALID_VTL_STATE,
            "ERROR_HV_NX_NOT_DETECTED" => USERMODE_HYPERVISOR::ERROR_HV_NX_NOT_DETECTED,
            "ERROR_HV_INVALID_DEVICE_ID" => USERMODE_HYPERVISOR::ERROR_HV_INVALID_DEVICE_ID,
            "ERROR_HV_INVALID_DEVICE_STATE" => USERMODE_HYPERVISOR::ERROR_HV_INVALID_DEVICE_STATE,
            "ERROR_HV_PAGE_REQUEST_INVALID" => USERMODE_HYPERVISOR::ERROR_HV_PAGE_REQUEST_INVALID,
            "ERROR_HV_INVALID_CPU_GROUP_ID" => USERMODE_HYPERVISOR::ERROR_HV_INVALID_CPU_GROUP_ID,
            "ERROR_HV_INVALID_CPU_GROUP_STATE" => USERMODE_HYPERVISOR::ERROR_HV_INVALID_CPU_GROUP_STATE,
            "ERROR_HV_OPERATION_FAILED" => USERMODE_HYPERVISOR::ERROR_HV_OPERATION_FAILED,
            "ERROR_HV_NOT_ALLOWED_WITH_NESTED_VIRT_ACTIVE" => USERMODE_HYPERVISOR::ERROR_HV_NOT_ALLOWED_WITH_NESTED_VIRT_ACTIVE,
            "ERROR_HV_INSUFFICIENT_ROOT_MEMORY" => USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_ROOT_MEMORY,
            "ERROR_HV_EVENT_BUFFER_ALREADY_FREED" => USERMODE_HYPERVISOR::ERROR_HV_EVENT_BUFFER_ALREADY_FREED,
            "ERROR_HV_INSUFFICIENT_CONTIGUOUS_MEMORY" => USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_CONTIGUOUS_MEMORY,
            "ERROR_HV_DEVICE_NOT_IN_DOMAIN" => USERMODE_HYPERVISOR::ERROR_HV_DEVICE_NOT_IN_DOMAIN,
            "ERROR_HV_NESTED_VM_EXIT" => USERMODE_HYPERVISOR::ERROR_HV_NESTED_VM_EXIT,
            "ERROR_HV_MSR_ACCESS_FAILED" => USERMODE_HYPERVISOR::ERROR_HV_MSR_ACCESS_FAILED,
            "ERROR_HV_INSUFFICIENT_MEMORY_MIRRORING" => USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_MEMORY_MIRRORING,
            "ERROR_HV_INSUFFICIENT_CONTIGUOUS_MEMORY_MIRRORING" => USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_CONTIGUOUS_MEMORY_MIRRORING,
            "ERROR_HV_INSUFFICIENT_CONTIGUOUS_ROOT_MEMORY" => USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_CONTIGUOUS_ROOT_MEMORY,
            "ERROR_HV_INSUFFICIENT_ROOT_MEMORY_MIRRORING" => USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_ROOT_MEMORY_MIRRORING,
            "ERROR_HV_INSUFFICIENT_CONTIGUOUS_ROOT_MEMORY_MIRRORING" => USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_CONTIGUOUS_ROOT_MEMORY_MIRRORING,
            "ERROR_HV_NOT_PRESENT" => USERMODE_HYPERVISOR::ERROR_HV_NOT_PRESENT,
        }
    }
    pub fn from_code(code: u32) -> USERMODE_HYPERVISOR {
        return match code {
            0x00350059 => USERMODE_HYPERVISOR::ERROR_HV_PENDING_PAGE_REQUESTS,
            0xC0350002 => USERMODE_HYPERVISOR::ERROR_HV_INVALID_HYPERCALL_CODE,
            0xC0350003 => USERMODE_HYPERVISOR::ERROR_HV_INVALID_HYPERCALL_INPUT,
            0xC0350004 => USERMODE_HYPERVISOR::ERROR_HV_INVALID_ALIGNMENT,
            0xC0350005 => USERMODE_HYPERVISOR::ERROR_HV_INVALID_PARAMETER,
            0xC0350006 => USERMODE_HYPERVISOR::ERROR_HV_ACCESS_DENIED,
            0xC0350007 => USERMODE_HYPERVISOR::ERROR_HV_INVALID_PARTITION_STATE,
            0xC0350008 => USERMODE_HYPERVISOR::ERROR_HV_OPERATION_DENIED,
            0xC0350009 => USERMODE_HYPERVISOR::ERROR_HV_UNKNOWN_PROPERTY,
            0xC035000A => USERMODE_HYPERVISOR::ERROR_HV_PROPERTY_VALUE_OUT_OF_RANGE,
            0xC035000B => USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_MEMORY,
            0xC035000C => USERMODE_HYPERVISOR::ERROR_HV_PARTITION_TOO_DEEP,
            0xC035000D => USERMODE_HYPERVISOR::ERROR_HV_INVALID_PARTITION_ID,
            0xC035000E => USERMODE_HYPERVISOR::ERROR_HV_INVALID_VP_INDEX,
            0xC0350011 => USERMODE_HYPERVISOR::ERROR_HV_INVALID_PORT_ID,
            0xC0350012 => USERMODE_HYPERVISOR::ERROR_HV_INVALID_CONNECTION_ID,
            0xC0350013 => USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_BUFFERS,
            0xC0350014 => USERMODE_HYPERVISOR::ERROR_HV_NOT_ACKNOWLEDGED,
            0xC0350015 => USERMODE_HYPERVISOR::ERROR_HV_INVALID_VP_STATE,
            0xC0350016 => USERMODE_HYPERVISOR::ERROR_HV_ACKNOWLEDGED,
            0xC0350017 => USERMODE_HYPERVISOR::ERROR_HV_INVALID_SAVE_RESTORE_STATE,
            0xC0350018 => USERMODE_HYPERVISOR::ERROR_HV_INVALID_SYNIC_STATE,
            0xC0350019 => USERMODE_HYPERVISOR::ERROR_HV_OBJECT_IN_USE,
            0xC035001A => USERMODE_HYPERVISOR::ERROR_HV_INVALID_PROXIMITY_DOMAIN_INFO,
            0xC035001B => USERMODE_HYPERVISOR::ERROR_HV_NO_DATA,
            0xC035001C => USERMODE_HYPERVISOR::ERROR_HV_INACTIVE,
            0xC035001D => USERMODE_HYPERVISOR::ERROR_HV_NO_RESOURCES,
            0xC035001E => USERMODE_HYPERVISOR::ERROR_HV_FEATURE_UNAVAILABLE,
            0xC0350033 => USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_BUFFER,
            0xC0350038 => USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_DEVICE_DOMAINS,
            0xC035003C => USERMODE_HYPERVISOR::ERROR_HV_CPUID_FEATURE_VALIDATION,
            0xC035003D => USERMODE_HYPERVISOR::ERROR_HV_CPUID_XSAVE_FEATURE_VALIDATION,
            0xC035003E => USERMODE_HYPERVISOR::ERROR_HV_PROCESSOR_STARTUP_TIMEOUT,
            0xC035003F => USERMODE_HYPERVISOR::ERROR_HV_SMX_ENABLED,
            0xC0350041 => USERMODE_HYPERVISOR::ERROR_HV_INVALID_LP_INDEX,
            0xC0350050 => USERMODE_HYPERVISOR::ERROR_HV_INVALID_REGISTER_VALUE,
            0xC0350051 => USERMODE_HYPERVISOR::ERROR_HV_INVALID_VTL_STATE,
            0xC0350055 => USERMODE_HYPERVISOR::ERROR_HV_NX_NOT_DETECTED,
            0xC0350057 => USERMODE_HYPERVISOR::ERROR_HV_INVALID_DEVICE_ID,
            0xC0350058 => USERMODE_HYPERVISOR::ERROR_HV_INVALID_DEVICE_STATE,
            0xC0350060 => USERMODE_HYPERVISOR::ERROR_HV_PAGE_REQUEST_INVALID,
            0xC035006F => USERMODE_HYPERVISOR::ERROR_HV_INVALID_CPU_GROUP_ID,
            0xC0350070 => USERMODE_HYPERVISOR::ERROR_HV_INVALID_CPU_GROUP_STATE,
            0xC0350071 => USERMODE_HYPERVISOR::ERROR_HV_OPERATION_FAILED,
            0xC0350072 => USERMODE_HYPERVISOR::ERROR_HV_NOT_ALLOWED_WITH_NESTED_VIRT_ACTIVE,
            0xC0350073 => USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_ROOT_MEMORY,
            0xC0350074 => USERMODE_HYPERVISOR::ERROR_HV_EVENT_BUFFER_ALREADY_FREED,
            0xC0350075 => USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_CONTIGUOUS_MEMORY,
            0xC0350076 => USERMODE_HYPERVISOR::ERROR_HV_DEVICE_NOT_IN_DOMAIN,
            0xC0350077 => USERMODE_HYPERVISOR::ERROR_HV_NESTED_VM_EXIT,
            0xC0350080 => USERMODE_HYPERVISOR::ERROR_HV_MSR_ACCESS_FAILED,
            0xC0350081 => USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_MEMORY_MIRRORING,
            0xC0350082 => USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_CONTIGUOUS_MEMORY_MIRRORING,
            0xC0350083 => USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_CONTIGUOUS_ROOT_MEMORY,
            0xC0350084 => USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_ROOT_MEMORY_MIRRORING,
            0xC0350085 => USERMODE_HYPERVISOR::ERROR_HV_INSUFFICIENT_CONTIGUOUS_ROOT_MEMORY_MIRRORING,
            0xC0351000 => USERMODE_HYPERVISOR::ERROR_HV_NOT_PRESENT,
        }
    }
}
