/*
 * Copyright 2023 Stanislav Mikhailov (xavetar)
 *
 * Licensed under the Creative Commons Zero v1.0 Universal (CC0) License.
 * You may obtain a copy of the License at
 *
 *     http://creativecommons.org/publicdomain/zero/1.0/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the CC0 license is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// It is strictly forbidden to use the from_code method, respected Microsoft decided to define
// duplicate codes, but by defining different constant names, therefore, when using these codes,
// it is mandatory to use through from_name, and not from_code. Otherwise, it may cause undefined
// behavior or an unknown exception. Because one code corresponds to several constants.

use super::{RawError};

#[derive(Clone, Debug, Copy, Eq, Hash, Ord, PartialEq, PartialOrd)]
pub enum WINDOWSUPDATE {
    WU_S_SERVICE_STOP,
    WU_S_SELFUPDATE,
    WU_S_UPDATE_ERROR,
    WU_S_MARKED_FOR_DISCONNECT,
    WU_S_REBOOT_REQUIRED,
    WU_S_ALREADY_INSTALLED,
    WU_S_ALREADY_UNINSTALLED,
    WU_S_ALREADY_DOWNLOADED,
    WU_E_NO_SERVICE,
    WU_E_MAX_CAPACITY_REACHED,
    WU_E_UNKNOWN_ID,
    WU_E_NOT_INITIALIZED,
    WU_E_RANGEOVERLAP,
    WU_E_TOOMANYRANGES,
    WU_E_INVALIDINDEX,
    WU_E_ITEMNOTFOUND,
    WU_E_OPERATIONINPROGRESS,
    WU_E_COULDNOTCANCEL,
    WU_E_CALL_CANCELLED,
    WU_E_NOOP,
    WU_E_XML_MISSINGDATA,
    WU_E_XML_INVALID,
    WU_E_CYCLE_DETECTED,
    WU_E_TOO_DEEP_RELATION,
    WU_E_INVALID_RELATIONSHIP,
    WU_E_REG_VALUE_INVALID,
    WU_E_DUPLICATE_ITEM,
    WU_E_INSTALL_NOT_ALLOWED,
    WU_E_NOT_APPLICABLE,
    WU_E_NO_USERTOKEN,
    WU_E_EXCLUSIVE_INSTALL_CONFLICT,
    WU_E_POLICY_NOT_SET,
    WU_E_SELFUPDATE_IN_PROGRESS,
    WU_E_INVALID_UPDATE,
    WU_E_SERVICE_STOP,
    WU_E_NO_CONNECTION,
    WU_E_NO_INTERACTIVE_USER,
    WU_E_TIME_OUT,
    WU_E_ALL_UPDATES_FAILED,
    WU_E_EULAS_DECLINED,
    WU_E_NO_UPDATE,
    WU_E_USER_ACCESS_DISABLED,
    WU_E_INVALID_UPDATE_TYPE,
    WU_E_URL_TOO_LONG,
    WU_E_UNINSTALL_NOT_ALLOWED,
    WU_E_INVALID_PRODUCT_LICENSE,
    WU_E_MISSING_HANDLER,
    WU_E_LEGACYSERVER,
    WU_E_BIN_SOURCE_ABSENT,
    WU_E_SOURCE_ABSENT,
    WU_E_WU_DISABLED,
    WU_E_CALL_CANCELLED_BY_POLICY,
    WU_E_INVALID_PROXY_SERVER,
    WU_E_INVALID_FILE,
    WU_E_INVALID_CRITERIA,
    WU_E_EULA_UNAVAILABLE,
    WU_E_DOWNLOAD_FAILED,
    WU_E_UPDATE_NOT_PROCESSED,
    WU_E_INVALID_OPERATION,
    WU_E_NOT_SUPPORTED,
    WU_E_WINHTTP_INVALID_FILE,
    WU_E_TOO_MANY_RESYNC,
    WU_E_NO_SERVER_CORE_SUPPORT,
    WU_E_SYSPREP_IN_PROGRESS,
    WU_E_UNKNOWN_SERVICE,
    WU_E_UNEXPECTED,
    WU_E_MSI_WRONG_VERSION,
    WU_E_MSI_NOT_CONFIGURED,
    WU_E_MSP_DISABLED,
    WU_E_MSI_WRONG_APP_CONTEXT,
    WU_E_MSP_UNEXPECTED,
    WU_E_UH_REMOTEUNAVAILABLE,
    WU_E_UH_LOCALONLY,
    WU_E_UH_UNKNOWNHANDLER,
    WU_E_UH_REMOTEALREADYACTIVE,
    WU_E_UH_DOESNOTSUPPORTACTION,
    WU_E_UH_WRONGHANDLER,
    WU_E_UH_INVALIDMETADATA,
    WU_E_UH_INSTALLERHUNG,
    WU_E_UH_OPERATIONCANCELLED,
    WU_E_UH_BADHANDLERXML,
    WU_E_UH_CANREQUIREINPUT,
    WU_E_UH_INSTALLERFAILURE,
    WU_E_UH_FALLBACKTOSELFCONTAINED,
    WU_E_UH_NEEDANOTHERDOWNLOAD,
    WU_E_UH_NOTIFYFAILURE,
    WU_E_UH_INCONSISTENT_FILE_NAMES,
    WU_E_UH_FALLBACKERROR,
    WU_E_UH_TOOMANYDOWNLOADREQUESTS,
    WU_E_UH_UNEXPECTEDCBSRESPONSE,
    WU_E_UH_BADCBSPACKAGEID,
    WU_E_UH_POSTREBOOTSTILLPENDING,
    WU_E_UH_POSTREBOOTRESULTUNKNOWN,
    WU_E_UH_POSTREBOOTUNEXPECTEDSTATE,
    WU_E_UH_NEW_SERVICING_STACK_REQUIRED,
    WU_E_UH_UNEXPECTED,
    WU_E_INSTALLATION_RESULTS_UNKNOWN_VERSION,
    WU_E_INSTALLATION_RESULTS_INVALID_DATA,
    WU_E_INSTALLATION_RESULTS_NOT_FOUND,
    WU_E_TRAYICON_FAILURE,
    WU_E_NON_UI_MODE,
    WU_E_WUCLTUI_UNSUPPORTED_VERSION,
    WU_E_AUCLIENT_UNEXPECTED,
    WU_E_PT_SOAPCLIENT_BASE,
    WU_E_PT_SOAPCLIENT_INITIALIZE,
    WU_E_PT_SOAPCLIENT_OUTOFMEMORY,
    WU_E_PT_SOAPCLIENT_GENERATE,
    WU_E_PT_SOAPCLIENT_CONNECT,
    WU_E_PT_SOAPCLIENT_SEND,
    WU_E_PT_SOAPCLIENT_SERVER,
    WU_E_PT_SOAPCLIENT_SOAPFAULT,
    WU_E_PT_SOAPCLIENT_PARSEFAULT,
    WU_E_PT_SOAPCLIENT_READ,
    WU_E_PT_SOAPCLIENT_PARSE,
    WU_E_PT_SOAP_VERSION,
    WU_E_PT_SOAP_MUST_UNDERSTAND,
    WU_E_PT_SOAP_CLIENT,
    WU_E_PT_SOAP_SERVER,
    WU_E_PT_WMI_ERROR,
    WU_E_PT_EXCEEDED_MAX_SERVER_TRIPS,
    WU_E_PT_SUS_SERVER_NOT_SET,
    WU_E_PT_DOUBLE_INITIALIZATION,
    WU_E_PT_INVALID_COMPUTER_NAME,
    WU_E_PT_REFRESH_CACHE_REQUIRED,
    WU_E_PT_HTTP_STATUS_BAD_REQUEST,
    WU_E_PT_HTTP_STATUS_DENIED,
    WU_E_PT_HTTP_STATUS_FORBIDDEN,
    WU_E_PT_HTTP_STATUS_NOT_FOUND,
    WU_E_PT_HTTP_STATUS_BAD_METHOD,
    WU_E_PT_HTTP_STATUS_PROXY_AUTH_REQ,
    WU_E_PT_HTTP_STATUS_REQUEST_TIMEOUT,
    WU_E_PT_HTTP_STATUS_CONFLICT,
    WU_E_PT_HTTP_STATUS_GONE,
    WU_E_PT_HTTP_STATUS_SERVER_ERROR,
    WU_E_PT_HTTP_STATUS_NOT_SUPPORTED,
    WU_E_PT_HTTP_STATUS_BAD_GATEWAY,
    WU_E_PT_HTTP_STATUS_SERVICE_UNAVAIL,
    WU_E_PT_HTTP_STATUS_GATEWAY_TIMEOUT,
    WU_E_PT_HTTP_STATUS_VERSION_NOT_SUP,
    WU_E_PT_FILE_LOCATIONS_CHANGED,
    WU_E_PT_REGISTRATION_NOT_SUPPORTED,
    WU_E_PT_NO_AUTH_PLUGINS_REQUESTED,
    WU_E_PT_NO_AUTH_COOKIES_CREATED,
    WU_E_PT_INVALID_CONFIG_PROP,
    WU_E_PT_CONFIG_PROP_MISSING,
    WU_E_PT_HTTP_STATUS_NOT_MAPPED,
    WU_E_PT_WINHTTP_NAME_NOT_RESOLVED,
    WU_E_PT_ECP_SUCCEEDED_WITH_ERRORS,
    WU_E_PT_ECP_INIT_FAILED,
    WU_E_PT_ECP_INVALID_FILE_FORMAT,
    WU_E_PT_ECP_INVALID_METADATA,
    WU_E_PT_ECP_FAILURE_TO_EXTRACT_DIGEST,
    WU_E_PT_ECP_FAILURE_TO_DECOMPRESS_CAB_FILE,
    WU_E_PT_ECP_FILE_LOCATION_ERROR,
    WU_E_PT_UNEXPECTED,
    WU_E_REDIRECTOR_LOAD_XML,
    WU_E_REDIRECTOR_S_FALSE,
    WU_E_REDIRECTOR_ID_SMALLER,
    WU_E_PT_SAME_REDIR_ID,
    WU_E_PT_NO_MANAGED_RECOVER,
    WU_E_REDIRECTOR_UNEXPECTED,
    WU_E_DM_URLNOTAVAILABLE,
    WU_E_DM_INCORRECTFILEHASH,
    WU_E_DM_UNKNOWNALGORITHM,
    WU_E_DM_NEEDDOWNLOADREQUEST,
    WU_E_DM_NONETWORK,
    WU_E_DM_WRONGBITSVERSION,
    WU_E_DM_NOTDOWNLOADED,
    WU_E_DM_FAILTOCONNECTTOBITS,
    WU_E_DM_BITSTRANSFERERROR,
    WU_E_DM_DOWNLOADLOCATIONCHANGED,
    WU_E_DM_CONTENTCHANGED,
    WU_E_DM_UNEXPECTED,
    WU_E_OL_INVALID_SCANFILE,
    WU_E_OL_NEWCLIENT_REQUIRED,
    WU_E_OL_UNEXPECTED,
    WU_E_DS_SHUTDOWN,
    WU_E_DS_INUSE,
    WU_E_DS_INVALID,
    WU_E_DS_TABLEMISSING,
    WU_E_DS_TABLEINCORRECT,
    WU_E_DS_INVALIDTABLENAME,
    WU_E_DS_BADVERSION,
    WU_E_DS_NODATA,
    WU_E_DS_MISSINGDATA,
    WU_E_DS_MISSINGREF,
    WU_E_DS_UNKNOWNHANDLER,
    WU_E_DS_CANTDELETE,
    WU_E_DS_LOCKTIMEOUTEXPIRED,
    WU_E_DS_NOCATEGORIES,
    WU_E_DS_ROWEXISTS,
    WU_E_DS_STOREFILELOCKED,
    WU_E_DS_CANNOTREGISTER,
    WU_E_DS_UNABLETOSTART,
    WU_E_DS_DUPLICATEUPDATEID,
    WU_E_DS_UNKNOWNSERVICE,
    WU_E_DS_SERVICEEXPIRED,
    WU_E_DS_DECLINENOTALLOWED,
    WU_E_DS_TABLESESSIONMISMATCH,
    WU_E_DS_SESSIONLOCKMISMATCH,
    WU_E_DS_NEEDWINDOWSSERVICE,
    WU_E_DS_INVALIDOPERATION,
    WU_E_DS_SCHEMAMISMATCH,
    WU_E_DS_RESETREQUIRED,
    WU_E_DS_IMPERSONATED,
    WU_E_DS_UNEXPECTED,
    WU_E_INVENTORY_PARSEFAILED,
    WU_E_INVENTORY_GET_INVENTORY_TYPE_FAILED,
    WU_E_INVENTORY_RESULT_UPLOAD_FAILED,
    WU_E_INVENTORY_UNEXPECTED,
    WU_E_INVENTORY_WMI_ERROR,
    WU_E_AU_NOSERVICE,
    WU_E_AU_NONLEGACYSERVER,
    WU_E_AU_LEGACYCLIENTDISABLED,
    WU_E_AU_PAUSED,
    WU_E_AU_NO_REGISTERED_SERVICE,
    WU_E_AU_UNEXPECTED,
    WU_E_DRV_PRUNED,
    WU_E_DRV_NOPROP_OR_LEGACY,
    WU_E_DRV_REG_MISMATCH,
    WU_E_DRV_NO_METADATA,
    WU_E_DRV_MISSING_ATTRIBUTE,
    WU_E_DRV_SYNC_FAILED,
    WU_E_DRV_NO_PRINTER_CONTENT,
    WU_E_DRV_UNEXPECTED,
    WU_E_SETUP_INVALID_INFDATA,
    WU_E_SETUP_INVALID_IDENTDATA,
    WU_E_SETUP_ALREADY_INITIALIZED,
    WU_E_SETUP_NOT_INITIALIZED,
    WU_E_SETUP_SOURCE_VERSION_MISMATCH,
    WU_E_SETUP_TARGET_VERSION_GREATER,
    WU_E_SETUP_REGISTRATION_FAILED,
    WU_E_SELFUPDATE_SKIP_ON_FAILURE,
    WU_E_SETUP_SKIP_UPDATE,
    WU_E_SETUP_UNSUPPORTED_CONFIGURATION,
    WU_E_SETUP_BLOCKED_CONFIGURATION,
    WU_E_SETUP_REBOOT_TO_FIX,
    WU_E_SETUP_ALREADYRUNNING,
    WU_E_SETUP_REBOOTREQUIRED,
    WU_E_SETUP_HANDLER_EXEC_FAILURE,
    WU_E_SETUP_INVALID_REGISTRY_DATA,
    WU_E_SELFUPDATE_REQUIRED,
    WU_E_SELFUPDATE_REQUIRED_ADMIN,
    WU_E_SETUP_WRONG_SERVER_VERSION,
    WU_E_SETUP_UNEXPECTED,
    WU_E_EE_UNKNOWN_EXPRESSION,
    WU_E_EE_INVALID_EXPRESSION,
    WU_E_EE_MISSING_METADATA,
    WU_E_EE_INVALID_VERSION,
    WU_E_EE_NOT_INITIALIZED,
    WU_E_EE_INVALID_ATTRIBUTEDATA,
    WU_E_EE_CLUSTER_ERROR,
    WU_E_EE_UNEXPECTED,
    WU_E_REPORTER_EVENTCACHECORRUPT,
    WU_E_REPORTER_EVENTNAMESPACEPARSEFAILED,
    WU_E_INVALID_EVENT,
    WU_E_SERVER_BUSY,
    WU_E_REPORTER_UNEXPECTED,
}

impl WINDOWSUPDATE {
    pub fn code(&self) -> u32 {
        return match self {
            WINDOWSUPDATE::WU_S_SERVICE_STOP => 0x00240001 as u32,
            WINDOWSUPDATE::WU_S_SELFUPDATE => 0x00240002 as u32,
            WINDOWSUPDATE::WU_S_UPDATE_ERROR => 0x00240003 as u32,
            WINDOWSUPDATE::WU_S_MARKED_FOR_DISCONNECT => 0x00240004 as u32,
            WINDOWSUPDATE::WU_S_REBOOT_REQUIRED => 0x00240005 as u32,
            WINDOWSUPDATE::WU_S_ALREADY_INSTALLED => 0x00240006 as u32,
            WINDOWSUPDATE::WU_S_ALREADY_UNINSTALLED => 0x00240007 as u32,
            WINDOWSUPDATE::WU_S_ALREADY_DOWNLOADED => 0x00240008 as u32,
            WINDOWSUPDATE::WU_E_NO_SERVICE => 0x80240001 as u32,
            WINDOWSUPDATE::WU_E_MAX_CAPACITY_REACHED => 0x80240002 as u32,
            WINDOWSUPDATE::WU_E_UNKNOWN_ID => 0x80240003 as u32,
            WINDOWSUPDATE::WU_E_NOT_INITIALIZED => 0x80240004 as u32,
            WINDOWSUPDATE::WU_E_RANGEOVERLAP => 0x80240005 as u32,
            WINDOWSUPDATE::WU_E_TOOMANYRANGES => 0x80240006 as u32,
            WINDOWSUPDATE::WU_E_INVALIDINDEX => 0x80240007 as u32,
            WINDOWSUPDATE::WU_E_ITEMNOTFOUND => 0x80240008 as u32,
            WINDOWSUPDATE::WU_E_OPERATIONINPROGRESS => 0x80240009 as u32,
            WINDOWSUPDATE::WU_E_COULDNOTCANCEL => 0x8024000A as u32,
            WINDOWSUPDATE::WU_E_CALL_CANCELLED => 0x8024000B as u32,
            WINDOWSUPDATE::WU_E_NOOP => 0x8024000C as u32,
            WINDOWSUPDATE::WU_E_XML_MISSINGDATA => 0x8024000D as u32,
            WINDOWSUPDATE::WU_E_XML_INVALID => 0x8024000E as u32,
            WINDOWSUPDATE::WU_E_CYCLE_DETECTED => 0x8024000F as u32,
            WINDOWSUPDATE::WU_E_TOO_DEEP_RELATION => 0x80240010 as u32,
            WINDOWSUPDATE::WU_E_INVALID_RELATIONSHIP => 0x80240011 as u32,
            WINDOWSUPDATE::WU_E_REG_VALUE_INVALID => 0x80240012 as u32,
            WINDOWSUPDATE::WU_E_DUPLICATE_ITEM => 0x80240013 as u32,
            WINDOWSUPDATE::WU_E_INSTALL_NOT_ALLOWED => 0x80240016 as u32,
            WINDOWSUPDATE::WU_E_NOT_APPLICABLE => 0x80240017 as u32,
            WINDOWSUPDATE::WU_E_NO_USERTOKEN => 0x80240018 as u32,
            WINDOWSUPDATE::WU_E_EXCLUSIVE_INSTALL_CONFLICT => 0x80240019 as u32,
            WINDOWSUPDATE::WU_E_POLICY_NOT_SET => 0x8024001A as u32,
            WINDOWSUPDATE::WU_E_SELFUPDATE_IN_PROGRESS => 0x8024001B as u32,
            WINDOWSUPDATE::WU_E_INVALID_UPDATE => 0x8024001D as u32,
            WINDOWSUPDATE::WU_E_SERVICE_STOP => 0x8024001E as u32,
            WINDOWSUPDATE::WU_E_NO_CONNECTION => 0x8024001F as u32,
            WINDOWSUPDATE::WU_E_NO_INTERACTIVE_USER => 0x80240020 as u32,
            WINDOWSUPDATE::WU_E_TIME_OUT => 0x80240021 as u32,
            WINDOWSUPDATE::WU_E_ALL_UPDATES_FAILED => 0x80240022 as u32,
            WINDOWSUPDATE::WU_E_EULAS_DECLINED => 0x80240023 as u32,
            WINDOWSUPDATE::WU_E_NO_UPDATE => 0x80240024 as u32,
            WINDOWSUPDATE::WU_E_USER_ACCESS_DISABLED => 0x80240025 as u32,
            WINDOWSUPDATE::WU_E_INVALID_UPDATE_TYPE => 0x80240026 as u32,
            WINDOWSUPDATE::WU_E_URL_TOO_LONG => 0x80240027 as u32,
            WINDOWSUPDATE::WU_E_UNINSTALL_NOT_ALLOWED => 0x80240028 as u32,
            WINDOWSUPDATE::WU_E_INVALID_PRODUCT_LICENSE => 0x80240029 as u32,
            WINDOWSUPDATE::WU_E_MISSING_HANDLER => 0x8024002A as u32,
            WINDOWSUPDATE::WU_E_LEGACYSERVER => 0x8024002B as u32,
            WINDOWSUPDATE::WU_E_BIN_SOURCE_ABSENT => 0x8024002C as u32,
            WINDOWSUPDATE::WU_E_SOURCE_ABSENT => 0x8024002D as u32,
            WINDOWSUPDATE::WU_E_WU_DISABLED => 0x8024002E as u32,
            WINDOWSUPDATE::WU_E_CALL_CANCELLED_BY_POLICY => 0x8024002F as u32,
            WINDOWSUPDATE::WU_E_INVALID_PROXY_SERVER => 0x80240030 as u32,
            WINDOWSUPDATE::WU_E_INVALID_FILE => 0x80240031 as u32,
            WINDOWSUPDATE::WU_E_INVALID_CRITERIA => 0x80240032 as u32,
            WINDOWSUPDATE::WU_E_EULA_UNAVAILABLE => 0x80240033 as u32,
            WINDOWSUPDATE::WU_E_DOWNLOAD_FAILED => 0x80240034 as u32,
            WINDOWSUPDATE::WU_E_UPDATE_NOT_PROCESSED => 0x80240035 as u32,
            WINDOWSUPDATE::WU_E_INVALID_OPERATION => 0x80240036 as u32,
            WINDOWSUPDATE::WU_E_NOT_SUPPORTED => 0x80240037 as u32,
            WINDOWSUPDATE::WU_E_WINHTTP_INVALID_FILE => 0x80240038 as u32,
            WINDOWSUPDATE::WU_E_TOO_MANY_RESYNC => 0x80240039 as u32,
            WINDOWSUPDATE::WU_E_NO_SERVER_CORE_SUPPORT => 0x80240040 as u32,
            WINDOWSUPDATE::WU_E_SYSPREP_IN_PROGRESS => 0x80240041 as u32,
            WINDOWSUPDATE::WU_E_UNKNOWN_SERVICE => 0x80240042 as u32,
            WINDOWSUPDATE::WU_E_UNEXPECTED => 0x80240FFF as u32,
            WINDOWSUPDATE::WU_E_MSI_WRONG_VERSION => 0x80241001 as u32,
            WINDOWSUPDATE::WU_E_MSI_NOT_CONFIGURED => 0x80241002 as u32,
            WINDOWSUPDATE::WU_E_MSP_DISABLED => 0x80241003 as u32,
            WINDOWSUPDATE::WU_E_MSI_WRONG_APP_CONTEXT => 0x80241004 as u32,
            WINDOWSUPDATE::WU_E_MSP_UNEXPECTED => 0x80241FFF as u32,
            WINDOWSUPDATE::WU_E_UH_REMOTEUNAVAILABLE => 0x80242000 as u32,
            WINDOWSUPDATE::WU_E_UH_LOCALONLY => 0x80242001 as u32,
            WINDOWSUPDATE::WU_E_UH_UNKNOWNHANDLER => 0x80242002 as u32,
            WINDOWSUPDATE::WU_E_UH_REMOTEALREADYACTIVE => 0x80242003 as u32,
            WINDOWSUPDATE::WU_E_UH_DOESNOTSUPPORTACTION => 0x80242004 as u32,
            WINDOWSUPDATE::WU_E_UH_WRONGHANDLER => 0x80242005 as u32,
            WINDOWSUPDATE::WU_E_UH_INVALIDMETADATA => 0x80242006 as u32,
            WINDOWSUPDATE::WU_E_UH_INSTALLERHUNG => 0x80242007 as u32,
            WINDOWSUPDATE::WU_E_UH_OPERATIONCANCELLED => 0x80242008 as u32,
            WINDOWSUPDATE::WU_E_UH_BADHANDLERXML => 0x80242009 as u32,
            WINDOWSUPDATE::WU_E_UH_CANREQUIREINPUT => 0x8024200A as u32,
            WINDOWSUPDATE::WU_E_UH_INSTALLERFAILURE => 0x8024200B as u32,
            WINDOWSUPDATE::WU_E_UH_FALLBACKTOSELFCONTAINED => 0x8024200C as u32,
            WINDOWSUPDATE::WU_E_UH_NEEDANOTHERDOWNLOAD => 0x8024200D as u32,
            WINDOWSUPDATE::WU_E_UH_NOTIFYFAILURE => 0x8024200E as u32,
            WINDOWSUPDATE::WU_E_UH_INCONSISTENT_FILE_NAMES => 0x8024200F as u32,
            WINDOWSUPDATE::WU_E_UH_FALLBACKERROR => 0x80242010 as u32,
            WINDOWSUPDATE::WU_E_UH_TOOMANYDOWNLOADREQUESTS => 0x80242011 as u32,
            WINDOWSUPDATE::WU_E_UH_UNEXPECTEDCBSRESPONSE => 0x80242012 as u32,
            WINDOWSUPDATE::WU_E_UH_BADCBSPACKAGEID => 0x80242013 as u32,
            WINDOWSUPDATE::WU_E_UH_POSTREBOOTSTILLPENDING => 0x80242014 as u32,
            WINDOWSUPDATE::WU_E_UH_POSTREBOOTRESULTUNKNOWN => 0x80242015 as u32,
            WINDOWSUPDATE::WU_E_UH_POSTREBOOTUNEXPECTEDSTATE => 0x80242016 as u32,
            WINDOWSUPDATE::WU_E_UH_NEW_SERVICING_STACK_REQUIRED => 0x80242017 as u32,
            WINDOWSUPDATE::WU_E_UH_UNEXPECTED => 0x80242FFF as u32,
            WINDOWSUPDATE::WU_E_INSTALLATION_RESULTS_UNKNOWN_VERSION => 0x80243001 as u32,
            WINDOWSUPDATE::WU_E_INSTALLATION_RESULTS_INVALID_DATA => 0x80243002 as u32,
            WINDOWSUPDATE::WU_E_INSTALLATION_RESULTS_NOT_FOUND => 0x80243003 as u32,
            WINDOWSUPDATE::WU_E_TRAYICON_FAILURE => 0x80243004 as u32,
            WINDOWSUPDATE::WU_E_NON_UI_MODE => 0x80243FFD as u32,
            WINDOWSUPDATE::WU_E_WUCLTUI_UNSUPPORTED_VERSION => 0x80243FFE as u32,
            WINDOWSUPDATE::WU_E_AUCLIENT_UNEXPECTED => 0x80243FFF as u32,
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_BASE => 0x80244000 as u32,
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_INITIALIZE => 0x80244001 as u32,
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_OUTOFMEMORY => 0x80244002 as u32,
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_GENERATE => 0x80244003 as u32,
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_CONNECT => 0x80244004 as u32,
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_SEND => 0x80244005 as u32,
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_SERVER => 0x80244006 as u32,
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_SOAPFAULT => 0x80244007 as u32,
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_PARSEFAULT => 0x80244008 as u32,
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_READ => 0x80244009 as u32,
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_PARSE => 0x8024400A as u32,
            WINDOWSUPDATE::WU_E_PT_SOAP_VERSION => 0x8024400B as u32,
            WINDOWSUPDATE::WU_E_PT_SOAP_MUST_UNDERSTAND => 0x8024400C as u32,
            WINDOWSUPDATE::WU_E_PT_SOAP_CLIENT => 0x8024400D as u32,
            WINDOWSUPDATE::WU_E_PT_SOAP_SERVER => 0x8024400E as u32,
            WINDOWSUPDATE::WU_E_PT_WMI_ERROR => 0x8024400F as u32,
            WINDOWSUPDATE::WU_E_PT_EXCEEDED_MAX_SERVER_TRIPS => 0x80244010 as u32,
            WINDOWSUPDATE::WU_E_PT_SUS_SERVER_NOT_SET => 0x80244011 as u32,
            WINDOWSUPDATE::WU_E_PT_DOUBLE_INITIALIZATION => 0x80244012 as u32,
            WINDOWSUPDATE::WU_E_PT_INVALID_COMPUTER_NAME => 0x80244013 as u32,
            WINDOWSUPDATE::WU_E_PT_REFRESH_CACHE_REQUIRED => 0x80244015 as u32,
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_BAD_REQUEST => 0x80244016 as u32,
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_DENIED => 0x80244017 as u32,
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_FORBIDDEN => 0x80244018 as u32,
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_NOT_FOUND => 0x80244019 as u32,
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_BAD_METHOD => 0x8024401A as u32,
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_PROXY_AUTH_REQ => 0x8024401B as u32,
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_REQUEST_TIMEOUT => 0x8024401C as u32,
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_CONFLICT => 0x8024401D as u32,
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_GONE => 0x8024401E as u32,
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_SERVER_ERROR => 0x8024401F as u32,
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_NOT_SUPPORTED => 0x80244020 as u32,
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_BAD_GATEWAY => 0x80244021 as u32,
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_SERVICE_UNAVAIL => 0x80244022 as u32,
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_GATEWAY_TIMEOUT => 0x80244023 as u32,
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_VERSION_NOT_SUP => 0x80244024 as u32,
            WINDOWSUPDATE::WU_E_PT_FILE_LOCATIONS_CHANGED => 0x80244025 as u32,
            WINDOWSUPDATE::WU_E_PT_REGISTRATION_NOT_SUPPORTED => 0x80244026 as u32,
            WINDOWSUPDATE::WU_E_PT_NO_AUTH_PLUGINS_REQUESTED => 0x80244027 as u32,
            WINDOWSUPDATE::WU_E_PT_NO_AUTH_COOKIES_CREATED => 0x80244028 as u32,
            WINDOWSUPDATE::WU_E_PT_INVALID_CONFIG_PROP => 0x80244029 as u32,
            WINDOWSUPDATE::WU_E_PT_CONFIG_PROP_MISSING => 0x8024402A as u32,
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_NOT_MAPPED => 0x8024402B as u32,
            WINDOWSUPDATE::WU_E_PT_WINHTTP_NAME_NOT_RESOLVED => 0x8024402C as u32,
            WINDOWSUPDATE::WU_E_PT_ECP_SUCCEEDED_WITH_ERRORS => 0x8024402F as u32,
            WINDOWSUPDATE::WU_E_PT_ECP_INIT_FAILED => 0x80244030 as u32,
            WINDOWSUPDATE::WU_E_PT_ECP_INVALID_FILE_FORMAT => 0x80244031 as u32,
            WINDOWSUPDATE::WU_E_PT_ECP_INVALID_METADATA => 0x80244032 as u32,
            WINDOWSUPDATE::WU_E_PT_ECP_FAILURE_TO_EXTRACT_DIGEST => 0x80244033 as u32,
            WINDOWSUPDATE::WU_E_PT_ECP_FAILURE_TO_DECOMPRESS_CAB_FILE => 0x80244034 as u32,
            WINDOWSUPDATE::WU_E_PT_ECP_FILE_LOCATION_ERROR => 0x80244035 as u32,
            WINDOWSUPDATE::WU_E_PT_UNEXPECTED => 0x80244FFF as u32,
            WINDOWSUPDATE::WU_E_REDIRECTOR_LOAD_XML => 0x80245001 as u32,
            WINDOWSUPDATE::WU_E_REDIRECTOR_S_FALSE => 0x80245002 as u32,
            WINDOWSUPDATE::WU_E_REDIRECTOR_ID_SMALLER => 0x80245003 as u32,
            WINDOWSUPDATE::WU_E_PT_SAME_REDIR_ID => 0x8024502D as u32,
            WINDOWSUPDATE::WU_E_PT_NO_MANAGED_RECOVER => 0x8024502E as u32,
            WINDOWSUPDATE::WU_E_REDIRECTOR_UNEXPECTED => 0x80245FFF as u32,
            WINDOWSUPDATE::WU_E_DM_URLNOTAVAILABLE => 0x80246001 as u32,
            WINDOWSUPDATE::WU_E_DM_INCORRECTFILEHASH => 0x80246002 as u32,
            WINDOWSUPDATE::WU_E_DM_UNKNOWNALGORITHM => 0x80246003 as u32,
            WINDOWSUPDATE::WU_E_DM_NEEDDOWNLOADREQUEST => 0x80246004 as u32,
            WINDOWSUPDATE::WU_E_DM_NONETWORK => 0x80246005 as u32,
            WINDOWSUPDATE::WU_E_DM_WRONGBITSVERSION => 0x80246006 as u32,
            WINDOWSUPDATE::WU_E_DM_NOTDOWNLOADED => 0x80246007 as u32,
            WINDOWSUPDATE::WU_E_DM_FAILTOCONNECTTOBITS => 0x80246008 as u32,
            WINDOWSUPDATE::WU_E_DM_BITSTRANSFERERROR => 0x80246009 as u32,
            WINDOWSUPDATE::WU_E_DM_DOWNLOADLOCATIONCHANGED => 0x8024600A as u32,
            WINDOWSUPDATE::WU_E_DM_CONTENTCHANGED => 0x8024600B as u32,
            WINDOWSUPDATE::WU_E_DM_UNEXPECTED => 0x80246FFF as u32,
            WINDOWSUPDATE::WU_E_OL_INVALID_SCANFILE => 0x80247001 as u32,
            WINDOWSUPDATE::WU_E_OL_NEWCLIENT_REQUIRED => 0x80247002 as u32,
            WINDOWSUPDATE::WU_E_OL_UNEXPECTED => 0x80247FFF as u32,
            WINDOWSUPDATE::WU_E_DS_SHUTDOWN => 0x80248000 as u32,
            WINDOWSUPDATE::WU_E_DS_INUSE => 0x80248001 as u32,
            WINDOWSUPDATE::WU_E_DS_INVALID => 0x80248002 as u32,
            WINDOWSUPDATE::WU_E_DS_TABLEMISSING => 0x80248003 as u32,
            WINDOWSUPDATE::WU_E_DS_TABLEINCORRECT => 0x80248004 as u32,
            WINDOWSUPDATE::WU_E_DS_INVALIDTABLENAME => 0x80248005 as u32,
            WINDOWSUPDATE::WU_E_DS_BADVERSION => 0x80248006 as u32,
            WINDOWSUPDATE::WU_E_DS_NODATA => 0x80248007 as u32,
            WINDOWSUPDATE::WU_E_DS_MISSINGDATA => 0x80248008 as u32,
            WINDOWSUPDATE::WU_E_DS_MISSINGREF => 0x80248009 as u32,
            WINDOWSUPDATE::WU_E_DS_UNKNOWNHANDLER => 0x8024800A as u32,
            WINDOWSUPDATE::WU_E_DS_CANTDELETE => 0x8024800B as u32,
            WINDOWSUPDATE::WU_E_DS_LOCKTIMEOUTEXPIRED => 0x8024800C as u32,
            WINDOWSUPDATE::WU_E_DS_NOCATEGORIES => 0x8024800D as u32,
            WINDOWSUPDATE::WU_E_DS_ROWEXISTS => 0x8024800E as u32,
            WINDOWSUPDATE::WU_E_DS_STOREFILELOCKED => 0x8024800F as u32,
            WINDOWSUPDATE::WU_E_DS_CANNOTREGISTER => 0x80248010 as u32,
            WINDOWSUPDATE::WU_E_DS_UNABLETOSTART => 0x80248011 as u32,
            WINDOWSUPDATE::WU_E_DS_DUPLICATEUPDATEID => 0x80248013 as u32,
            WINDOWSUPDATE::WU_E_DS_UNKNOWNSERVICE => 0x80248014 as u32,
            WINDOWSUPDATE::WU_E_DS_SERVICEEXPIRED => 0x80248015 as u32,
            WINDOWSUPDATE::WU_E_DS_DECLINENOTALLOWED => 0x80248016 as u32,
            WINDOWSUPDATE::WU_E_DS_TABLESESSIONMISMATCH => 0x80248017 as u32,
            WINDOWSUPDATE::WU_E_DS_SESSIONLOCKMISMATCH => 0x80248018 as u32,
            WINDOWSUPDATE::WU_E_DS_NEEDWINDOWSSERVICE => 0x80248019 as u32,
            WINDOWSUPDATE::WU_E_DS_INVALIDOPERATION => 0x8024801A as u32,
            WINDOWSUPDATE::WU_E_DS_SCHEMAMISMATCH => 0x8024801B as u32,
            WINDOWSUPDATE::WU_E_DS_RESETREQUIRED => 0x8024801C as u32,
            WINDOWSUPDATE::WU_E_DS_IMPERSONATED => 0x8024801D as u32,
            WINDOWSUPDATE::WU_E_DS_UNEXPECTED => 0x80248FFF as u32,
            WINDOWSUPDATE::WU_E_INVENTORY_PARSEFAILED => 0x80249001 as u32,
            WINDOWSUPDATE::WU_E_INVENTORY_GET_INVENTORY_TYPE_FAILED => 0x80249002 as u32,
            WINDOWSUPDATE::WU_E_INVENTORY_RESULT_UPLOAD_FAILED => 0x80249003 as u32,
            WINDOWSUPDATE::WU_E_INVENTORY_UNEXPECTED => 0x80249004 as u32,
            WINDOWSUPDATE::WU_E_INVENTORY_WMI_ERROR => 0x80249005 as u32,
            WINDOWSUPDATE::WU_E_AU_NOSERVICE => 0x8024A000 as u32,
            WINDOWSUPDATE::WU_E_AU_NONLEGACYSERVER => 0x8024A002 as u32,
            WINDOWSUPDATE::WU_E_AU_LEGACYCLIENTDISABLED => 0x8024A003 as u32,
            WINDOWSUPDATE::WU_E_AU_PAUSED => 0x8024A004 as u32,
            WINDOWSUPDATE::WU_E_AU_NO_REGISTERED_SERVICE => 0x8024A005 as u32,
            WINDOWSUPDATE::WU_E_AU_UNEXPECTED => 0x8024AFFF as u32,
            WINDOWSUPDATE::WU_E_DRV_PRUNED => 0x8024C001 as u32,
            WINDOWSUPDATE::WU_E_DRV_NOPROP_OR_LEGACY => 0x8024C002 as u32,
            WINDOWSUPDATE::WU_E_DRV_REG_MISMATCH => 0x8024C003 as u32,
            WINDOWSUPDATE::WU_E_DRV_NO_METADATA => 0x8024C004 as u32,
            WINDOWSUPDATE::WU_E_DRV_MISSING_ATTRIBUTE => 0x8024C005 as u32,
            WINDOWSUPDATE::WU_E_DRV_SYNC_FAILED => 0x8024C006 as u32,
            WINDOWSUPDATE::WU_E_DRV_NO_PRINTER_CONTENT => 0x8024C007 as u32,
            WINDOWSUPDATE::WU_E_DRV_UNEXPECTED => 0x8024CFFF as u32,
            WINDOWSUPDATE::WU_E_SETUP_INVALID_INFDATA => 0x8024D001 as u32,
            WINDOWSUPDATE::WU_E_SETUP_INVALID_IDENTDATA => 0x8024D002 as u32,
            WINDOWSUPDATE::WU_E_SETUP_ALREADY_INITIALIZED => 0x8024D003 as u32,
            WINDOWSUPDATE::WU_E_SETUP_NOT_INITIALIZED => 0x8024D004 as u32,
            WINDOWSUPDATE::WU_E_SETUP_SOURCE_VERSION_MISMATCH => 0x8024D005 as u32,
            WINDOWSUPDATE::WU_E_SETUP_TARGET_VERSION_GREATER => 0x8024D006 as u32,
            WINDOWSUPDATE::WU_E_SETUP_REGISTRATION_FAILED => 0x8024D007 as u32,
            WINDOWSUPDATE::WU_E_SELFUPDATE_SKIP_ON_FAILURE => 0x8024D008 as u32,
            WINDOWSUPDATE::WU_E_SETUP_SKIP_UPDATE => 0x8024D009 as u32,
            WINDOWSUPDATE::WU_E_SETUP_UNSUPPORTED_CONFIGURATION => 0x8024D00A as u32,
            WINDOWSUPDATE::WU_E_SETUP_BLOCKED_CONFIGURATION => 0x8024D00B as u32,
            WINDOWSUPDATE::WU_E_SETUP_REBOOT_TO_FIX => 0x8024D00C as u32,
            WINDOWSUPDATE::WU_E_SETUP_ALREADYRUNNING => 0x8024D00D as u32,
            WINDOWSUPDATE::WU_E_SETUP_REBOOTREQUIRED => 0x8024D00E as u32,
            WINDOWSUPDATE::WU_E_SETUP_HANDLER_EXEC_FAILURE => 0x8024D00F as u32,
            WINDOWSUPDATE::WU_E_SETUP_INVALID_REGISTRY_DATA => 0x8024D010 as u32,
            WINDOWSUPDATE::WU_E_SELFUPDATE_REQUIRED => 0x8024D011 as u32,
            WINDOWSUPDATE::WU_E_SELFUPDATE_REQUIRED_ADMIN => 0x8024D012 as u32,
            WINDOWSUPDATE::WU_E_SETUP_WRONG_SERVER_VERSION => 0x8024D013 as u32,
            WINDOWSUPDATE::WU_E_SETUP_UNEXPECTED => 0x8024DFFF as u32,
            WINDOWSUPDATE::WU_E_EE_UNKNOWN_EXPRESSION => 0x8024E001 as u32,
            WINDOWSUPDATE::WU_E_EE_INVALID_EXPRESSION => 0x8024E002 as u32,
            WINDOWSUPDATE::WU_E_EE_MISSING_METADATA => 0x8024E003 as u32,
            WINDOWSUPDATE::WU_E_EE_INVALID_VERSION => 0x8024E004 as u32,
            WINDOWSUPDATE::WU_E_EE_NOT_INITIALIZED => 0x8024E005 as u32,
            WINDOWSUPDATE::WU_E_EE_INVALID_ATTRIBUTEDATA => 0x8024E006 as u32,
            WINDOWSUPDATE::WU_E_EE_CLUSTER_ERROR => 0x8024E007 as u32,
            WINDOWSUPDATE::WU_E_EE_UNEXPECTED => 0x8024EFFF as u32,
            WINDOWSUPDATE::WU_E_REPORTER_EVENTCACHECORRUPT => 0x8024F001 as u32,
            WINDOWSUPDATE::WU_E_REPORTER_EVENTNAMESPACEPARSEFAILED => 0x8024F002 as u32,
            WINDOWSUPDATE::WU_E_INVALID_EVENT => 0x8024F003 as u32,
            WINDOWSUPDATE::WU_E_SERVER_BUSY => 0x8024F004 as u32,
            WINDOWSUPDATE::WU_E_REPORTER_UNEXPECTED => 0x8024FFFF as u32,
        }
    }

    pub fn error(&self) -> RawError {
        return match self {
            WINDOWSUPDATE::WU_S_SERVICE_STOP => RawError::Kind(WINDOWSUPDATE::WU_S_SERVICE_STOP),
            WINDOWSUPDATE::WU_S_SELFUPDATE => RawError::Kind(WINDOWSUPDATE::WU_S_SELFUPDATE),
            WINDOWSUPDATE::WU_S_UPDATE_ERROR => RawError::Kind(WINDOWSUPDATE::WU_S_UPDATE_ERROR),
            WINDOWSUPDATE::WU_S_MARKED_FOR_DISCONNECT => RawError::Kind(WINDOWSUPDATE::WU_S_MARKED_FOR_DISCONNECT),
            WINDOWSUPDATE::WU_S_REBOOT_REQUIRED => RawError::Kind(WINDOWSUPDATE::WU_S_REBOOT_REQUIRED),
            WINDOWSUPDATE::WU_S_ALREADY_INSTALLED => RawError::Kind(WINDOWSUPDATE::WU_S_ALREADY_INSTALLED),
            WINDOWSUPDATE::WU_S_ALREADY_UNINSTALLED => RawError::Kind(WINDOWSUPDATE::WU_S_ALREADY_UNINSTALLED),
            WINDOWSUPDATE::WU_S_ALREADY_DOWNLOADED => RawError::Kind(WINDOWSUPDATE::WU_S_ALREADY_DOWNLOADED),
            WINDOWSUPDATE::WU_E_NO_SERVICE => RawError::Kind(WINDOWSUPDATE::WU_E_NO_SERVICE),
            WINDOWSUPDATE::WU_E_MAX_CAPACITY_REACHED => RawError::Kind(WINDOWSUPDATE::WU_E_MAX_CAPACITY_REACHED),
            WINDOWSUPDATE::WU_E_UNKNOWN_ID => RawError::Kind(WINDOWSUPDATE::WU_E_UNKNOWN_ID),
            WINDOWSUPDATE::WU_E_NOT_INITIALIZED => RawError::Kind(WINDOWSUPDATE::WU_E_NOT_INITIALIZED),
            WINDOWSUPDATE::WU_E_RANGEOVERLAP => RawError::Kind(WINDOWSUPDATE::WU_E_RANGEOVERLAP),
            WINDOWSUPDATE::WU_E_TOOMANYRANGES => RawError::Kind(WINDOWSUPDATE::WU_E_TOOMANYRANGES),
            WINDOWSUPDATE::WU_E_INVALIDINDEX => RawError::Kind(WINDOWSUPDATE::WU_E_INVALIDINDEX),
            WINDOWSUPDATE::WU_E_ITEMNOTFOUND => RawError::Kind(WINDOWSUPDATE::WU_E_ITEMNOTFOUND),
            WINDOWSUPDATE::WU_E_OPERATIONINPROGRESS => RawError::Kind(WINDOWSUPDATE::WU_E_OPERATIONINPROGRESS),
            WINDOWSUPDATE::WU_E_COULDNOTCANCEL => RawError::Kind(WINDOWSUPDATE::WU_E_COULDNOTCANCEL),
            WINDOWSUPDATE::WU_E_CALL_CANCELLED => RawError::Kind(WINDOWSUPDATE::WU_E_CALL_CANCELLED),
            WINDOWSUPDATE::WU_E_NOOP => RawError::Kind(WINDOWSUPDATE::WU_E_NOOP),
            WINDOWSUPDATE::WU_E_XML_MISSINGDATA => RawError::Kind(WINDOWSUPDATE::WU_E_XML_MISSINGDATA),
            WINDOWSUPDATE::WU_E_XML_INVALID => RawError::Kind(WINDOWSUPDATE::WU_E_XML_INVALID),
            WINDOWSUPDATE::WU_E_CYCLE_DETECTED => RawError::Kind(WINDOWSUPDATE::WU_E_CYCLE_DETECTED),
            WINDOWSUPDATE::WU_E_TOO_DEEP_RELATION => RawError::Kind(WINDOWSUPDATE::WU_E_TOO_DEEP_RELATION),
            WINDOWSUPDATE::WU_E_INVALID_RELATIONSHIP => RawError::Kind(WINDOWSUPDATE::WU_E_INVALID_RELATIONSHIP),
            WINDOWSUPDATE::WU_E_REG_VALUE_INVALID => RawError::Kind(WINDOWSUPDATE::WU_E_REG_VALUE_INVALID),
            WINDOWSUPDATE::WU_E_DUPLICATE_ITEM => RawError::Kind(WINDOWSUPDATE::WU_E_DUPLICATE_ITEM),
            WINDOWSUPDATE::WU_E_INSTALL_NOT_ALLOWED => RawError::Kind(WINDOWSUPDATE::WU_E_INSTALL_NOT_ALLOWED),
            WINDOWSUPDATE::WU_E_NOT_APPLICABLE => RawError::Kind(WINDOWSUPDATE::WU_E_NOT_APPLICABLE),
            WINDOWSUPDATE::WU_E_NO_USERTOKEN => RawError::Kind(WINDOWSUPDATE::WU_E_NO_USERTOKEN),
            WINDOWSUPDATE::WU_E_EXCLUSIVE_INSTALL_CONFLICT => RawError::Kind(WINDOWSUPDATE::WU_E_EXCLUSIVE_INSTALL_CONFLICT),
            WINDOWSUPDATE::WU_E_POLICY_NOT_SET => RawError::Kind(WINDOWSUPDATE::WU_E_POLICY_NOT_SET),
            WINDOWSUPDATE::WU_E_SELFUPDATE_IN_PROGRESS => RawError::Kind(WINDOWSUPDATE::WU_E_SELFUPDATE_IN_PROGRESS),
            WINDOWSUPDATE::WU_E_INVALID_UPDATE => RawError::Kind(WINDOWSUPDATE::WU_E_INVALID_UPDATE),
            WINDOWSUPDATE::WU_E_SERVICE_STOP => RawError::Kind(WINDOWSUPDATE::WU_E_SERVICE_STOP),
            WINDOWSUPDATE::WU_E_NO_CONNECTION => RawError::Kind(WINDOWSUPDATE::WU_E_NO_CONNECTION),
            WINDOWSUPDATE::WU_E_NO_INTERACTIVE_USER => RawError::Kind(WINDOWSUPDATE::WU_E_NO_INTERACTIVE_USER),
            WINDOWSUPDATE::WU_E_TIME_OUT => RawError::Kind(WINDOWSUPDATE::WU_E_TIME_OUT),
            WINDOWSUPDATE::WU_E_ALL_UPDATES_FAILED => RawError::Kind(WINDOWSUPDATE::WU_E_ALL_UPDATES_FAILED),
            WINDOWSUPDATE::WU_E_EULAS_DECLINED => RawError::Kind(WINDOWSUPDATE::WU_E_EULAS_DECLINED),
            WINDOWSUPDATE::WU_E_NO_UPDATE => RawError::Kind(WINDOWSUPDATE::WU_E_NO_UPDATE),
            WINDOWSUPDATE::WU_E_USER_ACCESS_DISABLED => RawError::Kind(WINDOWSUPDATE::WU_E_USER_ACCESS_DISABLED),
            WINDOWSUPDATE::WU_E_INVALID_UPDATE_TYPE => RawError::Kind(WINDOWSUPDATE::WU_E_INVALID_UPDATE_TYPE),
            WINDOWSUPDATE::WU_E_URL_TOO_LONG => RawError::Kind(WINDOWSUPDATE::WU_E_URL_TOO_LONG),
            WINDOWSUPDATE::WU_E_UNINSTALL_NOT_ALLOWED => RawError::Kind(WINDOWSUPDATE::WU_E_UNINSTALL_NOT_ALLOWED),
            WINDOWSUPDATE::WU_E_INVALID_PRODUCT_LICENSE => RawError::Kind(WINDOWSUPDATE::WU_E_INVALID_PRODUCT_LICENSE),
            WINDOWSUPDATE::WU_E_MISSING_HANDLER => RawError::Kind(WINDOWSUPDATE::WU_E_MISSING_HANDLER),
            WINDOWSUPDATE::WU_E_LEGACYSERVER => RawError::Kind(WINDOWSUPDATE::WU_E_LEGACYSERVER),
            WINDOWSUPDATE::WU_E_BIN_SOURCE_ABSENT => RawError::Kind(WINDOWSUPDATE::WU_E_BIN_SOURCE_ABSENT),
            WINDOWSUPDATE::WU_E_SOURCE_ABSENT => RawError::Kind(WINDOWSUPDATE::WU_E_SOURCE_ABSENT),
            WINDOWSUPDATE::WU_E_WU_DISABLED => RawError::Kind(WINDOWSUPDATE::WU_E_WU_DISABLED),
            WINDOWSUPDATE::WU_E_CALL_CANCELLED_BY_POLICY => RawError::Kind(WINDOWSUPDATE::WU_E_CALL_CANCELLED_BY_POLICY),
            WINDOWSUPDATE::WU_E_INVALID_PROXY_SERVER => RawError::Kind(WINDOWSUPDATE::WU_E_INVALID_PROXY_SERVER),
            WINDOWSUPDATE::WU_E_INVALID_FILE => RawError::Kind(WINDOWSUPDATE::WU_E_INVALID_FILE),
            WINDOWSUPDATE::WU_E_INVALID_CRITERIA => RawError::Kind(WINDOWSUPDATE::WU_E_INVALID_CRITERIA),
            WINDOWSUPDATE::WU_E_EULA_UNAVAILABLE => RawError::Kind(WINDOWSUPDATE::WU_E_EULA_UNAVAILABLE),
            WINDOWSUPDATE::WU_E_DOWNLOAD_FAILED => RawError::Kind(WINDOWSUPDATE::WU_E_DOWNLOAD_FAILED),
            WINDOWSUPDATE::WU_E_UPDATE_NOT_PROCESSED => RawError::Kind(WINDOWSUPDATE::WU_E_UPDATE_NOT_PROCESSED),
            WINDOWSUPDATE::WU_E_INVALID_OPERATION => RawError::Kind(WINDOWSUPDATE::WU_E_INVALID_OPERATION),
            WINDOWSUPDATE::WU_E_NOT_SUPPORTED => RawError::Kind(WINDOWSUPDATE::WU_E_NOT_SUPPORTED),
            WINDOWSUPDATE::WU_E_WINHTTP_INVALID_FILE => RawError::Kind(WINDOWSUPDATE::WU_E_WINHTTP_INVALID_FILE),
            WINDOWSUPDATE::WU_E_TOO_MANY_RESYNC => RawError::Kind(WINDOWSUPDATE::WU_E_TOO_MANY_RESYNC),
            WINDOWSUPDATE::WU_E_NO_SERVER_CORE_SUPPORT => RawError::Kind(WINDOWSUPDATE::WU_E_NO_SERVER_CORE_SUPPORT),
            WINDOWSUPDATE::WU_E_SYSPREP_IN_PROGRESS => RawError::Kind(WINDOWSUPDATE::WU_E_SYSPREP_IN_PROGRESS),
            WINDOWSUPDATE::WU_E_UNKNOWN_SERVICE => RawError::Kind(WINDOWSUPDATE::WU_E_UNKNOWN_SERVICE),
            WINDOWSUPDATE::WU_E_UNEXPECTED => RawError::Kind(WINDOWSUPDATE::WU_E_UNEXPECTED),
            WINDOWSUPDATE::WU_E_MSI_WRONG_VERSION => RawError::Kind(WINDOWSUPDATE::WU_E_MSI_WRONG_VERSION),
            WINDOWSUPDATE::WU_E_MSI_NOT_CONFIGURED => RawError::Kind(WINDOWSUPDATE::WU_E_MSI_NOT_CONFIGURED),
            WINDOWSUPDATE::WU_E_MSP_DISABLED => RawError::Kind(WINDOWSUPDATE::WU_E_MSP_DISABLED),
            WINDOWSUPDATE::WU_E_MSI_WRONG_APP_CONTEXT => RawError::Kind(WINDOWSUPDATE::WU_E_MSI_WRONG_APP_CONTEXT),
            WINDOWSUPDATE::WU_E_MSP_UNEXPECTED => RawError::Kind(WINDOWSUPDATE::WU_E_MSP_UNEXPECTED),
            WINDOWSUPDATE::WU_E_UH_REMOTEUNAVAILABLE => RawError::Kind(WINDOWSUPDATE::WU_E_UH_REMOTEUNAVAILABLE),
            WINDOWSUPDATE::WU_E_UH_LOCALONLY => RawError::Kind(WINDOWSUPDATE::WU_E_UH_LOCALONLY),
            WINDOWSUPDATE::WU_E_UH_UNKNOWNHANDLER => RawError::Kind(WINDOWSUPDATE::WU_E_UH_UNKNOWNHANDLER),
            WINDOWSUPDATE::WU_E_UH_REMOTEALREADYACTIVE => RawError::Kind(WINDOWSUPDATE::WU_E_UH_REMOTEALREADYACTIVE),
            WINDOWSUPDATE::WU_E_UH_DOESNOTSUPPORTACTION => RawError::Kind(WINDOWSUPDATE::WU_E_UH_DOESNOTSUPPORTACTION),
            WINDOWSUPDATE::WU_E_UH_WRONGHANDLER => RawError::Kind(WINDOWSUPDATE::WU_E_UH_WRONGHANDLER),
            WINDOWSUPDATE::WU_E_UH_INVALIDMETADATA => RawError::Kind(WINDOWSUPDATE::WU_E_UH_INVALIDMETADATA),
            WINDOWSUPDATE::WU_E_UH_INSTALLERHUNG => RawError::Kind(WINDOWSUPDATE::WU_E_UH_INSTALLERHUNG),
            WINDOWSUPDATE::WU_E_UH_OPERATIONCANCELLED => RawError::Kind(WINDOWSUPDATE::WU_E_UH_OPERATIONCANCELLED),
            WINDOWSUPDATE::WU_E_UH_BADHANDLERXML => RawError::Kind(WINDOWSUPDATE::WU_E_UH_BADHANDLERXML),
            WINDOWSUPDATE::WU_E_UH_CANREQUIREINPUT => RawError::Kind(WINDOWSUPDATE::WU_E_UH_CANREQUIREINPUT),
            WINDOWSUPDATE::WU_E_UH_INSTALLERFAILURE => RawError::Kind(WINDOWSUPDATE::WU_E_UH_INSTALLERFAILURE),
            WINDOWSUPDATE::WU_E_UH_FALLBACKTOSELFCONTAINED => RawError::Kind(WINDOWSUPDATE::WU_E_UH_FALLBACKTOSELFCONTAINED),
            WINDOWSUPDATE::WU_E_UH_NEEDANOTHERDOWNLOAD => RawError::Kind(WINDOWSUPDATE::WU_E_UH_NEEDANOTHERDOWNLOAD),
            WINDOWSUPDATE::WU_E_UH_NOTIFYFAILURE => RawError::Kind(WINDOWSUPDATE::WU_E_UH_NOTIFYFAILURE),
            WINDOWSUPDATE::WU_E_UH_INCONSISTENT_FILE_NAMES => RawError::Kind(WINDOWSUPDATE::WU_E_UH_INCONSISTENT_FILE_NAMES),
            WINDOWSUPDATE::WU_E_UH_FALLBACKERROR => RawError::Kind(WINDOWSUPDATE::WU_E_UH_FALLBACKERROR),
            WINDOWSUPDATE::WU_E_UH_TOOMANYDOWNLOADREQUESTS => RawError::Kind(WINDOWSUPDATE::WU_E_UH_TOOMANYDOWNLOADREQUESTS),
            WINDOWSUPDATE::WU_E_UH_UNEXPECTEDCBSRESPONSE => RawError::Kind(WINDOWSUPDATE::WU_E_UH_UNEXPECTEDCBSRESPONSE),
            WINDOWSUPDATE::WU_E_UH_BADCBSPACKAGEID => RawError::Kind(WINDOWSUPDATE::WU_E_UH_BADCBSPACKAGEID),
            WINDOWSUPDATE::WU_E_UH_POSTREBOOTSTILLPENDING => RawError::Kind(WINDOWSUPDATE::WU_E_UH_POSTREBOOTSTILLPENDING),
            WINDOWSUPDATE::WU_E_UH_POSTREBOOTRESULTUNKNOWN => RawError::Kind(WINDOWSUPDATE::WU_E_UH_POSTREBOOTRESULTUNKNOWN),
            WINDOWSUPDATE::WU_E_UH_POSTREBOOTUNEXPECTEDSTATE => RawError::Kind(WINDOWSUPDATE::WU_E_UH_POSTREBOOTUNEXPECTEDSTATE),
            WINDOWSUPDATE::WU_E_UH_NEW_SERVICING_STACK_REQUIRED => RawError::Kind(WINDOWSUPDATE::WU_E_UH_NEW_SERVICING_STACK_REQUIRED),
            WINDOWSUPDATE::WU_E_UH_UNEXPECTED => RawError::Kind(WINDOWSUPDATE::WU_E_UH_UNEXPECTED),
            WINDOWSUPDATE::WU_E_INSTALLATION_RESULTS_UNKNOWN_VERSION => RawError::Kind(WINDOWSUPDATE::WU_E_INSTALLATION_RESULTS_UNKNOWN_VERSION),
            WINDOWSUPDATE::WU_E_INSTALLATION_RESULTS_INVALID_DATA => RawError::Kind(WINDOWSUPDATE::WU_E_INSTALLATION_RESULTS_INVALID_DATA),
            WINDOWSUPDATE::WU_E_INSTALLATION_RESULTS_NOT_FOUND => RawError::Kind(WINDOWSUPDATE::WU_E_INSTALLATION_RESULTS_NOT_FOUND),
            WINDOWSUPDATE::WU_E_TRAYICON_FAILURE => RawError::Kind(WINDOWSUPDATE::WU_E_TRAYICON_FAILURE),
            WINDOWSUPDATE::WU_E_NON_UI_MODE => RawError::Kind(WINDOWSUPDATE::WU_E_NON_UI_MODE),
            WINDOWSUPDATE::WU_E_WUCLTUI_UNSUPPORTED_VERSION => RawError::Kind(WINDOWSUPDATE::WU_E_WUCLTUI_UNSUPPORTED_VERSION),
            WINDOWSUPDATE::WU_E_AUCLIENT_UNEXPECTED => RawError::Kind(WINDOWSUPDATE::WU_E_AUCLIENT_UNEXPECTED),
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_BASE => RawError::Kind(WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_BASE),
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_INITIALIZE => RawError::Kind(WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_INITIALIZE),
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_OUTOFMEMORY => RawError::Kind(WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_OUTOFMEMORY),
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_GENERATE => RawError::Kind(WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_GENERATE),
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_CONNECT => RawError::Kind(WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_CONNECT),
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_SEND => RawError::Kind(WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_SEND),
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_SERVER => RawError::Kind(WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_SERVER),
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_SOAPFAULT => RawError::Kind(WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_SOAPFAULT),
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_PARSEFAULT => RawError::Kind(WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_PARSEFAULT),
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_READ => RawError::Kind(WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_READ),
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_PARSE => RawError::Kind(WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_PARSE),
            WINDOWSUPDATE::WU_E_PT_SOAP_VERSION => RawError::Kind(WINDOWSUPDATE::WU_E_PT_SOAP_VERSION),
            WINDOWSUPDATE::WU_E_PT_SOAP_MUST_UNDERSTAND => RawError::Kind(WINDOWSUPDATE::WU_E_PT_SOAP_MUST_UNDERSTAND),
            WINDOWSUPDATE::WU_E_PT_SOAP_CLIENT => RawError::Kind(WINDOWSUPDATE::WU_E_PT_SOAP_CLIENT),
            WINDOWSUPDATE::WU_E_PT_SOAP_SERVER => RawError::Kind(WINDOWSUPDATE::WU_E_PT_SOAP_SERVER),
            WINDOWSUPDATE::WU_E_PT_WMI_ERROR => RawError::Kind(WINDOWSUPDATE::WU_E_PT_WMI_ERROR),
            WINDOWSUPDATE::WU_E_PT_EXCEEDED_MAX_SERVER_TRIPS => RawError::Kind(WINDOWSUPDATE::WU_E_PT_EXCEEDED_MAX_SERVER_TRIPS),
            WINDOWSUPDATE::WU_E_PT_SUS_SERVER_NOT_SET => RawError::Kind(WINDOWSUPDATE::WU_E_PT_SUS_SERVER_NOT_SET),
            WINDOWSUPDATE::WU_E_PT_DOUBLE_INITIALIZATION => RawError::Kind(WINDOWSUPDATE::WU_E_PT_DOUBLE_INITIALIZATION),
            WINDOWSUPDATE::WU_E_PT_INVALID_COMPUTER_NAME => RawError::Kind(WINDOWSUPDATE::WU_E_PT_INVALID_COMPUTER_NAME),
            WINDOWSUPDATE::WU_E_PT_REFRESH_CACHE_REQUIRED => RawError::Kind(WINDOWSUPDATE::WU_E_PT_REFRESH_CACHE_REQUIRED),
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_BAD_REQUEST => RawError::Kind(WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_BAD_REQUEST),
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_DENIED => RawError::Kind(WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_DENIED),
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_FORBIDDEN => RawError::Kind(WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_FORBIDDEN),
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_NOT_FOUND => RawError::Kind(WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_NOT_FOUND),
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_BAD_METHOD => RawError::Kind(WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_BAD_METHOD),
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_PROXY_AUTH_REQ => RawError::Kind(WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_PROXY_AUTH_REQ),
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_REQUEST_TIMEOUT => RawError::Kind(WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_REQUEST_TIMEOUT),
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_CONFLICT => RawError::Kind(WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_CONFLICT),
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_GONE => RawError::Kind(WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_GONE),
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_SERVER_ERROR => RawError::Kind(WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_SERVER_ERROR),
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_NOT_SUPPORTED => RawError::Kind(WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_NOT_SUPPORTED),
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_BAD_GATEWAY => RawError::Kind(WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_BAD_GATEWAY),
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_SERVICE_UNAVAIL => RawError::Kind(WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_SERVICE_UNAVAIL),
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_GATEWAY_TIMEOUT => RawError::Kind(WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_GATEWAY_TIMEOUT),
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_VERSION_NOT_SUP => RawError::Kind(WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_VERSION_NOT_SUP),
            WINDOWSUPDATE::WU_E_PT_FILE_LOCATIONS_CHANGED => RawError::Kind(WINDOWSUPDATE::WU_E_PT_FILE_LOCATIONS_CHANGED),
            WINDOWSUPDATE::WU_E_PT_REGISTRATION_NOT_SUPPORTED => RawError::Kind(WINDOWSUPDATE::WU_E_PT_REGISTRATION_NOT_SUPPORTED),
            WINDOWSUPDATE::WU_E_PT_NO_AUTH_PLUGINS_REQUESTED => RawError::Kind(WINDOWSUPDATE::WU_E_PT_NO_AUTH_PLUGINS_REQUESTED),
            WINDOWSUPDATE::WU_E_PT_NO_AUTH_COOKIES_CREATED => RawError::Kind(WINDOWSUPDATE::WU_E_PT_NO_AUTH_COOKIES_CREATED),
            WINDOWSUPDATE::WU_E_PT_INVALID_CONFIG_PROP => RawError::Kind(WINDOWSUPDATE::WU_E_PT_INVALID_CONFIG_PROP),
            WINDOWSUPDATE::WU_E_PT_CONFIG_PROP_MISSING => RawError::Kind(WINDOWSUPDATE::WU_E_PT_CONFIG_PROP_MISSING),
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_NOT_MAPPED => RawError::Kind(WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_NOT_MAPPED),
            WINDOWSUPDATE::WU_E_PT_WINHTTP_NAME_NOT_RESOLVED => RawError::Kind(WINDOWSUPDATE::WU_E_PT_WINHTTP_NAME_NOT_RESOLVED),
            WINDOWSUPDATE::WU_E_PT_ECP_SUCCEEDED_WITH_ERRORS => RawError::Kind(WINDOWSUPDATE::WU_E_PT_ECP_SUCCEEDED_WITH_ERRORS),
            WINDOWSUPDATE::WU_E_PT_ECP_INIT_FAILED => RawError::Kind(WINDOWSUPDATE::WU_E_PT_ECP_INIT_FAILED),
            WINDOWSUPDATE::WU_E_PT_ECP_INVALID_FILE_FORMAT => RawError::Kind(WINDOWSUPDATE::WU_E_PT_ECP_INVALID_FILE_FORMAT),
            WINDOWSUPDATE::WU_E_PT_ECP_INVALID_METADATA => RawError::Kind(WINDOWSUPDATE::WU_E_PT_ECP_INVALID_METADATA),
            WINDOWSUPDATE::WU_E_PT_ECP_FAILURE_TO_EXTRACT_DIGEST => RawError::Kind(WINDOWSUPDATE::WU_E_PT_ECP_FAILURE_TO_EXTRACT_DIGEST),
            WINDOWSUPDATE::WU_E_PT_ECP_FAILURE_TO_DECOMPRESS_CAB_FILE => RawError::Kind(WINDOWSUPDATE::WU_E_PT_ECP_FAILURE_TO_DECOMPRESS_CAB_FILE),
            WINDOWSUPDATE::WU_E_PT_ECP_FILE_LOCATION_ERROR => RawError::Kind(WINDOWSUPDATE::WU_E_PT_ECP_FILE_LOCATION_ERROR),
            WINDOWSUPDATE::WU_E_PT_UNEXPECTED => RawError::Kind(WINDOWSUPDATE::WU_E_PT_UNEXPECTED),
            WINDOWSUPDATE::WU_E_REDIRECTOR_LOAD_XML => RawError::Kind(WINDOWSUPDATE::WU_E_REDIRECTOR_LOAD_XML),
            WINDOWSUPDATE::WU_E_REDIRECTOR_S_FALSE => RawError::Kind(WINDOWSUPDATE::WU_E_REDIRECTOR_S_FALSE),
            WINDOWSUPDATE::WU_E_REDIRECTOR_ID_SMALLER => RawError::Kind(WINDOWSUPDATE::WU_E_REDIRECTOR_ID_SMALLER),
            WINDOWSUPDATE::WU_E_PT_SAME_REDIR_ID => RawError::Kind(WINDOWSUPDATE::WU_E_PT_SAME_REDIR_ID),
            WINDOWSUPDATE::WU_E_PT_NO_MANAGED_RECOVER => RawError::Kind(WINDOWSUPDATE::WU_E_PT_NO_MANAGED_RECOVER),
            WINDOWSUPDATE::WU_E_REDIRECTOR_UNEXPECTED => RawError::Kind(WINDOWSUPDATE::WU_E_REDIRECTOR_UNEXPECTED),
            WINDOWSUPDATE::WU_E_DM_URLNOTAVAILABLE => RawError::Kind(WINDOWSUPDATE::WU_E_DM_URLNOTAVAILABLE),
            WINDOWSUPDATE::WU_E_DM_INCORRECTFILEHASH => RawError::Kind(WINDOWSUPDATE::WU_E_DM_INCORRECTFILEHASH),
            WINDOWSUPDATE::WU_E_DM_UNKNOWNALGORITHM => RawError::Kind(WINDOWSUPDATE::WU_E_DM_UNKNOWNALGORITHM),
            WINDOWSUPDATE::WU_E_DM_NEEDDOWNLOADREQUEST => RawError::Kind(WINDOWSUPDATE::WU_E_DM_NEEDDOWNLOADREQUEST),
            WINDOWSUPDATE::WU_E_DM_NONETWORK => RawError::Kind(WINDOWSUPDATE::WU_E_DM_NONETWORK),
            WINDOWSUPDATE::WU_E_DM_WRONGBITSVERSION => RawError::Kind(WINDOWSUPDATE::WU_E_DM_WRONGBITSVERSION),
            WINDOWSUPDATE::WU_E_DM_NOTDOWNLOADED => RawError::Kind(WINDOWSUPDATE::WU_E_DM_NOTDOWNLOADED),
            WINDOWSUPDATE::WU_E_DM_FAILTOCONNECTTOBITS => RawError::Kind(WINDOWSUPDATE::WU_E_DM_FAILTOCONNECTTOBITS),
            WINDOWSUPDATE::WU_E_DM_BITSTRANSFERERROR => RawError::Kind(WINDOWSUPDATE::WU_E_DM_BITSTRANSFERERROR),
            WINDOWSUPDATE::WU_E_DM_DOWNLOADLOCATIONCHANGED => RawError::Kind(WINDOWSUPDATE::WU_E_DM_DOWNLOADLOCATIONCHANGED),
            WINDOWSUPDATE::WU_E_DM_CONTENTCHANGED => RawError::Kind(WINDOWSUPDATE::WU_E_DM_CONTENTCHANGED),
            WINDOWSUPDATE::WU_E_DM_UNEXPECTED => RawError::Kind(WINDOWSUPDATE::WU_E_DM_UNEXPECTED),
            WINDOWSUPDATE::WU_E_OL_INVALID_SCANFILE => RawError::Kind(WINDOWSUPDATE::WU_E_OL_INVALID_SCANFILE),
            WINDOWSUPDATE::WU_E_OL_NEWCLIENT_REQUIRED => RawError::Kind(WINDOWSUPDATE::WU_E_OL_NEWCLIENT_REQUIRED),
            WINDOWSUPDATE::WU_E_OL_UNEXPECTED => RawError::Kind(WINDOWSUPDATE::WU_E_OL_UNEXPECTED),
            WINDOWSUPDATE::WU_E_DS_SHUTDOWN => RawError::Kind(WINDOWSUPDATE::WU_E_DS_SHUTDOWN),
            WINDOWSUPDATE::WU_E_DS_INUSE => RawError::Kind(WINDOWSUPDATE::WU_E_DS_INUSE),
            WINDOWSUPDATE::WU_E_DS_INVALID => RawError::Kind(WINDOWSUPDATE::WU_E_DS_INVALID),
            WINDOWSUPDATE::WU_E_DS_TABLEMISSING => RawError::Kind(WINDOWSUPDATE::WU_E_DS_TABLEMISSING),
            WINDOWSUPDATE::WU_E_DS_TABLEINCORRECT => RawError::Kind(WINDOWSUPDATE::WU_E_DS_TABLEINCORRECT),
            WINDOWSUPDATE::WU_E_DS_INVALIDTABLENAME => RawError::Kind(WINDOWSUPDATE::WU_E_DS_INVALIDTABLENAME),
            WINDOWSUPDATE::WU_E_DS_BADVERSION => RawError::Kind(WINDOWSUPDATE::WU_E_DS_BADVERSION),
            WINDOWSUPDATE::WU_E_DS_NODATA => RawError::Kind(WINDOWSUPDATE::WU_E_DS_NODATA),
            WINDOWSUPDATE::WU_E_DS_MISSINGDATA => RawError::Kind(WINDOWSUPDATE::WU_E_DS_MISSINGDATA),
            WINDOWSUPDATE::WU_E_DS_MISSINGREF => RawError::Kind(WINDOWSUPDATE::WU_E_DS_MISSINGREF),
            WINDOWSUPDATE::WU_E_DS_UNKNOWNHANDLER => RawError::Kind(WINDOWSUPDATE::WU_E_DS_UNKNOWNHANDLER),
            WINDOWSUPDATE::WU_E_DS_CANTDELETE => RawError::Kind(WINDOWSUPDATE::WU_E_DS_CANTDELETE),
            WINDOWSUPDATE::WU_E_DS_LOCKTIMEOUTEXPIRED => RawError::Kind(WINDOWSUPDATE::WU_E_DS_LOCKTIMEOUTEXPIRED),
            WINDOWSUPDATE::WU_E_DS_NOCATEGORIES => RawError::Kind(WINDOWSUPDATE::WU_E_DS_NOCATEGORIES),
            WINDOWSUPDATE::WU_E_DS_ROWEXISTS => RawError::Kind(WINDOWSUPDATE::WU_E_DS_ROWEXISTS),
            WINDOWSUPDATE::WU_E_DS_STOREFILELOCKED => RawError::Kind(WINDOWSUPDATE::WU_E_DS_STOREFILELOCKED),
            WINDOWSUPDATE::WU_E_DS_CANNOTREGISTER => RawError::Kind(WINDOWSUPDATE::WU_E_DS_CANNOTREGISTER),
            WINDOWSUPDATE::WU_E_DS_UNABLETOSTART => RawError::Kind(WINDOWSUPDATE::WU_E_DS_UNABLETOSTART),
            WINDOWSUPDATE::WU_E_DS_DUPLICATEUPDATEID => RawError::Kind(WINDOWSUPDATE::WU_E_DS_DUPLICATEUPDATEID),
            WINDOWSUPDATE::WU_E_DS_UNKNOWNSERVICE => RawError::Kind(WINDOWSUPDATE::WU_E_DS_UNKNOWNSERVICE),
            WINDOWSUPDATE::WU_E_DS_SERVICEEXPIRED => RawError::Kind(WINDOWSUPDATE::WU_E_DS_SERVICEEXPIRED),
            WINDOWSUPDATE::WU_E_DS_DECLINENOTALLOWED => RawError::Kind(WINDOWSUPDATE::WU_E_DS_DECLINENOTALLOWED),
            WINDOWSUPDATE::WU_E_DS_TABLESESSIONMISMATCH => RawError::Kind(WINDOWSUPDATE::WU_E_DS_TABLESESSIONMISMATCH),
            WINDOWSUPDATE::WU_E_DS_SESSIONLOCKMISMATCH => RawError::Kind(WINDOWSUPDATE::WU_E_DS_SESSIONLOCKMISMATCH),
            WINDOWSUPDATE::WU_E_DS_NEEDWINDOWSSERVICE => RawError::Kind(WINDOWSUPDATE::WU_E_DS_NEEDWINDOWSSERVICE),
            WINDOWSUPDATE::WU_E_DS_INVALIDOPERATION => RawError::Kind(WINDOWSUPDATE::WU_E_DS_INVALIDOPERATION),
            WINDOWSUPDATE::WU_E_DS_SCHEMAMISMATCH => RawError::Kind(WINDOWSUPDATE::WU_E_DS_SCHEMAMISMATCH),
            WINDOWSUPDATE::WU_E_DS_RESETREQUIRED => RawError::Kind(WINDOWSUPDATE::WU_E_DS_RESETREQUIRED),
            WINDOWSUPDATE::WU_E_DS_IMPERSONATED => RawError::Kind(WINDOWSUPDATE::WU_E_DS_IMPERSONATED),
            WINDOWSUPDATE::WU_E_DS_UNEXPECTED => RawError::Kind(WINDOWSUPDATE::WU_E_DS_UNEXPECTED),
            WINDOWSUPDATE::WU_E_INVENTORY_PARSEFAILED => RawError::Kind(WINDOWSUPDATE::WU_E_INVENTORY_PARSEFAILED),
            WINDOWSUPDATE::WU_E_INVENTORY_GET_INVENTORY_TYPE_FAILED => RawError::Kind(WINDOWSUPDATE::WU_E_INVENTORY_GET_INVENTORY_TYPE_FAILED),
            WINDOWSUPDATE::WU_E_INVENTORY_RESULT_UPLOAD_FAILED => RawError::Kind(WINDOWSUPDATE::WU_E_INVENTORY_RESULT_UPLOAD_FAILED),
            WINDOWSUPDATE::WU_E_INVENTORY_UNEXPECTED => RawError::Kind(WINDOWSUPDATE::WU_E_INVENTORY_UNEXPECTED),
            WINDOWSUPDATE::WU_E_INVENTORY_WMI_ERROR => RawError::Kind(WINDOWSUPDATE::WU_E_INVENTORY_WMI_ERROR),
            WINDOWSUPDATE::WU_E_AU_NOSERVICE => RawError::Kind(WINDOWSUPDATE::WU_E_AU_NOSERVICE),
            WINDOWSUPDATE::WU_E_AU_NONLEGACYSERVER => RawError::Kind(WINDOWSUPDATE::WU_E_AU_NONLEGACYSERVER),
            WINDOWSUPDATE::WU_E_AU_LEGACYCLIENTDISABLED => RawError::Kind(WINDOWSUPDATE::WU_E_AU_LEGACYCLIENTDISABLED),
            WINDOWSUPDATE::WU_E_AU_PAUSED => RawError::Kind(WINDOWSUPDATE::WU_E_AU_PAUSED),
            WINDOWSUPDATE::WU_E_AU_NO_REGISTERED_SERVICE => RawError::Kind(WINDOWSUPDATE::WU_E_AU_NO_REGISTERED_SERVICE),
            WINDOWSUPDATE::WU_E_AU_UNEXPECTED => RawError::Kind(WINDOWSUPDATE::WU_E_AU_UNEXPECTED),
            WINDOWSUPDATE::WU_E_DRV_PRUNED => RawError::Kind(WINDOWSUPDATE::WU_E_DRV_PRUNED),
            WINDOWSUPDATE::WU_E_DRV_NOPROP_OR_LEGACY => RawError::Kind(WINDOWSUPDATE::WU_E_DRV_NOPROP_OR_LEGACY),
            WINDOWSUPDATE::WU_E_DRV_REG_MISMATCH => RawError::Kind(WINDOWSUPDATE::WU_E_DRV_REG_MISMATCH),
            WINDOWSUPDATE::WU_E_DRV_NO_METADATA => RawError::Kind(WINDOWSUPDATE::WU_E_DRV_NO_METADATA),
            WINDOWSUPDATE::WU_E_DRV_MISSING_ATTRIBUTE => RawError::Kind(WINDOWSUPDATE::WU_E_DRV_MISSING_ATTRIBUTE),
            WINDOWSUPDATE::WU_E_DRV_SYNC_FAILED => RawError::Kind(WINDOWSUPDATE::WU_E_DRV_SYNC_FAILED),
            WINDOWSUPDATE::WU_E_DRV_NO_PRINTER_CONTENT => RawError::Kind(WINDOWSUPDATE::WU_E_DRV_NO_PRINTER_CONTENT),
            WINDOWSUPDATE::WU_E_DRV_UNEXPECTED => RawError::Kind(WINDOWSUPDATE::WU_E_DRV_UNEXPECTED),
            WINDOWSUPDATE::WU_E_SETUP_INVALID_INFDATA => RawError::Kind(WINDOWSUPDATE::WU_E_SETUP_INVALID_INFDATA),
            WINDOWSUPDATE::WU_E_SETUP_INVALID_IDENTDATA => RawError::Kind(WINDOWSUPDATE::WU_E_SETUP_INVALID_IDENTDATA),
            WINDOWSUPDATE::WU_E_SETUP_ALREADY_INITIALIZED => RawError::Kind(WINDOWSUPDATE::WU_E_SETUP_ALREADY_INITIALIZED),
            WINDOWSUPDATE::WU_E_SETUP_NOT_INITIALIZED => RawError::Kind(WINDOWSUPDATE::WU_E_SETUP_NOT_INITIALIZED),
            WINDOWSUPDATE::WU_E_SETUP_SOURCE_VERSION_MISMATCH => RawError::Kind(WINDOWSUPDATE::WU_E_SETUP_SOURCE_VERSION_MISMATCH),
            WINDOWSUPDATE::WU_E_SETUP_TARGET_VERSION_GREATER => RawError::Kind(WINDOWSUPDATE::WU_E_SETUP_TARGET_VERSION_GREATER),
            WINDOWSUPDATE::WU_E_SETUP_REGISTRATION_FAILED => RawError::Kind(WINDOWSUPDATE::WU_E_SETUP_REGISTRATION_FAILED),
            WINDOWSUPDATE::WU_E_SELFUPDATE_SKIP_ON_FAILURE => RawError::Kind(WINDOWSUPDATE::WU_E_SELFUPDATE_SKIP_ON_FAILURE),
            WINDOWSUPDATE::WU_E_SETUP_SKIP_UPDATE => RawError::Kind(WINDOWSUPDATE::WU_E_SETUP_SKIP_UPDATE),
            WINDOWSUPDATE::WU_E_SETUP_UNSUPPORTED_CONFIGURATION => RawError::Kind(WINDOWSUPDATE::WU_E_SETUP_UNSUPPORTED_CONFIGURATION),
            WINDOWSUPDATE::WU_E_SETUP_BLOCKED_CONFIGURATION => RawError::Kind(WINDOWSUPDATE::WU_E_SETUP_BLOCKED_CONFIGURATION),
            WINDOWSUPDATE::WU_E_SETUP_REBOOT_TO_FIX => RawError::Kind(WINDOWSUPDATE::WU_E_SETUP_REBOOT_TO_FIX),
            WINDOWSUPDATE::WU_E_SETUP_ALREADYRUNNING => RawError::Kind(WINDOWSUPDATE::WU_E_SETUP_ALREADYRUNNING),
            WINDOWSUPDATE::WU_E_SETUP_REBOOTREQUIRED => RawError::Kind(WINDOWSUPDATE::WU_E_SETUP_REBOOTREQUIRED),
            WINDOWSUPDATE::WU_E_SETUP_HANDLER_EXEC_FAILURE => RawError::Kind(WINDOWSUPDATE::WU_E_SETUP_HANDLER_EXEC_FAILURE),
            WINDOWSUPDATE::WU_E_SETUP_INVALID_REGISTRY_DATA => RawError::Kind(WINDOWSUPDATE::WU_E_SETUP_INVALID_REGISTRY_DATA),
            WINDOWSUPDATE::WU_E_SELFUPDATE_REQUIRED => RawError::Kind(WINDOWSUPDATE::WU_E_SELFUPDATE_REQUIRED),
            WINDOWSUPDATE::WU_E_SELFUPDATE_REQUIRED_ADMIN => RawError::Kind(WINDOWSUPDATE::WU_E_SELFUPDATE_REQUIRED_ADMIN),
            WINDOWSUPDATE::WU_E_SETUP_WRONG_SERVER_VERSION => RawError::Kind(WINDOWSUPDATE::WU_E_SETUP_WRONG_SERVER_VERSION),
            WINDOWSUPDATE::WU_E_SETUP_UNEXPECTED => RawError::Kind(WINDOWSUPDATE::WU_E_SETUP_UNEXPECTED),
            WINDOWSUPDATE::WU_E_EE_UNKNOWN_EXPRESSION => RawError::Kind(WINDOWSUPDATE::WU_E_EE_UNKNOWN_EXPRESSION),
            WINDOWSUPDATE::WU_E_EE_INVALID_EXPRESSION => RawError::Kind(WINDOWSUPDATE::WU_E_EE_INVALID_EXPRESSION),
            WINDOWSUPDATE::WU_E_EE_MISSING_METADATA => RawError::Kind(WINDOWSUPDATE::WU_E_EE_MISSING_METADATA),
            WINDOWSUPDATE::WU_E_EE_INVALID_VERSION => RawError::Kind(WINDOWSUPDATE::WU_E_EE_INVALID_VERSION),
            WINDOWSUPDATE::WU_E_EE_NOT_INITIALIZED => RawError::Kind(WINDOWSUPDATE::WU_E_EE_NOT_INITIALIZED),
            WINDOWSUPDATE::WU_E_EE_INVALID_ATTRIBUTEDATA => RawError::Kind(WINDOWSUPDATE::WU_E_EE_INVALID_ATTRIBUTEDATA),
            WINDOWSUPDATE::WU_E_EE_CLUSTER_ERROR => RawError::Kind(WINDOWSUPDATE::WU_E_EE_CLUSTER_ERROR),
            WINDOWSUPDATE::WU_E_EE_UNEXPECTED => RawError::Kind(WINDOWSUPDATE::WU_E_EE_UNEXPECTED),
            WINDOWSUPDATE::WU_E_REPORTER_EVENTCACHECORRUPT => RawError::Kind(WINDOWSUPDATE::WU_E_REPORTER_EVENTCACHECORRUPT),
            WINDOWSUPDATE::WU_E_REPORTER_EVENTNAMESPACEPARSEFAILED => RawError::Kind(WINDOWSUPDATE::WU_E_REPORTER_EVENTNAMESPACEPARSEFAILED),
            WINDOWSUPDATE::WU_E_INVALID_EVENT => RawError::Kind(WINDOWSUPDATE::WU_E_INVALID_EVENT),
            WINDOWSUPDATE::WU_E_SERVER_BUSY => RawError::Kind(WINDOWSUPDATE::WU_E_SERVER_BUSY),
            WINDOWSUPDATE::WU_E_REPORTER_UNEXPECTED => RawError::Kind(WINDOWSUPDATE::WU_E_REPORTER_UNEXPECTED),
        }
    }

    pub fn description(&self) -> &'static str {
        return match self {
            WINDOWSUPDATE::WU_S_SERVICE_STOP => "Windows Update Agent was stopped successfully.",
            WINDOWSUPDATE::WU_S_SELFUPDATE => "Windows Update Agent updated itself.",
            WINDOWSUPDATE::WU_S_UPDATE_ERROR => "Operation completed successfully but there were errors applying the updates.",
            WINDOWSUPDATE::WU_S_MARKED_FOR_DISCONNECT => "A callback was marked to be disconnected later because the request to disconnect the operation came while a callback was running.",
            WINDOWSUPDATE::WU_S_REBOOT_REQUIRED => "The system must be restarted to complete update installation.",
            WINDOWSUPDATE::WU_S_ALREADY_INSTALLED => "The update to be installed is already installed on the system.",
            WINDOWSUPDATE::WU_S_ALREADY_UNINSTALLED => "The update to be removed is not installed on the system.",
            WINDOWSUPDATE::WU_S_ALREADY_DOWNLOADED => "The update to be downloaded has already been downloaded.",
            WINDOWSUPDATE::WU_E_NO_SERVICE => "Windows Update Agent was unable to provide the service.",
            WINDOWSUPDATE::WU_E_MAX_CAPACITY_REACHED => "The maximum capacity of the service was exceeded.",
            WINDOWSUPDATE::WU_E_UNKNOWN_ID => "An ID cannot be found.",
            WINDOWSUPDATE::WU_E_NOT_INITIALIZED => "The object could not be initialized.",
            WINDOWSUPDATE::WU_E_RANGEOVERLAP => "The update handler requested a byte range that overlaps a previously requested range.",
            WINDOWSUPDATE::WU_E_TOOMANYRANGES => "The requested number of byte ranges exceeds the maximum number (2^31 - 1).",
            WINDOWSUPDATE::WU_E_INVALIDINDEX => "The index to a collection was invalid.",
            WINDOWSUPDATE::WU_E_ITEMNOTFOUND => "The key for the item queried could not be found.",
            WINDOWSUPDATE::WU_E_OPERATIONINPROGRESS => "A conflicting operation was in progress. Some operations (such as installation) cannot be performed simultaneously.",
            WINDOWSUPDATE::WU_E_COULDNOTCANCEL => "Cancellation of the operation was not allowed.",
            WINDOWSUPDATE::WU_E_CALL_CANCELLED => "Operation was cancelled.",
            WINDOWSUPDATE::WU_E_NOOP => "No operation was required.",
            WINDOWSUPDATE::WU_E_XML_MISSINGDATA => "Windows Update Agent could not find the required information in the update's XML data.",
            WINDOWSUPDATE::WU_E_XML_INVALID => "Windows Update Agent found invalid information in the update's XML data.",
            WINDOWSUPDATE::WU_E_CYCLE_DETECTED => "Circular update relationships were detected in the metadata.",
            WINDOWSUPDATE::WU_E_TOO_DEEP_RELATION => "Update relationships that are too deep were evaluated.",
            WINDOWSUPDATE::WU_E_INVALID_RELATIONSHIP => "An invalid update relationship was detected.",
            WINDOWSUPDATE::WU_E_REG_VALUE_INVALID => "An invalid registry value was read.",
            WINDOWSUPDATE::WU_E_DUPLICATE_ITEM => "Operation tried to add a duplicate item to a list.",
            WINDOWSUPDATE::WU_E_INSTALL_NOT_ALLOWED => "Operation tried to install while another installation was in progress or the system was pending a mandatory restart.",
            WINDOWSUPDATE::WU_E_NOT_APPLICABLE => "Operation was not performed because there are no applicable updates.",
            WINDOWSUPDATE::WU_E_NO_USERTOKEN => "Operation failed because a required user token is missing.",
            WINDOWSUPDATE::WU_E_EXCLUSIVE_INSTALL_CONFLICT => "An exclusive update cannot be installed with other updates at the same time.",
            WINDOWSUPDATE::WU_E_POLICY_NOT_SET => "A policy value was not set.",
            WINDOWSUPDATE::WU_E_SELFUPDATE_IN_PROGRESS => "The operation could not be performed because the Windows Update Agent is self-updating.",
            WINDOWSUPDATE::WU_E_INVALID_UPDATE => "An update contains invalid metadata.",
            WINDOWSUPDATE::WU_E_SERVICE_STOP => "Operation did not complete because the service or system was being shut down.",
            WINDOWSUPDATE::WU_E_NO_CONNECTION => "Operation did not complete because the network connection was unavailable.",
            WINDOWSUPDATE::WU_E_NO_INTERACTIVE_USER => "Operation did not complete because there is no logged-on interactive user.",
            WINDOWSUPDATE::WU_E_TIME_OUT => "Operation did not complete because it timed out.",
            WINDOWSUPDATE::WU_E_ALL_UPDATES_FAILED => "Operation failed for all the updates.",
            WINDOWSUPDATE::WU_E_EULAS_DECLINED => "The license terms for all updates were declined.",
            WINDOWSUPDATE::WU_E_NO_UPDATE => "There are no updates.",
            WINDOWSUPDATE::WU_E_USER_ACCESS_DISABLED => "Group Policy settings prevented access to Windows Update.",
            WINDOWSUPDATE::WU_E_INVALID_UPDATE_TYPE => "The type of update is invalid.",
            WINDOWSUPDATE::WU_E_URL_TOO_LONG => "The URL exceeded the maximum length.",
            WINDOWSUPDATE::WU_E_UNINSTALL_NOT_ALLOWED => "The update could not be uninstalled because the request did not originate from a WSUS server.",
            WINDOWSUPDATE::WU_E_INVALID_PRODUCT_LICENSE => "Search may have missed some updates before there is an unlicensed application on the system.",
            WINDOWSUPDATE::WU_E_MISSING_HANDLER => "A component that is required to detect applicable updates was missing.",
            WINDOWSUPDATE::WU_E_LEGACYSERVER => "An operation did not complete because it requires a newer version of server software.",
            WINDOWSUPDATE::WU_E_BIN_SOURCE_ABSENT => "A delta-compressed update could not be installed because it required the source.",
            WINDOWSUPDATE::WU_E_SOURCE_ABSENT => "A full-file update could not be installed because it required the source.",
            WINDOWSUPDATE::WU_E_WU_DISABLED => "Access to an unmanaged server is not allowed.",
            WINDOWSUPDATE::WU_E_CALL_CANCELLED_BY_POLICY => "Operation did not complete because the DisableWindowsUpdateAccess policy was set.",
            WINDOWSUPDATE::WU_E_INVALID_PROXY_SERVER => "The format of the proxy list was invalid.",
            WINDOWSUPDATE::WU_E_INVALID_FILE => "The file is in the wrong format.",
            WINDOWSUPDATE::WU_E_INVALID_CRITERIA => "The search criteria string was invalid.",
            WINDOWSUPDATE::WU_E_EULA_UNAVAILABLE => "License terms could not be downloaded.",
            WINDOWSUPDATE::WU_E_DOWNLOAD_FAILED => "Update failed to download.",
            WINDOWSUPDATE::WU_E_UPDATE_NOT_PROCESSED => "The update was not processed.",
            WINDOWSUPDATE::WU_E_INVALID_OPERATION => "The object's current state did not allow the operation.",
            WINDOWSUPDATE::WU_E_NOT_SUPPORTED => "The functionality for the operation is not supported.",
            WINDOWSUPDATE::WU_E_WINHTTP_INVALID_FILE => "The downloaded file has an unexpected content type.",
            WINDOWSUPDATE::WU_E_TOO_MANY_RESYNC => "The agent was asked by server to synchronize too many times.",
            WINDOWSUPDATE::WU_E_NO_SERVER_CORE_SUPPORT => "WUA API method does not run on a Server Core installation option of the Windows 2008 R2 operating system.",
            WINDOWSUPDATE::WU_E_SYSPREP_IN_PROGRESS => "Service is not available when sysprep is running.",
            WINDOWSUPDATE::WU_E_UNKNOWN_SERVICE => "The update service is no longer registered with Automatic Updates.",
            WINDOWSUPDATE::WU_E_UNEXPECTED => "An operation failed due to reasons not covered by another error code.",
            WINDOWSUPDATE::WU_E_MSI_WRONG_VERSION => "Search may have missed some updates because Windows Installer is less than version 3.1.",
            WINDOWSUPDATE::WU_E_MSI_NOT_CONFIGURED => "Search may have missed some updates because Windows Installer is not configured.",
            WINDOWSUPDATE::WU_E_MSP_DISABLED => "Search may have missed some updates because a policy setting disabled Windows Installer patching.",
            WINDOWSUPDATE::WU_E_MSI_WRONG_APP_CONTEXT => "An update could not be applied because the application is installed per-user.",
            WINDOWSUPDATE::WU_E_MSP_UNEXPECTED => "Search may have missed some updates because there was a failure of Windows Installer.",
            WINDOWSUPDATE::WU_E_UH_REMOTEUNAVAILABLE => "A request for a remote update handler could not be completed because no remote process is available.",
            WINDOWSUPDATE::WU_E_UH_LOCALONLY => "A request for a remote update handler could not be completed because the handler is local only.",
            WINDOWSUPDATE::WU_E_UH_UNKNOWNHANDLER => "A request for an update handler could not be completed because the handler could not be recognized.",
            WINDOWSUPDATE::WU_E_UH_REMOTEALREADYACTIVE => "A remote update handler could not be created because one already exists.",
            WINDOWSUPDATE::WU_E_UH_DOESNOTSUPPORTACTION => "A request for the handler to install (uninstall) an update could not be completed because the update does not support install (uninstall).",
            WINDOWSUPDATE::WU_E_UH_WRONGHANDLER => "An operation did not complete because the wrong handler was specified.",
            WINDOWSUPDATE::WU_E_UH_INVALIDMETADATA => "A handler operation could not be completed because the update contains invalid metadata.",
            WINDOWSUPDATE::WU_E_UH_INSTALLERHUNG => "An operation could not be completed because the installer exceeded the time limit.",
            WINDOWSUPDATE::WU_E_UH_OPERATIONCANCELLED => "An operation being done by the update handler was cancelled.",
            WINDOWSUPDATE::WU_E_UH_BADHANDLERXML => "An operation could not be completed because the handler-specific metadata is invalid.",
            WINDOWSUPDATE::WU_E_UH_CANREQUIREINPUT => "A request to the handler to install an update could not be completed because the update requires user input.",
            WINDOWSUPDATE::WU_E_UH_INSTALLERFAILURE => "The installer failed to install (uninstall) one or more updates.",
            WINDOWSUPDATE::WU_E_UH_FALLBACKTOSELFCONTAINED => "The update handler should download self-contained content rather than delta-compressed content for the update.",
            WINDOWSUPDATE::WU_E_UH_NEEDANOTHERDOWNLOAD => "The update handler did not install the update because the update needs to be downloaded again.",
            WINDOWSUPDATE::WU_E_UH_NOTIFYFAILURE => "The update handler failed to send notification of the status of the install (uninstall) operation.",
            WINDOWSUPDATE::WU_E_UH_INCONSISTENT_FILE_NAMES => "The file names in the update metadata are inconsistent with the file names in the update package.",
            WINDOWSUPDATE::WU_E_UH_FALLBACKERROR => "The update handler failed to fall back to the self-contained content.",
            WINDOWSUPDATE::WU_E_UH_TOOMANYDOWNLOADREQUESTS => "The update handler has exceeded the maximum number of download requests.",
            WINDOWSUPDATE::WU_E_UH_UNEXPECTEDCBSRESPONSE => "The update handler has received an unexpected response from CBS.",
            WINDOWSUPDATE::WU_E_UH_BADCBSPACKAGEID => "The update metadata contains an invalid CBS package identifier.",
            WINDOWSUPDATE::WU_E_UH_POSTREBOOTSTILLPENDING => "The post-reboot operation for the update is still in progress.",
            WINDOWSUPDATE::WU_E_UH_POSTREBOOTRESULTUNKNOWN => "The result of the post-reboot operation for the update could not be determined.",
            WINDOWSUPDATE::WU_E_UH_POSTREBOOTUNEXPECTEDSTATE => "The state of the update after its post-reboot operation has completed is unexpectedly.",
            WINDOWSUPDATE::WU_E_UH_NEW_SERVICING_STACK_REQUIRED => "The operating system servicing stack must be updated before this update is downloaded or installed.",
            WINDOWSUPDATE::WU_E_UH_UNEXPECTED => "This update handler error is not covered by another WU_E_UH_* code.",
            WINDOWSUPDATE::WU_E_INSTALLATION_RESULTS_UNKNOWN_VERSION => "The results of the download and installation could not be read in the registry due to an unrecognized data format version.",
            WINDOWSUPDATE::WU_E_INSTALLATION_RESULTS_INVALID_DATA => "The results of download and installation could not be read in the registry due to an invalid data format.",
            WINDOWSUPDATE::WU_E_INSTALLATION_RESULTS_NOT_FOUND => "The results of download and installation are not available; the operation may have failed to start.",
            WINDOWSUPDATE::WU_E_TRAYICON_FAILURE => "A failure occurred when trying to create an icon in the notification area.",
            WINDOWSUPDATE::WU_E_NON_UI_MODE => "Unable to show the user interface (UI) when in a non-UI mode; Windows Update (WU) client UI modules may not be installed.",
            WINDOWSUPDATE::WU_E_WUCLTUI_UNSUPPORTED_VERSION => "Unsupported version of WU client UI exported functions.",
            WINDOWSUPDATE::WU_E_AUCLIENT_UNEXPECTED => "There was a user interface error not covered by another WU_E_AUCLIENT_* error code.",
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_BASE => "WU_E_PT_SOAPCLIENT_* error codes map to the SOAPCLIENT_ERROR enum of the ATL Server Library.",
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_INITIALIZE => "Initialization of the SOAP client failed, possibly because of an MSXML installation failure.",
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_OUTOFMEMORY => "SOAP client failed because it ran out of memory.",
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_GENERATE => "SOAP client failed to generate the request.",
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_CONNECT => "SOAP client failed to connect to the server.",
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_SEND => "SOAP client failed to send a message due to WU_E_WINHTTP_* error codes.",
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_SERVER => "SOAP client failed because there was a server error.",
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_SOAPFAULT => "SOAP client failed because there was a SOAP fault due to WU_E_PT_SOAP_* error codes.",
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_PARSEFAULT => "SOAP client failed to parse a SOAP fault.",
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_READ => "SOAP client failed while reading the response from the server.",
            WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_PARSE => "SOAP client failed to parse the response from the server.",
            WINDOWSUPDATE::WU_E_PT_SOAP_VERSION => "SOAP client found an unrecognizable namespace for the SOAP envelope.",
            WINDOWSUPDATE::WU_E_PT_SOAP_MUST_UNDERSTAND => "SOAP client was unable to understand a header.",
            WINDOWSUPDATE::WU_E_PT_SOAP_CLIENT => "SOAP client found the message was malformed (fix before resending).",
            WINDOWSUPDATE::WU_E_PT_SOAP_SERVER => "The SOAP message could not be processed due to a server error (resend later).",
            WINDOWSUPDATE::WU_E_PT_WMI_ERROR => "There was an unspecified Windows Management Instrumentation (WMI) error.",
            WINDOWSUPDATE::WU_E_PT_EXCEEDED_MAX_SERVER_TRIPS => "The number of round trips to the server exceeded the maximum limit.",
            WINDOWSUPDATE::WU_E_PT_SUS_SERVER_NOT_SET => "WUServer policy value is missing in the registry.",
            WINDOWSUPDATE::WU_E_PT_DOUBLE_INITIALIZATION => "Initialization failed because the object was already initialized.",
            WINDOWSUPDATE::WU_E_PT_INVALID_COMPUTER_NAME => "The computer name could not be determined.",
            WINDOWSUPDATE::WU_E_PT_REFRESH_CACHE_REQUIRED => "The reply from the server indicates that the server was changed or the cookie was invalid; refresh the state of the internal cache and retry.",
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_BAD_REQUEST => "HTTP 400 - the server could not process the request due to invalid syntax.",
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_DENIED => "HTTP 401 - the requested resource requires user authentication.",
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_FORBIDDEN => "HTTP 403 - server understood the request, but declined to fulfill it.",
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_NOT_FOUND => "HTTP 404 - the server cannot find the requested Uniform Resource Identifier (URI).",
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_BAD_METHOD => "HTTP 405 - the HTTP method is not allowed.",
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_PROXY_AUTH_REQ => "HTTP 407 - proxy authentication is required.",
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_REQUEST_TIMEOUT => "HTTP 408 - the server timed out waiting for the request.",
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_CONFLICT => "HTTP 409 - the request was not completed due to a conflict with the current state of the resource.",
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_GONE => "HTTP 410 - the requested resource is no longer available at the server.",
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_SERVER_ERROR => "HTTP 500 - an error internal to the server prevented fulfilling the request.",
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_NOT_SUPPORTED => "HTTP 501 - server does not support the functionality that is required to fulfill the request.",
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_BAD_GATEWAY => "HTTP 502 - the server, while acting as a gateway or proxy, received an invalid response from the upstream server it accessed when attempting to fulfill the request.",
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_SERVICE_UNAVAIL => "HTTP 503 - the service is temporarily overloaded.",
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_GATEWAY_TIMEOUT => "HTTP 504 - the request was timed out waiting for a gateway.",
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_VERSION_NOT_SUP => "HTTP 505 - the server does not support the HTTP protocol version used for the request.",
            WINDOWSUPDATE::WU_E_PT_FILE_LOCATIONS_CHANGED => "Operation failed due to a changed file location; refresh internal state and resend.",
            WINDOWSUPDATE::WU_E_PT_REGISTRATION_NOT_SUPPORTED => "Operation failed because Windows Update Agent does not support registration with a non-WSUS server.",
            WINDOWSUPDATE::WU_E_PT_NO_AUTH_PLUGINS_REQUESTED => "The server returned an empty authentication information list.",
            WINDOWSUPDATE::WU_E_PT_NO_AUTH_COOKIES_CREATED => "Windows Update Agent was unable to create any valid authentication cookies.",
            WINDOWSUPDATE::WU_E_PT_INVALID_CONFIG_PROP => "A configuration property value was wrong.",
            WINDOWSUPDATE::WU_E_PT_CONFIG_PROP_MISSING => "A configuration property value was missing.",
            WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_NOT_MAPPED => "The HTTP request could not be completed and the reason did not correspond to any of the WU_E_PT_HTTP_* error codes.",
            WINDOWSUPDATE::WU_E_PT_WINHTTP_NAME_NOT_RESOLVED => "The proxy server or target server name cannot be resolved.",
            WINDOWSUPDATE::WU_E_PT_ECP_SUCCEEDED_WITH_ERRORS => "External .cab file processing completed with some errors.",
            WINDOWSUPDATE::WU_E_PT_ECP_INIT_FAILED => "The external .cab file processor initialization did not complete.",
            WINDOWSUPDATE::WU_E_PT_ECP_INVALID_FILE_FORMAT => "The format of a metadata file was invalid.",
            WINDOWSUPDATE::WU_E_PT_ECP_INVALID_METADATA => "External .cab file processor found invalid metadata.",
            WINDOWSUPDATE::WU_E_PT_ECP_FAILURE_TO_EXTRACT_DIGEST => "The file digest could not be extracted from an external .cab file.",
            WINDOWSUPDATE::WU_E_PT_ECP_FAILURE_TO_DECOMPRESS_CAB_FILE => "An external .cab file could not be decompressed.",
            WINDOWSUPDATE::WU_E_PT_ECP_FILE_LOCATION_ERROR => "External .cab processor was unable to get file locations.",
            WINDOWSUPDATE::WU_E_PT_UNEXPECTED => "There was a communication error not covered by another WU_E_PT_* error code",
            WINDOWSUPDATE::WU_E_REDIRECTOR_LOAD_XML => "The redirector XML document could not be loaded into the Document Object Model (DOM) class.",
            WINDOWSUPDATE::WU_E_REDIRECTOR_S_FALSE => "The redirector XML document is missing some required information.",
            WINDOWSUPDATE::WU_E_REDIRECTOR_ID_SMALLER => "The redirector ID in the downloaded redirector .cab file is less than in the cached .cab file.",
            WINDOWSUPDATE::WU_E_PT_SAME_REDIR_ID => "Windows Update Agent failed to download a redirector .cab file with a new redirector ID value from the server during the recovery.",
            WINDOWSUPDATE::WU_E_PT_NO_MANAGED_RECOVER => "A redirector recovery action did not complete because the server is managed.",
            WINDOWSUPDATE::WU_E_REDIRECTOR_UNEXPECTED => "The redirector failed for reasons not covered by another WU_E_REDIRECTOR_* error code.",
            WINDOWSUPDATE::WU_E_DM_URLNOTAVAILABLE => "A download manager operation could not be completed because the requested file does not have a URL.",
            WINDOWSUPDATE::WU_E_DM_INCORRECTFILEHASH => "A download manager operation could not be completed because the file digest was not recognized.",
            WINDOWSUPDATE::WU_E_DM_UNKNOWNALGORITHM => "A download manager operation could not be completed because the file metadata requested an unrecognized hash algorithm.",
            WINDOWSUPDATE::WU_E_DM_NEEDDOWNLOADREQUEST => "An operation could not be completed because a download request is required from the download handler.",
            WINDOWSUPDATE::WU_E_DM_NONETWORK => "A download manager operation could not be completed because the network connection was unavailable.",
            WINDOWSUPDATE::WU_E_DM_WRONGBITSVERSION => "A download manager operation could not be completed because the version of Background Intelligent Transfer Service (BITS) is incompatible.",
            WINDOWSUPDATE::WU_E_DM_NOTDOWNLOADED => "The update has not been downloaded.",
            WINDOWSUPDATE::WU_E_DM_FAILTOCONNECTTOBITS => "A download manager operation failed because the download manager was unable to connect the Background Intelligent Transfer Service (BITS).",
            WINDOWSUPDATE::WU_E_DM_BITSTRANSFERERROR => "A download manager operation failed because there was an unspecified Background Intelligent Transfer Service (BITS) transfer error.",
            WINDOWSUPDATE::WU_E_DM_DOWNLOADLOCATIONCHANGED => "A download must be restarted because the location of the source of the download has changed.",
            WINDOWSUPDATE::WU_E_DM_CONTENTCHANGED => "A download must be restarted because the update content changed in a new revision.",
            WINDOWSUPDATE::WU_E_DM_UNEXPECTED => "There was a download manager error not covered by another WU_E_DM_* error code.",
            WINDOWSUPDATE::WU_E_OL_INVALID_SCANFILE => "An operation could not be completed because the scan package was invalid.",
            WINDOWSUPDATE::WU_E_OL_NEWCLIENT_REQUIRED => "An operation could not be completed because the scan package requires a greater version of the Windows Update Agent.",
            WINDOWSUPDATE::WU_E_OL_UNEXPECTED => "Search using the scan package failed.",
            WINDOWSUPDATE::WU_E_DS_SHUTDOWN => "An operation failed because Windows Update Agent is shutting down.",
            WINDOWSUPDATE::WU_E_DS_INUSE => "An operation failed because the data store was in use.",
            WINDOWSUPDATE::WU_E_DS_INVALID => "The current and expected states of the data store do not match.",
            WINDOWSUPDATE::WU_E_DS_TABLEMISSING => "The data store is missing a table.",
            WINDOWSUPDATE::WU_E_DS_TABLEINCORRECT => "The data store contains a table with unexpected columns.",
            WINDOWSUPDATE::WU_E_DS_INVALIDTABLENAME => "A table could not be opened because the table is not in the data store.",
            WINDOWSUPDATE::WU_E_DS_BADVERSION => "The current and expected versions of the data store do not match.",
            WINDOWSUPDATE::WU_E_DS_NODATA => "The information requested is not in the data store.",
            WINDOWSUPDATE::WU_E_DS_MISSINGDATA => "The data store is missing required information or has a null value in a table column that requires a non-null value.",
            WINDOWSUPDATE::WU_E_DS_MISSINGREF => "The data store is missing required information or has a reference to missing license terms, a file, a localized property, or a linked row.",
            WINDOWSUPDATE::WU_E_DS_UNKNOWNHANDLER => "The update was not processed because its update handler could not be recognized.",
            WINDOWSUPDATE::WU_E_DS_CANTDELETE => "The update was not deleted because it is still referenced by one or more services.",
            WINDOWSUPDATE::WU_E_DS_LOCKTIMEOUTEXPIRED => "The data store section could not be locked within the allotted time.",
            WINDOWSUPDATE::WU_E_DS_NOCATEGORIES => "The category was not added because it contains no parent categories, and it is not a top-level category.",
            WINDOWSUPDATE::WU_E_DS_ROWEXISTS => "The row was not added because an existing row has the same primary key.",
            WINDOWSUPDATE::WU_E_DS_STOREFILELOCKED => "The data store could not be initialized because it was locked by another process.",
            WINDOWSUPDATE::WU_E_DS_CANNOTREGISTER => "The data store is not allowed to be registered with COM in the current process.",
            WINDOWSUPDATE::WU_E_DS_UNABLETOSTART => "Could not create a data store object in another process.",
            WINDOWSUPDATE::WU_E_DS_DUPLICATEUPDATEID => "The server sent the same update to the client computer, with two different revision IDs.",
            WINDOWSUPDATE::WU_E_DS_UNKNOWNSERVICE => "An operation did not complete because the service is not in the data store.",
            WINDOWSUPDATE::WU_E_DS_SERVICEEXPIRED => "An operation did not complete because the registration of the service has expired.",
            WINDOWSUPDATE::WU_E_DS_DECLINENOTALLOWED => "A request to hide an update was declined because it is a mandatory update or because it was deployed with a deadline.",
            WINDOWSUPDATE::WU_E_DS_TABLESESSIONMISMATCH => "A table was not closed because it is not associated with the session.",
            WINDOWSUPDATE::WU_E_DS_SESSIONLOCKMISMATCH => "A table was not closed because it is not associated with the session.",
            WINDOWSUPDATE::WU_E_DS_NEEDWINDOWSSERVICE => "A request to remove the Windows Update service or to unregister it with Automatic Updates was declined because it is a built-in service and Automatic Updates cannot fall back to another service.",
            WINDOWSUPDATE::WU_E_DS_INVALIDOPERATION => "A request was declined because the operation is not allowed.",
            WINDOWSUPDATE::WU_E_DS_SCHEMAMISMATCH => "The schema of the current data store and the schema of a table in a backup XML document do not match.",
            WINDOWSUPDATE::WU_E_DS_RESETREQUIRED => "The data store requires a session reset; release the session and retry with a new session.",
            WINDOWSUPDATE::WU_E_DS_IMPERSONATED => "A data store operation did not complete because it was requested with an impersonated identity.",
            WINDOWSUPDATE::WU_E_DS_UNEXPECTED => "There was a data store error not covered by another WU_E_DS_* code.",
            WINDOWSUPDATE::WU_E_INVENTORY_PARSEFAILED => "Parsing of the rule file failed.",
            WINDOWSUPDATE::WU_E_INVENTORY_GET_INVENTORY_TYPE_FAILED => "Failed to get the requested inventory type from the server.",
            WINDOWSUPDATE::WU_E_INVENTORY_RESULT_UPLOAD_FAILED => "Failed to upload inventory result to the server.",
            WINDOWSUPDATE::WU_E_INVENTORY_UNEXPECTED => "There was an inventory error not covered by another error code.",
            WINDOWSUPDATE::WU_E_INVENTORY_WMI_ERROR => "A WMI error occurred when enumerating the instances for a particular class.",
            WINDOWSUPDATE::WU_E_AU_NOSERVICE => "Automatic Updates was unable to service incoming requests.",
            WINDOWSUPDATE::WU_E_AU_NONLEGACYSERVER => "The old version of Automatic Updates has stopped because the WSUS server has been upgraded.",
            WINDOWSUPDATE::WU_E_AU_LEGACYCLIENTDISABLED => "The old version of Automatic Updates was disabled.",
            WINDOWSUPDATE::WU_E_AU_PAUSED => "Automatic Updates was unable to process incoming requests because it was paused.",
            WINDOWSUPDATE::WU_E_AU_NO_REGISTERED_SERVICE => "No unmanaged service is registered with AU.",
            WINDOWSUPDATE::WU_E_AU_UNEXPECTED => "There was an Automatic Updates error not covered by another WU_E_AU * code.",
            WINDOWSUPDATE::WU_E_DRV_PRUNED => "A driver was skipped.",
            WINDOWSUPDATE::WU_E_DRV_NOPROP_OR_LEGACY => "A property for the driver could not be found. It may not conform with required specifications.",
            WINDOWSUPDATE::WU_E_DRV_REG_MISMATCH => "The registry type read for the driver does not match the expected type.",
            WINDOWSUPDATE::WU_E_DRV_NO_METADATA => "The driver update is missing metadata.",
            WINDOWSUPDATE::WU_E_DRV_MISSING_ATTRIBUTE => "The driver update is missing a required attribute.",
            WINDOWSUPDATE::WU_E_DRV_SYNC_FAILED => "Driver synchronization failed.",
            WINDOWSUPDATE::WU_E_DRV_NO_PRINTER_CONTENT => "Information required for the synchronization of applicable printers is missing.",
            WINDOWSUPDATE::WU_E_DRV_UNEXPECTED => "There was a driver error not covered by another WU_E_DRV_* code.",
            WINDOWSUPDATE::WU_E_SETUP_INVALID_INFDATA => "Windows Update Agent could not be updated because an .inf file contains invalid information.",
            WINDOWSUPDATE::WU_E_SETUP_INVALID_IDENTDATA => "Windows Update Agent could not be updated because the wuident.cab file contains invalid information.",
            WINDOWSUPDATE::WU_E_SETUP_ALREADY_INITIALIZED => "Windows Update Agent could not be updated because of an internal error that caused setup initialization to be performed twice.",
            WINDOWSUPDATE::WU_E_SETUP_NOT_INITIALIZED => "Windows Update Agent could not be updated because setup initialization never completed successfully.",
            WINDOWSUPDATE::WU_E_SETUP_SOURCE_VERSION_MISMATCH => "Windows Update Agent could not be updated because the versions specified in the .inf file do not match the actual source file versions.",
            WINDOWSUPDATE::WU_E_SETUP_TARGET_VERSION_GREATER => "Windows Update Agent could not be updated because a Windows Update Agent file on the target system is newer than the corresponding source file.",
            WINDOWSUPDATE::WU_E_SETUP_REGISTRATION_FAILED => "Windows Update Agent could not be updated because regsvr32.exe returned an error.",
            WINDOWSUPDATE::WU_E_SELFUPDATE_SKIP_ON_FAILURE => "An update to the Windows Update Agent was skipped because previous attempts to update failed.",
            WINDOWSUPDATE::WU_E_SETUP_SKIP_UPDATE => "An update to the Windows Update Agent was skipped due to a directive in the wuident.cab file.",
            WINDOWSUPDATE::WU_E_SETUP_UNSUPPORTED_CONFIGURATION => "Windows Update Agent could not be updated because the current system configuration is not supported.",
            WINDOWSUPDATE::WU_E_SETUP_BLOCKED_CONFIGURATION => "Windows Update Agent could not be updated because the system is configured to block the update.",
            WINDOWSUPDATE::WU_E_SETUP_REBOOT_TO_FIX => "Windows Update Agent could not be updated because a restart of the system is required.",
            WINDOWSUPDATE::WU_E_SETUP_ALREADYRUNNING => "Windows Update Agent setup is already running.",
            WINDOWSUPDATE::WU_E_SETUP_REBOOTREQUIRED => "Windows Update Agent setup package requires a reboot to complete installation.",
            WINDOWSUPDATE::WU_E_SETUP_HANDLER_EXEC_FAILURE => "Windows Update Agent could not be updated because the setup handler failed when it was run.",
            WINDOWSUPDATE::WU_E_SETUP_INVALID_REGISTRY_DATA => "Windows Update Agent could not be updated because the registry contains invalid information.",
            WINDOWSUPDATE::WU_E_SELFUPDATE_REQUIRED => "Windows Update Agent must be updated before search can continue.",
            WINDOWSUPDATE::WU_E_SELFUPDATE_REQUIRED_ADMIN => "Windows Update Agent must be updated before search can continue. An administrator is required to perform the operation.",
            WINDOWSUPDATE::WU_E_SETUP_WRONG_SERVER_VERSION => "Windows Update Agent could not be updated because the server does not contain update information for this version.",
            WINDOWSUPDATE::WU_E_SETUP_UNEXPECTED => "Windows Update Agent could not be updated because of an error not covered by another WU_E_SETUP_* error code.",
            WINDOWSUPDATE::WU_E_EE_UNKNOWN_EXPRESSION => "An expression evaluator operation could not be completed because an expression was unrecognized.",
            WINDOWSUPDATE::WU_E_EE_INVALID_EXPRESSION => "An expression evaluator operation could not be completed because an expression was invalid.",
            WINDOWSUPDATE::WU_E_EE_MISSING_METADATA => "An expression evaluator operation could not be completed because an expression contains an incorrect number of metadata nodes.",
            WINDOWSUPDATE::WU_E_EE_INVALID_VERSION => "An expression evaluator operation could not be completed because the version of the serialized expression data is invalid.",
            WINDOWSUPDATE::WU_E_EE_NOT_INITIALIZED => "The expression evaluator could not be initialized.",
            WINDOWSUPDATE::WU_E_EE_INVALID_ATTRIBUTEDATA => "An expression evaluator operation could not be completed because there was an invalid attribute.",
            WINDOWSUPDATE::WU_E_EE_CLUSTER_ERROR => "An expression evaluator operation could not be completed because the cluster state of the computer could not be determined.",
            WINDOWSUPDATE::WU_E_EE_UNEXPECTED => "There was an expression evaluator error not covered by another WU_E_EE_* error code.",
            WINDOWSUPDATE::WU_E_REPORTER_EVENTCACHECORRUPT => "The event cache file was defective.",
            WINDOWSUPDATE::WU_E_REPORTER_EVENTNAMESPACEPARSEFAILED => "The XML in the event namespace descriptor could not be parsed.",
            WINDOWSUPDATE::WU_E_INVALID_EVENT => "The XML in the event namespace descriptor could not be parsed.",
            WINDOWSUPDATE::WU_E_SERVER_BUSY => "The server rejected an event because the server was too busy.",
            WINDOWSUPDATE::WU_E_REPORTER_UNEXPECTED => "There was a reporter error not covered by another error code.",
        }
    }

    pub fn from_name(name: &str) -> WINDOWSUPDATE {
        return match name {
            "WU_S_SERVICE_STOP" => WINDOWSUPDATE::WU_S_SERVICE_STOP,
            "WU_S_SELFUPDATE" => WINDOWSUPDATE::WU_S_SELFUPDATE,
            "WU_S_UPDATE_ERROR" => WINDOWSUPDATE::WU_S_UPDATE_ERROR,
            "WU_S_MARKED_FOR_DISCONNECT" => WINDOWSUPDATE::WU_S_MARKED_FOR_DISCONNECT,
            "WU_S_REBOOT_REQUIRED" => WINDOWSUPDATE::WU_S_REBOOT_REQUIRED,
            "WU_S_ALREADY_INSTALLED" => WINDOWSUPDATE::WU_S_ALREADY_INSTALLED,
            "WU_S_ALREADY_UNINSTALLED" => WINDOWSUPDATE::WU_S_ALREADY_UNINSTALLED,
            "WU_S_ALREADY_DOWNLOADED" => WINDOWSUPDATE::WU_S_ALREADY_DOWNLOADED,
            "WU_E_NO_SERVICE" => WINDOWSUPDATE::WU_E_NO_SERVICE,
            "WU_E_MAX_CAPACITY_REACHED" => WINDOWSUPDATE::WU_E_MAX_CAPACITY_REACHED,
            "WU_E_UNKNOWN_ID" => WINDOWSUPDATE::WU_E_UNKNOWN_ID,
            "WU_E_NOT_INITIALIZED" => WINDOWSUPDATE::WU_E_NOT_INITIALIZED,
            "WU_E_RANGEOVERLAP" => WINDOWSUPDATE::WU_E_RANGEOVERLAP,
            "WU_E_TOOMANYRANGES" => WINDOWSUPDATE::WU_E_TOOMANYRANGES,
            "WU_E_INVALIDINDEX" => WINDOWSUPDATE::WU_E_INVALIDINDEX,
            "WU_E_ITEMNOTFOUND" => WINDOWSUPDATE::WU_E_ITEMNOTFOUND,
            "WU_E_OPERATIONINPROGRESS" => WINDOWSUPDATE::WU_E_OPERATIONINPROGRESS,
            "WU_E_COULDNOTCANCEL" => WINDOWSUPDATE::WU_E_COULDNOTCANCEL,
            "WU_E_CALL_CANCELLED" => WINDOWSUPDATE::WU_E_CALL_CANCELLED,
            "WU_E_NOOP" => WINDOWSUPDATE::WU_E_NOOP,
            "WU_E_XML_MISSINGDATA" => WINDOWSUPDATE::WU_E_XML_MISSINGDATA,
            "WU_E_XML_INVALID" => WINDOWSUPDATE::WU_E_XML_INVALID,
            "WU_E_CYCLE_DETECTED" => WINDOWSUPDATE::WU_E_CYCLE_DETECTED,
            "WU_E_TOO_DEEP_RELATION" => WINDOWSUPDATE::WU_E_TOO_DEEP_RELATION,
            "WU_E_INVALID_RELATIONSHIP" => WINDOWSUPDATE::WU_E_INVALID_RELATIONSHIP,
            "WU_E_REG_VALUE_INVALID" => WINDOWSUPDATE::WU_E_REG_VALUE_INVALID,
            "WU_E_DUPLICATE_ITEM" => WINDOWSUPDATE::WU_E_DUPLICATE_ITEM,
            "WU_E_INSTALL_NOT_ALLOWED" => WINDOWSUPDATE::WU_E_INSTALL_NOT_ALLOWED,
            "WU_E_NOT_APPLICABLE" => WINDOWSUPDATE::WU_E_NOT_APPLICABLE,
            "WU_E_NO_USERTOKEN" => WINDOWSUPDATE::WU_E_NO_USERTOKEN,
            "WU_E_EXCLUSIVE_INSTALL_CONFLICT" => WINDOWSUPDATE::WU_E_EXCLUSIVE_INSTALL_CONFLICT,
            "WU_E_POLICY_NOT_SET" => WINDOWSUPDATE::WU_E_POLICY_NOT_SET,
            "WU_E_SELFUPDATE_IN_PROGRESS" => WINDOWSUPDATE::WU_E_SELFUPDATE_IN_PROGRESS,
            "WU_E_INVALID_UPDATE" => WINDOWSUPDATE::WU_E_INVALID_UPDATE,
            "WU_E_SERVICE_STOP" => WINDOWSUPDATE::WU_E_SERVICE_STOP,
            "WU_E_NO_CONNECTION" => WINDOWSUPDATE::WU_E_NO_CONNECTION,
            "WU_E_NO_INTERACTIVE_USER" => WINDOWSUPDATE::WU_E_NO_INTERACTIVE_USER,
            "WU_E_TIME_OUT" => WINDOWSUPDATE::WU_E_TIME_OUT,
            "WU_E_ALL_UPDATES_FAILED" => WINDOWSUPDATE::WU_E_ALL_UPDATES_FAILED,
            "WU_E_EULAS_DECLINED" => WINDOWSUPDATE::WU_E_EULAS_DECLINED,
            "WU_E_NO_UPDATE" => WINDOWSUPDATE::WU_E_NO_UPDATE,
            "WU_E_USER_ACCESS_DISABLED" => WINDOWSUPDATE::WU_E_USER_ACCESS_DISABLED,
            "WU_E_INVALID_UPDATE_TYPE" => WINDOWSUPDATE::WU_E_INVALID_UPDATE_TYPE,
            "WU_E_URL_TOO_LONG" => WINDOWSUPDATE::WU_E_URL_TOO_LONG,
            "WU_E_UNINSTALL_NOT_ALLOWED" => WINDOWSUPDATE::WU_E_UNINSTALL_NOT_ALLOWED,
            "WU_E_INVALID_PRODUCT_LICENSE" => WINDOWSUPDATE::WU_E_INVALID_PRODUCT_LICENSE,
            "WU_E_MISSING_HANDLER" => WINDOWSUPDATE::WU_E_MISSING_HANDLER,
            "WU_E_LEGACYSERVER" => WINDOWSUPDATE::WU_E_LEGACYSERVER,
            "WU_E_BIN_SOURCE_ABSENT" => WINDOWSUPDATE::WU_E_BIN_SOURCE_ABSENT,
            "WU_E_SOURCE_ABSENT" => WINDOWSUPDATE::WU_E_SOURCE_ABSENT,
            "WU_E_WU_DISABLED" => WINDOWSUPDATE::WU_E_WU_DISABLED,
            "WU_E_CALL_CANCELLED_BY_POLICY" => WINDOWSUPDATE::WU_E_CALL_CANCELLED_BY_POLICY,
            "WU_E_INVALID_PROXY_SERVER" => WINDOWSUPDATE::WU_E_INVALID_PROXY_SERVER,
            "WU_E_INVALID_FILE" => WINDOWSUPDATE::WU_E_INVALID_FILE,
            "WU_E_INVALID_CRITERIA" => WINDOWSUPDATE::WU_E_INVALID_CRITERIA,
            "WU_E_EULA_UNAVAILABLE" => WINDOWSUPDATE::WU_E_EULA_UNAVAILABLE,
            "WU_E_DOWNLOAD_FAILED" => WINDOWSUPDATE::WU_E_DOWNLOAD_FAILED,
            "WU_E_UPDATE_NOT_PROCESSED" => WINDOWSUPDATE::WU_E_UPDATE_NOT_PROCESSED,
            "WU_E_INVALID_OPERATION" => WINDOWSUPDATE::WU_E_INVALID_OPERATION,
            "WU_E_NOT_SUPPORTED" => WINDOWSUPDATE::WU_E_NOT_SUPPORTED,
            "WU_E_WINHTTP_INVALID_FILE" => WINDOWSUPDATE::WU_E_WINHTTP_INVALID_FILE,
            "WU_E_TOO_MANY_RESYNC" => WINDOWSUPDATE::WU_E_TOO_MANY_RESYNC,
            "WU_E_NO_SERVER_CORE_SUPPORT" => WINDOWSUPDATE::WU_E_NO_SERVER_CORE_SUPPORT,
            "WU_E_SYSPREP_IN_PROGRESS" => WINDOWSUPDATE::WU_E_SYSPREP_IN_PROGRESS,
            "WU_E_UNKNOWN_SERVICE" => WINDOWSUPDATE::WU_E_UNKNOWN_SERVICE,
            "WU_E_UNEXPECTED" => WINDOWSUPDATE::WU_E_UNEXPECTED,
            "WU_E_MSI_WRONG_VERSION" => WINDOWSUPDATE::WU_E_MSI_WRONG_VERSION,
            "WU_E_MSI_NOT_CONFIGURED" => WINDOWSUPDATE::WU_E_MSI_NOT_CONFIGURED,
            "WU_E_MSP_DISABLED" => WINDOWSUPDATE::WU_E_MSP_DISABLED,
            "WU_E_MSI_WRONG_APP_CONTEXT" => WINDOWSUPDATE::WU_E_MSI_WRONG_APP_CONTEXT,
            "WU_E_MSP_UNEXPECTED" => WINDOWSUPDATE::WU_E_MSP_UNEXPECTED,
            "WU_E_UH_REMOTEUNAVAILABLE" => WINDOWSUPDATE::WU_E_UH_REMOTEUNAVAILABLE,
            "WU_E_UH_LOCALONLY" => WINDOWSUPDATE::WU_E_UH_LOCALONLY,
            "WU_E_UH_UNKNOWNHANDLER" => WINDOWSUPDATE::WU_E_UH_UNKNOWNHANDLER,
            "WU_E_UH_REMOTEALREADYACTIVE" => WINDOWSUPDATE::WU_E_UH_REMOTEALREADYACTIVE,
            "WU_E_UH_DOESNOTSUPPORTACTION" => WINDOWSUPDATE::WU_E_UH_DOESNOTSUPPORTACTION,
            "WU_E_UH_WRONGHANDLER" => WINDOWSUPDATE::WU_E_UH_WRONGHANDLER,
            "WU_E_UH_INVALIDMETADATA" => WINDOWSUPDATE::WU_E_UH_INVALIDMETADATA,
            "WU_E_UH_INSTALLERHUNG" => WINDOWSUPDATE::WU_E_UH_INSTALLERHUNG,
            "WU_E_UH_OPERATIONCANCELLED" => WINDOWSUPDATE::WU_E_UH_OPERATIONCANCELLED,
            "WU_E_UH_BADHANDLERXML" => WINDOWSUPDATE::WU_E_UH_BADHANDLERXML,
            "WU_E_UH_CANREQUIREINPUT" => WINDOWSUPDATE::WU_E_UH_CANREQUIREINPUT,
            "WU_E_UH_INSTALLERFAILURE" => WINDOWSUPDATE::WU_E_UH_INSTALLERFAILURE,
            "WU_E_UH_FALLBACKTOSELFCONTAINED" => WINDOWSUPDATE::WU_E_UH_FALLBACKTOSELFCONTAINED,
            "WU_E_UH_NEEDANOTHERDOWNLOAD" => WINDOWSUPDATE::WU_E_UH_NEEDANOTHERDOWNLOAD,
            "WU_E_UH_NOTIFYFAILURE" => WINDOWSUPDATE::WU_E_UH_NOTIFYFAILURE,
            "WU_E_UH_INCONSISTENT_FILE_NAMES" => WINDOWSUPDATE::WU_E_UH_INCONSISTENT_FILE_NAMES,
            "WU_E_UH_FALLBACKERROR" => WINDOWSUPDATE::WU_E_UH_FALLBACKERROR,
            "WU_E_UH_TOOMANYDOWNLOADREQUESTS" => WINDOWSUPDATE::WU_E_UH_TOOMANYDOWNLOADREQUESTS,
            "WU_E_UH_UNEXPECTEDCBSRESPONSE" => WINDOWSUPDATE::WU_E_UH_UNEXPECTEDCBSRESPONSE,
            "WU_E_UH_BADCBSPACKAGEID" => WINDOWSUPDATE::WU_E_UH_BADCBSPACKAGEID,
            "WU_E_UH_POSTREBOOTSTILLPENDING" => WINDOWSUPDATE::WU_E_UH_POSTREBOOTSTILLPENDING,
            "WU_E_UH_POSTREBOOTRESULTUNKNOWN" => WINDOWSUPDATE::WU_E_UH_POSTREBOOTRESULTUNKNOWN,
            "WU_E_UH_POSTREBOOTUNEXPECTEDSTATE" => WINDOWSUPDATE::WU_E_UH_POSTREBOOTUNEXPECTEDSTATE,
            "WU_E_UH_NEW_SERVICING_STACK_REQUIRED" => WINDOWSUPDATE::WU_E_UH_NEW_SERVICING_STACK_REQUIRED,
            "WU_E_UH_UNEXPECTED" => WINDOWSUPDATE::WU_E_UH_UNEXPECTED,
            "WU_E_INSTALLATION_RESULTS_UNKNOWN_VERSION" => WINDOWSUPDATE::WU_E_INSTALLATION_RESULTS_UNKNOWN_VERSION,
            "WU_E_INSTALLATION_RESULTS_INVALID_DATA" => WINDOWSUPDATE::WU_E_INSTALLATION_RESULTS_INVALID_DATA,
            "WU_E_INSTALLATION_RESULTS_NOT_FOUND" => WINDOWSUPDATE::WU_E_INSTALLATION_RESULTS_NOT_FOUND,
            "WU_E_TRAYICON_FAILURE" => WINDOWSUPDATE::WU_E_TRAYICON_FAILURE,
            "WU_E_NON_UI_MODE" => WINDOWSUPDATE::WU_E_NON_UI_MODE,
            "WU_E_WUCLTUI_UNSUPPORTED_VERSION" => WINDOWSUPDATE::WU_E_WUCLTUI_UNSUPPORTED_VERSION,
            "WU_E_AUCLIENT_UNEXPECTED" => WINDOWSUPDATE::WU_E_AUCLIENT_UNEXPECTED,
            "WU_E_PT_SOAPCLIENT_BASE" => WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_BASE,
            "WU_E_PT_SOAPCLIENT_INITIALIZE" => WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_INITIALIZE,
            "WU_E_PT_SOAPCLIENT_OUTOFMEMORY" => WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_OUTOFMEMORY,
            "WU_E_PT_SOAPCLIENT_GENERATE" => WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_GENERATE,
            "WU_E_PT_SOAPCLIENT_CONNECT" => WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_CONNECT,
            "WU_E_PT_SOAPCLIENT_SEND" => WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_SEND,
            "WU_E_PT_SOAPCLIENT_SERVER" => WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_SERVER,
            "WU_E_PT_SOAPCLIENT_SOAPFAULT" => WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_SOAPFAULT,
            "WU_E_PT_SOAPCLIENT_PARSEFAULT" => WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_PARSEFAULT,
            "WU_E_PT_SOAPCLIENT_READ" => WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_READ,
            "WU_E_PT_SOAPCLIENT_PARSE" => WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_PARSE,
            "WU_E_PT_SOAP_VERSION" => WINDOWSUPDATE::WU_E_PT_SOAP_VERSION,
            "WU_E_PT_SOAP_MUST_UNDERSTAND" => WINDOWSUPDATE::WU_E_PT_SOAP_MUST_UNDERSTAND,
            "WU_E_PT_SOAP_CLIENT" => WINDOWSUPDATE::WU_E_PT_SOAP_CLIENT,
            "WU_E_PT_SOAP_SERVER" => WINDOWSUPDATE::WU_E_PT_SOAP_SERVER,
            "WU_E_PT_WMI_ERROR" => WINDOWSUPDATE::WU_E_PT_WMI_ERROR,
            "WU_E_PT_EXCEEDED_MAX_SERVER_TRIPS" => WINDOWSUPDATE::WU_E_PT_EXCEEDED_MAX_SERVER_TRIPS,
            "WU_E_PT_SUS_SERVER_NOT_SET" => WINDOWSUPDATE::WU_E_PT_SUS_SERVER_NOT_SET,
            "WU_E_PT_DOUBLE_INITIALIZATION" => WINDOWSUPDATE::WU_E_PT_DOUBLE_INITIALIZATION,
            "WU_E_PT_INVALID_COMPUTER_NAME" => WINDOWSUPDATE::WU_E_PT_INVALID_COMPUTER_NAME,
            "WU_E_PT_REFRESH_CACHE_REQUIRED" => WINDOWSUPDATE::WU_E_PT_REFRESH_CACHE_REQUIRED,
            "WU_E_PT_HTTP_STATUS_BAD_REQUEST" => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_BAD_REQUEST,
            "WU_E_PT_HTTP_STATUS_DENIED" => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_DENIED,
            "WU_E_PT_HTTP_STATUS_FORBIDDEN" => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_FORBIDDEN,
            "WU_E_PT_HTTP_STATUS_NOT_FOUND" => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_NOT_FOUND,
            "WU_E_PT_HTTP_STATUS_BAD_METHOD" => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_BAD_METHOD,
            "WU_E_PT_HTTP_STATUS_PROXY_AUTH_REQ" => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_PROXY_AUTH_REQ,
            "WU_E_PT_HTTP_STATUS_REQUEST_TIMEOUT" => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_REQUEST_TIMEOUT,
            "WU_E_PT_HTTP_STATUS_CONFLICT" => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_CONFLICT,
            "WU_E_PT_HTTP_STATUS_GONE" => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_GONE,
            "WU_E_PT_HTTP_STATUS_SERVER_ERROR" => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_SERVER_ERROR,
            "WU_E_PT_HTTP_STATUS_NOT_SUPPORTED" => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_NOT_SUPPORTED,
            "WU_E_PT_HTTP_STATUS_BAD_GATEWAY" => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_BAD_GATEWAY,
            "WU_E_PT_HTTP_STATUS_SERVICE_UNAVAIL" => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_SERVICE_UNAVAIL,
            "WU_E_PT_HTTP_STATUS_GATEWAY_TIMEOUT" => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_GATEWAY_TIMEOUT,
            "WU_E_PT_HTTP_STATUS_VERSION_NOT_SUP" => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_VERSION_NOT_SUP,
            "WU_E_PT_FILE_LOCATIONS_CHANGED" => WINDOWSUPDATE::WU_E_PT_FILE_LOCATIONS_CHANGED,
            "WU_E_PT_REGISTRATION_NOT_SUPPORTED" => WINDOWSUPDATE::WU_E_PT_REGISTRATION_NOT_SUPPORTED,
            "WU_E_PT_NO_AUTH_PLUGINS_REQUESTED" => WINDOWSUPDATE::WU_E_PT_NO_AUTH_PLUGINS_REQUESTED,
            "WU_E_PT_NO_AUTH_COOKIES_CREATED" => WINDOWSUPDATE::WU_E_PT_NO_AUTH_COOKIES_CREATED,
            "WU_E_PT_INVALID_CONFIG_PROP" => WINDOWSUPDATE::WU_E_PT_INVALID_CONFIG_PROP,
            "WU_E_PT_CONFIG_PROP_MISSING" => WINDOWSUPDATE::WU_E_PT_CONFIG_PROP_MISSING,
            "WU_E_PT_HTTP_STATUS_NOT_MAPPED" => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_NOT_MAPPED,
            "WU_E_PT_WINHTTP_NAME_NOT_RESOLVED" => WINDOWSUPDATE::WU_E_PT_WINHTTP_NAME_NOT_RESOLVED,
            "WU_E_PT_ECP_SUCCEEDED_WITH_ERRORS" => WINDOWSUPDATE::WU_E_PT_ECP_SUCCEEDED_WITH_ERRORS,
            "WU_E_PT_ECP_INIT_FAILED" => WINDOWSUPDATE::WU_E_PT_ECP_INIT_FAILED,
            "WU_E_PT_ECP_INVALID_FILE_FORMAT" => WINDOWSUPDATE::WU_E_PT_ECP_INVALID_FILE_FORMAT,
            "WU_E_PT_ECP_INVALID_METADATA" => WINDOWSUPDATE::WU_E_PT_ECP_INVALID_METADATA,
            "WU_E_PT_ECP_FAILURE_TO_EXTRACT_DIGEST" => WINDOWSUPDATE::WU_E_PT_ECP_FAILURE_TO_EXTRACT_DIGEST,
            "WU_E_PT_ECP_FAILURE_TO_DECOMPRESS_CAB_FILE" => WINDOWSUPDATE::WU_E_PT_ECP_FAILURE_TO_DECOMPRESS_CAB_FILE,
            "WU_E_PT_ECP_FILE_LOCATION_ERROR" => WINDOWSUPDATE::WU_E_PT_ECP_FILE_LOCATION_ERROR,
            "WU_E_PT_UNEXPECTED" => WINDOWSUPDATE::WU_E_PT_UNEXPECTED,
            "WU_E_REDIRECTOR_LOAD_XML" => WINDOWSUPDATE::WU_E_REDIRECTOR_LOAD_XML,
            "WU_E_REDIRECTOR_S_FALSE" => WINDOWSUPDATE::WU_E_REDIRECTOR_S_FALSE,
            "WU_E_REDIRECTOR_ID_SMALLER" => WINDOWSUPDATE::WU_E_REDIRECTOR_ID_SMALLER,
            "WU_E_PT_SAME_REDIR_ID" => WINDOWSUPDATE::WU_E_PT_SAME_REDIR_ID,
            "WU_E_PT_NO_MANAGED_RECOVER" => WINDOWSUPDATE::WU_E_PT_NO_MANAGED_RECOVER,
            "WU_E_REDIRECTOR_UNEXPECTED" => WINDOWSUPDATE::WU_E_REDIRECTOR_UNEXPECTED,
            "WU_E_DM_URLNOTAVAILABLE" => WINDOWSUPDATE::WU_E_DM_URLNOTAVAILABLE,
            "WU_E_DM_INCORRECTFILEHASH" => WINDOWSUPDATE::WU_E_DM_INCORRECTFILEHASH,
            "WU_E_DM_UNKNOWNALGORITHM" => WINDOWSUPDATE::WU_E_DM_UNKNOWNALGORITHM,
            "WU_E_DM_NEEDDOWNLOADREQUEST" => WINDOWSUPDATE::WU_E_DM_NEEDDOWNLOADREQUEST,
            "WU_E_DM_NONETWORK" => WINDOWSUPDATE::WU_E_DM_NONETWORK,
            "WU_E_DM_WRONGBITSVERSION" => WINDOWSUPDATE::WU_E_DM_WRONGBITSVERSION,
            "WU_E_DM_NOTDOWNLOADED" => WINDOWSUPDATE::WU_E_DM_NOTDOWNLOADED,
            "WU_E_DM_FAILTOCONNECTTOBITS" => WINDOWSUPDATE::WU_E_DM_FAILTOCONNECTTOBITS,
            "WU_E_DM_BITSTRANSFERERROR" => WINDOWSUPDATE::WU_E_DM_BITSTRANSFERERROR,
            "WU_E_DM_DOWNLOADLOCATIONCHANGED" => WINDOWSUPDATE::WU_E_DM_DOWNLOADLOCATIONCHANGED,
            "WU_E_DM_CONTENTCHANGED" => WINDOWSUPDATE::WU_E_DM_CONTENTCHANGED,
            "WU_E_DM_UNEXPECTED" => WINDOWSUPDATE::WU_E_DM_UNEXPECTED,
            "WU_E_OL_INVALID_SCANFILE" => WINDOWSUPDATE::WU_E_OL_INVALID_SCANFILE,
            "WU_E_OL_NEWCLIENT_REQUIRED" => WINDOWSUPDATE::WU_E_OL_NEWCLIENT_REQUIRED,
            "WU_E_OL_UNEXPECTED" => WINDOWSUPDATE::WU_E_OL_UNEXPECTED,
            "WU_E_DS_SHUTDOWN" => WINDOWSUPDATE::WU_E_DS_SHUTDOWN,
            "WU_E_DS_INUSE" => WINDOWSUPDATE::WU_E_DS_INUSE,
            "WU_E_DS_INVALID" => WINDOWSUPDATE::WU_E_DS_INVALID,
            "WU_E_DS_TABLEMISSING" => WINDOWSUPDATE::WU_E_DS_TABLEMISSING,
            "WU_E_DS_TABLEINCORRECT" => WINDOWSUPDATE::WU_E_DS_TABLEINCORRECT,
            "WU_E_DS_INVALIDTABLENAME" => WINDOWSUPDATE::WU_E_DS_INVALIDTABLENAME,
            "WU_E_DS_BADVERSION" => WINDOWSUPDATE::WU_E_DS_BADVERSION,
            "WU_E_DS_NODATA" => WINDOWSUPDATE::WU_E_DS_NODATA,
            "WU_E_DS_MISSINGDATA" => WINDOWSUPDATE::WU_E_DS_MISSINGDATA,
            "WU_E_DS_MISSINGREF" => WINDOWSUPDATE::WU_E_DS_MISSINGREF,
            "WU_E_DS_UNKNOWNHANDLER" => WINDOWSUPDATE::WU_E_DS_UNKNOWNHANDLER,
            "WU_E_DS_CANTDELETE" => WINDOWSUPDATE::WU_E_DS_CANTDELETE,
            "WU_E_DS_LOCKTIMEOUTEXPIRED" => WINDOWSUPDATE::WU_E_DS_LOCKTIMEOUTEXPIRED,
            "WU_E_DS_NOCATEGORIES" => WINDOWSUPDATE::WU_E_DS_NOCATEGORIES,
            "WU_E_DS_ROWEXISTS" => WINDOWSUPDATE::WU_E_DS_ROWEXISTS,
            "WU_E_DS_STOREFILELOCKED" => WINDOWSUPDATE::WU_E_DS_STOREFILELOCKED,
            "WU_E_DS_CANNOTREGISTER" => WINDOWSUPDATE::WU_E_DS_CANNOTREGISTER,
            "WU_E_DS_UNABLETOSTART" => WINDOWSUPDATE::WU_E_DS_UNABLETOSTART,
            "WU_E_DS_DUPLICATEUPDATEID" => WINDOWSUPDATE::WU_E_DS_DUPLICATEUPDATEID,
            "WU_E_DS_UNKNOWNSERVICE" => WINDOWSUPDATE::WU_E_DS_UNKNOWNSERVICE,
            "WU_E_DS_SERVICEEXPIRED" => WINDOWSUPDATE::WU_E_DS_SERVICEEXPIRED,
            "WU_E_DS_DECLINENOTALLOWED" => WINDOWSUPDATE::WU_E_DS_DECLINENOTALLOWED,
            "WU_E_DS_TABLESESSIONMISMATCH" => WINDOWSUPDATE::WU_E_DS_TABLESESSIONMISMATCH,
            "WU_E_DS_SESSIONLOCKMISMATCH" => WINDOWSUPDATE::WU_E_DS_SESSIONLOCKMISMATCH,
            "WU_E_DS_NEEDWINDOWSSERVICE" => WINDOWSUPDATE::WU_E_DS_NEEDWINDOWSSERVICE,
            "WU_E_DS_INVALIDOPERATION" => WINDOWSUPDATE::WU_E_DS_INVALIDOPERATION,
            "WU_E_DS_SCHEMAMISMATCH" => WINDOWSUPDATE::WU_E_DS_SCHEMAMISMATCH,
            "WU_E_DS_RESETREQUIRED" => WINDOWSUPDATE::WU_E_DS_RESETREQUIRED,
            "WU_E_DS_IMPERSONATED" => WINDOWSUPDATE::WU_E_DS_IMPERSONATED,
            "WU_E_DS_UNEXPECTED" => WINDOWSUPDATE::WU_E_DS_UNEXPECTED,
            "WU_E_INVENTORY_PARSEFAILED" => WINDOWSUPDATE::WU_E_INVENTORY_PARSEFAILED,
            "WU_E_INVENTORY_GET_INVENTORY_TYPE_FAILED" => WINDOWSUPDATE::WU_E_INVENTORY_GET_INVENTORY_TYPE_FAILED,
            "WU_E_INVENTORY_RESULT_UPLOAD_FAILED" => WINDOWSUPDATE::WU_E_INVENTORY_RESULT_UPLOAD_FAILED,
            "WU_E_INVENTORY_UNEXPECTED" => WINDOWSUPDATE::WU_E_INVENTORY_UNEXPECTED,
            "WU_E_INVENTORY_WMI_ERROR" => WINDOWSUPDATE::WU_E_INVENTORY_WMI_ERROR,
            "WU_E_AU_NOSERVICE" => WINDOWSUPDATE::WU_E_AU_NOSERVICE,
            "WU_E_AU_NONLEGACYSERVER" => WINDOWSUPDATE::WU_E_AU_NONLEGACYSERVER,
            "WU_E_AU_LEGACYCLIENTDISABLED" => WINDOWSUPDATE::WU_E_AU_LEGACYCLIENTDISABLED,
            "WU_E_AU_PAUSED" => WINDOWSUPDATE::WU_E_AU_PAUSED,
            "WU_E_AU_NO_REGISTERED_SERVICE" => WINDOWSUPDATE::WU_E_AU_NO_REGISTERED_SERVICE,
            "WU_E_AU_UNEXPECTED" => WINDOWSUPDATE::WU_E_AU_UNEXPECTED,
            "WU_E_DRV_PRUNED" => WINDOWSUPDATE::WU_E_DRV_PRUNED,
            "WU_E_DRV_NOPROP_OR_LEGACY" => WINDOWSUPDATE::WU_E_DRV_NOPROP_OR_LEGACY,
            "WU_E_DRV_REG_MISMATCH" => WINDOWSUPDATE::WU_E_DRV_REG_MISMATCH,
            "WU_E_DRV_NO_METADATA" => WINDOWSUPDATE::WU_E_DRV_NO_METADATA,
            "WU_E_DRV_MISSING_ATTRIBUTE" => WINDOWSUPDATE::WU_E_DRV_MISSING_ATTRIBUTE,
            "WU_E_DRV_SYNC_FAILED" => WINDOWSUPDATE::WU_E_DRV_SYNC_FAILED,
            "WU_E_DRV_NO_PRINTER_CONTENT" => WINDOWSUPDATE::WU_E_DRV_NO_PRINTER_CONTENT,
            "WU_E_DRV_UNEXPECTED" => WINDOWSUPDATE::WU_E_DRV_UNEXPECTED,
            "WU_E_SETUP_INVALID_INFDATA" => WINDOWSUPDATE::WU_E_SETUP_INVALID_INFDATA,
            "WU_E_SETUP_INVALID_IDENTDATA" => WINDOWSUPDATE::WU_E_SETUP_INVALID_IDENTDATA,
            "WU_E_SETUP_ALREADY_INITIALIZED" => WINDOWSUPDATE::WU_E_SETUP_ALREADY_INITIALIZED,
            "WU_E_SETUP_NOT_INITIALIZED" => WINDOWSUPDATE::WU_E_SETUP_NOT_INITIALIZED,
            "WU_E_SETUP_SOURCE_VERSION_MISMATCH" => WINDOWSUPDATE::WU_E_SETUP_SOURCE_VERSION_MISMATCH,
            "WU_E_SETUP_TARGET_VERSION_GREATER" => WINDOWSUPDATE::WU_E_SETUP_TARGET_VERSION_GREATER,
            "WU_E_SETUP_REGISTRATION_FAILED" => WINDOWSUPDATE::WU_E_SETUP_REGISTRATION_FAILED,
            "WU_E_SELFUPDATE_SKIP_ON_FAILURE" => WINDOWSUPDATE::WU_E_SELFUPDATE_SKIP_ON_FAILURE,
            "WU_E_SETUP_SKIP_UPDATE" => WINDOWSUPDATE::WU_E_SETUP_SKIP_UPDATE,
            "WU_E_SETUP_UNSUPPORTED_CONFIGURATION" => WINDOWSUPDATE::WU_E_SETUP_UNSUPPORTED_CONFIGURATION,
            "WU_E_SETUP_BLOCKED_CONFIGURATION" => WINDOWSUPDATE::WU_E_SETUP_BLOCKED_CONFIGURATION,
            "WU_E_SETUP_REBOOT_TO_FIX" => WINDOWSUPDATE::WU_E_SETUP_REBOOT_TO_FIX,
            "WU_E_SETUP_ALREADYRUNNING" => WINDOWSUPDATE::WU_E_SETUP_ALREADYRUNNING,
            "WU_E_SETUP_REBOOTREQUIRED" => WINDOWSUPDATE::WU_E_SETUP_REBOOTREQUIRED,
            "WU_E_SETUP_HANDLER_EXEC_FAILURE" => WINDOWSUPDATE::WU_E_SETUP_HANDLER_EXEC_FAILURE,
            "WU_E_SETUP_INVALID_REGISTRY_DATA" => WINDOWSUPDATE::WU_E_SETUP_INVALID_REGISTRY_DATA,
            "WU_E_SELFUPDATE_REQUIRED" => WINDOWSUPDATE::WU_E_SELFUPDATE_REQUIRED,
            "WU_E_SELFUPDATE_REQUIRED_ADMIN" => WINDOWSUPDATE::WU_E_SELFUPDATE_REQUIRED_ADMIN,
            "WU_E_SETUP_WRONG_SERVER_VERSION" => WINDOWSUPDATE::WU_E_SETUP_WRONG_SERVER_VERSION,
            "WU_E_SETUP_UNEXPECTED" => WINDOWSUPDATE::WU_E_SETUP_UNEXPECTED,
            "WU_E_EE_UNKNOWN_EXPRESSION" => WINDOWSUPDATE::WU_E_EE_UNKNOWN_EXPRESSION,
            "WU_E_EE_INVALID_EXPRESSION" => WINDOWSUPDATE::WU_E_EE_INVALID_EXPRESSION,
            "WU_E_EE_MISSING_METADATA" => WINDOWSUPDATE::WU_E_EE_MISSING_METADATA,
            "WU_E_EE_INVALID_VERSION" => WINDOWSUPDATE::WU_E_EE_INVALID_VERSION,
            "WU_E_EE_NOT_INITIALIZED" => WINDOWSUPDATE::WU_E_EE_NOT_INITIALIZED,
            "WU_E_EE_INVALID_ATTRIBUTEDATA" => WINDOWSUPDATE::WU_E_EE_INVALID_ATTRIBUTEDATA,
            "WU_E_EE_CLUSTER_ERROR" => WINDOWSUPDATE::WU_E_EE_CLUSTER_ERROR,
            "WU_E_EE_UNEXPECTED" => WINDOWSUPDATE::WU_E_EE_UNEXPECTED,
            "WU_E_REPORTER_EVENTCACHECORRUPT" => WINDOWSUPDATE::WU_E_REPORTER_EVENTCACHECORRUPT,
            "WU_E_REPORTER_EVENTNAMESPACEPARSEFAILED" => WINDOWSUPDATE::WU_E_REPORTER_EVENTNAMESPACEPARSEFAILED,
            "WU_E_INVALID_EVENT" => WINDOWSUPDATE::WU_E_INVALID_EVENT,
            "WU_E_SERVER_BUSY" => WINDOWSUPDATE::WU_E_SERVER_BUSY,
            "WU_E_REPORTER_UNEXPECTED" => WINDOWSUPDATE::WU_E_REPORTER_UNEXPECTED,
        }
    }
    pub fn from_code(code: u32) -> WINDOWSUPDATE {
        return match code {
            0x00240001 => WINDOWSUPDATE::WU_S_SERVICE_STOP,
            0x00240002 => WINDOWSUPDATE::WU_S_SELFUPDATE,
            0x00240003 => WINDOWSUPDATE::WU_S_UPDATE_ERROR,
            0x00240004 => WINDOWSUPDATE::WU_S_MARKED_FOR_DISCONNECT,
            0x00240005 => WINDOWSUPDATE::WU_S_REBOOT_REQUIRED,
            0x00240006 => WINDOWSUPDATE::WU_S_ALREADY_INSTALLED,
            0x00240007 => WINDOWSUPDATE::WU_S_ALREADY_UNINSTALLED,
            0x00240008 => WINDOWSUPDATE::WU_S_ALREADY_DOWNLOADED,
            0x80240001 => WINDOWSUPDATE::WU_E_NO_SERVICE,
            0x80240002 => WINDOWSUPDATE::WU_E_MAX_CAPACITY_REACHED,
            0x80240003 => WINDOWSUPDATE::WU_E_UNKNOWN_ID,
            0x80240004 => WINDOWSUPDATE::WU_E_NOT_INITIALIZED,
            0x80240005 => WINDOWSUPDATE::WU_E_RANGEOVERLAP,
            0x80240006 => WINDOWSUPDATE::WU_E_TOOMANYRANGES,
            0x80240007 => WINDOWSUPDATE::WU_E_INVALIDINDEX,
            0x80240008 => WINDOWSUPDATE::WU_E_ITEMNOTFOUND,
            0x80240009 => WINDOWSUPDATE::WU_E_OPERATIONINPROGRESS,
            0x8024000A => WINDOWSUPDATE::WU_E_COULDNOTCANCEL,
            0x8024000B => WINDOWSUPDATE::WU_E_CALL_CANCELLED,
            0x8024000C => WINDOWSUPDATE::WU_E_NOOP,
            0x8024000D => WINDOWSUPDATE::WU_E_XML_MISSINGDATA,
            0x8024000E => WINDOWSUPDATE::WU_E_XML_INVALID,
            0x8024000F => WINDOWSUPDATE::WU_E_CYCLE_DETECTED,
            0x80240010 => WINDOWSUPDATE::WU_E_TOO_DEEP_RELATION,
            0x80240011 => WINDOWSUPDATE::WU_E_INVALID_RELATIONSHIP,
            0x80240012 => WINDOWSUPDATE::WU_E_REG_VALUE_INVALID,
            0x80240013 => WINDOWSUPDATE::WU_E_DUPLICATE_ITEM,
            0x80240016 => WINDOWSUPDATE::WU_E_INSTALL_NOT_ALLOWED,
            0x80240017 => WINDOWSUPDATE::WU_E_NOT_APPLICABLE,
            0x80240018 => WINDOWSUPDATE::WU_E_NO_USERTOKEN,
            0x80240019 => WINDOWSUPDATE::WU_E_EXCLUSIVE_INSTALL_CONFLICT,
            0x8024001A => WINDOWSUPDATE::WU_E_POLICY_NOT_SET,
            0x8024001B => WINDOWSUPDATE::WU_E_SELFUPDATE_IN_PROGRESS,
            0x8024001D => WINDOWSUPDATE::WU_E_INVALID_UPDATE,
            0x8024001E => WINDOWSUPDATE::WU_E_SERVICE_STOP,
            0x8024001F => WINDOWSUPDATE::WU_E_NO_CONNECTION,
            0x80240020 => WINDOWSUPDATE::WU_E_NO_INTERACTIVE_USER,
            0x80240021 => WINDOWSUPDATE::WU_E_TIME_OUT,
            0x80240022 => WINDOWSUPDATE::WU_E_ALL_UPDATES_FAILED,
            0x80240023 => WINDOWSUPDATE::WU_E_EULAS_DECLINED,
            0x80240024 => WINDOWSUPDATE::WU_E_NO_UPDATE,
            0x80240025 => WINDOWSUPDATE::WU_E_USER_ACCESS_DISABLED,
            0x80240026 => WINDOWSUPDATE::WU_E_INVALID_UPDATE_TYPE,
            0x80240027 => WINDOWSUPDATE::WU_E_URL_TOO_LONG,
            0x80240028 => WINDOWSUPDATE::WU_E_UNINSTALL_NOT_ALLOWED,
            0x80240029 => WINDOWSUPDATE::WU_E_INVALID_PRODUCT_LICENSE,
            0x8024002A => WINDOWSUPDATE::WU_E_MISSING_HANDLER,
            0x8024002B => WINDOWSUPDATE::WU_E_LEGACYSERVER,
            0x8024002C => WINDOWSUPDATE::WU_E_BIN_SOURCE_ABSENT,
            0x8024002D => WINDOWSUPDATE::WU_E_SOURCE_ABSENT,
            0x8024002E => WINDOWSUPDATE::WU_E_WU_DISABLED,
            0x8024002F => WINDOWSUPDATE::WU_E_CALL_CANCELLED_BY_POLICY,
            0x80240030 => WINDOWSUPDATE::WU_E_INVALID_PROXY_SERVER,
            0x80240031 => WINDOWSUPDATE::WU_E_INVALID_FILE,
            0x80240032 => WINDOWSUPDATE::WU_E_INVALID_CRITERIA,
            0x80240033 => WINDOWSUPDATE::WU_E_EULA_UNAVAILABLE,
            0x80240034 => WINDOWSUPDATE::WU_E_DOWNLOAD_FAILED,
            0x80240035 => WINDOWSUPDATE::WU_E_UPDATE_NOT_PROCESSED,
            0x80240036 => WINDOWSUPDATE::WU_E_INVALID_OPERATION,
            0x80240037 => WINDOWSUPDATE::WU_E_NOT_SUPPORTED,
            0x80240038 => WINDOWSUPDATE::WU_E_WINHTTP_INVALID_FILE,
            0x80240039 => WINDOWSUPDATE::WU_E_TOO_MANY_RESYNC,
            0x80240040 => WINDOWSUPDATE::WU_E_NO_SERVER_CORE_SUPPORT,
            0x80240041 => WINDOWSUPDATE::WU_E_SYSPREP_IN_PROGRESS,
            0x80240042 => WINDOWSUPDATE::WU_E_UNKNOWN_SERVICE,
            0x80240FFF => WINDOWSUPDATE::WU_E_UNEXPECTED,
            0x80241001 => WINDOWSUPDATE::WU_E_MSI_WRONG_VERSION,
            0x80241002 => WINDOWSUPDATE::WU_E_MSI_NOT_CONFIGURED,
            0x80241003 => WINDOWSUPDATE::WU_E_MSP_DISABLED,
            0x80241004 => WINDOWSUPDATE::WU_E_MSI_WRONG_APP_CONTEXT,
            0x80241FFF => WINDOWSUPDATE::WU_E_MSP_UNEXPECTED,
            0x80242000 => WINDOWSUPDATE::WU_E_UH_REMOTEUNAVAILABLE,
            0x80242001 => WINDOWSUPDATE::WU_E_UH_LOCALONLY,
            0x80242002 => WINDOWSUPDATE::WU_E_UH_UNKNOWNHANDLER,
            0x80242003 => WINDOWSUPDATE::WU_E_UH_REMOTEALREADYACTIVE,
            0x80242004 => WINDOWSUPDATE::WU_E_UH_DOESNOTSUPPORTACTION,
            0x80242005 => WINDOWSUPDATE::WU_E_UH_WRONGHANDLER,
            0x80242006 => WINDOWSUPDATE::WU_E_UH_INVALIDMETADATA,
            0x80242007 => WINDOWSUPDATE::WU_E_UH_INSTALLERHUNG,
            0x80242008 => WINDOWSUPDATE::WU_E_UH_OPERATIONCANCELLED,
            0x80242009 => WINDOWSUPDATE::WU_E_UH_BADHANDLERXML,
            0x8024200A => WINDOWSUPDATE::WU_E_UH_CANREQUIREINPUT,
            0x8024200B => WINDOWSUPDATE::WU_E_UH_INSTALLERFAILURE,
            0x8024200C => WINDOWSUPDATE::WU_E_UH_FALLBACKTOSELFCONTAINED,
            0x8024200D => WINDOWSUPDATE::WU_E_UH_NEEDANOTHERDOWNLOAD,
            0x8024200E => WINDOWSUPDATE::WU_E_UH_NOTIFYFAILURE,
            0x8024200F => WINDOWSUPDATE::WU_E_UH_INCONSISTENT_FILE_NAMES,
            0x80242010 => WINDOWSUPDATE::WU_E_UH_FALLBACKERROR,
            0x80242011 => WINDOWSUPDATE::WU_E_UH_TOOMANYDOWNLOADREQUESTS,
            0x80242012 => WINDOWSUPDATE::WU_E_UH_UNEXPECTEDCBSRESPONSE,
            0x80242013 => WINDOWSUPDATE::WU_E_UH_BADCBSPACKAGEID,
            0x80242014 => WINDOWSUPDATE::WU_E_UH_POSTREBOOTSTILLPENDING,
            0x80242015 => WINDOWSUPDATE::WU_E_UH_POSTREBOOTRESULTUNKNOWN,
            0x80242016 => WINDOWSUPDATE::WU_E_UH_POSTREBOOTUNEXPECTEDSTATE,
            0x80242017 => WINDOWSUPDATE::WU_E_UH_NEW_SERVICING_STACK_REQUIRED,
            0x80242FFF => WINDOWSUPDATE::WU_E_UH_UNEXPECTED,
            0x80243001 => WINDOWSUPDATE::WU_E_INSTALLATION_RESULTS_UNKNOWN_VERSION,
            0x80243002 => WINDOWSUPDATE::WU_E_INSTALLATION_RESULTS_INVALID_DATA,
            0x80243003 => WINDOWSUPDATE::WU_E_INSTALLATION_RESULTS_NOT_FOUND,
            0x80243004 => WINDOWSUPDATE::WU_E_TRAYICON_FAILURE,
            0x80243FFD => WINDOWSUPDATE::WU_E_NON_UI_MODE,
            0x80243FFE => WINDOWSUPDATE::WU_E_WUCLTUI_UNSUPPORTED_VERSION,
            0x80243FFF => WINDOWSUPDATE::WU_E_AUCLIENT_UNEXPECTED,
            0x80244000 => WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_BASE,
            0x80244001 => WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_INITIALIZE,
            0x80244002 => WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_OUTOFMEMORY,
            0x80244003 => WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_GENERATE,
            0x80244004 => WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_CONNECT,
            0x80244005 => WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_SEND,
            0x80244006 => WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_SERVER,
            0x80244007 => WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_SOAPFAULT,
            0x80244008 => WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_PARSEFAULT,
            0x80244009 => WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_READ,
            0x8024400A => WINDOWSUPDATE::WU_E_PT_SOAPCLIENT_PARSE,
            0x8024400B => WINDOWSUPDATE::WU_E_PT_SOAP_VERSION,
            0x8024400C => WINDOWSUPDATE::WU_E_PT_SOAP_MUST_UNDERSTAND,
            0x8024400D => WINDOWSUPDATE::WU_E_PT_SOAP_CLIENT,
            0x8024400E => WINDOWSUPDATE::WU_E_PT_SOAP_SERVER,
            0x8024400F => WINDOWSUPDATE::WU_E_PT_WMI_ERROR,
            0x80244010 => WINDOWSUPDATE::WU_E_PT_EXCEEDED_MAX_SERVER_TRIPS,
            0x80244011 => WINDOWSUPDATE::WU_E_PT_SUS_SERVER_NOT_SET,
            0x80244012 => WINDOWSUPDATE::WU_E_PT_DOUBLE_INITIALIZATION,
            0x80244013 => WINDOWSUPDATE::WU_E_PT_INVALID_COMPUTER_NAME,
            0x80244015 => WINDOWSUPDATE::WU_E_PT_REFRESH_CACHE_REQUIRED,
            0x80244016 => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_BAD_REQUEST,
            0x80244017 => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_DENIED,
            0x80244018 => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_FORBIDDEN,
            0x80244019 => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_NOT_FOUND,
            0x8024401A => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_BAD_METHOD,
            0x8024401B => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_PROXY_AUTH_REQ,
            0x8024401C => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_REQUEST_TIMEOUT,
            0x8024401D => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_CONFLICT,
            0x8024401E => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_GONE,
            0x8024401F => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_SERVER_ERROR,
            0x80244020 => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_NOT_SUPPORTED,
            0x80244021 => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_BAD_GATEWAY,
            0x80244022 => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_SERVICE_UNAVAIL,
            0x80244023 => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_GATEWAY_TIMEOUT,
            0x80244024 => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_VERSION_NOT_SUP,
            0x80244025 => WINDOWSUPDATE::WU_E_PT_FILE_LOCATIONS_CHANGED,
            0x80244026 => WINDOWSUPDATE::WU_E_PT_REGISTRATION_NOT_SUPPORTED,
            0x80244027 => WINDOWSUPDATE::WU_E_PT_NO_AUTH_PLUGINS_REQUESTED,
            0x80244028 => WINDOWSUPDATE::WU_E_PT_NO_AUTH_COOKIES_CREATED,
            0x80244029 => WINDOWSUPDATE::WU_E_PT_INVALID_CONFIG_PROP,
            0x8024402A => WINDOWSUPDATE::WU_E_PT_CONFIG_PROP_MISSING,
            0x8024402B => WINDOWSUPDATE::WU_E_PT_HTTP_STATUS_NOT_MAPPED,
            0x8024402C => WINDOWSUPDATE::WU_E_PT_WINHTTP_NAME_NOT_RESOLVED,
            0x8024402F => WINDOWSUPDATE::WU_E_PT_ECP_SUCCEEDED_WITH_ERRORS,
            0x80244030 => WINDOWSUPDATE::WU_E_PT_ECP_INIT_FAILED,
            0x80244031 => WINDOWSUPDATE::WU_E_PT_ECP_INVALID_FILE_FORMAT,
            0x80244032 => WINDOWSUPDATE::WU_E_PT_ECP_INVALID_METADATA,
            0x80244033 => WINDOWSUPDATE::WU_E_PT_ECP_FAILURE_TO_EXTRACT_DIGEST,
            0x80244034 => WINDOWSUPDATE::WU_E_PT_ECP_FAILURE_TO_DECOMPRESS_CAB_FILE,
            0x80244035 => WINDOWSUPDATE::WU_E_PT_ECP_FILE_LOCATION_ERROR,
            0x80244FFF => WINDOWSUPDATE::WU_E_PT_UNEXPECTED,
            0x80245001 => WINDOWSUPDATE::WU_E_REDIRECTOR_LOAD_XML,
            0x80245002 => WINDOWSUPDATE::WU_E_REDIRECTOR_S_FALSE,
            0x80245003 => WINDOWSUPDATE::WU_E_REDIRECTOR_ID_SMALLER,
            0x8024502D => WINDOWSUPDATE::WU_E_PT_SAME_REDIR_ID,
            0x8024502E => WINDOWSUPDATE::WU_E_PT_NO_MANAGED_RECOVER,
            0x80245FFF => WINDOWSUPDATE::WU_E_REDIRECTOR_UNEXPECTED,
            0x80246001 => WINDOWSUPDATE::WU_E_DM_URLNOTAVAILABLE,
            0x80246002 => WINDOWSUPDATE::WU_E_DM_INCORRECTFILEHASH,
            0x80246003 => WINDOWSUPDATE::WU_E_DM_UNKNOWNALGORITHM,
            0x80246004 => WINDOWSUPDATE::WU_E_DM_NEEDDOWNLOADREQUEST,
            0x80246005 => WINDOWSUPDATE::WU_E_DM_NONETWORK,
            0x80246006 => WINDOWSUPDATE::WU_E_DM_WRONGBITSVERSION,
            0x80246007 => WINDOWSUPDATE::WU_E_DM_NOTDOWNLOADED,
            0x80246008 => WINDOWSUPDATE::WU_E_DM_FAILTOCONNECTTOBITS,
            0x80246009 => WINDOWSUPDATE::WU_E_DM_BITSTRANSFERERROR,
            0x8024600A => WINDOWSUPDATE::WU_E_DM_DOWNLOADLOCATIONCHANGED,
            0x8024600B => WINDOWSUPDATE::WU_E_DM_CONTENTCHANGED,
            0x80246FFF => WINDOWSUPDATE::WU_E_DM_UNEXPECTED,
            0x80247001 => WINDOWSUPDATE::WU_E_OL_INVALID_SCANFILE,
            0x80247002 => WINDOWSUPDATE::WU_E_OL_NEWCLIENT_REQUIRED,
            0x80247FFF => WINDOWSUPDATE::WU_E_OL_UNEXPECTED,
            0x80248000 => WINDOWSUPDATE::WU_E_DS_SHUTDOWN,
            0x80248001 => WINDOWSUPDATE::WU_E_DS_INUSE,
            0x80248002 => WINDOWSUPDATE::WU_E_DS_INVALID,
            0x80248003 => WINDOWSUPDATE::WU_E_DS_TABLEMISSING,
            0x80248004 => WINDOWSUPDATE::WU_E_DS_TABLEINCORRECT,
            0x80248005 => WINDOWSUPDATE::WU_E_DS_INVALIDTABLENAME,
            0x80248006 => WINDOWSUPDATE::WU_E_DS_BADVERSION,
            0x80248007 => WINDOWSUPDATE::WU_E_DS_NODATA,
            0x80248008 => WINDOWSUPDATE::WU_E_DS_MISSINGDATA,
            0x80248009 => WINDOWSUPDATE::WU_E_DS_MISSINGREF,
            0x8024800A => WINDOWSUPDATE::WU_E_DS_UNKNOWNHANDLER,
            0x8024800B => WINDOWSUPDATE::WU_E_DS_CANTDELETE,
            0x8024800C => WINDOWSUPDATE::WU_E_DS_LOCKTIMEOUTEXPIRED,
            0x8024800D => WINDOWSUPDATE::WU_E_DS_NOCATEGORIES,
            0x8024800E => WINDOWSUPDATE::WU_E_DS_ROWEXISTS,
            0x8024800F => WINDOWSUPDATE::WU_E_DS_STOREFILELOCKED,
            0x80248010 => WINDOWSUPDATE::WU_E_DS_CANNOTREGISTER,
            0x80248011 => WINDOWSUPDATE::WU_E_DS_UNABLETOSTART,
            0x80248013 => WINDOWSUPDATE::WU_E_DS_DUPLICATEUPDATEID,
            0x80248014 => WINDOWSUPDATE::WU_E_DS_UNKNOWNSERVICE,
            0x80248015 => WINDOWSUPDATE::WU_E_DS_SERVICEEXPIRED,
            0x80248016 => WINDOWSUPDATE::WU_E_DS_DECLINENOTALLOWED,
            0x80248017 => WINDOWSUPDATE::WU_E_DS_TABLESESSIONMISMATCH,
            0x80248018 => WINDOWSUPDATE::WU_E_DS_SESSIONLOCKMISMATCH,
            0x80248019 => WINDOWSUPDATE::WU_E_DS_NEEDWINDOWSSERVICE,
            0x8024801A => WINDOWSUPDATE::WU_E_DS_INVALIDOPERATION,
            0x8024801B => WINDOWSUPDATE::WU_E_DS_SCHEMAMISMATCH,
            0x8024801C => WINDOWSUPDATE::WU_E_DS_RESETREQUIRED,
            0x8024801D => WINDOWSUPDATE::WU_E_DS_IMPERSONATED,
            0x80248FFF => WINDOWSUPDATE::WU_E_DS_UNEXPECTED,
            0x80249001 => WINDOWSUPDATE::WU_E_INVENTORY_PARSEFAILED,
            0x80249002 => WINDOWSUPDATE::WU_E_INVENTORY_GET_INVENTORY_TYPE_FAILED,
            0x80249003 => WINDOWSUPDATE::WU_E_INVENTORY_RESULT_UPLOAD_FAILED,
            0x80249004 => WINDOWSUPDATE::WU_E_INVENTORY_UNEXPECTED,
            0x80249005 => WINDOWSUPDATE::WU_E_INVENTORY_WMI_ERROR,
            0x8024A000 => WINDOWSUPDATE::WU_E_AU_NOSERVICE,
            0x8024A002 => WINDOWSUPDATE::WU_E_AU_NONLEGACYSERVER,
            0x8024A003 => WINDOWSUPDATE::WU_E_AU_LEGACYCLIENTDISABLED,
            0x8024A004 => WINDOWSUPDATE::WU_E_AU_PAUSED,
            0x8024A005 => WINDOWSUPDATE::WU_E_AU_NO_REGISTERED_SERVICE,
            0x8024AFFF => WINDOWSUPDATE::WU_E_AU_UNEXPECTED,
            0x8024C001 => WINDOWSUPDATE::WU_E_DRV_PRUNED,
            0x8024C002 => WINDOWSUPDATE::WU_E_DRV_NOPROP_OR_LEGACY,
            0x8024C003 => WINDOWSUPDATE::WU_E_DRV_REG_MISMATCH,
            0x8024C004 => WINDOWSUPDATE::WU_E_DRV_NO_METADATA,
            0x8024C005 => WINDOWSUPDATE::WU_E_DRV_MISSING_ATTRIBUTE,
            0x8024C006 => WINDOWSUPDATE::WU_E_DRV_SYNC_FAILED,
            0x8024C007 => WINDOWSUPDATE::WU_E_DRV_NO_PRINTER_CONTENT,
            0x8024CFFF => WINDOWSUPDATE::WU_E_DRV_UNEXPECTED,
            0x8024D001 => WINDOWSUPDATE::WU_E_SETUP_INVALID_INFDATA,
            0x8024D002 => WINDOWSUPDATE::WU_E_SETUP_INVALID_IDENTDATA,
            0x8024D003 => WINDOWSUPDATE::WU_E_SETUP_ALREADY_INITIALIZED,
            0x8024D004 => WINDOWSUPDATE::WU_E_SETUP_NOT_INITIALIZED,
            0x8024D005 => WINDOWSUPDATE::WU_E_SETUP_SOURCE_VERSION_MISMATCH,
            0x8024D006 => WINDOWSUPDATE::WU_E_SETUP_TARGET_VERSION_GREATER,
            0x8024D007 => WINDOWSUPDATE::WU_E_SETUP_REGISTRATION_FAILED,
            0x8024D008 => WINDOWSUPDATE::WU_E_SELFUPDATE_SKIP_ON_FAILURE,
            0x8024D009 => WINDOWSUPDATE::WU_E_SETUP_SKIP_UPDATE,
            0x8024D00A => WINDOWSUPDATE::WU_E_SETUP_UNSUPPORTED_CONFIGURATION,
            0x8024D00B => WINDOWSUPDATE::WU_E_SETUP_BLOCKED_CONFIGURATION,
            0x8024D00C => WINDOWSUPDATE::WU_E_SETUP_REBOOT_TO_FIX,
            0x8024D00D => WINDOWSUPDATE::WU_E_SETUP_ALREADYRUNNING,
            0x8024D00E => WINDOWSUPDATE::WU_E_SETUP_REBOOTREQUIRED,
            0x8024D00F => WINDOWSUPDATE::WU_E_SETUP_HANDLER_EXEC_FAILURE,
            0x8024D010 => WINDOWSUPDATE::WU_E_SETUP_INVALID_REGISTRY_DATA,
            0x8024D011 => WINDOWSUPDATE::WU_E_SELFUPDATE_REQUIRED,
            0x8024D012 => WINDOWSUPDATE::WU_E_SELFUPDATE_REQUIRED_ADMIN,
            0x8024D013 => WINDOWSUPDATE::WU_E_SETUP_WRONG_SERVER_VERSION,
            0x8024DFFF => WINDOWSUPDATE::WU_E_SETUP_UNEXPECTED,
            0x8024E001 => WINDOWSUPDATE::WU_E_EE_UNKNOWN_EXPRESSION,
            0x8024E002 => WINDOWSUPDATE::WU_E_EE_INVALID_EXPRESSION,
            0x8024E003 => WINDOWSUPDATE::WU_E_EE_MISSING_METADATA,
            0x8024E004 => WINDOWSUPDATE::WU_E_EE_INVALID_VERSION,
            0x8024E005 => WINDOWSUPDATE::WU_E_EE_NOT_INITIALIZED,
            0x8024E006 => WINDOWSUPDATE::WU_E_EE_INVALID_ATTRIBUTEDATA,
            0x8024E007 => WINDOWSUPDATE::WU_E_EE_CLUSTER_ERROR,
            0x8024EFFF => WINDOWSUPDATE::WU_E_EE_UNEXPECTED,
            0x8024F001 => WINDOWSUPDATE::WU_E_REPORTER_EVENTCACHECORRUPT,
            0x8024F002 => WINDOWSUPDATE::WU_E_REPORTER_EVENTNAMESPACEPARSEFAILED,
            0x8024F003 => WINDOWSUPDATE::WU_E_INVALID_EVENT,
            0x8024F004 => WINDOWSUPDATE::WU_E_SERVER_BUSY,
            0x8024FFFF => WINDOWSUPDATE::WU_E_REPORTER_UNEXPECTED,
        }
    }
}
