/*
 * Copyright 2023 Stanislav Mikhailov (xavetar)
 *
 * Licensed under the Creative Commons Zero v1.0 Universal (CC0) License.
 * You may obtain a copy of the License at
 *
 *     http://creativecommons.org/publicdomain/zero/1.0/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the CC0 license is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// It is strictly forbidden to use the from_code method, respected Microsoft decided to define
// duplicate codes, but by defining different constant names, therefore, when using these codes,
// it is mandatory to use through from_name, and not from_code. Otherwise, it may cause undefined
// behavior or an unknown exception. Because one code corresponds to several constants.

use super::{RawError};

#[derive(Clone, Debug, Copy, Eq, Hash, Ord, PartialEq, PartialOrd)]
pub enum XPS {
    XPS_E_INVALID_LANGUAGE,
    XPS_E_INVALID_NAME,
    XPS_E_INVALID_RESOURCE_KEY,
    XPS_E_INVALID_PAGE_SIZE,
    XPS_E_INVALID_BLEED_BOX,
    XPS_E_INVALID_THUMBNAIL_IMAGE_TYPE,
    XPS_E_INVALID_LOOKUP_TYPE,
    XPS_E_INVALID_FLOAT,
    XPS_E_UNEXPECTED_CONTENT_TYPE,
    XPS_E_INVALID_FONT_URI,
    XPS_E_INVALID_CONTENT_BOX,
    XPS_E_INVALID_MARKUP,
    XPS_E_INVALID_XML_ENCODING,
    XPS_E_INVALID_CONTENT_TYPE,
    XPS_E_INVALID_OBFUSCATED_FONT_URI,
    XPS_E_UNEXPECTED_RELATIONSHIP_TYPE,
    XPS_E_UNEXPECTED_RESTRICTED_FONT_RELATIONSHIP,
    XPS_E_MISSING_NAME,
    XPS_E_MISSING_LOOKUP,
    XPS_E_MISSING_GLYPHS,
    XPS_E_MISSING_SEGMENT_DATA,
    XPS_E_MISSING_COLORPROFILE,
    XPS_E_MISSING_RELATIONSHIP_TARGET,
    XPS_E_MISSING_RESOURCE_RELATIONSHIP,
    XPS_E_MISSING_FONTURI,
    XPS_E_MISSING_DOCUMENTSEQUENCE_RELATIONSHIP,
    XPS_E_MISSING_DOCUMENT,
    XPS_E_MISSING_REFERRED_DOCUMENT,
    XPS_E_MISSING_REFERRED_PAGE,
    XPS_E_MISSING_PAGE_IN_DOCUMENT,
    XPS_E_MISSING_PAGE_IN_PAGEREFERENCE,
    XPS_E_MISSING_IMAGE_IN_IMAGEBRUSH,
    XPS_E_MISSING_RESOURCE_KEY,
    XPS_E_MISSING_PART_REFERENCE,
    XPS_E_MISSING_RESTRICTED_FONT_RELATIONSHIP,
    XPS_E_MISSING_DISCARDCONTROL,
    XPS_E_MISSING_PART_STREAM,
    XPS_E_UNAVAILABLE_PACKAGE,
    XPS_E_DUPLICATE_RESOURCE_KEYS,
    XPS_E_MULTIPLE_RESOURCES,
    XPS_E_MULTIPLE_DOCUMENTSEQUENCE_RELATIONSHIPS,
    XPS_E_MULTIPLE_THUMBNAILS_ON_PAGE,
    XPS_E_MULTIPLE_THUMBNAILS_ON_PACKAGE,
    XPS_E_MULTIPLE_PRINTTICKETS_ON_PAGE,
    XPS_E_MULTIPLE_PRINTTICKETS_ON_DOCUMENT,
    XPS_E_MULTIPLE_PRINTTICKETS_ON_DOCUMENTSEQUENCE,
    XPS_E_MULTIPLE_REFERENCES_TO_PART,
    XPS_E_DUPLICATE_NAMES,
    XPS_E_STRING_TOO_LONG,
    XPS_E_TOO_MANY_INDICES,
    XPS_E_MAPPING_OUT_OF_ORDER,
    XPS_E_MAPPING_OUTSIDE_STRING,
    XPS_E_MAPPING_OUTSIDE_INDICES,
    XPS_E_CARET_OUTSIDE_STRING,
    XPS_E_CARET_OUT_OF_ORDER,
    XPS_E_ODD_BIDILEVEL,
    XPS_E_ONE_TO_ONE_MAPPING_EXPECTED,
    XPS_E_RESTRICTED_FONT_NOT_OBFUSCATED,
    XPS_E_NEGATIVE_FLOAT,
    XPS_E_SIGREQUESTID_DUP,
    XPS_E_PACKAGE_NOT_OPENED,
    XPS_E_PACKAGE_ALREADY_OPENED,
    XPS_E_SIGNATUREID_DUP,
    XPS_E_MARKUP_COMPATIBILITY_ELEMENTS,
    XPS_E_OBJECT_DETACHED,
    XPS_E_INVALID_SIGNATUREBLOCK_MARKUP,
    XPS_E_XKEY_ATTR_PRESENT_OUTSIDE_RES_DICT,
    XPS_E_DICTIONARY_ITEM_NAMED,
    XPS_E_NESTED_REMOTE_DICTIONARY,
    XPS_E_INDEX_OUT_OF_RANGE,
    XPS_E_VISUAL_CIRCULAR_REF,
    XPS_E_NO_CUSTOM_OBJECTS,
    XPS_E_ALREADY_OWNED,
    XPS_E_RESOURCE_NOT_OWNED,
    XPS_E_UNEXPECTED_COLORPROFILE,
    XPS_E_COLOR_COMPONENT_OUT_OF_RANGE,
    XPS_E_BOTH_PATHFIGURE_AND_ABBR_SYNTAX_PRESENT,
    XPS_E_BOTH_RESOURCE_AND_SOURCEATTR_PRESENT,
    XPS_E_BLEED_BOX_PAGE_DIMENSIONS_NOT_IN_SYNC,
    XPS_E_RELATIONSHIP_EXTERNAL,
    XPS_E_NOT_ENOUGH_GRADIENT_STOPS,
    XPS_E_PACKAGE_WRITER_NOT_CLOSED,
    XPS_E_INVALID_NUMBER_OF_POINTS_IN_CURVE_SEGMENTS,
    XPS_E_ABSOLUTE_REFERENCE,
    XPS_E_INVALID_NUMBER_OF_COLOR_CHANNELS,
}

impl XPS {
    pub fn code(&self) -> u32 {
        return match self {
            XPS::XPS_E_INVALID_LANGUAGE => 0x80520000 as u32,
            XPS::XPS_E_INVALID_NAME => 0x80520001 as u32,
            XPS::XPS_E_INVALID_RESOURCE_KEY => 0x80520002 as u32,
            XPS::XPS_E_INVALID_PAGE_SIZE => 0x80520003 as u32,
            XPS::XPS_E_INVALID_BLEED_BOX => 0x80520004 as u32,
            XPS::XPS_E_INVALID_THUMBNAIL_IMAGE_TYPE => 0x80520005 as u32,
            XPS::XPS_E_INVALID_LOOKUP_TYPE => 0x80520006 as u32,
            XPS::XPS_E_INVALID_FLOAT => 0x80520007 as u32,
            XPS::XPS_E_UNEXPECTED_CONTENT_TYPE => 0x80520008 as u32,
            XPS::XPS_E_INVALID_FONT_URI => 0x8052000A as u32,
            XPS::XPS_E_INVALID_CONTENT_BOX => 0x8052000B as u32,
            XPS::XPS_E_INVALID_MARKUP => 0x8052000C as u32,
            XPS::XPS_E_INVALID_XML_ENCODING => 0x8052000D as u32,
            XPS::XPS_E_INVALID_CONTENT_TYPE => 0x8052000E as u32,
            XPS::XPS_E_INVALID_OBFUSCATED_FONT_URI => 0x8052000F as u32,
            XPS::XPS_E_UNEXPECTED_RELATIONSHIP_TYPE => 0x80520010 as u32,
            XPS::XPS_E_UNEXPECTED_RESTRICTED_FONT_RELATIONSHIP => 0x80520011 as u32,
            XPS::XPS_E_MISSING_NAME => 0x80520100 as u32,
            XPS::XPS_E_MISSING_LOOKUP => 0x80520101 as u32,
            XPS::XPS_E_MISSING_GLYPHS => 0x80520102 as u32,
            XPS::XPS_E_MISSING_SEGMENT_DATA => 0x80520103 as u32,
            XPS::XPS_E_MISSING_COLORPROFILE => 0x80520104 as u32,
            XPS::XPS_E_MISSING_RELATIONSHIP_TARGET => 0x80520105 as u32,
            XPS::XPS_E_MISSING_RESOURCE_RELATIONSHIP => 0x80520106 as u32,
            XPS::XPS_E_MISSING_FONTURI => 0x80520107 as u32,
            XPS::XPS_E_MISSING_DOCUMENTSEQUENCE_RELATIONSHIP => 0x80520108 as u32,
            XPS::XPS_E_MISSING_DOCUMENT => 0x80520109 as u32,
            XPS::XPS_E_MISSING_REFERRED_DOCUMENT => 0x8052010A as u32,
            XPS::XPS_E_MISSING_REFERRED_PAGE => 0x8052010B as u32,
            XPS::XPS_E_MISSING_PAGE_IN_DOCUMENT => 0x8052010C as u32,
            XPS::XPS_E_MISSING_PAGE_IN_PAGEREFERENCE => 0x8052010D as u32,
            XPS::XPS_E_MISSING_IMAGE_IN_IMAGEBRUSH => 0x8052010E as u32,
            XPS::XPS_E_MISSING_RESOURCE_KEY => 0x8052010F as u32,
            XPS::XPS_E_MISSING_PART_REFERENCE => 0x80520110 as u32,
            XPS::XPS_E_MISSING_RESTRICTED_FONT_RELATIONSHIP => 0x80520111 as u32,
            XPS::XPS_E_MISSING_DISCARDCONTROL => 0x80520112 as u32,
            XPS::XPS_E_MISSING_PART_STREAM => 0x80520113 as u32,
            XPS::XPS_E_UNAVAILABLE_PACKAGE => 0x80520114 as u32,
            XPS::XPS_E_DUPLICATE_RESOURCE_KEYS => 0x80520200 as u32,
            XPS::XPS_E_MULTIPLE_RESOURCES => 0x80520201 as u32,
            XPS::XPS_E_MULTIPLE_DOCUMENTSEQUENCE_RELATIONSHIPS => 0x80520202 as u32,
            XPS::XPS_E_MULTIPLE_THUMBNAILS_ON_PAGE => 0x80520203 as u32,
            XPS::XPS_E_MULTIPLE_THUMBNAILS_ON_PACKAGE => 0x80520204 as u32,
            XPS::XPS_E_MULTIPLE_PRINTTICKETS_ON_PAGE => 0x80520205 as u32,
            XPS::XPS_E_MULTIPLE_PRINTTICKETS_ON_DOCUMENT => 0x80520206 as u32,
            XPS::XPS_E_MULTIPLE_PRINTTICKETS_ON_DOCUMENTSEQUENCE => 0x80520207 as u32,
            XPS::XPS_E_MULTIPLE_REFERENCES_TO_PART => 0x80520208 as u32,
            XPS::XPS_E_DUPLICATE_NAMES => 0x80520209 as u32,
            XPS::XPS_E_STRING_TOO_LONG => 0x80520300 as u32,
            XPS::XPS_E_TOO_MANY_INDICES => 0x80520301 as u32,
            XPS::XPS_E_MAPPING_OUT_OF_ORDER => 0x80520302 as u32,
            XPS::XPS_E_MAPPING_OUTSIDE_STRING => 0x80520303 as u32,
            XPS::XPS_E_MAPPING_OUTSIDE_INDICES => 0x80520304 as u32,
            XPS::XPS_E_CARET_OUTSIDE_STRING => 0x80520305 as u32,
            XPS::XPS_E_CARET_OUT_OF_ORDER => 0x80520306 as u32,
            XPS::XPS_E_ODD_BIDILEVEL => 0x80520307 as u32,
            XPS::XPS_E_ONE_TO_ONE_MAPPING_EXPECTED => 0x80520308 as u32,
            XPS::XPS_E_RESTRICTED_FONT_NOT_OBFUSCATED => 0x80520309 as u32,
            XPS::XPS_E_NEGATIVE_FLOAT => 0x8052030A as u32,
            XPS::XPS_E_SIGREQUESTID_DUP => 0x80520385 as u32,
            XPS::XPS_E_PACKAGE_NOT_OPENED => 0x80520386 as u32,
            XPS::XPS_E_PACKAGE_ALREADY_OPENED => 0x80520387 as u32,
            XPS::XPS_E_SIGNATUREID_DUP => 0x80520388 as u32,
            XPS::XPS_E_MARKUP_COMPATIBILITY_ELEMENTS => 0x80520389 as u32,
            XPS::XPS_E_OBJECT_DETACHED => 0x8052038A as u32,
            XPS::XPS_E_INVALID_SIGNATUREBLOCK_MARKUP => 0x8052038B as u32,
            XPS::XPS_E_XKEY_ATTR_PRESENT_OUTSIDE_RES_DICT => 0x80520400 as u32,
            XPS::XPS_E_DICTIONARY_ITEM_NAMED => 0x80520401 as u32,
            XPS::XPS_E_NESTED_REMOTE_DICTIONARY => 0x80520402 as u32,
            XPS::XPS_E_INDEX_OUT_OF_RANGE => 0x80520500 as u32,
            XPS::XPS_E_VISUAL_CIRCULAR_REF => 0x80520501 as u32,
            XPS::XPS_E_NO_CUSTOM_OBJECTS => 0x80520502 as u32,
            XPS::XPS_E_ALREADY_OWNED => 0x80520503 as u32,
            XPS::XPS_E_RESOURCE_NOT_OWNED => 0x80520504 as u32,
            XPS::XPS_E_UNEXPECTED_COLORPROFILE => 0x80520505 as u32,
            XPS::XPS_E_COLOR_COMPONENT_OUT_OF_RANGE => 0x80520506 as u32,
            XPS::XPS_E_BOTH_PATHFIGURE_AND_ABBR_SYNTAX_PRESENT => 0x80520507 as u32,
            XPS::XPS_E_BOTH_RESOURCE_AND_SOURCEATTR_PRESENT => 0x80520508 as u32,
            XPS::XPS_E_BLEED_BOX_PAGE_DIMENSIONS_NOT_IN_SYNC => 0x80520509 as u32,
            XPS::XPS_E_RELATIONSHIP_EXTERNAL => 0x8052050A as u32,
            XPS::XPS_E_NOT_ENOUGH_GRADIENT_STOPS => 0x8052050B as u32,
            XPS::XPS_E_PACKAGE_WRITER_NOT_CLOSED => 0x8052050C as u32,
            XPS::XPS_E_INVALID_NUMBER_OF_POINTS_IN_CURVE_SEGMENTS => 0x80520600 as u32,
            XPS::XPS_E_ABSOLUTE_REFERENCE => 0x80520601 as u32,
            XPS::XPS_E_INVALID_NUMBER_OF_COLOR_CHANNELS => 0x80520602 as u32,
        }
    }

    pub fn error(&self) -> RawError {
        return match self {
            XPS::XPS_E_INVALID_LANGUAGE => RawError::Kind(XPS::XPS_E_INVALID_LANGUAGE),
            XPS::XPS_E_INVALID_NAME => RawError::Kind(XPS::XPS_E_INVALID_NAME),
            XPS::XPS_E_INVALID_RESOURCE_KEY => RawError::Kind(XPS::XPS_E_INVALID_RESOURCE_KEY),
            XPS::XPS_E_INVALID_PAGE_SIZE => RawError::Kind(XPS::XPS_E_INVALID_PAGE_SIZE),
            XPS::XPS_E_INVALID_BLEED_BOX => RawError::Kind(XPS::XPS_E_INVALID_BLEED_BOX),
            XPS::XPS_E_INVALID_THUMBNAIL_IMAGE_TYPE => RawError::Kind(XPS::XPS_E_INVALID_THUMBNAIL_IMAGE_TYPE),
            XPS::XPS_E_INVALID_LOOKUP_TYPE => RawError::Kind(XPS::XPS_E_INVALID_LOOKUP_TYPE),
            XPS::XPS_E_INVALID_FLOAT => RawError::Kind(XPS::XPS_E_INVALID_FLOAT),
            XPS::XPS_E_UNEXPECTED_CONTENT_TYPE => RawError::Kind(XPS::XPS_E_UNEXPECTED_CONTENT_TYPE),
            XPS::XPS_E_INVALID_FONT_URI => RawError::Kind(XPS::XPS_E_INVALID_FONT_URI),
            XPS::XPS_E_INVALID_CONTENT_BOX => RawError::Kind(XPS::XPS_E_INVALID_CONTENT_BOX),
            XPS::XPS_E_INVALID_MARKUP => RawError::Kind(XPS::XPS_E_INVALID_MARKUP),
            XPS::XPS_E_INVALID_XML_ENCODING => RawError::Kind(XPS::XPS_E_INVALID_XML_ENCODING),
            XPS::XPS_E_INVALID_CONTENT_TYPE => RawError::Kind(XPS::XPS_E_INVALID_CONTENT_TYPE),
            XPS::XPS_E_INVALID_OBFUSCATED_FONT_URI => RawError::Kind(XPS::XPS_E_INVALID_OBFUSCATED_FONT_URI),
            XPS::XPS_E_UNEXPECTED_RELATIONSHIP_TYPE => RawError::Kind(XPS::XPS_E_UNEXPECTED_RELATIONSHIP_TYPE),
            XPS::XPS_E_UNEXPECTED_RESTRICTED_FONT_RELATIONSHIP => RawError::Kind(XPS::XPS_E_UNEXPECTED_RESTRICTED_FONT_RELATIONSHIP),
            XPS::XPS_E_MISSING_NAME => RawError::Kind(XPS::XPS_E_MISSING_NAME),
            XPS::XPS_E_MISSING_LOOKUP => RawError::Kind(XPS::XPS_E_MISSING_LOOKUP),
            XPS::XPS_E_MISSING_GLYPHS => RawError::Kind(XPS::XPS_E_MISSING_GLYPHS),
            XPS::XPS_E_MISSING_SEGMENT_DATA => RawError::Kind(XPS::XPS_E_MISSING_SEGMENT_DATA),
            XPS::XPS_E_MISSING_COLORPROFILE => RawError::Kind(XPS::XPS_E_MISSING_COLORPROFILE),
            XPS::XPS_E_MISSING_RELATIONSHIP_TARGET => RawError::Kind(XPS::XPS_E_MISSING_RELATIONSHIP_TARGET),
            XPS::XPS_E_MISSING_RESOURCE_RELATIONSHIP => RawError::Kind(XPS::XPS_E_MISSING_RESOURCE_RELATIONSHIP),
            XPS::XPS_E_MISSING_FONTURI => RawError::Kind(XPS::XPS_E_MISSING_FONTURI),
            XPS::XPS_E_MISSING_DOCUMENTSEQUENCE_RELATIONSHIP => RawError::Kind(XPS::XPS_E_MISSING_DOCUMENTSEQUENCE_RELATIONSHIP),
            XPS::XPS_E_MISSING_DOCUMENT => RawError::Kind(XPS::XPS_E_MISSING_DOCUMENT),
            XPS::XPS_E_MISSING_REFERRED_DOCUMENT => RawError::Kind(XPS::XPS_E_MISSING_REFERRED_DOCUMENT),
            XPS::XPS_E_MISSING_REFERRED_PAGE => RawError::Kind(XPS::XPS_E_MISSING_REFERRED_PAGE),
            XPS::XPS_E_MISSING_PAGE_IN_DOCUMENT => RawError::Kind(XPS::XPS_E_MISSING_PAGE_IN_DOCUMENT),
            XPS::XPS_E_MISSING_PAGE_IN_PAGEREFERENCE => RawError::Kind(XPS::XPS_E_MISSING_PAGE_IN_PAGEREFERENCE),
            XPS::XPS_E_MISSING_IMAGE_IN_IMAGEBRUSH => RawError::Kind(XPS::XPS_E_MISSING_IMAGE_IN_IMAGEBRUSH),
            XPS::XPS_E_MISSING_RESOURCE_KEY => RawError::Kind(XPS::XPS_E_MISSING_RESOURCE_KEY),
            XPS::XPS_E_MISSING_PART_REFERENCE => RawError::Kind(XPS::XPS_E_MISSING_PART_REFERENCE),
            XPS::XPS_E_MISSING_RESTRICTED_FONT_RELATIONSHIP => RawError::Kind(XPS::XPS_E_MISSING_RESTRICTED_FONT_RELATIONSHIP),
            XPS::XPS_E_MISSING_DISCARDCONTROL => RawError::Kind(XPS::XPS_E_MISSING_DISCARDCONTROL),
            XPS::XPS_E_MISSING_PART_STREAM => RawError::Kind(XPS::XPS_E_MISSING_PART_STREAM),
            XPS::XPS_E_UNAVAILABLE_PACKAGE => RawError::Kind(XPS::XPS_E_UNAVAILABLE_PACKAGE),
            XPS::XPS_E_DUPLICATE_RESOURCE_KEYS => RawError::Kind(XPS::XPS_E_DUPLICATE_RESOURCE_KEYS),
            XPS::XPS_E_MULTIPLE_RESOURCES => RawError::Kind(XPS::XPS_E_MULTIPLE_RESOURCES),
            XPS::XPS_E_MULTIPLE_DOCUMENTSEQUENCE_RELATIONSHIPS => RawError::Kind(XPS::XPS_E_MULTIPLE_DOCUMENTSEQUENCE_RELATIONSHIPS),
            XPS::XPS_E_MULTIPLE_THUMBNAILS_ON_PAGE => RawError::Kind(XPS::XPS_E_MULTIPLE_THUMBNAILS_ON_PAGE),
            XPS::XPS_E_MULTIPLE_THUMBNAILS_ON_PACKAGE => RawError::Kind(XPS::XPS_E_MULTIPLE_THUMBNAILS_ON_PACKAGE),
            XPS::XPS_E_MULTIPLE_PRINTTICKETS_ON_PAGE => RawError::Kind(XPS::XPS_E_MULTIPLE_PRINTTICKETS_ON_PAGE),
            XPS::XPS_E_MULTIPLE_PRINTTICKETS_ON_DOCUMENT => RawError::Kind(XPS::XPS_E_MULTIPLE_PRINTTICKETS_ON_DOCUMENT),
            XPS::XPS_E_MULTIPLE_PRINTTICKETS_ON_DOCUMENTSEQUENCE => RawError::Kind(XPS::XPS_E_MULTIPLE_PRINTTICKETS_ON_DOCUMENTSEQUENCE),
            XPS::XPS_E_MULTIPLE_REFERENCES_TO_PART => RawError::Kind(XPS::XPS_E_MULTIPLE_REFERENCES_TO_PART),
            XPS::XPS_E_DUPLICATE_NAMES => RawError::Kind(XPS::XPS_E_DUPLICATE_NAMES),
            XPS::XPS_E_STRING_TOO_LONG => RawError::Kind(XPS::XPS_E_STRING_TOO_LONG),
            XPS::XPS_E_TOO_MANY_INDICES => RawError::Kind(XPS::XPS_E_TOO_MANY_INDICES),
            XPS::XPS_E_MAPPING_OUT_OF_ORDER => RawError::Kind(XPS::XPS_E_MAPPING_OUT_OF_ORDER),
            XPS::XPS_E_MAPPING_OUTSIDE_STRING => RawError::Kind(XPS::XPS_E_MAPPING_OUTSIDE_STRING),
            XPS::XPS_E_MAPPING_OUTSIDE_INDICES => RawError::Kind(XPS::XPS_E_MAPPING_OUTSIDE_INDICES),
            XPS::XPS_E_CARET_OUTSIDE_STRING => RawError::Kind(XPS::XPS_E_CARET_OUTSIDE_STRING),
            XPS::XPS_E_CARET_OUT_OF_ORDER => RawError::Kind(XPS::XPS_E_CARET_OUT_OF_ORDER),
            XPS::XPS_E_ODD_BIDILEVEL => RawError::Kind(XPS::XPS_E_ODD_BIDILEVEL),
            XPS::XPS_E_ONE_TO_ONE_MAPPING_EXPECTED => RawError::Kind(XPS::XPS_E_ONE_TO_ONE_MAPPING_EXPECTED),
            XPS::XPS_E_RESTRICTED_FONT_NOT_OBFUSCATED => RawError::Kind(XPS::XPS_E_RESTRICTED_FONT_NOT_OBFUSCATED),
            XPS::XPS_E_NEGATIVE_FLOAT => RawError::Kind(XPS::XPS_E_NEGATIVE_FLOAT),
            XPS::XPS_E_SIGREQUESTID_DUP => RawError::Kind(XPS::XPS_E_SIGREQUESTID_DUP),
            XPS::XPS_E_PACKAGE_NOT_OPENED => RawError::Kind(XPS::XPS_E_PACKAGE_NOT_OPENED),
            XPS::XPS_E_PACKAGE_ALREADY_OPENED => RawError::Kind(XPS::XPS_E_PACKAGE_ALREADY_OPENED),
            XPS::XPS_E_SIGNATUREID_DUP => RawError::Kind(XPS::XPS_E_SIGNATUREID_DUP),
            XPS::XPS_E_MARKUP_COMPATIBILITY_ELEMENTS => RawError::Kind(XPS::XPS_E_MARKUP_COMPATIBILITY_ELEMENTS),
            XPS::XPS_E_OBJECT_DETACHED => RawError::Kind(XPS::XPS_E_OBJECT_DETACHED),
            XPS::XPS_E_INVALID_SIGNATUREBLOCK_MARKUP => RawError::Kind(XPS::XPS_E_INVALID_SIGNATUREBLOCK_MARKUP),
            XPS::XPS_E_XKEY_ATTR_PRESENT_OUTSIDE_RES_DICT => RawError::Kind(XPS::XPS_E_XKEY_ATTR_PRESENT_OUTSIDE_RES_DICT),
            XPS::XPS_E_DICTIONARY_ITEM_NAMED => RawError::Kind(XPS::XPS_E_DICTIONARY_ITEM_NAMED),
            XPS::XPS_E_NESTED_REMOTE_DICTIONARY => RawError::Kind(XPS::XPS_E_NESTED_REMOTE_DICTIONARY),
            XPS::XPS_E_INDEX_OUT_OF_RANGE => RawError::Kind(XPS::XPS_E_INDEX_OUT_OF_RANGE),
            XPS::XPS_E_VISUAL_CIRCULAR_REF => RawError::Kind(XPS::XPS_E_VISUAL_CIRCULAR_REF),
            XPS::XPS_E_NO_CUSTOM_OBJECTS => RawError::Kind(XPS::XPS_E_NO_CUSTOM_OBJECTS),
            XPS::XPS_E_ALREADY_OWNED => RawError::Kind(XPS::XPS_E_ALREADY_OWNED),
            XPS::XPS_E_RESOURCE_NOT_OWNED => RawError::Kind(XPS::XPS_E_RESOURCE_NOT_OWNED),
            XPS::XPS_E_UNEXPECTED_COLORPROFILE => RawError::Kind(XPS::XPS_E_UNEXPECTED_COLORPROFILE),
            XPS::XPS_E_COLOR_COMPONENT_OUT_OF_RANGE => RawError::Kind(XPS::XPS_E_COLOR_COMPONENT_OUT_OF_RANGE),
            XPS::XPS_E_BOTH_PATHFIGURE_AND_ABBR_SYNTAX_PRESENT => RawError::Kind(XPS::XPS_E_BOTH_PATHFIGURE_AND_ABBR_SYNTAX_PRESENT),
            XPS::XPS_E_BOTH_RESOURCE_AND_SOURCEATTR_PRESENT => RawError::Kind(XPS::XPS_E_BOTH_RESOURCE_AND_SOURCEATTR_PRESENT),
            XPS::XPS_E_BLEED_BOX_PAGE_DIMENSIONS_NOT_IN_SYNC => RawError::Kind(XPS::XPS_E_BLEED_BOX_PAGE_DIMENSIONS_NOT_IN_SYNC),
            XPS::XPS_E_RELATIONSHIP_EXTERNAL => RawError::Kind(XPS::XPS_E_RELATIONSHIP_EXTERNAL),
            XPS::XPS_E_NOT_ENOUGH_GRADIENT_STOPS => RawError::Kind(XPS::XPS_E_NOT_ENOUGH_GRADIENT_STOPS),
            XPS::XPS_E_PACKAGE_WRITER_NOT_CLOSED => RawError::Kind(XPS::XPS_E_PACKAGE_WRITER_NOT_CLOSED),
            XPS::XPS_E_INVALID_NUMBER_OF_POINTS_IN_CURVE_SEGMENTS => RawError::Kind(XPS::XPS_E_INVALID_NUMBER_OF_POINTS_IN_CURVE_SEGMENTS),
            XPS::XPS_E_ABSOLUTE_REFERENCE => RawError::Kind(XPS::XPS_E_ABSOLUTE_REFERENCE),
            XPS::XPS_E_INVALID_NUMBER_OF_COLOR_CHANNELS => RawError::Kind(XPS::XPS_E_INVALID_NUMBER_OF_COLOR_CHANNELS),
        }
    }

    pub fn description(&self) -> &'static str {
        return match self {
            XPS::XPS_E_INVALID_LANGUAGE => "The specified language is either not valid or not correctly formatted.",
            XPS::XPS_E_INVALID_NAME => "The string that was passed is not a valid name, according to the XML Paper Specification.",
            XPS::XPS_E_INVALID_RESOURCE_KEY => "According to the XML Paper Specification, the lookup key string is not valid.",
            XPS::XPS_E_INVALID_PAGE_SIZE => "The page dimensions contain a page size value that is not valid.",
            XPS::XPS_E_INVALID_BLEED_BOX => "The bleed box rectangle contains one or more values that are not valid. See the parameter description for the valid values.",
            XPS::XPS_E_INVALID_THUMBNAIL_IMAGE_TYPE => "The thumbnail image type is not supported.",
            XPS::XPS_E_INVALID_LOOKUP_TYPE => "The lookup key name references an object that is not the correct type for the call; for example, if the method returns a brush but the lookup key name refers to a geometry object.",
            XPS::XPS_E_INVALID_FLOAT => "A FLOAT value is not valid. It is either an infinite or not a number (NAN).",
            XPS::XPS_E_UNEXPECTED_CONTENT_TYPE => "The target of a relationship is not the type expected by the context of the relationship.",
            XPS::XPS_E_INVALID_FONT_URI => "The font URI is not valid, possibly because it contains an empty fragment or characters that are not valid.",
            XPS::XPS_E_INVALID_CONTENT_BOX => "The content box rectangle contains one or more values that are not valid. See the parameter description for the valid values.",
            XPS::XPS_E_INVALID_MARKUP => "The markup being read contains an element or an attribute that does not conform to the XML Paper Specification. Note To represent floating-point values, the XPS OM uses the FLOAT data type instead of DOUBLE. If an XPS document has an element with floating-point data that does not fit into a FLOAT value, this error will be returned when that value is encountered during deserialization.",
            XPS::XPS_E_INVALID_XML_ENCODING => "Found improper or incorrectly formatted XML markup.",
            XPS::XPS_E_INVALID_CONTENT_TYPE => "The content type string is not valid.",
            XPS::XPS_E_INVALID_OBFUSCATED_FONT_URI => "Reserved.",
            XPS::XPS_E_UNEXPECTED_RELATIONSHIP_TYPE => "The relationship type was not recognized.",
            XPS::XPS_E_UNEXPECTED_RESTRICTED_FONT_RELATIONSHIP => "The restricted font collection contains an unrestricted font.",
            XPS::XPS_E_MISSING_NAME => "The page has not been named; the hyperlink target status can only be set if the page has a name.",
            XPS::XPS_E_MISSING_LOOKUP => "The remote resource has an unexpected object.",
            XPS::XPS_E_MISSING_GLYPHS => "The IXpsOMGlyphs interface without a Unicode string does not specify any glyph indices. An IXpsOMGlyphs interface must specify either a Unicode string or an array of glyph indices.",
            XPS::XPS_E_MISSING_SEGMENT_DATA => "The segment data array has fewer entries than the segment types array.",
            XPS::XPS_E_MISSING_COLORPROFILE => "The color profile parameter is NULL, but a color profile is expected. A color profile is required when the color type is XPS_COLOR_TYPE_CONTEXT.",
            XPS::XPS_E_MISSING_RELATIONSHIP_TARGET => "The relationship target part is not present in the package relationship.",
            XPS::XPS_E_MISSING_RESOURCE_RELATIONSHIP => "The resource referred to by the page or remote dictionary content does not exist as a page relationship.",
            XPS::XPS_E_MISSING_FONTURI => "The IXpsOMGlyphs interface requires a font URI, but one is not specified.",
            XPS::XPS_E_MISSING_DOCUMENTSEQUENCE_RELATIONSHIP => "The package does not contain a FixedDocumentSequence.",
            XPS::XPS_E_MISSING_DOCUMENT => "IXpsOMPackageWriter::AddPage was called before IXpsOMPackageWriter::StartNewDocument.",
            XPS::XPS_E_MISSING_REFERRED_DOCUMENT => "The FixedDocument part that is referenced by the FixedDocumentSequence could not be found. An XPS document must contain at least one FixedDocument.",
            XPS::XPS_E_MISSING_REFERRED_PAGE => "The FixedPage part that is referenced by the FixedDocument could not be found. An XPS document must contain at least one FixedPage part.",
            XPS::XPS_E_MISSING_PAGE_IN_DOCUMENT => "The FixedDocument does not contain any FixedPage parts. An XPS document must contain at least one FixedPage part.",
            XPS::XPS_E_MISSING_PAGE_IN_PAGEREFERENCE => "The page reference does not have a corresponding page.",
            XPS::XPS_E_MISSING_IMAGE_IN_IMAGEBRUSH => "An image resource could not be located for the image brush.",
            XPS::XPS_E_MISSING_RESOURCE_KEY => "No x:Key attribute was specified for the resource.",
            XPS::XPS_E_MISSING_PART_REFERENCE => "A required target part was not referenced.",
            XPS::XPS_E_MISSING_RESTRICTED_FONT_RELATIONSHIP => "The referenced restricted font was not specified in the call to IXpsOMPackageWriter::StartNewDocument.",
            XPS::XPS_E_MISSING_DISCARDCONTROL => "A page refers to discardable resources but does not specify a DiscardControl part name.",
            XPS::XPS_E_MISSING_PART_STREAM => "A stream was not specified for the resource.",
            XPS::XPS_E_UNAVAILABLE_PACKAGE => "A severe error occurred and the contents of the XPS OM might be unrecoverable. Some components of the XPS OM might still be usable, but they will need to be verified before being used further. Because the state of the XPS OM cannot be predicted after this error is returned, all components of the XPS OM should be released and discarded.",
            XPS::XPS_E_DUPLICATE_RESOURCE_KEYS => "An object with this key name already exists in the dictionary.",
            XPS::XPS_E_MULTIPLE_RESOURCES => "A resource by that part name already exists.",
            XPS::XPS_E_MULTIPLE_DOCUMENTSEQUENCE_RELATIONSHIPS => "An attempt was made to add a FixedDocumentSequence to a package that already has one. An XPS document must contain one and only one FixedDocumentSequence part.",
            XPS::XPS_E_MULTIPLE_THUMBNAILS_ON_PAGE => "An attempt was made to add a page-level thumbnail image to a FixedPage that already has one. A FixedPage in an XPS document can contain only one page-level thumbnail image.",
            XPS::XPS_E_MULTIPLE_THUMBNAILS_ON_PACKAGE => "An attempt was made to add a thumbnail image to a package that already has one. An XPS document can contain only one package-level thumbnail image.",
            XPS::XPS_E_MULTIPLE_PRINTTICKETS_ON_PAGE => "An attempt was made to add a page-level print ticket to a FixedPage that already has one. A FixedPage in an XPS document can contain only one page-level print ticket.",
            XPS::XPS_E_MULTIPLE_PRINTTICKETS_ON_DOCUMENT => "An attempt was made to add a document-level print ticket to a FixedDocument that already has one. A FixedDocument in an XPS document can contain only one document-level print ticket.",
            XPS::XPS_E_MULTIPLE_PRINTTICKETS_ON_DOCUMENTSEQUENCE => "An attempt was made to add a job-level print ticket to a FixedDocumentSequence that already has one. The FixedDocumentSequence in an XPS document can contain only one job-level print ticket.",
            XPS::XPS_E_MULTIPLE_REFERENCES_TO_PART => "The restricted font collection contained a restricted font entry that was repeated. Each font entry can occur in the collection only once.",
            XPS::XPS_E_DUPLICATE_NAMES => "An object with this name already exists in the dictionary.",
            XPS::XPS_E_STRING_TOO_LONG => "A size_t overflow occurred during an attempt to copy a string into a new buffer.",
            XPS::XPS_E_TOO_MANY_INDICES => "There were more glyph indices than Unicode code points. If there are no glyph mappings, the number of glyph indices must be less than or equal to the number of Unicode code points.",
            XPS::XPS_E_MAPPING_OUT_OF_ORDER => "In one or more XPS_GLYPH_MAPPING structures, an element is out of sequence.",
            XPS::XPS_E_MAPPING_OUTSIDE_STRING => "Error in the glyph mappings. If the Unicode string is empty, this error means that a glyph mapping was also defined. Glyph mappings must not be defined if the Unicode string is empty. If the Unicode string is not empty, this error means that a glyph mapping was defined for glyphs outside of the Unicode string. Glyph mappings cannot be defined for glyphs that fall outside the length of the Unicode string.",
            XPS::XPS_E_MAPPING_OUTSIDE_INDICES => "The glyph mappings exceed the number of glyph indices.",
            XPS::XPS_E_CARET_OUTSIDE_STRING => "Caret stops were specified for an empty string; or, the caret jump index has exceeded the length of the Unicode string.",
            XPS::XPS_E_CARET_OUT_OF_ORDER => "A caret location value is out of order. The location values must be sorted in ascending order.",
            XPS::XPS_E_ODD_BIDILEVEL => "The text string was specified as being oriented sideways and right-to-left. If the text is oriented sideways, it cannot have a bidi level that is an odd value (right-to-left). Likewise, if the bidi level is an odd value, the text cannot be oriented sideways.",
            XPS::XPS_E_ONE_TO_ONE_MAPPING_EXPECTED => "The glyph mappings do not match the Unicode string contents.",
            XPS::XPS_E_RESTRICTED_FONT_NOT_OBFUSCATED => "Reserved.",
            XPS::XPS_E_NEGATIVE_FLOAT => "An entry contains a negative value, but it must contain a non-negative value.",
            XPS::XPS_E_SIGREQUESTID_DUP => "XPS Digital Signature: Two or more signatures have the same ID.",
            XPS::XPS_E_PACKAGE_NOT_OPENED => "XPS Digital Signature: An XPS package has not yet been opened in the signature manager.",
            XPS::XPS_E_PACKAGE_ALREADY_OPENED => "XPS Digital Signature: An XPS package has already been opened in the signature manager.",
            XPS::XPS_E_SIGNATUREID_DUP => "XPS Digital Signature: The signature request ID is not unique within the signature block.",
            XPS::XPS_E_MARKUP_COMPATIBILITY_ELEMENTS => "XPS Digital Signature: The XPS_SIGN_FLAGS value specified that no markup compatibility elements were expected; however, markup compatibility elements were found.",
            XPS::XPS_E_OBJECT_DETACHED => "XPS Digital Signature: The interface is not associated with the signature manager.",
            XPS::XPS_E_INVALID_SIGNATUREBLOCK_MARKUP => "XPS Digital Signature: Encountered an error in the XML markup of the signature block while the signature markup was being read.",
            XPS::XPS_E_XKEY_ATTR_PRESENT_OUTSIDE_RES_DICT => "A path geometry that is not in a resource dictionary has an x:Key attribute specified. Path geometries that are not in a resource dictionary cannot have an x:Key attribute.",
            XPS::XPS_E_DICTIONARY_ITEM_NAMED => "A visual in a resource dictionary has the Name attribute, which may not be specified on any children of a ResourceDictionary element.",
            XPS::XPS_E_NESTED_REMOTE_DICTIONARY => "An attempt was made to add a remote dictionary reference to a remote dictionary. A remote dictionary cannot reference another remote dictionary.",
            XPS::XPS_E_INDEX_OUT_OF_RANGE => "Reserved.",
            XPS::XPS_E_VISUAL_CIRCULAR_REF => "Reserved.",
            XPS::XPS_E_NO_CUSTOM_OBJECTS => "An interface pointer does not point to a recognized interface implementation. Custom implementation of XPS Document API interfaces is not supported.",
            XPS::XPS_E_ALREADY_OWNED => "The interface already has an owner.",
            XPS::XPS_E_RESOURCE_NOT_OWNED => "Reserved.",
            XPS::XPS_E_UNEXPECTED_COLORPROFILE => "A color profile was present when one was not expected. A color profile is only allowed when the color type is XPS_COLOR_TYPE_CONTEXT.",
            XPS::XPS_E_COLOR_COMPONENT_OUT_OF_RANGE => "A color value is out of range. For XPS_COLOR_TYPE_SCRGB color types, the alpha channel value must be greater than or equal to 0.0 and less than or equal to +1.0. For XPS_COLOR_TYPE_CONTEXT color types, the channelValues[0] that represents the alpha channel value must be greater than or equal to 0.0 and less than or equal to +1.0.",
            XPS::XPS_E_BOTH_PATHFIGURE_AND_ABBR_SYNTAX_PRESENT => "A PathGeometry element contains a set of path figures that are specified either with the Figures attribute or with a child PathFigure element. The path figures of a geometry cannot have both the Figures attribute and a child PathFigure element.",
            XPS::XPS_E_BOTH_RESOURCE_AND_SOURCEATTR_PRESENT => "A ResourceDictionary element that specifies a remote resource dictionary in its Source attribute MUST NOT contain any resource definition children.",
            XPS::XPS_E_BLEED_BOX_PAGE_DIMENSIONS_NOT_IN_SYNC => "The bleed box dimensions are not compatible with the page dimensions. The bleed box width value must be greater than or equal to the page width plus the absolute value of the x-coordinate of the bleed box origin. The bleed box height value must be greater than or equal to the page height plus the absolute value of the y-coordinate of the bleed box origin.",
            XPS::XPS_E_RELATIONSHIP_EXTERNAL => "A relationship refers to a part that is outside of the XPS document. All content to be rendered in an XPS document must be contained in the XPS document.",
            XPS::XPS_E_NOT_ENOUGH_GRADIENT_STOPS => "The gradient stop collection has fewer than two stops. A gradient stop collection must have at least two gradient stops.",
            XPS::XPS_E_PACKAGE_WRITER_NOT_CLOSED => "The package writer was not closed before it was released.",
            XPS::XPS_E_INVALID_NUMBER_OF_POINTS_IN_CURVE_SEGMENTS => "The OpenXPS document contains an invalid number of color points in curve segments.",
            XPS::XPS_E_ABSOLUTE_REFERENCE => "The OpenXPS document contains XML elements that use absolute URIs to reference other parts in the document.",
            XPS::XPS_E_INVALID_NUMBER_OF_COLOR_CHANNELS => "The OpenXPS document contains an invalid number of color channels.",
        }
    }

    pub fn from_name(name: &str) -> XPS {
        return match name {
            "XPS_E_INVALID_LANGUAGE" => XPS::XPS_E_INVALID_LANGUAGE,
            "XPS_E_INVALID_NAME" => XPS::XPS_E_INVALID_NAME,
            "XPS_E_INVALID_RESOURCE_KEY" => XPS::XPS_E_INVALID_RESOURCE_KEY,
            "XPS_E_INVALID_PAGE_SIZE" => XPS::XPS_E_INVALID_PAGE_SIZE,
            "XPS_E_INVALID_BLEED_BOX" => XPS::XPS_E_INVALID_BLEED_BOX,
            "XPS_E_INVALID_THUMBNAIL_IMAGE_TYPE" => XPS::XPS_E_INVALID_THUMBNAIL_IMAGE_TYPE,
            "XPS_E_INVALID_LOOKUP_TYPE" => XPS::XPS_E_INVALID_LOOKUP_TYPE,
            "XPS_E_INVALID_FLOAT" => XPS::XPS_E_INVALID_FLOAT,
            "XPS_E_UNEXPECTED_CONTENT_TYPE" => XPS::XPS_E_UNEXPECTED_CONTENT_TYPE,
            "XPS_E_INVALID_FONT_URI" => XPS::XPS_E_INVALID_FONT_URI,
            "XPS_E_INVALID_CONTENT_BOX" => XPS::XPS_E_INVALID_CONTENT_BOX,
            "XPS_E_INVALID_MARKUP" => XPS::XPS_E_INVALID_MARKUP,
            "XPS_E_INVALID_XML_ENCODING" => XPS::XPS_E_INVALID_XML_ENCODING,
            "XPS_E_INVALID_CONTENT_TYPE" => XPS::XPS_E_INVALID_CONTENT_TYPE,
            "XPS_E_INVALID_OBFUSCATED_FONT_URI" => XPS::XPS_E_INVALID_OBFUSCATED_FONT_URI,
            "XPS_E_UNEXPECTED_RELATIONSHIP_TYPE" => XPS::XPS_E_UNEXPECTED_RELATIONSHIP_TYPE,
            "XPS_E_UNEXPECTED_RESTRICTED_FONT_RELATIONSHIP" => XPS::XPS_E_UNEXPECTED_RESTRICTED_FONT_RELATIONSHIP,
            "XPS_E_MISSING_NAME" => XPS::XPS_E_MISSING_NAME,
            "XPS_E_MISSING_LOOKUP" => XPS::XPS_E_MISSING_LOOKUP,
            "XPS_E_MISSING_GLYPHS" => XPS::XPS_E_MISSING_GLYPHS,
            "XPS_E_MISSING_SEGMENT_DATA" => XPS::XPS_E_MISSING_SEGMENT_DATA,
            "XPS_E_MISSING_COLORPROFILE" => XPS::XPS_E_MISSING_COLORPROFILE,
            "XPS_E_MISSING_RELATIONSHIP_TARGET" => XPS::XPS_E_MISSING_RELATIONSHIP_TARGET,
            "XPS_E_MISSING_RESOURCE_RELATIONSHIP" => XPS::XPS_E_MISSING_RESOURCE_RELATIONSHIP,
            "XPS_E_MISSING_FONTURI" => XPS::XPS_E_MISSING_FONTURI,
            "XPS_E_MISSING_DOCUMENTSEQUENCE_RELATIONSHIP" => XPS::XPS_E_MISSING_DOCUMENTSEQUENCE_RELATIONSHIP,
            "XPS_E_MISSING_DOCUMENT" => XPS::XPS_E_MISSING_DOCUMENT,
            "XPS_E_MISSING_REFERRED_DOCUMENT" => XPS::XPS_E_MISSING_REFERRED_DOCUMENT,
            "XPS_E_MISSING_REFERRED_PAGE" => XPS::XPS_E_MISSING_REFERRED_PAGE,
            "XPS_E_MISSING_PAGE_IN_DOCUMENT" => XPS::XPS_E_MISSING_PAGE_IN_DOCUMENT,
            "XPS_E_MISSING_PAGE_IN_PAGEREFERENCE" => XPS::XPS_E_MISSING_PAGE_IN_PAGEREFERENCE,
            "XPS_E_MISSING_IMAGE_IN_IMAGEBRUSH" => XPS::XPS_E_MISSING_IMAGE_IN_IMAGEBRUSH,
            "XPS_E_MISSING_RESOURCE_KEY" => XPS::XPS_E_MISSING_RESOURCE_KEY,
            "XPS_E_MISSING_PART_REFERENCE" => XPS::XPS_E_MISSING_PART_REFERENCE,
            "XPS_E_MISSING_RESTRICTED_FONT_RELATIONSHIP" => XPS::XPS_E_MISSING_RESTRICTED_FONT_RELATIONSHIP,
            "XPS_E_MISSING_DISCARDCONTROL" => XPS::XPS_E_MISSING_DISCARDCONTROL,
            "XPS_E_MISSING_PART_STREAM" => XPS::XPS_E_MISSING_PART_STREAM,
            "XPS_E_UNAVAILABLE_PACKAGE" => XPS::XPS_E_UNAVAILABLE_PACKAGE,
            "XPS_E_DUPLICATE_RESOURCE_KEYS" => XPS::XPS_E_DUPLICATE_RESOURCE_KEYS,
            "XPS_E_MULTIPLE_RESOURCES" => XPS::XPS_E_MULTIPLE_RESOURCES,
            "XPS_E_MULTIPLE_DOCUMENTSEQUENCE_RELATIONSHIPS" => XPS::XPS_E_MULTIPLE_DOCUMENTSEQUENCE_RELATIONSHIPS,
            "XPS_E_MULTIPLE_THUMBNAILS_ON_PAGE" => XPS::XPS_E_MULTIPLE_THUMBNAILS_ON_PAGE,
            "XPS_E_MULTIPLE_THUMBNAILS_ON_PACKAGE" => XPS::XPS_E_MULTIPLE_THUMBNAILS_ON_PACKAGE,
            "XPS_E_MULTIPLE_PRINTTICKETS_ON_PAGE" => XPS::XPS_E_MULTIPLE_PRINTTICKETS_ON_PAGE,
            "XPS_E_MULTIPLE_PRINTTICKETS_ON_DOCUMENT" => XPS::XPS_E_MULTIPLE_PRINTTICKETS_ON_DOCUMENT,
            "XPS_E_MULTIPLE_PRINTTICKETS_ON_DOCUMENTSEQUENCE" => XPS::XPS_E_MULTIPLE_PRINTTICKETS_ON_DOCUMENTSEQUENCE,
            "XPS_E_MULTIPLE_REFERENCES_TO_PART" => XPS::XPS_E_MULTIPLE_REFERENCES_TO_PART,
            "XPS_E_DUPLICATE_NAMES" => XPS::XPS_E_DUPLICATE_NAMES,
            "XPS_E_STRING_TOO_LONG" => XPS::XPS_E_STRING_TOO_LONG,
            "XPS_E_TOO_MANY_INDICES" => XPS::XPS_E_TOO_MANY_INDICES,
            "XPS_E_MAPPING_OUT_OF_ORDER" => XPS::XPS_E_MAPPING_OUT_OF_ORDER,
            "XPS_E_MAPPING_OUTSIDE_STRING" => XPS::XPS_E_MAPPING_OUTSIDE_STRING,
            "XPS_E_MAPPING_OUTSIDE_INDICES" => XPS::XPS_E_MAPPING_OUTSIDE_INDICES,
            "XPS_E_CARET_OUTSIDE_STRING" => XPS::XPS_E_CARET_OUTSIDE_STRING,
            "XPS_E_CARET_OUT_OF_ORDER" => XPS::XPS_E_CARET_OUT_OF_ORDER,
            "XPS_E_ODD_BIDILEVEL" => XPS::XPS_E_ODD_BIDILEVEL,
            "XPS_E_ONE_TO_ONE_MAPPING_EXPECTED" => XPS::XPS_E_ONE_TO_ONE_MAPPING_EXPECTED,
            "XPS_E_RESTRICTED_FONT_NOT_OBFUSCATED" => XPS::XPS_E_RESTRICTED_FONT_NOT_OBFUSCATED,
            "XPS_E_NEGATIVE_FLOAT" => XPS::XPS_E_NEGATIVE_FLOAT,
            "XPS_E_SIGREQUESTID_DUP" => XPS::XPS_E_SIGREQUESTID_DUP,
            "XPS_E_PACKAGE_NOT_OPENED" => XPS::XPS_E_PACKAGE_NOT_OPENED,
            "XPS_E_PACKAGE_ALREADY_OPENED" => XPS::XPS_E_PACKAGE_ALREADY_OPENED,
            "XPS_E_SIGNATUREID_DUP" => XPS::XPS_E_SIGNATUREID_DUP,
            "XPS_E_MARKUP_COMPATIBILITY_ELEMENTS" => XPS::XPS_E_MARKUP_COMPATIBILITY_ELEMENTS,
            "XPS_E_OBJECT_DETACHED" => XPS::XPS_E_OBJECT_DETACHED,
            "XPS_E_INVALID_SIGNATUREBLOCK_MARKUP" => XPS::XPS_E_INVALID_SIGNATUREBLOCK_MARKUP,
            "XPS_E_XKEY_ATTR_PRESENT_OUTSIDE_RES_DICT" => XPS::XPS_E_XKEY_ATTR_PRESENT_OUTSIDE_RES_DICT,
            "XPS_E_DICTIONARY_ITEM_NAMED" => XPS::XPS_E_DICTIONARY_ITEM_NAMED,
            "XPS_E_NESTED_REMOTE_DICTIONARY" => XPS::XPS_E_NESTED_REMOTE_DICTIONARY,
            "XPS_E_INDEX_OUT_OF_RANGE" => XPS::XPS_E_INDEX_OUT_OF_RANGE,
            "XPS_E_VISUAL_CIRCULAR_REF" => XPS::XPS_E_VISUAL_CIRCULAR_REF,
            "XPS_E_NO_CUSTOM_OBJECTS" => XPS::XPS_E_NO_CUSTOM_OBJECTS,
            "XPS_E_ALREADY_OWNED" => XPS::XPS_E_ALREADY_OWNED,
            "XPS_E_RESOURCE_NOT_OWNED" => XPS::XPS_E_RESOURCE_NOT_OWNED,
            "XPS_E_UNEXPECTED_COLORPROFILE" => XPS::XPS_E_UNEXPECTED_COLORPROFILE,
            "XPS_E_COLOR_COMPONENT_OUT_OF_RANGE" => XPS::XPS_E_COLOR_COMPONENT_OUT_OF_RANGE,
            "XPS_E_BOTH_PATHFIGURE_AND_ABBR_SYNTAX_PRESENT" => XPS::XPS_E_BOTH_PATHFIGURE_AND_ABBR_SYNTAX_PRESENT,
            "XPS_E_BOTH_RESOURCE_AND_SOURCEATTR_PRESENT" => XPS::XPS_E_BOTH_RESOURCE_AND_SOURCEATTR_PRESENT,
            "XPS_E_BLEED_BOX_PAGE_DIMENSIONS_NOT_IN_SYNC" => XPS::XPS_E_BLEED_BOX_PAGE_DIMENSIONS_NOT_IN_SYNC,
            "XPS_E_RELATIONSHIP_EXTERNAL" => XPS::XPS_E_RELATIONSHIP_EXTERNAL,
            "XPS_E_NOT_ENOUGH_GRADIENT_STOPS" => XPS::XPS_E_NOT_ENOUGH_GRADIENT_STOPS,
            "XPS_E_PACKAGE_WRITER_NOT_CLOSED" => XPS::XPS_E_PACKAGE_WRITER_NOT_CLOSED,
            "XPS_E_INVALID_NUMBER_OF_POINTS_IN_CURVE_SEGMENTS" => XPS::XPS_E_INVALID_NUMBER_OF_POINTS_IN_CURVE_SEGMENTS,
            "XPS_E_ABSOLUTE_REFERENCE" => XPS::XPS_E_ABSOLUTE_REFERENCE,
            "XPS_E_INVALID_NUMBER_OF_COLOR_CHANNELS" => XPS::XPS_E_INVALID_NUMBER_OF_COLOR_CHANNELS,
        }
    }
    pub fn from_code(code: u32) -> XPS {
        return match code {
            0x80520000 => XPS::XPS_E_INVALID_LANGUAGE,
            0x80520001 => XPS::XPS_E_INVALID_NAME,
            0x80520002 => XPS::XPS_E_INVALID_RESOURCE_KEY,
            0x80520003 => XPS::XPS_E_INVALID_PAGE_SIZE,
            0x80520004 => XPS::XPS_E_INVALID_BLEED_BOX,
            0x80520005 => XPS::XPS_E_INVALID_THUMBNAIL_IMAGE_TYPE,
            0x80520006 => XPS::XPS_E_INVALID_LOOKUP_TYPE,
            0x80520007 => XPS::XPS_E_INVALID_FLOAT,
            0x80520008 => XPS::XPS_E_UNEXPECTED_CONTENT_TYPE,
            0x8052000A => XPS::XPS_E_INVALID_FONT_URI,
            0x8052000B => XPS::XPS_E_INVALID_CONTENT_BOX,
            0x8052000C => XPS::XPS_E_INVALID_MARKUP,
            0x8052000D => XPS::XPS_E_INVALID_XML_ENCODING,
            0x8052000E => XPS::XPS_E_INVALID_CONTENT_TYPE,
            0x8052000F => XPS::XPS_E_INVALID_OBFUSCATED_FONT_URI,
            0x80520010 => XPS::XPS_E_UNEXPECTED_RELATIONSHIP_TYPE,
            0x80520011 => XPS::XPS_E_UNEXPECTED_RESTRICTED_FONT_RELATIONSHIP,
            0x80520100 => XPS::XPS_E_MISSING_NAME,
            0x80520101 => XPS::XPS_E_MISSING_LOOKUP,
            0x80520102 => XPS::XPS_E_MISSING_GLYPHS,
            0x80520103 => XPS::XPS_E_MISSING_SEGMENT_DATA,
            0x80520104 => XPS::XPS_E_MISSING_COLORPROFILE,
            0x80520105 => XPS::XPS_E_MISSING_RELATIONSHIP_TARGET,
            0x80520106 => XPS::XPS_E_MISSING_RESOURCE_RELATIONSHIP,
            0x80520107 => XPS::XPS_E_MISSING_FONTURI,
            0x80520108 => XPS::XPS_E_MISSING_DOCUMENTSEQUENCE_RELATIONSHIP,
            0x80520109 => XPS::XPS_E_MISSING_DOCUMENT,
            0x8052010A => XPS::XPS_E_MISSING_REFERRED_DOCUMENT,
            0x8052010B => XPS::XPS_E_MISSING_REFERRED_PAGE,
            0x8052010C => XPS::XPS_E_MISSING_PAGE_IN_DOCUMENT,
            0x8052010D => XPS::XPS_E_MISSING_PAGE_IN_PAGEREFERENCE,
            0x8052010E => XPS::XPS_E_MISSING_IMAGE_IN_IMAGEBRUSH,
            0x8052010F => XPS::XPS_E_MISSING_RESOURCE_KEY,
            0x80520110 => XPS::XPS_E_MISSING_PART_REFERENCE,
            0x80520111 => XPS::XPS_E_MISSING_RESTRICTED_FONT_RELATIONSHIP,
            0x80520112 => XPS::XPS_E_MISSING_DISCARDCONTROL,
            0x80520113 => XPS::XPS_E_MISSING_PART_STREAM,
            0x80520114 => XPS::XPS_E_UNAVAILABLE_PACKAGE,
            0x80520200 => XPS::XPS_E_DUPLICATE_RESOURCE_KEYS,
            0x80520201 => XPS::XPS_E_MULTIPLE_RESOURCES,
            0x80520202 => XPS::XPS_E_MULTIPLE_DOCUMENTSEQUENCE_RELATIONSHIPS,
            0x80520203 => XPS::XPS_E_MULTIPLE_THUMBNAILS_ON_PAGE,
            0x80520204 => XPS::XPS_E_MULTIPLE_THUMBNAILS_ON_PACKAGE,
            0x80520205 => XPS::XPS_E_MULTIPLE_PRINTTICKETS_ON_PAGE,
            0x80520206 => XPS::XPS_E_MULTIPLE_PRINTTICKETS_ON_DOCUMENT,
            0x80520207 => XPS::XPS_E_MULTIPLE_PRINTTICKETS_ON_DOCUMENTSEQUENCE,
            0x80520208 => XPS::XPS_E_MULTIPLE_REFERENCES_TO_PART,
            0x80520209 => XPS::XPS_E_DUPLICATE_NAMES,
            0x80520300 => XPS::XPS_E_STRING_TOO_LONG,
            0x80520301 => XPS::XPS_E_TOO_MANY_INDICES,
            0x80520302 => XPS::XPS_E_MAPPING_OUT_OF_ORDER,
            0x80520303 => XPS::XPS_E_MAPPING_OUTSIDE_STRING,
            0x80520304 => XPS::XPS_E_MAPPING_OUTSIDE_INDICES,
            0x80520305 => XPS::XPS_E_CARET_OUTSIDE_STRING,
            0x80520306 => XPS::XPS_E_CARET_OUT_OF_ORDER,
            0x80520307 => XPS::XPS_E_ODD_BIDILEVEL,
            0x80520308 => XPS::XPS_E_ONE_TO_ONE_MAPPING_EXPECTED,
            0x80520309 => XPS::XPS_E_RESTRICTED_FONT_NOT_OBFUSCATED,
            0x8052030A => XPS::XPS_E_NEGATIVE_FLOAT,
            0x80520385 => XPS::XPS_E_SIGREQUESTID_DUP,
            0x80520386 => XPS::XPS_E_PACKAGE_NOT_OPENED,
            0x80520387 => XPS::XPS_E_PACKAGE_ALREADY_OPENED,
            0x80520388 => XPS::XPS_E_SIGNATUREID_DUP,
            0x80520389 => XPS::XPS_E_MARKUP_COMPATIBILITY_ELEMENTS,
            0x8052038A => XPS::XPS_E_OBJECT_DETACHED,
            0x8052038B => XPS::XPS_E_INVALID_SIGNATUREBLOCK_MARKUP,
            0x80520400 => XPS::XPS_E_XKEY_ATTR_PRESENT_OUTSIDE_RES_DICT,
            0x80520401 => XPS::XPS_E_DICTIONARY_ITEM_NAMED,
            0x80520402 => XPS::XPS_E_NESTED_REMOTE_DICTIONARY,
            0x80520500 => XPS::XPS_E_INDEX_OUT_OF_RANGE,
            0x80520501 => XPS::XPS_E_VISUAL_CIRCULAR_REF,
            0x80520502 => XPS::XPS_E_NO_CUSTOM_OBJECTS,
            0x80520503 => XPS::XPS_E_ALREADY_OWNED,
            0x80520504 => XPS::XPS_E_RESOURCE_NOT_OWNED,
            0x80520505 => XPS::XPS_E_UNEXPECTED_COLORPROFILE,
            0x80520506 => XPS::XPS_E_COLOR_COMPONENT_OUT_OF_RANGE,
            0x80520507 => XPS::XPS_E_BOTH_PATHFIGURE_AND_ABBR_SYNTAX_PRESENT,
            0x80520508 => XPS::XPS_E_BOTH_RESOURCE_AND_SOURCEATTR_PRESENT,
            0x80520509 => XPS::XPS_E_BLEED_BOX_PAGE_DIMENSIONS_NOT_IN_SYNC,
            0x8052050A => XPS::XPS_E_RELATIONSHIP_EXTERNAL,
            0x8052050B => XPS::XPS_E_NOT_ENOUGH_GRADIENT_STOPS,
            0x8052050C => XPS::XPS_E_PACKAGE_WRITER_NOT_CLOSED,
            0x80520600 => XPS::XPS_E_INVALID_NUMBER_OF_POINTS_IN_CURVE_SEGMENTS,
            0x80520601 => XPS::XPS_E_ABSOLUTE_REFERENCE,
            0x80520602 => XPS::XPS_E_INVALID_NUMBER_OF_COLOR_CHANNELS,
        }
    }
}
