/*
 * Copyright 2023 Stanislav Mikhailov (xavetar)
 *
 * Licensed under the Creative Commons Zero v1.0 Universal (CC0) License.
 * You may obtain a copy of the License at
 *
 *     http://creativecommons.org/publicdomain/zero/1.0/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the CC0 license is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// It is strictly forbidden to use the from_code method, respected Microsoft decided to define
// duplicate codes, but by defining different constant names, therefore, when using these codes,
// it is mandatory to use through from_name, and not from_code. Otherwise, it may cause undefined
// behavior or an unknown exception. Because one code corresponds to several constants.

use super::{RawError};

#[derive(Clone, Debug, Copy, Eq, Hash, Ord, PartialEq, PartialOrd)]
pub enum SECURITY_SSPI {
    SEC_I_CONTINUE_NEEDED,
    SEC_I_COMPLETE_NEEDED,
    SEC_I_COMPLETE_AND_CONTINUE,
    SEC_I_LOCAL_LOGON,
    SEC_I_GENERIC_EXTENSION_RECEIVED,
    SEC_I_CONTEXT_EXPIRED,
    SEC_I_INCOMPLETE_CREDENTIALS,
    SEC_I_RENEGOTIATE,
    SEC_I_NO_LSA_CONTEXT,
    SEC_I_SIGNATURE_NEEDED,
    SEC_I_NO_RENEGOTIATION,
    SEC_I_MESSAGE_FRAGMENT,
    SEC_I_CONTINUE_NEEDED_MESSAGE_OK,
    SEC_I_ASYNC_CALL_PENDING,
    CRYPT_I_NEW_PROTECTION_REQUIRED,
    NTE_BAD_UID,
    NTE_BAD_HASH,
    NTE_BAD_KEY,
    NTE_BAD_LEN,
    NTE_BAD_DATA,
    NTE_BAD_SIGNATURE,
    NTE_BAD_VER,
    NTE_BAD_ALGID,
    NTE_BAD_FLAGS,
    NTE_BAD_TYPE,
    NTE_BAD_KEY_STATE,
    NTE_BAD_HASH_STATE,
    NTE_NO_KEY,
    NTE_NO_MEMORY,
    NTE_EXISTS,
    NTE_PERM,
    NTE_NOT_FOUND,
    NTE_DOUBLE_ENCRYPT,
    NTE_BAD_PROVIDER,
    NTE_BAD_PROV_TYPE,
    NTE_BAD_PUBLIC_KEY,
    NTE_BAD_KEYSET,
    NTE_PROV_TYPE_NOT_DEF,
    NTE_PROV_TYPE_ENTRY_BAD,
    NTE_KEYSET_NOT_DEF,
    NTE_KEYSET_ENTRY_BAD,
    NTE_PROV_TYPE_NO_MATCH,
    NTE_SIGNATURE_FILE_BAD,
    NTE_PROVIDER_DLL_FAIL,
    NTE_PROV_DLL_NOT_FOUND,
    NTE_BAD_KEYSET_PARAM,
    NTE_FAIL,
    NTE_SYS_ERR,
    NTE_SILENT_CONTEXT,
    NTE_TOKEN_KEYSET_STORAGE_FULL,
    NTE_TEMPORARY_PROFILE,
    NTE_FIXEDPARAMETER,
    NTE_INVALID_HANDLE,
    NTE_INVALID_PARAMETER,
    NTE_BUFFER_TOO_SMALL,
    NTE_NOT_SUPPORTED,
    NTE_NO_MORE_ITEMS,
    NTE_BUFFERS_OVERLAP,
    NTE_DECRYPTION_FAILURE,
    NTE_INTERNAL_ERROR,
    NTE_UI_REQUIRED,
    NTE_HMAC_NOT_SUPPORTED,
    NTE_DEVICE_NOT_READY,
    NTE_AUTHENTICATION_IGNORED,
    NTE_VALIDATION_FAILED,
    NTE_INCORRECT_PASSWORD,
    NTE_ENCRYPTION_FAILURE,
    NTE_DEVICE_NOT_FOUND,
    NTE_USER_CANCELLED,
    NTE_PASSWORD_CHANGE_REQUIRED,
    NTE_NOT_ACTIVE_CONSOLE,
    SEC_E_INSUFFICIENT_MEMORY,
    SEC_E_INVALID_HANDLE,
    SEC_E_UNSUPPORTED_FUNCTION,
    SEC_E_TARGET_UNKNOWN,
    SEC_E_INTERNAL_ERROR,
    SEC_E_SECPKG_NOT_FOUND,
    SEC_E_NOT_OWNER,
    SEC_E_CANNOT_INSTALL,
    SEC_E_INVALID_TOKEN,
    SEC_E_CANNOT_PACK,
    SEC_E_QOP_NOT_SUPPORTED,
    SEC_E_NO_IMPERSONATION,
    SEC_E_LOGON_DENIED,
    SEC_E_UNKNOWN_CREDENTIALS,
    SEC_E_NO_CREDENTIALS,
    SEC_E_MESSAGE_ALTERED,
    SEC_E_OUT_OF_SEQUENCE,
    SEC_E_NO_AUTHENTICATING_AUTHORITY,
    SEC_E_BAD_PKGID,
    SEC_E_CONTEXT_EXPIRED,
    SEC_E_INCOMPLETE_MESSAGE,
    SEC_E_INCOMPLETE_CREDENTIALS,
    SEC_E_BUFFER_TOO_SMALL,
    SEC_E_WRONG_PRINCIPAL,
    SEC_E_TIME_SKEW,
    SEC_E_UNTRUSTED_ROOT,
    SEC_E_ILLEGAL_MESSAGE,
    SEC_E_CERT_UNKNOWN,
    SEC_E_CERT_EXPIRED,
    SEC_E_ENCRYPT_FAILURE,
    SEC_E_DECRYPT_FAILURE,
    SEC_E_ALGORITHM_MISMATCH,
    SEC_E_SECURITY_QOS_FAILED,
    SEC_E_UNFINISHED_CONTEXT_DELETED,
    SEC_E_NO_TGT_REPLY,
    SEC_E_NO_IP_ADDRESSES,
    SEC_E_WRONG_CREDENTIAL_HANDLE,
    SEC_E_CRYPTO_SYSTEM_INVALID,
    SEC_E_MAX_REFERRALS_EXCEEDED,
    SEC_E_MUST_BE_KDC,
    SEC_E_STRONG_CRYPTO_NOT_SUPPORTED,
    SEC_E_TOO_MANY_PRINCIPALS,
    SEC_E_NO_PA_DATA,
    SEC_E_PKINIT_NAME_MISMATCH,
    SEC_E_SMARTCARD_LOGON_REQUIRED,
    SEC_E_SHUTDOWN_IN_PROGRESS,
    SEC_E_KDC_INVALID_REQUEST,
    SEC_E_KDC_UNABLE_TO_REFER,
    SEC_E_KDC_UNKNOWN_ETYPE,
    SEC_E_UNSUPPORTED_PREAUTH,
    SEC_E_DELEGATION_REQUIRED,
    SEC_E_BAD_BINDINGS,
    SEC_E_MULTIPLE_ACCOUNTS,
    SEC_E_NO_KERB_KEY,
    SEC_E_CERT_WRONG_USAGE,
    SEC_E_DOWNGRADE_DETECTED,
    SEC_E_SMARTCARD_CERT_REVOKED,
    SEC_E_ISSUING_CA_UNTRUSTED,
    SEC_E_REVOCATION_OFFLINE_C,
    SEC_E_PKINIT_CLIENT_FAILURE,
    SEC_E_SMARTCARD_CERT_EXPIRED,
    SEC_E_NO_S4U_PROT_SUPPORT,
    SEC_E_CROSSREALM_DELEGATION_FAILURE,
    SEC_E_REVOCATION_OFFLINE_KDC,
    SEC_E_ISSUING_CA_UNTRUSTED_KDC,
    SEC_E_KDC_CERT_EXPIRED,
    SEC_E_KDC_CERT_REVOKED,
    SEC_E_INVALID_PARAMETER,
    SEC_E_DELEGATION_POLICY,
    SEC_E_POLICY_NLTM_ONLY,
    SEC_E_NO_CONTEXT,
    SEC_E_PKU2U_CERT_FAILURE,
    SEC_E_MUTUAL_AUTH_FAILED,
    SEC_E_ONLY_HTTPS_ALLOWED,
    SEC_E_APPLICATION_PROTOCOL_MISMATCH,
    SEC_E_INVALID_UPN_NAME,
    SEC_E_EXT_BUFFER_TOO_SMALL,
    SEC_E_INSUFFICIENT_BUFFERS,
    CRYPT_E_MSG_ERROR,
    CRYPT_E_UNKNOWN_ALGO,
    CRYPT_E_OID_FORMAT,
    CRYPT_E_INVALID_MSG_TYPE,
    CRYPT_E_UNEXPECTED_ENCODING,
    CRYPT_E_AUTH_ATTR_MISSING,
    CRYPT_E_HASH_VALUE,
    CRYPT_E_INVALID_INDEX,
    CRYPT_E_ALREADY_DECRYPTED,
    CRYPT_E_NOT_DECRYPTED,
    CRYPT_E_RECIPIENT_NOT_FOUND,
    CRYPT_E_CONTROL_TYPE,
    CRYPT_E_ISSUER_SERIALNUMBER,
    CRYPT_E_SIGNER_NOT_FOUND,
    CRYPT_E_ATTRIBUTES_MISSING,
    CRYPT_E_STREAM_MSG_NOT_READY,
    CRYPT_E_STREAM_INSUFFICIENT_DATA,
    CRYPT_E_BAD_LEN,
    CRYPT_E_BAD_ENCODE,
    CRYPT_E_FILE_ERROR,
    CRYPT_E_NOT_FOUND,
    CRYPT_E_EXISTS,
    CRYPT_E_NO_PROVIDER,
    CRYPT_E_SELF_SIGNED,
    CRYPT_E_DELETED_PREV,
    CRYPT_E_NO_MATCH,
    CRYPT_E_UNEXPECTED_MSG_TYPE,
    CRYPT_E_NO_KEY_PROPERTY,
    CRYPT_E_NO_DECRYPT_CERT,
    CRYPT_E_BAD_MSG,
    CRYPT_E_NO_SIGNER,
    CRYPT_E_PENDING_CLOSE,
    CRYPT_E_REVOKED,
    CRYPT_E_NO_REVOCATION_DLL,
    CRYPT_E_NO_REVOCATION_CHECK,
    CRYPT_E_REVOCATION_OFFLINE,
    CRYPT_E_NOT_IN_REVOCATION_DATABASE,
    CRYPT_E_INVALID_NUMERIC_STRING,
    CRYPT_E_INVALID_PRINTABLE_STRING,
    CRYPT_E_INVALID_IA5_STRING,
    CRYPT_E_INVALID_X500_STRING,
    CRYPT_E_NOT_CHAR_STRING,
    CRYPT_E_FILERESIZED,
    CRYPT_E_SECURITY_SETTINGS,
    CRYPT_E_NO_VERIFY_USAGE_DLL,
    CRYPT_E_NO_VERIFY_USAGE_CHECK,
    CRYPT_E_VERIFY_USAGE_OFFLINE,
    CRYPT_E_NOT_IN_CTL,
    CRYPT_E_NO_TRUSTED_SIGNER,
    CRYPT_E_MISSING_PUBKEY_PARA,
    CRYPT_E_OBJECT_LOCATOR_OBJECT_NOT_FOUND,
    CRYPT_E_OSS_ERROR,
    OSS_MORE_BUF,
    OSS_NEGATIVE_UINTEGER,
    OSS_PDU_RANGE,
    OSS_MORE_INPUT,
    OSS_DATA_ERROR,
    OSS_BAD_ARG,
    OSS_BAD_VERSION,
    OSS_OUT_MEMORY,
    OSS_PDU_MISMATCH,
    OSS_LIMITED,
    OSS_BAD_PTR,
    OSS_BAD_TIME,
    OSS_INDEFINITE_NOT_SUPPORTED,
    OSS_MEM_ERROR,
    OSS_BAD_TABLE,
    OSS_TOO_LONG,
    OSS_CONSTRAINT_VIOLATED,
    OSS_FATAL_ERROR,
    OSS_ACCESS_SERIALIZATION_ERROR,
    OSS_NULL_TBL,
    OSS_NULL_FCN,
    OSS_BAD_ENCRULES,
    OSS_UNAVAIL_ENCRULES,
    OSS_CANT_OPEN_TRACE_WINDOW,
    OSS_UNIMPLEMENTED,
    OSS_OID_DLL_NOT_LINKED,
    OSS_CANT_OPEN_TRACE_FILE,
    OSS_TRACE_FILE_ALREADY_OPEN,
    OSS_TABLE_MISMATCH,
    OSS_TYPE_NOT_SUPPORTED,
    OSS_REAL_DLL_NOT_LINKED,
    OSS_REAL_CODE_NOT_LINKED,
    OSS_OUT_OF_RANGE,
    OSS_COPIER_DLL_NOT_LINKED,
    OSS_CONSTRAINT_DLL_NOT_LINKED,
    OSS_COMPARATOR_DLL_NOT_LINKED,
    OSS_COMPARATOR_CODE_NOT_LINKED,
    OSS_MEM_MGR_DLL_NOT_LINKED,
    OSS_PDV_DLL_NOT_LINKED,
    OSS_PDV_CODE_NOT_LINKED,
    OSS_API_DLL_NOT_LINKED,
    OSS_BERDER_DLL_NOT_LINKED,
    OSS_PER_DLL_NOT_LINKED,
    OSS_OPEN_TYPE_ERROR,
    OSS_MUTEX_NOT_CREATED,
    OSS_CANT_CLOSE_TRACE_FILE,
    CRYPT_E_ASN1_ERROR,
    CRYPT_E_ASN1_INTERNAL,
    CRYPT_E_ASN1_EOD,
    CRYPT_E_ASN1_CORRUPT,
    CRYPT_E_ASN1_LARGE,
    CRYPT_E_ASN1_CONSTRAINT,
    CRYPT_E_ASN1_MEMORY,
    CRYPT_E_ASN1_OVERFLOW,
    CRYPT_E_ASN1_BADPDU,
    CRYPT_E_ASN1_BADARGS,
    CRYPT_E_ASN1_BADREAL,
    CRYPT_E_ASN1_BADTAG,
    CRYPT_E_ASN1_CHOICE,
    CRYPT_E_ASN1_RULE,
    CRYPT_E_ASN1_UTF8,
    CRYPT_E_ASN1_PDU_TYPE,
    CRYPT_E_ASN1_NYI,
    CRYPT_E_ASN1_EXTENDED,
    CRYPT_E_ASN1_NOEOD,
    CERTSRV_E_BAD_REQUESTSUBJECT,
    CERTSRV_E_NO_REQUEST,
    CERTSRV_E_BAD_REQUESTSTATUS,
    CERTSRV_E_PROPERTY_EMPTY,
    CERTSRV_E_INVALID_CA_CERTIFICATE,
    CERTSRV_E_SERVER_SUSPENDED,
    CERTSRV_E_ENCODING_LENGTH,
    CERTSRV_E_ROLECONFLICT,
    CERTSRV_E_RESTRICTEDOFFICER,
    CERTSRV_E_KEY_ARCHIVAL_NOT_CONFIGURED,
    CERTSRV_E_NO_VALID_KRA,
    CERTSRV_E_BAD_REQUEST_KEY_ARCHIVAL,
    CERTSRV_E_NO_CAADMIN_DEFINED,
    CERTSRV_E_BAD_RENEWAL_CERT_ATTRIBUTE,
    CERTSRV_E_NO_DB_SESSIONS,
    CERTSRV_E_ALIGNMENT_FAULT,
    CERTSRV_E_ENROLL_DENIED,
    CERTSRV_E_TEMPLATE_DENIED,
    CERTSRV_E_DOWNLEVEL_DC_SSL_OR_UPGRADE,
    CERTSRV_E_ADMIN_DENIED_REQUEST,
    CERTSRV_E_NO_POLICY_SERVER,
    CERTSRV_E_WEAK_SIGNATURE_OR_KEY,
    CERTSRV_E_KEY_ATTESTATION_NOT_SUPPORTED,
    CERTSRV_E_ENCRYPTION_CERT_REQUIRED,
    CERTSRV_E_UNSUPPORTED_CERT_TYPE,
    CERTSRV_E_NO_CERT_TYPE,
    CERTSRV_E_TEMPLATE_CONFLICT,
    CERTSRV_E_SUBJECT_ALT_NAME_REQUIRED,
    CERTSRV_E_ARCHIVED_KEY_REQUIRED,
    CERTSRV_E_SMIME_REQUIRED,
    CERTSRV_E_BAD_RENEWAL_SUBJECT,
    CERTSRV_E_BAD_TEMPLATE_VERSION,
    CERTSRV_E_TEMPLATE_POLICY_REQUIRED,
    CERTSRV_E_SIGNATURE_POLICY_REQUIRED,
    CERTSRV_E_SIGNATURE_COUNT,
    CERTSRV_E_SIGNATURE_REJECTED,
    CERTSRV_E_ISSUANCE_POLICY_REQUIRED,
    CERTSRV_E_SUBJECT_UPN_REQUIRED,
    CERTSRV_E_SUBJECT_DIRECTORY_GUID_REQUIRED,
    CERTSRV_E_SUBJECT_DNS_REQUIRED,
    CERTSRV_E_ARCHIVED_KEY_UNEXPECTED,
    CERTSRV_E_KEY_LENGTH,
    CERTSRV_E_SUBJECT_EMAIL_REQUIRED,
    CERTSRV_E_UNKNOWN_CERT_TYPE,
    CERTSRV_E_CERT_TYPE_OVERLAP,
    CERTSRV_E_TOO_MANY_SIGNATURES,
    CERTSRV_E_RENEWAL_BAD_PUBLIC_KEY,
    CERTSRV_E_INVALID_EK,
    CERTSRV_E_INVALID_IDBINDING,
    CERTSRV_E_INVALID_ATTESTATION,
    CERTSRV_E_KEY_ATTESTATION,
    CERTSRV_E_CORRUPT_KEY_ATTESTATION,
    CERTSRV_E_EXPIRED_CHALLENGE,
    CERTSRV_E_INVALID_RESPONSE,
    CERTSRV_E_INVALID_REQUESTID,
    CERTSRV_E_REQUEST_PRECERTIFICATE_MISMATCH,
    CERTSRV_E_PENDING_CLIENT_RESPONSE,
    CERTSRV_E_SEC_EXT_DIRECTORY_SID_REQUIRED,
    XENROLL_E_KEY_NOT_EXPORTABLE,
    XENROLL_E_CANNOT_ADD_ROOT_CERT,
    XENROLL_E_RESPONSE_KA_HASH_NOT_FOUND,
    XENROLL_E_RESPONSE_UNEXPECTED_KA_HASH,
    XENROLL_E_RESPONSE_KA_HASH_MISMATCH,
    XENROLL_E_KEYSPEC_SMIME_MISMATCH,
    TRUST_E_SYSTEM_ERROR,
    TRUST_E_NO_SIGNER_CERT,
    TRUST_E_COUNTER_SIGNER,
    TRUST_E_CERT_SIGNATURE,
    TRUST_E_TIME_STAMP,
    TRUST_E_BAD_DIGEST,
    TRUST_E_MALFORMED_SIGNATURE,
    TRUST_E_BASIC_CONSTRAINTS,
    TRUST_E_FINANCIAL_CRITERIA,
    MSSIPOTF_E_OUTOFMEMRANGE,
    MSSIPOTF_E_CANTGETOBJECT,
    MSSIPOTF_E_NOHEADTABLE,
    MSSIPOTF_E_BAD_MAGICNUMBER,
    MSSIPOTF_E_BAD_OFFSET_TABLE,
    MSSIPOTF_E_TABLE_TAGORDER,
    MSSIPOTF_E_TABLE_LONGWORD,
    MSSIPOTF_E_BAD_FIRST_TABLE_PLACEMENT,
    MSSIPOTF_E_TABLES_OVERLAP,
    MSSIPOTF_E_TABLE_PADBYTES,
    MSSIPOTF_E_FILETOOSMALL,
    MSSIPOTF_E_TABLE_CHECKSUM,
    MSSIPOTF_E_FILE_CHECKSUM,
    MSSIPOTF_E_FAILED_POLICY,
    MSSIPOTF_E_FAILED_HINTS_CHECK,
    MSSIPOTF_E_NOT_OPENTYPE,
    MSSIPOTF_E_FILE,
    MSSIPOTF_E_CRYPT,
    MSSIPOTF_E_BADVERSION,
    MSSIPOTF_E_DSIG_STRUCTURE,
    MSSIPOTF_E_PCONST_CHECK,
    MSSIPOTF_E_STRUCTURE,
    ERROR_CRED_REQUIRES_CONFIRMATION,
    ERROR_AUDITING_DISABLED,
    ERROR_ALL_SIDS_FILTERED,
    ERROR_BIZRULES_NOT_ENABLED,
}

impl SECURITY_SSPI {
    pub fn code(&self) -> u32 {
        return match self {
            SECURITY_SSPI::SEC_I_CONTINUE_NEEDED => 0x00090312 as u32,
            SECURITY_SSPI::SEC_I_COMPLETE_NEEDED => 0x00090313 as u32,
            SECURITY_SSPI::SEC_I_COMPLETE_AND_CONTINUE => 0x00090314 as u32,
            SECURITY_SSPI::SEC_I_LOCAL_LOGON => 0x00090315 as u32,
            SECURITY_SSPI::SEC_I_GENERIC_EXTENSION_RECEIVED => 0x00090316 as u32,
            SECURITY_SSPI::SEC_I_CONTEXT_EXPIRED => 0x00090317 as u32,
            SECURITY_SSPI::SEC_I_INCOMPLETE_CREDENTIALS => 0x00090320 as u32,
            SECURITY_SSPI::SEC_I_RENEGOTIATE => 0x00090321 as u32,
            SECURITY_SSPI::SEC_I_NO_LSA_CONTEXT => 0x00090323 as u32,
            SECURITY_SSPI::SEC_I_SIGNATURE_NEEDED => 0x0009035C as u32,
            SECURITY_SSPI::SEC_I_NO_RENEGOTIATION => 0x00090360 as u32,
            SECURITY_SSPI::SEC_I_MESSAGE_FRAGMENT => 0x00090364 as u32,
            SECURITY_SSPI::SEC_I_CONTINUE_NEEDED_MESSAGE_OK => 0x00090366 as u32,
            SECURITY_SSPI::SEC_I_ASYNC_CALL_PENDING => 0x00090368 as u32,
            SECURITY_SSPI::CRYPT_I_NEW_PROTECTION_REQUIRED => 0x00091012 as u32,
            SECURITY_SSPI::NTE_BAD_UID => 0x80090001 as u32,
            SECURITY_SSPI::NTE_BAD_HASH => 0x80090002 as u32,
            SECURITY_SSPI::NTE_BAD_KEY => 0x80090003 as u32,
            SECURITY_SSPI::NTE_BAD_LEN => 0x80090004 as u32,
            SECURITY_SSPI::NTE_BAD_DATA => 0x80090005 as u32,
            SECURITY_SSPI::NTE_BAD_SIGNATURE => 0x80090006 as u32,
            SECURITY_SSPI::NTE_BAD_VER => 0x80090007 as u32,
            SECURITY_SSPI::NTE_BAD_ALGID => 0x80090008 as u32,
            SECURITY_SSPI::NTE_BAD_FLAGS => 0x80090009 as u32,
            SECURITY_SSPI::NTE_BAD_TYPE => 0x8009000A as u32,
            SECURITY_SSPI::NTE_BAD_KEY_STATE => 0x8009000B as u32,
            SECURITY_SSPI::NTE_BAD_HASH_STATE => 0x8009000C as u32,
            SECURITY_SSPI::NTE_NO_KEY => 0x8009000D as u32,
            SECURITY_SSPI::NTE_NO_MEMORY => 0x8009000E as u32,
            SECURITY_SSPI::NTE_EXISTS => 0x8009000F as u32,
            SECURITY_SSPI::NTE_PERM => 0x80090010 as u32,
            SECURITY_SSPI::NTE_NOT_FOUND => 0x80090011 as u32,
            SECURITY_SSPI::NTE_DOUBLE_ENCRYPT => 0x80090012 as u32,
            SECURITY_SSPI::NTE_BAD_PROVIDER => 0x80090013 as u32,
            SECURITY_SSPI::NTE_BAD_PROV_TYPE => 0x80090014 as u32,
            SECURITY_SSPI::NTE_BAD_PUBLIC_KEY => 0x80090015 as u32,
            SECURITY_SSPI::NTE_BAD_KEYSET => 0x80090016 as u32,
            SECURITY_SSPI::NTE_PROV_TYPE_NOT_DEF => 0x80090017 as u32,
            SECURITY_SSPI::NTE_PROV_TYPE_ENTRY_BAD => 0x80090018 as u32,
            SECURITY_SSPI::NTE_KEYSET_NOT_DEF => 0x80090019 as u32,
            SECURITY_SSPI::NTE_KEYSET_ENTRY_BAD => 0x8009001A as u32,
            SECURITY_SSPI::NTE_PROV_TYPE_NO_MATCH => 0x8009001B as u32,
            SECURITY_SSPI::NTE_SIGNATURE_FILE_BAD => 0x8009001C as u32,
            SECURITY_SSPI::NTE_PROVIDER_DLL_FAIL => 0x8009001D as u32,
            SECURITY_SSPI::NTE_PROV_DLL_NOT_FOUND => 0x8009001E as u32,
            SECURITY_SSPI::NTE_BAD_KEYSET_PARAM => 0x8009001F as u32,
            SECURITY_SSPI::NTE_FAIL => 0x80090020 as u32,
            SECURITY_SSPI::NTE_SYS_ERR => 0x80090021 as u32,
            SECURITY_SSPI::NTE_SILENT_CONTEXT => 0x80090022 as u32,
            SECURITY_SSPI::NTE_TOKEN_KEYSET_STORAGE_FULL => 0x80090023 as u32,
            SECURITY_SSPI::NTE_TEMPORARY_PROFILE => 0x80090024 as u32,
            SECURITY_SSPI::NTE_FIXEDPARAMETER => 0x80090025 as u32,
            SECURITY_SSPI::NTE_INVALID_HANDLE => 0x80090026 as u32,
            SECURITY_SSPI::NTE_INVALID_PARAMETER => 0x80090027 as u32,
            SECURITY_SSPI::NTE_BUFFER_TOO_SMALL => 0x80090028 as u32,
            SECURITY_SSPI::NTE_NOT_SUPPORTED => 0x80090029 as u32,
            SECURITY_SSPI::NTE_NO_MORE_ITEMS => 0x8009002A as u32,
            SECURITY_SSPI::NTE_BUFFERS_OVERLAP => 0x8009002B as u32,
            SECURITY_SSPI::NTE_DECRYPTION_FAILURE => 0x8009002C as u32,
            SECURITY_SSPI::NTE_INTERNAL_ERROR => 0x8009002D as u32,
            SECURITY_SSPI::NTE_UI_REQUIRED => 0x8009002E as u32,
            SECURITY_SSPI::NTE_HMAC_NOT_SUPPORTED => 0x8009002F as u32,
            SECURITY_SSPI::NTE_DEVICE_NOT_READY => 0x80090030 as u32,
            SECURITY_SSPI::NTE_AUTHENTICATION_IGNORED => 0x80090031 as u32,
            SECURITY_SSPI::NTE_VALIDATION_FAILED => 0x80090032 as u32,
            SECURITY_SSPI::NTE_INCORRECT_PASSWORD => 0x80090033 as u32,
            SECURITY_SSPI::NTE_ENCRYPTION_FAILURE => 0x80090034 as u32,
            SECURITY_SSPI::NTE_DEVICE_NOT_FOUND => 0x80090035 as u32,
            SECURITY_SSPI::NTE_USER_CANCELLED => 0x80090036 as u32,
            SECURITY_SSPI::NTE_PASSWORD_CHANGE_REQUIRED => 0x80090037 as u32,
            SECURITY_SSPI::NTE_NOT_ACTIVE_CONSOLE => 0x80090038 as u32,
            SECURITY_SSPI::SEC_E_INSUFFICIENT_MEMORY => 0x80090300 as u32,
            SECURITY_SSPI::SEC_E_INVALID_HANDLE => 0x80090301 as u32,
            SECURITY_SSPI::SEC_E_UNSUPPORTED_FUNCTION => 0x80090302 as u32,
            SECURITY_SSPI::SEC_E_TARGET_UNKNOWN => 0x80090303 as u32,
            SECURITY_SSPI::SEC_E_INTERNAL_ERROR => 0x80090304 as u32,
            SECURITY_SSPI::SEC_E_SECPKG_NOT_FOUND => 0x80090305 as u32,
            SECURITY_SSPI::SEC_E_NOT_OWNER => 0x80090306 as u32,
            SECURITY_SSPI::SEC_E_CANNOT_INSTALL => 0x80090307 as u32,
            SECURITY_SSPI::SEC_E_INVALID_TOKEN => 0x80090308 as u32,
            SECURITY_SSPI::SEC_E_CANNOT_PACK => 0x80090309 as u32,
            SECURITY_SSPI::SEC_E_QOP_NOT_SUPPORTED => 0x8009030A as u32,
            SECURITY_SSPI::SEC_E_NO_IMPERSONATION => 0x8009030B as u32,
            SECURITY_SSPI::SEC_E_LOGON_DENIED => 0x8009030C as u32,
            SECURITY_SSPI::SEC_E_UNKNOWN_CREDENTIALS => 0x8009030D as u32,
            SECURITY_SSPI::SEC_E_NO_CREDENTIALS => 0x8009030E as u32,
            SECURITY_SSPI::SEC_E_MESSAGE_ALTERED => 0x8009030F as u32,
            SECURITY_SSPI::SEC_E_OUT_OF_SEQUENCE => 0x80090310 as u32,
            SECURITY_SSPI::SEC_E_NO_AUTHENTICATING_AUTHORITY => 0x80090311 as u32,
            SECURITY_SSPI::SEC_E_BAD_PKGID => 0x80090316 as u32,
            SECURITY_SSPI::SEC_E_CONTEXT_EXPIRED => 0x80090317 as u32,
            SECURITY_SSPI::SEC_E_INCOMPLETE_MESSAGE => 0x80090318 as u32,
            SECURITY_SSPI::SEC_E_INCOMPLETE_CREDENTIALS => 0x80090320 as u32,
            SECURITY_SSPI::SEC_E_BUFFER_TOO_SMALL => 0x80090321 as u32,
            SECURITY_SSPI::SEC_E_WRONG_PRINCIPAL => 0x80090322 as u32,
            SECURITY_SSPI::SEC_E_TIME_SKEW => 0x80090324 as u32,
            SECURITY_SSPI::SEC_E_UNTRUSTED_ROOT => 0x80090325 as u32,
            SECURITY_SSPI::SEC_E_ILLEGAL_MESSAGE => 0x80090326 as u32,
            SECURITY_SSPI::SEC_E_CERT_UNKNOWN => 0x80090327 as u32,
            SECURITY_SSPI::SEC_E_CERT_EXPIRED => 0x80090328 as u32,
            SECURITY_SSPI::SEC_E_ENCRYPT_FAILURE => 0x80090329 as u32,
            SECURITY_SSPI::SEC_E_DECRYPT_FAILURE => 0x80090330 as u32,
            SECURITY_SSPI::SEC_E_ALGORITHM_MISMATCH => 0x80090331 as u32,
            SECURITY_SSPI::SEC_E_SECURITY_QOS_FAILED => 0x80090332 as u32,
            SECURITY_SSPI::SEC_E_UNFINISHED_CONTEXT_DELETED => 0x80090333 as u32,
            SECURITY_SSPI::SEC_E_NO_TGT_REPLY => 0x80090334 as u32,
            SECURITY_SSPI::SEC_E_NO_IP_ADDRESSES => 0x80090335 as u32,
            SECURITY_SSPI::SEC_E_WRONG_CREDENTIAL_HANDLE => 0x80090336 as u32,
            SECURITY_SSPI::SEC_E_CRYPTO_SYSTEM_INVALID => 0x80090337 as u32,
            SECURITY_SSPI::SEC_E_MAX_REFERRALS_EXCEEDED => 0x80090338 as u32,
            SECURITY_SSPI::SEC_E_MUST_BE_KDC => 0x80090339 as u32,
            SECURITY_SSPI::SEC_E_STRONG_CRYPTO_NOT_SUPPORTED => 0x8009033A as u32,
            SECURITY_SSPI::SEC_E_TOO_MANY_PRINCIPALS => 0x8009033B as u32,
            SECURITY_SSPI::SEC_E_NO_PA_DATA => 0x8009033C as u32,
            SECURITY_SSPI::SEC_E_PKINIT_NAME_MISMATCH => 0x8009033D as u32,
            SECURITY_SSPI::SEC_E_SMARTCARD_LOGON_REQUIRED => 0x8009033E as u32,
            SECURITY_SSPI::SEC_E_SHUTDOWN_IN_PROGRESS => 0x8009033F as u32,
            SECURITY_SSPI::SEC_E_KDC_INVALID_REQUEST => 0x80090340 as u32,
            SECURITY_SSPI::SEC_E_KDC_UNABLE_TO_REFER => 0x80090341 as u32,
            SECURITY_SSPI::SEC_E_KDC_UNKNOWN_ETYPE => 0x80090342 as u32,
            SECURITY_SSPI::SEC_E_UNSUPPORTED_PREAUTH => 0x80090343 as u32,
            SECURITY_SSPI::SEC_E_DELEGATION_REQUIRED => 0x80090345 as u32,
            SECURITY_SSPI::SEC_E_BAD_BINDINGS => 0x80090346 as u32,
            SECURITY_SSPI::SEC_E_MULTIPLE_ACCOUNTS => 0x80090347 as u32,
            SECURITY_SSPI::SEC_E_NO_KERB_KEY => 0x80090348 as u32,
            SECURITY_SSPI::SEC_E_CERT_WRONG_USAGE => 0x80090349 as u32,
            SECURITY_SSPI::SEC_E_DOWNGRADE_DETECTED => 0x80090350 as u32,
            SECURITY_SSPI::SEC_E_SMARTCARD_CERT_REVOKED => 0x80090351 as u32,
            SECURITY_SSPI::SEC_E_ISSUING_CA_UNTRUSTED => 0x80090352 as u32,
            SECURITY_SSPI::SEC_E_REVOCATION_OFFLINE_C => 0x80090353 as u32,
            SECURITY_SSPI::SEC_E_PKINIT_CLIENT_FAILURE => 0x80090354 as u32,
            SECURITY_SSPI::SEC_E_SMARTCARD_CERT_EXPIRED => 0x80090355 as u32,
            SECURITY_SSPI::SEC_E_NO_S4U_PROT_SUPPORT => 0x80090356 as u32,
            SECURITY_SSPI::SEC_E_CROSSREALM_DELEGATION_FAILURE => 0x80090357 as u32,
            SECURITY_SSPI::SEC_E_REVOCATION_OFFLINE_KDC => 0x80090358 as u32,
            SECURITY_SSPI::SEC_E_ISSUING_CA_UNTRUSTED_KDC => 0x80090359 as u32,
            SECURITY_SSPI::SEC_E_KDC_CERT_EXPIRED => 0x8009035A as u32,
            SECURITY_SSPI::SEC_E_KDC_CERT_REVOKED => 0x8009035B as u32,
            SECURITY_SSPI::SEC_E_INVALID_PARAMETER => 0x8009035D as u32,
            SECURITY_SSPI::SEC_E_DELEGATION_POLICY => 0x8009035E as u32,
            SECURITY_SSPI::SEC_E_POLICY_NLTM_ONLY => 0x8009035F as u32,
            SECURITY_SSPI::SEC_E_NO_CONTEXT => 0x80090361 as u32,
            SECURITY_SSPI::SEC_E_PKU2U_CERT_FAILURE => 0x80090362 as u32,
            SECURITY_SSPI::SEC_E_MUTUAL_AUTH_FAILED => 0x80090363 as u32,
            SECURITY_SSPI::SEC_E_ONLY_HTTPS_ALLOWED => 0x80090365 as u32,
            SECURITY_SSPI::SEC_E_APPLICATION_PROTOCOL_MISMATCH => 0x80090367 as u32,
            SECURITY_SSPI::SEC_E_INVALID_UPN_NAME => 0x80090369 as u32,
            SECURITY_SSPI::SEC_E_EXT_BUFFER_TOO_SMALL => 0x8009036A as u32,
            SECURITY_SSPI::SEC_E_INSUFFICIENT_BUFFERS => 0x8009036B as u32,
            SECURITY_SSPI::CRYPT_E_MSG_ERROR => 0x80091001 as u32,
            SECURITY_SSPI::CRYPT_E_UNKNOWN_ALGO => 0x80091002 as u32,
            SECURITY_SSPI::CRYPT_E_OID_FORMAT => 0x80091003 as u32,
            SECURITY_SSPI::CRYPT_E_INVALID_MSG_TYPE => 0x80091004 as u32,
            SECURITY_SSPI::CRYPT_E_UNEXPECTED_ENCODING => 0x80091005 as u32,
            SECURITY_SSPI::CRYPT_E_AUTH_ATTR_MISSING => 0x80091006 as u32,
            SECURITY_SSPI::CRYPT_E_HASH_VALUE => 0x80091007 as u32,
            SECURITY_SSPI::CRYPT_E_INVALID_INDEX => 0x80091008 as u32,
            SECURITY_SSPI::CRYPT_E_ALREADY_DECRYPTED => 0x80091009 as u32,
            SECURITY_SSPI::CRYPT_E_NOT_DECRYPTED => 0x8009100A as u32,
            SECURITY_SSPI::CRYPT_E_RECIPIENT_NOT_FOUND => 0x8009100B as u32,
            SECURITY_SSPI::CRYPT_E_CONTROL_TYPE => 0x8009100C as u32,
            SECURITY_SSPI::CRYPT_E_ISSUER_SERIALNUMBER => 0x8009100D as u32,
            SECURITY_SSPI::CRYPT_E_SIGNER_NOT_FOUND => 0x8009100E as u32,
            SECURITY_SSPI::CRYPT_E_ATTRIBUTES_MISSING => 0x8009100F as u32,
            SECURITY_SSPI::CRYPT_E_STREAM_MSG_NOT_READY => 0x80091010 as u32,
            SECURITY_SSPI::CRYPT_E_STREAM_INSUFFICIENT_DATA => 0x80091011 as u32,
            SECURITY_SSPI::CRYPT_E_BAD_LEN => 0x80092001 as u32,
            SECURITY_SSPI::CRYPT_E_BAD_ENCODE => 0x80092002 as u32,
            SECURITY_SSPI::CRYPT_E_FILE_ERROR => 0x80092003 as u32,
            SECURITY_SSPI::CRYPT_E_NOT_FOUND => 0x80092004 as u32,
            SECURITY_SSPI::CRYPT_E_EXISTS => 0x80092005 as u32,
            SECURITY_SSPI::CRYPT_E_NO_PROVIDER => 0x80092006 as u32,
            SECURITY_SSPI::CRYPT_E_SELF_SIGNED => 0x80092007 as u32,
            SECURITY_SSPI::CRYPT_E_DELETED_PREV => 0x80092008 as u32,
            SECURITY_SSPI::CRYPT_E_NO_MATCH => 0x80092009 as u32,
            SECURITY_SSPI::CRYPT_E_UNEXPECTED_MSG_TYPE => 0x8009200A as u32,
            SECURITY_SSPI::CRYPT_E_NO_KEY_PROPERTY => 0x8009200B as u32,
            SECURITY_SSPI::CRYPT_E_NO_DECRYPT_CERT => 0x8009200C as u32,
            SECURITY_SSPI::CRYPT_E_BAD_MSG => 0x8009200D as u32,
            SECURITY_SSPI::CRYPT_E_NO_SIGNER => 0x8009200E as u32,
            SECURITY_SSPI::CRYPT_E_PENDING_CLOSE => 0x8009200F as u32,
            SECURITY_SSPI::CRYPT_E_REVOKED => 0x80092010 as u32,
            SECURITY_SSPI::CRYPT_E_NO_REVOCATION_DLL => 0x80092011 as u32,
            SECURITY_SSPI::CRYPT_E_NO_REVOCATION_CHECK => 0x80092012 as u32,
            SECURITY_SSPI::CRYPT_E_REVOCATION_OFFLINE => 0x80092013 as u32,
            SECURITY_SSPI::CRYPT_E_NOT_IN_REVOCATION_DATABASE => 0x80092014 as u32,
            SECURITY_SSPI::CRYPT_E_INVALID_NUMERIC_STRING => 0x80092020 as u32,
            SECURITY_SSPI::CRYPT_E_INVALID_PRINTABLE_STRING => 0x80092021 as u32,
            SECURITY_SSPI::CRYPT_E_INVALID_IA5_STRING => 0x80092022 as u32,
            SECURITY_SSPI::CRYPT_E_INVALID_X500_STRING => 0x80092023 as u32,
            SECURITY_SSPI::CRYPT_E_NOT_CHAR_STRING => 0x80092024 as u32,
            SECURITY_SSPI::CRYPT_E_FILERESIZED => 0x80092025 as u32,
            SECURITY_SSPI::CRYPT_E_SECURITY_SETTINGS => 0x80092026 as u32,
            SECURITY_SSPI::CRYPT_E_NO_VERIFY_USAGE_DLL => 0x80092027 as u32,
            SECURITY_SSPI::CRYPT_E_NO_VERIFY_USAGE_CHECK => 0x80092028 as u32,
            SECURITY_SSPI::CRYPT_E_VERIFY_USAGE_OFFLINE => 0x80092029 as u32,
            SECURITY_SSPI::CRYPT_E_NOT_IN_CTL => 0x8009202A as u32,
            SECURITY_SSPI::CRYPT_E_NO_TRUSTED_SIGNER => 0x8009202B as u32,
            SECURITY_SSPI::CRYPT_E_MISSING_PUBKEY_PARA => 0x8009202C as u32,
            SECURITY_SSPI::CRYPT_E_OBJECT_LOCATOR_OBJECT_NOT_FOUND => 0x8009202D as u32,
            SECURITY_SSPI::CRYPT_E_OSS_ERROR => 0x80093000 as u32,
            SECURITY_SSPI::OSS_MORE_BUF => 0x80093001 as u32,
            SECURITY_SSPI::OSS_NEGATIVE_UINTEGER => 0x80093002 as u32,
            SECURITY_SSPI::OSS_PDU_RANGE => 0x80093003 as u32,
            SECURITY_SSPI::OSS_MORE_INPUT => 0x80093004 as u32,
            SECURITY_SSPI::OSS_DATA_ERROR => 0x80093005 as u32,
            SECURITY_SSPI::OSS_BAD_ARG => 0x80093006 as u32,
            SECURITY_SSPI::OSS_BAD_VERSION => 0x80093007 as u32,
            SECURITY_SSPI::OSS_OUT_MEMORY => 0x80093008 as u32,
            SECURITY_SSPI::OSS_PDU_MISMATCH => 0x80093009 as u32,
            SECURITY_SSPI::OSS_LIMITED => 0x8009300A as u32,
            SECURITY_SSPI::OSS_BAD_PTR => 0x8009300B as u32,
            SECURITY_SSPI::OSS_BAD_TIME => 0x8009300C as u32,
            SECURITY_SSPI::OSS_INDEFINITE_NOT_SUPPORTED => 0x8009300D as u32,
            SECURITY_SSPI::OSS_MEM_ERROR => 0x8009300E as u32,
            SECURITY_SSPI::OSS_BAD_TABLE => 0x8009300F as u32,
            SECURITY_SSPI::OSS_TOO_LONG => 0x80093010 as u32,
            SECURITY_SSPI::OSS_CONSTRAINT_VIOLATED => 0x80093011 as u32,
            SECURITY_SSPI::OSS_FATAL_ERROR => 0x80093012 as u32,
            SECURITY_SSPI::OSS_ACCESS_SERIALIZATION_ERROR => 0x80093013 as u32,
            SECURITY_SSPI::OSS_NULL_TBL => 0x80093014 as u32,
            SECURITY_SSPI::OSS_NULL_FCN => 0x80093015 as u32,
            SECURITY_SSPI::OSS_BAD_ENCRULES => 0x80093016 as u32,
            SECURITY_SSPI::OSS_UNAVAIL_ENCRULES => 0x80093017 as u32,
            SECURITY_SSPI::OSS_CANT_OPEN_TRACE_WINDOW => 0x80093018 as u32,
            SECURITY_SSPI::OSS_UNIMPLEMENTED => 0x80093019 as u32,
            SECURITY_SSPI::OSS_OID_DLL_NOT_LINKED => 0x8009301A as u32,
            SECURITY_SSPI::OSS_CANT_OPEN_TRACE_FILE => 0x8009301B as u32,
            SECURITY_SSPI::OSS_TRACE_FILE_ALREADY_OPEN => 0x8009301C as u32,
            SECURITY_SSPI::OSS_TABLE_MISMATCH => 0x8009301D as u32,
            SECURITY_SSPI::OSS_TYPE_NOT_SUPPORTED => 0x8009301E as u32,
            SECURITY_SSPI::OSS_REAL_DLL_NOT_LINKED => 0x8009301F as u32,
            SECURITY_SSPI::OSS_REAL_CODE_NOT_LINKED => 0x80093020 as u32,
            SECURITY_SSPI::OSS_OUT_OF_RANGE => 0x80093021 as u32,
            SECURITY_SSPI::OSS_COPIER_DLL_NOT_LINKED => 0x80093022 as u32,
            SECURITY_SSPI::OSS_CONSTRAINT_DLL_NOT_LINKED => 0x80093023 as u32,
            SECURITY_SSPI::OSS_COMPARATOR_DLL_NOT_LINKED => 0x80093024 as u32,
            SECURITY_SSPI::OSS_COMPARATOR_CODE_NOT_LINKED => 0x80093025 as u32,
            SECURITY_SSPI::OSS_MEM_MGR_DLL_NOT_LINKED => 0x80093026 as u32,
            SECURITY_SSPI::OSS_PDV_DLL_NOT_LINKED => 0x80093027 as u32,
            SECURITY_SSPI::OSS_PDV_CODE_NOT_LINKED => 0x80093028 as u32,
            SECURITY_SSPI::OSS_API_DLL_NOT_LINKED => 0x80093029 as u32,
            SECURITY_SSPI::OSS_BERDER_DLL_NOT_LINKED => 0x8009302A as u32,
            SECURITY_SSPI::OSS_PER_DLL_NOT_LINKED => 0x8009302B as u32,
            SECURITY_SSPI::OSS_OPEN_TYPE_ERROR => 0x8009302C as u32,
            SECURITY_SSPI::OSS_MUTEX_NOT_CREATED => 0x8009302D as u32,
            SECURITY_SSPI::OSS_CANT_CLOSE_TRACE_FILE => 0x8009302E as u32,
            SECURITY_SSPI::CRYPT_E_ASN1_ERROR => 0x80093100 as u32,
            SECURITY_SSPI::CRYPT_E_ASN1_INTERNAL => 0x80093101 as u32,
            SECURITY_SSPI::CRYPT_E_ASN1_EOD => 0x80093102 as u32,
            SECURITY_SSPI::CRYPT_E_ASN1_CORRUPT => 0x80093103 as u32,
            SECURITY_SSPI::CRYPT_E_ASN1_LARGE => 0x80093104 as u32,
            SECURITY_SSPI::CRYPT_E_ASN1_CONSTRAINT => 0x80093105 as u32,
            SECURITY_SSPI::CRYPT_E_ASN1_MEMORY => 0x80093106 as u32,
            SECURITY_SSPI::CRYPT_E_ASN1_OVERFLOW => 0x80093107 as u32,
            SECURITY_SSPI::CRYPT_E_ASN1_BADPDU => 0x80093108 as u32,
            SECURITY_SSPI::CRYPT_E_ASN1_BADARGS => 0x80093109 as u32,
            SECURITY_SSPI::CRYPT_E_ASN1_BADREAL => 0x8009310A as u32,
            SECURITY_SSPI::CRYPT_E_ASN1_BADTAG => 0x8009310B as u32,
            SECURITY_SSPI::CRYPT_E_ASN1_CHOICE => 0x8009310C as u32,
            SECURITY_SSPI::CRYPT_E_ASN1_RULE => 0x8009310D as u32,
            SECURITY_SSPI::CRYPT_E_ASN1_UTF8 => 0x8009310E as u32,
            SECURITY_SSPI::CRYPT_E_ASN1_PDU_TYPE => 0x80093133 as u32,
            SECURITY_SSPI::CRYPT_E_ASN1_NYI => 0x80093134 as u32,
            SECURITY_SSPI::CRYPT_E_ASN1_EXTENDED => 0x80093201 as u32,
            SECURITY_SSPI::CRYPT_E_ASN1_NOEOD => 0x80093202 as u32,
            SECURITY_SSPI::CERTSRV_E_BAD_REQUESTSUBJECT => 0x80094001 as u32,
            SECURITY_SSPI::CERTSRV_E_NO_REQUEST => 0x80094002 as u32,
            SECURITY_SSPI::CERTSRV_E_BAD_REQUESTSTATUS => 0x80094003 as u32,
            SECURITY_SSPI::CERTSRV_E_PROPERTY_EMPTY => 0x80094004 as u32,
            SECURITY_SSPI::CERTSRV_E_INVALID_CA_CERTIFICATE => 0x80094005 as u32,
            SECURITY_SSPI::CERTSRV_E_SERVER_SUSPENDED => 0x80094006 as u32,
            SECURITY_SSPI::CERTSRV_E_ENCODING_LENGTH => 0x80094007 as u32,
            SECURITY_SSPI::CERTSRV_E_ROLECONFLICT => 0x80094008 as u32,
            SECURITY_SSPI::CERTSRV_E_RESTRICTEDOFFICER => 0x80094009 as u32,
            SECURITY_SSPI::CERTSRV_E_KEY_ARCHIVAL_NOT_CONFIGURED => 0x8009400A as u32,
            SECURITY_SSPI::CERTSRV_E_NO_VALID_KRA => 0x8009400B as u32,
            SECURITY_SSPI::CERTSRV_E_BAD_REQUEST_KEY_ARCHIVAL => 0x8009400C as u32,
            SECURITY_SSPI::CERTSRV_E_NO_CAADMIN_DEFINED => 0x8009400D as u32,
            SECURITY_SSPI::CERTSRV_E_BAD_RENEWAL_CERT_ATTRIBUTE => 0x8009400E as u32,
            SECURITY_SSPI::CERTSRV_E_NO_DB_SESSIONS => 0x8009400F as u32,
            SECURITY_SSPI::CERTSRV_E_ALIGNMENT_FAULT => 0x80094010 as u32,
            SECURITY_SSPI::CERTSRV_E_ENROLL_DENIED => 0x80094011 as u32,
            SECURITY_SSPI::CERTSRV_E_TEMPLATE_DENIED => 0x80094012 as u32,
            SECURITY_SSPI::CERTSRV_E_DOWNLEVEL_DC_SSL_OR_UPGRADE => 0x80094013 as u32,
            SECURITY_SSPI::CERTSRV_E_ADMIN_DENIED_REQUEST => 0x80094014 as u32,
            SECURITY_SSPI::CERTSRV_E_NO_POLICY_SERVER => 0x80094015 as u32,
            SECURITY_SSPI::CERTSRV_E_WEAK_SIGNATURE_OR_KEY => 0x80094016 as u32,
            SECURITY_SSPI::CERTSRV_E_KEY_ATTESTATION_NOT_SUPPORTED => 0x80094017 as u32,
            SECURITY_SSPI::CERTSRV_E_ENCRYPTION_CERT_REQUIRED => 0x80094018 as u32,
            SECURITY_SSPI::CERTSRV_E_UNSUPPORTED_CERT_TYPE => 0x80094800 as u32,
            SECURITY_SSPI::CERTSRV_E_NO_CERT_TYPE => 0x80094801 as u32,
            SECURITY_SSPI::CERTSRV_E_TEMPLATE_CONFLICT => 0x80094802 as u32,
            SECURITY_SSPI::CERTSRV_E_SUBJECT_ALT_NAME_REQUIRED => 0x80094803 as u32,
            SECURITY_SSPI::CERTSRV_E_ARCHIVED_KEY_REQUIRED => 0x80094804 as u32,
            SECURITY_SSPI::CERTSRV_E_SMIME_REQUIRED => 0x80094805 as u32,
            SECURITY_SSPI::CERTSRV_E_BAD_RENEWAL_SUBJECT => 0x80094806 as u32,
            SECURITY_SSPI::CERTSRV_E_BAD_TEMPLATE_VERSION => 0x80094807 as u32,
            SECURITY_SSPI::CERTSRV_E_TEMPLATE_POLICY_REQUIRED => 0x80094808 as u32,
            SECURITY_SSPI::CERTSRV_E_SIGNATURE_POLICY_REQUIRED => 0x80094809 as u32,
            SECURITY_SSPI::CERTSRV_E_SIGNATURE_COUNT => 0x8009480A as u32,
            SECURITY_SSPI::CERTSRV_E_SIGNATURE_REJECTED => 0x8009480B as u32,
            SECURITY_SSPI::CERTSRV_E_ISSUANCE_POLICY_REQUIRED => 0x8009480C as u32,
            SECURITY_SSPI::CERTSRV_E_SUBJECT_UPN_REQUIRED => 0x8009480D as u32,
            SECURITY_SSPI::CERTSRV_E_SUBJECT_DIRECTORY_GUID_REQUIRED => 0x8009480E as u32,
            SECURITY_SSPI::CERTSRV_E_SUBJECT_DNS_REQUIRED => 0x8009480F as u32,
            SECURITY_SSPI::CERTSRV_E_ARCHIVED_KEY_UNEXPECTED => 0x80094810 as u32,
            SECURITY_SSPI::CERTSRV_E_KEY_LENGTH => 0x80094811 as u32,
            SECURITY_SSPI::CERTSRV_E_SUBJECT_EMAIL_REQUIRED => 0x80094812 as u32,
            SECURITY_SSPI::CERTSRV_E_UNKNOWN_CERT_TYPE => 0x80094813 as u32,
            SECURITY_SSPI::CERTSRV_E_CERT_TYPE_OVERLAP => 0x80094814 as u32,
            SECURITY_SSPI::CERTSRV_E_TOO_MANY_SIGNATURES => 0x80094815 as u32,
            SECURITY_SSPI::CERTSRV_E_RENEWAL_BAD_PUBLIC_KEY => 0x80094816 as u32,
            SECURITY_SSPI::CERTSRV_E_INVALID_EK => 0x80094817 as u32,
            SECURITY_SSPI::CERTSRV_E_INVALID_IDBINDING => 0x80094818 as u32,
            SECURITY_SSPI::CERTSRV_E_INVALID_ATTESTATION => 0x80094819 as u32,
            SECURITY_SSPI::CERTSRV_E_KEY_ATTESTATION => 0x8009481A as u32,
            SECURITY_SSPI::CERTSRV_E_CORRUPT_KEY_ATTESTATION => 0x8009481B as u32,
            SECURITY_SSPI::CERTSRV_E_EXPIRED_CHALLENGE => 0x8009481C as u32,
            SECURITY_SSPI::CERTSRV_E_INVALID_RESPONSE => 0x8009481D as u32,
            SECURITY_SSPI::CERTSRV_E_INVALID_REQUESTID => 0x8009481E as u32,
            SECURITY_SSPI::CERTSRV_E_REQUEST_PRECERTIFICATE_MISMATCH => 0x8009481F as u32,
            SECURITY_SSPI::CERTSRV_E_PENDING_CLIENT_RESPONSE => 0x80094820 as u32,
            SECURITY_SSPI::CERTSRV_E_SEC_EXT_DIRECTORY_SID_REQUIRED => 0x80094821 as u32,
            SECURITY_SSPI::XENROLL_E_KEY_NOT_EXPORTABLE => 0x80095000 as u32,
            SECURITY_SSPI::XENROLL_E_CANNOT_ADD_ROOT_CERT => 0x80095001 as u32,
            SECURITY_SSPI::XENROLL_E_RESPONSE_KA_HASH_NOT_FOUND => 0x80095002 as u32,
            SECURITY_SSPI::XENROLL_E_RESPONSE_UNEXPECTED_KA_HASH => 0x80095003 as u32,
            SECURITY_SSPI::XENROLL_E_RESPONSE_KA_HASH_MISMATCH => 0x80095004 as u32,
            SECURITY_SSPI::XENROLL_E_KEYSPEC_SMIME_MISMATCH => 0x80095005 as u32,
            SECURITY_SSPI::TRUST_E_SYSTEM_ERROR => 0x80096001 as u32,
            SECURITY_SSPI::TRUST_E_NO_SIGNER_CERT => 0x80096002 as u32,
            SECURITY_SSPI::TRUST_E_COUNTER_SIGNER => 0x80096003 as u32,
            SECURITY_SSPI::TRUST_E_CERT_SIGNATURE => 0x80096004 as u32,
            SECURITY_SSPI::TRUST_E_TIME_STAMP => 0x80096005 as u32,
            SECURITY_SSPI::TRUST_E_BAD_DIGEST => 0x80096010 as u32,
            SECURITY_SSPI::TRUST_E_MALFORMED_SIGNATURE => 0x80096011 as u32,
            SECURITY_SSPI::TRUST_E_BASIC_CONSTRAINTS => 0x80096019 as u32,
            SECURITY_SSPI::TRUST_E_FINANCIAL_CRITERIA => 0x8009601E as u32,
            SECURITY_SSPI::MSSIPOTF_E_OUTOFMEMRANGE => 0x80097001 as u32,
            SECURITY_SSPI::MSSIPOTF_E_CANTGETOBJECT => 0x80097002 as u32,
            SECURITY_SSPI::MSSIPOTF_E_NOHEADTABLE => 0x80097003 as u32,
            SECURITY_SSPI::MSSIPOTF_E_BAD_MAGICNUMBER => 0x80097004 as u32,
            SECURITY_SSPI::MSSIPOTF_E_BAD_OFFSET_TABLE => 0x80097005 as u32,
            SECURITY_SSPI::MSSIPOTF_E_TABLE_TAGORDER => 0x80097006 as u32,
            SECURITY_SSPI::MSSIPOTF_E_TABLE_LONGWORD => 0x80097007 as u32,
            SECURITY_SSPI::MSSIPOTF_E_BAD_FIRST_TABLE_PLACEMENT => 0x80097008 as u32,
            SECURITY_SSPI::MSSIPOTF_E_TABLES_OVERLAP => 0x80097009 as u32,
            SECURITY_SSPI::MSSIPOTF_E_TABLE_PADBYTES => 0x8009700A as u32,
            SECURITY_SSPI::MSSIPOTF_E_FILETOOSMALL => 0x8009700B as u32,
            SECURITY_SSPI::MSSIPOTF_E_TABLE_CHECKSUM => 0x8009700C as u32,
            SECURITY_SSPI::MSSIPOTF_E_FILE_CHECKSUM => 0x8009700D as u32,
            SECURITY_SSPI::MSSIPOTF_E_FAILED_POLICY => 0x80097010 as u32,
            SECURITY_SSPI::MSSIPOTF_E_FAILED_HINTS_CHECK => 0x80097011 as u32,
            SECURITY_SSPI::MSSIPOTF_E_NOT_OPENTYPE => 0x80097012 as u32,
            SECURITY_SSPI::MSSIPOTF_E_FILE => 0x80097013 as u32,
            SECURITY_SSPI::MSSIPOTF_E_CRYPT => 0x80097014 as u32,
            SECURITY_SSPI::MSSIPOTF_E_BADVERSION => 0x80097015 as u32,
            SECURITY_SSPI::MSSIPOTF_E_DSIG_STRUCTURE => 0x80097016 as u32,
            SECURITY_SSPI::MSSIPOTF_E_PCONST_CHECK => 0x80097017 as u32,
            SECURITY_SSPI::MSSIPOTF_E_STRUCTURE => 0x80097018 as u32,
            SECURITY_SSPI::ERROR_CRED_REQUIRES_CONFIRMATION => 0x80097019 as u32,
            SECURITY_SSPI::ERROR_AUDITING_DISABLED => 0xC0090001 as u32,
            SECURITY_SSPI::ERROR_ALL_SIDS_FILTERED => 0xC0090002 as u32,
            SECURITY_SSPI::ERROR_BIZRULES_NOT_ENABLED => 0xC0090003 as u32,
        }
    }

    pub fn error(&self) -> RawError {
        return match self {
            SECURITY_SSPI::SEC_I_CONTINUE_NEEDED => RawError::Kind(SECURITY_SSPI::SEC_I_CONTINUE_NEEDED),
            SECURITY_SSPI::SEC_I_COMPLETE_NEEDED => RawError::Kind(SECURITY_SSPI::SEC_I_COMPLETE_NEEDED),
            SECURITY_SSPI::SEC_I_COMPLETE_AND_CONTINUE => RawError::Kind(SECURITY_SSPI::SEC_I_COMPLETE_AND_CONTINUE),
            SECURITY_SSPI::SEC_I_LOCAL_LOGON => RawError::Kind(SECURITY_SSPI::SEC_I_LOCAL_LOGON),
            SECURITY_SSPI::SEC_I_GENERIC_EXTENSION_RECEIVED => RawError::Kind(SECURITY_SSPI::SEC_I_GENERIC_EXTENSION_RECEIVED),
            SECURITY_SSPI::SEC_I_CONTEXT_EXPIRED => RawError::Kind(SECURITY_SSPI::SEC_I_CONTEXT_EXPIRED),
            SECURITY_SSPI::SEC_I_INCOMPLETE_CREDENTIALS => RawError::Kind(SECURITY_SSPI::SEC_I_INCOMPLETE_CREDENTIALS),
            SECURITY_SSPI::SEC_I_RENEGOTIATE => RawError::Kind(SECURITY_SSPI::SEC_I_RENEGOTIATE),
            SECURITY_SSPI::SEC_I_NO_LSA_CONTEXT => RawError::Kind(SECURITY_SSPI::SEC_I_NO_LSA_CONTEXT),
            SECURITY_SSPI::SEC_I_SIGNATURE_NEEDED => RawError::Kind(SECURITY_SSPI::SEC_I_SIGNATURE_NEEDED),
            SECURITY_SSPI::SEC_I_NO_RENEGOTIATION => RawError::Kind(SECURITY_SSPI::SEC_I_NO_RENEGOTIATION),
            SECURITY_SSPI::SEC_I_MESSAGE_FRAGMENT => RawError::Kind(SECURITY_SSPI::SEC_I_MESSAGE_FRAGMENT),
            SECURITY_SSPI::SEC_I_CONTINUE_NEEDED_MESSAGE_OK => RawError::Kind(SECURITY_SSPI::SEC_I_CONTINUE_NEEDED_MESSAGE_OK),
            SECURITY_SSPI::SEC_I_ASYNC_CALL_PENDING => RawError::Kind(SECURITY_SSPI::SEC_I_ASYNC_CALL_PENDING),
            SECURITY_SSPI::CRYPT_I_NEW_PROTECTION_REQUIRED => RawError::Kind(SECURITY_SSPI::CRYPT_I_NEW_PROTECTION_REQUIRED),
            SECURITY_SSPI::NTE_BAD_UID => RawError::Kind(SECURITY_SSPI::NTE_BAD_UID),
            SECURITY_SSPI::NTE_BAD_HASH => RawError::Kind(SECURITY_SSPI::NTE_BAD_HASH),
            SECURITY_SSPI::NTE_BAD_KEY => RawError::Kind(SECURITY_SSPI::NTE_BAD_KEY),
            SECURITY_SSPI::NTE_BAD_LEN => RawError::Kind(SECURITY_SSPI::NTE_BAD_LEN),
            SECURITY_SSPI::NTE_BAD_DATA => RawError::Kind(SECURITY_SSPI::NTE_BAD_DATA),
            SECURITY_SSPI::NTE_BAD_SIGNATURE => RawError::Kind(SECURITY_SSPI::NTE_BAD_SIGNATURE),
            SECURITY_SSPI::NTE_BAD_VER => RawError::Kind(SECURITY_SSPI::NTE_BAD_VER),
            SECURITY_SSPI::NTE_BAD_ALGID => RawError::Kind(SECURITY_SSPI::NTE_BAD_ALGID),
            SECURITY_SSPI::NTE_BAD_FLAGS => RawError::Kind(SECURITY_SSPI::NTE_BAD_FLAGS),
            SECURITY_SSPI::NTE_BAD_TYPE => RawError::Kind(SECURITY_SSPI::NTE_BAD_TYPE),
            SECURITY_SSPI::NTE_BAD_KEY_STATE => RawError::Kind(SECURITY_SSPI::NTE_BAD_KEY_STATE),
            SECURITY_SSPI::NTE_BAD_HASH_STATE => RawError::Kind(SECURITY_SSPI::NTE_BAD_HASH_STATE),
            SECURITY_SSPI::NTE_NO_KEY => RawError::Kind(SECURITY_SSPI::NTE_NO_KEY),
            SECURITY_SSPI::NTE_NO_MEMORY => RawError::Kind(SECURITY_SSPI::NTE_NO_MEMORY),
            SECURITY_SSPI::NTE_EXISTS => RawError::Kind(SECURITY_SSPI::NTE_EXISTS),
            SECURITY_SSPI::NTE_PERM => RawError::Kind(SECURITY_SSPI::NTE_PERM),
            SECURITY_SSPI::NTE_NOT_FOUND => RawError::Kind(SECURITY_SSPI::NTE_NOT_FOUND),
            SECURITY_SSPI::NTE_DOUBLE_ENCRYPT => RawError::Kind(SECURITY_SSPI::NTE_DOUBLE_ENCRYPT),
            SECURITY_SSPI::NTE_BAD_PROVIDER => RawError::Kind(SECURITY_SSPI::NTE_BAD_PROVIDER),
            SECURITY_SSPI::NTE_BAD_PROV_TYPE => RawError::Kind(SECURITY_SSPI::NTE_BAD_PROV_TYPE),
            SECURITY_SSPI::NTE_BAD_PUBLIC_KEY => RawError::Kind(SECURITY_SSPI::NTE_BAD_PUBLIC_KEY),
            SECURITY_SSPI::NTE_BAD_KEYSET => RawError::Kind(SECURITY_SSPI::NTE_BAD_KEYSET),
            SECURITY_SSPI::NTE_PROV_TYPE_NOT_DEF => RawError::Kind(SECURITY_SSPI::NTE_PROV_TYPE_NOT_DEF),
            SECURITY_SSPI::NTE_PROV_TYPE_ENTRY_BAD => RawError::Kind(SECURITY_SSPI::NTE_PROV_TYPE_ENTRY_BAD),
            SECURITY_SSPI::NTE_KEYSET_NOT_DEF => RawError::Kind(SECURITY_SSPI::NTE_KEYSET_NOT_DEF),
            SECURITY_SSPI::NTE_KEYSET_ENTRY_BAD => RawError::Kind(SECURITY_SSPI::NTE_KEYSET_ENTRY_BAD),
            SECURITY_SSPI::NTE_PROV_TYPE_NO_MATCH => RawError::Kind(SECURITY_SSPI::NTE_PROV_TYPE_NO_MATCH),
            SECURITY_SSPI::NTE_SIGNATURE_FILE_BAD => RawError::Kind(SECURITY_SSPI::NTE_SIGNATURE_FILE_BAD),
            SECURITY_SSPI::NTE_PROVIDER_DLL_FAIL => RawError::Kind(SECURITY_SSPI::NTE_PROVIDER_DLL_FAIL),
            SECURITY_SSPI::NTE_PROV_DLL_NOT_FOUND => RawError::Kind(SECURITY_SSPI::NTE_PROV_DLL_NOT_FOUND),
            SECURITY_SSPI::NTE_BAD_KEYSET_PARAM => RawError::Kind(SECURITY_SSPI::NTE_BAD_KEYSET_PARAM),
            SECURITY_SSPI::NTE_FAIL => RawError::Kind(SECURITY_SSPI::NTE_FAIL),
            SECURITY_SSPI::NTE_SYS_ERR => RawError::Kind(SECURITY_SSPI::NTE_SYS_ERR),
            SECURITY_SSPI::NTE_SILENT_CONTEXT => RawError::Kind(SECURITY_SSPI::NTE_SILENT_CONTEXT),
            SECURITY_SSPI::NTE_TOKEN_KEYSET_STORAGE_FULL => RawError::Kind(SECURITY_SSPI::NTE_TOKEN_KEYSET_STORAGE_FULL),
            SECURITY_SSPI::NTE_TEMPORARY_PROFILE => RawError::Kind(SECURITY_SSPI::NTE_TEMPORARY_PROFILE),
            SECURITY_SSPI::NTE_FIXEDPARAMETER => RawError::Kind(SECURITY_SSPI::NTE_FIXEDPARAMETER),
            SECURITY_SSPI::NTE_INVALID_HANDLE => RawError::Kind(SECURITY_SSPI::NTE_INVALID_HANDLE),
            SECURITY_SSPI::NTE_INVALID_PARAMETER => RawError::Kind(SECURITY_SSPI::NTE_INVALID_PARAMETER),
            SECURITY_SSPI::NTE_BUFFER_TOO_SMALL => RawError::Kind(SECURITY_SSPI::NTE_BUFFER_TOO_SMALL),
            SECURITY_SSPI::NTE_NOT_SUPPORTED => RawError::Kind(SECURITY_SSPI::NTE_NOT_SUPPORTED),
            SECURITY_SSPI::NTE_NO_MORE_ITEMS => RawError::Kind(SECURITY_SSPI::NTE_NO_MORE_ITEMS),
            SECURITY_SSPI::NTE_BUFFERS_OVERLAP => RawError::Kind(SECURITY_SSPI::NTE_BUFFERS_OVERLAP),
            SECURITY_SSPI::NTE_DECRYPTION_FAILURE => RawError::Kind(SECURITY_SSPI::NTE_DECRYPTION_FAILURE),
            SECURITY_SSPI::NTE_INTERNAL_ERROR => RawError::Kind(SECURITY_SSPI::NTE_INTERNAL_ERROR),
            SECURITY_SSPI::NTE_UI_REQUIRED => RawError::Kind(SECURITY_SSPI::NTE_UI_REQUIRED),
            SECURITY_SSPI::NTE_HMAC_NOT_SUPPORTED => RawError::Kind(SECURITY_SSPI::NTE_HMAC_NOT_SUPPORTED),
            SECURITY_SSPI::NTE_DEVICE_NOT_READY => RawError::Kind(SECURITY_SSPI::NTE_DEVICE_NOT_READY),
            SECURITY_SSPI::NTE_AUTHENTICATION_IGNORED => RawError::Kind(SECURITY_SSPI::NTE_AUTHENTICATION_IGNORED),
            SECURITY_SSPI::NTE_VALIDATION_FAILED => RawError::Kind(SECURITY_SSPI::NTE_VALIDATION_FAILED),
            SECURITY_SSPI::NTE_INCORRECT_PASSWORD => RawError::Kind(SECURITY_SSPI::NTE_INCORRECT_PASSWORD),
            SECURITY_SSPI::NTE_ENCRYPTION_FAILURE => RawError::Kind(SECURITY_SSPI::NTE_ENCRYPTION_FAILURE),
            SECURITY_SSPI::NTE_DEVICE_NOT_FOUND => RawError::Kind(SECURITY_SSPI::NTE_DEVICE_NOT_FOUND),
            SECURITY_SSPI::NTE_USER_CANCELLED => RawError::Kind(SECURITY_SSPI::NTE_USER_CANCELLED),
            SECURITY_SSPI::NTE_PASSWORD_CHANGE_REQUIRED => RawError::Kind(SECURITY_SSPI::NTE_PASSWORD_CHANGE_REQUIRED),
            SECURITY_SSPI::NTE_NOT_ACTIVE_CONSOLE => RawError::Kind(SECURITY_SSPI::NTE_NOT_ACTIVE_CONSOLE),
            SECURITY_SSPI::SEC_E_INSUFFICIENT_MEMORY => RawError::Kind(SECURITY_SSPI::SEC_E_INSUFFICIENT_MEMORY),
            SECURITY_SSPI::SEC_E_INVALID_HANDLE => RawError::Kind(SECURITY_SSPI::SEC_E_INVALID_HANDLE),
            SECURITY_SSPI::SEC_E_UNSUPPORTED_FUNCTION => RawError::Kind(SECURITY_SSPI::SEC_E_UNSUPPORTED_FUNCTION),
            SECURITY_SSPI::SEC_E_TARGET_UNKNOWN => RawError::Kind(SECURITY_SSPI::SEC_E_TARGET_UNKNOWN),
            SECURITY_SSPI::SEC_E_INTERNAL_ERROR => RawError::Kind(SECURITY_SSPI::SEC_E_INTERNAL_ERROR),
            SECURITY_SSPI::SEC_E_SECPKG_NOT_FOUND => RawError::Kind(SECURITY_SSPI::SEC_E_SECPKG_NOT_FOUND),
            SECURITY_SSPI::SEC_E_NOT_OWNER => RawError::Kind(SECURITY_SSPI::SEC_E_NOT_OWNER),
            SECURITY_SSPI::SEC_E_CANNOT_INSTALL => RawError::Kind(SECURITY_SSPI::SEC_E_CANNOT_INSTALL),
            SECURITY_SSPI::SEC_E_INVALID_TOKEN => RawError::Kind(SECURITY_SSPI::SEC_E_INVALID_TOKEN),
            SECURITY_SSPI::SEC_E_CANNOT_PACK => RawError::Kind(SECURITY_SSPI::SEC_E_CANNOT_PACK),
            SECURITY_SSPI::SEC_E_QOP_NOT_SUPPORTED => RawError::Kind(SECURITY_SSPI::SEC_E_QOP_NOT_SUPPORTED),
            SECURITY_SSPI::SEC_E_NO_IMPERSONATION => RawError::Kind(SECURITY_SSPI::SEC_E_NO_IMPERSONATION),
            SECURITY_SSPI::SEC_E_LOGON_DENIED => RawError::Kind(SECURITY_SSPI::SEC_E_LOGON_DENIED),
            SECURITY_SSPI::SEC_E_UNKNOWN_CREDENTIALS => RawError::Kind(SECURITY_SSPI::SEC_E_UNKNOWN_CREDENTIALS),
            SECURITY_SSPI::SEC_E_NO_CREDENTIALS => RawError::Kind(SECURITY_SSPI::SEC_E_NO_CREDENTIALS),
            SECURITY_SSPI::SEC_E_MESSAGE_ALTERED => RawError::Kind(SECURITY_SSPI::SEC_E_MESSAGE_ALTERED),
            SECURITY_SSPI::SEC_E_OUT_OF_SEQUENCE => RawError::Kind(SECURITY_SSPI::SEC_E_OUT_OF_SEQUENCE),
            SECURITY_SSPI::SEC_E_NO_AUTHENTICATING_AUTHORITY => RawError::Kind(SECURITY_SSPI::SEC_E_NO_AUTHENTICATING_AUTHORITY),
            SECURITY_SSPI::SEC_E_BAD_PKGID => RawError::Kind(SECURITY_SSPI::SEC_E_BAD_PKGID),
            SECURITY_SSPI::SEC_E_CONTEXT_EXPIRED => RawError::Kind(SECURITY_SSPI::SEC_E_CONTEXT_EXPIRED),
            SECURITY_SSPI::SEC_E_INCOMPLETE_MESSAGE => RawError::Kind(SECURITY_SSPI::SEC_E_INCOMPLETE_MESSAGE),
            SECURITY_SSPI::SEC_E_INCOMPLETE_CREDENTIALS => RawError::Kind(SECURITY_SSPI::SEC_E_INCOMPLETE_CREDENTIALS),
            SECURITY_SSPI::SEC_E_BUFFER_TOO_SMALL => RawError::Kind(SECURITY_SSPI::SEC_E_BUFFER_TOO_SMALL),
            SECURITY_SSPI::SEC_E_WRONG_PRINCIPAL => RawError::Kind(SECURITY_SSPI::SEC_E_WRONG_PRINCIPAL),
            SECURITY_SSPI::SEC_E_TIME_SKEW => RawError::Kind(SECURITY_SSPI::SEC_E_TIME_SKEW),
            SECURITY_SSPI::SEC_E_UNTRUSTED_ROOT => RawError::Kind(SECURITY_SSPI::SEC_E_UNTRUSTED_ROOT),
            SECURITY_SSPI::SEC_E_ILLEGAL_MESSAGE => RawError::Kind(SECURITY_SSPI::SEC_E_ILLEGAL_MESSAGE),
            SECURITY_SSPI::SEC_E_CERT_UNKNOWN => RawError::Kind(SECURITY_SSPI::SEC_E_CERT_UNKNOWN),
            SECURITY_SSPI::SEC_E_CERT_EXPIRED => RawError::Kind(SECURITY_SSPI::SEC_E_CERT_EXPIRED),
            SECURITY_SSPI::SEC_E_ENCRYPT_FAILURE => RawError::Kind(SECURITY_SSPI::SEC_E_ENCRYPT_FAILURE),
            SECURITY_SSPI::SEC_E_DECRYPT_FAILURE => RawError::Kind(SECURITY_SSPI::SEC_E_DECRYPT_FAILURE),
            SECURITY_SSPI::SEC_E_ALGORITHM_MISMATCH => RawError::Kind(SECURITY_SSPI::SEC_E_ALGORITHM_MISMATCH),
            SECURITY_SSPI::SEC_E_SECURITY_QOS_FAILED => RawError::Kind(SECURITY_SSPI::SEC_E_SECURITY_QOS_FAILED),
            SECURITY_SSPI::SEC_E_UNFINISHED_CONTEXT_DELETED => RawError::Kind(SECURITY_SSPI::SEC_E_UNFINISHED_CONTEXT_DELETED),
            SECURITY_SSPI::SEC_E_NO_TGT_REPLY => RawError::Kind(SECURITY_SSPI::SEC_E_NO_TGT_REPLY),
            SECURITY_SSPI::SEC_E_NO_IP_ADDRESSES => RawError::Kind(SECURITY_SSPI::SEC_E_NO_IP_ADDRESSES),
            SECURITY_SSPI::SEC_E_WRONG_CREDENTIAL_HANDLE => RawError::Kind(SECURITY_SSPI::SEC_E_WRONG_CREDENTIAL_HANDLE),
            SECURITY_SSPI::SEC_E_CRYPTO_SYSTEM_INVALID => RawError::Kind(SECURITY_SSPI::SEC_E_CRYPTO_SYSTEM_INVALID),
            SECURITY_SSPI::SEC_E_MAX_REFERRALS_EXCEEDED => RawError::Kind(SECURITY_SSPI::SEC_E_MAX_REFERRALS_EXCEEDED),
            SECURITY_SSPI::SEC_E_MUST_BE_KDC => RawError::Kind(SECURITY_SSPI::SEC_E_MUST_BE_KDC),
            SECURITY_SSPI::SEC_E_STRONG_CRYPTO_NOT_SUPPORTED => RawError::Kind(SECURITY_SSPI::SEC_E_STRONG_CRYPTO_NOT_SUPPORTED),
            SECURITY_SSPI::SEC_E_TOO_MANY_PRINCIPALS => RawError::Kind(SECURITY_SSPI::SEC_E_TOO_MANY_PRINCIPALS),
            SECURITY_SSPI::SEC_E_NO_PA_DATA => RawError::Kind(SECURITY_SSPI::SEC_E_NO_PA_DATA),
            SECURITY_SSPI::SEC_E_PKINIT_NAME_MISMATCH => RawError::Kind(SECURITY_SSPI::SEC_E_PKINIT_NAME_MISMATCH),
            SECURITY_SSPI::SEC_E_SMARTCARD_LOGON_REQUIRED => RawError::Kind(SECURITY_SSPI::SEC_E_SMARTCARD_LOGON_REQUIRED),
            SECURITY_SSPI::SEC_E_SHUTDOWN_IN_PROGRESS => RawError::Kind(SECURITY_SSPI::SEC_E_SHUTDOWN_IN_PROGRESS),
            SECURITY_SSPI::SEC_E_KDC_INVALID_REQUEST => RawError::Kind(SECURITY_SSPI::SEC_E_KDC_INVALID_REQUEST),
            SECURITY_SSPI::SEC_E_KDC_UNABLE_TO_REFER => RawError::Kind(SECURITY_SSPI::SEC_E_KDC_UNABLE_TO_REFER),
            SECURITY_SSPI::SEC_E_KDC_UNKNOWN_ETYPE => RawError::Kind(SECURITY_SSPI::SEC_E_KDC_UNKNOWN_ETYPE),
            SECURITY_SSPI::SEC_E_UNSUPPORTED_PREAUTH => RawError::Kind(SECURITY_SSPI::SEC_E_UNSUPPORTED_PREAUTH),
            SECURITY_SSPI::SEC_E_DELEGATION_REQUIRED => RawError::Kind(SECURITY_SSPI::SEC_E_DELEGATION_REQUIRED),
            SECURITY_SSPI::SEC_E_BAD_BINDINGS => RawError::Kind(SECURITY_SSPI::SEC_E_BAD_BINDINGS),
            SECURITY_SSPI::SEC_E_MULTIPLE_ACCOUNTS => RawError::Kind(SECURITY_SSPI::SEC_E_MULTIPLE_ACCOUNTS),
            SECURITY_SSPI::SEC_E_NO_KERB_KEY => RawError::Kind(SECURITY_SSPI::SEC_E_NO_KERB_KEY),
            SECURITY_SSPI::SEC_E_CERT_WRONG_USAGE => RawError::Kind(SECURITY_SSPI::SEC_E_CERT_WRONG_USAGE),
            SECURITY_SSPI::SEC_E_DOWNGRADE_DETECTED => RawError::Kind(SECURITY_SSPI::SEC_E_DOWNGRADE_DETECTED),
            SECURITY_SSPI::SEC_E_SMARTCARD_CERT_REVOKED => RawError::Kind(SECURITY_SSPI::SEC_E_SMARTCARD_CERT_REVOKED),
            SECURITY_SSPI::SEC_E_ISSUING_CA_UNTRUSTED => RawError::Kind(SECURITY_SSPI::SEC_E_ISSUING_CA_UNTRUSTED),
            SECURITY_SSPI::SEC_E_REVOCATION_OFFLINE_C => RawError::Kind(SECURITY_SSPI::SEC_E_REVOCATION_OFFLINE_C),
            SECURITY_SSPI::SEC_E_PKINIT_CLIENT_FAILURE => RawError::Kind(SECURITY_SSPI::SEC_E_PKINIT_CLIENT_FAILURE),
            SECURITY_SSPI::SEC_E_SMARTCARD_CERT_EXPIRED => RawError::Kind(SECURITY_SSPI::SEC_E_SMARTCARD_CERT_EXPIRED),
            SECURITY_SSPI::SEC_E_NO_S4U_PROT_SUPPORT => RawError::Kind(SECURITY_SSPI::SEC_E_NO_S4U_PROT_SUPPORT),
            SECURITY_SSPI::SEC_E_CROSSREALM_DELEGATION_FAILURE => RawError::Kind(SECURITY_SSPI::SEC_E_CROSSREALM_DELEGATION_FAILURE),
            SECURITY_SSPI::SEC_E_REVOCATION_OFFLINE_KDC => RawError::Kind(SECURITY_SSPI::SEC_E_REVOCATION_OFFLINE_KDC),
            SECURITY_SSPI::SEC_E_ISSUING_CA_UNTRUSTED_KDC => RawError::Kind(SECURITY_SSPI::SEC_E_ISSUING_CA_UNTRUSTED_KDC),
            SECURITY_SSPI::SEC_E_KDC_CERT_EXPIRED => RawError::Kind(SECURITY_SSPI::SEC_E_KDC_CERT_EXPIRED),
            SECURITY_SSPI::SEC_E_KDC_CERT_REVOKED => RawError::Kind(SECURITY_SSPI::SEC_E_KDC_CERT_REVOKED),
            SECURITY_SSPI::SEC_E_INVALID_PARAMETER => RawError::Kind(SECURITY_SSPI::SEC_E_INVALID_PARAMETER),
            SECURITY_SSPI::SEC_E_DELEGATION_POLICY => RawError::Kind(SECURITY_SSPI::SEC_E_DELEGATION_POLICY),
            SECURITY_SSPI::SEC_E_POLICY_NLTM_ONLY => RawError::Kind(SECURITY_SSPI::SEC_E_POLICY_NLTM_ONLY),
            SECURITY_SSPI::SEC_E_NO_CONTEXT => RawError::Kind(SECURITY_SSPI::SEC_E_NO_CONTEXT),
            SECURITY_SSPI::SEC_E_PKU2U_CERT_FAILURE => RawError::Kind(SECURITY_SSPI::SEC_E_PKU2U_CERT_FAILURE),
            SECURITY_SSPI::SEC_E_MUTUAL_AUTH_FAILED => RawError::Kind(SECURITY_SSPI::SEC_E_MUTUAL_AUTH_FAILED),
            SECURITY_SSPI::SEC_E_ONLY_HTTPS_ALLOWED => RawError::Kind(SECURITY_SSPI::SEC_E_ONLY_HTTPS_ALLOWED),
            SECURITY_SSPI::SEC_E_APPLICATION_PROTOCOL_MISMATCH => RawError::Kind(SECURITY_SSPI::SEC_E_APPLICATION_PROTOCOL_MISMATCH),
            SECURITY_SSPI::SEC_E_INVALID_UPN_NAME => RawError::Kind(SECURITY_SSPI::SEC_E_INVALID_UPN_NAME),
            SECURITY_SSPI::SEC_E_EXT_BUFFER_TOO_SMALL => RawError::Kind(SECURITY_SSPI::SEC_E_EXT_BUFFER_TOO_SMALL),
            SECURITY_SSPI::SEC_E_INSUFFICIENT_BUFFERS => RawError::Kind(SECURITY_SSPI::SEC_E_INSUFFICIENT_BUFFERS),
            SECURITY_SSPI::CRYPT_E_MSG_ERROR => RawError::Kind(SECURITY_SSPI::CRYPT_E_MSG_ERROR),
            SECURITY_SSPI::CRYPT_E_UNKNOWN_ALGO => RawError::Kind(SECURITY_SSPI::CRYPT_E_UNKNOWN_ALGO),
            SECURITY_SSPI::CRYPT_E_OID_FORMAT => RawError::Kind(SECURITY_SSPI::CRYPT_E_OID_FORMAT),
            SECURITY_SSPI::CRYPT_E_INVALID_MSG_TYPE => RawError::Kind(SECURITY_SSPI::CRYPT_E_INVALID_MSG_TYPE),
            SECURITY_SSPI::CRYPT_E_UNEXPECTED_ENCODING => RawError::Kind(SECURITY_SSPI::CRYPT_E_UNEXPECTED_ENCODING),
            SECURITY_SSPI::CRYPT_E_AUTH_ATTR_MISSING => RawError::Kind(SECURITY_SSPI::CRYPT_E_AUTH_ATTR_MISSING),
            SECURITY_SSPI::CRYPT_E_HASH_VALUE => RawError::Kind(SECURITY_SSPI::CRYPT_E_HASH_VALUE),
            SECURITY_SSPI::CRYPT_E_INVALID_INDEX => RawError::Kind(SECURITY_SSPI::CRYPT_E_INVALID_INDEX),
            SECURITY_SSPI::CRYPT_E_ALREADY_DECRYPTED => RawError::Kind(SECURITY_SSPI::CRYPT_E_ALREADY_DECRYPTED),
            SECURITY_SSPI::CRYPT_E_NOT_DECRYPTED => RawError::Kind(SECURITY_SSPI::CRYPT_E_NOT_DECRYPTED),
            SECURITY_SSPI::CRYPT_E_RECIPIENT_NOT_FOUND => RawError::Kind(SECURITY_SSPI::CRYPT_E_RECIPIENT_NOT_FOUND),
            SECURITY_SSPI::CRYPT_E_CONTROL_TYPE => RawError::Kind(SECURITY_SSPI::CRYPT_E_CONTROL_TYPE),
            SECURITY_SSPI::CRYPT_E_ISSUER_SERIALNUMBER => RawError::Kind(SECURITY_SSPI::CRYPT_E_ISSUER_SERIALNUMBER),
            SECURITY_SSPI::CRYPT_E_SIGNER_NOT_FOUND => RawError::Kind(SECURITY_SSPI::CRYPT_E_SIGNER_NOT_FOUND),
            SECURITY_SSPI::CRYPT_E_ATTRIBUTES_MISSING => RawError::Kind(SECURITY_SSPI::CRYPT_E_ATTRIBUTES_MISSING),
            SECURITY_SSPI::CRYPT_E_STREAM_MSG_NOT_READY => RawError::Kind(SECURITY_SSPI::CRYPT_E_STREAM_MSG_NOT_READY),
            SECURITY_SSPI::CRYPT_E_STREAM_INSUFFICIENT_DATA => RawError::Kind(SECURITY_SSPI::CRYPT_E_STREAM_INSUFFICIENT_DATA),
            SECURITY_SSPI::CRYPT_E_BAD_LEN => RawError::Kind(SECURITY_SSPI::CRYPT_E_BAD_LEN),
            SECURITY_SSPI::CRYPT_E_BAD_ENCODE => RawError::Kind(SECURITY_SSPI::CRYPT_E_BAD_ENCODE),
            SECURITY_SSPI::CRYPT_E_FILE_ERROR => RawError::Kind(SECURITY_SSPI::CRYPT_E_FILE_ERROR),
            SECURITY_SSPI::CRYPT_E_NOT_FOUND => RawError::Kind(SECURITY_SSPI::CRYPT_E_NOT_FOUND),
            SECURITY_SSPI::CRYPT_E_EXISTS => RawError::Kind(SECURITY_SSPI::CRYPT_E_EXISTS),
            SECURITY_SSPI::CRYPT_E_NO_PROVIDER => RawError::Kind(SECURITY_SSPI::CRYPT_E_NO_PROVIDER),
            SECURITY_SSPI::CRYPT_E_SELF_SIGNED => RawError::Kind(SECURITY_SSPI::CRYPT_E_SELF_SIGNED),
            SECURITY_SSPI::CRYPT_E_DELETED_PREV => RawError::Kind(SECURITY_SSPI::CRYPT_E_DELETED_PREV),
            SECURITY_SSPI::CRYPT_E_NO_MATCH => RawError::Kind(SECURITY_SSPI::CRYPT_E_NO_MATCH),
            SECURITY_SSPI::CRYPT_E_UNEXPECTED_MSG_TYPE => RawError::Kind(SECURITY_SSPI::CRYPT_E_UNEXPECTED_MSG_TYPE),
            SECURITY_SSPI::CRYPT_E_NO_KEY_PROPERTY => RawError::Kind(SECURITY_SSPI::CRYPT_E_NO_KEY_PROPERTY),
            SECURITY_SSPI::CRYPT_E_NO_DECRYPT_CERT => RawError::Kind(SECURITY_SSPI::CRYPT_E_NO_DECRYPT_CERT),
            SECURITY_SSPI::CRYPT_E_BAD_MSG => RawError::Kind(SECURITY_SSPI::CRYPT_E_BAD_MSG),
            SECURITY_SSPI::CRYPT_E_NO_SIGNER => RawError::Kind(SECURITY_SSPI::CRYPT_E_NO_SIGNER),
            SECURITY_SSPI::CRYPT_E_PENDING_CLOSE => RawError::Kind(SECURITY_SSPI::CRYPT_E_PENDING_CLOSE),
            SECURITY_SSPI::CRYPT_E_REVOKED => RawError::Kind(SECURITY_SSPI::CRYPT_E_REVOKED),
            SECURITY_SSPI::CRYPT_E_NO_REVOCATION_DLL => RawError::Kind(SECURITY_SSPI::CRYPT_E_NO_REVOCATION_DLL),
            SECURITY_SSPI::CRYPT_E_NO_REVOCATION_CHECK => RawError::Kind(SECURITY_SSPI::CRYPT_E_NO_REVOCATION_CHECK),
            SECURITY_SSPI::CRYPT_E_REVOCATION_OFFLINE => RawError::Kind(SECURITY_SSPI::CRYPT_E_REVOCATION_OFFLINE),
            SECURITY_SSPI::CRYPT_E_NOT_IN_REVOCATION_DATABASE => RawError::Kind(SECURITY_SSPI::CRYPT_E_NOT_IN_REVOCATION_DATABASE),
            SECURITY_SSPI::CRYPT_E_INVALID_NUMERIC_STRING => RawError::Kind(SECURITY_SSPI::CRYPT_E_INVALID_NUMERIC_STRING),
            SECURITY_SSPI::CRYPT_E_INVALID_PRINTABLE_STRING => RawError::Kind(SECURITY_SSPI::CRYPT_E_INVALID_PRINTABLE_STRING),
            SECURITY_SSPI::CRYPT_E_INVALID_IA5_STRING => RawError::Kind(SECURITY_SSPI::CRYPT_E_INVALID_IA5_STRING),
            SECURITY_SSPI::CRYPT_E_INVALID_X500_STRING => RawError::Kind(SECURITY_SSPI::CRYPT_E_INVALID_X500_STRING),
            SECURITY_SSPI::CRYPT_E_NOT_CHAR_STRING => RawError::Kind(SECURITY_SSPI::CRYPT_E_NOT_CHAR_STRING),
            SECURITY_SSPI::CRYPT_E_FILERESIZED => RawError::Kind(SECURITY_SSPI::CRYPT_E_FILERESIZED),
            SECURITY_SSPI::CRYPT_E_SECURITY_SETTINGS => RawError::Kind(SECURITY_SSPI::CRYPT_E_SECURITY_SETTINGS),
            SECURITY_SSPI::CRYPT_E_NO_VERIFY_USAGE_DLL => RawError::Kind(SECURITY_SSPI::CRYPT_E_NO_VERIFY_USAGE_DLL),
            SECURITY_SSPI::CRYPT_E_NO_VERIFY_USAGE_CHECK => RawError::Kind(SECURITY_SSPI::CRYPT_E_NO_VERIFY_USAGE_CHECK),
            SECURITY_SSPI::CRYPT_E_VERIFY_USAGE_OFFLINE => RawError::Kind(SECURITY_SSPI::CRYPT_E_VERIFY_USAGE_OFFLINE),
            SECURITY_SSPI::CRYPT_E_NOT_IN_CTL => RawError::Kind(SECURITY_SSPI::CRYPT_E_NOT_IN_CTL),
            SECURITY_SSPI::CRYPT_E_NO_TRUSTED_SIGNER => RawError::Kind(SECURITY_SSPI::CRYPT_E_NO_TRUSTED_SIGNER),
            SECURITY_SSPI::CRYPT_E_MISSING_PUBKEY_PARA => RawError::Kind(SECURITY_SSPI::CRYPT_E_MISSING_PUBKEY_PARA),
            SECURITY_SSPI::CRYPT_E_OBJECT_LOCATOR_OBJECT_NOT_FOUND => RawError::Kind(SECURITY_SSPI::CRYPT_E_OBJECT_LOCATOR_OBJECT_NOT_FOUND),
            SECURITY_SSPI::CRYPT_E_OSS_ERROR => RawError::Kind(SECURITY_SSPI::CRYPT_E_OSS_ERROR),
            SECURITY_SSPI::OSS_MORE_BUF => RawError::Kind(SECURITY_SSPI::OSS_MORE_BUF),
            SECURITY_SSPI::OSS_NEGATIVE_UINTEGER => RawError::Kind(SECURITY_SSPI::OSS_NEGATIVE_UINTEGER),
            SECURITY_SSPI::OSS_PDU_RANGE => RawError::Kind(SECURITY_SSPI::OSS_PDU_RANGE),
            SECURITY_SSPI::OSS_MORE_INPUT => RawError::Kind(SECURITY_SSPI::OSS_MORE_INPUT),
            SECURITY_SSPI::OSS_DATA_ERROR => RawError::Kind(SECURITY_SSPI::OSS_DATA_ERROR),
            SECURITY_SSPI::OSS_BAD_ARG => RawError::Kind(SECURITY_SSPI::OSS_BAD_ARG),
            SECURITY_SSPI::OSS_BAD_VERSION => RawError::Kind(SECURITY_SSPI::OSS_BAD_VERSION),
            SECURITY_SSPI::OSS_OUT_MEMORY => RawError::Kind(SECURITY_SSPI::OSS_OUT_MEMORY),
            SECURITY_SSPI::OSS_PDU_MISMATCH => RawError::Kind(SECURITY_SSPI::OSS_PDU_MISMATCH),
            SECURITY_SSPI::OSS_LIMITED => RawError::Kind(SECURITY_SSPI::OSS_LIMITED),
            SECURITY_SSPI::OSS_BAD_PTR => RawError::Kind(SECURITY_SSPI::OSS_BAD_PTR),
            SECURITY_SSPI::OSS_BAD_TIME => RawError::Kind(SECURITY_SSPI::OSS_BAD_TIME),
            SECURITY_SSPI::OSS_INDEFINITE_NOT_SUPPORTED => RawError::Kind(SECURITY_SSPI::OSS_INDEFINITE_NOT_SUPPORTED),
            SECURITY_SSPI::OSS_MEM_ERROR => RawError::Kind(SECURITY_SSPI::OSS_MEM_ERROR),
            SECURITY_SSPI::OSS_BAD_TABLE => RawError::Kind(SECURITY_SSPI::OSS_BAD_TABLE),
            SECURITY_SSPI::OSS_TOO_LONG => RawError::Kind(SECURITY_SSPI::OSS_TOO_LONG),
            SECURITY_SSPI::OSS_CONSTRAINT_VIOLATED => RawError::Kind(SECURITY_SSPI::OSS_CONSTRAINT_VIOLATED),
            SECURITY_SSPI::OSS_FATAL_ERROR => RawError::Kind(SECURITY_SSPI::OSS_FATAL_ERROR),
            SECURITY_SSPI::OSS_ACCESS_SERIALIZATION_ERROR => RawError::Kind(SECURITY_SSPI::OSS_ACCESS_SERIALIZATION_ERROR),
            SECURITY_SSPI::OSS_NULL_TBL => RawError::Kind(SECURITY_SSPI::OSS_NULL_TBL),
            SECURITY_SSPI::OSS_NULL_FCN => RawError::Kind(SECURITY_SSPI::OSS_NULL_FCN),
            SECURITY_SSPI::OSS_BAD_ENCRULES => RawError::Kind(SECURITY_SSPI::OSS_BAD_ENCRULES),
            SECURITY_SSPI::OSS_UNAVAIL_ENCRULES => RawError::Kind(SECURITY_SSPI::OSS_UNAVAIL_ENCRULES),
            SECURITY_SSPI::OSS_CANT_OPEN_TRACE_WINDOW => RawError::Kind(SECURITY_SSPI::OSS_CANT_OPEN_TRACE_WINDOW),
            SECURITY_SSPI::OSS_UNIMPLEMENTED => RawError::Kind(SECURITY_SSPI::OSS_UNIMPLEMENTED),
            SECURITY_SSPI::OSS_OID_DLL_NOT_LINKED => RawError::Kind(SECURITY_SSPI::OSS_OID_DLL_NOT_LINKED),
            SECURITY_SSPI::OSS_CANT_OPEN_TRACE_FILE => RawError::Kind(SECURITY_SSPI::OSS_CANT_OPEN_TRACE_FILE),
            SECURITY_SSPI::OSS_TRACE_FILE_ALREADY_OPEN => RawError::Kind(SECURITY_SSPI::OSS_TRACE_FILE_ALREADY_OPEN),
            SECURITY_SSPI::OSS_TABLE_MISMATCH => RawError::Kind(SECURITY_SSPI::OSS_TABLE_MISMATCH),
            SECURITY_SSPI::OSS_TYPE_NOT_SUPPORTED => RawError::Kind(SECURITY_SSPI::OSS_TYPE_NOT_SUPPORTED),
            SECURITY_SSPI::OSS_REAL_DLL_NOT_LINKED => RawError::Kind(SECURITY_SSPI::OSS_REAL_DLL_NOT_LINKED),
            SECURITY_SSPI::OSS_REAL_CODE_NOT_LINKED => RawError::Kind(SECURITY_SSPI::OSS_REAL_CODE_NOT_LINKED),
            SECURITY_SSPI::OSS_OUT_OF_RANGE => RawError::Kind(SECURITY_SSPI::OSS_OUT_OF_RANGE),
            SECURITY_SSPI::OSS_COPIER_DLL_NOT_LINKED => RawError::Kind(SECURITY_SSPI::OSS_COPIER_DLL_NOT_LINKED),
            SECURITY_SSPI::OSS_CONSTRAINT_DLL_NOT_LINKED => RawError::Kind(SECURITY_SSPI::OSS_CONSTRAINT_DLL_NOT_LINKED),
            SECURITY_SSPI::OSS_COMPARATOR_DLL_NOT_LINKED => RawError::Kind(SECURITY_SSPI::OSS_COMPARATOR_DLL_NOT_LINKED),
            SECURITY_SSPI::OSS_COMPARATOR_CODE_NOT_LINKED => RawError::Kind(SECURITY_SSPI::OSS_COMPARATOR_CODE_NOT_LINKED),
            SECURITY_SSPI::OSS_MEM_MGR_DLL_NOT_LINKED => RawError::Kind(SECURITY_SSPI::OSS_MEM_MGR_DLL_NOT_LINKED),
            SECURITY_SSPI::OSS_PDV_DLL_NOT_LINKED => RawError::Kind(SECURITY_SSPI::OSS_PDV_DLL_NOT_LINKED),
            SECURITY_SSPI::OSS_PDV_CODE_NOT_LINKED => RawError::Kind(SECURITY_SSPI::OSS_PDV_CODE_NOT_LINKED),
            SECURITY_SSPI::OSS_API_DLL_NOT_LINKED => RawError::Kind(SECURITY_SSPI::OSS_API_DLL_NOT_LINKED),
            SECURITY_SSPI::OSS_BERDER_DLL_NOT_LINKED => RawError::Kind(SECURITY_SSPI::OSS_BERDER_DLL_NOT_LINKED),
            SECURITY_SSPI::OSS_PER_DLL_NOT_LINKED => RawError::Kind(SECURITY_SSPI::OSS_PER_DLL_NOT_LINKED),
            SECURITY_SSPI::OSS_OPEN_TYPE_ERROR => RawError::Kind(SECURITY_SSPI::OSS_OPEN_TYPE_ERROR),
            SECURITY_SSPI::OSS_MUTEX_NOT_CREATED => RawError::Kind(SECURITY_SSPI::OSS_MUTEX_NOT_CREATED),
            SECURITY_SSPI::OSS_CANT_CLOSE_TRACE_FILE => RawError::Kind(SECURITY_SSPI::OSS_CANT_CLOSE_TRACE_FILE),
            SECURITY_SSPI::CRYPT_E_ASN1_ERROR => RawError::Kind(SECURITY_SSPI::CRYPT_E_ASN1_ERROR),
            SECURITY_SSPI::CRYPT_E_ASN1_INTERNAL => RawError::Kind(SECURITY_SSPI::CRYPT_E_ASN1_INTERNAL),
            SECURITY_SSPI::CRYPT_E_ASN1_EOD => RawError::Kind(SECURITY_SSPI::CRYPT_E_ASN1_EOD),
            SECURITY_SSPI::CRYPT_E_ASN1_CORRUPT => RawError::Kind(SECURITY_SSPI::CRYPT_E_ASN1_CORRUPT),
            SECURITY_SSPI::CRYPT_E_ASN1_LARGE => RawError::Kind(SECURITY_SSPI::CRYPT_E_ASN1_LARGE),
            SECURITY_SSPI::CRYPT_E_ASN1_CONSTRAINT => RawError::Kind(SECURITY_SSPI::CRYPT_E_ASN1_CONSTRAINT),
            SECURITY_SSPI::CRYPT_E_ASN1_MEMORY => RawError::Kind(SECURITY_SSPI::CRYPT_E_ASN1_MEMORY),
            SECURITY_SSPI::CRYPT_E_ASN1_OVERFLOW => RawError::Kind(SECURITY_SSPI::CRYPT_E_ASN1_OVERFLOW),
            SECURITY_SSPI::CRYPT_E_ASN1_BADPDU => RawError::Kind(SECURITY_SSPI::CRYPT_E_ASN1_BADPDU),
            SECURITY_SSPI::CRYPT_E_ASN1_BADARGS => RawError::Kind(SECURITY_SSPI::CRYPT_E_ASN1_BADARGS),
            SECURITY_SSPI::CRYPT_E_ASN1_BADREAL => RawError::Kind(SECURITY_SSPI::CRYPT_E_ASN1_BADREAL),
            SECURITY_SSPI::CRYPT_E_ASN1_BADTAG => RawError::Kind(SECURITY_SSPI::CRYPT_E_ASN1_BADTAG),
            SECURITY_SSPI::CRYPT_E_ASN1_CHOICE => RawError::Kind(SECURITY_SSPI::CRYPT_E_ASN1_CHOICE),
            SECURITY_SSPI::CRYPT_E_ASN1_RULE => RawError::Kind(SECURITY_SSPI::CRYPT_E_ASN1_RULE),
            SECURITY_SSPI::CRYPT_E_ASN1_UTF8 => RawError::Kind(SECURITY_SSPI::CRYPT_E_ASN1_UTF8),
            SECURITY_SSPI::CRYPT_E_ASN1_PDU_TYPE => RawError::Kind(SECURITY_SSPI::CRYPT_E_ASN1_PDU_TYPE),
            SECURITY_SSPI::CRYPT_E_ASN1_NYI => RawError::Kind(SECURITY_SSPI::CRYPT_E_ASN1_NYI),
            SECURITY_SSPI::CRYPT_E_ASN1_EXTENDED => RawError::Kind(SECURITY_SSPI::CRYPT_E_ASN1_EXTENDED),
            SECURITY_SSPI::CRYPT_E_ASN1_NOEOD => RawError::Kind(SECURITY_SSPI::CRYPT_E_ASN1_NOEOD),
            SECURITY_SSPI::CERTSRV_E_BAD_REQUESTSUBJECT => RawError::Kind(SECURITY_SSPI::CERTSRV_E_BAD_REQUESTSUBJECT),
            SECURITY_SSPI::CERTSRV_E_NO_REQUEST => RawError::Kind(SECURITY_SSPI::CERTSRV_E_NO_REQUEST),
            SECURITY_SSPI::CERTSRV_E_BAD_REQUESTSTATUS => RawError::Kind(SECURITY_SSPI::CERTSRV_E_BAD_REQUESTSTATUS),
            SECURITY_SSPI::CERTSRV_E_PROPERTY_EMPTY => RawError::Kind(SECURITY_SSPI::CERTSRV_E_PROPERTY_EMPTY),
            SECURITY_SSPI::CERTSRV_E_INVALID_CA_CERTIFICATE => RawError::Kind(SECURITY_SSPI::CERTSRV_E_INVALID_CA_CERTIFICATE),
            SECURITY_SSPI::CERTSRV_E_SERVER_SUSPENDED => RawError::Kind(SECURITY_SSPI::CERTSRV_E_SERVER_SUSPENDED),
            SECURITY_SSPI::CERTSRV_E_ENCODING_LENGTH => RawError::Kind(SECURITY_SSPI::CERTSRV_E_ENCODING_LENGTH),
            SECURITY_SSPI::CERTSRV_E_ROLECONFLICT => RawError::Kind(SECURITY_SSPI::CERTSRV_E_ROLECONFLICT),
            SECURITY_SSPI::CERTSRV_E_RESTRICTEDOFFICER => RawError::Kind(SECURITY_SSPI::CERTSRV_E_RESTRICTEDOFFICER),
            SECURITY_SSPI::CERTSRV_E_KEY_ARCHIVAL_NOT_CONFIGURED => RawError::Kind(SECURITY_SSPI::CERTSRV_E_KEY_ARCHIVAL_NOT_CONFIGURED),
            SECURITY_SSPI::CERTSRV_E_NO_VALID_KRA => RawError::Kind(SECURITY_SSPI::CERTSRV_E_NO_VALID_KRA),
            SECURITY_SSPI::CERTSRV_E_BAD_REQUEST_KEY_ARCHIVAL => RawError::Kind(SECURITY_SSPI::CERTSRV_E_BAD_REQUEST_KEY_ARCHIVAL),
            SECURITY_SSPI::CERTSRV_E_NO_CAADMIN_DEFINED => RawError::Kind(SECURITY_SSPI::CERTSRV_E_NO_CAADMIN_DEFINED),
            SECURITY_SSPI::CERTSRV_E_BAD_RENEWAL_CERT_ATTRIBUTE => RawError::Kind(SECURITY_SSPI::CERTSRV_E_BAD_RENEWAL_CERT_ATTRIBUTE),
            SECURITY_SSPI::CERTSRV_E_NO_DB_SESSIONS => RawError::Kind(SECURITY_SSPI::CERTSRV_E_NO_DB_SESSIONS),
            SECURITY_SSPI::CERTSRV_E_ALIGNMENT_FAULT => RawError::Kind(SECURITY_SSPI::CERTSRV_E_ALIGNMENT_FAULT),
            SECURITY_SSPI::CERTSRV_E_ENROLL_DENIED => RawError::Kind(SECURITY_SSPI::CERTSRV_E_ENROLL_DENIED),
            SECURITY_SSPI::CERTSRV_E_TEMPLATE_DENIED => RawError::Kind(SECURITY_SSPI::CERTSRV_E_TEMPLATE_DENIED),
            SECURITY_SSPI::CERTSRV_E_DOWNLEVEL_DC_SSL_OR_UPGRADE => RawError::Kind(SECURITY_SSPI::CERTSRV_E_DOWNLEVEL_DC_SSL_OR_UPGRADE),
            SECURITY_SSPI::CERTSRV_E_ADMIN_DENIED_REQUEST => RawError::Kind(SECURITY_SSPI::CERTSRV_E_ADMIN_DENIED_REQUEST),
            SECURITY_SSPI::CERTSRV_E_NO_POLICY_SERVER => RawError::Kind(SECURITY_SSPI::CERTSRV_E_NO_POLICY_SERVER),
            SECURITY_SSPI::CERTSRV_E_WEAK_SIGNATURE_OR_KEY => RawError::Kind(SECURITY_SSPI::CERTSRV_E_WEAK_SIGNATURE_OR_KEY),
            SECURITY_SSPI::CERTSRV_E_KEY_ATTESTATION_NOT_SUPPORTED => RawError::Kind(SECURITY_SSPI::CERTSRV_E_KEY_ATTESTATION_NOT_SUPPORTED),
            SECURITY_SSPI::CERTSRV_E_ENCRYPTION_CERT_REQUIRED => RawError::Kind(SECURITY_SSPI::CERTSRV_E_ENCRYPTION_CERT_REQUIRED),
            SECURITY_SSPI::CERTSRV_E_UNSUPPORTED_CERT_TYPE => RawError::Kind(SECURITY_SSPI::CERTSRV_E_UNSUPPORTED_CERT_TYPE),
            SECURITY_SSPI::CERTSRV_E_NO_CERT_TYPE => RawError::Kind(SECURITY_SSPI::CERTSRV_E_NO_CERT_TYPE),
            SECURITY_SSPI::CERTSRV_E_TEMPLATE_CONFLICT => RawError::Kind(SECURITY_SSPI::CERTSRV_E_TEMPLATE_CONFLICT),
            SECURITY_SSPI::CERTSRV_E_SUBJECT_ALT_NAME_REQUIRED => RawError::Kind(SECURITY_SSPI::CERTSRV_E_SUBJECT_ALT_NAME_REQUIRED),
            SECURITY_SSPI::CERTSRV_E_ARCHIVED_KEY_REQUIRED => RawError::Kind(SECURITY_SSPI::CERTSRV_E_ARCHIVED_KEY_REQUIRED),
            SECURITY_SSPI::CERTSRV_E_SMIME_REQUIRED => RawError::Kind(SECURITY_SSPI::CERTSRV_E_SMIME_REQUIRED),
            SECURITY_SSPI::CERTSRV_E_BAD_RENEWAL_SUBJECT => RawError::Kind(SECURITY_SSPI::CERTSRV_E_BAD_RENEWAL_SUBJECT),
            SECURITY_SSPI::CERTSRV_E_BAD_TEMPLATE_VERSION => RawError::Kind(SECURITY_SSPI::CERTSRV_E_BAD_TEMPLATE_VERSION),
            SECURITY_SSPI::CERTSRV_E_TEMPLATE_POLICY_REQUIRED => RawError::Kind(SECURITY_SSPI::CERTSRV_E_TEMPLATE_POLICY_REQUIRED),
            SECURITY_SSPI::CERTSRV_E_SIGNATURE_POLICY_REQUIRED => RawError::Kind(SECURITY_SSPI::CERTSRV_E_SIGNATURE_POLICY_REQUIRED),
            SECURITY_SSPI::CERTSRV_E_SIGNATURE_COUNT => RawError::Kind(SECURITY_SSPI::CERTSRV_E_SIGNATURE_COUNT),
            SECURITY_SSPI::CERTSRV_E_SIGNATURE_REJECTED => RawError::Kind(SECURITY_SSPI::CERTSRV_E_SIGNATURE_REJECTED),
            SECURITY_SSPI::CERTSRV_E_ISSUANCE_POLICY_REQUIRED => RawError::Kind(SECURITY_SSPI::CERTSRV_E_ISSUANCE_POLICY_REQUIRED),
            SECURITY_SSPI::CERTSRV_E_SUBJECT_UPN_REQUIRED => RawError::Kind(SECURITY_SSPI::CERTSRV_E_SUBJECT_UPN_REQUIRED),
            SECURITY_SSPI::CERTSRV_E_SUBJECT_DIRECTORY_GUID_REQUIRED => RawError::Kind(SECURITY_SSPI::CERTSRV_E_SUBJECT_DIRECTORY_GUID_REQUIRED),
            SECURITY_SSPI::CERTSRV_E_SUBJECT_DNS_REQUIRED => RawError::Kind(SECURITY_SSPI::CERTSRV_E_SUBJECT_DNS_REQUIRED),
            SECURITY_SSPI::CERTSRV_E_ARCHIVED_KEY_UNEXPECTED => RawError::Kind(SECURITY_SSPI::CERTSRV_E_ARCHIVED_KEY_UNEXPECTED),
            SECURITY_SSPI::CERTSRV_E_KEY_LENGTH => RawError::Kind(SECURITY_SSPI::CERTSRV_E_KEY_LENGTH),
            SECURITY_SSPI::CERTSRV_E_SUBJECT_EMAIL_REQUIRED => RawError::Kind(SECURITY_SSPI::CERTSRV_E_SUBJECT_EMAIL_REQUIRED),
            SECURITY_SSPI::CERTSRV_E_UNKNOWN_CERT_TYPE => RawError::Kind(SECURITY_SSPI::CERTSRV_E_UNKNOWN_CERT_TYPE),
            SECURITY_SSPI::CERTSRV_E_CERT_TYPE_OVERLAP => RawError::Kind(SECURITY_SSPI::CERTSRV_E_CERT_TYPE_OVERLAP),
            SECURITY_SSPI::CERTSRV_E_TOO_MANY_SIGNATURES => RawError::Kind(SECURITY_SSPI::CERTSRV_E_TOO_MANY_SIGNATURES),
            SECURITY_SSPI::CERTSRV_E_RENEWAL_BAD_PUBLIC_KEY => RawError::Kind(SECURITY_SSPI::CERTSRV_E_RENEWAL_BAD_PUBLIC_KEY),
            SECURITY_SSPI::CERTSRV_E_INVALID_EK => RawError::Kind(SECURITY_SSPI::CERTSRV_E_INVALID_EK),
            SECURITY_SSPI::CERTSRV_E_INVALID_IDBINDING => RawError::Kind(SECURITY_SSPI::CERTSRV_E_INVALID_IDBINDING),
            SECURITY_SSPI::CERTSRV_E_INVALID_ATTESTATION => RawError::Kind(SECURITY_SSPI::CERTSRV_E_INVALID_ATTESTATION),
            SECURITY_SSPI::CERTSRV_E_KEY_ATTESTATION => RawError::Kind(SECURITY_SSPI::CERTSRV_E_KEY_ATTESTATION),
            SECURITY_SSPI::CERTSRV_E_CORRUPT_KEY_ATTESTATION => RawError::Kind(SECURITY_SSPI::CERTSRV_E_CORRUPT_KEY_ATTESTATION),
            SECURITY_SSPI::CERTSRV_E_EXPIRED_CHALLENGE => RawError::Kind(SECURITY_SSPI::CERTSRV_E_EXPIRED_CHALLENGE),
            SECURITY_SSPI::CERTSRV_E_INVALID_RESPONSE => RawError::Kind(SECURITY_SSPI::CERTSRV_E_INVALID_RESPONSE),
            SECURITY_SSPI::CERTSRV_E_INVALID_REQUESTID => RawError::Kind(SECURITY_SSPI::CERTSRV_E_INVALID_REQUESTID),
            SECURITY_SSPI::CERTSRV_E_REQUEST_PRECERTIFICATE_MISMATCH => RawError::Kind(SECURITY_SSPI::CERTSRV_E_REQUEST_PRECERTIFICATE_MISMATCH),
            SECURITY_SSPI::CERTSRV_E_PENDING_CLIENT_RESPONSE => RawError::Kind(SECURITY_SSPI::CERTSRV_E_PENDING_CLIENT_RESPONSE),
            SECURITY_SSPI::CERTSRV_E_SEC_EXT_DIRECTORY_SID_REQUIRED => RawError::Kind(SECURITY_SSPI::CERTSRV_E_SEC_EXT_DIRECTORY_SID_REQUIRED),
            SECURITY_SSPI::XENROLL_E_KEY_NOT_EXPORTABLE => RawError::Kind(SECURITY_SSPI::XENROLL_E_KEY_NOT_EXPORTABLE),
            SECURITY_SSPI::XENROLL_E_CANNOT_ADD_ROOT_CERT => RawError::Kind(SECURITY_SSPI::XENROLL_E_CANNOT_ADD_ROOT_CERT),
            SECURITY_SSPI::XENROLL_E_RESPONSE_KA_HASH_NOT_FOUND => RawError::Kind(SECURITY_SSPI::XENROLL_E_RESPONSE_KA_HASH_NOT_FOUND),
            SECURITY_SSPI::XENROLL_E_RESPONSE_UNEXPECTED_KA_HASH => RawError::Kind(SECURITY_SSPI::XENROLL_E_RESPONSE_UNEXPECTED_KA_HASH),
            SECURITY_SSPI::XENROLL_E_RESPONSE_KA_HASH_MISMATCH => RawError::Kind(SECURITY_SSPI::XENROLL_E_RESPONSE_KA_HASH_MISMATCH),
            SECURITY_SSPI::XENROLL_E_KEYSPEC_SMIME_MISMATCH => RawError::Kind(SECURITY_SSPI::XENROLL_E_KEYSPEC_SMIME_MISMATCH),
            SECURITY_SSPI::TRUST_E_SYSTEM_ERROR => RawError::Kind(SECURITY_SSPI::TRUST_E_SYSTEM_ERROR),
            SECURITY_SSPI::TRUST_E_NO_SIGNER_CERT => RawError::Kind(SECURITY_SSPI::TRUST_E_NO_SIGNER_CERT),
            SECURITY_SSPI::TRUST_E_COUNTER_SIGNER => RawError::Kind(SECURITY_SSPI::TRUST_E_COUNTER_SIGNER),
            SECURITY_SSPI::TRUST_E_CERT_SIGNATURE => RawError::Kind(SECURITY_SSPI::TRUST_E_CERT_SIGNATURE),
            SECURITY_SSPI::TRUST_E_TIME_STAMP => RawError::Kind(SECURITY_SSPI::TRUST_E_TIME_STAMP),
            SECURITY_SSPI::TRUST_E_BAD_DIGEST => RawError::Kind(SECURITY_SSPI::TRUST_E_BAD_DIGEST),
            SECURITY_SSPI::TRUST_E_MALFORMED_SIGNATURE => RawError::Kind(SECURITY_SSPI::TRUST_E_MALFORMED_SIGNATURE),
            SECURITY_SSPI::TRUST_E_BASIC_CONSTRAINTS => RawError::Kind(SECURITY_SSPI::TRUST_E_BASIC_CONSTRAINTS),
            SECURITY_SSPI::TRUST_E_FINANCIAL_CRITERIA => RawError::Kind(SECURITY_SSPI::TRUST_E_FINANCIAL_CRITERIA),
            SECURITY_SSPI::MSSIPOTF_E_OUTOFMEMRANGE => RawError::Kind(SECURITY_SSPI::MSSIPOTF_E_OUTOFMEMRANGE),
            SECURITY_SSPI::MSSIPOTF_E_CANTGETOBJECT => RawError::Kind(SECURITY_SSPI::MSSIPOTF_E_CANTGETOBJECT),
            SECURITY_SSPI::MSSIPOTF_E_NOHEADTABLE => RawError::Kind(SECURITY_SSPI::MSSIPOTF_E_NOHEADTABLE),
            SECURITY_SSPI::MSSIPOTF_E_BAD_MAGICNUMBER => RawError::Kind(SECURITY_SSPI::MSSIPOTF_E_BAD_MAGICNUMBER),
            SECURITY_SSPI::MSSIPOTF_E_BAD_OFFSET_TABLE => RawError::Kind(SECURITY_SSPI::MSSIPOTF_E_BAD_OFFSET_TABLE),
            SECURITY_SSPI::MSSIPOTF_E_TABLE_TAGORDER => RawError::Kind(SECURITY_SSPI::MSSIPOTF_E_TABLE_TAGORDER),
            SECURITY_SSPI::MSSIPOTF_E_TABLE_LONGWORD => RawError::Kind(SECURITY_SSPI::MSSIPOTF_E_TABLE_LONGWORD),
            SECURITY_SSPI::MSSIPOTF_E_BAD_FIRST_TABLE_PLACEMENT => RawError::Kind(SECURITY_SSPI::MSSIPOTF_E_BAD_FIRST_TABLE_PLACEMENT),
            SECURITY_SSPI::MSSIPOTF_E_TABLES_OVERLAP => RawError::Kind(SECURITY_SSPI::MSSIPOTF_E_TABLES_OVERLAP),
            SECURITY_SSPI::MSSIPOTF_E_TABLE_PADBYTES => RawError::Kind(SECURITY_SSPI::MSSIPOTF_E_TABLE_PADBYTES),
            SECURITY_SSPI::MSSIPOTF_E_FILETOOSMALL => RawError::Kind(SECURITY_SSPI::MSSIPOTF_E_FILETOOSMALL),
            SECURITY_SSPI::MSSIPOTF_E_TABLE_CHECKSUM => RawError::Kind(SECURITY_SSPI::MSSIPOTF_E_TABLE_CHECKSUM),
            SECURITY_SSPI::MSSIPOTF_E_FILE_CHECKSUM => RawError::Kind(SECURITY_SSPI::MSSIPOTF_E_FILE_CHECKSUM),
            SECURITY_SSPI::MSSIPOTF_E_FAILED_POLICY => RawError::Kind(SECURITY_SSPI::MSSIPOTF_E_FAILED_POLICY),
            SECURITY_SSPI::MSSIPOTF_E_FAILED_HINTS_CHECK => RawError::Kind(SECURITY_SSPI::MSSIPOTF_E_FAILED_HINTS_CHECK),
            SECURITY_SSPI::MSSIPOTF_E_NOT_OPENTYPE => RawError::Kind(SECURITY_SSPI::MSSIPOTF_E_NOT_OPENTYPE),
            SECURITY_SSPI::MSSIPOTF_E_FILE => RawError::Kind(SECURITY_SSPI::MSSIPOTF_E_FILE),
            SECURITY_SSPI::MSSIPOTF_E_CRYPT => RawError::Kind(SECURITY_SSPI::MSSIPOTF_E_CRYPT),
            SECURITY_SSPI::MSSIPOTF_E_BADVERSION => RawError::Kind(SECURITY_SSPI::MSSIPOTF_E_BADVERSION),
            SECURITY_SSPI::MSSIPOTF_E_DSIG_STRUCTURE => RawError::Kind(SECURITY_SSPI::MSSIPOTF_E_DSIG_STRUCTURE),
            SECURITY_SSPI::MSSIPOTF_E_PCONST_CHECK => RawError::Kind(SECURITY_SSPI::MSSIPOTF_E_PCONST_CHECK),
            SECURITY_SSPI::MSSIPOTF_E_STRUCTURE => RawError::Kind(SECURITY_SSPI::MSSIPOTF_E_STRUCTURE),
            SECURITY_SSPI::ERROR_CRED_REQUIRES_CONFIRMATION => RawError::Kind(SECURITY_SSPI::ERROR_CRED_REQUIRES_CONFIRMATION),
            SECURITY_SSPI::ERROR_AUDITING_DISABLED => RawError::Kind(SECURITY_SSPI::ERROR_AUDITING_DISABLED),
            SECURITY_SSPI::ERROR_ALL_SIDS_FILTERED => RawError::Kind(SECURITY_SSPI::ERROR_ALL_SIDS_FILTERED),
            SECURITY_SSPI::ERROR_BIZRULES_NOT_ENABLED => RawError::Kind(SECURITY_SSPI::ERROR_BIZRULES_NOT_ENABLED),
        }
    }

    pub fn description(&self) -> &'static str {
        return match self {
            SECURITY_SSPI::SEC_I_CONTINUE_NEEDED => "The function completed successfully, but must be called again to complete the context",
            SECURITY_SSPI::SEC_I_COMPLETE_NEEDED => "The function completed successfully, but CompleteToken must be called",
            SECURITY_SSPI::SEC_I_COMPLETE_AND_CONTINUE => "The function completed successfully, but both CompleteToken and this function must be called to complete the context",
            SECURITY_SSPI::SEC_I_LOCAL_LOGON => "The logon was completed, but no network authority was available. The logon was made using locally known information",
            SECURITY_SSPI::SEC_I_GENERIC_EXTENSION_RECEIVED => "Schannel has received a TLS extension the SSPI caller subscribed to.",
            SECURITY_SSPI::SEC_I_CONTEXT_EXPIRED => "The context has expired and can no longer be used.",
            SECURITY_SSPI::SEC_I_INCOMPLETE_CREDENTIALS => "The credentials supplied were not complete, and could not be verified. Additional information can be returned from the context.",
            SECURITY_SSPI::SEC_I_RENEGOTIATE => "The context data must be renegotiated with the peer.",
            SECURITY_SSPI::SEC_I_NO_LSA_CONTEXT => "There is no LSA mode context associated with this context.",
            SECURITY_SSPI::SEC_I_SIGNATURE_NEEDED => "A signature operation must be performed before the user can authenticate.",
            SECURITY_SSPI::SEC_I_NO_RENEGOTIATION => "The recipient rejected the renegotiation request.",
            SECURITY_SSPI::SEC_I_MESSAGE_FRAGMENT => "The returned buffer is only a fragment of the message. More fragments need to be returned.",
            SECURITY_SSPI::SEC_I_CONTINUE_NEEDED_MESSAGE_OK => "The function completed successfully, but must be called again to complete the context. Early start can be used.",
            SECURITY_SSPI::SEC_I_ASYNC_CALL_PENDING => "An asynchronous SSPI routine has been called and the work is pending completion.",
            SECURITY_SSPI::CRYPT_I_NEW_PROTECTION_REQUIRED => "The protected data needs to be re-protected.",
            SECURITY_SSPI::NTE_BAD_UID => "Bad UID.",
            SECURITY_SSPI::NTE_BAD_HASH => "Bad Hash.",
            SECURITY_SSPI::NTE_BAD_KEY => "Bad Key.",
            SECURITY_SSPI::NTE_BAD_LEN => "Bad Length.",
            SECURITY_SSPI::NTE_BAD_DATA => "Bad Data.",
            SECURITY_SSPI::NTE_BAD_SIGNATURE => "Invalid Signature.",
            SECURITY_SSPI::NTE_BAD_VER => "Bad Version of provider.",
            SECURITY_SSPI::NTE_BAD_ALGID => "Invalid algorithm specified.",
            SECURITY_SSPI::NTE_BAD_FLAGS => "Invalid flags specified.",
            SECURITY_SSPI::NTE_BAD_TYPE => "Invalid type specified.",
            SECURITY_SSPI::NTE_BAD_KEY_STATE => "Key not valid for use in specified state.",
            SECURITY_SSPI::NTE_BAD_HASH_STATE => "Hash not valid for use in specified state.",
            SECURITY_SSPI::NTE_NO_KEY => "Key does not exist.",
            SECURITY_SSPI::NTE_NO_MEMORY => "Insufficient memory available for the operation.",
            SECURITY_SSPI::NTE_EXISTS => "Object already exists.",
            SECURITY_SSPI::NTE_PERM => "Access denied.",
            SECURITY_SSPI::NTE_NOT_FOUND => "Object was not found.",
            SECURITY_SSPI::NTE_DOUBLE_ENCRYPT => "Data already encrypted.",
            SECURITY_SSPI::NTE_BAD_PROVIDER => "Invalid provider specified.",
            SECURITY_SSPI::NTE_BAD_PROV_TYPE => "Invalid provider type specified.",
            SECURITY_SSPI::NTE_BAD_PUBLIC_KEY => "Provider's public key is invalid.",
            SECURITY_SSPI::NTE_BAD_KEYSET => "Keyset does not exist",
            SECURITY_SSPI::NTE_PROV_TYPE_NOT_DEF => "Provider type not defined.",
            SECURITY_SSPI::NTE_PROV_TYPE_ENTRY_BAD => "Provider type as registered is invalid.",
            SECURITY_SSPI::NTE_KEYSET_NOT_DEF => "The keyset is not defined.",
            SECURITY_SSPI::NTE_KEYSET_ENTRY_BAD => "Keyset as registered is invalid.",
            SECURITY_SSPI::NTE_PROV_TYPE_NO_MATCH => "Provider type does not match registered value.",
            SECURITY_SSPI::NTE_SIGNATURE_FILE_BAD => "The digital signature file is corrupt.",
            SECURITY_SSPI::NTE_PROVIDER_DLL_FAIL => "Provider DLL failed to initialize correctly.",
            SECURITY_SSPI::NTE_PROV_DLL_NOT_FOUND => "Provider DLL could not be found.",
            SECURITY_SSPI::NTE_BAD_KEYSET_PARAM => "The Keyset parameter is invalid.",
            SECURITY_SSPI::NTE_FAIL => "An internal error occurred.",
            SECURITY_SSPI::NTE_SYS_ERR => "A base error occurred.",
            SECURITY_SSPI::NTE_SILENT_CONTEXT => "Provider could not perform the action since the context was acquired as silent.",
            SECURITY_SSPI::NTE_TOKEN_KEYSET_STORAGE_FULL => "The security token does not have storage space available for an additional container.",
            SECURITY_SSPI::NTE_TEMPORARY_PROFILE => "The profile for the user is a temporary profile.",
            SECURITY_SSPI::NTE_FIXEDPARAMETER => "The key parameters could not be set because the CSP uses fixed parameters.",
            SECURITY_SSPI::NTE_INVALID_HANDLE => "The supplied handle is invalid.",
            SECURITY_SSPI::NTE_INVALID_PARAMETER => "The parameter is incorrect.",
            SECURITY_SSPI::NTE_BUFFER_TOO_SMALL => "The buffer supplied to a function was too small.",
            SECURITY_SSPI::NTE_NOT_SUPPORTED => "The requested operation is not supported.",
            SECURITY_SSPI::NTE_NO_MORE_ITEMS => "No more data is available.",
            SECURITY_SSPI::NTE_BUFFERS_OVERLAP => "The supplied buffers overlap incorrectly.",
            SECURITY_SSPI::NTE_DECRYPTION_FAILURE => "The specified data could not be decrypted.",
            SECURITY_SSPI::NTE_INTERNAL_ERROR => "An internal consistency check failed.",
            SECURITY_SSPI::NTE_UI_REQUIRED => "This operation requires input from the user.",
            SECURITY_SSPI::NTE_HMAC_NOT_SUPPORTED => "The cryptographic provider does not support HMAC.",
            SECURITY_SSPI::NTE_DEVICE_NOT_READY => "The device that is required by this cryptographic provider is not ready for use.",
            SECURITY_SSPI::NTE_AUTHENTICATION_IGNORED => "The dictionary attack mitigation is triggered and the provided authorization was ignored by the provider.",
            SECURITY_SSPI::NTE_VALIDATION_FAILED => "The validation of the provided data failed the integrity or signature validation.",
            SECURITY_SSPI::NTE_INCORRECT_PASSWORD => "Incorrect password.",
            SECURITY_SSPI::NTE_ENCRYPTION_FAILURE => "Encryption failed.",
            SECURITY_SSPI::NTE_DEVICE_NOT_FOUND => "The device that is required by this cryptographic provider is not found on this platform.",
            SECURITY_SSPI::NTE_USER_CANCELLED => "The action was cancelled by the user.",
            SECURITY_SSPI::NTE_PASSWORD_CHANGE_REQUIRED => "The password is no longer valid and must be changed.",
            SECURITY_SSPI::NTE_NOT_ACTIVE_CONSOLE => "The operation cannot be completed from Terminal Server client sessions.",
            SECURITY_SSPI::SEC_E_INSUFFICIENT_MEMORY => "Not enough memory is available to complete this request",
            SECURITY_SSPI::SEC_E_INVALID_HANDLE => "The handle specified is invalid",
            SECURITY_SSPI::SEC_E_UNSUPPORTED_FUNCTION => "The function requested is not supported",
            SECURITY_SSPI::SEC_E_TARGET_UNKNOWN => "The specified target is unknown or unreachable",
            SECURITY_SSPI::SEC_E_INTERNAL_ERROR => "The Local Security Authority cannot be contacted",
            SECURITY_SSPI::SEC_E_SECPKG_NOT_FOUND => "The requested security package does not exist",
            SECURITY_SSPI::SEC_E_NOT_OWNER => "The caller is not the owner of the desired credentials",
            SECURITY_SSPI::SEC_E_CANNOT_INSTALL => "The security package failed to initialize, and cannot be installed",
            SECURITY_SSPI::SEC_E_INVALID_TOKEN => "The token supplied to the function is invalid",
            SECURITY_SSPI::SEC_E_CANNOT_PACK => "The security package is not able to marshall the logon buffer, so the logon attempt has failed",
            SECURITY_SSPI::SEC_E_QOP_NOT_SUPPORTED => "The per-message Quality of Protection is not supported by the security package",
            SECURITY_SSPI::SEC_E_NO_IMPERSONATION => "The security context does not allow impersonation of the client",
            SECURITY_SSPI::SEC_E_LOGON_DENIED => "The logon attempt failed",
            SECURITY_SSPI::SEC_E_UNKNOWN_CREDENTIALS => "The credentials supplied to the package were not recognized",
            SECURITY_SSPI::SEC_E_NO_CREDENTIALS => "No credentials are available in the security package",
            SECURITY_SSPI::SEC_E_MESSAGE_ALTERED => "The message or signature supplied for verification has been altered",
            SECURITY_SSPI::SEC_E_OUT_OF_SEQUENCE => "The message supplied for verification is out of sequence",
            SECURITY_SSPI::SEC_E_NO_AUTHENTICATING_AUTHORITY => "No authority could be contacted for authentication.",
            SECURITY_SSPI::SEC_E_BAD_PKGID => "The requested security package does not exist",
            SECURITY_SSPI::SEC_E_CONTEXT_EXPIRED => "The context has expired and can no longer be used.",
            SECURITY_SSPI::SEC_E_INCOMPLETE_MESSAGE => "The supplied message is incomplete. The signature was not verified.",
            SECURITY_SSPI::SEC_E_INCOMPLETE_CREDENTIALS => "The credentials supplied were not complete, and could not be verified. The context could not be initialized.",
            SECURITY_SSPI::SEC_E_BUFFER_TOO_SMALL => "The buffers supplied to a function was too small.",
            SECURITY_SSPI::SEC_E_WRONG_PRINCIPAL => "The target principal name is incorrect.",
            SECURITY_SSPI::SEC_E_TIME_SKEW => "The clocks on the client and server machines are skewed.",
            SECURITY_SSPI::SEC_E_UNTRUSTED_ROOT => "The certificate chain was issued by an authority that is not trusted.",
            SECURITY_SSPI::SEC_E_ILLEGAL_MESSAGE => "The message received was unexpected or badly formatted.",
            SECURITY_SSPI::SEC_E_CERT_UNKNOWN => "An unknown error occurred while processing the certificate.",
            SECURITY_SSPI::SEC_E_CERT_EXPIRED => "The received certificate has expired.",
            SECURITY_SSPI::SEC_E_ENCRYPT_FAILURE => "The specified data could not be encrypted.",
            SECURITY_SSPI::SEC_E_DECRYPT_FAILURE => "The specified data could not be decrypted.",
            SECURITY_SSPI::SEC_E_ALGORITHM_MISMATCH => "The client and server cannot communicate, because they do not possess a common algorithm.",
            SECURITY_SSPI::SEC_E_SECURITY_QOS_FAILED => "The security context could not be established due to a failure in the requested quality of service (e.g. mutual authentication or delegation).",
            SECURITY_SSPI::SEC_E_UNFINISHED_CONTEXT_DELETED => "A security context was deleted before the context was completed. This is considered a logon failure.",
            SECURITY_SSPI::SEC_E_NO_TGT_REPLY => "The client is trying to negotiate a context and the server requires user-to-user but didn't send a TGT reply.",
            SECURITY_SSPI::SEC_E_NO_IP_ADDRESSES => "Unable to accomplish the requested task because the local machine does not have any IP addresses.",
            SECURITY_SSPI::SEC_E_WRONG_CREDENTIAL_HANDLE => "The supplied credential handle does not match the credential associated with the security context.",
            SECURITY_SSPI::SEC_E_CRYPTO_SYSTEM_INVALID => "The crypto system or checksum function is invalid because a required function is unavailable.",
            SECURITY_SSPI::SEC_E_MAX_REFERRALS_EXCEEDED => "The number of maximum ticket referrals has been exceeded.",
            SECURITY_SSPI::SEC_E_MUST_BE_KDC => "The local machine must be a Kerberos KDC (domain controller) and it is not.",
            SECURITY_SSPI::SEC_E_STRONG_CRYPTO_NOT_SUPPORTED => "The other end of the security negotiation is requires strong crypto but it is not supported on the local machine.",
            SECURITY_SSPI::SEC_E_TOO_MANY_PRINCIPALS => "The KDC reply contained more than one principal name.",
            SECURITY_SSPI::SEC_E_NO_PA_DATA => "Expected to find PA data for a hint of what etype to use, but it was not found.",
            SECURITY_SSPI::SEC_E_PKINIT_NAME_MISMATCH => "The client certificate does not contain a valid UPN, or does not match the client name in the logon request. Please contact your administrator.",
            SECURITY_SSPI::SEC_E_SMARTCARD_LOGON_REQUIRED => "Smartcard logon is required and was not used.",
            SECURITY_SSPI::SEC_E_SHUTDOWN_IN_PROGRESS => "A system shutdown is in progress.",
            SECURITY_SSPI::SEC_E_KDC_INVALID_REQUEST => "An invalid request was sent to the KDC.",
            SECURITY_SSPI::SEC_E_KDC_UNABLE_TO_REFER => "The KDC was unable to generate a referral for the service requested.",
            SECURITY_SSPI::SEC_E_KDC_UNKNOWN_ETYPE => "The encryption type requested is not supported by the KDC.",
            SECURITY_SSPI::SEC_E_UNSUPPORTED_PREAUTH => "An unsupported preauthentication mechanism was presented to the Kerberos package.",
            SECURITY_SSPI::SEC_E_DELEGATION_REQUIRED => "The requested operation cannot be completed. The computer must be trusted for delegation and the current user account must be configured to allow delegation.",
            SECURITY_SSPI::SEC_E_BAD_BINDINGS => "Client's supplied SSPI channel bindings were incorrect.",
            SECURITY_SSPI::SEC_E_MULTIPLE_ACCOUNTS => "The received certificate was mapped to multiple accounts.",
            SECURITY_SSPI::SEC_E_NO_KERB_KEY => "SEC_E_NO_KERB_KEY",
            SECURITY_SSPI::SEC_E_CERT_WRONG_USAGE => "The certificate is not valid for the requested usage.",
            SECURITY_SSPI::SEC_E_DOWNGRADE_DETECTED => "The system cannot contact a domain controller to service the authentication request. Please try again later.",
            SECURITY_SSPI::SEC_E_SMARTCARD_CERT_REVOKED => "The smartcard certificate used for authentication has been revoked. Please contact your system administrator. There may be additional information in the event log.",
            SECURITY_SSPI::SEC_E_ISSUING_CA_UNTRUSTED => "An untrusted certificate authority was detected while processing the smartcard certificate used for authentication. Please contact your system administrator.",
            SECURITY_SSPI::SEC_E_REVOCATION_OFFLINE_C => "The revocation status of the smartcard certificate used for authentication could not be determined. Please contact your system administrator.",
            SECURITY_SSPI::SEC_E_PKINIT_CLIENT_FAILURE => "The smartcard certificate used for authentication was not trusted. Please contact your system administrator.",
            SECURITY_SSPI::SEC_E_SMARTCARD_CERT_EXPIRED => "The smartcard certificate used for authentication has expired. Please contact your system administrator.",
            SECURITY_SSPI::SEC_E_NO_S4U_PROT_SUPPORT => "The Kerberos subsystem encountered an error. A service for user protocol request was made against a domain controller which does not support service for user.",
            SECURITY_SSPI::SEC_E_CROSSREALM_DELEGATION_FAILURE => "An attempt was made by this server to make a Kerberos constrained delegation request for a target outside of the server's realm. This is not supported, and indicates a misconfiguration on this server's allowed to delegate to list. Please contact your administrator.",
            SECURITY_SSPI::SEC_E_REVOCATION_OFFLINE_KDC => "The revocation status of the domain controller certificate used for smartcard authentication could not be determined. There is additional information in the system event log. Please contact your system administrator.",
            SECURITY_SSPI::SEC_E_ISSUING_CA_UNTRUSTED_KDC => "An untrusted certificate authority was detected while processing the domain controller certificate used for authentication. There is additional information in the system event log. Please contact your system administrator.",
            SECURITY_SSPI::SEC_E_KDC_CERT_EXPIRED => "The domain controller certificate used for smartcard logon has expired. Please contact your system administrator with the contents of your system event log.",
            SECURITY_SSPI::SEC_E_KDC_CERT_REVOKED => "The domain controller certificate used for smartcard logon has been revoked. Please contact your system administrator with the contents of your system event log.",
            SECURITY_SSPI::SEC_E_INVALID_PARAMETER => "One or more of the parameters passed to the function was invalid.",
            SECURITY_SSPI::SEC_E_DELEGATION_POLICY => "Client policy does not allow credential delegation to target server.",
            SECURITY_SSPI::SEC_E_POLICY_NLTM_ONLY => "Client policy does not allow credential delegation to target server with NLTM only authentication.",
            SECURITY_SSPI::SEC_E_NO_CONTEXT => "The required security context does not exist.",
            SECURITY_SSPI::SEC_E_PKU2U_CERT_FAILURE => "The PKU2U protocol encountered an error while attempting to utilize the associated certificates.",
            SECURITY_SSPI::SEC_E_MUTUAL_AUTH_FAILED => "The identity of the server computer could not be verified.",
            SECURITY_SSPI::SEC_E_ONLY_HTTPS_ALLOWED => "Only https scheme is allowed.",
            SECURITY_SSPI::SEC_E_APPLICATION_PROTOCOL_MISMATCH => "No common application protocol exists between the client and the server. Application protocol negotiation failed.",
            SECURITY_SSPI::SEC_E_INVALID_UPN_NAME => "You can't sign in with a user ID in this format. Try using your email address instead.",
            SECURITY_SSPI::SEC_E_EXT_BUFFER_TOO_SMALL => "The buffer supplied by the SSPI caller to receive generic extensions is too small.",
            SECURITY_SSPI::SEC_E_INSUFFICIENT_BUFFERS => "Not enough secbuffers were supplied to generate a token.",
            SECURITY_SSPI::CRYPT_E_MSG_ERROR => "An error occurred while performing an operation on a cryptographic message.",
            SECURITY_SSPI::CRYPT_E_UNKNOWN_ALGO => "Unknown cryptographic algorithm.",
            SECURITY_SSPI::CRYPT_E_OID_FORMAT => "The object identifier is poorly formatted.",
            SECURITY_SSPI::CRYPT_E_INVALID_MSG_TYPE => "Invalid cryptographic message type.",
            SECURITY_SSPI::CRYPT_E_UNEXPECTED_ENCODING => "Unexpected cryptographic message encoding.",
            SECURITY_SSPI::CRYPT_E_AUTH_ATTR_MISSING => "The cryptographic message does not contain an expected authenticated attribute.",
            SECURITY_SSPI::CRYPT_E_HASH_VALUE => "The hash value is not correct.",
            SECURITY_SSPI::CRYPT_E_INVALID_INDEX => "The index value is not valid.",
            SECURITY_SSPI::CRYPT_E_ALREADY_DECRYPTED => "The content of the cryptographic message has already been decrypted.",
            SECURITY_SSPI::CRYPT_E_NOT_DECRYPTED => "The content of the cryptographic message has not been decrypted yet.",
            SECURITY_SSPI::CRYPT_E_RECIPIENT_NOT_FOUND => "The enveloped-data message does not contain the specified recipient.",
            SECURITY_SSPI::CRYPT_E_CONTROL_TYPE => "Invalid control type.",
            SECURITY_SSPI::CRYPT_E_ISSUER_SERIALNUMBER => "Invalid issuer and/or serial number.",
            SECURITY_SSPI::CRYPT_E_SIGNER_NOT_FOUND => "Cannot find the original signer.",
            SECURITY_SSPI::CRYPT_E_ATTRIBUTES_MISSING => "The cryptographic message does not contain all of the requested attributes.",
            SECURITY_SSPI::CRYPT_E_STREAM_MSG_NOT_READY => "The streamed cryptographic message is not ready to return data.",
            SECURITY_SSPI::CRYPT_E_STREAM_INSUFFICIENT_DATA => "The streamed cryptographic message requires more data to complete the decode operation.",
            SECURITY_SSPI::CRYPT_E_BAD_LEN => "The length specified for the output data was insufficient.",
            SECURITY_SSPI::CRYPT_E_BAD_ENCODE => "An error occurred during encode or decode operation.",
            SECURITY_SSPI::CRYPT_E_FILE_ERROR => "An error occurred while reading or writing to a file.",
            SECURITY_SSPI::CRYPT_E_NOT_FOUND => "Cannot find object or property.",
            SECURITY_SSPI::CRYPT_E_EXISTS => "The object or property already exists.",
            SECURITY_SSPI::CRYPT_E_NO_PROVIDER => "No provider was specified for the store or object.",
            SECURITY_SSPI::CRYPT_E_SELF_SIGNED => "The specified certificate is self signed.",
            SECURITY_SSPI::CRYPT_E_DELETED_PREV => "The previous certificate or CRL context was deleted.",
            SECURITY_SSPI::CRYPT_E_NO_MATCH => "Cannot find the requested object.",
            SECURITY_SSPI::CRYPT_E_UNEXPECTED_MSG_TYPE => "The certificate does not have a property that references a private key.",
            SECURITY_SSPI::CRYPT_E_NO_KEY_PROPERTY => "Cannot find the certificate and private key for decryption.",
            SECURITY_SSPI::CRYPT_E_NO_DECRYPT_CERT => "Cannot find the certificate and private key to use for decryption.",
            SECURITY_SSPI::CRYPT_E_BAD_MSG => "Not a cryptographic message or the cryptographic message is not formatted correctly.",
            SECURITY_SSPI::CRYPT_E_NO_SIGNER => "The signed cryptographic message does not have a signer for the specified signer index.",
            SECURITY_SSPI::CRYPT_E_PENDING_CLOSE => "Final closure is pending until additional frees or closes.",
            SECURITY_SSPI::CRYPT_E_REVOKED => "The certificate is revoked.",
            SECURITY_SSPI::CRYPT_E_NO_REVOCATION_DLL => "No Dll or exported function was found to verify revocation.",
            SECURITY_SSPI::CRYPT_E_NO_REVOCATION_CHECK => "The revocation function was unable to check revocation for the certificate.",
            SECURITY_SSPI::CRYPT_E_REVOCATION_OFFLINE => "The revocation function was unable to check revocation because the revocation server was offline.",
            SECURITY_SSPI::CRYPT_E_NOT_IN_REVOCATION_DATABASE => "The certificate is not in the revocation server's database.",
            SECURITY_SSPI::CRYPT_E_INVALID_NUMERIC_STRING => "The string contains a non-numeric character.",
            SECURITY_SSPI::CRYPT_E_INVALID_PRINTABLE_STRING => "The string contains a non-printable character.",
            SECURITY_SSPI::CRYPT_E_INVALID_IA5_STRING => "The string contains a character not in the 7 bit ASCII character set.",
            SECURITY_SSPI::CRYPT_E_INVALID_X500_STRING => "The string contains an invalid X500 name attribute key, oid, value or delimiter.",
            SECURITY_SSPI::CRYPT_E_NOT_CHAR_STRING => "The dwValueType for the CERT_NAME_VALUE is not one of the character strings. Most likely it is either a CERT_RDN_ENCODED_BLOB or CERT_RDN_OCTET_STRING.",
            SECURITY_SSPI::CRYPT_E_FILERESIZED => "The Put operation cannot continue. The file needs to be resized. However, there is already a signature present. A complete signing operation must be done.",
            SECURITY_SSPI::CRYPT_E_SECURITY_SETTINGS => "The cryptographic operation failed due to a local security option setting.",
            SECURITY_SSPI::CRYPT_E_NO_VERIFY_USAGE_DLL => "No DLL or exported function was found to verify subject usage.",
            SECURITY_SSPI::CRYPT_E_NO_VERIFY_USAGE_CHECK => "The called function was unable to do a usage check on the subject.",
            SECURITY_SSPI::CRYPT_E_VERIFY_USAGE_OFFLINE => "Since the server was offline, the called function was unable to complete the usage check.",
            SECURITY_SSPI::CRYPT_E_NOT_IN_CTL => "The subject was not found in a Certificate Trust List (CTL).",
            SECURITY_SSPI::CRYPT_E_NO_TRUSTED_SIGNER => "None of the signers of the cryptographic message or certificate trust list is trusted.",
            SECURITY_SSPI::CRYPT_E_MISSING_PUBKEY_PARA => "The public key's algorithm parameters are missing.",
            SECURITY_SSPI::CRYPT_E_OBJECT_LOCATOR_OBJECT_NOT_FOUND => "An object could not be located using the object locator infrastructure with the given name.",
            SECURITY_SSPI::CRYPT_E_OSS_ERROR => "OSS Certificate encode/decode error code base",
            SECURITY_SSPI::OSS_MORE_BUF => "OSS ASN.1 Error: Output Buffer is too small.",
            SECURITY_SSPI::OSS_NEGATIVE_UINTEGER => "OSS ASN.1 Error: Signed integer is encoded as a unsigned integer.",
            SECURITY_SSPI::OSS_PDU_RANGE => "OSS ASN.1 Error: Unknown ASN.1 data type.",
            SECURITY_SSPI::OSS_MORE_INPUT => "OSS ASN.1 Error: Output buffer is too small, the decoded data has been truncated.",
            SECURITY_SSPI::OSS_DATA_ERROR => "OSS ASN.1 Error: Invalid data.",
            SECURITY_SSPI::OSS_BAD_ARG => "OSS ASN.1 Error: Invalid argument.",
            SECURITY_SSPI::OSS_BAD_VERSION => "OSS ASN.1 Error: Encode/Decode version mismatch.",
            SECURITY_SSPI::OSS_OUT_MEMORY => "OSS ASN.1 Error: Out of memory.",
            SECURITY_SSPI::OSS_PDU_MISMATCH => "OSS ASN.1 Error: Encode/Decode Error.",
            SECURITY_SSPI::OSS_LIMITED => "OSS ASN.1 Error: Internal Error.",
            SECURITY_SSPI::OSS_BAD_PTR => "OSS ASN.1 Error: Invalid data.",
            SECURITY_SSPI::OSS_BAD_TIME => "OSS ASN.1 Error: Invalid data.",
            SECURITY_SSPI::OSS_INDEFINITE_NOT_SUPPORTED => "OSS ASN.1 Error: Unsupported BER indefinite-length encoding.",
            SECURITY_SSPI::OSS_MEM_ERROR => "OSS ASN.1 Error: Access violation.",
            SECURITY_SSPI::OSS_BAD_TABLE => "OSS ASN.1 Error: Invalid data.",
            SECURITY_SSPI::OSS_TOO_LONG => "OSS ASN.1 Error: Invalid data.",
            SECURITY_SSPI::OSS_CONSTRAINT_VIOLATED => "OSS ASN.1 Error: Invalid data.",
            SECURITY_SSPI::OSS_FATAL_ERROR => "OSS ASN.1 Error: Internal Error.",
            SECURITY_SSPI::OSS_ACCESS_SERIALIZATION_ERROR => "OSS ASN.1 Error: Multi-threading conflict.",
            SECURITY_SSPI::OSS_NULL_TBL => "OSS ASN.1 Error: Invalid data.",
            SECURITY_SSPI::OSS_NULL_FCN => "OSS ASN.1 Error: Invalid data.",
            SECURITY_SSPI::OSS_BAD_ENCRULES => "OSS ASN.1 Error: Invalid data.",
            SECURITY_SSPI::OSS_UNAVAIL_ENCRULES => "OSS ASN.1 Error: Encode/Decode function not implemented.",
            SECURITY_SSPI::OSS_CANT_OPEN_TRACE_WINDOW => "OSS ASN.1 Error: Trace file error.",
            SECURITY_SSPI::OSS_UNIMPLEMENTED => "OSS ASN.1 Error: Function not implemented.",
            SECURITY_SSPI::OSS_OID_DLL_NOT_LINKED => "OSS ASN.1 Error: Program link error.",
            SECURITY_SSPI::OSS_CANT_OPEN_TRACE_FILE => "OSS ASN.1 Error: Trace file error.",
            SECURITY_SSPI::OSS_TRACE_FILE_ALREADY_OPEN => "OSS ASN.1 Error: Trace file error.",
            SECURITY_SSPI::OSS_TABLE_MISMATCH => "OSS ASN.1 Error: Invalid data.",
            SECURITY_SSPI::OSS_TYPE_NOT_SUPPORTED => "OSS ASN.1 Error: Invalid data.",
            SECURITY_SSPI::OSS_REAL_DLL_NOT_LINKED => "OSS ASN.1 Error: Program link error.",
            SECURITY_SSPI::OSS_REAL_CODE_NOT_LINKED => "OSS ASN.1 Error: Program link error.",
            SECURITY_SSPI::OSS_OUT_OF_RANGE => "OSS ASN.1 Error: Program link error.",
            SECURITY_SSPI::OSS_COPIER_DLL_NOT_LINKED => "OSS ASN.1 Error: Program link error.",
            SECURITY_SSPI::OSS_CONSTRAINT_DLL_NOT_LINKED => "OSS ASN.1 Error: Program link error.",
            SECURITY_SSPI::OSS_COMPARATOR_DLL_NOT_LINKED => "OSS ASN.1 Error: Program link error.",
            SECURITY_SSPI::OSS_COMPARATOR_CODE_NOT_LINKED => "OSS ASN.1 Error: Program link error.",
            SECURITY_SSPI::OSS_MEM_MGR_DLL_NOT_LINKED => "OSS ASN.1 Error: Program link error.",
            SECURITY_SSPI::OSS_PDV_DLL_NOT_LINKED => "OSS ASN.1 Error: Program link error.",
            SECURITY_SSPI::OSS_PDV_CODE_NOT_LINKED => "OSS ASN.1 Error: Program link error.",
            SECURITY_SSPI::OSS_API_DLL_NOT_LINKED => "OSS ASN.1 Error: Program link error.",
            SECURITY_SSPI::OSS_BERDER_DLL_NOT_LINKED => "OSS ASN.1 Error: Program link error.",
            SECURITY_SSPI::OSS_PER_DLL_NOT_LINKED => "OSS ASN.1 Error: Program link error.",
            SECURITY_SSPI::OSS_OPEN_TYPE_ERROR => "OSS ASN.1 Error: Program link error.",
            SECURITY_SSPI::OSS_MUTEX_NOT_CREATED => "OSS ASN.1 Error: System resource error.",
            SECURITY_SSPI::OSS_CANT_CLOSE_TRACE_FILE => "OSS ASN.1 Error: Trace file error.",
            SECURITY_SSPI::CRYPT_E_ASN1_ERROR => "ASN1 Certificate encode/decode error code base. The ASN1 error values are offset by CRYPT_E_ASN1_ERROR.",
            SECURITY_SSPI::CRYPT_E_ASN1_INTERNAL => "ASN1 internal encode or decode error.",
            SECURITY_SSPI::CRYPT_E_ASN1_EOD => "ASN1 unexpected end of data.",
            SECURITY_SSPI::CRYPT_E_ASN1_CORRUPT => "ASN1 corrupted data.",
            SECURITY_SSPI::CRYPT_E_ASN1_LARGE => "ASN1 value too large.",
            SECURITY_SSPI::CRYPT_E_ASN1_CONSTRAINT => "ASN1 constraint violated.",
            SECURITY_SSPI::CRYPT_E_ASN1_MEMORY => "ASN1 out of memory.",
            SECURITY_SSPI::CRYPT_E_ASN1_OVERFLOW => "ASN1 buffer overflow.",
            SECURITY_SSPI::CRYPT_E_ASN1_BADPDU => "ASN1 function not supported for this PDU.",
            SECURITY_SSPI::CRYPT_E_ASN1_BADARGS => "ASN1 bad arguments to function call.",
            SECURITY_SSPI::CRYPT_E_ASN1_BADREAL => "ASN1 bad real value.",
            SECURITY_SSPI::CRYPT_E_ASN1_BADTAG => "ASN1 bad tag value met.",
            SECURITY_SSPI::CRYPT_E_ASN1_CHOICE => "ASN1 bad choice value.",
            SECURITY_SSPI::CRYPT_E_ASN1_RULE => "ASN1 bad encoding rule.",
            SECURITY_SSPI::CRYPT_E_ASN1_UTF8 => "ASN1 bad unicode (UTF8).",
            SECURITY_SSPI::CRYPT_E_ASN1_PDU_TYPE => "ASN1 bad PDU type.",
            SECURITY_SSPI::CRYPT_E_ASN1_NYI => "ASN1 not yet implemented.",
            SECURITY_SSPI::CRYPT_E_ASN1_EXTENDED => "ASN1 skipped unknown extension(s).",
            SECURITY_SSPI::CRYPT_E_ASN1_NOEOD => "ASN1 end of data expected",
            SECURITY_SSPI::CERTSRV_E_BAD_REQUESTSUBJECT => "The request subject name is invalid or too long.",
            SECURITY_SSPI::CERTSRV_E_NO_REQUEST => "The request does not exist.",
            SECURITY_SSPI::CERTSRV_E_BAD_REQUESTSTATUS => "The request's current status does not allow this operation.",
            SECURITY_SSPI::CERTSRV_E_PROPERTY_EMPTY => "The requested property value is empty.",
            SECURITY_SSPI::CERTSRV_E_INVALID_CA_CERTIFICATE => "The certification authority's certificate contains invalid data.",
            SECURITY_SSPI::CERTSRV_E_SERVER_SUSPENDED => "Certificate service has been suspended for a database restore operation.",
            SECURITY_SSPI::CERTSRV_E_ENCODING_LENGTH => "The certificate contains an encoded length that is potentially incompatible with older enrollment software.",
            SECURITY_SSPI::CERTSRV_E_ROLECONFLICT => "The operation is denied. The user has multiple roles assigned and the certification authority is configured to enforce role separation.",
            SECURITY_SSPI::CERTSRV_E_RESTRICTEDOFFICER => "The operation is denied. It can only be performed by a certificate manager that is allowed to manage certificates for the current requester.",
            SECURITY_SSPI::CERTSRV_E_KEY_ARCHIVAL_NOT_CONFIGURED => "Cannot archive private key. The certification authority is not configured for key archival.",
            SECURITY_SSPI::CERTSRV_E_NO_VALID_KRA => "Cannot archive private key. The certification authority could not verify one or more key recovery certificates.",
            SECURITY_SSPI::CERTSRV_E_BAD_REQUEST_KEY_ARCHIVAL => "The request is incorrectly formatted. The encrypted private key must be in an unauthenticated attribute in an outermost signature.",
            SECURITY_SSPI::CERTSRV_E_NO_CAADMIN_DEFINED => "At least one security principal must have the permission to manage this CA.",
            SECURITY_SSPI::CERTSRV_E_BAD_RENEWAL_CERT_ATTRIBUTE => "The request contains an invalid renewal certificate attribute.",
            SECURITY_SSPI::CERTSRV_E_NO_DB_SESSIONS => "An attempt was made to open a Certification Authority database session, but there are already too many active sessions. The server may need to be configured to allow additional sessions.",
            SECURITY_SSPI::CERTSRV_E_ALIGNMENT_FAULT => "A memory reference caused a data alignment fault.",
            SECURITY_SSPI::CERTSRV_E_ENROLL_DENIED => "The permissions on this certification authority do not allow the current user to enroll for certificates.",
            SECURITY_SSPI::CERTSRV_E_TEMPLATE_DENIED => "The permissions on the certificate template do not allow the current user to enroll for this type of certificate.",
            SECURITY_SSPI::CERTSRV_E_DOWNLEVEL_DC_SSL_OR_UPGRADE => "The contacted domain controller cannot support signed LDAP traffic. Update the domain controller or configure Certificate Services to use SSL for Active Directory access.",
            SECURITY_SSPI::CERTSRV_E_ADMIN_DENIED_REQUEST => "The request was denied by a certificate manager or CA administrator.",
            SECURITY_SSPI::CERTSRV_E_NO_POLICY_SERVER => "An enrollment policy server cannot be located.",
            SECURITY_SSPI::CERTSRV_E_WEAK_SIGNATURE_OR_KEY => "A signature algorithm or public key length does not meet the system's minimum required strength.",
            SECURITY_SSPI::CERTSRV_E_KEY_ATTESTATION_NOT_SUPPORTED => "Failed to create an attested key. This computer or the cryptographic provider may not meet the hardware requirements to support key attestation.",
            SECURITY_SSPI::CERTSRV_E_ENCRYPTION_CERT_REQUIRED => "No encryption certificate was specified.",
            SECURITY_SSPI::CERTSRV_E_UNSUPPORTED_CERT_TYPE => "The requested certificate template is not supported by this CA.",
            SECURITY_SSPI::CERTSRV_E_NO_CERT_TYPE => "The request contains no certificate template information.",
            SECURITY_SSPI::CERTSRV_E_TEMPLATE_CONFLICT => "The request contains conflicting template information.",
            SECURITY_SSPI::CERTSRV_E_SUBJECT_ALT_NAME_REQUIRED => "The request is missing a required Subject Alternate name extension.",
            SECURITY_SSPI::CERTSRV_E_ARCHIVED_KEY_REQUIRED => "The request is missing a required private key for archival by the server.",
            SECURITY_SSPI::CERTSRV_E_SMIME_REQUIRED => "The request is missing a required SMIME capabilities extension.",
            SECURITY_SSPI::CERTSRV_E_BAD_RENEWAL_SUBJECT => "The request was made on behalf of a subject other than the caller. The certificate template must be configured to require at least one signature to authorize the request.",
            SECURITY_SSPI::CERTSRV_E_BAD_TEMPLATE_VERSION => "The request template version is newer than the supported template version.",
            SECURITY_SSPI::CERTSRV_E_TEMPLATE_POLICY_REQUIRED => "The template is missing a required signature policy attribute.",
            SECURITY_SSPI::CERTSRV_E_SIGNATURE_POLICY_REQUIRED => "The request is missing required signature policy information.",
            SECURITY_SSPI::CERTSRV_E_SIGNATURE_COUNT => "The request is missing one or more required signatures.",
            SECURITY_SSPI::CERTSRV_E_SIGNATURE_REJECTED => "One or more signatures did not include the required application or issuance policies. The request is missing one or more required valid signatures.",
            SECURITY_SSPI::CERTSRV_E_ISSUANCE_POLICY_REQUIRED => "The request is missing one or more required signature issuance policies.",
            SECURITY_SSPI::CERTSRV_E_SUBJECT_UPN_REQUIRED => "The UPN is unavailable and cannot be added to the Subject Alternate name.",
            SECURITY_SSPI::CERTSRV_E_SUBJECT_DIRECTORY_GUID_REQUIRED => "The Active Directory GUID is unavailable and cannot be added to the Subject Alternate name.",
            SECURITY_SSPI::CERTSRV_E_SUBJECT_DNS_REQUIRED => "The DNS name is unavailable and cannot be added to the Subject Alternate name.",
            SECURITY_SSPI::CERTSRV_E_ARCHIVED_KEY_UNEXPECTED => "The request includes a private key for archival by the server, but key archival is not enabled for the specified certificate template.",
            SECURITY_SSPI::CERTSRV_E_KEY_LENGTH => "The public key does not meet the minimum size required by the specified certificate template.",
            SECURITY_SSPI::CERTSRV_E_SUBJECT_EMAIL_REQUIRED => "The EMail name is unavailable and cannot be added to the Subject or Subject Alternate name.",
            SECURITY_SSPI::CERTSRV_E_UNKNOWN_CERT_TYPE => "One or more certificate templates to be enabled on this certification authority could not be found.",
            SECURITY_SSPI::CERTSRV_E_CERT_TYPE_OVERLAP => "The certificate template renewal period is longer than the certificate validity period. The template should be reconfigured or the CA certificate renewed.",
            SECURITY_SSPI::CERTSRV_E_TOO_MANY_SIGNATURES => "The certificate template requires too many RA signatures. Only one RA signature is allowed.",
            SECURITY_SSPI::CERTSRV_E_RENEWAL_BAD_PUBLIC_KEY => "The certificate template requires renewal with the same public key, but the request uses a different public key.",
            SECURITY_SSPI::CERTSRV_E_INVALID_EK => "The certification authority cannot interpret or verify the endorsement key information supplied in the request, or the information is inconsistent.",
            SECURITY_SSPI::CERTSRV_E_INVALID_IDBINDING => "The certification authority cannot validate the Attestation Identity Key Id Binding.",
            SECURITY_SSPI::CERTSRV_E_INVALID_ATTESTATION => "The certification authority cannot validate the private key attestation data.",
            SECURITY_SSPI::CERTSRV_E_KEY_ATTESTATION => "The request does not support private key attestation as defined in the certificate template.",
            SECURITY_SSPI::CERTSRV_E_CORRUPT_KEY_ATTESTATION => "The request public key is not consistent with the private key attestation data.",
            SECURITY_SSPI::CERTSRV_E_EXPIRED_CHALLENGE => "The private key attestation challenge cannot be validated because the encryption certificate has expired, or the certificate or key is unavailable.",
            SECURITY_SSPI::CERTSRV_E_INVALID_RESPONSE => "The client's response could not be validated. It is either unexpected or incorrect.",
            SECURITY_SSPI::CERTSRV_E_INVALID_REQUESTID => "A valid Request ID was not detected in the request attributes, or an invalid one was submitted.",
            SECURITY_SSPI::CERTSRV_E_REQUEST_PRECERTIFICATE_MISMATCH => "The request is not consistent with the previously generated precertificate.",
            SECURITY_SSPI::CERTSRV_E_PENDING_CLIENT_RESPONSE => "The request is locked against edits until a response is received from the client.",
            SECURITY_SSPI::CERTSRV_E_SEC_EXT_DIRECTORY_SID_REQUIRED => "The Active Directory SID is unavailable and cannot be added to the custom security extension.",
            SECURITY_SSPI::XENROLL_E_KEY_NOT_EXPORTABLE => "The key is not exportable.",
            SECURITY_SSPI::XENROLL_E_CANNOT_ADD_ROOT_CERT => "You cannot add the root CA certificate into your local store.",
            SECURITY_SSPI::XENROLL_E_RESPONSE_KA_HASH_NOT_FOUND => "The key archival hash attribute was not found in the response.",
            SECURITY_SSPI::XENROLL_E_RESPONSE_UNEXPECTED_KA_HASH => "An unexpected key archival hash attribute was found in the response.",
            SECURITY_SSPI::XENROLL_E_RESPONSE_KA_HASH_MISMATCH => "There is a key archival hash mismatch between the request and the response.",
            SECURITY_SSPI::XENROLL_E_KEYSPEC_SMIME_MISMATCH => "Signing certificate cannot include SMIME extension.",
            SECURITY_SSPI::TRUST_E_SYSTEM_ERROR => "A system-level error occurred while verifying trust.",
            SECURITY_SSPI::TRUST_E_NO_SIGNER_CERT => "The certificate for the signer of the message is invalid or not found.",
            SECURITY_SSPI::TRUST_E_COUNTER_SIGNER => "One of the counter signatures was invalid.",
            SECURITY_SSPI::TRUST_E_CERT_SIGNATURE => "The signature of the certificate cannot be verified.",
            SECURITY_SSPI::TRUST_E_TIME_STAMP => "The timestamp signature and/or certificate could not be verified or is malformed.",
            SECURITY_SSPI::TRUST_E_BAD_DIGEST => "The digital signature of the object did not verify.",
            SECURITY_SSPI::TRUST_E_MALFORMED_SIGNATURE => "The digital signature of the object is malformed. For technical detail, see security bulletin MS13-098.",
            SECURITY_SSPI::TRUST_E_BASIC_CONSTRAINTS => "A certificate's basic constraint extension has not been observed.",
            SECURITY_SSPI::TRUST_E_FINANCIAL_CRITERIA => "The certificate does not meet or contain the Authenticode(tm) financial extensions.",
            SECURITY_SSPI::MSSIPOTF_E_OUTOFMEMRANGE => "Tried to reference a part of the file outside the proper range.",
            SECURITY_SSPI::MSSIPOTF_E_CANTGETOBJECT => "Could not retrieve an object from the file.",
            SECURITY_SSPI::MSSIPOTF_E_NOHEADTABLE => "Could not find the head table in the file.",
            SECURITY_SSPI::MSSIPOTF_E_BAD_MAGICNUMBER => "The magic number in the head table is incorrect.",
            SECURITY_SSPI::MSSIPOTF_E_BAD_OFFSET_TABLE => "The offset table has incorrect values.",
            SECURITY_SSPI::MSSIPOTF_E_TABLE_TAGORDER => "Duplicate table tags or tags out of alphabetical order.",
            SECURITY_SSPI::MSSIPOTF_E_TABLE_LONGWORD => "A table does not start on a long word boundary.",
            SECURITY_SSPI::MSSIPOTF_E_BAD_FIRST_TABLE_PLACEMENT => "First table does not appear after header information.",
            SECURITY_SSPI::MSSIPOTF_E_TABLES_OVERLAP => "Two or more tables overlap.",
            SECURITY_SSPI::MSSIPOTF_E_TABLE_PADBYTES => "Too many pad bytes between tables or pad bytes are not 0.",
            SECURITY_SSPI::MSSIPOTF_E_FILETOOSMALL => "File is too small to contain the last table.",
            SECURITY_SSPI::MSSIPOTF_E_TABLE_CHECKSUM => "A table checksum is incorrect.",
            SECURITY_SSPI::MSSIPOTF_E_FILE_CHECKSUM => "The file checksum is incorrect.",
            SECURITY_SSPI::MSSIPOTF_E_FAILED_POLICY => "The signature does not have the correct attributes for the policy.",
            SECURITY_SSPI::MSSIPOTF_E_FAILED_HINTS_CHECK => "The file did not pass the hints check.",
            SECURITY_SSPI::MSSIPOTF_E_NOT_OPENTYPE => "The file is not an OpenType file.",
            SECURITY_SSPI::MSSIPOTF_E_FILE => "Failed on a file operation (open, map, read, write).",
            SECURITY_SSPI::MSSIPOTF_E_CRYPT => "A call to a CryptoAPI function failed.",
            SECURITY_SSPI::MSSIPOTF_E_BADVERSION => "There is a bad version number in the file.",
            SECURITY_SSPI::MSSIPOTF_E_DSIG_STRUCTURE => "The structure of the DSIG table is incorrect.",
            SECURITY_SSPI::MSSIPOTF_E_PCONST_CHECK => "A check failed in a partially constant table.",
            SECURITY_SSPI::MSSIPOTF_E_STRUCTURE => "Some kind of structural error.",
            SECURITY_SSPI::ERROR_CRED_REQUIRES_CONFIRMATION => "The requested credential requires confirmation.",
            SECURITY_SSPI::ERROR_AUDITING_DISABLED => "The specified event is currently not being audited.",
            SECURITY_SSPI::ERROR_ALL_SIDS_FILTERED => "The SID filtering operation removed all SIDs.",
            SECURITY_SSPI::ERROR_BIZRULES_NOT_ENABLED => "Business rule scripts are disabled for the calling application.",
        }
    }

    pub fn from_name(name: &str) -> SECURITY_SSPI {
        return match name {
            "SEC_I_CONTINUE_NEEDED" => SECURITY_SSPI::SEC_I_CONTINUE_NEEDED,
            "SEC_I_COMPLETE_NEEDED" => SECURITY_SSPI::SEC_I_COMPLETE_NEEDED,
            "SEC_I_COMPLETE_AND_CONTINUE" => SECURITY_SSPI::SEC_I_COMPLETE_AND_CONTINUE,
            "SEC_I_LOCAL_LOGON" => SECURITY_SSPI::SEC_I_LOCAL_LOGON,
            "SEC_I_GENERIC_EXTENSION_RECEIVED" => SECURITY_SSPI::SEC_I_GENERIC_EXTENSION_RECEIVED,
            "SEC_I_CONTEXT_EXPIRED" => SECURITY_SSPI::SEC_I_CONTEXT_EXPIRED,
            "SEC_I_INCOMPLETE_CREDENTIALS" => SECURITY_SSPI::SEC_I_INCOMPLETE_CREDENTIALS,
            "SEC_I_RENEGOTIATE" => SECURITY_SSPI::SEC_I_RENEGOTIATE,
            "SEC_I_NO_LSA_CONTEXT" => SECURITY_SSPI::SEC_I_NO_LSA_CONTEXT,
            "SEC_I_SIGNATURE_NEEDED" => SECURITY_SSPI::SEC_I_SIGNATURE_NEEDED,
            "SEC_I_NO_RENEGOTIATION" => SECURITY_SSPI::SEC_I_NO_RENEGOTIATION,
            "SEC_I_MESSAGE_FRAGMENT" => SECURITY_SSPI::SEC_I_MESSAGE_FRAGMENT,
            "SEC_I_CONTINUE_NEEDED_MESSAGE_OK" => SECURITY_SSPI::SEC_I_CONTINUE_NEEDED_MESSAGE_OK,
            "SEC_I_ASYNC_CALL_PENDING" => SECURITY_SSPI::SEC_I_ASYNC_CALL_PENDING,
            "CRYPT_I_NEW_PROTECTION_REQUIRED" => SECURITY_SSPI::CRYPT_I_NEW_PROTECTION_REQUIRED,
            "NTE_BAD_UID" => SECURITY_SSPI::NTE_BAD_UID,
            "NTE_BAD_HASH" => SECURITY_SSPI::NTE_BAD_HASH,
            "NTE_BAD_KEY" => SECURITY_SSPI::NTE_BAD_KEY,
            "NTE_BAD_LEN" => SECURITY_SSPI::NTE_BAD_LEN,
            "NTE_BAD_DATA" => SECURITY_SSPI::NTE_BAD_DATA,
            "NTE_BAD_SIGNATURE" => SECURITY_SSPI::NTE_BAD_SIGNATURE,
            "NTE_BAD_VER" => SECURITY_SSPI::NTE_BAD_VER,
            "NTE_BAD_ALGID" => SECURITY_SSPI::NTE_BAD_ALGID,
            "NTE_BAD_FLAGS" => SECURITY_SSPI::NTE_BAD_FLAGS,
            "NTE_BAD_TYPE" => SECURITY_SSPI::NTE_BAD_TYPE,
            "NTE_BAD_KEY_STATE" => SECURITY_SSPI::NTE_BAD_KEY_STATE,
            "NTE_BAD_HASH_STATE" => SECURITY_SSPI::NTE_BAD_HASH_STATE,
            "NTE_NO_KEY" => SECURITY_SSPI::NTE_NO_KEY,
            "NTE_NO_MEMORY" => SECURITY_SSPI::NTE_NO_MEMORY,
            "NTE_EXISTS" => SECURITY_SSPI::NTE_EXISTS,
            "NTE_PERM" => SECURITY_SSPI::NTE_PERM,
            "NTE_NOT_FOUND" => SECURITY_SSPI::NTE_NOT_FOUND,
            "NTE_DOUBLE_ENCRYPT" => SECURITY_SSPI::NTE_DOUBLE_ENCRYPT,
            "NTE_BAD_PROVIDER" => SECURITY_SSPI::NTE_BAD_PROVIDER,
            "NTE_BAD_PROV_TYPE" => SECURITY_SSPI::NTE_BAD_PROV_TYPE,
            "NTE_BAD_PUBLIC_KEY" => SECURITY_SSPI::NTE_BAD_PUBLIC_KEY,
            "NTE_BAD_KEYSET" => SECURITY_SSPI::NTE_BAD_KEYSET,
            "NTE_PROV_TYPE_NOT_DEF" => SECURITY_SSPI::NTE_PROV_TYPE_NOT_DEF,
            "NTE_PROV_TYPE_ENTRY_BAD" => SECURITY_SSPI::NTE_PROV_TYPE_ENTRY_BAD,
            "NTE_KEYSET_NOT_DEF" => SECURITY_SSPI::NTE_KEYSET_NOT_DEF,
            "NTE_KEYSET_ENTRY_BAD" => SECURITY_SSPI::NTE_KEYSET_ENTRY_BAD,
            "NTE_PROV_TYPE_NO_MATCH" => SECURITY_SSPI::NTE_PROV_TYPE_NO_MATCH,
            "NTE_SIGNATURE_FILE_BAD" => SECURITY_SSPI::NTE_SIGNATURE_FILE_BAD,
            "NTE_PROVIDER_DLL_FAIL" => SECURITY_SSPI::NTE_PROVIDER_DLL_FAIL,
            "NTE_PROV_DLL_NOT_FOUND" => SECURITY_SSPI::NTE_PROV_DLL_NOT_FOUND,
            "NTE_BAD_KEYSET_PARAM" => SECURITY_SSPI::NTE_BAD_KEYSET_PARAM,
            "NTE_FAIL" => SECURITY_SSPI::NTE_FAIL,
            "NTE_SYS_ERR" => SECURITY_SSPI::NTE_SYS_ERR,
            "NTE_SILENT_CONTEXT" => SECURITY_SSPI::NTE_SILENT_CONTEXT,
            "NTE_TOKEN_KEYSET_STORAGE_FULL" => SECURITY_SSPI::NTE_TOKEN_KEYSET_STORAGE_FULL,
            "NTE_TEMPORARY_PROFILE" => SECURITY_SSPI::NTE_TEMPORARY_PROFILE,
            "NTE_FIXEDPARAMETER" => SECURITY_SSPI::NTE_FIXEDPARAMETER,
            "NTE_INVALID_HANDLE" => SECURITY_SSPI::NTE_INVALID_HANDLE,
            "NTE_INVALID_PARAMETER" => SECURITY_SSPI::NTE_INVALID_PARAMETER,
            "NTE_BUFFER_TOO_SMALL" => SECURITY_SSPI::NTE_BUFFER_TOO_SMALL,
            "NTE_NOT_SUPPORTED" => SECURITY_SSPI::NTE_NOT_SUPPORTED,
            "NTE_NO_MORE_ITEMS" => SECURITY_SSPI::NTE_NO_MORE_ITEMS,
            "NTE_BUFFERS_OVERLAP" => SECURITY_SSPI::NTE_BUFFERS_OVERLAP,
            "NTE_DECRYPTION_FAILURE" => SECURITY_SSPI::NTE_DECRYPTION_FAILURE,
            "NTE_INTERNAL_ERROR" => SECURITY_SSPI::NTE_INTERNAL_ERROR,
            "NTE_UI_REQUIRED" => SECURITY_SSPI::NTE_UI_REQUIRED,
            "NTE_HMAC_NOT_SUPPORTED" => SECURITY_SSPI::NTE_HMAC_NOT_SUPPORTED,
            "NTE_DEVICE_NOT_READY" => SECURITY_SSPI::NTE_DEVICE_NOT_READY,
            "NTE_AUTHENTICATION_IGNORED" => SECURITY_SSPI::NTE_AUTHENTICATION_IGNORED,
            "NTE_VALIDATION_FAILED" => SECURITY_SSPI::NTE_VALIDATION_FAILED,
            "NTE_INCORRECT_PASSWORD" => SECURITY_SSPI::NTE_INCORRECT_PASSWORD,
            "NTE_ENCRYPTION_FAILURE" => SECURITY_SSPI::NTE_ENCRYPTION_FAILURE,
            "NTE_DEVICE_NOT_FOUND" => SECURITY_SSPI::NTE_DEVICE_NOT_FOUND,
            "NTE_USER_CANCELLED" => SECURITY_SSPI::NTE_USER_CANCELLED,
            "NTE_PASSWORD_CHANGE_REQUIRED" => SECURITY_SSPI::NTE_PASSWORD_CHANGE_REQUIRED,
            "NTE_NOT_ACTIVE_CONSOLE" => SECURITY_SSPI::NTE_NOT_ACTIVE_CONSOLE,
            "SEC_E_INSUFFICIENT_MEMORY" => SECURITY_SSPI::SEC_E_INSUFFICIENT_MEMORY,
            "SEC_E_INVALID_HANDLE" => SECURITY_SSPI::SEC_E_INVALID_HANDLE,
            "SEC_E_UNSUPPORTED_FUNCTION" => SECURITY_SSPI::SEC_E_UNSUPPORTED_FUNCTION,
            "SEC_E_TARGET_UNKNOWN" => SECURITY_SSPI::SEC_E_TARGET_UNKNOWN,
            "SEC_E_INTERNAL_ERROR" => SECURITY_SSPI::SEC_E_INTERNAL_ERROR,
            "SEC_E_SECPKG_NOT_FOUND" => SECURITY_SSPI::SEC_E_SECPKG_NOT_FOUND,
            "SEC_E_NOT_OWNER" => SECURITY_SSPI::SEC_E_NOT_OWNER,
            "SEC_E_CANNOT_INSTALL" => SECURITY_SSPI::SEC_E_CANNOT_INSTALL,
            "SEC_E_INVALID_TOKEN" => SECURITY_SSPI::SEC_E_INVALID_TOKEN,
            "SEC_E_CANNOT_PACK" => SECURITY_SSPI::SEC_E_CANNOT_PACK,
            "SEC_E_QOP_NOT_SUPPORTED" => SECURITY_SSPI::SEC_E_QOP_NOT_SUPPORTED,
            "SEC_E_NO_IMPERSONATION" => SECURITY_SSPI::SEC_E_NO_IMPERSONATION,
            "SEC_E_LOGON_DENIED" => SECURITY_SSPI::SEC_E_LOGON_DENIED,
            "SEC_E_UNKNOWN_CREDENTIALS" => SECURITY_SSPI::SEC_E_UNKNOWN_CREDENTIALS,
            "SEC_E_NO_CREDENTIALS" => SECURITY_SSPI::SEC_E_NO_CREDENTIALS,
            "SEC_E_MESSAGE_ALTERED" => SECURITY_SSPI::SEC_E_MESSAGE_ALTERED,
            "SEC_E_OUT_OF_SEQUENCE" => SECURITY_SSPI::SEC_E_OUT_OF_SEQUENCE,
            "SEC_E_NO_AUTHENTICATING_AUTHORITY" => SECURITY_SSPI::SEC_E_NO_AUTHENTICATING_AUTHORITY,
            "SEC_E_BAD_PKGID" => SECURITY_SSPI::SEC_E_BAD_PKGID,
            "SEC_E_CONTEXT_EXPIRED" => SECURITY_SSPI::SEC_E_CONTEXT_EXPIRED,
            "SEC_E_INCOMPLETE_MESSAGE" => SECURITY_SSPI::SEC_E_INCOMPLETE_MESSAGE,
            "SEC_E_INCOMPLETE_CREDENTIALS" => SECURITY_SSPI::SEC_E_INCOMPLETE_CREDENTIALS,
            "SEC_E_BUFFER_TOO_SMALL" => SECURITY_SSPI::SEC_E_BUFFER_TOO_SMALL,
            "SEC_E_WRONG_PRINCIPAL" => SECURITY_SSPI::SEC_E_WRONG_PRINCIPAL,
            "SEC_E_TIME_SKEW" => SECURITY_SSPI::SEC_E_TIME_SKEW,
            "SEC_E_UNTRUSTED_ROOT" => SECURITY_SSPI::SEC_E_UNTRUSTED_ROOT,
            "SEC_E_ILLEGAL_MESSAGE" => SECURITY_SSPI::SEC_E_ILLEGAL_MESSAGE,
            "SEC_E_CERT_UNKNOWN" => SECURITY_SSPI::SEC_E_CERT_UNKNOWN,
            "SEC_E_CERT_EXPIRED" => SECURITY_SSPI::SEC_E_CERT_EXPIRED,
            "SEC_E_ENCRYPT_FAILURE" => SECURITY_SSPI::SEC_E_ENCRYPT_FAILURE,
            "SEC_E_DECRYPT_FAILURE" => SECURITY_SSPI::SEC_E_DECRYPT_FAILURE,
            "SEC_E_ALGORITHM_MISMATCH" => SECURITY_SSPI::SEC_E_ALGORITHM_MISMATCH,
            "SEC_E_SECURITY_QOS_FAILED" => SECURITY_SSPI::SEC_E_SECURITY_QOS_FAILED,
            "SEC_E_UNFINISHED_CONTEXT_DELETED" => SECURITY_SSPI::SEC_E_UNFINISHED_CONTEXT_DELETED,
            "SEC_E_NO_TGT_REPLY" => SECURITY_SSPI::SEC_E_NO_TGT_REPLY,
            "SEC_E_NO_IP_ADDRESSES" => SECURITY_SSPI::SEC_E_NO_IP_ADDRESSES,
            "SEC_E_WRONG_CREDENTIAL_HANDLE" => SECURITY_SSPI::SEC_E_WRONG_CREDENTIAL_HANDLE,
            "SEC_E_CRYPTO_SYSTEM_INVALID" => SECURITY_SSPI::SEC_E_CRYPTO_SYSTEM_INVALID,
            "SEC_E_MAX_REFERRALS_EXCEEDED" => SECURITY_SSPI::SEC_E_MAX_REFERRALS_EXCEEDED,
            "SEC_E_MUST_BE_KDC" => SECURITY_SSPI::SEC_E_MUST_BE_KDC,
            "SEC_E_STRONG_CRYPTO_NOT_SUPPORTED" => SECURITY_SSPI::SEC_E_STRONG_CRYPTO_NOT_SUPPORTED,
            "SEC_E_TOO_MANY_PRINCIPALS" => SECURITY_SSPI::SEC_E_TOO_MANY_PRINCIPALS,
            "SEC_E_NO_PA_DATA" => SECURITY_SSPI::SEC_E_NO_PA_DATA,
            "SEC_E_PKINIT_NAME_MISMATCH" => SECURITY_SSPI::SEC_E_PKINIT_NAME_MISMATCH,
            "SEC_E_SMARTCARD_LOGON_REQUIRED" => SECURITY_SSPI::SEC_E_SMARTCARD_LOGON_REQUIRED,
            "SEC_E_SHUTDOWN_IN_PROGRESS" => SECURITY_SSPI::SEC_E_SHUTDOWN_IN_PROGRESS,
            "SEC_E_KDC_INVALID_REQUEST" => SECURITY_SSPI::SEC_E_KDC_INVALID_REQUEST,
            "SEC_E_KDC_UNABLE_TO_REFER" => SECURITY_SSPI::SEC_E_KDC_UNABLE_TO_REFER,
            "SEC_E_KDC_UNKNOWN_ETYPE" => SECURITY_SSPI::SEC_E_KDC_UNKNOWN_ETYPE,
            "SEC_E_UNSUPPORTED_PREAUTH" => SECURITY_SSPI::SEC_E_UNSUPPORTED_PREAUTH,
            "SEC_E_DELEGATION_REQUIRED" => SECURITY_SSPI::SEC_E_DELEGATION_REQUIRED,
            "SEC_E_BAD_BINDINGS" => SECURITY_SSPI::SEC_E_BAD_BINDINGS,
            "SEC_E_MULTIPLE_ACCOUNTS" => SECURITY_SSPI::SEC_E_MULTIPLE_ACCOUNTS,
            "SEC_E_NO_KERB_KEY" => SECURITY_SSPI::SEC_E_NO_KERB_KEY,
            "SEC_E_CERT_WRONG_USAGE" => SECURITY_SSPI::SEC_E_CERT_WRONG_USAGE,
            "SEC_E_DOWNGRADE_DETECTED" => SECURITY_SSPI::SEC_E_DOWNGRADE_DETECTED,
            "SEC_E_SMARTCARD_CERT_REVOKED" => SECURITY_SSPI::SEC_E_SMARTCARD_CERT_REVOKED,
            "SEC_E_ISSUING_CA_UNTRUSTED" => SECURITY_SSPI::SEC_E_ISSUING_CA_UNTRUSTED,
            "SEC_E_REVOCATION_OFFLINE_C" => SECURITY_SSPI::SEC_E_REVOCATION_OFFLINE_C,
            "SEC_E_PKINIT_CLIENT_FAILURE" => SECURITY_SSPI::SEC_E_PKINIT_CLIENT_FAILURE,
            "SEC_E_SMARTCARD_CERT_EXPIRED" => SECURITY_SSPI::SEC_E_SMARTCARD_CERT_EXPIRED,
            "SEC_E_NO_S4U_PROT_SUPPORT" => SECURITY_SSPI::SEC_E_NO_S4U_PROT_SUPPORT,
            "SEC_E_CROSSREALM_DELEGATION_FAILURE" => SECURITY_SSPI::SEC_E_CROSSREALM_DELEGATION_FAILURE,
            "SEC_E_REVOCATION_OFFLINE_KDC" => SECURITY_SSPI::SEC_E_REVOCATION_OFFLINE_KDC,
            "SEC_E_ISSUING_CA_UNTRUSTED_KDC" => SECURITY_SSPI::SEC_E_ISSUING_CA_UNTRUSTED_KDC,
            "SEC_E_KDC_CERT_EXPIRED" => SECURITY_SSPI::SEC_E_KDC_CERT_EXPIRED,
            "SEC_E_KDC_CERT_REVOKED" => SECURITY_SSPI::SEC_E_KDC_CERT_REVOKED,
            "SEC_E_INVALID_PARAMETER" => SECURITY_SSPI::SEC_E_INVALID_PARAMETER,
            "SEC_E_DELEGATION_POLICY" => SECURITY_SSPI::SEC_E_DELEGATION_POLICY,
            "SEC_E_POLICY_NLTM_ONLY" => SECURITY_SSPI::SEC_E_POLICY_NLTM_ONLY,
            "SEC_E_NO_CONTEXT" => SECURITY_SSPI::SEC_E_NO_CONTEXT,
            "SEC_E_PKU2U_CERT_FAILURE" => SECURITY_SSPI::SEC_E_PKU2U_CERT_FAILURE,
            "SEC_E_MUTUAL_AUTH_FAILED" => SECURITY_SSPI::SEC_E_MUTUAL_AUTH_FAILED,
            "SEC_E_ONLY_HTTPS_ALLOWED" => SECURITY_SSPI::SEC_E_ONLY_HTTPS_ALLOWED,
            "SEC_E_APPLICATION_PROTOCOL_MISMATCH" => SECURITY_SSPI::SEC_E_APPLICATION_PROTOCOL_MISMATCH,
            "SEC_E_INVALID_UPN_NAME" => SECURITY_SSPI::SEC_E_INVALID_UPN_NAME,
            "SEC_E_EXT_BUFFER_TOO_SMALL" => SECURITY_SSPI::SEC_E_EXT_BUFFER_TOO_SMALL,
            "SEC_E_INSUFFICIENT_BUFFERS" => SECURITY_SSPI::SEC_E_INSUFFICIENT_BUFFERS,
            "CRYPT_E_MSG_ERROR" => SECURITY_SSPI::CRYPT_E_MSG_ERROR,
            "CRYPT_E_UNKNOWN_ALGO" => SECURITY_SSPI::CRYPT_E_UNKNOWN_ALGO,
            "CRYPT_E_OID_FORMAT" => SECURITY_SSPI::CRYPT_E_OID_FORMAT,
            "CRYPT_E_INVALID_MSG_TYPE" => SECURITY_SSPI::CRYPT_E_INVALID_MSG_TYPE,
            "CRYPT_E_UNEXPECTED_ENCODING" => SECURITY_SSPI::CRYPT_E_UNEXPECTED_ENCODING,
            "CRYPT_E_AUTH_ATTR_MISSING" => SECURITY_SSPI::CRYPT_E_AUTH_ATTR_MISSING,
            "CRYPT_E_HASH_VALUE" => SECURITY_SSPI::CRYPT_E_HASH_VALUE,
            "CRYPT_E_INVALID_INDEX" => SECURITY_SSPI::CRYPT_E_INVALID_INDEX,
            "CRYPT_E_ALREADY_DECRYPTED" => SECURITY_SSPI::CRYPT_E_ALREADY_DECRYPTED,
            "CRYPT_E_NOT_DECRYPTED" => SECURITY_SSPI::CRYPT_E_NOT_DECRYPTED,
            "CRYPT_E_RECIPIENT_NOT_FOUND" => SECURITY_SSPI::CRYPT_E_RECIPIENT_NOT_FOUND,
            "CRYPT_E_CONTROL_TYPE" => SECURITY_SSPI::CRYPT_E_CONTROL_TYPE,
            "CRYPT_E_ISSUER_SERIALNUMBER" => SECURITY_SSPI::CRYPT_E_ISSUER_SERIALNUMBER,
            "CRYPT_E_SIGNER_NOT_FOUND" => SECURITY_SSPI::CRYPT_E_SIGNER_NOT_FOUND,
            "CRYPT_E_ATTRIBUTES_MISSING" => SECURITY_SSPI::CRYPT_E_ATTRIBUTES_MISSING,
            "CRYPT_E_STREAM_MSG_NOT_READY" => SECURITY_SSPI::CRYPT_E_STREAM_MSG_NOT_READY,
            "CRYPT_E_STREAM_INSUFFICIENT_DATA" => SECURITY_SSPI::CRYPT_E_STREAM_INSUFFICIENT_DATA,
            "CRYPT_E_BAD_LEN" => SECURITY_SSPI::CRYPT_E_BAD_LEN,
            "CRYPT_E_BAD_ENCODE" => SECURITY_SSPI::CRYPT_E_BAD_ENCODE,
            "CRYPT_E_FILE_ERROR" => SECURITY_SSPI::CRYPT_E_FILE_ERROR,
            "CRYPT_E_NOT_FOUND" => SECURITY_SSPI::CRYPT_E_NOT_FOUND,
            "CRYPT_E_EXISTS" => SECURITY_SSPI::CRYPT_E_EXISTS,
            "CRYPT_E_NO_PROVIDER" => SECURITY_SSPI::CRYPT_E_NO_PROVIDER,
            "CRYPT_E_SELF_SIGNED" => SECURITY_SSPI::CRYPT_E_SELF_SIGNED,
            "CRYPT_E_DELETED_PREV" => SECURITY_SSPI::CRYPT_E_DELETED_PREV,
            "CRYPT_E_NO_MATCH" => SECURITY_SSPI::CRYPT_E_NO_MATCH,
            "CRYPT_E_UNEXPECTED_MSG_TYPE" => SECURITY_SSPI::CRYPT_E_UNEXPECTED_MSG_TYPE,
            "CRYPT_E_NO_KEY_PROPERTY" => SECURITY_SSPI::CRYPT_E_NO_KEY_PROPERTY,
            "CRYPT_E_NO_DECRYPT_CERT" => SECURITY_SSPI::CRYPT_E_NO_DECRYPT_CERT,
            "CRYPT_E_BAD_MSG" => SECURITY_SSPI::CRYPT_E_BAD_MSG,
            "CRYPT_E_NO_SIGNER" => SECURITY_SSPI::CRYPT_E_NO_SIGNER,
            "CRYPT_E_PENDING_CLOSE" => SECURITY_SSPI::CRYPT_E_PENDING_CLOSE,
            "CRYPT_E_REVOKED" => SECURITY_SSPI::CRYPT_E_REVOKED,
            "CRYPT_E_NO_REVOCATION_DLL" => SECURITY_SSPI::CRYPT_E_NO_REVOCATION_DLL,
            "CRYPT_E_NO_REVOCATION_CHECK" => SECURITY_SSPI::CRYPT_E_NO_REVOCATION_CHECK,
            "CRYPT_E_REVOCATION_OFFLINE" => SECURITY_SSPI::CRYPT_E_REVOCATION_OFFLINE,
            "CRYPT_E_NOT_IN_REVOCATION_DATABASE" => SECURITY_SSPI::CRYPT_E_NOT_IN_REVOCATION_DATABASE,
            "CRYPT_E_INVALID_NUMERIC_STRING" => SECURITY_SSPI::CRYPT_E_INVALID_NUMERIC_STRING,
            "CRYPT_E_INVALID_PRINTABLE_STRING" => SECURITY_SSPI::CRYPT_E_INVALID_PRINTABLE_STRING,
            "CRYPT_E_INVALID_IA5_STRING" => SECURITY_SSPI::CRYPT_E_INVALID_IA5_STRING,
            "CRYPT_E_INVALID_X500_STRING" => SECURITY_SSPI::CRYPT_E_INVALID_X500_STRING,
            "CRYPT_E_NOT_CHAR_STRING" => SECURITY_SSPI::CRYPT_E_NOT_CHAR_STRING,
            "CRYPT_E_FILERESIZED" => SECURITY_SSPI::CRYPT_E_FILERESIZED,
            "CRYPT_E_SECURITY_SETTINGS" => SECURITY_SSPI::CRYPT_E_SECURITY_SETTINGS,
            "CRYPT_E_NO_VERIFY_USAGE_DLL" => SECURITY_SSPI::CRYPT_E_NO_VERIFY_USAGE_DLL,
            "CRYPT_E_NO_VERIFY_USAGE_CHECK" => SECURITY_SSPI::CRYPT_E_NO_VERIFY_USAGE_CHECK,
            "CRYPT_E_VERIFY_USAGE_OFFLINE" => SECURITY_SSPI::CRYPT_E_VERIFY_USAGE_OFFLINE,
            "CRYPT_E_NOT_IN_CTL" => SECURITY_SSPI::CRYPT_E_NOT_IN_CTL,
            "CRYPT_E_NO_TRUSTED_SIGNER" => SECURITY_SSPI::CRYPT_E_NO_TRUSTED_SIGNER,
            "CRYPT_E_MISSING_PUBKEY_PARA" => SECURITY_SSPI::CRYPT_E_MISSING_PUBKEY_PARA,
            "CRYPT_E_OBJECT_LOCATOR_OBJECT_NOT_FOUND" => SECURITY_SSPI::CRYPT_E_OBJECT_LOCATOR_OBJECT_NOT_FOUND,
            "CRYPT_E_OSS_ERROR" => SECURITY_SSPI::CRYPT_E_OSS_ERROR,
            "OSS_MORE_BUF" => SECURITY_SSPI::OSS_MORE_BUF,
            "OSS_NEGATIVE_UINTEGER" => SECURITY_SSPI::OSS_NEGATIVE_UINTEGER,
            "OSS_PDU_RANGE" => SECURITY_SSPI::OSS_PDU_RANGE,
            "OSS_MORE_INPUT" => SECURITY_SSPI::OSS_MORE_INPUT,
            "OSS_DATA_ERROR" => SECURITY_SSPI::OSS_DATA_ERROR,
            "OSS_BAD_ARG" => SECURITY_SSPI::OSS_BAD_ARG,
            "OSS_BAD_VERSION" => SECURITY_SSPI::OSS_BAD_VERSION,
            "OSS_OUT_MEMORY" => SECURITY_SSPI::OSS_OUT_MEMORY,
            "OSS_PDU_MISMATCH" => SECURITY_SSPI::OSS_PDU_MISMATCH,
            "OSS_LIMITED" => SECURITY_SSPI::OSS_LIMITED,
            "OSS_BAD_PTR" => SECURITY_SSPI::OSS_BAD_PTR,
            "OSS_BAD_TIME" => SECURITY_SSPI::OSS_BAD_TIME,
            "OSS_INDEFINITE_NOT_SUPPORTED" => SECURITY_SSPI::OSS_INDEFINITE_NOT_SUPPORTED,
            "OSS_MEM_ERROR" => SECURITY_SSPI::OSS_MEM_ERROR,
            "OSS_BAD_TABLE" => SECURITY_SSPI::OSS_BAD_TABLE,
            "OSS_TOO_LONG" => SECURITY_SSPI::OSS_TOO_LONG,
            "OSS_CONSTRAINT_VIOLATED" => SECURITY_SSPI::OSS_CONSTRAINT_VIOLATED,
            "OSS_FATAL_ERROR" => SECURITY_SSPI::OSS_FATAL_ERROR,
            "OSS_ACCESS_SERIALIZATION_ERROR" => SECURITY_SSPI::OSS_ACCESS_SERIALIZATION_ERROR,
            "OSS_NULL_TBL" => SECURITY_SSPI::OSS_NULL_TBL,
            "OSS_NULL_FCN" => SECURITY_SSPI::OSS_NULL_FCN,
            "OSS_BAD_ENCRULES" => SECURITY_SSPI::OSS_BAD_ENCRULES,
            "OSS_UNAVAIL_ENCRULES" => SECURITY_SSPI::OSS_UNAVAIL_ENCRULES,
            "OSS_CANT_OPEN_TRACE_WINDOW" => SECURITY_SSPI::OSS_CANT_OPEN_TRACE_WINDOW,
            "OSS_UNIMPLEMENTED" => SECURITY_SSPI::OSS_UNIMPLEMENTED,
            "OSS_OID_DLL_NOT_LINKED" => SECURITY_SSPI::OSS_OID_DLL_NOT_LINKED,
            "OSS_CANT_OPEN_TRACE_FILE" => SECURITY_SSPI::OSS_CANT_OPEN_TRACE_FILE,
            "OSS_TRACE_FILE_ALREADY_OPEN" => SECURITY_SSPI::OSS_TRACE_FILE_ALREADY_OPEN,
            "OSS_TABLE_MISMATCH" => SECURITY_SSPI::OSS_TABLE_MISMATCH,
            "OSS_TYPE_NOT_SUPPORTED" => SECURITY_SSPI::OSS_TYPE_NOT_SUPPORTED,
            "OSS_REAL_DLL_NOT_LINKED" => SECURITY_SSPI::OSS_REAL_DLL_NOT_LINKED,
            "OSS_REAL_CODE_NOT_LINKED" => SECURITY_SSPI::OSS_REAL_CODE_NOT_LINKED,
            "OSS_OUT_OF_RANGE" => SECURITY_SSPI::OSS_OUT_OF_RANGE,
            "OSS_COPIER_DLL_NOT_LINKED" => SECURITY_SSPI::OSS_COPIER_DLL_NOT_LINKED,
            "OSS_CONSTRAINT_DLL_NOT_LINKED" => SECURITY_SSPI::OSS_CONSTRAINT_DLL_NOT_LINKED,
            "OSS_COMPARATOR_DLL_NOT_LINKED" => SECURITY_SSPI::OSS_COMPARATOR_DLL_NOT_LINKED,
            "OSS_COMPARATOR_CODE_NOT_LINKED" => SECURITY_SSPI::OSS_COMPARATOR_CODE_NOT_LINKED,
            "OSS_MEM_MGR_DLL_NOT_LINKED" => SECURITY_SSPI::OSS_MEM_MGR_DLL_NOT_LINKED,
            "OSS_PDV_DLL_NOT_LINKED" => SECURITY_SSPI::OSS_PDV_DLL_NOT_LINKED,
            "OSS_PDV_CODE_NOT_LINKED" => SECURITY_SSPI::OSS_PDV_CODE_NOT_LINKED,
            "OSS_API_DLL_NOT_LINKED" => SECURITY_SSPI::OSS_API_DLL_NOT_LINKED,
            "OSS_BERDER_DLL_NOT_LINKED" => SECURITY_SSPI::OSS_BERDER_DLL_NOT_LINKED,
            "OSS_PER_DLL_NOT_LINKED" => SECURITY_SSPI::OSS_PER_DLL_NOT_LINKED,
            "OSS_OPEN_TYPE_ERROR" => SECURITY_SSPI::OSS_OPEN_TYPE_ERROR,
            "OSS_MUTEX_NOT_CREATED" => SECURITY_SSPI::OSS_MUTEX_NOT_CREATED,
            "OSS_CANT_CLOSE_TRACE_FILE" => SECURITY_SSPI::OSS_CANT_CLOSE_TRACE_FILE,
            "CRYPT_E_ASN1_ERROR" => SECURITY_SSPI::CRYPT_E_ASN1_ERROR,
            "CRYPT_E_ASN1_INTERNAL" => SECURITY_SSPI::CRYPT_E_ASN1_INTERNAL,
            "CRYPT_E_ASN1_EOD" => SECURITY_SSPI::CRYPT_E_ASN1_EOD,
            "CRYPT_E_ASN1_CORRUPT" => SECURITY_SSPI::CRYPT_E_ASN1_CORRUPT,
            "CRYPT_E_ASN1_LARGE" => SECURITY_SSPI::CRYPT_E_ASN1_LARGE,
            "CRYPT_E_ASN1_CONSTRAINT" => SECURITY_SSPI::CRYPT_E_ASN1_CONSTRAINT,
            "CRYPT_E_ASN1_MEMORY" => SECURITY_SSPI::CRYPT_E_ASN1_MEMORY,
            "CRYPT_E_ASN1_OVERFLOW" => SECURITY_SSPI::CRYPT_E_ASN1_OVERFLOW,
            "CRYPT_E_ASN1_BADPDU" => SECURITY_SSPI::CRYPT_E_ASN1_BADPDU,
            "CRYPT_E_ASN1_BADARGS" => SECURITY_SSPI::CRYPT_E_ASN1_BADARGS,
            "CRYPT_E_ASN1_BADREAL" => SECURITY_SSPI::CRYPT_E_ASN1_BADREAL,
            "CRYPT_E_ASN1_BADTAG" => SECURITY_SSPI::CRYPT_E_ASN1_BADTAG,
            "CRYPT_E_ASN1_CHOICE" => SECURITY_SSPI::CRYPT_E_ASN1_CHOICE,
            "CRYPT_E_ASN1_RULE" => SECURITY_SSPI::CRYPT_E_ASN1_RULE,
            "CRYPT_E_ASN1_UTF8" => SECURITY_SSPI::CRYPT_E_ASN1_UTF8,
            "CRYPT_E_ASN1_PDU_TYPE" => SECURITY_SSPI::CRYPT_E_ASN1_PDU_TYPE,
            "CRYPT_E_ASN1_NYI" => SECURITY_SSPI::CRYPT_E_ASN1_NYI,
            "CRYPT_E_ASN1_EXTENDED" => SECURITY_SSPI::CRYPT_E_ASN1_EXTENDED,
            "CRYPT_E_ASN1_NOEOD" => SECURITY_SSPI::CRYPT_E_ASN1_NOEOD,
            "CERTSRV_E_BAD_REQUESTSUBJECT" => SECURITY_SSPI::CERTSRV_E_BAD_REQUESTSUBJECT,
            "CERTSRV_E_NO_REQUEST" => SECURITY_SSPI::CERTSRV_E_NO_REQUEST,
            "CERTSRV_E_BAD_REQUESTSTATUS" => SECURITY_SSPI::CERTSRV_E_BAD_REQUESTSTATUS,
            "CERTSRV_E_PROPERTY_EMPTY" => SECURITY_SSPI::CERTSRV_E_PROPERTY_EMPTY,
            "CERTSRV_E_INVALID_CA_CERTIFICATE" => SECURITY_SSPI::CERTSRV_E_INVALID_CA_CERTIFICATE,
            "CERTSRV_E_SERVER_SUSPENDED" => SECURITY_SSPI::CERTSRV_E_SERVER_SUSPENDED,
            "CERTSRV_E_ENCODING_LENGTH" => SECURITY_SSPI::CERTSRV_E_ENCODING_LENGTH,
            "CERTSRV_E_ROLECONFLICT" => SECURITY_SSPI::CERTSRV_E_ROLECONFLICT,
            "CERTSRV_E_RESTRICTEDOFFICER" => SECURITY_SSPI::CERTSRV_E_RESTRICTEDOFFICER,
            "CERTSRV_E_KEY_ARCHIVAL_NOT_CONFIGURED" => SECURITY_SSPI::CERTSRV_E_KEY_ARCHIVAL_NOT_CONFIGURED,
            "CERTSRV_E_NO_VALID_KRA" => SECURITY_SSPI::CERTSRV_E_NO_VALID_KRA,
            "CERTSRV_E_BAD_REQUEST_KEY_ARCHIVAL" => SECURITY_SSPI::CERTSRV_E_BAD_REQUEST_KEY_ARCHIVAL,
            "CERTSRV_E_NO_CAADMIN_DEFINED" => SECURITY_SSPI::CERTSRV_E_NO_CAADMIN_DEFINED,
            "CERTSRV_E_BAD_RENEWAL_CERT_ATTRIBUTE" => SECURITY_SSPI::CERTSRV_E_BAD_RENEWAL_CERT_ATTRIBUTE,
            "CERTSRV_E_NO_DB_SESSIONS" => SECURITY_SSPI::CERTSRV_E_NO_DB_SESSIONS,
            "CERTSRV_E_ALIGNMENT_FAULT" => SECURITY_SSPI::CERTSRV_E_ALIGNMENT_FAULT,
            "CERTSRV_E_ENROLL_DENIED" => SECURITY_SSPI::CERTSRV_E_ENROLL_DENIED,
            "CERTSRV_E_TEMPLATE_DENIED" => SECURITY_SSPI::CERTSRV_E_TEMPLATE_DENIED,
            "CERTSRV_E_DOWNLEVEL_DC_SSL_OR_UPGRADE" => SECURITY_SSPI::CERTSRV_E_DOWNLEVEL_DC_SSL_OR_UPGRADE,
            "CERTSRV_E_ADMIN_DENIED_REQUEST" => SECURITY_SSPI::CERTSRV_E_ADMIN_DENIED_REQUEST,
            "CERTSRV_E_NO_POLICY_SERVER" => SECURITY_SSPI::CERTSRV_E_NO_POLICY_SERVER,
            "CERTSRV_E_WEAK_SIGNATURE_OR_KEY" => SECURITY_SSPI::CERTSRV_E_WEAK_SIGNATURE_OR_KEY,
            "CERTSRV_E_KEY_ATTESTATION_NOT_SUPPORTED" => SECURITY_SSPI::CERTSRV_E_KEY_ATTESTATION_NOT_SUPPORTED,
            "CERTSRV_E_ENCRYPTION_CERT_REQUIRED" => SECURITY_SSPI::CERTSRV_E_ENCRYPTION_CERT_REQUIRED,
            "CERTSRV_E_UNSUPPORTED_CERT_TYPE" => SECURITY_SSPI::CERTSRV_E_UNSUPPORTED_CERT_TYPE,
            "CERTSRV_E_NO_CERT_TYPE" => SECURITY_SSPI::CERTSRV_E_NO_CERT_TYPE,
            "CERTSRV_E_TEMPLATE_CONFLICT" => SECURITY_SSPI::CERTSRV_E_TEMPLATE_CONFLICT,
            "CERTSRV_E_SUBJECT_ALT_NAME_REQUIRED" => SECURITY_SSPI::CERTSRV_E_SUBJECT_ALT_NAME_REQUIRED,
            "CERTSRV_E_ARCHIVED_KEY_REQUIRED" => SECURITY_SSPI::CERTSRV_E_ARCHIVED_KEY_REQUIRED,
            "CERTSRV_E_SMIME_REQUIRED" => SECURITY_SSPI::CERTSRV_E_SMIME_REQUIRED,
            "CERTSRV_E_BAD_RENEWAL_SUBJECT" => SECURITY_SSPI::CERTSRV_E_BAD_RENEWAL_SUBJECT,
            "CERTSRV_E_BAD_TEMPLATE_VERSION" => SECURITY_SSPI::CERTSRV_E_BAD_TEMPLATE_VERSION,
            "CERTSRV_E_TEMPLATE_POLICY_REQUIRED" => SECURITY_SSPI::CERTSRV_E_TEMPLATE_POLICY_REQUIRED,
            "CERTSRV_E_SIGNATURE_POLICY_REQUIRED" => SECURITY_SSPI::CERTSRV_E_SIGNATURE_POLICY_REQUIRED,
            "CERTSRV_E_SIGNATURE_COUNT" => SECURITY_SSPI::CERTSRV_E_SIGNATURE_COUNT,
            "CERTSRV_E_SIGNATURE_REJECTED" => SECURITY_SSPI::CERTSRV_E_SIGNATURE_REJECTED,
            "CERTSRV_E_ISSUANCE_POLICY_REQUIRED" => SECURITY_SSPI::CERTSRV_E_ISSUANCE_POLICY_REQUIRED,
            "CERTSRV_E_SUBJECT_UPN_REQUIRED" => SECURITY_SSPI::CERTSRV_E_SUBJECT_UPN_REQUIRED,
            "CERTSRV_E_SUBJECT_DIRECTORY_GUID_REQUIRED" => SECURITY_SSPI::CERTSRV_E_SUBJECT_DIRECTORY_GUID_REQUIRED,
            "CERTSRV_E_SUBJECT_DNS_REQUIRED" => SECURITY_SSPI::CERTSRV_E_SUBJECT_DNS_REQUIRED,
            "CERTSRV_E_ARCHIVED_KEY_UNEXPECTED" => SECURITY_SSPI::CERTSRV_E_ARCHIVED_KEY_UNEXPECTED,
            "CERTSRV_E_KEY_LENGTH" => SECURITY_SSPI::CERTSRV_E_KEY_LENGTH,
            "CERTSRV_E_SUBJECT_EMAIL_REQUIRED" => SECURITY_SSPI::CERTSRV_E_SUBJECT_EMAIL_REQUIRED,
            "CERTSRV_E_UNKNOWN_CERT_TYPE" => SECURITY_SSPI::CERTSRV_E_UNKNOWN_CERT_TYPE,
            "CERTSRV_E_CERT_TYPE_OVERLAP" => SECURITY_SSPI::CERTSRV_E_CERT_TYPE_OVERLAP,
            "CERTSRV_E_TOO_MANY_SIGNATURES" => SECURITY_SSPI::CERTSRV_E_TOO_MANY_SIGNATURES,
            "CERTSRV_E_RENEWAL_BAD_PUBLIC_KEY" => SECURITY_SSPI::CERTSRV_E_RENEWAL_BAD_PUBLIC_KEY,
            "CERTSRV_E_INVALID_EK" => SECURITY_SSPI::CERTSRV_E_INVALID_EK,
            "CERTSRV_E_INVALID_IDBINDING" => SECURITY_SSPI::CERTSRV_E_INVALID_IDBINDING,
            "CERTSRV_E_INVALID_ATTESTATION" => SECURITY_SSPI::CERTSRV_E_INVALID_ATTESTATION,
            "CERTSRV_E_KEY_ATTESTATION" => SECURITY_SSPI::CERTSRV_E_KEY_ATTESTATION,
            "CERTSRV_E_CORRUPT_KEY_ATTESTATION" => SECURITY_SSPI::CERTSRV_E_CORRUPT_KEY_ATTESTATION,
            "CERTSRV_E_EXPIRED_CHALLENGE" => SECURITY_SSPI::CERTSRV_E_EXPIRED_CHALLENGE,
            "CERTSRV_E_INVALID_RESPONSE" => SECURITY_SSPI::CERTSRV_E_INVALID_RESPONSE,
            "CERTSRV_E_INVALID_REQUESTID" => SECURITY_SSPI::CERTSRV_E_INVALID_REQUESTID,
            "CERTSRV_E_REQUEST_PRECERTIFICATE_MISMATCH" => SECURITY_SSPI::CERTSRV_E_REQUEST_PRECERTIFICATE_MISMATCH,
            "CERTSRV_E_PENDING_CLIENT_RESPONSE" => SECURITY_SSPI::CERTSRV_E_PENDING_CLIENT_RESPONSE,
            "CERTSRV_E_SEC_EXT_DIRECTORY_SID_REQUIRED" => SECURITY_SSPI::CERTSRV_E_SEC_EXT_DIRECTORY_SID_REQUIRED,
            "XENROLL_E_KEY_NOT_EXPORTABLE" => SECURITY_SSPI::XENROLL_E_KEY_NOT_EXPORTABLE,
            "XENROLL_E_CANNOT_ADD_ROOT_CERT" => SECURITY_SSPI::XENROLL_E_CANNOT_ADD_ROOT_CERT,
            "XENROLL_E_RESPONSE_KA_HASH_NOT_FOUND" => SECURITY_SSPI::XENROLL_E_RESPONSE_KA_HASH_NOT_FOUND,
            "XENROLL_E_RESPONSE_UNEXPECTED_KA_HASH" => SECURITY_SSPI::XENROLL_E_RESPONSE_UNEXPECTED_KA_HASH,
            "XENROLL_E_RESPONSE_KA_HASH_MISMATCH" => SECURITY_SSPI::XENROLL_E_RESPONSE_KA_HASH_MISMATCH,
            "XENROLL_E_KEYSPEC_SMIME_MISMATCH" => SECURITY_SSPI::XENROLL_E_KEYSPEC_SMIME_MISMATCH,
            "TRUST_E_SYSTEM_ERROR" => SECURITY_SSPI::TRUST_E_SYSTEM_ERROR,
            "TRUST_E_NO_SIGNER_CERT" => SECURITY_SSPI::TRUST_E_NO_SIGNER_CERT,
            "TRUST_E_COUNTER_SIGNER" => SECURITY_SSPI::TRUST_E_COUNTER_SIGNER,
            "TRUST_E_CERT_SIGNATURE" => SECURITY_SSPI::TRUST_E_CERT_SIGNATURE,
            "TRUST_E_TIME_STAMP" => SECURITY_SSPI::TRUST_E_TIME_STAMP,
            "TRUST_E_BAD_DIGEST" => SECURITY_SSPI::TRUST_E_BAD_DIGEST,
            "TRUST_E_MALFORMED_SIGNATURE" => SECURITY_SSPI::TRUST_E_MALFORMED_SIGNATURE,
            "TRUST_E_BASIC_CONSTRAINTS" => SECURITY_SSPI::TRUST_E_BASIC_CONSTRAINTS,
            "TRUST_E_FINANCIAL_CRITERIA" => SECURITY_SSPI::TRUST_E_FINANCIAL_CRITERIA,
            "MSSIPOTF_E_OUTOFMEMRANGE" => SECURITY_SSPI::MSSIPOTF_E_OUTOFMEMRANGE,
            "MSSIPOTF_E_CANTGETOBJECT" => SECURITY_SSPI::MSSIPOTF_E_CANTGETOBJECT,
            "MSSIPOTF_E_NOHEADTABLE" => SECURITY_SSPI::MSSIPOTF_E_NOHEADTABLE,
            "MSSIPOTF_E_BAD_MAGICNUMBER" => SECURITY_SSPI::MSSIPOTF_E_BAD_MAGICNUMBER,
            "MSSIPOTF_E_BAD_OFFSET_TABLE" => SECURITY_SSPI::MSSIPOTF_E_BAD_OFFSET_TABLE,
            "MSSIPOTF_E_TABLE_TAGORDER" => SECURITY_SSPI::MSSIPOTF_E_TABLE_TAGORDER,
            "MSSIPOTF_E_TABLE_LONGWORD" => SECURITY_SSPI::MSSIPOTF_E_TABLE_LONGWORD,
            "MSSIPOTF_E_BAD_FIRST_TABLE_PLACEMENT" => SECURITY_SSPI::MSSIPOTF_E_BAD_FIRST_TABLE_PLACEMENT,
            "MSSIPOTF_E_TABLES_OVERLAP" => SECURITY_SSPI::MSSIPOTF_E_TABLES_OVERLAP,
            "MSSIPOTF_E_TABLE_PADBYTES" => SECURITY_SSPI::MSSIPOTF_E_TABLE_PADBYTES,
            "MSSIPOTF_E_FILETOOSMALL" => SECURITY_SSPI::MSSIPOTF_E_FILETOOSMALL,
            "MSSIPOTF_E_TABLE_CHECKSUM" => SECURITY_SSPI::MSSIPOTF_E_TABLE_CHECKSUM,
            "MSSIPOTF_E_FILE_CHECKSUM" => SECURITY_SSPI::MSSIPOTF_E_FILE_CHECKSUM,
            "MSSIPOTF_E_FAILED_POLICY" => SECURITY_SSPI::MSSIPOTF_E_FAILED_POLICY,
            "MSSIPOTF_E_FAILED_HINTS_CHECK" => SECURITY_SSPI::MSSIPOTF_E_FAILED_HINTS_CHECK,
            "MSSIPOTF_E_NOT_OPENTYPE" => SECURITY_SSPI::MSSIPOTF_E_NOT_OPENTYPE,
            "MSSIPOTF_E_FILE" => SECURITY_SSPI::MSSIPOTF_E_FILE,
            "MSSIPOTF_E_CRYPT" => SECURITY_SSPI::MSSIPOTF_E_CRYPT,
            "MSSIPOTF_E_BADVERSION" => SECURITY_SSPI::MSSIPOTF_E_BADVERSION,
            "MSSIPOTF_E_DSIG_STRUCTURE" => SECURITY_SSPI::MSSIPOTF_E_DSIG_STRUCTURE,
            "MSSIPOTF_E_PCONST_CHECK" => SECURITY_SSPI::MSSIPOTF_E_PCONST_CHECK,
            "MSSIPOTF_E_STRUCTURE" => SECURITY_SSPI::MSSIPOTF_E_STRUCTURE,
            "ERROR_CRED_REQUIRES_CONFIRMATION" => SECURITY_SSPI::ERROR_CRED_REQUIRES_CONFIRMATION,
            "ERROR_AUDITING_DISABLED" => SECURITY_SSPI::ERROR_AUDITING_DISABLED,
            "ERROR_ALL_SIDS_FILTERED" => SECURITY_SSPI::ERROR_ALL_SIDS_FILTERED,
            "ERROR_BIZRULES_NOT_ENABLED" => SECURITY_SSPI::ERROR_BIZRULES_NOT_ENABLED,
        }
    }
    pub fn from_code(code: u32) -> SECURITY_SSPI {
        return match code {
            0x00090312 => SECURITY_SSPI::SEC_I_CONTINUE_NEEDED,
            0x00090313 => SECURITY_SSPI::SEC_I_COMPLETE_NEEDED,
            0x00090314 => SECURITY_SSPI::SEC_I_COMPLETE_AND_CONTINUE,
            0x00090315 => SECURITY_SSPI::SEC_I_LOCAL_LOGON,
            0x00090316 => SECURITY_SSPI::SEC_I_GENERIC_EXTENSION_RECEIVED,
            0x00090317 => SECURITY_SSPI::SEC_I_CONTEXT_EXPIRED,
            0x00090320 => SECURITY_SSPI::SEC_I_INCOMPLETE_CREDENTIALS,
            0x00090321 => SECURITY_SSPI::SEC_I_RENEGOTIATE,
            0x00090323 => SECURITY_SSPI::SEC_I_NO_LSA_CONTEXT,
            0x0009035C => SECURITY_SSPI::SEC_I_SIGNATURE_NEEDED,
            0x00090360 => SECURITY_SSPI::SEC_I_NO_RENEGOTIATION,
            0x00090364 => SECURITY_SSPI::SEC_I_MESSAGE_FRAGMENT,
            0x00090366 => SECURITY_SSPI::SEC_I_CONTINUE_NEEDED_MESSAGE_OK,
            0x00090368 => SECURITY_SSPI::SEC_I_ASYNC_CALL_PENDING,
            0x00091012 => SECURITY_SSPI::CRYPT_I_NEW_PROTECTION_REQUIRED,
            0x80090001 => SECURITY_SSPI::NTE_BAD_UID,
            0x80090002 => SECURITY_SSPI::NTE_BAD_HASH,
            0x80090003 => SECURITY_SSPI::NTE_BAD_KEY,
            0x80090004 => SECURITY_SSPI::NTE_BAD_LEN,
            0x80090005 => SECURITY_SSPI::NTE_BAD_DATA,
            0x80090006 => SECURITY_SSPI::NTE_BAD_SIGNATURE,
            0x80090007 => SECURITY_SSPI::NTE_BAD_VER,
            0x80090008 => SECURITY_SSPI::NTE_BAD_ALGID,
            0x80090009 => SECURITY_SSPI::NTE_BAD_FLAGS,
            0x8009000A => SECURITY_SSPI::NTE_BAD_TYPE,
            0x8009000B => SECURITY_SSPI::NTE_BAD_KEY_STATE,
            0x8009000C => SECURITY_SSPI::NTE_BAD_HASH_STATE,
            0x8009000D => SECURITY_SSPI::NTE_NO_KEY,
            0x8009000E => SECURITY_SSPI::NTE_NO_MEMORY,
            0x8009000F => SECURITY_SSPI::NTE_EXISTS,
            0x80090010 => SECURITY_SSPI::NTE_PERM,
            0x80090011 => SECURITY_SSPI::NTE_NOT_FOUND,
            0x80090012 => SECURITY_SSPI::NTE_DOUBLE_ENCRYPT,
            0x80090013 => SECURITY_SSPI::NTE_BAD_PROVIDER,
            0x80090014 => SECURITY_SSPI::NTE_BAD_PROV_TYPE,
            0x80090015 => SECURITY_SSPI::NTE_BAD_PUBLIC_KEY,
            0x80090016 => SECURITY_SSPI::NTE_BAD_KEYSET,
            0x80090017 => SECURITY_SSPI::NTE_PROV_TYPE_NOT_DEF,
            0x80090018 => SECURITY_SSPI::NTE_PROV_TYPE_ENTRY_BAD,
            0x80090019 => SECURITY_SSPI::NTE_KEYSET_NOT_DEF,
            0x8009001A => SECURITY_SSPI::NTE_KEYSET_ENTRY_BAD,
            0x8009001B => SECURITY_SSPI::NTE_PROV_TYPE_NO_MATCH,
            0x8009001C => SECURITY_SSPI::NTE_SIGNATURE_FILE_BAD,
            0x8009001D => SECURITY_SSPI::NTE_PROVIDER_DLL_FAIL,
            0x8009001E => SECURITY_SSPI::NTE_PROV_DLL_NOT_FOUND,
            0x8009001F => SECURITY_SSPI::NTE_BAD_KEYSET_PARAM,
            0x80090020 => SECURITY_SSPI::NTE_FAIL,
            0x80090021 => SECURITY_SSPI::NTE_SYS_ERR,
            0x80090022 => SECURITY_SSPI::NTE_SILENT_CONTEXT,
            0x80090023 => SECURITY_SSPI::NTE_TOKEN_KEYSET_STORAGE_FULL,
            0x80090024 => SECURITY_SSPI::NTE_TEMPORARY_PROFILE,
            0x80090025 => SECURITY_SSPI::NTE_FIXEDPARAMETER,
            0x80090026 => SECURITY_SSPI::NTE_INVALID_HANDLE,
            0x80090027 => SECURITY_SSPI::NTE_INVALID_PARAMETER,
            0x80090028 => SECURITY_SSPI::NTE_BUFFER_TOO_SMALL,
            0x80090029 => SECURITY_SSPI::NTE_NOT_SUPPORTED,
            0x8009002A => SECURITY_SSPI::NTE_NO_MORE_ITEMS,
            0x8009002B => SECURITY_SSPI::NTE_BUFFERS_OVERLAP,
            0x8009002C => SECURITY_SSPI::NTE_DECRYPTION_FAILURE,
            0x8009002D => SECURITY_SSPI::NTE_INTERNAL_ERROR,
            0x8009002E => SECURITY_SSPI::NTE_UI_REQUIRED,
            0x8009002F => SECURITY_SSPI::NTE_HMAC_NOT_SUPPORTED,
            0x80090030 => SECURITY_SSPI::NTE_DEVICE_NOT_READY,
            0x80090031 => SECURITY_SSPI::NTE_AUTHENTICATION_IGNORED,
            0x80090032 => SECURITY_SSPI::NTE_VALIDATION_FAILED,
            0x80090033 => SECURITY_SSPI::NTE_INCORRECT_PASSWORD,
            0x80090034 => SECURITY_SSPI::NTE_ENCRYPTION_FAILURE,
            0x80090035 => SECURITY_SSPI::NTE_DEVICE_NOT_FOUND,
            0x80090036 => SECURITY_SSPI::NTE_USER_CANCELLED,
            0x80090037 => SECURITY_SSPI::NTE_PASSWORD_CHANGE_REQUIRED,
            0x80090038 => SECURITY_SSPI::NTE_NOT_ACTIVE_CONSOLE,
            0x80090300 => SECURITY_SSPI::SEC_E_INSUFFICIENT_MEMORY,
            0x80090301 => SECURITY_SSPI::SEC_E_INVALID_HANDLE,
            0x80090302 => SECURITY_SSPI::SEC_E_UNSUPPORTED_FUNCTION,
            0x80090303 => SECURITY_SSPI::SEC_E_TARGET_UNKNOWN,
            0x80090304 => SECURITY_SSPI::SEC_E_INTERNAL_ERROR,
            0x80090305 => SECURITY_SSPI::SEC_E_SECPKG_NOT_FOUND,
            0x80090306 => SECURITY_SSPI::SEC_E_NOT_OWNER,
            0x80090307 => SECURITY_SSPI::SEC_E_CANNOT_INSTALL,
            0x80090308 => SECURITY_SSPI::SEC_E_INVALID_TOKEN,
            0x80090309 => SECURITY_SSPI::SEC_E_CANNOT_PACK,
            0x8009030A => SECURITY_SSPI::SEC_E_QOP_NOT_SUPPORTED,
            0x8009030B => SECURITY_SSPI::SEC_E_NO_IMPERSONATION,
            0x8009030C => SECURITY_SSPI::SEC_E_LOGON_DENIED,
            0x8009030D => SECURITY_SSPI::SEC_E_UNKNOWN_CREDENTIALS,
            0x8009030E => SECURITY_SSPI::SEC_E_NO_CREDENTIALS,
            0x8009030F => SECURITY_SSPI::SEC_E_MESSAGE_ALTERED,
            0x80090310 => SECURITY_SSPI::SEC_E_OUT_OF_SEQUENCE,
            0x80090311 => SECURITY_SSPI::SEC_E_NO_AUTHENTICATING_AUTHORITY,
            0x80090316 => SECURITY_SSPI::SEC_E_BAD_PKGID,
            0x80090317 => SECURITY_SSPI::SEC_E_CONTEXT_EXPIRED,
            0x80090318 => SECURITY_SSPI::SEC_E_INCOMPLETE_MESSAGE,
            0x80090320 => SECURITY_SSPI::SEC_E_INCOMPLETE_CREDENTIALS,
            0x80090321 => SECURITY_SSPI::SEC_E_BUFFER_TOO_SMALL,
            0x80090322 => SECURITY_SSPI::SEC_E_WRONG_PRINCIPAL,
            0x80090324 => SECURITY_SSPI::SEC_E_TIME_SKEW,
            0x80090325 => SECURITY_SSPI::SEC_E_UNTRUSTED_ROOT,
            0x80090326 => SECURITY_SSPI::SEC_E_ILLEGAL_MESSAGE,
            0x80090327 => SECURITY_SSPI::SEC_E_CERT_UNKNOWN,
            0x80090328 => SECURITY_SSPI::SEC_E_CERT_EXPIRED,
            0x80090329 => SECURITY_SSPI::SEC_E_ENCRYPT_FAILURE,
            0x80090330 => SECURITY_SSPI::SEC_E_DECRYPT_FAILURE,
            0x80090331 => SECURITY_SSPI::SEC_E_ALGORITHM_MISMATCH,
            0x80090332 => SECURITY_SSPI::SEC_E_SECURITY_QOS_FAILED,
            0x80090333 => SECURITY_SSPI::SEC_E_UNFINISHED_CONTEXT_DELETED,
            0x80090334 => SECURITY_SSPI::SEC_E_NO_TGT_REPLY,
            0x80090335 => SECURITY_SSPI::SEC_E_NO_IP_ADDRESSES,
            0x80090336 => SECURITY_SSPI::SEC_E_WRONG_CREDENTIAL_HANDLE,
            0x80090337 => SECURITY_SSPI::SEC_E_CRYPTO_SYSTEM_INVALID,
            0x80090338 => SECURITY_SSPI::SEC_E_MAX_REFERRALS_EXCEEDED,
            0x80090339 => SECURITY_SSPI::SEC_E_MUST_BE_KDC,
            0x8009033A => SECURITY_SSPI::SEC_E_STRONG_CRYPTO_NOT_SUPPORTED,
            0x8009033B => SECURITY_SSPI::SEC_E_TOO_MANY_PRINCIPALS,
            0x8009033C => SECURITY_SSPI::SEC_E_NO_PA_DATA,
            0x8009033D => SECURITY_SSPI::SEC_E_PKINIT_NAME_MISMATCH,
            0x8009033E => SECURITY_SSPI::SEC_E_SMARTCARD_LOGON_REQUIRED,
            0x8009033F => SECURITY_SSPI::SEC_E_SHUTDOWN_IN_PROGRESS,
            0x80090340 => SECURITY_SSPI::SEC_E_KDC_INVALID_REQUEST,
            0x80090341 => SECURITY_SSPI::SEC_E_KDC_UNABLE_TO_REFER,
            0x80090342 => SECURITY_SSPI::SEC_E_KDC_UNKNOWN_ETYPE,
            0x80090343 => SECURITY_SSPI::SEC_E_UNSUPPORTED_PREAUTH,
            0x80090345 => SECURITY_SSPI::SEC_E_DELEGATION_REQUIRED,
            0x80090346 => SECURITY_SSPI::SEC_E_BAD_BINDINGS,
            0x80090347 => SECURITY_SSPI::SEC_E_MULTIPLE_ACCOUNTS,
            0x80090348 => SECURITY_SSPI::SEC_E_NO_KERB_KEY,
            0x80090349 => SECURITY_SSPI::SEC_E_CERT_WRONG_USAGE,
            0x80090350 => SECURITY_SSPI::SEC_E_DOWNGRADE_DETECTED,
            0x80090351 => SECURITY_SSPI::SEC_E_SMARTCARD_CERT_REVOKED,
            0x80090352 => SECURITY_SSPI::SEC_E_ISSUING_CA_UNTRUSTED,
            0x80090353 => SECURITY_SSPI::SEC_E_REVOCATION_OFFLINE_C,
            0x80090354 => SECURITY_SSPI::SEC_E_PKINIT_CLIENT_FAILURE,
            0x80090355 => SECURITY_SSPI::SEC_E_SMARTCARD_CERT_EXPIRED,
            0x80090356 => SECURITY_SSPI::SEC_E_NO_S4U_PROT_SUPPORT,
            0x80090357 => SECURITY_SSPI::SEC_E_CROSSREALM_DELEGATION_FAILURE,
            0x80090358 => SECURITY_SSPI::SEC_E_REVOCATION_OFFLINE_KDC,
            0x80090359 => SECURITY_SSPI::SEC_E_ISSUING_CA_UNTRUSTED_KDC,
            0x8009035A => SECURITY_SSPI::SEC_E_KDC_CERT_EXPIRED,
            0x8009035B => SECURITY_SSPI::SEC_E_KDC_CERT_REVOKED,
            0x8009035D => SECURITY_SSPI::SEC_E_INVALID_PARAMETER,
            0x8009035E => SECURITY_SSPI::SEC_E_DELEGATION_POLICY,
            0x8009035F => SECURITY_SSPI::SEC_E_POLICY_NLTM_ONLY,
            0x80090361 => SECURITY_SSPI::SEC_E_NO_CONTEXT,
            0x80090362 => SECURITY_SSPI::SEC_E_PKU2U_CERT_FAILURE,
            0x80090363 => SECURITY_SSPI::SEC_E_MUTUAL_AUTH_FAILED,
            0x80090365 => SECURITY_SSPI::SEC_E_ONLY_HTTPS_ALLOWED,
            0x80090367 => SECURITY_SSPI::SEC_E_APPLICATION_PROTOCOL_MISMATCH,
            0x80090369 => SECURITY_SSPI::SEC_E_INVALID_UPN_NAME,
            0x8009036A => SECURITY_SSPI::SEC_E_EXT_BUFFER_TOO_SMALL,
            0x8009036B => SECURITY_SSPI::SEC_E_INSUFFICIENT_BUFFERS,
            0x80091001 => SECURITY_SSPI::CRYPT_E_MSG_ERROR,
            0x80091002 => SECURITY_SSPI::CRYPT_E_UNKNOWN_ALGO,
            0x80091003 => SECURITY_SSPI::CRYPT_E_OID_FORMAT,
            0x80091004 => SECURITY_SSPI::CRYPT_E_INVALID_MSG_TYPE,
            0x80091005 => SECURITY_SSPI::CRYPT_E_UNEXPECTED_ENCODING,
            0x80091006 => SECURITY_SSPI::CRYPT_E_AUTH_ATTR_MISSING,
            0x80091007 => SECURITY_SSPI::CRYPT_E_HASH_VALUE,
            0x80091008 => SECURITY_SSPI::CRYPT_E_INVALID_INDEX,
            0x80091009 => SECURITY_SSPI::CRYPT_E_ALREADY_DECRYPTED,
            0x8009100A => SECURITY_SSPI::CRYPT_E_NOT_DECRYPTED,
            0x8009100B => SECURITY_SSPI::CRYPT_E_RECIPIENT_NOT_FOUND,
            0x8009100C => SECURITY_SSPI::CRYPT_E_CONTROL_TYPE,
            0x8009100D => SECURITY_SSPI::CRYPT_E_ISSUER_SERIALNUMBER,
            0x8009100E => SECURITY_SSPI::CRYPT_E_SIGNER_NOT_FOUND,
            0x8009100F => SECURITY_SSPI::CRYPT_E_ATTRIBUTES_MISSING,
            0x80091010 => SECURITY_SSPI::CRYPT_E_STREAM_MSG_NOT_READY,
            0x80091011 => SECURITY_SSPI::CRYPT_E_STREAM_INSUFFICIENT_DATA,
            0x80092001 => SECURITY_SSPI::CRYPT_E_BAD_LEN,
            0x80092002 => SECURITY_SSPI::CRYPT_E_BAD_ENCODE,
            0x80092003 => SECURITY_SSPI::CRYPT_E_FILE_ERROR,
            0x80092004 => SECURITY_SSPI::CRYPT_E_NOT_FOUND,
            0x80092005 => SECURITY_SSPI::CRYPT_E_EXISTS,
            0x80092006 => SECURITY_SSPI::CRYPT_E_NO_PROVIDER,
            0x80092007 => SECURITY_SSPI::CRYPT_E_SELF_SIGNED,
            0x80092008 => SECURITY_SSPI::CRYPT_E_DELETED_PREV,
            0x80092009 => SECURITY_SSPI::CRYPT_E_NO_MATCH,
            0x8009200A => SECURITY_SSPI::CRYPT_E_UNEXPECTED_MSG_TYPE,
            0x8009200B => SECURITY_SSPI::CRYPT_E_NO_KEY_PROPERTY,
            0x8009200C => SECURITY_SSPI::CRYPT_E_NO_DECRYPT_CERT,
            0x8009200D => SECURITY_SSPI::CRYPT_E_BAD_MSG,
            0x8009200E => SECURITY_SSPI::CRYPT_E_NO_SIGNER,
            0x8009200F => SECURITY_SSPI::CRYPT_E_PENDING_CLOSE,
            0x80092010 => SECURITY_SSPI::CRYPT_E_REVOKED,
            0x80092011 => SECURITY_SSPI::CRYPT_E_NO_REVOCATION_DLL,
            0x80092012 => SECURITY_SSPI::CRYPT_E_NO_REVOCATION_CHECK,
            0x80092013 => SECURITY_SSPI::CRYPT_E_REVOCATION_OFFLINE,
            0x80092014 => SECURITY_SSPI::CRYPT_E_NOT_IN_REVOCATION_DATABASE,
            0x80092020 => SECURITY_SSPI::CRYPT_E_INVALID_NUMERIC_STRING,
            0x80092021 => SECURITY_SSPI::CRYPT_E_INVALID_PRINTABLE_STRING,
            0x80092022 => SECURITY_SSPI::CRYPT_E_INVALID_IA5_STRING,
            0x80092023 => SECURITY_SSPI::CRYPT_E_INVALID_X500_STRING,
            0x80092024 => SECURITY_SSPI::CRYPT_E_NOT_CHAR_STRING,
            0x80092025 => SECURITY_SSPI::CRYPT_E_FILERESIZED,
            0x80092026 => SECURITY_SSPI::CRYPT_E_SECURITY_SETTINGS,
            0x80092027 => SECURITY_SSPI::CRYPT_E_NO_VERIFY_USAGE_DLL,
            0x80092028 => SECURITY_SSPI::CRYPT_E_NO_VERIFY_USAGE_CHECK,
            0x80092029 => SECURITY_SSPI::CRYPT_E_VERIFY_USAGE_OFFLINE,
            0x8009202A => SECURITY_SSPI::CRYPT_E_NOT_IN_CTL,
            0x8009202B => SECURITY_SSPI::CRYPT_E_NO_TRUSTED_SIGNER,
            0x8009202C => SECURITY_SSPI::CRYPT_E_MISSING_PUBKEY_PARA,
            0x8009202D => SECURITY_SSPI::CRYPT_E_OBJECT_LOCATOR_OBJECT_NOT_FOUND,
            0x80093000 => SECURITY_SSPI::CRYPT_E_OSS_ERROR,
            0x80093001 => SECURITY_SSPI::OSS_MORE_BUF,
            0x80093002 => SECURITY_SSPI::OSS_NEGATIVE_UINTEGER,
            0x80093003 => SECURITY_SSPI::OSS_PDU_RANGE,
            0x80093004 => SECURITY_SSPI::OSS_MORE_INPUT,
            0x80093005 => SECURITY_SSPI::OSS_DATA_ERROR,
            0x80093006 => SECURITY_SSPI::OSS_BAD_ARG,
            0x80093007 => SECURITY_SSPI::OSS_BAD_VERSION,
            0x80093008 => SECURITY_SSPI::OSS_OUT_MEMORY,
            0x80093009 => SECURITY_SSPI::OSS_PDU_MISMATCH,
            0x8009300A => SECURITY_SSPI::OSS_LIMITED,
            0x8009300B => SECURITY_SSPI::OSS_BAD_PTR,
            0x8009300C => SECURITY_SSPI::OSS_BAD_TIME,
            0x8009300D => SECURITY_SSPI::OSS_INDEFINITE_NOT_SUPPORTED,
            0x8009300E => SECURITY_SSPI::OSS_MEM_ERROR,
            0x8009300F => SECURITY_SSPI::OSS_BAD_TABLE,
            0x80093010 => SECURITY_SSPI::OSS_TOO_LONG,
            0x80093011 => SECURITY_SSPI::OSS_CONSTRAINT_VIOLATED,
            0x80093012 => SECURITY_SSPI::OSS_FATAL_ERROR,
            0x80093013 => SECURITY_SSPI::OSS_ACCESS_SERIALIZATION_ERROR,
            0x80093014 => SECURITY_SSPI::OSS_NULL_TBL,
            0x80093015 => SECURITY_SSPI::OSS_NULL_FCN,
            0x80093016 => SECURITY_SSPI::OSS_BAD_ENCRULES,
            0x80093017 => SECURITY_SSPI::OSS_UNAVAIL_ENCRULES,
            0x80093018 => SECURITY_SSPI::OSS_CANT_OPEN_TRACE_WINDOW,
            0x80093019 => SECURITY_SSPI::OSS_UNIMPLEMENTED,
            0x8009301A => SECURITY_SSPI::OSS_OID_DLL_NOT_LINKED,
            0x8009301B => SECURITY_SSPI::OSS_CANT_OPEN_TRACE_FILE,
            0x8009301C => SECURITY_SSPI::OSS_TRACE_FILE_ALREADY_OPEN,
            0x8009301D => SECURITY_SSPI::OSS_TABLE_MISMATCH,
            0x8009301E => SECURITY_SSPI::OSS_TYPE_NOT_SUPPORTED,
            0x8009301F => SECURITY_SSPI::OSS_REAL_DLL_NOT_LINKED,
            0x80093020 => SECURITY_SSPI::OSS_REAL_CODE_NOT_LINKED,
            0x80093021 => SECURITY_SSPI::OSS_OUT_OF_RANGE,
            0x80093022 => SECURITY_SSPI::OSS_COPIER_DLL_NOT_LINKED,
            0x80093023 => SECURITY_SSPI::OSS_CONSTRAINT_DLL_NOT_LINKED,
            0x80093024 => SECURITY_SSPI::OSS_COMPARATOR_DLL_NOT_LINKED,
            0x80093025 => SECURITY_SSPI::OSS_COMPARATOR_CODE_NOT_LINKED,
            0x80093026 => SECURITY_SSPI::OSS_MEM_MGR_DLL_NOT_LINKED,
            0x80093027 => SECURITY_SSPI::OSS_PDV_DLL_NOT_LINKED,
            0x80093028 => SECURITY_SSPI::OSS_PDV_CODE_NOT_LINKED,
            0x80093029 => SECURITY_SSPI::OSS_API_DLL_NOT_LINKED,
            0x8009302A => SECURITY_SSPI::OSS_BERDER_DLL_NOT_LINKED,
            0x8009302B => SECURITY_SSPI::OSS_PER_DLL_NOT_LINKED,
            0x8009302C => SECURITY_SSPI::OSS_OPEN_TYPE_ERROR,
            0x8009302D => SECURITY_SSPI::OSS_MUTEX_NOT_CREATED,
            0x8009302E => SECURITY_SSPI::OSS_CANT_CLOSE_TRACE_FILE,
            0x80093100 => SECURITY_SSPI::CRYPT_E_ASN1_ERROR,
            0x80093101 => SECURITY_SSPI::CRYPT_E_ASN1_INTERNAL,
            0x80093102 => SECURITY_SSPI::CRYPT_E_ASN1_EOD,
            0x80093103 => SECURITY_SSPI::CRYPT_E_ASN1_CORRUPT,
            0x80093104 => SECURITY_SSPI::CRYPT_E_ASN1_LARGE,
            0x80093105 => SECURITY_SSPI::CRYPT_E_ASN1_CONSTRAINT,
            0x80093106 => SECURITY_SSPI::CRYPT_E_ASN1_MEMORY,
            0x80093107 => SECURITY_SSPI::CRYPT_E_ASN1_OVERFLOW,
            0x80093108 => SECURITY_SSPI::CRYPT_E_ASN1_BADPDU,
            0x80093109 => SECURITY_SSPI::CRYPT_E_ASN1_BADARGS,
            0x8009310A => SECURITY_SSPI::CRYPT_E_ASN1_BADREAL,
            0x8009310B => SECURITY_SSPI::CRYPT_E_ASN1_BADTAG,
            0x8009310C => SECURITY_SSPI::CRYPT_E_ASN1_CHOICE,
            0x8009310D => SECURITY_SSPI::CRYPT_E_ASN1_RULE,
            0x8009310E => SECURITY_SSPI::CRYPT_E_ASN1_UTF8,
            0x80093133 => SECURITY_SSPI::CRYPT_E_ASN1_PDU_TYPE,
            0x80093134 => SECURITY_SSPI::CRYPT_E_ASN1_NYI,
            0x80093201 => SECURITY_SSPI::CRYPT_E_ASN1_EXTENDED,
            0x80093202 => SECURITY_SSPI::CRYPT_E_ASN1_NOEOD,
            0x80094001 => SECURITY_SSPI::CERTSRV_E_BAD_REQUESTSUBJECT,
            0x80094002 => SECURITY_SSPI::CERTSRV_E_NO_REQUEST,
            0x80094003 => SECURITY_SSPI::CERTSRV_E_BAD_REQUESTSTATUS,
            0x80094004 => SECURITY_SSPI::CERTSRV_E_PROPERTY_EMPTY,
            0x80094005 => SECURITY_SSPI::CERTSRV_E_INVALID_CA_CERTIFICATE,
            0x80094006 => SECURITY_SSPI::CERTSRV_E_SERVER_SUSPENDED,
            0x80094007 => SECURITY_SSPI::CERTSRV_E_ENCODING_LENGTH,
            0x80094008 => SECURITY_SSPI::CERTSRV_E_ROLECONFLICT,
            0x80094009 => SECURITY_SSPI::CERTSRV_E_RESTRICTEDOFFICER,
            0x8009400A => SECURITY_SSPI::CERTSRV_E_KEY_ARCHIVAL_NOT_CONFIGURED,
            0x8009400B => SECURITY_SSPI::CERTSRV_E_NO_VALID_KRA,
            0x8009400C => SECURITY_SSPI::CERTSRV_E_BAD_REQUEST_KEY_ARCHIVAL,
            0x8009400D => SECURITY_SSPI::CERTSRV_E_NO_CAADMIN_DEFINED,
            0x8009400E => SECURITY_SSPI::CERTSRV_E_BAD_RENEWAL_CERT_ATTRIBUTE,
            0x8009400F => SECURITY_SSPI::CERTSRV_E_NO_DB_SESSIONS,
            0x80094010 => SECURITY_SSPI::CERTSRV_E_ALIGNMENT_FAULT,
            0x80094011 => SECURITY_SSPI::CERTSRV_E_ENROLL_DENIED,
            0x80094012 => SECURITY_SSPI::CERTSRV_E_TEMPLATE_DENIED,
            0x80094013 => SECURITY_SSPI::CERTSRV_E_DOWNLEVEL_DC_SSL_OR_UPGRADE,
            0x80094014 => SECURITY_SSPI::CERTSRV_E_ADMIN_DENIED_REQUEST,
            0x80094015 => SECURITY_SSPI::CERTSRV_E_NO_POLICY_SERVER,
            0x80094016 => SECURITY_SSPI::CERTSRV_E_WEAK_SIGNATURE_OR_KEY,
            0x80094017 => SECURITY_SSPI::CERTSRV_E_KEY_ATTESTATION_NOT_SUPPORTED,
            0x80094018 => SECURITY_SSPI::CERTSRV_E_ENCRYPTION_CERT_REQUIRED,
            0x80094800 => SECURITY_SSPI::CERTSRV_E_UNSUPPORTED_CERT_TYPE,
            0x80094801 => SECURITY_SSPI::CERTSRV_E_NO_CERT_TYPE,
            0x80094802 => SECURITY_SSPI::CERTSRV_E_TEMPLATE_CONFLICT,
            0x80094803 => SECURITY_SSPI::CERTSRV_E_SUBJECT_ALT_NAME_REQUIRED,
            0x80094804 => SECURITY_SSPI::CERTSRV_E_ARCHIVED_KEY_REQUIRED,
            0x80094805 => SECURITY_SSPI::CERTSRV_E_SMIME_REQUIRED,
            0x80094806 => SECURITY_SSPI::CERTSRV_E_BAD_RENEWAL_SUBJECT,
            0x80094807 => SECURITY_SSPI::CERTSRV_E_BAD_TEMPLATE_VERSION,
            0x80094808 => SECURITY_SSPI::CERTSRV_E_TEMPLATE_POLICY_REQUIRED,
            0x80094809 => SECURITY_SSPI::CERTSRV_E_SIGNATURE_POLICY_REQUIRED,
            0x8009480A => SECURITY_SSPI::CERTSRV_E_SIGNATURE_COUNT,
            0x8009480B => SECURITY_SSPI::CERTSRV_E_SIGNATURE_REJECTED,
            0x8009480C => SECURITY_SSPI::CERTSRV_E_ISSUANCE_POLICY_REQUIRED,
            0x8009480D => SECURITY_SSPI::CERTSRV_E_SUBJECT_UPN_REQUIRED,
            0x8009480E => SECURITY_SSPI::CERTSRV_E_SUBJECT_DIRECTORY_GUID_REQUIRED,
            0x8009480F => SECURITY_SSPI::CERTSRV_E_SUBJECT_DNS_REQUIRED,
            0x80094810 => SECURITY_SSPI::CERTSRV_E_ARCHIVED_KEY_UNEXPECTED,
            0x80094811 => SECURITY_SSPI::CERTSRV_E_KEY_LENGTH,
            0x80094812 => SECURITY_SSPI::CERTSRV_E_SUBJECT_EMAIL_REQUIRED,
            0x80094813 => SECURITY_SSPI::CERTSRV_E_UNKNOWN_CERT_TYPE,
            0x80094814 => SECURITY_SSPI::CERTSRV_E_CERT_TYPE_OVERLAP,
            0x80094815 => SECURITY_SSPI::CERTSRV_E_TOO_MANY_SIGNATURES,
            0x80094816 => SECURITY_SSPI::CERTSRV_E_RENEWAL_BAD_PUBLIC_KEY,
            0x80094817 => SECURITY_SSPI::CERTSRV_E_INVALID_EK,
            0x80094818 => SECURITY_SSPI::CERTSRV_E_INVALID_IDBINDING,
            0x80094819 => SECURITY_SSPI::CERTSRV_E_INVALID_ATTESTATION,
            0x8009481A => SECURITY_SSPI::CERTSRV_E_KEY_ATTESTATION,
            0x8009481B => SECURITY_SSPI::CERTSRV_E_CORRUPT_KEY_ATTESTATION,
            0x8009481C => SECURITY_SSPI::CERTSRV_E_EXPIRED_CHALLENGE,
            0x8009481D => SECURITY_SSPI::CERTSRV_E_INVALID_RESPONSE,
            0x8009481E => SECURITY_SSPI::CERTSRV_E_INVALID_REQUESTID,
            0x8009481F => SECURITY_SSPI::CERTSRV_E_REQUEST_PRECERTIFICATE_MISMATCH,
            0x80094820 => SECURITY_SSPI::CERTSRV_E_PENDING_CLIENT_RESPONSE,
            0x80094821 => SECURITY_SSPI::CERTSRV_E_SEC_EXT_DIRECTORY_SID_REQUIRED,
            0x80095000 => SECURITY_SSPI::XENROLL_E_KEY_NOT_EXPORTABLE,
            0x80095001 => SECURITY_SSPI::XENROLL_E_CANNOT_ADD_ROOT_CERT,
            0x80095002 => SECURITY_SSPI::XENROLL_E_RESPONSE_KA_HASH_NOT_FOUND,
            0x80095003 => SECURITY_SSPI::XENROLL_E_RESPONSE_UNEXPECTED_KA_HASH,
            0x80095004 => SECURITY_SSPI::XENROLL_E_RESPONSE_KA_HASH_MISMATCH,
            0x80095005 => SECURITY_SSPI::XENROLL_E_KEYSPEC_SMIME_MISMATCH,
            0x80096001 => SECURITY_SSPI::TRUST_E_SYSTEM_ERROR,
            0x80096002 => SECURITY_SSPI::TRUST_E_NO_SIGNER_CERT,
            0x80096003 => SECURITY_SSPI::TRUST_E_COUNTER_SIGNER,
            0x80096004 => SECURITY_SSPI::TRUST_E_CERT_SIGNATURE,
            0x80096005 => SECURITY_SSPI::TRUST_E_TIME_STAMP,
            0x80096010 => SECURITY_SSPI::TRUST_E_BAD_DIGEST,
            0x80096011 => SECURITY_SSPI::TRUST_E_MALFORMED_SIGNATURE,
            0x80096019 => SECURITY_SSPI::TRUST_E_BASIC_CONSTRAINTS,
            0x8009601E => SECURITY_SSPI::TRUST_E_FINANCIAL_CRITERIA,
            0x80097001 => SECURITY_SSPI::MSSIPOTF_E_OUTOFMEMRANGE,
            0x80097002 => SECURITY_SSPI::MSSIPOTF_E_CANTGETOBJECT,
            0x80097003 => SECURITY_SSPI::MSSIPOTF_E_NOHEADTABLE,
            0x80097004 => SECURITY_SSPI::MSSIPOTF_E_BAD_MAGICNUMBER,
            0x80097005 => SECURITY_SSPI::MSSIPOTF_E_BAD_OFFSET_TABLE,
            0x80097006 => SECURITY_SSPI::MSSIPOTF_E_TABLE_TAGORDER,
            0x80097007 => SECURITY_SSPI::MSSIPOTF_E_TABLE_LONGWORD,
            0x80097008 => SECURITY_SSPI::MSSIPOTF_E_BAD_FIRST_TABLE_PLACEMENT,
            0x80097009 => SECURITY_SSPI::MSSIPOTF_E_TABLES_OVERLAP,
            0x8009700A => SECURITY_SSPI::MSSIPOTF_E_TABLE_PADBYTES,
            0x8009700B => SECURITY_SSPI::MSSIPOTF_E_FILETOOSMALL,
            0x8009700C => SECURITY_SSPI::MSSIPOTF_E_TABLE_CHECKSUM,
            0x8009700D => SECURITY_SSPI::MSSIPOTF_E_FILE_CHECKSUM,
            0x80097010 => SECURITY_SSPI::MSSIPOTF_E_FAILED_POLICY,
            0x80097011 => SECURITY_SSPI::MSSIPOTF_E_FAILED_HINTS_CHECK,
            0x80097012 => SECURITY_SSPI::MSSIPOTF_E_NOT_OPENTYPE,
            0x80097013 => SECURITY_SSPI::MSSIPOTF_E_FILE,
            0x80097014 => SECURITY_SSPI::MSSIPOTF_E_CRYPT,
            0x80097015 => SECURITY_SSPI::MSSIPOTF_E_BADVERSION,
            0x80097016 => SECURITY_SSPI::MSSIPOTF_E_DSIG_STRUCTURE,
            0x80097017 => SECURITY_SSPI::MSSIPOTF_E_PCONST_CHECK,
            0x80097018 => SECURITY_SSPI::MSSIPOTF_E_STRUCTURE,
            0x80097019 => SECURITY_SSPI::ERROR_CRED_REQUIRES_CONFIRMATION,
            0xC0090001 => SECURITY_SSPI::ERROR_AUDITING_DISABLED,
            0xC0090002 => SECURITY_SSPI::ERROR_ALL_SIDS_FILTERED,
            0xC0090003 => SECURITY_SSPI::ERROR_BIZRULES_NOT_ENABLED,
        }
    }
}
