/*
 * Copyright 2023 Stanislav Mikhailov (xavetar)
 *
 * Licensed under the Creative Commons Zero v1.0 Universal (CC0) License.
 * You may obtain a copy of the License at
 *
 *     http://creativecommons.org/publicdomain/zero/1.0/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the CC0 license is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// It is strictly forbidden to use the from_code method, respected Microsoft decided to define
// duplicate codes, but by defining different constant names, therefore, when using these codes,
// it is mandatory to use through from_name, and not from_code. Otherwise, it may cause undefined
// behavior or an unknown exception. Because one code corresponds to several constants.

use super::{RawError};

#[derive(Clone, Debug, Copy, Eq, Hash, Ord, PartialEq, PartialOrd)]
pub enum URT {
    VLDTR_S_WRN,
    VLDTR_S_ERR,
    VLDTR_S_WRNERR,
    CORDBG_S_BAD_START_SEQUENCE_POINT,
    CORDBG_S_BAD_END_SEQUENCE_POINT,
    CORDBG_S_INSUFFICIENT_INFO_FOR_SET_IP,
    CORDBG_S_FUNC_EVAL_HAS_NO_RESULT,
    CORDBG_S_VALUE_POINTS_TO_VOID,
    CORDBG_S_FUNC_EVAL_ABORTED,
    CEE_E_ENTRYPOINT,
    CEE_E_CVTRES_NOT_FOUND,
    MSEE_E_LOADLIBFAILED,
    MSEE_E_GETPROCFAILED,
    MSEE_E_MULTCOPIESLOADED,
    COR_E_TYPEUNLOADED,
    COR_E_APPDOMAINUNLOADED,
    COR_E_CANNOTUNLOADAPPDOMAIN,
    MSEE_E_ASSEMBLYLOADINPROGRESS,
    MSEE_E_CANNOTCREATEAPPDOMAIN,
    COR_E_ASSEMBLYEXPECTED,
    COR_E_FIXUPSINEXE,
    COR_E_NO_LOADLIBRARY_ALLOWED,
    COR_E_NEWER_RUNTIME,
    COR_E_CANNOT_SET_POLICY,
    COR_E_CANNOT_SPECIFY_EVIDENCE,
    COR_E_MULTIMODULEASSEMBLIESDIALLOWED,
    HOST_E_DEADLOCK,
    HOST_E_INTERRUPTED,
    HOST_E_INVALIDOPERATION,
    HOST_E_CLRNOTAVAILABLE,
    HOST_E_TIMEOUT,
    HOST_E_NOT_OWNER,
    HOST_E_ABANDONED,
    HOST_E_EXITPROCESS_THREADABORT,
    HOST_E_EXITPROCESS_ADUNLOAD,
    HOST_E_EXITPROCESS_TIMEOUT,
    HOST_E_EXITPROCESS_OUTOFMEMORY,
    HOST_E_EXITPROCESS_STACKOVERFLOW,
    COR_E_MODULE_HASH_CHECK_FAILED,
    FUSION_E_REF_DEF_MISMATCH,
    FUSION_E_INVALID_PRIVATE_ASM_LOCATION,
    FUSION_E_ASM_MODULE_MISSING,
    FUSION_E_UNEXPECTED_MODULE_FOUND,
    FUSION_E_PRIVATE_ASM_DISALLOWED,
    FUSION_E_SIGNATURE_CHECK_FAILED,
    FUSION_E_DATABASE_ERROR,
    FUSION_E_INVALID_NAME,
    FUSION_E_CODE_DOWNLOAD_DISABLED,
    FUSION_E_UNINSTALL_DISALLOWED,
    FUSION_E_HOST_GAC_ASM_MISMATCH,
    FUSION_E_LOADFROM_BLOCKED,
    FUSION_E_CACHEFILE_FAILED,
    FUSION_E_APP_DOMAIN_LOCKED,
    FUSION_E_CONFIGURATION_ERROR,
    FUSION_E_MANIFEST_PARSE_ERROR,
    FUSION_E_INVALID_ASSEMBLY_REFERENCE,
    COR_E_ASSEMBLY_NOT_EXPECTED,
    COR_E_LOADING_REFERENCE_ASSEMBLY,
    CLDB_E_FILE_BADREAD,
    CLDB_E_FILE_BADWRITE,
    CLDB_E_FILE_READONLY,
    CLDB_E_NAME_ERROR,
    CLDB_E_TRUNCATION,
    CLDB_E_FILE_OLDVER,
    CLDB_E_RELOCATED,
    CLDB_E_SMDUPLICATE,
    CLDB_E_NO_DATA,
    CLDB_E_READONLY,
    CLDB_E_INCOMPATIBLE,
    CLDB_E_FILE_CORRUPT,
    CLDB_E_SCHEMA_VERNOTFOUND,
    CLDB_E_BADUPDATEMODE,
    CLDB_E_INDEX_NONULLKEYS,
    CLDB_E_INDEX_DUPLICATE,
    CLDB_E_INDEX_BADTYPE,
    CLDB_E_INDEX_NOTFOUND,
    CLDB_E_RECORD_NOTFOUND,
    CLDB_E_RECORD_OVERFLOW,
    CLDB_E_RECORD_DUPLICATE,
    CLDB_E_RECORD_PKREQUIRED,
    CLDB_E_RECORD_DELETED,
    CLDB_E_RECORD_OUTOFORDER,
    CLDB_E_COLUMN_OVERFLOW,
    CLDB_E_COLUMN_READONLY,
    CLDB_E_COLUMN_SPECIALCOL,
    CLDB_E_COLUMN_PKNONULLS,
    CLDB_E_TABLE_CANTDROP,
    CLDB_E_OBJECT_NOTFOUND,
    CLDB_E_OBJECT_COLNOTFOUND,
    CLDB_E_VECTOR_BADINDEX,
    CLDB_E_TOO_BIG,
    META_E_INVALID_TOKEN_TYPE,
    TLBX_E_INVALID_TYPEINFO,
    TLBX_E_INVALID_TYPEINFO_UNNAMED,
    TLBX_E_CTX_NESTED,
    TLBX_E_ERROR_MESSAGE,
    TLBX_E_CANT_SAVE,
    TLBX_W_LIBNOTREGISTERED,
    TLBX_E_CANTLOADLIBRARY,
    TLBX_E_BAD_VT_TYPE,
    TLBX_E_NO_MSCOREE_TLB,
    TLBX_E_BAD_MSCOREE_TLB,
    TLBX_E_TLB_EXCEPTION,
    TLBX_E_MULTIPLE_LCIDS,
    TLBX_E_AMBIGUOUS_RETURN,
    TLBX_E_DUPLICATE_TYPE_NAME,
    TLBX_I_NONSEQUENTIALSTRUCT,
    TLBX_I_RESOLVEREFFAILED,
    TLBX_E_ASANY,
    TLBX_E_INVALIDLCIDPARAM,
    TLBX_E_LCIDONDISPONLYITF,
    TLBX_E_NONPUBLIC_FIELD,
    TLBX_E_BAD_NAMES,
    TLBX_E_GENERICINST_SIGNATURE,
    TLBX_E_GENERICPAR_SIGNATURE,
    META_E_DUPLICATE,
    META_E_GUID_REQUIRED,
    META_E_TYPEDEF_MISMATCH,
    META_E_MERGE_COLLISION,
    TLBX_E_NO_SAFEHANDLE_ARRAYS,
    META_E_METHD_NOT_FOUND,
    META_E_FIELD_NOT_FOUND,
    META_E_PARAM_MISMATCH,
    META_E_BADMETADATA,
    META_E_INTFCEIMPL_NOT_FOUND,
    TLBX_E_NO_CRITICALHANDLE_ARRAYS,
    META_E_CLASS_LAYOUT_INCONSISTENT,
    META_E_FIELD_MARSHAL_NOT_FOUND,
    META_E_METHODSEM_NOT_FOUND,
    META_E_EVENT_NOT_FOUND,
    META_E_PROP_NOT_FOUND,
    META_E_BAD_SIGNATURE,
    META_E_BAD_INPUT_PARAMETER,
    META_E_METHDIMPL_INCONSISTENT,
    META_E_MD_INCONSISTENCY,
    META_E_CANNOTRESOLVETYPEREF,
    META_E_STRINGSPACE_FULL,
    META_E_UNEXPECTED_REMAP,
    META_E_HAS_UNMARKALL,
    META_E_MUST_CALL_UNMARKALL,
    META_E_GENERICPARAM_INCONSISTENT,
    META_E_EVENT_COUNTS,
    META_E_PROPERTY_COUNTS,
    META_E_TYPEDEF_MISSING,
    TLBX_E_CANT_LOAD_MODULE,
    TLBX_E_CANT_LOAD_CLASS,
    TLBX_E_NULL_MODULE,
    TLBX_E_NO_CLSID_KEY,
    TLBX_E_CIRCULAR_EXPORT,
    TLBX_E_CIRCULAR_IMPORT,
    TLBX_E_BAD_NATIVETYPE,
    TLBX_E_BAD_VTABLE,
    TLBX_E_CRM_NON_STATIC,
    TLBX_E_CRM_INVALID_SIG,
    TLBX_E_CLASS_LOAD_EXCEPTION,
    TLBX_E_UNKNOWN_SIGNATURE,
    TLBX_E_REFERENCED_TYPELIB,
    TLBX_E_INVALID_NAMESPACE,
    TLBX_E_LAYOUT_ERROR,
    TLBX_E_NOTIUNKNOWN,
    TLBX_E_NONVISIBLEVALUECLASS,
    TLBX_E_LPTSTR_NOT_ALLOWED,
    TLBX_E_AUTO_CS_NOT_ALLOWED,
    TLBX_S_NOSTDINTERFACE,
    TLBX_S_DUPLICATE_DISPID,
    TLBX_E_ENUM_VALUE_INVALID,
    TLBX_E_DUPLICATE_IID,
    TLBX_E_NO_NESTED_ARRAYS,
    TLBX_E_PARAM_ERROR_NAMED,
    TLBX_E_PARAM_ERROR_UNNAMED,
    TLBX_E_AGNOST_SIGNATURE,
    TLBX_E_CONVERT_FAIL,
    TLBX_W_DUAL_NOT_DISPATCH,
    TLBX_E_BAD_SIGNATURE,
    TLBX_E_ARRAY_NEEDS_NT_FIXED,
    TLBX_E_CLASS_NEEDS_NT_INTF,
    META_E_CA_INVALID_TARGET,
    META_E_CA_INVALID_VALUE,
    META_E_CA_INVALID_BLOB,
    META_E_CA_REPEATED_ARG,
    META_E_CA_UNKNOWN_ARGUMENT,
    META_E_CA_VARIANT_NYI,
    META_E_CA_ARRAY_NYI,
    META_E_CA_UNEXPECTED_TYPE,
    META_E_CA_INVALID_ARGTYPE,
    META_E_CA_INVALID_ARG_FOR_TYPE,
    META_E_CA_INVALID_UUID,
    META_E_CA_INVALID_MARSHALAS_FIELDS,
    META_E_CA_NT_FIELDONLY,
    META_E_CA_NEGATIVE_PARAMINDEX,
    META_E_CA_NEGATIVE_MULTIPLIER,
    META_E_CA_NEGATIVE_CONSTSIZE,
    META_E_CA_FIXEDSTR_SIZE_REQUIRED,
    META_E_CA_CUSTMARSH_TYPE_REQUIRED,
    META_E_CA_FILENAME_REQUIRED,
    TLBX_W_NO_PROPS_IN_EVENTS,
    META_E_NOT_IN_ENC_MODE,
    META_E_METHOD_COUNTS,
    META_E_FIELD_COUNTS,
    META_E_PARAM_COUNTS,
    TLBX_E_TYPED_REF,
    TLBX_W_DEFAULT_INTF_NOT_VISIBLE,
    TLBX_W_BAD_SAFEARRAYFIELD_NO_ELEMENTVT,
    TLBX_W_LAYOUTCLASS_AS_INTERFACE,
    TLBX_I_GENERIC_BASE_TYPE,
    TLBX_E_BITNESS_MISMATCH,
    TLBX_E_EVENT_WITH_NEWENUM,
    TLBX_E_PROPGET_WITHOUT_RETURN,
    META_E_MISMATCHED_VISIBLITY,
    META_E_CA_BAD_FRIENDS_ARGS,
    META_E_CA_FRIENDS_SN_REQUIRED,
    VLDTR_E_RID_OUTOFRANGE,
    VLDTR_E_CDTKN_OUTOFRANGE,
    VLDTR_E_CDRID_OUTOFRANGE,
    VLDTR_E_STRING_INVALID,
    VLDTR_E_GUID_INVALID,
    VLDTR_E_BLOB_INVALID,
    VLDTR_E_MOD_MULTI,
    VLDTR_E_MOD_NULLMVID,
    VLDTR_E_TR_NAMENULL,
    VLDTR_E_TR_DUP,
    VLDTR_E_TD_NAMENULL,
    VLDTR_E_TD_DUPNAME,
    VLDTR_E_TD_DUPGUID,
    VLDTR_E_TD_NOTIFACEOBJEXTNULL,
    VLDTR_E_TD_OBJEXTENDSNONNULL,
    VLDTR_E_TD_EXTENDSSEALED,
    VLDTR_E_TD_DLTNORTSPCL,
    VLDTR_E_TD_RTSPCLNOTDLT,
    VLDTR_E_MI_DECLPRIV,
    VLDTR_E_AS_BADNAME,
    VLDTR_E_FILE_SYSNAME,
    VLDTR_E_MI_BODYSTATIC,
    VLDTR_E_TD_IFACENOTABS,
    VLDTR_E_TD_IFACEPARNOTNIL,
    VLDTR_E_TD_IFACEGUIDNULL,
    VLDTR_E_MI_DECLFINAL,
    VLDTR_E_TD_VTNOTSEAL,
    VLDTR_E_PD_BADFLAGS,
    VLDTR_E_IFACE_DUP,
    VLDTR_E_MR_NAMENULL,
    VLDTR_E_MR_VTBLNAME,
    VLDTR_E_MR_DELNAME,
    VLDTR_E_MR_PARNIL,
    VLDTR_E_MR_BADCALLINGCONV,
    VLDTR_E_MR_NOTVARARG,
    VLDTR_E_MR_NAMEDIFF,
    VLDTR_E_MR_SIGDIFF,
    VLDTR_E_MR_DUP,
    VLDTR_E_CL_TDAUTO,
    VLDTR_E_CL_BADPCKSZ,
    VLDTR_E_CL_DUP,
    VLDTR_E_FL_BADOFFSET,
    VLDTR_E_FL_TDNIL,
    VLDTR_E_FL_NOCL,
    VLDTR_E_FL_TDNOTEXPLCT,
    VLDTR_E_FL_FLDSTATIC,
    VLDTR_E_FL_DUP,
    VLDTR_E_MODREF_NAMENULL,
    VLDTR_E_MODREF_DUP,
    VLDTR_E_TR_BADSCOPE,
    VLDTR_E_TD_NESTEDNOENCL,
    VLDTR_E_TD_EXTTRRES,
    VLDTR_E_SIGNULL,
    VLDTR_E_SIGNODATA,
    VLDTR_E_MD_BADCALLINGCONV,
    VLDTR_E_MD_THISSTATIC,
    VLDTR_E_MD_NOTTHISNOTSTATIC,
    VLDTR_E_MD_NOARGCNT,
    VLDTR_E_SIG_MISSELTYPE,
    VLDTR_E_SIG_MISSTKN,
    VLDTR_E_SIG_TKNBAD,
    VLDTR_E_SIG_MISSFPTR,
    VLDTR_E_SIG_MISSFPTRARGCNT,
    VLDTR_E_SIG_MISSRANK,
    VLDTR_E_SIG_MISSNSIZE,
    VLDTR_E_SIG_MISSSIZE,
    VLDTR_E_SIG_MISSNLBND,
    VLDTR_E_SIG_MISSLBND,
    VLDTR_E_SIG_BADELTYPE,
    VLDTR_E_SIG_MISSVASIZE,
    VLDTR_E_FD_BADCALLINGCONV,
    VLDTR_E_MD_NAMENULL,
    VLDTR_E_MD_PARNIL,
    VLDTR_E_MD_DUP,
    VLDTR_E_FD_NAMENULL,
    VLDTR_E_FD_PARNIL,
    VLDTR_E_FD_DUP,
    VLDTR_E_AS_MULTI,
    VLDTR_E_AS_NAMENULL,
    VLDTR_E_SIG_TOKTYPEMISMATCH,
    VLDTR_E_CL_TDINTF,
    VLDTR_E_ASOS_OSPLTFRMIDINVAL,
    VLDTR_E_AR_NAMENULL,
    VLDTR_E_TD_ENCLNOTNESTED,
    VLDTR_E_AROS_OSPLTFRMIDINVAL,
    VLDTR_E_FILE_NAMENULL,
    VLDTR_E_CT_NAMENULL,
    VLDTR_E_TD_EXTENDSCHILD,
    VLDTR_E_MAR_NAMENULL,
    VLDTR_E_FILE_DUP,
    VLDTR_E_FILE_NAMEFULLQLFD,
    VLDTR_E_CT_DUP,
    VLDTR_E_MAR_DUP,
    VLDTR_E_MAR_NOTPUBPRIV,
    VLDTR_E_TD_ENUMNOVALUE,
    VLDTR_E_TD_ENUMVALSTATIC,
    VLDTR_E_TD_ENUMVALNOTSN,
    VLDTR_E_TD_ENUMFLDNOTST,
    VLDTR_E_TD_ENUMFLDNOTLIT,
    VLDTR_E_TD_ENUMNOLITFLDS,
    VLDTR_E_TD_ENUMFLDSIGMISMATCH,
    VLDTR_E_TD_ENUMVALNOT1ST,
    VLDTR_E_FD_NOTVALUERTSN,
    VLDTR_E_FD_VALUEPARNOTENUM,
    VLDTR_E_FD_INSTINIFACE,
    VLDTR_E_FD_NOTPUBINIFACE,
    VLDTR_E_FMD_GLOBALNOTPUBPRIVSC,
    VLDTR_E_FMD_GLOBALNOTSTATIC,
    VLDTR_E_FD_GLOBALNORVA,
    VLDTR_E_MD_CTORZERORVA,
    VLDTR_E_FD_MARKEDNOMARSHAL,
    VLDTR_E_FD_MARSHALNOTMARKED,
    VLDTR_E_FD_MARKEDNODEFLT,
    VLDTR_E_FD_DEFLTNOTMARKED,
    VLDTR_E_FMD_MARKEDNOSECUR,
    VLDTR_E_FMD_SECURNOTMARKED,
    VLDTR_E_FMD_PINVOKENOTSTATIC,
    VLDTR_E_FMD_MARKEDNOPINVOKE,
    VLDTR_E_FMD_PINVOKENOTMARKED,
    VLDTR_E_FMD_BADIMPLMAP,
    VLDTR_E_IMAP_BADMODREF,
    VLDTR_E_IMAP_BADMEMBER,
    VLDTR_E_IMAP_BADIMPORTNAME,
    VLDTR_E_IMAP_BADCALLCONV,
    VLDTR_E_FMD_BADACCESSFLAG,
    VLDTR_E_FD_INITONLYANDLITERAL,
    VLDTR_E_FD_LITERALNOTSTATIC,
    VLDTR_E_FMD_RTSNNOTSN,
    VLDTR_E_MD_ABSTPARNOTABST,
    VLDTR_E_MD_NOTSTATABSTININTF,
    VLDTR_E_MD_NOTPUBININTF,
    VLDTR_E_MD_CTORININTF,
    VLDTR_E_MD_GLOBALCTORCCTOR,
    VLDTR_E_MD_CTORSTATIC,
    VLDTR_E_MD_CTORNOTSNRTSN,
    VLDTR_E_MD_CTORVIRT,
    VLDTR_E_MD_CTORABST,
    VLDTR_E_MD_CCTORNOTSTATIC,
    VLDTR_E_MD_ZERORVA,
    VLDTR_E_MD_FINNOTVIRT,
    VLDTR_E_MD_STATANDFINORVIRT,
    VLDTR_E_MD_ABSTANDFINAL,
    VLDTR_E_MD_ABSTANDIMPL,
    VLDTR_E_MD_ABSTANDPINVOKE,
    VLDTR_E_MD_ABSTNOTVIRT,
    VLDTR_E_MD_NOTABSTNOTIMPL,
    VLDTR_E_MD_NOTABSTBADFLAGSRVA,
    VLDTR_E_MD_PRIVSCOPENORVA,
    VLDTR_E_MD_GLOBALABSTORVIRT,
    VLDTR_E_SIG_LONGFORM,
    VLDTR_E_MD_MULTIPLESEMANTICS,
    VLDTR_E_MD_INVALIDSEMANTICS,
    VLDTR_E_MD_SEMANTICSNOTEXIST,
    VLDTR_E_MI_DECLNOTVIRT,
    VLDTR_E_FMD_GLOBALITEM,
    VLDTR_E_MD_MULTSEMANTICFLAGS,
    VLDTR_E_MD_NOSEMANTICFLAGS,
    VLDTR_E_FD_FLDINIFACE,
    VLDTR_E_AS_HASHALGID,
    VLDTR_E_AS_PROCID,
    VLDTR_E_AR_PROCID,
    VLDTR_E_CN_PARENTRANGE,
    VLDTR_E_AS_BADFLAGS,
    VLDTR_E_TR_HASTYPEDEF,
    VLDTR_E_IFACE_BADIMPL,
    VLDTR_E_IFACE_BADIFACE,
    VLDTR_E_TD_SECURNOTMARKED,
    VLDTR_E_TD_MARKEDNOSECUR,
    VLDTR_E_MD_CCTORHASARGS,
    VLDTR_E_CT_BADIMPL,
    VLDTR_E_MI_ALIENBODY,
    VLDTR_E_MD_CCTORCALLCONV,
    VLDTR_E_MI_BADCLASS,
    VLDTR_E_MI_CLASSISINTF,
    VLDTR_E_MI_BADDECL,
    VLDTR_E_MI_BADBODY,
    VLDTR_E_MI_DUP,
    VLDTR_E_FD_BADPARENT,
    VLDTR_E_MD_PARAMOUTOFSEQ,
    VLDTR_E_MD_PARASEQTOOBIG,
    VLDTR_E_MD_PARMMARKEDNOMARSHAL,
    VLDTR_E_MD_PARMMARSHALNOTMARKED,
    VLDTR_E_MD_PARMMARKEDNODEFLT,
    VLDTR_E_MD_PARMDEFLTNOTMARKED,
    VLDTR_E_PR_BADSCOPE,
    VLDTR_E_PR_NONAME,
    VLDTR_E_PR_NOSIG,
    VLDTR_E_PR_DUP,
    VLDTR_E_PR_BADCALLINGCONV,
    VLDTR_E_PR_MARKEDNODEFLT,
    VLDTR_E_PR_DEFLTNOTMARKED,
    VLDTR_E_PR_BADSEMANTICS,
    VLDTR_E_PR_BADMETHOD,
    VLDTR_E_PR_ALIENMETHOD,
    VLDTR_E_CN_BLOBNOTNULL,
    VLDTR_E_CN_BLOBNULL,
    VLDTR_E_EV_BADSCOPE,
    VLDTR_E_EV_NONAME,
    VLDTR_E_EV_DUP,
    VLDTR_E_EV_BADEVTYPE,
    VLDTR_E_EV_EVTYPENOTCLASS,
    VLDTR_E_EV_BADSEMANTICS,
    VLDTR_E_EV_BADMETHOD,
    VLDTR_E_EV_ALIENMETHOD,
    VLDTR_E_EV_NOADDON,
    VLDTR_E_EV_NOREMOVEON,
    VLDTR_E_CT_DUPTDNAME,
    VLDTR_E_MAR_BADOFFSET,
    VLDTR_E_DS_BADOWNER,
    VLDTR_E_DS_BADFLAGS,
    VLDTR_E_DS_NOBLOB,
    VLDTR_E_MAR_BADIMPL,
    VLDTR_E_MR_VARARGCALLINGCONV,
    VLDTR_E_MD_CTORNOTVOID,
    VLDTR_E_EV_FIRENOTVOID,
    VLDTR_E_AS_BADLOCALE,
    VLDTR_E_CN_PARENTTYPE,
    VLDTR_E_SIG_SENTINMETHODDEF,
    VLDTR_E_SIG_SENTMUSTVARARG,
    VLDTR_E_SIG_MULTSENTINELS,
    VLDTR_E_SIG_LASTSENTINEL,
    VLDTR_E_SIG_MISSARG,
    VLDTR_E_SIG_BYREFINFIELD,
    VLDTR_E_MD_SYNCMETHODINVTYPE,
    VLDTR_E_TD_NAMETOOLONG,
    VLDTR_E_AS_PROCDUP,
    VLDTR_E_ASOS_DUP,
    VLDTR_E_MAR_BADFLAGS,
    VLDTR_E_CT_NOTYPEDEFID,
    VLDTR_E_FILE_BADFLAGS,
    VLDTR_E_FILE_NULLHASH,
    VLDTR_E_MOD_NONAME,
    VLDTR_E_MOD_NAMEFULLQLFD,
    VLDTR_E_TD_RTSPCLNOTSPCL,
    VLDTR_E_TD_EXTENDSIFACE,
    VLDTR_E_MD_CTORPINVOKE,
    VLDTR_E_TD_SYSENUMNOTCLASS,
    VLDTR_E_TD_SYSENUMNOTEXTVTYPE,
    VLDTR_E_MI_SIGMISMATCH,
    VLDTR_E_TD_ENUMHASMETHODS,
    VLDTR_E_TD_ENUMIMPLIFACE,
    VLDTR_E_TD_ENUMHASPROP,
    VLDTR_E_TD_ENUMHASEVENT,
    VLDTR_E_TD_BADMETHODLST,
    VLDTR_E_TD_BADFIELDLST,
    VLDTR_E_CN_BADTYPE,
    VLDTR_E_TD_ENUMNOINSTFLD,
    VLDTR_E_TD_ENUMMULINSTFLD,
    VLDTR_E_INTERRUPTED,
    VLDTR_E_NOTINIT,
    CORDBG_E_UNRECOVERABLE_ERROR,
    CORDBG_E_PROCESS_TERMINATED,
    CORDBG_E_PROCESS_NOT_SYNCHRONIZED,
    CORDBG_E_CLASS_NOT_LOADED,
    CORDBG_E_IL_VAR_NOT_AVAILABLE,
    CORDBG_E_BAD_REFERENCE_VALUE,
    CORDBG_E_FIELD_NOT_AVAILABLE,
    CORDBG_E_NON_NATIVE_FRAME,
    CORDBG_E_NONCONTINUABLE_EXCEPTION,
    CORDBG_E_CODE_NOT_AVAILABLE,
    CORDBG_E_FUNCTION_NOT_IL,
    CORDBG_E_CANT_SET_IP_INTO_FINALLY,
    CORDBG_E_CANT_SET_IP_OUT_OF_FINALLY,
    CORDBG_E_CANT_SET_IP_INTO_CATCH,
    CORDBG_E_SET_IP_NOT_ALLOWED_ON_NONLEAF_FRAME,
    CORDBG_E_SET_IP_IMPOSSIBLE,
    CORDBG_E_FUNC_EVAL_BAD_START_POINT,
    CORDBG_E_INVALID_OBJECT,
    CORDBG_E_FUNC_EVAL_NOT_COMPLETE,
    CORDBG_E_INPROC_NOT_IMPL,
    CORDBG_E_STATIC_VAR_NOT_AVAILABLE,
    CORDBG_E_OBJECT_IS_NOT_COPYABLE_VALUE_CLASS,
    CORDBG_E_CANT_SETIP_INTO_OR_OUT_OF_FILTER,
    CORDBG_E_CANT_CHANGE_JIT_SETTING_FOR_ZAP_MODULE,
    CORDBG_E_CANT_SET_IP_OUT_OF_FINALLY_ON_WIN64,
    CORDBG_E_CANT_SET_IP_OUT_OF_CATCH_ON_WIN64,
    CORDBG_E_REMOTE_CONNECTION_CONN_RESET,
    CORDBG_E_REMOTE_CONNECTION_KEEP_ALIVE,
    CORDBG_E_REMOTE_CONNECTION_FATAL_ERROR,
    CORDBG_E_CANT_SET_TO_JMC,
    CORDBG_E_NO_CONTEXT_FOR_INTERNAL_FRAME,
    CORDBG_E_NOT_CHILD_FRAME,
    CORDBG_E_NON_MATCHING_CONTEXT,
    CORDBG_E_PAST_END_OF_STACK,
    CORDBG_E_BAD_THREAD_STATE,
    CORDBG_E_FUNC_EVAL_CANNOT_UPDATE_REGISTER_IN_NONLEAF_FRAME,
    CORDBG_E_DEBUGGER_ALREADY_ATTACHED,
    CORDBG_E_SUPERFLOUS_CONTINUE,
    CORDBG_E_SET_VALUE_NOT_ALLOWED_ON_NONLEAF_FRAME,
    CORDBG_E_ENC_EH_MAX_NESTING_LEVEL_CANT_INCREASE,
    CORDBG_E_ENC_MODULE_NOT_ENC_ENABLED,
    CORDBG_E_SET_IP_NOT_ALLOWED_ON_EXCEPTION,
    CORDBG_E_VARIABLE_IS_ACTUALLY_LITERAL,
    CORDBG_E_PROCESS_DETACHED,
    CORDBG_E_ENC_METHOD_SIG_CHANGED,
    CORDBG_E_ENC_METHOD_NO_LOCAL_SIG,
    CORDBG_E_ENC_CANT_ADD_FIELD_TO_VALUE_OR_LAYOUT_CLASS,
    CORDBG_E_ENC_CANT_CHANGE_FIELD,
    CORDBG_E_ENC_CANT_ADD_NON_PRIVATE_MEMBER,
    CORDBG_E_FIELD_NOT_STATIC,
    CORDBG_E_FIELD_NOT_INSTANCE,
    CORDBG_E_ENC_ZAPPED_WITHOUT_ENC,
    CORDBG_E_ENC_BAD_METHOD_INFO,
    CORDBG_E_ENC_JIT_CANT_UPDATE,
    CORDBG_E_ENC_MISSING_CLASS,
    CORDBG_E_ENC_INTERNAL_ERROR,
    CORDBG_E_ENC_HANGING_FIELD,
    CORDBG_E_MODULE_NOT_LOADED,
    CORDBG_E_ENC_CANT_CHANGE_SUPERCLASS,
    CORDBG_E_UNABLE_TO_SET_BREAKPOINT,
    CORDBG_E_DEBUGGING_NOT_POSSIBLE,
    CORDBG_E_KERNEL_DEBUGGER_ENABLED,
    CORDBG_E_KERNEL_DEBUGGER_PRESENT,
    CORDBG_E_HELPER_THREAD_DEAD,
    CORDBG_E_INTERFACE_INHERITANCE_CANT_CHANGE,
    CORDBG_E_INCOMPATIBLE_PROTOCOL,
    CORDBG_E_TOO_MANY_PROCESSES,
    CORDBG_E_INTEROP_NOT_SUPPORTED,
    CORDBG_E_NO_REMAP_BREAKPIONT,
    CORDBG_E_OBJECT_NEUTERED,
    CORPROF_E_FUNCTION_NOT_COMPILED,
    CORPROF_E_DATAINCOMPLETE,
    CORPROF_E_NOT_REJITABLE_METHODS,
    CORPROF_E_CANNOT_UPDATE_METHOD,
    CORPROF_E_FUNCTION_NOT_IL,
    CORPROF_E_NOT_MANAGED_THREAD,
    CORPROF_E_CALL_ONLY_FROM_INIT,
    CORPROF_E_INPROC_NOT_ENABLED,
    CORPROF_E_JITMAPS_NOT_ENABLED,
    CORPROF_E_INPROC_ALREADY_BEGUN,
    CORPROF_E_INPROC_NOT_AVAILABLE,
    CORPROF_E_NOT_YET_AVAILABLE,
    CORPROF_E_TYPE_IS_PARAMETERIZED,
    CORPROF_E_FUNCTION_IS_PARAMETERIZED,
    CORPROF_E_STACKSNAPSHOT_INVALID_TGT_THREAD,
    CORPROF_E_STACKSNAPSHOT_UNMANAGED_CTX,
    CORPROF_E_STACKSNAPSHOT_UNSAFE,
    CORPROF_E_STACKSNAPSHOT_ABORTED,
    CORPROF_E_LITERALS_HAVE_NO_ADDRESS,
    CORPROF_E_UNSUPPORTED_CALL_SEQUENCE,
    CORPROF_E_ASYNCHRONOUS_UNSAFE,
    CORPROF_E_CLASSID_IS_ARRAY,
    CORPROF_E_CLASSID_IS_COMPOSITE,
    CORPROF_E_PROFILER_DETACHING,
    CORPROF_E_PROFILER_NOT_ATTACHABLE,
    CORPROF_E_UNRECOGNIZED_PIPE_MSG_FORMAT,
    CORPROF_E_PROFILER_ALREADY_ACTIVE,
    CORPROF_E_PROFILEE_INCOMPATIBLE_WITH_TRIGGER,
    CORPROF_E_IPC_FAILED,
    CORPROF_E_PROFILEE_PROCESS_NOT_FOUND,
    CORPROF_E_CALLBACK3_REQUIRED,
    CORPROF_E_UNSUPPORTED_FOR_ATTACHING_PROFILER,
    CORPROF_E_IRREVERSIBLE_INSTRUMENTATION_PRESENT,
    CORPROF_E_RUNTIME_UNINITIALIZED,
    CORPROF_E_IMMUTABLE_FLAGS_SET,
    CORPROF_E_PROFILER_NOT_YET_INITIALIZED,
    CORPROF_E_INCONSISTENT_WITH_FLAGS,
    CORPROF_E_PROFILER_CANCEL_ACTIVATION,
    CORPROF_E_CONCURRENT_GC_NOT_PROFILABLE,
    CORPROF_E_INCONSISTENT_FLAGS_WITH_HOST_PROTECTION_SETTING,
    CORPROF_E_DEBUGGING_DISABLED,
    CORPROF_E_TIMEOUT_WAITING_FOR_CONCURRENT_GC,
    CORPROF_E_MODULE_IS_DYNAMIC,
    CORPROF_E_CALLBACK4_REQUIRED,
    CORPROF_E_REJIT_NOT_ENABLED,
    CORPROF_E_ACTIVE_REJIT_REQUEST_NOT_FOUND,
    CORPROF_E_FUNCTION_IS_COLLECTIBLE,
    CORPROF_E_REJIT_REQUIRES_DISABLE_NGEN,
    CORPROF_E_CALLBACK6_REQUIRED,
    CORPROF_E_RUNTIME_SUSPEND_REQUIRED,
    CORPROF_E_CALLBACK7_REQUIRED,
    SECURITY_E_XML_TO_ASN_ENCODING,
    SECURITY_E_INCOMPATIBLE_SHARE,
    SECURITY_E_UNVERIFIABLE,
    SECURITY_E_INCOMPATIBLE_EVIDENCE,
    CORSEC_E_DECODE_SET,
    CORSEC_E_ENCODE_SET,
    CORSEC_E_UNSUPPORTED_FORMAT,
    SN_CRYPTOAPI_CALL_FAILED,
    CORSEC_E_CRYPTOAPI_CALL_FAILED,
    SN_NO_SUITABLE_CSP,
    CORSEC_E_NO_SUITABLE_CSP,
    CORSEC_E_INVALID_ATTR,
    CORSEC_E_POLICY_EXCEPTION,
    CORSEC_E_MIN_GRANT_FAIL,
    CORSEC_E_NO_EXEC_PERM,
    CORSEC_E_XMLSYNTAX,
    CORSEC_E_INVALID_STRONGNAME,
    CORSEC_E_MISSING_STRONGNAME,
    CORSEC_E_CONTAINER_NOT_FOUND,
    CORSEC_E_INVALID_IMAGE_FORMAT,
    CORSEC_E_INVALID_PUBLICKEY,
    CORSEC_E_SIGNATURE_MISMATCH,
    SN_E_PUBLICKEY_MISMATCH,
    CORSEC_E_CRYPTO,
    CORSEC_E_CRYPTO_UNEX_OPER,
    CORSECATTR_E_BAD_ATTRIBUTE,
    CORSECATTR_E_MISSING_CONSTRUCTOR,
    CORSECATTR_E_FAILED_TO_CREATE_PERM,
    CORSECATTR_E_BAD_ACTION_ASM,
    CORSECATTR_E_BAD_ACTION_OTHER,
    CORSECATTR_E_BAD_PARENT,
    CORSECATTR_E_TRUNCATED,
    CORSECATTR_E_BAD_VERSION,
    CORSECATTR_E_BAD_ACTION,
    CORSECATTR_E_NO_SELF_REF,
    CORSECATTR_E_BAD_NONCAS,
    CORSECATTR_E_ASSEMBLY_LOAD_FAILED,
    CORSECATTR_E_ASSEMBLY_LOAD_FAILED_EX,
    CORSECATTR_E_TYPE_LOAD_FAILED,
    CORSECATTR_E_TYPE_LOAD_FAILED_EX,
    CORSECATTR_E_ABSTRACT,
    CORSECATTR_E_UNSUPPORTED_TYPE,
    CORSECATTR_E_UNSUPPORTED_ENUM_TYPE,
    CORSECATTR_E_NO_FIELD,
    CORSECATTR_E_NO_PROPERTY,
    CORSECATTR_E_EXCEPTION,
    CORSECATTR_E_EXCEPTION_HR,
    ISS_E_ISOSTORE_START,
    ISS_E_ISOSTORE,
    ISS_E_OPEN_STORE_FILE,
    ISS_E_OPEN_FILE_MAPPING,
    ISS_E_MAP_VIEW_OF_FILE,
    ISS_E_GET_FILE_SIZE,
    ISS_E_CREATE_MUTEX,
    ISS_E_LOCK_FAILED,
    ISS_E_FILE_WRITE,
    ISS_E_SET_FILE_POINTER,
    ISS_E_CREATE_DIR,
    ISS_E_STORE_NOT_OPEN,
    ISS_E_CORRUPTED_STORE_FILE,
    ISS_E_STORE_VERSION,
    ISS_E_FILE_NOT_MAPPED,
    ISS_E_BLOCK_SIZE_TOO_SMALL,
    ISS_E_ALLOC_TOO_LARGE,
    ISS_E_USAGE_WILL_EXCEED_QUOTA,
    ISS_E_TABLE_ROW_NOT_FOUND,
    ISS_E_DEPRECATE,
    ISS_E_CALLER,
    ISS_E_PATH_LENGTH,
    ISS_E_MACHINE,
    ISS_E_MACHINE_DACL,
    ISS_E_ISOSTORE_END,
    COR_E_EXCEPTION,
    COR_E_SYSTEM,
    COR_E_ARGUMENTOUTOFRANGE,
    COR_E_ARRAYTYPEMISMATCH,
    COR_E_CONTEXTMARSHAL,
    COR_E_TIMEOUT,
    COR_E_EXECUTIONENGINE,
    COR_E_FIELDACCESS,
    COR_E_INDEXOUTOFRANGE,
    COR_E_INVALIDOPERATION,
    COR_E_SECURITY,
    COR_E_REMOTING,
    COR_E_SERIALIZATION,
    COR_E_VERIFICATION,
    COR_E_SERVER,
    COR_E_SERVICEDCOMPONENT,
    COR_E_METHODACCESS,
    COR_E_MISSINGFIELD,
    COR_E_MISSINGMEMBER,
    COR_E_MISSINGMETHOD,
    COR_E_MULTICASTNOTSUPPORTED,
    COR_E_NOTSUPPORTED,
    COR_E_OVERFLOW,
    COR_E_RANK,
    COR_E_SYNCHRONIZATIONLOCK,
    COR_E_THREADINTERRUPTED,
    COR_E_MEMBERACCESS,
    COR_E_THREADSTATE,
    COR_E_THREADSTOP,
    COR_E_TYPELOAD,
    COR_E_ENTRYPOINTNOTFOUND,
    COR_E_DLLNOTFOUND,
    COR_E_THREADSTART,
    COR_E_INVALIDCOMOBJECT,
    COR_E_NOTFINITENUMBER,
    COR_E_DUPLICATEWAITOBJECT,
    COR_E_SEMAPHOREFULL,
    COR_E_WAITHANDLECANNOTBEOPENED,
    COR_E_ABANDONEDMUTEX,
    COR_E_THREADABORTED,
    COR_E_INVALIDOLEVARIANTTYPE,
    COR_E_MISSINGMANIFESTRESOURCE,
    COR_E_SAFEARRAYTYPEMISMATCH,
    COR_E_TYPEINITIALIZATION,
    COR_E_MARSHALDIRECTIVE,
    COR_E_MISSINGSATELLITEASSEMBLY,
    COR_E_FORMAT,
    COR_E_SAFEARRAYRANKMISMATCH,
    COR_E_PLATFORMNOTSUPPORTED,
    COR_E_INVALIDPROGRAM,
    COR_E_OPERATIONCANCELED,
    COR_E_INSUFFICIENTMEMORY,
    COR_E_RUNTIMEWRAPPED,
    COR_E_DEVICESNOTSUPPORTED,
    COR_E_DATAMISALIGNED,
    COR_E_CODECONTRACTFAILED,
    COR_E_TYPEACCESS,
    COR_E_KEYNOTFOUND,
    COR_E_INSUFFICIENTEXECUTIONSTACK,
    COR_E_APPLICATION,
    COR_E_INVALIDFILTERCRITERIA,
    COR_E_REFLECTIONTYPELOAD,
    COR_E_TARGET,
    COR_E_TARGETINVOCATION,
    COR_E_CUSTOMATTRIBUTEFORMAT,
    COR_E_IO,
    COR_E_FILELOAD,
    COR_E_OBJECTDISPOSED,
    COR_E_FAILFAST,
    COR_E_HOSTPROTECTION,
    COR_E_ILLEGAL_REENTRANCY,
    CLR_E_SHIM_RUNTIMELOAD,
    CLR_E_SHIM_RUNTIMEEXPORT,
    CLR_E_SHIM_INSTALLROOT,
    CLR_E_SHIM_INSTALLCOMP,
    CLR_E_SHIM_LEGACYRUNTIMEALREADYBOUND,
    CLR_E_SHIM_SHUTDOWNINPROGRESS,
    VER_E_HRESULT,
    VER_E_OFFSET,
    VER_E_OPCODE,
    VER_E_OPERAND,
    VER_E_TOKEN,
    VER_E_EXCEPT,
    VER_E_STACK_SLOT,
    VER_E_LOC,
    VER_E_ARG,
    VER_E_FOUND,
    VER_E_EXPECTED,
    VER_E_LOC_BYNAME,
    VER_E_UNKNOWN_OPCODE,
    VER_E_SIG_CALLCONV,
    VER_E_SIG_ELEMTYPE,
    VER_E_RET_SIG,
    VER_E_FIELD_SIG,
    VER_E_INTERNAL,
    VER_E_STACK_TOO_LARGE,
    VER_E_ARRAY_NAME_LONG,
    VER_E_FALLTHRU,
    VER_E_TRY_GTEQ_END,
    VER_E_TRYEND_GT_CS,
    VER_E_HND_GTEQ_END,
    VER_E_HNDEND_GT_CS,
    VER_E_FLT_GTEQ_CS,
    VER_E_TRY_START,
    VER_E_HND_START,
    VER_E_FLT_START,
    VER_E_TRY_OVERLAP,
    VER_E_TRY_EQ_HND_FIL,
    VER_E_TRY_SHARE_FIN_FAL,
    VER_E_HND_OVERLAP,
    VER_E_HND_EQ,
    VER_E_FIL_OVERLAP,
    VER_E_FIL_EQ,
    VER_E_FIL_CONT_TRY,
    VER_E_FIL_CONT_HND,
    VER_E_FIL_CONT_FIL,
    VER_E_FIL_GTEQ_CS,
    VER_E_FIL_START,
    VER_E_FALLTHRU_EXCEP,
    VER_E_FALLTHRU_INTO_HND,
    VER_E_FALLTHRU_INTO_FIL,
    VER_E_LEAVE,
    VER_E_RETHROW,
    VER_E_ENDFINALLY,
    VER_E_ENDFILTER,
    VER_E_ENDFILTER_MISSING,
    VER_E_BR_INTO_TRY,
    VER_E_BR_INTO_HND,
    VER_E_BR_INTO_FIL,
    VER_E_BR_OUTOF_TRY,
    VER_E_BR_OUTOF_HND,
    VER_E_BR_OUTOF_FIL,
    VER_E_BR_OUTOF_FIN,
    VER_E_RET_FROM_TRY,
    VER_E_RET_FROM_HND,
    VER_E_RET_FROM_FIL,
    VER_E_BAD_JMP_TARGET,
    VER_E_PATH_LOC,
    VER_E_PATH_THIS,
    VER_E_PATH_STACK,
    VER_E_PATH_STACK_DEPTH,
    VER_E_THIS,
    VER_E_THIS_UNINIT_EXCEP,
    VER_E_THIS_UNINIT_STORE,
    VER_E_THIS_UNINIT_RET,
    VER_E_THIS_UNINIT_V_RET,
    VER_E_THIS_UNINIT_BR,
    VER_E_LDFTN_CTOR,
    VER_E_STACK_NOT_EQ,
    VER_E_STACK_UNEXPECTED,
    VER_E_STACK_EXCEPTION,
    VER_E_STACK_OVERFLOW,
    VER_E_STACK_UNDERFLOW,
    VER_E_STACK_EMPTY,
    VER_E_STACK_UNINIT,
    VER_E_STACK_I_I4_I8,
    VER_E_STACK_R_R4_R8,
    VER_E_STACK_NO_R_I8,
    VER_E_STACK_NUMERIC,
    VER_E_STACK_OBJREF,
    VER_E_STACK_P_OBJREF,
    VER_E_STACK_BYREF,
    VER_E_STACK_METHOD,
    VER_E_STACK_ARRAY_SD,
    VER_E_STACK_VALCLASS,
    VER_E_STACK_P_VALCLASS,
    VER_E_STACK_NO_VALCLASS,
    VER_E_LOC_DEAD,
    VER_E_LOC_NUM,
    VER_E_ARG_NUM,
    VER_E_TOKEN_RESOLVE,
    VER_E_TOKEN_TYPE,
    VER_E_TOKEN_TYPE_MEMBER,
    VER_E_TOKEN_TYPE_FIELD,
    VER_E_TOKEN_TYPE_SIG,
    VER_E_UNVERIFIABLE,
    VER_E_LDSTR_OPERAND,
    VER_E_RET_PTR_TO_STACK,
    VER_E_RET_VOID,
    VER_E_RET_MISSING,
    VER_E_RET_EMPTY,
    VER_E_RET_UNINIT,
    VER_E_ARRAY_ACCESS,
    VER_E_ARRAY_V_STORE,
    VER_E_ARRAY_SD,
    VER_E_ARRAY_SD_PTR,
    VER_E_ARRAY_FIELD,
    VER_E_ARGLIST,
    VER_E_VALCLASS,
    VER_E_METHOD_ACCESS,
    VER_E_FIELD_ACCESS,
    VER_E_DEAD,
    VER_E_FIELD_STATIC,
    VER_E_FIELD_NO_STATIC,
    VER_E_ADDR,
    VER_E_ADDR_BYREF,
    VER_E_ADDR_LITERAL,
    VER_E_INITONLY,
    VER_E_THROW,
    VER_E_CALLVIRT_VALCLASS,
    VER_E_CALL_SIG,
    VER_E_CALL_STATIC,
    VER_E_CTOR,
    VER_E_CTOR_VIRT,
    VER_E_CTOR_OR_SUPER,
    VER_E_CTOR_MUL_INIT,
    VER_E_SIG,
    VER_E_SIG_ARRAY,
    VER_E_SIG_ARRAY_PTR,
    VER_E_SIG_ARRAY_BYREF,
    VER_E_SIG_ELEM_PTR,
    VER_E_SIG_VARARG,
    VER_E_SIG_VOID,
    VER_E_SIG_BYREF_BYREF,
    VER_E_CODE_SIZE_ZERO,
    VER_E_BAD_VARARG,
    VER_E_TAIL_CALL,
    VER_E_TAIL_BYREF,
    VER_E_TAIL_RET,
    VER_E_TAIL_RET_VOID,
    VER_E_TAIL_RET_TYPE,
    VER_E_TAIL_STACK_EMPTY,
    VER_E_METHOD_END,
    VER_E_BAD_BRANCH,
    VER_E_FIN_OVERLAP,
    VER_E_LEXICAL_NESTING,
    VER_E_VOLATILE,
    VER_E_UNALIGNED,
    VER_E_INNERMOST_FIRST,
    VER_E_CALLI_VIRTUAL,
    VER_E_CALL_ABSTRACT,
    VER_E_STACK_UNEXP_ARRAY,
    VER_E_NOT_IN_GC_HEAP,
    VER_E_TRY_N_EMPTY_STACK,
    VER_E_DLGT_CTOR,
    VER_E_DLGT_BB,
    VER_E_DLGT_PATTERN,
    VER_E_DLGT_LDFTN,
    VER_E_FTN_ABSTRACT,
    VER_E_SIG_C_VC,
    VER_E_SIG_VC_C,
    VER_E_BOX_PTR_TO_STACK,
    VER_E_SIG_BYREF_TB_AH,
    VER_E_SIG_ARRAY_TB_AH,
    VER_E_ENDFILTER_STACK,
    VER_E_DLGT_SIG_I,
    VER_E_DLGT_SIG_O,
    VER_E_RA_PTR_TO_STACK,
    VER_E_CATCH_VALUE_TYPE,
    VER_E_CATCH_BYREF,
    VER_E_FIL_PRECEED_HND,
    VER_E_LDVIRTFTN_STATIC,
    VER_E_CALLVIRT_STATIC,
    VER_E_INITLOCALS,
    VER_E_BR_TO_EXCEPTION,
    VER_E_CALL_CTOR,
    VER_E_VALCLASS_OBJREF_VAR,
    VER_E_STACK_P_VALCLASS_OBJREF_VAR,
    VER_E_SIG_VAR_PARAM,
    VER_E_SIG_MVAR_PARAM,
    VER_E_SIG_VAR_ARG,
    VER_E_SIG_MVAR_ARG,
    VER_E_SIG_GENERICINST,
    VER_E_SIG_METHOD_INST,
    VER_E_SIG_METHOD_PARENT_INST,
    VER_E_SIG_FIELD_PARENT_INST,
    VER_E_CALLCONV_NOT_GENERICINST,
    VER_E_TOKEN_BAD_METHOD_SPEC,
    VER_E_BAD_READONLY_PREFIX,
    VER_E_BAD_CONSTRAINED_PREFIX,
    VER_E_CIRCULAR_VAR_CONSTRAINTS,
    VER_E_CIRCULAR_MVAR_CONSTRAINTS,
    VER_E_UNSATISFIED_METHOD_INST,
    VER_E_UNSATISFIED_METHOD_PARENT_INST,
    VER_E_UNSATISFIED_FIELD_PARENT_INST,
    VER_E_UNSATISFIED_BOX_OPERAND,
    VER_E_CONSTRAINED_CALL_WITH_NON_BYREF_THIS,
    VER_E_CONSTRAINED_OF_NON_VARIABLE_TYPE,
    VER_E_READONLY_UNEXPECTED_CALLEE,
    VER_E_READONLY_ILLEGAL_WRITE,
    VER_E_READONLY_IN_MKREFANY,
    VER_E_UNALIGNED_ALIGNMENT,
    VER_E_TAILCALL_INSIDE_EH,
    VER_E_BACKWARD_BRANCH,
    VER_E_CALL_TO_VTYPE_BASE,
    VER_E_NEWOBJ_OF_ABSTRACT_CLASS,
    VER_E_UNMANAGED_POINTER,
    VER_E_LDFTN_NON_FINAL_VIRTUAL,
    VER_E_FIELD_OVERLAP,
    VER_E_THIS_MISMATCH,
    VER_E_STACK_I_I4,
    VER_E_BAD_PE,
    VER_E_BAD_MD,
    VER_E_BAD_APPDOMAIN,
    VER_E_TYPELOAD,
    VER_E_PE_LOAD,
    VER_E_WRITE_RVA_STATIC,
    COR_E_SqlException,
    COR_E_Data,
    COR_E_DataDeletedRowInaccessible,
    COR_E_DataDuplicateName,
    COR_E_DataInRowChangingEvent,
    COR_E_DataInvalidConstraint,
    COR_E_DataMissingPrimaryKey,
    COR_E_DataNoNullAllowed,
    COR_E_DataReadOnly,
    COR_E_DataRowNotInTable,
    COR_E_DataVersionNotFound,
    COR_E_DataConstraint,
    COR_E_StrongTyping,
    COR_E_SqlType,
    COR_E_SqlNullValue,
    COR_E_SqlTruncate,
    COR_E_AdapterMapping,
    COR_E_DataAdapter,
    COR_E_DBConcurrency,
    COR_E_OperationAborted,
    COR_E_InvalidUdt,
    COR_E_OdbcException,
    COR_E_OracleException,
    COR_E_Xml,
    COR_E_XmlSchema,
    COR_E_XmlXslt,
    COR_E_XmlXPath,
    COR_E_XmlQuery,
    VLDTR_E_IFACE_NOTIFACE,
    TLBX_E_CIRCULAR_EXPORT2,
    VLDTR_E_FD_RVAHASNORVA,
    VLDTR_E_FD_RVAHASZERORVA,
    VLDTR_E_MD_RVAANDIMPLMAP,
    VLDTR_E_TD_EXTRAFLAGS,
    VLDTR_E_TD_EXTENDSITSELF,
    VLDTR_E_TD_SYSVTNOTEXTOBJ,
    VLDTR_E_TD_EXTTYPESPEC,
    VLDTR_E_TD_VTNOSIZE,
    VLDTR_E_TD_IFACESEALED,
    VLDTR_E_NC_BADNESTED,
    VLDTR_E_NC_BADENCLOSER,
    VLDTR_E_NC_DUP,
    VLDTR_E_NC_DUPENCLOSER,
    VLDTR_E_FRVA_ZERORVA,
    VLDTR_E_FRVA_BADFIELD,
    VLDTR_E_FRVA_DUPRVA,
    VLDTR_E_FRVA_DUPFIELD,
    VLDTR_E_EP_BADTOKEN,
    VLDTR_E_EP_INSTANCE,
    VLDTR_E_TD_ENUMFLDBADTYPE,
    VLDTR_E_MD_BADRVA,
    VLDTR_E_FD_LITERALNODEFAULT,
    VLDTR_E_IFACE_METHNOTIMPL,
    VLDTR_E_CA_BADPARENT,
    VLDTR_E_CA_BADTYPE,
    VLDTR_E_CA_NOTCTOR,
    VLDTR_E_CA_BADSIG,
    VLDTR_E_CA_NOSIG,
    VLDTR_E_CA_BADPROLOG,
    VLDTR_E_MD_BADLOCALSIGTOK,
    VLDTR_E_MD_BADHEADER,
    VLDTR_E_EP_TOOMANYARGS,
    VLDTR_E_EP_BADRET,
    VLDTR_E_EP_BADARG,
    VLDTR_E_SIG_BADVOID,
    VLDTR_E_IFACE_METHMULTIMPL,
    VLDTR_E_GP_NAMENULL,
    VLDTR_E_GP_OWNERNIL,
    VLDTR_E_GP_DUPNAME,
    VLDTR_E_GP_DUPNUMBER,
    VLDTR_E_GP_NONSEQ_BY_OWNER,
    VLDTR_E_GP_NONSEQ_BY_NUMBER,
    VLDTR_E_GP_UNEXPECTED_OWNER_FOR_VARIANT_VAR,
    VLDTR_E_GP_ILLEGAL_VARIANT_MVAR,
    VLDTR_E_GP_ILLEGAL_VARIANCE_FLAGS,
    VLDTR_E_GP_REFANDVALUETYPE,
    VLDTR_E_GPC_OWNERNIL,
    VLDTR_E_GPC_DUP,
    VLDTR_E_GPC_NONCONTIGUOUS,
    VLDTR_E_MS_METHODNIL,
    VLDTR_E_MS_DUP,
    VLDTR_E_MS_BADCALLINGCONV,
    VLDTR_E_MS_MISSARITY,
    VLDTR_E_MS_MISSARG,
    VLDTR_E_MS_ARITYMISMATCH,
    VLDTR_E_MS_METHODNOTGENERIC,
    VLDTR_E_SIG_MISSARITY,
    VLDTR_E_SIG_ARITYMISMATCH,
    VLDTR_E_MD_GENERIC_CCTOR,
    VLDTR_E_MD_GENERIC_CTOR,
    VLDTR_E_MD_GENERIC_IMPORT,
    VLDTR_E_MD_GENERIC_BADCALLCONV,
    VLDTR_E_MD_GENERIC_GLOBAL,
    VLDTR_E_EP_GENERIC_METHOD,
    VLDTR_E_MD_MISSARITY,
    VLDTR_E_MD_ARITYZERO,
    VLDTR_E_SIG_ARITYZERO,
    VLDTR_E_MS_ARITYZERO,
    VLDTR_E_MD_GPMISMATCH,
    VLDTR_E_EP_GENERIC_TYPE,
    VLDTR_E_MI_DECLNOTGENERIC,
    VLDTR_E_MI_IMPLNOTGENERIC,
    VLDTR_E_MI_ARITYMISMATCH,
    VLDTR_E_TD_EXTBADTYPESPEC,
    VLDTR_E_SIG_BYREFINST,
    VLDTR_E_MS_BYREFINST,
    VLDTR_E_TS_EMPTY,
    VLDTR_E_TS_HASSENTINALS,
    VLDTR_E_TD_GENERICHASEXPLAYOUT,
    VLDTR_E_SIG_BADTOKTYPE,
    VLDTR_E_IFACE_METHNOTIMPLTHISMOD,
    CORDBG_E_THREAD_NOT_SCHEDULED,
    CORDBG_E_HANDLE_HAS_BEEN_DISPOSED,
    CORDBG_E_NONINTERCEPTABLE_EXCEPTION,
    CORDBG_E_CANT_UNWIND_ABOVE_CALLBACK,
    CORDBG_E_INTERCEPT_FRAME_ALREADY_SET,
    CORDBG_E_NO_NATIVE_PATCH_AT_ADDR,
    CORDBG_E_MUST_BE_INTEROP_DEBUGGING,
    CORDBG_E_NATIVE_PATCH_ALREADY_AT_ADDR,
    CORDBG_E_TIMEOUT,
    CORDBG_E_CANT_CALL_ON_THIS_THREAD,
    CORDBG_E_ENC_INFOLESS_METHOD,
    CORDBG_E_ENC_NESTED_HANLDERS,
    CORDBG_E_ENC_IN_FUNCLET,
    CORDBG_E_ENC_LOCALLOC,
    CORDBG_E_ENC_EDIT_NOT_SUPPORTED,
    CORDBG_E_FEABORT_DELAYED_UNTIL_THREAD_RESUMED,
    CORDBG_E_NOTREADY,
    CORDBG_E_CANNOT_RESOLVE_ASSEMBLY,
    CORDBG_E_MUST_BE_IN_LOAD_MODULE,
    CORDBG_E_CANNOT_BE_ON_ATTACH,
    CORDBG_E_NGEN_NOT_SUPPORTED,
    CORDBG_E_ILLEGAL_SHUTDOWN_ORDER,
    CORDBG_E_CANNOT_DEBUG_FIBER_PROCESS,
    CORDBG_E_MUST_BE_IN_CREATE_PROCESS,
    CORDBG_E_DETACH_FAILED_OUTSTANDING_EVALS,
    CORDBG_E_DETACH_FAILED_OUTSTANDING_STEPPERS,
    CORDBG_E_CANT_INTEROP_STEP_OUT,
    CORDBG_E_DETACH_FAILED_OUTSTANDING_BREAKPOINTS,
    CORDBG_E_ILLEGAL_IN_STACK_OVERFLOW,
    CORDBG_E_ILLEGAL_AT_GC_UNSAFE_POINT,
    CORDBG_E_ILLEGAL_IN_PROLOG,
    CORDBG_E_ILLEGAL_IN_NATIVE_CODE,
    CORDBG_E_ILLEGAL_IN_OPTIMIZED_CODE,
    CORDBG_E_MINIDUMP_UNSUPPORTED,
    CORDBG_E_APPDOMAIN_MISMATCH,
    CORDBG_E_CONTEXT_UNVAILABLE,
    CORDBG_E_UNCOMPATIBLE_PLATFORMS,
    CORDBG_E_DEBUGGING_DISABLED,
    CORDBG_E_DETACH_FAILED_ON_ENC,
    CORDBG_E_CURRENT_EXCEPTION_IS_OUTSIDE_CURRENT_EXECUTION_SCOPE,
    CORDBG_E_HELPER_MAY_DEADLOCK,
    CORDBG_E_MISSING_METADATA,
    CORDBG_E_TARGET_INCONSISTENT,
    CORDBG_E_DETACH_FAILED_OUTSTANDING_TARGET_RESOURCES,
    CORDBG_E_TARGET_READONLY,
    CORDBG_E_MISMATCHED_CORWKS_AND_DACWKS_DLLS,
    CORDBG_E_MODULE_LOADED_FROM_DISK,
    CORDBG_E_SYMBOLS_NOT_AVAILABLE,
    CORDBG_E_DEBUG_COMPONENT_MISSING,
    CORDBG_E_REMOTE_MISMATCHED_CERTS,
    CORDBG_E_REMOTE_NETWORK_FAILURE,
    CORDBG_E_REMOTE_NO_LISTENER,
    CORDBG_E_REMOTE_UNKNOWN_TARGET,
    CORDBG_E_REMOTE_INVALID_CONFIG,
    CORDBG_E_REMOTE_MISMATCHED_PROTOCOLS,
    CORDBG_E_LIBRARY_PROVIDER_ERROR,
    CORDBG_E_NOT_CLR,
    CORDBG_E_MISSING_DATA_TARGET_INTERFACE,
    CORDBG_E_UNSUPPORTED_DEBUGGING_MODEL,
    CORDBG_E_UNSUPPORTED_FORWARD_COMPAT,
    CORDBG_E_UNSUPPORTED_VERSION_STRUCT,
    CORDBG_E_READVIRTUAL_FAILURE,
    CORDBG_E_VALUE_POINTS_TO_FUNCTION,
    CORDBG_E_CORRUPT_OBJECT,
    CORDBG_E_GC_STRUCTURES_INVALID,
    CORDBG_E_INVALID_OPCODE,
    CORDBG_E_UNSUPPORTED,
    CORDBG_E_MISSING_DEBUGGER_EXPORTS,
    PEFMT_E_NO_CONTENTS,
    PEFMT_E_NO_NTHEADERS,
    PEFMT_E_64BIT,
    PEFMT_E_NO_CORHEADER,
    PEFMT_E_NOT_ILONLY,
    PEFMT_E_IMPORT_DLLS,
    PEFMT_E_EXE_NOENTRYPOINT,
    PEFMT_E_BASE_RELOCS,
    PEFMT_E_ENTRYPOINT,
    PEFMT_E_ZERO_SIZEOFCODE,
    PEFMT_E_BAD_CORHEADER,
    PEFMT_E_32BIT,
    CLR_OPTSVC_E_CONTROLLER_INTERRUPT,
    NGEN_FAILED_GET_DEPENDENCIES,
    NGEN_FAILED_NATIVE_IMAGE_DELETE,
    NGEN_E_TOO_MANY_INTERFACES,
    NGEN_E_OLDER_RUNTIME,
    NGEN_E_WORKER_UNEXPECTED_EXIT,
    NGEN_E_WORKER_UNEXPECTED_SYNC,
    NGEN_E_SYS_ASM_NI_MISSING,
    CLDB_E_INTERNALERROR,
}

impl URT {
    pub fn code(&self) -> u32 {
        return match self {
            URT::VLDTR_S_WRN => 0x00131200 as u32,
            URT::VLDTR_S_ERR => 0x00131201 as u32,
            URT::VLDTR_S_WRNERR => 0x00131202 as u32,
            URT::CORDBG_S_BAD_START_SEQUENCE_POINT => 0x0013130B as u32,
            URT::CORDBG_S_BAD_END_SEQUENCE_POINT => 0x0013130C as u32,
            URT::CORDBG_S_INSUFFICIENT_INFO_FOR_SET_IP => 0x0013130D as u32,
            URT::CORDBG_S_FUNC_EVAL_HAS_NO_RESULT => 0x00131316 as u32,
            URT::CORDBG_S_VALUE_POINTS_TO_VOID => 0x00131317 as u32,
            URT::CORDBG_S_FUNC_EVAL_ABORTED => 0x00131319 as u32,
            URT::CEE_E_ENTRYPOINT => 0x80131000 as u32,
            URT::CEE_E_CVTRES_NOT_FOUND => 0x80131001 as u32,
            URT::MSEE_E_LOADLIBFAILED => 0x80131010 as u32,
            URT::MSEE_E_GETPROCFAILED => 0x80131011 as u32,
            URT::MSEE_E_MULTCOPIESLOADED => 0x80131012 as u32,
            URT::COR_E_TYPEUNLOADED => 0x80131013 as u32,
            URT::COR_E_APPDOMAINUNLOADED => 0x80131014 as u32,
            URT::COR_E_CANNOTUNLOADAPPDOMAIN => 0x80131015 as u32,
            URT::MSEE_E_ASSEMBLYLOADINPROGRESS => 0x80131016 as u32,
            URT::MSEE_E_CANNOTCREATEAPPDOMAIN => 0x80131017 as u32,
            URT::COR_E_ASSEMBLYEXPECTED => 0x80131018 as u32,
            URT::COR_E_FIXUPSINEXE => 0x80131019 as u32,
            URT::COR_E_NO_LOADLIBRARY_ALLOWED => 0x8013101A as u32,
            URT::COR_E_NEWER_RUNTIME => 0x8013101B as u32,
            URT::COR_E_CANNOT_SET_POLICY => 0x8013101C as u32,
            URT::COR_E_CANNOT_SPECIFY_EVIDENCE => 0x8013101D as u32,
            URT::COR_E_MULTIMODULEASSEMBLIESDIALLOWED => 0x8013101E as u32,
            URT::HOST_E_DEADLOCK => 0x80131020 as u32,
            URT::HOST_E_INTERRUPTED => 0x80131021 as u32,
            URT::HOST_E_INVALIDOPERATION => 0x80131022 as u32,
            URT::HOST_E_CLRNOTAVAILABLE => 0x80131023 as u32,
            URT::HOST_E_TIMEOUT => 0x80131024 as u32,
            URT::HOST_E_NOT_OWNER => 0x80131025 as u32,
            URT::HOST_E_ABANDONED => 0x80131026 as u32,
            URT::HOST_E_EXITPROCESS_THREADABORT => 0x80131027 as u32,
            URT::HOST_E_EXITPROCESS_ADUNLOAD => 0x80131028 as u32,
            URT::HOST_E_EXITPROCESS_TIMEOUT => 0x80131029 as u32,
            URT::HOST_E_EXITPROCESS_OUTOFMEMORY => 0x8013102A as u32,
            URT::HOST_E_EXITPROCESS_STACKOVERFLOW => 0x8013102B as u32,
            URT::COR_E_MODULE_HASH_CHECK_FAILED => 0x80131039 as u32,
            URT::FUSION_E_REF_DEF_MISMATCH => 0x80131040 as u32,
            URT::FUSION_E_INVALID_PRIVATE_ASM_LOCATION => 0x80131041 as u32,
            URT::FUSION_E_ASM_MODULE_MISSING => 0x80131042 as u32,
            URT::FUSION_E_UNEXPECTED_MODULE_FOUND => 0x80131043 as u32,
            URT::FUSION_E_PRIVATE_ASM_DISALLOWED => 0x80131044 as u32,
            URT::FUSION_E_SIGNATURE_CHECK_FAILED => 0x80131045 as u32,
            URT::FUSION_E_DATABASE_ERROR => 0x80131046 as u32,
            URT::FUSION_E_INVALID_NAME => 0x80131047 as u32,
            URT::FUSION_E_CODE_DOWNLOAD_DISABLED => 0x80131048 as u32,
            URT::FUSION_E_UNINSTALL_DISALLOWED => 0x80131049 as u32,
            URT::FUSION_E_HOST_GAC_ASM_MISMATCH => 0x80131050 as u32,
            URT::FUSION_E_LOADFROM_BLOCKED => 0x80131051 as u32,
            URT::FUSION_E_CACHEFILE_FAILED => 0x80131052 as u32,
            URT::FUSION_E_APP_DOMAIN_LOCKED => 0x80131053 as u32,
            URT::FUSION_E_CONFIGURATION_ERROR => 0x80131054 as u32,
            URT::FUSION_E_MANIFEST_PARSE_ERROR => 0x80131055 as u32,
            URT::FUSION_E_INVALID_ASSEMBLY_REFERENCE => 0x80131056 as u32,
            URT::COR_E_ASSEMBLY_NOT_EXPECTED => 0x80131057 as u32,
            URT::COR_E_LOADING_REFERENCE_ASSEMBLY => 0x80131058 as u32,
            URT::CLDB_E_FILE_BADREAD => 0x80131100 as u32,
            URT::CLDB_E_FILE_BADWRITE => 0x80131101 as u32,
            URT::CLDB_E_FILE_READONLY => 0x80131103 as u32,
            URT::CLDB_E_NAME_ERROR => 0x80131105 as u32,
            URT::CLDB_E_TRUNCATION => 0x80131106 as u32,
            URT::CLDB_E_FILE_OLDVER => 0x80131107 as u32,
            URT::CLDB_E_RELOCATED => 0x80131108 as u32,
            URT::CLDB_E_SMDUPLICATE => 0x8013110A as u32,
            URT::CLDB_E_NO_DATA => 0x8013110B as u32,
            URT::CLDB_E_READONLY => 0x8013110C as u32,
            URT::CLDB_E_INCOMPATIBLE => 0x8013110D as u32,
            URT::CLDB_E_FILE_CORRUPT => 0x8013110E as u32,
            URT::CLDB_E_SCHEMA_VERNOTFOUND => 0x8013110F as u32,
            URT::CLDB_E_BADUPDATEMODE => 0x80131110 as u32,
            URT::CLDB_E_INDEX_NONULLKEYS => 0x80131121 as u32,
            URT::CLDB_E_INDEX_DUPLICATE => 0x80131122 as u32,
            URT::CLDB_E_INDEX_BADTYPE => 0x80131123 as u32,
            URT::CLDB_E_INDEX_NOTFOUND => 0x80131124 as u32,
            URT::CLDB_E_RECORD_NOTFOUND => 0x80131130 as u32,
            URT::CLDB_E_RECORD_OVERFLOW => 0x80131131 as u32,
            URT::CLDB_E_RECORD_DUPLICATE => 0x80131132 as u32,
            URT::CLDB_E_RECORD_PKREQUIRED => 0x80131133 as u32,
            URT::CLDB_E_RECORD_DELETED => 0x80131134 as u32,
            URT::CLDB_E_RECORD_OUTOFORDER => 0x80131135 as u32,
            URT::CLDB_E_COLUMN_OVERFLOW => 0x80131140 as u32,
            URT::CLDB_E_COLUMN_READONLY => 0x80131141 as u32,
            URT::CLDB_E_COLUMN_SPECIALCOL => 0x80131142 as u32,
            URT::CLDB_E_COLUMN_PKNONULLS => 0x80131143 as u32,
            URT::CLDB_E_TABLE_CANTDROP => 0x80131150 as u32,
            URT::CLDB_E_OBJECT_NOTFOUND => 0x80131151 as u32,
            URT::CLDB_E_OBJECT_COLNOTFOUND => 0x80131152 as u32,
            URT::CLDB_E_VECTOR_BADINDEX => 0x80131153 as u32,
            URT::CLDB_E_TOO_BIG => 0x80131154 as u32,
            URT::META_E_INVALID_TOKEN_TYPE => 0x8013115F as u32,
            URT::TLBX_E_INVALID_TYPEINFO => 0x80131160 as u32,
            URT::TLBX_E_INVALID_TYPEINFO_UNNAMED => 0x80131161 as u32,
            URT::TLBX_E_CTX_NESTED => 0x80131162 as u32,
            URT::TLBX_E_ERROR_MESSAGE => 0x80131163 as u32,
            URT::TLBX_E_CANT_SAVE => 0x80131164 as u32,
            URT::TLBX_W_LIBNOTREGISTERED => 0x80131165 as u32,
            URT::TLBX_E_CANTLOADLIBRARY => 0x80131166 as u32,
            URT::TLBX_E_BAD_VT_TYPE => 0x80131167 as u32,
            URT::TLBX_E_NO_MSCOREE_TLB => 0x80131168 as u32,
            URT::TLBX_E_BAD_MSCOREE_TLB => 0x80131169 as u32,
            URT::TLBX_E_TLB_EXCEPTION => 0x8013116A as u32,
            URT::TLBX_E_MULTIPLE_LCIDS => 0x8013116B as u32,
            URT::TLBX_E_AMBIGUOUS_RETURN => 0x8013116D as u32,
            URT::TLBX_E_DUPLICATE_TYPE_NAME => 0x8013116E as u32,
            URT::TLBX_I_NONSEQUENTIALSTRUCT => 0x80131172 as u32,
            URT::TLBX_I_RESOLVEREFFAILED => 0x80131174 as u32,
            URT::TLBX_E_ASANY => 0x80131175 as u32,
            URT::TLBX_E_INVALIDLCIDPARAM => 0x80131176 as u32,
            URT::TLBX_E_LCIDONDISPONLYITF => 0x80131177 as u32,
            URT::TLBX_E_NONPUBLIC_FIELD => 0x80131178 as u32,
            URT::TLBX_E_BAD_NAMES => 0x8013117B as u32,
            URT::TLBX_E_GENERICINST_SIGNATURE => 0x8013117D as u32,
            URT::TLBX_E_GENERICPAR_SIGNATURE => 0x8013117E as u32,
            URT::META_E_DUPLICATE => 0x80131180 as u32,
            URT::META_E_GUID_REQUIRED => 0x80131181 as u32,
            URT::META_E_TYPEDEF_MISMATCH => 0x80131182 as u32,
            URT::META_E_MERGE_COLLISION => 0x80131183 as u32,
            URT::TLBX_E_NO_SAFEHANDLE_ARRAYS => 0x80131186 as u32,
            URT::META_E_METHD_NOT_FOUND => 0x80131187 as u32,
            URT::META_E_FIELD_NOT_FOUND => 0x80131188 as u32,
            URT::META_E_PARAM_MISMATCH => 0x80131189 as u32,
            URT::META_E_BADMETADATA => 0x8013118A as u32,
            URT::META_E_INTFCEIMPL_NOT_FOUND => 0x8013118B as u32,
            URT::TLBX_E_NO_CRITICALHANDLE_ARRAYS => 0x8013118C as u32,
            URT::META_E_CLASS_LAYOUT_INCONSISTENT => 0x8013118D as u32,
            URT::META_E_FIELD_MARSHAL_NOT_FOUND => 0x8013118E as u32,
            URT::META_E_METHODSEM_NOT_FOUND => 0x8013118F as u32,
            URT::META_E_EVENT_NOT_FOUND => 0x80131190 as u32,
            URT::META_E_PROP_NOT_FOUND => 0x80131191 as u32,
            URT::META_E_BAD_SIGNATURE => 0x80131192 as u32,
            URT::META_E_BAD_INPUT_PARAMETER => 0x80131193 as u32,
            URT::META_E_METHDIMPL_INCONSISTENT => 0x80131194 as u32,
            URT::META_E_MD_INCONSISTENCY => 0x80131195 as u32,
            URT::META_E_CANNOTRESOLVETYPEREF => 0x80131196 as u32,
            URT::META_E_STRINGSPACE_FULL => 0x80131198 as u32,
            URT::META_E_UNEXPECTED_REMAP => 0x80131199 as u32,
            URT::META_E_HAS_UNMARKALL => 0x8013119A as u32,
            URT::META_E_MUST_CALL_UNMARKALL => 0x8013119B as u32,
            URT::META_E_GENERICPARAM_INCONSISTENT => 0x8013119C as u32,
            URT::META_E_EVENT_COUNTS => 0x8013119D as u32,
            URT::META_E_PROPERTY_COUNTS => 0x8013119E as u32,
            URT::META_E_TYPEDEF_MISSING => 0x8013119F as u32,
            URT::TLBX_E_CANT_LOAD_MODULE => 0x801311A0 as u32,
            URT::TLBX_E_CANT_LOAD_CLASS => 0x801311A1 as u32,
            URT::TLBX_E_NULL_MODULE => 0x801311A2 as u32,
            URT::TLBX_E_NO_CLSID_KEY => 0x801311A3 as u32,
            URT::TLBX_E_CIRCULAR_EXPORT => 0x801311A4 as u32,
            URT::TLBX_E_CIRCULAR_IMPORT => 0x801311A5 as u32,
            URT::TLBX_E_BAD_NATIVETYPE => 0x801311A6 as u32,
            URT::TLBX_E_BAD_VTABLE => 0x801311A7 as u32,
            URT::TLBX_E_CRM_NON_STATIC => 0x801311A8 as u32,
            URT::TLBX_E_CRM_INVALID_SIG => 0x801311A9 as u32,
            URT::TLBX_E_CLASS_LOAD_EXCEPTION => 0x801311AA as u32,
            URT::TLBX_E_UNKNOWN_SIGNATURE => 0x801311AB as u32,
            URT::TLBX_E_REFERENCED_TYPELIB => 0x801311AC as u32,
            URT::TLBX_E_INVALID_NAMESPACE => 0x801311AD as u32,
            URT::TLBX_E_LAYOUT_ERROR => 0x801311AE as u32,
            URT::TLBX_E_NOTIUNKNOWN => 0x801311AF as u32,
            URT::TLBX_E_NONVISIBLEVALUECLASS => 0x801311B0 as u32,
            URT::TLBX_E_LPTSTR_NOT_ALLOWED => 0x801311B1 as u32,
            URT::TLBX_E_AUTO_CS_NOT_ALLOWED => 0x801311B2 as u32,
            URT::TLBX_S_NOSTDINTERFACE => 0x801311B3 as u32,
            URT::TLBX_S_DUPLICATE_DISPID => 0x801311B4 as u32,
            URT::TLBX_E_ENUM_VALUE_INVALID => 0x801311B5 as u32,
            URT::TLBX_E_DUPLICATE_IID => 0x801311B6 as u32,
            URT::TLBX_E_NO_NESTED_ARRAYS => 0x801311B7 as u32,
            URT::TLBX_E_PARAM_ERROR_NAMED => 0x801311B8 as u32,
            URT::TLBX_E_PARAM_ERROR_UNNAMED => 0x801311B9 as u32,
            URT::TLBX_E_AGNOST_SIGNATURE => 0x801311BA as u32,
            URT::TLBX_E_CONVERT_FAIL => 0x801311BB as u32,
            URT::TLBX_W_DUAL_NOT_DISPATCH => 0x801311BC as u32,
            URT::TLBX_E_BAD_SIGNATURE => 0x801311BD as u32,
            URT::TLBX_E_ARRAY_NEEDS_NT_FIXED => 0x801311BE as u32,
            URT::TLBX_E_CLASS_NEEDS_NT_INTF => 0x801311BF as u32,
            URT::META_E_CA_INVALID_TARGET => 0x801311C0 as u32,
            URT::META_E_CA_INVALID_VALUE => 0x801311C1 as u32,
            URT::META_E_CA_INVALID_BLOB => 0x801311C2 as u32,
            URT::META_E_CA_REPEATED_ARG => 0x801311C3 as u32,
            URT::META_E_CA_UNKNOWN_ARGUMENT => 0x801311C4 as u32,
            URT::META_E_CA_VARIANT_NYI => 0x801311C5 as u32,
            URT::META_E_CA_ARRAY_NYI => 0x801311C6 as u32,
            URT::META_E_CA_UNEXPECTED_TYPE => 0x801311C7 as u32,
            URT::META_E_CA_INVALID_ARGTYPE => 0x801311C8 as u32,
            URT::META_E_CA_INVALID_ARG_FOR_TYPE => 0x801311C9 as u32,
            URT::META_E_CA_INVALID_UUID => 0x801311CA as u32,
            URT::META_E_CA_INVALID_MARSHALAS_FIELDS => 0x801311CB as u32,
            URT::META_E_CA_NT_FIELDONLY => 0x801311CC as u32,
            URT::META_E_CA_NEGATIVE_PARAMINDEX => 0x801311CD as u32,
            URT::META_E_CA_NEGATIVE_MULTIPLIER => 0x801311CE as u32,
            URT::META_E_CA_NEGATIVE_CONSTSIZE => 0x801311CF as u32,
            URT::META_E_CA_FIXEDSTR_SIZE_REQUIRED => 0x801311D0 as u32,
            URT::META_E_CA_CUSTMARSH_TYPE_REQUIRED => 0x801311D1 as u32,
            URT::META_E_CA_FILENAME_REQUIRED => 0x801311D2 as u32,
            URT::TLBX_W_NO_PROPS_IN_EVENTS => 0x801311D3 as u32,
            URT::META_E_NOT_IN_ENC_MODE => 0x801311D4 as u32,
            URT::META_E_METHOD_COUNTS => 0x801311D6 as u32,
            URT::META_E_FIELD_COUNTS => 0x801311D7 as u32,
            URT::META_E_PARAM_COUNTS => 0x801311D8 as u32,
            URT::TLBX_E_TYPED_REF => 0x801311DA as u32,
            URT::TLBX_W_DEFAULT_INTF_NOT_VISIBLE => 0x801311DB as u32,
            URT::TLBX_W_BAD_SAFEARRAYFIELD_NO_ELEMENTVT => 0x801311DE as u32,
            URT::TLBX_W_LAYOUTCLASS_AS_INTERFACE => 0x801311DF as u32,
            URT::TLBX_I_GENERIC_BASE_TYPE => 0x801311E0 as u32,
            URT::TLBX_E_BITNESS_MISMATCH => 0x801311E1 as u32,
            URT::TLBX_E_EVENT_WITH_NEWENUM => 0x801311E2 as u32,
            URT::TLBX_E_PROPGET_WITHOUT_RETURN => 0x801311E3 as u32,
            URT::META_E_MISMATCHED_VISIBLITY => 0x801311E4 as u32,
            URT::META_E_CA_BAD_FRIENDS_ARGS => 0x801311E5 as u32,
            URT::META_E_CA_FRIENDS_SN_REQUIRED => 0x801311E6 as u32,
            URT::VLDTR_E_RID_OUTOFRANGE => 0x80131203 as u32,
            URT::VLDTR_E_CDTKN_OUTOFRANGE => 0x80131204 as u32,
            URT::VLDTR_E_CDRID_OUTOFRANGE => 0x80131205 as u32,
            URT::VLDTR_E_STRING_INVALID => 0x80131206 as u32,
            URT::VLDTR_E_GUID_INVALID => 0x80131207 as u32,
            URT::VLDTR_E_BLOB_INVALID => 0x80131208 as u32,
            URT::VLDTR_E_MOD_MULTI => 0x80131209 as u32,
            URT::VLDTR_E_MOD_NULLMVID => 0x8013120A as u32,
            URT::VLDTR_E_TR_NAMENULL => 0x8013120B as u32,
            URT::VLDTR_E_TR_DUP => 0x8013120C as u32,
            URT::VLDTR_E_TD_NAMENULL => 0x8013120D as u32,
            URT::VLDTR_E_TD_DUPNAME => 0x8013120E as u32,
            URT::VLDTR_E_TD_DUPGUID => 0x8013120F as u32,
            URT::VLDTR_E_TD_NOTIFACEOBJEXTNULL => 0x80131210 as u32,
            URT::VLDTR_E_TD_OBJEXTENDSNONNULL => 0x80131211 as u32,
            URT::VLDTR_E_TD_EXTENDSSEALED => 0x80131212 as u32,
            URT::VLDTR_E_TD_DLTNORTSPCL => 0x80131213 as u32,
            URT::VLDTR_E_TD_RTSPCLNOTDLT => 0x80131214 as u32,
            URT::VLDTR_E_MI_DECLPRIV => 0x80131215 as u32,
            URT::VLDTR_E_AS_BADNAME => 0x80131216 as u32,
            URT::VLDTR_E_FILE_SYSNAME => 0x80131217 as u32,
            URT::VLDTR_E_MI_BODYSTATIC => 0x80131218 as u32,
            URT::VLDTR_E_TD_IFACENOTABS => 0x80131219 as u32,
            URT::VLDTR_E_TD_IFACEPARNOTNIL => 0x8013121A as u32,
            URT::VLDTR_E_TD_IFACEGUIDNULL => 0x8013121B as u32,
            URT::VLDTR_E_MI_DECLFINAL => 0x8013121C as u32,
            URT::VLDTR_E_TD_VTNOTSEAL => 0x8013121D as u32,
            URT::VLDTR_E_PD_BADFLAGS => 0x8013121E as u32,
            URT::VLDTR_E_IFACE_DUP => 0x8013121F as u32,
            URT::VLDTR_E_MR_NAMENULL => 0x80131220 as u32,
            URT::VLDTR_E_MR_VTBLNAME => 0x80131221 as u32,
            URT::VLDTR_E_MR_DELNAME => 0x80131222 as u32,
            URT::VLDTR_E_MR_PARNIL => 0x80131223 as u32,
            URT::VLDTR_E_MR_BADCALLINGCONV => 0x80131224 as u32,
            URT::VLDTR_E_MR_NOTVARARG => 0x80131225 as u32,
            URT::VLDTR_E_MR_NAMEDIFF => 0x80131226 as u32,
            URT::VLDTR_E_MR_SIGDIFF => 0x80131227 as u32,
            URT::VLDTR_E_MR_DUP => 0x80131228 as u32,
            URT::VLDTR_E_CL_TDAUTO => 0x80131229 as u32,
            URT::VLDTR_E_CL_BADPCKSZ => 0x8013122A as u32,
            URT::VLDTR_E_CL_DUP => 0x8013122B as u32,
            URT::VLDTR_E_FL_BADOFFSET => 0x8013122C as u32,
            URT::VLDTR_E_FL_TDNIL => 0x8013122D as u32,
            URT::VLDTR_E_FL_NOCL => 0x8013122E as u32,
            URT::VLDTR_E_FL_TDNOTEXPLCT => 0x8013122F as u32,
            URT::VLDTR_E_FL_FLDSTATIC => 0x80131230 as u32,
            URT::VLDTR_E_FL_DUP => 0x80131231 as u32,
            URT::VLDTR_E_MODREF_NAMENULL => 0x80131232 as u32,
            URT::VLDTR_E_MODREF_DUP => 0x80131233 as u32,
            URT::VLDTR_E_TR_BADSCOPE => 0x80131234 as u32,
            URT::VLDTR_E_TD_NESTEDNOENCL => 0x80131235 as u32,
            URT::VLDTR_E_TD_EXTTRRES => 0x80131236 as u32,
            URT::VLDTR_E_SIGNULL => 0x80131237 as u32,
            URT::VLDTR_E_SIGNODATA => 0x80131238 as u32,
            URT::VLDTR_E_MD_BADCALLINGCONV => 0x80131239 as u32,
            URT::VLDTR_E_MD_THISSTATIC => 0x8013123A as u32,
            URT::VLDTR_E_MD_NOTTHISNOTSTATIC => 0x8013123B as u32,
            URT::VLDTR_E_MD_NOARGCNT => 0x8013123C as u32,
            URT::VLDTR_E_SIG_MISSELTYPE => 0x8013123D as u32,
            URT::VLDTR_E_SIG_MISSTKN => 0x8013123E as u32,
            URT::VLDTR_E_SIG_TKNBAD => 0x8013123F as u32,
            URT::VLDTR_E_SIG_MISSFPTR => 0x80131240 as u32,
            URT::VLDTR_E_SIG_MISSFPTRARGCNT => 0x80131241 as u32,
            URT::VLDTR_E_SIG_MISSRANK => 0x80131242 as u32,
            URT::VLDTR_E_SIG_MISSNSIZE => 0x80131243 as u32,
            URT::VLDTR_E_SIG_MISSSIZE => 0x80131244 as u32,
            URT::VLDTR_E_SIG_MISSNLBND => 0x80131245 as u32,
            URT::VLDTR_E_SIG_MISSLBND => 0x80131246 as u32,
            URT::VLDTR_E_SIG_BADELTYPE => 0x80131247 as u32,
            URT::VLDTR_E_SIG_MISSVASIZE => 0x80131248 as u32,
            URT::VLDTR_E_FD_BADCALLINGCONV => 0x80131249 as u32,
            URT::VLDTR_E_MD_NAMENULL => 0x8013124A as u32,
            URT::VLDTR_E_MD_PARNIL => 0x8013124B as u32,
            URT::VLDTR_E_MD_DUP => 0x8013124C as u32,
            URT::VLDTR_E_FD_NAMENULL => 0x8013124D as u32,
            URT::VLDTR_E_FD_PARNIL => 0x8013124E as u32,
            URT::VLDTR_E_FD_DUP => 0x8013124F as u32,
            URT::VLDTR_E_AS_MULTI => 0x80131250 as u32,
            URT::VLDTR_E_AS_NAMENULL => 0x80131251 as u32,
            URT::VLDTR_E_SIG_TOKTYPEMISMATCH => 0x80131252 as u32,
            URT::VLDTR_E_CL_TDINTF => 0x80131253 as u32,
            URT::VLDTR_E_ASOS_OSPLTFRMIDINVAL => 0x80131254 as u32,
            URT::VLDTR_E_AR_NAMENULL => 0x80131255 as u32,
            URT::VLDTR_E_TD_ENCLNOTNESTED => 0x80131256 as u32,
            URT::VLDTR_E_AROS_OSPLTFRMIDINVAL => 0x80131257 as u32,
            URT::VLDTR_E_FILE_NAMENULL => 0x80131258 as u32,
            URT::VLDTR_E_CT_NAMENULL => 0x80131259 as u32,
            URT::VLDTR_E_TD_EXTENDSCHILD => 0x8013125A as u32,
            URT::VLDTR_E_MAR_NAMENULL => 0x8013125B as u32,
            URT::VLDTR_E_FILE_DUP => 0x8013125C as u32,
            URT::VLDTR_E_FILE_NAMEFULLQLFD => 0x8013125D as u32,
            URT::VLDTR_E_CT_DUP => 0x8013125E as u32,
            URT::VLDTR_E_MAR_DUP => 0x8013125F as u32,
            URT::VLDTR_E_MAR_NOTPUBPRIV => 0x80131260 as u32,
            URT::VLDTR_E_TD_ENUMNOVALUE => 0x80131261 as u32,
            URT::VLDTR_E_TD_ENUMVALSTATIC => 0x80131262 as u32,
            URT::VLDTR_E_TD_ENUMVALNOTSN => 0x80131263 as u32,
            URT::VLDTR_E_TD_ENUMFLDNOTST => 0x80131264 as u32,
            URT::VLDTR_E_TD_ENUMFLDNOTLIT => 0x80131265 as u32,
            URT::VLDTR_E_TD_ENUMNOLITFLDS => 0x80131266 as u32,
            URT::VLDTR_E_TD_ENUMFLDSIGMISMATCH => 0x80131267 as u32,
            URT::VLDTR_E_TD_ENUMVALNOT1ST => 0x80131268 as u32,
            URT::VLDTR_E_FD_NOTVALUERTSN => 0x80131269 as u32,
            URT::VLDTR_E_FD_VALUEPARNOTENUM => 0x8013126A as u32,
            URT::VLDTR_E_FD_INSTINIFACE => 0x8013126B as u32,
            URT::VLDTR_E_FD_NOTPUBINIFACE => 0x8013126C as u32,
            URT::VLDTR_E_FMD_GLOBALNOTPUBPRIVSC => 0x8013126D as u32,
            URT::VLDTR_E_FMD_GLOBALNOTSTATIC => 0x8013126E as u32,
            URT::VLDTR_E_FD_GLOBALNORVA => 0x8013126F as u32,
            URT::VLDTR_E_MD_CTORZERORVA => 0x80131270 as u32,
            URT::VLDTR_E_FD_MARKEDNOMARSHAL => 0x80131271 as u32,
            URT::VLDTR_E_FD_MARSHALNOTMARKED => 0x80131272 as u32,
            URT::VLDTR_E_FD_MARKEDNODEFLT => 0x80131273 as u32,
            URT::VLDTR_E_FD_DEFLTNOTMARKED => 0x80131274 as u32,
            URT::VLDTR_E_FMD_MARKEDNOSECUR => 0x80131275 as u32,
            URT::VLDTR_E_FMD_SECURNOTMARKED => 0x80131276 as u32,
            URT::VLDTR_E_FMD_PINVOKENOTSTATIC => 0x80131277 as u32,
            URT::VLDTR_E_FMD_MARKEDNOPINVOKE => 0x80131278 as u32,
            URT::VLDTR_E_FMD_PINVOKENOTMARKED => 0x80131279 as u32,
            URT::VLDTR_E_FMD_BADIMPLMAP => 0x8013127A as u32,
            URT::VLDTR_E_IMAP_BADMODREF => 0x8013127B as u32,
            URT::VLDTR_E_IMAP_BADMEMBER => 0x8013127C as u32,
            URT::VLDTR_E_IMAP_BADIMPORTNAME => 0x8013127D as u32,
            URT::VLDTR_E_IMAP_BADCALLCONV => 0x8013127E as u32,
            URT::VLDTR_E_FMD_BADACCESSFLAG => 0x8013127F as u32,
            URT::VLDTR_E_FD_INITONLYANDLITERAL => 0x80131280 as u32,
            URT::VLDTR_E_FD_LITERALNOTSTATIC => 0x80131281 as u32,
            URT::VLDTR_E_FMD_RTSNNOTSN => 0x80131282 as u32,
            URT::VLDTR_E_MD_ABSTPARNOTABST => 0x80131283 as u32,
            URT::VLDTR_E_MD_NOTSTATABSTININTF => 0x80131284 as u32,
            URT::VLDTR_E_MD_NOTPUBININTF => 0x80131285 as u32,
            URT::VLDTR_E_MD_CTORININTF => 0x80131286 as u32,
            URT::VLDTR_E_MD_GLOBALCTORCCTOR => 0x80131287 as u32,
            URT::VLDTR_E_MD_CTORSTATIC => 0x80131288 as u32,
            URT::VLDTR_E_MD_CTORNOTSNRTSN => 0x80131289 as u32,
            URT::VLDTR_E_MD_CTORVIRT => 0x8013128A as u32,
            URT::VLDTR_E_MD_CTORABST => 0x8013128B as u32,
            URT::VLDTR_E_MD_CCTORNOTSTATIC => 0x8013128C as u32,
            URT::VLDTR_E_MD_ZERORVA => 0x8013128D as u32,
            URT::VLDTR_E_MD_FINNOTVIRT => 0x8013128E as u32,
            URT::VLDTR_E_MD_STATANDFINORVIRT => 0x8013128F as u32,
            URT::VLDTR_E_MD_ABSTANDFINAL => 0x80131290 as u32,
            URT::VLDTR_E_MD_ABSTANDIMPL => 0x80131291 as u32,
            URT::VLDTR_E_MD_ABSTANDPINVOKE => 0x80131292 as u32,
            URT::VLDTR_E_MD_ABSTNOTVIRT => 0x80131293 as u32,
            URT::VLDTR_E_MD_NOTABSTNOTIMPL => 0x80131294 as u32,
            URT::VLDTR_E_MD_NOTABSTBADFLAGSRVA => 0x80131295 as u32,
            URT::VLDTR_E_MD_PRIVSCOPENORVA => 0x80131296 as u32,
            URT::VLDTR_E_MD_GLOBALABSTORVIRT => 0x80131297 as u32,
            URT::VLDTR_E_SIG_LONGFORM => 0x80131298 as u32,
            URT::VLDTR_E_MD_MULTIPLESEMANTICS => 0x80131299 as u32,
            URT::VLDTR_E_MD_INVALIDSEMANTICS => 0x8013129A as u32,
            URT::VLDTR_E_MD_SEMANTICSNOTEXIST => 0x8013129B as u32,
            URT::VLDTR_E_MI_DECLNOTVIRT => 0x8013129C as u32,
            URT::VLDTR_E_FMD_GLOBALITEM => 0x8013129D as u32,
            URT::VLDTR_E_MD_MULTSEMANTICFLAGS => 0x8013129E as u32,
            URT::VLDTR_E_MD_NOSEMANTICFLAGS => 0x8013129F as u32,
            URT::VLDTR_E_FD_FLDINIFACE => 0x801312A0 as u32,
            URT::VLDTR_E_AS_HASHALGID => 0x801312A1 as u32,
            URT::VLDTR_E_AS_PROCID => 0x801312A2 as u32,
            URT::VLDTR_E_AR_PROCID => 0x801312A3 as u32,
            URT::VLDTR_E_CN_PARENTRANGE => 0x801312A4 as u32,
            URT::VLDTR_E_AS_BADFLAGS => 0x801312A5 as u32,
            URT::VLDTR_E_TR_HASTYPEDEF => 0x801312A6 as u32,
            URT::VLDTR_E_IFACE_BADIMPL => 0x801312A7 as u32,
            URT::VLDTR_E_IFACE_BADIFACE => 0x801312A8 as u32,
            URT::VLDTR_E_TD_SECURNOTMARKED => 0x801312A9 as u32,
            URT::VLDTR_E_TD_MARKEDNOSECUR => 0x801312AA as u32,
            URT::VLDTR_E_MD_CCTORHASARGS => 0x801312AB as u32,
            URT::VLDTR_E_CT_BADIMPL => 0x801312AC as u32,
            URT::VLDTR_E_MI_ALIENBODY => 0x801312AD as u32,
            URT::VLDTR_E_MD_CCTORCALLCONV => 0x801312AE as u32,
            URT::VLDTR_E_MI_BADCLASS => 0x801312AF as u32,
            URT::VLDTR_E_MI_CLASSISINTF => 0x801312B0 as u32,
            URT::VLDTR_E_MI_BADDECL => 0x801312B1 as u32,
            URT::VLDTR_E_MI_BADBODY => 0x801312B2 as u32,
            URT::VLDTR_E_MI_DUP => 0x801312B3 as u32,
            URT::VLDTR_E_FD_BADPARENT => 0x801312B4 as u32,
            URT::VLDTR_E_MD_PARAMOUTOFSEQ => 0x801312B5 as u32,
            URT::VLDTR_E_MD_PARASEQTOOBIG => 0x801312B6 as u32,
            URT::VLDTR_E_MD_PARMMARKEDNOMARSHAL => 0x801312B7 as u32,
            URT::VLDTR_E_MD_PARMMARSHALNOTMARKED => 0x801312B8 as u32,
            URT::VLDTR_E_MD_PARMMARKEDNODEFLT => 0x801312BA as u32,
            URT::VLDTR_E_MD_PARMDEFLTNOTMARKED => 0x801312BB as u32,
            URT::VLDTR_E_PR_BADSCOPE => 0x801312BC as u32,
            URT::VLDTR_E_PR_NONAME => 0x801312BD as u32,
            URT::VLDTR_E_PR_NOSIG => 0x801312BE as u32,
            URT::VLDTR_E_PR_DUP => 0x801312BF as u32,
            URT::VLDTR_E_PR_BADCALLINGCONV => 0x801312C0 as u32,
            URT::VLDTR_E_PR_MARKEDNODEFLT => 0x801312C1 as u32,
            URT::VLDTR_E_PR_DEFLTNOTMARKED => 0x801312C2 as u32,
            URT::VLDTR_E_PR_BADSEMANTICS => 0x801312C3 as u32,
            URT::VLDTR_E_PR_BADMETHOD => 0x801312C4 as u32,
            URT::VLDTR_E_PR_ALIENMETHOD => 0x801312C5 as u32,
            URT::VLDTR_E_CN_BLOBNOTNULL => 0x801312C6 as u32,
            URT::VLDTR_E_CN_BLOBNULL => 0x801312C7 as u32,
            URT::VLDTR_E_EV_BADSCOPE => 0x801312C8 as u32,
            URT::VLDTR_E_EV_NONAME => 0x801312CA as u32,
            URT::VLDTR_E_EV_DUP => 0x801312CB as u32,
            URT::VLDTR_E_EV_BADEVTYPE => 0x801312CC as u32,
            URT::VLDTR_E_EV_EVTYPENOTCLASS => 0x801312CD as u32,
            URT::VLDTR_E_EV_BADSEMANTICS => 0x801312CE as u32,
            URT::VLDTR_E_EV_BADMETHOD => 0x801312CF as u32,
            URT::VLDTR_E_EV_ALIENMETHOD => 0x801312D0 as u32,
            URT::VLDTR_E_EV_NOADDON => 0x801312D1 as u32,
            URT::VLDTR_E_EV_NOREMOVEON => 0x801312D2 as u32,
            URT::VLDTR_E_CT_DUPTDNAME => 0x801312D3 as u32,
            URT::VLDTR_E_MAR_BADOFFSET => 0x801312D4 as u32,
            URT::VLDTR_E_DS_BADOWNER => 0x801312D5 as u32,
            URT::VLDTR_E_DS_BADFLAGS => 0x801312D6 as u32,
            URT::VLDTR_E_DS_NOBLOB => 0x801312D7 as u32,
            URT::VLDTR_E_MAR_BADIMPL => 0x801312D8 as u32,
            URT::VLDTR_E_MR_VARARGCALLINGCONV => 0x801312DA as u32,
            URT::VLDTR_E_MD_CTORNOTVOID => 0x801312DB as u32,
            URT::VLDTR_E_EV_FIRENOTVOID => 0x801312DC as u32,
            URT::VLDTR_E_AS_BADLOCALE => 0x801312DD as u32,
            URT::VLDTR_E_CN_PARENTTYPE => 0x801312DE as u32,
            URT::VLDTR_E_SIG_SENTINMETHODDEF => 0x801312DF as u32,
            URT::VLDTR_E_SIG_SENTMUSTVARARG => 0x801312E0 as u32,
            URT::VLDTR_E_SIG_MULTSENTINELS => 0x801312E1 as u32,
            URT::VLDTR_E_SIG_LASTSENTINEL => 0x801312E2 as u32,
            URT::VLDTR_E_SIG_MISSARG => 0x801312E3 as u32,
            URT::VLDTR_E_SIG_BYREFINFIELD => 0x801312E4 as u32,
            URT::VLDTR_E_MD_SYNCMETHODINVTYPE => 0x801312E5 as u32,
            URT::VLDTR_E_TD_NAMETOOLONG => 0x801312E6 as u32,
            URT::VLDTR_E_AS_PROCDUP => 0x801312E7 as u32,
            URT::VLDTR_E_ASOS_DUP => 0x801312E8 as u32,
            URT::VLDTR_E_MAR_BADFLAGS => 0x801312E9 as u32,
            URT::VLDTR_E_CT_NOTYPEDEFID => 0x801312EA as u32,
            URT::VLDTR_E_FILE_BADFLAGS => 0x801312EB as u32,
            URT::VLDTR_E_FILE_NULLHASH => 0x801312EC as u32,
            URT::VLDTR_E_MOD_NONAME => 0x801312ED as u32,
            URT::VLDTR_E_MOD_NAMEFULLQLFD => 0x801312EE as u32,
            URT::VLDTR_E_TD_RTSPCLNOTSPCL => 0x801312EF as u32,
            URT::VLDTR_E_TD_EXTENDSIFACE => 0x801312F0 as u32,
            URT::VLDTR_E_MD_CTORPINVOKE => 0x801312F1 as u32,
            URT::VLDTR_E_TD_SYSENUMNOTCLASS => 0x801312F2 as u32,
            URT::VLDTR_E_TD_SYSENUMNOTEXTVTYPE => 0x801312F3 as u32,
            URT::VLDTR_E_MI_SIGMISMATCH => 0x801312F4 as u32,
            URT::VLDTR_E_TD_ENUMHASMETHODS => 0x801312F5 as u32,
            URT::VLDTR_E_TD_ENUMIMPLIFACE => 0x801312F6 as u32,
            URT::VLDTR_E_TD_ENUMHASPROP => 0x801312F7 as u32,
            URT::VLDTR_E_TD_ENUMHASEVENT => 0x801312F8 as u32,
            URT::VLDTR_E_TD_BADMETHODLST => 0x801312F9 as u32,
            URT::VLDTR_E_TD_BADFIELDLST => 0x801312FA as u32,
            URT::VLDTR_E_CN_BADTYPE => 0x801312FB as u32,
            URT::VLDTR_E_TD_ENUMNOINSTFLD => 0x801312FC as u32,
            URT::VLDTR_E_TD_ENUMMULINSTFLD => 0x801312FD as u32,
            URT::VLDTR_E_INTERRUPTED => 0x801312FE as u32,
            URT::VLDTR_E_NOTINIT => 0x801312FF as u32,
            URT::CORDBG_E_UNRECOVERABLE_ERROR => 0x80131300 as u32,
            URT::CORDBG_E_PROCESS_TERMINATED => 0x80131301 as u32,
            URT::CORDBG_E_PROCESS_NOT_SYNCHRONIZED => 0x80131302 as u32,
            URT::CORDBG_E_CLASS_NOT_LOADED => 0x80131303 as u32,
            URT::CORDBG_E_IL_VAR_NOT_AVAILABLE => 0x80131304 as u32,
            URT::CORDBG_E_BAD_REFERENCE_VALUE => 0x80131305 as u32,
            URT::CORDBG_E_FIELD_NOT_AVAILABLE => 0x80131306 as u32,
            URT::CORDBG_E_NON_NATIVE_FRAME => 0x80131307 as u32,
            URT::CORDBG_E_NONCONTINUABLE_EXCEPTION => 0x80131308 as u32,
            URT::CORDBG_E_CODE_NOT_AVAILABLE => 0x80131309 as u32,
            URT::CORDBG_E_FUNCTION_NOT_IL => 0x8013130A as u32,
            URT::CORDBG_E_CANT_SET_IP_INTO_FINALLY => 0x8013130E as u32,
            URT::CORDBG_E_CANT_SET_IP_OUT_OF_FINALLY => 0x8013130F as u32,
            URT::CORDBG_E_CANT_SET_IP_INTO_CATCH => 0x80131310 as u32,
            URT::CORDBG_E_SET_IP_NOT_ALLOWED_ON_NONLEAF_FRAME => 0x80131311 as u32,
            URT::CORDBG_E_SET_IP_IMPOSSIBLE => 0x80131312 as u32,
            URT::CORDBG_E_FUNC_EVAL_BAD_START_POINT => 0x80131313 as u32,
            URT::CORDBG_E_INVALID_OBJECT => 0x80131314 as u32,
            URT::CORDBG_E_FUNC_EVAL_NOT_COMPLETE => 0x80131315 as u32,
            URT::CORDBG_E_INPROC_NOT_IMPL => 0x80131318 as u32,
            URT::CORDBG_E_STATIC_VAR_NOT_AVAILABLE => 0x8013131A as u32,
            URT::CORDBG_E_OBJECT_IS_NOT_COPYABLE_VALUE_CLASS => 0x8013131B as u32,
            URT::CORDBG_E_CANT_SETIP_INTO_OR_OUT_OF_FILTER => 0x8013131C as u32,
            URT::CORDBG_E_CANT_CHANGE_JIT_SETTING_FOR_ZAP_MODULE => 0x8013131D as u32,
            URT::CORDBG_E_CANT_SET_IP_OUT_OF_FINALLY_ON_WIN64 => 0x8013131E as u32,
            URT::CORDBG_E_CANT_SET_IP_OUT_OF_CATCH_ON_WIN64 => 0x8013131F as u32,
            URT::CORDBG_E_REMOTE_CONNECTION_CONN_RESET => 0x80131320 as u32,
            URT::CORDBG_E_REMOTE_CONNECTION_KEEP_ALIVE => 0x80131321 as u32,
            URT::CORDBG_E_REMOTE_CONNECTION_FATAL_ERROR => 0x80131322 as u32,
            URT::CORDBG_E_CANT_SET_TO_JMC => 0x80131323 as u32,
            URT::CORDBG_E_NO_CONTEXT_FOR_INTERNAL_FRAME => 0x80131326 as u32,
            URT::CORDBG_E_NOT_CHILD_FRAME => 0x80131327 as u32,
            URT::CORDBG_E_NON_MATCHING_CONTEXT => 0x80131328 as u32,
            URT::CORDBG_E_PAST_END_OF_STACK => 0x80131329 as u32,
            URT::CORDBG_E_BAD_THREAD_STATE => 0x8013132D as u32,
            URT::CORDBG_E_FUNC_EVAL_CANNOT_UPDATE_REGISTER_IN_NONLEAF_FRAME => 0x8013132D as u32,
            URT::CORDBG_E_DEBUGGER_ALREADY_ATTACHED => 0x8013132E as u32,
            URT::CORDBG_E_SUPERFLOUS_CONTINUE => 0x8013132F as u32,
            URT::CORDBG_E_SET_VALUE_NOT_ALLOWED_ON_NONLEAF_FRAME => 0x80131330 as u32,
            URT::CORDBG_E_ENC_EH_MAX_NESTING_LEVEL_CANT_INCREASE => 0x80131331 as u32,
            URT::CORDBG_E_ENC_MODULE_NOT_ENC_ENABLED => 0x80131332 as u32,
            URT::CORDBG_E_SET_IP_NOT_ALLOWED_ON_EXCEPTION => 0x80131333 as u32,
            URT::CORDBG_E_VARIABLE_IS_ACTUALLY_LITERAL => 0x80131334 as u32,
            URT::CORDBG_E_PROCESS_DETACHED => 0x80131335 as u32,
            URT::CORDBG_E_ENC_METHOD_SIG_CHANGED => 0x80131336 as u32,
            URT::CORDBG_E_ENC_METHOD_NO_LOCAL_SIG => 0x80131337 as u32,
            URT::CORDBG_E_ENC_CANT_ADD_FIELD_TO_VALUE_OR_LAYOUT_CLASS => 0x80131338 as u32,
            URT::CORDBG_E_ENC_CANT_CHANGE_FIELD => 0x80131339 as u32,
            URT::CORDBG_E_ENC_CANT_ADD_NON_PRIVATE_MEMBER => 0x8013133A as u32,
            URT::CORDBG_E_FIELD_NOT_STATIC => 0x8013133B as u32,
            URT::CORDBG_E_FIELD_NOT_INSTANCE => 0x8013133C as u32,
            URT::CORDBG_E_ENC_ZAPPED_WITHOUT_ENC => 0x8013133D as u32,
            URT::CORDBG_E_ENC_BAD_METHOD_INFO => 0x8013133E as u32,
            URT::CORDBG_E_ENC_JIT_CANT_UPDATE => 0x8013133F as u32,
            URT::CORDBG_E_ENC_MISSING_CLASS => 0x80131340 as u32,
            URT::CORDBG_E_ENC_INTERNAL_ERROR => 0x80131341 as u32,
            URT::CORDBG_E_ENC_HANGING_FIELD => 0x80131342 as u32,
            URT::CORDBG_E_MODULE_NOT_LOADED => 0x80131343 as u32,
            URT::CORDBG_E_ENC_CANT_CHANGE_SUPERCLASS => 0x80131344 as u32,
            URT::CORDBG_E_UNABLE_TO_SET_BREAKPOINT => 0x80131345 as u32,
            URT::CORDBG_E_DEBUGGING_NOT_POSSIBLE => 0x80131346 as u32,
            URT::CORDBG_E_KERNEL_DEBUGGER_ENABLED => 0x80131347 as u32,
            URT::CORDBG_E_KERNEL_DEBUGGER_PRESENT => 0x80131348 as u32,
            URT::CORDBG_E_HELPER_THREAD_DEAD => 0x80131349 as u32,
            URT::CORDBG_E_INTERFACE_INHERITANCE_CANT_CHANGE => 0x8013134A as u32,
            URT::CORDBG_E_INCOMPATIBLE_PROTOCOL => 0x8013134B as u32,
            URT::CORDBG_E_TOO_MANY_PROCESSES => 0x8013134C as u32,
            URT::CORDBG_E_INTEROP_NOT_SUPPORTED => 0x8013134D as u32,
            URT::CORDBG_E_NO_REMAP_BREAKPIONT => 0x8013134E as u32,
            URT::CORDBG_E_OBJECT_NEUTERED => 0x8013134F as u32,
            URT::CORPROF_E_FUNCTION_NOT_COMPILED => 0x80131350 as u32,
            URT::CORPROF_E_DATAINCOMPLETE => 0x80131351 as u32,
            URT::CORPROF_E_NOT_REJITABLE_METHODS => 0x80131352 as u32,
            URT::CORPROF_E_CANNOT_UPDATE_METHOD => 0x80131353 as u32,
            URT::CORPROF_E_FUNCTION_NOT_IL => 0x80131354 as u32,
            URT::CORPROF_E_NOT_MANAGED_THREAD => 0x80131355 as u32,
            URT::CORPROF_E_CALL_ONLY_FROM_INIT => 0x80131356 as u32,
            URT::CORPROF_E_INPROC_NOT_ENABLED => 0x80131357 as u32,
            URT::CORPROF_E_JITMAPS_NOT_ENABLED => 0x80131358 as u32,
            URT::CORPROF_E_INPROC_ALREADY_BEGUN => 0x80131359 as u32,
            URT::CORPROF_E_INPROC_NOT_AVAILABLE => 0x8013135A as u32,
            URT::CORPROF_E_NOT_YET_AVAILABLE => 0x8013135B as u32,
            URT::CORPROF_E_TYPE_IS_PARAMETERIZED => 0x8013135C as u32,
            URT::CORPROF_E_FUNCTION_IS_PARAMETERIZED => 0x8013135D as u32,
            URT::CORPROF_E_STACKSNAPSHOT_INVALID_TGT_THREAD => 0x8013135E as u32,
            URT::CORPROF_E_STACKSNAPSHOT_UNMANAGED_CTX => 0x8013135F as u32,
            URT::CORPROF_E_STACKSNAPSHOT_UNSAFE => 0x80131360 as u32,
            URT::CORPROF_E_STACKSNAPSHOT_ABORTED => 0x80131361 as u32,
            URT::CORPROF_E_LITERALS_HAVE_NO_ADDRESS => 0x80131362 as u32,
            URT::CORPROF_E_UNSUPPORTED_CALL_SEQUENCE => 0x80131363 as u32,
            URT::CORPROF_E_ASYNCHRONOUS_UNSAFE => 0x80131364 as u32,
            URT::CORPROF_E_CLASSID_IS_ARRAY => 0x80131365 as u32,
            URT::CORPROF_E_CLASSID_IS_COMPOSITE => 0x80131366 as u32,
            URT::CORPROF_E_PROFILER_DETACHING => 0x80131367 as u32,
            URT::CORPROF_E_PROFILER_NOT_ATTACHABLE => 0x80131368 as u32,
            URT::CORPROF_E_UNRECOGNIZED_PIPE_MSG_FORMAT => 0x80131369 as u32,
            URT::CORPROF_E_PROFILER_ALREADY_ACTIVE => 0x8013136A as u32,
            URT::CORPROF_E_PROFILEE_INCOMPATIBLE_WITH_TRIGGER => 0x8013136B as u32,
            URT::CORPROF_E_IPC_FAILED => 0x8013136C as u32,
            URT::CORPROF_E_PROFILEE_PROCESS_NOT_FOUND => 0x8013136D as u32,
            URT::CORPROF_E_CALLBACK3_REQUIRED => 0x8013136E as u32,
            URT::CORPROF_E_UNSUPPORTED_FOR_ATTACHING_PROFILER => 0x8013136F as u32,
            URT::CORPROF_E_IRREVERSIBLE_INSTRUMENTATION_PRESENT => 0x80131370 as u32,
            URT::CORPROF_E_RUNTIME_UNINITIALIZED => 0x80131371 as u32,
            URT::CORPROF_E_IMMUTABLE_FLAGS_SET => 0x80131372 as u32,
            URT::CORPROF_E_PROFILER_NOT_YET_INITIALIZED => 0x80131373 as u32,
            URT::CORPROF_E_INCONSISTENT_WITH_FLAGS => 0x80131374 as u32,
            URT::CORPROF_E_PROFILER_CANCEL_ACTIVATION => 0x80131375 as u32,
            URT::CORPROF_E_CONCURRENT_GC_NOT_PROFILABLE => 0x80131376 as u32,
            URT::CORPROF_E_INCONSISTENT_FLAGS_WITH_HOST_PROTECTION_SETTING => 0x80131377 as u32,
            URT::CORPROF_E_DEBUGGING_DISABLED => 0x80131378 as u32,
            URT::CORPROF_E_TIMEOUT_WAITING_FOR_CONCURRENT_GC => 0x80131379 as u32,
            URT::CORPROF_E_MODULE_IS_DYNAMIC => 0x8013137A as u32,
            URT::CORPROF_E_CALLBACK4_REQUIRED => 0x8013137B as u32,
            URT::CORPROF_E_REJIT_NOT_ENABLED => 0x8013137C as u32,
            URT::CORPROF_E_ACTIVE_REJIT_REQUEST_NOT_FOUND => 0x8013137D as u32,
            URT::CORPROF_E_FUNCTION_IS_COLLECTIBLE => 0x8013137E as u32,
            URT::CORPROF_E_REJIT_REQUIRES_DISABLE_NGEN => 0x8013137F as u32,
            URT::CORPROF_E_CALLBACK6_REQUIRED => 0x80131380 as u32,
            URT::CORPROF_E_RUNTIME_SUSPEND_REQUIRED => 0x80131381 as u32,
            URT::CORPROF_E_CALLBACK7_REQUIRED => 0x80131382 as u32,
            URT::SECURITY_E_XML_TO_ASN_ENCODING => 0x80131400 as u32,
            URT::SECURITY_E_INCOMPATIBLE_SHARE => 0x80131401 as u32,
            URT::SECURITY_E_UNVERIFIABLE => 0x80131402 as u32,
            URT::SECURITY_E_INCOMPATIBLE_EVIDENCE => 0x80131403 as u32,
            URT::CORSEC_E_DECODE_SET => 0x80131410 as u32,
            URT::CORSEC_E_ENCODE_SET => 0x80131411 as u32,
            URT::CORSEC_E_UNSUPPORTED_FORMAT => 0x80131412 as u32,
            URT::SN_CRYPTOAPI_CALL_FAILED => 0x80131413 as u32,
            URT::CORSEC_E_CRYPTOAPI_CALL_FAILED => 0x80131413 as u32,
            URT::SN_NO_SUITABLE_CSP => 0x80131414 as u32,
            URT::CORSEC_E_NO_SUITABLE_CSP => 0x80131414 as u32,
            URT::CORSEC_E_INVALID_ATTR => 0x80131415 as u32,
            URT::CORSEC_E_POLICY_EXCEPTION => 0x80131416 as u32,
            URT::CORSEC_E_MIN_GRANT_FAIL => 0x80131417 as u32,
            URT::CORSEC_E_NO_EXEC_PERM => 0x80131418 as u32,
            URT::CORSEC_E_XMLSYNTAX => 0x80131419 as u32,
            URT::CORSEC_E_INVALID_STRONGNAME => 0x8013141A as u32,
            URT::CORSEC_E_MISSING_STRONGNAME => 0x8013141B as u32,
            URT::CORSEC_E_CONTAINER_NOT_FOUND => 0x8013141C as u32,
            URT::CORSEC_E_INVALID_IMAGE_FORMAT => 0x8013141D as u32,
            URT::CORSEC_E_INVALID_PUBLICKEY => 0x8013141E as u32,
            URT::CORSEC_E_SIGNATURE_MISMATCH => 0x80131420 as u32,
            URT::SN_E_PUBLICKEY_MISMATCH => 0x80131421 as u32,
            URT::CORSEC_E_CRYPTO => 0x80131430 as u32,
            URT::CORSEC_E_CRYPTO_UNEX_OPER => 0x80131431 as u32,
            URT::CORSECATTR_E_BAD_ATTRIBUTE => 0x8013143A as u32,
            URT::CORSECATTR_E_MISSING_CONSTRUCTOR => 0x8013143B as u32,
            URT::CORSECATTR_E_FAILED_TO_CREATE_PERM => 0x8013143C as u32,
            URT::CORSECATTR_E_BAD_ACTION_ASM => 0x8013143D as u32,
            URT::CORSECATTR_E_BAD_ACTION_OTHER => 0x8013143E as u32,
            URT::CORSECATTR_E_BAD_PARENT => 0x8013143F as u32,
            URT::CORSECATTR_E_TRUNCATED => 0x80131440 as u32,
            URT::CORSECATTR_E_BAD_VERSION => 0x80131441 as u32,
            URT::CORSECATTR_E_BAD_ACTION => 0x80131442 as u32,
            URT::CORSECATTR_E_NO_SELF_REF => 0x80131443 as u32,
            URT::CORSECATTR_E_BAD_NONCAS => 0x80131444 as u32,
            URT::CORSECATTR_E_ASSEMBLY_LOAD_FAILED => 0x80131445 as u32,
            URT::CORSECATTR_E_ASSEMBLY_LOAD_FAILED_EX => 0x80131446 as u32,
            URT::CORSECATTR_E_TYPE_LOAD_FAILED => 0x80131447 as u32,
            URT::CORSECATTR_E_TYPE_LOAD_FAILED_EX => 0x80131448 as u32,
            URT::CORSECATTR_E_ABSTRACT => 0x80131449 as u32,
            URT::CORSECATTR_E_UNSUPPORTED_TYPE => 0x8013144A as u32,
            URT::CORSECATTR_E_UNSUPPORTED_ENUM_TYPE => 0x8013144B as u32,
            URT::CORSECATTR_E_NO_FIELD => 0x8013144C as u32,
            URT::CORSECATTR_E_NO_PROPERTY => 0x8013144D as u32,
            URT::CORSECATTR_E_EXCEPTION => 0x8013144E as u32,
            URT::CORSECATTR_E_EXCEPTION_HR => 0x8013144F as u32,
            URT::ISS_E_ISOSTORE_START => 0x80131450 as u32,
            URT::ISS_E_ISOSTORE => 0x80131450 as u32,
            URT::ISS_E_OPEN_STORE_FILE => 0x80131460 as u32,
            URT::ISS_E_OPEN_FILE_MAPPING => 0x80131461 as u32,
            URT::ISS_E_MAP_VIEW_OF_FILE => 0x80131462 as u32,
            URT::ISS_E_GET_FILE_SIZE => 0x80131463 as u32,
            URT::ISS_E_CREATE_MUTEX => 0x80131464 as u32,
            URT::ISS_E_LOCK_FAILED => 0x80131465 as u32,
            URT::ISS_E_FILE_WRITE => 0x80131466 as u32,
            URT::ISS_E_SET_FILE_POINTER => 0x80131467 as u32,
            URT::ISS_E_CREATE_DIR => 0x80131468 as u32,
            URT::ISS_E_STORE_NOT_OPEN => 0x80131469 as u32,
            URT::ISS_E_CORRUPTED_STORE_FILE => 0x80131480 as u32,
            URT::ISS_E_STORE_VERSION => 0x80131481 as u32,
            URT::ISS_E_FILE_NOT_MAPPED => 0x80131482 as u32,
            URT::ISS_E_BLOCK_SIZE_TOO_SMALL => 0x80131483 as u32,
            URT::ISS_E_ALLOC_TOO_LARGE => 0x80131484 as u32,
            URT::ISS_E_USAGE_WILL_EXCEED_QUOTA => 0x80131485 as u32,
            URT::ISS_E_TABLE_ROW_NOT_FOUND => 0x80131486 as u32,
            URT::ISS_E_DEPRECATE => 0x801314A0 as u32,
            URT::ISS_E_CALLER => 0x801314A1 as u32,
            URT::ISS_E_PATH_LENGTH => 0x801314A2 as u32,
            URT::ISS_E_MACHINE => 0x801314A3 as u32,
            URT::ISS_E_MACHINE_DACL => 0x801314A4 as u32,
            URT::ISS_E_ISOSTORE_END => 0x801314FF as u32,
            URT::COR_E_EXCEPTION => 0x80131500 as u32,
            URT::COR_E_SYSTEM => 0x80131501 as u32,
            URT::COR_E_ARGUMENTOUTOFRANGE => 0x80131502 as u32,
            URT::COR_E_ARRAYTYPEMISMATCH => 0x80131503 as u32,
            URT::COR_E_CONTEXTMARSHAL => 0x80131504 as u32,
            URT::COR_E_TIMEOUT => 0x80131505 as u32,
            URT::COR_E_EXECUTIONENGINE => 0x80131506 as u32,
            URT::COR_E_FIELDACCESS => 0x80131507 as u32,
            URT::COR_E_INDEXOUTOFRANGE => 0x80131508 as u32,
            URT::COR_E_INVALIDOPERATION => 0x80131509 as u32,
            URT::COR_E_SECURITY => 0x8013150A as u32,
            URT::COR_E_REMOTING => 0x8013150B as u32,
            URT::COR_E_SERIALIZATION => 0x8013150C as u32,
            URT::COR_E_VERIFICATION => 0x8013150D as u32,
            URT::COR_E_SERVER => 0x8013150E as u32,
            URT::COR_E_SERVICEDCOMPONENT => 0x8013150F as u32,
            URT::COR_E_METHODACCESS => 0x80131510 as u32,
            URT::COR_E_MISSINGFIELD => 0x80131511 as u32,
            URT::COR_E_MISSINGMEMBER => 0x80131512 as u32,
            URT::COR_E_MISSINGMETHOD => 0x80131513 as u32,
            URT::COR_E_MULTICASTNOTSUPPORTED => 0x80131514 as u32,
            URT::COR_E_NOTSUPPORTED => 0x80131515 as u32,
            URT::COR_E_OVERFLOW => 0x80131516 as u32,
            URT::COR_E_RANK => 0x80131517 as u32,
            URT::COR_E_SYNCHRONIZATIONLOCK => 0x80131518 as u32,
            URT::COR_E_THREADINTERRUPTED => 0x80131519 as u32,
            URT::COR_E_MEMBERACCESS => 0x8013151A as u32,
            URT::COR_E_THREADSTATE => 0x80131520 as u32,
            URT::COR_E_THREADSTOP => 0x80131521 as u32,
            URT::COR_E_TYPELOAD => 0x80131522 as u32,
            URT::COR_E_ENTRYPOINTNOTFOUND => 0x80131523 as u32,
            URT::COR_E_DLLNOTFOUND => 0x80131524 as u32,
            URT::COR_E_THREADSTART => 0x80131525 as u32,
            URT::COR_E_INVALIDCOMOBJECT => 0x80131527 as u32,
            URT::COR_E_NOTFINITENUMBER => 0x80131528 as u32,
            URT::COR_E_DUPLICATEWAITOBJECT => 0x80131529 as u32,
            URT::COR_E_SEMAPHOREFULL => 0x8013152B as u32,
            URT::COR_E_WAITHANDLECANNOTBEOPENED => 0x8013152C as u32,
            URT::COR_E_ABANDONEDMUTEX => 0x8013152D as u32,
            URT::COR_E_THREADABORTED => 0x80131530 as u32,
            URT::COR_E_INVALIDOLEVARIANTTYPE => 0x80131531 as u32,
            URT::COR_E_MISSINGMANIFESTRESOURCE => 0x80131532 as u32,
            URT::COR_E_SAFEARRAYTYPEMISMATCH => 0x80131533 as u32,
            URT::COR_E_TYPEINITIALIZATION => 0x80131534 as u32,
            URT::COR_E_MARSHALDIRECTIVE => 0x80131535 as u32,
            URT::COR_E_MISSINGSATELLITEASSEMBLY => 0x80131536 as u32,
            URT::COR_E_FORMAT => 0x80131537 as u32,
            URT::COR_E_SAFEARRAYRANKMISMATCH => 0x80131538 as u32,
            URT::COR_E_PLATFORMNOTSUPPORTED => 0x80131539 as u32,
            URT::COR_E_INVALIDPROGRAM => 0x8013153A as u32,
            URT::COR_E_OPERATIONCANCELED => 0x8013153B as u32,
            URT::COR_E_INSUFFICIENTMEMORY => 0x8013153D as u32,
            URT::COR_E_RUNTIMEWRAPPED => 0x8013153E as u32,
            URT::COR_E_DEVICESNOTSUPPORTED => 0x80131540 as u32,
            URT::COR_E_DATAMISALIGNED => 0x80131541 as u32,
            URT::COR_E_CODECONTRACTFAILED => 0x80131542 as u32,
            URT::COR_E_TYPEACCESS => 0x80131543 as u32,
            URT::COR_E_KEYNOTFOUND => 0x80131577 as u32,
            URT::COR_E_INSUFFICIENTEXECUTIONSTACK => 0x80131578 as u32,
            URT::COR_E_APPLICATION => 0x80131600 as u32,
            URT::COR_E_INVALIDFILTERCRITERIA => 0x80131601 as u32,
            URT::COR_E_REFLECTIONTYPELOAD => 0x80131602 as u32,
            URT::COR_E_TARGET => 0x80131603 as u32,
            URT::COR_E_TARGETINVOCATION => 0x80131604 as u32,
            URT::COR_E_CUSTOMATTRIBUTEFORMAT => 0x80131605 as u32,
            URT::COR_E_IO => 0x80131620 as u32,
            URT::COR_E_FILELOAD => 0x80131621 as u32,
            URT::COR_E_OBJECTDISPOSED => 0x80131622 as u32,
            URT::COR_E_FAILFAST => 0x80131623 as u32,
            URT::COR_E_HOSTPROTECTION => 0x80131640 as u32,
            URT::COR_E_ILLEGAL_REENTRANCY => 0x80131641 as u32,
            URT::CLR_E_SHIM_RUNTIMELOAD => 0x80131700 as u32,
            URT::CLR_E_SHIM_RUNTIMEEXPORT => 0x80131701 as u32,
            URT::CLR_E_SHIM_INSTALLROOT => 0x80131702 as u32,
            URT::CLR_E_SHIM_INSTALLCOMP => 0x80131703 as u32,
            URT::CLR_E_SHIM_LEGACYRUNTIMEALREADYBOUND => 0x80131704 as u32,
            URT::CLR_E_SHIM_SHUTDOWNINPROGRESS => 0x80131705 as u32,
            URT::VER_E_HRESULT => 0x80131801 as u32,
            URT::VER_E_OFFSET => 0x80131802 as u32,
            URT::VER_E_OPCODE => 0x80131803 as u32,
            URT::VER_E_OPERAND => 0x80131804 as u32,
            URT::VER_E_TOKEN => 0x80131805 as u32,
            URT::VER_E_EXCEPT => 0x80131806 as u32,
            URT::VER_E_STACK_SLOT => 0x80131807 as u32,
            URT::VER_E_LOC => 0x80131808 as u32,
            URT::VER_E_ARG => 0x80131809 as u32,
            URT::VER_E_FOUND => 0x8013180A as u32,
            URT::VER_E_EXPECTED => 0x8013180B as u32,
            URT::VER_E_LOC_BYNAME => 0x8013180C as u32,
            URT::VER_E_UNKNOWN_OPCODE => 0x80131810 as u32,
            URT::VER_E_SIG_CALLCONV => 0x80131811 as u32,
            URT::VER_E_SIG_ELEMTYPE => 0x80131812 as u32,
            URT::VER_E_RET_SIG => 0x80131814 as u32,
            URT::VER_E_FIELD_SIG => 0x80131815 as u32,
            URT::VER_E_INTERNAL => 0x80131818 as u32,
            URT::VER_E_STACK_TOO_LARGE => 0x80131819 as u32,
            URT::VER_E_ARRAY_NAME_LONG => 0x8013181A as u32,
            URT::VER_E_FALLTHRU => 0x80131820 as u32,
            URT::VER_E_TRY_GTEQ_END => 0x80131821 as u32,
            URT::VER_E_TRYEND_GT_CS => 0x80131822 as u32,
            URT::VER_E_HND_GTEQ_END => 0x80131823 as u32,
            URT::VER_E_HNDEND_GT_CS => 0x80131824 as u32,
            URT::VER_E_FLT_GTEQ_CS => 0x80131825 as u32,
            URT::VER_E_TRY_START => 0x80131826 as u32,
            URT::VER_E_HND_START => 0x80131827 as u32,
            URT::VER_E_FLT_START => 0x80131828 as u32,
            URT::VER_E_TRY_OVERLAP => 0x80131829 as u32,
            URT::VER_E_TRY_EQ_HND_FIL => 0x8013182A as u32,
            URT::VER_E_TRY_SHARE_FIN_FAL => 0x8013182B as u32,
            URT::VER_E_HND_OVERLAP => 0x8013182C as u32,
            URT::VER_E_HND_EQ => 0x8013182D as u32,
            URT::VER_E_FIL_OVERLAP => 0x8013182E as u32,
            URT::VER_E_FIL_EQ => 0x8013182F as u32,
            URT::VER_E_FIL_CONT_TRY => 0x80131830 as u32,
            URT::VER_E_FIL_CONT_HND => 0x80131831 as u32,
            URT::VER_E_FIL_CONT_FIL => 0x80131832 as u32,
            URT::VER_E_FIL_GTEQ_CS => 0x80131833 as u32,
            URT::VER_E_FIL_START => 0x80131834 as u32,
            URT::VER_E_FALLTHRU_EXCEP => 0x80131835 as u32,
            URT::VER_E_FALLTHRU_INTO_HND => 0x80131836 as u32,
            URT::VER_E_FALLTHRU_INTO_FIL => 0x80131837 as u32,
            URT::VER_E_LEAVE => 0x80131838 as u32,
            URT::VER_E_RETHROW => 0x80131839 as u32,
            URT::VER_E_ENDFINALLY => 0x8013183A as u32,
            URT::VER_E_ENDFILTER => 0x8013183B as u32,
            URT::VER_E_ENDFILTER_MISSING => 0x8013183C as u32,
            URT::VER_E_BR_INTO_TRY => 0x8013183D as u32,
            URT::VER_E_BR_INTO_HND => 0x8013183E as u32,
            URT::VER_E_BR_INTO_FIL => 0x8013183F as u32,
            URT::VER_E_BR_OUTOF_TRY => 0x80131840 as u32,
            URT::VER_E_BR_OUTOF_HND => 0x80131841 as u32,
            URT::VER_E_BR_OUTOF_FIL => 0x80131842 as u32,
            URT::VER_E_BR_OUTOF_FIN => 0x80131843 as u32,
            URT::VER_E_RET_FROM_TRY => 0x80131844 as u32,
            URT::VER_E_RET_FROM_HND => 0x80131845 as u32,
            URT::VER_E_RET_FROM_FIL => 0x80131846 as u32,
            URT::VER_E_BAD_JMP_TARGET => 0x80131847 as u32,
            URT::VER_E_PATH_LOC => 0x80131848 as u32,
            URT::VER_E_PATH_THIS => 0x80131849 as u32,
            URT::VER_E_PATH_STACK => 0x8013184A as u32,
            URT::VER_E_PATH_STACK_DEPTH => 0x8013184B as u32,
            URT::VER_E_THIS => 0x8013184C as u32,
            URT::VER_E_THIS_UNINIT_EXCEP => 0x8013184D as u32,
            URT::VER_E_THIS_UNINIT_STORE => 0x8013184E as u32,
            URT::VER_E_THIS_UNINIT_RET => 0x8013184F as u32,
            URT::VER_E_THIS_UNINIT_V_RET => 0x80131850 as u32,
            URT::VER_E_THIS_UNINIT_BR => 0x80131851 as u32,
            URT::VER_E_LDFTN_CTOR => 0x80131852 as u32,
            URT::VER_E_STACK_NOT_EQ => 0x80131853 as u32,
            URT::VER_E_STACK_UNEXPECTED => 0x80131854 as u32,
            URT::VER_E_STACK_EXCEPTION => 0x80131855 as u32,
            URT::VER_E_STACK_OVERFLOW => 0x80131856 as u32,
            URT::VER_E_STACK_UNDERFLOW => 0x80131857 as u32,
            URT::VER_E_STACK_EMPTY => 0x80131858 as u32,
            URT::VER_E_STACK_UNINIT => 0x80131859 as u32,
            URT::VER_E_STACK_I_I4_I8 => 0x8013185A as u32,
            URT::VER_E_STACK_R_R4_R8 => 0x8013185B as u32,
            URT::VER_E_STACK_NO_R_I8 => 0x8013185C as u32,
            URT::VER_E_STACK_NUMERIC => 0x8013185D as u32,
            URT::VER_E_STACK_OBJREF => 0x8013185E as u32,
            URT::VER_E_STACK_P_OBJREF => 0x8013185F as u32,
            URT::VER_E_STACK_BYREF => 0x80131860 as u32,
            URT::VER_E_STACK_METHOD => 0x80131861 as u32,
            URT::VER_E_STACK_ARRAY_SD => 0x80131862 as u32,
            URT::VER_E_STACK_VALCLASS => 0x80131863 as u32,
            URT::VER_E_STACK_P_VALCLASS => 0x80131864 as u32,
            URT::VER_E_STACK_NO_VALCLASS => 0x80131865 as u32,
            URT::VER_E_LOC_DEAD => 0x80131866 as u32,
            URT::VER_E_LOC_NUM => 0x80131867 as u32,
            URT::VER_E_ARG_NUM => 0x80131868 as u32,
            URT::VER_E_TOKEN_RESOLVE => 0x80131869 as u32,
            URT::VER_E_TOKEN_TYPE => 0x8013186A as u32,
            URT::VER_E_TOKEN_TYPE_MEMBER => 0x8013186B as u32,
            URT::VER_E_TOKEN_TYPE_FIELD => 0x8013186C as u32,
            URT::VER_E_TOKEN_TYPE_SIG => 0x8013186D as u32,
            URT::VER_E_UNVERIFIABLE => 0x8013186E as u32,
            URT::VER_E_LDSTR_OPERAND => 0x8013186F as u32,
            URT::VER_E_RET_PTR_TO_STACK => 0x80131870 as u32,
            URT::VER_E_RET_VOID => 0x80131871 as u32,
            URT::VER_E_RET_MISSING => 0x80131872 as u32,
            URT::VER_E_RET_EMPTY => 0x80131873 as u32,
            URT::VER_E_RET_UNINIT => 0x80131874 as u32,
            URT::VER_E_ARRAY_ACCESS => 0x80131875 as u32,
            URT::VER_E_ARRAY_V_STORE => 0x80131876 as u32,
            URT::VER_E_ARRAY_SD => 0x80131877 as u32,
            URT::VER_E_ARRAY_SD_PTR => 0x80131878 as u32,
            URT::VER_E_ARRAY_FIELD => 0x80131879 as u32,
            URT::VER_E_ARGLIST => 0x8013187A as u32,
            URT::VER_E_VALCLASS => 0x8013187B as u32,
            URT::VER_E_METHOD_ACCESS => 0x8013187C as u32,
            URT::VER_E_FIELD_ACCESS => 0x8013187D as u32,
            URT::VER_E_DEAD => 0x8013187E as u32,
            URT::VER_E_FIELD_STATIC => 0x8013187F as u32,
            URT::VER_E_FIELD_NO_STATIC => 0x80131880 as u32,
            URT::VER_E_ADDR => 0x80131881 as u32,
            URT::VER_E_ADDR_BYREF => 0x80131882 as u32,
            URT::VER_E_ADDR_LITERAL => 0x80131883 as u32,
            URT::VER_E_INITONLY => 0x80131884 as u32,
            URT::VER_E_THROW => 0x80131885 as u32,
            URT::VER_E_CALLVIRT_VALCLASS => 0x80131886 as u32,
            URT::VER_E_CALL_SIG => 0x80131887 as u32,
            URT::VER_E_CALL_STATIC => 0x80131888 as u32,
            URT::VER_E_CTOR => 0x80131889 as u32,
            URT::VER_E_CTOR_VIRT => 0x8013188A as u32,
            URT::VER_E_CTOR_OR_SUPER => 0x8013188B as u32,
            URT::VER_E_CTOR_MUL_INIT => 0x8013188C as u32,
            URT::VER_E_SIG => 0x8013188D as u32,
            URT::VER_E_SIG_ARRAY => 0x8013188E as u32,
            URT::VER_E_SIG_ARRAY_PTR => 0x8013188F as u32,
            URT::VER_E_SIG_ARRAY_BYREF => 0x80131890 as u32,
            URT::VER_E_SIG_ELEM_PTR => 0x80131891 as u32,
            URT::VER_E_SIG_VARARG => 0x80131892 as u32,
            URT::VER_E_SIG_VOID => 0x80131893 as u32,
            URT::VER_E_SIG_BYREF_BYREF => 0x80131894 as u32,
            URT::VER_E_CODE_SIZE_ZERO => 0x80131896 as u32,
            URT::VER_E_BAD_VARARG => 0x80131897 as u32,
            URT::VER_E_TAIL_CALL => 0x80131898 as u32,
            URT::VER_E_TAIL_BYREF => 0x80131899 as u32,
            URT::VER_E_TAIL_RET => 0x8013189A as u32,
            URT::VER_E_TAIL_RET_VOID => 0x8013189B as u32,
            URT::VER_E_TAIL_RET_TYPE => 0x8013189C as u32,
            URT::VER_E_TAIL_STACK_EMPTY => 0x8013189D as u32,
            URT::VER_E_METHOD_END => 0x8013189E as u32,
            URT::VER_E_BAD_BRANCH => 0x8013189F as u32,
            URT::VER_E_FIN_OVERLAP => 0x801318A0 as u32,
            URT::VER_E_LEXICAL_NESTING => 0x801318A1 as u32,
            URT::VER_E_VOLATILE => 0x801318A2 as u32,
            URT::VER_E_UNALIGNED => 0x801318A3 as u32,
            URT::VER_E_INNERMOST_FIRST => 0x801318A4 as u32,
            URT::VER_E_CALLI_VIRTUAL => 0x801318A5 as u32,
            URT::VER_E_CALL_ABSTRACT => 0x801318A6 as u32,
            URT::VER_E_STACK_UNEXP_ARRAY => 0x801318A7 as u32,
            URT::VER_E_NOT_IN_GC_HEAP => 0x801318A8 as u32,
            URT::VER_E_TRY_N_EMPTY_STACK => 0x801318A9 as u32,
            URT::VER_E_DLGT_CTOR => 0x801318AA as u32,
            URT::VER_E_DLGT_BB => 0x801318AB as u32,
            URT::VER_E_DLGT_PATTERN => 0x801318AC as u32,
            URT::VER_E_DLGT_LDFTN => 0x801318AD as u32,
            URT::VER_E_FTN_ABSTRACT => 0x801318AE as u32,
            URT::VER_E_SIG_C_VC => 0x801318AF as u32,
            URT::VER_E_SIG_VC_C => 0x801318B0 as u32,
            URT::VER_E_BOX_PTR_TO_STACK => 0x801318B1 as u32,
            URT::VER_E_SIG_BYREF_TB_AH => 0x801318B2 as u32,
            URT::VER_E_SIG_ARRAY_TB_AH => 0x801318B3 as u32,
            URT::VER_E_ENDFILTER_STACK => 0x801318B4 as u32,
            URT::VER_E_DLGT_SIG_I => 0x801318B5 as u32,
            URT::VER_E_DLGT_SIG_O => 0x801318B6 as u32,
            URT::VER_E_RA_PTR_TO_STACK => 0x801318B7 as u32,
            URT::VER_E_CATCH_VALUE_TYPE => 0x801318B8 as u32,
            URT::VER_E_CATCH_BYREF => 0x801318B9 as u32,
            URT::VER_E_FIL_PRECEED_HND => 0x801318BA as u32,
            URT::VER_E_LDVIRTFTN_STATIC => 0x801318BB as u32,
            URT::VER_E_CALLVIRT_STATIC => 0x801318BC as u32,
            URT::VER_E_INITLOCALS => 0x801318BD as u32,
            URT::VER_E_BR_TO_EXCEPTION => 0x801318BE as u32,
            URT::VER_E_CALL_CTOR => 0x801318BF as u32,
            URT::VER_E_VALCLASS_OBJREF_VAR => 0x801318C0 as u32,
            URT::VER_E_STACK_P_VALCLASS_OBJREF_VAR => 0x801318C1 as u32,
            URT::VER_E_SIG_VAR_PARAM => 0x801318C2 as u32,
            URT::VER_E_SIG_MVAR_PARAM => 0x801318C3 as u32,
            URT::VER_E_SIG_VAR_ARG => 0x801318C4 as u32,
            URT::VER_E_SIG_MVAR_ARG => 0x801318C5 as u32,
            URT::VER_E_SIG_GENERICINST => 0x801318C6 as u32,
            URT::VER_E_SIG_METHOD_INST => 0x801318C7 as u32,
            URT::VER_E_SIG_METHOD_PARENT_INST => 0x801318C8 as u32,
            URT::VER_E_SIG_FIELD_PARENT_INST => 0x801318C9 as u32,
            URT::VER_E_CALLCONV_NOT_GENERICINST => 0x801318CA as u32,
            URT::VER_E_TOKEN_BAD_METHOD_SPEC => 0x801318CB as u32,
            URT::VER_E_BAD_READONLY_PREFIX => 0x801318CC as u32,
            URT::VER_E_BAD_CONSTRAINED_PREFIX => 0x801318CD as u32,
            URT::VER_E_CIRCULAR_VAR_CONSTRAINTS => 0x801318CE as u32,
            URT::VER_E_CIRCULAR_MVAR_CONSTRAINTS => 0x801318CF as u32,
            URT::VER_E_UNSATISFIED_METHOD_INST => 0x801318D0 as u32,
            URT::VER_E_UNSATISFIED_METHOD_PARENT_INST => 0x801318D1 as u32,
            URT::VER_E_UNSATISFIED_FIELD_PARENT_INST => 0x801318D2 as u32,
            URT::VER_E_UNSATISFIED_BOX_OPERAND => 0x801318D3 as u32,
            URT::VER_E_CONSTRAINED_CALL_WITH_NON_BYREF_THIS => 0x801318D4 as u32,
            URT::VER_E_CONSTRAINED_OF_NON_VARIABLE_TYPE => 0x801318D5 as u32,
            URT::VER_E_READONLY_UNEXPECTED_CALLEE => 0x801318D6 as u32,
            URT::VER_E_READONLY_ILLEGAL_WRITE => 0x801318D7 as u32,
            URT::VER_E_READONLY_IN_MKREFANY => 0x801318D8 as u32,
            URT::VER_E_UNALIGNED_ALIGNMENT => 0x801318D9 as u32,
            URT::VER_E_TAILCALL_INSIDE_EH => 0x801318DA as u32,
            URT::VER_E_BACKWARD_BRANCH => 0x801318DB as u32,
            URT::VER_E_CALL_TO_VTYPE_BASE => 0x801318DC as u32,
            URT::VER_E_NEWOBJ_OF_ABSTRACT_CLASS => 0x801318DD as u32,
            URT::VER_E_UNMANAGED_POINTER => 0x801318DE as u32,
            URT::VER_E_LDFTN_NON_FINAL_VIRTUAL => 0x801318DF as u32,
            URT::VER_E_FIELD_OVERLAP => 0x801318E0 as u32,
            URT::VER_E_THIS_MISMATCH => 0x801318E1 as u32,
            URT::VER_E_STACK_I_I4 => 0x801318E2 as u32,
            URT::VER_E_BAD_PE => 0x801318F0 as u32,
            URT::VER_E_BAD_MD => 0x801318F1 as u32,
            URT::VER_E_BAD_APPDOMAIN => 0x801318F2 as u32,
            URT::VER_E_TYPELOAD => 0x801318F3 as u32,
            URT::VER_E_PE_LOAD => 0x801318F4 as u32,
            URT::VER_E_WRITE_RVA_STATIC => 0x801318F5 as u32,
            URT::COR_E_SqlException => 0x80131904 as u32,
            URT::COR_E_Data => 0x80131920 as u32,
            URT::COR_E_DataDeletedRowInaccessible => 0x80131921 as u32,
            URT::COR_E_DataDuplicateName => 0x80131922 as u32,
            URT::COR_E_DataInRowChangingEvent => 0x80131923 as u32,
            URT::COR_E_DataInvalidConstraint => 0x80131924 as u32,
            URT::COR_E_DataMissingPrimaryKey => 0x80131925 as u32,
            URT::COR_E_DataNoNullAllowed => 0x80131926 as u32,
            URT::COR_E_DataReadOnly => 0x80131927 as u32,
            URT::COR_E_DataRowNotInTable => 0x80131928 as u32,
            URT::COR_E_DataVersionNotFound => 0x80131929 as u32,
            URT::COR_E_DataConstraint => 0x8013192A as u32,
            URT::COR_E_StrongTyping => 0x8013192B as u32,
            URT::COR_E_SqlType => 0x80131930 as u32,
            URT::COR_E_SqlNullValue => 0x80131931 as u32,
            URT::COR_E_SqlTruncate => 0x80131932 as u32,
            URT::COR_E_AdapterMapping => 0x80131933 as u32,
            URT::COR_E_DataAdapter => 0x80131934 as u32,
            URT::COR_E_DBConcurrency => 0x80131935 as u32,
            URT::COR_E_OperationAborted => 0x80131936 as u32,
            URT::COR_E_InvalidUdt => 0x80131937 as u32,
            URT::COR_E_OdbcException => 0x80131937 as u32,
            URT::COR_E_OracleException => 0x80131938 as u32,
            URT::COR_E_Xml => 0x80131940 as u32,
            URT::COR_E_XmlSchema => 0x80131941 as u32,
            URT::COR_E_XmlXslt => 0x80131942 as u32,
            URT::COR_E_XmlXPath => 0x80131943 as u32,
            URT::COR_E_XmlQuery => 0x80131944 as u32,
            URT::VLDTR_E_IFACE_NOTIFACE => 0x80131B00 as u32,
            URT::TLBX_E_CIRCULAR_EXPORT2 => 0x80131B00 as u32,
            URT::VLDTR_E_FD_RVAHASNORVA => 0x80131B01 as u32,
            URT::VLDTR_E_FD_RVAHASZERORVA => 0x80131B02 as u32,
            URT::VLDTR_E_MD_RVAANDIMPLMAP => 0x80131B03 as u32,
            URT::VLDTR_E_TD_EXTRAFLAGS => 0x80131B04 as u32,
            URT::VLDTR_E_TD_EXTENDSITSELF => 0x80131B05 as u32,
            URT::VLDTR_E_TD_SYSVTNOTEXTOBJ => 0x80131B06 as u32,
            URT::VLDTR_E_TD_EXTTYPESPEC => 0x80131B07 as u32,
            URT::VLDTR_E_TD_VTNOSIZE => 0x80131B09 as u32,
            URT::VLDTR_E_TD_IFACESEALED => 0x80131B0A as u32,
            URT::VLDTR_E_NC_BADNESTED => 0x80131B0B as u32,
            URT::VLDTR_E_NC_BADENCLOSER => 0x80131B0C as u32,
            URT::VLDTR_E_NC_DUP => 0x80131B0D as u32,
            URT::VLDTR_E_NC_DUPENCLOSER => 0x80131B0E as u32,
            URT::VLDTR_E_FRVA_ZERORVA => 0x80131B0F as u32,
            URT::VLDTR_E_FRVA_BADFIELD => 0x80131B10 as u32,
            URT::VLDTR_E_FRVA_DUPRVA => 0x80131B11 as u32,
            URT::VLDTR_E_FRVA_DUPFIELD => 0x80131B12 as u32,
            URT::VLDTR_E_EP_BADTOKEN => 0x80131B13 as u32,
            URT::VLDTR_E_EP_INSTANCE => 0x80131B14 as u32,
            URT::VLDTR_E_TD_ENUMFLDBADTYPE => 0x80131B15 as u32,
            URT::VLDTR_E_MD_BADRVA => 0x80131B16 as u32,
            URT::VLDTR_E_FD_LITERALNODEFAULT => 0x80131B17 as u32,
            URT::VLDTR_E_IFACE_METHNOTIMPL => 0x80131B18 as u32,
            URT::VLDTR_E_CA_BADPARENT => 0x80131B19 as u32,
            URT::VLDTR_E_CA_BADTYPE => 0x80131B1A as u32,
            URT::VLDTR_E_CA_NOTCTOR => 0x80131B1B as u32,
            URT::VLDTR_E_CA_BADSIG => 0x80131B1C as u32,
            URT::VLDTR_E_CA_NOSIG => 0x80131B1D as u32,
            URT::VLDTR_E_CA_BADPROLOG => 0x80131B1E as u32,
            URT::VLDTR_E_MD_BADLOCALSIGTOK => 0x80131B1F as u32,
            URT::VLDTR_E_MD_BADHEADER => 0x80131B20 as u32,
            URT::VLDTR_E_EP_TOOMANYARGS => 0x80131B21 as u32,
            URT::VLDTR_E_EP_BADRET => 0x80131B22 as u32,
            URT::VLDTR_E_EP_BADARG => 0x80131B23 as u32,
            URT::VLDTR_E_SIG_BADVOID => 0x80131B24 as u32,
            URT::VLDTR_E_IFACE_METHMULTIMPL => 0x80131B25 as u32,
            URT::VLDTR_E_GP_NAMENULL => 0x80131B26 as u32,
            URT::VLDTR_E_GP_OWNERNIL => 0x80131B27 as u32,
            URT::VLDTR_E_GP_DUPNAME => 0x80131B28 as u32,
            URT::VLDTR_E_GP_DUPNUMBER => 0x80131B29 as u32,
            URT::VLDTR_E_GP_NONSEQ_BY_OWNER => 0x80131B2A as u32,
            URT::VLDTR_E_GP_NONSEQ_BY_NUMBER => 0x80131B2B as u32,
            URT::VLDTR_E_GP_UNEXPECTED_OWNER_FOR_VARIANT_VAR => 0x80131B2C as u32,
            URT::VLDTR_E_GP_ILLEGAL_VARIANT_MVAR => 0x80131B2D as u32,
            URT::VLDTR_E_GP_ILLEGAL_VARIANCE_FLAGS => 0x80131B2E as u32,
            URT::VLDTR_E_GP_REFANDVALUETYPE => 0x80131B2F as u32,
            URT::VLDTR_E_GPC_OWNERNIL => 0x80131B30 as u32,
            URT::VLDTR_E_GPC_DUP => 0x80131B31 as u32,
            URT::VLDTR_E_GPC_NONCONTIGUOUS => 0x80131B32 as u32,
            URT::VLDTR_E_MS_METHODNIL => 0x80131B33 as u32,
            URT::VLDTR_E_MS_DUP => 0x80131B34 as u32,
            URT::VLDTR_E_MS_BADCALLINGCONV => 0x80131B35 as u32,
            URT::VLDTR_E_MS_MISSARITY => 0x80131B36 as u32,
            URT::VLDTR_E_MS_MISSARG => 0x80131B37 as u32,
            URT::VLDTR_E_MS_ARITYMISMATCH => 0x80131B38 as u32,
            URT::VLDTR_E_MS_METHODNOTGENERIC => 0x80131B39 as u32,
            URT::VLDTR_E_SIG_MISSARITY => 0x80131B3A as u32,
            URT::VLDTR_E_SIG_ARITYMISMATCH => 0x80131B3B as u32,
            URT::VLDTR_E_MD_GENERIC_CCTOR => 0x80131B3C as u32,
            URT::VLDTR_E_MD_GENERIC_CTOR => 0x80131B3D as u32,
            URT::VLDTR_E_MD_GENERIC_IMPORT => 0x80131B3E as u32,
            URT::VLDTR_E_MD_GENERIC_BADCALLCONV => 0x80131B3F as u32,
            URT::VLDTR_E_MD_GENERIC_GLOBAL => 0x80131B40 as u32,
            URT::VLDTR_E_EP_GENERIC_METHOD => 0x80131B41 as u32,
            URT::VLDTR_E_MD_MISSARITY => 0x80131B42 as u32,
            URT::VLDTR_E_MD_ARITYZERO => 0x80131B43 as u32,
            URT::VLDTR_E_SIG_ARITYZERO => 0x80131B44 as u32,
            URT::VLDTR_E_MS_ARITYZERO => 0x80131B45 as u32,
            URT::VLDTR_E_MD_GPMISMATCH => 0x80131B46 as u32,
            URT::VLDTR_E_EP_GENERIC_TYPE => 0x80131B47 as u32,
            URT::VLDTR_E_MI_DECLNOTGENERIC => 0x80131B48 as u32,
            URT::VLDTR_E_MI_IMPLNOTGENERIC => 0x80131B49 as u32,
            URT::VLDTR_E_MI_ARITYMISMATCH => 0x80131B4A as u32,
            URT::VLDTR_E_TD_EXTBADTYPESPEC => 0x80131B4B as u32,
            URT::VLDTR_E_SIG_BYREFINST => 0x80131B4C as u32,
            URT::VLDTR_E_MS_BYREFINST => 0x80131B4D as u32,
            URT::VLDTR_E_TS_EMPTY => 0x80131B4E as u32,
            URT::VLDTR_E_TS_HASSENTINALS => 0x80131B4F as u32,
            URT::VLDTR_E_TD_GENERICHASEXPLAYOUT => 0x80131B50 as u32,
            URT::VLDTR_E_SIG_BADTOKTYPE => 0x80131B51 as u32,
            URT::VLDTR_E_IFACE_METHNOTIMPLTHISMOD => 0x80131B52 as u32,
            URT::CORDBG_E_THREAD_NOT_SCHEDULED => 0x80131C00 as u32,
            URT::CORDBG_E_HANDLE_HAS_BEEN_DISPOSED => 0x80131C01 as u32,
            URT::CORDBG_E_NONINTERCEPTABLE_EXCEPTION => 0x80131C02 as u32,
            URT::CORDBG_E_CANT_UNWIND_ABOVE_CALLBACK => 0x80131C03 as u32,
            URT::CORDBG_E_INTERCEPT_FRAME_ALREADY_SET => 0x80131C04 as u32,
            URT::CORDBG_E_NO_NATIVE_PATCH_AT_ADDR => 0x80131C05 as u32,
            URT::CORDBG_E_MUST_BE_INTEROP_DEBUGGING => 0x80131C06 as u32,
            URT::CORDBG_E_NATIVE_PATCH_ALREADY_AT_ADDR => 0x80131C07 as u32,
            URT::CORDBG_E_TIMEOUT => 0x80131C08 as u32,
            URT::CORDBG_E_CANT_CALL_ON_THIS_THREAD => 0x80131C09 as u32,
            URT::CORDBG_E_ENC_INFOLESS_METHOD => 0x80131C0A as u32,
            URT::CORDBG_E_ENC_NESTED_HANLDERS => 0x80131C0B as u32,
            URT::CORDBG_E_ENC_IN_FUNCLET => 0x80131C0C as u32,
            URT::CORDBG_E_ENC_LOCALLOC => 0x80131C0D as u32,
            URT::CORDBG_E_ENC_EDIT_NOT_SUPPORTED => 0x80131C0E as u32,
            URT::CORDBG_E_FEABORT_DELAYED_UNTIL_THREAD_RESUMED => 0x80131C0F as u32,
            URT::CORDBG_E_NOTREADY => 0x80131C10 as u32,
            URT::CORDBG_E_CANNOT_RESOLVE_ASSEMBLY => 0x80131C11 as u32,
            URT::CORDBG_E_MUST_BE_IN_LOAD_MODULE => 0x80131C12 as u32,
            URT::CORDBG_E_CANNOT_BE_ON_ATTACH => 0x80131C13 as u32,
            URT::CORDBG_E_NGEN_NOT_SUPPORTED => 0x80131C14 as u32,
            URT::CORDBG_E_ILLEGAL_SHUTDOWN_ORDER => 0x80131C15 as u32,
            URT::CORDBG_E_CANNOT_DEBUG_FIBER_PROCESS => 0x80131C16 as u32,
            URT::CORDBG_E_MUST_BE_IN_CREATE_PROCESS => 0x80131C17 as u32,
            URT::CORDBG_E_DETACH_FAILED_OUTSTANDING_EVALS => 0x80131C18 as u32,
            URT::CORDBG_E_DETACH_FAILED_OUTSTANDING_STEPPERS => 0x80131C19 as u32,
            URT::CORDBG_E_CANT_INTEROP_STEP_OUT => 0x80131C20 as u32,
            URT::CORDBG_E_DETACH_FAILED_OUTSTANDING_BREAKPOINTS => 0x80131C21 as u32,
            URT::CORDBG_E_ILLEGAL_IN_STACK_OVERFLOW => 0x80131C22 as u32,
            URT::CORDBG_E_ILLEGAL_AT_GC_UNSAFE_POINT => 0x80131C23 as u32,
            URT::CORDBG_E_ILLEGAL_IN_PROLOG => 0x80131C24 as u32,
            URT::CORDBG_E_ILLEGAL_IN_NATIVE_CODE => 0x80131C25 as u32,
            URT::CORDBG_E_ILLEGAL_IN_OPTIMIZED_CODE => 0x80131C26 as u32,
            URT::CORDBG_E_MINIDUMP_UNSUPPORTED => 0x80131C27 as u32,
            URT::CORDBG_E_APPDOMAIN_MISMATCH => 0x80131C28 as u32,
            URT::CORDBG_E_CONTEXT_UNVAILABLE => 0x80131C29 as u32,
            URT::CORDBG_E_UNCOMPATIBLE_PLATFORMS => 0x80131C30 as u32,
            URT::CORDBG_E_DEBUGGING_DISABLED => 0x80131C31 as u32,
            URT::CORDBG_E_DETACH_FAILED_ON_ENC => 0x80131C32 as u32,
            URT::CORDBG_E_CURRENT_EXCEPTION_IS_OUTSIDE_CURRENT_EXECUTION_SCOPE => 0x80131C33 as u32,
            URT::CORDBG_E_HELPER_MAY_DEADLOCK => 0x80131C34 as u32,
            URT::CORDBG_E_MISSING_METADATA => 0x80131C35 as u32,
            URT::CORDBG_E_TARGET_INCONSISTENT => 0x80131C36 as u32,
            URT::CORDBG_E_DETACH_FAILED_OUTSTANDING_TARGET_RESOURCES => 0x80131C37 as u32,
            URT::CORDBG_E_TARGET_READONLY => 0x80131C38 as u32,
            URT::CORDBG_E_MISMATCHED_CORWKS_AND_DACWKS_DLLS => 0x80131C39 as u32,
            URT::CORDBG_E_MODULE_LOADED_FROM_DISK => 0x80131C3A as u32,
            URT::CORDBG_E_SYMBOLS_NOT_AVAILABLE => 0x80131C3B as u32,
            URT::CORDBG_E_DEBUG_COMPONENT_MISSING => 0x80131C3C as u32,
            URT::CORDBG_E_REMOTE_MISMATCHED_CERTS => 0x80131C3D as u32,
            URT::CORDBG_E_REMOTE_NETWORK_FAILURE => 0x80131C3E as u32,
            URT::CORDBG_E_REMOTE_NO_LISTENER => 0x80131C3F as u32,
            URT::CORDBG_E_REMOTE_UNKNOWN_TARGET => 0x80131C40 as u32,
            URT::CORDBG_E_REMOTE_INVALID_CONFIG => 0x80131C41 as u32,
            URT::CORDBG_E_REMOTE_MISMATCHED_PROTOCOLS => 0x80131C42 as u32,
            URT::CORDBG_E_LIBRARY_PROVIDER_ERROR => 0x80131C43 as u32,
            URT::CORDBG_E_NOT_CLR => 0x80131C44 as u32,
            URT::CORDBG_E_MISSING_DATA_TARGET_INTERFACE => 0x80131C45 as u32,
            URT::CORDBG_E_UNSUPPORTED_DEBUGGING_MODEL => 0x80131C46 as u32,
            URT::CORDBG_E_UNSUPPORTED_FORWARD_COMPAT => 0x80131C47 as u32,
            URT::CORDBG_E_UNSUPPORTED_VERSION_STRUCT => 0x80131C48 as u32,
            URT::CORDBG_E_READVIRTUAL_FAILURE => 0x80131C49 as u32,
            URT::CORDBG_E_VALUE_POINTS_TO_FUNCTION => 0x80131C4A as u32,
            URT::CORDBG_E_CORRUPT_OBJECT => 0x80131C4B as u32,
            URT::CORDBG_E_GC_STRUCTURES_INVALID => 0x80131C4C as u32,
            URT::CORDBG_E_INVALID_OPCODE => 0x80131C4D as u32,
            URT::CORDBG_E_UNSUPPORTED => 0x80131C4E as u32,
            URT::CORDBG_E_MISSING_DEBUGGER_EXPORTS => 0x80131C4F as u32,
            URT::PEFMT_E_NO_CONTENTS => 0x80131D00 as u32,
            URT::PEFMT_E_NO_NTHEADERS => 0x80131D01 as u32,
            URT::PEFMT_E_64BIT => 0x80131D02 as u32,
            URT::PEFMT_E_NO_CORHEADER => 0x80131D03 as u32,
            URT::PEFMT_E_NOT_ILONLY => 0x80131D04 as u32,
            URT::PEFMT_E_IMPORT_DLLS => 0x80131D05 as u32,
            URT::PEFMT_E_EXE_NOENTRYPOINT => 0x80131D06 as u32,
            URT::PEFMT_E_BASE_RELOCS => 0x80131D07 as u32,
            URT::PEFMT_E_ENTRYPOINT => 0x80131D08 as u32,
            URT::PEFMT_E_ZERO_SIZEOFCODE => 0x80131D09 as u32,
            URT::PEFMT_E_BAD_CORHEADER => 0x80131D0A as u32,
            URT::PEFMT_E_32BIT => 0x80131D0B as u32,
            URT::CLR_OPTSVC_E_CONTROLLER_INTERRUPT => 0x80131E00 as u32,
            URT::NGEN_FAILED_GET_DEPENDENCIES => 0x80131F00 as u32,
            URT::NGEN_FAILED_NATIVE_IMAGE_DELETE => 0x80131F01 as u32,
            URT::NGEN_E_TOO_MANY_INTERFACES => 0x80131F02 as u32,
            URT::NGEN_E_OLDER_RUNTIME => 0x80131F03 as u32,
            URT::NGEN_E_WORKER_UNEXPECTED_EXIT => 0x80131F04 as u32,
            URT::NGEN_E_WORKER_UNEXPECTED_SYNC => 0x80131F05 as u32,
            URT::NGEN_E_SYS_ASM_NI_MISSING => 0x80131F06 as u32,
            URT::CLDB_E_INTERNALERROR => 0x80131FFF as u32,
        }
    }

    pub fn error(&self) -> RawError {
        return match self {
            URT::VLDTR_S_WRN => RawError::Kind(URT::VLDTR_S_WRN),
            URT::VLDTR_S_ERR => RawError::Kind(URT::VLDTR_S_ERR),
            URT::VLDTR_S_WRNERR => RawError::Kind(URT::VLDTR_S_WRNERR),
            URT::CORDBG_S_BAD_START_SEQUENCE_POINT => RawError::Kind(URT::CORDBG_S_BAD_START_SEQUENCE_POINT),
            URT::CORDBG_S_BAD_END_SEQUENCE_POINT => RawError::Kind(URT::CORDBG_S_BAD_END_SEQUENCE_POINT),
            URT::CORDBG_S_INSUFFICIENT_INFO_FOR_SET_IP => RawError::Kind(URT::CORDBG_S_INSUFFICIENT_INFO_FOR_SET_IP),
            URT::CORDBG_S_FUNC_EVAL_HAS_NO_RESULT => RawError::Kind(URT::CORDBG_S_FUNC_EVAL_HAS_NO_RESULT),
            URT::CORDBG_S_VALUE_POINTS_TO_VOID => RawError::Kind(URT::CORDBG_S_VALUE_POINTS_TO_VOID),
            URT::CORDBG_S_FUNC_EVAL_ABORTED => RawError::Kind(URT::CORDBG_S_FUNC_EVAL_ABORTED),
            URT::CEE_E_ENTRYPOINT => RawError::Kind(URT::CEE_E_ENTRYPOINT),
            URT::CEE_E_CVTRES_NOT_FOUND => RawError::Kind(URT::CEE_E_CVTRES_NOT_FOUND),
            URT::MSEE_E_LOADLIBFAILED => RawError::Kind(URT::MSEE_E_LOADLIBFAILED),
            URT::MSEE_E_GETPROCFAILED => RawError::Kind(URT::MSEE_E_GETPROCFAILED),
            URT::MSEE_E_MULTCOPIESLOADED => RawError::Kind(URT::MSEE_E_MULTCOPIESLOADED),
            URT::COR_E_TYPEUNLOADED => RawError::Kind(URT::COR_E_TYPEUNLOADED),
            URT::COR_E_APPDOMAINUNLOADED => RawError::Kind(URT::COR_E_APPDOMAINUNLOADED),
            URT::COR_E_CANNOTUNLOADAPPDOMAIN => RawError::Kind(URT::COR_E_CANNOTUNLOADAPPDOMAIN),
            URT::MSEE_E_ASSEMBLYLOADINPROGRESS => RawError::Kind(URT::MSEE_E_ASSEMBLYLOADINPROGRESS),
            URT::MSEE_E_CANNOTCREATEAPPDOMAIN => RawError::Kind(URT::MSEE_E_CANNOTCREATEAPPDOMAIN),
            URT::COR_E_ASSEMBLYEXPECTED => RawError::Kind(URT::COR_E_ASSEMBLYEXPECTED),
            URT::COR_E_FIXUPSINEXE => RawError::Kind(URT::COR_E_FIXUPSINEXE),
            URT::COR_E_NO_LOADLIBRARY_ALLOWED => RawError::Kind(URT::COR_E_NO_LOADLIBRARY_ALLOWED),
            URT::COR_E_NEWER_RUNTIME => RawError::Kind(URT::COR_E_NEWER_RUNTIME),
            URT::COR_E_CANNOT_SET_POLICY => RawError::Kind(URT::COR_E_CANNOT_SET_POLICY),
            URT::COR_E_CANNOT_SPECIFY_EVIDENCE => RawError::Kind(URT::COR_E_CANNOT_SPECIFY_EVIDENCE),
            URT::COR_E_MULTIMODULEASSEMBLIESDIALLOWED => RawError::Kind(URT::COR_E_MULTIMODULEASSEMBLIESDIALLOWED),
            URT::HOST_E_DEADLOCK => RawError::Kind(URT::HOST_E_DEADLOCK),
            URT::HOST_E_INTERRUPTED => RawError::Kind(URT::HOST_E_INTERRUPTED),
            URT::HOST_E_INVALIDOPERATION => RawError::Kind(URT::HOST_E_INVALIDOPERATION),
            URT::HOST_E_CLRNOTAVAILABLE => RawError::Kind(URT::HOST_E_CLRNOTAVAILABLE),
            URT::HOST_E_TIMEOUT => RawError::Kind(URT::HOST_E_TIMEOUT),
            URT::HOST_E_NOT_OWNER => RawError::Kind(URT::HOST_E_NOT_OWNER),
            URT::HOST_E_ABANDONED => RawError::Kind(URT::HOST_E_ABANDONED),
            URT::HOST_E_EXITPROCESS_THREADABORT => RawError::Kind(URT::HOST_E_EXITPROCESS_THREADABORT),
            URT::HOST_E_EXITPROCESS_ADUNLOAD => RawError::Kind(URT::HOST_E_EXITPROCESS_ADUNLOAD),
            URT::HOST_E_EXITPROCESS_TIMEOUT => RawError::Kind(URT::HOST_E_EXITPROCESS_TIMEOUT),
            URT::HOST_E_EXITPROCESS_OUTOFMEMORY => RawError::Kind(URT::HOST_E_EXITPROCESS_OUTOFMEMORY),
            URT::HOST_E_EXITPROCESS_STACKOVERFLOW => RawError::Kind(URT::HOST_E_EXITPROCESS_STACKOVERFLOW),
            URT::COR_E_MODULE_HASH_CHECK_FAILED => RawError::Kind(URT::COR_E_MODULE_HASH_CHECK_FAILED),
            URT::FUSION_E_REF_DEF_MISMATCH => RawError::Kind(URT::FUSION_E_REF_DEF_MISMATCH),
            URT::FUSION_E_INVALID_PRIVATE_ASM_LOCATION => RawError::Kind(URT::FUSION_E_INVALID_PRIVATE_ASM_LOCATION),
            URT::FUSION_E_ASM_MODULE_MISSING => RawError::Kind(URT::FUSION_E_ASM_MODULE_MISSING),
            URT::FUSION_E_UNEXPECTED_MODULE_FOUND => RawError::Kind(URT::FUSION_E_UNEXPECTED_MODULE_FOUND),
            URT::FUSION_E_PRIVATE_ASM_DISALLOWED => RawError::Kind(URT::FUSION_E_PRIVATE_ASM_DISALLOWED),
            URT::FUSION_E_SIGNATURE_CHECK_FAILED => RawError::Kind(URT::FUSION_E_SIGNATURE_CHECK_FAILED),
            URT::FUSION_E_DATABASE_ERROR => RawError::Kind(URT::FUSION_E_DATABASE_ERROR),
            URT::FUSION_E_INVALID_NAME => RawError::Kind(URT::FUSION_E_INVALID_NAME),
            URT::FUSION_E_CODE_DOWNLOAD_DISABLED => RawError::Kind(URT::FUSION_E_CODE_DOWNLOAD_DISABLED),
            URT::FUSION_E_UNINSTALL_DISALLOWED => RawError::Kind(URT::FUSION_E_UNINSTALL_DISALLOWED),
            URT::FUSION_E_HOST_GAC_ASM_MISMATCH => RawError::Kind(URT::FUSION_E_HOST_GAC_ASM_MISMATCH),
            URT::FUSION_E_LOADFROM_BLOCKED => RawError::Kind(URT::FUSION_E_LOADFROM_BLOCKED),
            URT::FUSION_E_CACHEFILE_FAILED => RawError::Kind(URT::FUSION_E_CACHEFILE_FAILED),
            URT::FUSION_E_APP_DOMAIN_LOCKED => RawError::Kind(URT::FUSION_E_APP_DOMAIN_LOCKED),
            URT::FUSION_E_CONFIGURATION_ERROR => RawError::Kind(URT::FUSION_E_CONFIGURATION_ERROR),
            URT::FUSION_E_MANIFEST_PARSE_ERROR => RawError::Kind(URT::FUSION_E_MANIFEST_PARSE_ERROR),
            URT::FUSION_E_INVALID_ASSEMBLY_REFERENCE => RawError::Kind(URT::FUSION_E_INVALID_ASSEMBLY_REFERENCE),
            URT::COR_E_ASSEMBLY_NOT_EXPECTED => RawError::Kind(URT::COR_E_ASSEMBLY_NOT_EXPECTED),
            URT::COR_E_LOADING_REFERENCE_ASSEMBLY => RawError::Kind(URT::COR_E_LOADING_REFERENCE_ASSEMBLY),
            URT::CLDB_E_FILE_BADREAD => RawError::Kind(URT::CLDB_E_FILE_BADREAD),
            URT::CLDB_E_FILE_BADWRITE => RawError::Kind(URT::CLDB_E_FILE_BADWRITE),
            URT::CLDB_E_FILE_READONLY => RawError::Kind(URT::CLDB_E_FILE_READONLY),
            URT::CLDB_E_NAME_ERROR => RawError::Kind(URT::CLDB_E_NAME_ERROR),
            URT::CLDB_E_TRUNCATION => RawError::Kind(URT::CLDB_E_TRUNCATION),
            URT::CLDB_E_FILE_OLDVER => RawError::Kind(URT::CLDB_E_FILE_OLDVER),
            URT::CLDB_E_RELOCATED => RawError::Kind(URT::CLDB_E_RELOCATED),
            URT::CLDB_E_SMDUPLICATE => RawError::Kind(URT::CLDB_E_SMDUPLICATE),
            URT::CLDB_E_NO_DATA => RawError::Kind(URT::CLDB_E_NO_DATA),
            URT::CLDB_E_READONLY => RawError::Kind(URT::CLDB_E_READONLY),
            URT::CLDB_E_INCOMPATIBLE => RawError::Kind(URT::CLDB_E_INCOMPATIBLE),
            URT::CLDB_E_FILE_CORRUPT => RawError::Kind(URT::CLDB_E_FILE_CORRUPT),
            URT::CLDB_E_SCHEMA_VERNOTFOUND => RawError::Kind(URT::CLDB_E_SCHEMA_VERNOTFOUND),
            URT::CLDB_E_BADUPDATEMODE => RawError::Kind(URT::CLDB_E_BADUPDATEMODE),
            URT::CLDB_E_INDEX_NONULLKEYS => RawError::Kind(URT::CLDB_E_INDEX_NONULLKEYS),
            URT::CLDB_E_INDEX_DUPLICATE => RawError::Kind(URT::CLDB_E_INDEX_DUPLICATE),
            URT::CLDB_E_INDEX_BADTYPE => RawError::Kind(URT::CLDB_E_INDEX_BADTYPE),
            URT::CLDB_E_INDEX_NOTFOUND => RawError::Kind(URT::CLDB_E_INDEX_NOTFOUND),
            URT::CLDB_E_RECORD_NOTFOUND => RawError::Kind(URT::CLDB_E_RECORD_NOTFOUND),
            URT::CLDB_E_RECORD_OVERFLOW => RawError::Kind(URT::CLDB_E_RECORD_OVERFLOW),
            URT::CLDB_E_RECORD_DUPLICATE => RawError::Kind(URT::CLDB_E_RECORD_DUPLICATE),
            URT::CLDB_E_RECORD_PKREQUIRED => RawError::Kind(URT::CLDB_E_RECORD_PKREQUIRED),
            URT::CLDB_E_RECORD_DELETED => RawError::Kind(URT::CLDB_E_RECORD_DELETED),
            URT::CLDB_E_RECORD_OUTOFORDER => RawError::Kind(URT::CLDB_E_RECORD_OUTOFORDER),
            URT::CLDB_E_COLUMN_OVERFLOW => RawError::Kind(URT::CLDB_E_COLUMN_OVERFLOW),
            URT::CLDB_E_COLUMN_READONLY => RawError::Kind(URT::CLDB_E_COLUMN_READONLY),
            URT::CLDB_E_COLUMN_SPECIALCOL => RawError::Kind(URT::CLDB_E_COLUMN_SPECIALCOL),
            URT::CLDB_E_COLUMN_PKNONULLS => RawError::Kind(URT::CLDB_E_COLUMN_PKNONULLS),
            URT::CLDB_E_TABLE_CANTDROP => RawError::Kind(URT::CLDB_E_TABLE_CANTDROP),
            URT::CLDB_E_OBJECT_NOTFOUND => RawError::Kind(URT::CLDB_E_OBJECT_NOTFOUND),
            URT::CLDB_E_OBJECT_COLNOTFOUND => RawError::Kind(URT::CLDB_E_OBJECT_COLNOTFOUND),
            URT::CLDB_E_VECTOR_BADINDEX => RawError::Kind(URT::CLDB_E_VECTOR_BADINDEX),
            URT::CLDB_E_TOO_BIG => RawError::Kind(URT::CLDB_E_TOO_BIG),
            URT::META_E_INVALID_TOKEN_TYPE => RawError::Kind(URT::META_E_INVALID_TOKEN_TYPE),
            URT::TLBX_E_INVALID_TYPEINFO => RawError::Kind(URT::TLBX_E_INVALID_TYPEINFO),
            URT::TLBX_E_INVALID_TYPEINFO_UNNAMED => RawError::Kind(URT::TLBX_E_INVALID_TYPEINFO_UNNAMED),
            URT::TLBX_E_CTX_NESTED => RawError::Kind(URT::TLBX_E_CTX_NESTED),
            URT::TLBX_E_ERROR_MESSAGE => RawError::Kind(URT::TLBX_E_ERROR_MESSAGE),
            URT::TLBX_E_CANT_SAVE => RawError::Kind(URT::TLBX_E_CANT_SAVE),
            URT::TLBX_W_LIBNOTREGISTERED => RawError::Kind(URT::TLBX_W_LIBNOTREGISTERED),
            URT::TLBX_E_CANTLOADLIBRARY => RawError::Kind(URT::TLBX_E_CANTLOADLIBRARY),
            URT::TLBX_E_BAD_VT_TYPE => RawError::Kind(URT::TLBX_E_BAD_VT_TYPE),
            URT::TLBX_E_NO_MSCOREE_TLB => RawError::Kind(URT::TLBX_E_NO_MSCOREE_TLB),
            URT::TLBX_E_BAD_MSCOREE_TLB => RawError::Kind(URT::TLBX_E_BAD_MSCOREE_TLB),
            URT::TLBX_E_TLB_EXCEPTION => RawError::Kind(URT::TLBX_E_TLB_EXCEPTION),
            URT::TLBX_E_MULTIPLE_LCIDS => RawError::Kind(URT::TLBX_E_MULTIPLE_LCIDS),
            URT::TLBX_E_AMBIGUOUS_RETURN => RawError::Kind(URT::TLBX_E_AMBIGUOUS_RETURN),
            URT::TLBX_E_DUPLICATE_TYPE_NAME => RawError::Kind(URT::TLBX_E_DUPLICATE_TYPE_NAME),
            URT::TLBX_I_NONSEQUENTIALSTRUCT => RawError::Kind(URT::TLBX_I_NONSEQUENTIALSTRUCT),
            URT::TLBX_I_RESOLVEREFFAILED => RawError::Kind(URT::TLBX_I_RESOLVEREFFAILED),
            URT::TLBX_E_ASANY => RawError::Kind(URT::TLBX_E_ASANY),
            URT::TLBX_E_INVALIDLCIDPARAM => RawError::Kind(URT::TLBX_E_INVALIDLCIDPARAM),
            URT::TLBX_E_LCIDONDISPONLYITF => RawError::Kind(URT::TLBX_E_LCIDONDISPONLYITF),
            URT::TLBX_E_NONPUBLIC_FIELD => RawError::Kind(URT::TLBX_E_NONPUBLIC_FIELD),
            URT::TLBX_E_BAD_NAMES => RawError::Kind(URT::TLBX_E_BAD_NAMES),
            URT::TLBX_E_GENERICINST_SIGNATURE => RawError::Kind(URT::TLBX_E_GENERICINST_SIGNATURE),
            URT::TLBX_E_GENERICPAR_SIGNATURE => RawError::Kind(URT::TLBX_E_GENERICPAR_SIGNATURE),
            URT::META_E_DUPLICATE => RawError::Kind(URT::META_E_DUPLICATE),
            URT::META_E_GUID_REQUIRED => RawError::Kind(URT::META_E_GUID_REQUIRED),
            URT::META_E_TYPEDEF_MISMATCH => RawError::Kind(URT::META_E_TYPEDEF_MISMATCH),
            URT::META_E_MERGE_COLLISION => RawError::Kind(URT::META_E_MERGE_COLLISION),
            URT::TLBX_E_NO_SAFEHANDLE_ARRAYS => RawError::Kind(URT::TLBX_E_NO_SAFEHANDLE_ARRAYS),
            URT::META_E_METHD_NOT_FOUND => RawError::Kind(URT::META_E_METHD_NOT_FOUND),
            URT::META_E_FIELD_NOT_FOUND => RawError::Kind(URT::META_E_FIELD_NOT_FOUND),
            URT::META_E_PARAM_MISMATCH => RawError::Kind(URT::META_E_PARAM_MISMATCH),
            URT::META_E_BADMETADATA => RawError::Kind(URT::META_E_BADMETADATA),
            URT::META_E_INTFCEIMPL_NOT_FOUND => RawError::Kind(URT::META_E_INTFCEIMPL_NOT_FOUND),
            URT::TLBX_E_NO_CRITICALHANDLE_ARRAYS => RawError::Kind(URT::TLBX_E_NO_CRITICALHANDLE_ARRAYS),
            URT::META_E_CLASS_LAYOUT_INCONSISTENT => RawError::Kind(URT::META_E_CLASS_LAYOUT_INCONSISTENT),
            URT::META_E_FIELD_MARSHAL_NOT_FOUND => RawError::Kind(URT::META_E_FIELD_MARSHAL_NOT_FOUND),
            URT::META_E_METHODSEM_NOT_FOUND => RawError::Kind(URT::META_E_METHODSEM_NOT_FOUND),
            URT::META_E_EVENT_NOT_FOUND => RawError::Kind(URT::META_E_EVENT_NOT_FOUND),
            URT::META_E_PROP_NOT_FOUND => RawError::Kind(URT::META_E_PROP_NOT_FOUND),
            URT::META_E_BAD_SIGNATURE => RawError::Kind(URT::META_E_BAD_SIGNATURE),
            URT::META_E_BAD_INPUT_PARAMETER => RawError::Kind(URT::META_E_BAD_INPUT_PARAMETER),
            URT::META_E_METHDIMPL_INCONSISTENT => RawError::Kind(URT::META_E_METHDIMPL_INCONSISTENT),
            URT::META_E_MD_INCONSISTENCY => RawError::Kind(URT::META_E_MD_INCONSISTENCY),
            URT::META_E_CANNOTRESOLVETYPEREF => RawError::Kind(URT::META_E_CANNOTRESOLVETYPEREF),
            URT::META_E_STRINGSPACE_FULL => RawError::Kind(URT::META_E_STRINGSPACE_FULL),
            URT::META_E_UNEXPECTED_REMAP => RawError::Kind(URT::META_E_UNEXPECTED_REMAP),
            URT::META_E_HAS_UNMARKALL => RawError::Kind(URT::META_E_HAS_UNMARKALL),
            URT::META_E_MUST_CALL_UNMARKALL => RawError::Kind(URT::META_E_MUST_CALL_UNMARKALL),
            URT::META_E_GENERICPARAM_INCONSISTENT => RawError::Kind(URT::META_E_GENERICPARAM_INCONSISTENT),
            URT::META_E_EVENT_COUNTS => RawError::Kind(URT::META_E_EVENT_COUNTS),
            URT::META_E_PROPERTY_COUNTS => RawError::Kind(URT::META_E_PROPERTY_COUNTS),
            URT::META_E_TYPEDEF_MISSING => RawError::Kind(URT::META_E_TYPEDEF_MISSING),
            URT::TLBX_E_CANT_LOAD_MODULE => RawError::Kind(URT::TLBX_E_CANT_LOAD_MODULE),
            URT::TLBX_E_CANT_LOAD_CLASS => RawError::Kind(URT::TLBX_E_CANT_LOAD_CLASS),
            URT::TLBX_E_NULL_MODULE => RawError::Kind(URT::TLBX_E_NULL_MODULE),
            URT::TLBX_E_NO_CLSID_KEY => RawError::Kind(URT::TLBX_E_NO_CLSID_KEY),
            URT::TLBX_E_CIRCULAR_EXPORT => RawError::Kind(URT::TLBX_E_CIRCULAR_EXPORT),
            URT::TLBX_E_CIRCULAR_IMPORT => RawError::Kind(URT::TLBX_E_CIRCULAR_IMPORT),
            URT::TLBX_E_BAD_NATIVETYPE => RawError::Kind(URT::TLBX_E_BAD_NATIVETYPE),
            URT::TLBX_E_BAD_VTABLE => RawError::Kind(URT::TLBX_E_BAD_VTABLE),
            URT::TLBX_E_CRM_NON_STATIC => RawError::Kind(URT::TLBX_E_CRM_NON_STATIC),
            URT::TLBX_E_CRM_INVALID_SIG => RawError::Kind(URT::TLBX_E_CRM_INVALID_SIG),
            URT::TLBX_E_CLASS_LOAD_EXCEPTION => RawError::Kind(URT::TLBX_E_CLASS_LOAD_EXCEPTION),
            URT::TLBX_E_UNKNOWN_SIGNATURE => RawError::Kind(URT::TLBX_E_UNKNOWN_SIGNATURE),
            URT::TLBX_E_REFERENCED_TYPELIB => RawError::Kind(URT::TLBX_E_REFERENCED_TYPELIB),
            URT::TLBX_E_INVALID_NAMESPACE => RawError::Kind(URT::TLBX_E_INVALID_NAMESPACE),
            URT::TLBX_E_LAYOUT_ERROR => RawError::Kind(URT::TLBX_E_LAYOUT_ERROR),
            URT::TLBX_E_NOTIUNKNOWN => RawError::Kind(URT::TLBX_E_NOTIUNKNOWN),
            URT::TLBX_E_NONVISIBLEVALUECLASS => RawError::Kind(URT::TLBX_E_NONVISIBLEVALUECLASS),
            URT::TLBX_E_LPTSTR_NOT_ALLOWED => RawError::Kind(URT::TLBX_E_LPTSTR_NOT_ALLOWED),
            URT::TLBX_E_AUTO_CS_NOT_ALLOWED => RawError::Kind(URT::TLBX_E_AUTO_CS_NOT_ALLOWED),
            URT::TLBX_S_NOSTDINTERFACE => RawError::Kind(URT::TLBX_S_NOSTDINTERFACE),
            URT::TLBX_S_DUPLICATE_DISPID => RawError::Kind(URT::TLBX_S_DUPLICATE_DISPID),
            URT::TLBX_E_ENUM_VALUE_INVALID => RawError::Kind(URT::TLBX_E_ENUM_VALUE_INVALID),
            URT::TLBX_E_DUPLICATE_IID => RawError::Kind(URT::TLBX_E_DUPLICATE_IID),
            URT::TLBX_E_NO_NESTED_ARRAYS => RawError::Kind(URT::TLBX_E_NO_NESTED_ARRAYS),
            URT::TLBX_E_PARAM_ERROR_NAMED => RawError::Kind(URT::TLBX_E_PARAM_ERROR_NAMED),
            URT::TLBX_E_PARAM_ERROR_UNNAMED => RawError::Kind(URT::TLBX_E_PARAM_ERROR_UNNAMED),
            URT::TLBX_E_AGNOST_SIGNATURE => RawError::Kind(URT::TLBX_E_AGNOST_SIGNATURE),
            URT::TLBX_E_CONVERT_FAIL => RawError::Kind(URT::TLBX_E_CONVERT_FAIL),
            URT::TLBX_W_DUAL_NOT_DISPATCH => RawError::Kind(URT::TLBX_W_DUAL_NOT_DISPATCH),
            URT::TLBX_E_BAD_SIGNATURE => RawError::Kind(URT::TLBX_E_BAD_SIGNATURE),
            URT::TLBX_E_ARRAY_NEEDS_NT_FIXED => RawError::Kind(URT::TLBX_E_ARRAY_NEEDS_NT_FIXED),
            URT::TLBX_E_CLASS_NEEDS_NT_INTF => RawError::Kind(URT::TLBX_E_CLASS_NEEDS_NT_INTF),
            URT::META_E_CA_INVALID_TARGET => RawError::Kind(URT::META_E_CA_INVALID_TARGET),
            URT::META_E_CA_INVALID_VALUE => RawError::Kind(URT::META_E_CA_INVALID_VALUE),
            URT::META_E_CA_INVALID_BLOB => RawError::Kind(URT::META_E_CA_INVALID_BLOB),
            URT::META_E_CA_REPEATED_ARG => RawError::Kind(URT::META_E_CA_REPEATED_ARG),
            URT::META_E_CA_UNKNOWN_ARGUMENT => RawError::Kind(URT::META_E_CA_UNKNOWN_ARGUMENT),
            URT::META_E_CA_VARIANT_NYI => RawError::Kind(URT::META_E_CA_VARIANT_NYI),
            URT::META_E_CA_ARRAY_NYI => RawError::Kind(URT::META_E_CA_ARRAY_NYI),
            URT::META_E_CA_UNEXPECTED_TYPE => RawError::Kind(URT::META_E_CA_UNEXPECTED_TYPE),
            URT::META_E_CA_INVALID_ARGTYPE => RawError::Kind(URT::META_E_CA_INVALID_ARGTYPE),
            URT::META_E_CA_INVALID_ARG_FOR_TYPE => RawError::Kind(URT::META_E_CA_INVALID_ARG_FOR_TYPE),
            URT::META_E_CA_INVALID_UUID => RawError::Kind(URT::META_E_CA_INVALID_UUID),
            URT::META_E_CA_INVALID_MARSHALAS_FIELDS => RawError::Kind(URT::META_E_CA_INVALID_MARSHALAS_FIELDS),
            URT::META_E_CA_NT_FIELDONLY => RawError::Kind(URT::META_E_CA_NT_FIELDONLY),
            URT::META_E_CA_NEGATIVE_PARAMINDEX => RawError::Kind(URT::META_E_CA_NEGATIVE_PARAMINDEX),
            URT::META_E_CA_NEGATIVE_MULTIPLIER => RawError::Kind(URT::META_E_CA_NEGATIVE_MULTIPLIER),
            URT::META_E_CA_NEGATIVE_CONSTSIZE => RawError::Kind(URT::META_E_CA_NEGATIVE_CONSTSIZE),
            URT::META_E_CA_FIXEDSTR_SIZE_REQUIRED => RawError::Kind(URT::META_E_CA_FIXEDSTR_SIZE_REQUIRED),
            URT::META_E_CA_CUSTMARSH_TYPE_REQUIRED => RawError::Kind(URT::META_E_CA_CUSTMARSH_TYPE_REQUIRED),
            URT::META_E_CA_FILENAME_REQUIRED => RawError::Kind(URT::META_E_CA_FILENAME_REQUIRED),
            URT::TLBX_W_NO_PROPS_IN_EVENTS => RawError::Kind(URT::TLBX_W_NO_PROPS_IN_EVENTS),
            URT::META_E_NOT_IN_ENC_MODE => RawError::Kind(URT::META_E_NOT_IN_ENC_MODE),
            URT::META_E_METHOD_COUNTS => RawError::Kind(URT::META_E_METHOD_COUNTS),
            URT::META_E_FIELD_COUNTS => RawError::Kind(URT::META_E_FIELD_COUNTS),
            URT::META_E_PARAM_COUNTS => RawError::Kind(URT::META_E_PARAM_COUNTS),
            URT::TLBX_E_TYPED_REF => RawError::Kind(URT::TLBX_E_TYPED_REF),
            URT::TLBX_W_DEFAULT_INTF_NOT_VISIBLE => RawError::Kind(URT::TLBX_W_DEFAULT_INTF_NOT_VISIBLE),
            URT::TLBX_W_BAD_SAFEARRAYFIELD_NO_ELEMENTVT => RawError::Kind(URT::TLBX_W_BAD_SAFEARRAYFIELD_NO_ELEMENTVT),
            URT::TLBX_W_LAYOUTCLASS_AS_INTERFACE => RawError::Kind(URT::TLBX_W_LAYOUTCLASS_AS_INTERFACE),
            URT::TLBX_I_GENERIC_BASE_TYPE => RawError::Kind(URT::TLBX_I_GENERIC_BASE_TYPE),
            URT::TLBX_E_BITNESS_MISMATCH => RawError::Kind(URT::TLBX_E_BITNESS_MISMATCH),
            URT::TLBX_E_EVENT_WITH_NEWENUM => RawError::Kind(URT::TLBX_E_EVENT_WITH_NEWENUM),
            URT::TLBX_E_PROPGET_WITHOUT_RETURN => RawError::Kind(URT::TLBX_E_PROPGET_WITHOUT_RETURN),
            URT::META_E_MISMATCHED_VISIBLITY => RawError::Kind(URT::META_E_MISMATCHED_VISIBLITY),
            URT::META_E_CA_BAD_FRIENDS_ARGS => RawError::Kind(URT::META_E_CA_BAD_FRIENDS_ARGS),
            URT::META_E_CA_FRIENDS_SN_REQUIRED => RawError::Kind(URT::META_E_CA_FRIENDS_SN_REQUIRED),
            URT::VLDTR_E_RID_OUTOFRANGE => RawError::Kind(URT::VLDTR_E_RID_OUTOFRANGE),
            URT::VLDTR_E_CDTKN_OUTOFRANGE => RawError::Kind(URT::VLDTR_E_CDTKN_OUTOFRANGE),
            URT::VLDTR_E_CDRID_OUTOFRANGE => RawError::Kind(URT::VLDTR_E_CDRID_OUTOFRANGE),
            URT::VLDTR_E_STRING_INVALID => RawError::Kind(URT::VLDTR_E_STRING_INVALID),
            URT::VLDTR_E_GUID_INVALID => RawError::Kind(URT::VLDTR_E_GUID_INVALID),
            URT::VLDTR_E_BLOB_INVALID => RawError::Kind(URT::VLDTR_E_BLOB_INVALID),
            URT::VLDTR_E_MOD_MULTI => RawError::Kind(URT::VLDTR_E_MOD_MULTI),
            URT::VLDTR_E_MOD_NULLMVID => RawError::Kind(URT::VLDTR_E_MOD_NULLMVID),
            URT::VLDTR_E_TR_NAMENULL => RawError::Kind(URT::VLDTR_E_TR_NAMENULL),
            URT::VLDTR_E_TR_DUP => RawError::Kind(URT::VLDTR_E_TR_DUP),
            URT::VLDTR_E_TD_NAMENULL => RawError::Kind(URT::VLDTR_E_TD_NAMENULL),
            URT::VLDTR_E_TD_DUPNAME => RawError::Kind(URT::VLDTR_E_TD_DUPNAME),
            URT::VLDTR_E_TD_DUPGUID => RawError::Kind(URT::VLDTR_E_TD_DUPGUID),
            URT::VLDTR_E_TD_NOTIFACEOBJEXTNULL => RawError::Kind(URT::VLDTR_E_TD_NOTIFACEOBJEXTNULL),
            URT::VLDTR_E_TD_OBJEXTENDSNONNULL => RawError::Kind(URT::VLDTR_E_TD_OBJEXTENDSNONNULL),
            URT::VLDTR_E_TD_EXTENDSSEALED => RawError::Kind(URT::VLDTR_E_TD_EXTENDSSEALED),
            URT::VLDTR_E_TD_DLTNORTSPCL => RawError::Kind(URT::VLDTR_E_TD_DLTNORTSPCL),
            URT::VLDTR_E_TD_RTSPCLNOTDLT => RawError::Kind(URT::VLDTR_E_TD_RTSPCLNOTDLT),
            URT::VLDTR_E_MI_DECLPRIV => RawError::Kind(URT::VLDTR_E_MI_DECLPRIV),
            URT::VLDTR_E_AS_BADNAME => RawError::Kind(URT::VLDTR_E_AS_BADNAME),
            URT::VLDTR_E_FILE_SYSNAME => RawError::Kind(URT::VLDTR_E_FILE_SYSNAME),
            URT::VLDTR_E_MI_BODYSTATIC => RawError::Kind(URT::VLDTR_E_MI_BODYSTATIC),
            URT::VLDTR_E_TD_IFACENOTABS => RawError::Kind(URT::VLDTR_E_TD_IFACENOTABS),
            URT::VLDTR_E_TD_IFACEPARNOTNIL => RawError::Kind(URT::VLDTR_E_TD_IFACEPARNOTNIL),
            URT::VLDTR_E_TD_IFACEGUIDNULL => RawError::Kind(URT::VLDTR_E_TD_IFACEGUIDNULL),
            URT::VLDTR_E_MI_DECLFINAL => RawError::Kind(URT::VLDTR_E_MI_DECLFINAL),
            URT::VLDTR_E_TD_VTNOTSEAL => RawError::Kind(URT::VLDTR_E_TD_VTNOTSEAL),
            URT::VLDTR_E_PD_BADFLAGS => RawError::Kind(URT::VLDTR_E_PD_BADFLAGS),
            URT::VLDTR_E_IFACE_DUP => RawError::Kind(URT::VLDTR_E_IFACE_DUP),
            URT::VLDTR_E_MR_NAMENULL => RawError::Kind(URT::VLDTR_E_MR_NAMENULL),
            URT::VLDTR_E_MR_VTBLNAME => RawError::Kind(URT::VLDTR_E_MR_VTBLNAME),
            URT::VLDTR_E_MR_DELNAME => RawError::Kind(URT::VLDTR_E_MR_DELNAME),
            URT::VLDTR_E_MR_PARNIL => RawError::Kind(URT::VLDTR_E_MR_PARNIL),
            URT::VLDTR_E_MR_BADCALLINGCONV => RawError::Kind(URT::VLDTR_E_MR_BADCALLINGCONV),
            URT::VLDTR_E_MR_NOTVARARG => RawError::Kind(URT::VLDTR_E_MR_NOTVARARG),
            URT::VLDTR_E_MR_NAMEDIFF => RawError::Kind(URT::VLDTR_E_MR_NAMEDIFF),
            URT::VLDTR_E_MR_SIGDIFF => RawError::Kind(URT::VLDTR_E_MR_SIGDIFF),
            URT::VLDTR_E_MR_DUP => RawError::Kind(URT::VLDTR_E_MR_DUP),
            URT::VLDTR_E_CL_TDAUTO => RawError::Kind(URT::VLDTR_E_CL_TDAUTO),
            URT::VLDTR_E_CL_BADPCKSZ => RawError::Kind(URT::VLDTR_E_CL_BADPCKSZ),
            URT::VLDTR_E_CL_DUP => RawError::Kind(URT::VLDTR_E_CL_DUP),
            URT::VLDTR_E_FL_BADOFFSET => RawError::Kind(URT::VLDTR_E_FL_BADOFFSET),
            URT::VLDTR_E_FL_TDNIL => RawError::Kind(URT::VLDTR_E_FL_TDNIL),
            URT::VLDTR_E_FL_NOCL => RawError::Kind(URT::VLDTR_E_FL_NOCL),
            URT::VLDTR_E_FL_TDNOTEXPLCT => RawError::Kind(URT::VLDTR_E_FL_TDNOTEXPLCT),
            URT::VLDTR_E_FL_FLDSTATIC => RawError::Kind(URT::VLDTR_E_FL_FLDSTATIC),
            URT::VLDTR_E_FL_DUP => RawError::Kind(URT::VLDTR_E_FL_DUP),
            URT::VLDTR_E_MODREF_NAMENULL => RawError::Kind(URT::VLDTR_E_MODREF_NAMENULL),
            URT::VLDTR_E_MODREF_DUP => RawError::Kind(URT::VLDTR_E_MODREF_DUP),
            URT::VLDTR_E_TR_BADSCOPE => RawError::Kind(URT::VLDTR_E_TR_BADSCOPE),
            URT::VLDTR_E_TD_NESTEDNOENCL => RawError::Kind(URT::VLDTR_E_TD_NESTEDNOENCL),
            URT::VLDTR_E_TD_EXTTRRES => RawError::Kind(URT::VLDTR_E_TD_EXTTRRES),
            URT::VLDTR_E_SIGNULL => RawError::Kind(URT::VLDTR_E_SIGNULL),
            URT::VLDTR_E_SIGNODATA => RawError::Kind(URT::VLDTR_E_SIGNODATA),
            URT::VLDTR_E_MD_BADCALLINGCONV => RawError::Kind(URT::VLDTR_E_MD_BADCALLINGCONV),
            URT::VLDTR_E_MD_THISSTATIC => RawError::Kind(URT::VLDTR_E_MD_THISSTATIC),
            URT::VLDTR_E_MD_NOTTHISNOTSTATIC => RawError::Kind(URT::VLDTR_E_MD_NOTTHISNOTSTATIC),
            URT::VLDTR_E_MD_NOARGCNT => RawError::Kind(URT::VLDTR_E_MD_NOARGCNT),
            URT::VLDTR_E_SIG_MISSELTYPE => RawError::Kind(URT::VLDTR_E_SIG_MISSELTYPE),
            URT::VLDTR_E_SIG_MISSTKN => RawError::Kind(URT::VLDTR_E_SIG_MISSTKN),
            URT::VLDTR_E_SIG_TKNBAD => RawError::Kind(URT::VLDTR_E_SIG_TKNBAD),
            URT::VLDTR_E_SIG_MISSFPTR => RawError::Kind(URT::VLDTR_E_SIG_MISSFPTR),
            URT::VLDTR_E_SIG_MISSFPTRARGCNT => RawError::Kind(URT::VLDTR_E_SIG_MISSFPTRARGCNT),
            URT::VLDTR_E_SIG_MISSRANK => RawError::Kind(URT::VLDTR_E_SIG_MISSRANK),
            URT::VLDTR_E_SIG_MISSNSIZE => RawError::Kind(URT::VLDTR_E_SIG_MISSNSIZE),
            URT::VLDTR_E_SIG_MISSSIZE => RawError::Kind(URT::VLDTR_E_SIG_MISSSIZE),
            URT::VLDTR_E_SIG_MISSNLBND => RawError::Kind(URT::VLDTR_E_SIG_MISSNLBND),
            URT::VLDTR_E_SIG_MISSLBND => RawError::Kind(URT::VLDTR_E_SIG_MISSLBND),
            URT::VLDTR_E_SIG_BADELTYPE => RawError::Kind(URT::VLDTR_E_SIG_BADELTYPE),
            URT::VLDTR_E_SIG_MISSVASIZE => RawError::Kind(URT::VLDTR_E_SIG_MISSVASIZE),
            URT::VLDTR_E_FD_BADCALLINGCONV => RawError::Kind(URT::VLDTR_E_FD_BADCALLINGCONV),
            URT::VLDTR_E_MD_NAMENULL => RawError::Kind(URT::VLDTR_E_MD_NAMENULL),
            URT::VLDTR_E_MD_PARNIL => RawError::Kind(URT::VLDTR_E_MD_PARNIL),
            URT::VLDTR_E_MD_DUP => RawError::Kind(URT::VLDTR_E_MD_DUP),
            URT::VLDTR_E_FD_NAMENULL => RawError::Kind(URT::VLDTR_E_FD_NAMENULL),
            URT::VLDTR_E_FD_PARNIL => RawError::Kind(URT::VLDTR_E_FD_PARNIL),
            URT::VLDTR_E_FD_DUP => RawError::Kind(URT::VLDTR_E_FD_DUP),
            URT::VLDTR_E_AS_MULTI => RawError::Kind(URT::VLDTR_E_AS_MULTI),
            URT::VLDTR_E_AS_NAMENULL => RawError::Kind(URT::VLDTR_E_AS_NAMENULL),
            URT::VLDTR_E_SIG_TOKTYPEMISMATCH => RawError::Kind(URT::VLDTR_E_SIG_TOKTYPEMISMATCH),
            URT::VLDTR_E_CL_TDINTF => RawError::Kind(URT::VLDTR_E_CL_TDINTF),
            URT::VLDTR_E_ASOS_OSPLTFRMIDINVAL => RawError::Kind(URT::VLDTR_E_ASOS_OSPLTFRMIDINVAL),
            URT::VLDTR_E_AR_NAMENULL => RawError::Kind(URT::VLDTR_E_AR_NAMENULL),
            URT::VLDTR_E_TD_ENCLNOTNESTED => RawError::Kind(URT::VLDTR_E_TD_ENCLNOTNESTED),
            URT::VLDTR_E_AROS_OSPLTFRMIDINVAL => RawError::Kind(URT::VLDTR_E_AROS_OSPLTFRMIDINVAL),
            URT::VLDTR_E_FILE_NAMENULL => RawError::Kind(URT::VLDTR_E_FILE_NAMENULL),
            URT::VLDTR_E_CT_NAMENULL => RawError::Kind(URT::VLDTR_E_CT_NAMENULL),
            URT::VLDTR_E_TD_EXTENDSCHILD => RawError::Kind(URT::VLDTR_E_TD_EXTENDSCHILD),
            URT::VLDTR_E_MAR_NAMENULL => RawError::Kind(URT::VLDTR_E_MAR_NAMENULL),
            URT::VLDTR_E_FILE_DUP => RawError::Kind(URT::VLDTR_E_FILE_DUP),
            URT::VLDTR_E_FILE_NAMEFULLQLFD => RawError::Kind(URT::VLDTR_E_FILE_NAMEFULLQLFD),
            URT::VLDTR_E_CT_DUP => RawError::Kind(URT::VLDTR_E_CT_DUP),
            URT::VLDTR_E_MAR_DUP => RawError::Kind(URT::VLDTR_E_MAR_DUP),
            URT::VLDTR_E_MAR_NOTPUBPRIV => RawError::Kind(URT::VLDTR_E_MAR_NOTPUBPRIV),
            URT::VLDTR_E_TD_ENUMNOVALUE => RawError::Kind(URT::VLDTR_E_TD_ENUMNOVALUE),
            URT::VLDTR_E_TD_ENUMVALSTATIC => RawError::Kind(URT::VLDTR_E_TD_ENUMVALSTATIC),
            URT::VLDTR_E_TD_ENUMVALNOTSN => RawError::Kind(URT::VLDTR_E_TD_ENUMVALNOTSN),
            URT::VLDTR_E_TD_ENUMFLDNOTST => RawError::Kind(URT::VLDTR_E_TD_ENUMFLDNOTST),
            URT::VLDTR_E_TD_ENUMFLDNOTLIT => RawError::Kind(URT::VLDTR_E_TD_ENUMFLDNOTLIT),
            URT::VLDTR_E_TD_ENUMNOLITFLDS => RawError::Kind(URT::VLDTR_E_TD_ENUMNOLITFLDS),
            URT::VLDTR_E_TD_ENUMFLDSIGMISMATCH => RawError::Kind(URT::VLDTR_E_TD_ENUMFLDSIGMISMATCH),
            URT::VLDTR_E_TD_ENUMVALNOT1ST => RawError::Kind(URT::VLDTR_E_TD_ENUMVALNOT1ST),
            URT::VLDTR_E_FD_NOTVALUERTSN => RawError::Kind(URT::VLDTR_E_FD_NOTVALUERTSN),
            URT::VLDTR_E_FD_VALUEPARNOTENUM => RawError::Kind(URT::VLDTR_E_FD_VALUEPARNOTENUM),
            URT::VLDTR_E_FD_INSTINIFACE => RawError::Kind(URT::VLDTR_E_FD_INSTINIFACE),
            URT::VLDTR_E_FD_NOTPUBINIFACE => RawError::Kind(URT::VLDTR_E_FD_NOTPUBINIFACE),
            URT::VLDTR_E_FMD_GLOBALNOTPUBPRIVSC => RawError::Kind(URT::VLDTR_E_FMD_GLOBALNOTPUBPRIVSC),
            URT::VLDTR_E_FMD_GLOBALNOTSTATIC => RawError::Kind(URT::VLDTR_E_FMD_GLOBALNOTSTATIC),
            URT::VLDTR_E_FD_GLOBALNORVA => RawError::Kind(URT::VLDTR_E_FD_GLOBALNORVA),
            URT::VLDTR_E_MD_CTORZERORVA => RawError::Kind(URT::VLDTR_E_MD_CTORZERORVA),
            URT::VLDTR_E_FD_MARKEDNOMARSHAL => RawError::Kind(URT::VLDTR_E_FD_MARKEDNOMARSHAL),
            URT::VLDTR_E_FD_MARSHALNOTMARKED => RawError::Kind(URT::VLDTR_E_FD_MARSHALNOTMARKED),
            URT::VLDTR_E_FD_MARKEDNODEFLT => RawError::Kind(URT::VLDTR_E_FD_MARKEDNODEFLT),
            URT::VLDTR_E_FD_DEFLTNOTMARKED => RawError::Kind(URT::VLDTR_E_FD_DEFLTNOTMARKED),
            URT::VLDTR_E_FMD_MARKEDNOSECUR => RawError::Kind(URT::VLDTR_E_FMD_MARKEDNOSECUR),
            URT::VLDTR_E_FMD_SECURNOTMARKED => RawError::Kind(URT::VLDTR_E_FMD_SECURNOTMARKED),
            URT::VLDTR_E_FMD_PINVOKENOTSTATIC => RawError::Kind(URT::VLDTR_E_FMD_PINVOKENOTSTATIC),
            URT::VLDTR_E_FMD_MARKEDNOPINVOKE => RawError::Kind(URT::VLDTR_E_FMD_MARKEDNOPINVOKE),
            URT::VLDTR_E_FMD_PINVOKENOTMARKED => RawError::Kind(URT::VLDTR_E_FMD_PINVOKENOTMARKED),
            URT::VLDTR_E_FMD_BADIMPLMAP => RawError::Kind(URT::VLDTR_E_FMD_BADIMPLMAP),
            URT::VLDTR_E_IMAP_BADMODREF => RawError::Kind(URT::VLDTR_E_IMAP_BADMODREF),
            URT::VLDTR_E_IMAP_BADMEMBER => RawError::Kind(URT::VLDTR_E_IMAP_BADMEMBER),
            URT::VLDTR_E_IMAP_BADIMPORTNAME => RawError::Kind(URT::VLDTR_E_IMAP_BADIMPORTNAME),
            URT::VLDTR_E_IMAP_BADCALLCONV => RawError::Kind(URT::VLDTR_E_IMAP_BADCALLCONV),
            URT::VLDTR_E_FMD_BADACCESSFLAG => RawError::Kind(URT::VLDTR_E_FMD_BADACCESSFLAG),
            URT::VLDTR_E_FD_INITONLYANDLITERAL => RawError::Kind(URT::VLDTR_E_FD_INITONLYANDLITERAL),
            URT::VLDTR_E_FD_LITERALNOTSTATIC => RawError::Kind(URT::VLDTR_E_FD_LITERALNOTSTATIC),
            URT::VLDTR_E_FMD_RTSNNOTSN => RawError::Kind(URT::VLDTR_E_FMD_RTSNNOTSN),
            URT::VLDTR_E_MD_ABSTPARNOTABST => RawError::Kind(URT::VLDTR_E_MD_ABSTPARNOTABST),
            URT::VLDTR_E_MD_NOTSTATABSTININTF => RawError::Kind(URT::VLDTR_E_MD_NOTSTATABSTININTF),
            URT::VLDTR_E_MD_NOTPUBININTF => RawError::Kind(URT::VLDTR_E_MD_NOTPUBININTF),
            URT::VLDTR_E_MD_CTORININTF => RawError::Kind(URT::VLDTR_E_MD_CTORININTF),
            URT::VLDTR_E_MD_GLOBALCTORCCTOR => RawError::Kind(URT::VLDTR_E_MD_GLOBALCTORCCTOR),
            URT::VLDTR_E_MD_CTORSTATIC => RawError::Kind(URT::VLDTR_E_MD_CTORSTATIC),
            URT::VLDTR_E_MD_CTORNOTSNRTSN => RawError::Kind(URT::VLDTR_E_MD_CTORNOTSNRTSN),
            URT::VLDTR_E_MD_CTORVIRT => RawError::Kind(URT::VLDTR_E_MD_CTORVIRT),
            URT::VLDTR_E_MD_CTORABST => RawError::Kind(URT::VLDTR_E_MD_CTORABST),
            URT::VLDTR_E_MD_CCTORNOTSTATIC => RawError::Kind(URT::VLDTR_E_MD_CCTORNOTSTATIC),
            URT::VLDTR_E_MD_ZERORVA => RawError::Kind(URT::VLDTR_E_MD_ZERORVA),
            URT::VLDTR_E_MD_FINNOTVIRT => RawError::Kind(URT::VLDTR_E_MD_FINNOTVIRT),
            URT::VLDTR_E_MD_STATANDFINORVIRT => RawError::Kind(URT::VLDTR_E_MD_STATANDFINORVIRT),
            URT::VLDTR_E_MD_ABSTANDFINAL => RawError::Kind(URT::VLDTR_E_MD_ABSTANDFINAL),
            URT::VLDTR_E_MD_ABSTANDIMPL => RawError::Kind(URT::VLDTR_E_MD_ABSTANDIMPL),
            URT::VLDTR_E_MD_ABSTANDPINVOKE => RawError::Kind(URT::VLDTR_E_MD_ABSTANDPINVOKE),
            URT::VLDTR_E_MD_ABSTNOTVIRT => RawError::Kind(URT::VLDTR_E_MD_ABSTNOTVIRT),
            URT::VLDTR_E_MD_NOTABSTNOTIMPL => RawError::Kind(URT::VLDTR_E_MD_NOTABSTNOTIMPL),
            URT::VLDTR_E_MD_NOTABSTBADFLAGSRVA => RawError::Kind(URT::VLDTR_E_MD_NOTABSTBADFLAGSRVA),
            URT::VLDTR_E_MD_PRIVSCOPENORVA => RawError::Kind(URT::VLDTR_E_MD_PRIVSCOPENORVA),
            URT::VLDTR_E_MD_GLOBALABSTORVIRT => RawError::Kind(URT::VLDTR_E_MD_GLOBALABSTORVIRT),
            URT::VLDTR_E_SIG_LONGFORM => RawError::Kind(URT::VLDTR_E_SIG_LONGFORM),
            URT::VLDTR_E_MD_MULTIPLESEMANTICS => RawError::Kind(URT::VLDTR_E_MD_MULTIPLESEMANTICS),
            URT::VLDTR_E_MD_INVALIDSEMANTICS => RawError::Kind(URT::VLDTR_E_MD_INVALIDSEMANTICS),
            URT::VLDTR_E_MD_SEMANTICSNOTEXIST => RawError::Kind(URT::VLDTR_E_MD_SEMANTICSNOTEXIST),
            URT::VLDTR_E_MI_DECLNOTVIRT => RawError::Kind(URT::VLDTR_E_MI_DECLNOTVIRT),
            URT::VLDTR_E_FMD_GLOBALITEM => RawError::Kind(URT::VLDTR_E_FMD_GLOBALITEM),
            URT::VLDTR_E_MD_MULTSEMANTICFLAGS => RawError::Kind(URT::VLDTR_E_MD_MULTSEMANTICFLAGS),
            URT::VLDTR_E_MD_NOSEMANTICFLAGS => RawError::Kind(URT::VLDTR_E_MD_NOSEMANTICFLAGS),
            URT::VLDTR_E_FD_FLDINIFACE => RawError::Kind(URT::VLDTR_E_FD_FLDINIFACE),
            URT::VLDTR_E_AS_HASHALGID => RawError::Kind(URT::VLDTR_E_AS_HASHALGID),
            URT::VLDTR_E_AS_PROCID => RawError::Kind(URT::VLDTR_E_AS_PROCID),
            URT::VLDTR_E_AR_PROCID => RawError::Kind(URT::VLDTR_E_AR_PROCID),
            URT::VLDTR_E_CN_PARENTRANGE => RawError::Kind(URT::VLDTR_E_CN_PARENTRANGE),
            URT::VLDTR_E_AS_BADFLAGS => RawError::Kind(URT::VLDTR_E_AS_BADFLAGS),
            URT::VLDTR_E_TR_HASTYPEDEF => RawError::Kind(URT::VLDTR_E_TR_HASTYPEDEF),
            URT::VLDTR_E_IFACE_BADIMPL => RawError::Kind(URT::VLDTR_E_IFACE_BADIMPL),
            URT::VLDTR_E_IFACE_BADIFACE => RawError::Kind(URT::VLDTR_E_IFACE_BADIFACE),
            URT::VLDTR_E_TD_SECURNOTMARKED => RawError::Kind(URT::VLDTR_E_TD_SECURNOTMARKED),
            URT::VLDTR_E_TD_MARKEDNOSECUR => RawError::Kind(URT::VLDTR_E_TD_MARKEDNOSECUR),
            URT::VLDTR_E_MD_CCTORHASARGS => RawError::Kind(URT::VLDTR_E_MD_CCTORHASARGS),
            URT::VLDTR_E_CT_BADIMPL => RawError::Kind(URT::VLDTR_E_CT_BADIMPL),
            URT::VLDTR_E_MI_ALIENBODY => RawError::Kind(URT::VLDTR_E_MI_ALIENBODY),
            URT::VLDTR_E_MD_CCTORCALLCONV => RawError::Kind(URT::VLDTR_E_MD_CCTORCALLCONV),
            URT::VLDTR_E_MI_BADCLASS => RawError::Kind(URT::VLDTR_E_MI_BADCLASS),
            URT::VLDTR_E_MI_CLASSISINTF => RawError::Kind(URT::VLDTR_E_MI_CLASSISINTF),
            URT::VLDTR_E_MI_BADDECL => RawError::Kind(URT::VLDTR_E_MI_BADDECL),
            URT::VLDTR_E_MI_BADBODY => RawError::Kind(URT::VLDTR_E_MI_BADBODY),
            URT::VLDTR_E_MI_DUP => RawError::Kind(URT::VLDTR_E_MI_DUP),
            URT::VLDTR_E_FD_BADPARENT => RawError::Kind(URT::VLDTR_E_FD_BADPARENT),
            URT::VLDTR_E_MD_PARAMOUTOFSEQ => RawError::Kind(URT::VLDTR_E_MD_PARAMOUTOFSEQ),
            URT::VLDTR_E_MD_PARASEQTOOBIG => RawError::Kind(URT::VLDTR_E_MD_PARASEQTOOBIG),
            URT::VLDTR_E_MD_PARMMARKEDNOMARSHAL => RawError::Kind(URT::VLDTR_E_MD_PARMMARKEDNOMARSHAL),
            URT::VLDTR_E_MD_PARMMARSHALNOTMARKED => RawError::Kind(URT::VLDTR_E_MD_PARMMARSHALNOTMARKED),
            URT::VLDTR_E_MD_PARMMARKEDNODEFLT => RawError::Kind(URT::VLDTR_E_MD_PARMMARKEDNODEFLT),
            URT::VLDTR_E_MD_PARMDEFLTNOTMARKED => RawError::Kind(URT::VLDTR_E_MD_PARMDEFLTNOTMARKED),
            URT::VLDTR_E_PR_BADSCOPE => RawError::Kind(URT::VLDTR_E_PR_BADSCOPE),
            URT::VLDTR_E_PR_NONAME => RawError::Kind(URT::VLDTR_E_PR_NONAME),
            URT::VLDTR_E_PR_NOSIG => RawError::Kind(URT::VLDTR_E_PR_NOSIG),
            URT::VLDTR_E_PR_DUP => RawError::Kind(URT::VLDTR_E_PR_DUP),
            URT::VLDTR_E_PR_BADCALLINGCONV => RawError::Kind(URT::VLDTR_E_PR_BADCALLINGCONV),
            URT::VLDTR_E_PR_MARKEDNODEFLT => RawError::Kind(URT::VLDTR_E_PR_MARKEDNODEFLT),
            URT::VLDTR_E_PR_DEFLTNOTMARKED => RawError::Kind(URT::VLDTR_E_PR_DEFLTNOTMARKED),
            URT::VLDTR_E_PR_BADSEMANTICS => RawError::Kind(URT::VLDTR_E_PR_BADSEMANTICS),
            URT::VLDTR_E_PR_BADMETHOD => RawError::Kind(URT::VLDTR_E_PR_BADMETHOD),
            URT::VLDTR_E_PR_ALIENMETHOD => RawError::Kind(URT::VLDTR_E_PR_ALIENMETHOD),
            URT::VLDTR_E_CN_BLOBNOTNULL => RawError::Kind(URT::VLDTR_E_CN_BLOBNOTNULL),
            URT::VLDTR_E_CN_BLOBNULL => RawError::Kind(URT::VLDTR_E_CN_BLOBNULL),
            URT::VLDTR_E_EV_BADSCOPE => RawError::Kind(URT::VLDTR_E_EV_BADSCOPE),
            URT::VLDTR_E_EV_NONAME => RawError::Kind(URT::VLDTR_E_EV_NONAME),
            URT::VLDTR_E_EV_DUP => RawError::Kind(URT::VLDTR_E_EV_DUP),
            URT::VLDTR_E_EV_BADEVTYPE => RawError::Kind(URT::VLDTR_E_EV_BADEVTYPE),
            URT::VLDTR_E_EV_EVTYPENOTCLASS => RawError::Kind(URT::VLDTR_E_EV_EVTYPENOTCLASS),
            URT::VLDTR_E_EV_BADSEMANTICS => RawError::Kind(URT::VLDTR_E_EV_BADSEMANTICS),
            URT::VLDTR_E_EV_BADMETHOD => RawError::Kind(URT::VLDTR_E_EV_BADMETHOD),
            URT::VLDTR_E_EV_ALIENMETHOD => RawError::Kind(URT::VLDTR_E_EV_ALIENMETHOD),
            URT::VLDTR_E_EV_NOADDON => RawError::Kind(URT::VLDTR_E_EV_NOADDON),
            URT::VLDTR_E_EV_NOREMOVEON => RawError::Kind(URT::VLDTR_E_EV_NOREMOVEON),
            URT::VLDTR_E_CT_DUPTDNAME => RawError::Kind(URT::VLDTR_E_CT_DUPTDNAME),
            URT::VLDTR_E_MAR_BADOFFSET => RawError::Kind(URT::VLDTR_E_MAR_BADOFFSET),
            URT::VLDTR_E_DS_BADOWNER => RawError::Kind(URT::VLDTR_E_DS_BADOWNER),
            URT::VLDTR_E_DS_BADFLAGS => RawError::Kind(URT::VLDTR_E_DS_BADFLAGS),
            URT::VLDTR_E_DS_NOBLOB => RawError::Kind(URT::VLDTR_E_DS_NOBLOB),
            URT::VLDTR_E_MAR_BADIMPL => RawError::Kind(URT::VLDTR_E_MAR_BADIMPL),
            URT::VLDTR_E_MR_VARARGCALLINGCONV => RawError::Kind(URT::VLDTR_E_MR_VARARGCALLINGCONV),
            URT::VLDTR_E_MD_CTORNOTVOID => RawError::Kind(URT::VLDTR_E_MD_CTORNOTVOID),
            URT::VLDTR_E_EV_FIRENOTVOID => RawError::Kind(URT::VLDTR_E_EV_FIRENOTVOID),
            URT::VLDTR_E_AS_BADLOCALE => RawError::Kind(URT::VLDTR_E_AS_BADLOCALE),
            URT::VLDTR_E_CN_PARENTTYPE => RawError::Kind(URT::VLDTR_E_CN_PARENTTYPE),
            URT::VLDTR_E_SIG_SENTINMETHODDEF => RawError::Kind(URT::VLDTR_E_SIG_SENTINMETHODDEF),
            URT::VLDTR_E_SIG_SENTMUSTVARARG => RawError::Kind(URT::VLDTR_E_SIG_SENTMUSTVARARG),
            URT::VLDTR_E_SIG_MULTSENTINELS => RawError::Kind(URT::VLDTR_E_SIG_MULTSENTINELS),
            URT::VLDTR_E_SIG_LASTSENTINEL => RawError::Kind(URT::VLDTR_E_SIG_LASTSENTINEL),
            URT::VLDTR_E_SIG_MISSARG => RawError::Kind(URT::VLDTR_E_SIG_MISSARG),
            URT::VLDTR_E_SIG_BYREFINFIELD => RawError::Kind(URT::VLDTR_E_SIG_BYREFINFIELD),
            URT::VLDTR_E_MD_SYNCMETHODINVTYPE => RawError::Kind(URT::VLDTR_E_MD_SYNCMETHODINVTYPE),
            URT::VLDTR_E_TD_NAMETOOLONG => RawError::Kind(URT::VLDTR_E_TD_NAMETOOLONG),
            URT::VLDTR_E_AS_PROCDUP => RawError::Kind(URT::VLDTR_E_AS_PROCDUP),
            URT::VLDTR_E_ASOS_DUP => RawError::Kind(URT::VLDTR_E_ASOS_DUP),
            URT::VLDTR_E_MAR_BADFLAGS => RawError::Kind(URT::VLDTR_E_MAR_BADFLAGS),
            URT::VLDTR_E_CT_NOTYPEDEFID => RawError::Kind(URT::VLDTR_E_CT_NOTYPEDEFID),
            URT::VLDTR_E_FILE_BADFLAGS => RawError::Kind(URT::VLDTR_E_FILE_BADFLAGS),
            URT::VLDTR_E_FILE_NULLHASH => RawError::Kind(URT::VLDTR_E_FILE_NULLHASH),
            URT::VLDTR_E_MOD_NONAME => RawError::Kind(URT::VLDTR_E_MOD_NONAME),
            URT::VLDTR_E_MOD_NAMEFULLQLFD => RawError::Kind(URT::VLDTR_E_MOD_NAMEFULLQLFD),
            URT::VLDTR_E_TD_RTSPCLNOTSPCL => RawError::Kind(URT::VLDTR_E_TD_RTSPCLNOTSPCL),
            URT::VLDTR_E_TD_EXTENDSIFACE => RawError::Kind(URT::VLDTR_E_TD_EXTENDSIFACE),
            URT::VLDTR_E_MD_CTORPINVOKE => RawError::Kind(URT::VLDTR_E_MD_CTORPINVOKE),
            URT::VLDTR_E_TD_SYSENUMNOTCLASS => RawError::Kind(URT::VLDTR_E_TD_SYSENUMNOTCLASS),
            URT::VLDTR_E_TD_SYSENUMNOTEXTVTYPE => RawError::Kind(URT::VLDTR_E_TD_SYSENUMNOTEXTVTYPE),
            URT::VLDTR_E_MI_SIGMISMATCH => RawError::Kind(URT::VLDTR_E_MI_SIGMISMATCH),
            URT::VLDTR_E_TD_ENUMHASMETHODS => RawError::Kind(URT::VLDTR_E_TD_ENUMHASMETHODS),
            URT::VLDTR_E_TD_ENUMIMPLIFACE => RawError::Kind(URT::VLDTR_E_TD_ENUMIMPLIFACE),
            URT::VLDTR_E_TD_ENUMHASPROP => RawError::Kind(URT::VLDTR_E_TD_ENUMHASPROP),
            URT::VLDTR_E_TD_ENUMHASEVENT => RawError::Kind(URT::VLDTR_E_TD_ENUMHASEVENT),
            URT::VLDTR_E_TD_BADMETHODLST => RawError::Kind(URT::VLDTR_E_TD_BADMETHODLST),
            URT::VLDTR_E_TD_BADFIELDLST => RawError::Kind(URT::VLDTR_E_TD_BADFIELDLST),
            URT::VLDTR_E_CN_BADTYPE => RawError::Kind(URT::VLDTR_E_CN_BADTYPE),
            URT::VLDTR_E_TD_ENUMNOINSTFLD => RawError::Kind(URT::VLDTR_E_TD_ENUMNOINSTFLD),
            URT::VLDTR_E_TD_ENUMMULINSTFLD => RawError::Kind(URT::VLDTR_E_TD_ENUMMULINSTFLD),
            URT::VLDTR_E_INTERRUPTED => RawError::Kind(URT::VLDTR_E_INTERRUPTED),
            URT::VLDTR_E_NOTINIT => RawError::Kind(URT::VLDTR_E_NOTINIT),
            URT::CORDBG_E_UNRECOVERABLE_ERROR => RawError::Kind(URT::CORDBG_E_UNRECOVERABLE_ERROR),
            URT::CORDBG_E_PROCESS_TERMINATED => RawError::Kind(URT::CORDBG_E_PROCESS_TERMINATED),
            URT::CORDBG_E_PROCESS_NOT_SYNCHRONIZED => RawError::Kind(URT::CORDBG_E_PROCESS_NOT_SYNCHRONIZED),
            URT::CORDBG_E_CLASS_NOT_LOADED => RawError::Kind(URT::CORDBG_E_CLASS_NOT_LOADED),
            URT::CORDBG_E_IL_VAR_NOT_AVAILABLE => RawError::Kind(URT::CORDBG_E_IL_VAR_NOT_AVAILABLE),
            URT::CORDBG_E_BAD_REFERENCE_VALUE => RawError::Kind(URT::CORDBG_E_BAD_REFERENCE_VALUE),
            URT::CORDBG_E_FIELD_NOT_AVAILABLE => RawError::Kind(URT::CORDBG_E_FIELD_NOT_AVAILABLE),
            URT::CORDBG_E_NON_NATIVE_FRAME => RawError::Kind(URT::CORDBG_E_NON_NATIVE_FRAME),
            URT::CORDBG_E_NONCONTINUABLE_EXCEPTION => RawError::Kind(URT::CORDBG_E_NONCONTINUABLE_EXCEPTION),
            URT::CORDBG_E_CODE_NOT_AVAILABLE => RawError::Kind(URT::CORDBG_E_CODE_NOT_AVAILABLE),
            URT::CORDBG_E_FUNCTION_NOT_IL => RawError::Kind(URT::CORDBG_E_FUNCTION_NOT_IL),
            URT::CORDBG_E_CANT_SET_IP_INTO_FINALLY => RawError::Kind(URT::CORDBG_E_CANT_SET_IP_INTO_FINALLY),
            URT::CORDBG_E_CANT_SET_IP_OUT_OF_FINALLY => RawError::Kind(URT::CORDBG_E_CANT_SET_IP_OUT_OF_FINALLY),
            URT::CORDBG_E_CANT_SET_IP_INTO_CATCH => RawError::Kind(URT::CORDBG_E_CANT_SET_IP_INTO_CATCH),
            URT::CORDBG_E_SET_IP_NOT_ALLOWED_ON_NONLEAF_FRAME => RawError::Kind(URT::CORDBG_E_SET_IP_NOT_ALLOWED_ON_NONLEAF_FRAME),
            URT::CORDBG_E_SET_IP_IMPOSSIBLE => RawError::Kind(URT::CORDBG_E_SET_IP_IMPOSSIBLE),
            URT::CORDBG_E_FUNC_EVAL_BAD_START_POINT => RawError::Kind(URT::CORDBG_E_FUNC_EVAL_BAD_START_POINT),
            URT::CORDBG_E_INVALID_OBJECT => RawError::Kind(URT::CORDBG_E_INVALID_OBJECT),
            URT::CORDBG_E_FUNC_EVAL_NOT_COMPLETE => RawError::Kind(URT::CORDBG_E_FUNC_EVAL_NOT_COMPLETE),
            URT::CORDBG_E_INPROC_NOT_IMPL => RawError::Kind(URT::CORDBG_E_INPROC_NOT_IMPL),
            URT::CORDBG_E_STATIC_VAR_NOT_AVAILABLE => RawError::Kind(URT::CORDBG_E_STATIC_VAR_NOT_AVAILABLE),
            URT::CORDBG_E_OBJECT_IS_NOT_COPYABLE_VALUE_CLASS => RawError::Kind(URT::CORDBG_E_OBJECT_IS_NOT_COPYABLE_VALUE_CLASS),
            URT::CORDBG_E_CANT_SETIP_INTO_OR_OUT_OF_FILTER => RawError::Kind(URT::CORDBG_E_CANT_SETIP_INTO_OR_OUT_OF_FILTER),
            URT::CORDBG_E_CANT_CHANGE_JIT_SETTING_FOR_ZAP_MODULE => RawError::Kind(URT::CORDBG_E_CANT_CHANGE_JIT_SETTING_FOR_ZAP_MODULE),
            URT::CORDBG_E_CANT_SET_IP_OUT_OF_FINALLY_ON_WIN64 => RawError::Kind(URT::CORDBG_E_CANT_SET_IP_OUT_OF_FINALLY_ON_WIN64),
            URT::CORDBG_E_CANT_SET_IP_OUT_OF_CATCH_ON_WIN64 => RawError::Kind(URT::CORDBG_E_CANT_SET_IP_OUT_OF_CATCH_ON_WIN64),
            URT::CORDBG_E_REMOTE_CONNECTION_CONN_RESET => RawError::Kind(URT::CORDBG_E_REMOTE_CONNECTION_CONN_RESET),
            URT::CORDBG_E_REMOTE_CONNECTION_KEEP_ALIVE => RawError::Kind(URT::CORDBG_E_REMOTE_CONNECTION_KEEP_ALIVE),
            URT::CORDBG_E_REMOTE_CONNECTION_FATAL_ERROR => RawError::Kind(URT::CORDBG_E_REMOTE_CONNECTION_FATAL_ERROR),
            URT::CORDBG_E_CANT_SET_TO_JMC => RawError::Kind(URT::CORDBG_E_CANT_SET_TO_JMC),
            URT::CORDBG_E_NO_CONTEXT_FOR_INTERNAL_FRAME => RawError::Kind(URT::CORDBG_E_NO_CONTEXT_FOR_INTERNAL_FRAME),
            URT::CORDBG_E_NOT_CHILD_FRAME => RawError::Kind(URT::CORDBG_E_NOT_CHILD_FRAME),
            URT::CORDBG_E_NON_MATCHING_CONTEXT => RawError::Kind(URT::CORDBG_E_NON_MATCHING_CONTEXT),
            URT::CORDBG_E_PAST_END_OF_STACK => RawError::Kind(URT::CORDBG_E_PAST_END_OF_STACK),
            URT::CORDBG_E_BAD_THREAD_STATE => RawError::Kind(URT::CORDBG_E_BAD_THREAD_STATE),
            URT::CORDBG_E_FUNC_EVAL_CANNOT_UPDATE_REGISTER_IN_NONLEAF_FRAME => RawError::Kind(URT::CORDBG_E_FUNC_EVAL_CANNOT_UPDATE_REGISTER_IN_NONLEAF_FRAME),
            URT::CORDBG_E_DEBUGGER_ALREADY_ATTACHED => RawError::Kind(URT::CORDBG_E_DEBUGGER_ALREADY_ATTACHED),
            URT::CORDBG_E_SUPERFLOUS_CONTINUE => RawError::Kind(URT::CORDBG_E_SUPERFLOUS_CONTINUE),
            URT::CORDBG_E_SET_VALUE_NOT_ALLOWED_ON_NONLEAF_FRAME => RawError::Kind(URT::CORDBG_E_SET_VALUE_NOT_ALLOWED_ON_NONLEAF_FRAME),
            URT::CORDBG_E_ENC_EH_MAX_NESTING_LEVEL_CANT_INCREASE => RawError::Kind(URT::CORDBG_E_ENC_EH_MAX_NESTING_LEVEL_CANT_INCREASE),
            URT::CORDBG_E_ENC_MODULE_NOT_ENC_ENABLED => RawError::Kind(URT::CORDBG_E_ENC_MODULE_NOT_ENC_ENABLED),
            URT::CORDBG_E_SET_IP_NOT_ALLOWED_ON_EXCEPTION => RawError::Kind(URT::CORDBG_E_SET_IP_NOT_ALLOWED_ON_EXCEPTION),
            URT::CORDBG_E_VARIABLE_IS_ACTUALLY_LITERAL => RawError::Kind(URT::CORDBG_E_VARIABLE_IS_ACTUALLY_LITERAL),
            URT::CORDBG_E_PROCESS_DETACHED => RawError::Kind(URT::CORDBG_E_PROCESS_DETACHED),
            URT::CORDBG_E_ENC_METHOD_SIG_CHANGED => RawError::Kind(URT::CORDBG_E_ENC_METHOD_SIG_CHANGED),
            URT::CORDBG_E_ENC_METHOD_NO_LOCAL_SIG => RawError::Kind(URT::CORDBG_E_ENC_METHOD_NO_LOCAL_SIG),
            URT::CORDBG_E_ENC_CANT_ADD_FIELD_TO_VALUE_OR_LAYOUT_CLASS => RawError::Kind(URT::CORDBG_E_ENC_CANT_ADD_FIELD_TO_VALUE_OR_LAYOUT_CLASS),
            URT::CORDBG_E_ENC_CANT_CHANGE_FIELD => RawError::Kind(URT::CORDBG_E_ENC_CANT_CHANGE_FIELD),
            URT::CORDBG_E_ENC_CANT_ADD_NON_PRIVATE_MEMBER => RawError::Kind(URT::CORDBG_E_ENC_CANT_ADD_NON_PRIVATE_MEMBER),
            URT::CORDBG_E_FIELD_NOT_STATIC => RawError::Kind(URT::CORDBG_E_FIELD_NOT_STATIC),
            URT::CORDBG_E_FIELD_NOT_INSTANCE => RawError::Kind(URT::CORDBG_E_FIELD_NOT_INSTANCE),
            URT::CORDBG_E_ENC_ZAPPED_WITHOUT_ENC => RawError::Kind(URT::CORDBG_E_ENC_ZAPPED_WITHOUT_ENC),
            URT::CORDBG_E_ENC_BAD_METHOD_INFO => RawError::Kind(URT::CORDBG_E_ENC_BAD_METHOD_INFO),
            URT::CORDBG_E_ENC_JIT_CANT_UPDATE => RawError::Kind(URT::CORDBG_E_ENC_JIT_CANT_UPDATE),
            URT::CORDBG_E_ENC_MISSING_CLASS => RawError::Kind(URT::CORDBG_E_ENC_MISSING_CLASS),
            URT::CORDBG_E_ENC_INTERNAL_ERROR => RawError::Kind(URT::CORDBG_E_ENC_INTERNAL_ERROR),
            URT::CORDBG_E_ENC_HANGING_FIELD => RawError::Kind(URT::CORDBG_E_ENC_HANGING_FIELD),
            URT::CORDBG_E_MODULE_NOT_LOADED => RawError::Kind(URT::CORDBG_E_MODULE_NOT_LOADED),
            URT::CORDBG_E_ENC_CANT_CHANGE_SUPERCLASS => RawError::Kind(URT::CORDBG_E_ENC_CANT_CHANGE_SUPERCLASS),
            URT::CORDBG_E_UNABLE_TO_SET_BREAKPOINT => RawError::Kind(URT::CORDBG_E_UNABLE_TO_SET_BREAKPOINT),
            URT::CORDBG_E_DEBUGGING_NOT_POSSIBLE => RawError::Kind(URT::CORDBG_E_DEBUGGING_NOT_POSSIBLE),
            URT::CORDBG_E_KERNEL_DEBUGGER_ENABLED => RawError::Kind(URT::CORDBG_E_KERNEL_DEBUGGER_ENABLED),
            URT::CORDBG_E_KERNEL_DEBUGGER_PRESENT => RawError::Kind(URT::CORDBG_E_KERNEL_DEBUGGER_PRESENT),
            URT::CORDBG_E_HELPER_THREAD_DEAD => RawError::Kind(URT::CORDBG_E_HELPER_THREAD_DEAD),
            URT::CORDBG_E_INTERFACE_INHERITANCE_CANT_CHANGE => RawError::Kind(URT::CORDBG_E_INTERFACE_INHERITANCE_CANT_CHANGE),
            URT::CORDBG_E_INCOMPATIBLE_PROTOCOL => RawError::Kind(URT::CORDBG_E_INCOMPATIBLE_PROTOCOL),
            URT::CORDBG_E_TOO_MANY_PROCESSES => RawError::Kind(URT::CORDBG_E_TOO_MANY_PROCESSES),
            URT::CORDBG_E_INTEROP_NOT_SUPPORTED => RawError::Kind(URT::CORDBG_E_INTEROP_NOT_SUPPORTED),
            URT::CORDBG_E_NO_REMAP_BREAKPIONT => RawError::Kind(URT::CORDBG_E_NO_REMAP_BREAKPIONT),
            URT::CORDBG_E_OBJECT_NEUTERED => RawError::Kind(URT::CORDBG_E_OBJECT_NEUTERED),
            URT::CORPROF_E_FUNCTION_NOT_COMPILED => RawError::Kind(URT::CORPROF_E_FUNCTION_NOT_COMPILED),
            URT::CORPROF_E_DATAINCOMPLETE => RawError::Kind(URT::CORPROF_E_DATAINCOMPLETE),
            URT::CORPROF_E_NOT_REJITABLE_METHODS => RawError::Kind(URT::CORPROF_E_NOT_REJITABLE_METHODS),
            URT::CORPROF_E_CANNOT_UPDATE_METHOD => RawError::Kind(URT::CORPROF_E_CANNOT_UPDATE_METHOD),
            URT::CORPROF_E_FUNCTION_NOT_IL => RawError::Kind(URT::CORPROF_E_FUNCTION_NOT_IL),
            URT::CORPROF_E_NOT_MANAGED_THREAD => RawError::Kind(URT::CORPROF_E_NOT_MANAGED_THREAD),
            URT::CORPROF_E_CALL_ONLY_FROM_INIT => RawError::Kind(URT::CORPROF_E_CALL_ONLY_FROM_INIT),
            URT::CORPROF_E_INPROC_NOT_ENABLED => RawError::Kind(URT::CORPROF_E_INPROC_NOT_ENABLED),
            URT::CORPROF_E_JITMAPS_NOT_ENABLED => RawError::Kind(URT::CORPROF_E_JITMAPS_NOT_ENABLED),
            URT::CORPROF_E_INPROC_ALREADY_BEGUN => RawError::Kind(URT::CORPROF_E_INPROC_ALREADY_BEGUN),
            URT::CORPROF_E_INPROC_NOT_AVAILABLE => RawError::Kind(URT::CORPROF_E_INPROC_NOT_AVAILABLE),
            URT::CORPROF_E_NOT_YET_AVAILABLE => RawError::Kind(URT::CORPROF_E_NOT_YET_AVAILABLE),
            URT::CORPROF_E_TYPE_IS_PARAMETERIZED => RawError::Kind(URT::CORPROF_E_TYPE_IS_PARAMETERIZED),
            URT::CORPROF_E_FUNCTION_IS_PARAMETERIZED => RawError::Kind(URT::CORPROF_E_FUNCTION_IS_PARAMETERIZED),
            URT::CORPROF_E_STACKSNAPSHOT_INVALID_TGT_THREAD => RawError::Kind(URT::CORPROF_E_STACKSNAPSHOT_INVALID_TGT_THREAD),
            URT::CORPROF_E_STACKSNAPSHOT_UNMANAGED_CTX => RawError::Kind(URT::CORPROF_E_STACKSNAPSHOT_UNMANAGED_CTX),
            URT::CORPROF_E_STACKSNAPSHOT_UNSAFE => RawError::Kind(URT::CORPROF_E_STACKSNAPSHOT_UNSAFE),
            URT::CORPROF_E_STACKSNAPSHOT_ABORTED => RawError::Kind(URT::CORPROF_E_STACKSNAPSHOT_ABORTED),
            URT::CORPROF_E_LITERALS_HAVE_NO_ADDRESS => RawError::Kind(URT::CORPROF_E_LITERALS_HAVE_NO_ADDRESS),
            URT::CORPROF_E_UNSUPPORTED_CALL_SEQUENCE => RawError::Kind(URT::CORPROF_E_UNSUPPORTED_CALL_SEQUENCE),
            URT::CORPROF_E_ASYNCHRONOUS_UNSAFE => RawError::Kind(URT::CORPROF_E_ASYNCHRONOUS_UNSAFE),
            URT::CORPROF_E_CLASSID_IS_ARRAY => RawError::Kind(URT::CORPROF_E_CLASSID_IS_ARRAY),
            URT::CORPROF_E_CLASSID_IS_COMPOSITE => RawError::Kind(URT::CORPROF_E_CLASSID_IS_COMPOSITE),
            URT::CORPROF_E_PROFILER_DETACHING => RawError::Kind(URT::CORPROF_E_PROFILER_DETACHING),
            URT::CORPROF_E_PROFILER_NOT_ATTACHABLE => RawError::Kind(URT::CORPROF_E_PROFILER_NOT_ATTACHABLE),
            URT::CORPROF_E_UNRECOGNIZED_PIPE_MSG_FORMAT => RawError::Kind(URT::CORPROF_E_UNRECOGNIZED_PIPE_MSG_FORMAT),
            URT::CORPROF_E_PROFILER_ALREADY_ACTIVE => RawError::Kind(URT::CORPROF_E_PROFILER_ALREADY_ACTIVE),
            URT::CORPROF_E_PROFILEE_INCOMPATIBLE_WITH_TRIGGER => RawError::Kind(URT::CORPROF_E_PROFILEE_INCOMPATIBLE_WITH_TRIGGER),
            URT::CORPROF_E_IPC_FAILED => RawError::Kind(URT::CORPROF_E_IPC_FAILED),
            URT::CORPROF_E_PROFILEE_PROCESS_NOT_FOUND => RawError::Kind(URT::CORPROF_E_PROFILEE_PROCESS_NOT_FOUND),
            URT::CORPROF_E_CALLBACK3_REQUIRED => RawError::Kind(URT::CORPROF_E_CALLBACK3_REQUIRED),
            URT::CORPROF_E_UNSUPPORTED_FOR_ATTACHING_PROFILER => RawError::Kind(URT::CORPROF_E_UNSUPPORTED_FOR_ATTACHING_PROFILER),
            URT::CORPROF_E_IRREVERSIBLE_INSTRUMENTATION_PRESENT => RawError::Kind(URT::CORPROF_E_IRREVERSIBLE_INSTRUMENTATION_PRESENT),
            URT::CORPROF_E_RUNTIME_UNINITIALIZED => RawError::Kind(URT::CORPROF_E_RUNTIME_UNINITIALIZED),
            URT::CORPROF_E_IMMUTABLE_FLAGS_SET => RawError::Kind(URT::CORPROF_E_IMMUTABLE_FLAGS_SET),
            URT::CORPROF_E_PROFILER_NOT_YET_INITIALIZED => RawError::Kind(URT::CORPROF_E_PROFILER_NOT_YET_INITIALIZED),
            URT::CORPROF_E_INCONSISTENT_WITH_FLAGS => RawError::Kind(URT::CORPROF_E_INCONSISTENT_WITH_FLAGS),
            URT::CORPROF_E_PROFILER_CANCEL_ACTIVATION => RawError::Kind(URT::CORPROF_E_PROFILER_CANCEL_ACTIVATION),
            URT::CORPROF_E_CONCURRENT_GC_NOT_PROFILABLE => RawError::Kind(URT::CORPROF_E_CONCURRENT_GC_NOT_PROFILABLE),
            URT::CORPROF_E_INCONSISTENT_FLAGS_WITH_HOST_PROTECTION_SETTING => RawError::Kind(URT::CORPROF_E_INCONSISTENT_FLAGS_WITH_HOST_PROTECTION_SETTING),
            URT::CORPROF_E_DEBUGGING_DISABLED => RawError::Kind(URT::CORPROF_E_DEBUGGING_DISABLED),
            URT::CORPROF_E_TIMEOUT_WAITING_FOR_CONCURRENT_GC => RawError::Kind(URT::CORPROF_E_TIMEOUT_WAITING_FOR_CONCURRENT_GC),
            URT::CORPROF_E_MODULE_IS_DYNAMIC => RawError::Kind(URT::CORPROF_E_MODULE_IS_DYNAMIC),
            URT::CORPROF_E_CALLBACK4_REQUIRED => RawError::Kind(URT::CORPROF_E_CALLBACK4_REQUIRED),
            URT::CORPROF_E_REJIT_NOT_ENABLED => RawError::Kind(URT::CORPROF_E_REJIT_NOT_ENABLED),
            URT::CORPROF_E_ACTIVE_REJIT_REQUEST_NOT_FOUND => RawError::Kind(URT::CORPROF_E_ACTIVE_REJIT_REQUEST_NOT_FOUND),
            URT::CORPROF_E_FUNCTION_IS_COLLECTIBLE => RawError::Kind(URT::CORPROF_E_FUNCTION_IS_COLLECTIBLE),
            URT::CORPROF_E_REJIT_REQUIRES_DISABLE_NGEN => RawError::Kind(URT::CORPROF_E_REJIT_REQUIRES_DISABLE_NGEN),
            URT::CORPROF_E_CALLBACK6_REQUIRED => RawError::Kind(URT::CORPROF_E_CALLBACK6_REQUIRED),
            URT::CORPROF_E_RUNTIME_SUSPEND_REQUIRED => RawError::Kind(URT::CORPROF_E_RUNTIME_SUSPEND_REQUIRED),
            URT::CORPROF_E_CALLBACK7_REQUIRED => RawError::Kind(URT::CORPROF_E_CALLBACK7_REQUIRED),
            URT::SECURITY_E_XML_TO_ASN_ENCODING => RawError::Kind(URT::SECURITY_E_XML_TO_ASN_ENCODING),
            URT::SECURITY_E_INCOMPATIBLE_SHARE => RawError::Kind(URT::SECURITY_E_INCOMPATIBLE_SHARE),
            URT::SECURITY_E_UNVERIFIABLE => RawError::Kind(URT::SECURITY_E_UNVERIFIABLE),
            URT::SECURITY_E_INCOMPATIBLE_EVIDENCE => RawError::Kind(URT::SECURITY_E_INCOMPATIBLE_EVIDENCE),
            URT::CORSEC_E_DECODE_SET => RawError::Kind(URT::CORSEC_E_DECODE_SET),
            URT::CORSEC_E_ENCODE_SET => RawError::Kind(URT::CORSEC_E_ENCODE_SET),
            URT::CORSEC_E_UNSUPPORTED_FORMAT => RawError::Kind(URT::CORSEC_E_UNSUPPORTED_FORMAT),
            URT::SN_CRYPTOAPI_CALL_FAILED => RawError::Kind(URT::SN_CRYPTOAPI_CALL_FAILED),
            URT::CORSEC_E_CRYPTOAPI_CALL_FAILED => RawError::Kind(URT::CORSEC_E_CRYPTOAPI_CALL_FAILED),
            URT::SN_NO_SUITABLE_CSP => RawError::Kind(URT::SN_NO_SUITABLE_CSP),
            URT::CORSEC_E_NO_SUITABLE_CSP => RawError::Kind(URT::CORSEC_E_NO_SUITABLE_CSP),
            URT::CORSEC_E_INVALID_ATTR => RawError::Kind(URT::CORSEC_E_INVALID_ATTR),
            URT::CORSEC_E_POLICY_EXCEPTION => RawError::Kind(URT::CORSEC_E_POLICY_EXCEPTION),
            URT::CORSEC_E_MIN_GRANT_FAIL => RawError::Kind(URT::CORSEC_E_MIN_GRANT_FAIL),
            URT::CORSEC_E_NO_EXEC_PERM => RawError::Kind(URT::CORSEC_E_NO_EXEC_PERM),
            URT::CORSEC_E_XMLSYNTAX => RawError::Kind(URT::CORSEC_E_XMLSYNTAX),
            URT::CORSEC_E_INVALID_STRONGNAME => RawError::Kind(URT::CORSEC_E_INVALID_STRONGNAME),
            URT::CORSEC_E_MISSING_STRONGNAME => RawError::Kind(URT::CORSEC_E_MISSING_STRONGNAME),
            URT::CORSEC_E_CONTAINER_NOT_FOUND => RawError::Kind(URT::CORSEC_E_CONTAINER_NOT_FOUND),
            URT::CORSEC_E_INVALID_IMAGE_FORMAT => RawError::Kind(URT::CORSEC_E_INVALID_IMAGE_FORMAT),
            URT::CORSEC_E_INVALID_PUBLICKEY => RawError::Kind(URT::CORSEC_E_INVALID_PUBLICKEY),
            URT::CORSEC_E_SIGNATURE_MISMATCH => RawError::Kind(URT::CORSEC_E_SIGNATURE_MISMATCH),
            URT::SN_E_PUBLICKEY_MISMATCH => RawError::Kind(URT::SN_E_PUBLICKEY_MISMATCH),
            URT::CORSEC_E_CRYPTO => RawError::Kind(URT::CORSEC_E_CRYPTO),
            URT::CORSEC_E_CRYPTO_UNEX_OPER => RawError::Kind(URT::CORSEC_E_CRYPTO_UNEX_OPER),
            URT::CORSECATTR_E_BAD_ATTRIBUTE => RawError::Kind(URT::CORSECATTR_E_BAD_ATTRIBUTE),
            URT::CORSECATTR_E_MISSING_CONSTRUCTOR => RawError::Kind(URT::CORSECATTR_E_MISSING_CONSTRUCTOR),
            URT::CORSECATTR_E_FAILED_TO_CREATE_PERM => RawError::Kind(URT::CORSECATTR_E_FAILED_TO_CREATE_PERM),
            URT::CORSECATTR_E_BAD_ACTION_ASM => RawError::Kind(URT::CORSECATTR_E_BAD_ACTION_ASM),
            URT::CORSECATTR_E_BAD_ACTION_OTHER => RawError::Kind(URT::CORSECATTR_E_BAD_ACTION_OTHER),
            URT::CORSECATTR_E_BAD_PARENT => RawError::Kind(URT::CORSECATTR_E_BAD_PARENT),
            URT::CORSECATTR_E_TRUNCATED => RawError::Kind(URT::CORSECATTR_E_TRUNCATED),
            URT::CORSECATTR_E_BAD_VERSION => RawError::Kind(URT::CORSECATTR_E_BAD_VERSION),
            URT::CORSECATTR_E_BAD_ACTION => RawError::Kind(URT::CORSECATTR_E_BAD_ACTION),
            URT::CORSECATTR_E_NO_SELF_REF => RawError::Kind(URT::CORSECATTR_E_NO_SELF_REF),
            URT::CORSECATTR_E_BAD_NONCAS => RawError::Kind(URT::CORSECATTR_E_BAD_NONCAS),
            URT::CORSECATTR_E_ASSEMBLY_LOAD_FAILED => RawError::Kind(URT::CORSECATTR_E_ASSEMBLY_LOAD_FAILED),
            URT::CORSECATTR_E_ASSEMBLY_LOAD_FAILED_EX => RawError::Kind(URT::CORSECATTR_E_ASSEMBLY_LOAD_FAILED_EX),
            URT::CORSECATTR_E_TYPE_LOAD_FAILED => RawError::Kind(URT::CORSECATTR_E_TYPE_LOAD_FAILED),
            URT::CORSECATTR_E_TYPE_LOAD_FAILED_EX => RawError::Kind(URT::CORSECATTR_E_TYPE_LOAD_FAILED_EX),
            URT::CORSECATTR_E_ABSTRACT => RawError::Kind(URT::CORSECATTR_E_ABSTRACT),
            URT::CORSECATTR_E_UNSUPPORTED_TYPE => RawError::Kind(URT::CORSECATTR_E_UNSUPPORTED_TYPE),
            URT::CORSECATTR_E_UNSUPPORTED_ENUM_TYPE => RawError::Kind(URT::CORSECATTR_E_UNSUPPORTED_ENUM_TYPE),
            URT::CORSECATTR_E_NO_FIELD => RawError::Kind(URT::CORSECATTR_E_NO_FIELD),
            URT::CORSECATTR_E_NO_PROPERTY => RawError::Kind(URT::CORSECATTR_E_NO_PROPERTY),
            URT::CORSECATTR_E_EXCEPTION => RawError::Kind(URT::CORSECATTR_E_EXCEPTION),
            URT::CORSECATTR_E_EXCEPTION_HR => RawError::Kind(URT::CORSECATTR_E_EXCEPTION_HR),
            URT::ISS_E_ISOSTORE_START => RawError::Kind(URT::ISS_E_ISOSTORE_START),
            URT::ISS_E_ISOSTORE => RawError::Kind(URT::ISS_E_ISOSTORE),
            URT::ISS_E_OPEN_STORE_FILE => RawError::Kind(URT::ISS_E_OPEN_STORE_FILE),
            URT::ISS_E_OPEN_FILE_MAPPING => RawError::Kind(URT::ISS_E_OPEN_FILE_MAPPING),
            URT::ISS_E_MAP_VIEW_OF_FILE => RawError::Kind(URT::ISS_E_MAP_VIEW_OF_FILE),
            URT::ISS_E_GET_FILE_SIZE => RawError::Kind(URT::ISS_E_GET_FILE_SIZE),
            URT::ISS_E_CREATE_MUTEX => RawError::Kind(URT::ISS_E_CREATE_MUTEX),
            URT::ISS_E_LOCK_FAILED => RawError::Kind(URT::ISS_E_LOCK_FAILED),
            URT::ISS_E_FILE_WRITE => RawError::Kind(URT::ISS_E_FILE_WRITE),
            URT::ISS_E_SET_FILE_POINTER => RawError::Kind(URT::ISS_E_SET_FILE_POINTER),
            URT::ISS_E_CREATE_DIR => RawError::Kind(URT::ISS_E_CREATE_DIR),
            URT::ISS_E_STORE_NOT_OPEN => RawError::Kind(URT::ISS_E_STORE_NOT_OPEN),
            URT::ISS_E_CORRUPTED_STORE_FILE => RawError::Kind(URT::ISS_E_CORRUPTED_STORE_FILE),
            URT::ISS_E_STORE_VERSION => RawError::Kind(URT::ISS_E_STORE_VERSION),
            URT::ISS_E_FILE_NOT_MAPPED => RawError::Kind(URT::ISS_E_FILE_NOT_MAPPED),
            URT::ISS_E_BLOCK_SIZE_TOO_SMALL => RawError::Kind(URT::ISS_E_BLOCK_SIZE_TOO_SMALL),
            URT::ISS_E_ALLOC_TOO_LARGE => RawError::Kind(URT::ISS_E_ALLOC_TOO_LARGE),
            URT::ISS_E_USAGE_WILL_EXCEED_QUOTA => RawError::Kind(URT::ISS_E_USAGE_WILL_EXCEED_QUOTA),
            URT::ISS_E_TABLE_ROW_NOT_FOUND => RawError::Kind(URT::ISS_E_TABLE_ROW_NOT_FOUND),
            URT::ISS_E_DEPRECATE => RawError::Kind(URT::ISS_E_DEPRECATE),
            URT::ISS_E_CALLER => RawError::Kind(URT::ISS_E_CALLER),
            URT::ISS_E_PATH_LENGTH => RawError::Kind(URT::ISS_E_PATH_LENGTH),
            URT::ISS_E_MACHINE => RawError::Kind(URT::ISS_E_MACHINE),
            URT::ISS_E_MACHINE_DACL => RawError::Kind(URT::ISS_E_MACHINE_DACL),
            URT::ISS_E_ISOSTORE_END => RawError::Kind(URT::ISS_E_ISOSTORE_END),
            URT::COR_E_EXCEPTION => RawError::Kind(URT::COR_E_EXCEPTION),
            URT::COR_E_SYSTEM => RawError::Kind(URT::COR_E_SYSTEM),
            URT::COR_E_ARGUMENTOUTOFRANGE => RawError::Kind(URT::COR_E_ARGUMENTOUTOFRANGE),
            URT::COR_E_ARRAYTYPEMISMATCH => RawError::Kind(URT::COR_E_ARRAYTYPEMISMATCH),
            URT::COR_E_CONTEXTMARSHAL => RawError::Kind(URT::COR_E_CONTEXTMARSHAL),
            URT::COR_E_TIMEOUT => RawError::Kind(URT::COR_E_TIMEOUT),
            URT::COR_E_EXECUTIONENGINE => RawError::Kind(URT::COR_E_EXECUTIONENGINE),
            URT::COR_E_FIELDACCESS => RawError::Kind(URT::COR_E_FIELDACCESS),
            URT::COR_E_INDEXOUTOFRANGE => RawError::Kind(URT::COR_E_INDEXOUTOFRANGE),
            URT::COR_E_INVALIDOPERATION => RawError::Kind(URT::COR_E_INVALIDOPERATION),
            URT::COR_E_SECURITY => RawError::Kind(URT::COR_E_SECURITY),
            URT::COR_E_REMOTING => RawError::Kind(URT::COR_E_REMOTING),
            URT::COR_E_SERIALIZATION => RawError::Kind(URT::COR_E_SERIALIZATION),
            URT::COR_E_VERIFICATION => RawError::Kind(URT::COR_E_VERIFICATION),
            URT::COR_E_SERVER => RawError::Kind(URT::COR_E_SERVER),
            URT::COR_E_SERVICEDCOMPONENT => RawError::Kind(URT::COR_E_SERVICEDCOMPONENT),
            URT::COR_E_METHODACCESS => RawError::Kind(URT::COR_E_METHODACCESS),
            URT::COR_E_MISSINGFIELD => RawError::Kind(URT::COR_E_MISSINGFIELD),
            URT::COR_E_MISSINGMEMBER => RawError::Kind(URT::COR_E_MISSINGMEMBER),
            URT::COR_E_MISSINGMETHOD => RawError::Kind(URT::COR_E_MISSINGMETHOD),
            URT::COR_E_MULTICASTNOTSUPPORTED => RawError::Kind(URT::COR_E_MULTICASTNOTSUPPORTED),
            URT::COR_E_NOTSUPPORTED => RawError::Kind(URT::COR_E_NOTSUPPORTED),
            URT::COR_E_OVERFLOW => RawError::Kind(URT::COR_E_OVERFLOW),
            URT::COR_E_RANK => RawError::Kind(URT::COR_E_RANK),
            URT::COR_E_SYNCHRONIZATIONLOCK => RawError::Kind(URT::COR_E_SYNCHRONIZATIONLOCK),
            URT::COR_E_THREADINTERRUPTED => RawError::Kind(URT::COR_E_THREADINTERRUPTED),
            URT::COR_E_MEMBERACCESS => RawError::Kind(URT::COR_E_MEMBERACCESS),
            URT::COR_E_THREADSTATE => RawError::Kind(URT::COR_E_THREADSTATE),
            URT::COR_E_THREADSTOP => RawError::Kind(URT::COR_E_THREADSTOP),
            URT::COR_E_TYPELOAD => RawError::Kind(URT::COR_E_TYPELOAD),
            URT::COR_E_ENTRYPOINTNOTFOUND => RawError::Kind(URT::COR_E_ENTRYPOINTNOTFOUND),
            URT::COR_E_DLLNOTFOUND => RawError::Kind(URT::COR_E_DLLNOTFOUND),
            URT::COR_E_THREADSTART => RawError::Kind(URT::COR_E_THREADSTART),
            URT::COR_E_INVALIDCOMOBJECT => RawError::Kind(URT::COR_E_INVALIDCOMOBJECT),
            URT::COR_E_NOTFINITENUMBER => RawError::Kind(URT::COR_E_NOTFINITENUMBER),
            URT::COR_E_DUPLICATEWAITOBJECT => RawError::Kind(URT::COR_E_DUPLICATEWAITOBJECT),
            URT::COR_E_SEMAPHOREFULL => RawError::Kind(URT::COR_E_SEMAPHOREFULL),
            URT::COR_E_WAITHANDLECANNOTBEOPENED => RawError::Kind(URT::COR_E_WAITHANDLECANNOTBEOPENED),
            URT::COR_E_ABANDONEDMUTEX => RawError::Kind(URT::COR_E_ABANDONEDMUTEX),
            URT::COR_E_THREADABORTED => RawError::Kind(URT::COR_E_THREADABORTED),
            URT::COR_E_INVALIDOLEVARIANTTYPE => RawError::Kind(URT::COR_E_INVALIDOLEVARIANTTYPE),
            URT::COR_E_MISSINGMANIFESTRESOURCE => RawError::Kind(URT::COR_E_MISSINGMANIFESTRESOURCE),
            URT::COR_E_SAFEARRAYTYPEMISMATCH => RawError::Kind(URT::COR_E_SAFEARRAYTYPEMISMATCH),
            URT::COR_E_TYPEINITIALIZATION => RawError::Kind(URT::COR_E_TYPEINITIALIZATION),
            URT::COR_E_MARSHALDIRECTIVE => RawError::Kind(URT::COR_E_MARSHALDIRECTIVE),
            URT::COR_E_MISSINGSATELLITEASSEMBLY => RawError::Kind(URT::COR_E_MISSINGSATELLITEASSEMBLY),
            URT::COR_E_FORMAT => RawError::Kind(URT::COR_E_FORMAT),
            URT::COR_E_SAFEARRAYRANKMISMATCH => RawError::Kind(URT::COR_E_SAFEARRAYRANKMISMATCH),
            URT::COR_E_PLATFORMNOTSUPPORTED => RawError::Kind(URT::COR_E_PLATFORMNOTSUPPORTED),
            URT::COR_E_INVALIDPROGRAM => RawError::Kind(URT::COR_E_INVALIDPROGRAM),
            URT::COR_E_OPERATIONCANCELED => RawError::Kind(URT::COR_E_OPERATIONCANCELED),
            URT::COR_E_INSUFFICIENTMEMORY => RawError::Kind(URT::COR_E_INSUFFICIENTMEMORY),
            URT::COR_E_RUNTIMEWRAPPED => RawError::Kind(URT::COR_E_RUNTIMEWRAPPED),
            URT::COR_E_DEVICESNOTSUPPORTED => RawError::Kind(URT::COR_E_DEVICESNOTSUPPORTED),
            URT::COR_E_DATAMISALIGNED => RawError::Kind(URT::COR_E_DATAMISALIGNED),
            URT::COR_E_CODECONTRACTFAILED => RawError::Kind(URT::COR_E_CODECONTRACTFAILED),
            URT::COR_E_TYPEACCESS => RawError::Kind(URT::COR_E_TYPEACCESS),
            URT::COR_E_KEYNOTFOUND => RawError::Kind(URT::COR_E_KEYNOTFOUND),
            URT::COR_E_INSUFFICIENTEXECUTIONSTACK => RawError::Kind(URT::COR_E_INSUFFICIENTEXECUTIONSTACK),
            URT::COR_E_APPLICATION => RawError::Kind(URT::COR_E_APPLICATION),
            URT::COR_E_INVALIDFILTERCRITERIA => RawError::Kind(URT::COR_E_INVALIDFILTERCRITERIA),
            URT::COR_E_REFLECTIONTYPELOAD => RawError::Kind(URT::COR_E_REFLECTIONTYPELOAD),
            URT::COR_E_TARGET => RawError::Kind(URT::COR_E_TARGET),
            URT::COR_E_TARGETINVOCATION => RawError::Kind(URT::COR_E_TARGETINVOCATION),
            URT::COR_E_CUSTOMATTRIBUTEFORMAT => RawError::Kind(URT::COR_E_CUSTOMATTRIBUTEFORMAT),
            URT::COR_E_IO => RawError::Kind(URT::COR_E_IO),
            URT::COR_E_FILELOAD => RawError::Kind(URT::COR_E_FILELOAD),
            URT::COR_E_OBJECTDISPOSED => RawError::Kind(URT::COR_E_OBJECTDISPOSED),
            URT::COR_E_FAILFAST => RawError::Kind(URT::COR_E_FAILFAST),
            URT::COR_E_HOSTPROTECTION => RawError::Kind(URT::COR_E_HOSTPROTECTION),
            URT::COR_E_ILLEGAL_REENTRANCY => RawError::Kind(URT::COR_E_ILLEGAL_REENTRANCY),
            URT::CLR_E_SHIM_RUNTIMELOAD => RawError::Kind(URT::CLR_E_SHIM_RUNTIMELOAD),
            URT::CLR_E_SHIM_RUNTIMEEXPORT => RawError::Kind(URT::CLR_E_SHIM_RUNTIMEEXPORT),
            URT::CLR_E_SHIM_INSTALLROOT => RawError::Kind(URT::CLR_E_SHIM_INSTALLROOT),
            URT::CLR_E_SHIM_INSTALLCOMP => RawError::Kind(URT::CLR_E_SHIM_INSTALLCOMP),
            URT::CLR_E_SHIM_LEGACYRUNTIMEALREADYBOUND => RawError::Kind(URT::CLR_E_SHIM_LEGACYRUNTIMEALREADYBOUND),
            URT::CLR_E_SHIM_SHUTDOWNINPROGRESS => RawError::Kind(URT::CLR_E_SHIM_SHUTDOWNINPROGRESS),
            URT::VER_E_HRESULT => RawError::Kind(URT::VER_E_HRESULT),
            URT::VER_E_OFFSET => RawError::Kind(URT::VER_E_OFFSET),
            URT::VER_E_OPCODE => RawError::Kind(URT::VER_E_OPCODE),
            URT::VER_E_OPERAND => RawError::Kind(URT::VER_E_OPERAND),
            URT::VER_E_TOKEN => RawError::Kind(URT::VER_E_TOKEN),
            URT::VER_E_EXCEPT => RawError::Kind(URT::VER_E_EXCEPT),
            URT::VER_E_STACK_SLOT => RawError::Kind(URT::VER_E_STACK_SLOT),
            URT::VER_E_LOC => RawError::Kind(URT::VER_E_LOC),
            URT::VER_E_ARG => RawError::Kind(URT::VER_E_ARG),
            URT::VER_E_FOUND => RawError::Kind(URT::VER_E_FOUND),
            URT::VER_E_EXPECTED => RawError::Kind(URT::VER_E_EXPECTED),
            URT::VER_E_LOC_BYNAME => RawError::Kind(URT::VER_E_LOC_BYNAME),
            URT::VER_E_UNKNOWN_OPCODE => RawError::Kind(URT::VER_E_UNKNOWN_OPCODE),
            URT::VER_E_SIG_CALLCONV => RawError::Kind(URT::VER_E_SIG_CALLCONV),
            URT::VER_E_SIG_ELEMTYPE => RawError::Kind(URT::VER_E_SIG_ELEMTYPE),
            URT::VER_E_RET_SIG => RawError::Kind(URT::VER_E_RET_SIG),
            URT::VER_E_FIELD_SIG => RawError::Kind(URT::VER_E_FIELD_SIG),
            URT::VER_E_INTERNAL => RawError::Kind(URT::VER_E_INTERNAL),
            URT::VER_E_STACK_TOO_LARGE => RawError::Kind(URT::VER_E_STACK_TOO_LARGE),
            URT::VER_E_ARRAY_NAME_LONG => RawError::Kind(URT::VER_E_ARRAY_NAME_LONG),
            URT::VER_E_FALLTHRU => RawError::Kind(URT::VER_E_FALLTHRU),
            URT::VER_E_TRY_GTEQ_END => RawError::Kind(URT::VER_E_TRY_GTEQ_END),
            URT::VER_E_TRYEND_GT_CS => RawError::Kind(URT::VER_E_TRYEND_GT_CS),
            URT::VER_E_HND_GTEQ_END => RawError::Kind(URT::VER_E_HND_GTEQ_END),
            URT::VER_E_HNDEND_GT_CS => RawError::Kind(URT::VER_E_HNDEND_GT_CS),
            URT::VER_E_FLT_GTEQ_CS => RawError::Kind(URT::VER_E_FLT_GTEQ_CS),
            URT::VER_E_TRY_START => RawError::Kind(URT::VER_E_TRY_START),
            URT::VER_E_HND_START => RawError::Kind(URT::VER_E_HND_START),
            URT::VER_E_FLT_START => RawError::Kind(URT::VER_E_FLT_START),
            URT::VER_E_TRY_OVERLAP => RawError::Kind(URT::VER_E_TRY_OVERLAP),
            URT::VER_E_TRY_EQ_HND_FIL => RawError::Kind(URT::VER_E_TRY_EQ_HND_FIL),
            URT::VER_E_TRY_SHARE_FIN_FAL => RawError::Kind(URT::VER_E_TRY_SHARE_FIN_FAL),
            URT::VER_E_HND_OVERLAP => RawError::Kind(URT::VER_E_HND_OVERLAP),
            URT::VER_E_HND_EQ => RawError::Kind(URT::VER_E_HND_EQ),
            URT::VER_E_FIL_OVERLAP => RawError::Kind(URT::VER_E_FIL_OVERLAP),
            URT::VER_E_FIL_EQ => RawError::Kind(URT::VER_E_FIL_EQ),
            URT::VER_E_FIL_CONT_TRY => RawError::Kind(URT::VER_E_FIL_CONT_TRY),
            URT::VER_E_FIL_CONT_HND => RawError::Kind(URT::VER_E_FIL_CONT_HND),
            URT::VER_E_FIL_CONT_FIL => RawError::Kind(URT::VER_E_FIL_CONT_FIL),
            URT::VER_E_FIL_GTEQ_CS => RawError::Kind(URT::VER_E_FIL_GTEQ_CS),
            URT::VER_E_FIL_START => RawError::Kind(URT::VER_E_FIL_START),
            URT::VER_E_FALLTHRU_EXCEP => RawError::Kind(URT::VER_E_FALLTHRU_EXCEP),
            URT::VER_E_FALLTHRU_INTO_HND => RawError::Kind(URT::VER_E_FALLTHRU_INTO_HND),
            URT::VER_E_FALLTHRU_INTO_FIL => RawError::Kind(URT::VER_E_FALLTHRU_INTO_FIL),
            URT::VER_E_LEAVE => RawError::Kind(URT::VER_E_LEAVE),
            URT::VER_E_RETHROW => RawError::Kind(URT::VER_E_RETHROW),
            URT::VER_E_ENDFINALLY => RawError::Kind(URT::VER_E_ENDFINALLY),
            URT::VER_E_ENDFILTER => RawError::Kind(URT::VER_E_ENDFILTER),
            URT::VER_E_ENDFILTER_MISSING => RawError::Kind(URT::VER_E_ENDFILTER_MISSING),
            URT::VER_E_BR_INTO_TRY => RawError::Kind(URT::VER_E_BR_INTO_TRY),
            URT::VER_E_BR_INTO_HND => RawError::Kind(URT::VER_E_BR_INTO_HND),
            URT::VER_E_BR_INTO_FIL => RawError::Kind(URT::VER_E_BR_INTO_FIL),
            URT::VER_E_BR_OUTOF_TRY => RawError::Kind(URT::VER_E_BR_OUTOF_TRY),
            URT::VER_E_BR_OUTOF_HND => RawError::Kind(URT::VER_E_BR_OUTOF_HND),
            URT::VER_E_BR_OUTOF_FIL => RawError::Kind(URT::VER_E_BR_OUTOF_FIL),
            URT::VER_E_BR_OUTOF_FIN => RawError::Kind(URT::VER_E_BR_OUTOF_FIN),
            URT::VER_E_RET_FROM_TRY => RawError::Kind(URT::VER_E_RET_FROM_TRY),
            URT::VER_E_RET_FROM_HND => RawError::Kind(URT::VER_E_RET_FROM_HND),
            URT::VER_E_RET_FROM_FIL => RawError::Kind(URT::VER_E_RET_FROM_FIL),
            URT::VER_E_BAD_JMP_TARGET => RawError::Kind(URT::VER_E_BAD_JMP_TARGET),
            URT::VER_E_PATH_LOC => RawError::Kind(URT::VER_E_PATH_LOC),
            URT::VER_E_PATH_THIS => RawError::Kind(URT::VER_E_PATH_THIS),
            URT::VER_E_PATH_STACK => RawError::Kind(URT::VER_E_PATH_STACK),
            URT::VER_E_PATH_STACK_DEPTH => RawError::Kind(URT::VER_E_PATH_STACK_DEPTH),
            URT::VER_E_THIS => RawError::Kind(URT::VER_E_THIS),
            URT::VER_E_THIS_UNINIT_EXCEP => RawError::Kind(URT::VER_E_THIS_UNINIT_EXCEP),
            URT::VER_E_THIS_UNINIT_STORE => RawError::Kind(URT::VER_E_THIS_UNINIT_STORE),
            URT::VER_E_THIS_UNINIT_RET => RawError::Kind(URT::VER_E_THIS_UNINIT_RET),
            URT::VER_E_THIS_UNINIT_V_RET => RawError::Kind(URT::VER_E_THIS_UNINIT_V_RET),
            URT::VER_E_THIS_UNINIT_BR => RawError::Kind(URT::VER_E_THIS_UNINIT_BR),
            URT::VER_E_LDFTN_CTOR => RawError::Kind(URT::VER_E_LDFTN_CTOR),
            URT::VER_E_STACK_NOT_EQ => RawError::Kind(URT::VER_E_STACK_NOT_EQ),
            URT::VER_E_STACK_UNEXPECTED => RawError::Kind(URT::VER_E_STACK_UNEXPECTED),
            URT::VER_E_STACK_EXCEPTION => RawError::Kind(URT::VER_E_STACK_EXCEPTION),
            URT::VER_E_STACK_OVERFLOW => RawError::Kind(URT::VER_E_STACK_OVERFLOW),
            URT::VER_E_STACK_UNDERFLOW => RawError::Kind(URT::VER_E_STACK_UNDERFLOW),
            URT::VER_E_STACK_EMPTY => RawError::Kind(URT::VER_E_STACK_EMPTY),
            URT::VER_E_STACK_UNINIT => RawError::Kind(URT::VER_E_STACK_UNINIT),
            URT::VER_E_STACK_I_I4_I8 => RawError::Kind(URT::VER_E_STACK_I_I4_I8),
            URT::VER_E_STACK_R_R4_R8 => RawError::Kind(URT::VER_E_STACK_R_R4_R8),
            URT::VER_E_STACK_NO_R_I8 => RawError::Kind(URT::VER_E_STACK_NO_R_I8),
            URT::VER_E_STACK_NUMERIC => RawError::Kind(URT::VER_E_STACK_NUMERIC),
            URT::VER_E_STACK_OBJREF => RawError::Kind(URT::VER_E_STACK_OBJREF),
            URT::VER_E_STACK_P_OBJREF => RawError::Kind(URT::VER_E_STACK_P_OBJREF),
            URT::VER_E_STACK_BYREF => RawError::Kind(URT::VER_E_STACK_BYREF),
            URT::VER_E_STACK_METHOD => RawError::Kind(URT::VER_E_STACK_METHOD),
            URT::VER_E_STACK_ARRAY_SD => RawError::Kind(URT::VER_E_STACK_ARRAY_SD),
            URT::VER_E_STACK_VALCLASS => RawError::Kind(URT::VER_E_STACK_VALCLASS),
            URT::VER_E_STACK_P_VALCLASS => RawError::Kind(URT::VER_E_STACK_P_VALCLASS),
            URT::VER_E_STACK_NO_VALCLASS => RawError::Kind(URT::VER_E_STACK_NO_VALCLASS),
            URT::VER_E_LOC_DEAD => RawError::Kind(URT::VER_E_LOC_DEAD),
            URT::VER_E_LOC_NUM => RawError::Kind(URT::VER_E_LOC_NUM),
            URT::VER_E_ARG_NUM => RawError::Kind(URT::VER_E_ARG_NUM),
            URT::VER_E_TOKEN_RESOLVE => RawError::Kind(URT::VER_E_TOKEN_RESOLVE),
            URT::VER_E_TOKEN_TYPE => RawError::Kind(URT::VER_E_TOKEN_TYPE),
            URT::VER_E_TOKEN_TYPE_MEMBER => RawError::Kind(URT::VER_E_TOKEN_TYPE_MEMBER),
            URT::VER_E_TOKEN_TYPE_FIELD => RawError::Kind(URT::VER_E_TOKEN_TYPE_FIELD),
            URT::VER_E_TOKEN_TYPE_SIG => RawError::Kind(URT::VER_E_TOKEN_TYPE_SIG),
            URT::VER_E_UNVERIFIABLE => RawError::Kind(URT::VER_E_UNVERIFIABLE),
            URT::VER_E_LDSTR_OPERAND => RawError::Kind(URT::VER_E_LDSTR_OPERAND),
            URT::VER_E_RET_PTR_TO_STACK => RawError::Kind(URT::VER_E_RET_PTR_TO_STACK),
            URT::VER_E_RET_VOID => RawError::Kind(URT::VER_E_RET_VOID),
            URT::VER_E_RET_MISSING => RawError::Kind(URT::VER_E_RET_MISSING),
            URT::VER_E_RET_EMPTY => RawError::Kind(URT::VER_E_RET_EMPTY),
            URT::VER_E_RET_UNINIT => RawError::Kind(URT::VER_E_RET_UNINIT),
            URT::VER_E_ARRAY_ACCESS => RawError::Kind(URT::VER_E_ARRAY_ACCESS),
            URT::VER_E_ARRAY_V_STORE => RawError::Kind(URT::VER_E_ARRAY_V_STORE),
            URT::VER_E_ARRAY_SD => RawError::Kind(URT::VER_E_ARRAY_SD),
            URT::VER_E_ARRAY_SD_PTR => RawError::Kind(URT::VER_E_ARRAY_SD_PTR),
            URT::VER_E_ARRAY_FIELD => RawError::Kind(URT::VER_E_ARRAY_FIELD),
            URT::VER_E_ARGLIST => RawError::Kind(URT::VER_E_ARGLIST),
            URT::VER_E_VALCLASS => RawError::Kind(URT::VER_E_VALCLASS),
            URT::VER_E_METHOD_ACCESS => RawError::Kind(URT::VER_E_METHOD_ACCESS),
            URT::VER_E_FIELD_ACCESS => RawError::Kind(URT::VER_E_FIELD_ACCESS),
            URT::VER_E_DEAD => RawError::Kind(URT::VER_E_DEAD),
            URT::VER_E_FIELD_STATIC => RawError::Kind(URT::VER_E_FIELD_STATIC),
            URT::VER_E_FIELD_NO_STATIC => RawError::Kind(URT::VER_E_FIELD_NO_STATIC),
            URT::VER_E_ADDR => RawError::Kind(URT::VER_E_ADDR),
            URT::VER_E_ADDR_BYREF => RawError::Kind(URT::VER_E_ADDR_BYREF),
            URT::VER_E_ADDR_LITERAL => RawError::Kind(URT::VER_E_ADDR_LITERAL),
            URT::VER_E_INITONLY => RawError::Kind(URT::VER_E_INITONLY),
            URT::VER_E_THROW => RawError::Kind(URT::VER_E_THROW),
            URT::VER_E_CALLVIRT_VALCLASS => RawError::Kind(URT::VER_E_CALLVIRT_VALCLASS),
            URT::VER_E_CALL_SIG => RawError::Kind(URT::VER_E_CALL_SIG),
            URT::VER_E_CALL_STATIC => RawError::Kind(URT::VER_E_CALL_STATIC),
            URT::VER_E_CTOR => RawError::Kind(URT::VER_E_CTOR),
            URT::VER_E_CTOR_VIRT => RawError::Kind(URT::VER_E_CTOR_VIRT),
            URT::VER_E_CTOR_OR_SUPER => RawError::Kind(URT::VER_E_CTOR_OR_SUPER),
            URT::VER_E_CTOR_MUL_INIT => RawError::Kind(URT::VER_E_CTOR_MUL_INIT),
            URT::VER_E_SIG => RawError::Kind(URT::VER_E_SIG),
            URT::VER_E_SIG_ARRAY => RawError::Kind(URT::VER_E_SIG_ARRAY),
            URT::VER_E_SIG_ARRAY_PTR => RawError::Kind(URT::VER_E_SIG_ARRAY_PTR),
            URT::VER_E_SIG_ARRAY_BYREF => RawError::Kind(URT::VER_E_SIG_ARRAY_BYREF),
            URT::VER_E_SIG_ELEM_PTR => RawError::Kind(URT::VER_E_SIG_ELEM_PTR),
            URT::VER_E_SIG_VARARG => RawError::Kind(URT::VER_E_SIG_VARARG),
            URT::VER_E_SIG_VOID => RawError::Kind(URT::VER_E_SIG_VOID),
            URT::VER_E_SIG_BYREF_BYREF => RawError::Kind(URT::VER_E_SIG_BYREF_BYREF),
            URT::VER_E_CODE_SIZE_ZERO => RawError::Kind(URT::VER_E_CODE_SIZE_ZERO),
            URT::VER_E_BAD_VARARG => RawError::Kind(URT::VER_E_BAD_VARARG),
            URT::VER_E_TAIL_CALL => RawError::Kind(URT::VER_E_TAIL_CALL),
            URT::VER_E_TAIL_BYREF => RawError::Kind(URT::VER_E_TAIL_BYREF),
            URT::VER_E_TAIL_RET => RawError::Kind(URT::VER_E_TAIL_RET),
            URT::VER_E_TAIL_RET_VOID => RawError::Kind(URT::VER_E_TAIL_RET_VOID),
            URT::VER_E_TAIL_RET_TYPE => RawError::Kind(URT::VER_E_TAIL_RET_TYPE),
            URT::VER_E_TAIL_STACK_EMPTY => RawError::Kind(URT::VER_E_TAIL_STACK_EMPTY),
            URT::VER_E_METHOD_END => RawError::Kind(URT::VER_E_METHOD_END),
            URT::VER_E_BAD_BRANCH => RawError::Kind(URT::VER_E_BAD_BRANCH),
            URT::VER_E_FIN_OVERLAP => RawError::Kind(URT::VER_E_FIN_OVERLAP),
            URT::VER_E_LEXICAL_NESTING => RawError::Kind(URT::VER_E_LEXICAL_NESTING),
            URT::VER_E_VOLATILE => RawError::Kind(URT::VER_E_VOLATILE),
            URT::VER_E_UNALIGNED => RawError::Kind(URT::VER_E_UNALIGNED),
            URT::VER_E_INNERMOST_FIRST => RawError::Kind(URT::VER_E_INNERMOST_FIRST),
            URT::VER_E_CALLI_VIRTUAL => RawError::Kind(URT::VER_E_CALLI_VIRTUAL),
            URT::VER_E_CALL_ABSTRACT => RawError::Kind(URT::VER_E_CALL_ABSTRACT),
            URT::VER_E_STACK_UNEXP_ARRAY => RawError::Kind(URT::VER_E_STACK_UNEXP_ARRAY),
            URT::VER_E_NOT_IN_GC_HEAP => RawError::Kind(URT::VER_E_NOT_IN_GC_HEAP),
            URT::VER_E_TRY_N_EMPTY_STACK => RawError::Kind(URT::VER_E_TRY_N_EMPTY_STACK),
            URT::VER_E_DLGT_CTOR => RawError::Kind(URT::VER_E_DLGT_CTOR),
            URT::VER_E_DLGT_BB => RawError::Kind(URT::VER_E_DLGT_BB),
            URT::VER_E_DLGT_PATTERN => RawError::Kind(URT::VER_E_DLGT_PATTERN),
            URT::VER_E_DLGT_LDFTN => RawError::Kind(URT::VER_E_DLGT_LDFTN),
            URT::VER_E_FTN_ABSTRACT => RawError::Kind(URT::VER_E_FTN_ABSTRACT),
            URT::VER_E_SIG_C_VC => RawError::Kind(URT::VER_E_SIG_C_VC),
            URT::VER_E_SIG_VC_C => RawError::Kind(URT::VER_E_SIG_VC_C),
            URT::VER_E_BOX_PTR_TO_STACK => RawError::Kind(URT::VER_E_BOX_PTR_TO_STACK),
            URT::VER_E_SIG_BYREF_TB_AH => RawError::Kind(URT::VER_E_SIG_BYREF_TB_AH),
            URT::VER_E_SIG_ARRAY_TB_AH => RawError::Kind(URT::VER_E_SIG_ARRAY_TB_AH),
            URT::VER_E_ENDFILTER_STACK => RawError::Kind(URT::VER_E_ENDFILTER_STACK),
            URT::VER_E_DLGT_SIG_I => RawError::Kind(URT::VER_E_DLGT_SIG_I),
            URT::VER_E_DLGT_SIG_O => RawError::Kind(URT::VER_E_DLGT_SIG_O),
            URT::VER_E_RA_PTR_TO_STACK => RawError::Kind(URT::VER_E_RA_PTR_TO_STACK),
            URT::VER_E_CATCH_VALUE_TYPE => RawError::Kind(URT::VER_E_CATCH_VALUE_TYPE),
            URT::VER_E_CATCH_BYREF => RawError::Kind(URT::VER_E_CATCH_BYREF),
            URT::VER_E_FIL_PRECEED_HND => RawError::Kind(URT::VER_E_FIL_PRECEED_HND),
            URT::VER_E_LDVIRTFTN_STATIC => RawError::Kind(URT::VER_E_LDVIRTFTN_STATIC),
            URT::VER_E_CALLVIRT_STATIC => RawError::Kind(URT::VER_E_CALLVIRT_STATIC),
            URT::VER_E_INITLOCALS => RawError::Kind(URT::VER_E_INITLOCALS),
            URT::VER_E_BR_TO_EXCEPTION => RawError::Kind(URT::VER_E_BR_TO_EXCEPTION),
            URT::VER_E_CALL_CTOR => RawError::Kind(URT::VER_E_CALL_CTOR),
            URT::VER_E_VALCLASS_OBJREF_VAR => RawError::Kind(URT::VER_E_VALCLASS_OBJREF_VAR),
            URT::VER_E_STACK_P_VALCLASS_OBJREF_VAR => RawError::Kind(URT::VER_E_STACK_P_VALCLASS_OBJREF_VAR),
            URT::VER_E_SIG_VAR_PARAM => RawError::Kind(URT::VER_E_SIG_VAR_PARAM),
            URT::VER_E_SIG_MVAR_PARAM => RawError::Kind(URT::VER_E_SIG_MVAR_PARAM),
            URT::VER_E_SIG_VAR_ARG => RawError::Kind(URT::VER_E_SIG_VAR_ARG),
            URT::VER_E_SIG_MVAR_ARG => RawError::Kind(URT::VER_E_SIG_MVAR_ARG),
            URT::VER_E_SIG_GENERICINST => RawError::Kind(URT::VER_E_SIG_GENERICINST),
            URT::VER_E_SIG_METHOD_INST => RawError::Kind(URT::VER_E_SIG_METHOD_INST),
            URT::VER_E_SIG_METHOD_PARENT_INST => RawError::Kind(URT::VER_E_SIG_METHOD_PARENT_INST),
            URT::VER_E_SIG_FIELD_PARENT_INST => RawError::Kind(URT::VER_E_SIG_FIELD_PARENT_INST),
            URT::VER_E_CALLCONV_NOT_GENERICINST => RawError::Kind(URT::VER_E_CALLCONV_NOT_GENERICINST),
            URT::VER_E_TOKEN_BAD_METHOD_SPEC => RawError::Kind(URT::VER_E_TOKEN_BAD_METHOD_SPEC),
            URT::VER_E_BAD_READONLY_PREFIX => RawError::Kind(URT::VER_E_BAD_READONLY_PREFIX),
            URT::VER_E_BAD_CONSTRAINED_PREFIX => RawError::Kind(URT::VER_E_BAD_CONSTRAINED_PREFIX),
            URT::VER_E_CIRCULAR_VAR_CONSTRAINTS => RawError::Kind(URT::VER_E_CIRCULAR_VAR_CONSTRAINTS),
            URT::VER_E_CIRCULAR_MVAR_CONSTRAINTS => RawError::Kind(URT::VER_E_CIRCULAR_MVAR_CONSTRAINTS),
            URT::VER_E_UNSATISFIED_METHOD_INST => RawError::Kind(URT::VER_E_UNSATISFIED_METHOD_INST),
            URT::VER_E_UNSATISFIED_METHOD_PARENT_INST => RawError::Kind(URT::VER_E_UNSATISFIED_METHOD_PARENT_INST),
            URT::VER_E_UNSATISFIED_FIELD_PARENT_INST => RawError::Kind(URT::VER_E_UNSATISFIED_FIELD_PARENT_INST),
            URT::VER_E_UNSATISFIED_BOX_OPERAND => RawError::Kind(URT::VER_E_UNSATISFIED_BOX_OPERAND),
            URT::VER_E_CONSTRAINED_CALL_WITH_NON_BYREF_THIS => RawError::Kind(URT::VER_E_CONSTRAINED_CALL_WITH_NON_BYREF_THIS),
            URT::VER_E_CONSTRAINED_OF_NON_VARIABLE_TYPE => RawError::Kind(URT::VER_E_CONSTRAINED_OF_NON_VARIABLE_TYPE),
            URT::VER_E_READONLY_UNEXPECTED_CALLEE => RawError::Kind(URT::VER_E_READONLY_UNEXPECTED_CALLEE),
            URT::VER_E_READONLY_ILLEGAL_WRITE => RawError::Kind(URT::VER_E_READONLY_ILLEGAL_WRITE),
            URT::VER_E_READONLY_IN_MKREFANY => RawError::Kind(URT::VER_E_READONLY_IN_MKREFANY),
            URT::VER_E_UNALIGNED_ALIGNMENT => RawError::Kind(URT::VER_E_UNALIGNED_ALIGNMENT),
            URT::VER_E_TAILCALL_INSIDE_EH => RawError::Kind(URT::VER_E_TAILCALL_INSIDE_EH),
            URT::VER_E_BACKWARD_BRANCH => RawError::Kind(URT::VER_E_BACKWARD_BRANCH),
            URT::VER_E_CALL_TO_VTYPE_BASE => RawError::Kind(URT::VER_E_CALL_TO_VTYPE_BASE),
            URT::VER_E_NEWOBJ_OF_ABSTRACT_CLASS => RawError::Kind(URT::VER_E_NEWOBJ_OF_ABSTRACT_CLASS),
            URT::VER_E_UNMANAGED_POINTER => RawError::Kind(URT::VER_E_UNMANAGED_POINTER),
            URT::VER_E_LDFTN_NON_FINAL_VIRTUAL => RawError::Kind(URT::VER_E_LDFTN_NON_FINAL_VIRTUAL),
            URT::VER_E_FIELD_OVERLAP => RawError::Kind(URT::VER_E_FIELD_OVERLAP),
            URT::VER_E_THIS_MISMATCH => RawError::Kind(URT::VER_E_THIS_MISMATCH),
            URT::VER_E_STACK_I_I4 => RawError::Kind(URT::VER_E_STACK_I_I4),
            URT::VER_E_BAD_PE => RawError::Kind(URT::VER_E_BAD_PE),
            URT::VER_E_BAD_MD => RawError::Kind(URT::VER_E_BAD_MD),
            URT::VER_E_BAD_APPDOMAIN => RawError::Kind(URT::VER_E_BAD_APPDOMAIN),
            URT::VER_E_TYPELOAD => RawError::Kind(URT::VER_E_TYPELOAD),
            URT::VER_E_PE_LOAD => RawError::Kind(URT::VER_E_PE_LOAD),
            URT::VER_E_WRITE_RVA_STATIC => RawError::Kind(URT::VER_E_WRITE_RVA_STATIC),
            URT::COR_E_SqlException => RawError::Kind(URT::COR_E_SqlException),
            URT::COR_E_Data => RawError::Kind(URT::COR_E_Data),
            URT::COR_E_DataDeletedRowInaccessible => RawError::Kind(URT::COR_E_DataDeletedRowInaccessible),
            URT::COR_E_DataDuplicateName => RawError::Kind(URT::COR_E_DataDuplicateName),
            URT::COR_E_DataInRowChangingEvent => RawError::Kind(URT::COR_E_DataInRowChangingEvent),
            URT::COR_E_DataInvalidConstraint => RawError::Kind(URT::COR_E_DataInvalidConstraint),
            URT::COR_E_DataMissingPrimaryKey => RawError::Kind(URT::COR_E_DataMissingPrimaryKey),
            URT::COR_E_DataNoNullAllowed => RawError::Kind(URT::COR_E_DataNoNullAllowed),
            URT::COR_E_DataReadOnly => RawError::Kind(URT::COR_E_DataReadOnly),
            URT::COR_E_DataRowNotInTable => RawError::Kind(URT::COR_E_DataRowNotInTable),
            URT::COR_E_DataVersionNotFound => RawError::Kind(URT::COR_E_DataVersionNotFound),
            URT::COR_E_DataConstraint => RawError::Kind(URT::COR_E_DataConstraint),
            URT::COR_E_StrongTyping => RawError::Kind(URT::COR_E_StrongTyping),
            URT::COR_E_SqlType => RawError::Kind(URT::COR_E_SqlType),
            URT::COR_E_SqlNullValue => RawError::Kind(URT::COR_E_SqlNullValue),
            URT::COR_E_SqlTruncate => RawError::Kind(URT::COR_E_SqlTruncate),
            URT::COR_E_AdapterMapping => RawError::Kind(URT::COR_E_AdapterMapping),
            URT::COR_E_DataAdapter => RawError::Kind(URT::COR_E_DataAdapter),
            URT::COR_E_DBConcurrency => RawError::Kind(URT::COR_E_DBConcurrency),
            URT::COR_E_OperationAborted => RawError::Kind(URT::COR_E_OperationAborted),
            URT::COR_E_InvalidUdt => RawError::Kind(URT::COR_E_InvalidUdt),
            URT::COR_E_OdbcException => RawError::Kind(URT::COR_E_OdbcException),
            URT::COR_E_OracleException => RawError::Kind(URT::COR_E_OracleException),
            URT::COR_E_Xml => RawError::Kind(URT::COR_E_Xml),
            URT::COR_E_XmlSchema => RawError::Kind(URT::COR_E_XmlSchema),
            URT::COR_E_XmlXslt => RawError::Kind(URT::COR_E_XmlXslt),
            URT::COR_E_XmlXPath => RawError::Kind(URT::COR_E_XmlXPath),
            URT::COR_E_XmlQuery => RawError::Kind(URT::COR_E_XmlQuery),
            URT::VLDTR_E_IFACE_NOTIFACE => RawError::Kind(URT::VLDTR_E_IFACE_NOTIFACE),
            URT::TLBX_E_CIRCULAR_EXPORT2 => RawError::Kind(URT::TLBX_E_CIRCULAR_EXPORT2),
            URT::VLDTR_E_FD_RVAHASNORVA => RawError::Kind(URT::VLDTR_E_FD_RVAHASNORVA),
            URT::VLDTR_E_FD_RVAHASZERORVA => RawError::Kind(URT::VLDTR_E_FD_RVAHASZERORVA),
            URT::VLDTR_E_MD_RVAANDIMPLMAP => RawError::Kind(URT::VLDTR_E_MD_RVAANDIMPLMAP),
            URT::VLDTR_E_TD_EXTRAFLAGS => RawError::Kind(URT::VLDTR_E_TD_EXTRAFLAGS),
            URT::VLDTR_E_TD_EXTENDSITSELF => RawError::Kind(URT::VLDTR_E_TD_EXTENDSITSELF),
            URT::VLDTR_E_TD_SYSVTNOTEXTOBJ => RawError::Kind(URT::VLDTR_E_TD_SYSVTNOTEXTOBJ),
            URT::VLDTR_E_TD_EXTTYPESPEC => RawError::Kind(URT::VLDTR_E_TD_EXTTYPESPEC),
            URT::VLDTR_E_TD_VTNOSIZE => RawError::Kind(URT::VLDTR_E_TD_VTNOSIZE),
            URT::VLDTR_E_TD_IFACESEALED => RawError::Kind(URT::VLDTR_E_TD_IFACESEALED),
            URT::VLDTR_E_NC_BADNESTED => RawError::Kind(URT::VLDTR_E_NC_BADNESTED),
            URT::VLDTR_E_NC_BADENCLOSER => RawError::Kind(URT::VLDTR_E_NC_BADENCLOSER),
            URT::VLDTR_E_NC_DUP => RawError::Kind(URT::VLDTR_E_NC_DUP),
            URT::VLDTR_E_NC_DUPENCLOSER => RawError::Kind(URT::VLDTR_E_NC_DUPENCLOSER),
            URT::VLDTR_E_FRVA_ZERORVA => RawError::Kind(URT::VLDTR_E_FRVA_ZERORVA),
            URT::VLDTR_E_FRVA_BADFIELD => RawError::Kind(URT::VLDTR_E_FRVA_BADFIELD),
            URT::VLDTR_E_FRVA_DUPRVA => RawError::Kind(URT::VLDTR_E_FRVA_DUPRVA),
            URT::VLDTR_E_FRVA_DUPFIELD => RawError::Kind(URT::VLDTR_E_FRVA_DUPFIELD),
            URT::VLDTR_E_EP_BADTOKEN => RawError::Kind(URT::VLDTR_E_EP_BADTOKEN),
            URT::VLDTR_E_EP_INSTANCE => RawError::Kind(URT::VLDTR_E_EP_INSTANCE),
            URT::VLDTR_E_TD_ENUMFLDBADTYPE => RawError::Kind(URT::VLDTR_E_TD_ENUMFLDBADTYPE),
            URT::VLDTR_E_MD_BADRVA => RawError::Kind(URT::VLDTR_E_MD_BADRVA),
            URT::VLDTR_E_FD_LITERALNODEFAULT => RawError::Kind(URT::VLDTR_E_FD_LITERALNODEFAULT),
            URT::VLDTR_E_IFACE_METHNOTIMPL => RawError::Kind(URT::VLDTR_E_IFACE_METHNOTIMPL),
            URT::VLDTR_E_CA_BADPARENT => RawError::Kind(URT::VLDTR_E_CA_BADPARENT),
            URT::VLDTR_E_CA_BADTYPE => RawError::Kind(URT::VLDTR_E_CA_BADTYPE),
            URT::VLDTR_E_CA_NOTCTOR => RawError::Kind(URT::VLDTR_E_CA_NOTCTOR),
            URT::VLDTR_E_CA_BADSIG => RawError::Kind(URT::VLDTR_E_CA_BADSIG),
            URT::VLDTR_E_CA_NOSIG => RawError::Kind(URT::VLDTR_E_CA_NOSIG),
            URT::VLDTR_E_CA_BADPROLOG => RawError::Kind(URT::VLDTR_E_CA_BADPROLOG),
            URT::VLDTR_E_MD_BADLOCALSIGTOK => RawError::Kind(URT::VLDTR_E_MD_BADLOCALSIGTOK),
            URT::VLDTR_E_MD_BADHEADER => RawError::Kind(URT::VLDTR_E_MD_BADHEADER),
            URT::VLDTR_E_EP_TOOMANYARGS => RawError::Kind(URT::VLDTR_E_EP_TOOMANYARGS),
            URT::VLDTR_E_EP_BADRET => RawError::Kind(URT::VLDTR_E_EP_BADRET),
            URT::VLDTR_E_EP_BADARG => RawError::Kind(URT::VLDTR_E_EP_BADARG),
            URT::VLDTR_E_SIG_BADVOID => RawError::Kind(URT::VLDTR_E_SIG_BADVOID),
            URT::VLDTR_E_IFACE_METHMULTIMPL => RawError::Kind(URT::VLDTR_E_IFACE_METHMULTIMPL),
            URT::VLDTR_E_GP_NAMENULL => RawError::Kind(URT::VLDTR_E_GP_NAMENULL),
            URT::VLDTR_E_GP_OWNERNIL => RawError::Kind(URT::VLDTR_E_GP_OWNERNIL),
            URT::VLDTR_E_GP_DUPNAME => RawError::Kind(URT::VLDTR_E_GP_DUPNAME),
            URT::VLDTR_E_GP_DUPNUMBER => RawError::Kind(URT::VLDTR_E_GP_DUPNUMBER),
            URT::VLDTR_E_GP_NONSEQ_BY_OWNER => RawError::Kind(URT::VLDTR_E_GP_NONSEQ_BY_OWNER),
            URT::VLDTR_E_GP_NONSEQ_BY_NUMBER => RawError::Kind(URT::VLDTR_E_GP_NONSEQ_BY_NUMBER),
            URT::VLDTR_E_GP_UNEXPECTED_OWNER_FOR_VARIANT_VAR => RawError::Kind(URT::VLDTR_E_GP_UNEXPECTED_OWNER_FOR_VARIANT_VAR),
            URT::VLDTR_E_GP_ILLEGAL_VARIANT_MVAR => RawError::Kind(URT::VLDTR_E_GP_ILLEGAL_VARIANT_MVAR),
            URT::VLDTR_E_GP_ILLEGAL_VARIANCE_FLAGS => RawError::Kind(URT::VLDTR_E_GP_ILLEGAL_VARIANCE_FLAGS),
            URT::VLDTR_E_GP_REFANDVALUETYPE => RawError::Kind(URT::VLDTR_E_GP_REFANDVALUETYPE),
            URT::VLDTR_E_GPC_OWNERNIL => RawError::Kind(URT::VLDTR_E_GPC_OWNERNIL),
            URT::VLDTR_E_GPC_DUP => RawError::Kind(URT::VLDTR_E_GPC_DUP),
            URT::VLDTR_E_GPC_NONCONTIGUOUS => RawError::Kind(URT::VLDTR_E_GPC_NONCONTIGUOUS),
            URT::VLDTR_E_MS_METHODNIL => RawError::Kind(URT::VLDTR_E_MS_METHODNIL),
            URT::VLDTR_E_MS_DUP => RawError::Kind(URT::VLDTR_E_MS_DUP),
            URT::VLDTR_E_MS_BADCALLINGCONV => RawError::Kind(URT::VLDTR_E_MS_BADCALLINGCONV),
            URT::VLDTR_E_MS_MISSARITY => RawError::Kind(URT::VLDTR_E_MS_MISSARITY),
            URT::VLDTR_E_MS_MISSARG => RawError::Kind(URT::VLDTR_E_MS_MISSARG),
            URT::VLDTR_E_MS_ARITYMISMATCH => RawError::Kind(URT::VLDTR_E_MS_ARITYMISMATCH),
            URT::VLDTR_E_MS_METHODNOTGENERIC => RawError::Kind(URT::VLDTR_E_MS_METHODNOTGENERIC),
            URT::VLDTR_E_SIG_MISSARITY => RawError::Kind(URT::VLDTR_E_SIG_MISSARITY),
            URT::VLDTR_E_SIG_ARITYMISMATCH => RawError::Kind(URT::VLDTR_E_SIG_ARITYMISMATCH),
            URT::VLDTR_E_MD_GENERIC_CCTOR => RawError::Kind(URT::VLDTR_E_MD_GENERIC_CCTOR),
            URT::VLDTR_E_MD_GENERIC_CTOR => RawError::Kind(URT::VLDTR_E_MD_GENERIC_CTOR),
            URT::VLDTR_E_MD_GENERIC_IMPORT => RawError::Kind(URT::VLDTR_E_MD_GENERIC_IMPORT),
            URT::VLDTR_E_MD_GENERIC_BADCALLCONV => RawError::Kind(URT::VLDTR_E_MD_GENERIC_BADCALLCONV),
            URT::VLDTR_E_MD_GENERIC_GLOBAL => RawError::Kind(URT::VLDTR_E_MD_GENERIC_GLOBAL),
            URT::VLDTR_E_EP_GENERIC_METHOD => RawError::Kind(URT::VLDTR_E_EP_GENERIC_METHOD),
            URT::VLDTR_E_MD_MISSARITY => RawError::Kind(URT::VLDTR_E_MD_MISSARITY),
            URT::VLDTR_E_MD_ARITYZERO => RawError::Kind(URT::VLDTR_E_MD_ARITYZERO),
            URT::VLDTR_E_SIG_ARITYZERO => RawError::Kind(URT::VLDTR_E_SIG_ARITYZERO),
            URT::VLDTR_E_MS_ARITYZERO => RawError::Kind(URT::VLDTR_E_MS_ARITYZERO),
            URT::VLDTR_E_MD_GPMISMATCH => RawError::Kind(URT::VLDTR_E_MD_GPMISMATCH),
            URT::VLDTR_E_EP_GENERIC_TYPE => RawError::Kind(URT::VLDTR_E_EP_GENERIC_TYPE),
            URT::VLDTR_E_MI_DECLNOTGENERIC => RawError::Kind(URT::VLDTR_E_MI_DECLNOTGENERIC),
            URT::VLDTR_E_MI_IMPLNOTGENERIC => RawError::Kind(URT::VLDTR_E_MI_IMPLNOTGENERIC),
            URT::VLDTR_E_MI_ARITYMISMATCH => RawError::Kind(URT::VLDTR_E_MI_ARITYMISMATCH),
            URT::VLDTR_E_TD_EXTBADTYPESPEC => RawError::Kind(URT::VLDTR_E_TD_EXTBADTYPESPEC),
            URT::VLDTR_E_SIG_BYREFINST => RawError::Kind(URT::VLDTR_E_SIG_BYREFINST),
            URT::VLDTR_E_MS_BYREFINST => RawError::Kind(URT::VLDTR_E_MS_BYREFINST),
            URT::VLDTR_E_TS_EMPTY => RawError::Kind(URT::VLDTR_E_TS_EMPTY),
            URT::VLDTR_E_TS_HASSENTINALS => RawError::Kind(URT::VLDTR_E_TS_HASSENTINALS),
            URT::VLDTR_E_TD_GENERICHASEXPLAYOUT => RawError::Kind(URT::VLDTR_E_TD_GENERICHASEXPLAYOUT),
            URT::VLDTR_E_SIG_BADTOKTYPE => RawError::Kind(URT::VLDTR_E_SIG_BADTOKTYPE),
            URT::VLDTR_E_IFACE_METHNOTIMPLTHISMOD => RawError::Kind(URT::VLDTR_E_IFACE_METHNOTIMPLTHISMOD),
            URT::CORDBG_E_THREAD_NOT_SCHEDULED => RawError::Kind(URT::CORDBG_E_THREAD_NOT_SCHEDULED),
            URT::CORDBG_E_HANDLE_HAS_BEEN_DISPOSED => RawError::Kind(URT::CORDBG_E_HANDLE_HAS_BEEN_DISPOSED),
            URT::CORDBG_E_NONINTERCEPTABLE_EXCEPTION => RawError::Kind(URT::CORDBG_E_NONINTERCEPTABLE_EXCEPTION),
            URT::CORDBG_E_CANT_UNWIND_ABOVE_CALLBACK => RawError::Kind(URT::CORDBG_E_CANT_UNWIND_ABOVE_CALLBACK),
            URT::CORDBG_E_INTERCEPT_FRAME_ALREADY_SET => RawError::Kind(URT::CORDBG_E_INTERCEPT_FRAME_ALREADY_SET),
            URT::CORDBG_E_NO_NATIVE_PATCH_AT_ADDR => RawError::Kind(URT::CORDBG_E_NO_NATIVE_PATCH_AT_ADDR),
            URT::CORDBG_E_MUST_BE_INTEROP_DEBUGGING => RawError::Kind(URT::CORDBG_E_MUST_BE_INTEROP_DEBUGGING),
            URT::CORDBG_E_NATIVE_PATCH_ALREADY_AT_ADDR => RawError::Kind(URT::CORDBG_E_NATIVE_PATCH_ALREADY_AT_ADDR),
            URT::CORDBG_E_TIMEOUT => RawError::Kind(URT::CORDBG_E_TIMEOUT),
            URT::CORDBG_E_CANT_CALL_ON_THIS_THREAD => RawError::Kind(URT::CORDBG_E_CANT_CALL_ON_THIS_THREAD),
            URT::CORDBG_E_ENC_INFOLESS_METHOD => RawError::Kind(URT::CORDBG_E_ENC_INFOLESS_METHOD),
            URT::CORDBG_E_ENC_NESTED_HANLDERS => RawError::Kind(URT::CORDBG_E_ENC_NESTED_HANLDERS),
            URT::CORDBG_E_ENC_IN_FUNCLET => RawError::Kind(URT::CORDBG_E_ENC_IN_FUNCLET),
            URT::CORDBG_E_ENC_LOCALLOC => RawError::Kind(URT::CORDBG_E_ENC_LOCALLOC),
            URT::CORDBG_E_ENC_EDIT_NOT_SUPPORTED => RawError::Kind(URT::CORDBG_E_ENC_EDIT_NOT_SUPPORTED),
            URT::CORDBG_E_FEABORT_DELAYED_UNTIL_THREAD_RESUMED => RawError::Kind(URT::CORDBG_E_FEABORT_DELAYED_UNTIL_THREAD_RESUMED),
            URT::CORDBG_E_NOTREADY => RawError::Kind(URT::CORDBG_E_NOTREADY),
            URT::CORDBG_E_CANNOT_RESOLVE_ASSEMBLY => RawError::Kind(URT::CORDBG_E_CANNOT_RESOLVE_ASSEMBLY),
            URT::CORDBG_E_MUST_BE_IN_LOAD_MODULE => RawError::Kind(URT::CORDBG_E_MUST_BE_IN_LOAD_MODULE),
            URT::CORDBG_E_CANNOT_BE_ON_ATTACH => RawError::Kind(URT::CORDBG_E_CANNOT_BE_ON_ATTACH),
            URT::CORDBG_E_NGEN_NOT_SUPPORTED => RawError::Kind(URT::CORDBG_E_NGEN_NOT_SUPPORTED),
            URT::CORDBG_E_ILLEGAL_SHUTDOWN_ORDER => RawError::Kind(URT::CORDBG_E_ILLEGAL_SHUTDOWN_ORDER),
            URT::CORDBG_E_CANNOT_DEBUG_FIBER_PROCESS => RawError::Kind(URT::CORDBG_E_CANNOT_DEBUG_FIBER_PROCESS),
            URT::CORDBG_E_MUST_BE_IN_CREATE_PROCESS => RawError::Kind(URT::CORDBG_E_MUST_BE_IN_CREATE_PROCESS),
            URT::CORDBG_E_DETACH_FAILED_OUTSTANDING_EVALS => RawError::Kind(URT::CORDBG_E_DETACH_FAILED_OUTSTANDING_EVALS),
            URT::CORDBG_E_DETACH_FAILED_OUTSTANDING_STEPPERS => RawError::Kind(URT::CORDBG_E_DETACH_FAILED_OUTSTANDING_STEPPERS),
            URT::CORDBG_E_CANT_INTEROP_STEP_OUT => RawError::Kind(URT::CORDBG_E_CANT_INTEROP_STEP_OUT),
            URT::CORDBG_E_DETACH_FAILED_OUTSTANDING_BREAKPOINTS => RawError::Kind(URT::CORDBG_E_DETACH_FAILED_OUTSTANDING_BREAKPOINTS),
            URT::CORDBG_E_ILLEGAL_IN_STACK_OVERFLOW => RawError::Kind(URT::CORDBG_E_ILLEGAL_IN_STACK_OVERFLOW),
            URT::CORDBG_E_ILLEGAL_AT_GC_UNSAFE_POINT => RawError::Kind(URT::CORDBG_E_ILLEGAL_AT_GC_UNSAFE_POINT),
            URT::CORDBG_E_ILLEGAL_IN_PROLOG => RawError::Kind(URT::CORDBG_E_ILLEGAL_IN_PROLOG),
            URT::CORDBG_E_ILLEGAL_IN_NATIVE_CODE => RawError::Kind(URT::CORDBG_E_ILLEGAL_IN_NATIVE_CODE),
            URT::CORDBG_E_ILLEGAL_IN_OPTIMIZED_CODE => RawError::Kind(URT::CORDBG_E_ILLEGAL_IN_OPTIMIZED_CODE),
            URT::CORDBG_E_MINIDUMP_UNSUPPORTED => RawError::Kind(URT::CORDBG_E_MINIDUMP_UNSUPPORTED),
            URT::CORDBG_E_APPDOMAIN_MISMATCH => RawError::Kind(URT::CORDBG_E_APPDOMAIN_MISMATCH),
            URT::CORDBG_E_CONTEXT_UNVAILABLE => RawError::Kind(URT::CORDBG_E_CONTEXT_UNVAILABLE),
            URT::CORDBG_E_UNCOMPATIBLE_PLATFORMS => RawError::Kind(URT::CORDBG_E_UNCOMPATIBLE_PLATFORMS),
            URT::CORDBG_E_DEBUGGING_DISABLED => RawError::Kind(URT::CORDBG_E_DEBUGGING_DISABLED),
            URT::CORDBG_E_DETACH_FAILED_ON_ENC => RawError::Kind(URT::CORDBG_E_DETACH_FAILED_ON_ENC),
            URT::CORDBG_E_CURRENT_EXCEPTION_IS_OUTSIDE_CURRENT_EXECUTION_SCOPE => RawError::Kind(URT::CORDBG_E_CURRENT_EXCEPTION_IS_OUTSIDE_CURRENT_EXECUTION_SCOPE),
            URT::CORDBG_E_HELPER_MAY_DEADLOCK => RawError::Kind(URT::CORDBG_E_HELPER_MAY_DEADLOCK),
            URT::CORDBG_E_MISSING_METADATA => RawError::Kind(URT::CORDBG_E_MISSING_METADATA),
            URT::CORDBG_E_TARGET_INCONSISTENT => RawError::Kind(URT::CORDBG_E_TARGET_INCONSISTENT),
            URT::CORDBG_E_DETACH_FAILED_OUTSTANDING_TARGET_RESOURCES => RawError::Kind(URT::CORDBG_E_DETACH_FAILED_OUTSTANDING_TARGET_RESOURCES),
            URT::CORDBG_E_TARGET_READONLY => RawError::Kind(URT::CORDBG_E_TARGET_READONLY),
            URT::CORDBG_E_MISMATCHED_CORWKS_AND_DACWKS_DLLS => RawError::Kind(URT::CORDBG_E_MISMATCHED_CORWKS_AND_DACWKS_DLLS),
            URT::CORDBG_E_MODULE_LOADED_FROM_DISK => RawError::Kind(URT::CORDBG_E_MODULE_LOADED_FROM_DISK),
            URT::CORDBG_E_SYMBOLS_NOT_AVAILABLE => RawError::Kind(URT::CORDBG_E_SYMBOLS_NOT_AVAILABLE),
            URT::CORDBG_E_DEBUG_COMPONENT_MISSING => RawError::Kind(URT::CORDBG_E_DEBUG_COMPONENT_MISSING),
            URT::CORDBG_E_REMOTE_MISMATCHED_CERTS => RawError::Kind(URT::CORDBG_E_REMOTE_MISMATCHED_CERTS),
            URT::CORDBG_E_REMOTE_NETWORK_FAILURE => RawError::Kind(URT::CORDBG_E_REMOTE_NETWORK_FAILURE),
            URT::CORDBG_E_REMOTE_NO_LISTENER => RawError::Kind(URT::CORDBG_E_REMOTE_NO_LISTENER),
            URT::CORDBG_E_REMOTE_UNKNOWN_TARGET => RawError::Kind(URT::CORDBG_E_REMOTE_UNKNOWN_TARGET),
            URT::CORDBG_E_REMOTE_INVALID_CONFIG => RawError::Kind(URT::CORDBG_E_REMOTE_INVALID_CONFIG),
            URT::CORDBG_E_REMOTE_MISMATCHED_PROTOCOLS => RawError::Kind(URT::CORDBG_E_REMOTE_MISMATCHED_PROTOCOLS),
            URT::CORDBG_E_LIBRARY_PROVIDER_ERROR => RawError::Kind(URT::CORDBG_E_LIBRARY_PROVIDER_ERROR),
            URT::CORDBG_E_NOT_CLR => RawError::Kind(URT::CORDBG_E_NOT_CLR),
            URT::CORDBG_E_MISSING_DATA_TARGET_INTERFACE => RawError::Kind(URT::CORDBG_E_MISSING_DATA_TARGET_INTERFACE),
            URT::CORDBG_E_UNSUPPORTED_DEBUGGING_MODEL => RawError::Kind(URT::CORDBG_E_UNSUPPORTED_DEBUGGING_MODEL),
            URT::CORDBG_E_UNSUPPORTED_FORWARD_COMPAT => RawError::Kind(URT::CORDBG_E_UNSUPPORTED_FORWARD_COMPAT),
            URT::CORDBG_E_UNSUPPORTED_VERSION_STRUCT => RawError::Kind(URT::CORDBG_E_UNSUPPORTED_VERSION_STRUCT),
            URT::CORDBG_E_READVIRTUAL_FAILURE => RawError::Kind(URT::CORDBG_E_READVIRTUAL_FAILURE),
            URT::CORDBG_E_VALUE_POINTS_TO_FUNCTION => RawError::Kind(URT::CORDBG_E_VALUE_POINTS_TO_FUNCTION),
            URT::CORDBG_E_CORRUPT_OBJECT => RawError::Kind(URT::CORDBG_E_CORRUPT_OBJECT),
            URT::CORDBG_E_GC_STRUCTURES_INVALID => RawError::Kind(URT::CORDBG_E_GC_STRUCTURES_INVALID),
            URT::CORDBG_E_INVALID_OPCODE => RawError::Kind(URT::CORDBG_E_INVALID_OPCODE),
            URT::CORDBG_E_UNSUPPORTED => RawError::Kind(URT::CORDBG_E_UNSUPPORTED),
            URT::CORDBG_E_MISSING_DEBUGGER_EXPORTS => RawError::Kind(URT::CORDBG_E_MISSING_DEBUGGER_EXPORTS),
            URT::PEFMT_E_NO_CONTENTS => RawError::Kind(URT::PEFMT_E_NO_CONTENTS),
            URT::PEFMT_E_NO_NTHEADERS => RawError::Kind(URT::PEFMT_E_NO_NTHEADERS),
            URT::PEFMT_E_64BIT => RawError::Kind(URT::PEFMT_E_64BIT),
            URT::PEFMT_E_NO_CORHEADER => RawError::Kind(URT::PEFMT_E_NO_CORHEADER),
            URT::PEFMT_E_NOT_ILONLY => RawError::Kind(URT::PEFMT_E_NOT_ILONLY),
            URT::PEFMT_E_IMPORT_DLLS => RawError::Kind(URT::PEFMT_E_IMPORT_DLLS),
            URT::PEFMT_E_EXE_NOENTRYPOINT => RawError::Kind(URT::PEFMT_E_EXE_NOENTRYPOINT),
            URT::PEFMT_E_BASE_RELOCS => RawError::Kind(URT::PEFMT_E_BASE_RELOCS),
            URT::PEFMT_E_ENTRYPOINT => RawError::Kind(URT::PEFMT_E_ENTRYPOINT),
            URT::PEFMT_E_ZERO_SIZEOFCODE => RawError::Kind(URT::PEFMT_E_ZERO_SIZEOFCODE),
            URT::PEFMT_E_BAD_CORHEADER => RawError::Kind(URT::PEFMT_E_BAD_CORHEADER),
            URT::PEFMT_E_32BIT => RawError::Kind(URT::PEFMT_E_32BIT),
            URT::CLR_OPTSVC_E_CONTROLLER_INTERRUPT => RawError::Kind(URT::CLR_OPTSVC_E_CONTROLLER_INTERRUPT),
            URT::NGEN_FAILED_GET_DEPENDENCIES => RawError::Kind(URT::NGEN_FAILED_GET_DEPENDENCIES),
            URT::NGEN_FAILED_NATIVE_IMAGE_DELETE => RawError::Kind(URT::NGEN_FAILED_NATIVE_IMAGE_DELETE),
            URT::NGEN_E_TOO_MANY_INTERFACES => RawError::Kind(URT::NGEN_E_TOO_MANY_INTERFACES),
            URT::NGEN_E_OLDER_RUNTIME => RawError::Kind(URT::NGEN_E_OLDER_RUNTIME),
            URT::NGEN_E_WORKER_UNEXPECTED_EXIT => RawError::Kind(URT::NGEN_E_WORKER_UNEXPECTED_EXIT),
            URT::NGEN_E_WORKER_UNEXPECTED_SYNC => RawError::Kind(URT::NGEN_E_WORKER_UNEXPECTED_SYNC),
            URT::NGEN_E_SYS_ASM_NI_MISSING => RawError::Kind(URT::NGEN_E_SYS_ASM_NI_MISSING),
            URT::CLDB_E_INTERNALERROR => RawError::Kind(URT::CLDB_E_INTERNALERROR),
        }
    }

    pub fn description(&self) -> &'static str {
        return match self {
            URT::VLDTR_S_WRN => "Warnings found in the validator.",
            URT::VLDTR_S_ERR => "Errors found in the validator.",
            URT::VLDTR_S_WRNERR => "Warnings and errors found in the validator.",
            URT::CORDBG_S_BAD_START_SEQUENCE_POINT => "Attempt to SetIP not at a sequence point",
            URT::CORDBG_S_BAD_END_SEQUENCE_POINT => "Attempt to SetIP when not going to a sequence point. If both this and CORDBG_E_BAD_START_SEQUENCE_POINT are true, only CORDBG_E_BAD_START_SEQUENCE_POINT will be reported.",
            URT::CORDBG_S_INSUFFICIENT_INFO_FOR_SET_IP => "SetIP is possible, but the debugger doesn't have enough info to fix variable locations, GC refs, or anything else. Use at your own risk.",
            URT::CORDBG_S_FUNC_EVAL_HAS_NO_RESULT => "Some Func evals will lack a return value, such as those whose return type is void.",
            URT::CORDBG_S_VALUE_POINTS_TO_VOID => "The Debugging API doesn't support dereferencing pointers of type void.",
            URT::CORDBG_S_FUNC_EVAL_ABORTED => "The func eval completed, but was aborted.",
            URT::CEE_E_ENTRYPOINT => "Invalid entrypoint information.",
            URT::CEE_E_CVTRES_NOT_FOUND => "cvtres.exe not found.",
            URT::MSEE_E_LOADLIBFAILED => "Failed to delayload a library.",
            URT::MSEE_E_GETPROCFAILED => "Failed to get dll entrypoint.",
            URT::MSEE_E_MULTCOPIESLOADED => "Multiple copies of mscoree.dll have been loaded into the same process.",
            URT::COR_E_TYPEUNLOADED => "Type has been unloaded.",
            URT::COR_E_APPDOMAINUNLOADED => "Attempted to access an unloaded appdomain.",
            URT::COR_E_CANNOTUNLOADAPPDOMAIN => "Error while unloading appdomain.",
            URT::MSEE_E_ASSEMBLYLOADINPROGRESS => "Assembly is still being loaded.",
            URT::MSEE_E_CANNOTCREATEAPPDOMAIN => "Attempt to create appdomain failed.",
            URT::COR_E_ASSEMBLYEXPECTED => "The module was expected to contain an assembly manifest.",
            URT::COR_E_FIXUPSINEXE => "Attempt to load an unverifiable executable with fixups (IAT with more than 2 sections or a TLS section.)",
            URT::COR_E_NO_LOADLIBRARY_ALLOWED => "The private assembly was located outside the appbase directory.",
            URT::COR_E_NEWER_RUNTIME => "A module specified in the manifest was not found.",
            URT::COR_E_CANNOT_SET_POLICY => "Modules which are not in the manifest were streamed in.",
            URT::COR_E_CANNOT_SPECIFY_EVIDENCE => "A strongly-named assembly is required.",
            URT::COR_E_MULTIMODULEASSEMBLIESDIALLOWED => "Strong name signature could not be verified. The assembly may have been tampered with, or it was delay signed but not fully signed with the correct private key.",
            URT::HOST_E_DEADLOCK => "Host detected a deadlock on a blocking operation.",
            URT::HOST_E_INTERRUPTED => "Host interrupted a wait.",
            URT::HOST_E_INVALIDOPERATION => "Invalid operation.",
            URT::HOST_E_CLRNOTAVAILABLE => "CLR has been disabled due to unrecoverable error.",
            URT::HOST_E_TIMEOUT => "A wait has timed out.",
            URT::HOST_E_NOT_OWNER => "The leave operation has been attempted on a synchronization primitive that is not owned by the current thread.",
            URT::HOST_E_ABANDONED => "An event has been abandoned.",
            URT::HOST_E_EXITPROCESS_THREADABORT => "Process exited due to ThreadAbort escalation.",
            URT::HOST_E_EXITPROCESS_ADUNLOAD => "Process exited due to AD Unload escalation.",
            URT::HOST_E_EXITPROCESS_TIMEOUT => "Process exited due to Timeout escalation.",
            URT::HOST_E_EXITPROCESS_OUTOFMEMORY => "LoadFrom(), LoadFile(), Load(byte[]) and LoadModule() have been disabled by the host.",
            URT::HOST_E_EXITPROCESS_STACKOVERFLOW => "Failed to add file to AppDomain cache.",
            URT::COR_E_MODULE_HASH_CHECK_FAILED => "The check of the module's hash failed.",
            URT::FUSION_E_REF_DEF_MISMATCH => "The located assembly's manifest definition does not match the assembly reference.",
            URT::FUSION_E_INVALID_PRIVATE_ASM_LOCATION => "The private assembly was located outside the appbase directory.",
            URT::FUSION_E_ASM_MODULE_MISSING => "A module specified in the manifest was not found.",
            URT::FUSION_E_UNEXPECTED_MODULE_FOUND => "Modules which are not in the manifest were streamed in.",
            URT::FUSION_E_PRIVATE_ASM_DISALLOWED => "A strongly-named assembly is required.",
            URT::FUSION_E_SIGNATURE_CHECK_FAILED => "Strong name signature could not be verified. The assembly may have been tampered with, or it was delay signed but not fully signed with the correct private key.",
            URT::FUSION_E_DATABASE_ERROR => "An unexpected error was encountered in the Assembly Cache database.",
            URT::FUSION_E_INVALID_NAME => "The given assembly name or codebase was invalid.",
            URT::FUSION_E_CODE_DOWNLOAD_DISABLED => "HTTP download of assemblies has been disabled for this appdomain.",
            URT::FUSION_E_UNINSTALL_DISALLOWED => "Uninstall of given assembly is not allowed.",
            URT::FUSION_E_HOST_GAC_ASM_MISMATCH => "Assembly in host store has a different signature than assembly in GAC.",
            URT::FUSION_E_LOADFROM_BLOCKED => "LoadFrom(), LoadFile(), Load(byte[]) and LoadModule() have been disabled by the host.",
            URT::FUSION_E_CACHEFILE_FAILED => "Failed to add file to AppDomain cache.",
            URT::FUSION_E_APP_DOMAIN_LOCKED => "The requested assembly version conflicts with what is already bound in the app domain or specified in the manifest.",
            URT::FUSION_E_CONFIGURATION_ERROR => "The requested assembly name was neither found in the GAC nor in the manifest or the manifest's specified location is wrong.",
            URT::FUSION_E_MANIFEST_PARSE_ERROR => "Unexpected error while parsing the specified manifest.",
            URT::FUSION_E_INVALID_ASSEMBLY_REFERENCE => "The given assembly name is invalid because a processor architecture is specified.",
            URT::COR_E_ASSEMBLY_NOT_EXPECTED => "The module was expected to not contain an assembly manifest.",
            URT::COR_E_LOADING_REFERENCE_ASSEMBLY => "Reference assemblies should not be loaded for execution. They can only be loaded in the Reflection-only loader context.",
            URT::CLDB_E_FILE_BADREAD => "Error occurred during a read.",
            URT::CLDB_E_FILE_BADWRITE => "Error occurred during a write.",
            URT::CLDB_E_FILE_READONLY => "File is read only.",
            URT::CLDB_E_NAME_ERROR => "Ill-formed name.",
            URT::CLDB_E_TRUNCATION => "Data value was truncated.",
            URT::CLDB_E_FILE_OLDVER => "Old version error.",
            URT::CLDB_E_RELOCATED => "A shared memory open failed to open at the originally assigned memory address.",
            URT::CLDB_E_SMDUPLICATE => "Too many records were returned for criteria.",
            URT::CLDB_E_NO_DATA => "Record is a duplicate.",
            URT::CLDB_E_READONLY => "Primary key value is required.",
            URT::CLDB_E_INCOMPATIBLE => "Record is valid but deleted.",
            URT::CLDB_E_FILE_CORRUPT => "Record is emitted out of order.",
            URT::CLDB_E_SCHEMA_VERNOTFOUND => "Version %d of schema '%s' not found.",
            URT::CLDB_E_BADUPDATEMODE => "Cannot open a incrementally build scope for full update.",
            URT::CLDB_E_INDEX_NONULLKEYS => "Null value not allowed in unique index or primary key.",
            URT::CLDB_E_INDEX_DUPLICATE => "Index has been duplicated.",
            URT::CLDB_E_INDEX_BADTYPE => "The columns data type is not allowed in an index.",
            URT::CLDB_E_INDEX_NOTFOUND => "Index not found.",
            URT::CLDB_E_RECORD_NOTFOUND => "Record not found on lookup.",
            URT::CLDB_E_RECORD_OVERFLOW => "Too many records were returned for criteria.",
            URT::CLDB_E_RECORD_DUPLICATE => "Record is a duplicate.",
            URT::CLDB_E_RECORD_PKREQUIRED => "Primary key value is required.",
            URT::CLDB_E_RECORD_DELETED => "Record is valid but deleted.",
            URT::CLDB_E_RECORD_OUTOFORDER => "Record is emitted out of order.",
            URT::CLDB_E_COLUMN_OVERFLOW => "Data too large.",
            URT::CLDB_E_COLUMN_READONLY => "Column cannot be changed.",
            URT::CLDB_E_COLUMN_SPECIALCOL => "Too many RID or primary key columns, 1 is max.",
            URT::CLDB_E_COLUMN_PKNONULLS => "Primary key column may not allow the null value.",
            URT::CLDB_E_TABLE_CANTDROP => "Attempted auto-drop of table while open.",
            URT::CLDB_E_OBJECT_NOTFOUND => "Object not found in the database.",
            URT::CLDB_E_OBJECT_COLNOTFOUND => "Column not found.",
            URT::CLDB_E_VECTOR_BADINDEX => "Invalid index.",
            URT::CLDB_E_TOO_BIG => "A blob or string was too big.",
            URT::META_E_INVALID_TOKEN_TYPE => "TypeLib export: Detected array of SafeHandles.",
            URT::TLBX_E_INVALID_TYPEINFO => "Typelib import: Invalid type, not converted.",
            URT::TLBX_E_INVALID_TYPEINFO_UNNAMED => "Typelib import: Invalid type, not converted - name unknown.",
            URT::TLBX_E_CTX_NESTED => "Typelib export: TLBX_E_CTX_NESTED",
            URT::TLBX_E_ERROR_MESSAGE => "Typelib export: General error. See IError info for more information.",
            URT::TLBX_E_CANT_SAVE => "Typelib export: SaveAllChanges() failed.",
            URT::TLBX_W_LIBNOTREGISTERED => "Typelib export: Type library is not registered.",
            URT::TLBX_E_CANTLOADLIBRARY => "Typelib export: Type library could not be loaded.",
            URT::TLBX_E_BAD_VT_TYPE => "Typelib import: Invalid vartype, not converted.",
            URT::TLBX_E_NO_MSCOREE_TLB => "Typelib export: Could not load mscoree.tlb.",
            URT::TLBX_E_BAD_MSCOREE_TLB => "Typelib export: Could not get a required typeinfo from mscoree.tlb.",
            URT::TLBX_E_TLB_EXCEPTION => "Merge: Method is duplicated but no matching property info.",
            URT::TLBX_E_MULTIPLE_LCIDS => "Bad binary signature.",
            URT::TLBX_E_AMBIGUOUS_RETURN => "Merge: duplicated methods have inconsistent ImplFlags.",
            URT::TLBX_E_DUPLICATE_TYPE_NAME => "Merge: Inconsistency in meta data.",
            URT::TLBX_I_NONSEQUENTIALSTRUCT => "Typelib export: Cannot convert non-sequential structs.",
            URT::TLBX_I_RESOLVEREFFAILED => "Typelib import: The resolve ref call failed.",
            URT::TLBX_E_ASANY => "Typelib export: Encountered AsAny - ignored.",
            URT::TLBX_E_INVALIDLCIDPARAM => "Typelib export: Encountered an [lcid] attribute set to an invalid parameter.",
            URT::TLBX_E_LCIDONDISPONLYITF => "Typelib export: Encountered an [lcid] attribute on a pure dispatch interface.",
            URT::TLBX_E_NONPUBLIC_FIELD => "Typelib export: Non-public field in public struct.",
            URT::TLBX_E_BAD_NAMES => "TypeLib export: the hModule of a loaded class is 0; cannot export it.",
            URT::TLBX_E_GENERICINST_SIGNATURE => "TypeLib export: attempted to export an Assembly imported from a TLB.",
            URT::TLBX_E_GENERICPAR_SIGNATURE => "TypeLib import: attempted to import a TLB exported from an Assembly.",
            URT::META_E_DUPLICATE => "Attempted to define an object that already exists.",
            URT::META_E_GUID_REQUIRED => "A guid was not provided where one was required.",
            URT::META_E_TYPEDEF_MISMATCH => "Merge: an import typedef matched ns.name, but not version and guid.",
            URT::META_E_MERGE_COLLISION => "Merge: conflict between import and emit.",
            URT::TLBX_E_NO_SAFEHANDLE_ARRAYS => "TypeLib export: Detected array of SafeHandles.",
            URT::META_E_METHD_NOT_FOUND => "Merge: Class already in emit scope, but member not found.",
            URT::META_E_FIELD_NOT_FOUND => "Merge: Class already in emit scope, but member not found.",
            URT::META_E_PARAM_MISMATCH => "Merge: Parameter information mismatched.",
            URT::META_E_BADMETADATA => "Typelib export: Types which contain the native type NATIVE_TYPE_LPTSTR are not allowed to be exported to COM.",
            URT::META_E_INTFCEIMPL_NOT_FOUND => "Typelib export: Types with a charset of auto are not allowed to be exported to COM.",
            URT::TLBX_E_NO_CRITICALHANDLE_ARRAYS => "TypeLib export: detected array of CriticalHandles",
            URT::META_E_CLASS_LAYOUT_INCONSISTENT => "Merge: Class is duplicated but class layout information is not consistent",
            URT::META_E_FIELD_MARSHAL_NOT_FOUND => "Typelib export: The enum value is not legal for a typelib.",
            URT::META_E_METHODSEM_NOT_FOUND => "Typelib export: Duplicate IID.",
            URT::META_E_EVENT_NOT_FOUND => "Merge: Method is duplicated but no matching event info.",
            URT::META_E_PROP_NOT_FOUND => "Merge: Method is duplicated but no matching property info.",
            URT::META_E_BAD_SIGNATURE => "Bad binary signature.",
            URT::META_E_BAD_INPUT_PARAMETER => "Bad input parameters.",
            URT::META_E_METHDIMPL_INCONSISTENT => "Merge: duplicated methods have inconsistent ImplFlags.",
            URT::META_E_MD_INCONSISTENCY => "Merge: Inconsistency in meta data.",
            URT::META_E_CANNOTRESOLVETYPEREF => "Cannot resolve typeref.",
            URT::META_E_STRINGSPACE_FULL => "No logical space left to create more user strings.",
            URT::META_E_UNEXPECTED_REMAP => "Unexpected TokenRemap.",
            URT::META_E_HAS_UNMARKALL => "Known custom attribute had invalid value.",
            URT::META_E_MUST_CALL_UNMARKALL => "Known custom attribute blob has bad format.",
            URT::META_E_GENERICPARAM_INCONSISTENT => "Known custom attribute blob has repeated named argument.",
            URT::META_E_EVENT_COUNTS => "Known custom attribute named argument not recognized.",
            URT::META_E_PROPERTY_COUNTS => "Known attribute named argument does not support variant.",
            URT::META_E_TYPEDEF_MISSING => "Known attribute named argument does not support array.",
            URT::TLBX_E_CANT_LOAD_MODULE => "Failure decoding permission set.",
            URT::TLBX_E_CANT_LOAD_CLASS => "Failure encoding permission set.",
            URT::TLBX_E_NULL_MODULE => "Unrecognized encoding format.",
            URT::TLBX_E_NO_CLSID_KEY => "StrongName APIs not supported on system.",
            URT::TLBX_E_CIRCULAR_EXPORT => "StrongName APIs could not locate a matching CSP.",
            URT::TLBX_E_CIRCULAR_IMPORT => "Invalid security custom attribute.",
            URT::TLBX_E_BAD_NATIVETYPE => "PolicyException thrown.",
            URT::TLBX_E_BAD_VTABLE => "Failed to grant minimum permission requests.",
            URT::TLBX_E_CRM_NON_STATIC => "Failed to grant permission to execute.",
            URT::TLBX_E_CRM_INVALID_SIG => "XML Syntax error.",
            URT::TLBX_E_CLASS_LOAD_EXCEPTION => "Bad custom attribute serialized blob version.",
            URT::TLBX_E_UNKNOWN_SIGNATURE => "Invalid security action code.",
            URT::TLBX_E_REFERENCED_TYPELIB => "CA reference to CA definition in same assembly.",
            URT::TLBX_E_INVALID_NAMESPACE => "Use of non-CAS permission with invalid action.",
            URT::TLBX_E_LAYOUT_ERROR => "Failed to load assembly containing CA (or required CA type).",
            URT::TLBX_E_NOTIUNKNOWN => "Failed to load assembly containing CA (or required CA type).",
            URT::TLBX_E_NONVISIBLEVALUECLASS => "Signature size mismatch.",
            URT::TLBX_E_LPTSTR_NOT_ALLOWED => "Public key of assembly did not match signing public key.",
            URT::TLBX_E_AUTO_CS_NOT_ALLOWED => "Typelib export: Types with a char set of auto are not allowed to be exported to COM.",
            URT::TLBX_S_NOSTDINTERFACE => "Typelib export: Found an interface marked as IID_IDispatch or IID_IUnknown.",
            URT::TLBX_S_DUPLICATE_DISPID => "ypelib export: duplicate dispid found; ignored.",
            URT::TLBX_E_ENUM_VALUE_INVALID => "Typelib export: The enum value is not legal for a typelib.",
            URT::TLBX_E_DUPLICATE_IID => "Typelib export: Duplicate IID",
            URT::TLBX_E_NO_NESTED_ARRAYS => "Tyeplib export: detected nested arrays.",
            URT::TLBX_E_PARAM_ERROR_NAMED => "Typelib import: param type couldn't be converted.",
            URT::TLBX_E_PARAM_ERROR_UNNAMED => "Typelib import: param type couldn't be converted -- param name unknown.",
            URT::TLBX_E_AGNOST_SIGNATURE => "TypeLib export: size agnostic element in signature.",
            URT::TLBX_E_CONVERT_FAIL => "TypeLib export: exporter failed.",
            URT::TLBX_W_DUAL_NOT_DISPATCH => "Typelib import: [dual] interface not derived from IDispatch.",
            URT::TLBX_E_BAD_SIGNATURE => "Typelib export: unconvertable signature (use specific error for reporting!)",
            URT::TLBX_E_ARRAY_NEEDS_NT_FIXED => "Typelib export: non-fixed/non-safearray array in struct",
            URT::TLBX_E_CLASS_NEEDS_NT_INTF => "Typelib export: non-interface class in struct",
            URT::META_E_CA_INVALID_TARGET => "Failure during Cryptographic operation.",
            URT::META_E_CA_INVALID_VALUE => "Unexpected Cryptographic operation.",
            URT::META_E_CA_INVALID_BLOB => "Known custom attribute blob is bad format.",
            URT::META_E_CA_REPEATED_ARG => "Known custom attribute blob has repeated named argument.",
            URT::META_E_CA_UNKNOWN_ARGUMENT => "Known custom attrubte named arg not recognized.",
            URT::META_E_CA_VARIANT_NYI => "Known attribute named argument doesn't support variant.",
            URT::META_E_CA_ARRAY_NYI => "Known attribute named argument doesn't support array.",
            URT::META_E_CA_UNEXPECTED_TYPE => "Known attribute parser found unexpected type.",
            URT::META_E_CA_INVALID_ARGTYPE => "Known attribute parser only handles fields -- no properties.",
            URT::META_E_CA_INVALID_ARG_FOR_TYPE => "Known attribute parser found an argument that is invalid for the object it is applied to.",
            URT::META_E_CA_INVALID_UUID => "Unable to create store file mapping.",
            URT::META_E_CA_INVALID_MARSHALAS_FIELDS => "Unable to map the store file.",
            URT::META_E_CA_NT_FIELDONLY => "Unable to determine store file size.",
            URT::META_E_CA_NEGATIVE_PARAMINDEX => "Unable to create mutex.",
            URT::META_E_CA_NEGATIVE_MULTIPLIER => "Unable to lock the store.",
            URT::META_E_CA_NEGATIVE_CONSTSIZE => "File Write failed.",
            URT::META_E_CA_FIXEDSTR_SIZE_REQUIRED => "Bad custom attribute serialized blob.",
            URT::META_E_CA_CUSTMARSH_TYPE_REQUIRED => "Bad custom attribute serialized blob version.",
            URT::META_E_CA_FILENAME_REQUIRED => "Invalid security action code.",
            URT::TLBX_W_NO_PROPS_IN_EVENTS => "CA reference to CA definition in same assembly.",
            URT::META_E_NOT_IN_ENC_MODE => "Use of non-CAS permission with invalid action.",
            URT::META_E_METHOD_COUNTS => "Failed to load assembly containing CA (or required CA type).",
            URT::META_E_FIELD_COUNTS => "Failed to load CA type (or required CA type).",
            URT::META_E_PARAM_COUNTS => "Failed to load CA type (or required CA type).",
            URT::TLBX_E_TYPED_REF => "TypeLib export: Exporting a TypedReference.",
            URT::TLBX_W_DEFAULT_INTF_NOT_VISIBLE => "TypeLib export: ComDefaultInterface is not COMVisible.",
            URT::TLBX_W_BAD_SAFEARRAYFIELD_NO_ELEMENTVT => "TypeLib export: System.Array SAFEARRAY field without a SafeArraySubType.",
            URT::TLBX_W_LAYOUTCLASS_AS_INTERFACE => "TypeLib export: Class with layout parameter of field marked with UnmanagedType.Interface",
            URT::TLBX_I_GENERIC_BASE_TYPE => "TypeLib export: type deriving from a generic type.",
            URT::TLBX_E_BITNESS_MISMATCH => "TypeLib export: bitness of assembly doesn't match bitness of output type library",
            URT::TLBX_E_EVENT_WITH_NEWENUM => "TypeLib import: source interface with NewEnum member.",
            URT::TLBX_E_PROPGET_WITHOUT_RETURN => "TypeLib import: propget without return type",
            URT::META_E_MISMATCHED_VISIBLITY => "Merge - Match found for type/method/etc but differs in visiblity",
            URT::META_E_CA_BAD_FRIENDS_ARGS => "InternalsVisibileTo can't have a version, culture, or processor architecture",
            URT::META_E_CA_FRIENDS_SN_REQUIRED => "Strong-name signed assemblies can only grant friend access to strong name-signed assemblies",
            URT::VLDTR_E_RID_OUTOFRANGE => "Rid is out of range.",
            URT::VLDTR_E_CDTKN_OUTOFRANGE => "Coded token type is out of range.",
            URT::VLDTR_E_CDRID_OUTOFRANGE => "Coded rid is out of range.",
            URT::VLDTR_E_STRING_INVALID => "String offset is invalid.",
            URT::VLDTR_E_GUID_INVALID => "GUID offset is invalid.",
            URT::VLDTR_E_BLOB_INVALID => "Blob offset if invalid.",
            URT::VLDTR_E_MOD_MULTI => "Multiple module records found.",
            URT::VLDTR_E_MOD_NULLMVID => "Module has null MVID.",
            URT::VLDTR_E_TR_NAMENULL => "TypeRef name is NULL.",
            URT::VLDTR_E_TR_DUP => "TypeRef has a dup.",
            URT::VLDTR_E_TD_NAMENULL => "TypeDef name is NULL.",
            URT::VLDTR_E_TD_DUPNAME => "TypeDef has a dup based on name+namespace.",
            URT::VLDTR_E_TD_DUPGUID => "TypeDef has a dup based on GUID.",
            URT::VLDTR_E_TD_NOTIFACEOBJEXTNULL => "TypeDef that's not an Interface and not System.Object extends nil parent.",
            URT::VLDTR_E_TD_OBJEXTENDSNONNULL => "System.Object extends a non-nil parent.",
            URT::VLDTR_E_TD_EXTENDSSEALED => "TypeDef extends sealed class.",
            URT::VLDTR_E_TD_DLTNORTSPCL => "TypeDef is Deleted but not marked with RTSpecialName.",
            URT::VLDTR_E_TD_RTSPCLNOTDLT => "TypeDef is marked RTSpecialName, but is not a Deleted record.",
            URT::VLDTR_E_MI_DECLPRIV => "MethodImpl's Decl is private",
            URT::VLDTR_E_AS_BADNAME => "Assembly [Ref] name has path and/or extension.",
            URT::VLDTR_E_FILE_SYSNAME => "File has a system name (con, com, aux, etc.).",
            URT::VLDTR_E_MI_BODYSTATIC => "MethodImpl's body is static.",
            URT::VLDTR_E_TD_IFACENOTABS => "TypeDef is marked Interface but not Abstract.",
            URT::VLDTR_E_TD_IFACEPARNOTNIL => "TypeDef is marked Interface but parent is not Nil.",
            URT::VLDTR_E_TD_IFACEGUIDNULL => "TypeDef is marked Interface but GUID is NULL.",
            URT::VLDTR_E_MI_DECLFINAL => "TMethodImpl's Decl is final.",
            URT::VLDTR_E_TD_VTNOTSEAL => "TypeDef is marked ValueType but not marked Sealed.",
            URT::VLDTR_E_PD_BADFLAGS => "Param has extra bits in flags.",
            URT::VLDTR_E_IFACE_DUP => "InterfaceImpl has a dup.",
            URT::VLDTR_E_MR_NAMENULL => "MemberRef name is NULL.",
            URT::VLDTR_E_MR_VTBLNAME => "MemberRef has an invalid name, _VtblGap*.",
            URT::VLDTR_E_MR_DELNAME => "MemberRef has an invalid name, _Deleted*.",
            URT::VLDTR_E_MR_PARNIL => "MemberRef parent Nil in a PE file.",
            URT::VLDTR_E_MR_BADCALLINGCONV => "MemberRef has invalid calling convention.",
            URT::VLDTR_E_MR_NOTVARARG => "MemberRef has Method parent but calling convention is not VARARG.",
            URT::VLDTR_E_MR_NAMEDIFF => "MemberRef name different from parent MethodDef.",
            URT::VLDTR_E_MR_SIGDIFF => "MemberRef signature different from parent MethodDef.",
            URT::VLDTR_E_MR_DUP => "MemberRef has a dup.",
            URT::VLDTR_E_CL_TDAUTO => "ClassLayout parent TypeDef is marked AutoLayout.",
            URT::VLDTR_E_CL_BADPCKSZ => "ClassLayout has bad PackingSize.",
            URT::VLDTR_E_CL_DUP => "ClassLayout has dup.",
            URT::VLDTR_E_FL_BADOFFSET => "FieldLayout2 has bad offset.",
            URT::VLDTR_E_FL_TDNIL => "FieldLayout2 has field with nil parent.",
            URT::VLDTR_E_FL_NOCL => "FieldLayout2 has no ClassLayout record.",
            URT::VLDTR_E_FL_TDNOTEXPLCT => "FieldLayout2 parent TypeDef is not marked with ExplicitLayout.",
            URT::VLDTR_E_FL_FLDSTATIC => "FieldLayout2 has field marked Static.",
            URT::VLDTR_E_FL_DUP => "FieldLayout2 has a dup.",
            URT::VLDTR_E_MODREF_NAMENULL => "ModuleRef name is NULL.",
            URT::VLDTR_E_MODREF_DUP => "ModuleRef has a dup.",
            URT::VLDTR_E_TR_BADSCOPE => "TypeRef has a bad resolution scope.",
            URT::VLDTR_E_TD_NESTEDNOENCL => "TypeDef marked nested has no encloser.",
            URT::VLDTR_E_TD_EXTTRRES => "TypeDef extends a TypeRef which resolves to a TypeDef in the same module.",
            URT::VLDTR_E_SIGNULL => "Signature specified is zero-sized.",
            URT::VLDTR_E_SIGNODATA => "Signature does not have enough data at specified byte.",
            URT::VLDTR_E_MD_BADCALLINGCONV => "Method signature has invalid calling convention.",
            URT::VLDTR_E_MD_THISSTATIC => "Method is marked static but has HASTHIS/EXPLICITTHIS set on the calling convention.",
            URT::VLDTR_E_MD_NOTTHISNOTSTATIC => "Method is not marked static but is not HASTHIS/EXPLICITTHIS.",
            URT::VLDTR_E_MD_NOARGCNT => "Method signature is missing the argument count.",
            URT::VLDTR_E_SIG_MISSELTYPE => "Signature missing element type.",
            URT::VLDTR_E_SIG_MISSTKN => "Signature missing token.",
            URT::VLDTR_E_SIG_TKNBAD => "Signature has bad token.",
            URT::VLDTR_E_SIG_MISSFPTR => "Signature is missing function pointer.",
            URT::VLDTR_E_SIG_MISSFPTRARGCNT => "Signature has function pointer missing argument count.",
            URT::VLDTR_E_SIG_MISSRANK => "Signature is missing rank specification.",
            URT::VLDTR_E_SIG_MISSNSIZE => "Signature is missing count of sized dimensions.",
            URT::VLDTR_E_SIG_MISSSIZE => "Signature is missing size of dimension.",
            URT::VLDTR_E_SIG_MISSNLBND => "Signature is missing count of lower bounds.",
            URT::VLDTR_E_SIG_MISSLBND => "Signature is missing a lower bound.",
            URT::VLDTR_E_SIG_BADELTYPE => "Signature has bad element type.",
            URT::VLDTR_E_SIG_MISSVASIZE => "Signature has value array missing size.",
            URT::VLDTR_E_FD_BADCALLINGCONV => "Field signature has invalid calling convention.",
            URT::VLDTR_E_MD_NAMENULL => "Method name is NULL.",
            URT::VLDTR_E_MD_PARNIL => "Method has parent NIL.",
            URT::VLDTR_E_MD_DUP => "Method has dup.",
            URT::VLDTR_E_FD_NAMENULL => "Field name is NULL.",
            URT::VLDTR_E_FD_PARNIL => "Field parent is Nil.",
            URT::VLDTR_E_FD_DUP => "Field has dup.",
            URT::VLDTR_E_AS_MULTI => "Multiple Assembly records found.",
            URT::VLDTR_E_AS_NAMENULL => "Assembly name is NULL.",
            URT::VLDTR_E_SIG_TOKTYPEMISMATCH => "E_T_VALUETYPE<class token> or E_T_CLASS<vtype token>.",
            URT::VLDTR_E_CL_TDINTF => "Class layout on an Interface.",
            URT::VLDTR_E_ASOS_OSPLTFRMIDINVAL => "AssemblyOS platform ID invalid.",
            URT::VLDTR_E_AR_NAMENULL => "AssemblyRef name is NULL.",
            URT::VLDTR_E_TD_ENCLNOTNESTED => "TypeDef not nested has encloser.",
            URT::VLDTR_E_AROS_OSPLTFRMIDINVAL => "AssemblyRefOS has invalid platform ID.",
            URT::VLDTR_E_FILE_NAMENULL => "File name is NULL.",
            URT::VLDTR_E_CT_NAMENULL => "ComType name is NULL.",
            URT::VLDTR_E_TD_EXTENDSCHILD => "TypeDef extends its own child.",
            URT::VLDTR_E_MAR_NAMENULL => "ManifestResource name is NULL.",
            URT::VLDTR_E_FILE_DUP => "File has dup.",
            URT::VLDTR_E_FILE_NAMEFULLQLFD => "File name is fully qualified.",
            URT::VLDTR_E_CT_DUP => "ComType has dup.",
            URT::VLDTR_E_MAR_DUP => "ManifestResource has dup.",
            URT::VLDTR_E_MAR_NOTPUBPRIV => "ManifestResource is neither Public not Private.",
            URT::VLDTR_E_TD_ENUMNOVALUE => "Enum has no 'value__' field.",
            URT::VLDTR_E_TD_ENUMVALSTATIC => "Enum's 'value__' field is static.",
            URT::VLDTR_E_TD_ENUMVALNOTSN => "Enum's 'value__' field is not SpecialName.",
            URT::VLDTR_E_TD_ENUMFLDNOTST => "Enum's field is not static.",
            URT::VLDTR_E_TD_ENUMFLDNOTLIT => "Enum's field is not literal.",
            URT::VLDTR_E_TD_ENUMNOLITFLDS => "Enum has no literal fields.",
            URT::VLDTR_E_TD_ENUMFLDSIGMISMATCH => "Enum's field sig does not match value__ sig.",
            URT::VLDTR_E_TD_ENUMVALNOT1ST => "Enum's 'value__' field is not first.",
            URT::VLDTR_E_FD_NOTVALUERTSN => "Field is RTSpecialName but name is not 'value__'.",
            URT::VLDTR_E_FD_VALUEPARNOTENUM => "Field 'value__' in not Enum class.",
            URT::VLDTR_E_FD_INSTINIFACE => "Instance field in interface.",
            URT::VLDTR_E_FD_NOTPUBINIFACE => "Non-public field in interface.",
            URT::VLDTR_E_FMD_GLOBALNOTPUBPRIVSC => "Global field/method neither Public nor PrivateScope.",
            URT::VLDTR_E_FMD_GLOBALNOTSTATIC => "Global field/method not static.",
            URT::VLDTR_E_FD_GLOBALNORVA => "Global field has no RVA.",
            URT::VLDTR_E_MD_CTORZERORVA => ".ctor,.cctor has zero RVA.",
            URT::VLDTR_E_FD_MARKEDNOMARSHAL => "Field is marked marshaled but has no marshaling rec.",
            URT::VLDTR_E_FD_MARSHALNOTMARKED => "Field has marshaling rec but is not marked marshaled.",
            URT::VLDTR_E_FD_MARKEDNODEFLT => "Field is marked HasDefault but has no const value.",
            URT::VLDTR_E_FD_DEFLTNOTMARKED => "Field has const value rec but is not marked HasDefault.",
            URT::VLDTR_E_FMD_MARKEDNOSECUR => "Field/method is marked HasSecurity but has no security rec.",
            URT::VLDTR_E_FMD_SECURNOTMARKED => "Field/method has security rec but is not marked HasSecurity.",
            URT::VLDTR_E_FMD_PINVOKENOTSTATIC => "Field/method is PInvoke but is not marked Static.",
            URT::VLDTR_E_FMD_MARKEDNOPINVOKE => "Field/method is marked PInvoke but has no ImplMap.",
            URT::VLDTR_E_FMD_PINVOKENOTMARKED => "Field/method has ImplMap but is not marked PInvoke.",
            URT::VLDTR_E_FMD_BADIMPLMAP => "Field/method has invalid ImplMap",
            URT::VLDTR_E_IMAP_BADMODREF => "ImplMap has invalid ModuleRef",
            URT::VLDTR_E_IMAP_BADMEMBER => "ImplMap has invalid MemberForwarded",
            URT::VLDTR_E_IMAP_BADIMPORTNAME => "ImplMap has invalid ImportName",
            URT::VLDTR_E_IMAP_BADCALLCONV => "ImplMap has invalid call conv",
            URT::VLDTR_E_FMD_BADACCESSFLAG => "Field/method has invalid access flag",
            URT::VLDTR_E_FD_INITONLYANDLITERAL => "Field is InitOnly and Literal",
            URT::VLDTR_E_FD_LITERALNOTSTATIC => "Field is Literal but not Static",
            URT::VLDTR_E_FMD_RTSNNOTSN => "Field/method is RTSpec.Name but not Spec.Name",
            URT::VLDTR_E_MD_ABSTPARNOTABST => "Method is abstract, parent is not",
            URT::VLDTR_E_MD_NOTSTATABSTININTF => "Method not static or abstract in interface",
            URT::VLDTR_E_MD_NOTPUBININTF => "Method not public in interface",
            URT::VLDTR_E_MD_CTORININTF => "ctor in interface",
            URT::VLDTR_E_MD_GLOBALCTORCCTOR => "global ctor or cctor",
            URT::VLDTR_E_MD_CTORSTATIC => "static ctor",
            URT::VLDTR_E_MD_CTORNOTSNRTSN => "ctor,cctor not marked SpecialName,RTSpecialName",
            URT::VLDTR_E_MD_CTORVIRT => "virtual ctor,cctor",
            URT::VLDTR_E_MD_CTORABST => "abstract ctor,cctor",
            URT::VLDTR_E_MD_CCTORNOTSTATIC => "instance cctor",
            URT::VLDTR_E_MD_ZERORVA => "RVA=0, method not abstract or pinvoke or runtime, or reverse",
            URT::VLDTR_E_MD_FINNOTVIRT => "Method is final and not virtual",
            URT::VLDTR_E_MD_STATANDFINORVIRT => "Method is static and final or virtual",
            URT::VLDTR_E_MD_ABSTANDFINAL => "Method is abstract and final",
            URT::VLDTR_E_MD_ABSTANDIMPL => "Method is abstract and implemented",
            URT::VLDTR_E_MD_ABSTANDPINVOKE => "Method is abstract and pinvoke",
            URT::VLDTR_E_MD_ABSTNOTVIRT => "Method is abstract and not virtual",
            URT::VLDTR_E_MD_NOTABSTNOTIMPL => "Method is not abstract and not implemented",
            URT::VLDTR_E_MD_NOTABSTBADFLAGSRVA => "Method is not abstract and not (RVA!=0 or pinvoke or runtime)",
            URT::VLDTR_E_MD_PRIVSCOPENORVA => "Method is PrivateScope and has RVA==0",
            URT::VLDTR_E_MD_GLOBALABSTORVIRT => "Global method is abstract or virtual",
            URT::VLDTR_E_SIG_LONGFORM => "Signature uses long form",
            URT::VLDTR_E_MD_MULTIPLESEMANTICS => "Method has multiple semantics (warning)",
            URT::VLDTR_E_MD_INVALIDSEMANTICS => "Method has invalid semantics (not event or prop)",
            URT::VLDTR_E_MD_SEMANTICSNOTEXIST => "Method has semantics assoc that does not exist",
            URT::VLDTR_E_MI_DECLNOTVIRT => "MethodImpl's Decl is not virtual",
            URT::VLDTR_E_FMD_GLOBALITEM => "Global field/method (warning,CLS)",
            URT::VLDTR_E_MD_MULTSEMANTICFLAGS => "Method has multiple semantic flags set",
            URT::VLDTR_E_MD_NOSEMANTICFLAGS => "Method has no semantic flags set",
            URT::VLDTR_E_FD_FLDINIFACE => "Field in Interface (warning, CLS)",
            URT::VLDTR_E_AS_HASHALGID => "Unrecognized Hash Alg ID (warning)",
            URT::VLDTR_E_AS_PROCID => "Unrecognized Processor ID in Assembly(warning)",
            URT::VLDTR_E_AR_PROCID => "Unrecognized Processor ID in AssemblyRef(warning)",
            URT::VLDTR_E_CN_PARENTRANGE => "Constant: parent token out of range",
            URT::VLDTR_E_AS_BADFLAGS => "Invalid flags in Assembly",
            URT::VLDTR_E_TR_HASTYPEDEF => "There is TypeDef with same name as TypeRef (warning)",
            URT::VLDTR_E_IFACE_BADIMPL => "In InterfaceImpl, the implementing token is not TypeDef",
            URT::VLDTR_E_IFACE_BADIFACE => "In InterfaceImpl, the implemented token is not TypeDef or TypeRef",
            URT::VLDTR_E_TD_SECURNOTMARKED => "TypeDef has security rec but not marked HasSecurity",
            URT::VLDTR_E_TD_MARKEDNOSECUR => "TypeDef marked HasSecurity but has no security rec",
            URT::VLDTR_E_MD_CCTORHASARGS => ".cctor has arguments",
            URT::VLDTR_E_CT_BADIMPL => "COMType has invalid Implementation",
            URT::VLDTR_E_MI_ALIENBODY => "MethodImpl has body from other class",
            URT::VLDTR_E_MD_CCTORCALLCONV => ".cctor has invalid calling convention",
            URT::VLDTR_E_MI_BADCLASS => "MethodImpl has invalid Class token",
            URT::VLDTR_E_MI_CLASSISINTF => "MethodImpl declared in Interface",
            URT::VLDTR_E_MI_BADDECL => "MethodImpl has invalid MethodDeclaration token",
            URT::VLDTR_E_MI_BADBODY => "MethodImpl has invalid MethodBody token",
            URT::VLDTR_E_MI_DUP => "MethodImpl has duplicate",
            URT::VLDTR_E_FD_BADPARENT => "Bad field parent",
            URT::VLDTR_E_MD_PARAMOUTOFSEQ => "Param out of sequence (warning)",
            URT::VLDTR_E_MD_PARASEQTOOBIG => "Param's sequence num exceeds num of args",
            URT::VLDTR_E_MD_PARMMARKEDNOMARSHAL => "Param marked HasMarshal, has no marshaling info",
            URT::VLDTR_E_MD_PARMMARSHALNOTMARKED => "Param has marshaling info, not marked HasMarshal",
            URT::VLDTR_E_MD_PARMMARKEDNODEFLT => "Param marked HasDefault, has no const value",
            URT::VLDTR_E_MD_PARMDEFLTNOTMARKED => "Param has const value, not marked HasDefault",
            URT::VLDTR_E_PR_BADSCOPE => "Prop has invalid scope",
            URT::VLDTR_E_PR_NONAME => "Prop has no name",
            URT::VLDTR_E_PR_NOSIG => "Prop has no signature",
            URT::VLDTR_E_PR_DUP => "Prop has a duplicate",
            URT::VLDTR_E_PR_BADCALLINGCONV => "Prop has bad calling convention",
            URT::VLDTR_E_PR_MARKEDNODEFLT => "Prop marked HasDefault, has no const value",
            URT::VLDTR_E_PR_DEFLTNOTMARKED => "Prop has const value, not marked HasDefault",
            URT::VLDTR_E_PR_BADSEMANTICS => "Prop has method not (Setter,Getter, or Other)",
            URT::VLDTR_E_PR_BADMETHOD => "Prop has method with invalid token",
            URT::VLDTR_E_PR_ALIENMETHOD => "Prop has method from another class",
            URT::VLDTR_E_CN_BLOBNOTNULL => "Const has non-null blob when it should not",
            URT::VLDTR_E_CN_BLOBNULL => "Const has null value blob",
            URT::VLDTR_E_EV_BADSCOPE => "Event has invalid scope",
            URT::VLDTR_E_EV_NONAME => "Event has no name",
            URT::VLDTR_E_EV_DUP => "Event has a duplicate",
            URT::VLDTR_E_EV_BADEVTYPE => "Event has invalid EventType",
            URT::VLDTR_E_EV_EVTYPENOTCLASS => "Event's EventType is not a class",
            URT::VLDTR_E_EV_BADSEMANTICS => "Event has method not (AddOn,RemoveOn,Fire,Other)",
            URT::VLDTR_E_EV_BADMETHOD => "Event has method with invalid token",
            URT::VLDTR_E_EV_ALIENMETHOD => "Event has method from another class",
            URT::VLDTR_E_EV_NOADDON => "Event has no AddOn method",
            URT::VLDTR_E_EV_NOREMOVEON => "Event has no RemoveOn method",
            URT::VLDTR_E_CT_DUPTDNAME => "ExportedType has same name as TypeDef",
            URT::VLDTR_E_MAR_BADOFFSET => "MRes refers to non-PE file with offset !=0",
            URT::VLDTR_E_DS_BADOWNER => "Decl.security has invalid owner token",
            URT::VLDTR_E_DS_BADFLAGS => "Decl.security has invalid action flags",
            URT::VLDTR_E_DS_NOBLOB => "Decl.security has no permission blob",
            URT::VLDTR_E_MAR_BADIMPL => "Manifest resource has invalid Implementation",
            URT::VLDTR_E_MR_VARARGCALLINGCONV => "MemberRef has VARARG calling conv. (CLS warning)",
            URT::VLDTR_E_MD_CTORNOTVOID => ".ctor,.cctor returning not void",
            URT::VLDTR_E_EV_FIRENOTVOID => "Fire method returning not void",
            URT::VLDTR_E_AS_BADLOCALE => "Invalid locale",
            URT::VLDTR_E_CN_PARENTTYPE => "Constant has parent of invalid type",
            URT::VLDTR_E_SIG_SENTINMETHODDEF => "E_T_SENTINEL in MethodDef signature",
            URT::VLDTR_E_SIG_SENTMUSTVARARG => "E_T_SENTINEL <=> VARARG",
            URT::VLDTR_E_SIG_MULTSENTINELS => "Multiple E_T_SENTINELs",
            URT::VLDTR_E_SIG_LASTSENTINEL => "E_T_SENTINEL not followed by type",
            URT::VLDTR_E_SIG_MISSARG => "Signature missing argument",
            URT::VLDTR_E_SIG_BYREFINFIELD => "Field of ByRef type",
            URT::VLDTR_E_MD_SYNCMETHODINVTYPE => "Synchronized method in value class",
            URT::VLDTR_E_TD_NAMETOOLONG => "TypeDef name too long",
            URT::VLDTR_E_AS_PROCDUP => "Duplicate Assembly Processor",
            URT::VLDTR_E_ASOS_DUP => "Duplicate Assembly OS (ID+ver.major+ver.minor)",
            URT::VLDTR_E_MAR_BADFLAGS => "Manifest Resource has bad flags",
            URT::VLDTR_E_CT_NOTYPEDEFID => "COMType has nil TypeDefId",
            URT::VLDTR_E_FILE_BADFLAGS => "File has bad flags",
            URT::VLDTR_E_FILE_NULLHASH => "File has no hash blob",
            URT::VLDTR_E_MOD_NONAME => "Module has no name",
            URT::VLDTR_E_MOD_NAMEFULLQLFD => "Module has fully-qualified name",
            URT::VLDTR_E_TD_RTSPCLNOTSPCL => "TypeDef is tdRTSpecialName but not tdSpecialName",
            URT::VLDTR_E_TD_EXTENDSIFACE => "TypeDef extends interface",
            URT::VLDTR_E_MD_CTORPINVOKE => ".ctor,.cctor is PInvokeImpl",
            URT::VLDTR_E_TD_SYSENUMNOTCLASS => "System.Enum is not a class",
            URT::VLDTR_E_TD_SYSENUMNOTEXTVTYPE => "System.Enum extends not System.ValueType",
            URT::VLDTR_E_MI_SIGMISMATCH => "MethodImpl's Decl and Body signatures mismatch",
            URT::VLDTR_E_TD_ENUMHASMETHODS => "TypeDef extends System.Enum but has methods",
            URT::VLDTR_E_TD_ENUMIMPLIFACE => "TypeDef extends System.Enum but impls interface(s)",
            URT::VLDTR_E_TD_ENUMHASPROP => "TypeDef extends System.Enum but has prop(s)",
            URT::VLDTR_E_TD_ENUMHASEVENT => "TypeDef extends System.Enum but has event(s)",
            URT::VLDTR_E_TD_BADMETHODLST => "TypeDef has MethodList > Nmethods+1",
            URT::VLDTR_E_TD_BADFIELDLST => "TypeDef has FieldList > Nfields+1",
            URT::VLDTR_E_CN_BADTYPE => "Constant has wrong type",
            URT::VLDTR_E_TD_ENUMNOINSTFLD => "Enum has no instance fields",
            URT::VLDTR_E_TD_ENUMMULINSTFLD => "Enum has multiple instance fields",
            URT::VLDTR_E_INTERRUPTED => "Validator has been interrupted by the VEHandler.",
            URT::VLDTR_E_NOTINIT => "Validator failed to initialize correctly.",
            URT::CORDBG_E_UNRECOVERABLE_ERROR => "Unrecoverable API error.",
            URT::CORDBG_E_PROCESS_TERMINATED => "Process was terminated.",
            URT::CORDBG_E_PROCESS_NOT_SYNCHRONIZED => "Process not synchronized.",
            URT::CORDBG_E_CLASS_NOT_LOADED => "A class is not loaded.",
            URT::CORDBG_E_IL_VAR_NOT_AVAILABLE => "An IL variable is not available at the current native IP.",
            URT::CORDBG_E_BAD_REFERENCE_VALUE => "A reference value was found to be bad during dereferencing.",
            URT::CORDBG_E_FIELD_NOT_AVAILABLE => "A field in a class is not available, because the runtime optimized it away.",
            URT::CORDBG_E_NON_NATIVE_FRAME => "Native frame only operation on-native frame",
            URT::CORDBG_E_NONCONTINUABLE_EXCEPTION => "Continue on non-continuable exception",
            URT::CORDBG_E_CODE_NOT_AVAILABLE => "The code is currently unavailable",
            URT::CORDBG_E_FUNCTION_NOT_IL => "Attempt to get a ICorDebugFunction for a function that is not IL.",
            URT::CORDBG_E_CANT_SET_IP_INTO_FINALLY => "SetIP isn't possible, because SetIP would move EIP from outside of an exception handling finally clause to a point inside of one.",
            URT::CORDBG_E_CANT_SET_IP_OUT_OF_FINALLY => "SetIP isn't possible because it would move EIP from within an exception handling finally clause to a point outside of one.",
            URT::CORDBG_E_CANT_SET_IP_INTO_CATCH => "SetIP isn't possible, because SetIP would move EIP from outside of an exception handling catch clause to a point inside of one.",
            URT::CORDBG_E_SET_IP_NOT_ALLOWED_ON_NONLEAF_FRAME => "Setip cannot be done on any frame except the leaf frame.",
            URT::CORDBG_E_SET_IP_IMPOSSIBLE => "SetIP isn't allowed. For example, there is insufficient memory to perform SetIP.",
            URT::CORDBG_E_FUNC_EVAL_BAD_START_POINT => "Func eval can't work if we're, for example, not stopped at a GC safe point.",
            URT::CORDBG_E_INVALID_OBJECT => "This object value is no longer valid.",
            URT::CORDBG_E_FUNC_EVAL_NOT_COMPLETE => "If you call CordbEval::GetResult before the func eval has finished, you'll get this result.",
            URT::CORDBG_E_INPROC_NOT_IMPL => "The inproc version of the debugging API doesn't implement this function,",
            URT::CORDBG_E_STATIC_VAR_NOT_AVAILABLE => "A static variable isn't available because it hasn't been initialized yet.",
            URT::CORDBG_E_OBJECT_IS_NOT_COPYABLE_VALUE_CLASS => "Can't copy a VC with object refs in it.",
            URT::CORDBG_E_CANT_SETIP_INTO_OR_OUT_OF_FILTER => "SetIP can't leave or enter a filter",
            URT::CORDBG_E_CANT_CHANGE_JIT_SETTING_FOR_ZAP_MODULE => "You can't change JIT settings for ZAP modules.",
            URT::CORDBG_E_CANT_SET_IP_OUT_OF_FINALLY_ON_WIN64 => "SetIP isn't possible because it would move EIP from within a finally clause to a point outside of one on WIN64 platforms.",
            URT::CORDBG_E_CANT_SET_IP_OUT_OF_CATCH_ON_WIN64 => "SetIP isn't possible because it would move EIP from within a catch clause to a point outside of one on WIN64 platforms.",
            URT::CORDBG_E_REMOTE_CONNECTION_CONN_RESET => "The remote device closed the connection.",
            URT::CORDBG_E_REMOTE_CONNECTION_KEEP_ALIVE => "The connection was closed due to akeep alive failure.",
            URT::CORDBG_E_REMOTE_CONNECTION_FATAL_ERROR => "Generic error that the device connection has been broken with no chance for recovery.",
            URT::CORDBG_E_CANT_SET_TO_JMC => "Can't use JMC on this code (likely wrong jit settings).",
            URT::CORDBG_E_NO_CONTEXT_FOR_INTERNAL_FRAME => "The current frame is not a child frame.",
            URT::CORDBG_E_NOT_CHILD_FRAME => "The provided CONTEXT does not match the specified thread.",
            URT::CORDBG_E_NON_MATCHING_CONTEXT => "The stackwalker is now past the end of stack. No information is available.",
            URT::CORDBG_E_PAST_END_OF_STACK => "Func eval cannot update a variable stored in a register on a non-leaf frame. The most likely cause is that such a variable is passed as a ref/out argument.",
            URT::CORDBG_E_BAD_THREAD_STATE => "The state of the thread is invalid.",
            URT::CORDBG_E_FUNC_EVAL_CANNOT_UPDATE_REGISTER_IN_NONLEAF_FRAME => "The Method has no associated IL.",
            URT::CORDBG_E_DEBUGGER_ALREADY_ATTACHED => "This process has already been attached to",
            URT::CORDBG_E_SUPERFLOUS_CONTINUE => "Returned from a call to Continue that was Not matched with a stopping event.",
            URT::CORDBG_E_SET_VALUE_NOT_ALLOWED_ON_NONLEAF_FRAME => "Can't perfrom SetValue on non-leaf frames.",
            URT::CORDBG_E_ENC_EH_MAX_NESTING_LEVEL_CANT_INCREASE => "When doing EnC, some JITters don't let you increase the maximum level to which exception handling can be nested.",
            URT::CORDBG_E_ENC_MODULE_NOT_ENC_ENABLED => "Tried to do EnC on a module that wasn't started in EnC mode.",
            URT::CORDBG_E_SET_IP_NOT_ALLOWED_ON_EXCEPTION => "Setip cannot be done on any exception",
            URT::CORDBG_E_VARIABLE_IS_ACTUALLY_LITERAL => "The 'variable' doesn't exist because it is a literal optimized away by the compiler - ask Metadata for it's default value, instead.",
            URT::CORDBG_E_PROCESS_DETACHED => "Process has been detached from",
            URT::CORDBG_E_ENC_METHOD_SIG_CHANGED => "Not allowed to change the signature of an existing method - compiler should make new method instead.",
            URT::CORDBG_E_ENC_METHOD_NO_LOCAL_SIG => "Can't get the local signature for the method we're trying to EnC.",
            URT::CORDBG_E_ENC_CANT_ADD_FIELD_TO_VALUE_OR_LAYOUT_CLASS => "Adding a field to a value or layout class is prohibitted, since we can't guarantee the new field is contiguous to VC's on the stack, embedded in other objects, etc.",
            URT::CORDBG_E_ENC_CANT_CHANGE_FIELD => "Once you've got a field, you're not allowed to change it, since that would change the size of the type it belongs to.",
            URT::CORDBG_E_ENC_CANT_ADD_NON_PRIVATE_MEMBER => "Only support addition of private members.",
            URT::CORDBG_E_FIELD_NOT_STATIC => "Returned if someone tries to call GetStaticFieldValue on a non-static field",
            URT::CORDBG_E_FIELD_NOT_INSTANCE => "Returned if someone tries to call GetStaticFieldValue on a non-instance field",
            URT::CORDBG_E_ENC_ZAPPED_WITHOUT_ENC => "If a zap file was created without the EnC flag set, then we can't do EnC on it, no matter what.",
            URT::CORDBG_E_ENC_BAD_METHOD_INFO => "Lacking information about method.",
            URT::CORDBG_E_ENC_JIT_CANT_UPDATE => "The JIT is unable to update the method.",
            URT::CORDBG_E_ENC_MISSING_CLASS => "An internal structure about the class is missing",
            URT::CORDBG_E_ENC_INTERNAL_ERROR => "Generic message for 'Something user doesn't control went wrong' message.",
            URT::CORDBG_E_ENC_HANGING_FIELD => "The field was added via enc after the class was loaded, and so instead of the field being contiguous with the other fields, it's 'hanging' off the instance, so the right side will have to go & get (instance-specific info based on the particular object.",
            URT::CORDBG_E_MODULE_NOT_LOADED => "If the module isn't loaded, including if it's been unloaded.",
            URT::CORDBG_E_ENC_CANT_CHANGE_SUPERCLASS => "Not allowed to change which class something inherits from",
            URT::CORDBG_E_UNABLE_TO_SET_BREAKPOINT => "Can't set a breakpoint here.",
            URT::CORDBG_E_DEBUGGING_NOT_POSSIBLE => "Debugging isn't possible due to an incompatability within the CLR implementation.",
            URT::CORDBG_E_KERNEL_DEBUGGER_ENABLED => "Debugging isn't possible because a kernel debugger is enabled on the system.",
            URT::CORDBG_E_KERNEL_DEBUGGER_PRESENT => "Debugging isn't possible because a kernel debugger is present on the system.",
            URT::CORDBG_E_HELPER_THREAD_DEAD => "The debugger's internal helper thread is dead.",
            URT::CORDBG_E_INTERFACE_INHERITANCE_CANT_CHANGE => "Not allowed to change interface inheritance.",
            URT::CORDBG_E_INCOMPATIBLE_PROTOCOL => "The debugger's protocol is incompatible with the debuggee.",
            URT::CORDBG_E_TOO_MANY_PROCESSES => "The debugger can only handle a finite number of debuggees.",
            URT::CORDBG_E_INTEROP_NOT_SUPPORTED => "Interop is not allowed on a win9x platform",
            URT::CORDBG_E_NO_REMAP_BREAKPIONT => "Cannot call RemapFunction until have received RemapBreakpoint",
            URT::CORDBG_E_OBJECT_NEUTERED => "Object has been neutered (it's in a zombie state).",
            URT::CORPROF_E_FUNCTION_NOT_COMPILED => "Function not yet compiled.",
            URT::CORPROF_E_DATAINCOMPLETE => "The ID is not fully loaded/defined yet.",
            URT::CORPROF_E_NOT_REJITABLE_METHODS => "The Module is not configured for updateable methods.",
            URT::CORPROF_E_CANNOT_UPDATE_METHOD => "The Method could not be updated for re-jit.",
            URT::CORPROF_E_FUNCTION_NOT_IL => "The Method has no associated IL",
            URT::CORPROF_E_NOT_MANAGED_THREAD => "The thread has never run managed code before",
            URT::CORPROF_E_CALL_ONLY_FROM_INIT => "The function may only be called during profiler init",
            URT::CORPROF_E_INPROC_NOT_ENABLED => "Inprocess debugging must be enabled during init Also returned when BeginInprocDebugging not called before using the inprocess debugging services",
            URT::CORPROF_E_JITMAPS_NOT_ENABLED => "Can't get a JIT map becuase they are not enabled",
            URT::CORPROF_E_INPROC_ALREADY_BEGUN => "If a profiler tries to call BeginInprocDebugging more than once, it will get this error.",
            URT::CORPROF_E_INPROC_NOT_AVAILABLE => "States that inprocess debugging not allowed at this point (for example during GC callbacks or RuntimeSuspention callbacks",
            URT::CORPROF_E_NOT_YET_AVAILABLE => "This is a general error used to indicated that the information requested is not yet available",
            URT::CORPROF_E_TYPE_IS_PARAMETERIZED => "The given type is a generic and cannot be used with this method.",
            URT::CORPROF_E_FUNCTION_IS_PARAMETERIZED => "The given function is a generic and cannot be used with this method.",
            URT::CORPROF_E_STACKSNAPSHOT_INVALID_TGT_THREAD => "A profiler tried to walk the stack of an invalid thread",
            URT::CORPROF_E_STACKSNAPSHOT_UNMANAGED_CTX => "A profiler can not walk a thread that is currently executing unmanaged code",
            URT::CORPROF_E_STACKSNAPSHOT_UNSAFE => "A stackwalk at this point may cause dead locks or data corruption",
            URT::CORPROF_E_STACKSNAPSHOT_ABORTED => "Stackwalking callback requested the walk to abort",
            URT::CORPROF_E_LITERALS_HAVE_NO_ADDRESS => "Returned when asked for the address of a static that is a literal.",
            URT::CORPROF_E_UNSUPPORTED_CALL_SEQUENCE => "A call was made at an unsupported time. Examples include illegally calling a profiling API method asynchronously, calling a method that might trigger a GC at an unsafe time, and calling a method at a time that could cause locks to be taken out of order.",
            URT::CORPROF_E_ASYNCHRONOUS_UNSAFE => "A legal asynchronous call was made at an unsafe time (e.g., CLR locks are held)",
            URT::CORPROF_E_CLASSID_IS_ARRAY => "The specified ClassID cannot be inspected by this function because it is an array",
            URT::CORPROF_E_CLASSID_IS_COMPOSITE => "The specified ClassID is a non-array composite type (e.g., ref) and cannot be inspected",
            URT::CORPROF_E_PROFILER_DETACHING => "The profiler's call into the CLR is disallowed because the profiler is attempting to detach.",
            URT::CORPROF_E_PROFILER_NOT_ATTACHABLE => "The profiler does not support attaching to a live process.",
            URT::CORPROF_E_UNRECOGNIZED_PIPE_MSG_FORMAT => "The message sent on the profiling API attach pipe is in an unrecognized format.",
            URT::CORPROF_E_PROFILER_ALREADY_ACTIVE => "The request to attach a profiler was denied because a profiler is already loaded.",
            URT::CORPROF_E_PROFILEE_INCOMPATIBLE_WITH_TRIGGER => "Unable to request a profiler attach because the target profilee's runtime is of a version incompatible with the current process calling AttachProfiler().",
            URT::CORPROF_E_IPC_FAILED => "AttachProfiler() encountered an error while communicating on the pipe to the target profilee. This is often caused by a target profilee that is shutting down or killed while AttachProfiler() is reading or writing the pipe.",
            URT::CORPROF_E_PROFILEE_PROCESS_NOT_FOUND => "AttachProfiler() was unable to find a profilee with the specified process ID.",
            URT::CORPROF_E_CALLBACK3_REQUIRED => "Profiler must implement ICorProfilerCallback3 interface for this call to be supported.",
            URT::CORPROF_E_UNSUPPORTED_FOR_ATTACHING_PROFILER => "This call was attempted by a profiler that attached to the process after startup, but this call is only supported by profilers that are loaded into the process on startup.",
            URT::CORPROF_E_IRREVERSIBLE_INSTRUMENTATION_PRESENT => "Detach is impossible because the profiler has either instrumented IL or inserted enter/leave hooks. Detach was not attempted; the profiler is still fully attached",
            URT::CORPROF_E_RUNTIME_UNINITIALIZED => "The profiler called a function that cannot complete because the CLR is not yet fully initialized. The profiler may try again once the CLR has fully started.",
            URT::CORPROF_E_IMMUTABLE_FLAGS_SET => "Detach is impossible because immutable flags were set by the profiler at startup. Detach was not attempted; the profiler is still fully attached.",
            URT::CORPROF_E_PROFILER_NOT_YET_INITIALIZED => "The profiler called a function that cannot complete because the profiler is not yet fully initialized.",
            URT::CORPROF_E_INCONSISTENT_WITH_FLAGS => "The profiler called a function that first requires additional flags to be set in the event mask. This HRESULT may also indicate that the profiler called a function that first requires that some of the flags currently set in the event mask be reset.",
            URT::CORPROF_E_PROFILER_CANCEL_ACTIVATION => "The profiler has requested that the CLR instance not load the profiler into this process.",
            URT::CORPROF_E_CONCURRENT_GC_NOT_PROFILABLE => "Concurrent GC mode is enabled, which prevents use of COR_PRF_MONITOR_GC",
            URT::CORPROF_E_INCONSISTENT_FLAGS_WITH_HOST_PROTECTION_SETTING => "Host enforces host protection attributes, which prevents use of COR_PRF_USE_PROFILE_IMAGES, COR_PRF_MONITOR_CODE_TRANSITIONS, and COR_PRF_MONITOR_ENTERLEAVE",
            URT::CORPROF_E_DEBUGGING_DISABLED => "This functionality requires CoreCLR debugging to be enabled.",
            URT::CORPROF_E_TIMEOUT_WAITING_FOR_CONCURRENT_GC => "Timed out on waiting for concurrent GC to finish during attach.",
            URT::CORPROF_E_MODULE_IS_DYNAMIC => "The specified module was dynamically generated (e.g., via Reflection.Emit API), and is thus not supported by this API method.",
            URT::CORPROF_E_CALLBACK4_REQUIRED => "Profiler must implement ICorProfilerCallback4 interface for this call to be supported.",
            URT::CORPROF_E_REJIT_NOT_ENABLED => "This call is not supported unless ReJIT is first enabled during initialization by setting COR_PRF_ENABLE_REJIT via SetEventMask.",
            URT::CORPROF_E_ACTIVE_REJIT_REQUEST_NOT_FOUND => "Could not revert the specified method, because a corresponding active ReJIT request was not found. Either the method was never requested to be reJITted OR the method was already reverted.",
            URT::CORPROF_E_FUNCTION_IS_COLLECTIBLE => "The specified function is instantiated into a collectible assembly, and is thus not supported by this API method.",
            URT::CORPROF_E_REJIT_REQUIRES_DISABLE_NGEN => "If the profiler specifies COR_PRF_ENABLE_REJIT to SetEventMask, it must also specify COR_PRF_DISABLE_ALL_NGEN_IMAGES.",
            URT::CORPROF_E_CALLBACK6_REQUIRED => "Profiler must implement ICorProfilerCallback6 interface for this call to be supported.",
            URT::CORPROF_E_RUNTIME_SUSPEND_REQUIRED => "This call can't be completed safely because the runtime is not suspended (This HRESULT is used only internally by our rejit implementation right now. If it ever appears in a public API HRESULT it is a bug. It is included here to prevent anyone else from defining a new failure with the same value and causing confusion)",
            URT::CORPROF_E_CALLBACK7_REQUIRED => "Profiler must implement ICorProfilerCallback7 interface for this call to be supported.",
            URT::SECURITY_E_XML_TO_ASN_ENCODING => "Failed to convert XML to ASN",
            URT::SECURITY_E_INCOMPATIBLE_SHARE => "Loading this assembly would produce a different grant set from other instances",
            URT::SECURITY_E_UNVERIFIABLE => "Unverifable code failed policy check",
            URT::SECURITY_E_INCOMPATIBLE_EVIDENCE => "Assembly already loaded without additional security evidence.",
            URT::CORSEC_E_DECODE_SET => "Failure decoding permission set.",
            URT::CORSEC_E_ENCODE_SET => "Failure encoding permission set.",
            URT::CORSEC_E_UNSUPPORTED_FORMAT => "Unrecognized encoding format.",
            URT::SN_CRYPTOAPI_CALL_FAILED => "StrongName APIs not supported on system.",
            URT::CORSEC_E_CRYPTOAPI_CALL_FAILED => "StrongName APIs not supported on system.",
            URT::SN_NO_SUITABLE_CSP => "StrongName APIs could not locate a matching CSP.",
            URT::CORSEC_E_NO_SUITABLE_CSP => "StrongName APIs could not locate a matching CSP.",
            URT::CORSEC_E_INVALID_ATTR => "Invalid security custom attribute.",
            URT::CORSEC_E_POLICY_EXCEPTION => "PolicyException thrown.",
            URT::CORSEC_E_MIN_GRANT_FAIL => "Failed to grant minimum permission requests.",
            URT::CORSEC_E_NO_EXEC_PERM => "Failed to grant permission to execute.",
            URT::CORSEC_E_XMLSYNTAX => "XML Syntax error.",
            URT::CORSEC_E_INVALID_STRONGNAME => "Bad custom attribute serialized blob version.",
            URT::CORSEC_E_MISSING_STRONGNAME => "Invalid security action code.",
            URT::CORSEC_E_CONTAINER_NOT_FOUND => "CA reference to CA definition in same assembly.",
            URT::CORSEC_E_INVALID_IMAGE_FORMAT => "Use of non-CAS permission with invalid action.",
            URT::CORSEC_E_INVALID_PUBLICKEY => "Failed to load assembly containing CA (or required CA type).",
            URT::CORSEC_E_SIGNATURE_MISMATCH => "Signature size mismatch.",
            URT::SN_E_PUBLICKEY_MISMATCH => "Public key of assembly did not match signing public key.",
            URT::CORSEC_E_CRYPTO => "Failure during Cryptographic operation.",
            URT::CORSEC_E_CRYPTO_UNEX_OPER => "Unexpected Cryptographic operation.",
            URT::CORSECATTR_E_BAD_ATTRIBUTE => "Unable to create store file mapping.",
            URT::CORSECATTR_E_MISSING_CONSTRUCTOR => "Unable to map the store file.",
            URT::CORSECATTR_E_FAILED_TO_CREATE_PERM => "Unable to determine store file size.",
            URT::CORSECATTR_E_BAD_ACTION_ASM => "Unable to create mutex.",
            URT::CORSECATTR_E_BAD_ACTION_OTHER => "Unable to lock the store.",
            URT::CORSECATTR_E_BAD_PARENT => "File Write failed.",
            URT::CORSECATTR_E_TRUNCATED => "Bad custom attribute serialized blob.",
            URT::CORSECATTR_E_BAD_VERSION => "Bad custom attribute serialized blob version.",
            URT::CORSECATTR_E_BAD_ACTION => "Invalid security action code.",
            URT::CORSECATTR_E_NO_SELF_REF => "CA reference to CA definition in same assembly.",
            URT::CORSECATTR_E_BAD_NONCAS => "Use of non-CAS permission with invalid action.",
            URT::CORSECATTR_E_ASSEMBLY_LOAD_FAILED => "Failed to load assembly containing CA (or required CA type).",
            URT::CORSECATTR_E_ASSEMBLY_LOAD_FAILED_EX => "Failed to load assembly containing CA (or required CA type).",
            URT::CORSECATTR_E_TYPE_LOAD_FAILED => "Failed to load CA type (or required CA type).",
            URT::CORSECATTR_E_TYPE_LOAD_FAILED_EX => "Failed to load CA type (or required CA type).",
            URT::CORSECATTR_E_ABSTRACT => "CA type is abstract.",
            URT::CORSECATTR_E_UNSUPPORTED_TYPE => "CA type is abstract",
            URT::CORSECATTR_E_UNSUPPORTED_ENUM_TYPE => "Unsupported type for field/property setter",
            URT::CORSECATTR_E_NO_FIELD => "Unsupported base type for enum field/property",
            URT::CORSECATTR_E_NO_PROPERTY => "Couldn't find a CA field",
            URT::CORSECATTR_E_EXCEPTION => "Couldn't find a CA property",
            URT::CORSECATTR_E_EXCEPTION_HR => "Unexpected exception",
            URT::ISS_E_ISOSTORE_START => "Unexpected exception",
            URT::ISS_E_ISOSTORE => "IsolatedStorage operation failed.",
            URT::ISS_E_OPEN_STORE_FILE => "Unable to open the store.",
            URT::ISS_E_OPEN_FILE_MAPPING => "Unable to create store file mapping.",
            URT::ISS_E_MAP_VIEW_OF_FILE => "Unable to map the store file.",
            URT::ISS_E_GET_FILE_SIZE => "Unable to determine store file size.",
            URT::ISS_E_CREATE_MUTEX => "Unable to create mutex.",
            URT::ISS_E_LOCK_FAILED => "Unable to lock the store.",
            URT::ISS_E_FILE_WRITE => "File Write failed.",
            URT::ISS_E_SET_FILE_POINTER => "Cannot set file pointer.",
            URT::ISS_E_CREATE_DIR => "Unable to create the store directory.",
            URT::ISS_E_STORE_NOT_OPEN => "Store must be open for this operation.",
            URT::ISS_E_CORRUPTED_STORE_FILE => "Store file is corrupt.",
            URT::ISS_E_STORE_VERSION => "Store version is not supported.",
            URT::ISS_E_FILE_NOT_MAPPED => "Store file is not mapped.",
            URT::ISS_E_BLOCK_SIZE_TOO_SMALL => "Block size is too small.",
            URT::ISS_E_ALLOC_TOO_LARGE => "Allocation size is too large.",
            URT::ISS_E_USAGE_WILL_EXCEED_QUOTA => "Allowed quota is fully used.",
            URT::ISS_E_TABLE_ROW_NOT_FOUND => "Row not found.",
            URT::ISS_E_DEPRECATE => "Unable to deprecate old store",
            URT::ISS_E_CALLER => "Unable to determine the caller",
            URT::ISS_E_PATH_LENGTH => "Path length is too long",
            URT::ISS_E_MACHINE => "Machine Store is not supported",
            URT::ISS_E_MACHINE_DACL => "The DACL for the machine store is incorrect or could not be created",
            URT::ISS_E_ISOSTORE_END => "Description is missing",
            URT::COR_E_EXCEPTION => "General Exception (Base class for all exceptions in the runtime)",
            URT::COR_E_SYSTEM => "System.Exception (The base class for the runtime's 'less serious' exceptions)",
            URT::COR_E_ARGUMENTOUTOFRANGE => "An argument was out of its legal range.",
            URT::COR_E_ARRAYTYPEMISMATCH => "Attempted to store an object of the wrong type in an array.",
            URT::COR_E_CONTEXTMARSHAL => "Attempted to marshal an object across a context boundary.",
            URT::COR_E_TIMEOUT => "Operation timed out.",
            URT::COR_E_EXECUTIONENGINE => "Internal CLR error.",
            URT::COR_E_FIELDACCESS => "Access to this field is denied.",
            URT::COR_E_INDEXOUTOFRANGE => "Array subscript out of range.",
            URT::COR_E_INVALIDOPERATION => "An operation is not legal in the current state.",
            URT::COR_E_SECURITY => "OLE Variant has an invalid type.",
            URT::COR_E_REMOTING => "An expected resource in the assembly manifest was missing.",
            URT::COR_E_SERIALIZATION => "A mismatch has occurred between the runtime type of the array and the sub type recorded in the metadata.",
            URT::COR_E_VERIFICATION => "Uncaught exception during type initialization.",
            URT::COR_E_SERVER => "Invalid marshaling directives.",
            URT::COR_E_SERVICEDCOMPONENT => "An expected satellite assembly containing the ultimate fallback resources for a given culture was not found or could not be loaded.",
            URT::COR_E_METHODACCESS => "Access to this method is denied.",
            URT::COR_E_MISSINGFIELD => "Field does not exist.",
            URT::COR_E_MISSINGMEMBER => "Member does not exist.",
            URT::COR_E_MISSINGMETHOD => "Method does not exist.",
            URT::COR_E_MULTICASTNOTSUPPORTED => "Attempt to combine delegates that are not multicast.",
            URT::COR_E_NOTSUPPORTED => "Operation is not supported.",
            URT::COR_E_OVERFLOW => "Arithmetic, casting or conversion operation overflowed or underflowed.",
            URT::COR_E_RANK => "An array has the wrong number of dimensions for a particular operation.",
            URT::COR_E_SYNCHRONIZATIONLOCK => "This operation must be called from a synchronized block.",
            URT::COR_E_THREADINTERRUPTED => "Thread was interrupted from a waiting state.",
            URT::COR_E_MEMBERACCESS => "A datatype misalignment was detected in a load or store instruction.",
            URT::COR_E_THREADSTATE => "Thread is in an invalid state for this operation.",
            URT::COR_E_THREADSTOP => "Thread is stopping.",
            URT::COR_E_TYPELOAD => "Could not find or load a type.",
            URT::COR_E_ENTRYPOINTNOTFOUND => "Could not find the specified DllImport entrypoint.",
            URT::COR_E_DLLNOTFOUND => "Could not find the specified DllImport Dll.",
            URT::COR_E_THREADSTART => "Indicate that a user thread fails to start",
            URT::COR_E_INVALIDCOMOBJECT => "An invalid __ComObject has been used.",
            URT::COR_E_NOTFINITENUMBER => "Not a Number.",
            URT::COR_E_DUPLICATEWAITOBJECT => "An object appears more than once in the wait objects array.",
            URT::COR_E_SEMAPHOREFULL => "Reached maximum count for semaphore. Adding the given count to the semaphore would cause it to exceed its maximum count",
            URT::COR_E_WAITHANDLECANNOTBEOPENED => "No Semaphore of the given name exists",
            URT::COR_E_ABANDONEDMUTEX => "The wait completed due to an abandoned mutex",
            URT::COR_E_THREADABORTED => "Thread has aborted.",
            URT::COR_E_INVALIDOLEVARIANTTYPE => "OLE Variant has an invalid type.",
            URT::COR_E_MISSINGMANIFESTRESOURCE => "An expected resource in the assembly manifest was missing.",
            URT::COR_E_SAFEARRAYTYPEMISMATCH => "A mismatch has occurred between the runtime type of the array and the sub type recorded in the metadata.",
            URT::COR_E_TYPEINITIALIZATION => "Uncaught exception during type initialization.",
            URT::COR_E_MARSHALDIRECTIVE => "Invalid marshaling directives.",
            URT::COR_E_MISSINGSATELLITEASSEMBLY => "An expected satellite assembly containing the ultimate fallback resources for a given culture was not found or could not be loaded.",
            URT::COR_E_FORMAT => "The format of one argument does not meet the contract of the method.",
            URT::COR_E_SAFEARRAYRANKMISMATCH => "A mismatch has occurred between the runtime rank of the array and the rank recorded in the metadata.",
            URT::COR_E_PLATFORMNOTSUPPORTED => "Operation is not supported on this platform.",
            URT::COR_E_INVALIDPROGRAM => "Invalid IL or CLR metadata. A program contained invalid IL or bad metadata. Usually this is a compiler bug.",
            URT::COR_E_OPERATIONCANCELED => "The operation was cancelled.",
            URT::COR_E_INSUFFICIENTMEMORY => "Not enough memory was available for an operation.",
            URT::COR_E_RUNTIMEWRAPPED => "An object that does not derive from System.Exception has been wrapped in a RuntimeWrappedException.",
            URT::COR_E_DEVICESNOTSUPPORTED => "Devices not supported.",
            URT::COR_E_DATAMISALIGNED => "A datatype misalignment was detected in a load or store instruction.",
            URT::COR_E_CODECONTRACTFAILED => "A managed code contract (ie, precondition, postcondition, invariant, or assert) failed.",
            URT::COR_E_TYPEACCESS => "Access to this type is denied.",
            URT::COR_E_KEYNOTFOUND => "The given key was not present in the dictionary.",
            URT::COR_E_INSUFFICIENTEXECUTIONSTACK => "Insufficient stack to continue executing the program safely. This can happen from having too many functions on the call stack or function on the stack using too much stack space.",
            URT::COR_E_APPLICATION => "Application exception",
            URT::COR_E_INVALIDFILTERCRITERIA => "The given filter criteria does not match the filter content.",
            URT::COR_E_REFLECTIONTYPELOAD => "Could not find or load a specific class that was requested through Reflection.",
            URT::COR_E_TARGET => "Attempt to invoke non-static method with a null Object.",
            URT::COR_E_TARGETINVOCATION => "Uncaught exception thrown by method called through Reflection.",
            URT::COR_E_CUSTOMATTRIBUTEFORMAT => "Custom attribute has invalid format.",
            URT::COR_E_IO => "Error during managed I/O.",
            URT::COR_E_FILELOAD => "Could not find or load a specific file.",
            URT::COR_E_OBJECTDISPOSED => "The object has already been disposed.",
            URT::COR_E_FAILFAST => "Runtime operation halted by call to System.Environment.FailFast().",
            URT::COR_E_HOSTPROTECTION => "The host has forbidden this operation.",
            URT::COR_E_ILLEGAL_REENTRANCY => "Attempted to call into managed code when executing inside a low level extensibility point.",
            URT::CLR_E_SHIM_RUNTIMELOAD => "Failed to load the runtime.",
            URT::CLR_E_SHIM_RUNTIMEEXPORT => "Failed to find a required export in the runtime.",
            URT::CLR_E_SHIM_INSTALLROOT => "Install root is not defined.",
            URT::CLR_E_SHIM_INSTALLCOMP => "Expected component of the runtime is not available.",
            URT::CLR_E_SHIM_LEGACYRUNTIMEALREADYBOUND => "A runtime has already been bound for legacy activation policy use.",
            URT::CLR_E_SHIM_SHUTDOWNINPROGRESS => "The operation is invalid because the process may be shutting down.",
            URT::VER_E_HRESULT => "Description is missing",
            URT::VER_E_OFFSET => "Description is missing",
            URT::VER_E_OPCODE => "Description is missing",
            URT::VER_E_OPERAND => "Description is missing",
            URT::VER_E_TOKEN => "Description is missing",
            URT::VER_E_EXCEPT => "Description is missing",
            URT::VER_E_STACK_SLOT => "Description is missing",
            URT::VER_E_LOC => "Description is missing",
            URT::VER_E_ARG => "Description is missing",
            URT::VER_E_FOUND => "Filter contains handler.",
            URT::VER_E_EXPECTED => "Nested filters.",
            URT::VER_E_LOC_BYNAME => "filter >= code size",
            URT::VER_E_UNKNOWN_OPCODE => "Unknown opcode.",
            URT::VER_E_SIG_CALLCONV => "Unknown calling convention.",
            URT::VER_E_SIG_ELEMTYPE => "Unknown ELEMENT_TYPE.",
            URT::VER_E_RET_SIG => "[return sig]",
            URT::VER_E_FIELD_SIG => "[field sig]",
            URT::VER_E_INTERNAL => "Internal error.",
            URT::VER_E_STACK_TOO_LARGE => "Stack is too large.",
            URT::VER_E_ARRAY_NAME_LONG => "Branch out of exception handler block.",
            URT::VER_E_FALLTHRU => "fall through end of the method without returning",
            URT::VER_E_TRY_GTEQ_END => "try start >= try end",
            URT::VER_E_TRYEND_GT_CS => "try end > code size",
            URT::VER_E_HND_GTEQ_END => "handler start >= handler end",
            URT::VER_E_HNDEND_GT_CS => "handler end > code size",
            URT::VER_E_FLT_GTEQ_CS => "Description is missing",
            URT::VER_E_TRY_START => "Try starts in the middle of an instruction.",
            URT::VER_E_HND_START => "Handler starts in the middle of an instruction.",
            URT::VER_E_FLT_START => "Description is missing",
            URT::VER_E_TRY_OVERLAP => "Try block overlap with another block.",
            URT::VER_E_TRY_EQ_HND_FIL => "Branch back when this is uninitialized.",
            URT::VER_E_TRY_SHARE_FIN_FAL => "ldftn and ldvirtftn not allowed on .ctor.",
            URT::VER_E_HND_OVERLAP => "Non-compatible types on the stack.",
            URT::VER_E_HND_EQ => "Unexpected type on the stack.",
            URT::VER_E_FIL_OVERLAP => "Missing stack slot for exception.",
            URT::VER_E_FIL_EQ => "Stack overflow.",
            URT::VER_E_FIL_CONT_TRY => "Filter contains try.",
            URT::VER_E_FIL_CONT_HND => "Filter contains handler.",
            URT::VER_E_FIL_CONT_FIL => "Nested filters.",
            URT::VER_E_FIL_GTEQ_CS => "filter >= code size",
            URT::VER_E_FIL_START => "Filter starts in the middle of an instruction.",
            URT::VER_E_FALLTHRU_EXCEP => "fallthru the end of an exception block",
            URT::VER_E_FALLTHRU_INTO_HND => "fallthru into an exception handler",
            URT::VER_E_FALLTHRU_INTO_FIL => "fallthru into an exception filter",
            URT::VER_E_LEAVE => "Leave from outside a try or catch block.",
            URT::VER_E_RETHROW => "Rethrow from outside a catch handler.",
            URT::VER_E_ENDFINALLY => "Expected pointer to function on the stack.",
            URT::VER_E_ENDFILTER => "Expected single dimension array on the stack.",
            URT::VER_E_ENDFILTER_MISSING => "Expected value type instance on the stack.",
            URT::VER_E_BR_INTO_TRY => "Expected address of value type on the stack.",
            URT::VER_E_BR_INTO_HND => "Unexpected value type instance on the stack.",
            URT::VER_E_BR_INTO_FIL => "Local variable is unusable at this point.",
            URT::VER_E_BR_OUTOF_TRY => "Branch out of try block.",
            URT::VER_E_BR_OUTOF_HND => "Branch out of exception handler block.",
            URT::VER_E_BR_OUTOF_FIL => "Branch out of exception filter block.",
            URT::VER_E_BR_OUTOF_FIN => "Branch out of finally block.",
            URT::VER_E_RET_FROM_TRY => "Return out of try block.",
            URT::VER_E_RET_FROM_HND => "Return out of exception handler block.",
            URT::VER_E_RET_FROM_FIL => "Return out of exception filter block.",
            URT::VER_E_BAD_JMP_TARGET => "jmp / exception into the middle of an instruction.",
            URT::VER_E_PATH_LOC => "Non-compatible types depending on path.",
            URT::VER_E_PATH_THIS => "Init state for this differs depending on path.",
            URT::VER_E_PATH_STACK => "Stack must be empty on return from a void function.",
            URT::VER_E_PATH_STACK_DEPTH => "Return value missing on the stack.",
            URT::VER_E_THIS => "Stack must contain only the return value.",
            URT::VER_E_THIS_UNINIT_EXCEP => "Return uninitialized data.",
            URT::VER_E_THIS_UNINIT_STORE => "Illegal array access.",
            URT::VER_E_THIS_UNINIT_RET => "Store non Object type into Object array.",
            URT::VER_E_THIS_UNINIT_V_RET => "Return from .ctor before all fields are initialized.",
            URT::VER_E_THIS_UNINIT_BR => "Branch back when this is uninitialized.",
            URT::VER_E_LDFTN_CTOR => "ldftn and ldvirtftn not allowed on .ctor.",
            URT::VER_E_STACK_NOT_EQ => "Non-compatible types on the stack.",
            URT::VER_E_STACK_UNEXPECTED => "Unexpected type on the stack.",
            URT::VER_E_STACK_EXCEPTION => "Missing stack slot for exception.",
            URT::VER_E_STACK_OVERFLOW => "Stack overflow.",
            URT::VER_E_STACK_UNDERFLOW => "Stack underflow.",
            URT::VER_E_STACK_EMPTY => "Stack empty.",
            URT::VER_E_STACK_UNINIT => "Uninitialized item on stack.",
            URT::VER_E_STACK_I_I4_I8 => "Address of not allowed for this item.",
            URT::VER_E_STACK_R_R4_R8 => "Address of not allowed for ByRef.",
            URT::VER_E_STACK_NO_R_I8 => "Address of not allowed for literal field.",
            URT::VER_E_STACK_NUMERIC => "Cannot change initonly field outside its .ctor.",
            URT::VER_E_STACK_OBJREF => "Cannot throw this object.",
            URT::VER_E_STACK_P_OBJREF => "Callvirt on a value type method.",
            URT::VER_E_STACK_BYREF => "Expected ByRef on the stack.",
            URT::VER_E_STACK_METHOD => "Expected pointer to function on the stack.",
            URT::VER_E_STACK_ARRAY_SD => "Expected single dimension array on the stack.",
            URT::VER_E_STACK_VALCLASS => "Expected value type instance on the stack.",
            URT::VER_E_STACK_P_VALCLASS => "Expected address of value type on the stack.",
            URT::VER_E_STACK_NO_VALCLASS => "Unexpected value type instance on the stack.",
            URT::VER_E_LOC_DEAD => "Local variable is unusable at this point.",
            URT::VER_E_LOC_NUM => "Unrecognized local variable number.",
            URT::VER_E_ARG_NUM => "Unrecognized argument number.",
            URT::VER_E_TOKEN_RESOLVE => "Unable to resolve token.",
            URT::VER_E_TOKEN_TYPE => "ELEMENT_TYPE_PTR cannot be verified.",
            URT::VER_E_TOKEN_TYPE_MEMBER => "Unexpected vararg.",
            URT::VER_E_TOKEN_TYPE_FIELD => "Unexpected Void.",
            URT::VER_E_TOKEN_TYPE_SIG => "ByRef of ByRef",
            URT::VER_E_UNVERIFIABLE => "Instruction cannot be verified.",
            URT::VER_E_LDSTR_OPERAND => "Code size is zero.",
            URT::VER_E_RET_PTR_TO_STACK => "Return type is ByRef, TypedReference, ArgHandle, or ArgIterator.",
            URT::VER_E_RET_VOID => "Stack must be empty on return from a void function.",
            URT::VER_E_RET_MISSING => "Return value missing on the stack.",
            URT::VER_E_RET_EMPTY => "Stack must contain only the return value.",
            URT::VER_E_RET_UNINIT => "Return uninitialized data.",
            URT::VER_E_ARRAY_ACCESS => "Illegal array access.",
            URT::VER_E_ARRAY_V_STORE => "Store non Object type into Object array.",
            URT::VER_E_ARRAY_SD => "Expected single dimension array.",
            URT::VER_E_ARRAY_SD_PTR => "Expected single dimension array of pointer types.",
            URT::VER_E_ARRAY_FIELD => "Array field access is denied.",
            URT::VER_E_ARGLIST => "Lexical nesting.",
            URT::VER_E_VALCLASS => "Missing ldsfld, stsfld, ldind, stind, ldfld, stfld, ldobj, stobj, initblk or cpblk.",
            URT::VER_E_METHOD_ACCESS => "Missing ldind, stind, ldfld, stfld, ldobj, stobj, initblk or cpblk.",
            URT::VER_E_FIELD_ACCESS => "Innermost exception blocks should be declared first.",
            URT::VER_E_DEAD => "Calli not allowed on virtual methods.",
            URT::VER_E_FIELD_STATIC => "Call not allowed on abstract methods.",
            URT::VER_E_FIELD_NO_STATIC => "Expected non-static field.",
            URT::VER_E_ADDR => "Address of not allowed for this item.",
            URT::VER_E_ADDR_BYREF => "Address of not allowed for ByRef.",
            URT::VER_E_ADDR_LITERAL => "Address of not allowed for literal field.",
            URT::VER_E_INITONLY => "Cannot change initonly field outside its .ctor.",
            URT::VER_E_THROW => "Cannot throw this object.",
            URT::VER_E_CALLVIRT_VALCLASS => "Callvirt on a value type method.",
            URT::VER_E_CALL_SIG => "Call signature mismatch.",
            URT::VER_E_CALL_STATIC => "Static function expected.",
            URT::VER_E_CTOR => ".ctor expected.",
            URT::VER_E_CTOR_VIRT => "Box operation on TypedReference, ArgHandle, or ArgIterator.",
            URT::VER_E_CTOR_OR_SUPER => "ByRef of TypedReference, ArgHandle, or ArgIterator.",
            URT::VER_E_CTOR_MUL_INIT => "Array of TypedReference, ArgHandle, or ArgIterator.",
            URT::VER_E_SIG => "Stack not empty when leaving an exception filter.",
            URT::VER_E_SIG_ARRAY => "Unrecognized delegate .ctor signature; expected I.",
            URT::VER_E_SIG_ARRAY_PTR => "Unrecognized delegate .ctor signature; expected Object.",
            URT::VER_E_SIG_ARRAY_BYREF => "Array of ELEMENT_TYPE_BYREF or ELEMENT_TYPE_TYPEDBYREF.",
            URT::VER_E_SIG_ELEM_PTR => "ELEMENT_TYPE_PTR cannot be verified.",
            URT::VER_E_SIG_VARARG => "Unexpected vararg.",
            URT::VER_E_SIG_VOID => "Unexpected Void.",
            URT::VER_E_SIG_BYREF_BYREF => "ByRef of ByRef",
            URT::VER_E_CODE_SIZE_ZERO => "Code size is zero.",
            URT::VER_E_BAD_VARARG => "Unrecognized use of vararg.",
            URT::VER_E_TAIL_CALL => "Missing call, callvirt or calli.",
            URT::VER_E_TAIL_BYREF => "Cannot pass ByRef to a tail call.",
            URT::VER_E_TAIL_RET => "Expected address of value type, ObjRef type or variable type on the stack.",
            URT::VER_E_TAIL_RET_VOID => "Unrecognized type parameter of enclosing class.",
            URT::VER_E_TAIL_RET_TYPE => "Unrecognized type parameter of enclosing method.",
            URT::VER_E_TAIL_STACK_EMPTY => "Unrecognized type argument of referenced class instantiation.",
            URT::VER_E_METHOD_END => "Unrecognized type argument of referenced method instantiation.",
            URT::VER_E_BAD_BRANCH => "Cannot resolve generic type.",
            URT::VER_E_FIN_OVERLAP => "Invalid field token in FieldRVA record.",
            URT::VER_E_LEXICAL_NESTING => "Duplicate RVA in FieldRVA record.",
            URT::VER_E_VOLATILE => "Duplicate field in FieldRVA record.",
            URT::VER_E_UNALIGNED => "Bad token as entry point in CLR header.",
            URT::VER_E_INNERMOST_FIRST => "Entry point in CLR header is a token of instance method.",
            URT::VER_E_CALLI_VIRTUAL => "Enum has non-integral underlying type.",
            URT::VER_E_CALL_ABSTRACT => "Method has bogus RVA.",
            URT::VER_E_STACK_UNEXP_ARRAY => "Literal field has no const value.",
            URT::VER_E_NOT_IN_GC_HEAP => "Class implementing an interface does not implement one of methods.",
            URT::VER_E_TRY_N_EMPTY_STACK => "CA has invalid owner.",
            URT::VER_E_DLGT_CTOR => "Method signature is generic but is missing its arity.",
            URT::VER_E_DLGT_BB => "Method signature is generic but its arity is zero.",
            URT::VER_E_DLGT_PATTERN => "Signature has generic type instantiated at arity 0.",
            URT::VER_E_DLGT_LDFTN => "MethodSpec signature has arity 0.",
            URT::VER_E_FTN_ABSTRACT => "MethodDef signature has arity n but owns m GenericParams.",
            URT::VER_E_SIG_C_VC => "Entry point in CLR header is the token for a method in a generic type.",
            URT::VER_E_SIG_VC_C => "Method has invalid header.",
            URT::VER_E_BOX_PTR_TO_STACK => "Entry point has more than one argument.",
            URT::VER_E_SIG_BYREF_TB_AH => "Entry point has bad return type.",
            URT::VER_E_SIG_ARRAY_TB_AH => "Entry point has bad argument.",
            URT::VER_E_ENDFILTER_STACK => "Illegal 'void' in signature.",
            URT::VER_E_DLGT_SIG_I => "Multiple implementation of method.",
            URT::VER_E_DLGT_SIG_O => "GenericParam name is NULL.",
            URT::VER_E_RA_PTR_TO_STACK => "GenericParam has nil owner.",
            URT::VER_E_CATCH_VALUE_TYPE => "GenericParam has duplicate by owner and name.",
            URT::VER_E_CATCH_BYREF => "GenericParam has duplicate by owner and number.",
            URT::VER_E_FIL_PRECEED_HND => "Warning: Class does not implement interface method in this module.",
            URT::VER_E_LDVIRTFTN_STATIC => "TypeLib export: attempted to export an Assembly imported from a TLB.",
            URT::VER_E_CALLVIRT_STATIC => "Callvirt on static",
            URT::VER_E_INITLOCALS => "Initlocals must be set for verifiable methods with one or more local variables.",
            URT::VER_E_BR_TO_EXCEPTION => "Branch or leave to the beginning of a catch/filter handler",
            URT::VER_E_CALL_CTOR => "Call to .ctor only allowed to initialize this pointer from within a .ctor. Try newobj.",
            URT::VER_E_VALCLASS_OBJREF_VAR => "GenericParamConstraint has nil owner.",
            URT::VER_E_STACK_P_VALCLASS_OBJREF_VAR => "GenericParamConstraint has duplicate by owner and constraint.",
            URT::VER_E_SIG_VAR_PARAM => "GenericParamConstraint is non-contiguous with preceeding constraints for same owner.",
            URT::VER_E_SIG_MVAR_PARAM => "MethodSpec has nil method.",
            URT::VER_E_SIG_VAR_ARG => "MethodSpec has duplicate based on method and instantiation.",
            URT::VER_E_SIG_MVAR_ARG => "MethodSpec signature has invalid calling convention.",
            URT::VER_E_SIG_GENERICINST => "MethodSpec signature is missing arity specification.",
            URT::VER_E_SIG_METHOD_INST => "MethodSpec signature is missing type argument.",
            URT::VER_E_SIG_METHOD_PARENT_INST => "MethodSpec arity of generic method and instantiation do not match.",
            URT::VER_E_SIG_FIELD_PARENT_INST => "MethodSpec method is not generic.",
            URT::VER_E_CALLCONV_NOT_GENERICINST => "Unrecognized calling convention for an instantiated generic method.",
            URT::VER_E_TOKEN_BAD_METHOD_SPEC => "Unrecognized generic method in method instantiation.",
            URT::VER_E_BAD_READONLY_PREFIX => "Missing ldelema or call following readonly. prefix.",
            URT::VER_E_BAD_CONSTRAINED_PREFIX => "Missing callvirt following constrained. prefix.",
            URT::VER_E_CIRCULAR_VAR_CONSTRAINTS => "Method parent has circular class type parameter constraints.",
            URT::VER_E_CIRCULAR_MVAR_CONSTRAINTS => "Method has circular method type parameter constraints.",
            URT::VER_E_UNSATISFIED_METHOD_INST => "Entry point in CLR header is the token for a generic method.",
            URT::VER_E_UNSATISFIED_METHOD_PARENT_INST => "Method signature is generic but is missing its arity.",
            URT::VER_E_UNSATISFIED_FIELD_PARENT_INST => "Method signature is generic but its arity is zero.",
            URT::VER_E_UNSATISFIED_BOX_OPERAND => "Signature has generic type instantiated at arity 0.",
            URT::VER_E_CONSTRAINED_CALL_WITH_NON_BYREF_THIS => "MethodSpec signature has arity 0.",
            URT::VER_E_CONSTRAINED_OF_NON_VARIABLE_TYPE => "MethodDef signature has arity n but owns m GenericParams.",
            URT::VER_E_READONLY_UNEXPECTED_CALLEE => "Entry point in CLR header is the token for a method in a generic type.",
            URT::VER_E_READONLY_ILLEGAL_WRITE => "MethodImpl overrides non-generic method with generic method.",
            URT::VER_E_READONLY_IN_MKREFANY => "MethodImpl overrides non-generic method with generic method.",
            URT::VER_E_UNALIGNED_ALIGNMENT => "MethodImpl overrides generic method of arity n with generic method of arity m.",
            URT::VER_E_TAILCALL_INSIDE_EH => "The tail.call (or calli or callvirt) instruction cannot be used to transfer control out of a try, filter, catch, or finally block.",
            URT::VER_E_BACKWARD_BRANCH => "Stack height at all points must be determinable in a single forward scan of IL.",
            URT::VER_E_CALL_TO_VTYPE_BASE => "Call to base type of valuetype.",
            URT::VER_E_NEWOBJ_OF_ABSTRACT_CLASS => "Cannot construct an instance of abstract class.",
            URT::VER_E_UNMANAGED_POINTER => "Unmanaged pointers are not a verifiable type.",
            URT::VER_E_LDFTN_NON_FINAL_VIRTUAL => "Cannot LDFTN a non-final virtual method.",
            URT::VER_E_FIELD_OVERLAP => "Signature has token following ELEMENT_TYPE_CLASS (_VALUETYPE) that is not a TypeDef or TypeRef.",
            URT::VER_E_THIS_MISMATCH => "Warning: Class does not implement interface method in this module.",
            URT::VER_E_STACK_I_I4 => "TypeLib export: attempted to export an Assembly imported from a TLB.",
            URT::VER_E_BAD_PE => "Unverifiable PE Header/native stub.",
            URT::VER_E_BAD_MD => "Unrecognized metadata, unable to verify IL.",
            URT::VER_E_BAD_APPDOMAIN => "Unrecognized appdomain pointer.",
            URT::VER_E_TYPELOAD => "Type load failed.",
            URT::VER_E_PE_LOAD => "Module load failed.",
            URT::VER_E_WRITE_RVA_STATIC => "Cannot modify an imaged based (RVA) static",
            URT::COR_E_SqlException => "Description is missing",
            URT::COR_E_Data => "Description is missing",
            URT::COR_E_DataDeletedRowInaccessible => "Description is missing",
            URT::COR_E_DataDuplicateName => "Description is missing",
            URT::COR_E_DataInRowChangingEvent => "Description is missing",
            URT::COR_E_DataInvalidConstraint => "Description is missing",
            URT::COR_E_DataMissingPrimaryKey => "Description is missing",
            URT::COR_E_DataNoNullAllowed => "Description is missing",
            URT::COR_E_DataReadOnly => "Description is missing",
            URT::COR_E_DataRowNotInTable => "Description is missing",
            URT::COR_E_DataVersionNotFound => "Description is missing",
            URT::COR_E_DataConstraint => "Description is missing",
            URT::COR_E_StrongTyping => "Description is missing",
            URT::COR_E_SqlType => "Description is missing",
            URT::COR_E_SqlNullValue => "Description is missing",
            URT::COR_E_SqlTruncate => "Description is missing",
            URT::COR_E_AdapterMapping => "Description is missing",
            URT::COR_E_DataAdapter => "Description is missing",
            URT::COR_E_DBConcurrency => "Description is missing",
            URT::COR_E_OperationAborted => "Description is missing",
            URT::COR_E_InvalidUdt => "Description is missing",
            URT::COR_E_OdbcException => "Description is missing",
            URT::COR_E_OracleException => "Description is missing",
            URT::COR_E_Xml => "Description is missing",
            URT::COR_E_XmlSchema => "Description is missing",
            URT::COR_E_XmlXslt => "Description is missing",
            URT::COR_E_XmlXPath => "Description is missing",
            URT::COR_E_XmlQuery => "Description is missing",
            URT::VLDTR_E_IFACE_NOTIFACE => "Interface in InterfaceImpl is not marked tdInterface",
            URT::TLBX_E_CIRCULAR_EXPORT2 => "Interface in InterfaceImpl is not marked tdInterface",
            URT::VLDTR_E_FD_RVAHASNORVA => "Field marked fdHasFieldRVA but has no RVA rec",
            URT::VLDTR_E_FD_RVAHASZERORVA => "Field marked fdHasFieldRVA has RVA =0",
            URT::VLDTR_E_MD_RVAANDIMPLMAP => "Method has both RVA !=0 and ImplMap",
            URT::VLDTR_E_TD_EXTRAFLAGS => "TypeDef has extraneous bits in flags",
            URT::VLDTR_E_TD_EXTENDSITSELF => "TypeDef extends itself",
            URT::VLDTR_E_TD_SYSVTNOTEXTOBJ => "System.ValueType does not extend System.Object",
            URT::VLDTR_E_TD_EXTTYPESPEC => "Class extends TypeSpec (warning)",
            URT::VLDTR_E_TD_VTNOSIZE => "Value Class has zero size",
            URT::VLDTR_E_TD_IFACESEALED => "Interface is sealed",
            URT::VLDTR_E_NC_BADNESTED => "Bad 'nested' token in NestedClass",
            URT::VLDTR_E_NC_BADENCLOSER => "Bad 'enclosing' token in NestedClass",
            URT::VLDTR_E_NC_DUP => "Duplicate NestedClass record",
            URT::VLDTR_E_NC_DUPENCLOSER => "Duplicate NestedClass with different encloser",
            URT::VLDTR_E_FRVA_ZERORVA => "RVA=0 in FieldRVA record",
            URT::VLDTR_E_FRVA_BADFIELD => "Invalid field token in FieldRVA record",
            URT::VLDTR_E_FRVA_DUPRVA => "Duplicate RVA in FieldRVA record",
            URT::VLDTR_E_FRVA_DUPFIELD => "Duplicate field in FieldRVA record",
            URT::VLDTR_E_EP_BADTOKEN => "Bad token as entry point in CLR header",
            URT::VLDTR_E_EP_INSTANCE => "Entry point in CLR header is a token of instance method",
            URT::VLDTR_E_TD_ENUMFLDBADTYPE => "Enum has non-integral underlying type",
            URT::VLDTR_E_MD_BADRVA => "Method has bogus RVA",
            URT::VLDTR_E_FD_LITERALNODEFAULT => "Literal field has no const value",
            URT::VLDTR_E_IFACE_METHNOTIMPL => "Class implementing an interface doesn't impl.one of methods",
            URT::VLDTR_E_CA_BADPARENT => "CA has invalid owner",
            URT::VLDTR_E_CA_BADTYPE => "CA has invalid type",
            URT::VLDTR_E_CA_NOTCTOR => "CA type is not .ctor",
            URT::VLDTR_E_CA_BADSIG => "CA type has bad signature",
            URT::VLDTR_E_CA_NOSIG => "CA type has no signature",
            URT::VLDTR_E_CA_BADPROLOG => "CA blob has bad prolog (not 0x01 0x00)",
            URT::VLDTR_E_MD_BADLOCALSIGTOK => "Method has invalid LocalSig token",
            URT::VLDTR_E_MD_BADHEADER => "Method has invalid header",
            URT::VLDTR_E_EP_TOOMANYARGS => "Entry point has more than one arg",
            URT::VLDTR_E_EP_BADRET => "Entry point has bad return type",
            URT::VLDTR_E_EP_BADARG => "Entry point has bad argument",
            URT::VLDTR_E_SIG_BADVOID => "Illegal 'void' in signature",
            URT::VLDTR_E_IFACE_METHMULTIMPL => "Multiple implementation of method",
            URT::VLDTR_E_GP_NAMENULL => "GenericParam name is NULL",
            URT::VLDTR_E_GP_OWNERNIL => "GenericParam has nil owner.",
            URT::VLDTR_E_GP_DUPNAME => "GenericParam has duplicate by owner and name.",
            URT::VLDTR_E_GP_DUPNUMBER => "GenericParam has duplicate by owner and number.",
            URT::VLDTR_E_GP_NONSEQ_BY_OWNER => "GenericParam is non sequential by owner",
            URT::VLDTR_E_GP_NONSEQ_BY_NUMBER => "GenericParam is non sequential by number",
            URT::VLDTR_E_GP_UNEXPECTED_OWNER_FOR_VARIANT_VAR => "GenericParam has variance but its owner is not an interface or delegate",
            URT::VLDTR_E_GP_ILLEGAL_VARIANT_MVAR => "GenericParam is a method type parameter and must be non-variant",
            URT::VLDTR_E_GP_ILLEGAL_VARIANCE_FLAGS => "GenericParam has illegal value for variance flags",
            URT::VLDTR_E_GP_REFANDVALUETYPE => "GenericParam has incompatible special constraints reference type and valuetype",
            URT::VLDTR_E_GPC_OWNERNIL => "GenericParamConstraint has nil owner",
            URT::VLDTR_E_GPC_DUP => "GenericParamConstraint has duplicate by owner and constraint",
            URT::VLDTR_E_GPC_NONCONTIGUOUS => "GenericParamConstraint is non-contiguous with preceeding constraints for same owner",
            URT::VLDTR_E_MS_METHODNIL => "MethodSpec has nil method",
            URT::VLDTR_E_MS_DUP => "MethodSpec has duplicate based own method and instantiation",
            URT::VLDTR_E_MS_BADCALLINGCONV => "MethodSpec signature has invalid calling convention",
            URT::VLDTR_E_MS_MISSARITY => "MethodSpec signature is missing arity specification",
            URT::VLDTR_E_MS_MISSARG => "MethodSpec signature is missing type argument",
            URT::VLDTR_E_MS_ARITYMISMATCH => "MethodSpec arity of generic method and instantiation do not match",
            URT::VLDTR_E_MS_METHODNOTGENERIC => "MethodSpec method is not generic",
            URT::VLDTR_E_SIG_MISSARITY => "Signature missing arity of instantiated generic type",
            URT::VLDTR_E_SIG_ARITYMISMATCH => "Signature has generic type of arity instantiated at different arity",
            URT::VLDTR_E_MD_GENERIC_CCTOR => "Method cannot be both generic and a class constructor",
            URT::VLDTR_E_MD_GENERIC_CTOR => "Method cannot be both generic and an instance constructor",
            URT::VLDTR_E_MD_GENERIC_IMPORT => "Method cannot be both generic and defined on an imported type",
            URT::VLDTR_E_MD_GENERIC_BADCALLCONV => "Method cannot be both generic and have non-default calling convention",
            URT::VLDTR_E_MD_GENERIC_GLOBAL => "Method cannot be both generic and global",
            URT::VLDTR_E_EP_GENERIC_METHOD => "Entry point in CLR header is the token for a generic method",
            URT::VLDTR_E_MD_MISSARITY => "Method signature is generic but is missing its arity",
            URT::VLDTR_E_MD_ARITYZERO => "Method signature is generic but its arity is zero",
            URT::VLDTR_E_SIG_ARITYZERO => "Signature has generic type instantiated at arity 0",
            URT::VLDTR_E_MS_ARITYZERO => "MethodSpec signature has arity 0",
            URT::VLDTR_E_MD_GPMISMATCH => "MethodDef signature has arity n but owns m GenericParams",
            URT::VLDTR_E_EP_GENERIC_TYPE => "Entry point in CLR header is the token for a method in a generic type",
            URT::VLDTR_E_MI_DECLNOTGENERIC => "MethodImpl overrides non-generic method with generic method",
            URT::VLDTR_E_MI_IMPLNOTGENERIC => "MethodImpl overrides non-generic method with generic method",
            URT::VLDTR_E_MI_ARITYMISMATCH => "MethodImpl overrides generic method of arity n with generic method of arity m",
            URT::VLDTR_E_TD_EXTBADTYPESPEC => "TypeDef extends a TypeSpec that is not an instantiated type",
            URT::VLDTR_E_SIG_BYREFINST => "Signature has type instantiated at byref at offset i",
            URT::VLDTR_E_MS_BYREFINST => "Signature has type instantiated at byref at offset i",
            URT::VLDTR_E_TS_EMPTY => "TypeSpec has empty signature",
            URT::VLDTR_E_TS_HASSENTINALS => "TypeSpec has signature containing one or more sentinals",
            URT::VLDTR_E_TD_GENERICHASEXPLAYOUT => "TypeDef is generic but has explicit layout",
            URT::VLDTR_E_SIG_BADTOKTYPE => "Signature has token following ELEMENT_TYPE_CLASS (_VALUETYPE) that is not a TypeDef or TypeRef",
            URT::VLDTR_E_IFACE_METHNOTIMPLTHISMOD => "Warn:Class doesn't implement interface method in this module",
            URT::CORDBG_E_THREAD_NOT_SCHEDULED => "Thread is not scheduled. Thus we may not have OSThreadId, handle, or context",
            URT::CORDBG_E_HANDLE_HAS_BEEN_DISPOSED => "Handle has been disposed.",
            URT::CORDBG_E_NONINTERCEPTABLE_EXCEPTION => "Cant intercept this exception.",
            URT::CORDBG_E_CANT_UNWIND_ABOVE_CALLBACK => "When intercepting an exception, cannot intercept above the current frame.",
            URT::CORDBG_E_INTERCEPT_FRAME_ALREADY_SET => "The intercept frame for this exception has already been set.",
            URT::CORDBG_E_NO_NATIVE_PATCH_AT_ADDR => "There's no native patch at the given address.",
            URT::CORDBG_E_MUST_BE_INTEROP_DEBUGGING => "This API is only allowed when interop debugging.",
            URT::CORDBG_E_NATIVE_PATCH_ALREADY_AT_ADDR => "There's already a native patch at the address.",
            URT::CORDBG_E_TIMEOUT => "A wait timed out, likely an indication of deadlock.",
            URT::CORDBG_E_CANT_CALL_ON_THIS_THREAD => "Can't use the API on this thread.",
            URT::CORDBG_E_ENC_INFOLESS_METHOD => "Method was not JIT'd in EnC mode.",
            URT::CORDBG_E_ENC_NESTED_HANLDERS => "Frame cannot be updated due to change in max nesting of handlers.",
            URT::CORDBG_E_ENC_IN_FUNCLET => "Method is in a callable handler/filter. Cannot increase stack.",
            URT::CORDBG_E_ENC_LOCALLOC => "Frame cannot be updated due to localloc.",
            URT::CORDBG_E_ENC_EDIT_NOT_SUPPORTED => "Attempt to perform unsupported edit.",
            URT::CORDBG_E_FEABORT_DELAYED_UNTIL_THREAD_RESUMED => "Attempt to func eval abort on a suspended thread.",
            URT::CORDBG_E_NOTREADY => "The LS is not in a good spot to perform the requested operation.",
            URT::CORDBG_E_CANNOT_RESOLVE_ASSEMBLY => "We failed to resolve assembly given an AssemblyRef token. Assembly may be not loaded yet or not a valid token.",
            URT::CORDBG_E_MUST_BE_IN_LOAD_MODULE => "Must be in context of LoadModule callback to perform requested operation.",
            URT::CORDBG_E_CANNOT_BE_ON_ATTACH => "Requested operation cannot be performed during an attach operation.",
            URT::CORDBG_E_NGEN_NOT_SUPPORTED => "NGEN must be supported to perform the requested operation.",
            URT::CORDBG_E_ILLEGAL_SHUTDOWN_ORDER => "Trying to shutdown out of order.",
            URT::CORDBG_E_CANNOT_DEBUG_FIBER_PROCESS => "Debugging fiber mode managed process is not supported. For Whidbey, we don't support debugging fiber mode managed process",
            URT::CORDBG_E_MUST_BE_IN_CREATE_PROCESS => "Must be in context of CreateProcess callback to perform requested operation",
            URT::CORDBG_E_DETACH_FAILED_OUTSTANDING_EVALS => "All outstanding func-evals have not completed, detaching is not allowed at this time.",
            URT::CORDBG_E_DETACH_FAILED_OUTSTANDING_STEPPERS => "All outstanding steppers have not been closed, detaching is not allowed at this time.",
            URT::CORDBG_E_CANT_INTEROP_STEP_OUT => "Can't have an ICorDebugStepper do a native step-out.",
            URT::CORDBG_E_DETACH_FAILED_OUTSTANDING_BREAKPOINTS => "All outstanding breakpoints have not been closed, detaching is not allowed at this time.",
            URT::CORDBG_E_ILLEGAL_IN_STACK_OVERFLOW => "The operation is illegal because of a stack overflow.",
            URT::CORDBG_E_ILLEGAL_AT_GC_UNSAFE_POINT => "The operation failed because it is a GC unsafe point.",
            URT::CORDBG_E_ILLEGAL_IN_PROLOG => "The operation failed because the thread is in the prolog.",
            URT::CORDBG_E_ILLEGAL_IN_NATIVE_CODE => "The operation failed because the thread is in native code.",
            URT::CORDBG_E_ILLEGAL_IN_OPTIMIZED_CODE => "The operation failed because the thread is in optimized code.",
            URT::CORDBG_E_MINIDUMP_UNSUPPORTED => "The information requested is not supported by minidumps.",
            URT::CORDBG_E_APPDOMAIN_MISMATCH => "A supplied object or type belongs to the wrong AppDomain.",
            URT::CORDBG_E_CONTEXT_UNVAILABLE => "The thread's context is not available.",
            URT::CORDBG_E_UNCOMPATIBLE_PLATFORMS => "The operation failed because debuggee and debugger are on incompatible platforms.",
            URT::CORDBG_E_DEBUGGING_DISABLED => "The operation failed because the debugging has been disabled",
            URT::CORDBG_E_DETACH_FAILED_ON_ENC => "Detach is illegal after an Edit and Continue on a module.",
            URT::CORDBG_E_CURRENT_EXCEPTION_IS_OUTSIDE_CURRENT_EXECUTION_SCOPE => "Cannot intercept the current exception at the specified frame.",
            URT::CORDBG_E_HELPER_MAY_DEADLOCK => "The debugger helper thread cannot obtain the locks it needs to perform this operation.",
            URT::CORDBG_E_MISSING_METADATA => "The operation failed because the debugger could not get the metadata.",
            URT::CORDBG_E_TARGET_INCONSISTENT => "The debuggee is in a corrupt state.",
            URT::CORDBG_E_DETACH_FAILED_OUTSTANDING_TARGET_RESOURCES => "Detach failed because there are outstanding resources in the target. The debugger is holding resource in the target (such as GC handles, function evaluations, etc). These resources must be released through the appropriate ICorDebug API before detach can succeed.",
            URT::CORDBG_E_TARGET_READONLY => "The debuggee is read-only. The provided ICorDebugDataTarget does not implement ICorDebugMutableDataTarget.",
            URT::CORDBG_E_MISMATCHED_CORWKS_AND_DACWKS_DLLS => "The version of clr.dll in the target does not match the one mscordacwks.dll was built for. A clr/mscordacwks mismatch will cause DAC to fail to initialize in ClrDataAccess::Initialize",
            URT::CORDBG_E_MODULE_LOADED_FROM_DISK => "Symbols are not supplied for modules loaded from disk.",
            URT::CORDBG_E_SYMBOLS_NOT_AVAILABLE => "The application did not supply symbols when it loaded or created this module, or they are not yet available.",
            URT::CORDBG_E_DEBUG_COMPONENT_MISSING => "A debug component is not installed.",
            URT::CORDBG_E_REMOTE_MISMATCHED_CERTS => "Connection authentication failed due to mismatched certificates.",
            URT::CORDBG_E_REMOTE_NETWORK_FAILURE => "Connection failed due to a miscellaneous network error.",
            URT::CORDBG_E_REMOTE_NO_LISTENER => "Connection failed due to no endpoint at remote machine (no proxy configured?).",
            URT::CORDBG_E_REMOTE_UNKNOWN_TARGET => "Connection failed due to inability to locate remote machine.",
            URT::CORDBG_E_REMOTE_INVALID_CONFIG => "Local debugger configuration was missing or invalid.",
            URT::CORDBG_E_REMOTE_MISMATCHED_PROTOCOLS => "Connection failed due to protocol version mismatch between local and remote components.",
            URT::CORDBG_E_LIBRARY_PROVIDER_ERROR => "The ICLRDebuggingLibraryProvider callback returned an error or did not provide a valid handle.",
            URT::CORDBG_E_NOT_CLR => "The module at the base address indicated was not recognized as a CLR.",
            URT::CORDBG_E_MISSING_DATA_TARGET_INTERFACE => "The provided data target does not implement the required interfaces for this version of the runtime.",
            URT::CORDBG_E_UNSUPPORTED_DEBUGGING_MODEL => "This debugging model is unsupported by the specified runtime.",
            URT::CORDBG_E_UNSUPPORTED_FORWARD_COMPAT => "The debugger is not designed to support the version of the CLR the debuggee is using.",
            URT::CORDBG_E_UNSUPPORTED_VERSION_STRUCT => "The version struct has an unrecognized value for wStructVersion.",
            URT::CORDBG_E_READVIRTUAL_FAILURE => "A call into a ReadVirtual implementation returned failure.",
            URT::CORDBG_E_VALUE_POINTS_TO_FUNCTION => "The Debugging API doesn't support dereferencing function pointers.",
            URT::CORDBG_E_CORRUPT_OBJECT => "The address provided does not point to a valid managed object.",
            URT::CORDBG_E_GC_STRUCTURES_INVALID => "The GC heap structures are not in a valid state for traversal.",
            URT::CORDBG_E_INVALID_OPCODE => "The specified IL offset or opcode is not supported for this operation.",
            URT::CORDBG_E_UNSUPPORTED => "The specified action is unsupported by this version of the runtime.",
            URT::CORDBG_E_MISSING_DEBUGGER_EXPORTS => "The debuggee memory space does not have the expected debugging export table.",
            URT::PEFMT_E_NO_CONTENTS => "File is empty.",
            URT::PEFMT_E_NO_NTHEADERS => "File has no NT headers",
            URT::PEFMT_E_64BIT => "File is PE32+.",
            URT::PEFMT_E_NO_CORHEADER => "File has no COR header.",
            URT::PEFMT_E_NOT_ILONLY => "Flag IL_ONLY not set.",
            URT::PEFMT_E_IMPORT_DLLS => "Bad import DLLs.",
            URT::PEFMT_E_EXE_NOENTRYPOINT => "EXE file has no mgd entry point.",
            URT::PEFMT_E_BASE_RELOCS => "Bad base relocations.",
            URT::PEFMT_E_ENTRYPOINT => "Bad managed entry point.",
            URT::PEFMT_E_ZERO_SIZEOFCODE => "OptHeader.SizeOfCode is set to zero.",
            URT::PEFMT_E_BAD_CORHEADER => "File has invalid COR header.",
            URT::PEFMT_E_32BIT => "File is PE32",
            URT::CLR_OPTSVC_E_CONTROLLER_INTERRUPT => "The operation was interrupted by the CLR Optimization Service controller.",
            URT::NGEN_FAILED_GET_DEPENDENCIES => "Failed to get dependencies for assembly. Service manager failed to get ICorSvcDependencies interface",
            URT::NGEN_FAILED_NATIVE_IMAGE_DELETE => "Failed to delete native image.",
            URT::NGEN_E_TOO_MANY_INTERFACES => "Module contains too many interfaces to successfully compile all methods.",
            URT::NGEN_E_OLDER_RUNTIME => "Requested runtime does not support side-by-side NGen.",
            URT::NGEN_E_WORKER_UNEXPECTED_EXIT => "Worker exited unexpectedly during startup. An NGen worker process exited before it could be reached via DCOM.",
            URT::NGEN_E_WORKER_UNEXPECTED_SYNC => "Failed to synchronize with worker during startup. Synchronizing with an NGen worker process returned an unexpected result.",
            URT::NGEN_E_SYS_ASM_NI_MISSING => "NGen cannot proceed because Mscorlib.dll does not have a native image. Compiling any assembly other than mscorlib in the absence of mscorlib.ni.dll is not allowed.",
            URT::CLDB_E_INTERNALERROR => "Description is missing",
        }
    }

    pub fn from_name(name: &str) -> URT {
        return match name {
            "VLDTR_S_WRN" => URT::VLDTR_S_WRN,
            "VLDTR_S_ERR" => URT::VLDTR_S_ERR,
            "VLDTR_S_WRNERR" => URT::VLDTR_S_WRNERR,
            "CORDBG_S_BAD_START_SEQUENCE_POINT" => URT::CORDBG_S_BAD_START_SEQUENCE_POINT,
            "CORDBG_S_BAD_END_SEQUENCE_POINT" => URT::CORDBG_S_BAD_END_SEQUENCE_POINT,
            "CORDBG_S_INSUFFICIENT_INFO_FOR_SET_IP" => URT::CORDBG_S_INSUFFICIENT_INFO_FOR_SET_IP,
            "CORDBG_S_FUNC_EVAL_HAS_NO_RESULT" => URT::CORDBG_S_FUNC_EVAL_HAS_NO_RESULT,
            "CORDBG_S_VALUE_POINTS_TO_VOID" => URT::CORDBG_S_VALUE_POINTS_TO_VOID,
            "CORDBG_S_FUNC_EVAL_ABORTED" => URT::CORDBG_S_FUNC_EVAL_ABORTED,
            "CEE_E_ENTRYPOINT" => URT::CEE_E_ENTRYPOINT,
            "CEE_E_CVTRES_NOT_FOUND" => URT::CEE_E_CVTRES_NOT_FOUND,
            "MSEE_E_LOADLIBFAILED" => URT::MSEE_E_LOADLIBFAILED,
            "MSEE_E_GETPROCFAILED" => URT::MSEE_E_GETPROCFAILED,
            "MSEE_E_MULTCOPIESLOADED" => URT::MSEE_E_MULTCOPIESLOADED,
            "COR_E_TYPEUNLOADED" => URT::COR_E_TYPEUNLOADED,
            "COR_E_APPDOMAINUNLOADED" => URT::COR_E_APPDOMAINUNLOADED,
            "COR_E_CANNOTUNLOADAPPDOMAIN" => URT::COR_E_CANNOTUNLOADAPPDOMAIN,
            "MSEE_E_ASSEMBLYLOADINPROGRESS" => URT::MSEE_E_ASSEMBLYLOADINPROGRESS,
            "MSEE_E_CANNOTCREATEAPPDOMAIN" => URT::MSEE_E_CANNOTCREATEAPPDOMAIN,
            "COR_E_ASSEMBLYEXPECTED" => URT::COR_E_ASSEMBLYEXPECTED,
            "COR_E_FIXUPSINEXE" => URT::COR_E_FIXUPSINEXE,
            "COR_E_NO_LOADLIBRARY_ALLOWED" => URT::COR_E_NO_LOADLIBRARY_ALLOWED,
            "COR_E_NEWER_RUNTIME" => URT::COR_E_NEWER_RUNTIME,
            "COR_E_CANNOT_SET_POLICY" => URT::COR_E_CANNOT_SET_POLICY,
            "COR_E_CANNOT_SPECIFY_EVIDENCE" => URT::COR_E_CANNOT_SPECIFY_EVIDENCE,
            "COR_E_MULTIMODULEASSEMBLIESDIALLOWED" => URT::COR_E_MULTIMODULEASSEMBLIESDIALLOWED,
            "HOST_E_DEADLOCK" => URT::HOST_E_DEADLOCK,
            "HOST_E_INTERRUPTED" => URT::HOST_E_INTERRUPTED,
            "HOST_E_INVALIDOPERATION" => URT::HOST_E_INVALIDOPERATION,
            "HOST_E_CLRNOTAVAILABLE" => URT::HOST_E_CLRNOTAVAILABLE,
            "HOST_E_TIMEOUT" => URT::HOST_E_TIMEOUT,
            "HOST_E_NOT_OWNER" => URT::HOST_E_NOT_OWNER,
            "HOST_E_ABANDONED" => URT::HOST_E_ABANDONED,
            "HOST_E_EXITPROCESS_THREADABORT" => URT::HOST_E_EXITPROCESS_THREADABORT,
            "HOST_E_EXITPROCESS_ADUNLOAD" => URT::HOST_E_EXITPROCESS_ADUNLOAD,
            "HOST_E_EXITPROCESS_TIMEOUT" => URT::HOST_E_EXITPROCESS_TIMEOUT,
            "HOST_E_EXITPROCESS_OUTOFMEMORY" => URT::HOST_E_EXITPROCESS_OUTOFMEMORY,
            "HOST_E_EXITPROCESS_STACKOVERFLOW" => URT::HOST_E_EXITPROCESS_STACKOVERFLOW,
            "COR_E_MODULE_HASH_CHECK_FAILED" => URT::COR_E_MODULE_HASH_CHECK_FAILED,
            "FUSION_E_REF_DEF_MISMATCH" => URT::FUSION_E_REF_DEF_MISMATCH,
            "FUSION_E_INVALID_PRIVATE_ASM_LOCATION" => URT::FUSION_E_INVALID_PRIVATE_ASM_LOCATION,
            "FUSION_E_ASM_MODULE_MISSING" => URT::FUSION_E_ASM_MODULE_MISSING,
            "FUSION_E_UNEXPECTED_MODULE_FOUND" => URT::FUSION_E_UNEXPECTED_MODULE_FOUND,
            "FUSION_E_PRIVATE_ASM_DISALLOWED" => URT::FUSION_E_PRIVATE_ASM_DISALLOWED,
            "FUSION_E_SIGNATURE_CHECK_FAILED" => URT::FUSION_E_SIGNATURE_CHECK_FAILED,
            "FUSION_E_DATABASE_ERROR" => URT::FUSION_E_DATABASE_ERROR,
            "FUSION_E_INVALID_NAME" => URT::FUSION_E_INVALID_NAME,
            "FUSION_E_CODE_DOWNLOAD_DISABLED" => URT::FUSION_E_CODE_DOWNLOAD_DISABLED,
            "FUSION_E_UNINSTALL_DISALLOWED" => URT::FUSION_E_UNINSTALL_DISALLOWED,
            "FUSION_E_HOST_GAC_ASM_MISMATCH" => URT::FUSION_E_HOST_GAC_ASM_MISMATCH,
            "FUSION_E_LOADFROM_BLOCKED" => URT::FUSION_E_LOADFROM_BLOCKED,
            "FUSION_E_CACHEFILE_FAILED" => URT::FUSION_E_CACHEFILE_FAILED,
            "FUSION_E_APP_DOMAIN_LOCKED" => URT::FUSION_E_APP_DOMAIN_LOCKED,
            "FUSION_E_CONFIGURATION_ERROR" => URT::FUSION_E_CONFIGURATION_ERROR,
            "FUSION_E_MANIFEST_PARSE_ERROR" => URT::FUSION_E_MANIFEST_PARSE_ERROR,
            "FUSION_E_INVALID_ASSEMBLY_REFERENCE" => URT::FUSION_E_INVALID_ASSEMBLY_REFERENCE,
            "COR_E_ASSEMBLY_NOT_EXPECTED" => URT::COR_E_ASSEMBLY_NOT_EXPECTED,
            "COR_E_LOADING_REFERENCE_ASSEMBLY" => URT::COR_E_LOADING_REFERENCE_ASSEMBLY,
            "CLDB_E_FILE_BADREAD" => URT::CLDB_E_FILE_BADREAD,
            "CLDB_E_FILE_BADWRITE" => URT::CLDB_E_FILE_BADWRITE,
            "CLDB_E_FILE_READONLY" => URT::CLDB_E_FILE_READONLY,
            "CLDB_E_NAME_ERROR" => URT::CLDB_E_NAME_ERROR,
            "CLDB_E_TRUNCATION" => URT::CLDB_E_TRUNCATION,
            "CLDB_E_FILE_OLDVER" => URT::CLDB_E_FILE_OLDVER,
            "CLDB_E_RELOCATED" => URT::CLDB_E_RELOCATED,
            "CLDB_E_SMDUPLICATE" => URT::CLDB_E_SMDUPLICATE,
            "CLDB_E_NO_DATA" => URT::CLDB_E_NO_DATA,
            "CLDB_E_READONLY" => URT::CLDB_E_READONLY,
            "CLDB_E_INCOMPATIBLE" => URT::CLDB_E_INCOMPATIBLE,
            "CLDB_E_FILE_CORRUPT" => URT::CLDB_E_FILE_CORRUPT,
            "CLDB_E_SCHEMA_VERNOTFOUND" => URT::CLDB_E_SCHEMA_VERNOTFOUND,
            "CLDB_E_BADUPDATEMODE" => URT::CLDB_E_BADUPDATEMODE,
            "CLDB_E_INDEX_NONULLKEYS" => URT::CLDB_E_INDEX_NONULLKEYS,
            "CLDB_E_INDEX_DUPLICATE" => URT::CLDB_E_INDEX_DUPLICATE,
            "CLDB_E_INDEX_BADTYPE" => URT::CLDB_E_INDEX_BADTYPE,
            "CLDB_E_INDEX_NOTFOUND" => URT::CLDB_E_INDEX_NOTFOUND,
            "CLDB_E_RECORD_NOTFOUND" => URT::CLDB_E_RECORD_NOTFOUND,
            "CLDB_E_RECORD_OVERFLOW" => URT::CLDB_E_RECORD_OVERFLOW,
            "CLDB_E_RECORD_DUPLICATE" => URT::CLDB_E_RECORD_DUPLICATE,
            "CLDB_E_RECORD_PKREQUIRED" => URT::CLDB_E_RECORD_PKREQUIRED,
            "CLDB_E_RECORD_DELETED" => URT::CLDB_E_RECORD_DELETED,
            "CLDB_E_RECORD_OUTOFORDER" => URT::CLDB_E_RECORD_OUTOFORDER,
            "CLDB_E_COLUMN_OVERFLOW" => URT::CLDB_E_COLUMN_OVERFLOW,
            "CLDB_E_COLUMN_READONLY" => URT::CLDB_E_COLUMN_READONLY,
            "CLDB_E_COLUMN_SPECIALCOL" => URT::CLDB_E_COLUMN_SPECIALCOL,
            "CLDB_E_COLUMN_PKNONULLS" => URT::CLDB_E_COLUMN_PKNONULLS,
            "CLDB_E_TABLE_CANTDROP" => URT::CLDB_E_TABLE_CANTDROP,
            "CLDB_E_OBJECT_NOTFOUND" => URT::CLDB_E_OBJECT_NOTFOUND,
            "CLDB_E_OBJECT_COLNOTFOUND" => URT::CLDB_E_OBJECT_COLNOTFOUND,
            "CLDB_E_VECTOR_BADINDEX" => URT::CLDB_E_VECTOR_BADINDEX,
            "CLDB_E_TOO_BIG" => URT::CLDB_E_TOO_BIG,
            "META_E_INVALID_TOKEN_TYPE" => URT::META_E_INVALID_TOKEN_TYPE,
            "TLBX_E_INVALID_TYPEINFO" => URT::TLBX_E_INVALID_TYPEINFO,
            "TLBX_E_INVALID_TYPEINFO_UNNAMED" => URT::TLBX_E_INVALID_TYPEINFO_UNNAMED,
            "TLBX_E_CTX_NESTED" => URT::TLBX_E_CTX_NESTED,
            "TLBX_E_ERROR_MESSAGE" => URT::TLBX_E_ERROR_MESSAGE,
            "TLBX_E_CANT_SAVE" => URT::TLBX_E_CANT_SAVE,
            "TLBX_W_LIBNOTREGISTERED" => URT::TLBX_W_LIBNOTREGISTERED,
            "TLBX_E_CANTLOADLIBRARY" => URT::TLBX_E_CANTLOADLIBRARY,
            "TLBX_E_BAD_VT_TYPE" => URT::TLBX_E_BAD_VT_TYPE,
            "TLBX_E_NO_MSCOREE_TLB" => URT::TLBX_E_NO_MSCOREE_TLB,
            "TLBX_E_BAD_MSCOREE_TLB" => URT::TLBX_E_BAD_MSCOREE_TLB,
            "TLBX_E_TLB_EXCEPTION" => URT::TLBX_E_TLB_EXCEPTION,
            "TLBX_E_MULTIPLE_LCIDS" => URT::TLBX_E_MULTIPLE_LCIDS,
            "TLBX_E_AMBIGUOUS_RETURN" => URT::TLBX_E_AMBIGUOUS_RETURN,
            "TLBX_E_DUPLICATE_TYPE_NAME" => URT::TLBX_E_DUPLICATE_TYPE_NAME,
            "TLBX_I_NONSEQUENTIALSTRUCT" => URT::TLBX_I_NONSEQUENTIALSTRUCT,
            "TLBX_I_RESOLVEREFFAILED" => URT::TLBX_I_RESOLVEREFFAILED,
            "TLBX_E_ASANY" => URT::TLBX_E_ASANY,
            "TLBX_E_INVALIDLCIDPARAM" => URT::TLBX_E_INVALIDLCIDPARAM,
            "TLBX_E_LCIDONDISPONLYITF" => URT::TLBX_E_LCIDONDISPONLYITF,
            "TLBX_E_NONPUBLIC_FIELD" => URT::TLBX_E_NONPUBLIC_FIELD,
            "TLBX_E_BAD_NAMES" => URT::TLBX_E_BAD_NAMES,
            "TLBX_E_GENERICINST_SIGNATURE" => URT::TLBX_E_GENERICINST_SIGNATURE,
            "TLBX_E_GENERICPAR_SIGNATURE" => URT::TLBX_E_GENERICPAR_SIGNATURE,
            "META_E_DUPLICATE" => URT::META_E_DUPLICATE,
            "META_E_GUID_REQUIRED" => URT::META_E_GUID_REQUIRED,
            "META_E_TYPEDEF_MISMATCH" => URT::META_E_TYPEDEF_MISMATCH,
            "META_E_MERGE_COLLISION" => URT::META_E_MERGE_COLLISION,
            "TLBX_E_NO_SAFEHANDLE_ARRAYS" => URT::TLBX_E_NO_SAFEHANDLE_ARRAYS,
            "META_E_METHD_NOT_FOUND" => URT::META_E_METHD_NOT_FOUND,
            "META_E_FIELD_NOT_FOUND" => URT::META_E_FIELD_NOT_FOUND,
            "META_E_PARAM_MISMATCH" => URT::META_E_PARAM_MISMATCH,
            "META_E_BADMETADATA" => URT::META_E_BADMETADATA,
            "META_E_INTFCEIMPL_NOT_FOUND" => URT::META_E_INTFCEIMPL_NOT_FOUND,
            "TLBX_E_NO_CRITICALHANDLE_ARRAYS" => URT::TLBX_E_NO_CRITICALHANDLE_ARRAYS,
            "META_E_CLASS_LAYOUT_INCONSISTENT" => URT::META_E_CLASS_LAYOUT_INCONSISTENT,
            "META_E_FIELD_MARSHAL_NOT_FOUND" => URT::META_E_FIELD_MARSHAL_NOT_FOUND,
            "META_E_METHODSEM_NOT_FOUND" => URT::META_E_METHODSEM_NOT_FOUND,
            "META_E_EVENT_NOT_FOUND" => URT::META_E_EVENT_NOT_FOUND,
            "META_E_PROP_NOT_FOUND" => URT::META_E_PROP_NOT_FOUND,
            "META_E_BAD_SIGNATURE" => URT::META_E_BAD_SIGNATURE,
            "META_E_BAD_INPUT_PARAMETER" => URT::META_E_BAD_INPUT_PARAMETER,
            "META_E_METHDIMPL_INCONSISTENT" => URT::META_E_METHDIMPL_INCONSISTENT,
            "META_E_MD_INCONSISTENCY" => URT::META_E_MD_INCONSISTENCY,
            "META_E_CANNOTRESOLVETYPEREF" => URT::META_E_CANNOTRESOLVETYPEREF,
            "META_E_STRINGSPACE_FULL" => URT::META_E_STRINGSPACE_FULL,
            "META_E_UNEXPECTED_REMAP" => URT::META_E_UNEXPECTED_REMAP,
            "META_E_HAS_UNMARKALL" => URT::META_E_HAS_UNMARKALL,
            "META_E_MUST_CALL_UNMARKALL" => URT::META_E_MUST_CALL_UNMARKALL,
            "META_E_GENERICPARAM_INCONSISTENT" => URT::META_E_GENERICPARAM_INCONSISTENT,
            "META_E_EVENT_COUNTS" => URT::META_E_EVENT_COUNTS,
            "META_E_PROPERTY_COUNTS" => URT::META_E_PROPERTY_COUNTS,
            "META_E_TYPEDEF_MISSING" => URT::META_E_TYPEDEF_MISSING,
            "TLBX_E_CANT_LOAD_MODULE" => URT::TLBX_E_CANT_LOAD_MODULE,
            "TLBX_E_CANT_LOAD_CLASS" => URT::TLBX_E_CANT_LOAD_CLASS,
            "TLBX_E_NULL_MODULE" => URT::TLBX_E_NULL_MODULE,
            "TLBX_E_NO_CLSID_KEY" => URT::TLBX_E_NO_CLSID_KEY,
            "TLBX_E_CIRCULAR_EXPORT" => URT::TLBX_E_CIRCULAR_EXPORT,
            "TLBX_E_CIRCULAR_IMPORT" => URT::TLBX_E_CIRCULAR_IMPORT,
            "TLBX_E_BAD_NATIVETYPE" => URT::TLBX_E_BAD_NATIVETYPE,
            "TLBX_E_BAD_VTABLE" => URT::TLBX_E_BAD_VTABLE,
            "TLBX_E_CRM_NON_STATIC" => URT::TLBX_E_CRM_NON_STATIC,
            "TLBX_E_CRM_INVALID_SIG" => URT::TLBX_E_CRM_INVALID_SIG,
            "TLBX_E_CLASS_LOAD_EXCEPTION" => URT::TLBX_E_CLASS_LOAD_EXCEPTION,
            "TLBX_E_UNKNOWN_SIGNATURE" => URT::TLBX_E_UNKNOWN_SIGNATURE,
            "TLBX_E_REFERENCED_TYPELIB" => URT::TLBX_E_REFERENCED_TYPELIB,
            "TLBX_E_INVALID_NAMESPACE" => URT::TLBX_E_INVALID_NAMESPACE,
            "TLBX_E_LAYOUT_ERROR" => URT::TLBX_E_LAYOUT_ERROR,
            "TLBX_E_NOTIUNKNOWN" => URT::TLBX_E_NOTIUNKNOWN,
            "TLBX_E_NONVISIBLEVALUECLASS" => URT::TLBX_E_NONVISIBLEVALUECLASS,
            "TLBX_E_LPTSTR_NOT_ALLOWED" => URT::TLBX_E_LPTSTR_NOT_ALLOWED,
            "TLBX_E_AUTO_CS_NOT_ALLOWED" => URT::TLBX_E_AUTO_CS_NOT_ALLOWED,
            "TLBX_S_NOSTDINTERFACE" => URT::TLBX_S_NOSTDINTERFACE,
            "TLBX_S_DUPLICATE_DISPID" => URT::TLBX_S_DUPLICATE_DISPID,
            "TLBX_E_ENUM_VALUE_INVALID" => URT::TLBX_E_ENUM_VALUE_INVALID,
            "TLBX_E_DUPLICATE_IID" => URT::TLBX_E_DUPLICATE_IID,
            "TLBX_E_NO_NESTED_ARRAYS" => URT::TLBX_E_NO_NESTED_ARRAYS,
            "TLBX_E_PARAM_ERROR_NAMED" => URT::TLBX_E_PARAM_ERROR_NAMED,
            "TLBX_E_PARAM_ERROR_UNNAMED" => URT::TLBX_E_PARAM_ERROR_UNNAMED,
            "TLBX_E_AGNOST_SIGNATURE" => URT::TLBX_E_AGNOST_SIGNATURE,
            "TLBX_E_CONVERT_FAIL" => URT::TLBX_E_CONVERT_FAIL,
            "TLBX_W_DUAL_NOT_DISPATCH" => URT::TLBX_W_DUAL_NOT_DISPATCH,
            "TLBX_E_BAD_SIGNATURE" => URT::TLBX_E_BAD_SIGNATURE,
            "TLBX_E_ARRAY_NEEDS_NT_FIXED" => URT::TLBX_E_ARRAY_NEEDS_NT_FIXED,
            "TLBX_E_CLASS_NEEDS_NT_INTF" => URT::TLBX_E_CLASS_NEEDS_NT_INTF,
            "META_E_CA_INVALID_TARGET" => URT::META_E_CA_INVALID_TARGET,
            "META_E_CA_INVALID_VALUE" => URT::META_E_CA_INVALID_VALUE,
            "META_E_CA_INVALID_BLOB" => URT::META_E_CA_INVALID_BLOB,
            "META_E_CA_REPEATED_ARG" => URT::META_E_CA_REPEATED_ARG,
            "META_E_CA_UNKNOWN_ARGUMENT" => URT::META_E_CA_UNKNOWN_ARGUMENT,
            "META_E_CA_VARIANT_NYI" => URT::META_E_CA_VARIANT_NYI,
            "META_E_CA_ARRAY_NYI" => URT::META_E_CA_ARRAY_NYI,
            "META_E_CA_UNEXPECTED_TYPE" => URT::META_E_CA_UNEXPECTED_TYPE,
            "META_E_CA_INVALID_ARGTYPE" => URT::META_E_CA_INVALID_ARGTYPE,
            "META_E_CA_INVALID_ARG_FOR_TYPE" => URT::META_E_CA_INVALID_ARG_FOR_TYPE,
            "META_E_CA_INVALID_UUID" => URT::META_E_CA_INVALID_UUID,
            "META_E_CA_INVALID_MARSHALAS_FIELDS" => URT::META_E_CA_INVALID_MARSHALAS_FIELDS,
            "META_E_CA_NT_FIELDONLY" => URT::META_E_CA_NT_FIELDONLY,
            "META_E_CA_NEGATIVE_PARAMINDEX" => URT::META_E_CA_NEGATIVE_PARAMINDEX,
            "META_E_CA_NEGATIVE_MULTIPLIER" => URT::META_E_CA_NEGATIVE_MULTIPLIER,
            "META_E_CA_NEGATIVE_CONSTSIZE" => URT::META_E_CA_NEGATIVE_CONSTSIZE,
            "META_E_CA_FIXEDSTR_SIZE_REQUIRED" => URT::META_E_CA_FIXEDSTR_SIZE_REQUIRED,
            "META_E_CA_CUSTMARSH_TYPE_REQUIRED" => URT::META_E_CA_CUSTMARSH_TYPE_REQUIRED,
            "META_E_CA_FILENAME_REQUIRED" => URT::META_E_CA_FILENAME_REQUIRED,
            "TLBX_W_NO_PROPS_IN_EVENTS" => URT::TLBX_W_NO_PROPS_IN_EVENTS,
            "META_E_NOT_IN_ENC_MODE" => URT::META_E_NOT_IN_ENC_MODE,
            "META_E_METHOD_COUNTS" => URT::META_E_METHOD_COUNTS,
            "META_E_FIELD_COUNTS" => URT::META_E_FIELD_COUNTS,
            "META_E_PARAM_COUNTS" => URT::META_E_PARAM_COUNTS,
            "TLBX_E_TYPED_REF" => URT::TLBX_E_TYPED_REF,
            "TLBX_W_DEFAULT_INTF_NOT_VISIBLE" => URT::TLBX_W_DEFAULT_INTF_NOT_VISIBLE,
            "TLBX_W_BAD_SAFEARRAYFIELD_NO_ELEMENTVT" => URT::TLBX_W_BAD_SAFEARRAYFIELD_NO_ELEMENTVT,
            "TLBX_W_LAYOUTCLASS_AS_INTERFACE" => URT::TLBX_W_LAYOUTCLASS_AS_INTERFACE,
            "TLBX_I_GENERIC_BASE_TYPE" => URT::TLBX_I_GENERIC_BASE_TYPE,
            "TLBX_E_BITNESS_MISMATCH" => URT::TLBX_E_BITNESS_MISMATCH,
            "TLBX_E_EVENT_WITH_NEWENUM" => URT::TLBX_E_EVENT_WITH_NEWENUM,
            "TLBX_E_PROPGET_WITHOUT_RETURN" => URT::TLBX_E_PROPGET_WITHOUT_RETURN,
            "META_E_MISMATCHED_VISIBLITY" => URT::META_E_MISMATCHED_VISIBLITY,
            "META_E_CA_BAD_FRIENDS_ARGS" => URT::META_E_CA_BAD_FRIENDS_ARGS,
            "META_E_CA_FRIENDS_SN_REQUIRED" => URT::META_E_CA_FRIENDS_SN_REQUIRED,
            "VLDTR_E_RID_OUTOFRANGE" => URT::VLDTR_E_RID_OUTOFRANGE,
            "VLDTR_E_CDTKN_OUTOFRANGE" => URT::VLDTR_E_CDTKN_OUTOFRANGE,
            "VLDTR_E_CDRID_OUTOFRANGE" => URT::VLDTR_E_CDRID_OUTOFRANGE,
            "VLDTR_E_STRING_INVALID" => URT::VLDTR_E_STRING_INVALID,
            "VLDTR_E_GUID_INVALID" => URT::VLDTR_E_GUID_INVALID,
            "VLDTR_E_BLOB_INVALID" => URT::VLDTR_E_BLOB_INVALID,
            "VLDTR_E_MOD_MULTI" => URT::VLDTR_E_MOD_MULTI,
            "VLDTR_E_MOD_NULLMVID" => URT::VLDTR_E_MOD_NULLMVID,
            "VLDTR_E_TR_NAMENULL" => URT::VLDTR_E_TR_NAMENULL,
            "VLDTR_E_TR_DUP" => URT::VLDTR_E_TR_DUP,
            "VLDTR_E_TD_NAMENULL" => URT::VLDTR_E_TD_NAMENULL,
            "VLDTR_E_TD_DUPNAME" => URT::VLDTR_E_TD_DUPNAME,
            "VLDTR_E_TD_DUPGUID" => URT::VLDTR_E_TD_DUPGUID,
            "VLDTR_E_TD_NOTIFACEOBJEXTNULL" => URT::VLDTR_E_TD_NOTIFACEOBJEXTNULL,
            "VLDTR_E_TD_OBJEXTENDSNONNULL" => URT::VLDTR_E_TD_OBJEXTENDSNONNULL,
            "VLDTR_E_TD_EXTENDSSEALED" => URT::VLDTR_E_TD_EXTENDSSEALED,
            "VLDTR_E_TD_DLTNORTSPCL" => URT::VLDTR_E_TD_DLTNORTSPCL,
            "VLDTR_E_TD_RTSPCLNOTDLT" => URT::VLDTR_E_TD_RTSPCLNOTDLT,
            "VLDTR_E_MI_DECLPRIV" => URT::VLDTR_E_MI_DECLPRIV,
            "VLDTR_E_AS_BADNAME" => URT::VLDTR_E_AS_BADNAME,
            "VLDTR_E_FILE_SYSNAME" => URT::VLDTR_E_FILE_SYSNAME,
            "VLDTR_E_MI_BODYSTATIC" => URT::VLDTR_E_MI_BODYSTATIC,
            "VLDTR_E_TD_IFACENOTABS" => URT::VLDTR_E_TD_IFACENOTABS,
            "VLDTR_E_TD_IFACEPARNOTNIL" => URT::VLDTR_E_TD_IFACEPARNOTNIL,
            "VLDTR_E_TD_IFACEGUIDNULL" => URT::VLDTR_E_TD_IFACEGUIDNULL,
            "VLDTR_E_MI_DECLFINAL" => URT::VLDTR_E_MI_DECLFINAL,
            "VLDTR_E_TD_VTNOTSEAL" => URT::VLDTR_E_TD_VTNOTSEAL,
            "VLDTR_E_PD_BADFLAGS" => URT::VLDTR_E_PD_BADFLAGS,
            "VLDTR_E_IFACE_DUP" => URT::VLDTR_E_IFACE_DUP,
            "VLDTR_E_MR_NAMENULL" => URT::VLDTR_E_MR_NAMENULL,
            "VLDTR_E_MR_VTBLNAME" => URT::VLDTR_E_MR_VTBLNAME,
            "VLDTR_E_MR_DELNAME" => URT::VLDTR_E_MR_DELNAME,
            "VLDTR_E_MR_PARNIL" => URT::VLDTR_E_MR_PARNIL,
            "VLDTR_E_MR_BADCALLINGCONV" => URT::VLDTR_E_MR_BADCALLINGCONV,
            "VLDTR_E_MR_NOTVARARG" => URT::VLDTR_E_MR_NOTVARARG,
            "VLDTR_E_MR_NAMEDIFF" => URT::VLDTR_E_MR_NAMEDIFF,
            "VLDTR_E_MR_SIGDIFF" => URT::VLDTR_E_MR_SIGDIFF,
            "VLDTR_E_MR_DUP" => URT::VLDTR_E_MR_DUP,
            "VLDTR_E_CL_TDAUTO" => URT::VLDTR_E_CL_TDAUTO,
            "VLDTR_E_CL_BADPCKSZ" => URT::VLDTR_E_CL_BADPCKSZ,
            "VLDTR_E_CL_DUP" => URT::VLDTR_E_CL_DUP,
            "VLDTR_E_FL_BADOFFSET" => URT::VLDTR_E_FL_BADOFFSET,
            "VLDTR_E_FL_TDNIL" => URT::VLDTR_E_FL_TDNIL,
            "VLDTR_E_FL_NOCL" => URT::VLDTR_E_FL_NOCL,
            "VLDTR_E_FL_TDNOTEXPLCT" => URT::VLDTR_E_FL_TDNOTEXPLCT,
            "VLDTR_E_FL_FLDSTATIC" => URT::VLDTR_E_FL_FLDSTATIC,
            "VLDTR_E_FL_DUP" => URT::VLDTR_E_FL_DUP,
            "VLDTR_E_MODREF_NAMENULL" => URT::VLDTR_E_MODREF_NAMENULL,
            "VLDTR_E_MODREF_DUP" => URT::VLDTR_E_MODREF_DUP,
            "VLDTR_E_TR_BADSCOPE" => URT::VLDTR_E_TR_BADSCOPE,
            "VLDTR_E_TD_NESTEDNOENCL" => URT::VLDTR_E_TD_NESTEDNOENCL,
            "VLDTR_E_TD_EXTTRRES" => URT::VLDTR_E_TD_EXTTRRES,
            "VLDTR_E_SIGNULL" => URT::VLDTR_E_SIGNULL,
            "VLDTR_E_SIGNODATA" => URT::VLDTR_E_SIGNODATA,
            "VLDTR_E_MD_BADCALLINGCONV" => URT::VLDTR_E_MD_BADCALLINGCONV,
            "VLDTR_E_MD_THISSTATIC" => URT::VLDTR_E_MD_THISSTATIC,
            "VLDTR_E_MD_NOTTHISNOTSTATIC" => URT::VLDTR_E_MD_NOTTHISNOTSTATIC,
            "VLDTR_E_MD_NOARGCNT" => URT::VLDTR_E_MD_NOARGCNT,
            "VLDTR_E_SIG_MISSELTYPE" => URT::VLDTR_E_SIG_MISSELTYPE,
            "VLDTR_E_SIG_MISSTKN" => URT::VLDTR_E_SIG_MISSTKN,
            "VLDTR_E_SIG_TKNBAD" => URT::VLDTR_E_SIG_TKNBAD,
            "VLDTR_E_SIG_MISSFPTR" => URT::VLDTR_E_SIG_MISSFPTR,
            "VLDTR_E_SIG_MISSFPTRARGCNT" => URT::VLDTR_E_SIG_MISSFPTRARGCNT,
            "VLDTR_E_SIG_MISSRANK" => URT::VLDTR_E_SIG_MISSRANK,
            "VLDTR_E_SIG_MISSNSIZE" => URT::VLDTR_E_SIG_MISSNSIZE,
            "VLDTR_E_SIG_MISSSIZE" => URT::VLDTR_E_SIG_MISSSIZE,
            "VLDTR_E_SIG_MISSNLBND" => URT::VLDTR_E_SIG_MISSNLBND,
            "VLDTR_E_SIG_MISSLBND" => URT::VLDTR_E_SIG_MISSLBND,
            "VLDTR_E_SIG_BADELTYPE" => URT::VLDTR_E_SIG_BADELTYPE,
            "VLDTR_E_SIG_MISSVASIZE" => URT::VLDTR_E_SIG_MISSVASIZE,
            "VLDTR_E_FD_BADCALLINGCONV" => URT::VLDTR_E_FD_BADCALLINGCONV,
            "VLDTR_E_MD_NAMENULL" => URT::VLDTR_E_MD_NAMENULL,
            "VLDTR_E_MD_PARNIL" => URT::VLDTR_E_MD_PARNIL,
            "VLDTR_E_MD_DUP" => URT::VLDTR_E_MD_DUP,
            "VLDTR_E_FD_NAMENULL" => URT::VLDTR_E_FD_NAMENULL,
            "VLDTR_E_FD_PARNIL" => URT::VLDTR_E_FD_PARNIL,
            "VLDTR_E_FD_DUP" => URT::VLDTR_E_FD_DUP,
            "VLDTR_E_AS_MULTI" => URT::VLDTR_E_AS_MULTI,
            "VLDTR_E_AS_NAMENULL" => URT::VLDTR_E_AS_NAMENULL,
            "VLDTR_E_SIG_TOKTYPEMISMATCH" => URT::VLDTR_E_SIG_TOKTYPEMISMATCH,
            "VLDTR_E_CL_TDINTF" => URT::VLDTR_E_CL_TDINTF,
            "VLDTR_E_ASOS_OSPLTFRMIDINVAL" => URT::VLDTR_E_ASOS_OSPLTFRMIDINVAL,
            "VLDTR_E_AR_NAMENULL" => URT::VLDTR_E_AR_NAMENULL,
            "VLDTR_E_TD_ENCLNOTNESTED" => URT::VLDTR_E_TD_ENCLNOTNESTED,
            "VLDTR_E_AROS_OSPLTFRMIDINVAL" => URT::VLDTR_E_AROS_OSPLTFRMIDINVAL,
            "VLDTR_E_FILE_NAMENULL" => URT::VLDTR_E_FILE_NAMENULL,
            "VLDTR_E_CT_NAMENULL" => URT::VLDTR_E_CT_NAMENULL,
            "VLDTR_E_TD_EXTENDSCHILD" => URT::VLDTR_E_TD_EXTENDSCHILD,
            "VLDTR_E_MAR_NAMENULL" => URT::VLDTR_E_MAR_NAMENULL,
            "VLDTR_E_FILE_DUP" => URT::VLDTR_E_FILE_DUP,
            "VLDTR_E_FILE_NAMEFULLQLFD" => URT::VLDTR_E_FILE_NAMEFULLQLFD,
            "VLDTR_E_CT_DUP" => URT::VLDTR_E_CT_DUP,
            "VLDTR_E_MAR_DUP" => URT::VLDTR_E_MAR_DUP,
            "VLDTR_E_MAR_NOTPUBPRIV" => URT::VLDTR_E_MAR_NOTPUBPRIV,
            "VLDTR_E_TD_ENUMNOVALUE" => URT::VLDTR_E_TD_ENUMNOVALUE,
            "VLDTR_E_TD_ENUMVALSTATIC" => URT::VLDTR_E_TD_ENUMVALSTATIC,
            "VLDTR_E_TD_ENUMVALNOTSN" => URT::VLDTR_E_TD_ENUMVALNOTSN,
            "VLDTR_E_TD_ENUMFLDNOTST" => URT::VLDTR_E_TD_ENUMFLDNOTST,
            "VLDTR_E_TD_ENUMFLDNOTLIT" => URT::VLDTR_E_TD_ENUMFLDNOTLIT,
            "VLDTR_E_TD_ENUMNOLITFLDS" => URT::VLDTR_E_TD_ENUMNOLITFLDS,
            "VLDTR_E_TD_ENUMFLDSIGMISMATCH" => URT::VLDTR_E_TD_ENUMFLDSIGMISMATCH,
            "VLDTR_E_TD_ENUMVALNOT1ST" => URT::VLDTR_E_TD_ENUMVALNOT1ST,
            "VLDTR_E_FD_NOTVALUERTSN" => URT::VLDTR_E_FD_NOTVALUERTSN,
            "VLDTR_E_FD_VALUEPARNOTENUM" => URT::VLDTR_E_FD_VALUEPARNOTENUM,
            "VLDTR_E_FD_INSTINIFACE" => URT::VLDTR_E_FD_INSTINIFACE,
            "VLDTR_E_FD_NOTPUBINIFACE" => URT::VLDTR_E_FD_NOTPUBINIFACE,
            "VLDTR_E_FMD_GLOBALNOTPUBPRIVSC" => URT::VLDTR_E_FMD_GLOBALNOTPUBPRIVSC,
            "VLDTR_E_FMD_GLOBALNOTSTATIC" => URT::VLDTR_E_FMD_GLOBALNOTSTATIC,
            "VLDTR_E_FD_GLOBALNORVA" => URT::VLDTR_E_FD_GLOBALNORVA,
            "VLDTR_E_MD_CTORZERORVA" => URT::VLDTR_E_MD_CTORZERORVA,
            "VLDTR_E_FD_MARKEDNOMARSHAL" => URT::VLDTR_E_FD_MARKEDNOMARSHAL,
            "VLDTR_E_FD_MARSHALNOTMARKED" => URT::VLDTR_E_FD_MARSHALNOTMARKED,
            "VLDTR_E_FD_MARKEDNODEFLT" => URT::VLDTR_E_FD_MARKEDNODEFLT,
            "VLDTR_E_FD_DEFLTNOTMARKED" => URT::VLDTR_E_FD_DEFLTNOTMARKED,
            "VLDTR_E_FMD_MARKEDNOSECUR" => URT::VLDTR_E_FMD_MARKEDNOSECUR,
            "VLDTR_E_FMD_SECURNOTMARKED" => URT::VLDTR_E_FMD_SECURNOTMARKED,
            "VLDTR_E_FMD_PINVOKENOTSTATIC" => URT::VLDTR_E_FMD_PINVOKENOTSTATIC,
            "VLDTR_E_FMD_MARKEDNOPINVOKE" => URT::VLDTR_E_FMD_MARKEDNOPINVOKE,
            "VLDTR_E_FMD_PINVOKENOTMARKED" => URT::VLDTR_E_FMD_PINVOKENOTMARKED,
            "VLDTR_E_FMD_BADIMPLMAP" => URT::VLDTR_E_FMD_BADIMPLMAP,
            "VLDTR_E_IMAP_BADMODREF" => URT::VLDTR_E_IMAP_BADMODREF,
            "VLDTR_E_IMAP_BADMEMBER" => URT::VLDTR_E_IMAP_BADMEMBER,
            "VLDTR_E_IMAP_BADIMPORTNAME" => URT::VLDTR_E_IMAP_BADIMPORTNAME,
            "VLDTR_E_IMAP_BADCALLCONV" => URT::VLDTR_E_IMAP_BADCALLCONV,
            "VLDTR_E_FMD_BADACCESSFLAG" => URT::VLDTR_E_FMD_BADACCESSFLAG,
            "VLDTR_E_FD_INITONLYANDLITERAL" => URT::VLDTR_E_FD_INITONLYANDLITERAL,
            "VLDTR_E_FD_LITERALNOTSTATIC" => URT::VLDTR_E_FD_LITERALNOTSTATIC,
            "VLDTR_E_FMD_RTSNNOTSN" => URT::VLDTR_E_FMD_RTSNNOTSN,
            "VLDTR_E_MD_ABSTPARNOTABST" => URT::VLDTR_E_MD_ABSTPARNOTABST,
            "VLDTR_E_MD_NOTSTATABSTININTF" => URT::VLDTR_E_MD_NOTSTATABSTININTF,
            "VLDTR_E_MD_NOTPUBININTF" => URT::VLDTR_E_MD_NOTPUBININTF,
            "VLDTR_E_MD_CTORININTF" => URT::VLDTR_E_MD_CTORININTF,
            "VLDTR_E_MD_GLOBALCTORCCTOR" => URT::VLDTR_E_MD_GLOBALCTORCCTOR,
            "VLDTR_E_MD_CTORSTATIC" => URT::VLDTR_E_MD_CTORSTATIC,
            "VLDTR_E_MD_CTORNOTSNRTSN" => URT::VLDTR_E_MD_CTORNOTSNRTSN,
            "VLDTR_E_MD_CTORVIRT" => URT::VLDTR_E_MD_CTORVIRT,
            "VLDTR_E_MD_CTORABST" => URT::VLDTR_E_MD_CTORABST,
            "VLDTR_E_MD_CCTORNOTSTATIC" => URT::VLDTR_E_MD_CCTORNOTSTATIC,
            "VLDTR_E_MD_ZERORVA" => URT::VLDTR_E_MD_ZERORVA,
            "VLDTR_E_MD_FINNOTVIRT" => URT::VLDTR_E_MD_FINNOTVIRT,
            "VLDTR_E_MD_STATANDFINORVIRT" => URT::VLDTR_E_MD_STATANDFINORVIRT,
            "VLDTR_E_MD_ABSTANDFINAL" => URT::VLDTR_E_MD_ABSTANDFINAL,
            "VLDTR_E_MD_ABSTANDIMPL" => URT::VLDTR_E_MD_ABSTANDIMPL,
            "VLDTR_E_MD_ABSTANDPINVOKE" => URT::VLDTR_E_MD_ABSTANDPINVOKE,
            "VLDTR_E_MD_ABSTNOTVIRT" => URT::VLDTR_E_MD_ABSTNOTVIRT,
            "VLDTR_E_MD_NOTABSTNOTIMPL" => URT::VLDTR_E_MD_NOTABSTNOTIMPL,
            "VLDTR_E_MD_NOTABSTBADFLAGSRVA" => URT::VLDTR_E_MD_NOTABSTBADFLAGSRVA,
            "VLDTR_E_MD_PRIVSCOPENORVA" => URT::VLDTR_E_MD_PRIVSCOPENORVA,
            "VLDTR_E_MD_GLOBALABSTORVIRT" => URT::VLDTR_E_MD_GLOBALABSTORVIRT,
            "VLDTR_E_SIG_LONGFORM" => URT::VLDTR_E_SIG_LONGFORM,
            "VLDTR_E_MD_MULTIPLESEMANTICS" => URT::VLDTR_E_MD_MULTIPLESEMANTICS,
            "VLDTR_E_MD_INVALIDSEMANTICS" => URT::VLDTR_E_MD_INVALIDSEMANTICS,
            "VLDTR_E_MD_SEMANTICSNOTEXIST" => URT::VLDTR_E_MD_SEMANTICSNOTEXIST,
            "VLDTR_E_MI_DECLNOTVIRT" => URT::VLDTR_E_MI_DECLNOTVIRT,
            "VLDTR_E_FMD_GLOBALITEM" => URT::VLDTR_E_FMD_GLOBALITEM,
            "VLDTR_E_MD_MULTSEMANTICFLAGS" => URT::VLDTR_E_MD_MULTSEMANTICFLAGS,
            "VLDTR_E_MD_NOSEMANTICFLAGS" => URT::VLDTR_E_MD_NOSEMANTICFLAGS,
            "VLDTR_E_FD_FLDINIFACE" => URT::VLDTR_E_FD_FLDINIFACE,
            "VLDTR_E_AS_HASHALGID" => URT::VLDTR_E_AS_HASHALGID,
            "VLDTR_E_AS_PROCID" => URT::VLDTR_E_AS_PROCID,
            "VLDTR_E_AR_PROCID" => URT::VLDTR_E_AR_PROCID,
            "VLDTR_E_CN_PARENTRANGE" => URT::VLDTR_E_CN_PARENTRANGE,
            "VLDTR_E_AS_BADFLAGS" => URT::VLDTR_E_AS_BADFLAGS,
            "VLDTR_E_TR_HASTYPEDEF" => URT::VLDTR_E_TR_HASTYPEDEF,
            "VLDTR_E_IFACE_BADIMPL" => URT::VLDTR_E_IFACE_BADIMPL,
            "VLDTR_E_IFACE_BADIFACE" => URT::VLDTR_E_IFACE_BADIFACE,
            "VLDTR_E_TD_SECURNOTMARKED" => URT::VLDTR_E_TD_SECURNOTMARKED,
            "VLDTR_E_TD_MARKEDNOSECUR" => URT::VLDTR_E_TD_MARKEDNOSECUR,
            "VLDTR_E_MD_CCTORHASARGS" => URT::VLDTR_E_MD_CCTORHASARGS,
            "VLDTR_E_CT_BADIMPL" => URT::VLDTR_E_CT_BADIMPL,
            "VLDTR_E_MI_ALIENBODY" => URT::VLDTR_E_MI_ALIENBODY,
            "VLDTR_E_MD_CCTORCALLCONV" => URT::VLDTR_E_MD_CCTORCALLCONV,
            "VLDTR_E_MI_BADCLASS" => URT::VLDTR_E_MI_BADCLASS,
            "VLDTR_E_MI_CLASSISINTF" => URT::VLDTR_E_MI_CLASSISINTF,
            "VLDTR_E_MI_BADDECL" => URT::VLDTR_E_MI_BADDECL,
            "VLDTR_E_MI_BADBODY" => URT::VLDTR_E_MI_BADBODY,
            "VLDTR_E_MI_DUP" => URT::VLDTR_E_MI_DUP,
            "VLDTR_E_FD_BADPARENT" => URT::VLDTR_E_FD_BADPARENT,
            "VLDTR_E_MD_PARAMOUTOFSEQ" => URT::VLDTR_E_MD_PARAMOUTOFSEQ,
            "VLDTR_E_MD_PARASEQTOOBIG" => URT::VLDTR_E_MD_PARASEQTOOBIG,
            "VLDTR_E_MD_PARMMARKEDNOMARSHAL" => URT::VLDTR_E_MD_PARMMARKEDNOMARSHAL,
            "VLDTR_E_MD_PARMMARSHALNOTMARKED" => URT::VLDTR_E_MD_PARMMARSHALNOTMARKED,
            "VLDTR_E_MD_PARMMARKEDNODEFLT" => URT::VLDTR_E_MD_PARMMARKEDNODEFLT,
            "VLDTR_E_MD_PARMDEFLTNOTMARKED" => URT::VLDTR_E_MD_PARMDEFLTNOTMARKED,
            "VLDTR_E_PR_BADSCOPE" => URT::VLDTR_E_PR_BADSCOPE,
            "VLDTR_E_PR_NONAME" => URT::VLDTR_E_PR_NONAME,
            "VLDTR_E_PR_NOSIG" => URT::VLDTR_E_PR_NOSIG,
            "VLDTR_E_PR_DUP" => URT::VLDTR_E_PR_DUP,
            "VLDTR_E_PR_BADCALLINGCONV" => URT::VLDTR_E_PR_BADCALLINGCONV,
            "VLDTR_E_PR_MARKEDNODEFLT" => URT::VLDTR_E_PR_MARKEDNODEFLT,
            "VLDTR_E_PR_DEFLTNOTMARKED" => URT::VLDTR_E_PR_DEFLTNOTMARKED,
            "VLDTR_E_PR_BADSEMANTICS" => URT::VLDTR_E_PR_BADSEMANTICS,
            "VLDTR_E_PR_BADMETHOD" => URT::VLDTR_E_PR_BADMETHOD,
            "VLDTR_E_PR_ALIENMETHOD" => URT::VLDTR_E_PR_ALIENMETHOD,
            "VLDTR_E_CN_BLOBNOTNULL" => URT::VLDTR_E_CN_BLOBNOTNULL,
            "VLDTR_E_CN_BLOBNULL" => URT::VLDTR_E_CN_BLOBNULL,
            "VLDTR_E_EV_BADSCOPE" => URT::VLDTR_E_EV_BADSCOPE,
            "VLDTR_E_EV_NONAME" => URT::VLDTR_E_EV_NONAME,
            "VLDTR_E_EV_DUP" => URT::VLDTR_E_EV_DUP,
            "VLDTR_E_EV_BADEVTYPE" => URT::VLDTR_E_EV_BADEVTYPE,
            "VLDTR_E_EV_EVTYPENOTCLASS" => URT::VLDTR_E_EV_EVTYPENOTCLASS,
            "VLDTR_E_EV_BADSEMANTICS" => URT::VLDTR_E_EV_BADSEMANTICS,
            "VLDTR_E_EV_BADMETHOD" => URT::VLDTR_E_EV_BADMETHOD,
            "VLDTR_E_EV_ALIENMETHOD" => URT::VLDTR_E_EV_ALIENMETHOD,
            "VLDTR_E_EV_NOADDON" => URT::VLDTR_E_EV_NOADDON,
            "VLDTR_E_EV_NOREMOVEON" => URT::VLDTR_E_EV_NOREMOVEON,
            "VLDTR_E_CT_DUPTDNAME" => URT::VLDTR_E_CT_DUPTDNAME,
            "VLDTR_E_MAR_BADOFFSET" => URT::VLDTR_E_MAR_BADOFFSET,
            "VLDTR_E_DS_BADOWNER" => URT::VLDTR_E_DS_BADOWNER,
            "VLDTR_E_DS_BADFLAGS" => URT::VLDTR_E_DS_BADFLAGS,
            "VLDTR_E_DS_NOBLOB" => URT::VLDTR_E_DS_NOBLOB,
            "VLDTR_E_MAR_BADIMPL" => URT::VLDTR_E_MAR_BADIMPL,
            "VLDTR_E_MR_VARARGCALLINGCONV" => URT::VLDTR_E_MR_VARARGCALLINGCONV,
            "VLDTR_E_MD_CTORNOTVOID" => URT::VLDTR_E_MD_CTORNOTVOID,
            "VLDTR_E_EV_FIRENOTVOID" => URT::VLDTR_E_EV_FIRENOTVOID,
            "VLDTR_E_AS_BADLOCALE" => URT::VLDTR_E_AS_BADLOCALE,
            "VLDTR_E_CN_PARENTTYPE" => URT::VLDTR_E_CN_PARENTTYPE,
            "VLDTR_E_SIG_SENTINMETHODDEF" => URT::VLDTR_E_SIG_SENTINMETHODDEF,
            "VLDTR_E_SIG_SENTMUSTVARARG" => URT::VLDTR_E_SIG_SENTMUSTVARARG,
            "VLDTR_E_SIG_MULTSENTINELS" => URT::VLDTR_E_SIG_MULTSENTINELS,
            "VLDTR_E_SIG_LASTSENTINEL" => URT::VLDTR_E_SIG_LASTSENTINEL,
            "VLDTR_E_SIG_MISSARG" => URT::VLDTR_E_SIG_MISSARG,
            "VLDTR_E_SIG_BYREFINFIELD" => URT::VLDTR_E_SIG_BYREFINFIELD,
            "VLDTR_E_MD_SYNCMETHODINVTYPE" => URT::VLDTR_E_MD_SYNCMETHODINVTYPE,
            "VLDTR_E_TD_NAMETOOLONG" => URT::VLDTR_E_TD_NAMETOOLONG,
            "VLDTR_E_AS_PROCDUP" => URT::VLDTR_E_AS_PROCDUP,
            "VLDTR_E_ASOS_DUP" => URT::VLDTR_E_ASOS_DUP,
            "VLDTR_E_MAR_BADFLAGS" => URT::VLDTR_E_MAR_BADFLAGS,
            "VLDTR_E_CT_NOTYPEDEFID" => URT::VLDTR_E_CT_NOTYPEDEFID,
            "VLDTR_E_FILE_BADFLAGS" => URT::VLDTR_E_FILE_BADFLAGS,
            "VLDTR_E_FILE_NULLHASH" => URT::VLDTR_E_FILE_NULLHASH,
            "VLDTR_E_MOD_NONAME" => URT::VLDTR_E_MOD_NONAME,
            "VLDTR_E_MOD_NAMEFULLQLFD" => URT::VLDTR_E_MOD_NAMEFULLQLFD,
            "VLDTR_E_TD_RTSPCLNOTSPCL" => URT::VLDTR_E_TD_RTSPCLNOTSPCL,
            "VLDTR_E_TD_EXTENDSIFACE" => URT::VLDTR_E_TD_EXTENDSIFACE,
            "VLDTR_E_MD_CTORPINVOKE" => URT::VLDTR_E_MD_CTORPINVOKE,
            "VLDTR_E_TD_SYSENUMNOTCLASS" => URT::VLDTR_E_TD_SYSENUMNOTCLASS,
            "VLDTR_E_TD_SYSENUMNOTEXTVTYPE" => URT::VLDTR_E_TD_SYSENUMNOTEXTVTYPE,
            "VLDTR_E_MI_SIGMISMATCH" => URT::VLDTR_E_MI_SIGMISMATCH,
            "VLDTR_E_TD_ENUMHASMETHODS" => URT::VLDTR_E_TD_ENUMHASMETHODS,
            "VLDTR_E_TD_ENUMIMPLIFACE" => URT::VLDTR_E_TD_ENUMIMPLIFACE,
            "VLDTR_E_TD_ENUMHASPROP" => URT::VLDTR_E_TD_ENUMHASPROP,
            "VLDTR_E_TD_ENUMHASEVENT" => URT::VLDTR_E_TD_ENUMHASEVENT,
            "VLDTR_E_TD_BADMETHODLST" => URT::VLDTR_E_TD_BADMETHODLST,
            "VLDTR_E_TD_BADFIELDLST" => URT::VLDTR_E_TD_BADFIELDLST,
            "VLDTR_E_CN_BADTYPE" => URT::VLDTR_E_CN_BADTYPE,
            "VLDTR_E_TD_ENUMNOINSTFLD" => URT::VLDTR_E_TD_ENUMNOINSTFLD,
            "VLDTR_E_TD_ENUMMULINSTFLD" => URT::VLDTR_E_TD_ENUMMULINSTFLD,
            "VLDTR_E_INTERRUPTED" => URT::VLDTR_E_INTERRUPTED,
            "VLDTR_E_NOTINIT" => URT::VLDTR_E_NOTINIT,
            "CORDBG_E_UNRECOVERABLE_ERROR" => URT::CORDBG_E_UNRECOVERABLE_ERROR,
            "CORDBG_E_PROCESS_TERMINATED" => URT::CORDBG_E_PROCESS_TERMINATED,
            "CORDBG_E_PROCESS_NOT_SYNCHRONIZED" => URT::CORDBG_E_PROCESS_NOT_SYNCHRONIZED,
            "CORDBG_E_CLASS_NOT_LOADED" => URT::CORDBG_E_CLASS_NOT_LOADED,
            "CORDBG_E_IL_VAR_NOT_AVAILABLE" => URT::CORDBG_E_IL_VAR_NOT_AVAILABLE,
            "CORDBG_E_BAD_REFERENCE_VALUE" => URT::CORDBG_E_BAD_REFERENCE_VALUE,
            "CORDBG_E_FIELD_NOT_AVAILABLE" => URT::CORDBG_E_FIELD_NOT_AVAILABLE,
            "CORDBG_E_NON_NATIVE_FRAME" => URT::CORDBG_E_NON_NATIVE_FRAME,
            "CORDBG_E_NONCONTINUABLE_EXCEPTION" => URT::CORDBG_E_NONCONTINUABLE_EXCEPTION,
            "CORDBG_E_CODE_NOT_AVAILABLE" => URT::CORDBG_E_CODE_NOT_AVAILABLE,
            "CORDBG_E_FUNCTION_NOT_IL" => URT::CORDBG_E_FUNCTION_NOT_IL,
            "CORDBG_E_CANT_SET_IP_INTO_FINALLY" => URT::CORDBG_E_CANT_SET_IP_INTO_FINALLY,
            "CORDBG_E_CANT_SET_IP_OUT_OF_FINALLY" => URT::CORDBG_E_CANT_SET_IP_OUT_OF_FINALLY,
            "CORDBG_E_CANT_SET_IP_INTO_CATCH" => URT::CORDBG_E_CANT_SET_IP_INTO_CATCH,
            "CORDBG_E_SET_IP_NOT_ALLOWED_ON_NONLEAF_FRAME" => URT::CORDBG_E_SET_IP_NOT_ALLOWED_ON_NONLEAF_FRAME,
            "CORDBG_E_SET_IP_IMPOSSIBLE" => URT::CORDBG_E_SET_IP_IMPOSSIBLE,
            "CORDBG_E_FUNC_EVAL_BAD_START_POINT" => URT::CORDBG_E_FUNC_EVAL_BAD_START_POINT,
            "CORDBG_E_INVALID_OBJECT" => URT::CORDBG_E_INVALID_OBJECT,
            "CORDBG_E_FUNC_EVAL_NOT_COMPLETE" => URT::CORDBG_E_FUNC_EVAL_NOT_COMPLETE,
            "CORDBG_E_INPROC_NOT_IMPL" => URT::CORDBG_E_INPROC_NOT_IMPL,
            "CORDBG_E_STATIC_VAR_NOT_AVAILABLE" => URT::CORDBG_E_STATIC_VAR_NOT_AVAILABLE,
            "CORDBG_E_OBJECT_IS_NOT_COPYABLE_VALUE_CLASS" => URT::CORDBG_E_OBJECT_IS_NOT_COPYABLE_VALUE_CLASS,
            "CORDBG_E_CANT_SETIP_INTO_OR_OUT_OF_FILTER" => URT::CORDBG_E_CANT_SETIP_INTO_OR_OUT_OF_FILTER,
            "CORDBG_E_CANT_CHANGE_JIT_SETTING_FOR_ZAP_MODULE" => URT::CORDBG_E_CANT_CHANGE_JIT_SETTING_FOR_ZAP_MODULE,
            "CORDBG_E_CANT_SET_IP_OUT_OF_FINALLY_ON_WIN64" => URT::CORDBG_E_CANT_SET_IP_OUT_OF_FINALLY_ON_WIN64,
            "CORDBG_E_CANT_SET_IP_OUT_OF_CATCH_ON_WIN64" => URT::CORDBG_E_CANT_SET_IP_OUT_OF_CATCH_ON_WIN64,
            "CORDBG_E_REMOTE_CONNECTION_CONN_RESET" => URT::CORDBG_E_REMOTE_CONNECTION_CONN_RESET,
            "CORDBG_E_REMOTE_CONNECTION_KEEP_ALIVE" => URT::CORDBG_E_REMOTE_CONNECTION_KEEP_ALIVE,
            "CORDBG_E_REMOTE_CONNECTION_FATAL_ERROR" => URT::CORDBG_E_REMOTE_CONNECTION_FATAL_ERROR,
            "CORDBG_E_CANT_SET_TO_JMC" => URT::CORDBG_E_CANT_SET_TO_JMC,
            "CORDBG_E_NO_CONTEXT_FOR_INTERNAL_FRAME" => URT::CORDBG_E_NO_CONTEXT_FOR_INTERNAL_FRAME,
            "CORDBG_E_NOT_CHILD_FRAME" => URT::CORDBG_E_NOT_CHILD_FRAME,
            "CORDBG_E_NON_MATCHING_CONTEXT" => URT::CORDBG_E_NON_MATCHING_CONTEXT,
            "CORDBG_E_PAST_END_OF_STACK" => URT::CORDBG_E_PAST_END_OF_STACK,
            "CORDBG_E_BAD_THREAD_STATE" => URT::CORDBG_E_BAD_THREAD_STATE,
            "CORDBG_E_FUNC_EVAL_CANNOT_UPDATE_REGISTER_IN_NONLEAF_FRAME" => URT::CORDBG_E_FUNC_EVAL_CANNOT_UPDATE_REGISTER_IN_NONLEAF_FRAME,
            "CORDBG_E_DEBUGGER_ALREADY_ATTACHED" => URT::CORDBG_E_DEBUGGER_ALREADY_ATTACHED,
            "CORDBG_E_SUPERFLOUS_CONTINUE" => URT::CORDBG_E_SUPERFLOUS_CONTINUE,
            "CORDBG_E_SET_VALUE_NOT_ALLOWED_ON_NONLEAF_FRAME" => URT::CORDBG_E_SET_VALUE_NOT_ALLOWED_ON_NONLEAF_FRAME,
            "CORDBG_E_ENC_EH_MAX_NESTING_LEVEL_CANT_INCREASE" => URT::CORDBG_E_ENC_EH_MAX_NESTING_LEVEL_CANT_INCREASE,
            "CORDBG_E_ENC_MODULE_NOT_ENC_ENABLED" => URT::CORDBG_E_ENC_MODULE_NOT_ENC_ENABLED,
            "CORDBG_E_SET_IP_NOT_ALLOWED_ON_EXCEPTION" => URT::CORDBG_E_SET_IP_NOT_ALLOWED_ON_EXCEPTION,
            "CORDBG_E_VARIABLE_IS_ACTUALLY_LITERAL" => URT::CORDBG_E_VARIABLE_IS_ACTUALLY_LITERAL,
            "CORDBG_E_PROCESS_DETACHED" => URT::CORDBG_E_PROCESS_DETACHED,
            "CORDBG_E_ENC_METHOD_SIG_CHANGED" => URT::CORDBG_E_ENC_METHOD_SIG_CHANGED,
            "CORDBG_E_ENC_METHOD_NO_LOCAL_SIG" => URT::CORDBG_E_ENC_METHOD_NO_LOCAL_SIG,
            "CORDBG_E_ENC_CANT_ADD_FIELD_TO_VALUE_OR_LAYOUT_CLASS" => URT::CORDBG_E_ENC_CANT_ADD_FIELD_TO_VALUE_OR_LAYOUT_CLASS,
            "CORDBG_E_ENC_CANT_CHANGE_FIELD" => URT::CORDBG_E_ENC_CANT_CHANGE_FIELD,
            "CORDBG_E_ENC_CANT_ADD_NON_PRIVATE_MEMBER" => URT::CORDBG_E_ENC_CANT_ADD_NON_PRIVATE_MEMBER,
            "CORDBG_E_FIELD_NOT_STATIC" => URT::CORDBG_E_FIELD_NOT_STATIC,
            "CORDBG_E_FIELD_NOT_INSTANCE" => URT::CORDBG_E_FIELD_NOT_INSTANCE,
            "CORDBG_E_ENC_ZAPPED_WITHOUT_ENC" => URT::CORDBG_E_ENC_ZAPPED_WITHOUT_ENC,
            "CORDBG_E_ENC_BAD_METHOD_INFO" => URT::CORDBG_E_ENC_BAD_METHOD_INFO,
            "CORDBG_E_ENC_JIT_CANT_UPDATE" => URT::CORDBG_E_ENC_JIT_CANT_UPDATE,
            "CORDBG_E_ENC_MISSING_CLASS" => URT::CORDBG_E_ENC_MISSING_CLASS,
            "CORDBG_E_ENC_INTERNAL_ERROR" => URT::CORDBG_E_ENC_INTERNAL_ERROR,
            "CORDBG_E_ENC_HANGING_FIELD" => URT::CORDBG_E_ENC_HANGING_FIELD,
            "CORDBG_E_MODULE_NOT_LOADED" => URT::CORDBG_E_MODULE_NOT_LOADED,
            "CORDBG_E_ENC_CANT_CHANGE_SUPERCLASS" => URT::CORDBG_E_ENC_CANT_CHANGE_SUPERCLASS,
            "CORDBG_E_UNABLE_TO_SET_BREAKPOINT" => URT::CORDBG_E_UNABLE_TO_SET_BREAKPOINT,
            "CORDBG_E_DEBUGGING_NOT_POSSIBLE" => URT::CORDBG_E_DEBUGGING_NOT_POSSIBLE,
            "CORDBG_E_KERNEL_DEBUGGER_ENABLED" => URT::CORDBG_E_KERNEL_DEBUGGER_ENABLED,
            "CORDBG_E_KERNEL_DEBUGGER_PRESENT" => URT::CORDBG_E_KERNEL_DEBUGGER_PRESENT,
            "CORDBG_E_HELPER_THREAD_DEAD" => URT::CORDBG_E_HELPER_THREAD_DEAD,
            "CORDBG_E_INTERFACE_INHERITANCE_CANT_CHANGE" => URT::CORDBG_E_INTERFACE_INHERITANCE_CANT_CHANGE,
            "CORDBG_E_INCOMPATIBLE_PROTOCOL" => URT::CORDBG_E_INCOMPATIBLE_PROTOCOL,
            "CORDBG_E_TOO_MANY_PROCESSES" => URT::CORDBG_E_TOO_MANY_PROCESSES,
            "CORDBG_E_INTEROP_NOT_SUPPORTED" => URT::CORDBG_E_INTEROP_NOT_SUPPORTED,
            "CORDBG_E_NO_REMAP_BREAKPIONT" => URT::CORDBG_E_NO_REMAP_BREAKPIONT,
            "CORDBG_E_OBJECT_NEUTERED" => URT::CORDBG_E_OBJECT_NEUTERED,
            "CORPROF_E_FUNCTION_NOT_COMPILED" => URT::CORPROF_E_FUNCTION_NOT_COMPILED,
            "CORPROF_E_DATAINCOMPLETE" => URT::CORPROF_E_DATAINCOMPLETE,
            "CORPROF_E_NOT_REJITABLE_METHODS" => URT::CORPROF_E_NOT_REJITABLE_METHODS,
            "CORPROF_E_CANNOT_UPDATE_METHOD" => URT::CORPROF_E_CANNOT_UPDATE_METHOD,
            "CORPROF_E_FUNCTION_NOT_IL" => URT::CORPROF_E_FUNCTION_NOT_IL,
            "CORPROF_E_NOT_MANAGED_THREAD" => URT::CORPROF_E_NOT_MANAGED_THREAD,
            "CORPROF_E_CALL_ONLY_FROM_INIT" => URT::CORPROF_E_CALL_ONLY_FROM_INIT,
            "CORPROF_E_INPROC_NOT_ENABLED" => URT::CORPROF_E_INPROC_NOT_ENABLED,
            "CORPROF_E_JITMAPS_NOT_ENABLED" => URT::CORPROF_E_JITMAPS_NOT_ENABLED,
            "CORPROF_E_INPROC_ALREADY_BEGUN" => URT::CORPROF_E_INPROC_ALREADY_BEGUN,
            "CORPROF_E_INPROC_NOT_AVAILABLE" => URT::CORPROF_E_INPROC_NOT_AVAILABLE,
            "CORPROF_E_NOT_YET_AVAILABLE" => URT::CORPROF_E_NOT_YET_AVAILABLE,
            "CORPROF_E_TYPE_IS_PARAMETERIZED" => URT::CORPROF_E_TYPE_IS_PARAMETERIZED,
            "CORPROF_E_FUNCTION_IS_PARAMETERIZED" => URT::CORPROF_E_FUNCTION_IS_PARAMETERIZED,
            "CORPROF_E_STACKSNAPSHOT_INVALID_TGT_THREAD" => URT::CORPROF_E_STACKSNAPSHOT_INVALID_TGT_THREAD,
            "CORPROF_E_STACKSNAPSHOT_UNMANAGED_CTX" => URT::CORPROF_E_STACKSNAPSHOT_UNMANAGED_CTX,
            "CORPROF_E_STACKSNAPSHOT_UNSAFE" => URT::CORPROF_E_STACKSNAPSHOT_UNSAFE,
            "CORPROF_E_STACKSNAPSHOT_ABORTED" => URT::CORPROF_E_STACKSNAPSHOT_ABORTED,
            "CORPROF_E_LITERALS_HAVE_NO_ADDRESS" => URT::CORPROF_E_LITERALS_HAVE_NO_ADDRESS,
            "CORPROF_E_UNSUPPORTED_CALL_SEQUENCE" => URT::CORPROF_E_UNSUPPORTED_CALL_SEQUENCE,
            "CORPROF_E_ASYNCHRONOUS_UNSAFE" => URT::CORPROF_E_ASYNCHRONOUS_UNSAFE,
            "CORPROF_E_CLASSID_IS_ARRAY" => URT::CORPROF_E_CLASSID_IS_ARRAY,
            "CORPROF_E_CLASSID_IS_COMPOSITE" => URT::CORPROF_E_CLASSID_IS_COMPOSITE,
            "CORPROF_E_PROFILER_DETACHING" => URT::CORPROF_E_PROFILER_DETACHING,
            "CORPROF_E_PROFILER_NOT_ATTACHABLE" => URT::CORPROF_E_PROFILER_NOT_ATTACHABLE,
            "CORPROF_E_UNRECOGNIZED_PIPE_MSG_FORMAT" => URT::CORPROF_E_UNRECOGNIZED_PIPE_MSG_FORMAT,
            "CORPROF_E_PROFILER_ALREADY_ACTIVE" => URT::CORPROF_E_PROFILER_ALREADY_ACTIVE,
            "CORPROF_E_PROFILEE_INCOMPATIBLE_WITH_TRIGGER" => URT::CORPROF_E_PROFILEE_INCOMPATIBLE_WITH_TRIGGER,
            "CORPROF_E_IPC_FAILED" => URT::CORPROF_E_IPC_FAILED,
            "CORPROF_E_PROFILEE_PROCESS_NOT_FOUND" => URT::CORPROF_E_PROFILEE_PROCESS_NOT_FOUND,
            "CORPROF_E_CALLBACK3_REQUIRED" => URT::CORPROF_E_CALLBACK3_REQUIRED,
            "CORPROF_E_UNSUPPORTED_FOR_ATTACHING_PROFILER" => URT::CORPROF_E_UNSUPPORTED_FOR_ATTACHING_PROFILER,
            "CORPROF_E_IRREVERSIBLE_INSTRUMENTATION_PRESENT" => URT::CORPROF_E_IRREVERSIBLE_INSTRUMENTATION_PRESENT,
            "CORPROF_E_RUNTIME_UNINITIALIZED" => URT::CORPROF_E_RUNTIME_UNINITIALIZED,
            "CORPROF_E_IMMUTABLE_FLAGS_SET" => URT::CORPROF_E_IMMUTABLE_FLAGS_SET,
            "CORPROF_E_PROFILER_NOT_YET_INITIALIZED" => URT::CORPROF_E_PROFILER_NOT_YET_INITIALIZED,
            "CORPROF_E_INCONSISTENT_WITH_FLAGS" => URT::CORPROF_E_INCONSISTENT_WITH_FLAGS,
            "CORPROF_E_PROFILER_CANCEL_ACTIVATION" => URT::CORPROF_E_PROFILER_CANCEL_ACTIVATION,
            "CORPROF_E_CONCURRENT_GC_NOT_PROFILABLE" => URT::CORPROF_E_CONCURRENT_GC_NOT_PROFILABLE,
            "CORPROF_E_INCONSISTENT_FLAGS_WITH_HOST_PROTECTION_SETTING" => URT::CORPROF_E_INCONSISTENT_FLAGS_WITH_HOST_PROTECTION_SETTING,
            "CORPROF_E_DEBUGGING_DISABLED" => URT::CORPROF_E_DEBUGGING_DISABLED,
            "CORPROF_E_TIMEOUT_WAITING_FOR_CONCURRENT_GC" => URT::CORPROF_E_TIMEOUT_WAITING_FOR_CONCURRENT_GC,
            "CORPROF_E_MODULE_IS_DYNAMIC" => URT::CORPROF_E_MODULE_IS_DYNAMIC,
            "CORPROF_E_CALLBACK4_REQUIRED" => URT::CORPROF_E_CALLBACK4_REQUIRED,
            "CORPROF_E_REJIT_NOT_ENABLED" => URT::CORPROF_E_REJIT_NOT_ENABLED,
            "CORPROF_E_ACTIVE_REJIT_REQUEST_NOT_FOUND" => URT::CORPROF_E_ACTIVE_REJIT_REQUEST_NOT_FOUND,
            "CORPROF_E_FUNCTION_IS_COLLECTIBLE" => URT::CORPROF_E_FUNCTION_IS_COLLECTIBLE,
            "CORPROF_E_REJIT_REQUIRES_DISABLE_NGEN" => URT::CORPROF_E_REJIT_REQUIRES_DISABLE_NGEN,
            "CORPROF_E_CALLBACK6_REQUIRED" => URT::CORPROF_E_CALLBACK6_REQUIRED,
            "CORPROF_E_RUNTIME_SUSPEND_REQUIRED" => URT::CORPROF_E_RUNTIME_SUSPEND_REQUIRED,
            "CORPROF_E_CALLBACK7_REQUIRED" => URT::CORPROF_E_CALLBACK7_REQUIRED,
            "SECURITY_E_XML_TO_ASN_ENCODING" => URT::SECURITY_E_XML_TO_ASN_ENCODING,
            "SECURITY_E_INCOMPATIBLE_SHARE" => URT::SECURITY_E_INCOMPATIBLE_SHARE,
            "SECURITY_E_UNVERIFIABLE" => URT::SECURITY_E_UNVERIFIABLE,
            "SECURITY_E_INCOMPATIBLE_EVIDENCE" => URT::SECURITY_E_INCOMPATIBLE_EVIDENCE,
            "CORSEC_E_DECODE_SET" => URT::CORSEC_E_DECODE_SET,
            "CORSEC_E_ENCODE_SET" => URT::CORSEC_E_ENCODE_SET,
            "CORSEC_E_UNSUPPORTED_FORMAT" => URT::CORSEC_E_UNSUPPORTED_FORMAT,
            "SN_CRYPTOAPI_CALL_FAILED" => URT::SN_CRYPTOAPI_CALL_FAILED,
            "CORSEC_E_CRYPTOAPI_CALL_FAILED" => URT::CORSEC_E_CRYPTOAPI_CALL_FAILED,
            "SN_NO_SUITABLE_CSP" => URT::SN_NO_SUITABLE_CSP,
            "CORSEC_E_NO_SUITABLE_CSP" => URT::CORSEC_E_NO_SUITABLE_CSP,
            "CORSEC_E_INVALID_ATTR" => URT::CORSEC_E_INVALID_ATTR,
            "CORSEC_E_POLICY_EXCEPTION" => URT::CORSEC_E_POLICY_EXCEPTION,
            "CORSEC_E_MIN_GRANT_FAIL" => URT::CORSEC_E_MIN_GRANT_FAIL,
            "CORSEC_E_NO_EXEC_PERM" => URT::CORSEC_E_NO_EXEC_PERM,
            "CORSEC_E_XMLSYNTAX" => URT::CORSEC_E_XMLSYNTAX,
            "CORSEC_E_INVALID_STRONGNAME" => URT::CORSEC_E_INVALID_STRONGNAME,
            "CORSEC_E_MISSING_STRONGNAME" => URT::CORSEC_E_MISSING_STRONGNAME,
            "CORSEC_E_CONTAINER_NOT_FOUND" => URT::CORSEC_E_CONTAINER_NOT_FOUND,
            "CORSEC_E_INVALID_IMAGE_FORMAT" => URT::CORSEC_E_INVALID_IMAGE_FORMAT,
            "CORSEC_E_INVALID_PUBLICKEY" => URT::CORSEC_E_INVALID_PUBLICKEY,
            "CORSEC_E_SIGNATURE_MISMATCH" => URT::CORSEC_E_SIGNATURE_MISMATCH,
            "SN_E_PUBLICKEY_MISMATCH" => URT::SN_E_PUBLICKEY_MISMATCH,
            "CORSEC_E_CRYPTO" => URT::CORSEC_E_CRYPTO,
            "CORSEC_E_CRYPTO_UNEX_OPER" => URT::CORSEC_E_CRYPTO_UNEX_OPER,
            "CORSECATTR_E_BAD_ATTRIBUTE" => URT::CORSECATTR_E_BAD_ATTRIBUTE,
            "CORSECATTR_E_MISSING_CONSTRUCTOR" => URT::CORSECATTR_E_MISSING_CONSTRUCTOR,
            "CORSECATTR_E_FAILED_TO_CREATE_PERM" => URT::CORSECATTR_E_FAILED_TO_CREATE_PERM,
            "CORSECATTR_E_BAD_ACTION_ASM" => URT::CORSECATTR_E_BAD_ACTION_ASM,
            "CORSECATTR_E_BAD_ACTION_OTHER" => URT::CORSECATTR_E_BAD_ACTION_OTHER,
            "CORSECATTR_E_BAD_PARENT" => URT::CORSECATTR_E_BAD_PARENT,
            "CORSECATTR_E_TRUNCATED" => URT::CORSECATTR_E_TRUNCATED,
            "CORSECATTR_E_BAD_VERSION" => URT::CORSECATTR_E_BAD_VERSION,
            "CORSECATTR_E_BAD_ACTION" => URT::CORSECATTR_E_BAD_ACTION,
            "CORSECATTR_E_NO_SELF_REF" => URT::CORSECATTR_E_NO_SELF_REF,
            "CORSECATTR_E_BAD_NONCAS" => URT::CORSECATTR_E_BAD_NONCAS,
            "CORSECATTR_E_ASSEMBLY_LOAD_FAILED" => URT::CORSECATTR_E_ASSEMBLY_LOAD_FAILED,
            "CORSECATTR_E_ASSEMBLY_LOAD_FAILED_EX" => URT::CORSECATTR_E_ASSEMBLY_LOAD_FAILED_EX,
            "CORSECATTR_E_TYPE_LOAD_FAILED" => URT::CORSECATTR_E_TYPE_LOAD_FAILED,
            "CORSECATTR_E_TYPE_LOAD_FAILED_EX" => URT::CORSECATTR_E_TYPE_LOAD_FAILED_EX,
            "CORSECATTR_E_ABSTRACT" => URT::CORSECATTR_E_ABSTRACT,
            "CORSECATTR_E_UNSUPPORTED_TYPE" => URT::CORSECATTR_E_UNSUPPORTED_TYPE,
            "CORSECATTR_E_UNSUPPORTED_ENUM_TYPE" => URT::CORSECATTR_E_UNSUPPORTED_ENUM_TYPE,
            "CORSECATTR_E_NO_FIELD" => URT::CORSECATTR_E_NO_FIELD,
            "CORSECATTR_E_NO_PROPERTY" => URT::CORSECATTR_E_NO_PROPERTY,
            "CORSECATTR_E_EXCEPTION" => URT::CORSECATTR_E_EXCEPTION,
            "CORSECATTR_E_EXCEPTION_HR" => URT::CORSECATTR_E_EXCEPTION_HR,
            "ISS_E_ISOSTORE_START" => URT::ISS_E_ISOSTORE_START,
            "ISS_E_ISOSTORE" => URT::ISS_E_ISOSTORE,
            "ISS_E_OPEN_STORE_FILE" => URT::ISS_E_OPEN_STORE_FILE,
            "ISS_E_OPEN_FILE_MAPPING" => URT::ISS_E_OPEN_FILE_MAPPING,
            "ISS_E_MAP_VIEW_OF_FILE" => URT::ISS_E_MAP_VIEW_OF_FILE,
            "ISS_E_GET_FILE_SIZE" => URT::ISS_E_GET_FILE_SIZE,
            "ISS_E_CREATE_MUTEX" => URT::ISS_E_CREATE_MUTEX,
            "ISS_E_LOCK_FAILED" => URT::ISS_E_LOCK_FAILED,
            "ISS_E_FILE_WRITE" => URT::ISS_E_FILE_WRITE,
            "ISS_E_SET_FILE_POINTER" => URT::ISS_E_SET_FILE_POINTER,
            "ISS_E_CREATE_DIR" => URT::ISS_E_CREATE_DIR,
            "ISS_E_STORE_NOT_OPEN" => URT::ISS_E_STORE_NOT_OPEN,
            "ISS_E_CORRUPTED_STORE_FILE" => URT::ISS_E_CORRUPTED_STORE_FILE,
            "ISS_E_STORE_VERSION" => URT::ISS_E_STORE_VERSION,
            "ISS_E_FILE_NOT_MAPPED" => URT::ISS_E_FILE_NOT_MAPPED,
            "ISS_E_BLOCK_SIZE_TOO_SMALL" => URT::ISS_E_BLOCK_SIZE_TOO_SMALL,
            "ISS_E_ALLOC_TOO_LARGE" => URT::ISS_E_ALLOC_TOO_LARGE,
            "ISS_E_USAGE_WILL_EXCEED_QUOTA" => URT::ISS_E_USAGE_WILL_EXCEED_QUOTA,
            "ISS_E_TABLE_ROW_NOT_FOUND" => URT::ISS_E_TABLE_ROW_NOT_FOUND,
            "ISS_E_DEPRECATE" => URT::ISS_E_DEPRECATE,
            "ISS_E_CALLER" => URT::ISS_E_CALLER,
            "ISS_E_PATH_LENGTH" => URT::ISS_E_PATH_LENGTH,
            "ISS_E_MACHINE" => URT::ISS_E_MACHINE,
            "ISS_E_MACHINE_DACL" => URT::ISS_E_MACHINE_DACL,
            "ISS_E_ISOSTORE_END" => URT::ISS_E_ISOSTORE_END,
            "COR_E_EXCEPTION" => URT::COR_E_EXCEPTION,
            "COR_E_SYSTEM" => URT::COR_E_SYSTEM,
            "COR_E_ARGUMENTOUTOFRANGE" => URT::COR_E_ARGUMENTOUTOFRANGE,
            "COR_E_ARRAYTYPEMISMATCH" => URT::COR_E_ARRAYTYPEMISMATCH,
            "COR_E_CONTEXTMARSHAL" => URT::COR_E_CONTEXTMARSHAL,
            "COR_E_TIMEOUT" => URT::COR_E_TIMEOUT,
            "COR_E_EXECUTIONENGINE" => URT::COR_E_EXECUTIONENGINE,
            "COR_E_FIELDACCESS" => URT::COR_E_FIELDACCESS,
            "COR_E_INDEXOUTOFRANGE" => URT::COR_E_INDEXOUTOFRANGE,
            "COR_E_INVALIDOPERATION" => URT::COR_E_INVALIDOPERATION,
            "COR_E_SECURITY" => URT::COR_E_SECURITY,
            "COR_E_REMOTING" => URT::COR_E_REMOTING,
            "COR_E_SERIALIZATION" => URT::COR_E_SERIALIZATION,
            "COR_E_VERIFICATION" => URT::COR_E_VERIFICATION,
            "COR_E_SERVER" => URT::COR_E_SERVER,
            "COR_E_SERVICEDCOMPONENT" => URT::COR_E_SERVICEDCOMPONENT,
            "COR_E_METHODACCESS" => URT::COR_E_METHODACCESS,
            "COR_E_MISSINGFIELD" => URT::COR_E_MISSINGFIELD,
            "COR_E_MISSINGMEMBER" => URT::COR_E_MISSINGMEMBER,
            "COR_E_MISSINGMETHOD" => URT::COR_E_MISSINGMETHOD,
            "COR_E_MULTICASTNOTSUPPORTED" => URT::COR_E_MULTICASTNOTSUPPORTED,
            "COR_E_NOTSUPPORTED" => URT::COR_E_NOTSUPPORTED,
            "COR_E_OVERFLOW" => URT::COR_E_OVERFLOW,
            "COR_E_RANK" => URT::COR_E_RANK,
            "COR_E_SYNCHRONIZATIONLOCK" => URT::COR_E_SYNCHRONIZATIONLOCK,
            "COR_E_THREADINTERRUPTED" => URT::COR_E_THREADINTERRUPTED,
            "COR_E_MEMBERACCESS" => URT::COR_E_MEMBERACCESS,
            "COR_E_THREADSTATE" => URT::COR_E_THREADSTATE,
            "COR_E_THREADSTOP" => URT::COR_E_THREADSTOP,
            "COR_E_TYPELOAD" => URT::COR_E_TYPELOAD,
            "COR_E_ENTRYPOINTNOTFOUND" => URT::COR_E_ENTRYPOINTNOTFOUND,
            "COR_E_DLLNOTFOUND" => URT::COR_E_DLLNOTFOUND,
            "COR_E_THREADSTART" => URT::COR_E_THREADSTART,
            "COR_E_INVALIDCOMOBJECT" => URT::COR_E_INVALIDCOMOBJECT,
            "COR_E_NOTFINITENUMBER" => URT::COR_E_NOTFINITENUMBER,
            "COR_E_DUPLICATEWAITOBJECT" => URT::COR_E_DUPLICATEWAITOBJECT,
            "COR_E_SEMAPHOREFULL" => URT::COR_E_SEMAPHOREFULL,
            "COR_E_WAITHANDLECANNOTBEOPENED" => URT::COR_E_WAITHANDLECANNOTBEOPENED,
            "COR_E_ABANDONEDMUTEX" => URT::COR_E_ABANDONEDMUTEX,
            "COR_E_THREADABORTED" => URT::COR_E_THREADABORTED,
            "COR_E_INVALIDOLEVARIANTTYPE" => URT::COR_E_INVALIDOLEVARIANTTYPE,
            "COR_E_MISSINGMANIFESTRESOURCE" => URT::COR_E_MISSINGMANIFESTRESOURCE,
            "COR_E_SAFEARRAYTYPEMISMATCH" => URT::COR_E_SAFEARRAYTYPEMISMATCH,
            "COR_E_TYPEINITIALIZATION" => URT::COR_E_TYPEINITIALIZATION,
            "COR_E_MARSHALDIRECTIVE" => URT::COR_E_MARSHALDIRECTIVE,
            "COR_E_MISSINGSATELLITEASSEMBLY" => URT::COR_E_MISSINGSATELLITEASSEMBLY,
            "COR_E_FORMAT" => URT::COR_E_FORMAT,
            "COR_E_SAFEARRAYRANKMISMATCH" => URT::COR_E_SAFEARRAYRANKMISMATCH,
            "COR_E_PLATFORMNOTSUPPORTED" => URT::COR_E_PLATFORMNOTSUPPORTED,
            "COR_E_INVALIDPROGRAM" => URT::COR_E_INVALIDPROGRAM,
            "COR_E_OPERATIONCANCELED" => URT::COR_E_OPERATIONCANCELED,
            "COR_E_INSUFFICIENTMEMORY" => URT::COR_E_INSUFFICIENTMEMORY,
            "COR_E_RUNTIMEWRAPPED" => URT::COR_E_RUNTIMEWRAPPED,
            "COR_E_DEVICESNOTSUPPORTED" => URT::COR_E_DEVICESNOTSUPPORTED,
            "COR_E_DATAMISALIGNED" => URT::COR_E_DATAMISALIGNED,
            "COR_E_CODECONTRACTFAILED" => URT::COR_E_CODECONTRACTFAILED,
            "COR_E_TYPEACCESS" => URT::COR_E_TYPEACCESS,
            "COR_E_KEYNOTFOUND" => URT::COR_E_KEYNOTFOUND,
            "COR_E_INSUFFICIENTEXECUTIONSTACK" => URT::COR_E_INSUFFICIENTEXECUTIONSTACK,
            "COR_E_APPLICATION" => URT::COR_E_APPLICATION,
            "COR_E_INVALIDFILTERCRITERIA" => URT::COR_E_INVALIDFILTERCRITERIA,
            "COR_E_REFLECTIONTYPELOAD" => URT::COR_E_REFLECTIONTYPELOAD,
            "COR_E_TARGET" => URT::COR_E_TARGET,
            "COR_E_TARGETINVOCATION" => URT::COR_E_TARGETINVOCATION,
            "COR_E_CUSTOMATTRIBUTEFORMAT" => URT::COR_E_CUSTOMATTRIBUTEFORMAT,
            "COR_E_IO" => URT::COR_E_IO,
            "COR_E_FILELOAD" => URT::COR_E_FILELOAD,
            "COR_E_OBJECTDISPOSED" => URT::COR_E_OBJECTDISPOSED,
            "COR_E_FAILFAST" => URT::COR_E_FAILFAST,
            "COR_E_HOSTPROTECTION" => URT::COR_E_HOSTPROTECTION,
            "COR_E_ILLEGAL_REENTRANCY" => URT::COR_E_ILLEGAL_REENTRANCY,
            "CLR_E_SHIM_RUNTIMELOAD" => URT::CLR_E_SHIM_RUNTIMELOAD,
            "CLR_E_SHIM_RUNTIMEEXPORT" => URT::CLR_E_SHIM_RUNTIMEEXPORT,
            "CLR_E_SHIM_INSTALLROOT" => URT::CLR_E_SHIM_INSTALLROOT,
            "CLR_E_SHIM_INSTALLCOMP" => URT::CLR_E_SHIM_INSTALLCOMP,
            "CLR_E_SHIM_LEGACYRUNTIMEALREADYBOUND" => URT::CLR_E_SHIM_LEGACYRUNTIMEALREADYBOUND,
            "CLR_E_SHIM_SHUTDOWNINPROGRESS" => URT::CLR_E_SHIM_SHUTDOWNINPROGRESS,
            "VER_E_HRESULT" => URT::VER_E_HRESULT,
            "VER_E_OFFSET" => URT::VER_E_OFFSET,
            "VER_E_OPCODE" => URT::VER_E_OPCODE,
            "VER_E_OPERAND" => URT::VER_E_OPERAND,
            "VER_E_TOKEN" => URT::VER_E_TOKEN,
            "VER_E_EXCEPT" => URT::VER_E_EXCEPT,
            "VER_E_STACK_SLOT" => URT::VER_E_STACK_SLOT,
            "VER_E_LOC" => URT::VER_E_LOC,
            "VER_E_ARG" => URT::VER_E_ARG,
            "VER_E_FOUND" => URT::VER_E_FOUND,
            "VER_E_EXPECTED" => URT::VER_E_EXPECTED,
            "VER_E_LOC_BYNAME" => URT::VER_E_LOC_BYNAME,
            "VER_E_UNKNOWN_OPCODE" => URT::VER_E_UNKNOWN_OPCODE,
            "VER_E_SIG_CALLCONV" => URT::VER_E_SIG_CALLCONV,
            "VER_E_SIG_ELEMTYPE" => URT::VER_E_SIG_ELEMTYPE,
            "VER_E_RET_SIG" => URT::VER_E_RET_SIG,
            "VER_E_FIELD_SIG" => URT::VER_E_FIELD_SIG,
            "VER_E_INTERNAL" => URT::VER_E_INTERNAL,
            "VER_E_STACK_TOO_LARGE" => URT::VER_E_STACK_TOO_LARGE,
            "VER_E_ARRAY_NAME_LONG" => URT::VER_E_ARRAY_NAME_LONG,
            "VER_E_FALLTHRU" => URT::VER_E_FALLTHRU,
            "VER_E_TRY_GTEQ_END" => URT::VER_E_TRY_GTEQ_END,
            "VER_E_TRYEND_GT_CS" => URT::VER_E_TRYEND_GT_CS,
            "VER_E_HND_GTEQ_END" => URT::VER_E_HND_GTEQ_END,
            "VER_E_HNDEND_GT_CS" => URT::VER_E_HNDEND_GT_CS,
            "VER_E_FLT_GTEQ_CS" => URT::VER_E_FLT_GTEQ_CS,
            "VER_E_TRY_START" => URT::VER_E_TRY_START,
            "VER_E_HND_START" => URT::VER_E_HND_START,
            "VER_E_FLT_START" => URT::VER_E_FLT_START,
            "VER_E_TRY_OVERLAP" => URT::VER_E_TRY_OVERLAP,
            "VER_E_TRY_EQ_HND_FIL" => URT::VER_E_TRY_EQ_HND_FIL,
            "VER_E_TRY_SHARE_FIN_FAL" => URT::VER_E_TRY_SHARE_FIN_FAL,
            "VER_E_HND_OVERLAP" => URT::VER_E_HND_OVERLAP,
            "VER_E_HND_EQ" => URT::VER_E_HND_EQ,
            "VER_E_FIL_OVERLAP" => URT::VER_E_FIL_OVERLAP,
            "VER_E_FIL_EQ" => URT::VER_E_FIL_EQ,
            "VER_E_FIL_CONT_TRY" => URT::VER_E_FIL_CONT_TRY,
            "VER_E_FIL_CONT_HND" => URT::VER_E_FIL_CONT_HND,
            "VER_E_FIL_CONT_FIL" => URT::VER_E_FIL_CONT_FIL,
            "VER_E_FIL_GTEQ_CS" => URT::VER_E_FIL_GTEQ_CS,
            "VER_E_FIL_START" => URT::VER_E_FIL_START,
            "VER_E_FALLTHRU_EXCEP" => URT::VER_E_FALLTHRU_EXCEP,
            "VER_E_FALLTHRU_INTO_HND" => URT::VER_E_FALLTHRU_INTO_HND,
            "VER_E_FALLTHRU_INTO_FIL" => URT::VER_E_FALLTHRU_INTO_FIL,
            "VER_E_LEAVE" => URT::VER_E_LEAVE,
            "VER_E_RETHROW" => URT::VER_E_RETHROW,
            "VER_E_ENDFINALLY" => URT::VER_E_ENDFINALLY,
            "VER_E_ENDFILTER" => URT::VER_E_ENDFILTER,
            "VER_E_ENDFILTER_MISSING" => URT::VER_E_ENDFILTER_MISSING,
            "VER_E_BR_INTO_TRY" => URT::VER_E_BR_INTO_TRY,
            "VER_E_BR_INTO_HND" => URT::VER_E_BR_INTO_HND,
            "VER_E_BR_INTO_FIL" => URT::VER_E_BR_INTO_FIL,
            "VER_E_BR_OUTOF_TRY" => URT::VER_E_BR_OUTOF_TRY,
            "VER_E_BR_OUTOF_HND" => URT::VER_E_BR_OUTOF_HND,
            "VER_E_BR_OUTOF_FIL" => URT::VER_E_BR_OUTOF_FIL,
            "VER_E_BR_OUTOF_FIN" => URT::VER_E_BR_OUTOF_FIN,
            "VER_E_RET_FROM_TRY" => URT::VER_E_RET_FROM_TRY,
            "VER_E_RET_FROM_HND" => URT::VER_E_RET_FROM_HND,
            "VER_E_RET_FROM_FIL" => URT::VER_E_RET_FROM_FIL,
            "VER_E_BAD_JMP_TARGET" => URT::VER_E_BAD_JMP_TARGET,
            "VER_E_PATH_LOC" => URT::VER_E_PATH_LOC,
            "VER_E_PATH_THIS" => URT::VER_E_PATH_THIS,
            "VER_E_PATH_STACK" => URT::VER_E_PATH_STACK,
            "VER_E_PATH_STACK_DEPTH" => URT::VER_E_PATH_STACK_DEPTH,
            "VER_E_THIS" => URT::VER_E_THIS,
            "VER_E_THIS_UNINIT_EXCEP" => URT::VER_E_THIS_UNINIT_EXCEP,
            "VER_E_THIS_UNINIT_STORE" => URT::VER_E_THIS_UNINIT_STORE,
            "VER_E_THIS_UNINIT_RET" => URT::VER_E_THIS_UNINIT_RET,
            "VER_E_THIS_UNINIT_V_RET" => URT::VER_E_THIS_UNINIT_V_RET,
            "VER_E_THIS_UNINIT_BR" => URT::VER_E_THIS_UNINIT_BR,
            "VER_E_LDFTN_CTOR" => URT::VER_E_LDFTN_CTOR,
            "VER_E_STACK_NOT_EQ" => URT::VER_E_STACK_NOT_EQ,
            "VER_E_STACK_UNEXPECTED" => URT::VER_E_STACK_UNEXPECTED,
            "VER_E_STACK_EXCEPTION" => URT::VER_E_STACK_EXCEPTION,
            "VER_E_STACK_OVERFLOW" => URT::VER_E_STACK_OVERFLOW,
            "VER_E_STACK_UNDERFLOW" => URT::VER_E_STACK_UNDERFLOW,
            "VER_E_STACK_EMPTY" => URT::VER_E_STACK_EMPTY,
            "VER_E_STACK_UNINIT" => URT::VER_E_STACK_UNINIT,
            "VER_E_STACK_I_I4_I8" => URT::VER_E_STACK_I_I4_I8,
            "VER_E_STACK_R_R4_R8" => URT::VER_E_STACK_R_R4_R8,
            "VER_E_STACK_NO_R_I8" => URT::VER_E_STACK_NO_R_I8,
            "VER_E_STACK_NUMERIC" => URT::VER_E_STACK_NUMERIC,
            "VER_E_STACK_OBJREF" => URT::VER_E_STACK_OBJREF,
            "VER_E_STACK_P_OBJREF" => URT::VER_E_STACK_P_OBJREF,
            "VER_E_STACK_BYREF" => URT::VER_E_STACK_BYREF,
            "VER_E_STACK_METHOD" => URT::VER_E_STACK_METHOD,
            "VER_E_STACK_ARRAY_SD" => URT::VER_E_STACK_ARRAY_SD,
            "VER_E_STACK_VALCLASS" => URT::VER_E_STACK_VALCLASS,
            "VER_E_STACK_P_VALCLASS" => URT::VER_E_STACK_P_VALCLASS,
            "VER_E_STACK_NO_VALCLASS" => URT::VER_E_STACK_NO_VALCLASS,
            "VER_E_LOC_DEAD" => URT::VER_E_LOC_DEAD,
            "VER_E_LOC_NUM" => URT::VER_E_LOC_NUM,
            "VER_E_ARG_NUM" => URT::VER_E_ARG_NUM,
            "VER_E_TOKEN_RESOLVE" => URT::VER_E_TOKEN_RESOLVE,
            "VER_E_TOKEN_TYPE" => URT::VER_E_TOKEN_TYPE,
            "VER_E_TOKEN_TYPE_MEMBER" => URT::VER_E_TOKEN_TYPE_MEMBER,
            "VER_E_TOKEN_TYPE_FIELD" => URT::VER_E_TOKEN_TYPE_FIELD,
            "VER_E_TOKEN_TYPE_SIG" => URT::VER_E_TOKEN_TYPE_SIG,
            "VER_E_UNVERIFIABLE" => URT::VER_E_UNVERIFIABLE,
            "VER_E_LDSTR_OPERAND" => URT::VER_E_LDSTR_OPERAND,
            "VER_E_RET_PTR_TO_STACK" => URT::VER_E_RET_PTR_TO_STACK,
            "VER_E_RET_VOID" => URT::VER_E_RET_VOID,
            "VER_E_RET_MISSING" => URT::VER_E_RET_MISSING,
            "VER_E_RET_EMPTY" => URT::VER_E_RET_EMPTY,
            "VER_E_RET_UNINIT" => URT::VER_E_RET_UNINIT,
            "VER_E_ARRAY_ACCESS" => URT::VER_E_ARRAY_ACCESS,
            "VER_E_ARRAY_V_STORE" => URT::VER_E_ARRAY_V_STORE,
            "VER_E_ARRAY_SD" => URT::VER_E_ARRAY_SD,
            "VER_E_ARRAY_SD_PTR" => URT::VER_E_ARRAY_SD_PTR,
            "VER_E_ARRAY_FIELD" => URT::VER_E_ARRAY_FIELD,
            "VER_E_ARGLIST" => URT::VER_E_ARGLIST,
            "VER_E_VALCLASS" => URT::VER_E_VALCLASS,
            "VER_E_METHOD_ACCESS" => URT::VER_E_METHOD_ACCESS,
            "VER_E_FIELD_ACCESS" => URT::VER_E_FIELD_ACCESS,
            "VER_E_DEAD" => URT::VER_E_DEAD,
            "VER_E_FIELD_STATIC" => URT::VER_E_FIELD_STATIC,
            "VER_E_FIELD_NO_STATIC" => URT::VER_E_FIELD_NO_STATIC,
            "VER_E_ADDR" => URT::VER_E_ADDR,
            "VER_E_ADDR_BYREF" => URT::VER_E_ADDR_BYREF,
            "VER_E_ADDR_LITERAL" => URT::VER_E_ADDR_LITERAL,
            "VER_E_INITONLY" => URT::VER_E_INITONLY,
            "VER_E_THROW" => URT::VER_E_THROW,
            "VER_E_CALLVIRT_VALCLASS" => URT::VER_E_CALLVIRT_VALCLASS,
            "VER_E_CALL_SIG" => URT::VER_E_CALL_SIG,
            "VER_E_CALL_STATIC" => URT::VER_E_CALL_STATIC,
            "VER_E_CTOR" => URT::VER_E_CTOR,
            "VER_E_CTOR_VIRT" => URT::VER_E_CTOR_VIRT,
            "VER_E_CTOR_OR_SUPER" => URT::VER_E_CTOR_OR_SUPER,
            "VER_E_CTOR_MUL_INIT" => URT::VER_E_CTOR_MUL_INIT,
            "VER_E_SIG" => URT::VER_E_SIG,
            "VER_E_SIG_ARRAY" => URT::VER_E_SIG_ARRAY,
            "VER_E_SIG_ARRAY_PTR" => URT::VER_E_SIG_ARRAY_PTR,
            "VER_E_SIG_ARRAY_BYREF" => URT::VER_E_SIG_ARRAY_BYREF,
            "VER_E_SIG_ELEM_PTR" => URT::VER_E_SIG_ELEM_PTR,
            "VER_E_SIG_VARARG" => URT::VER_E_SIG_VARARG,
            "VER_E_SIG_VOID" => URT::VER_E_SIG_VOID,
            "VER_E_SIG_BYREF_BYREF" => URT::VER_E_SIG_BYREF_BYREF,
            "VER_E_CODE_SIZE_ZERO" => URT::VER_E_CODE_SIZE_ZERO,
            "VER_E_BAD_VARARG" => URT::VER_E_BAD_VARARG,
            "VER_E_TAIL_CALL" => URT::VER_E_TAIL_CALL,
            "VER_E_TAIL_BYREF" => URT::VER_E_TAIL_BYREF,
            "VER_E_TAIL_RET" => URT::VER_E_TAIL_RET,
            "VER_E_TAIL_RET_VOID" => URT::VER_E_TAIL_RET_VOID,
            "VER_E_TAIL_RET_TYPE" => URT::VER_E_TAIL_RET_TYPE,
            "VER_E_TAIL_STACK_EMPTY" => URT::VER_E_TAIL_STACK_EMPTY,
            "VER_E_METHOD_END" => URT::VER_E_METHOD_END,
            "VER_E_BAD_BRANCH" => URT::VER_E_BAD_BRANCH,
            "VER_E_FIN_OVERLAP" => URT::VER_E_FIN_OVERLAP,
            "VER_E_LEXICAL_NESTING" => URT::VER_E_LEXICAL_NESTING,
            "VER_E_VOLATILE" => URT::VER_E_VOLATILE,
            "VER_E_UNALIGNED" => URT::VER_E_UNALIGNED,
            "VER_E_INNERMOST_FIRST" => URT::VER_E_INNERMOST_FIRST,
            "VER_E_CALLI_VIRTUAL" => URT::VER_E_CALLI_VIRTUAL,
            "VER_E_CALL_ABSTRACT" => URT::VER_E_CALL_ABSTRACT,
            "VER_E_STACK_UNEXP_ARRAY" => URT::VER_E_STACK_UNEXP_ARRAY,
            "VER_E_NOT_IN_GC_HEAP" => URT::VER_E_NOT_IN_GC_HEAP,
            "VER_E_TRY_N_EMPTY_STACK" => URT::VER_E_TRY_N_EMPTY_STACK,
            "VER_E_DLGT_CTOR" => URT::VER_E_DLGT_CTOR,
            "VER_E_DLGT_BB" => URT::VER_E_DLGT_BB,
            "VER_E_DLGT_PATTERN" => URT::VER_E_DLGT_PATTERN,
            "VER_E_DLGT_LDFTN" => URT::VER_E_DLGT_LDFTN,
            "VER_E_FTN_ABSTRACT" => URT::VER_E_FTN_ABSTRACT,
            "VER_E_SIG_C_VC" => URT::VER_E_SIG_C_VC,
            "VER_E_SIG_VC_C" => URT::VER_E_SIG_VC_C,
            "VER_E_BOX_PTR_TO_STACK" => URT::VER_E_BOX_PTR_TO_STACK,
            "VER_E_SIG_BYREF_TB_AH" => URT::VER_E_SIG_BYREF_TB_AH,
            "VER_E_SIG_ARRAY_TB_AH" => URT::VER_E_SIG_ARRAY_TB_AH,
            "VER_E_ENDFILTER_STACK" => URT::VER_E_ENDFILTER_STACK,
            "VER_E_DLGT_SIG_I" => URT::VER_E_DLGT_SIG_I,
            "VER_E_DLGT_SIG_O" => URT::VER_E_DLGT_SIG_O,
            "VER_E_RA_PTR_TO_STACK" => URT::VER_E_RA_PTR_TO_STACK,
            "VER_E_CATCH_VALUE_TYPE" => URT::VER_E_CATCH_VALUE_TYPE,
            "VER_E_CATCH_BYREF" => URT::VER_E_CATCH_BYREF,
            "VER_E_FIL_PRECEED_HND" => URT::VER_E_FIL_PRECEED_HND,
            "VER_E_LDVIRTFTN_STATIC" => URT::VER_E_LDVIRTFTN_STATIC,
            "VER_E_CALLVIRT_STATIC" => URT::VER_E_CALLVIRT_STATIC,
            "VER_E_INITLOCALS" => URT::VER_E_INITLOCALS,
            "VER_E_BR_TO_EXCEPTION" => URT::VER_E_BR_TO_EXCEPTION,
            "VER_E_CALL_CTOR" => URT::VER_E_CALL_CTOR,
            "VER_E_VALCLASS_OBJREF_VAR" => URT::VER_E_VALCLASS_OBJREF_VAR,
            "VER_E_STACK_P_VALCLASS_OBJREF_VAR" => URT::VER_E_STACK_P_VALCLASS_OBJREF_VAR,
            "VER_E_SIG_VAR_PARAM" => URT::VER_E_SIG_VAR_PARAM,
            "VER_E_SIG_MVAR_PARAM" => URT::VER_E_SIG_MVAR_PARAM,
            "VER_E_SIG_VAR_ARG" => URT::VER_E_SIG_VAR_ARG,
            "VER_E_SIG_MVAR_ARG" => URT::VER_E_SIG_MVAR_ARG,
            "VER_E_SIG_GENERICINST" => URT::VER_E_SIG_GENERICINST,
            "VER_E_SIG_METHOD_INST" => URT::VER_E_SIG_METHOD_INST,
            "VER_E_SIG_METHOD_PARENT_INST" => URT::VER_E_SIG_METHOD_PARENT_INST,
            "VER_E_SIG_FIELD_PARENT_INST" => URT::VER_E_SIG_FIELD_PARENT_INST,
            "VER_E_CALLCONV_NOT_GENERICINST" => URT::VER_E_CALLCONV_NOT_GENERICINST,
            "VER_E_TOKEN_BAD_METHOD_SPEC" => URT::VER_E_TOKEN_BAD_METHOD_SPEC,
            "VER_E_BAD_READONLY_PREFIX" => URT::VER_E_BAD_READONLY_PREFIX,
            "VER_E_BAD_CONSTRAINED_PREFIX" => URT::VER_E_BAD_CONSTRAINED_PREFIX,
            "VER_E_CIRCULAR_VAR_CONSTRAINTS" => URT::VER_E_CIRCULAR_VAR_CONSTRAINTS,
            "VER_E_CIRCULAR_MVAR_CONSTRAINTS" => URT::VER_E_CIRCULAR_MVAR_CONSTRAINTS,
            "VER_E_UNSATISFIED_METHOD_INST" => URT::VER_E_UNSATISFIED_METHOD_INST,
            "VER_E_UNSATISFIED_METHOD_PARENT_INST" => URT::VER_E_UNSATISFIED_METHOD_PARENT_INST,
            "VER_E_UNSATISFIED_FIELD_PARENT_INST" => URT::VER_E_UNSATISFIED_FIELD_PARENT_INST,
            "VER_E_UNSATISFIED_BOX_OPERAND" => URT::VER_E_UNSATISFIED_BOX_OPERAND,
            "VER_E_CONSTRAINED_CALL_WITH_NON_BYREF_THIS" => URT::VER_E_CONSTRAINED_CALL_WITH_NON_BYREF_THIS,
            "VER_E_CONSTRAINED_OF_NON_VARIABLE_TYPE" => URT::VER_E_CONSTRAINED_OF_NON_VARIABLE_TYPE,
            "VER_E_READONLY_UNEXPECTED_CALLEE" => URT::VER_E_READONLY_UNEXPECTED_CALLEE,
            "VER_E_READONLY_ILLEGAL_WRITE" => URT::VER_E_READONLY_ILLEGAL_WRITE,
            "VER_E_READONLY_IN_MKREFANY" => URT::VER_E_READONLY_IN_MKREFANY,
            "VER_E_UNALIGNED_ALIGNMENT" => URT::VER_E_UNALIGNED_ALIGNMENT,
            "VER_E_TAILCALL_INSIDE_EH" => URT::VER_E_TAILCALL_INSIDE_EH,
            "VER_E_BACKWARD_BRANCH" => URT::VER_E_BACKWARD_BRANCH,
            "VER_E_CALL_TO_VTYPE_BASE" => URT::VER_E_CALL_TO_VTYPE_BASE,
            "VER_E_NEWOBJ_OF_ABSTRACT_CLASS" => URT::VER_E_NEWOBJ_OF_ABSTRACT_CLASS,
            "VER_E_UNMANAGED_POINTER" => URT::VER_E_UNMANAGED_POINTER,
            "VER_E_LDFTN_NON_FINAL_VIRTUAL" => URT::VER_E_LDFTN_NON_FINAL_VIRTUAL,
            "VER_E_FIELD_OVERLAP" => URT::VER_E_FIELD_OVERLAP,
            "VER_E_THIS_MISMATCH" => URT::VER_E_THIS_MISMATCH,
            "VER_E_STACK_I_I4" => URT::VER_E_STACK_I_I4,
            "VER_E_BAD_PE" => URT::VER_E_BAD_PE,
            "VER_E_BAD_MD" => URT::VER_E_BAD_MD,
            "VER_E_BAD_APPDOMAIN" => URT::VER_E_BAD_APPDOMAIN,
            "VER_E_TYPELOAD" => URT::VER_E_TYPELOAD,
            "VER_E_PE_LOAD" => URT::VER_E_PE_LOAD,
            "VER_E_WRITE_RVA_STATIC" => URT::VER_E_WRITE_RVA_STATIC,
            "COR_E_SqlException" => URT::COR_E_SqlException,
            "COR_E_Data" => URT::COR_E_Data,
            "COR_E_DataDeletedRowInaccessible" => URT::COR_E_DataDeletedRowInaccessible,
            "COR_E_DataDuplicateName" => URT::COR_E_DataDuplicateName,
            "COR_E_DataInRowChangingEvent" => URT::COR_E_DataInRowChangingEvent,
            "COR_E_DataInvalidConstraint" => URT::COR_E_DataInvalidConstraint,
            "COR_E_DataMissingPrimaryKey" => URT::COR_E_DataMissingPrimaryKey,
            "COR_E_DataNoNullAllowed" => URT::COR_E_DataNoNullAllowed,
            "COR_E_DataReadOnly" => URT::COR_E_DataReadOnly,
            "COR_E_DataRowNotInTable" => URT::COR_E_DataRowNotInTable,
            "COR_E_DataVersionNotFound" => URT::COR_E_DataVersionNotFound,
            "COR_E_DataConstraint" => URT::COR_E_DataConstraint,
            "COR_E_StrongTyping" => URT::COR_E_StrongTyping,
            "COR_E_SqlType" => URT::COR_E_SqlType,
            "COR_E_SqlNullValue" => URT::COR_E_SqlNullValue,
            "COR_E_SqlTruncate" => URT::COR_E_SqlTruncate,
            "COR_E_AdapterMapping" => URT::COR_E_AdapterMapping,
            "COR_E_DataAdapter" => URT::COR_E_DataAdapter,
            "COR_E_DBConcurrency" => URT::COR_E_DBConcurrency,
            "COR_E_OperationAborted" => URT::COR_E_OperationAborted,
            "COR_E_InvalidUdt" => URT::COR_E_InvalidUdt,
            "COR_E_OdbcException" => URT::COR_E_OdbcException,
            "COR_E_OracleException" => URT::COR_E_OracleException,
            "COR_E_Xml" => URT::COR_E_Xml,
            "COR_E_XmlSchema" => URT::COR_E_XmlSchema,
            "COR_E_XmlXslt" => URT::COR_E_XmlXslt,
            "COR_E_XmlXPath" => URT::COR_E_XmlXPath,
            "COR_E_XmlQuery" => URT::COR_E_XmlQuery,
            "VLDTR_E_IFACE_NOTIFACE" => URT::VLDTR_E_IFACE_NOTIFACE,
            "TLBX_E_CIRCULAR_EXPORT2" => URT::TLBX_E_CIRCULAR_EXPORT2,
            "VLDTR_E_FD_RVAHASNORVA" => URT::VLDTR_E_FD_RVAHASNORVA,
            "VLDTR_E_FD_RVAHASZERORVA" => URT::VLDTR_E_FD_RVAHASZERORVA,
            "VLDTR_E_MD_RVAANDIMPLMAP" => URT::VLDTR_E_MD_RVAANDIMPLMAP,
            "VLDTR_E_TD_EXTRAFLAGS" => URT::VLDTR_E_TD_EXTRAFLAGS,
            "VLDTR_E_TD_EXTENDSITSELF" => URT::VLDTR_E_TD_EXTENDSITSELF,
            "VLDTR_E_TD_SYSVTNOTEXTOBJ" => URT::VLDTR_E_TD_SYSVTNOTEXTOBJ,
            "VLDTR_E_TD_EXTTYPESPEC" => URT::VLDTR_E_TD_EXTTYPESPEC,
            "VLDTR_E_TD_VTNOSIZE" => URT::VLDTR_E_TD_VTNOSIZE,
            "VLDTR_E_TD_IFACESEALED" => URT::VLDTR_E_TD_IFACESEALED,
            "VLDTR_E_NC_BADNESTED" => URT::VLDTR_E_NC_BADNESTED,
            "VLDTR_E_NC_BADENCLOSER" => URT::VLDTR_E_NC_BADENCLOSER,
            "VLDTR_E_NC_DUP" => URT::VLDTR_E_NC_DUP,
            "VLDTR_E_NC_DUPENCLOSER" => URT::VLDTR_E_NC_DUPENCLOSER,
            "VLDTR_E_FRVA_ZERORVA" => URT::VLDTR_E_FRVA_ZERORVA,
            "VLDTR_E_FRVA_BADFIELD" => URT::VLDTR_E_FRVA_BADFIELD,
            "VLDTR_E_FRVA_DUPRVA" => URT::VLDTR_E_FRVA_DUPRVA,
            "VLDTR_E_FRVA_DUPFIELD" => URT::VLDTR_E_FRVA_DUPFIELD,
            "VLDTR_E_EP_BADTOKEN" => URT::VLDTR_E_EP_BADTOKEN,
            "VLDTR_E_EP_INSTANCE" => URT::VLDTR_E_EP_INSTANCE,
            "VLDTR_E_TD_ENUMFLDBADTYPE" => URT::VLDTR_E_TD_ENUMFLDBADTYPE,
            "VLDTR_E_MD_BADRVA" => URT::VLDTR_E_MD_BADRVA,
            "VLDTR_E_FD_LITERALNODEFAULT" => URT::VLDTR_E_FD_LITERALNODEFAULT,
            "VLDTR_E_IFACE_METHNOTIMPL" => URT::VLDTR_E_IFACE_METHNOTIMPL,
            "VLDTR_E_CA_BADPARENT" => URT::VLDTR_E_CA_BADPARENT,
            "VLDTR_E_CA_BADTYPE" => URT::VLDTR_E_CA_BADTYPE,
            "VLDTR_E_CA_NOTCTOR" => URT::VLDTR_E_CA_NOTCTOR,
            "VLDTR_E_CA_BADSIG" => URT::VLDTR_E_CA_BADSIG,
            "VLDTR_E_CA_NOSIG" => URT::VLDTR_E_CA_NOSIG,
            "VLDTR_E_CA_BADPROLOG" => URT::VLDTR_E_CA_BADPROLOG,
            "VLDTR_E_MD_BADLOCALSIGTOK" => URT::VLDTR_E_MD_BADLOCALSIGTOK,
            "VLDTR_E_MD_BADHEADER" => URT::VLDTR_E_MD_BADHEADER,
            "VLDTR_E_EP_TOOMANYARGS" => URT::VLDTR_E_EP_TOOMANYARGS,
            "VLDTR_E_EP_BADRET" => URT::VLDTR_E_EP_BADRET,
            "VLDTR_E_EP_BADARG" => URT::VLDTR_E_EP_BADARG,
            "VLDTR_E_SIG_BADVOID" => URT::VLDTR_E_SIG_BADVOID,
            "VLDTR_E_IFACE_METHMULTIMPL" => URT::VLDTR_E_IFACE_METHMULTIMPL,
            "VLDTR_E_GP_NAMENULL" => URT::VLDTR_E_GP_NAMENULL,
            "VLDTR_E_GP_OWNERNIL" => URT::VLDTR_E_GP_OWNERNIL,
            "VLDTR_E_GP_DUPNAME" => URT::VLDTR_E_GP_DUPNAME,
            "VLDTR_E_GP_DUPNUMBER" => URT::VLDTR_E_GP_DUPNUMBER,
            "VLDTR_E_GP_NONSEQ_BY_OWNER" => URT::VLDTR_E_GP_NONSEQ_BY_OWNER,
            "VLDTR_E_GP_NONSEQ_BY_NUMBER" => URT::VLDTR_E_GP_NONSEQ_BY_NUMBER,
            "VLDTR_E_GP_UNEXPECTED_OWNER_FOR_VARIANT_VAR" => URT::VLDTR_E_GP_UNEXPECTED_OWNER_FOR_VARIANT_VAR,
            "VLDTR_E_GP_ILLEGAL_VARIANT_MVAR" => URT::VLDTR_E_GP_ILLEGAL_VARIANT_MVAR,
            "VLDTR_E_GP_ILLEGAL_VARIANCE_FLAGS" => URT::VLDTR_E_GP_ILLEGAL_VARIANCE_FLAGS,
            "VLDTR_E_GP_REFANDVALUETYPE" => URT::VLDTR_E_GP_REFANDVALUETYPE,
            "VLDTR_E_GPC_OWNERNIL" => URT::VLDTR_E_GPC_OWNERNIL,
            "VLDTR_E_GPC_DUP" => URT::VLDTR_E_GPC_DUP,
            "VLDTR_E_GPC_NONCONTIGUOUS" => URT::VLDTR_E_GPC_NONCONTIGUOUS,
            "VLDTR_E_MS_METHODNIL" => URT::VLDTR_E_MS_METHODNIL,
            "VLDTR_E_MS_DUP" => URT::VLDTR_E_MS_DUP,
            "VLDTR_E_MS_BADCALLINGCONV" => URT::VLDTR_E_MS_BADCALLINGCONV,
            "VLDTR_E_MS_MISSARITY" => URT::VLDTR_E_MS_MISSARITY,
            "VLDTR_E_MS_MISSARG" => URT::VLDTR_E_MS_MISSARG,
            "VLDTR_E_MS_ARITYMISMATCH" => URT::VLDTR_E_MS_ARITYMISMATCH,
            "VLDTR_E_MS_METHODNOTGENERIC" => URT::VLDTR_E_MS_METHODNOTGENERIC,
            "VLDTR_E_SIG_MISSARITY" => URT::VLDTR_E_SIG_MISSARITY,
            "VLDTR_E_SIG_ARITYMISMATCH" => URT::VLDTR_E_SIG_ARITYMISMATCH,
            "VLDTR_E_MD_GENERIC_CCTOR" => URT::VLDTR_E_MD_GENERIC_CCTOR,
            "VLDTR_E_MD_GENERIC_CTOR" => URT::VLDTR_E_MD_GENERIC_CTOR,
            "VLDTR_E_MD_GENERIC_IMPORT" => URT::VLDTR_E_MD_GENERIC_IMPORT,
            "VLDTR_E_MD_GENERIC_BADCALLCONV" => URT::VLDTR_E_MD_GENERIC_BADCALLCONV,
            "VLDTR_E_MD_GENERIC_GLOBAL" => URT::VLDTR_E_MD_GENERIC_GLOBAL,
            "VLDTR_E_EP_GENERIC_METHOD" => URT::VLDTR_E_EP_GENERIC_METHOD,
            "VLDTR_E_MD_MISSARITY" => URT::VLDTR_E_MD_MISSARITY,
            "VLDTR_E_MD_ARITYZERO" => URT::VLDTR_E_MD_ARITYZERO,
            "VLDTR_E_SIG_ARITYZERO" => URT::VLDTR_E_SIG_ARITYZERO,
            "VLDTR_E_MS_ARITYZERO" => URT::VLDTR_E_MS_ARITYZERO,
            "VLDTR_E_MD_GPMISMATCH" => URT::VLDTR_E_MD_GPMISMATCH,
            "VLDTR_E_EP_GENERIC_TYPE" => URT::VLDTR_E_EP_GENERIC_TYPE,
            "VLDTR_E_MI_DECLNOTGENERIC" => URT::VLDTR_E_MI_DECLNOTGENERIC,
            "VLDTR_E_MI_IMPLNOTGENERIC" => URT::VLDTR_E_MI_IMPLNOTGENERIC,
            "VLDTR_E_MI_ARITYMISMATCH" => URT::VLDTR_E_MI_ARITYMISMATCH,
            "VLDTR_E_TD_EXTBADTYPESPEC" => URT::VLDTR_E_TD_EXTBADTYPESPEC,
            "VLDTR_E_SIG_BYREFINST" => URT::VLDTR_E_SIG_BYREFINST,
            "VLDTR_E_MS_BYREFINST" => URT::VLDTR_E_MS_BYREFINST,
            "VLDTR_E_TS_EMPTY" => URT::VLDTR_E_TS_EMPTY,
            "VLDTR_E_TS_HASSENTINALS" => URT::VLDTR_E_TS_HASSENTINALS,
            "VLDTR_E_TD_GENERICHASEXPLAYOUT" => URT::VLDTR_E_TD_GENERICHASEXPLAYOUT,
            "VLDTR_E_SIG_BADTOKTYPE" => URT::VLDTR_E_SIG_BADTOKTYPE,
            "VLDTR_E_IFACE_METHNOTIMPLTHISMOD" => URT::VLDTR_E_IFACE_METHNOTIMPLTHISMOD,
            "CORDBG_E_THREAD_NOT_SCHEDULED" => URT::CORDBG_E_THREAD_NOT_SCHEDULED,
            "CORDBG_E_HANDLE_HAS_BEEN_DISPOSED" => URT::CORDBG_E_HANDLE_HAS_BEEN_DISPOSED,
            "CORDBG_E_NONINTERCEPTABLE_EXCEPTION" => URT::CORDBG_E_NONINTERCEPTABLE_EXCEPTION,
            "CORDBG_E_CANT_UNWIND_ABOVE_CALLBACK" => URT::CORDBG_E_CANT_UNWIND_ABOVE_CALLBACK,
            "CORDBG_E_INTERCEPT_FRAME_ALREADY_SET" => URT::CORDBG_E_INTERCEPT_FRAME_ALREADY_SET,
            "CORDBG_E_NO_NATIVE_PATCH_AT_ADDR" => URT::CORDBG_E_NO_NATIVE_PATCH_AT_ADDR,
            "CORDBG_E_MUST_BE_INTEROP_DEBUGGING" => URT::CORDBG_E_MUST_BE_INTEROP_DEBUGGING,
            "CORDBG_E_NATIVE_PATCH_ALREADY_AT_ADDR" => URT::CORDBG_E_NATIVE_PATCH_ALREADY_AT_ADDR,
            "CORDBG_E_TIMEOUT" => URT::CORDBG_E_TIMEOUT,
            "CORDBG_E_CANT_CALL_ON_THIS_THREAD" => URT::CORDBG_E_CANT_CALL_ON_THIS_THREAD,
            "CORDBG_E_ENC_INFOLESS_METHOD" => URT::CORDBG_E_ENC_INFOLESS_METHOD,
            "CORDBG_E_ENC_NESTED_HANLDERS" => URT::CORDBG_E_ENC_NESTED_HANLDERS,
            "CORDBG_E_ENC_IN_FUNCLET" => URT::CORDBG_E_ENC_IN_FUNCLET,
            "CORDBG_E_ENC_LOCALLOC" => URT::CORDBG_E_ENC_LOCALLOC,
            "CORDBG_E_ENC_EDIT_NOT_SUPPORTED" => URT::CORDBG_E_ENC_EDIT_NOT_SUPPORTED,
            "CORDBG_E_FEABORT_DELAYED_UNTIL_THREAD_RESUMED" => URT::CORDBG_E_FEABORT_DELAYED_UNTIL_THREAD_RESUMED,
            "CORDBG_E_NOTREADY" => URT::CORDBG_E_NOTREADY,
            "CORDBG_E_CANNOT_RESOLVE_ASSEMBLY" => URT::CORDBG_E_CANNOT_RESOLVE_ASSEMBLY,
            "CORDBG_E_MUST_BE_IN_LOAD_MODULE" => URT::CORDBG_E_MUST_BE_IN_LOAD_MODULE,
            "CORDBG_E_CANNOT_BE_ON_ATTACH" => URT::CORDBG_E_CANNOT_BE_ON_ATTACH,
            "CORDBG_E_NGEN_NOT_SUPPORTED" => URT::CORDBG_E_NGEN_NOT_SUPPORTED,
            "CORDBG_E_ILLEGAL_SHUTDOWN_ORDER" => URT::CORDBG_E_ILLEGAL_SHUTDOWN_ORDER,
            "CORDBG_E_CANNOT_DEBUG_FIBER_PROCESS" => URT::CORDBG_E_CANNOT_DEBUG_FIBER_PROCESS,
            "CORDBG_E_MUST_BE_IN_CREATE_PROCESS" => URT::CORDBG_E_MUST_BE_IN_CREATE_PROCESS,
            "CORDBG_E_DETACH_FAILED_OUTSTANDING_EVALS" => URT::CORDBG_E_DETACH_FAILED_OUTSTANDING_EVALS,
            "CORDBG_E_DETACH_FAILED_OUTSTANDING_STEPPERS" => URT::CORDBG_E_DETACH_FAILED_OUTSTANDING_STEPPERS,
            "CORDBG_E_CANT_INTEROP_STEP_OUT" => URT::CORDBG_E_CANT_INTEROP_STEP_OUT,
            "CORDBG_E_DETACH_FAILED_OUTSTANDING_BREAKPOINTS" => URT::CORDBG_E_DETACH_FAILED_OUTSTANDING_BREAKPOINTS,
            "CORDBG_E_ILLEGAL_IN_STACK_OVERFLOW" => URT::CORDBG_E_ILLEGAL_IN_STACK_OVERFLOW,
            "CORDBG_E_ILLEGAL_AT_GC_UNSAFE_POINT" => URT::CORDBG_E_ILLEGAL_AT_GC_UNSAFE_POINT,
            "CORDBG_E_ILLEGAL_IN_PROLOG" => URT::CORDBG_E_ILLEGAL_IN_PROLOG,
            "CORDBG_E_ILLEGAL_IN_NATIVE_CODE" => URT::CORDBG_E_ILLEGAL_IN_NATIVE_CODE,
            "CORDBG_E_ILLEGAL_IN_OPTIMIZED_CODE" => URT::CORDBG_E_ILLEGAL_IN_OPTIMIZED_CODE,
            "CORDBG_E_MINIDUMP_UNSUPPORTED" => URT::CORDBG_E_MINIDUMP_UNSUPPORTED,
            "CORDBG_E_APPDOMAIN_MISMATCH" => URT::CORDBG_E_APPDOMAIN_MISMATCH,
            "CORDBG_E_CONTEXT_UNVAILABLE" => URT::CORDBG_E_CONTEXT_UNVAILABLE,
            "CORDBG_E_UNCOMPATIBLE_PLATFORMS" => URT::CORDBG_E_UNCOMPATIBLE_PLATFORMS,
            "CORDBG_E_DEBUGGING_DISABLED" => URT::CORDBG_E_DEBUGGING_DISABLED,
            "CORDBG_E_DETACH_FAILED_ON_ENC" => URT::CORDBG_E_DETACH_FAILED_ON_ENC,
            "CORDBG_E_CURRENT_EXCEPTION_IS_OUTSIDE_CURRENT_EXECUTION_SCOPE" => URT::CORDBG_E_CURRENT_EXCEPTION_IS_OUTSIDE_CURRENT_EXECUTION_SCOPE,
            "CORDBG_E_HELPER_MAY_DEADLOCK" => URT::CORDBG_E_HELPER_MAY_DEADLOCK,
            "CORDBG_E_MISSING_METADATA" => URT::CORDBG_E_MISSING_METADATA,
            "CORDBG_E_TARGET_INCONSISTENT" => URT::CORDBG_E_TARGET_INCONSISTENT,
            "CORDBG_E_DETACH_FAILED_OUTSTANDING_TARGET_RESOURCES" => URT::CORDBG_E_DETACH_FAILED_OUTSTANDING_TARGET_RESOURCES,
            "CORDBG_E_TARGET_READONLY" => URT::CORDBG_E_TARGET_READONLY,
            "CORDBG_E_MISMATCHED_CORWKS_AND_DACWKS_DLLS" => URT::CORDBG_E_MISMATCHED_CORWKS_AND_DACWKS_DLLS,
            "CORDBG_E_MODULE_LOADED_FROM_DISK" => URT::CORDBG_E_MODULE_LOADED_FROM_DISK,
            "CORDBG_E_SYMBOLS_NOT_AVAILABLE" => URT::CORDBG_E_SYMBOLS_NOT_AVAILABLE,
            "CORDBG_E_DEBUG_COMPONENT_MISSING" => URT::CORDBG_E_DEBUG_COMPONENT_MISSING,
            "CORDBG_E_REMOTE_MISMATCHED_CERTS" => URT::CORDBG_E_REMOTE_MISMATCHED_CERTS,
            "CORDBG_E_REMOTE_NETWORK_FAILURE" => URT::CORDBG_E_REMOTE_NETWORK_FAILURE,
            "CORDBG_E_REMOTE_NO_LISTENER" => URT::CORDBG_E_REMOTE_NO_LISTENER,
            "CORDBG_E_REMOTE_UNKNOWN_TARGET" => URT::CORDBG_E_REMOTE_UNKNOWN_TARGET,
            "CORDBG_E_REMOTE_INVALID_CONFIG" => URT::CORDBG_E_REMOTE_INVALID_CONFIG,
            "CORDBG_E_REMOTE_MISMATCHED_PROTOCOLS" => URT::CORDBG_E_REMOTE_MISMATCHED_PROTOCOLS,
            "CORDBG_E_LIBRARY_PROVIDER_ERROR" => URT::CORDBG_E_LIBRARY_PROVIDER_ERROR,
            "CORDBG_E_NOT_CLR" => URT::CORDBG_E_NOT_CLR,
            "CORDBG_E_MISSING_DATA_TARGET_INTERFACE" => URT::CORDBG_E_MISSING_DATA_TARGET_INTERFACE,
            "CORDBG_E_UNSUPPORTED_DEBUGGING_MODEL" => URT::CORDBG_E_UNSUPPORTED_DEBUGGING_MODEL,
            "CORDBG_E_UNSUPPORTED_FORWARD_COMPAT" => URT::CORDBG_E_UNSUPPORTED_FORWARD_COMPAT,
            "CORDBG_E_UNSUPPORTED_VERSION_STRUCT" => URT::CORDBG_E_UNSUPPORTED_VERSION_STRUCT,
            "CORDBG_E_READVIRTUAL_FAILURE" => URT::CORDBG_E_READVIRTUAL_FAILURE,
            "CORDBG_E_VALUE_POINTS_TO_FUNCTION" => URT::CORDBG_E_VALUE_POINTS_TO_FUNCTION,
            "CORDBG_E_CORRUPT_OBJECT" => URT::CORDBG_E_CORRUPT_OBJECT,
            "CORDBG_E_GC_STRUCTURES_INVALID" => URT::CORDBG_E_GC_STRUCTURES_INVALID,
            "CORDBG_E_INVALID_OPCODE" => URT::CORDBG_E_INVALID_OPCODE,
            "CORDBG_E_UNSUPPORTED" => URT::CORDBG_E_UNSUPPORTED,
            "CORDBG_E_MISSING_DEBUGGER_EXPORTS" => URT::CORDBG_E_MISSING_DEBUGGER_EXPORTS,
            "PEFMT_E_NO_CONTENTS" => URT::PEFMT_E_NO_CONTENTS,
            "PEFMT_E_NO_NTHEADERS" => URT::PEFMT_E_NO_NTHEADERS,
            "PEFMT_E_64BIT" => URT::PEFMT_E_64BIT,
            "PEFMT_E_NO_CORHEADER" => URT::PEFMT_E_NO_CORHEADER,
            "PEFMT_E_NOT_ILONLY" => URT::PEFMT_E_NOT_ILONLY,
            "PEFMT_E_IMPORT_DLLS" => URT::PEFMT_E_IMPORT_DLLS,
            "PEFMT_E_EXE_NOENTRYPOINT" => URT::PEFMT_E_EXE_NOENTRYPOINT,
            "PEFMT_E_BASE_RELOCS" => URT::PEFMT_E_BASE_RELOCS,
            "PEFMT_E_ENTRYPOINT" => URT::PEFMT_E_ENTRYPOINT,
            "PEFMT_E_ZERO_SIZEOFCODE" => URT::PEFMT_E_ZERO_SIZEOFCODE,
            "PEFMT_E_BAD_CORHEADER" => URT::PEFMT_E_BAD_CORHEADER,
            "PEFMT_E_32BIT" => URT::PEFMT_E_32BIT,
            "CLR_OPTSVC_E_CONTROLLER_INTERRUPT" => URT::CLR_OPTSVC_E_CONTROLLER_INTERRUPT,
            "NGEN_FAILED_GET_DEPENDENCIES" => URT::NGEN_FAILED_GET_DEPENDENCIES,
            "NGEN_FAILED_NATIVE_IMAGE_DELETE" => URT::NGEN_FAILED_NATIVE_IMAGE_DELETE,
            "NGEN_E_TOO_MANY_INTERFACES" => URT::NGEN_E_TOO_MANY_INTERFACES,
            "NGEN_E_OLDER_RUNTIME" => URT::NGEN_E_OLDER_RUNTIME,
            "NGEN_E_WORKER_UNEXPECTED_EXIT" => URT::NGEN_E_WORKER_UNEXPECTED_EXIT,
            "NGEN_E_WORKER_UNEXPECTED_SYNC" => URT::NGEN_E_WORKER_UNEXPECTED_SYNC,
            "NGEN_E_SYS_ASM_NI_MISSING" => URT::NGEN_E_SYS_ASM_NI_MISSING,
            "CLDB_E_INTERNALERROR" => URT::CLDB_E_INTERNALERROR,
        }
    }
    pub fn from_code(code: u32) -> URT {
        return match code {
            0x00131200 => URT::VLDTR_S_WRN,
            0x00131201 => URT::VLDTR_S_ERR,
            0x00131202 => URT::VLDTR_S_WRNERR,
            0x0013130B => URT::CORDBG_S_BAD_START_SEQUENCE_POINT,
            0x0013130C => URT::CORDBG_S_BAD_END_SEQUENCE_POINT,
            0x0013130D => URT::CORDBG_S_INSUFFICIENT_INFO_FOR_SET_IP,
            0x00131316 => URT::CORDBG_S_FUNC_EVAL_HAS_NO_RESULT,
            0x00131317 => URT::CORDBG_S_VALUE_POINTS_TO_VOID,
            0x00131319 => URT::CORDBG_S_FUNC_EVAL_ABORTED,
            0x80131000 => URT::CEE_E_ENTRYPOINT,
            0x80131001 => URT::CEE_E_CVTRES_NOT_FOUND,
            0x80131010 => URT::MSEE_E_LOADLIBFAILED,
            0x80131011 => URT::MSEE_E_GETPROCFAILED,
            0x80131012 => URT::MSEE_E_MULTCOPIESLOADED,
            0x80131013 => URT::COR_E_TYPEUNLOADED,
            0x80131014 => URT::COR_E_APPDOMAINUNLOADED,
            0x80131015 => URT::COR_E_CANNOTUNLOADAPPDOMAIN,
            0x80131016 => URT::MSEE_E_ASSEMBLYLOADINPROGRESS,
            0x80131017 => URT::MSEE_E_CANNOTCREATEAPPDOMAIN,
            0x80131018 => URT::COR_E_ASSEMBLYEXPECTED,
            0x80131019 => URT::COR_E_FIXUPSINEXE,
            0x8013101A => URT::COR_E_NO_LOADLIBRARY_ALLOWED,
            0x8013101B => URT::COR_E_NEWER_RUNTIME,
            0x8013101C => URT::COR_E_CANNOT_SET_POLICY,
            0x8013101D => URT::COR_E_CANNOT_SPECIFY_EVIDENCE,
            0x8013101E => URT::COR_E_MULTIMODULEASSEMBLIESDIALLOWED,
            0x80131020 => URT::HOST_E_DEADLOCK,
            0x80131021 => URT::HOST_E_INTERRUPTED,
            0x80131022 => URT::HOST_E_INVALIDOPERATION,
            0x80131023 => URT::HOST_E_CLRNOTAVAILABLE,
            0x80131024 => URT::HOST_E_TIMEOUT,
            0x80131025 => URT::HOST_E_NOT_OWNER,
            0x80131026 => URT::HOST_E_ABANDONED,
            0x80131027 => URT::HOST_E_EXITPROCESS_THREADABORT,
            0x80131028 => URT::HOST_E_EXITPROCESS_ADUNLOAD,
            0x80131029 => URT::HOST_E_EXITPROCESS_TIMEOUT,
            0x8013102A => URT::HOST_E_EXITPROCESS_OUTOFMEMORY,
            0x8013102B => URT::HOST_E_EXITPROCESS_STACKOVERFLOW,
            0x80131039 => URT::COR_E_MODULE_HASH_CHECK_FAILED,
            0x80131040 => URT::FUSION_E_REF_DEF_MISMATCH,
            0x80131041 => URT::FUSION_E_INVALID_PRIVATE_ASM_LOCATION,
            0x80131042 => URT::FUSION_E_ASM_MODULE_MISSING,
            0x80131043 => URT::FUSION_E_UNEXPECTED_MODULE_FOUND,
            0x80131044 => URT::FUSION_E_PRIVATE_ASM_DISALLOWED,
            0x80131045 => URT::FUSION_E_SIGNATURE_CHECK_FAILED,
            0x80131046 => URT::FUSION_E_DATABASE_ERROR,
            0x80131047 => URT::FUSION_E_INVALID_NAME,
            0x80131048 => URT::FUSION_E_CODE_DOWNLOAD_DISABLED,
            0x80131049 => URT::FUSION_E_UNINSTALL_DISALLOWED,
            0x80131050 => URT::FUSION_E_HOST_GAC_ASM_MISMATCH,
            0x80131051 => URT::FUSION_E_LOADFROM_BLOCKED,
            0x80131052 => URT::FUSION_E_CACHEFILE_FAILED,
            0x80131053 => URT::FUSION_E_APP_DOMAIN_LOCKED,
            0x80131054 => URT::FUSION_E_CONFIGURATION_ERROR,
            0x80131055 => URT::FUSION_E_MANIFEST_PARSE_ERROR,
            0x80131056 => URT::FUSION_E_INVALID_ASSEMBLY_REFERENCE,
            0x80131057 => URT::COR_E_ASSEMBLY_NOT_EXPECTED,
            0x80131058 => URT::COR_E_LOADING_REFERENCE_ASSEMBLY,
            0x80131100 => URT::CLDB_E_FILE_BADREAD,
            0x80131101 => URT::CLDB_E_FILE_BADWRITE,
            0x80131103 => URT::CLDB_E_FILE_READONLY,
            0x80131105 => URT::CLDB_E_NAME_ERROR,
            0x80131106 => URT::CLDB_E_TRUNCATION,
            0x80131107 => URT::CLDB_E_FILE_OLDVER,
            0x80131108 => URT::CLDB_E_RELOCATED,
            0x8013110A => URT::CLDB_E_SMDUPLICATE,
            0x8013110B => URT::CLDB_E_NO_DATA,
            0x8013110C => URT::CLDB_E_READONLY,
            0x8013110D => URT::CLDB_E_INCOMPATIBLE,
            0x8013110E => URT::CLDB_E_FILE_CORRUPT,
            0x8013110F => URT::CLDB_E_SCHEMA_VERNOTFOUND,
            0x80131110 => URT::CLDB_E_BADUPDATEMODE,
            0x80131121 => URT::CLDB_E_INDEX_NONULLKEYS,
            0x80131122 => URT::CLDB_E_INDEX_DUPLICATE,
            0x80131123 => URT::CLDB_E_INDEX_BADTYPE,
            0x80131124 => URT::CLDB_E_INDEX_NOTFOUND,
            0x80131130 => URT::CLDB_E_RECORD_NOTFOUND,
            0x80131131 => URT::CLDB_E_RECORD_OVERFLOW,
            0x80131132 => URT::CLDB_E_RECORD_DUPLICATE,
            0x80131133 => URT::CLDB_E_RECORD_PKREQUIRED,
            0x80131134 => URT::CLDB_E_RECORD_DELETED,
            0x80131135 => URT::CLDB_E_RECORD_OUTOFORDER,
            0x80131140 => URT::CLDB_E_COLUMN_OVERFLOW,
            0x80131141 => URT::CLDB_E_COLUMN_READONLY,
            0x80131142 => URT::CLDB_E_COLUMN_SPECIALCOL,
            0x80131143 => URT::CLDB_E_COLUMN_PKNONULLS,
            0x80131150 => URT::CLDB_E_TABLE_CANTDROP,
            0x80131151 => URT::CLDB_E_OBJECT_NOTFOUND,
            0x80131152 => URT::CLDB_E_OBJECT_COLNOTFOUND,
            0x80131153 => URT::CLDB_E_VECTOR_BADINDEX,
            0x80131154 => URT::CLDB_E_TOO_BIG,
            0x8013115F => URT::META_E_INVALID_TOKEN_TYPE,
            0x80131160 => URT::TLBX_E_INVALID_TYPEINFO,
            0x80131161 => URT::TLBX_E_INVALID_TYPEINFO_UNNAMED,
            0x80131162 => URT::TLBX_E_CTX_NESTED,
            0x80131163 => URT::TLBX_E_ERROR_MESSAGE,
            0x80131164 => URT::TLBX_E_CANT_SAVE,
            0x80131165 => URT::TLBX_W_LIBNOTREGISTERED,
            0x80131166 => URT::TLBX_E_CANTLOADLIBRARY,
            0x80131167 => URT::TLBX_E_BAD_VT_TYPE,
            0x80131168 => URT::TLBX_E_NO_MSCOREE_TLB,
            0x80131169 => URT::TLBX_E_BAD_MSCOREE_TLB,
            0x8013116A => URT::TLBX_E_TLB_EXCEPTION,
            0x8013116B => URT::TLBX_E_MULTIPLE_LCIDS,
            0x8013116D => URT::TLBX_E_AMBIGUOUS_RETURN,
            0x8013116E => URT::TLBX_E_DUPLICATE_TYPE_NAME,
            0x80131172 => URT::TLBX_I_NONSEQUENTIALSTRUCT,
            0x80131174 => URT::TLBX_I_RESOLVEREFFAILED,
            0x80131175 => URT::TLBX_E_ASANY,
            0x80131176 => URT::TLBX_E_INVALIDLCIDPARAM,
            0x80131177 => URT::TLBX_E_LCIDONDISPONLYITF,
            0x80131178 => URT::TLBX_E_NONPUBLIC_FIELD,
            0x8013117B => URT::TLBX_E_BAD_NAMES,
            0x8013117D => URT::TLBX_E_GENERICINST_SIGNATURE,
            0x8013117E => URT::TLBX_E_GENERICPAR_SIGNATURE,
            0x80131180 => URT::META_E_DUPLICATE,
            0x80131181 => URT::META_E_GUID_REQUIRED,
            0x80131182 => URT::META_E_TYPEDEF_MISMATCH,
            0x80131183 => URT::META_E_MERGE_COLLISION,
            0x80131186 => URT::TLBX_E_NO_SAFEHANDLE_ARRAYS,
            0x80131187 => URT::META_E_METHD_NOT_FOUND,
            0x80131188 => URT::META_E_FIELD_NOT_FOUND,
            0x80131189 => URT::META_E_PARAM_MISMATCH,
            0x8013118A => URT::META_E_BADMETADATA,
            0x8013118B => URT::META_E_INTFCEIMPL_NOT_FOUND,
            0x8013118C => URT::TLBX_E_NO_CRITICALHANDLE_ARRAYS,
            0x8013118D => URT::META_E_CLASS_LAYOUT_INCONSISTENT,
            0x8013118E => URT::META_E_FIELD_MARSHAL_NOT_FOUND,
            0x8013118F => URT::META_E_METHODSEM_NOT_FOUND,
            0x80131190 => URT::META_E_EVENT_NOT_FOUND,
            0x80131191 => URT::META_E_PROP_NOT_FOUND,
            0x80131192 => URT::META_E_BAD_SIGNATURE,
            0x80131193 => URT::META_E_BAD_INPUT_PARAMETER,
            0x80131194 => URT::META_E_METHDIMPL_INCONSISTENT,
            0x80131195 => URT::META_E_MD_INCONSISTENCY,
            0x80131196 => URT::META_E_CANNOTRESOLVETYPEREF,
            0x80131198 => URT::META_E_STRINGSPACE_FULL,
            0x80131199 => URT::META_E_UNEXPECTED_REMAP,
            0x8013119A => URT::META_E_HAS_UNMARKALL,
            0x8013119B => URT::META_E_MUST_CALL_UNMARKALL,
            0x8013119C => URT::META_E_GENERICPARAM_INCONSISTENT,
            0x8013119D => URT::META_E_EVENT_COUNTS,
            0x8013119E => URT::META_E_PROPERTY_COUNTS,
            0x8013119F => URT::META_E_TYPEDEF_MISSING,
            0x801311A0 => URT::TLBX_E_CANT_LOAD_MODULE,
            0x801311A1 => URT::TLBX_E_CANT_LOAD_CLASS,
            0x801311A2 => URT::TLBX_E_NULL_MODULE,
            0x801311A3 => URT::TLBX_E_NO_CLSID_KEY,
            0x801311A4 => URT::TLBX_E_CIRCULAR_EXPORT,
            0x801311A5 => URT::TLBX_E_CIRCULAR_IMPORT,
            0x801311A6 => URT::TLBX_E_BAD_NATIVETYPE,
            0x801311A7 => URT::TLBX_E_BAD_VTABLE,
            0x801311A8 => URT::TLBX_E_CRM_NON_STATIC,
            0x801311A9 => URT::TLBX_E_CRM_INVALID_SIG,
            0x801311AA => URT::TLBX_E_CLASS_LOAD_EXCEPTION,
            0x801311AB => URT::TLBX_E_UNKNOWN_SIGNATURE,
            0x801311AC => URT::TLBX_E_REFERENCED_TYPELIB,
            0x801311AD => URT::TLBX_E_INVALID_NAMESPACE,
            0x801311AE => URT::TLBX_E_LAYOUT_ERROR,
            0x801311AF => URT::TLBX_E_NOTIUNKNOWN,
            0x801311B0 => URT::TLBX_E_NONVISIBLEVALUECLASS,
            0x801311B1 => URT::TLBX_E_LPTSTR_NOT_ALLOWED,
            0x801311B2 => URT::TLBX_E_AUTO_CS_NOT_ALLOWED,
            0x801311B3 => URT::TLBX_S_NOSTDINTERFACE,
            0x801311B4 => URT::TLBX_S_DUPLICATE_DISPID,
            0x801311B5 => URT::TLBX_E_ENUM_VALUE_INVALID,
            0x801311B6 => URT::TLBX_E_DUPLICATE_IID,
            0x801311B7 => URT::TLBX_E_NO_NESTED_ARRAYS,
            0x801311B8 => URT::TLBX_E_PARAM_ERROR_NAMED,
            0x801311B9 => URT::TLBX_E_PARAM_ERROR_UNNAMED,
            0x801311BA => URT::TLBX_E_AGNOST_SIGNATURE,
            0x801311BB => URT::TLBX_E_CONVERT_FAIL,
            0x801311BC => URT::TLBX_W_DUAL_NOT_DISPATCH,
            0x801311BD => URT::TLBX_E_BAD_SIGNATURE,
            0x801311BE => URT::TLBX_E_ARRAY_NEEDS_NT_FIXED,
            0x801311BF => URT::TLBX_E_CLASS_NEEDS_NT_INTF,
            0x801311C0 => URT::META_E_CA_INVALID_TARGET,
            0x801311C1 => URT::META_E_CA_INVALID_VALUE,
            0x801311C2 => URT::META_E_CA_INVALID_BLOB,
            0x801311C3 => URT::META_E_CA_REPEATED_ARG,
            0x801311C4 => URT::META_E_CA_UNKNOWN_ARGUMENT,
            0x801311C5 => URT::META_E_CA_VARIANT_NYI,
            0x801311C6 => URT::META_E_CA_ARRAY_NYI,
            0x801311C7 => URT::META_E_CA_UNEXPECTED_TYPE,
            0x801311C8 => URT::META_E_CA_INVALID_ARGTYPE,
            0x801311C9 => URT::META_E_CA_INVALID_ARG_FOR_TYPE,
            0x801311CA => URT::META_E_CA_INVALID_UUID,
            0x801311CB => URT::META_E_CA_INVALID_MARSHALAS_FIELDS,
            0x801311CC => URT::META_E_CA_NT_FIELDONLY,
            0x801311CD => URT::META_E_CA_NEGATIVE_PARAMINDEX,
            0x801311CE => URT::META_E_CA_NEGATIVE_MULTIPLIER,
            0x801311CF => URT::META_E_CA_NEGATIVE_CONSTSIZE,
            0x801311D0 => URT::META_E_CA_FIXEDSTR_SIZE_REQUIRED,
            0x801311D1 => URT::META_E_CA_CUSTMARSH_TYPE_REQUIRED,
            0x801311D2 => URT::META_E_CA_FILENAME_REQUIRED,
            0x801311D3 => URT::TLBX_W_NO_PROPS_IN_EVENTS,
            0x801311D4 => URT::META_E_NOT_IN_ENC_MODE,
            0x801311D6 => URT::META_E_METHOD_COUNTS,
            0x801311D7 => URT::META_E_FIELD_COUNTS,
            0x801311D8 => URT::META_E_PARAM_COUNTS,
            0x801311DA => URT::TLBX_E_TYPED_REF,
            0x801311DB => URT::TLBX_W_DEFAULT_INTF_NOT_VISIBLE,
            0x801311DE => URT::TLBX_W_BAD_SAFEARRAYFIELD_NO_ELEMENTVT,
            0x801311DF => URT::TLBX_W_LAYOUTCLASS_AS_INTERFACE,
            0x801311E0 => URT::TLBX_I_GENERIC_BASE_TYPE,
            0x801311E1 => URT::TLBX_E_BITNESS_MISMATCH,
            0x801311E2 => URT::TLBX_E_EVENT_WITH_NEWENUM,
            0x801311E3 => URT::TLBX_E_PROPGET_WITHOUT_RETURN,
            0x801311E4 => URT::META_E_MISMATCHED_VISIBLITY,
            0x801311E5 => URT::META_E_CA_BAD_FRIENDS_ARGS,
            0x801311E6 => URT::META_E_CA_FRIENDS_SN_REQUIRED,
            0x80131203 => URT::VLDTR_E_RID_OUTOFRANGE,
            0x80131204 => URT::VLDTR_E_CDTKN_OUTOFRANGE,
            0x80131205 => URT::VLDTR_E_CDRID_OUTOFRANGE,
            0x80131206 => URT::VLDTR_E_STRING_INVALID,
            0x80131207 => URT::VLDTR_E_GUID_INVALID,
            0x80131208 => URT::VLDTR_E_BLOB_INVALID,
            0x80131209 => URT::VLDTR_E_MOD_MULTI,
            0x8013120A => URT::VLDTR_E_MOD_NULLMVID,
            0x8013120B => URT::VLDTR_E_TR_NAMENULL,
            0x8013120C => URT::VLDTR_E_TR_DUP,
            0x8013120D => URT::VLDTR_E_TD_NAMENULL,
            0x8013120E => URT::VLDTR_E_TD_DUPNAME,
            0x8013120F => URT::VLDTR_E_TD_DUPGUID,
            0x80131210 => URT::VLDTR_E_TD_NOTIFACEOBJEXTNULL,
            0x80131211 => URT::VLDTR_E_TD_OBJEXTENDSNONNULL,
            0x80131212 => URT::VLDTR_E_TD_EXTENDSSEALED,
            0x80131213 => URT::VLDTR_E_TD_DLTNORTSPCL,
            0x80131214 => URT::VLDTR_E_TD_RTSPCLNOTDLT,
            0x80131215 => URT::VLDTR_E_MI_DECLPRIV,
            0x80131216 => URT::VLDTR_E_AS_BADNAME,
            0x80131217 => URT::VLDTR_E_FILE_SYSNAME,
            0x80131218 => URT::VLDTR_E_MI_BODYSTATIC,
            0x80131219 => URT::VLDTR_E_TD_IFACENOTABS,
            0x8013121A => URT::VLDTR_E_TD_IFACEPARNOTNIL,
            0x8013121B => URT::VLDTR_E_TD_IFACEGUIDNULL,
            0x8013121C => URT::VLDTR_E_MI_DECLFINAL,
            0x8013121D => URT::VLDTR_E_TD_VTNOTSEAL,
            0x8013121E => URT::VLDTR_E_PD_BADFLAGS,
            0x8013121F => URT::VLDTR_E_IFACE_DUP,
            0x80131220 => URT::VLDTR_E_MR_NAMENULL,
            0x80131221 => URT::VLDTR_E_MR_VTBLNAME,
            0x80131222 => URT::VLDTR_E_MR_DELNAME,
            0x80131223 => URT::VLDTR_E_MR_PARNIL,
            0x80131224 => URT::VLDTR_E_MR_BADCALLINGCONV,
            0x80131225 => URT::VLDTR_E_MR_NOTVARARG,
            0x80131226 => URT::VLDTR_E_MR_NAMEDIFF,
            0x80131227 => URT::VLDTR_E_MR_SIGDIFF,
            0x80131228 => URT::VLDTR_E_MR_DUP,
            0x80131229 => URT::VLDTR_E_CL_TDAUTO,
            0x8013122A => URT::VLDTR_E_CL_BADPCKSZ,
            0x8013122B => URT::VLDTR_E_CL_DUP,
            0x8013122C => URT::VLDTR_E_FL_BADOFFSET,
            0x8013122D => URT::VLDTR_E_FL_TDNIL,
            0x8013122E => URT::VLDTR_E_FL_NOCL,
            0x8013122F => URT::VLDTR_E_FL_TDNOTEXPLCT,
            0x80131230 => URT::VLDTR_E_FL_FLDSTATIC,
            0x80131231 => URT::VLDTR_E_FL_DUP,
            0x80131232 => URT::VLDTR_E_MODREF_NAMENULL,
            0x80131233 => URT::VLDTR_E_MODREF_DUP,
            0x80131234 => URT::VLDTR_E_TR_BADSCOPE,
            0x80131235 => URT::VLDTR_E_TD_NESTEDNOENCL,
            0x80131236 => URT::VLDTR_E_TD_EXTTRRES,
            0x80131237 => URT::VLDTR_E_SIGNULL,
            0x80131238 => URT::VLDTR_E_SIGNODATA,
            0x80131239 => URT::VLDTR_E_MD_BADCALLINGCONV,
            0x8013123A => URT::VLDTR_E_MD_THISSTATIC,
            0x8013123B => URT::VLDTR_E_MD_NOTTHISNOTSTATIC,
            0x8013123C => URT::VLDTR_E_MD_NOARGCNT,
            0x8013123D => URT::VLDTR_E_SIG_MISSELTYPE,
            0x8013123E => URT::VLDTR_E_SIG_MISSTKN,
            0x8013123F => URT::VLDTR_E_SIG_TKNBAD,
            0x80131240 => URT::VLDTR_E_SIG_MISSFPTR,
            0x80131241 => URT::VLDTR_E_SIG_MISSFPTRARGCNT,
            0x80131242 => URT::VLDTR_E_SIG_MISSRANK,
            0x80131243 => URT::VLDTR_E_SIG_MISSNSIZE,
            0x80131244 => URT::VLDTR_E_SIG_MISSSIZE,
            0x80131245 => URT::VLDTR_E_SIG_MISSNLBND,
            0x80131246 => URT::VLDTR_E_SIG_MISSLBND,
            0x80131247 => URT::VLDTR_E_SIG_BADELTYPE,
            0x80131248 => URT::VLDTR_E_SIG_MISSVASIZE,
            0x80131249 => URT::VLDTR_E_FD_BADCALLINGCONV,
            0x8013124A => URT::VLDTR_E_MD_NAMENULL,
            0x8013124B => URT::VLDTR_E_MD_PARNIL,
            0x8013124C => URT::VLDTR_E_MD_DUP,
            0x8013124D => URT::VLDTR_E_FD_NAMENULL,
            0x8013124E => URT::VLDTR_E_FD_PARNIL,
            0x8013124F => URT::VLDTR_E_FD_DUP,
            0x80131250 => URT::VLDTR_E_AS_MULTI,
            0x80131251 => URT::VLDTR_E_AS_NAMENULL,
            0x80131252 => URT::VLDTR_E_SIG_TOKTYPEMISMATCH,
            0x80131253 => URT::VLDTR_E_CL_TDINTF,
            0x80131254 => URT::VLDTR_E_ASOS_OSPLTFRMIDINVAL,
            0x80131255 => URT::VLDTR_E_AR_NAMENULL,
            0x80131256 => URT::VLDTR_E_TD_ENCLNOTNESTED,
            0x80131257 => URT::VLDTR_E_AROS_OSPLTFRMIDINVAL,
            0x80131258 => URT::VLDTR_E_FILE_NAMENULL,
            0x80131259 => URT::VLDTR_E_CT_NAMENULL,
            0x8013125A => URT::VLDTR_E_TD_EXTENDSCHILD,
            0x8013125B => URT::VLDTR_E_MAR_NAMENULL,
            0x8013125C => URT::VLDTR_E_FILE_DUP,
            0x8013125D => URT::VLDTR_E_FILE_NAMEFULLQLFD,
            0x8013125E => URT::VLDTR_E_CT_DUP,
            0x8013125F => URT::VLDTR_E_MAR_DUP,
            0x80131260 => URT::VLDTR_E_MAR_NOTPUBPRIV,
            0x80131261 => URT::VLDTR_E_TD_ENUMNOVALUE,
            0x80131262 => URT::VLDTR_E_TD_ENUMVALSTATIC,
            0x80131263 => URT::VLDTR_E_TD_ENUMVALNOTSN,
            0x80131264 => URT::VLDTR_E_TD_ENUMFLDNOTST,
            0x80131265 => URT::VLDTR_E_TD_ENUMFLDNOTLIT,
            0x80131266 => URT::VLDTR_E_TD_ENUMNOLITFLDS,
            0x80131267 => URT::VLDTR_E_TD_ENUMFLDSIGMISMATCH,
            0x80131268 => URT::VLDTR_E_TD_ENUMVALNOT1ST,
            0x80131269 => URT::VLDTR_E_FD_NOTVALUERTSN,
            0x8013126A => URT::VLDTR_E_FD_VALUEPARNOTENUM,
            0x8013126B => URT::VLDTR_E_FD_INSTINIFACE,
            0x8013126C => URT::VLDTR_E_FD_NOTPUBINIFACE,
            0x8013126D => URT::VLDTR_E_FMD_GLOBALNOTPUBPRIVSC,
            0x8013126E => URT::VLDTR_E_FMD_GLOBALNOTSTATIC,
            0x8013126F => URT::VLDTR_E_FD_GLOBALNORVA,
            0x80131270 => URT::VLDTR_E_MD_CTORZERORVA,
            0x80131271 => URT::VLDTR_E_FD_MARKEDNOMARSHAL,
            0x80131272 => URT::VLDTR_E_FD_MARSHALNOTMARKED,
            0x80131273 => URT::VLDTR_E_FD_MARKEDNODEFLT,
            0x80131274 => URT::VLDTR_E_FD_DEFLTNOTMARKED,
            0x80131275 => URT::VLDTR_E_FMD_MARKEDNOSECUR,
            0x80131276 => URT::VLDTR_E_FMD_SECURNOTMARKED,
            0x80131277 => URT::VLDTR_E_FMD_PINVOKENOTSTATIC,
            0x80131278 => URT::VLDTR_E_FMD_MARKEDNOPINVOKE,
            0x80131279 => URT::VLDTR_E_FMD_PINVOKENOTMARKED,
            0x8013127A => URT::VLDTR_E_FMD_BADIMPLMAP,
            0x8013127B => URT::VLDTR_E_IMAP_BADMODREF,
            0x8013127C => URT::VLDTR_E_IMAP_BADMEMBER,
            0x8013127D => URT::VLDTR_E_IMAP_BADIMPORTNAME,
            0x8013127E => URT::VLDTR_E_IMAP_BADCALLCONV,
            0x8013127F => URT::VLDTR_E_FMD_BADACCESSFLAG,
            0x80131280 => URT::VLDTR_E_FD_INITONLYANDLITERAL,
            0x80131281 => URT::VLDTR_E_FD_LITERALNOTSTATIC,
            0x80131282 => URT::VLDTR_E_FMD_RTSNNOTSN,
            0x80131283 => URT::VLDTR_E_MD_ABSTPARNOTABST,
            0x80131284 => URT::VLDTR_E_MD_NOTSTATABSTININTF,
            0x80131285 => URT::VLDTR_E_MD_NOTPUBININTF,
            0x80131286 => URT::VLDTR_E_MD_CTORININTF,
            0x80131287 => URT::VLDTR_E_MD_GLOBALCTORCCTOR,
            0x80131288 => URT::VLDTR_E_MD_CTORSTATIC,
            0x80131289 => URT::VLDTR_E_MD_CTORNOTSNRTSN,
            0x8013128A => URT::VLDTR_E_MD_CTORVIRT,
            0x8013128B => URT::VLDTR_E_MD_CTORABST,
            0x8013128C => URT::VLDTR_E_MD_CCTORNOTSTATIC,
            0x8013128D => URT::VLDTR_E_MD_ZERORVA,
            0x8013128E => URT::VLDTR_E_MD_FINNOTVIRT,
            0x8013128F => URT::VLDTR_E_MD_STATANDFINORVIRT,
            0x80131290 => URT::VLDTR_E_MD_ABSTANDFINAL,
            0x80131291 => URT::VLDTR_E_MD_ABSTANDIMPL,
            0x80131292 => URT::VLDTR_E_MD_ABSTANDPINVOKE,
            0x80131293 => URT::VLDTR_E_MD_ABSTNOTVIRT,
            0x80131294 => URT::VLDTR_E_MD_NOTABSTNOTIMPL,
            0x80131295 => URT::VLDTR_E_MD_NOTABSTBADFLAGSRVA,
            0x80131296 => URT::VLDTR_E_MD_PRIVSCOPENORVA,
            0x80131297 => URT::VLDTR_E_MD_GLOBALABSTORVIRT,
            0x80131298 => URT::VLDTR_E_SIG_LONGFORM,
            0x80131299 => URT::VLDTR_E_MD_MULTIPLESEMANTICS,
            0x8013129A => URT::VLDTR_E_MD_INVALIDSEMANTICS,
            0x8013129B => URT::VLDTR_E_MD_SEMANTICSNOTEXIST,
            0x8013129C => URT::VLDTR_E_MI_DECLNOTVIRT,
            0x8013129D => URT::VLDTR_E_FMD_GLOBALITEM,
            0x8013129E => URT::VLDTR_E_MD_MULTSEMANTICFLAGS,
            0x8013129F => URT::VLDTR_E_MD_NOSEMANTICFLAGS,
            0x801312A0 => URT::VLDTR_E_FD_FLDINIFACE,
            0x801312A1 => URT::VLDTR_E_AS_HASHALGID,
            0x801312A2 => URT::VLDTR_E_AS_PROCID,
            0x801312A3 => URT::VLDTR_E_AR_PROCID,
            0x801312A4 => URT::VLDTR_E_CN_PARENTRANGE,
            0x801312A5 => URT::VLDTR_E_AS_BADFLAGS,
            0x801312A6 => URT::VLDTR_E_TR_HASTYPEDEF,
            0x801312A7 => URT::VLDTR_E_IFACE_BADIMPL,
            0x801312A8 => URT::VLDTR_E_IFACE_BADIFACE,
            0x801312A9 => URT::VLDTR_E_TD_SECURNOTMARKED,
            0x801312AA => URT::VLDTR_E_TD_MARKEDNOSECUR,
            0x801312AB => URT::VLDTR_E_MD_CCTORHASARGS,
            0x801312AC => URT::VLDTR_E_CT_BADIMPL,
            0x801312AD => URT::VLDTR_E_MI_ALIENBODY,
            0x801312AE => URT::VLDTR_E_MD_CCTORCALLCONV,
            0x801312AF => URT::VLDTR_E_MI_BADCLASS,
            0x801312B0 => URT::VLDTR_E_MI_CLASSISINTF,
            0x801312B1 => URT::VLDTR_E_MI_BADDECL,
            0x801312B2 => URT::VLDTR_E_MI_BADBODY,
            0x801312B3 => URT::VLDTR_E_MI_DUP,
            0x801312B4 => URT::VLDTR_E_FD_BADPARENT,
            0x801312B5 => URT::VLDTR_E_MD_PARAMOUTOFSEQ,
            0x801312B6 => URT::VLDTR_E_MD_PARASEQTOOBIG,
            0x801312B7 => URT::VLDTR_E_MD_PARMMARKEDNOMARSHAL,
            0x801312B8 => URT::VLDTR_E_MD_PARMMARSHALNOTMARKED,
            0x801312BA => URT::VLDTR_E_MD_PARMMARKEDNODEFLT,
            0x801312BB => URT::VLDTR_E_MD_PARMDEFLTNOTMARKED,
            0x801312BC => URT::VLDTR_E_PR_BADSCOPE,
            0x801312BD => URT::VLDTR_E_PR_NONAME,
            0x801312BE => URT::VLDTR_E_PR_NOSIG,
            0x801312BF => URT::VLDTR_E_PR_DUP,
            0x801312C0 => URT::VLDTR_E_PR_BADCALLINGCONV,
            0x801312C1 => URT::VLDTR_E_PR_MARKEDNODEFLT,
            0x801312C2 => URT::VLDTR_E_PR_DEFLTNOTMARKED,
            0x801312C3 => URT::VLDTR_E_PR_BADSEMANTICS,
            0x801312C4 => URT::VLDTR_E_PR_BADMETHOD,
            0x801312C5 => URT::VLDTR_E_PR_ALIENMETHOD,
            0x801312C6 => URT::VLDTR_E_CN_BLOBNOTNULL,
            0x801312C7 => URT::VLDTR_E_CN_BLOBNULL,
            0x801312C8 => URT::VLDTR_E_EV_BADSCOPE,
            0x801312CA => URT::VLDTR_E_EV_NONAME,
            0x801312CB => URT::VLDTR_E_EV_DUP,
            0x801312CC => URT::VLDTR_E_EV_BADEVTYPE,
            0x801312CD => URT::VLDTR_E_EV_EVTYPENOTCLASS,
            0x801312CE => URT::VLDTR_E_EV_BADSEMANTICS,
            0x801312CF => URT::VLDTR_E_EV_BADMETHOD,
            0x801312D0 => URT::VLDTR_E_EV_ALIENMETHOD,
            0x801312D1 => URT::VLDTR_E_EV_NOADDON,
            0x801312D2 => URT::VLDTR_E_EV_NOREMOVEON,
            0x801312D3 => URT::VLDTR_E_CT_DUPTDNAME,
            0x801312D4 => URT::VLDTR_E_MAR_BADOFFSET,
            0x801312D5 => URT::VLDTR_E_DS_BADOWNER,
            0x801312D6 => URT::VLDTR_E_DS_BADFLAGS,
            0x801312D7 => URT::VLDTR_E_DS_NOBLOB,
            0x801312D8 => URT::VLDTR_E_MAR_BADIMPL,
            0x801312DA => URT::VLDTR_E_MR_VARARGCALLINGCONV,
            0x801312DB => URT::VLDTR_E_MD_CTORNOTVOID,
            0x801312DC => URT::VLDTR_E_EV_FIRENOTVOID,
            0x801312DD => URT::VLDTR_E_AS_BADLOCALE,
            0x801312DE => URT::VLDTR_E_CN_PARENTTYPE,
            0x801312DF => URT::VLDTR_E_SIG_SENTINMETHODDEF,
            0x801312E0 => URT::VLDTR_E_SIG_SENTMUSTVARARG,
            0x801312E1 => URT::VLDTR_E_SIG_MULTSENTINELS,
            0x801312E2 => URT::VLDTR_E_SIG_LASTSENTINEL,
            0x801312E3 => URT::VLDTR_E_SIG_MISSARG,
            0x801312E4 => URT::VLDTR_E_SIG_BYREFINFIELD,
            0x801312E5 => URT::VLDTR_E_MD_SYNCMETHODINVTYPE,
            0x801312E6 => URT::VLDTR_E_TD_NAMETOOLONG,
            0x801312E7 => URT::VLDTR_E_AS_PROCDUP,
            0x801312E8 => URT::VLDTR_E_ASOS_DUP,
            0x801312E9 => URT::VLDTR_E_MAR_BADFLAGS,
            0x801312EA => URT::VLDTR_E_CT_NOTYPEDEFID,
            0x801312EB => URT::VLDTR_E_FILE_BADFLAGS,
            0x801312EC => URT::VLDTR_E_FILE_NULLHASH,
            0x801312ED => URT::VLDTR_E_MOD_NONAME,
            0x801312EE => URT::VLDTR_E_MOD_NAMEFULLQLFD,
            0x801312EF => URT::VLDTR_E_TD_RTSPCLNOTSPCL,
            0x801312F0 => URT::VLDTR_E_TD_EXTENDSIFACE,
            0x801312F1 => URT::VLDTR_E_MD_CTORPINVOKE,
            0x801312F2 => URT::VLDTR_E_TD_SYSENUMNOTCLASS,
            0x801312F3 => URT::VLDTR_E_TD_SYSENUMNOTEXTVTYPE,
            0x801312F4 => URT::VLDTR_E_MI_SIGMISMATCH,
            0x801312F5 => URT::VLDTR_E_TD_ENUMHASMETHODS,
            0x801312F6 => URT::VLDTR_E_TD_ENUMIMPLIFACE,
            0x801312F7 => URT::VLDTR_E_TD_ENUMHASPROP,
            0x801312F8 => URT::VLDTR_E_TD_ENUMHASEVENT,
            0x801312F9 => URT::VLDTR_E_TD_BADMETHODLST,
            0x801312FA => URT::VLDTR_E_TD_BADFIELDLST,
            0x801312FB => URT::VLDTR_E_CN_BADTYPE,
            0x801312FC => URT::VLDTR_E_TD_ENUMNOINSTFLD,
            0x801312FD => URT::VLDTR_E_TD_ENUMMULINSTFLD,
            0x801312FE => URT::VLDTR_E_INTERRUPTED,
            0x801312FF => URT::VLDTR_E_NOTINIT,
            0x80131300 => URT::CORDBG_E_UNRECOVERABLE_ERROR,
            0x80131301 => URT::CORDBG_E_PROCESS_TERMINATED,
            0x80131302 => URT::CORDBG_E_PROCESS_NOT_SYNCHRONIZED,
            0x80131303 => URT::CORDBG_E_CLASS_NOT_LOADED,
            0x80131304 => URT::CORDBG_E_IL_VAR_NOT_AVAILABLE,
            0x80131305 => URT::CORDBG_E_BAD_REFERENCE_VALUE,
            0x80131306 => URT::CORDBG_E_FIELD_NOT_AVAILABLE,
            0x80131307 => URT::CORDBG_E_NON_NATIVE_FRAME,
            0x80131308 => URT::CORDBG_E_NONCONTINUABLE_EXCEPTION,
            0x80131309 => URT::CORDBG_E_CODE_NOT_AVAILABLE,
            0x8013130A => URT::CORDBG_E_FUNCTION_NOT_IL,
            0x8013130E => URT::CORDBG_E_CANT_SET_IP_INTO_FINALLY,
            0x8013130F => URT::CORDBG_E_CANT_SET_IP_OUT_OF_FINALLY,
            0x80131310 => URT::CORDBG_E_CANT_SET_IP_INTO_CATCH,
            0x80131311 => URT::CORDBG_E_SET_IP_NOT_ALLOWED_ON_NONLEAF_FRAME,
            0x80131312 => URT::CORDBG_E_SET_IP_IMPOSSIBLE,
            0x80131313 => URT::CORDBG_E_FUNC_EVAL_BAD_START_POINT,
            0x80131314 => URT::CORDBG_E_INVALID_OBJECT,
            0x80131315 => URT::CORDBG_E_FUNC_EVAL_NOT_COMPLETE,
            0x80131318 => URT::CORDBG_E_INPROC_NOT_IMPL,
            0x8013131A => URT::CORDBG_E_STATIC_VAR_NOT_AVAILABLE,
            0x8013131B => URT::CORDBG_E_OBJECT_IS_NOT_COPYABLE_VALUE_CLASS,
            0x8013131C => URT::CORDBG_E_CANT_SETIP_INTO_OR_OUT_OF_FILTER,
            0x8013131D => URT::CORDBG_E_CANT_CHANGE_JIT_SETTING_FOR_ZAP_MODULE,
            0x8013131E => URT::CORDBG_E_CANT_SET_IP_OUT_OF_FINALLY_ON_WIN64,
            0x8013131F => URT::CORDBG_E_CANT_SET_IP_OUT_OF_CATCH_ON_WIN64,
            0x80131320 => URT::CORDBG_E_REMOTE_CONNECTION_CONN_RESET,
            0x80131321 => URT::CORDBG_E_REMOTE_CONNECTION_KEEP_ALIVE,
            0x80131322 => URT::CORDBG_E_REMOTE_CONNECTION_FATAL_ERROR,
            0x80131323 => URT::CORDBG_E_CANT_SET_TO_JMC,
            0x80131326 => URT::CORDBG_E_NO_CONTEXT_FOR_INTERNAL_FRAME,
            0x80131327 => URT::CORDBG_E_NOT_CHILD_FRAME,
            0x80131328 => URT::CORDBG_E_NON_MATCHING_CONTEXT,
            0x80131329 => URT::CORDBG_E_PAST_END_OF_STACK,
            0x8013132D => URT::CORDBG_E_BAD_THREAD_STATE,
            0x8013132D => URT::CORDBG_E_FUNC_EVAL_CANNOT_UPDATE_REGISTER_IN_NONLEAF_FRAME,
            0x8013132E => URT::CORDBG_E_DEBUGGER_ALREADY_ATTACHED,
            0x8013132F => URT::CORDBG_E_SUPERFLOUS_CONTINUE,
            0x80131330 => URT::CORDBG_E_SET_VALUE_NOT_ALLOWED_ON_NONLEAF_FRAME,
            0x80131331 => URT::CORDBG_E_ENC_EH_MAX_NESTING_LEVEL_CANT_INCREASE,
            0x80131332 => URT::CORDBG_E_ENC_MODULE_NOT_ENC_ENABLED,
            0x80131333 => URT::CORDBG_E_SET_IP_NOT_ALLOWED_ON_EXCEPTION,
            0x80131334 => URT::CORDBG_E_VARIABLE_IS_ACTUALLY_LITERAL,
            0x80131335 => URT::CORDBG_E_PROCESS_DETACHED,
            0x80131336 => URT::CORDBG_E_ENC_METHOD_SIG_CHANGED,
            0x80131337 => URT::CORDBG_E_ENC_METHOD_NO_LOCAL_SIG,
            0x80131338 => URT::CORDBG_E_ENC_CANT_ADD_FIELD_TO_VALUE_OR_LAYOUT_CLASS,
            0x80131339 => URT::CORDBG_E_ENC_CANT_CHANGE_FIELD,
            0x8013133A => URT::CORDBG_E_ENC_CANT_ADD_NON_PRIVATE_MEMBER,
            0x8013133B => URT::CORDBG_E_FIELD_NOT_STATIC,
            0x8013133C => URT::CORDBG_E_FIELD_NOT_INSTANCE,
            0x8013133D => URT::CORDBG_E_ENC_ZAPPED_WITHOUT_ENC,
            0x8013133E => URT::CORDBG_E_ENC_BAD_METHOD_INFO,
            0x8013133F => URT::CORDBG_E_ENC_JIT_CANT_UPDATE,
            0x80131340 => URT::CORDBG_E_ENC_MISSING_CLASS,
            0x80131341 => URT::CORDBG_E_ENC_INTERNAL_ERROR,
            0x80131342 => URT::CORDBG_E_ENC_HANGING_FIELD,
            0x80131343 => URT::CORDBG_E_MODULE_NOT_LOADED,
            0x80131344 => URT::CORDBG_E_ENC_CANT_CHANGE_SUPERCLASS,
            0x80131345 => URT::CORDBG_E_UNABLE_TO_SET_BREAKPOINT,
            0x80131346 => URT::CORDBG_E_DEBUGGING_NOT_POSSIBLE,
            0x80131347 => URT::CORDBG_E_KERNEL_DEBUGGER_ENABLED,
            0x80131348 => URT::CORDBG_E_KERNEL_DEBUGGER_PRESENT,
            0x80131349 => URT::CORDBG_E_HELPER_THREAD_DEAD,
            0x8013134A => URT::CORDBG_E_INTERFACE_INHERITANCE_CANT_CHANGE,
            0x8013134B => URT::CORDBG_E_INCOMPATIBLE_PROTOCOL,
            0x8013134C => URT::CORDBG_E_TOO_MANY_PROCESSES,
            0x8013134D => URT::CORDBG_E_INTEROP_NOT_SUPPORTED,
            0x8013134E => URT::CORDBG_E_NO_REMAP_BREAKPIONT,
            0x8013134F => URT::CORDBG_E_OBJECT_NEUTERED,
            0x80131350 => URT::CORPROF_E_FUNCTION_NOT_COMPILED,
            0x80131351 => URT::CORPROF_E_DATAINCOMPLETE,
            0x80131352 => URT::CORPROF_E_NOT_REJITABLE_METHODS,
            0x80131353 => URT::CORPROF_E_CANNOT_UPDATE_METHOD,
            0x80131354 => URT::CORPROF_E_FUNCTION_NOT_IL,
            0x80131355 => URT::CORPROF_E_NOT_MANAGED_THREAD,
            0x80131356 => URT::CORPROF_E_CALL_ONLY_FROM_INIT,
            0x80131357 => URT::CORPROF_E_INPROC_NOT_ENABLED,
            0x80131358 => URT::CORPROF_E_JITMAPS_NOT_ENABLED,
            0x80131359 => URT::CORPROF_E_INPROC_ALREADY_BEGUN,
            0x8013135A => URT::CORPROF_E_INPROC_NOT_AVAILABLE,
            0x8013135B => URT::CORPROF_E_NOT_YET_AVAILABLE,
            0x8013135C => URT::CORPROF_E_TYPE_IS_PARAMETERIZED,
            0x8013135D => URT::CORPROF_E_FUNCTION_IS_PARAMETERIZED,
            0x8013135E => URT::CORPROF_E_STACKSNAPSHOT_INVALID_TGT_THREAD,
            0x8013135F => URT::CORPROF_E_STACKSNAPSHOT_UNMANAGED_CTX,
            0x80131360 => URT::CORPROF_E_STACKSNAPSHOT_UNSAFE,
            0x80131361 => URT::CORPROF_E_STACKSNAPSHOT_ABORTED,
            0x80131362 => URT::CORPROF_E_LITERALS_HAVE_NO_ADDRESS,
            0x80131363 => URT::CORPROF_E_UNSUPPORTED_CALL_SEQUENCE,
            0x80131364 => URT::CORPROF_E_ASYNCHRONOUS_UNSAFE,
            0x80131365 => URT::CORPROF_E_CLASSID_IS_ARRAY,
            0x80131366 => URT::CORPROF_E_CLASSID_IS_COMPOSITE,
            0x80131367 => URT::CORPROF_E_PROFILER_DETACHING,
            0x80131368 => URT::CORPROF_E_PROFILER_NOT_ATTACHABLE,
            0x80131369 => URT::CORPROF_E_UNRECOGNIZED_PIPE_MSG_FORMAT,
            0x8013136A => URT::CORPROF_E_PROFILER_ALREADY_ACTIVE,
            0x8013136B => URT::CORPROF_E_PROFILEE_INCOMPATIBLE_WITH_TRIGGER,
            0x8013136C => URT::CORPROF_E_IPC_FAILED,
            0x8013136D => URT::CORPROF_E_PROFILEE_PROCESS_NOT_FOUND,
            0x8013136E => URT::CORPROF_E_CALLBACK3_REQUIRED,
            0x8013136F => URT::CORPROF_E_UNSUPPORTED_FOR_ATTACHING_PROFILER,
            0x80131370 => URT::CORPROF_E_IRREVERSIBLE_INSTRUMENTATION_PRESENT,
            0x80131371 => URT::CORPROF_E_RUNTIME_UNINITIALIZED,
            0x80131372 => URT::CORPROF_E_IMMUTABLE_FLAGS_SET,
            0x80131373 => URT::CORPROF_E_PROFILER_NOT_YET_INITIALIZED,
            0x80131374 => URT::CORPROF_E_INCONSISTENT_WITH_FLAGS,
            0x80131375 => URT::CORPROF_E_PROFILER_CANCEL_ACTIVATION,
            0x80131376 => URT::CORPROF_E_CONCURRENT_GC_NOT_PROFILABLE,
            0x80131377 => URT::CORPROF_E_INCONSISTENT_FLAGS_WITH_HOST_PROTECTION_SETTING,
            0x80131378 => URT::CORPROF_E_DEBUGGING_DISABLED,
            0x80131379 => URT::CORPROF_E_TIMEOUT_WAITING_FOR_CONCURRENT_GC,
            0x8013137A => URT::CORPROF_E_MODULE_IS_DYNAMIC,
            0x8013137B => URT::CORPROF_E_CALLBACK4_REQUIRED,
            0x8013137C => URT::CORPROF_E_REJIT_NOT_ENABLED,
            0x8013137D => URT::CORPROF_E_ACTIVE_REJIT_REQUEST_NOT_FOUND,
            0x8013137E => URT::CORPROF_E_FUNCTION_IS_COLLECTIBLE,
            0x8013137F => URT::CORPROF_E_REJIT_REQUIRES_DISABLE_NGEN,
            0x80131380 => URT::CORPROF_E_CALLBACK6_REQUIRED,
            0x80131381 => URT::CORPROF_E_RUNTIME_SUSPEND_REQUIRED,
            0x80131382 => URT::CORPROF_E_CALLBACK7_REQUIRED,
            0x80131400 => URT::SECURITY_E_XML_TO_ASN_ENCODING,
            0x80131401 => URT::SECURITY_E_INCOMPATIBLE_SHARE,
            0x80131402 => URT::SECURITY_E_UNVERIFIABLE,
            0x80131403 => URT::SECURITY_E_INCOMPATIBLE_EVIDENCE,
            0x80131410 => URT::CORSEC_E_DECODE_SET,
            0x80131411 => URT::CORSEC_E_ENCODE_SET,
            0x80131412 => URT::CORSEC_E_UNSUPPORTED_FORMAT,
            0x80131413 => URT::SN_CRYPTOAPI_CALL_FAILED,
            0x80131413 => URT::CORSEC_E_CRYPTOAPI_CALL_FAILED,
            0x80131414 => URT::SN_NO_SUITABLE_CSP,
            0x80131414 => URT::CORSEC_E_NO_SUITABLE_CSP,
            0x80131415 => URT::CORSEC_E_INVALID_ATTR,
            0x80131416 => URT::CORSEC_E_POLICY_EXCEPTION,
            0x80131417 => URT::CORSEC_E_MIN_GRANT_FAIL,
            0x80131418 => URT::CORSEC_E_NO_EXEC_PERM,
            0x80131419 => URT::CORSEC_E_XMLSYNTAX,
            0x8013141A => URT::CORSEC_E_INVALID_STRONGNAME,
            0x8013141B => URT::CORSEC_E_MISSING_STRONGNAME,
            0x8013141C => URT::CORSEC_E_CONTAINER_NOT_FOUND,
            0x8013141D => URT::CORSEC_E_INVALID_IMAGE_FORMAT,
            0x8013141E => URT::CORSEC_E_INVALID_PUBLICKEY,
            0x80131420 => URT::CORSEC_E_SIGNATURE_MISMATCH,
            0x80131421 => URT::SN_E_PUBLICKEY_MISMATCH,
            0x80131430 => URT::CORSEC_E_CRYPTO,
            0x80131431 => URT::CORSEC_E_CRYPTO_UNEX_OPER,
            0x8013143A => URT::CORSECATTR_E_BAD_ATTRIBUTE,
            0x8013143B => URT::CORSECATTR_E_MISSING_CONSTRUCTOR,
            0x8013143C => URT::CORSECATTR_E_FAILED_TO_CREATE_PERM,
            0x8013143D => URT::CORSECATTR_E_BAD_ACTION_ASM,
            0x8013143E => URT::CORSECATTR_E_BAD_ACTION_OTHER,
            0x8013143F => URT::CORSECATTR_E_BAD_PARENT,
            0x80131440 => URT::CORSECATTR_E_TRUNCATED,
            0x80131441 => URT::CORSECATTR_E_BAD_VERSION,
            0x80131442 => URT::CORSECATTR_E_BAD_ACTION,
            0x80131443 => URT::CORSECATTR_E_NO_SELF_REF,
            0x80131444 => URT::CORSECATTR_E_BAD_NONCAS,
            0x80131445 => URT::CORSECATTR_E_ASSEMBLY_LOAD_FAILED,
            0x80131446 => URT::CORSECATTR_E_ASSEMBLY_LOAD_FAILED_EX,
            0x80131447 => URT::CORSECATTR_E_TYPE_LOAD_FAILED,
            0x80131448 => URT::CORSECATTR_E_TYPE_LOAD_FAILED_EX,
            0x80131449 => URT::CORSECATTR_E_ABSTRACT,
            0x8013144A => URT::CORSECATTR_E_UNSUPPORTED_TYPE,
            0x8013144B => URT::CORSECATTR_E_UNSUPPORTED_ENUM_TYPE,
            0x8013144C => URT::CORSECATTR_E_NO_FIELD,
            0x8013144D => URT::CORSECATTR_E_NO_PROPERTY,
            0x8013144E => URT::CORSECATTR_E_EXCEPTION,
            0x8013144F => URT::CORSECATTR_E_EXCEPTION_HR,
            0x80131450 => URT::ISS_E_ISOSTORE_START,
            0x80131450 => URT::ISS_E_ISOSTORE,
            0x80131460 => URT::ISS_E_OPEN_STORE_FILE,
            0x80131461 => URT::ISS_E_OPEN_FILE_MAPPING,
            0x80131462 => URT::ISS_E_MAP_VIEW_OF_FILE,
            0x80131463 => URT::ISS_E_GET_FILE_SIZE,
            0x80131464 => URT::ISS_E_CREATE_MUTEX,
            0x80131465 => URT::ISS_E_LOCK_FAILED,
            0x80131466 => URT::ISS_E_FILE_WRITE,
            0x80131467 => URT::ISS_E_SET_FILE_POINTER,
            0x80131468 => URT::ISS_E_CREATE_DIR,
            0x80131469 => URT::ISS_E_STORE_NOT_OPEN,
            0x80131480 => URT::ISS_E_CORRUPTED_STORE_FILE,
            0x80131481 => URT::ISS_E_STORE_VERSION,
            0x80131482 => URT::ISS_E_FILE_NOT_MAPPED,
            0x80131483 => URT::ISS_E_BLOCK_SIZE_TOO_SMALL,
            0x80131484 => URT::ISS_E_ALLOC_TOO_LARGE,
            0x80131485 => URT::ISS_E_USAGE_WILL_EXCEED_QUOTA,
            0x80131486 => URT::ISS_E_TABLE_ROW_NOT_FOUND,
            0x801314A0 => URT::ISS_E_DEPRECATE,
            0x801314A1 => URT::ISS_E_CALLER,
            0x801314A2 => URT::ISS_E_PATH_LENGTH,
            0x801314A3 => URT::ISS_E_MACHINE,
            0x801314A4 => URT::ISS_E_MACHINE_DACL,
            0x801314FF => URT::ISS_E_ISOSTORE_END,
            0x80131500 => URT::COR_E_EXCEPTION,
            0x80131501 => URT::COR_E_SYSTEM,
            0x80131502 => URT::COR_E_ARGUMENTOUTOFRANGE,
            0x80131503 => URT::COR_E_ARRAYTYPEMISMATCH,
            0x80131504 => URT::COR_E_CONTEXTMARSHAL,
            0x80131505 => URT::COR_E_TIMEOUT,
            0x80131506 => URT::COR_E_EXECUTIONENGINE,
            0x80131507 => URT::COR_E_FIELDACCESS,
            0x80131508 => URT::COR_E_INDEXOUTOFRANGE,
            0x80131509 => URT::COR_E_INVALIDOPERATION,
            0x8013150A => URT::COR_E_SECURITY,
            0x8013150B => URT::COR_E_REMOTING,
            0x8013150C => URT::COR_E_SERIALIZATION,
            0x8013150D => URT::COR_E_VERIFICATION,
            0x8013150E => URT::COR_E_SERVER,
            0x8013150F => URT::COR_E_SERVICEDCOMPONENT,
            0x80131510 => URT::COR_E_METHODACCESS,
            0x80131511 => URT::COR_E_MISSINGFIELD,
            0x80131512 => URT::COR_E_MISSINGMEMBER,
            0x80131513 => URT::COR_E_MISSINGMETHOD,
            0x80131514 => URT::COR_E_MULTICASTNOTSUPPORTED,
            0x80131515 => URT::COR_E_NOTSUPPORTED,
            0x80131516 => URT::COR_E_OVERFLOW,
            0x80131517 => URT::COR_E_RANK,
            0x80131518 => URT::COR_E_SYNCHRONIZATIONLOCK,
            0x80131519 => URT::COR_E_THREADINTERRUPTED,
            0x8013151A => URT::COR_E_MEMBERACCESS,
            0x80131520 => URT::COR_E_THREADSTATE,
            0x80131521 => URT::COR_E_THREADSTOP,
            0x80131522 => URT::COR_E_TYPELOAD,
            0x80131523 => URT::COR_E_ENTRYPOINTNOTFOUND,
            0x80131524 => URT::COR_E_DLLNOTFOUND,
            0x80131525 => URT::COR_E_THREADSTART,
            0x80131527 => URT::COR_E_INVALIDCOMOBJECT,
            0x80131528 => URT::COR_E_NOTFINITENUMBER,
            0x80131529 => URT::COR_E_DUPLICATEWAITOBJECT,
            0x8013152B => URT::COR_E_SEMAPHOREFULL,
            0x8013152C => URT::COR_E_WAITHANDLECANNOTBEOPENED,
            0x8013152D => URT::COR_E_ABANDONEDMUTEX,
            0x80131530 => URT::COR_E_THREADABORTED,
            0x80131531 => URT::COR_E_INVALIDOLEVARIANTTYPE,
            0x80131532 => URT::COR_E_MISSINGMANIFESTRESOURCE,
            0x80131533 => URT::COR_E_SAFEARRAYTYPEMISMATCH,
            0x80131534 => URT::COR_E_TYPEINITIALIZATION,
            0x80131535 => URT::COR_E_MARSHALDIRECTIVE,
            0x80131536 => URT::COR_E_MISSINGSATELLITEASSEMBLY,
            0x80131537 => URT::COR_E_FORMAT,
            0x80131538 => URT::COR_E_SAFEARRAYRANKMISMATCH,
            0x80131539 => URT::COR_E_PLATFORMNOTSUPPORTED,
            0x8013153A => URT::COR_E_INVALIDPROGRAM,
            0x8013153B => URT::COR_E_OPERATIONCANCELED,
            0x8013153D => URT::COR_E_INSUFFICIENTMEMORY,
            0x8013153E => URT::COR_E_RUNTIMEWRAPPED,
            0x80131540 => URT::COR_E_DEVICESNOTSUPPORTED,
            0x80131541 => URT::COR_E_DATAMISALIGNED,
            0x80131542 => URT::COR_E_CODECONTRACTFAILED,
            0x80131543 => URT::COR_E_TYPEACCESS,
            0x80131577 => URT::COR_E_KEYNOTFOUND,
            0x80131578 => URT::COR_E_INSUFFICIENTEXECUTIONSTACK,
            0x80131600 => URT::COR_E_APPLICATION,
            0x80131601 => URT::COR_E_INVALIDFILTERCRITERIA,
            0x80131602 => URT::COR_E_REFLECTIONTYPELOAD,
            0x80131603 => URT::COR_E_TARGET,
            0x80131604 => URT::COR_E_TARGETINVOCATION,
            0x80131605 => URT::COR_E_CUSTOMATTRIBUTEFORMAT,
            0x80131620 => URT::COR_E_IO,
            0x80131621 => URT::COR_E_FILELOAD,
            0x80131622 => URT::COR_E_OBJECTDISPOSED,
            0x80131623 => URT::COR_E_FAILFAST,
            0x80131640 => URT::COR_E_HOSTPROTECTION,
            0x80131641 => URT::COR_E_ILLEGAL_REENTRANCY,
            0x80131700 => URT::CLR_E_SHIM_RUNTIMELOAD,
            0x80131701 => URT::CLR_E_SHIM_RUNTIMEEXPORT,
            0x80131702 => URT::CLR_E_SHIM_INSTALLROOT,
            0x80131703 => URT::CLR_E_SHIM_INSTALLCOMP,
            0x80131704 => URT::CLR_E_SHIM_LEGACYRUNTIMEALREADYBOUND,
            0x80131705 => URT::CLR_E_SHIM_SHUTDOWNINPROGRESS,
            0x80131801 => URT::VER_E_HRESULT,
            0x80131802 => URT::VER_E_OFFSET,
            0x80131803 => URT::VER_E_OPCODE,
            0x80131804 => URT::VER_E_OPERAND,
            0x80131805 => URT::VER_E_TOKEN,
            0x80131806 => URT::VER_E_EXCEPT,
            0x80131807 => URT::VER_E_STACK_SLOT,
            0x80131808 => URT::VER_E_LOC,
            0x80131809 => URT::VER_E_ARG,
            0x8013180A => URT::VER_E_FOUND,
            0x8013180B => URT::VER_E_EXPECTED,
            0x8013180C => URT::VER_E_LOC_BYNAME,
            0x80131810 => URT::VER_E_UNKNOWN_OPCODE,
            0x80131811 => URT::VER_E_SIG_CALLCONV,
            0x80131812 => URT::VER_E_SIG_ELEMTYPE,
            0x80131814 => URT::VER_E_RET_SIG,
            0x80131815 => URT::VER_E_FIELD_SIG,
            0x80131818 => URT::VER_E_INTERNAL,
            0x80131819 => URT::VER_E_STACK_TOO_LARGE,
            0x8013181A => URT::VER_E_ARRAY_NAME_LONG,
            0x80131820 => URT::VER_E_FALLTHRU,
            0x80131821 => URT::VER_E_TRY_GTEQ_END,
            0x80131822 => URT::VER_E_TRYEND_GT_CS,
            0x80131823 => URT::VER_E_HND_GTEQ_END,
            0x80131824 => URT::VER_E_HNDEND_GT_CS,
            0x80131825 => URT::VER_E_FLT_GTEQ_CS,
            0x80131826 => URT::VER_E_TRY_START,
            0x80131827 => URT::VER_E_HND_START,
            0x80131828 => URT::VER_E_FLT_START,
            0x80131829 => URT::VER_E_TRY_OVERLAP,
            0x8013182A => URT::VER_E_TRY_EQ_HND_FIL,
            0x8013182B => URT::VER_E_TRY_SHARE_FIN_FAL,
            0x8013182C => URT::VER_E_HND_OVERLAP,
            0x8013182D => URT::VER_E_HND_EQ,
            0x8013182E => URT::VER_E_FIL_OVERLAP,
            0x8013182F => URT::VER_E_FIL_EQ,
            0x80131830 => URT::VER_E_FIL_CONT_TRY,
            0x80131831 => URT::VER_E_FIL_CONT_HND,
            0x80131832 => URT::VER_E_FIL_CONT_FIL,
            0x80131833 => URT::VER_E_FIL_GTEQ_CS,
            0x80131834 => URT::VER_E_FIL_START,
            0x80131835 => URT::VER_E_FALLTHRU_EXCEP,
            0x80131836 => URT::VER_E_FALLTHRU_INTO_HND,
            0x80131837 => URT::VER_E_FALLTHRU_INTO_FIL,
            0x80131838 => URT::VER_E_LEAVE,
            0x80131839 => URT::VER_E_RETHROW,
            0x8013183A => URT::VER_E_ENDFINALLY,
            0x8013183B => URT::VER_E_ENDFILTER,
            0x8013183C => URT::VER_E_ENDFILTER_MISSING,
            0x8013183D => URT::VER_E_BR_INTO_TRY,
            0x8013183E => URT::VER_E_BR_INTO_HND,
            0x8013183F => URT::VER_E_BR_INTO_FIL,
            0x80131840 => URT::VER_E_BR_OUTOF_TRY,
            0x80131841 => URT::VER_E_BR_OUTOF_HND,
            0x80131842 => URT::VER_E_BR_OUTOF_FIL,
            0x80131843 => URT::VER_E_BR_OUTOF_FIN,
            0x80131844 => URT::VER_E_RET_FROM_TRY,
            0x80131845 => URT::VER_E_RET_FROM_HND,
            0x80131846 => URT::VER_E_RET_FROM_FIL,
            0x80131847 => URT::VER_E_BAD_JMP_TARGET,
            0x80131848 => URT::VER_E_PATH_LOC,
            0x80131849 => URT::VER_E_PATH_THIS,
            0x8013184A => URT::VER_E_PATH_STACK,
            0x8013184B => URT::VER_E_PATH_STACK_DEPTH,
            0x8013184C => URT::VER_E_THIS,
            0x8013184D => URT::VER_E_THIS_UNINIT_EXCEP,
            0x8013184E => URT::VER_E_THIS_UNINIT_STORE,
            0x8013184F => URT::VER_E_THIS_UNINIT_RET,
            0x80131850 => URT::VER_E_THIS_UNINIT_V_RET,
            0x80131851 => URT::VER_E_THIS_UNINIT_BR,
            0x80131852 => URT::VER_E_LDFTN_CTOR,
            0x80131853 => URT::VER_E_STACK_NOT_EQ,
            0x80131854 => URT::VER_E_STACK_UNEXPECTED,
            0x80131855 => URT::VER_E_STACK_EXCEPTION,
            0x80131856 => URT::VER_E_STACK_OVERFLOW,
            0x80131857 => URT::VER_E_STACK_UNDERFLOW,
            0x80131858 => URT::VER_E_STACK_EMPTY,
            0x80131859 => URT::VER_E_STACK_UNINIT,
            0x8013185A => URT::VER_E_STACK_I_I4_I8,
            0x8013185B => URT::VER_E_STACK_R_R4_R8,
            0x8013185C => URT::VER_E_STACK_NO_R_I8,
            0x8013185D => URT::VER_E_STACK_NUMERIC,
            0x8013185E => URT::VER_E_STACK_OBJREF,
            0x8013185F => URT::VER_E_STACK_P_OBJREF,
            0x80131860 => URT::VER_E_STACK_BYREF,
            0x80131861 => URT::VER_E_STACK_METHOD,
            0x80131862 => URT::VER_E_STACK_ARRAY_SD,
            0x80131863 => URT::VER_E_STACK_VALCLASS,
            0x80131864 => URT::VER_E_STACK_P_VALCLASS,
            0x80131865 => URT::VER_E_STACK_NO_VALCLASS,
            0x80131866 => URT::VER_E_LOC_DEAD,
            0x80131867 => URT::VER_E_LOC_NUM,
            0x80131868 => URT::VER_E_ARG_NUM,
            0x80131869 => URT::VER_E_TOKEN_RESOLVE,
            0x8013186A => URT::VER_E_TOKEN_TYPE,
            0x8013186B => URT::VER_E_TOKEN_TYPE_MEMBER,
            0x8013186C => URT::VER_E_TOKEN_TYPE_FIELD,
            0x8013186D => URT::VER_E_TOKEN_TYPE_SIG,
            0x8013186E => URT::VER_E_UNVERIFIABLE,
            0x8013186F => URT::VER_E_LDSTR_OPERAND,
            0x80131870 => URT::VER_E_RET_PTR_TO_STACK,
            0x80131871 => URT::VER_E_RET_VOID,
            0x80131872 => URT::VER_E_RET_MISSING,
            0x80131873 => URT::VER_E_RET_EMPTY,
            0x80131874 => URT::VER_E_RET_UNINIT,
            0x80131875 => URT::VER_E_ARRAY_ACCESS,
            0x80131876 => URT::VER_E_ARRAY_V_STORE,
            0x80131877 => URT::VER_E_ARRAY_SD,
            0x80131878 => URT::VER_E_ARRAY_SD_PTR,
            0x80131879 => URT::VER_E_ARRAY_FIELD,
            0x8013187A => URT::VER_E_ARGLIST,
            0x8013187B => URT::VER_E_VALCLASS,
            0x8013187C => URT::VER_E_METHOD_ACCESS,
            0x8013187D => URT::VER_E_FIELD_ACCESS,
            0x8013187E => URT::VER_E_DEAD,
            0x8013187F => URT::VER_E_FIELD_STATIC,
            0x80131880 => URT::VER_E_FIELD_NO_STATIC,
            0x80131881 => URT::VER_E_ADDR,
            0x80131882 => URT::VER_E_ADDR_BYREF,
            0x80131883 => URT::VER_E_ADDR_LITERAL,
            0x80131884 => URT::VER_E_INITONLY,
            0x80131885 => URT::VER_E_THROW,
            0x80131886 => URT::VER_E_CALLVIRT_VALCLASS,
            0x80131887 => URT::VER_E_CALL_SIG,
            0x80131888 => URT::VER_E_CALL_STATIC,
            0x80131889 => URT::VER_E_CTOR,
            0x8013188A => URT::VER_E_CTOR_VIRT,
            0x8013188B => URT::VER_E_CTOR_OR_SUPER,
            0x8013188C => URT::VER_E_CTOR_MUL_INIT,
            0x8013188D => URT::VER_E_SIG,
            0x8013188E => URT::VER_E_SIG_ARRAY,
            0x8013188F => URT::VER_E_SIG_ARRAY_PTR,
            0x80131890 => URT::VER_E_SIG_ARRAY_BYREF,
            0x80131891 => URT::VER_E_SIG_ELEM_PTR,
            0x80131892 => URT::VER_E_SIG_VARARG,
            0x80131893 => URT::VER_E_SIG_VOID,
            0x80131894 => URT::VER_E_SIG_BYREF_BYREF,
            0x80131896 => URT::VER_E_CODE_SIZE_ZERO,
            0x80131897 => URT::VER_E_BAD_VARARG,
            0x80131898 => URT::VER_E_TAIL_CALL,
            0x80131899 => URT::VER_E_TAIL_BYREF,
            0x8013189A => URT::VER_E_TAIL_RET,
            0x8013189B => URT::VER_E_TAIL_RET_VOID,
            0x8013189C => URT::VER_E_TAIL_RET_TYPE,
            0x8013189D => URT::VER_E_TAIL_STACK_EMPTY,
            0x8013189E => URT::VER_E_METHOD_END,
            0x8013189F => URT::VER_E_BAD_BRANCH,
            0x801318A0 => URT::VER_E_FIN_OVERLAP,
            0x801318A1 => URT::VER_E_LEXICAL_NESTING,
            0x801318A2 => URT::VER_E_VOLATILE,
            0x801318A3 => URT::VER_E_UNALIGNED,
            0x801318A4 => URT::VER_E_INNERMOST_FIRST,
            0x801318A5 => URT::VER_E_CALLI_VIRTUAL,
            0x801318A6 => URT::VER_E_CALL_ABSTRACT,
            0x801318A7 => URT::VER_E_STACK_UNEXP_ARRAY,
            0x801318A8 => URT::VER_E_NOT_IN_GC_HEAP,
            0x801318A9 => URT::VER_E_TRY_N_EMPTY_STACK,
            0x801318AA => URT::VER_E_DLGT_CTOR,
            0x801318AB => URT::VER_E_DLGT_BB,
            0x801318AC => URT::VER_E_DLGT_PATTERN,
            0x801318AD => URT::VER_E_DLGT_LDFTN,
            0x801318AE => URT::VER_E_FTN_ABSTRACT,
            0x801318AF => URT::VER_E_SIG_C_VC,
            0x801318B0 => URT::VER_E_SIG_VC_C,
            0x801318B1 => URT::VER_E_BOX_PTR_TO_STACK,
            0x801318B2 => URT::VER_E_SIG_BYREF_TB_AH,
            0x801318B3 => URT::VER_E_SIG_ARRAY_TB_AH,
            0x801318B4 => URT::VER_E_ENDFILTER_STACK,
            0x801318B5 => URT::VER_E_DLGT_SIG_I,
            0x801318B6 => URT::VER_E_DLGT_SIG_O,
            0x801318B7 => URT::VER_E_RA_PTR_TO_STACK,
            0x801318B8 => URT::VER_E_CATCH_VALUE_TYPE,
            0x801318B9 => URT::VER_E_CATCH_BYREF,
            0x801318BA => URT::VER_E_FIL_PRECEED_HND,
            0x801318BB => URT::VER_E_LDVIRTFTN_STATIC,
            0x801318BC => URT::VER_E_CALLVIRT_STATIC,
            0x801318BD => URT::VER_E_INITLOCALS,
            0x801318BE => URT::VER_E_BR_TO_EXCEPTION,
            0x801318BF => URT::VER_E_CALL_CTOR,
            0x801318C0 => URT::VER_E_VALCLASS_OBJREF_VAR,
            0x801318C1 => URT::VER_E_STACK_P_VALCLASS_OBJREF_VAR,
            0x801318C2 => URT::VER_E_SIG_VAR_PARAM,
            0x801318C3 => URT::VER_E_SIG_MVAR_PARAM,
            0x801318C4 => URT::VER_E_SIG_VAR_ARG,
            0x801318C5 => URT::VER_E_SIG_MVAR_ARG,
            0x801318C6 => URT::VER_E_SIG_GENERICINST,
            0x801318C7 => URT::VER_E_SIG_METHOD_INST,
            0x801318C8 => URT::VER_E_SIG_METHOD_PARENT_INST,
            0x801318C9 => URT::VER_E_SIG_FIELD_PARENT_INST,
            0x801318CA => URT::VER_E_CALLCONV_NOT_GENERICINST,
            0x801318CB => URT::VER_E_TOKEN_BAD_METHOD_SPEC,
            0x801318CC => URT::VER_E_BAD_READONLY_PREFIX,
            0x801318CD => URT::VER_E_BAD_CONSTRAINED_PREFIX,
            0x801318CE => URT::VER_E_CIRCULAR_VAR_CONSTRAINTS,
            0x801318CF => URT::VER_E_CIRCULAR_MVAR_CONSTRAINTS,
            0x801318D0 => URT::VER_E_UNSATISFIED_METHOD_INST,
            0x801318D1 => URT::VER_E_UNSATISFIED_METHOD_PARENT_INST,
            0x801318D2 => URT::VER_E_UNSATISFIED_FIELD_PARENT_INST,
            0x801318D3 => URT::VER_E_UNSATISFIED_BOX_OPERAND,
            0x801318D4 => URT::VER_E_CONSTRAINED_CALL_WITH_NON_BYREF_THIS,
            0x801318D5 => URT::VER_E_CONSTRAINED_OF_NON_VARIABLE_TYPE,
            0x801318D6 => URT::VER_E_READONLY_UNEXPECTED_CALLEE,
            0x801318D7 => URT::VER_E_READONLY_ILLEGAL_WRITE,
            0x801318D8 => URT::VER_E_READONLY_IN_MKREFANY,
            0x801318D9 => URT::VER_E_UNALIGNED_ALIGNMENT,
            0x801318DA => URT::VER_E_TAILCALL_INSIDE_EH,
            0x801318DB => URT::VER_E_BACKWARD_BRANCH,
            0x801318DC => URT::VER_E_CALL_TO_VTYPE_BASE,
            0x801318DD => URT::VER_E_NEWOBJ_OF_ABSTRACT_CLASS,
            0x801318DE => URT::VER_E_UNMANAGED_POINTER,
            0x801318DF => URT::VER_E_LDFTN_NON_FINAL_VIRTUAL,
            0x801318E0 => URT::VER_E_FIELD_OVERLAP,
            0x801318E1 => URT::VER_E_THIS_MISMATCH,
            0x801318E2 => URT::VER_E_STACK_I_I4,
            0x801318F0 => URT::VER_E_BAD_PE,
            0x801318F1 => URT::VER_E_BAD_MD,
            0x801318F2 => URT::VER_E_BAD_APPDOMAIN,
            0x801318F3 => URT::VER_E_TYPELOAD,
            0x801318F4 => URT::VER_E_PE_LOAD,
            0x801318F5 => URT::VER_E_WRITE_RVA_STATIC,
            0x80131904 => URT::COR_E_SqlException,
            0x80131920 => URT::COR_E_Data,
            0x80131921 => URT::COR_E_DataDeletedRowInaccessible,
            0x80131922 => URT::COR_E_DataDuplicateName,
            0x80131923 => URT::COR_E_DataInRowChangingEvent,
            0x80131924 => URT::COR_E_DataInvalidConstraint,
            0x80131925 => URT::COR_E_DataMissingPrimaryKey,
            0x80131926 => URT::COR_E_DataNoNullAllowed,
            0x80131927 => URT::COR_E_DataReadOnly,
            0x80131928 => URT::COR_E_DataRowNotInTable,
            0x80131929 => URT::COR_E_DataVersionNotFound,
            0x8013192A => URT::COR_E_DataConstraint,
            0x8013192B => URT::COR_E_StrongTyping,
            0x80131930 => URT::COR_E_SqlType,
            0x80131931 => URT::COR_E_SqlNullValue,
            0x80131932 => URT::COR_E_SqlTruncate,
            0x80131933 => URT::COR_E_AdapterMapping,
            0x80131934 => URT::COR_E_DataAdapter,
            0x80131935 => URT::COR_E_DBConcurrency,
            0x80131936 => URT::COR_E_OperationAborted,
            0x80131937 => URT::COR_E_InvalidUdt,
            0x80131937 => URT::COR_E_OdbcException,
            0x80131938 => URT::COR_E_OracleException,
            0x80131940 => URT::COR_E_Xml,
            0x80131941 => URT::COR_E_XmlSchema,
            0x80131942 => URT::COR_E_XmlXslt,
            0x80131943 => URT::COR_E_XmlXPath,
            0x80131944 => URT::COR_E_XmlQuery,
            0x80131B00 => URT::VLDTR_E_IFACE_NOTIFACE,
            0x80131B00 => URT::TLBX_E_CIRCULAR_EXPORT2,
            0x80131B01 => URT::VLDTR_E_FD_RVAHASNORVA,
            0x80131B02 => URT::VLDTR_E_FD_RVAHASZERORVA,
            0x80131B03 => URT::VLDTR_E_MD_RVAANDIMPLMAP,
            0x80131B04 => URT::VLDTR_E_TD_EXTRAFLAGS,
            0x80131B05 => URT::VLDTR_E_TD_EXTENDSITSELF,
            0x80131B06 => URT::VLDTR_E_TD_SYSVTNOTEXTOBJ,
            0x80131B07 => URT::VLDTR_E_TD_EXTTYPESPEC,
            0x80131B09 => URT::VLDTR_E_TD_VTNOSIZE,
            0x80131B0A => URT::VLDTR_E_TD_IFACESEALED,
            0x80131B0B => URT::VLDTR_E_NC_BADNESTED,
            0x80131B0C => URT::VLDTR_E_NC_BADENCLOSER,
            0x80131B0D => URT::VLDTR_E_NC_DUP,
            0x80131B0E => URT::VLDTR_E_NC_DUPENCLOSER,
            0x80131B0F => URT::VLDTR_E_FRVA_ZERORVA,
            0x80131B10 => URT::VLDTR_E_FRVA_BADFIELD,
            0x80131B11 => URT::VLDTR_E_FRVA_DUPRVA,
            0x80131B12 => URT::VLDTR_E_FRVA_DUPFIELD,
            0x80131B13 => URT::VLDTR_E_EP_BADTOKEN,
            0x80131B14 => URT::VLDTR_E_EP_INSTANCE,
            0x80131B15 => URT::VLDTR_E_TD_ENUMFLDBADTYPE,
            0x80131B16 => URT::VLDTR_E_MD_BADRVA,
            0x80131B17 => URT::VLDTR_E_FD_LITERALNODEFAULT,
            0x80131B18 => URT::VLDTR_E_IFACE_METHNOTIMPL,
            0x80131B19 => URT::VLDTR_E_CA_BADPARENT,
            0x80131B1A => URT::VLDTR_E_CA_BADTYPE,
            0x80131B1B => URT::VLDTR_E_CA_NOTCTOR,
            0x80131B1C => URT::VLDTR_E_CA_BADSIG,
            0x80131B1D => URT::VLDTR_E_CA_NOSIG,
            0x80131B1E => URT::VLDTR_E_CA_BADPROLOG,
            0x80131B1F => URT::VLDTR_E_MD_BADLOCALSIGTOK,
            0x80131B20 => URT::VLDTR_E_MD_BADHEADER,
            0x80131B21 => URT::VLDTR_E_EP_TOOMANYARGS,
            0x80131B22 => URT::VLDTR_E_EP_BADRET,
            0x80131B23 => URT::VLDTR_E_EP_BADARG,
            0x80131B24 => URT::VLDTR_E_SIG_BADVOID,
            0x80131B25 => URT::VLDTR_E_IFACE_METHMULTIMPL,
            0x80131B26 => URT::VLDTR_E_GP_NAMENULL,
            0x80131B27 => URT::VLDTR_E_GP_OWNERNIL,
            0x80131B28 => URT::VLDTR_E_GP_DUPNAME,
            0x80131B29 => URT::VLDTR_E_GP_DUPNUMBER,
            0x80131B2A => URT::VLDTR_E_GP_NONSEQ_BY_OWNER,
            0x80131B2B => URT::VLDTR_E_GP_NONSEQ_BY_NUMBER,
            0x80131B2C => URT::VLDTR_E_GP_UNEXPECTED_OWNER_FOR_VARIANT_VAR,
            0x80131B2D => URT::VLDTR_E_GP_ILLEGAL_VARIANT_MVAR,
            0x80131B2E => URT::VLDTR_E_GP_ILLEGAL_VARIANCE_FLAGS,
            0x80131B2F => URT::VLDTR_E_GP_REFANDVALUETYPE,
            0x80131B30 => URT::VLDTR_E_GPC_OWNERNIL,
            0x80131B31 => URT::VLDTR_E_GPC_DUP,
            0x80131B32 => URT::VLDTR_E_GPC_NONCONTIGUOUS,
            0x80131B33 => URT::VLDTR_E_MS_METHODNIL,
            0x80131B34 => URT::VLDTR_E_MS_DUP,
            0x80131B35 => URT::VLDTR_E_MS_BADCALLINGCONV,
            0x80131B36 => URT::VLDTR_E_MS_MISSARITY,
            0x80131B37 => URT::VLDTR_E_MS_MISSARG,
            0x80131B38 => URT::VLDTR_E_MS_ARITYMISMATCH,
            0x80131B39 => URT::VLDTR_E_MS_METHODNOTGENERIC,
            0x80131B3A => URT::VLDTR_E_SIG_MISSARITY,
            0x80131B3B => URT::VLDTR_E_SIG_ARITYMISMATCH,
            0x80131B3C => URT::VLDTR_E_MD_GENERIC_CCTOR,
            0x80131B3D => URT::VLDTR_E_MD_GENERIC_CTOR,
            0x80131B3E => URT::VLDTR_E_MD_GENERIC_IMPORT,
            0x80131B3F => URT::VLDTR_E_MD_GENERIC_BADCALLCONV,
            0x80131B40 => URT::VLDTR_E_MD_GENERIC_GLOBAL,
            0x80131B41 => URT::VLDTR_E_EP_GENERIC_METHOD,
            0x80131B42 => URT::VLDTR_E_MD_MISSARITY,
            0x80131B43 => URT::VLDTR_E_MD_ARITYZERO,
            0x80131B44 => URT::VLDTR_E_SIG_ARITYZERO,
            0x80131B45 => URT::VLDTR_E_MS_ARITYZERO,
            0x80131B46 => URT::VLDTR_E_MD_GPMISMATCH,
            0x80131B47 => URT::VLDTR_E_EP_GENERIC_TYPE,
            0x80131B48 => URT::VLDTR_E_MI_DECLNOTGENERIC,
            0x80131B49 => URT::VLDTR_E_MI_IMPLNOTGENERIC,
            0x80131B4A => URT::VLDTR_E_MI_ARITYMISMATCH,
            0x80131B4B => URT::VLDTR_E_TD_EXTBADTYPESPEC,
            0x80131B4C => URT::VLDTR_E_SIG_BYREFINST,
            0x80131B4D => URT::VLDTR_E_MS_BYREFINST,
            0x80131B4E => URT::VLDTR_E_TS_EMPTY,
            0x80131B4F => URT::VLDTR_E_TS_HASSENTINALS,
            0x80131B50 => URT::VLDTR_E_TD_GENERICHASEXPLAYOUT,
            0x80131B51 => URT::VLDTR_E_SIG_BADTOKTYPE,
            0x80131B52 => URT::VLDTR_E_IFACE_METHNOTIMPLTHISMOD,
            0x80131C00 => URT::CORDBG_E_THREAD_NOT_SCHEDULED,
            0x80131C01 => URT::CORDBG_E_HANDLE_HAS_BEEN_DISPOSED,
            0x80131C02 => URT::CORDBG_E_NONINTERCEPTABLE_EXCEPTION,
            0x80131C03 => URT::CORDBG_E_CANT_UNWIND_ABOVE_CALLBACK,
            0x80131C04 => URT::CORDBG_E_INTERCEPT_FRAME_ALREADY_SET,
            0x80131C05 => URT::CORDBG_E_NO_NATIVE_PATCH_AT_ADDR,
            0x80131C06 => URT::CORDBG_E_MUST_BE_INTEROP_DEBUGGING,
            0x80131C07 => URT::CORDBG_E_NATIVE_PATCH_ALREADY_AT_ADDR,
            0x80131C08 => URT::CORDBG_E_TIMEOUT,
            0x80131C09 => URT::CORDBG_E_CANT_CALL_ON_THIS_THREAD,
            0x80131C0A => URT::CORDBG_E_ENC_INFOLESS_METHOD,
            0x80131C0B => URT::CORDBG_E_ENC_NESTED_HANLDERS,
            0x80131C0C => URT::CORDBG_E_ENC_IN_FUNCLET,
            0x80131C0D => URT::CORDBG_E_ENC_LOCALLOC,
            0x80131C0E => URT::CORDBG_E_ENC_EDIT_NOT_SUPPORTED,
            0x80131C0F => URT::CORDBG_E_FEABORT_DELAYED_UNTIL_THREAD_RESUMED,
            0x80131C10 => URT::CORDBG_E_NOTREADY,
            0x80131C11 => URT::CORDBG_E_CANNOT_RESOLVE_ASSEMBLY,
            0x80131C12 => URT::CORDBG_E_MUST_BE_IN_LOAD_MODULE,
            0x80131C13 => URT::CORDBG_E_CANNOT_BE_ON_ATTACH,
            0x80131C14 => URT::CORDBG_E_NGEN_NOT_SUPPORTED,
            0x80131C15 => URT::CORDBG_E_ILLEGAL_SHUTDOWN_ORDER,
            0x80131C16 => URT::CORDBG_E_CANNOT_DEBUG_FIBER_PROCESS,
            0x80131C17 => URT::CORDBG_E_MUST_BE_IN_CREATE_PROCESS,
            0x80131C18 => URT::CORDBG_E_DETACH_FAILED_OUTSTANDING_EVALS,
            0x80131C19 => URT::CORDBG_E_DETACH_FAILED_OUTSTANDING_STEPPERS,
            0x80131C20 => URT::CORDBG_E_CANT_INTEROP_STEP_OUT,
            0x80131C21 => URT::CORDBG_E_DETACH_FAILED_OUTSTANDING_BREAKPOINTS,
            0x80131C22 => URT::CORDBG_E_ILLEGAL_IN_STACK_OVERFLOW,
            0x80131C23 => URT::CORDBG_E_ILLEGAL_AT_GC_UNSAFE_POINT,
            0x80131C24 => URT::CORDBG_E_ILLEGAL_IN_PROLOG,
            0x80131C25 => URT::CORDBG_E_ILLEGAL_IN_NATIVE_CODE,
            0x80131C26 => URT::CORDBG_E_ILLEGAL_IN_OPTIMIZED_CODE,
            0x80131C27 => URT::CORDBG_E_MINIDUMP_UNSUPPORTED,
            0x80131C28 => URT::CORDBG_E_APPDOMAIN_MISMATCH,
            0x80131C29 => URT::CORDBG_E_CONTEXT_UNVAILABLE,
            0x80131C30 => URT::CORDBG_E_UNCOMPATIBLE_PLATFORMS,
            0x80131C31 => URT::CORDBG_E_DEBUGGING_DISABLED,
            0x80131C32 => URT::CORDBG_E_DETACH_FAILED_ON_ENC,
            0x80131C33 => URT::CORDBG_E_CURRENT_EXCEPTION_IS_OUTSIDE_CURRENT_EXECUTION_SCOPE,
            0x80131C34 => URT::CORDBG_E_HELPER_MAY_DEADLOCK,
            0x80131C35 => URT::CORDBG_E_MISSING_METADATA,
            0x80131C36 => URT::CORDBG_E_TARGET_INCONSISTENT,
            0x80131C37 => URT::CORDBG_E_DETACH_FAILED_OUTSTANDING_TARGET_RESOURCES,
            0x80131C38 => URT::CORDBG_E_TARGET_READONLY,
            0x80131C39 => URT::CORDBG_E_MISMATCHED_CORWKS_AND_DACWKS_DLLS,
            0x80131C3A => URT::CORDBG_E_MODULE_LOADED_FROM_DISK,
            0x80131C3B => URT::CORDBG_E_SYMBOLS_NOT_AVAILABLE,
            0x80131C3C => URT::CORDBG_E_DEBUG_COMPONENT_MISSING,
            0x80131C3D => URT::CORDBG_E_REMOTE_MISMATCHED_CERTS,
            0x80131C3E => URT::CORDBG_E_REMOTE_NETWORK_FAILURE,
            0x80131C3F => URT::CORDBG_E_REMOTE_NO_LISTENER,
            0x80131C40 => URT::CORDBG_E_REMOTE_UNKNOWN_TARGET,
            0x80131C41 => URT::CORDBG_E_REMOTE_INVALID_CONFIG,
            0x80131C42 => URT::CORDBG_E_REMOTE_MISMATCHED_PROTOCOLS,
            0x80131C43 => URT::CORDBG_E_LIBRARY_PROVIDER_ERROR,
            0x80131C44 => URT::CORDBG_E_NOT_CLR,
            0x80131C45 => URT::CORDBG_E_MISSING_DATA_TARGET_INTERFACE,
            0x80131C46 => URT::CORDBG_E_UNSUPPORTED_DEBUGGING_MODEL,
            0x80131C47 => URT::CORDBG_E_UNSUPPORTED_FORWARD_COMPAT,
            0x80131C48 => URT::CORDBG_E_UNSUPPORTED_VERSION_STRUCT,
            0x80131C49 => URT::CORDBG_E_READVIRTUAL_FAILURE,
            0x80131C4A => URT::CORDBG_E_VALUE_POINTS_TO_FUNCTION,
            0x80131C4B => URT::CORDBG_E_CORRUPT_OBJECT,
            0x80131C4C => URT::CORDBG_E_GC_STRUCTURES_INVALID,
            0x80131C4D => URT::CORDBG_E_INVALID_OPCODE,
            0x80131C4E => URT::CORDBG_E_UNSUPPORTED,
            0x80131C4F => URT::CORDBG_E_MISSING_DEBUGGER_EXPORTS,
            0x80131D00 => URT::PEFMT_E_NO_CONTENTS,
            0x80131D01 => URT::PEFMT_E_NO_NTHEADERS,
            0x80131D02 => URT::PEFMT_E_64BIT,
            0x80131D03 => URT::PEFMT_E_NO_CORHEADER,
            0x80131D04 => URT::PEFMT_E_NOT_ILONLY,
            0x80131D05 => URT::PEFMT_E_IMPORT_DLLS,
            0x80131D06 => URT::PEFMT_E_EXE_NOENTRYPOINT,
            0x80131D07 => URT::PEFMT_E_BASE_RELOCS,
            0x80131D08 => URT::PEFMT_E_ENTRYPOINT,
            0x80131D09 => URT::PEFMT_E_ZERO_SIZEOFCODE,
            0x80131D0A => URT::PEFMT_E_BAD_CORHEADER,
            0x80131D0B => URT::PEFMT_E_32BIT,
            0x80131E00 => URT::CLR_OPTSVC_E_CONTROLLER_INTERRUPT,
            0x80131F00 => URT::NGEN_FAILED_GET_DEPENDENCIES,
            0x80131F01 => URT::NGEN_FAILED_NATIVE_IMAGE_DELETE,
            0x80131F02 => URT::NGEN_E_TOO_MANY_INTERFACES,
            0x80131F03 => URT::NGEN_E_OLDER_RUNTIME,
            0x80131F04 => URT::NGEN_E_WORKER_UNEXPECTED_EXIT,
            0x80131F05 => URT::NGEN_E_WORKER_UNEXPECTED_SYNC,
            0x80131F06 => URT::NGEN_E_SYS_ASM_NI_MISSING,
            0x80131FFF => URT::CLDB_E_INTERNALERROR,
        }
    }
}
